// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// clientdefs.h

// Global #defines and C++ constants
// Please nothing else - no #includes, no types

// In general keep this file as small as possible.

#ifndef __clientdefs_h
#define __clientdefs_h


// To be defined in env var (cl), 2000-3-13 MarcP.
// Always define INTERNAL_BUILD when _DEBUG is defined
#if (!defined(INTERNAL_BUILD)) && defined(_DEBUG)
#   define INTERNAL_BUILD
#endif

#define FAKE_PORT	69
 
// Should be same as FRPKT_NUM_ROUTES_MAX in <fr_rcpkt.h>
#define MAX_AIPS_IN_ROUTE	    5

#define MAX_OPERATOR_LEN		64

// Maximum number of concurrent routes
#define MAX_ROUTES				10

#define MAX_ROUTES_TO_CALCULATE_FOR_CHECK_RULES 10

// Nym routes can have variable number of hops
#define MIN_HOPS	              1
#define MAX_HOPS	              3
#define DEFAULT_HOPS_IN_NYM_ROUTE 1

#define OFFENSIVE_WORD_SZ	64

// Key expiration policy needs to be visited once key rotation is enabled.
// Because we don't rotate then our keys need to last a long long time....

#define DAYS_UNTIL_FREEDOM_KEYS_EXPIRE		1000


// We know that the first nym (index 0) is the User Nym
#define USER_NYM_INDEX 0

// NIQS reports these in 0-255 range, we absorb 0-5 only for the UI
#define RATING(x) ((x/255.) * 5)

// Replies from the GUI concerning socket connections
#define PREVENT_CONNECTION		1
#define ALLOW_CONNECTION		2

#define		WARN_BOGUS_MAIL_SERVER		"NoWarnCheckingBogusMailServer"
#define		NO_HTTP_PROX_FOR_USER	"NoHttpProxForUser"

// For users without a real mail server - If the SMTP/POP proxies see this, they 
// will not connect to the user's server.
static const char DUMMY_MAIL_SERVER[]		= "mail.freedom.net";
static const char BOGUS_EMAIL_ADDRESS[]		= "freedom@mail.freedom.net";

#define		ZK_MAIL_CHECK_ALL 1
#define		ZK_MAIL_CHECK_ACTIVE 2
#define		ZK_MAIL_CHECK_BLOCK 3

#define DEFAULT_NYM_ROUTE_SECS			32 * 60 // 32 minutes
#define DEFAULT_SERVER_ROUTE_SECS		180 * 60 // 3 hours
#define DEFAULT_MAXKEYQUERY_TIME_SECS	3	

// Special ID for route that should be blocked
#define INVALID_ROUTE_ID 0xFFFFFFFF

//Used by IZkAppState and ZKAppState as global mutexes
static const char MUTEX_FREEDOM_RUNNING[] = "OneInstanceZKSApps";
static const char MUTEX_FREEDOM_INIT[] = "FreedomInitMutex";
static const char MUTEX_WALLET_READY[] = "WalletReadyMutex";
static const char MUTEX_FIREWALL_STATE[] = "FirewallStateMutex";


// XP Macros to extract the upper or lower bytes of an int (word)
// In windows parlance these are the HIWORD and LOWORD macros
// which extract a 'WORD' from a 'DWORD' int. Of course int is the CPU 
// word size ... pretty soon we'll have QuadWORD on our brand spanking new 
// 64 bit word size processors. Thank god windows never shipped a version for 808Xs!

#define ZK_LOSHORT(l)           ((uint16_t)(l))
#define ZK_HISHORT(l)           ((uint16_t)((uint32_t)(l) >> 16))



#endif
