// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// clientenum.h

// Global enumerations for the client

// In general keep this file as small as possible.

#ifndef __clientenum_h
#define __clientenum_h

typedef enum INET_PROTOCOLS{
	NO_PROTOCOL		 = 0,
	PROTOCOL_HTTP    = 1,
	PROTOCOL_POP3    = 2,
	PROTOCOL_SMTP    = 3,
	PROTOCOL_IRC     = 4,
	PROTOCOL_NNTP1   = 5,
	PROTOCOL_TELNET  = 6,
	PROTOCOL_SSL     = 7,
	PROTOCOL_SSH	 = 8,
	UNKNOWN_PROTOCOL = 9
}INET_PROTOCOLS;

// Basic firewall functionality for handling the ports
// that we do not support (ie not a Freedom Server port and not a port handled by the proxies such as 80)
enum UnsupportedPortAction 
{
	UndefinedPortAction = -1,
	BlockAllPorts = 0,
	OpenAllPorts = 1,
	OpenSelectPorts = 2			// Preselected ports have been opened, rest are all blocked
} ;

// Used in FirewallAppPersist structure
enum eTrafficType { unknowntraffic, cloud, clear };

// Used in FirewallEvent_UsersChoice structure 
enum eSelection { unknownselection, allow, block };

// Used in ZKDialogs::LauchFirewallAppDialog parameter
enum eDirection { in, out };

// Used in connection_info structure
// The post pre prefixes indicate when the call is passed to the firewall in relation to the underlying provider
enum ESOCKETAPI { epostbind, epresendto, epreconnect, epostaccept, eprelisten, eprebind };

// Used in endpoint structure
enum eTcpState { unknown, closed, listening, syn_sent, syn_received, established, fin_wait, fin_wait2, close_wait, closing, last_ack, time_wait };

enum FirewallAction
{	
	FirewallAllowOut			= 0,	// Always make == NO_ERROR, that is, 0. Allow the data to continue, unhindered, to its destination
	FirewallBlock				= 1,	// If Freedom returns this the Winsock api will fail with SOCKET_ERROR and a specific error in lpErrno
	FirewallForwardSockaddr		= 2,	// Send real address
	FirewallRouteThroughCloud   = 3,	// Data will be re-routed anonymously through our Aips
	FirewallAskUser             = 4		// Ask
} ;

enum eConsoleVerbosity
{
	high,			// everything goes onto the console
	medium,			// everything except Netbios and Dhcp
	low				// No Netbios, Dhcp, DNS or any other incoming packet, the default
};

enum RULEZ{
	RULE_NONE = 0,
	RULE_RANDOM = 1,
	RULE_SPECIFY = 2,
	RULE_FIRSTHOP = 4,
	RULE_AVOID = 8,
	RULE_MUSTUSE = 16,
	RULE_EXITFUNNEL = 32
};

enum NYM_LIFE_STAGES {
	NYM_LIFE_STAGE_UNDEFINED	= 0,
	NYM_LIFE_STAGE_ACTIVE		= 1,		// Normal nym state
	NYM_LIFE_STAGE_EXPIRED		= 2,		// Means that time has run out, nym is inactive but
											// visible in the client, can be recoved
	NYM_LIFE_STAGE_RELEASED		= 3			// As far as the client is concerned the nym is dead, gone
											// forever, although it may still have a presence on the
											// server
} ;
// Nym Preference Bit Fields
// Nym specific preferences stored in the frRemoteNym
enum NYM_PREFS {
	PREF_COOKIES_DISABLED		= 1,
	PREF_NO_HIDE_REFERER		= 1 << 2,
	PREF_INTERACTIVE_ROUTES     = 1 << 3
};

// Freedom status types
// Used when showing status AVIs and Bitmaps - Global so that
// anyone can set the app to a particular state without knowing the actual resource ID

enum ZK_STATUS_TYPE {
	ZK_STATUS_UNDEFINED = 0,
	ZK_STATUS_SECURE	= 1,		// Whether a nym is active or not, in future with multiple
									// levels of nym security might become a multistate

	ZK_STATUS_MAILBROKEN = 2,       // Show that email is not activated yet for a particular nym

	// Busy States - show that Freedom is doing something

	ZK_STATUS_PROGRESS_BUSY = 100,	// General busy
	ZK_STATUS_NETWORK_BUSY  = 101,
	ZK_STATUS_TASKBAR_BUSY  = 102,
	ZK_STATUS_MAIL_BUSY     = 103,	// Handles mail and news currently
	ZK_STATUS_CRYPTO_BUSY   = 104,	// Encryption of mail, etc.

	// Protocol connection states
	ZK_STATUS_MAIL_CONNECTION   = 200,
	ZK_STATUS_WEB_CONNECTION    = 201,
	ZK_STATUS_SSL_CONNECTION    = 202,
	ZK_STATUS_TELNET_CONNECTION = 203,
	ZK_STATUS_IRC_CONNECTION    = 204,
	ZK_STATUS_NEWS_CONNECTION   = 205,
	
	// Brief Events
	ZK_STATUS_WEB_COOKIE		= 300,
	ZK_STATUS_REFUSE_CONNECTION = 301   // Firewall event showing we block outgoing connection with nym active
} ;

// Defines the condition we are now in
// For toggle types this is simple ON or OFF.  For items with multiple states
// there are different possibilities

enum ZK_STATUS_CONDITION {
	ZK_CONDITION_DEFAULT			= -1,
	ZK_CONDITION_OFF				= -1,
	ZK_CONDITION_ON					= 0,

	// Special multi-state toggles (values associate
	// with position in the bitmap array that represents all the possible states)
	ZK_CONDITION_USERACTIVE			= 0,
	ZK_CONDITION_NYMACTIVE			= 1
};

enum RouteType 
{ 
	RouteTypeUnknown     = 0,
	AuthenticatedRoute   = 1,
	UnauthenticatedRoute = 2
};

enum RouteState 
{
	RouteInvalid         = 0,		// No route exist

	RouteValidNotActive  = 1,	    // Route has been created locally, not yet active

	RouteActive			 = 2,	    // Route create ACK has been created - traffic can
									// flow.  This state is valid until the route is prolonged,
									// the route times out, the route is destroyed by client, or the route
									// destroy packet is recieved

	RouteRetired         = 3        // A new route has been created to prolong an old one.
									// the old one is retired and only around to decrypt late coming
									// packets.  If the route is explicitly destroyed then it never
									// lives in this state
};


// Different statistical information can be useful for a particular value
// For example for route creates we are interested in Min (best), Avg, Max (worst)
enum StatisticType
{
	UndefinedStatisticType = 0,
	CountDatums	           = 1,		// Number of data types
	AvgOfDatums            = 2,   
	MinDatum			   = 3,		// Lowest value of all datum ("Best" is small is good)
	MaxDatum               = 4	
} ;


enum SigCheckResult
{
	SigCheckNotAttempted = 0 ,
	SigCheckSuccessful   = 1 ,	// We have verified that the sending entity matchs the signature
	SigCheckFailed       = 2 ,  // The signature check failed
	SigCheckKeyNotAvail  = 3 ,  // We could not check the signature because we could not get the public signing key
	SigCheckKeyNotFound  = 4 	// The key was not found on the server
} ;

#endif