// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// clientwin32.h

// Main header file for Windows specific types,
// values etc 

#ifndef __clientwin32_h
#define __clientwin32_h

#include "clientWin32WM.h"
#include <winsock2.h>
#include <client.h>


/////////////// Use ZKS critical sections /////////////////////

class ZKCriticalSection
{
public:
	ZKCriticalSection() { InitializeCriticalSection(&cs); }
	~ZKCriticalSection() { DeleteCriticalSection(&cs); }

	void Lock() { EnterCriticalSection(&cs); }
	void Unlock() { LeaveCriticalSection(&cs); }
private:
	CRITICAL_SECTION cs;
};
///////////////////////////////////////////////////////////////


static const char FRWINDOWNAME[]			= "Freedom Login";

static const char FRRESTOREUTILWINDOWNAME[] = "Freedom Backup Recovery";

static const char FREEDOM_FIREWALL_WNDNAME[]= "Freedom Firewall Window";

static const char ZERO_REG_KEY[]			= "SOFTWARE\\Zero Knowledge\\Freedom";
static const char FREEDOM_PATH_KEY[]		= "Path";

/////////////////////////////////////////////////////////////////////////////////
//
// The names of mail clients we support.
//
static const char STR_MUA_OUTLOOK[]				= "Microsoft Outlook";
static const char STR_MUA_OUTLOOK_EXPRESS[]		= "Outlook Express";
static const char STR_MUA_NETSCAPE_MESSENGER[]	= "Netscape Messenger";
static const char STR_MUA_EUDORA[]				= "Eudora";

// It is alot better to prevent defect than to try to find them later
// this is built into vc7
#if _MSC_VER == 1200
#ifdef  END_MESSAGE_MAP  // last #define before <afxmsg_.h> is #included

#undef ON_MESSAGE
#define ON_MESSAGE(message, memberFxn) \
	{ message, 0, 0, 0, AfxSig_lwl, \
		(AFX_PMSG)(AFX_PMSGW) (static_cast<LRESULT (AFX_MSG_CALL CWnd::*)(WPARAM, LPARAM)>(&memberFxn)) },

#undef  ON_REGISTERED_MESSAGE
#define ON_REGISTERED_MESSAGE(nMessageVariable, memberFxn) \
{ 0xC000, 0, 0, 0, (unsigned int)(UINT*)(&nMessageVariable), \
(AFX_PMSG)(AFX_PMSGW)(static_cast<LRESULT (AFX_MSG_CALL CWnd::*)(WPARAM, LPARAM)>(&memberFxn)) },



#undef  ON_THREAD_MESSAGE
#define ON_THREAD_MESSAGE(message, memberFxn) \
{ message, 0, 0, 0, AfxSig_vwl,\
(AFX_PMSG)(AFX_PMSGT)(static_cast<void (AFX_MSG_CALL  CWinThread::*)(WPARAM, LPARAM)>(&memberFxn)) },



#undef  ON_REGISTERED_THREAD_MESSAGE
#define ON_REGISTERED_THREAD_MESSAGE(nMessageVariable, memberFxn) \
{ 0xC000, 0, 0, 0, (unsigned int)(UINT*)(&nMessageVariable), \
(AFX_PMSG)(AFX_PMSGT)(static_cast<void (AFX_MSG_CALL  WinThread::*)(WPARAM, LPARAM)>(&memberFxn)) }, 
   

#endif	// END_MESSAGE_MAP
#endif	// _MSC_VER == 1200

// Close an event created with WSACreateEvent() if it isn't WSA_INVALID_EVENT
#define SAFECLOSEWSAEVENT(x)	if( (x) != WSA_INVALID_EVENT)		{ WSACloseEvent(x); (x) = WSA_INVALID_EVENT; }

// Close a handle created with CreateEvent() if it isn't NULL
#define SAFECLOSEHANDLE(x)		if( (x) != NULL)					{ CloseHandle(x); (x) = NULL; }

// Close a handle created with CreateFile() if it isn't INVALID_HANDLE_VALUE
#define SAFECLOSEFILEHANDLE(x)	if( (x) != INVALID_HANDLE_VALUE)	{ CloseHandle(x); (x) = INVALID_HANDLE_VALUE; }

// Free a library handle created with LoadLibrary
#define SAFEFREELIB(x)			if( (x) != NULL)					{ FreeLibrary(x); (x) = NULL; }

#endif
