// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// clientwin32WM.h

// Win32 Window Message IDs
// All freedom used messages (PostMessage, SendMessage or variations thereof)

#ifndef __clientwin32WM_h
#define __clientwin32WM_h

// This is sent when offensive material has been found in the body of an Email, HTTP request, etc..
// WPARAM is reserved (1 means nntp proxy, 2 is from the http proxy, 3 the irc proxy and 4 the smtp proxy)
// LPARAM is reserved
// Returning LET_REQ_OUT tells the proxy to send the request out
#define WM_WARN_WORD_SCAN			WM_APP + 10

// This is sent when an outgoing mail is found to be signed by the MUA
// WPARAM is reserved (send NULL)
// LPARAM is reserved (send NULL)
// Returning LET_REQ_OUT tells the proxy to send the request out
#define WM_WARN_SIGNED_MAIL			WM_APP + 11

// This is sent when the user first checks their mail for more than one Nym
// WPARAM is reserved (send NULL)
// LPARAM is reserved (send NULL)
// Returning LET_REQ_OUT tells the proxy to send the request out
#define WM_FIRST_MAIL_CHECK 		WM_APP + 12

// This is sent when the user tries to make an NNTP connection with a nym
// that either has network access or mail rights.
// WPARAM is reserved (send NULL)
// LPARAM is reserved (send NULL)
// Returning LET_REQ_OUT tells the proxy to send the request out
#define WM_WARN_NNTP_CONNECTION 		WM_APP + 13

// This is sent when the user tries to make an SSL connection (with any nym
// or true identity selected
// WPARAM is reserved (send NULL)
// LPARAM is reserved (send NULL)
// Returning LET_REQ_OUT tells the proxy to send the request out
#define WM_WARN_SSL_CONNECTION 		WM_APP + 14

// Pass this to winsock when calling WSAAsyncGetHostByAddr.
// Used by the offending words dialog to give the user details about
// the destination of the offensive message.
// WPARAM is reserved
// LPARAM : High 16 bits contain the error code. Error code of 0
//			means success.
#define WM_WSAASYNCGETHOSTBYADDR_COMPLETE WM_APP + 15

// This is sent when the user exceeds the maximum sent mail for a day on a Nym
// WPARAM is reserved (send NULL)
// LPARAM is reserved (send NULL)
#define WM_MAIL_SENT_COUNT			WM_APP + 16

// This is sent when a client wants to update the status of the 
// Status Display section of the toolbar, eg to start an AVI during
// a long network access
#define WM_UPDATE_STATUS        WM_APP + 21

// WPARAM is the authenticated route id which died and we were unable to rebuild automatically
// LPARAM unused
#define	WM_SELECTUSER_NYM		WM_APP + 70

// Private message used by ZKFreedomWizard ZKSetupTokenDlg
// WPARAM unused
// LPARAM unused
#define WM_CALLSETUPTOKENINIT   WM_APP + 86

// Used internally by  EditClass
// WPARAM unused
// LPARAM unused
#define EDITCLASSMSG			WM_APP + 100


// Allows anyone to update the console
// WPARAM is &ConsInfo
#define FBM_ADDCONSOLE	        WM_APP + 105

// Return the console to what ever default message we
// show.  Call this when an operation is complete
// No arguments
#define FBM_DEFAULT_CONSOLE		WM_APP + 106

// Call the console without waiting.  Receipient is
// responsible to free ConsInfo object sent
#define FBM_ADDCONSOLE_NOWAIT   WM_APP + 107

// Used internally by FreedomBar
#define FBM_TASKBARCALLBACK     WM_APP + 115
#define WM_SMARTEDIT			WM_APP + 125   

// Sent when the smtp proxy has captured an email message and is
// giving feedback to the user as to what is happening to their mail.
// Allows the user a chance to cancel the mail operation.
#define WM_SMTPPROXYFEEDBACK	WM_APP + 135


// Sent when the nntp proxy has captured a news posting
// and is giving feedback to the user as to what is happening
// to their posting.  Allows the user a chance to cancel the 
// operation.
#define WM_NNTPPROXYFEEDBACK	WM_APP + 140


// Sent by the any proxy with information used to update
// the console regarding the proxy activities.
#define WM_ANYPROXY_TO_CONS		WM_APP + 145

// Communication after ping completed - sent to ZKServerCtrl
// and its parent
#define WM_PING_COMPLETE		WM_APP + 160

// Private inside ZKServerCtrl
#define WM_PING_UPDATE_UI		WM_APP + 161

// Show a particular page of the property page
#define WM_SHOWNYMPREFPAGE	    WM_APP + 162

// Messages for the wallet
#define WM_WALLET_SET_ACTIVE_NYM		WM_APP + 163
#define WM_WALLET_SELECT_LOGIN_INFO		WM_APP + 164
#define WM_WALLET_LOGIN_NORMALLY		WM_APP + 165
#define WM_WALLET_CANCEL_DIALOG			WM_APP + 166
#define WM_WALLET_REFRESH_DIALOG		WM_APP + 167
#define WM_WALLET_ONE_CLICK_FORM_FILL	WM_APP + 169
#define WM_WALLET_IS_DRAGGABLE			WM_APP + 170

// Show the dropped packet dialog
#define WM_SHOW_DROPPED_PACKET_DLG		WM_APP + 175

// When using WM_COPYDATA, use CS_xxx as the format for the dwData value of the COPYDATASTRUCT

// Message for connection_info structure
// Return value from app contains
// bits 0-15	loword, FirewallAction enumeration equivalent
// bits 16-31	hiword, Local port (host byte order) to switch the connection to
#define CS_CONNECTION_INFO				WM_APP + 176


// Message for postconnection_info structure 
#define CS_POSTCONNECTION_INFO			WM_APP + 177

// Message for postsendto_info structure
#define CS_POSTSENDTO_INFO				WM_APP + 179

// This is the message for which only the secondary
// intermediate firewall window handles which is when
// a UI pop needs to be displayed
#define SHOWFIREUI						WM_APP+200

// Messages used by ctrls in ZkCommonUI
#define WM_WEBLINK_CLICKED				WM_APP + 180
#define WM_TABCTRL_TAB_SELECTED			WM_APP + 281


// Use to tell a dialog to redraw itself
// For example with Subject informs an UI observer from
// a background thread, the observer might post this 
// message to itself to redraw.
#define WM_DATA_CHANGED_PLEASE_REDRAW	WM_APP + 290

// Used in the Upgrade code to comm with thread
#define WM_DOWNLOADUPGRADE  WM_APP + 1000
#define WM_FREEDOMUPGRADE   WM_APP + 1001
#define WM_FINISHEDUPGRADE	WM_APP + 1002
#define WM_CANCELUPGRADE    WM_APP + 1003
#define WM_FINISHEDDOWNLOAD WM_APP + 1004
#define WM_STARTINSTALL		WM_APP + 1005
#define WM_DOWNLOADINFO		WM_APP + 1006
#define WM_UPGRADE_ERROR	WM_APP + 1007

// Test Framework messages
// Launch a unit test. The unit test is specified into the first parameter
// the unit tests are defined in inc/test_commands.h
#define WM_TF_RUN_TEST WM_APP + 2000

#endif
