// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "ZkPrngClientImpl.h"
#include <clientdebug.h>
#include <zkrand.h>

ZkPrngClientImpl::ZkPrngClientImpl() 
	: m_keyboard_source_id(0)
	, m_mouse_source_id(0)
{
	ZKVERIFY( 1 == zkRandEntropyAddSource(&m_err, &m_keyboard_source_id) );

	ZKVERIFY( 1 == zkRandEntropyAddSource(&m_err, &m_mouse_source_id) );
}


int ZkPrngClientImpl::Input(
	void * in_sample, 
	const size_t in_size,
	const unsigned int bits,
	const source in_source)
{
	ZKASSERT( ZKIsValidAddress(in_sample, in_size, 0) );

	switch(in_source)
	{
	case KEYBOARD:
		return ( 1 == zkRandEntropyAddFromSource(&m_err, m_keyboard_source_id, in_sample, in_size, bits) );
		break;
	case MOUSE:
		return ( 1 == zkRandEntropyAddFromSource(&m_err, m_mouse_source_id, in_sample, in_size, bits) );
		break;
	}

	ZKASSERT(0);
	return 0;
}
