// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

//#include "stdafx.h"
#include <clientdefs.h>

#ifdef INTERNAL_BUILD

#include <unistd.h>

#include <IZkAppState.h>
#include <ZkStatus.h>

#include "ZKConfiguration.h"
#include "ZKFileConfiguration.h"
#include "IZkConsoleMsg.h"
#include "IZkFileHelp.h"

#include <ZkUnicode.h>

#include "IZkRegistry.h"

#include "ClientBaseUnitTest.h"

#include "IZkPopup.h"
#include <string_hlp.h>
#include <ZkErrorHandler.h>
#include <IZKErrorReporter.h>
#include <IZkException.h>

#include <fstream>
//#include <file_hlp.h>

// For linux test
#include <ZkSmallSharpTool.h>

#include <unistd.h>
#define TEST_MODE ,755


long ClientBaseUnitTest::Execute( void* )
{
	ZKConfigurationUnitTest();
	ZKFileConfigurationUnitTest() ;
	ZKConsoleMsgUnitTest();
	ZKFileHelpUnitTest();
	ZKFileHelpUnitTest2();
	ZKFileHelpUnitTest3();
    GetLinesExceedingWidthUnitTest();
    ExtractFileNameUnitTest();
	ZKRegistryUnitTest();
	ZKUnicodeUnitTest();

	return 1;
}

void ClientBaseUnitTest::ZKConfigurationUnitTest()
{
	ZKConfiguration::DoUnitTest();
}

void ClientBaseUnitTest::ZKFileConfigurationUnitTest()
{
	ZKFileConfiguration::DoUnitTest() ;
}

void ClientBaseUnitTest::ZKConsoleMsgUnitTest()
{
	IZkConsoleMsg::ShowMsg( IDS_CONS_UNIT_TEST, 1 );
	IZkConsoleMsg::ShowMsg( CONSIMAGE_QUESTION, IDS_CONS_UNIT_TEST, 2 );
	IZkConsoleMsg::ShowMsg( CONS_VERY_IMPORTANT, CONSIMAGE_COOKIE, IDS_CONS_UNIT_TEST, 3 );

	IZkConsoleMsg::ShowImportantMsg( IDS_CONS_UNIT_TEST, 4 );
	IZkConsoleMsg::ShowImportantMsg( CONSIMAGE_SERIOUS, IDS_CONS_UNIT_TEST, 5 );

	IZkConsoleMsg::ShowMsgForceUpdate( IDS_CONS_UNIT_TEST, 6 );
	IZkConsoleMsg::ShowMsgForceUpdate( CONSIMAGE_COMP_TERM, IDS_CONS_UNIT_TEST, 7 );
	IZkConsoleMsg::ShowMsgForceUpdate( CONS_NORMAL, CONSIMAGE_CONNECTION, IDS_CONS_UNIT_TEST, 8 );

}

void ClientBaseUnitTest::ZKFileHelpUnitTest()
{
	char strOrigFileName[MAX_PATH] = { 0 };
	char strNewFileName[MAX_PATH+1] = { 0 };
	unsigned char* pBuffer = NULL;
	size_t bufSize = 0;

    zkCStrlcpy(strOrigFileName, "/boot/vmlinuz", sizeof(strOrigFileName));
	try
	{
        if( IZkFileHelp::ReadBinaryFile(strOrigFileName, &pBuffer, bufSize) )
        {
                tmpnam(strNewFileName);
        }
        else
        {
            ZKTEST (false);
            return;
        }        
	}
	catch(IZkException& e)
	{
		ZkErrorHandler::GetIZkErrorReporter()->ShowError(e);
		SAFEFREE( pBuffer );
		return;
	}
	catch( ZKSTD::exception& )
	{
		ZKTEST( false );
		SAFEFREE( pBuffer );
		return;
	}

	try
	{
		ZKASSERT( pBuffer && bufSize > 0 );
		IZkFileHelp::WriteBinaryFile( strNewFileName, pBuffer, bufSize );
	}
	catch(IZkException& e)
	{
		ZkErrorHandler::GetIZkErrorReporter()->ShowError(e);
		SAFEFREE( pBuffer );
		return;
	}
	catch( ZKSTD::exception& )
	{
		ZKTEST( false );
		SAFEFREE( pBuffer );
		return;
	}


	unsigned char* pCopiedBuffer = NULL;
	try
	{
		size_t copiedBufSize = 0;
		
		ZKTEST( IZkFileHelp::ReadBinaryFile( strNewFileName, &pCopiedBuffer, copiedBufSize ) );

		ZKTEST( copiedBufSize == bufSize );
		
		for( unsigned int i = 0; i < copiedBufSize; i++ )
			ZKTEST( pBuffer[i] == pCopiedBuffer[i] );
	}
	catch(IZkException& e)
	{
		ZkErrorHandler::GetIZkErrorReporter()->ShowError(e);
		SAFEFREE( pBuffer );
		SAFEFREE( pCopiedBuffer );
		return;
	}
	catch( ZKSTD::exception& )
	{
		ZKTEST( false );
		SAFEFREE( pBuffer );
		SAFEFREE( pCopiedBuffer );
		return;
	}

	SAFEFREE( pBuffer );
	SAFEFREE( pCopiedBuffer );

	unlink( strNewFileName );

	// Backup and Erase functionality
	const ZKSTD::string strFileName( "/tmp/backupnerase.test" );

	ZKSTD::ofstream theFile( strFileName.c_str() );
	theFile.close();
	ZKSTD::string strBackupName( IZkFileHelp::BackupAndErase( strFileName ) );
	ZKTEST( strFileName != strBackupName );
	ZKTEST( "Unlink returned 0" && ( unlink( strBackupName.c_str() ) == 0));

	// DoesFileExist
	ZKTEST( IZkFileHelp::DoesFileOrDirExist( ZKFileConfiguration::GetFreedomExeFilePath().c_str() ) ) ;
	ZKTEST( !IZkFileHelp::DoesFileOrDirExist( "" ) ) ;

	// Create Directory

	try
	{
		// What directory is most likely to fail?

        // DO NOT RUN AS ROOT!
        IZkFileHelp::CreateDirectory( "/root/WhyDidYouRunFreedomAsRoot/" ) ; 

		ZKTEST( false ) ;
	}
	catch( IZkException& /*ex*/ )
	{
//		ZkErrorHandler::GetIZkErrorReporter()->ShowError( ex );
	}
}


// Unit test for GetTempFileName/filecopy
void ClientBaseUnitTest::ZKFileHelpUnitTest2()
{
	char file1 [L_tmpnam * 2] = { 0 };
	char file2 [L_tmpnam * 2] = { 0 };

	IZkFileHelp::GetTempFileName(file1);
	IZkFileHelp::GetTempFileName(file2);

	ZKTEST(file1);
	ZKTEST(file2);

	FILE* f1 = NULL;

	if(file1 && file2)
	{
		f1 = fopen(file1, "wb");
		FILE* f2 = fopen(file2, "w+b");
		ZKTEST(f1);
		ZKTEST(f2);
		if(f1 && f2)
		{
			ZKTEST( 0 == IZkFileHelp::FileCopy((FILE*)NULL, (FILE*)NULL) );
			
			fprintf(f2, "This is a test");

			ZKTEST( 1 == IZkFileHelp::FileCopy(f1, f2) ); // write from f2 into f1

			fclose(f2);
		}
	}

//////
//  XXX Removed ifstream ofstream FileCopy function.
//  Andrew and I agreed we want to standardize on using
//  FILE * io instead of the less robust / portable C++ lib --ol
//////
// 	char stream_out[L_tmpnam * 2] = { 0 };
// 	IZkFileHelp::GetTempFileName(stream_out);

// 	// "This is a test" will be in 'file1' only if f1 has been closed, which it has not yet
// 	ZKSTD::ifstream ifs(file1); 
// 	ZKSTD::ofstream of(stream_out);
	
// 	ZKTEST(0 == IZkFileHelp::FileCopy(of, ifs));

// 	fclose(f1);
// 	of.clear(); ifs.clear();
// 	ZKTEST(1 == IZkFileHelp::FileCopy(of, ifs));

// 	ZKSTD::ifstream ifsbad("");
// 	ZKTEST(0 == IZkFileHelp::FileCopy(of, ifsbad));

// 	ZKSTD::ofstream ofsbad("");
// 	ZKTEST(0 == IZkFileHelp::FileCopy(ofsbad, ifsbad));

// 	ZKTEST(IZkFileHelp::FileExists(file1));
// 	ZKTEST(IZkFileHelp::FileExists(file2));

// 	ZKTEST(!IZkFileHelp::FileExists(""));

// 	ifs.close();
// 	of.close();

#ifdef ZK_LINUX
    // Set umask to avoid permission problems
    umask(022);
#endif

	ZKTEST(0 == mkdir("./tests_t" TEST_MODE));

	ZKTEST(1 == IZkFileHelp::FileCopyToDir(file1, "./tests_t") );
	
	char buf[FILENAME_MAX] = { 0 };

#ifdef WIN32
	snprintf(buf, sizeof buf, "./tests_t/%s", file1);
#else
	snprintf(buf, sizeof buf, "./tests_t/%s", basename(file1));
#endif

	ZKTEST(0 == remove(buf));
	ZKTEST(0 == rmdir("./tests_t"));
}

// Unit test for GetTempFileName/ReadBinaryFile/readtestfile/FileCopyWithVerification
void ClientBaseUnitTest::ZKFileHelpUnitTest3()
{

	char stream_outbin[L_tmpnam * 2] = { 0 };
	char stream_outtext[L_tmpnam * 2] = { 0 };
	char stream_outtext2[L_tmpnam * 2] = { 0 };
	char stream_outtext3[L_tmpnam * 2] = { 0 };
	char stream_outtext4[L_tmpnam * 2] = { 0 };
	char stream_outtext5[L_tmpnam * 2] = { 0 };

	IZkFileHelp::GetTempFileName(stream_outbin);
	IZkFileHelp::GetTempFileName(stream_outtext);
	IZkFileHelp::GetTempFileName(stream_outtext2);
	IZkFileHelp::GetTempFileName(stream_outtext3);
	IZkFileHelp::GetTempFileName(stream_outtext4);
	IZkFileHelp::GetTempFileName(stream_outtext5);

	ZKSTD::string str;
	

	{ // create the binary file on disk
	ZKSTD::ofstream of(stream_outbin, ZKSTD::ios::binary);
	of << 0x01 << 0x02 << 0x03 << 0x04 << 0x05 << 0x06 << 0x07 << 0x08 << 0x09 << 0x10 << 0x11 << 0x12 << 0x13;
	}
	
	{ // read the same binary file into memory
	unsigned char* pbuf = NULL;
	size_t siz = 0;

	ZKTEST(IZkFileHelp::ReadBinaryFile(stream_outbin, &pbuf, siz));
	ZKTEST(17 == siz);
	ZKTEST(0  == memcmp("12345678916171819", pbuf, 13) );
	SAFEFREE(pbuf);
	}


	{ // create the text file on disk
	ZKSTD::ofstream of(stream_outtext);
	of << "This is a story";
	of << "Of a man named Brady";
	of << "Who should of been neutered\r\n";
	of << "This is a lady\r\n";
	ZKTEST(!IZkFileHelp::ReadTextFile(stream_outtext, str));
	}

	{ // read the same text file into memory
	str = "";
	ZKTEST(IZkFileHelp::ReadTextFile(stream_outtext, str));
	ZKTEST(str 
		== "This is a storyOf a man named BradyWho should of been neutered\r\nThis is a lady\r\n");	
	}

	{ 
	ZKTEST( !IZkFileHelp::FileCopyWithVerification(stream_outbin, stream_outtext, true) );
	ZKTEST( IZkFileHelp::FileCopyWithVerification(stream_outbin, stream_outtext, false) );
	ZKTEST(0 == remove(stream_outtext));	
	ZKTEST( IZkFileHelp::FileCopyWithVerification(stream_outbin, stream_outtext, true) );
	ZKTEST(0  == remove(stream_outbin));
	}


	{
	ZKSTD::ofstream of(stream_outtext2);
	of << "This is another story" ;	
	ZKTEST(!IZkFileHelp::ReadTextFile(stream_outtext2, str));
	}

	str = "";    
	ZKTEST(IZkFileHelp::ReadTextFile(stream_outtext2, str));
	ZKTEST(str == "This is another story\n");
	ZKTEST(0 == remove(stream_outtext2));

	//////////////////////////////////////////////////////////////////////////////////////////

	// Make sure that we won't get duplication of the eol delimiter wanted.
	{
		ZKSTD::ofstream of(stream_outtext3);
		of << "Once upon\na time\r\nat 5000ft under a big cu...\n";
		ZKTEST(!IZkFileHelp::ReadTextFile(stream_outtext3, str));
	}
	str = "";
	ZKTEST(IZkFileHelp::ReadTextFile(stream_outtext3, str, "\r\n", '\n'));
	ZKTEST(str == "Once upon\r\na time\r\nat 5000ft under a big cu...\r\n");
	ZKTEST(0 == remove(stream_outtext3));

	//////////////////////////////////////////////////////////////////////////////////////////

	{
		ZKSTD::ofstream of(stream_outtext4);
		of << "The vario\rwas crazy and the\ncloud base was at 9500ft.";
		ZKTEST(!IZkFileHelp::ReadTextFile(stream_outtext4, str));
	}
	str = "";
	ZKTEST(IZkFileHelp::ReadTextFile(stream_outtext4, str, "\r", '\n'));
	ZKTEST(str == "The vario\rwas crazy and the\rcloud base was at 9500ft.\r");
	ZKTEST(0 == remove(stream_outtext4));

	//////////////////////////////////////////////////////////////////////////////////////////

	{
		ZKSTD::ofstream of(stream_outtext5);
		of << "I determined\r\r\nmy ceiling at 8000ft\n\n\rso I thermalled the best I could.\r";
		ZKTEST(!IZkFileHelp::ReadTextFile(stream_outtext5, str));
	}
	str = "";
	ZKTEST(IZkFileHelp::ReadTextFile(stream_outtext5, str, "\r", '\n'));
	ZKTEST(str 
			== "I determined\rmy ceiling at 8000ft\r\r\rso I thermalled the best I could.\r");
	ZKTEST(0 == remove(stream_outtext5));

}

// Unit test for GetLinesExceedingWidth
void ClientBaseUnitTest::GetLinesExceedingWidthUnitTest()
{
	ZKSTD::map<size_t, ZKSTD::string> mymap;
	const size_t nBadFile = IZkFileHelp::GetLinesExceedingWidth("resource.h", 50, mymap);
	if(nBadFile)
	{
		// I've removed the htmltag helper class and code to show
		// results because it wasn't used enough to warrent its usage.  Generally
		// unit test should be silent, not spam people who run it,
		// just assert is some expected value doesn't happen. -Andrew


		for(ZKSTD::map<size_t, ZKSTD::string>::const_iterator it = mymap.begin(); 
			it != mymap.end(); 
			++it)
		{
			ZKTRACE2( "Found line too long %d %s\n", (*it).first, (*it).second.c_str() );
		}
	}
}

// Unit test for ExtractFileName
void ClientBaseUnitTest::ExtractFileNameUnitTest()
{
	// Let's build a vector of dummy files with random path (absolute/relative/none)
	ZKSTD::vector<ZKSTD::string> fileNameVector;
	fileNameVector.push_back( "/var/log/messages" );
	fileNameVector.push_back( "ld.so.conf" );
	fileNameVector.push_back( "foo/bar/baz/quz/quux/../quux/quuux" );
	fileNameVector.push_back( "file" );

	// Let's extract the file name.
	ZKSTD::vector<ZKSTD::string> fileNameOnlyVector;
	IZkFileHelp::ExtractFileName( fileNameVector, fileNameOnlyVector );

	// Now, make sure it works.
	ZKTEST( fileNameVector.size() == fileNameOnlyVector.size() );
	for( ZKSTD::vector<ZKSTD::string>::const_iterator it = fileNameOnlyVector.begin();
			it != fileNameOnlyVector.end();
			++it )
				ZKTEST( ( (*it).find( ":" ) ==  ZKSTD::string::npos )
							&& ( (*it).find( "/" ) ==  ZKSTD::string::npos ) );
}


void ClientBaseUnitTest::ZKRegistryUnitTest()
{
	const char strUnitTestKey[] = { "ZKRegistryUnitTest" };
	
	ZKTEST( !IZkRegistry::GetFlag( strUnitTestKey ) );
	IZkRegistry::SetFlag( strUnitTestKey, true );
	ZKTEST( IZkRegistry::GetFlag( strUnitTestKey ) );
	IZkRegistry::DeleteValue( strUnitTestKey );

	unsigned long dwDummyVal = 42, dwRetDummyVal = 0;
	ZKTEST( !IZkRegistry::GetDWORD( strUnitTestKey, dwRetDummyVal ) );
	IZkRegistry::SetDWORD( strUnitTestKey, dwDummyVal );
	ZKTEST( IZkRegistry::GetDWORD( strUnitTestKey, dwRetDummyVal ) );
	ZKTEST( dwDummyVal == dwRetDummyVal );
	IZkRegistry::DeleteValue( strUnitTestKey );

	ZKSTD::vector<uint16_t> inputVector;
	ZKSTD::vector<uint16_t> outputVector;
	for( unsigned int i = 0; i < 5; ++i )
		inputVector.push_back( i );
	IZkRegistry::GetShortArray( strUnitTestKey, outputVector );
	ZKTEST( outputVector.empty() );
	IZkRegistry::SetShortArray( strUnitTestKey, inputVector );
	IZkRegistry::GetShortArray( strUnitTestKey, outputVector );
	ZKTEST( inputVector.size() == outputVector.size() );
	for( i = 0; i < outputVector.size(); ++i )
		ZKTEST( inputVector[i] == outputVector[i] );
	IZkRegistry::DeleteValue( strUnitTestKey );
	
	const char strInput[] = { "MyUnitTest" };
	ZKSTD::string strOutput;
	ZKTEST( !IZkRegistry::GetString( strUnitTestKey, strOutput ) );
	ZKTEST( strOutput.empty() );
	IZkRegistry::SetString( strUnitTestKey, strInput );
	ZKTEST( IZkRegistry::GetString( strUnitTestKey, strOutput ) );
	ZKTEST( strInput == strOutput );
	IZkRegistry::DeleteValue( strUnitTestKey );
}


#define CP_ACP 0
#define CP_OEMCP 1
void ClientBaseUnitTest::ZKUnicodeUnitTest()
{
	/*
	// Useful for looking at the symbols on current code page
	for ( unsigned char i = 127 ; i < 255 ; i++ )
	{
		TRACE2( "Code Value - %d - %c\n", i, i ) ;
	}
	*/
		
	uint32_t existingPage = ZkUnicode::GetWindowsCodePage() ;

	// Doesn't really convert the code page of the machine - just
	// makes the conversion function think the machine is set
	// to a particular code page
	ZkUnicode::SetWindowsCodePage( CP_ACP ) ;

	// Simple string with accented e acute as represented on Windows and Mac
	const char WindowsLatinEAcute[] = "\xE9" ;
	const char MacLatinEAcute[] = "\x8E" ;
	const char MSDocEAcute[] = "\x82" ;

	uint16_t * pUnicodeVersion = NULL;
	size_t     cntUnicodeVersion = 0 ;  // Size in Bytes

	ZkUnicode::StringToUnicodeBuffer( ZKSTD::string( WindowsLatinEAcute ), &pUnicodeVersion, cntUnicodeVersion ) ;

	ZKASSERT( pUnicodeVersion != NULL ) ;
	ZKASSERT( cntUnicodeVersion > 0 ) ;

#warning "No code page support on Linux"

// 	if(ZKBaseUI::RunningUnderNt())
// 	{
// 		// Now pretend we are on a Mac - we should get a different value for e acute
// 		// Unfortunately only win2K has this code page, not 9X
// 		ZkUnicode::SetWindowsCodePage( CP_MACCP ) ;

// 		ZKSTD::string strMacRepresentation ;
// 		ZkUnicode::UnicodeBufferToString( pUnicodeVersion, cntUnicodeVersion, strMacRepresentation ) ;

// 		ZKTEST( strMacRepresentation[ 0 ] == MacLatinEAcute[ 0 ] ) ;
// 	}

	// Now pretend we are using old MSDOS character set
//	ZkUnicode::SetWindowsCodePage( CP_OEMCP ) ;

	ZKSTD::string strMsdosRepresentation ;
	ZkUnicode::UnicodeBufferToString( pUnicodeVersion, cntUnicodeVersion, strMsdosRepresentation ) ;

//	ZKTEST( strMsdosRepresentation[ 0 ] == MSDocEAcute[ 0 ] ) ;

	
	// Restore state
//	ZkUnicode::SetWindowsCodePage( existingPage ) ;

	// Free Unicode String
	free( pUnicodeVersion ) ;
}	

//--------------------------------------------------------------------------------------------
//
//ClientBaseUnitTestInteraction g_clientBaseUnitTestInteraction;

long ClientBaseUnitTestInteraction::Execute( void* )
{
	ZKBaseUIUnitTestUserInter();
	ZKFileHelpUnitTestUserInter();

	return 1;
}

void ClientBaseUnitTestInteraction::ZKBaseUIUnitTestUserInter()
{
	// Test all the message boxes.

	IZkPopup::MessageBox( tstring("Some message boxes will popup."
									"Please discard them all with OK.") );
    
    IZkPopup::MessageCheckBox( tstring("This is a message check box"));
    IZkPopup::MessageCheckBox( tstring("This is a message check box with the check box ticked"),
                               IDS_DONT_SHOW_AGAIN_DEFAULT,
                               true);

	IZkPopup::MessageBox( IDS_UNIT_TEST_STRING );
	IZkPopup::MessageBox( tstring("ZK_MESSAGE"), ZK_MESSAGE );
	IZkPopup::MessageBox( tstring("ZK_WARNING"), ZK_WARNING );
	IZkPopup::MessageBox( tstring("ZK_ERROR"), ZK_ERROR );

	IZkPopup::QuestionBox( IDS_UNIT_TEST_STRING, ZK_OKCANCEL );
	IZkPopup::QuestionBox( IDS_UNIT_TEST_STRING, ZK_YESNO );
	IZkPopup::QuestionBox( tstring("ZK_MESSAGE"), ZK_YESNO,  ZK_MESSAGE);
	IZkPopup::QuestionBox( tstring("ZK_ERROR"), ZK_YESNO,  ZK_ERROR);
	IZkPopup::QuestionBox( tstring("ZK_QUESTION"), ZK_YESNO,  ZK_QUESTION);
	IZkPopup::QuestionBox( tstring("ZK_WARNING"), ZK_YESNO,  ZK_WARNING);
	IZkPopup::QuestionBox( tstring("ZK_INFORMATION"), ZK_YESNO,  ZK_INFORMATION);
	IZkPopup::QuestionBox( tstring("default button should be NO"), ZK_YESNO,  ZK_INFORMATION, false);
	IZkPopup::QuestionBox( tstring("default button should be CANCEL"), ZK_OKCANCEL,  ZK_INFORMATION, false);
}

void ClientBaseUnitTestInteraction::ZKFileHelpUnitTestUserInter()
{
	ZKSTD::string strFolderPicked;
	ZKTEST( IZkFileHelp::FolderPicker( "Please choose a folder before clicking OK." ,
										strFolderPicked ) );
}


 long DiodeTestUnitTest::Execute( void* )
 {
 	{ 
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, 
											CONSIMAGE_QUESTION, 
											IDS_CONS_JUST_STRING, 
											"(Not) Testing Network Busy Indication" );
//		ZKBusyAVI Busy ;
		sleep( 5 ) ;
	}
	{
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, 
											CONSIMAGE_QUESTION, 
											IDS_CONS_JUST_STRING, 
											"Testing Mail Decrypt/encrypt Indication" );
		ZkStatus Busy2( ZK_STATUS_CRYPTO_BUSY ) ;
		sleep( 5 ) ;
	}

	{
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, 
											CONSIMAGE_QUESTION, 
											IDS_CONS_JUST_STRING, 
											"Testing Mail Connection Indication" );
		ZkStatus Busy2( ZK_STATUS_MAIL_CONNECTION ) ;
		sleep( 5 ) ;
	}

	{
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, 
											CONSIMAGE_QUESTION, 
											IDS_CONS_JUST_STRING, 
											"Testing Web Connection Indication" ) ;
		ZkStatus Busy2( ZK_STATUS_WEB_CONNECTION ) ;
		sleep( 5 ) ;
	}

	{
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, 
											CONSIMAGE_QUESTION, 
											IDS_CONS_JUST_STRING, 
											"Testing SSL Connection Indication" );
		ZkStatus Busy2( ZK_STATUS_SSL_CONNECTION ) ;
		sleep( 5 ) ;
	}

	{
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, 
											CONSIMAGE_QUESTION, 
											IDS_CONS_JUST_STRING, 
											"Testing NEWS Connection Indication" );
		ZkStatus Busy2( ZK_STATUS_NEWS_CONNECTION ) ;
		sleep( 5 ) ;
	}

	{
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, 
											CONSIMAGE_QUESTION, 
											IDS_CONS_JUST_STRING, 
											"Testing IRC Connection Indication" );
		ZkStatus Busy2( ZK_STATUS_IRC_CONNECTION ) ;
		sleep( 5 ) ;
	}

	{
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, 
											CONSIMAGE_QUESTION, 
											IDS_CONS_JUST_STRING, 
											"Testing Telnet Indication" );
		ZkStatus Busy2( ZK_STATUS_TELNET_CONNECTION ) ;
		sleep( 5 ) ;
	}
	
	return 1;
}


//===========================================================================
// CLASS ClientBaseLinuxUnitTest
//===========================================================================
int ClientBaseLinuxUnitTest::TestSmallSharpTool( )
{
    int infd, outfd;
    char c;
    size_t count;
    char *wbuf = "Failure.\n";

    ZKSTD::vector<ZKSTD::string> args;

    args.push_back("-e");
    args.push_back("s/Failure/SUCCESS/g");
//    args.push_back("127.0.0.1");

    ZkSmallSharpTool ping("/bin/sed");
    ping.Execute(args, IO_INPUT | IO_OUTPUT);
    infd = ping.GetInputFD();
    outfd = ping.GetOutputFD();

    write(outfd, wbuf, strlen(wbuf));
    close(outfd);
    while ((count = read(infd, &c, 1)) > 0) write(STDOUT_FILENO, &c, count);
    close(infd);
    infd = ping.Wait();
    printf("Return Value: %d\n",infd);
    fflush(stdout);
    return infd;

}

//---------------------------------------------------------------------------
//
int ClientBaseLinuxUnitTest::TestSmallSharpToolTimeout( )
{
    int infd;
    char c;
    size_t count;
    char *wbuf = "Failure.\n";

    ZKSTD::vector<ZKSTD::string> args;

    args.push_back("-c");
    args.push_back("50");
    args.push_back("www.microsoft.com");

    ZkSmallSharpTool ping("/bin/ping");
    ping.Execute(args, IO_INPUT , 4000);
    infd = ping.GetInputFD();


    while ((count = read(infd, &c, 1)) > 0) write(STDOUT_FILENO, &c, count);
    close(infd);
    infd = ping.Wait();
    printf("Return Value: %d\n",infd);
    fflush(stdout);
    return infd;

}

//---------------------------------------------------------------------------
//

long ClientBaseLinuxUnitTest::Execute( void* in_pParam )
{
    (void)in_pParam; //no warn
    int rval;

    rval = TestSmallSharpTool();
    rval &= TestSmallSharpToolTimeout();
    return !rval;
}
#endif
