// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// ClientBaseUnitTest.h

#ifndef _ClientBaseUnitTest_H
#define _ClientBaseUnitTest_H

#ifdef INTERNAL_BUILD

#include "../../libzktest/ZkTestBase.h"

class ClientBaseUnitTest : public ZkTestBase
{

public:
	ClientBaseUnitTest()
		: ZkTestBase( "zkbase/clientbase/ClientBaseUnitTest", 
						"ClientBase unit tests that do not need user interaction" )
	{}

	// PURPOSE: Run the unit test for the ClientBase project without
	//			user interaction.
	// REQUIRE: Nothing.
	// PROMISE: ASSERT in debug and CrtDbgBreak in release.
	virtual long Execute( void* );

private:
	// REQUIRE: Nothing
	// PROMISE: Call a static member function in ZKConfiguration that will
	//			do the unit tests.
	//			Will assert if something goes wrong.
	static void ZKConfigurationUnitTest();

	// PURPOSE: Test ZKFileConfiguration.
	// PROMISE: Call a static member function in ZKFileConfiguration that will
	//			do the unit tests.
	//			Will assert if something goes wrong.

	static void ZKFileConfigurationUnitTest() ;

	// PURPOSE: Test the ZKConsole.
	// REQUIRE: The testing person should take a look at the output in the console.
	// PROMISE: Call each function at least once.
	//			Will not call the private member functions since they are called
	//			from the public ones.
	static void ZKConsoleMsgUnitTest();

	// PURPOSE: Test the file helpers that doesn't require user
	//			interaction.
	// REQUIRE: Nothing
	// PROMISE: Same as other tests.
	static void ZKFileHelpUnitTest();
	static void ZKFileHelpUnitTest2();
	static void ZKFileHelpUnitTest3();

	// PURPOSE: Test the misc file functions in IZkFileHelp helpers.
	// REQUIRE: Nothing
	// PROMISE: Same
    void GetLinesExceedingWidthUnitTest();
    void ExtractFileNameUnitTest();


	// PURPOSE: Test the ZKRegistry helpers.
	// REQUIRE: Nothing
	// PROMISE: To leave the registry as it was before except if errors occur.
	static void ZKRegistryUnitTest();

	
	// PURPOSE: Test the functionnality of ZKUnicodeUnitTest.
	// REQUIRE: Nothing
	// PROMISE: Will leave active code page settings etc as before
	static void ZKUnicodeUnitTest() ;
};

class ClientBaseUnitTestInteraction : public ZkTestBase
{
public:
	ClientBaseUnitTestInteraction()
		: ZkTestBase( "zkbase/clientbase/ClientBaseUnitTestInteraction", 
						"ClientBase unit tests that need user interaction", 
						ZkTestBase::INTERACTION )
	{}
	
	~ClientBaseUnitTestInteraction()
	{}

	// PURPOSE: Run the unit test for the ClientBase project with user
	//			interaction.
	// REQUIRE: Be called by the test agent.
	// PROMISE: Will need the user interaction.
	virtual long Execute( void* );

private:
	// PURPOSE: Test functionnality related to UI that requires user interaction.
	// REQUIRE: Be called by the test Agent.
	// PROMISE: To pop up some dialog boxes.
	static void ZKBaseUIUnitTestUserInter();

	// PURPOSE: Test the file helpers that require user interaction.
	// REQUIRE: Be called by the test agent.
	// PROMISE: Nothing
	static void ZKFileHelpUnitTestUserInter();
};

class DiodeTestUnitTest : public ZkTestBase
{
public:
 	DiodeTestUnitTest()
 		: ZkTestBase( "zkbase/DiodeTestUnitTest",
                      "Unit test for the different diodes" )
 	{}
    
 	~DiodeTestUnitTest()
 	{}
    
 	virtual long Execute( void* );
};


// This is intended to hold test functions for
// any purely linux specific classes.
class ClientBaseLinuxUnitTest 
    : public ZkTestBase 
{

public: 
    ClientBaseLinuxUnitTest()
        :ZkTestBase( "zkbase/clientbase/linux/ClientBaseLinuxUnitTest", "Client Base Linux Unit Test" )
        {};

    ~ClientBaseLinuxUnitTest() {};

    virtual long Execute( void * );

private: 

    static int TestSmallSharpTool();
    static int TestSmallSharpToolTimeout();
};

#endif // INTERNAL_BUILD

#endif
