// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include <client.h>	
#include <clientdebug.h>
#include "RZkFileHelp.h"
#include <sys/stat.h>
#include <unistd.h>
#include <../linux/RZkException.h>

int
RZkFileHelp::onFolderPicker( const char* in_pszTitle, ZKSTD::string& out_strPickedFile )
{
    ZKTRACE0("RZkFileHelp::onFolderPicker is not implemented on linux.\n");
    ZKASSERT(0);
    return 0;
}

bool
RZkFileHelp::onIsNetWorkDrive(const ZKSTD::string& in_strPath) 
{
    ZKTRACE0("RZkFileHelp::onIsNetworkDrive is not implemented on linux.\n");
    ZKASSERT(0);
    return false;
}

bool
RZkFileHelp::onDoesFileOrDirExist( const char* in_pszFile ) throw(RZkException) 
{
	ZKFunctionEntry( "ZKFileHelp::onDoesFileOrDirExist" ) ;
	ZKTestPointer( in_pszFile ) ;

    struct stat st = {0};

    return !stat( in_pszFile, &st );
}

void
RZkFileHelp::onCreateDirectory( const char* in_pszDirectory ) throw(RZkException ) 
{
	ZKFunctionEntry( "ZKFileHelp::CreateDirectory" ) ;
	ZKTestPointer( in_pszDirectory ) ;

	if ( !DoesFileOrDirExist( in_pszDirectory ) )
	{
		if (mkdir( in_pszDirectory, 0755 ) )
		{
			throw RZkException(
				"ZKFileHelp::CreateDirectory", 
				"Failed to create directory", 
				IDS_FAILED_CREATE_DIR, 
				in_pszDirectory);
		}
	}
}

ZKSTD::string
RZkFileHelp::onBackupAndErase( const ZKSTD::string& in_rstrFileName ) 
{
	char szPath[ MAX_PATH ];
	
	if( onDoesFileOrDirExist( in_rstrFileName.c_str() ) )
	{
		// this is platform specific ?
		struct tm* t;  
		t = localtime ( NULL );

		if( t != NULL )
		{
			snprintf(szPath, sizeof(szPath), "%s.%d.%02d.%02d.%d.%02d.%02d.Backup", in_rstrFileName.data(), t->tm_year, t->tm_mon, t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec ) ;
		}
		else
		{
			// if the system has no time available, we put a random number
			// randomize timer
            ZKTRACE0("RZkFileHelp: Using Random Backup filename.\n");
			srand( (unsigned)time( NULL ) );
			snprintf(szPath,sizeof(szPath),"%s.%d.Backup", in_rstrFileName.data(), rand()*rand() ) ;
		}

		if(rename( in_rstrFileName.data(), szPath)) {
        }
	}
		
	return szPath;
}
