// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):


//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <RZkPopup.h>
#include <ZkCatalogRegistry.h>
#include <clientdebug.h>


//===========================================================================
// CLASS RZkPopup
//===========================================================================

//---------------------------------------------------------------------------
//
RZkPopup::RZkPopup( ZKSTD::auto_ptr< ZkPopupHandler > handler )
    : m_handler( handler )
{
    ZKASSERT( m_handler.get() != 0 );
}

//---------------------------------------------------------------------------
//
RZkPopup::~RZkPopup( )
{
}


//---------------------------------------------------------------------------
//
void RZkPopup::openMessageBox( uint32_t msgId, IconType iconType, PopupPosition, uint32_t titleId )
{
    // FIXME - For now it will compile as long as we do not use unicode defines.
    m_handler->openMessageBox( ZkCatalogRegistry::GetString( msgId ), 
                               ZkCatalogRegistry::GetString( titleId ), 
                               iconType );
}

//---------------------------------------------------------------------------
//
void RZkPopup::openMessageBox( const tstring& msg, IconType iconType, PopupPosition, const tstring& title )
{
    m_handler->openMessageBox( msg, title, iconType );
}

//---------------------------------------------------------------------------
//
bool RZkPopup::openQuestionBox( uint32_t msgId, BtnType btnType, PopupPosition )
{
    // FIXME - For now it will compile as long as we do not use unicode defines.
    return m_handler->openQuestionBox( ZkCatalogRegistry::GetString( msgId ), 
                                       btnType,
                                       ZK_QUESTION,
                                       0 );
}

//---------------------------------------------------------------------------
//
bool RZkPopup::openQuestionBox( 
    const tstring& msg, 
    BtnType btnType, 
    IconType iconType, 
    bool defBtn,
	PopupPosition )
{
    return m_handler->openQuestionBox( msg, btnType, iconType, defBtn );
}

//---------------------------------------------------------------------------
//
bool RZkPopup::openMessageCheckBox(	
    uint32_t  msgId, 
    uint32_t  in_checkMsgId,
    bool	  in_bDefaultChecked, 
    IconType  icon,
	PopupPosition )    
{
    return m_handler->openMessageCheckBox( 
        ZkCatalogRegistry::GetString( msgId ), 
        ZkCatalogRegistry::GetString( in_checkMsgId ), 
        in_bDefaultChecked,
        icon ); 
}

//---------------------------------------------------------------------------
//
bool RZkPopup::openMessageCheckBox(	
    const tstring& str,
    uint32_t  in_checkMsgId,
    bool	  in_bDefaultChecked, 
    IconType  icon,
	PopupPosition )
{
    return m_handler->openMessageCheckBox( 
        str, 
        ZkCatalogRegistry::GetString( in_checkMsgId ), 
        in_bDefaultChecked,
        icon ); 
}


//---------------------------------------------------------------------------
//
bool RZkPopup::openQuestionCheckBox(	
	uint32_t in_questionId,
	bool& out_bChecked,
	uint32_t in_btn1textId,
	uint32_t in_btn2textId,
	uint32_t in_checkMsgId,
	bool in_bDefaultChecked, 
	IconType in_icon,
	PopupPosition /*in_popupPos*/ )
{
    return m_handler->openQuestionCheckBox( ZkCatalogRegistry::GetString( in_questionId ),
                                            out_bChecked,
                                            in_btn1textId,
                                            in_btn2textId,
                                            in_checkMsgId,
                                            in_bDefaultChecked,
                                            in_icon );
}

//---------------------------------------------------------------------------
//
bool RZkPopup::openQuestionCheckBox(
	const tstring& in_strQuestion,
	bool& out_bChecked,
	uint32_t in_btn1textId,
	uint32_t in_btn2textId,
	uint32_t in_checkMsgId,
	bool in_bDefaultChecked, 
	IconType in_icon,
	PopupPosition /*in_popupPos*/ )
{
    return m_handler->openQuestionCheckBox( in_strQuestion,
                                            out_bChecked,
                                            in_btn1textId,
                                            in_btn2textId,
                                            in_checkMsgId,
                                            in_bDefaultChecked,
                                            in_icon );

}



//---------------------------------------------------------------------------
//
tstring RZkPopup::openRequestStringBox(	const tstring& /*in_strTitle*/,
										const tstring& /*in_strPrompt*/,
										const tstring& /*in_strInitialValue*/,
                                        int /*in_nMinLength*/,
										PopupPosition)
{
	ZKASSERT(false && "Not implemented on Linux yet");

	return "";
}
