// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include "ZkCatalog.h"
#include "ZkCatalogRegistry.h"

#include <clientdebug.h>

//===========================================================================
// CLASS ZkCatalog
//===========================================================================

//---------------------------------------------------------------------------
//
ZkCatalog::ZkCatalog( 
    int index, 
    int size, 
    const char** table 
)
    : m_index( index )
    , m_size( size )
    , m_table( table )
{
    ZKASSERT( m_size != 0 );

    ZkCatalogRegistry::RegisterCatalog( this );
}

//---------------------------------------------------------------------------
//
ZkCatalog::~ZkCatalog( )
{
}

//---------------------------------------------------------------------------
//
const char* ZkCatalog::GetString( uint32_t id ) const
{
    // check the string is actually in the table scope.
    if( id > GetSize() )
    {
        ZKTRACE3( "In catalog %d, string id %d is out of range (0, %d).\n",
                  GetIndex(), id, GetSize() );
        ZKASSERT( false );
        return "String ID out of range";
    }

    return m_table[ id ];
}



