/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_ZKCATALOG
#define ZKS_INCL_ZKCATALOG

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include <client.h>

/*===========================================================================
 * CLASS ZkCatalog
 *=========================================================================*/

/*

  This class simply defines the attributes of a string catalog. 
  At the moment, these attributes are:

    - catalog index
    - catalog size
    - catalog static string table.

  The catalog constructor takes care of the registration with the catalog 
  registry. (which introduces a nice cyclic dependency, but who cares really...)

*/

class ZkCatalog {

public: 

    /*----- member functions -----*/

    ZkCatalog( int index, int size, const char** table );
    virtual ~ZkCatalog();

    const char*  GetString( uint32_t id ) const;
    int          GetIndex() const;
    unsigned int GetSize() const;

protected: 

    /*----- member functions -----*/

	// Prohibited funcs - not implemented.
	ZkCatalog( const ZkCatalog& );
	const ZkCatalog& operator=( const ZkCatalog& );


private: 

    /*----- data members -----*/

    int          m_index;
    unsigned int m_size;
    const char** m_table;

};

//---------------------------------------------------------------------------
//
inline int ZkCatalog::GetIndex() const
{
    return m_index;
}

//---------------------------------------------------------------------------
//
inline unsigned int ZkCatalog::GetSize() const
{
    return m_size;
}

#endif
