// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================


#include "ZkCatalogRegistry.h"
#include "ZkCatalog.h"

#include <clientdebug.h>

//===========================================================================
// CLASS ZkCatalogRegistry
//===========================================================================

ZkCatalogRegistry::CatalogTable ZkCatalogRegistry::m_catalogs;

//---------------------------------------------------------------------------
//
void ZkCatalogRegistry::RegisterCatalog( ZkCatalog* catalog )
{
    ZKASSERT( catalog != 0 );

    // make sure the catalog hasn't been registered yet.
    CatalogTable::iterator iter;
    iter = m_catalogs.find( catalog->GetIndex() );

    if( iter != m_catalogs.end() ) {
        ZKTRACE1( "String catalog %d is already registered.\n", 
                  catalog->GetIndex() );
        ZKASSERT( false );        
    }
    
    m_catalogs[ catalog->GetIndex() ] = catalog;
}

//---------------------------------------------------------------------------
//
const char* ZkCatalogRegistry::GetString( long id )
{
    unsigned int iCat = GetCatalogIndex( id );
    unsigned int iStr = GetStringIndex( id );

// FIXME - legitimate test, but enable when everything is a bit more stable.
//    ZKTRACE1( "Catalog : %d is not registered.\n", iCat );  
//    ZKASSERT( iCat < m_catalogs.size() );

    CatalogTable::iterator iter;
    iter = m_catalogs.find( iCat );

    if( iter != m_catalogs.end() ) 
	{
	    return (*iter).second->GetString( iStr );
	}
	else
	{
		ZKTRACE1( "Catalog : %d is not registered.\n", iCat );  
		ZKASSERT( false ) ;

		return "Wrong catalog number." ;
	}
}




