/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_ZKCATALOGREGISTRY
#define ZKS_INCL_ZKCATALOGREGISTRY

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include "ZkCatalog.h" // so it can be friends with it.

#include <map>
#include <clientstl.h>


// Those two values go together.
#define CAT_SHIFT 10
#define CAT_MASK  0x03FF


/*===========================================================================
 * CLASS ZkCatalogRegistry
 *=========================================================================*/

class ZkCatalogRegistry {
    friend class ZkCatalog;

public: 

    /*----- types and enumerations ----*/

    typedef long Id;
    typedef ZKSTD::map< int, ZkCatalog* > CatalogTable;

    /*----- static member functions -----*/

    static const char*  GetString( long in_stringId );

    static unsigned int GetCatalogIndex( long in_stringId );
    static unsigned int GetStringIndex( long in_stringId );

protected:

    /*----- static member functions -----*/

    // Only ZkCatalogs themselves can register with this class.
    static void RegisterCatalog( ZkCatalog* catalog );


private: 

    /*----- data members -----*/

    static CatalogTable m_catalogs;

};


//---------------------------------------------------------------------------
//
inline unsigned int ZkCatalogRegistry::GetCatalogIndex( long id )
{
    return static_cast< short >( id >> CAT_SHIFT );
}

//---------------------------------------------------------------------------
//
inline unsigned int ZkCatalogRegistry::GetStringIndex( long id )
{
    return static_cast< short >( id & CAT_MASK );
}


#endif
