// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <ZkPipe.h>
#include <client.h>

#include <unistd.h>
#include <errno.h>
#include <stdio.h>


//===========================================================================
// CLASS ZkPipe
//===========================================================================

//---------------------------------------------------------------------------
//
ZkPipe::ZkPipe( )
{
    // Create new set of piped FDs
    ZKVERIFY( !pipe(m_fdset) );
    m_state = BIDIR_PIPE;
}

//---------------------------------------------------------------------------
//
ZkPipe::~ZkPipe( )
{
    Close();
}

//---------------------------------------------------------------------------
//
void ZkPipe::MakeReadOnly( )
{
    ZKASSERT( m_state == BIDIR_PIPE );
    // Clsoe write fd
    if ( m_fdset[1] >= 0 ) {
        ZKVERIFY( !close(m_fdset[1]) );
        m_fdset[1] = -1;
    }
    m_state = READ_PIPE;
    
}

//---------------------------------------------------------------------------
//
void ZkPipe::MakeWriteOnly( )
{
    ZKASSERT( m_state == BIDIR_PIPE );
    // Clsoe read fd
    if ( m_fdset[0] >= 0 ) {
        ZKVERIFY( !close( m_fdset[0] ) );
        m_fdset[0] = -1;
    }
    m_state = WRITE_PIPE;
}

//---------------------------------------------------------------------------
//
void ZkPipe::Close( )
{
    if (m_state == CLOSED_PIPE) return;
    // No checks. Silent error if one is already closed
    if (m_fdset[0] >= 0) {
        close(m_fdset[0]);
        m_fdset[0] = -1;
    }
    if (m_fdset[1] >= 0) {
        close(m_fdset[1]);
        m_fdset[1] = -1;
    }
    m_state = CLOSED_PIPE;
}

//---------------------------------------------------------------------------
//
void ZkPipe::RedirectRead( int in_fd, bool close_local /* = false */)
{
    ZKASSERT( m_state == READ_PIPE || m_state == BIDIR_PIPE );
    ZKVERIFY( dup2(GetRead(), in_fd) >= 0);
    if ( close_local && m_fdset[0] >= 0 ) {
        ZKVERIFY( !close( m_fdset[0] ) );
        m_fdset[0] = -1;

        if (m_state == READ_PIPE) m_state = CLOSED_PIPE;
        else if (m_state == BIDIR_PIPE) m_state = WRITE_PIPE;
    }
}

//---------------------------------------------------------------------------
//
void ZkPipe::RedirectWrite( int in_fd, bool close_local /* = false */)
{
    ZKASSERT( m_state == WRITE_PIPE || m_state == BIDIR_PIPE );
    ZKVERIFY( dup2( GetWrite(), in_fd) >= 0);
    if ( close_local && m_fdset[1] >= 0 ) {
        ZKVERIFY( !close( m_fdset[1] ) );
        m_fdset[1] = -1;

        if (m_state == WRITE_PIPE) m_state = CLOSED_PIPE;
        else if (m_state == BIDIR_PIPE) m_state = READ_PIPE;
    }
}
