/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_ZKSMALLSHARPTOOL
#define ZKS_INCL_ZKSMALLSHARPTOOL

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/
#include <../linux/ZkPipe.h>
#include <zkThread.h>
#include <client.h>
//#include <clientstl.h>
#include <../linux/RZkException.h>

#include <vector>

/*===========================================================================
 * CLASS ZkSmallSharpTool
 *=========================================================================*/

/*
 * Helper class for running "Small Sharp Tools"; other executables that communicate with
 * us via stdin/stdout. Ping will be implemented this way for example.
 * --ol <oliver@zks.net>
 */

#define IO_INPUT 1
#define IO_OUTPUT (IO_INPUT << 1)

class ZkSmallSharpTool AUTO_INSTANCE_COUNTING1( ZkSmallSharpTool ) {
public: 
    /*----- member functions -----*/
      
    ZkSmallSharpTool( char *in_command );
    
    virtual ~ZkSmallSharpTool();

    virtual void Execute( ZKSTD::vector<ZKSTD::string> &argv, unsigned int in_io_flags = 0,
                          unsigned long in_timeout = ZK_INFINITE) throw ( RZkException );

    virtual int Wait() throw ( RZkException );

    virtual void Terminate() throw ( RZkException );

    int GetInputFD() 
        { return (m_State == RUNNING && m_InputPipe) ? m_InputPipe->GetRead() : -1; };

    int GetOutputFD() 
        { return (m_State == RUNNING && m_InputPipe) ? m_OutputPipe->GetWrite() : -1; };

    pid_t GetPID() 
        { return (m_State == RUNNING)? m_ChildPID : 0; };

    ZKSTD::string GetCommand() const
        { return m_Command; }

    void SetCommand( const ZKSTD::string &command ) 
        { m_Command = command; }

    void SetTimer();

    void KillTimer();

protected: 

    /*----- static member functions -----*/

    static void ZK_ITIMER_CALLBACK timerProc( void *, unsigned int, 
                                              unsigned int, unsigned long millis );
    /*----- member functions -----*/

    virtual void ChildFunction(char **argv) throw ( RZkException );

    /*----- static data members -----*/

    static ZkMutex m_Mutex;
    static ZkITimer m_Timer;
    static ZkSmallSharpTool *m_ActiveChild;

    /*----- data members -----*/
    enum {
        RUNNING,
        STOPPED        
    };

    ZkPipe *m_InputPipe;
    ZkPipe *m_OutputPipe;
    pid_t m_ChildPID;
    unsigned long m_Timeout;
    ZKSTD::string m_Command;
    int m_State;

 private:
	// Prohibited funcs - not implemented.
	ZkSmallSharpTool( const ZkSmallSharpTool& );
	const ZkSmallSharpTool& operator= ( const ZkSmallSharpTool& );

};

#endif
