// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <ZkStringMgr.h>
#include <ZkCatalogRegistry.h>

#include <glib.h>
#include <cstdarg>

//===========================================================================
// CLASS ZkStringMgr
//===========================================================================

//---------------------------------------------------------------------------
//
void ZkStringMgr::Load(         
    ZKSTD::string& out_str, 
    uint32_t id, 
    ... 
)
{
	va_list argList;
	va_start( argList, id );

	Load( out_str, id, argList ) ;
}

//---------------------------------------------------------------------------
//

void ZkStringMgr::Load( 
        ZKSTD::string& out_str, 
        uint32_t id, 
        va_list args
    )
{
    // First make sure we empty the given string, so we don't return garbage if
    // something goes wrong.
    out_str = "";

    char* result = g_strdup_vprintf( ZkCatalogRegistry::GetString( id ), args );

    out_str = result;
    g_free( result );
}

//---------------------------------------------------------------------------
//
void ZkStringMgr::Load( 
    char* out_str, 
    int len, 
    uint32_t id, 
    ... 
)
{
    ZKASSERT( out_str != 0 );
    ZKASSERT( len > 1 );

	va_list argList;
	va_start( argList, id );

    // First make sure we empty the given string, so we don't return garbage if
    // something goes wrong.
    memset( out_str, 0, len );

    char* result = g_strdup_vprintf( ZkCatalogRegistry::GetString( id ), argList );
    strncpy( out_str, result, len-1 ); // -1 here since we prefer a zero terminated string..
    g_free( result );
}



