/* NSPR glue code, to keep us from requiring all of NSPR for the registry */

#include "prmem.h"
#include <assert.h>

void *
PR_Malloc(PRUint32 size)
{
    return malloc(size);
}

void *
PR_Calloc(PRUint32 nelem, PRUint32 elsize)
{
    return calloc(nelem, elsize);
}

void *
PR_Realloc(void * ptr, PRUint32 size)
{
    return realloc(ptr, size);
}

void
PR_Free(void *ptr)
{
    free(ptr);
}

#include "prlock.h"

/*
 * Our registry usage doesn't currently require threadsafety, so we just
 * punt here and dummy up some stub functions.  If we require threadsafety
 * in the future, we should either pull in more of NSPR or make these call out
 * to libzkthread.
 */

struct PRLock {
    /* empty */
};

/* can't return NULL, so use this dummy structure */
static struct PRLock sDummyLock;

PRLock *
PR_NewLock(void)
{
    return &sDummyLock;
}

void
PR_DestroyLock(PRLock *lock)
{
    assert(lock == &sDummyLock);
    /* no-op */
}

void
PR_Lock(PRLock *lock)
{
    assert(lock == &sDummyLock);
    /* no-op */
}

PRStatus
PR_Unlock(PRLock *lock)
{
    assert(lock == &sDummyLock);
    return PR_SUCCESS;
}

#include <stdio.h>
#include <errno.h>

/* vr_OpenFile, so we can open files */
FILE *
vr_fileOpen (const char *name, const char * mode)
{
    FILE * fh; 
    if (!name)
        return NULL;
    
    fh = fopen( name, mode);
    if (!fh && errno == ENOENT)
        fh = fopen( name, "w+");
    return fh;
}
