/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef IZkAppState_H__
#define IZkAppState_H__

#include "ClientBaseInc.h"	// for CLIENTBASE_DLL
#include "ZkSingleton.h"	// for base class


#ifdef _WIN32
#pragma warning(disable: 4275)
#endif

class CLIENTBASE_DLL IZkAppState
	: public ZkSingleton < IZkAppState >
{

public:
	IZkAppState();
	virtual ~IZkAppState(){};

	// Determines whether any of the Freedom applications
	// are running.  This includes Freedom Client, the Diagnostic utility,
	// and the Restore utility
	static bool IsFreedomRunning() ;

	// Set to true once Freedom is completely initialized
	// and the proxies are allowed to start running
	static bool IsFreedomInitialized() ;

	// Set to true once Freedom is completely initialized
	// and the proxies are allowed to start running,
	// and the MainDialog has been initialized
	static bool IsWalletReady() ;

	// Ensures IsFreedomRunning() returns true by creating a mutex
	static void* SetFreedomRunning();

	// Ensures IsFreedomInitialized() returns true by creating a mutex
	static void* SetFreedomInitialized();

	// Ensures IsWalletReady() returns true by creating a mutex
	static void* SetWalletReady();

	// Returns true if the firewall is enabled
	static bool IsFirewallOn();

	// Turns firewall on/off
	static void SetFirewall(const bool);

protected:

	virtual bool onIsFreedomRunning() = 0;

	// Set to true once Freedom is completely initialized
	// and the proxies are allowed to start running
	virtual bool onIsFreedomInitialized() = 0 ;

	// Set to true once Freedom is completely initialized
	// and the proxies are allowed to start running,
	// and the MainDialog has been initialized
	virtual bool onIsWalletReady() = 0 ;

	// Ensures IsFreedomRunning() returns true by creating a mutex
	virtual void* onSetFreedomRunning() = 0;

	// Ensures IsFreedomInitialized() returns true by creating a mutex
	virtual void* onSetFreedomInitialized() = 0;

	// Ensures IsFreedomInitialized() returns true by creating a mutex
	virtual void* onSetWalletReady() = 0;

	virtual bool onIsFirewallOn() = 0;

	virtual void onSetFirewall(const bool) = 0;
};


#endif	// IZkAppState_H__