// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include "IZkConsoleMsg.h"
#include "ZkStringMgr.h"
#include "ZkConsoleHint.h"

#include <memory> // auto_ptr

//---------------------------------------------------------------------------
//
IZkConsoleMsg::IZkConsoleMsg()
{
}

void 
IZkConsoleMsg::ShowMsg( uint32_t msgId, ... )
{
	va_list argList;
	va_start(argList, msgId);

	get().ShowMsgImplementation( CONS_NORMAL, CONSIMAGE_NORMAL, msgId, argList ) ;	

	va_end(argList);
}

void 
IZkConsoleMsg::ShowMsg( ConsImage eImage, uint32_t idStringResource, ... )
{
	va_list argList;
	va_start(argList, idStringResource);
	
	get().ShowMsgImplementation( CONS_NORMAL, eImage, idStringResource, argList ) ;

	va_end(argList);	
}


void 
IZkConsoleMsg::ShowMsg( ConsSeverity eSeverity, ConsImage eImage, uint32_t idStringResource, ... )
{
	va_list argList;
	va_start(argList, idStringResource);
	
	get().ShowMsgImplementation( eSeverity, eImage, idStringResource, argList ) ;

	va_end(argList);	
}


void 
IZkConsoleMsg::ShowImportantMsg( uint32_t idStringResource, ... )
{
	va_list argList;
	va_start(argList, idStringResource);
	
	get().ShowMsgImplementation( CONS_IMPORTANT, CONSIMAGE_NORMAL, idStringResource, argList ) ;

	va_end(argList);	
}

void 
IZkConsoleMsg::ShowImportantMsg( ConsImage eImage, uint32_t idStringResource, ... )
{
	va_list argList;
	va_start(argList, idStringResource);
	
	get().ShowMsgImplementation( CONS_IMPORTANT, eImage, idStringResource, argList ) ;

	va_end(argList);	
}


void 
IZkConsoleMsg::ShowMsgForceUpdate( uint32_t idStringResource, ... )
{
	va_list argList;
	va_start(argList, idStringResource);
	
	get().ShowMsgImplementation( CONS_IMPORTANT, CONSIMAGE_NORMAL, idStringResource, argList, true ) ;

	va_end(argList);
}

void 
IZkConsoleMsg::ShowMsgForceUpdate( ConsImage eImage, uint32_t idStringResource, ... )
{
	va_list argList;
	va_start(argList, idStringResource);
	
	get().ShowMsgImplementation( CONS_IMPORTANT, eImage, idStringResource, argList, true ) ;

	va_end(argList);	
}

void 
IZkConsoleMsg::ShowMsgForceUpdate( ConsSeverity eSeverity, ConsImage eImage, uint32_t idStringResource, ... )
{
	va_list argList;
	va_start(argList, idStringResource);
	
	get().ShowMsgImplementation( eSeverity, eImage, idStringResource, argList, true ) ;

	va_end(argList);	
}

                
//---------------------------------------------------------------------------
//
void IZkConsoleMsg::ShowMsgImplementation( 
    ConsSeverity severity,
    ConsImage t, 
    uint32_t msgId,
    va_list argList,
    bool /*forceUpdate*/
)
{

	ZKSTD::string msg ;
	ZkStringMgr::Load( msg, msgId, argList  )  ;

    ZKSTD::auto_ptr< ZkConsoleHint > cHint( new ZkConsoleHint( msg, t, severity ));
    notify( cHint.get() );    
}

void IZkConsoleMsg::connectHack( ZkObserver* in_pObserver )
{
	get().connect( in_pObserver ) ;
}

void IZkConsoleMsg::disconnectHack( ZkObserver* in_pObserver )
{
	get().disconnect( in_pObserver ) ;
}