/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#ifndef ZKS_INCL_IZKCONSOLEMSG
#define ZKS_INCL_IZKCONSOLEMSG

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include "ClientBaseInc.h"

#ifndef ZKS_INCL_ZKSINGLETON
#include "ZkSingleton.h"
#endif

#ifndef ZKS_INCL_ZKCONSTYPES
#include "ZkConsTypes.h"
#endif

#ifndef ZKS_INCL_ZKSUBJECT
#include "ZkSubject.h"
#endif

#ifndef ZKS_INCL_ZKHINT
#include "ZkHint.h"
#endif

#include <cstdarg>
#include <string>

#include "ConsMessages.h"		// for the string ID's (this is for convenience,
								// we assume that if you want to use the console
								// you also want access to the catalog of console messages)

// This registry key is used to enable the verbose mode of
// the console.  However it is up to each observer to decide
// if they want to respect this flag.  For example
// the "log to file" observer can choose to log everything to
// file even if this registry key is not set
#define CONSOLE_VERBOSE_REGISTRY_KEY "ConsoleDebug"

// The registry key that is use to create a file that log the console
// content
#define CONSOLE_LOG_REGKEY "ConsoleLog"

// Console log base filename
#define CONSOLE_LOG_FILENAME "console"


/*===========================================================================
 * CLASS IZkConsoleMsg
 *=========================================================================*/

/*
  This base class offers a standard API for all platforms.
  Even though it is a singleton in that there is only one instance of the thing,
  the instance has to be explicitly created at program startup.
  Users of the instance have no knowledge of the platform specific instance
  to create.

  The ZkConsleMsg is a ZkSubject. Any object inetrested in knowing when a message
  is ready to display can derive from ZkObserver and connect with the console.
  ZkHint objects are sent to registered observers, hints are defined in 
  ZkConsoleHint.h

  The main hint type, ZkConsoleHint, contains the same information ConsInfo
  used to provide and should now be used instead.
*/

#ifdef _WIN32
#pragma warning( disable : 4275 )
#endif

class CLIENTBASE_DLL IZkConsoleMsg 
    : public ZkSingleton< IZkConsoleMsg > 
    , public ZkSubject 
{

public: 

    /*----- static member functions -----*/
    
    IZkConsoleMsg();
    virtual ~IZkConsoleMsg(){}

    static void ShowMsg( uint32_t msgId, ... ) ;
    static void ShowMsg( ConsImage, uint32_t msgId, ... ) ;
    static void ShowMsg( ConsSeverity, ConsImage, uint32_t msgId, ... ) ;

	// An "Important" Message will actually appear on the Main UI.
    // (on some platforms, this may depend on options set in the main UI
    //  message display)

	static void ShowImportantMsg( uint32_t msgId, ... ) ;
	static void ShowImportantMsg( ConsImage eImage, uint32_t msgId, ... ) ;

    // These functions will wait for the ui to be updated before returning.
    // Doesn't make any difference on Linux platform.
	// OBSOLETE - No longer makes a difference on Windows either

    static void ShowMsgForceUpdate( uint32_t msgId, ... ) ;
    static void ShowMsgForceUpdate( ConsImage, uint32_t msgId, ... ) ;
    static void ShowMsgForceUpdate( ConsSeverity, ConsImage,  uint32_t msgId, ...);
                 
	// Hack alert - On windows calling "get" directly was failing,
	// needed to add this temporarily to allow subject to connect
	static void connectHack( ZkObserver* in_pObserver ) ;
	static void disconnectHack( ZkObserver* in_pObserver ) ;

protected:

	virtual void ShowMsgImplementation( 
		ConsSeverity , 
		ConsImage , 
		uint32_t idStringResource, 
		va_list in_args, 
		bool bForceDisplay = false ) ;
                 
};

#endif
