/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef IZkRegistry_H__
#define IZkRegistry_H__

#include "ClientBaseInc.h"	// for CLIENTBASE_DLL
#include "ZkSingleton.h"	// for base class
#include <client.h>			// for ZKSTD
#include <string>
#include <vector>

#ifdef _WIN32
#pragma warning(disable: 4275)
#endif

class CLIENTBASE_DLL IZkRegistry 
	: public ZkSingleton < IZkRegistry >
{

public:
	IZkRegistry();
	virtual ~IZkRegistry(){};

	static bool GetFlag( const char* in_pszKeyName ) ;
	static void SetFlag( const char* in_pszKeyName, bool in_bValue ) ;

	static bool GetDWORD( const char*in_pszKeyName, unsigned long &dwValue ) ;
	static void SetDWORD( const char*in_pszKeyName, unsigned long in_dwValue ) ;

	static void GetShortArray( const char*in_pszKeyName, ZKSTD::vector<uint16_t >& out_vec ) ;
	static void SetShortArray( const char*in_pszKeyName, const ZKSTD::vector<uint16_t > in_vec ) ;

	static bool GetString( const char*in_pszKeyName, ZKSTD::string& out_pszValue) ;
	static void SetString( const char*in_pszKeyName, const char*in_pszValue ) ;

	static long DeleteValue(const char* in_pszKeyName );

protected:

	virtual bool onGetFlag( const char* in_pszKeyName ) = 0;
	virtual void onSetFlag( const char* in_pszKeyName, bool in_bValue ) = 0;

	virtual bool onGetDWORD( const char*in_pszKeyName, unsigned long &dwValue ) = 0;
	virtual void onSetDWORD( const char*in_pszKeyName, unsigned long in_dwValue ) = 0;

	virtual void onGetShortArray( const char*in_pszKeyName, ZKSTD::vector<uint16_t >& out_vec ) = 0;
	virtual void onSetShortArray( const char*in_pszKeyName, const ZKSTD::vector<uint16_t > in_vec ) = 0;

	virtual bool onGetString( const char*in_pszKeyName, ZKSTD::string& out_pszValue) = 0;
	virtual void onSetString( const char*in_pszKeyName, const char*in_pszValue ) = 0;

	virtual long onDeleteValue(const char* in_pszKeyName ) = 0;

};


#endif	// IZkRegistry_H__
