/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/* 

   Small wrapper so that we can have scoped mutex locks.

*/

#ifndef ZKS_INCL_MUTEXGUARD
#define ZKS_INCL_MUTEXGUARD

#include <zkThread.h>
#include <clientdebug.h>	// for ZKASSERT

/*===========================================================================
 * CLASS MutexGuard
 *=========================================================================*/

class MutexGuard{
public:
	MutexGuard( ZkMutex m ) : m_lock( m ) { Lock(); }
	~MutexGuard() { Unlock(); }

	void Lock() ;
	void Unlock();

private:
	ZkMutex m_lock;
};


// We don't throw here because too many callers use us, either directly
// or indirectly, in their destructor.
inline void MutexGuard::Lock()
{
	ZkError error;
	zkErrorNew(&error);
    ZKVERIFY( zkMutexLock( error, m_lock ) != ZK_FALSE ) ;    
	zkErrorFree(&error);
}

//---------------------------------------------------------------------------
//
inline void MutexGuard::Unlock()
{
    ZkError error;
    zkErrorNew(&error);
    if( zkMutexUnlock( error, m_lock ) == ZK_FALSE ) 
    {
        ZKASSERT(0);
		// Never throw in DTORs
    }    
    zkErrorFree(&error);
}

#endif
