// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// Helpful functions to get information
// about the configuration of Freedom
// 
// By configuration I mean:
// What directory are we installed in, what file names
// do we save into?
// What network are we installed on?
// Etc.
//
// Related things not handled by this object include:
// User preferences for App Behavior
// 

#ifndef _ZKCONFIGURATION_H
#define _ZKCONFIGURATION_H

#ifdef _WIN32
#pragma once
#endif

#include <client.h>			// for ZKSTD
#include "ClientBaseInc.h"	// for CLIENTBASE_DLL
#include <string>


enum SerialNumberTypes 
{
	NYM_TYPE1_CREATE = 500,	// mail and network (serial id = 500)
	NYM_TYPE1_EXTEND = 500,	// mail and network (serial id = 500)
	NYM_TYPE2_CREATE = 501,	// mail only (serial id = 501)
	NYM_TYPE2_EXTEND = 501,	// mail only (serial id = 501)
	NYM_TYPE3_CREATE = 502,	// nym only (serial id = 502)
	NYM_TYPE3_EXTEND = 503	// nym only (serial id = 503)
} ;

class CLIENTBASE_DLL ZKConfiguration 
{
private:
	ZKConfiguration(); // this class cannot be instantiated.
	
public:

	// We use multiple networks internally
	// to test and develop
	//
	// The setting is stored as a numeric value in the
	// registry use the FreedomCfg utility to adjust
	// 
	// Normally values from this enum should not be used by
	// code external to this class

	enum ZKFreedomNetworks 
	{
		ProductionFreedomNetwork = 0,
		
		PrayFreedomNetwork    = 1, /* also called religious network */
		VeggieFreedomNetwork  = 2,
		AlphaFreedomNetwork   = 3,

		area51 = 4,
		aussie = 5,
		groomlake = 6,
		beta=7,
		delta=8,
		gamma=9,
		zeta=10,
		beta1=11,
		beta2=12,

		// There are four hard coded networks
		// This will simplified in the future once scalper problem is fixed
		BaseFreedomNetworkCount = 13,

		// The registry can be specified to use a specific network
		// by its name
		RegistrySpecifiedNetwork = 1000,
		DefaultFreedomNetwork = ProductionFreedomNetwork
	};
		

	// Returns the NIQS domain name for selected network
	static ZKSTD::string GetNiqsAddress();

	static ZKSTD::string GetTrialSerialNumberUrl();
	static ZKSTD::string GetSerialNumberUrl();
	static ZKSTD::string GetSerialNumberUrl(SerialNumberTypes serialtype);


	// Set the network
	static void ChooseNetwork( unsigned long in_dwNetwork, const char* in_pszNetworkName = NULL );

	// Returns the chosen network (value from ZKFreedomNetworks enum)
	static unsigned long GetChosenNetwork();
	
	// Retrieve information about what version of the client is 
	// running.  This is normally only for informational, debugging information,
	// should not be the basis for making code logic decisions
	static ZKSTD::string GetFreedomPlatform() ;
	static ZKSTD::string GetFreedomProductName() ;
	static ZKSTD::string GetFreedomProductVersion() ;

	// Name of the current network, for example "t1.dev.zks.net" or
	// "freedom.net"
	static ZKSTD::string GetNetworkName();

	static void SetUseProdNetwork(bool in_bUseProdNetwork) 
		{ s_bUseProductionNetwork = in_bUseProdNetwork; } ;

#ifdef INTERNAL_BUILD
	static void DoUnitTest();
#endif //INTERNAL_BUILD


private:

	// Build a scalper URL for internal networks.
	// Depending on the serial number id different types of tokens are available
	static ZKSTD::string BuildInternalScalperUrl( const ZKSTD::string& in_rstrNetwork, 
												const int in_snid );

	static bool s_bUseProductionNetwork;
};

#endif
