// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):





// ZKFileConfiguration.h

// This class manages all the File Names, File Directory locations
// on a client.  Locations include the Freedom Install directory,
// the User Data directory.  Files include the main freedom data file
// (freedom.nym)

// Each platform has its own conventions for file locations and even file 
// naming.  This class should hide these details from the rest of the
// client.  Note: We do expect that the freedom.nym file has the same 
// name on all platforms.

// This class is not intended to actually perform file I/O functions.
// Please see ZKFileHelp for such functions.



#ifndef _ZKFILECONFIGURATION_H
#define _ZKFILECONFIGURATION_H

#include <client.h>
#include "ClientBaseInc.h"
#include <string>

class RZkException ;

class CLIENTBASE_DLL ZKFileConfiguration
{
public:

	// Return the directory that contains the exe and dlls
	// Path includes the "\" at the end
	static ZKSTD::string GetFreedomInstallDir();


	// Location to place the users files
	// see CSIDL_APPDATA
	// Path includes the "\" at the end
	static ZKSTD::string GetFreedomUserDataDir();


	// Path where we can store files that might apply to more
	// than just the current user.  For example cached network
	// information - similar to HKEY_LOCAL_MACHINE versus 
	// HKEY_CURRENT_USER, see CSIDL_COMMON_APPDATA
	static ZKSTD::string GetFreedomCommonDataDir();

	// The wallet scripts apply to all users
	static ZKSTD::string GetWalletScriptsDir();
	static ZKSTD::string GetWalletScriptPath(const ZKSTD::string& in_strDomain);

	// Get Executable Name and Path
	static ZKSTD::string GetFreedomExeFilePath();

	static ZKSTD::string GetKeyCacheFilePath();

	static ZKSTD::string GetNiqsCacheFilePath();

	static ZKSTD::string GetTokenDescFilePath();

	static ZKSTD::string GetEntropySeedFilePath();


	// Return the file name (with path) where we store our data (nyms etc)
	// Must be called from the Freedom process or a executable running in the
	// same directory as Freedom
	static ZKSTD::string GetFreedomDataFilePath();
	static ZKSTD::string GetFreedomDataFilePathProd();


	// Version 1.X saved the file in the installation directory.
	// In order to convert we remember this
	static ZKSTD::string GetOldFreedomDataFilePath() ;

	// Return the file name where we store our data (nyms etc)
	// Does NOT include the path
	static ZKSTD::string GetFreedomDataFileName();

	// Return our standard file name for a backup file
	// based on the current network
	static ZKSTD::string GetFreedomDataBackupFileName() ;
	// Return the old format of it to be supported by the backup utility
	static ZKSTD::string GetOldFreedomDataBackupFileName() ;

	// Returns the file name to save into as a backup
	static ZKSTD::string GetFreedomAutoBackupFilePath() ;


	// return the value in the registry contains in TEST_DATA_DIR_KEY
	// otherwise, return "C:\dev\bin"
	static ZKSTD::string GetFreedomTestDataDir();

    // Return the directory in which we want to put copies of the console output.
    static ZKSTD::string ZKFileConfiguration::GetFreedomConsoleLogDir();

#ifdef ZK_LINUX
    // Returns the directory containing the XML description of the UI.
    // (Linux Specific)
    static ZKSTD::string GetFreedomXMLDir();

    // Returns the directory containing all pixmaps used in the UI.
    // (Linux Specific)
	static ZKSTD::string GetFreedomPixmapDir();

    // Returns the directory containing all Freedom help.
    // (Linux Specific)
	static ZKSTD::string GetFreedomHelpDir();
	static ZKSTD::string GetFreedomNymWizardHelp();
	static ZKSTD::string GetFreedomRouteWizardHelp();
	static ZKSTD::string GetFreedomSetupWizardHelp();

    // Set value for GetFreedomExeFlePath from argv0
    static void SetFreedomExeFilePath(char *argv0);
#endif

	// Test that the file system has the necessary
	// folders.  This can be useful for self-repair 
	// and can halt the program immediately if system is
	// in bad shape
	static void EnsureFileConfigurationValid() throw (RZkException);


#ifdef INTERNAL_BUILD
	static void DoUnitTest();
#endif //INTERNAL_BUILD

private:
	ZKFileConfiguration() ;
	static const char szBackupExt[];

#ifdef ZK_LINUX
    static char *m_ExeFilePath;
#endif

	static const char szNymFileBackupName[];
	static const char szOldNymFileBackupName[];
} ;

#endif
