/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_ZKCONSOLEHINT
#define ZKS_INCL_ZKCONSOLEHINT

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#ifndef ZKS_INCL_ZKHINT
#include "ZkHint.h"
#endif

#ifndef ZKS_INCL_ZKCONSTYPES
#include <ZkConsTypes.h>
#endif

#include "ClientBaseInc.h"
#include <clientstl.h>
#include <zks.h>
#include <string>

/*===========================================================================
 * CLASS ZkConsoleHint
 *=========================================================================*/

/*
  Hint sent by the ZkConsoleMsg to notify any interested observer that a
  new message is ready to be displayed.
  Display can thn derive from ZkObserver and connect to the console.
*/

class CLIENTBASE_DLL ZkConsoleHint : public ZkHint {

public:

    /*----- member functions -----*/

	ZkConsoleHint();
    ZkConsoleHint( 
        const ZKSTD::string& msg, 
        ConsImage image = CONSIMAGE_NORMAL, 
        ConsSeverity severity = CONS_IMPORTANT
    );

	ZkConsoleHint( const ZkConsoleHint& );


    ~ZkConsoleHint();

    ConsSeverity getSeverity() const;
    ConsImage    getImageId() const;

    const ZKSTD::string& getMessage() const;

private:

    /*----- member functions -----*/

	// Prohibited funcs - not implemented.
	const ZkConsoleHint& operator=( const ZkConsoleHint& );


    /*----- data members -----*/

	ConsSeverity m_eSeverity;
	ConsImage    m_eImage;    // image index
    ZKSTD::string  m_msg;

};

//---------------------------------------------------------------------------
//
inline ConsSeverity ZkConsoleHint::getSeverity() const
{
    return m_eSeverity;
}

//---------------------------------------------------------------------------
//
inline ConsImage ZkConsoleHint::getImageId() const
{
    return m_eImage;
}

//---------------------------------------------------------------------------
//
inline const ZKSTD::string& ZkConsoleHint::getMessage() const
{
    return m_msg;
}


#endif
