/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 *         
 */

/*$Id: ZkConsoleLogger.h,v 1.1.2.4 2001/01/23 15:54:15 ericb Exp $*/


#ifndef ZKS_INCL_ZKCONSOLELOGGER
#define ZKS_INCL_ZKCONSOLELOGGER

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

// System Includes
#include <stdio.h>

// Freedom Includes
#include <clientstl.h>

// Module Includes
#include "ZkObserver.h"
#include "ZkSingleton.h"

using namespace ZKSTD;


class IWordScan;


class ZkHint;

/*===========================================================================
 * CLASS ZkConsoleLogger
 *=========================================================================*/

class CLIENTBASE_DLL ZkConsoleLogger 
    : public ZkObserver
    , public ZkSingleton< ZkConsoleLogger >
{
public: 

    /*----- constants -----*/

    static const char * const SeverityStrings[];

    /*----- member functions -----*/
      
    ZkConsoleLogger();
    virtual ~ZkConsoleLogger();

    virtual void update( ZkHint* );

    void SetFileName( string logFileName );

	void SetIWordScan( IWordScan* pWordScan );

private: 

	// Prohibited funcs - not implemented.
	ZkConsoleLogger( const ZkConsoleLogger& );
	const ZkConsoleLogger& operator=( const ZkConsoleLogger& );

    /*----- member functions -----*/

	// Open the log file for writing.
    // retuns true if successful
    bool OpenFile();

    // Close the log file. Furthur notifications will not be written
    void CloseFile();

    /*----- data members -----*/

    FILE *m_file;
    string m_fileName;
	IWordScan* m_pWordScan;
};
#endif
