// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include "ZkObserver.h"
#include "ZkSubject.h"
#include <clientdebug.h>
#include <algorithm>

//===========================================================================
// CLASS ZkObserver
//===========================================================================

//---------------------------------------------------------------------------
//
ZkObserver::ZkObserver( )
{
}

//---------------------------------------------------------------------------
//
ZkObserver::~ZkObserver( )
{
    // Make sure we disconnect from any ZkSubject we had connected with.

    disconnectAll();
}


//---------------------------------------------------------------------------
//
void ZkObserver::registerSubject( ZkSubject* subject )
{
    // Registering twice has no effect.
    SubjectVector::iterator iter;
    iter = ZKSTD::find( m_subjects.begin(), m_subjects.end(), subject );

    if( iter == m_subjects.end() ) 
    {
        m_subjects.push_back( subject );
    }
}

//---------------------------------------------------------------------------
//
void ZkObserver::deregisterSubject( ZkSubject* subject )
{

    // Deregistering has no effect if not in the list. 
    // (FIXME - could assert, this is not a normal situation)
    SubjectVector::iterator iter;
    iter = ZKSTD::find( m_subjects.begin(), m_subjects.end(), subject );

    if( iter != m_subjects.end() ) 
    {
        m_subjects.erase( iter );
    }
	else
	{
		ZKASSERT(0);
	}

}

//---------------------------------------------------------------------------
//
void ZkObserver::disconnectAll()
{
    SubjectVector::reverse_iterator iter;

    // Start from the end, since in return the subjects are going to 
    // deregister themselves.
    for( iter = m_subjects.rbegin(); iter != m_subjects.rend(); ++iter )
    {
        (*iter)->disconnect( this );
    }
}




