/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_ZKOBSERVER
#define ZKS_INCL_ZKOBSERVER


#include <vector>
#include <clientstl.h>
#include "ClientBaseInc.h"

class ZkSubject;
class ZkHint;

#ifdef WIN32
#pragma warning( disable : 4251 )
#endif

/*
  This class can be used by an object to register itself with a ZkSubject.
  A ZkSubject will notify all its ZkObervers when its state changes.
  The advantage of using the ZkSubject/ZkObserver is to get for free all
  the registration mangement. ZkObservers automatically deregister themselves
  when they go out of scope. Also, registration and notifycation should
  soon be thread safe.
*/

class CLIENTBASE_DLL ZkObserver {

    // ZkSubject is a friend of ZkObserver so that only a subject can deregister itself.

	friend class ZkSubject;

public: 

    /*----- types and enumerations ----*/

    typedef ZKSTD::vector< ZkSubject* > SubjectVector;

    /*----- member functions -----*/

    ZkObserver();
    virtual ~ZkObserver();

	// Purpose:  Called by a ZkSubject to send a hint notification.
	// Throws:   FIXME - should probably throw if the hint is NULL.
    virtual void update( ZkHint* ) = 0;
      
	// Purpose:  For tests only.
	// Promise:  I'll kick your butt if you use it.
    const ZkObserver::SubjectVector& getSubjectVector( ) const;



protected:

    /*----- member functions -----*/

    void disconnectAll();


private: 

    void registerSubject( ZkSubject* );

    void deregisterSubject( ZkSubject* );



    /*----- data members -----*/

    SubjectVector m_subjects;

};


//---------------------------------------------------------------------------
//
inline  const ZkObserver::SubjectVector& ZkObserver::getSubjectVector( ) const
{
    return m_subjects;
}



#endif
