/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_ZKPROXYDIALOGS
#define ZKS_INCL_ZKPROXYDIALOGS

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include <ZkSingleton.h>
#include <ClientBaseInc.h>      // for CLIENTBASE_DLL

#include <client.h>
#include <memory>
#include <vector>

class WordScanSummary;

/*===========================================================================
 * CLASS IProxyDialogs
 *=========================================================================*/

class CLIENTBASE_DLL IProxyDialogs {

public: 

    /*----- member functions -----*/

    // See class below for a description of each function.
      
	virtual long LaunchOutgoingMsgDlg( 
        ZKSTD::vector<ZKSTD::string>* pVecAddresses, 
        bool in_fSMTP = true,
		const ZKSTD::string* in_pStrUserEmailAddr = NULL) = 0;

	virtual bool LaunchWordScanMsgDlg( 
        unsigned int in_Protocol, 
        WordScanSummary* in_pScanResults ) = 0;

	virtual bool LaunchWarnSignedMailDlg( ) = 0;

	virtual long LaunchFirstMailCheckDlg( ) = 0;
	virtual void LaunchMailSentCountDlg( time_t resetTime,
					     ZKSTD::vector<ZKSTD::string> * pvecRecipients ) = 0;

    virtual bool LaunchWarnNNTPConnectionDlg( ) = 0;

    virtual bool LaunchWarnSSLConnectionDlg( ) = 0;
};


/*===========================================================================
 * CLASS ZkProxyDialogs
 *=========================================================================*/

class  CLIENTBASE_DLL ZkProxyDialogs 
    : public ZkSingleton< ZkProxyDialogs > {

public: 

    /*----- static member functions -----*/

	// Purpose:  Opens up the Outgoing Mail dialog box, asking the user
    //           which Nym is to be used.
    // Returns:  -1 if user chooses to block outgoing message.
    //           The nym index otherwise.
	static long LaunchOutgoingMsgDlg( 
        ZKSTD::vector<ZKSTD::string>* pVecAddresses, 
        bool in_fSMTP = true,
		const ZKSTD::string* in_pStrUserEmailAddr = NULL);

	// Purpose:  Opens up the Word Scan dialog box, warning the user
    //           about a word found in outgoig traffic that corresponds to an
    //           entry in the Word Scan list.
    // Returns:  False if user chooses to block outgoing message.
    //           True  if the message is to be sent anyway.
	static bool LaunchWordScanMsgDlg( 
        unsigned int in_Protocol, 
        WordScanSummary* in_pScanResults );

	// Purpose:  Opens the Signed Mail warning dialog box, explaining to the 
    //           user that the email about to be sent contain a digital signature.
    // Returns:  False if user chooses to block outgoing message.
    //           True  if the message is to be sent anyway.
	static bool LaunchWarnSignedMailDlg( );
    
	// Purpose: Opens a dialog explaining the risks of Mail Correlation attack,
	//			and allows the user to choose whether to check all their mail,
	//			or only the active nym's.
	// Returns: ZK_MAIL_CHECK_ALL, ZK_MAIL_CHECK_ACTIVE, ZK_MAIL_CHECK_BLOCK
	static long LaunchFirstMailCheckDlg( );

	// Purpose:
	// Returns:
	static void LaunchMailSentCountDlg( time_t resetTime,
                                        ZKSTD::vector<ZKSTD::string> * pvecRecipients );

	// Purpose: Opens a dialog explaining the problem with reading news with a 
    //          nym selected.  It also contains a link to an news.freedom.net
    //          which redirects to nes.freedom.net.
	// Returns: true if the connection should be allowed, and false if it should
    //          be blocked
    static bool LaunchWarnNNTPConnectionDlg( );

	// Purpose: Opens a dialog explaining the cookie and word scanning problems
    //          with SSL.
	// Returns: true if the connection should be allowed, and false if it should
    //          be blocked
    static bool LaunchWarnSSLConnectionDlg( );

    /*----- member functions -----*/

    // Purpose:  Initializes the single instance of the Proxy Dialogs API.
    // Requires: A valid instance of a handler must be provided at construction.
    //           The handler takes care of openening a platform specific 
    //           implementation of the dialogs. Handlers are usually found in
    //           /zkui/zkdialogs.
    ZkProxyDialogs( ZKSTD::auto_ptr< IProxyDialogs > handler );
    ~ZkProxyDialogs( );
      
private: 

    /*----- static data members -----*/

    ZKSTD::auto_ptr< IProxyDialogs >  m_pHandler;
};


#endif
