/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef _ZKS_INCL_ZKSTATUSSUBJECT
#define _ZKS_INCL_ZKSTATUSSUBJECT

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#ifndef _ZKS_INCL_ZKSTATUSID
#include <ZkStatusId.h>
#endif

#ifndef _ZKS_INCL_ZKSINGLETON
#include <ZkSingleton.h>
#endif

#ifndef _ZKS_INCL_ZKSUBJECT
#include <ZkSubject.h>
#endif

#ifndef _ZKS_INCL_ZKHINT
#include <ZkHint.h>
#endif



/*===========================================================================
 * CLASS ZkStatusHint
 *=========================================================================*/

/*
  A state hint is sent to every observer of this object.
  The hint contains that state ID along with a boolean indicating the new state.
  It is up to whoever gets the state to implement counters if a state is set
  multiple times.
*/

class ZkStatusHint : public ZkHint {
public:


    /*----- member functions -----*/
      
    // Note: the forceUpdate parameter is there only to make it easier to 
    //       use this class for windows eventually.
    ZkStatusHint( ZkStatusId, bool state, bool forceUpdate=false );
    ~ZkStatusHint();

    // Self explanatory. If not, please call marketing, they are hiring.
    ZkStatusId GetStatusId() const;
    bool       GetStatus() const;
    bool       ShouldForceUpdate() const;

private:
    
    /*----- data members -----*/

    ZkStatusId m_id;
    bool       m_state;
    bool       m_force;
};


/*===========================================================================
 * CLASS ZkStatusSubject
 *=========================================================================*/

/*
  This class takes care of notifying whoever is interested that a state has
  changed. It is a subject, so notifications are sent as hints.
  For now we will only have one type of hint, and use a state enum.
  If it turns out we need too many different states we will have to use one
  hint class per possible state (later...).
*/

class ZkStatusSubject  
    : public ZkSingleton< ZkStatusSubject > 
    , public ZkSubject {

public: 

    /*----- member functions -----*/
      
    ZkStatusSubject();
    ~ZkStatusSubject();

    static void UpdateStatus( 
        ZkStatusId id, 
        bool state, 
        bool forceUpdate=false 
    );

};

#endif
