/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#ifndef ZKS_INCL_ZKSTRINGMGR
#define ZKS_INCL_ZKSTRINGMGR

#include <client.h>
#include "ClientBaseInc.h"

/*===========================================================================
 * CLASS ZkStringMgr
 *=========================================================================*/

/*
  This class provides the API to load strings accross platforms.
  It supports the use of string IDs which are portable and map to different 
  string tables according to the platform. 

  For example, the string IDS_CONS_DNS_TIME maps to "DNS web lookup took %d seconds"
  on all platforms, but it is obtained from aresource file on windows and
  a ZkCatalogRegistry on Linux (which is really a glorified static string table).

  Only the header is provided in the /xp directory, you will find the platform
  specific implementation in the corresponding sub-directory.
*/

class CLIENTBASE_DLL ZkStringMgr 
{

public: 

    /*----- member functions -----*/

	// Purpose:  Finds the string identfied by the Id and fills in the given
    //           string. 
	// Promise:  Will assert if the string id is invalid.
    static void Load( 
        ZKSTD::string& out_str, 
        uint32_t id, 
        ... 
    );

    static void Load( 
        ZKSTD::string& out_str, 
        uint32_t id, 
        va_list in_args
    );


	// Purpose:  Finds the string identfied by the Id and fills in the given
    //           buffer. 
	// Promise:  Will assert if the string id is invalid.
    // Requires: A valid pointer, with the given length already allocated.
    static void Load( 
        char* out_str, 
        int len, 
        uint32_t id, 
        ... 
    );      

protected:

    /*----- member functions -----*/

    // Just an API, not for instantiation.
    ZkStringMgr();

};
#endif
