// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// ZKUnicode

// Conversion between 8 bit strings and the UNICODE 
// library is platform specific.  These conversions,
// messy conversions etc are wrapped up in this class

// Not sure what the final form of this code will be

#ifndef _ZKUNICODE_H
#define _ZKUNICODE_H

#include <string>
#include <zkIntTypes.h>
#include <client.h>
#include "ClientBaseInc.h"

class CLIENTBASE_DLL ZkUnicode
{
public:

	// Call to fill in a provided buffer with unicode.
	// For error handling matches behavior of MultiByteToWideChar()
	// - need to compare with other platform API and determine a more
	// consistent behavior.  For example maybe it be assumed to always succeed
	// and throw.  It could also take care of allocating the array if necessary.

	//User is required to zkMemFree the returned buffer
	static int StringToUnicodeBuffer( const ZKSTD::string& in_str, uint16_t ** out_ppUnicode, size_t& out_BufferSize ) ;

	static int UnicodeBufferToString( const uint16_t * in_pUnicode, size_t in_BufferSize, ZKSTD::string& out_str ) ;

	static int StorageUnicodeBufferToString ( const uint16_t * in_pUnicode, size_t in_BufferSize, ZKSTD::string& out_str );
        
	//User is required to zkMemFree the returned buffer, regardless of the returned value from the function 
	static int ZkUnicode::StringToStorageUnicodeBuffer( const ZKSTD::string &in_str, uint16_t **io_ppUnicode, size_t &out_BufferSize );

	// I guess we'll need similar methods for other platforms
	// This function establishes what code page is being used.  All 8 bit strings
	// will be interpreted as following the active code page value
	static void SetWindowsCodePage( uint32_t in_NewCodePage ) { m_WindowsCodePage = in_NewCodePage ; }

	// Basically just for testcode - get the active code page
	static uint32_t GetWindowsCodePage() { return m_WindowsCodePage ; }

private:

	ZkUnicode() ;

	// Code Page active
	static uint32_t m_WindowsCodePage ;

} ;

#endif
