// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "ZkWebIntegration.h"

#include "clientdefs.h"			// For INTERNAL_BUILD
#include "IZkPopup.h"			// for internal builds
#include "ZKConfiguration.h"	// used to get the urls
#include "ZkStringMgr.h"
#include "MiscMessages.h"

#ifdef WIN32
// This include in xp code avoids the need for an entire R class.
#include <../win/Url.h>
#include <time.h>
#endif

#ifdef ZK_LINUX
#include <gnome.h>
#endif

void
ZkWebIntegration::LaunchWebSite(unsigned int nID)
{
	if (( nID == URL_TRIAL_SERIAL_NUM ) ||
		( nID == URL_BUY_SERIAL_NUM )   ||
		  nID == URL_BUY_SERIAL_NUM_DURING_SETUP ) {

		// These are special case because they vary depending
		// on network
		LaunchScalperWebSite( nID ) ;
	}
	else {

		ZKSTD::string strURL = GetURL( nID );

		if ( !strURL.empty() ) {
			LaunchURL( strURL.c_str() ) ;
		}
	}
}


void 
ZkWebIntegration::LaunchOperatorInfoSite(int in_OperatorID)
{
    ZKSTD::string strURL;
    ZkStringMgr::Load( strURL, IDS_OPERATOR_URL, in_OperatorID );

	LaunchURL( strURL.c_str() ) ;	
}

void 
ZkWebIntegration::LaunchURL(const char* in_pszUrl)
{
#ifdef WIN32
	GotoURL(in_pszUrl);
#endif

#ifdef ZK_LINUX
    gnome_url_show( in_pszUrl );
#endif
}

void 
ZkWebIntegration::LaunchScalperWebSite(unsigned int nID)
{
	ZKSTD::string strURL;
	switch(nID)
	{
	case URL_TRIAL_SERIAL_NUM:
		strURL = ZKConfiguration::GetTrialSerialNumberUrl() ;
		break;
	case URL_BUY_SERIAL_NUM:
	{
		strURL = ZKConfiguration::GetSerialNumberUrl(NYM_TYPE1_CREATE) ;
		
		// we finally append the parameter, see bug 7569
		ZKSTD::string strPostfix;
		ZkStringMgr::Load( strPostfix, IDS_SERIAL_POSTFIX );
		AppendURLParameter( strURL , strPostfix );

		break;
	}
	case URL_BUY_SERIAL_NUM_DURING_SETUP:
	{
		strURL = ZKConfiguration::GetSerialNumberUrl(NYM_TYPE1_CREATE);

		// we finally append the parameter, see bug 7569
		ZKSTD::string strPostfix;
		ZkStringMgr::Load( strPostfix, IDS_SERIAL_DURING_SETUP_POSTFIX );
		AppendURLParameter( strURL , strPostfix );

		break;
	}
	default:
		ZKASSERT( false ) ;
		break;
	}

	if ( !strURL.empty() ) {
		LaunchURL( strURL.c_str() ) ;
	}
    else
	{
#ifdef INTERNAL_BUILD
		IZkPopup::MessageBox( "No Scalper is Currently Configured for this Network,\nplease consult network owner for URL",
                              ZK_MESSAGE,
                              OVER_CENTER );
#else
		// Ouch - we shipped outside the company something 
		// that doesn't know where to get
		// an valid serial number!
		ZKASSERT( false ) ;
#endif
	}
}

ZKSTD::string 
ZkWebIntegration::GetURL( unsigned int nID )
{
    ZKSTD::string strURL;

    switch( nID )
    {
		case URL_HOMEPAGE_ID:
			ZkStringMgr::Load( strURL, IDS_HOMEPAGE_URL );
			break;
		case URL_SUPPORT_ID:
			ZkStringMgr::Load( strURL, IDS_SUPPORT_URL );
			break;
		case URL_GETTINGSTARTED_ID:
			ZkStringMgr::Load( strURL, IDS_GETTINGSTARTED_URL );
			break;
		case URL_NEWS_PARTNER:
			ZkStringMgr::Load( strURL, IDS_NEWS_URL );
			break;
		case URL_TEAM:
			ZkStringMgr::Load( strURL, IDS_TEAM_URL );
			break;
		case URL_KB_MAIL_CORRELATION:
			ZkStringMgr::Load( strURL, IDS_KB_MAIL_CORRELATION_URL );
			break;
		case URL_VIRAL_EMAIL_WARNING:
			ZkStringMgr::Load( strURL, IDS_VIRAL_EMAIL_WARNING );
			break;
		case URL_TESTDRIVE_ID:
			ZkStringMgr::Load( strURL, IDS_TESTDRIVE_URL );
			break;
		case URL_TESTDRIVE_STD_NYM_ID:
			ZkStringMgr::Load( strURL, IDS_TESTDRIVE_URL_STD_NYM );
			break;
		case URL_KB_SSL_LOGIN_PROBLEMS:
			ZkStringMgr::Load( strURL, IDS_KB_SSL_LOGIN_PROBLEMS);
			break;
		case URL_PREMIUM_INFO_ID:
			ZkStringMgr::Load( strURL, IDS_PREMIUM_INFO_URL );
			break;
		default:
			ZkStringMgr::Load( strURL, IDS_HOMEPAGE_URL );
			break;
    }

    return strURL;
}

void
ZkWebIntegration::AppendURLParameter( ZKSTD::string& io_strURL, 
									 const ZKSTD::string& in_strParam)
{
	if( io_strURL.find('?') != ZKSTD::string::npos )
	{
		// mean that the URL has already an '?', so we append '&'
		io_strURL += '&';
	}
	else
	{
		// mean that the URL don't have any parameter, so we append '?'
		io_strURL += '?';
	}

	// we finally append the parameter
	io_strURL += in_strParam;
}


#ifdef INTERNAL_BUILD

void 
ZkWebIntegration::DoUnitTest()
{
	::srand( time( NULL ) );

#ifdef WIN32
// FIXME - in a hurry, find replacement for Sleep.
	// Display one of our defined URL.
	LaunchWebSite( ::rand() % ( URL_TEAM + 1 ) );
	::Sleep( 1500 );
	// Display the default URL.
	LaunchWebSite( ::rand() + URL_TEAM + 1 );
	::Sleep( 1500 );

	LaunchOperatorInfoSite( ( rand() % 100 ) + 1 );
	::Sleep( 1500 );

	LaunchURL( "url:http://www.avvc.qc.ca" );
	::Sleep( 1500 );

	LaunchScalperWebSite( URL_TRIAL_SERIAL_NUM );
	::Sleep( 1500 );

	LaunchScalperWebSite( URL_BUY_SERIAL_NUM );
	::Sleep( 1500 );
#endif
}

#endif	// INTERNAL_BUILD
