// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// includes here
#include "RZKErrorReporter.h"
#include "ExceptionMessages.h"
#include "../xp/IZkException.h"
#include "RZKStringLoader.h"
#include "ZkAbstractReporter.h"

// includes from other projects
#include <clientdebug.h>
#include <clientlinux.h>
#include <errno.h>
#include <memory> // for auto_ptr

// C++ includes
#include <strstream>

RZkErrorReporter::RZkErrorReporter()
{
    // empty for noe.
}

RZkErrorReporter::~RZkErrorReporter()
{
    // empty for now.
}


// pure in base
void 
RZkErrorReporter::ShowError( const IZkException& in_except ) throw()
{
    ZkReporterInstance::ShowError( in_except );

/*
    printf("AN ERROR HAS OCCURED. There is no error dialog box currently\n");
    printf("ERROR: %s\n", GetErrorString(in_except).c_str());
    ZKASSERT(0);
*/
}

// pure in base
// Get a suitable string representation of the error object along with the DZkError object
// Does not include the 'm_strDescription' in the string
ZKSTD::string
RZkErrorReporter::GetErrorString( const IZkException& in_except ) throw()
{

	ZKSTD::ostrstream s;
	ZKSTD::string strFormattedString;

	// Do something nice with the DZkError
	try
	{
		if( in_except.GetDZkError().getfnc().size() > 0 )
			s << " [" << in_except.GetDZkError().getfnc() << "] ";

		if( in_except.GetDZkError().geterrcode() > 0 )
			s << " [Error Code: " << in_except.GetDZkError().geterrcode() << "] ";

		if( in_except.GetDZkError().getdetail().size() > 0 )
			s << " [" << in_except.GetDZkError().getdetail() << "] ";

		// Get the real error
		strFormattedString = in_except.GetFormattedString();
	}
	catch(...)
	{
		ZKASSERT( 0 );
	}

	// Add the value of errno
	s << " [errno: " << errno << "]";

	return strFormattedString + s.str();
}

