/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
	This is a concrete class derived from IZkErrorReporter used
	for reporting exceptions.

    Under Linux this class happens to be a ZkSubject. When an exception occurs
    it notifies whicever ZkObserver is registered. This way we can create a 
    ui observer that will report the exception in a popup dialog, but we can 
    also develop a non-ui version that will log the error to a file.
    It also avoids some ugly dependencies on the high level ui code.
*/

#ifndef RZKErrorReporter_H__
#define RZKErrorReporter_H__

#include "IZKErrorReporter.h"
#include "DZkError.h"

#include <string>
#include <AutoInstCount.h>

class RZkErrorReporter;
class RZKStringLoader;
class IZkException;


/*===========================================================================
 * CLASS RZkErrorReporter
 *=========================================================================*/

class  RZkErrorReporter 
	: public IZkErrorReporter AUTO_INSTANCE_COUNTING2( RZkErrorReporter )
{
public:
	RZkErrorReporter();
	virtual ~RZkErrorReporter();

	
	// PURPOSE: override. Will show a dialog containing error information
	// REQUIRE:	An IZkException pointer of type RZkException
	// PROMISE: To use RZKStringLoader to load strings from resources
	//			GetCallerFunctionName(), GetDeepFunction(), GetDeepErrorCode()
	//			and GetDeepDetail() will all be displayed in the details portion
	//			of the error dialog	
	virtual void ShowError( const IZkException& in_except ) throw();

	// PURPOSE: override. Will silently return error information
	// REQUIRE:	An IZkException pointer of type RZkException
	// PROMISE: To use RZKStringLoader to load strings from resources
	//			The string will be composed of the following components
	//			The GetDZkError().getfnc() string, the GetDZkError().geterrcode()
	//			the GetDZkError().getdetail() and the GetFormattedString()
	//			as returned by the exception object passed here.
	virtual ZKSTD::string GetErrorString( const IZkException& in_except ) throw();

};

#endif // RZKErrorReporter_H__
