// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "RZkException.h"
#include <clientdebug.h>
#include "RZKStringLoader.h"
#include <stdarg.h>


const unsigned int RZkException::kMAXIMUM_STRING_LENGTH = 1024;


RZkException::RZkException(
	const DZkError&			in_err, 
	const ZKSTD::string		in_strCallerFunction,
	const ZKSTD::string		in_strDescription,
	const uint32_t			in_MessageId,
    ...)
		: m_zkError(in_err)
		, m_strCallerFunction(in_strCallerFunction)
		, m_strDescription(in_strDescription)
		, m_nMessageId(in_MessageId)
{

	ZKTRACE1("RZkException CTOR => [%s]\n", in_strDescription.c_str() );

	va_list argList;

	va_start(argList, in_MessageId);

	RZKStringLoader loader;

	const ZKSTD::string strFormat = loader.frLoadString( in_MessageId );

	char buffer[kMAXIMUM_STRING_LENGTH] = { 0 };
	vsnprintf( buffer, sizeof buffer, strFormat.c_str() , argList ) ;
	m_strFormatted = buffer;

	ZKTRACE1("RZkException CTOR after formatting => [%s]\n", m_strFormatted.c_str() );
    va_end(argList);
}


RZkException::RZkException(
	const ZKSTD::string		in_strCallerFunction,
	const ZKSTD::string		in_strDescription,
	const uint32_t			in_MessageId,
    ...)
		: m_strCallerFunction(in_strCallerFunction)
		, m_strDescription(in_strDescription)
		, m_nMessageId(in_MessageId)
{

	ZKTRACE1("RZkException CTOR => [%s]\n", in_strDescription.c_str() );

	va_list argList;

	va_start(argList, in_MessageId);

	RZKStringLoader loader;

	const ZKSTD::string strFormat = loader.frLoadString( in_MessageId );

	char buffer[kMAXIMUM_STRING_LENGTH] = { 0 };
	vsnprintf( buffer, sizeof buffer, strFormat.c_str() , argList ) ;
	m_strFormatted = buffer;

	ZKTRACE1("RZkException CTOR after formatting => [%s]\n", m_strFormatted.c_str() );
    va_end(argList);
}

RZkException::~RZkException()
{
}

// pure in base
uint32_t RZkException::GetErrorCode() const throw()
{
	return m_zkError.geterrcode();
}

// pure in base
uint32_t RZkException::GetErrorStringId() const throw()
{	
	return m_nMessageId;
}

const DZkError& RZkException::GetDZkError() const
{
	return m_zkError;
}

const ZKSTD::string& RZkException::GetCallerFunctionName() const throw()
{
	return m_strCallerFunction;
}

const ZKSTD::string& RZkException::GetFormattedString() const throw()
{
	return	m_strFormatted;
}

const ZKSTD::string& RZkException::GetDescription() const throw()
{
	return m_strDescription;
}
