// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#ifndef ZKS_INCL_ZKABSTRACTREPORTER
#define ZKS_INCL_ZKABSTRACTREPORTER

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include "ZkAbstractReporter.h"

// FIXME - will be moved soon
#include <../../clientbase/xp/ZkSingleton.h>
#include <memory> // for auto_ptr
#include <client.h>

class IZkException;


/*===========================================================================
 * CLASS ZkAbstractReporter
 *=========================================================================*/

/*
  FIXME - this is getting way too convoluted, the original idea changed
          was partly deleted, recoded. Cleanup needed.

          The main reason for this indirection is that we need a single instance
          of the UI reporter because of thread issues, and we want to avoid
          dependencies on higher level libraries such as.. UI.
          This is why we have a ZkAbstractReporter class, so that a UiReporter 
          can be derived, and a ZkReporterInstance class, so that the x-platform
          api can have a static instance to call.

*/

class ZkAbstractReporter {

public: 

    /*----- member functions -----*/

    ZkAbstractReporter();
    virtual ~ZkAbstractReporter();

	// Purpose:  Called by RZKErrorReporter to have the error reported.
    //           It is up to the concrete reporter to handle the error.
	virtual void ShowError( const IZkException& in_except ) throw() = 0;
     
};

/*===========================================================================
 * CLASS ZkReporterInstance
 *=========================================================================*/

/*
  This class is used by the RZKErrorReporter to get a concrete instance of an 
  error reporter. 

  It has to be explicitly created, and the reporter prototype has to be provided
  at creation to ensure only one prototype is used.
*/

class ZkReporterInstance 
    : public ZkSingleton< ZkReporterInstance > 
{

public: 

    /*----- member functions -----*/
    
	// Purpose:  Creates the only instance of the reporter.
    //           An auto_ptr is asked for to the caller has to give up the
    //           prototype instance. (no leaks, no scope issues)
	// Promise:  Will get rid of the given instance when it dies.
	// Requires: A valid pointer to a reporter prototype.
    ZkReporterInstance( ZKSTD::auto_ptr< ZkAbstractReporter> inst );
    virtual ~ZkReporterInstance();

	// Purpose:  Called by RZKErrorReporter to have the error reported.
    //           It is up to the concrete reporter to handle the error.
	static void ShowError( const IZkException& in_except );
     

protected: 

    ZkAbstractReporter* m_reporter;
};


#endif
