// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// DZkError.cpp: implementation of the DZkError class.
//
//////////////////////////////////////////////////////////////////////

#include "DZkError.h"
#include <zkLog.h>
#include <clientdebug.h>

DZkError::DZkError(const DZkError& rhs) 
    AUTO_INSTANCE_COPY_CTOR1(DZkError, rhs)
{
	DZkError::operator=(rhs);
}

DZkError::DZkError()
{
	T_ZkError::flags = 0;	
	T_ZkError::msgtype = 0;
	T_ZkError::log = NULL;

	memset( &context, 0, sizeof(T_ZkError::context) );
}

DZkError::~DZkError()
{	
}

DZkError& DZkError::operator=(const DZkError& rhs)
{
	if(this == &rhs) return *this; // address equality

	T_ZkError::flags = 0;	
	T_ZkError::msgtype = 0;
	T_ZkError::log = NULL;

	T_ZkError::flags   = rhs.T_ZkError::flags;
	T_ZkError::msgtype = rhs.T_ZkError::msgtype;	
	
	memcpy( &context, &rhs.context, sizeof(T_ZkError::context) );

	return *this;
}


ZKSTD::string DZkError::getfnc() const
{
	return context.detail.func;
}

void DZkError::setfunction(const char* in_pszFunction)
{
	ZKASSERT( ZKIsValidString(in_pszFunction) );
	strncpy(context.detail.func, in_pszFunction, ZKS_ERR_FUNCNAME_MAX);
}

unsigned long DZkError::geterrcode() const
{
	return context.code;
}

void DZkError::seterrorcode(const unsigned long in_code)
{
	context.code = in_code;
}

ZKSTD::string DZkError::getdetail() const 
{
	return context.detail.mesg;
}

void DZkError::setdetail(const char* in_pszDetail)
{
	ZKASSERT( ZKIsValidString(in_pszDetail) );	
	strncpy(context.detail.mesg, in_pszDetail, ZKS_ERR_MESG_MAX);
}
	

void DZkError::Dump()
{
	ZKTRACE4("Flags [%d] ErrorCode [%ld] function[%s] detail [%s]\n",
			T_ZkError::flags,
			geterrcode(),
			getfnc().c_str(),
			getdetail().c_str());
	
}

#ifdef INTERNAL_BUILD

void DZkError::UnitTest()
{
	DZkError zkError1;
	zkError1.seterrorcode( 69 );
	zkError1.setfunction( "DZkError::UnitTest()" );
	zkError1.setdetail( "This is the unit test for DZkError." );

	DZkError zkError2( zkError1 );	// will call operator=

	ZKTEST( zkError1.geterrcode() == zkError2.geterrcode() );
	ZKTEST( zkError1.getfnc() == zkError2.getfnc() );
	ZKTEST( zkError1.getdetail() == zkError2.getdetail() );

	ZKTRACE0( "The following 2 dumps should be exactly the same."
				"If not, DZkError::UnitTest() hasn't succeeded." );
	zkError1.Dump();
	zkError2.Dump();
}

#endif // INTERNAL_BUILD
