/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
	A simple class which is used to obtain a pointer
	to an existing concrete IZkErrorReporter object
	during the lifetime of the client application.
*/

#ifndef _ZkErrorHandler_H__
#define _ZkErrorHandler_H__

#include "ErrorHandling.h"
#include <client.h>
#include <AutoInstCount.h>

class IZkErrorReporter;
class RZkErrorReporter;

class ZKEXCEPTION_DLL ZkErrorHandler AUTO_INSTANCE_COUNTING1( ZkErrorHandler )
{
public:

	// PURPOSE: To return a pointer to a previously created object
	// REQUIRE:	Init() be called previously. Though this could of been
	//			done internally, we use the Init/Term idiom since our
	//			memory leak reporting tools handle that better.	
	static IZkErrorReporter* GetIZkErrorReporter() ;

	// PURPOSE: To create the instance of the reporting object for future use
	// PROMISE:	To use new
	static void Init() throw(ZKSTD::bad_alloc);

	// PURPOSE: To free the memory allocated by Init()
	// REQUIRE:	Init() be successfully called previously
	// PROMISE: To use SAFEDELETE
	static void Term();

#ifdef INTERNAL_BUILD
	static void UnitTest();
#endif // INTERNAL_BUILD

private:
	
	ZkErrorHandler();
	ZkErrorHandler& operator=(const ZkErrorHandler&);

	static RZkErrorReporter* m_pErrorReporterInstance;
};

#endif	// _ZkErrorHandler_H__
