/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/* Case-insensitive string - Mario Contestabile 30 March 2000

	Provides a string compatible class 'ci_string' which is case insensitive

	Test19 demonstrates

	See Herb Sutter's 'Exceptional C++' book for this example

	Limitations:
	I haven't determined how to construct one of these from another ZKSTD::string
	so use a ".."
	
*/

#ifndef _ZKS_CISTRING_H__
#define _ZKS_CISTRING_H__

#ifdef ZK_LINUX
#define strnicmp strncasecmp
#endif

namespace ZKS
{ 

struct ci_char_traits : public ZKSTD::char_traits<char>
                // just inherit all the other functions
                //  that we don't need to override
{
	static bool eq( char c1, char c2 ) {
	  return tolower(c1) == tolower(c2);
	}

	static bool ne( char c1, char c2 ) {
	  return tolower(c1) != tolower(c2);
	}

	static bool lt( char c1, char c2 ) {
	  return tolower(c1) < tolower(c2);
	}

	static int compare( const char* s1,
						const char* s2,
						size_t n ) {
	  return strnicmp( s1, s2, n );
			 // if available on your compiler,
			 //  otherwise you can roll your own
	}

	static const char*
	find( const char* s, int n, char a ) {
	  while( n-- > 0 && tolower(*s) != tolower(a) ) {
		  ++s;
	  }
	  return s;
	}
};

typedef ZKSTD::basic_string< char, ci_char_traits >	ci_string;


}	//namespace ZKS

#endif	// CISTRING
