/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/* cross platform debugging output - Mario Contestabile 21 Nov 1999

The objective of the DebugStreamBuf and DebugStream classes
is to provide a very easy way for debugger output on any platform.
Nothing in this translation unit is not dictated by the C++ standard.

  Here's an example providing debugging under Win32:

	#include <windows.h>
	#include <debugstream.h>
	
	class DebugStreamWin32{
	public:
		void print(char* s){
			OutputDebugString(s);
		}
	};

	DebugStream<DebugStreamWin32> dout;

	dout << "calling foo()" << endl;	
	dout << 1 << 2 << 0x12 << "test" << endl;

A simpler variant of this concept was presented by James Curran in microsoft.public.vc.stl
from which I've generated this version. If you want alot more information on iostreams
feel free to look at http://www.informatik.uni-konstanz.de/~kuehl/c++/iostream/

  The DebugStream class is in the ZKS namespace

*/
#ifndef _DEBUGSTREAM_HPP__
#define _DEBUGSTREAM_HPP__

#include <strstream>
#include <client.h>
using ZKSTD::ostream;
using ZKSTD::endl;


#ifndef NDEBUG

namespace{
template<typename T>
class DebugStreamBuf : public ZKSTD::strstreambuf{

	T m_dbg;

	public:
		DebugStreamBuf(){}
		DebugStreamBuf(const T& dbg) : m_dbg(dbg) {}

	protected:
		virtual int sync(){
			sputc('\0');
			m_dbg.print(str());
			freeze(false);
			setp(pbase(), pbase(), epptr());
			return 0;
		}
};
}	// unnamed namespace

namespace ZKS{

template<typename T>
class DebugStream : public ostream
{
public:
	DebugStream() : ostream(&m_buf)  {}
	~DebugStream()  { m_buf.pubsync();}

private:
  DebugStreamBuf<T> m_buf;

};

#else // do nothing in release

namespace ZKS{

template<typename T>
class DebugStream
{
public:
  template <typename T>
  inline const DebugStream& operator<<(T) const  {return(*this);}

  typedef ZKSTD::basic_ostream<char>& (__cdecl *endl_type)(ZKSTD::basic_ostream<char>&);

  inline const DebugStream& operator<<(const endl_type T) const  {return(*this);}
};

#endif	// NDEBUG
}		// namespace ZKS
#endif	// _DEBUGSTREAM_HPP__