/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/* cross platform synchronization - Mario Contestabile 22 Nov 1999

The objective of the Lock_Guard class is to provide a standard method
of preventing re-entrancy problems without using any os specific calls.

I got this class straight out of a CUJ.
Here's an example to solve multithread issues under Win32

	class ZKCriticalSection{
	public:
		ZKCriticalSection() { InitializeCriticalSection(&cs); }
		~ZKCriticalSection() { DeleteCriticalSection(&cs); }

		void Lock() { EnterCriticalSection(&cs); }
		void Unlock() { LeaveCriticalSection(&cs); }
	private:
		CRITICAL_SECTION	cs;
	};

	ZKCriticalSection classX::_key;

	classX::foo(){
		...
		Lock_Guard<ZKCriticalSection> gate(_key);
	}

  What I've done is given you a little more flexibility. I've purposely
  used methods Unlock() and Lock(), so if you're using MFC, you don't
  have to write your own ZKCriticalSection class as above, simply use
  CCriticalSection!

*/

#ifndef _LOCK_GUARD_HPP__
#define _LOCK_GUARD_HPP__

namespace ZKS{


template<typename T> 
class Lock_Guard{
public:
	Lock_Guard(T& a_key) : _key(a_key) { lock(); }
	~Lock_Guard() { unlock(); }

	void lock() { _key.Lock(); }
	void unlock() { _key.Unlock(); }
private:
	T&	_key;
};

}		// namespace ZKS

#endif	// _LOCK_GUARD_HPP__