// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "clientdefs.h"

#ifdef INTERNAL_BUILD

// This library
#include "math_hlp.h"
#include "date_hlp.h"
#include "bits_hlp.h"
#include "string_hlp.h"
#include "endian.h"
#include "DynamicLibrary.h"
#include "AutoInstCount.h"
#include "ZKInverseSemaphore.h"
#include "CircularVector.h"

#include "../../libzktest/ZkTestBase.h"

#define INTEL_VER_SUPPORTING_HARD_TEMPLATES	500

#include "ci_string.h"
#include <zkThread.h>

// C++
#include <vector>
#include <list>
// C
#ifdef WIN32
#  include <direct.h>
#  define TEST_MODE
#else
#  include <unistd.h>
#  define TEST_MODE ,666
#endif

// Unit test for Duplicate.
void Test()
{
	ZKSTD::vector<int> x;
	x.push_back(1);
	x.push_back(2);
	x.push_back(3);
	x.push_back(3);
	x.push_back(4);
	x.push_back(5);

	int val2 = ZKS::Duplicate<ZKSTD::vector<int> >(x); 
	ZKTEST(val2 == 3);

	ZKSTD::list<float> f;
	f.push_back(1);
	f.push_back(2);
	f.push_back(3);
	f.push_back(3);
	f.push_back(4);
	f.push_back(5);
	float val3 = ZKS::Duplicate<ZKSTD::list<float> >(f); 
	ZKTEST(val3 == 3.00);
}

// Unit test for Sum.
// XXX Review the implementation of this function.
void Test2()
{
	ZKTEST(15 == ZKS::sum(5));
	ZKTEST(21 == ZKS::sum(6));
	ZKTEST(55 == ZKS::sum(10));
	ZKTEST(66 == ZKS::sum(11));
	ZKTEST(5050 == ZKS::sum(100));
	ZKTEST(0 == ZKS::sum(0));
	ZKTEST(-1 == ZKS::sum(-1));
	ZKTEST(15 ==	ZKS::sum<long>(5));
	ZKTEST(15 == ZKS::sum<unsigned long>(5));
	ZKTEST(15 == ZKS::sum<unsigned short>(5));
	ZKTEST(15 == ZKS::sum<short>(5));
	ZKTEST(15 == ZKS::sum<int>(5));
	ZKTEST(15 == ZKS::sum<unsigned int>(5));
}

// Unit test for IsLeapYear.
void Test3()
{

	ZKTEST(ZKS::IsLeapYear(2000));
	ZKTEST(!ZKS::IsLeapYear(1999));
	ZKTEST(!ZKS::IsLeapYear(1998));
	ZKTEST(!ZKS::IsLeapYear(1997));
	ZKTEST(ZKS::IsLeapYear(1996));

}

// Unit test for DayOfWeek
void Test4()
{

	ZKTEST(ZKS::DayOfWeek(2,  1,  2000) == 6); // sunday
	ZKTEST(ZKS::DayOfWeek(28, 1,  1980) == 0); // monday
	ZKTEST(ZKS::DayOfWeek(20, 9, 2080) == 4); // friday
	ZKTEST(ZKS::DayOfWeek(9,  3, 2038) == 1); // tuesday
}

// Unit test for is_bit_on
void Test5()
{

	ZKTEST(ZKS::is_bit_on(18,   1));
	ZKTEST(ZKS::is_bit_on(18,   4));
	ZKTEST(ZKS::is_bit_on(64,   6));
	ZKTEST(ZKS::is_bit_on(0xFF, 0));
	ZKTEST(ZKS::is_bit_on(0xFF, 1));
	ZKTEST(ZKS::is_bit_on(0xFF, 2));
	ZKTEST(ZKS::is_bit_on(0xFF, 3));
	ZKTEST(ZKS::is_bit_on(0xFF, 4));
	ZKTEST(ZKS::is_bit_on(0xFF, 5));
	ZKTEST(ZKS::is_bit_on(0xFF, 6));
	ZKTEST(ZKS::is_bit_on(0xFF, 7));	
}

// Unit test for set_bit_on/set_bit_off
void Test6()
{

	unsigned int x = 0;

	ZKS::set_bit_on(x, 0);
	ZKTEST(ZKS::is_bit_on(x, 0));

	ZKS::set_bit_on(x, 1);
	ZKTEST(ZKS::is_bit_on(x, 1));

	ZKS::set_bit_on(x, 2);
	ZKTEST(ZKS::is_bit_on(x, 2));

	ZKS::set_bit_on(x, 3);
	ZKTEST(ZKS::is_bit_on(x, 3));

	ZKS::set_bit_on(x, 4);
	ZKTEST(ZKS::is_bit_on(x, 4));

	ZKS::set_bit_on(x, 32);
	ZKTEST(ZKS::is_bit_on(x, 32));

	ZKS::set_bit_off(x, 0);
	ZKS::set_bit_off(x, 1);
	ZKS::set_bit_off(x, 2);
	ZKS::set_bit_off(x, 3);
	ZKS::set_bit_off(x, 4);
	ZKS::set_bit_off(x, 32);
	ZKTEST(!ZKS::is_bit_on(x, 0));
	ZKTEST(!ZKS::is_bit_on(x, 1));
	ZKTEST(!ZKS::is_bit_on(x, 2));
	ZKTEST(!ZKS::is_bit_on(x, 3));
	ZKTEST(!ZKS::is_bit_on(x, 4));
	ZKTEST(!ZKS::is_bit_on(x, 32));
}

// Unit test for getbits
void Test7()
{

	unsigned int x = 16; // 10000
	ZKTEST(ZKS::getbits(x, 4, 1) == 1);
	ZKTEST(ZKS::getbits(x, 3, 1) == 0);
	ZKTEST(ZKS::getbits(x, 2, 1) == 0);
	ZKTEST(ZKS::getbits(x, 1, 1) == 0);
	ZKTEST(ZKS::getbits(x, 0, 1) == 0);

	x = 1; // 00001
	ZKTEST(ZKS::getbits(x, 0, 1) == 1);
	ZKTEST(ZKS::getbits(x, 1, 1) == 0);
	ZKTEST(ZKS::getbits(x, 2, 1) == 0);
	ZKTEST(ZKS::getbits(x, 3, 1) == 0);
	ZKTEST(ZKS::getbits(x, 4, 1) == 0);

	x = 21; // 10101
	ZKTEST(ZKS::getbits(x, 0, 1) == 1);
	ZKTEST(ZKS::getbits(x, 1, 1) == 0);
	ZKTEST(ZKS::getbits(x, 2, 1) == 1);
	ZKTEST(ZKS::getbits(x, 3, 1) == 0);
	ZKTEST(ZKS::getbits(x, 4, 1) == 1);

	ZKTEST(ZKS::getbits(x, 2, 3) == 5); // 101 LS
	ZKTEST(ZKS::getbits(x, 1, 2) == 1); //  01 LS
	ZKTEST(ZKS::getbits(x, 4, 5) == 21);
	ZKTEST(ZKS::getbits(x, 4, 3) == 5); // 101 MS
}

// Unit test for ZKFACTORIAL and factorial.
void Test8()
{
	
	ZKTEST(ZKSFACTORIAL(6) == 720);
	ZKTEST(ZKSFACTORIAL(2) == 2);
	ZKTEST(ZKSFACTORIAL(9) == 362880);	

	ZKTEST( 720 == ZKS::factorial(6) );
	
	ZKTEST( 1 == ZKS::factorial(-1) );	

	ZKTEST( 1 == ZKS::factorial(0) );

	ZKTEST( 1 == ZKS::factorial(1) );	
}

// Unit test for trim/pad/trimLeadingWhite
void Test9()
{
	ZKSTD::string testString = "   Hello";
	ZKS::trimLeadingWhite(testString);
	ZKTEST(testString == "Hello");

	ZKS::trimLeadingWhite(testString);
	ZKTEST(testString == "Hello");

	testString = " 1 2 3 ";
	ZKS::trimLeadingWhite(testString);
	ZKTEST(testString == "1 2 3 ");

	ZKS::trim(testString);
	ZKTEST(testString == "1 2 3");

	testString = "123 ";
	ZKS::trim(testString);
	ZKTEST(testString == "123");

	testString = " 1 2 3   ";
	ZKS::trim(testString);
	ZKTEST(testString == " 1 2 3");

	testString = "123";
	ZKS::pad(testString, 4, 'x');
	ZKTEST(testString == "123x");
}

// Unit test for SplitOnCharacter.
void Test10()
{
	ZKSTD::string testString = ";123;abc;456;;";
	ZKSTD::vector<ZKSTD::string> testVector;

	ZKS::SplitOnCharacter(testString, testVector);

	ZKTEST(testVector[0] == "123");
	ZKTEST(testVector[1] == "abc");
	ZKTEST(testVector[2] == "456");
	ZKTEST(testVector.size() == 3);

	testString = "'123;abc'456;;";
	testVector.clear();

	ZKS::SplitOnCharacter(testString, testVector, '\'');

	ZKTEST(testVector[0] == "123;abc");
	ZKTEST(testVector[1] == "456;;");
	ZKTEST(testVector.size() == 2);
}

// Unit test for Little/big endian functionnality.
void Test11()
{
#ifdef LITTLE_ENDIAN
	ZKTEST( isLittleEndian() );
#else
	ZKTEST( !isLittleEndian() );
#endif

   // volatile so the compiler doesn't cut corners
   const volatile uint16_t numIts = 255;

   for (uint16_t i=0;i<numIts;++i)
   {
#ifdef TEST_NATIVE
      uint16_t bn = i;
#else
      big_endian<uint16_t> bn = i;
#endif

      uint16_t bcheck = bn;
      bn = bn + 1;
      ZKTEST(bcheck + 1 == bn);

      bcheck = bn;
      bn = bn * bn;
	  // carefull not to overflow 255*255 = 65025, or numIts * numIts must be < USHRT_MAX uint16_t
      ZKTEST(bcheck * bcheck == bn); 

      bcheck = bn;
      bn = bn/2;
      ZKTEST(bcheck/2 == bn);

	  // little_endian template test
#ifdef TEST_NATIVE
      uint32_t ln = i;
#else
      little_endian<uint32_t> ln = i;
#endif

      uint32_t lcheck = ln;
      ln = ln + 1;
      ZKTEST(lcheck + 1 == ln);

      lcheck = ln;
      ln = ln * ln;
      ZKTEST(lcheck * lcheck == ln);

      lcheck = ln;
      ln = ln/2;
      ZKTEST(lcheck/2 == ln);
   }
}

// Unit test for ascitohex
void Test12()
{
	ZKTEST( ZKS::ascitohex("0xABC") == 0xabc);
	
	ZKTEST( ZKS::ascitohex("0xFF") == 0xff);

	ZKTEST( ZKS::ascitohex("FF") == 0xff);

	ZKTEST( ZKS::ascitohex("mario") == 0x00);
}

// Unit test for greatest common divisor (gcd)
void Test13()
{
	ZKTEST(11 == ZKS::gcd(55, 11));

	ZKTEST(25 == ZKS::gcd(25, 550));

	ZKTEST(-1 == ZKS::gcd(-1, -2));

	ZKTEST(-1 == ZKS::gcd(-1, -1));

	ZKTEST(0 == ZKS::gcd(0, 0));

	ZKTEST(1 == ZKS::gcd(1, 1));
}

// Unit test for replace_all
void Test14()
{
	const ZKSTD::string str = "salut lynee, c'est mario, oui Mario, pas oiraM! ";

	ZKSTD::string stuff = ZKS::replace_all(str, "mario", "1234567");

	ZKTEST(stuff == "salut lynee, c'est 1234567, oui Mario, pas oiraM! ");

	stuff = ZKS::replace_all(str, "mario", "");

	ZKTEST(stuff == "salut lynee, c'est , oui Mario, pas oiraM! ");
}

//////
// XXX MOVED to ClientBase --ol
//////
// // Unit test for GetTempFileName/filecopy
// void Test15()
// {
// 	char file1 [L_tmpnam * 2] = { 0 };
// 	char file2 [L_tmpnam * 2] = { 0 };
// 	char file3 [L_tmpnam * 2] = { 0 };

// 	ZKS::GetTempFileName(file1);
// 	ZKS::GetTempFileName(file2);
// 	ZKS::GetTempFileName(file3);

// 	ZKTEST(file1);
// 	ZKTEST(file2);
// 	ZKTEST(file3);

// 	FILE* f1 = NULL;

// 	if(file1 && file2)
// 	{
// 		f1 = fopen(file1, "wb");
// 		FILE* f2 = fopen(file2, "w+b");
// 		ZKTEST(f1);
// 		ZKTEST(f2);
// 		if(f1 && f2)
// 		{
// 			ZKTEST( 0 == ZKS::FileCopy((FILE*)NULL, (FILE*)NULL) );
			
// 			fprintf(f2, "This is a test");

// 			ZKTEST( 1 == ZKS::FileCopy(f1, f2) ); // write from f2 into f1

// 			fclose(f2);

// 			// File should be equal with itself
// 			ZKTEST(ZKS::AreFilesEqual(file2, file2 ) ) ;
// 		}
// 	}

// 	// File that didn't exist yet shouldn't be equal
// 	ZKTEST( 0 == ZKS::AreFilesEqual(file2, file3 ) ) ;

// 	// Create a different file to compare with file2
// 	FILE* f3 = fopen(file3, "w+b" ) ;
// 	fprintf( f3, "Different than f2" ) ;
// 	fclose( f3 ) ;
// 	ZKTEST( 0 == ZKS::AreFilesEqual(file2, file3 ) ) ;

// 	char stream_out[L_tmpnam * 2] = { 0 };
// 	ZKS::GetTempFileName(stream_out);

// 	// "This is a test" will be in 'file1' only if f1 has been closed, which it has not yet
// 	ZKSTD::ifstream ifs(file1); 
// 	ZKSTD::ofstream of(stream_out);
	
// 	ZKTEST(0 == ZKS::FileCopy(of, ifs));

// 	// Close f1 and compare with f2
// 	fclose(f1);
// 	ZKTEST(ZKS::AreFilesEqual(file1, file2 ) ) ;

// 	of.clear(); ifs.clear();
// 	ZKTEST(1 == ZKS::FileCopy(of, ifs));

// 	ZKSTD::ifstream ifsbad("");
// 	ZKTEST(0 == ZKS::FileCopy(of, ifsbad));

// 	ZKSTD::ofstream ofsbad("");
// 	ZKTEST(0 == ZKS::FileCopy(ofsbad, ifsbad));

// 	ZKTEST(ZKS::FileExists(file1));
// 	ZKTEST(ZKS::FileExists(file2));

// 	ZKTEST(!ZKS::FileExists(""));

// 	ifs.close();
// 	of.close();

// 	ZKTEST(0 == mkdir("/tests_t" TEST_MODE));

// 	ZKTEST(1 == ZKS::FileCopyToDir(file1, "/tests_t") );
	
// 	char buf[FILENAME_MAX] = { 0 };
// 	snprintf(buf, sizeof buf, "/tests_t/%s", file1);
// 	ZKTEST(0 == remove(buf));
// 	ZKTEST(0 == rmdir("/tests_t"));
// }

// // Unit test for GetTempFileName/ReadBinaryFile/readtestfile/FileCopyWithVerification
// void Test16()
// {

// 	char stream_outbin[L_tmpnam * 2] = { 0 };
// 	char stream_outtext[L_tmpnam * 2] = { 0 };
// 	char stream_outtext2[L_tmpnam * 2] = { 0 };
// 	char stream_outtext3[L_tmpnam * 2] = { 0 };
// 	char stream_outtext4[L_tmpnam * 2] = { 0 };
// 	char stream_outtext5[L_tmpnam * 2] = { 0 };

// 	ZKS::GetTempFileName(stream_outbin);
// 	ZKS::GetTempFileName(stream_outtext);
// 	ZKS::GetTempFileName(stream_outtext2);
// 	ZKS::GetTempFileName(stream_outtext3);
// 	ZKS::GetTempFileName(stream_outtext4);
// 	ZKS::GetTempFileName(stream_outtext5);

// 	ZKSTD::string str;
	

// 	{ // create the binary file on disk
// 	ZKSTD::ofstream of(stream_outbin, ZKSTD::ios::binary);
// 	of << 0x01 << 0x02 << 0x03 << 0x04 << 0x05 << 0x06 << 0x07 << 0x08 << 0x09 << 0x10 << 0x11 << 0x12 << 0x13;
// 	}
	
// 	{ // read the same binary file into memory
// 	unsigned char* pbuf = NULL;
// 	size_t siz = 0;

// 	ZKTEST(ZKS::ReadBinaryFile(stream_outbin, &pbuf, siz));
// 	ZKTEST(17 == siz);
// 	ZKTEST(0  == memcmp("12345678916171819", pbuf, 13) );
// 	SAFEFREE(pbuf);
// 	}


// 	{ // create the text file on disk
// 	ZKSTD::ofstream of(stream_outtext);
// 	of << "This is a story";
// 	of << "Of a man named Brady";
// 	of << "Who should of been neutered\r\n";
// 	of << "This is a lady\r\n";
// 	ZKTEST(!ZKS::ReadTextFile(stream_outtext, str));
// 	}

// 	{ // read the same text file into memory
// 	str = "";
// 	ZKTEST(ZKS::ReadTextFile(stream_outtext, str));
// 	ZKTEST(str 
// 		== "This is a storyOf a man named BradyWho should of been neutered\r\nThis is a lady\r\n");	
// 	}

// 	{ 
// 	ZKTEST( !ZKS::FileCopyWithVerification(stream_outbin, stream_outtext, true) );
// 	ZKTEST( ZKS::FileCopyWithVerification(stream_outbin, stream_outtext, false) );
// 	ZKTEST(0 == remove(stream_outtext));	
// 	ZKTEST( ZKS::FileCopyWithVerification(stream_outbin, stream_outtext, true) );
// 	ZKTEST(0  == remove(stream_outbin));
// 	}


// 	{
// 	ZKSTD::ofstream of(stream_outtext2);
// 	of << "This is another story";	
// 	ZKTEST(!ZKS::ReadTextFile(stream_outtext2, str));
// 	}

// 	str = "";
// 	ZKTEST(ZKS::ReadTextFile(stream_outtext2, str));
// 	ZKTEST(str == "This is another story\n");
// 	ZKTEST(0 == remove(stream_outtext2));

// 	//////////////////////////////////////////////////////////////////////////////////////////

// 	// Make sure that we won't get duplication of the eol delimiter wanted.
// 	{
// 		ZKSTD::ofstream of(stream_outtext3);
// 		of << "Once upon\na time\r\nat 5000ft under a big cu...\n";
// 		ZKTEST(!ZKS::ReadTextFile(stream_outtext3, str));
// 	}
// 	str = "";
// 	ZKTEST(ZKS::ReadTextFile(stream_outtext3, str, "\r\n", '\n'));
// 	ZKTEST(str == "Once upon\r\na time\r\nat 5000ft under a big cu...\r\n");
// 	ZKTEST(0 == remove(stream_outtext3));

// 	//////////////////////////////////////////////////////////////////////////////////////////

// 	{
// 		ZKSTD::ofstream of(stream_outtext4);
// 		of << "The vario\rwas crazy and the\ncloud base was at 9500ft.";
// 		ZKTEST(!ZKS::ReadTextFile(stream_outtext4, str));
// 	}
// 	str = "";
// 	ZKTEST(ZKS::ReadTextFile(stream_outtext4, str, "\r", '\n'));
// 	ZKTEST(str == "The vario\rwas crazy and the\rcloud base was at 9500ft.\r");
// 	ZKTEST(0 == remove(stream_outtext4));

// 	//////////////////////////////////////////////////////////////////////////////////////////

// 	{
// 		ZKSTD::ofstream of(stream_outtext5);
// 		of << "I determined\r\r\nmy ceiling at 8000ft\n\n\rso I thermalled the best I could.\r";
// 		ZKTEST(!ZKS::ReadTextFile(stream_outtext5, str));
// 	}
// 	str = "";
// 	ZKTEST(ZKS::ReadTextFile(stream_outtext5, str, "\r", '\n'));
// 	ZKTEST(str 
// 			== "I determined\rmy ceiling at 8000ft\r\r\rso I thermalled the best I could.\r");
// 	ZKTEST(0 == remove(stream_outtext5));

// }

// Unit test for ci_string : case-insensitive strings.
void Test19()
{
	ZKS::ci_string s( "AbCdE" );

	// case insensitive
    ZKTEST( s == "abcde" );
    ZKTEST( s == "ABCDE" );

    // still case-preserving, of course
    ZKTEST( strcmp( s.c_str(), "AbCdE" ) == 0 );
	ZKTEST( strcmp( s.c_str(), "abcde" ) != 0 );
}

//////
// XXX MOVED to ClientBase --ol
//////
// // Unit test for GetLinesExceedingWidth
// void Test20()
// {
// 	ZKSTD::map<size_t, ZKSTD::string> mymap;
// 	const size_t nBadFile = ZKS::GetLinesExceedingWidth("resource.h", 50, mymap);
// 	if(nBadFile)
// 	{
// 		// I've removed the htmltag helper class and code to show
// 		// results because it wasn't used enough to warrent its usage.  Generally
// 		// unit test should be silent, not spam people who run it,
// 		// just assert is some expected value doesn't happen. -Andrew


// 		for(ZKSTD::map<size_t, ZKSTD::string>::const_iterator it = mymap.begin(); 
// 			it != mymap.end(); 
// 			++it)
// 		{
// 			ZKTRACE2( "Found line too long %d %s\n", (*it).first, (*it).second.c_str() );
// 		}
// 	}
// }

// // Unit test for ExtractFileName
// void Test21()
// {
// 	// Let's build a vector of dummy files with random path (absolute/relative/none)
// 	ZKSTD::vector<ZKSTD::string> fileNameVector;
// 	fileNameVector.push_back( "c:\\windoze\\reboot.dll" );
// 	fileNameVector.push_back( "pagefile.sys" );
// 	fileNameVector.push_back( "zk\\freedom\\client.cpp" );
// 	fileNameVector.push_back( "file" );

// 	// Let's extract the file name.
// 	ZKSTD::vector<ZKSTD::string> fileNameOnlyVector;
// 	ZKS::ExtractFileName( fileNameVector, fileNameOnlyVector );

// 	// Now, make sure it works.
// 	ZKTEST( fileNameVector.size() == fileNameOnlyVector.size() );
// 	for( ZKSTD::vector<ZKSTD::string>::const_iterator it = fileNameOnlyVector.begin();
// 			it != fileNameOnlyVector.end();
// 			++it )
// 				ZKTEST( ( (*it).find( ":" ) ==  ZKSTD::string::npos )
// 							&& ( (*it).find( "\\" ) ==  ZKSTD::string::npos ) );
// }

// Unit test for InsertInFront
void Test22()
{
	ZKSTD::vector<ZKSTD::string> origStringVector;
	origStringVector.push_back( "Hello" );
	origStringVector.push_back( "HowAreYou" );
	origStringVector.push_back( "Bye" );

	ZKSTD::vector<ZKSTD::string> modStringVector;
	ZKS::InsertInFront( origStringVector, modStringVector, "**" );

	ZKTEST( origStringVector.size() == modStringVector.size() );

	ZKTEST( modStringVector[0] == "**Hello" );
	ZKTEST( modStringVector[1] == "**HowAreYou" );
	ZKTEST( modStringVector[2] == "**Bye" );

	ZKS::InsertInFront( origStringVector, modStringVector );

	ZKTEST( modStringVector[0] == "__Hello" );
	ZKTEST( modStringVector[1] == "__HowAreYou" );
	ZKTEST( modStringVector[2] == "__Bye" );

}

// Unit test for RDynamicLibrary
// This unit test is specific for Windows.
// Linux should change the string so it loads a valid library.
void Test23()
{
#ifdef WIN32
	char strDynLibName[] = { "ws2_32.dll" };
	char strValidFunc[] = { "accept" };
	char strInvalidFunc[] = { "NotValidFunc" };
	try
	{
		ZKS::RDynamicLibrary dynLib;
		ZKTEST( dynLib.GetProcAddr( 1 ) == NULL );
		ZKTEST( dynLib.GetProcAddr( strInvalidFunc ) == NULL );
		ZKTEST( dynLib.GetProcAddrCached( 1 ) == NULL );
		// don't know how to use procID
//		ZKTEST( dynLib.GetProcAddrCached( "SomeFcn", procID ) == NULL );
		ZKTEST( !dynLib.IsLoaded() );

		ZKTEST( dynLib.Load( strDynLibName ) );
		ZKTEST( dynLib.IsLoaded() );
		ZKTEST( dynLib.GetProcAddr( 1 ) );
		ZKTEST( dynLib.GetProcAddr( strValidFunc ) );
		ZKTEST( dynLib.GetProcAddrCached( 1 ) );
		// don't know how to use procID
//		ZKTEST( dynLib.GetProcAddrCached( "AddAtomW" ) );
		ZKTEST( dynLib.Unload() );
		ZKTEST( !dynLib.IsLoaded() );
	}
	catch( ZKSTD::exception&)
	{
		ZKTEST( false );
	}

	try
	{
		ZKS::RDynamicLibrary dynLib( strDynLibName );
		ZKTEST( dynLib.IsLoaded() );
	}
	catch( ZKSTD::exception& )
	{
		ZKTEST( false );
	}
#endif
}

// Unit test for LookForLine
void Test24()
{
	// Note:
	// If LookForLine find a \r\n, the working buffer returned is empty.
	// If it finds a \n, the working buffer contain the reminder of the
	// string pass in parameter.
	// LookForLine appends the string pass in parameter to the working buffer.
	
	const ZKSTD::string strInitialBuffer( "This is my\r\nbuffer." );
	const ZKSTD::string strInitialBuffer2( "This is my\nbuffer\non three lines." );
	const ZKSTD::string strInitialBuffer3( "This is my\nbuffer\r\non three lines." );
	const ZKSTD::string strInitialBuffer4( "This is my buffer on a single line." );
	const ZKSTD::string strInitialBuffer5( "This is my buffer on a single line.\r\n" );
	ZKSTD::string strLineReceived;
	ZKSTD::string strWorkBuf;
	ZKTEST( ZKS::LookForLine( strInitialBuffer.c_str(), 
								strInitialBuffer.length(), 
								strLineReceived, 
								strWorkBuf ) );

	ZKTEST( strLineReceived == "This is my\r\n" );
	ZKTEST( strWorkBuf.empty() );

	ZKTEST( ZKS::LookForLine( strInitialBuffer2.c_str(),
								strInitialBuffer2.length(),
								strLineReceived,
								strWorkBuf ) );
	ZKTEST( strLineReceived == "This is my" );
	ZKTEST( strWorkBuf == "buffer\non three lines." );
	ZKTEST( ZKS::LookForLine( strWorkBuf.c_str(),
								strInitialBuffer2.length(),
								strLineReceived,
								strWorkBuf ) );
	ZKTEST( strLineReceived == "buffer" );
	// strWorkBuf will contain the remaining of the line of the value of the buffer pass to LookForLine
	// + the value of the buffer pass to LookForLine.
	ZKTEST( strWorkBuf == ( ZKSTD::string( "on three lines." ) + "buffer\non three lines." ) );
	strWorkBuf.erase();

	ZKTEST( ZKS::LookForLine( strInitialBuffer3.c_str(),
								strInitialBuffer3.length(),
								strLineReceived,
								strWorkBuf ) );
	ZKTEST( strLineReceived == "This is my\nbuffer\r\n" );
	ZKTEST( strWorkBuf.empty() );

	ZKTEST( !ZKS::LookForLine( strInitialBuffer4.c_str(),
								strInitialBuffer4.length(),
								strLineReceived,
								strWorkBuf ) );
	ZKTEST( strLineReceived == strInitialBuffer4 );
	ZKTEST( strWorkBuf == strInitialBuffer4 );

	strWorkBuf.erase();
	ZKTEST( ZKS::LookForLine( strInitialBuffer5.c_str(),
								strInitialBuffer5.length(),
								strLineReceived,
								strWorkBuf ) );
	ZKTEST( strLineReceived == strInitialBuffer5 );
	ZKTEST( strWorkBuf.empty() );
}

// Unit test for GetDomain
void Test25()
{
	ZKSTD::string strDomain = ZKS::ExtractDomain("www.freedom.net");
	ZKTEST(strDomain == "freedom.net");

	strDomain = ZKS::ExtractDomain("http://erik.is.a.very.young.lad.com");
	ZKTEST(strDomain == "lad.com");

	strDomain = ZKS::ExtractDomain("http://mutt.freedom.net:8080");
	ZKTEST(strDomain == "freedom.net");

	strDomain =  ZKS::ExtractDomain("www.FREEDOM.neT/sdasda:sdajhf/asd");
	ZKTEST(strDomain == "FREEDOM.neT");

	strDomain = ZKS::ExtractDomain("www.good.url/");
	ZKTEST(strDomain == "good.url");

	strDomain = ZKS::ExtractDomain( "freedom/sdsfsf");
	ZKTEST(strDomain == "freedom");

	strDomain = ZKS::ExtractDomain( "http://devnet/bugfree");
	ZKTEST(strDomain == "devnet");

	strDomain = ZKS::ExtractDomain( "devnet");
	ZKTEST(strDomain == "devnet");

	strDomain = ZKS::ExtractDomain( "http://devnet/bugfree");
	ZKTEST(strDomain == "devnet");

	strDomain = ZKS::ExtractDomain("");
	ZKTEST(strDomain == ZK_LOCAL_COMPUTER);

	strDomain = ZKS::ExtractDomain(strDomain);
	ZKTEST(strDomain == ZK_LOCAL_COMPUTER);
}

// Unit test for EscapeString
void Test26()
{
	ZKSTD::string strEscaped = ZKS::EscapeString("hello");
	ZKTEST(strEscaped == "hello");

	strEscaped = ZKS::EscapeString("Hello there");
	ZKTEST(strEscaped == "Hello+there");

	strEscaped = ZKS::EscapeString("H%i&");
	ZKTEST(strEscaped == "H%25i%26");

	strEscaped = ZKS::EscapeString("Hi)");
	ZKTEST(strEscaped == "Hi%29");

	strEscaped = ZKS::EscapeString("Hi/");
	ZKTEST(strEscaped == "Hi%2F");

	strEscaped = ZKS::EscapeString("Hi*");
	ZKTEST(strEscaped == "Hi%2A");

	strEscaped = ZKS::EscapeString("Hi`");
	ZKTEST(strEscaped == "Hi%60");

}

// Unit test for UnEscapeString
void Test27()
{
	ZKSTD::string strUnEscaped = ZKS::UnEscapeString("badString%2");
	ZKTEST(strUnEscaped == "");

	strUnEscaped = ZKS::UnEscapeString("bad%GString");
	ZKTEST(strUnEscaped == "");

	strUnEscaped = ZKS::UnEscapeString("Hi%29");
	ZKTEST(strUnEscaped == "Hi)");

	strUnEscaped = ZKS::UnEscapeString("Hi%2F");
	ZKTEST(strUnEscaped == "Hi/");

	strUnEscaped = ZKS::UnEscapeString("Hi%2A");
	ZKTEST(strUnEscaped == "Hi*");

	strUnEscaped = ZKS::UnEscapeString("Hi%60");
	ZKTEST(strUnEscaped == "Hi`");

	ZKSTD::string strEscaped = ZKS::EscapeString("H 7+876kd");
	ZKTEST(ZKS::UnEscapeString(strEscaped) == "H 7+876kd");

	strUnEscaped = ZKS::UnEscapeString("H%25i%26");
	ZKTEST(strUnEscaped == "H%i&");

}

#ifdef INSTANCE_COUNTING
struct Tracked AUTO_INSTANCE_COUNTING1( Tracked )
{
	int m_nDummy;

	static size_t HowMany() { return RInstanceCounter<Tracked>::HowMany(); }
};
#endif // INSTANCE_COUNTING

void Test28()
{
#ifdef INSTANCE_COUNTING
	const Tracked* pTracked1 = new Tracked;
	{
		const Tracked tracked2;

		ZKTEST( Tracked::HowMany() == 2 );
	}

	ZKTEST( Tracked::HowMany() == 1 );

	SAFEDELETE( pTracked1 );

	ZKTEST( Tracked::HowMany() == 0 );
#endif // INSTANCE_COUNTING
};

void * ZK_THREAD_CALLBACK Test29Helper(void *pVoid)
{
	ZKInverseSemaphore* pInvSem = (ZKInverseSemaphore*) pVoid;

	ZKInverseSemaphoreLock lock(*pInvSem);

#ifdef WIN32
	::Sleep(50);
#else 
    usleep(5);
#endif;
    return NULL;
}

void Test29()
{
	ZKInverseSemaphore invSem, invSem2, invSem3;
	{
		ZKInverseSemaphoreLock lock(invSem);
	}
	ZKTEST(invSem.WaitUntilCountIsZero(100));

	ZKTEST(invSem2.WaitUntilCountIsZero(100));
	
	ZKInverseSemaphoreLock lock(invSem2);
	ZKTEST(!invSem2.WaitUntilCountIsZero(100));

    DZkError err;

	zkThreadCreateDetached(&err, Test29Helper, &invSem3);	
#ifdef WIN32
	::Sleep(200);
#else
    usleep(20);
#endif

	ZKTEST(invSem3.WaitUntilCountIsZero(200));
}

void Test30()
{
	ZKSTD::string s("MorPheus");
	ZKS::MakeLower(s);
	ZKTEST( s == "morpheus" );

	ZKS::MakeUpper(s);
	ZKTEST( s == "MORPHEUS" );
}

//////
// XXX MOVED to ClientBase --ol
//////
// void
// Test31()
// {
// #ifdef WIN32

// 	char strOrigFileName[_MAX_PATH] = { 0 };
// 	ZKTEST( ::GetModuleFileName( NULL, strOrigFileName, _MAX_PATH ) );

// 	unsigned char* pBuffer = NULL;
// 	size_t bufSize = 0;
// 	ZKTEST( ZKS::ReadBinaryFile(strOrigFileName, &pBuffer, bufSize) );

// 	ZKTEST( ZKS::GetFileSize(strOrigFileName) == bufSize );

// 	const size_t original_buf_size = bufSize;

// 	char strNewFileName[_MAX_PATH] = { 0 };
// 	::strncpy( strNewFileName, strOrigFileName, _MAX_PATH );
// 	::strncat( strNewFileName, "_", _MAX_PATH);

// 	// Write this.exe into this.exe_
// 	ZKTEST( ZKS::WriteBinaryFile(strNewFileName, pBuffer, bufSize) );	

// 	SAFEFREE(pBuffer);

// 	// Read this.exe_ again
// 	ZKTEST( ZKS::ReadBinaryFile(strNewFileName, &pBuffer, bufSize) );

// 	ZKTEST(bufSize == original_buf_size);

// 	SAFEFREE(pBuffer);
// #else
// #warning "Please write a test for your platform"
// #endif
// }


void Test32()
{
		// Test date parsing code
	
/*
[Mon, ]20 Jun 82 23:59[:59] GMT			**Supported
Weekday, DD-Mon-YY HH:MM:SS TIMEZONE	**Supported
Wed Jan 02 02:03:55 1980\n\0			**Supported (Finally!)
*/

	tm tmExpires;
	ZKSTD::string temp("Sun");
	memset(&tmExpires, 0, sizeof(tmExpires));
	ZKS::ParseDayMonthOrZone(temp, tmExpires);
	ZKTEST(tmExpires.tm_wday == 0);

	temp = "Wed";
	memset(&tmExpires, 0, sizeof(tmExpires));
	ZKS::ParseDayMonthOrZone(temp, tmExpires);
	ZKTEST(tmExpires.tm_wday == 3);

	temp = "Sat";
	memset(&tmExpires, 0, sizeof(tmExpires));
	ZKS::ParseDayMonthOrZone(temp, tmExpires);
	ZKTEST(tmExpires.tm_wday == 6);

	temp = "Jan";
	memset(&tmExpires, 0, sizeof(tmExpires));
	ZKS::ParseDayMonthOrZone(temp, tmExpires);
	ZKTEST(tmExpires.tm_mon == 0);

	temp = "Dec";
	memset(&tmExpires, 0, sizeof(tmExpires));
	ZKS::ParseDayMonthOrZone(temp, tmExpires);
	ZKTEST(tmExpires.tm_mon == 11);

	long lSeconds1 = 0, lSeconds2 = 0, lSeconds3 = 0;
	ZKS::ParseDateString("Wednesday, 09-Nov-99 23:12:40 GMT", tmExpires);
	ZKS::GetSecondsCUT(tmExpires, lSeconds1);

	ZKTEST(tmExpires.tm_hour == 23);
	ZKTEST(tmExpires.tm_min  == 12);
	ZKTEST(tmExpires.tm_sec  == 40);
	ZKTEST(tmExpires.tm_mday == 9);
	ZKTEST(tmExpires.tm_wday == 3);
	ZKTEST(tmExpires.tm_year == 99);

	ZKS::ConvertTmToString(temp, tmExpires);
	ZKTEST(temp == "Wed, 09 Nov 1999 23:12:40 GMT");

	ZKS::ParseDateString("Wed Nov 09 23:12:40 1999", tmExpires);
	ZKS::GetSecondsCUT(tmExpires, lSeconds2);

	ZKTEST(tmExpires.tm_hour == 23);
	ZKTEST(tmExpires.tm_min  == 12);
	ZKTEST(tmExpires.tm_sec  == 40);
	ZKTEST(tmExpires.tm_mday == 9);
	ZKTEST(tmExpires.tm_wday == 3);
	ZKTEST(tmExpires.tm_year == 99);


	ZKS::ParseDateString("Wed, 09 Nov 99 23:12:40 GMT", tmExpires);
	ZKS::GetSecondsCUT(tmExpires, lSeconds3);

	ZKTEST(tmExpires.tm_hour == 23);
	ZKTEST(tmExpires.tm_min  == 12);
	ZKTEST(tmExpires.tm_sec  == 40);
	ZKTEST(tmExpires.tm_mday == 9);
	ZKTEST(tmExpires.tm_wday == 3);
	ZKTEST(tmExpires.tm_year == 99);
	
	ZKTEST((lSeconds1 == lSeconds2) && (lSeconds1 == lSeconds3) && (lSeconds1 == 942361960));


	ZKS::ParseDateString("Thu, 10 Nov 2000 1:09:02 GMT", tmExpires);
	ZKS::ConvertTmToString(temp, tmExpires);
	ZKTEST(temp == "Thu, 10 Nov 2000 1:09:02 GMT");
}


void
Test33()
{
	
	ZKASSERT(ZKS::is_palindrome("abba"));

	ZKASSERT(ZKS::is_palindrome("ABBA"));

	ZKASSERT(!ZKS::is_palindrome("ABBAc"));

	ZKASSERT(!ZKS::is_palindrome("cABBA"));
}

void
Test34()
{
	using ZKS::CircularVector;

	// Testing basic functionality
	// Make the begin index and the end index move around.
	{
		CircularVector<int> circVec( 2 );
		
		circVec.PushBack( 0 );
		circVec.PushBack( 1 );

		ZKTEST( circVec.PopHead() == 0 );
		circVec.PushBack( 2 );
		ZKTEST( circVec.PopHead() == 1 );
		ZKTEST( circVec.Size() == 1 );
		ZKTEST( circVec.PopHead() == 2 );
	}

	// Testing growing.
	{
		CircularVector<int> circVec( 1 );

		ZKTEST( circVec.Capacity() == 1 );
		ZKTEST( circVec.Size() == 0 );
		
		circVec.PushBack( 0 );
		circVec.PushBack( 1 );

		ZKTEST( circVec.Capacity() == 2 );
		ZKTEST( circVec.Size() == 2 );

		// Make sure we can't reserve less that actual size.
		circVec.Reserve( 1 );
		ZKTEST( circVec.Capacity() == 2 );
		
		circVec.Reserve( 3 );
		ZKTEST( circVec.Capacity() == 3 );
		circVec.PushBack( 2 );

		ZKTEST( circVec.PopHead() == 0 );
		ZKTEST( circVec.PopHead() == 1 );
		ZKTEST( circVec.PopHead() == 2 );

		circVec.PushBack( 3 );
		circVec.PushBack( 4 );
		circVec.PushBack( 5 );
		circVec.PushBack( 6 );

		ZKTEST( circVec.Capacity() == 5 );

		ZKTEST( circVec.PopHead() == 3 );
	}
}

// Caller of all the unit tests.
void MainSDUTests()
{
	Test();	
	Test2();
	Test3();
	Test4();
	Test5(); 
	Test6();
	Test7();
	Test8();
	Test9();
	Test10();
	Test11();
	Test12();
	Test13();
	Test14();
// 	Test15();
// 	Test16();
	Test19();
// 	Test20();
// 	Test21();
	Test22();
	Test23();
	Test24();
	Test25();
	Test26();
	Test27();
	Test28();

	Test29();
	Test30();
// 	Test31();
	Test32();
	Test33();

	Test34();
}

class GeneralUnitTest : public ZkTestBase
{
public:
	GeneralUnitTest()
		: ZkTestBase( "zkbase/GeneralUnitTest", "Unit tests for the zkbase/general" )
	{}

	~GeneralUnitTest()
	{}

	virtual long Execute( void* )
	{
		MainSDUTests();

		return 1;
	}
};

GeneralUnitTest g_anInstance2;

#endif // INTERNAL_BUILD

