// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "ZkPingImpl.h"
#include <zksocket.h> // parse_inet_addr
#include <clientdebug.h>	// zkassert
#include <time.h>
#include <TraceControl.h> // For TRACE_PINGS



//---------------------------------------------------------------------------
//
// This isn't vrey robust. We rely on the output of gnu ping. which is fine
// for linux, but probably won't work on other platforms (Solaris, for example
// has a very different ping). This could be our own ping implementation, but we
// could also just ship versions of gnu ping on platforms that don't have it.
// --ol

bool
ZkPingImpl::Ping(	DZkError&		out_rErr,
					const char*		in_pszHostName, 
					unsigned long&	out_rReply, 
					const unsigned long in_dwTimeout, 
					const int			in_nPacketSize)
{
    (void)out_rErr;
    bool bRet = false;

    int infd;
    FILE * infile;
    char rBuf[1024];

    char sizeBuf[10];
    unsigned long l_Timeout = in_dwTimeout;
    if (l_Timeout < 1000 && l_Timeout != ZK_INFINITE) l_Timeout = 1000;

    out_rReply = 0;


    ZKSTD::vector<ZKSTD::string> args;

    args.push_back("-q");
    args.push_back("-n");
    args.push_back("-c");
    args.push_back("1");

    if (in_nPacketSize) 
    {
        snprintf(sizeBuf, 10, "%d", in_nPacketSize);
        args.push_back("-s");
        args.push_back(sizeBuf);
    }

    args.push_back(in_pszHostName);

    Execute(args, IO_INPUT, l_Timeout);

    infd = GetInputFD();
    ZKVERIFY( infile = fdopen(infd, "r") );

    memset(rBuf, 0, sizeof(rBuf));
    while ( fgets(rBuf, 1023, infile) )
    {
        if ( !strncmp(rBuf, "round-trip", strlen("round-trip")) )
        {
            int i;
            bool foundEqual = false;
            bool foundSlash = false;
            char *avgStr;
            float avgFloat;
            
            // Found a valid summary line.
            // (worst case scenario we bail on the 1024th NULL if something went wrong
            // shouldn't overrun the buffer)
            for (i = 0; rBuf[i]; i++)
            {
                if (foundEqual) 
                {
                    if (foundSlash) 
                    {
                        if (rBuf[i] == '/')
                        {
                            // got the second slash.
                            rBuf[i] = '\0';
                            if ( sscanf(avgStr, "%f", &avgFloat) == 1 ) 
                            {
                                ZKASSERT( (in_dwTimeout + 200) > avgFloat >= 0 );
                                out_rReply = (unsigned long)avgFloat;
                                bRet = true;
                                break;
                            }
                            
                        }
                    }
                    if (rBuf[i] == '/') 
                    {
                        avgStr = &rBuf[i+1];
                        foundSlash = true;
                    }
                }
                if (rBuf[i] == '=') foundEqual = true;
            }
        }
    }

    fclose(infile);

    infd = Wait();

    ZKCTRACE1( TRACE_PINGS, "PingImpl child Return Value: %d\n", infd);
    ZKCTRACE2( TRACE_PINGS, "Ping time for %s is %lu\n", in_pszHostName, out_rReply);
	return bRet;
}

