/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef HTTPHELPER_H__
#define HTTPHELPER_H__


#include <client.h>	// for ZKSTD
#include "inethelp.h"
#include <zkError.h>

class INETHELP_DLL HttpHelper
{
public:

	///////////////////////////////////////////////////////////////////////////////////////////////
	// PURPOSE: To make an HTTP GET request for the specified URL
	//
	// REQUIRE: out_ppResponse != NULL
	//			in_strURL must contain a valid absolute URL, like
	//				-"http://www.freedom.net:8080/index.html", or
	//				-"www.freedom.net/index.html"
	//			If this method returns true, the caller must free out_ppResponse using zkMemFree.
	//
	// PROMISE:	If (in_pbCancel != NULL), *in_pbCancel will be checked every in_TimeoutSec.  If it
	//			is true, this request will be very shortly aborted.
	//			if (!out_pResponseLen), this method will not choke. out_ppResponse will be NULL
	//			terminated.  The caller should obviously only pass in out_pResponseLen == NULL
	//			if a string (ie. no '\0's in the middle) is expected.

	static bool GetRequest( ZkError in_err, const char* in_strURL, char** out_ppResponse,
							unsigned int* out_pResponseLen = NULL,	bool* in_pbCancel = NULL, 
							int in_TimeoutSec = 5);


	///////////////////////////////////////////////////////////////////////////////////////////////
	// PURPOSE: To make an HTTP POST request for the specified URL
	//
	// REQUIRE: out_ppResponse != NULL
	//			in_strPostBody != NULL
	//			in_strURL must contain a valid absolute URL, like
	//				-"http://www.freedom.net:8080/index.html", or
	//				-"www.freedom.net/index.html"
	//			If this method returns true, the caller must free out_ppResponse using zkMemFree.
	//
	// PROMISE:	If (in_pbCancel != NULL), *in_pbCancel will be checked every in_TimeoutSec.  If it
	//			is true, this request will be very shortly aborted.

	static bool PostRequest(ZkError in_err, const char* in_strURL, const char *in_strPostBody,
							char** out_ppResponse, unsigned int* out_pResponseLen, 
							bool* in_pbCancel = NULL, int in_TimeoutSec = 5);

private:
	HttpHelper();
	HttpHelper(HttpHelper&);

	static bool Request(ZkError in_err, const char* in_strURL, const char *in_strPostBody,
						char** out_ppResponse, unsigned int* out_pResponseLen,
						bool* in_pbCancel = NULL, int in_TimeoutSec = 5);
	static size_t GetContentLength(const char* in_strResponse);
	static bool ResponseIsSuccess(const char* in_strResponse);
	static bool SplitURL (const char * in_strUrl, int* out_Port, char *out_strHostName, char *out_strResource );
};


#endif	// HTTPHELPER_H__
