/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "inethelp.h"
#include <zkIntTypes.h>	// for uint8_t
#include <clientstl.h>	// for ZKSTD

class INETHELP_DLL PacketHelper
{
public:
	
	PacketHelper( 
		const int in_proto , 
		const unsigned int in_srcip,
		const unsigned int in_dstip,
		const uint8_t in_type, 
		const uint8_t in_code);


	PacketHelper( 
		const int in_proto,
		const unsigned int in_dstip,
		const unsigned short in_dstport);

						
	PacketHelper( 
		const int in_proto,
		const unsigned int in_srcip,
		const unsigned int in_dstip,
		const unsigned short in_srcport,
		const unsigned short in_dstport);

	PacketHelper(const PacketHelper&);

	ZKSTD::string	GetProtocolDesc_str() const throw();
	ZKSTD::string	GetProtocolPacketDesc_str() const throw();
	ZKSTD::string	GetAdditionalPacketDesc_DestPort_str() const throw();
	ZKSTD::string	GetAdditionalPacketDesc_SrcPort_str() const throw();
	ZKSTD::string	GetSourceIp_str() const throw();
	ZKSTD::string	GetDestinationIp_str() const throw();
	int				GetProtocol() const throw();
	unsigned int	GetSourceIp() const throw();
	unsigned int	GetDestinationIp() const throw();
	unsigned short	GetDestinationPort() const throw();
	unsigned short	GetSourcePort() const throw();
	uint8_t			GetIcmpType() const throw();
	ZKSTD::string   GetPossibleTrojans_DestPort_str() const throw();
	time_t			GetDateTime() throw();

	PacketHelper& operator=(const PacketHelper&);

	void InitDateTime();

private:
	PacketHelper::PacketHelper(); // prevent default ctor

	// ctor member init
	int			m_protocol;
	unsigned int	m_srcip;
	unsigned int	m_dstip;
	uint8_t		m_Icmptype;
	uint8_t		m_Icmpcode;
	unsigned short m_srcport;
	unsigned short m_dstport;

	time_t m_datetime;
};
