// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "ProtocolInfo.h"
#include "PortProtocols.h"	// for g_PortInfo
#include "ZkNetMisc.h"		// for GetServiceInformation
#include <clientdebug.h>	// for ZKASSERT

bool ProtocolInfo::m_bInitialized = false;

ProtocolInfo&
ProtocolInfo::theProtocolInfo()
{
	static ProtocolInfo x;
	return x;
}


ZKSTD::string 
ProtocolInfo::GetPortService_str(const int in_proto, const unsigned short in_port)  
{
	Init();

	char* protocol = "xxx";

	if(IPPROTO_TCP == in_proto)
	{
		protocol = "tcp";
	}
	else if (IPPROTO_UDP == in_proto)
	{
		protocol = "udp";
	}
	else
	{
		return "Unknown";
	}

	const char* service = ZKS::GetServiceInformation(in_port, "tcp");

	ZKSTD::string strRet;

	if(service)
	{
		return service;
	}
	else
	{
		ZKSTD::map<unsigned short, const PortInfo*>::iterator i = m_map.find(in_port);
		if( i != m_map.end() )
		{
			return (*i).second->protocol;
		}
	}

	return "Unknown";
}


ZKSTD::string 
ProtocolInfo::GetTrojans_str(const unsigned short in_port) 
{
	Init();

	ZKSTD::map<unsigned short, const PortInfo*>::iterator i = m_map.find(in_port);
	if( i != m_map.end() )
	{
		return (*i).second->attacks;
	}

	return "";
}

#define ELEMENTS(t) sizeof(t)/sizeof(t[0])

void
ProtocolInfo::Init()
{
	if(m_bInitialized) return;

	for(int i=0; i<ELEMENTS(g_PortInfo); ++i)
	{
#ifndef NDEBUG
		ZKSTD::map<unsigned short, const PortInfo*>::iterator it = m_map.find(g_PortInfo[i].port);
		ZKASSERT( it == m_map.end() );
#endif
		m_map[g_PortInfo[i].port] = &g_PortInfo[i];
	}
	
	m_bInitialized = true;
}

ZKSTD::string 
ProtocolInfo::GetPortRangeDescription(const unsigned short in_port) const
{
	if(in_port >= 6970 && in_port <= 7170)
	{
		return "RealAudio";
	}
	else if(in_port >= 6660 && in_port <= 6690)
	{
		return "IRC";
	}
	else if(in_port >= 8778 && in_port <= 8786)
	{
		return "Unreal Tournament";
	}
	else if(in_port >= 27910 && in_port <= 27961)
	{
		return "Quake";
	}
	else if(in_port >= 33434 && in_port <= 33600)
	{
		return "traceroute";
	}
	return "";
}