/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ProtocolInfo_H__
#define ProtocolInfo_H__

#include "inethelp.h"
#include <clientstl.h>

#ifdef _WIN32
#pragma warning(disable: 4251)
#endif

struct PortInfo;

class INETHELP_DLL ProtocolInfo
{
public:
	static ProtocolInfo&	theProtocolInfo();


	// PURPOSE: Returns string representation of what runs on that port
	// REQUIRE:	Only valid for tcp and udp protocols
	// PROMISE: Returns "Unknown" when no service is known
	ZKSTD::string GetPortService_str(const int in_proto, const unsigned short in_port)  throw();

	// PURPOSE: Returns string representation of trojans known to exist on that port
	// PROMISE: Returns "" is none is known
	ZKSTD::string GetTrojans_str(const unsigned short in_port) throw();

	// PURPOSE: Some well known protocols use a wide range
	// PROMISE: Returns "" for ports not in a known range
	ZKSTD::string GetPortRangeDescription(const unsigned short in_port) const throw();

private:
	static bool m_bInitialized;
	
	ZKSTD::map<unsigned short, const PortInfo*>	m_map;

	void Init();
};

#endif	// ProtocolInfo_H__
