// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// ZKTestScalper.cpp: implementation of the ZKTestScalper class.
// Test functions about scalper

#include "ZKTestScalper.h"
#include <DZkError.h>
#include <WAutoBuffer.h>
#include "HttpHelper.h"

#define SERIALNUMLENGTH 14 // this is also defined in ITokenService.h
							// as ZKSERIAL_NUMBER_LENGTH but managing the
							// #includes in the project settings was too
							// difficult, it brought in the world.
bool 
ZKTestScalper::GetSerialNumber(
		ZKSTD::string& out_serialNumber, 
		SerialNumberTypes SerialType)
{
	bool bReturnValue = false;
	DZkError io_err;
	ZKSTD::string in_strUrl( ZKConfiguration::GetSerialNumberUrl(SerialType) );
	ZKASSERT( !in_strUrl.empty() ) ;

	WAutoCharBuffer ResponseBuf;

	char strserial[SERIALNUMLENGTH + 1] = {0};
	
	
	if (!HttpHelper::GetRequest(&io_err, 
							in_strUrl.c_str(), 
							&ResponseBuf) )
	{
		bReturnValue = false;	
	}
	else
	{
		if( !ResponseBuf.IsValid() ||
			strlen(ResponseBuf) < SERIALNUMLENGTH ||
			strlen(ResponseBuf) > (SERIALNUMLENGTH + 4) ) 
		{
			// what are we sending back these days ..
			bReturnValue = false;	

		}
		else
		{
			// grab the first ZKSERIAL_NUMBER_LENGTH chars
			// (http reqests typically come with \r\n\r\n
			// tagged on to the end ...)
			memcpy(strserial, ResponseBuf, SERIALNUMLENGTH);
			out_serialNumber = strserial;
			bReturnValue = true;
		}
	}
	
	return bReturnValue;
}
		

/* 
*
* This test code was replaced by with a newer version by 
* Marty, 08-21-00
*
*/

#if 0

bool ZKTestScalper::GetSerialNumber(ZKSTD::string& out_serialNumber)
{
	DZkError err;

	bool bFailed = false;
	
	ZKSTD::string SNLocation = ZKConfiguration::GetSerialNumberUrl();
	ZKSTD::string str_HostName = SNLocation.substr(7, SNLocation.find_first_of('/', 7) - 7);
	ZKSTD::string str_request =	"GET " + SNLocation + " HTTP/1.1\r\n"
								"Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/vnd.ms-excel, application/msword, application/pdf, */*\r\n"
								"Content-Type: application/x-www-form-urlencoded\r\n"
								"Accept-Encoding: gzip, deflate\r\n"
								"Host: " + str_HostName + "\r\n"
								"\r\nConnection: Keep-Alive\r\n\r\n";

	str_HostName += ":80";

	ZkSocket sock;
#ifdef WIN32
	WSADATA stWSAData;
	ZeroMemory(&stWSAData, sizeof(WSADATA));

	if(0 == ::WSAStartup(MAKEWORD(MAJOR_VERSION_REQUIRED, MINOR_VERSION_REQUIRED), &stWSAData))
	{
#endif
		if(!zkSocketNew(&err, AF_INET, SOCK_STREAM, IPPROTO_TCP, &sock))
		{
			bFailed = true;
		}
		else
		{
			if(!zkSocketConnect(&err, sock, str_HostName.c_str()))
			{
				bFailed = true;
			}
			else if(!zkSocketWriteAll(&err, sock, str_request.c_str(), str_request.length()))
			{
				bFailed = true;
			}
			else{

				char recvbuf[2000] = {0};
				unsigned int recvdLength = 0;
				if(!zkSocketRead(&err, sock, recvbuf, sizeof(recvbuf), &recvdLength))
				{
					bFailed = true;
				}
				else{
					// Parse out SN
					ZKSTD::string ssRecv(recvbuf);

					// i look for 2 enters
					size_t start = ssRecv.find("\r\n\r\n");
					// skip them
					start += 4;
					out_serialNumber = ssRecv.substr(start,14);

					// look if the serial number is valid (capital letters and numbers)
					for( size_t i = 0; i<14 ; ++i)
					{
						if(!((out_serialNumber[i] >= 65 && out_serialNumber[i] <= 90) ||
							(out_serialNumber[i] >= 49 && out_serialNumber[i] <= 57)))
						{
							bFailed = true;
							break;
						}
					}
				}
			}
			zkSocketFree(&err, &sock);
		}

#ifdef WIN32
		WSACleanup();
	}
	else{
		bFailed = true;
	}
#endif

	return bFailed;
}

#endif // 0
