// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "ZkIcmp.h"


#ifdef ZK_LINUX
#include "../linux/ZkPingImpl.h"
#else
#include "../win/ZkPingImpl.h"
#endif

#include <client.h>	// safedelete
#include <iostream> // cout

ZkIcmp::ZkIcmp() 
	: m_pImpl( new ZkPingImpl )
{	 
}

ZkIcmp::~ZkIcmp() 
{
	SAFEDELETE(m_pImpl);
}


bool
ZkIcmp::Ping(	DZkError&		out_rErr,
				const char*		in_pszHostName, 
				unsigned long&	out_rReply, 
				const unsigned long in_dwTimeout /* = 5000 */, 
				const int			in_nPacketSize /* = 32 */)
		
{
	return m_pImpl->Ping( out_rErr, in_pszHostName, out_rReply, in_dwTimeout, in_nPacketSize );
}


void
ZkIcmp::UnitTest()
{
	unsigned long reply = 0;
	DZkError err;

	ZkIcmp theIcmp;

	// Could fail if you're not connected, or the remote server is down
	ZKTEST( theIcmp.Ping( err, "www.starwars.com", reply, 5000, 1000 ) );
	ZKTEST( theIcmp.Ping( err, "smartypants.zks.net", reply, 500, 500 ) );
	ZKTEST( theIcmp.Ping( err, "www.startrekvoyager.com", reply, 700, 250 ) );

	// We are ensuring the timeout is working, should fail 
	// This test will fail if microsoft starts to respond to ping.
	const unsigned long start = ZKGetTickCount();
	ZKTEST( !theIcmp.Ping( err, "www.microsoft.com", reply, 9000, 128 ) );
	const unsigned long stop = ZKGetTickCount();
	// XXX 9000 <=  <= 10000
	const unsigned long delay = stop - start;
	ZKTEST( delay >= 9000 && delay <= 10000 );

#ifdef WIN32
	ZkPingImpl::UnitTest();
#endif

    ZKSTD::cout << "Test Complete" << ZKSTD::endl;
}
