/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#if !defined(AFX_DWINSOCK2APP_H__D4F316A6_50B1_11D2_8097_00104B6B4FA4__INCLUDED_)
#define AFX_DWINSOCK2APP_H__D4F316A6_50B1_11D2_8097_00104B6B4FA4__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "inethelp.h"
#include <string>
#include <zksocket.h>
#include <DZkError.h>
#include <vector>

class INETHELP_DLL ZkNetHelp  
{
public:
	ZkNetHelp();
	~ZkNetHelp();

	ZkSocket& GetSocket() { return m_socket; }
	const DZkError& GetError() const { return m_err; }

	static bool ReceiveLine(ZkError in_err, ZkSocket in_Socket, ZKSTD::string& out_strLine);
	static bool IsRemoteHostDummyMailServer(ZkSocketAddress& in_RemoteAddr);
	static bool SmtpResponseIsSuccess(const ZKSTD::string& in_strResponse);

    // Get a vector of the machine's IP addresses in string form
    static const ZKSTD::vector< ZKSTD::string > &GetVectorOfStringIPs();

    // Get a vector of the machine's IP addresses in binary form. (Network byte order)
    static const ZKSTD::vector< uint32_t > &GetVectorOfBinaryIPs();

	// This base class implements it, that is, receiving a SOCKADDR.sa_data from the LSP
	bool InitRealSockAddr();

	// Make sure that InitRealSocket has been called before calling this if you want
	// to receive an initialized value.
	ZkSocketAddress& GetOriginalDestinationSocketAddress() { return m_realSockAddr; }

#ifdef INTERNAL_BUILD
	static void UnitTest();
#endif //INTERNAL_BUILD

private:
	// Don't allow copy construction or assignment
	ZkNetHelp(const ZkNetHelp&);
	ZkNetHelp& operator=(const ZkNetHelp&);	

    // Implementation of getting IP addresses
    static void GetIPsForThisHost();

#ifdef ZK_LINUX
// Helpers for GetIPsForThisHost
    static int GetDefaultRouteIF( char *ifbuf, size_t ifbuflen );
    static int FetchIFAddr( struct ifreq *ifr, char *ifname );
#endif

	ZkSocket		m_socket;	
	ZkSocketAddress	m_realSockAddr;
	DZkError		m_err;

    // IP addresses cached in both formats. 
    static ZKSTD::vector< ZKSTD::string > m_vecStrIPs;
    static ZKSTD::vector< uint32_t > m_vecIPs;
    static bool m_bGrokedIPsAlready;
};

#endif // !defined(AFX_DWINSOCK2APP_H__D4F316A6_50B1_11D2_8097_00104B6B4FA4__INCLUDED_)
