// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <ZkTestBase.h>
#include <ZkTest.h>

//===========================================================================
// CLASS ZkTestBase
//===========================================================================

bool ZkTestBase::s_bTrace = true;

//---------------------------------------------------------------------------
//
ZkTestBase::ZkTestBase( const char* in_pName, bool in_bInitialize )
	: m_strName( in_pName )
	, m_strDescription( "N/A" )
	, m_testType( STANDARD )
{
	Init( in_bInitialize );
}

//---------------------------------------------------------------------------
//
ZkTestBase::ZkTestBase( 
	const char* in_pName, 
	const char* in_pDescr,
	TestType in_testType,
	bool in_bInitialize )
		: m_strName( in_pName )
		, m_strDescription( in_pDescr )
		, m_testType( in_testType )
{
	Init( in_bInitialize );
}


//---------------------------------------------------------------------------
//
ZkTestBase::~ZkTestBase( )
{
}

//---------------------------------------------------------------------------
//
void ZkTestBase::Init( bool in_bInitialize )
{
    if( in_bInitialize ) 
    {
        ZkTest::RegisterInit( this );
    }
    else 
    {
        ZkTest::RegisterTest( this );
    }
}

//---------------------------------------------------------------------------
//
const char* ZkTestBase::GetName() const
{
    return m_strName;
}

//---------------------------------------------------------------------------
//
const char* ZkTestBase::GetDescription() const
{
	return m_strDescription;
}

//---------------------------------------------------------------------------
//
ZkTestBase::TestType ZkTestBase::GetType() const
{
	return m_testType;
}


