// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// AciInfo.cpp: implementation of the AciInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"

#include "AciInfo.h"
#include "ZKConvert.h"
#include <TraceControl.h>

AciInfo::AciInfo()
{
	Clear() ;
}

void AciInfo::Clear()
{
	m_RouteState = RouteInvalid ;
	m_RouteType = RouteTypeUnknown ;

	m_nDuration = 0 ;
	m_StartTime = 0 ;
	memset( &m_FirstAipAddr, 0, sizeof( m_FirstAipAddr ) ) ;

	m_iNym = 0 ;
	m_DestPort = 0 ;
    SetRouteIdleCounter( 0 ) ;
}

AciInfo::~AciInfo()
{
}

void
AciInfo::TraceState() const
{
#ifdef _DEBUG
	
	if ( TRACE_ROUTEMANAGER )
	{

		if ( m_RouteType == AuthenticatedRoute )
		{
			ZKTRACE0( "		Authenticated Route, " ) ;
			ZKTRACE1( "Nym Index %d", m_iNym ) ;

		}
		else if ( m_RouteType == UnauthenticatedRoute )
		{
			ZKTRACE0( "		Unauthenticated Route, " ) ;
			ZKTRACE2( "Destination Port %d (0x%x)", m_DestPort, m_DestPort ) ;

			ZKTRACE1( "- %s", ZKConvert::GetServerNameFromPort( m_DestPort ).c_str() ) ;		
		}
		else
		{
			ZKTRACE0( "		Not a route - is data corrupt?\n" ) ;
		}

		if ( m_RouteState == RouteInvalid )
		{
			ZKTRACE0( "- Route is invalid - is table corrupt?\n" ) ;
		}
		else if ( m_RouteState == RouteValidNotActive )
		{
			ZKTRACE0( "- Route has been created but isn't active\n" ) ;
		}
		else if ( m_RouteState == RouteActive )
		{
			ZKTRACE0( "- Route is active\n" ) ;
		}
		else if ( m_RouteState == RouteRetired )
		{
			ZKTRACE0( "- Route has been retired\n" ) ;
		}

		ZKTRACE2( "		Duration: %d, Start Time: %d\n", m_nDuration, m_StartTime ) ;
	}
#endif
}
