// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// ClientTypesUnitTest

// This class provides the interface for different unit tests
// available in for this module.

#ifndef _ClientTypesUnitTest_H
#define _ClientTypesUnitTest_H

#ifdef INTERNAL_BUILD

#include <AutoInstCount.h>

#include "ClientTypesInc.h"
#include "WCoreTypes.h" // For WMemBuf

#include <zkbase/libzktest/ZkTestBase.h>

class ClientTypes_API ClientTypesUnitTest
	: public ZkTestBase AUTO_INSTANCE_COUNTING2( ClientTypesUnitTest )
{
public:
	ClientTypesUnitTest()
		: ZkTestBase( "zkcentral/clienttypes/ClientTypesUnitTest",
						"Client types unit test" )
	{}
	~ClientTypesUnitTest()
	{}
	
	virtual long Execute( void* );

private:
	// Unit Tests
	static void TestWCypherMailMsg() ;
	static void TestWAutoBuffer() ;
	static void TestWReadBuffer() ;
	static void TestChunks() ;
	static void TestZKDocument() ;
	static void TestWNymIconInfo() ;
	static void TestWNetInfo() ;
	static void TestWZkipClient() ;


	// Helper functions
	static void zkAllocationFunction1( unsigned char ** out_Buffer, size_t * out_Size ) ;
	static void zkAllocationFunction2( char ** out_Buffer, size_t * out_Size ) ;
	static void zkAllocationFunction3( void ** out_Buffer, size_t * out_Size ) ;

	static void WriteChunks( WMemBuf& io_Memory ) ;
	static void ReadChunks( unsigned char * in_pRawBuffer, size_t in_cntBuffer ) ;
} ;

#endif // INTERNAL_BUILD

#endif
