// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// DContext.h
//
// This is a shared base class for the plugins.
//
// 
// It is not really a basic type for the client,
// so its presence in ClientTypes should be considered
// temporary.  A revamp of the plugins will probably make
// this class obsolete.  For more information please contact
// Marty.

#ifndef _DContext_H
#define _DContext_H

#include <string>
#include <map>
#include <vector>
#include <winsock2.h>
#include <client.h>

#include <AutoInstCount.h>

typedef ZKSTD::vector<ZKSTD::string>			vctSTRING;
typedef ZKSTD::map<ZKSTD::string, ZKSTD::string>	mapOs2s;

#define BUFSIZE 2048

class DContext AUTO_INSTANCE_COUNTING1( DContext )
{
	// No copying or assignment
	DContext(const DContext&);
	DContext& operator=(const DContext&);
	
	int				m_nCounter;
	int				m_nStatus;
	int				m_nCmd;
	mapOs2s			m_StrStrMap;
	WSABUF			m_sendwsabuf;
	WSABUF			m_recvdwsabuf;
	unsigned char			m_pRecvBuf[BUFSIZE];
	unsigned long			m_dwRecv;
	int				m_nState;	// enumeration specific to each derivation
	bool			m_bSwitch;
	ZKSTD::string		m_strCommandLine;
	vctSTRING		m_VectorOfStrings;
	
	// marty ZkEmail			m_zMail;
	// marty ZkEmailHeader		m_zHeader;
	

	// ELW: We'll store the domain name of a host here
	// (taken from the client request). This is used
	// while interpreting cookies in the response.
	ZKSTD::string m_DomainName;


public:
	DContext()  : m_nCounter(0)
			    , m_nStatus(0)
				, m_nCmd(0)
				, m_dwRecv(0)
				, m_nState(0)
	{
		memset(m_pRecvBuf, 0, BUFSIZE);
		m_sendwsabuf.buf = NULL;
		m_sendwsabuf.len = 0;
		m_recvdwsabuf.buf = NULL;
		m_recvdwsabuf.len = 0;
	}

	
	virtual int& GetCount()						{ return m_nCounter; }
	virtual int& GetStatus()					{ return m_nStatus; }
	virtual mapOs2s& GetmapOs2s()				{ return m_StrStrMap; }
	virtual int& GetCommand()					{ return m_nCmd; }
	virtual WSABUF& GetSendBuff()				{ return m_sendwsabuf; }
	virtual WSABUF& GetRecvBuff()				{ return m_recvdwsabuf; }
	// marty virtual ZkEmail& GetZkEmail()				{ return m_zMail; }
	// marty virtual ZkEmailHeader& GetZkEmailHeader()	{ return m_zHeader; }
	virtual int& GetState()						{ return m_nState; }
	virtual unsigned long& GetBytesReceived()			{ return m_dwRecv; }
	virtual ZKSTD::string& GetCommandLine()		{ return m_strCommandLine; }
	virtual vctSTRING& GetVectorOfStrings()		{ return m_VectorOfStrings; } 
	virtual bool& GetSwitch()					{ return m_bSwitch; }
	virtual unsigned char* GetByteBuffer()				{ return m_pRecvBuf; }
//	virtual mapInt2CString& GetMapInt2CString()	{ return m_MapInt2CString; }
	ZKSTD::string& GetDomainName()				{ return m_DomainName; }
};


#endif