// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include <StdAfx.h>
#include "RPortToProxyMap.h"

#include <TraceControl.h>

RuleForPort 
RPortToProxyMap::GetPortRule(uint16_t in_port) const
{
	RPortToProxyMap::const_iterator it = find(in_port);

	if(it != end())
	{
		return (*it).second ;
	}
	else
	{
		// Not in the map, not supported port as
		// far as we are concerned
		return PortRuleUndefined ;
	}
}

bool 
RPortToProxyMap::IsHttpPort(uint16_t in_port ) const
{
	return ( PortRuleUseHTTPProxy == GetPortRule( in_port ) ) ;
}

bool 
RPortToProxyMap::IsIRCPort(uint16_t in_port ) const
{
	return ( PortRuleUseIRCProxy == GetPortRule( in_port ) ) ;
}

bool 
RPortToProxyMap::IsTelnetPort(uint16_t in_port ) const
{
	return ( PortRuleUseTelnetProxy == GetPortRule( in_port ) ) ;
}

bool 
RPortToProxyMap::IsProxyPort(uint16_t in_port ) const 
{
	// Warning this assumption assumes all port rules are proxy port rules
	return ( PortRuleUndefined != GetPortRule( in_port ) ) ;
}

void 
RPortToProxyMap::DeletePort(uint16_t in_port)
{
	RPortToProxyMap::iterator it = find(in_port);

	if(it != end())
	{
		ZKCTRACE1( TRACE_MYINTAC, "Port %d - removing from map\n", in_port ) ;
		erase( it ) ;
	}	
	else
	{
		ZKCTRACE1( TRACE_MYINTAC, "Not deleting Port %d - it is not in the map\n", in_port ) ;
	}
}