// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RStatCollector.h

// This simple class implements an 
// Min/Max/Avg etc calculation for a stream
// of data points
//
// We could make a template if other types
// (eg double) also need to be collected

// REVIEW: Converting this interface to use doubles
// will slow it down but provide most flexibility for
// statistic collection

#ifndef RSTATCOLLECTOR_H
#define RSTATCOLLECTOR_H

#include "ClientTypesInc.h"

#include <zkIntTypes.h>
#include <clientenum.h>

class ClientTypes_API RStatCollector
{
public:
	RStatCollector() ;

	void Reset() ;

	void AddDataPoint( uint32_t in_data ) ;

	// Returns a stat about the data, for example AvgOfDatums

	// These functions return zero if there is no data yet so be
	// careful
	uint32_t GetStat( StatisticType in_whichStat ) const ;

private:
	// We are collecting avg as a uint - assuming that
	// fractions don't matter, ie avg of 1 2, 4 is 2
	uint32_t m_avg ;
	uint32_t m_min ;
	uint32_t m_max ;
	uint32_t m_cntItems ;  // Number of items that we have received

} ;



#endif