// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "StdAfx.h"
#include "WClientInfo.h"

WClientInfo::WClientInfo()
{
	Init();
}

WClientInfo::~WClientInfo()
{
	Clear();	
}

bool
WClientInfo::IsValid() const
{
	return ( 0 != GetCurrentUTC() ) ;
}

ZKSTD::string WClientInfo::GetDomain() const throw()
{
	return m_Datum.domain;
}

ZKSTD::string WClientInfo::GetSerialNoUrl() const throw()
{
	return m_Datum.sn_url;
}

ZKSTD::string WClientInfo::GetClientId() const throw()
{
	return m_Datum.client_id;
}

ZKSTD::string WClientInfo::GetUpgradeUrl() const throw()
{
	return m_Datum.upgrade_url;
}

time_t WClientInfo::GetCurrentUTC() const throw()
{
	return m_Datum.current_UTC;
}

time_t WClientInfo::ExpiresIn() const throw()
{

	return m_Datum.expire_in;
}

int WClientInfo::GetNiqsUpdateFrequency() const throw()
{
	return m_Datum.niqs_update;
}

size_t WClientInfo::GetMessageOfTheDaySize() const throw()
{
	return m_Datum.motd_sz;
}

ZKSTD::string WClientInfo::GetMessageOfTheDay() const throw()
{
	return ZKSTD::string(m_Datum.motd, m_Datum.motd_sz);
}

int WClientInfo::GetMessageOfTheDayUrgency() const throw()
{
	return m_Datum.motd_urg;
}

void WClientInfo::Clear()
{

	ZKVERIFY( frNiqsClientInfoFree(&m_err, &m_Datum) );
}

void WClientInfo::Init()
{

	memset(&m_Datum, 0, sizeof(m_Datum) );	
	m_Datum.niqs_update = 1800; // Default Niqs updates to once every 30 minutes
}

WClientInfo& WClientInfo::operator=(const WClientInfo& rhs) 
{

	if(this == &rhs) return *this; // address equality

	// Free our object resources
	Clear();

	const FrNiqsClientInfo * pRhsData = (const FrNiqsClientInfo*) rhs ;

	// Copy the right hand side object onto ours
	memcpy( &m_Datum, pRhsData, sizeof(m_Datum) );

	m_Datum.motd_sz = rhs.GetMessageOfTheDaySize();

	if(m_Datum.motd_sz)
	{
		m_Datum.motd = (char*)malloc( m_Datum.motd_sz );
		strncpy( m_Datum.motd, rhs.GetMessageOfTheDay().c_str(), m_Datum.motd_sz );
	}
	else
	{
		// Without this else case, m_Datum.motd is a pointer into memory owned by rhs,
		// which causes a crash on shutdown in frNiqsClientInfoFree (Bug 8473) Corey.

		//m_Datum.motd = NULL;  // This should work, but this is an LBF fix, so the 
								// following 2 lines are more safe.
		m_Datum.motd = (char*)malloc( sizeof(char) );
		*m_Datum.motd = '\0';
	}

	return *this;
}

WClientInfo::WClientInfo(const WClientInfo& rhs)
{
	Init();
	WClientInfo::operator=(rhs);
}

int WClientInfo::GetExpiryYear() const throw()
{
	return m_Datum.expiry.date.year;
}

int WClientInfo::GetExpiryMonth() const throw()
{
	return m_Datum.expiry.date.month;
}

int WClientInfo::GetExpiryDay() const throw()
{
	return m_Datum.expiry.date.day;
}
