// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

//  WClientInfo is a wrapper for the frNiqsClientInfo
//  class.  It takes care of memory management and
//  provides nice C++ accessors and data types for the
//  internal data.


#ifndef WCLIENT_INFO_H
#define WCLIENT_INFO_H

#include "ClientTypesInc.h"

#include <zks.h>
#include <frNiqs.h>
#include <string>
#include <DZkError.h>

#include <AutoInstCount.h>

#define TIME_TO_UPGRADE 3456000 // 40 days

class ClientTypes_API WClientInfo AUTO_INSTANCE_COUNTING1( WClientInfo )
{

public:
	// Provide access to underlying structure - eg when
	// making calls that expect a FrNiqsClientInfo
	operator FrNiqsClientInfo*()		throw() { return &m_Datum ; }
	operator const FrNiqsClientInfo*() const	throw() { return &m_Datum ; }

	WClientInfo();

	~WClientInfo();

	WClientInfo(const WClientInfo&);

	WClientInfo& operator=(const WClientInfo&);	

	// Return whether the object actually has any valid
	// information
	bool IsValid() const ;

	// Year is 0 to 9999
	int GetExpiryYear() const throw();

	// Month is 1 to 12
	int GetExpiryMonth() const throw();

	// Day is 1 to 31
	int GetExpiryDay() const throw();


	// The domain name (Usually freedom.net)
	ZKSTD::string GetDomain() const throw();

	// The serial number URL (points to the scalper) 
	ZKSTD::string GetSerialNoUrl() const throw();

	// The client identifier string
	ZKSTD::string GetClientId() const throw();

	// URL which points to client upgrade
	ZKSTD::string GetUpgradeUrl() const throw();

	// The current UTC time in seconds since 00:00:00 UTC, january 1, 1970
	time_t GetCurrentUTC() const throw();

	// Number of seconds left until the client expires.
	time_t ExpiresIn() const throw();

	// NIQS Update Frequency (in seconds)
	int GetNiqsUpdateFrequency() const throw();

	ZKSTD::string GetMessageOfTheDay() const throw(); 

	// Size of the MOTD string (not counting terminating NUL)
	size_t GetMessageOfTheDaySize() const throw();

	 // MOTD urgency (for future use, always 1)
	int GetMessageOfTheDayUrgency() const throw();

private:

	FrNiqsClientInfo		m_Datum;
	DZkError				m_err;	

	void Init();
	void Clear();
};


#endif	
