// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "StdAfx.h"
#include "WCypherMailMsg.h"

const size_t WCypherMailMsg::UnknownValue = 99999 ; 


WCypherMailMsg::WCypherMailMsg()
	: m_nReplyBlock( WCypherMailMsg::UnknownValue )
	, m_FmgSignatureResult( SigCheckNotAttempted ) 
	, m_SignatureResult( SigCheckNotAttempted ) 
	, m_pEncryptedText( NULL )
	, m_cntEncryptedText( 0 )
	, m_bActivatedRB( false )
{
}

/*
WCypherMailMsg::WCypherMailMsg(const WCypherMailMsg &)
{
}
*/

WCypherMailMsg::~WCypherMailMsg()
{
}

void WCypherMailMsg::SetEncryptedText( 
							const unsigned char * in_pBuf, 
							size_t in_szBuf ) 
{ 
	m_pEncryptedText = in_pBuf ; 
	m_cntEncryptedText = in_szBuf ; 
}


void WCypherMailMsg::Clear()
{
	m_nReplyBlock = WCypherMailMsg::UnknownValue ;
	m_pEncryptedText = NULL ;
	m_cntEncryptedText = 0 ;
	m_ClearText.Clear() ;
	m_strSendingEntity.erase() ;
	m_strRecipientNym.erase() ;
	m_FmgSignatureResult = SigCheckNotAttempted ;
	m_SignatureResult = SigCheckNotAttempted ;
}

void PrintSigResult( SigCheckResult in_Sig )
{
	if ( in_Sig == SigCheckNotAttempted ) {
		ZKTRACE0( "Signature Check Not Attempted\n" ) ;
	}
	else if ( in_Sig ==SigCheckSuccessful ) {
		ZKTRACE0( "Signature Check Successful\n" ) ;
	}
	else if (in_Sig ==SigCheckFailed) {
		ZKTRACE0( "Signature Check Failed\n" ) ;
	}
	else if (in_Sig ==SigCheckKeyNotAvail ) {
		ZKTRACE0( "Key not retrieved for Signature Check\n" ) ;
	}
	else if (in_Sig ==SigCheckKeyNotFound ) {
		ZKTRACE0( "Key not Found\n" ) ;
	}
}

void WCypherMailMsg::SelfTest() const
{
#ifdef INTERNAL_BUILD

	// Verify various assumptions about what states
	// are valid

	if ( m_cntEncryptedText == 0 ) {
		ZKTEST( m_pEncryptedText == NULL ) ;
		ZKTEST( m_ClearText.Size() == 0 ) ;
		ZKTEST( m_strSendingEntity.empty() ) ;
		ZKTEST( m_FmgSignatureResult == SigCheckNotAttempted ) ;
		ZKTEST( m_SignatureResult == SigCheckNotAttempted ) ;
		ZKTEST( !m_bActivatedRB );

		// It is legal to fill these in before setting the encryption
		// ZKTEST( m_strRecipientNym.empty() ) ;
		// ZKTEST( m_nReplyBlock == UnknownValue ) ;
	}
	else if ( m_ClearText.Size() > 0 ) {
		ZKTEST( m_cntEncryptedText > 0 ) ;
		ZKTEST( m_pEncryptedText != NULL ) ;
		ZKTEST( !m_strRecipientNym.empty() ) ;
		ZKTEST( !m_strSendingEntity.empty() ) ;
		ZKTEST( m_nReplyBlock != UnknownValue ) ;
		ZKTEST( m_FmgSignatureResult != SigCheckNotAttempted ) ;
		ZKTEST( m_SignatureResult != SigCheckNotAttempted ) ;
	}
#endif
}

void WCypherMailMsg::DumpToDebugOutput() const
{
#ifdef _DEBUG

	if ( m_cntEncryptedText == 0 ) {
		ZKTRACE0( "WCypherMailMsg has not be provided with an buffer to decrypt yet\n" ) ;
	}

	if ( m_ClearText.Size() > 0 ) {
		ZKTRACE0( "Message has been decrypted\n" ) ;
	}

	if ( !m_strSendingEntity.empty() ) {
		ZKTRACE1( "Message was sent by %s\n", m_strSendingEntity.c_str() ) ;
	}

	if ( !m_strRecipientNym.empty() ) {
		ZKTRACE1( "Message was sent to %s\n", m_strRecipientNym.c_str() ) ;
	}

	if ( m_nReplyBlock != UnknownValue ) {
		ZKTRACE1( "Reply block to decrypt is %d\n", m_nReplyBlock ) ;
	}

	ZKTRACE0( "Fmg Signature Test Result: " )  ;
	PrintSigResult( m_FmgSignatureResult ) ;

	ZKTRACE0( "Sender Signature Test Result: " )  ;
	PrintSigResult( m_SignatureResult ) ;

	if ( m_bActivatedRB ) {
		ZKTRACE0( "Message was a reply block activation token - reply block was successfully activated\n" ) ;
	}

	ZKTRACE0( "\n\n" ) ;

#endif
}
