// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "StdAfx.h"
#include "WNymIconInfo.h"

WNymIconInfo::WNymIconInfo()
	: m_IconType( NymIconTypeUndefined )
	, m_IconIndex( DEFAULT_NYM_ICON_INDEX )
{
}

WNymIconInfo::WNymIconInfo( const WNymIconInfo& in_rhs )
{
	m_IconType = in_rhs.m_IconType ;
	m_IconIndex = in_rhs.m_IconIndex ;
	m_strNymIconFilePath = in_rhs.m_strNymIconFilePath ;
}

WNymIconInfo& 
WNymIconInfo::operator=( const WNymIconInfo& in_rhs )
{
	if ( &in_rhs != this )
	{
		m_IconType = in_rhs.m_IconType ;
		m_IconIndex = in_rhs.m_IconIndex ;
		m_strNymIconFilePath = in_rhs.m_strNymIconFilePath ;
	}

	return *this ; 
}

void 
WNymIconInfo::SetIconIndex( size_t in_IconIndex ) 
{ 
	m_IconType = NymIconIndex ; 
	m_IconIndex = in_IconIndex ; 
}

void 
WNymIconInfo::SetIconFilePath( ZKSTD::string in_IconFilePath ) 
{ 
	m_IconType = NymIconFilePath ; 
	m_strNymIconFilePath = in_IconFilePath ; 
}

size_t 
WNymIconInfo::GetIconIndex() const
{
	ZKASSERT( m_IconType != NymIconTypeUndefined ) ;

	if ( m_IconType == NymIconIndex )
	{
		return m_IconIndex ;
	}
	else
	{
		return DEFAULT_NYM_ICON_INDEX ;
	}
}

ZKSTD::string 
WNymIconInfo::GetFilePath() const
{
	// It has to be legal for this object to work
	// on different platforms.  This means until all
	// platforms support all possible encodings we
	// return default values rather than failing
	
	ZKASSERT( m_IconType != NymIconTypeUndefined ) ;

	if ( m_IconType == NymIconFilePath )
	{
		return m_strNymIconFilePath ;
	}
	else
	{
		return ZKSTD::string( "" ) ;
	}
}

void 
WNymIconInfo::Save( WMemBuf& io_Buffer ) throw(RZkException)
{
	ZKChunkWriter cw( io_Buffer, ICON_CHUNK_ID, 1 ) ;

	cw.WriteUInt32( (uint32_t) m_IconType ) ;
	
	if ( m_IconType == NymIconFilePath )
	{
		cw.WriteNonUIString( m_strNymIconFilePath ) ;
	}
	else if ( m_IconType == NymIconIndex )
	{
		cw.WriteUInt32(m_IconIndex ) ;
	}	
}

void 
WNymIconInfo::Load( ZKChunkReader& io_ChunkReader ) throw(RZkException)
{
	ZKASSERT( io_ChunkReader.GetChunkID() == ICON_CHUNK_ID ) ;	

	m_IconType = (NymIconType) io_ChunkReader.ReadUInt32() ;

	if ( m_IconType == NymIconFilePath )
	{
		io_ChunkReader.ReadNonUIString( m_strNymIconFilePath ) ;
	}
	else if ( m_IconType == NymIconIndex )
	{
		m_IconIndex = io_ChunkReader.ReadUInt32() ;
	}
}
