// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// WReadBuffer.h

// Similar to a FILE structure, this 
// class tracks a read pointer inside 
// an "in-memory" buffer/stream.

// Buffer is read only, it must not change sizes, and
// the memory is not owned by the ReadBuffer() - so it
// shouldn't go away until after any instance of this class
// do.

#ifndef _ZKWREADBUFFER_H
#define _ZKWREADBUFFER_H

#include <AutoInstCount.h>

#include "WCoreTypes.h"

#include "ClientTypesInc.h"

class ClientTypes_API WReadBuffer AUTO_INSTANCE_COUNTING1( WReadBuffer )
{
public:

	WReadBuffer( const unsigned char * in_pBuf, size_t in_cntBuffer ) ;

	// Gets some data from the stream at
	// the read point and moves the read point
	// Data is copied into provided buffer, which must have
	// size at least "in_cnt"
	// (See standard C _read() function for more details )

	int Read( void * io_buffer, unsigned int in_cnt ) ;

	// Position the ReadPosition in the stream
	int Seek( size_t in_newReadPos ) ;

	// Allows a client to remember a position in the stream, possibly in 
	// order to Seek() back in the future.
	size_t GetReadPos() const { return m_ReadPos ; }
	
	size_t GetBufferSize() const { return m_cntBuffer ; }

	bool IsEOF() const { return m_cntBuffer <= m_ReadPos ; }  

	// Get access to the buffer directly - for advanced use only
	const unsigned char * GetBuffer() const { return m_pBuf ; } 

private:
	const unsigned char * m_pBuf ;			//Buffer that we are reading

	size_t				  m_cntBuffer ;		//Buffer
	size_t				  m_ReadPos ;		//Current Read position
} ;

#endif