// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

//
// WRemoteNym.h
//
// This class wraps the frRemoteNym data type
// This is a versatile structure that can represent
// a set of parameters of various types.
//
// This is used to represent the attributes
// of a Nym.  However it can also be used to
// record other information, for example
// tokens.

#ifndef _CREMOTENYM_H
#define _CREMOTENYM_H

#include <DZkError.h>
#include <AutoInstCount.h>
#include <frNym.h>

class RZkException;

struct T_FrRemoteNym;

#include "ClientTypesInc.h"

class ClientTypes_API WRemoteNym AUTO_INSTANCE_COUNTING1( WRemoteNym )
{
public:
	WRemoteNym() ;
	~WRemoteNym() ;

	// Low level access
	T_FrRemoteNym* GetFrRemoteNym() { return m_pFrRemoteNym; }

	operator T_FrRemoteNym*() const { return m_pFrRemoteNym; }

	// Initialize our state with the contents of a buffer
	bool InitializeFromBuffer( 
			const unsigned char* in_pBuffer, 
			size_t in_cntBuffer) ;

	// Extract a buffer representation of the state of this
	// object.
	// Useful for persistence.
	bool GetBuffer( unsigned char** out_ppBuffer, size_t* cntBuffer ) const ;

	void GetDateParam( 
			enum FrRemoteNymParameter in_NymDateParameter, 
			struct T_ZkDateTime& out_Date ) 
			throw(RZkException);

	// Retrive a specific string from the Remote Nym
 	void GetStringParam( enum FrRemoteNymParameter, ZKSTD::string & out_string ) ;

	// Remember a string inside the Remote Nym
	void SetStringParam( enum FrRemoteNymParameter, ZKSTD::string & in_string ) ;

	// Copy a frRemoteNym
	// Replace with a more C++ approach (eg constructor)

	bool DupFrRemoteNym(T_FrRemoteNym* in_pRemoteNymToCopy, 
						T_FrRemoteNym** out_ppCopy);

	bool DupFrRemoteNymWithSignatureKey(T_FrRemoteNym* in_pRemoteNymToCopy, 
										T_FrRemoteNym** out_ppCopy);

	bool CopyReqdFrRemoteNymParamsToLocalFrRemoteNym(T_FrRemoteNym* io_ClientSide, 
													 T_FrRemoteNym* in_ServerSide);
	// Debug Helpers - TODO - should act on "this"
	void DebugTraceRemoteNym(T_FrRemoteNym*) ;

private:
	mutable DZkError	m_ZkError ;
	T_FrRemoteNym*		m_pFrRemoteNym ;
} ;

#endif
