// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "StdAfx.h"

#include "WRoute.h"
#include <clientdebug.h>


WRoute&
WRoute::operator=( const WRoute& in_rhs ) 
{ 
	if ( &in_rhs != this ) 
	{
		memcpy( m_hops, in_rhs.m_hops, sizeof( m_hops ) ) ;
	}

	return *this ;
}

bool 
WRoute::operator==(const WRoute& in_rhs) const
{
	return ( 0 == memcmp( m_hops, in_rhs.m_hops, sizeof( m_hops ) ) ) ;
}

size_t 
WRoute::GetSize() const 
{
	size_t retVal = 0 ;

	while(( retVal < MAX_AIPS_IN_ROUTE ) &&
		  ( m_hops[ retVal ] != 0 ) )
	{
		retVal++ ;
	}

	return retVal ;
}

bool 
WRoute::IsHopInRoute( uint32_t in_entity ) const
{
	ZKASSERT( in_entity != 0 ) ;

	bool bRet = false ;

	size_t pos = 0 ;

	while(( pos < MAX_AIPS_IN_ROUTE ) &&
		  ( m_hops[ pos ] != 0 ) )
	{

		if ( m_hops[ pos ] == in_entity )
		{
			bRet = true ;
			break ;
		}

		pos++ ;
	}

	return bRet ;
}

void 
WRoute::Clear()
{
	memset( m_hops, 0, sizeof( m_hops ) ) ;
}

void 
WRoute::AddHop( uint32_t in_entity )
{
	size_t cntHops = GetSize() ;

	if ( cntHops >= MAX_AIPS_IN_ROUTE )
	{
		// Error - we've gone over the 
		// maximum number of hops, we can't add it
		ZKASSERT( false ) ;
		return ;
	}

	m_hops[ cntHops ] = in_entity ;
}

void 
WRoute::PopHop()
{
	size_t cntHops = GetSize() ;

	if ( cntHops == 0 )
	{
		// Error - stack is empty
		ZKASSERT( false ) ;
		return ;
	}

	m_hops[ cntHops - 1 ] = 0 ;

	ZKASSERT( GetSize() == cntHops - 1 ) ;
}

uint32_t 
WRoute::GetHop( size_t in_WhichHop ) const
{
	if ( in_WhichHop >= MAX_AIPS_IN_ROUTE )
	{
		ZKASSERT( false ) ;
		return 0 ;		
	}

	return m_hops[ in_WhichHop ] ;
}


void
WRoute::SelfTest()
{
	WRoute r1 ;

	ZKTEST( r1.GetSize() == 0 ) ;
	r1.AddHop( 5 ) ;

	ZKTEST( r1.GetHop( 0 ) == 5 ) ;
	ZKTEST( r1.GetSize() == 1 ) ;

	ZKTEST( r1.IsHopInRoute( 5 ) ) ;
	ZKTEST( !r1.IsHopInRoute( 45 ) ) ;

	WRoute r2( r1 ) ;

	ZKTEST( r2.GetHop( 0 ) == 5 ) ;
	ZKTEST( r2.GetSize() == 1 ) ;

	r1.AddHop( 6 ) ;
	r1.AddHop( 7 ) ;

	ZKTEST( r1.GetSize() == 3 ) ;

	r2 = r1 ;

	ZKTEST( r2.GetSize() == 3 ) ;
	ZKTEST( r2.GetHop( 1 ) == 6 ) ;

	ZKTEST( r1 == r2 ) ;
	ZKTEST( r2 == r1 ) ;

	// Classic assignment operator bug
	r2 = r2 ;
	ZKTEST( r2.GetSize() == 3 ) ;

	// Demonstrate low level access for core library
	// usage
	const uint32_t * pRawHops = r2.GetHopArray() ;
	ZKTEST( pRawHops != NULL ) ;
	ZKTEST( pRawHops[ 2 ] == 7 ) ;
}