// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// class WRoute

// This class describes the hops in a route.

// All entities have an unique entity
// id on the system so we can specify routes with very
// little memory consumption

// There is a limit to the number of hops allowed
// (MAX_AIPS_IN_ROUTE)

#ifndef WROUTE_H
#define WROUTE_H

#include <clientdefs.h>
#include <zkIntTypes.h>
#include <AutoInstCount.h>

#include "ClientTypesInc.h"

// Represents a single route
class ClientTypes_API WRoute AUTO_INSTANCE_COUNTING1( WRoute )
{
public:
	WRoute() { Clear() ; } 
	WRoute( const WRoute& in_route ) 
        AUTO_INSTANCE_COPY_CTOR1( WRoute, in_route )
        { memcpy( m_hops, in_route.m_hops, sizeof( m_hops ) ) ; } 

	WRoute& operator=( const WRoute& in_rhs ) ;

	bool operator==(const WRoute& rhs) const ;

	// Reset the contents to no hops
	void Clear() ;

	// Get number of hops
	size_t GetSize() const ;

	// Add a hop to the route - this must respect the 
	// MAX_AIPS_IN_ROUTE limitation
	void AddHop( uint32_t in_entity ) ;

	// Remove the last hop ( expects at least
	// one hop to exist)
	void PopHop() ;

	// Get the hop specified by the index.
	// Index is zero based (eg 0 is the first hop)
	// Returns 0 as entity if hop doesn't exist yet
	uint32_t GetHop( size_t in_WhichHop ) const ;

	// Make sure this class is up and ready,
	// demonstrate its usage
	static void SelfTest() ;

	// For usage when calling the core libaries
	// than expect an array of Hops.  In client
	// code we should deal with WRoute
	uint32_t* GetHopArray() { return m_hops ; } 

	// Test if a hop is already in a route
	bool IsHopInRoute( uint32_t in_entity ) const ;

private:
	// Data
	// All entities have an unique entity
	// id on the system so we can specify routes with very
	// little memory consumption
	uint32_t m_hops[MAX_AIPS_IN_ROUTE];
} ;

#include <vector>
typedef ZKSTD::vector<WRoute> WRouteVec;

#endif
