// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// WSerialNumber.cpp

// 

#include "StdAfx.h"
#include "WSerialNumber.h"
#include <frCapability.h> 

WSerialNumber::WSerialNumber( const char * in_strSerialNumber ) 
	: m_pFrSerialNumber( NULL )
	, m_bWellFormed( false )
	, m_bCapabilitiesSet( false )
	, m_cntTokens( 0 )
	, m_sn_type( SERIAL_TYPE_MAX )
	, m_token_type( 0 )
	, m_activateTime( 0 )
{
	frSerialNumberNew( &m_Error, &m_pFrSerialNumber ) ;
	ZKASSERT( m_pFrSerialNumber != NULL ) ;
	ZKASSERT( in_strSerialNumber != NULL ) ;

	if ( in_strSerialNumber != NULL ) 
	{	
		if( frSerialNumberParse (
				&m_Error ,
				m_pFrSerialNumber ,
				const_cast<char*>(in_strSerialNumber) ,
				strlen( in_strSerialNumber ) )) 
		{
			// Serial number is bogus, probably too long or user hacking around
			m_bWellFormed = true ;
		}
	}
}

WSerialNumber::~WSerialNumber() 
{ 
	if ( m_pFrSerialNumber != NULL ) 
	{
		frSerialNumberFree( &m_Error, &m_pFrSerialNumber ) ;
	}
}

void
WSerialNumber::SetCapabilities( 
	char * in_pCapabilitiesBuffer, 
	size_t in_szCapabilitiesBuffer )
{
	ZKASSERT( m_bWellFormed ) ;
	
	if (( in_pCapabilitiesBuffer == NULL ) ||
		( in_szCapabilitiesBuffer == 0 ) ) 
	{
		ZKASSERT( false ) ;
		return ;
	}

	if ( !frCapabilityCodeParse( 
							&m_Error, 
							in_pCapabilitiesBuffer, 
							in_szCapabilitiesBuffer, 
							&m_cntTokens, 
							&m_token_type, 
							&m_sn_type, 
							&m_activateTime ) ) {
		// Bad input parameters?
		ZKASSERT( false ) ;
		return ;
	}

	m_bCapabilitiesSet = true ;
}


bool
WSerialNumber::IsTrialSerialNumber() const
{
	ZKASSERT( m_bWellFormed ) ;
	ZKASSERT( m_bCapabilitiesSet ) ;

	if (( m_sn_type >= SERIAL_TYPE_TRIAL_MIN ) &&
		( m_sn_type <= SERIAL_TYPE_TRIAL_MAX ) ) 
	{
		return true ;
	}
	
	return false ;
}
