// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// WSerialNumber.h

// Serial Number Object

// Represents a serial number, containing both
// the underlying internal representation of a serial number
// information about the validity and and capabilities of
// the serial number.

// Usage: A serial number is represented by a short string 
// for example - W23T2KYNWPSL9P.
// This object can also hold capability information about the
// serial number.  This is done by calling SetCapabilities()
// with information returned from the token server.


#ifndef _WSERIALNUMBER_H
#define _WSERIALNUMBER_H


#include <ErrorHandling.h>
#include <DZkError.h>
#include <zkList.h>
#include <frSerial.h>

#include <AutoInstCount.h>

#include "ClientTypesInc.h"

// Forward Declarations

// Class WSerialNumber
//

class ClientTypes_API WSerialNumber AUTO_INSTANCE_COUNTING1( WSerialNumber )
{
public:
	explicit WSerialNumber( const char * in_strSerialNumber ) ;
	~WSerialNumber() ;

	// Returns whether the serial number appears value (but this
	// has not necessarily be confirmed with the server)
	bool   IsWellFormed() const { return m_bWellFormed ; }

	// The following properties are available once SetCapabilities() is called:

	bool   IsTrialSerialNumber() const ;

	// Number of tokens this serial number will give
	size_t GetTokenCount() const { ZKASSERT( m_bCapabilitiesSet ) ; return m_cntTokens ; }

	// Returns type of serial number.  Range of number has significance, eg see SERIAL_TYPE_TRIAL_MIN
	size_t GetSerialNumberType() const { ZKASSERT( m_bCapabilitiesSet ) ; return m_sn_type ; }

	// Type of tokens this serial number will return.  The numbers ranges have 
	// significance
	size_t GetTokenType() const { ZKASSERT( m_bCapabilitiesSet ) ; return m_token_type ; }

	// A security 
	time_t GetSerialNumberActivateTime() const { ZKASSERT( m_bCapabilitiesSet ) ; return m_activateTime ; }

	// Set information about what the capabilities of this serial number
	// The WSerialNumber doesn't know how to contact the token server itself to get
	// this information
	void SetCapabilities( char * in_CapabilitiesBuffer, size_t in_szCapabilitiesBuffer ) ;

	// Operators allow this object to appear like the underlying FrSerialNumber
	// structure for the core library calls.
	operator FrSerialNumber()		 { return m_pFrSerialNumber ; } ;
	operator FrSerialNumber() const  { return m_pFrSerialNumber ; } ;

private:

	WSerialNumber() ;
	WSerialNumber( const WSerialNumber & in_rhs ) ;
	WSerialNumber& operator =( const WSerialNumber & in_rhs ) ;

	mutable	DZkError	 m_Error ;

	FrSerialNumber  m_pFrSerialNumber ;
	bool            m_bWellFormed ;	// Whether Serial number has correct format to be a real serial number
	bool            m_bCapabilitiesSet ;
	
	size_t          m_cntTokens ;
	unsigned short  m_sn_type ;
	unsigned short  m_token_type ;
	time_t          m_activateTime ;
} ;

#endif
