// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// WTokenDescriptions.cpp: implementation of the WTokenDescriptions class.
//
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "WTokenDescriptions.h"
#include <frTokenClient.h>
#include <frTokenCache.h>
#include <frTokenType.h>

WTokenDescriptions::WTokenDescriptions()
{
	m_type_ID = 0; // REVIEW need "special" Type ID
	m_duration = 0;
	m_reserved = 0;
	m_label = "Zero Knowledge Token";			//Default if we can't find get description from server
	m_email_domain = "Unknown Email domain";	//Default Email Domain - if exposed to user we should say something better
	m_access_level = 0;
	m_email_limit = 0;
	m_flags       = 0;
}

WTokenDescriptions::~WTokenDescriptions()
{

}

WTokenDescriptions::WTokenDescriptions(const FrTokenTypeDescription* in_pfrTypeDesc)
{
	m_type_ID = in_pfrTypeDesc->type_ID;
	m_duration = in_pfrTypeDesc->duration;
	m_reserved = in_pfrTypeDesc->reserved;
	m_label = in_pfrTypeDesc->label;
	m_email_domain = in_pfrTypeDesc->email_domain;
	m_access_level = in_pfrTypeDesc->access_level;
	m_email_limit = in_pfrTypeDesc->email_limit;
	m_flags = in_pfrTypeDesc->flags;
}

WTokenDescriptions::WTokenDescriptions(const WTokenDescriptions& rhs)

    AUTO_INSTANCE_COPY_CTOR1(WTokenDescriptions, rhs)
{
	WTokenDescriptions::operator=(rhs);
}

WTokenDescriptions& WTokenDescriptions::operator=(const WTokenDescriptions& rhs)
{
	if (this != &rhs){
		m_type_ID = rhs.m_type_ID;
		m_duration = rhs.m_duration;
		m_reserved = rhs.m_reserved;
		m_label = rhs.m_label;


		m_email_domain = rhs.m_email_domain;
		m_access_level = rhs.m_access_level;

		m_email_limit = rhs.m_email_limit;
		m_strTokenRandomNumber = rhs.m_strTokenRandomNumber;

		m_flags = rhs.m_flags ;
	}
	return (*this) ;
}

bool WTokenDescriptions::IsTrialToken()			
{
	return 0 != ( m_flags & FL_TOKEN_TRIAL ) ;
}


void WTokenDescriptions::TraceContents()
{
	ZKTRACE3( "Token Label: %s, Email Domain: %s, TypeID %d ", m_label.c_str(), m_email_domain.c_str(), m_type_ID ) ;
	ZKTRACE3( "\tDuration %d, Reserved %d, AccessLevel %d, ", m_duration, m_reserved, m_access_level);
	ZKTRACE2( "\tEmailLimit %d, Flags %d ", m_email_limit, m_flags ) ;

	if ( IsTrialToken() )
	{
		ZKTRACE0( "[Trial Token]" ) ;
	}

	ZKTRACE0( "\n" ) ;
}
