// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

//////////////////////////////////////////////////////////////////////
// WTokenDescriptions.h
// 
// This class represents the attributes of
// a single token, or group of tokens with
// identical attributes.
//
// This class is approximately a wrapper for
// FrTokenTypeDescription.  

// TODO: it needs to be changed implemented
// to include an instance of FrTokenTypeDescription,
// rather than duplicating the member data
//////////////////////////////////////////////////////////////////////

#ifndef _WTOKENDESCRIPTION_H
#define _WTOKENDESCRIPTION_H

#include "ClientTypesInc.h"

#include <zks.h>
#include <frToken.h>
#include <frCapability.h>
#include <frTokenType.h>
#include <client.h>

#include <AutoInstCount.h>

class ClientTypes_API WTokenDescriptions AUTO_INSTANCE_COUNTING1( WTokenDescriptions )
{
public:	
	WTokenDescriptions();
	virtual ~WTokenDescriptions();

	WTokenDescriptions(const FrTokenTypeDescription* );
	WTokenDescriptions(const WTokenDescriptions& rhs);
	WTokenDescriptions& operator=(const WTokenDescriptions& rhs);

	// clients use Get...()s
	unsigned short	GetTypeId() const		{return m_type_ID;}
	unsigned int	GetDurationInDays()		{return m_duration;}
	unsigned short	GetReserveTimeInDays()	{return m_reserved;}
	ZKSTD::string		GetTokenLabels()		{return m_label;}
	ZKSTD::string		GetEmailDomain()		{return m_email_domain;}		
	unsigned short	GetAccessLevel()		{return m_access_level;}
	unsigned short	GetEmailLimit()			{return m_email_limit;}
	ZKSTD::string		GetRandomNumber()		{return m_strTokenRandomNumber;}
	uint32_t		GetFlags()				{return m_flags;}

	bool			IsTrialToken();
	
	// Debug helper
	void			TraceContents() ;

private:
	unsigned short	m_type_ID;		/* Type ID number */
	unsigned int	m_duration;		/* Lifetime in days */
	unsigned short	m_reserved;		/* Reserve/hold time in days */
	ZKSTD::string	m_label;		/* Token label string (informational) */
	ZKSTD::string	m_email_domain; /* nym email address domain */
	unsigned short	m_access_level; /* Wormhole access level */
	unsigned short	m_email_limit;	/* Maximum number of mails per day */
	ZKSTD::string	m_strTokenRandomNumber;
	uint32_t		m_flags;
};

#endif

