// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

///////////////////////////////////////////////////////////////////////////////
//
// WZkipClient
//
// This class wraps the ZkipClient structure.   The benefit of using this class
// instead of the ZkipClient struct is that it is automatically disconnected and
// freed by the destructor when it goes out of scope.
// 
///////////////////////////////////////////////////////////////////////////////

#ifndef WZKIPCLIENT_H
#define WZKIPCLIENT_H

#include <zkipClient.h>		// Defines underlying code structure
#include "ClientTypesInc.h"
#include <AutoInstCount.h>

class ClientTypes_API WZkipClient AUTO_INSTANCE_COUNTING1( WZkipClient )
{
public:

	WZkipClient() ;
	virtual ~WZkipClient() ;

	// Provide access to underlying structure - eg when
	// making calls that expect a ZkipClient
	operator ZkipClient()		{ return m_pData ; }
	operator ZkipClient() const	{ return m_pData ; }
	ZkipClient* operator&()		{ return &m_pData ; }

	void DisconnectAndReset() throw();

private:
	WZkipClient( const WZkipClient& ) ;
	WZkipClient& operator= ( const WZkipClient & ) ;

	void DisconnectAndFree() throw();

	ZkipClient	m_pData ;
} ;

#endif //WZKIPCLIENT_H