// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// ZKConvert.cpp
//
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "ZKConvert.h"
#include <DZkError.h>
#include <zkDateTime.h>


void ZKConvert::TimeTFromZKDateTime( struct T_ZkDateTime& in_ZkDate,
											 time_t& out_Date )
{	
	DZkError	Err ;
	
	ZKVERIFY( zkDateTimeMkTime (&Err, 
								&in_ZkDate ,
								ZKTIME_FL_LOCAL ,
								&out_Date ) );
}


void ZKConvert::StringFromZKDateTime( struct T_ZkDateTime& in_ZkDate,
									  ZKSTD::string& out_strDate )
{
    time_t	    timetDate ;
	char        szDate[64];
	
	DZkError	Err ;

	if ( zkDateTimeMkTime (&Err, 
							&in_ZkDate ,
							ZKTIME_FL_LOCAL ,
							&timetDate ) ) {

		struct tm * ptmDate;
		ptmDate = localtime( &timetDate );		
		strftime( szDate, 64, "%d %B %Y", ptmDate );
		out_strDate = szDate ;

		/*
		// Standard format, but long

		char * pszDate = ctime( &timetDate );
		zkCStrlcpy( szDate, pszDate, sizeof(szDate) );
		szDate[strlen( szDate ) - 1] = 0;
		out_strBirthDate = szDate;
		*/
	}
	else
	{
		out_strDate = "";
	}
}


FrEntIdType 
ZKConvert::GetEntityTypeFromPort(const uint16_t in_DestPort)
{
	FrEntIdType retVal = FRENT_TYPE_UNUSED_ ;

	switch(in_DestPort)
	{
	case FR_DFLT_TOKEN_SERVER_PORT:
		retVal = FRENT_TYPE_TOK_SRV;
		break ;

	case FR_DFLT_KEYQRY_SERVER_PORT:
		retVal = FRENT_TYPE_KEYQRY_SRV;
		break ;

	case FR_DFLT_NYM_SERVER_PORT:
		retVal = FRENT_TYPE_NYM_SRV;
		break ;
	
	case FR_DFLT_NIQS_SERVER_PORT:
		retVal = FRENT_TYPE_NIQS;
		break ;

	case FR_DFLT_FMGMAIP_SERVER_PORT:
		retVal = FRENT_TYPE_FMG;
		break ;

	case FR_DFLT_MAIP_PORT :
		retVal = FRENT_TYPE_MAIP ;
		break ;

	case( FR_DFLT_NMTA_PORT ):
		retVal = FRENT_TYPE_NMTA ;
		break ;

	case( FR_DFLT_POP_PORT ):
		retVal = FRENT_TYPE_POP ;
		break ;

	case FR_DFLT_AIP_UDP_PORT :
	case FR_DFLT_AIP_TCP_PORT :
		retVal = FRENT_TYPE_AIP ;
		break ;		  

	}

	return retVal;
}

uint16_t 
ZKConvert::GetPortFromEntityType(const FrEntIdType& in_entType)
{
	uint16_t retVal = 0;

	switch(in_entType)
	{
	case FRENT_TYPE_TOK_SRV:
		retVal = FR_DFLT_TOKEN_SERVER_PORT;
		break ;

	case FRENT_TYPE_KEYQRY_SRV:
		retVal = FR_DFLT_KEYQRY_SERVER_PORT;
		break ;

	case FRENT_TYPE_NYM_SRV:
		retVal = FR_DFLT_NYM_SERVER_PORT;
		break ;
	
	case FRENT_TYPE_NIQS:
		retVal = FR_DFLT_NIQS_SERVER_PORT;
		break ;

	case FRENT_TYPE_FMG:
		retVal = FR_DFLT_FMGMAIP_SERVER_PORT;
		break ;

	case FRENT_TYPE_MAIP:
		retVal = FR_DFLT_MAIP_PORT;
		break ;

	case FRENT_TYPE_NMTA:
		retVal = FR_DFLT_NMTA_PORT;
		break ;

	case FRENT_TYPE_POP:
		retVal = FR_DFLT_POP_PORT;
		break ;

	default:
		ZKASSERT(false);
	}

	return retVal;
}

ZKSTD::string
ZKConvert::GetServerNameFromType( FrEntIdType in_Type )
{
	// Resource this....

	switch(in_Type)
	{
		case FRENT_TYPE_AIP :
			return ZKSTD::string( "Freedom AIP" ) ;
		
		case FRENT_TYPE_MAIP :
			return ZKSTD::string( "MAIP Server" ) ;

		case FRENT_TYPE_TOK_SRV:
			return ZKSTD::string( "Token Server" ) ;

		case FRENT_TYPE_NYM_SRV:
			return ZKSTD::string( "Nym Server" ) ;

		case FRENT_TYPE_NIQS:
			return ZKSTD::string( "NIQS Server" ) ;

		case FRENT_TYPE_KEYQRY_SRV:
			return ZKSTD::string( "Key Query Server" ) ;

		case FRENT_TYPE_FMG:
			return ZKSTD::string( "FMG Server" ) ;

		case( FRENT_TYPE_NMTA ):
			return ZKSTD::string( "Nym Mail Transfer Agent Server" ) ;

		case( FRENT_TYPE_POP ):
			return ZKSTD::string( "Freedom POP Server" ) ;

		// client should never see this on
		case( FRENT_TYPE_IMEP ):
			return ZKSTD::string( "Internet Mail Encryption Proxy Server" ) ;
	};

	ZKASSERT( false ) ;
	return ZKSTD::string( "Unknown Server" ) ;
}

ZKSTD::string
ZKConvert::GetServerNameFromPort( uint16_t in_DestPort )
{
	return GetServerNameFromType( GetEntityTypeFromPort( in_DestPort ) ) ;
}
