// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

//
// ZKCONVERT
//
// This helper class provides conversion assitance for dealing with the UNIX library helper 
// Because we try to hide the UNIX structures from code higher than the Facade it is unlikely
// that we would want to export this class from the Facade DLL
// 

#ifndef _ZKCONVERT_H
#define _ZKCONVERT_H

#include "ClientTypesInc.h"
#include <frNetInfo.h>
#include <client.h>	// for ZKSTD

#include <AutoInstCount.h>

class ClientTypes_API ZKConvert /* AUTO_INSTANCE_COUNTING1( ZKConvert ) */
{
private:
	ZKConvert(); // this class cannot be instantiated.
	
public:
	static void TimeTFromZKDateTime ( struct T_ZkDateTime& in_ZkDate, time_t& out_Date ) ;
	static void StringFromZKDateTime( struct T_ZkDateTime& in_ZkDate,
									  ZKSTD::string& out_strDate ) ;

	// PURPOSE: We use this function simply to determine the entity type associated with  a port
	//			Notice it returns the enumeration 'FrEntIdType'
	// PROMISE:	Returns FRENT_TYPE_UNUSED_ after asserting if the port is not one for our unauthenticated services

	static FrEntIdType GetEntityTypeFromPort(const uint16_t in_DestPort) ; 

	// PURPOSE: We use this function simply to determine the port associated with an entity type 
	// PROMISE:	Returns 0 if the entity type is not known
	static uint16_t GetPortFromEntityType(const FrEntIdType& in_entType) ;

	static ZKSTD::string GetServerNameFromPort( uint16_t in_DestPort ) ;
	static ZKSTD::string GetServerNameFromType( FrEntIdType in_Type ) ;
} ;

#endif
