// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

////////////////////////////////////////////////////////////////////////////////
// IAdBlocker.h

////////////////////////////////////////////////////////////////////////////////

#ifndef _IAdBlocker_H
#define _IAdBlocker_H

////////////////////////////////////////////////////////////////////////////////
// Includes - (please keep to a minimum)
////////////////////////////////////////////////////////////////////////////////

#include <ErrorHandling.h>
#include <client.h>
#include "ZkServiceInc.h"
#include <vector>

#define BLOCK_LIST_FILE_NAME "adblocker.dat"
#define AD_BLOCKER_CHUNK_VERSION 1

////////////////////////////////////////////////////////////////////////////////
// Interface
////////////////////////////////////////////////////////////////////////////////

class ZkService_API AdDescription
{
public:
	AdDescription(const ZKSTD::string& in_strFriendlyName, const ZKSTD::string& in_strURL,
								bool in_bException);
	AdDescription(const ZKSTD::string& in_strFriendlyName, const ZKSTD::string& in_strURL);
	AdDescription(const ZKSTD::string& in_strRule);
	AdDescription(void);

	void GetRuleString(ZKSTD::string& out_strRule);
	const ZKSTD::string& GetURL() const { return m_strURL; }
	const ZKSTD::string& GetFriendlyName() const { return m_strFriendlyName; }
	bool IsException() { return m_bException; }
	bool IsBadData() { return m_bBadData; }



#ifdef INTERNAL_BUILD
	void MarkForUpload(bool in_bUpload = true) { m_bUpload = in_bUpload;}
	bool IsMarkedForUpload() { return m_bUpload;}
#endif
private:
	ZKSTD::string	m_strFriendlyName;
	ZKSTD::string	m_strURL;
	bool			m_bException;
	bool			m_bUpload;
	bool			m_bBadData;
};

class IAdBlocker
{
public:
	////////////////////////////////////////////////////////////////
	// PURPOSE: Determine if a URI should be blocked. Checks for
	//			matches in the block list file, and returns true if
	//			the file should be replaced by a bogus response.
	//
	// REQUIRE: strUri is a null terminated string.
	//
	// PROMISE: Not to die. Returns false if an error occurs
	//			(block list	hasn't been loaded)
	virtual bool	UriIsBlocked(const char *strUri) PURE;
	
	////////////////////////////////////////////////////////////////
	// PURPOSE: Get the response that should be given to the user
	//			following a blocked request.
	//
	// REQUIRE: nothing.
	//
	// PROMISE: Returns a buffer (and length) to be sent as-is to the
	//			browser.
	virtual void	GetBogusResponse(const char **out_ppBogusResponse,
									size_t *out_pBogusResponseLength) PURE;	
	////////////////////////////////////////////////////////////////
	// PURPOSE: Reload the blocklist (call after an update from the script server)
	//
	// REQUIRE: nothing.
	//
	// PROMISE: Loads the blocklist from disk.
	virtual void	ReloadBlocklist() PURE;	
	
	virtual void	SaveToDisk() PURE;
	virtual void	GetBlockList(ZKSTD::vector<AdDescription>& out_vecAdList) PURE;
	virtual void	SetBlockList(ZKSTD::vector<AdDescription>& in_vecAdList, bool bSaveToDisk = true) PURE;
	virtual void	AddEntry(const ZKSTD::string &in_strRule, bool bSaveToDisk = true) PURE;
	virtual void	RemoveEntry(const ZKSTD::string &in_strURL, bool bSaveToDisk = true) PURE;
	virtual long	GetNumberOfAdsBlocked() PURE;
	virtual void	SetNumberOfAdsBlocked(long) PURE;
	virtual long	GetEstimatedSavedBandwidth() PURE;

#ifdef INTERNAL_BUILD
	virtual void UploadToScriptServer() PURE;
#endif
	virtual void	SelfTest() PURE ;
} ;

#endif
