// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// IClientPersist

// This service provides our on disk persisted
// state.

//Usage
//When loading up it is possible to determine
//if there is any persisted state using IsFirstTimeRun()

//The persisted state can be loaded with LoadClientState()
//Because the persisted data is encrypted the correct 
//passphrase must be provided

//Each major change, for example creation of
//tokens, adjustment of a nym properties etc, should
//be committed by saving the state

//GetActiveFilePath() can be used by low-level services such
//as Backup.  However the backup support itself could
//move into this interface as a clean up.

//REVIEW: The design pattern "momento" is interesting related
//to what this class does.  Please review the design pattern book
//before making big changes to this class.  It might give
//some good ideas.


#ifndef _ICLIENTPERSIST_H
#define _ICLIENTPERSIST_H
#include <client.h>

#include <string>
class RZkException;

class IClientPersist
{
public:	

	// Freedom state needs to be created (eg with SetupWizard) if it
	// is not already setup. 
	virtual bool IsFirstTimeRun() PURE ;

	// Loads out of our persisted file.  because the file is encrypted a passphrase is necessary
	// Will throw if the passphrase is incorrect, the file does not exist, etc etc
	virtual void LoadClientState(const ZKSTD::string& in_SuggestedPassphrase) throw(RZkException) PURE ;

	// Saves state.  Client calls whenever a major change as been made
	// REVIEW: in future the ZkService may manage its dirty state itself and
	// the UI will never know when the saves actually occur
	virtual void SaveClientState() throw(RZkException) PURE ;



	// If the state change is very important we should save immediately by
	// calling SaveClientState().  However if the change is more minor,
	// we mark our state as dirty so that we know to save next periodic change.
	// Normally only object at the ZkService level should call this,
	// the UI and other higher level components don't know exactly when they
	// dirty the Client State.
	virtual void SetDirty() PURE ;

	// Test if Client State is marked as dirty
	virtual bool IsDirty() PURE ;

	// Tells the complete path that we persist into
	// Note: this is only for use by low level services such as backup
	virtual void GetActiveFilePath( ZKSTD::string& out_FileWithPath )throw(RZkException) PURE ;
} ;

#endif