// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

////////////////////////////////////////////////////////////////////////////////
// ICookieManager.h

////////////////////////////////////////////////////////////////////////////////

#ifndef _ICookieManager_H
#define _ICookieManager_H

////////////////////////////////////////////////////////////////////////////////
// Includes - (please keep to a minimum)
////////////////////////////////////////////////////////////////////////////////

#include <ErrorHandling.h>
#include <client.h>	// for ZKSTD
#include <string>

////////////////////////////////////////////////////////////////////////////////
// Enumerations
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Interface
////////////////////////////////////////////////////////////////////////////////

class ICookieManager
{
public:		

	// HTTP Proxy interface

	// Given the entire cookie and its intended domain this parses out the
	// details 
	// REVIEW: This call should be combined with SetNymCookieMapEntry

	virtual void    AddCookie( 
						const ZKSTD::string& in_cookie, 
						const ZKSTD::string& in_domain ) PURE ;


	// ELW: Build the appropriate cookie-field for an outgoing request based on the domain name.
	virtual bool	BuildCookieField(
						const ZKSTD::string& in_strDomainToLookUp, 
						ZKSTD::string&       out_strCookiesForThatDomain ) PURE ;
	
	// TDB: In the future we can keep seperate counters and have different
	// UI indication for incoming and outgoing cookies
	virtual unsigned long * GetCookieCounter() PURE ;

	// Used by Nyms to notify that they have updated their cookie
	// data or sent out cookies
	virtual void    SignalCookieEvent() PURE ;


	// Self Test
	virtual void	SelfTest() PURE ;
	
} ;

#endif
