// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// ICoreServerConnect Interface declaration

#ifndef _ICORESERVERCONNECT_H__
#define _ICORESERVERCONNECT_H__

#include <zkIntTypes.h>
#include <fr_entid.h>
#include <RZkException.h>

class WZkipClient;

class RZkException;

class ICoreServerConnect
{
public:

	// PURPOSE:	Connect to a core server through an unauthenticated route.
	//				
	// REQUIRE: The caller must disconnect and free out_client if this function does not throw.
	//			in_nTimeoutPerConnect is in seconds.
	//
	// PROMISE: To throw if the client doesn't have any info about this entity type or
	//			if unable to build a route or connect even after in_nAttempts.
	//			(out_client == NULL) if this function throws
	//			(out_client != NULL) if this function doesn't throw,
	//			If (in_nAttempts == UINT32_MAX) a default number of attempts will be
	//			tried (as specified in the "CoreServerNumAttempts" registry key or 
	//			DEFAULT_NUM_ATTEMPTS if unspecified).
	//			An attempt fails if either the route create or the socket connect fails.

	virtual void SmartConnectThroughCloud(
			const FrEntIdType& in_entityType,
			WZkipClient& out_client,
			const int& in_nTimeoutPerConnect,
			const size_t& in_nAttempts = UINT32_MAX) throw(RZkException) PURE ;


	// PROMISE: Provide a connection to the NIQS service
	// The NIQS is a special case because, as the provider of topology information
	// it must be able to bootstrap when there is no topology information yet
	virtual void ConnectToNiqs(
			WZkipClient& out_client,
			const int& in_nTimeoutPerConnect,
			const size_t& in_nAttempts = UINT32_MAX) const throw(RZkException) PURE ;
} ;

#endif //_ICORESERVERCONNECT_H__
