// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// IKeyQry Interface declaration

// This interface offers a key lookup service for 
// the client.

// Public keys for ZKS entities are available
// from a Key Query Server.  Entities include the
// freedom servers, MAIPs, Nyms etc.

// Some key lookups will respond without network
// access because the keys are found on a local cache.


#if defined(WIN32)

#elif defined(ZK_LINUX)
#	include <clientlinux.h>
#endif

#include <zksocket.h>
#include <fr_keyqry.h>
#include <fr_entid.h>
#include <fr_gencrypt.h>
#include <WNetInfoDatum.h>

// forward declarations
class DZkError;
class RZkException;


class IKeyQry
{
public:

	// This performs a Key Lookup from the Cache and a 
	// Key Query to the Key Query Server if the key is not in the local cache.

	// NETWORK ACCESS: KeyQrySrvr if key is not cached.  Hence it can
	// be expected to fail from time to time.

	// Throw could replace use of DZkError argument
	// shouldn't return both bool AND throw
	virtual bool		GetKey(	DZkError&, 
								const FrEntIdType  ent, 
								const int          nTimeout, 
								const char*        ccEntityName, 
								const unsigned int         uKeyVersion, 
								const FrPubKeyType keytype, 
								OUT FrCryptPubKey* outkey ) throw(RZkException) PURE;


	// Get the public key for an entity.  For example "Get the Public Signing KeyFromCache
	// for the Nym "DarkAvenger"
	// This version never makes any network access, but will return the key if
	// it is in the local key cache.

	virtual bool GetKeyFromCache( 
								DZkError& err, 
								const FrEntIdType	in_entity, 
								const char*			in_strEntityName, 
								const unsigned int	in_uKeyVersion, 
								const FrPubKeyType  in_keytype, 
								OUT FrCryptPubKey*	out_pKey ) PURE ;

	virtual void GetMultipleKeys(FrEntIdType			 in_EntityType, 
								 const WNetInfoDatumVec& in_EntitiesToLookup, 
								 FrPubKeyType			 in_keytype            // Type of key to lookup
								) throw(RZkException) PURE ;

	// TODO - Breaks abstraction
	virtual FrKeyQry	GetKeyCache() PURE ;

	// TODO - Get the listening socket.  
	// This breaks abstraction, is necessary because of the design of server libraries,
	// do not use it if possible!
	virtual		ZkSocketDescriptor	GetKeyQuerySocket() PURE ;

	// PURPOSE: For all the Aips download their keys if it isn't in our cache
	//			Keys of type FRCRYPT_KEYTYPE_CRYPTION and FRCRYPT_KEYTYPE_SIGNING are retrieved.
	//			Also downloads keys for all FRENT_TYPE_FMG entities.
	// PROMISE: To Save the latest state of the Key cache before returning
	//			To Get the FRENT_TYPE_AIP and FRENT_TYPE_FMG entities from the INiqs interface.
	// REQUIRE: That The INiqs return some entities for any downloading to occur
	virtual void		DownLoadKeysIfRequired() PURE;
} ;


