// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// INetworkPerformance

// The interface provides access to a service which 
// implements 


#ifndef INETWORKPERFORMANCE_H_
#define INETWORKPERFORMANCE_H_

#include <zkIntTypes.h>
#include <clientenum.h>
#include <WCoreTypes.h>

#define MAX_PERFORMANCE_RESULT	   255	// This is good
#define DEFAULT_PERFORMANCE_RESULT 128
#define MIN_PERFORMANCE_RESULT	     0	// This is bad server

class WRoute ;
class WReadBuffer ;
class RZkException ;

class INetworkPerformance
{
public:
	
	// Route Create time management

	// This function is used to feed the route create data to the 
	// service.
	// In future we may add argument for which hops were used
	// so that we collect route create times per aip

	virtual void	 AddRouteCreateTime( uint32_t in_msCreateTime ) = 0 ;

	// Pull stats out of the historic route creation data
	virtual uint32_t GetRouteCreateStat( StatisticType in_whichStat ) = 0 ;

	// Route Create Statistics

	virtual void     IncrementRouteCreateAttempts() = 0 ;
	virtual void     IncrementRouteCreateSuccess() = 0 ;

	// Core Server Connect Stats
	virtual void     IncrementCoreServerConnectCloudAttempts() = 0 ;
	virtual void     IncrementCoreServerConnectCloudSuccess() = 0 ;

	// Route Black List - this tracks routes that failed in the past so
	// that we can avoid the aips in future route creates

	virtual void     AddRouteToBlackList( const WRoute& in_route ) = 0 ;
	virtual bool     IsRouteBlackListed( const WRoute& in_route ) const = 0 ;

	// Entity performance management

	// New Entities might be added to the network, this
	// forces us to update our structures to recognize new 
	// entities.  It also provides the initial bootstapped.  When called
	// the Niqs will be consulted to discover latest lists of entities
	// that we track performance info for.

	virtual void UpdateEntityList() = 0 ;
	virtual uint32_t GetEntityPerformanceResult( uint32_t in_Entity ) = 0;

	// This function launches the performance test
	// If in_bStartImmediately is false it will perform the
	// test at some random time (within a certain range) in the future 
	virtual void CalculatePerformance( bool in_bStartImmediately ) = 0 ;

	// Part of the interface only to make it visible from an internal timer
	virtual void HandleCalculatePerformanceTimer() = 0;

	// This functions sends the current network statistics to the console/trace window
	virtual void ShowCurrentStats(ConsSeverity in_Severity = CONS_DEBUG ) = 0 ;

	// PROMISE - flush any remembered statistics for the Aip, route creates,
	// route failures etc.  Basically for debugging purposes
	virtual void ResetAllStats() = 0 ;

	// Provides access to an array of Entities ids sorted by current
	// preformance rating "down" servers.  Caller is responsible to
	// zkMemFree() the list
	// Perhaps this can be replaced by an iterator over the local copy
	// help by Network performance
	virtual void GetAipIdsSortedByRating( 
					unsigned int ** out_pEntities, 
					unsigned int & out_cntEntities ) = 0 ;

	// Persistence Methods

	// PROMISE: Serialize the state into a chunked buffer
	// NETWORK: No access required
	virtual void Save( WMemBuf& io_Buffer ) throw(RZkException) = 0 ;

	// PROMISE: Any existing state is completely lost and a new state is read from the
	//			stream
	// NETWORK: No access required
	virtual void Load( WReadBuffer& io_Buffer ) throw(RZkException) = 0 ;

	virtual void SelfTest() = 0 ;
} ;

#endif
