// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// INymCookieMgr Manages the cookies
// for an individual nym, providing
// query, persistence, and update capabilities to 
// the "Cookie Jar" of this nym

#ifndef _INYMCOOKIEMGR_H
#define _INYMCOOKIEMGR_H

#include <client.h>
#include <WCoreTypes.h>
#include <ZkSubject.h>
#include <ZkHint.h>

////////////////////////////////////////////////////////////////////////////////
// Types
////////////////////////////////////////////////////////////////////////////////

class DomainAndDate
{
public:
	DomainAndDate() : SecondsExpire( 0 ) { ; }

	ZKSTD::string DateRecd;
	ZKSTD::string DateExpire;
	long		  SecondsExpire;
	ZKSTD::string Domain;
} ;

// COOKIEJARMAP maps from Cookie Data to the Attributes of a Cookie

typedef std::multimap<std::string, DomainAndDate> COOKIEJARMAP;

class ZKChunkReader ;

class INymCookieMgr : public ZkSubject
{
public:

	// HTTP Proxy interface

	// Given the entire cookie and its intended domain this parses out the
	// details 
	// REVIEW: This call should be combined with SetNymCookieMapEntry

	virtual void    AddCookie( 
						const ZKSTD::string& in_cookie, 
						const ZKSTD::string& in_domain ) PURE ;


	// ELW: Build the appropriate cookie-field for an outgoing request based on the domain name.
	virtual bool	BuildCookieField(
						const ZKSTD::string& in_strDomainToLookUp, 
						ZKSTD::string&       out_strCookiesForThatDomain ) PURE ;
	
	// Access to the entire state
	// This is useful for test code
	// It is also used by the UI, although for proper multi-threaded access
	// the UI should perform individul Cookie operations.
	virtual bool	GetNymCookieMap(COOKIEJARMAP& out_Cookies ) PURE ;
	virtual bool    SetNymCookieMap(COOKIEJARMAP& in_Cookies ) PURE ;

	virtual void	Save( WMemBuf& io_Buffer ) PURE ;
	virtual void	Load( ZKChunkReader& io_ChunkReader ) PURE ;

	// Call to load the nyms cookies out of the old file format
	// Nym Cookies used to be stored in individual files in the installation dir
	// We expect the the crypto sub-system has already been primed with the correct
	// passphrase
	// Losing cookies is not a fatal error so we don't throw
	virtual void	LoadOldV1XFormat() PURE ;

	// Self Test
	virtual void	SelfTest() PURE ;


	// Hint class returned by NymCookieMgr when events happen
    class NymCookieMgrHint 
        : public ZkHint
    {
    public:

        // Add other events which cause a change in route manager's state here
        enum {
            NYMCOOKIEMGR_ADDED_COOKIE,			// New cookie arrives
            NYMCOOKIEMGR_UPDATED_COOKIE			// Value of an existing cookie changed
        } ;
        
        NymCookieMgrHint( int stateChange )
            : m_StateChange( stateChange )
        { } ;

        ~NymCookieMgrHint() {} ;

        int GetStateChange() { return m_StateChange ; } ;
    private:
        int m_StateChange;
    } ;


} ;

#endif
