// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// INymMail

// Provided mail related functionality related to
// a particular nym 

// Functionality includes 
// - Reply blocks, reply block activation
// - Email Block List
// - Email Address
// - Decryption service on buffers encrypted with nyms key and
//    possibly wrapped in reply blocks

#ifndef _INYMMAIL_H
#define _INYMMAIL_H

#include <zkError.h>
#include <zkList.h>
#include <zkMIME.h>
#include <frNym.h>
#include <DZkError.h>
#include <WNetInfoDatum.h>
#include <WCoreTypes.h>
#include <WCypherMailMsg.h>

#define MAX_NUM_REPLY_BLOCKS 5
#define MIN_NUM_REPLY_BLOCKS 3

#define OPTIMAL_ACTIVE_REPLYBLOCKS	3
#define MIN_HOPS_IN_REPLYBLOCK		2
#define MAX_HOPS_IN_REPLYBLOCK		4

// The nyms email state can be described as 
// one of the following simplified states:

enum NymEmailState {
	NymEmailFullyFunctional   = 1,   // At least 3 active reply blocks
	NymEmailPartialFunctional = 2,   // At least 1 active reply blocks
	NymEmailNotFunctional     = 3    // No active reply blocks (maybe not active at all
} ;

class RZkException;
class ZKChunkReader ;

class INymMail
{
public:

	// 2.0 Mail Functionality

	virtual bool	NymHasTaglineChoice() PURE;

	virtual void	TransitionIfNecessary() PURE;
	// Used to create account on POP server
	virtual bool	RetrieveNymMailCert() throw(RZkException) PURE;

	void CreatePopAccountOnPopServer() throw(RZkException);

	// Used if our old certificate is rejected - try and get a new one
	virtual bool	UpdateMailCertificate() PURE;

	// Used to log in to POP/ESMTP servers
	virtual void	GetMailCertificate(ZKSTD::string& out_strCert) PURE;

	// Used to validate FMG signature of Internet->Nym mail.
	virtual FrCryptMacKey GetMailMacKey() PURE;


	///////////////////////////////////////////////////////////////////////////
	// Block list support
	///////////////////////////////////////////////////////////////////////////

	virtual void	GetEmailAddressBlockList(ZKSTD::vector<ZKSTD::string>&) PURE ;
	virtual void	SetEmailAddressBlockList(const ZKSTD::vector<ZKSTD::string>&) PURE ;
	virtual void	SetSpamPreference(bool) PURE ;
	virtual bool	GetSpamPreference()  PURE ;

	virtual void	SetTaglinePreference(bool) PURE ;
	virtual bool	GetTaglinePreference() PURE ;
  
        ///////////////////////////////////////////////////////////////////////////
        // Mail limit support
        ///////////////////////////////////////////////////////////////////////////
	virtual void     SetCurrentMailSentCount( const unsigned int ) PURE;
        virtual unsigned int    GetCurrentMailSentCount() const PURE;

        virtual void     SetMailSentInitTime( const time_t ) PURE;
        virtual time_t          GetMailSentInitTime() const PURE;



 	///////////////////////////////////////////////////////////////////////////
 	// (Legacy) Reply Block Mail Decryption support 
 	///////////////////////////////////////////////////////////////////////////
 
 	// Given a email message with its layers of encryption and signatures, this will determine
 	// the plain text version
 	// All details of the process, including the final decrypted message will be filled into
 	// the WCypherMailMsg argument
 
#ifdef WIN32
 	virtual void    DecryptMessage( 
 						WCypherMailMsg&  io_MailMsg ) throw(RZkException) PURE;
#endif

	// Self Test
	virtual void SelfTest() PURE ;
	
	virtual void	Save( WMemBuf& io_Buffer ) PURE ;
	virtual void	Load( ZKChunkReader& io_ChunkReader ) PURE ;
} ;

#endif
