// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// This interface handles the routing preferences
// and other settings for a nym 
// It can also collect routing statistics and other 
// information for a nym

// This interface does not provide any actual routing
// logic or functionality - please see RRouteManager for
// that code.  Perhaps if we someday offer multiple routes
// for multiple nyms at the same time this interface would
// help manage the different active nym routes in a sensible way

#ifndef INYMROUTE_H
#define INYMROUTE_H


#include <WCoreTypes.h>

class ZKChunkReader ;
class WRoute ;
class RZkException ;

class INymRoute 
{
public:
//	void IsInteractiveModeOn( 

	// PROMISE: Fills in the argument with 
	// the last successful route.  If no route has
	// ever been successful for this poor nym then 
	// route is returned empty
	// Watch out if nym properties for hop count changes
	// - the returned route might not match the required number of
	// hops
	virtual void GetLastSuccessfulRoute( WRoute& out_LastRoute ) = 0 ;

	// Called when a route is successfully created for this nym
	// so that it can be remembered by INymRoute
	virtual void SetSuccessfulRoute( const WRoute& in_Route ) = 0 ;

	/////////////////////////////////////////////////////////////////////////
	// Persistence
	/////////////////////////////////////////////////////////////////////////

	virtual void Save( WMemBuf& io_Buffer ) throw(RZkException) = 0 ;
	virtual void Load( ZKChunkReader& io_ChunkReader ) throw(RZkException) = 0 ;

} ;



#endif