// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

///////////////////////////////////////////////////////////////////////////////
// IPacketFirewall

// This interface provides the logic for handling
// packet traffic leaving the machine

// Potentially all such events are responded to in the following
// way:
// -Block them
// -Allow them out un-hindered
// -Reroute them.  In the case of TCP socket connection this means to reroute to a
//                 localhost proxy
//                 In the case of packets this means to send the packet as encrypted
//				   data through the Freedom Cloud

// These are the low level Methods called by the TrafficFilter (Packet handling code)
// Higher level Firewall functionality is covered by IFirewall

#ifndef _IPacketFirewall_H
#define _IPacketFirewall_H

#ifdef ZK_LINUX
#	include <clientlinux.h>
#endif // ZK_LINUX

#include <zkIntTypes.h>
#include <client.h>

struct  ZkBuffer;
struct	ZkNetFrame;

class IPacketFirewall
{
public:
	// PURPOSE: Called when we received incoming packet 
	//			Returning true will allow the packet into the stack
	//			Returning false will prevent it from doing so
	virtual bool 
	HandleIncomingPacket(const ZkNetFrame* ) = 0;

	// PURPOSE:	This function determines how to treat an outgoing packet of data
	//			Lowlevel TrafficFilter can use this information to determine 
	//			how to handle all traffic going out of the machine.
	// 
	// USAGE:	Should you want it to take the cloud, return true and 
	//          specify which route by setting the second parameter
	//			Otherwise simply return false and it will exit your packet as per usual
	virtual FirewallAction	
	GetActionForOutgoingTCPPacket( ZkBuffer *in_pbuf ) = 0;

	// PURPOSE:	This function determines how to treat an outgoing UDP packet of data
	//			Lowlevel TrafficFilter can use this information to determine 
	//			how to handle all traffic going out of the machine.
	// 
	// USAGE:	Should you want it to take the cloud, return true and 
	//          specify which route by setting the second parameter
	//			Otherwise simply return false and it will exit your packet as per usual

	virtual FirewallAction	
	GetActionForOutgoingUDPPacket( ZkBuffer *in_pbuf ) = 0;

	// PURPOSE: Given the the icmp message type, a decision needs to be made
	//			on what to do with this packet. Currently ICMP's are only
	//			blocked, or not. They are not anonymized.
	// REQUIRE: A valid icmp type as parameter, see <zkpacket.h>
	virtual FirewallAction 
	GetActionForOutGoingIcmp(int in_type) = 0;
} ;

#endif