/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
	Interface used between the firewall and the upper-layers to communicate
	various notifications. It was deemed better suited than using subject-observer
	since at times we will want the results from notifications, so it will
	be more of a synchronous invocation


	REVIEW - This interface doesn't only provide notification services.
	Functions like "AppRequestsOutNetAccess" actually return with a user decision
	about firewall behavior.  In otherwords is acts as a two way communication channel
	between user and firewall.  (Otherwise subject/observer would have been fine)
	
*/

#include <clientenum.h> // For eSelection

class PacketHelper;

class IPacketNotification
{
public:
	IPacketNotification() {};

	// PURPOSE: This method is called when an incoming packet is being dropped
	virtual bool NotifyDroppingIncomingPacket(  PacketHelper* ) = 0;

	// PURPOSE: This method is called when an incoming packet is accepted
	virtual bool NotifyAcceptingIncomingPacket(  PacketHelper* ) = 0;

	// PURPOSE: This method is called when a connection to our local http 
	//			proxy is being done.
	virtual bool NotifyConnectionHttp(const unsigned int in_ip, const unsigned short in_port) = 0;

	// PURPOSE: This method is called when a connection to our local irc
	//			proxy is being done.
	virtual bool NotifyConnectionIrc(const unsigned int in_ip, const unsigned short in_port) = 0;

	// PURPOSE: This method is called when a connection to our local nntp
	//			proxy is being done.
	virtual bool NotifyConnectionNntp(const unsigned int in_ip, const unsigned short in_port) = 0;

	// PURPOSE: This method is called when a connection to our local ssl
	//			proxy is being done.
	virtual bool NotifyConnectionSsl(const unsigned int in_ip, const unsigned short in_port) = 0;

	// PURPOSE: This method is called when a connection to our local telnet
	//			proxy is being done.
	virtual bool NotifyConnectionTelnet(const unsigned int in_ip, const unsigned short in_port) = 0;

	// PURPOSE: This method is called when a connection to our local telnet
	//			proxy is being done over port 22
	virtual bool NotifyConnectionSsh(const unsigned int in_ip, const unsigned short in_port) = 0;

	// PURPOSE: This method is called when a connection to our local smtp
	//			proxy is being done.
	virtual bool NotifyConnectionSmtp(const unsigned int in_ip, const unsigned short in_port) = 0;

	// PURPOSE: This method is called when a connection to our local pop3
	//			proxy is being done.
	virtual bool NotifyConnectionPop3(const unsigned int in_ip, const unsigned short in_port) = 0;	
	

	// PURPOSE: This method is called when a connection is about to be let out
	virtual bool NotifyAllowingSocketConnection( PacketHelper* ) = 0;

	// PURPOSE: This method is called when a connection is about to be refused
	virtual bool NotifyDisallowingSocketConnection( PacketHelper* ) = 0;

	// PURPOSE: To ask the user what to do with this application
	// REQUIRE:	You need to call SetSpecificAppSettingsOut when you have the answer
	virtual eSelection
	AppRequestsOutNetAccess(const char in_app[MAX_PATH], 
							const unsigned int in_dstIp, 
							const unsigned short in_dstPort, 
							const bool in_bSpecialWarning) = 0;

	// PURPOSE: To ask the user what to do with this application
	// REQUIRE: You need to call SetSpecificAppSettingsIn when you have the answer
	virtual eSelection
	AppRequestsInNetAccess(const char in_app[MAX_PATH]) = 0;
};