// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// IRoutePacket.h

// This interface provides low level API
// for handling packets that are incoming/outgoing
// through the cloud.  It provides the means for the driver
// to use the cloud, without having indepth knowledge of
// how to encrypt or decrypt the data.

#ifndef _IROUTEPACKET_H
#define _IROUTEPACKET_H

#include <client.h>
#include <fr_genpkt.h>
#include <frAips.h>
#include <clientstl.h>
#include <string>

class RZkException ;
struct ZkBuffer ;

class IRoutePacket
{
public:

	// Determine the IP of the exit hop of the Authenticated route
	// so that we can take care of DNS

	virtual bool GetExitAipIPForDNS( sockaddr_in & out_addrExitAip ) = 0 ;

	// Given a particular Packet, coming from
	// the specified address, this looks at the
	// format and decides what to do.
	// The type of the packet is returned
	// For Route Creates/Route Destroys the
	// there is no information in the ZkBuffer.
	// For FRPKTTYPE_AIP_DATA the payload is decrypted
	// and filled into the structure
	// If there is a failure FRPKTTYPE_UNUSED_ is
	// returned and the packet should be ignored.
	virtual FrPktType ProcessIncomingCloudPacket( 
				sockaddr_in & in_AipAddr, 
				char * in_pBuffer, 
				const size_t in_cntBuffer, 
				ZkBuffer& out_DecryptedPacket ) = 0 ;

	// CreateOutgoingPacket prepares a packet for
	// sending out.  The function returned the packet to send and
	// the internet address to set it to.
	// Returns false is something has gone wrong that
	// doesn't allow us to send the packet, for 
	// example if no suitable route already exists.

    virtual bool CreateOutgoingPacket( 
				ZkBuffer& in_Buffer,
				char * out_pEncryptedPacket,
				size_t& io_cntEncryptedPacket,
				sockaddr_in & out_FirstHopAddr ) = 0 ;
} ;

#endif
