// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

////////////////////////////////////////////////////////////////////////////////
// IScriptServer.h
////////////////////////////////////////////////////////////////////////////////

#ifndef _ISCRIPTSERVER_H
#define _ISCRIPTSERVER_H

#include <ErrorHandling.h>
#include <client.h>
#include <clientstl.h>

// Interface
///////////////////////////////////////////////////////////////////////////////////////////////////

class IScriptServer
{
public:
	///////////////////////////////////////////////////////////////////////////////////////////////
	// PURPOSE: To spawn a thread that contacts the script server to update the wallet scripts 
	//			and ad blocker rules.
	//
	// REQUIRE: Two simulaneous updates are not permitted.  This method returns false if an update
	//			is already happening
	//
	// PROMISE:	To return to the caller immediately.
	//			To periodically retry the update if the update fails.

	virtual bool UpdateAllInNonBlockingThread() PURE;


	///////////////////////////////////////////////////////////////////////////////////////////////
	// PURPOSE: To stop a thread that was spawned by a call to UpdateAllInNonBlockingThread
	//
	// REQUIRE: 
	//
	// PROMISE:	If a thread that was launched by call to UpdateAllInNonBlockingThread is has not
	//			exited, this method will make it exit within 5 sec and then return
	
	virtual void AbortIfStillUpdating() PURE;


	///////////////////////////////////////////////////////////////////////////////////////////////
	// PURPOSE: To update the wallet scripts and/or ad blocker rules in a blocking fashion.
	//
	// REQUIRE: At least one of in_bUpdateWalletScripts and in_bUpdateAdBlockerRules must be true.
	//
	// PROMISE:	This method will only try to contact the ScriptServer once.  It returns true on
	//			a successful update.

	virtual bool Update(bool in_bUpdateWalletScripts = true, 
						bool in_bUpdateAdBlockerRules = true) PURE;
	

#ifdef INTERNAL_BUILD
	///////////////////////////////////////////////////////////////////////////////////////////////
	// PURPOSE: To add script mappings to the remote ScriptServer. 
	//
	// REQUIRE: Nothing
	//
	// PROMISE:	If in_bOverwrite, and conflicts will be overwritten on the server.  Otherwise,
	//			the user wil be prompted for each conflict.
	//			This method will update the local scripts after adding scripts to the remote
	//			ScrpitServer

	virtual bool AddScriptMappings(	const ZKSTD::string& in_strDomain, 
									const String2StringMap& in_realName2ZkNameMap, 
									bool in_bOverwrite = false) PURE;

	virtual bool UploadAdBlockerRules(const ZKSTD::string& in_strRuleList) PURE;
#endif //INTERNAL_BUILD

	virtual void SelfTest() PURE;
};

#endif //_ISCRIPTSERVER_H