// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):


#ifndef ITOKENDESC_H
#define ITOKENDESC_H

// Token descriptions is dynamically configurable information that comes from
// the network (eg NIQS).  It defines user friendly strings for token type ids
// eg "Premium Token" and also attributes of the tokens (eg how long the token
// will last).  
// This information is entirely informational for the user - don't think that
// changing the duration of a token in the description will affect anything
// on the server!


#include <WTokenDescriptions.h>

class ITokenDesc
{
public:

	// Given a token it is possible to determine the token type ID. 
	// From the token type ID this API allows you to get 
	// the details of the particular token type (eg the user friendly name).
	// Will return a default value and ASSERT if the type is bogus
	// or we don't have any valid token descriptions
	virtual bool GetTokenDesc( unsigned short in_TypeId, WTokenDescriptions& out_TokenDesc ) = 0 ;

	virtual void InitFromCache( const char* in_strFile ) = 0;
	virtual void SaveTokenDescriptions( const char* in_strFile ) = 0 ;

	// Set a new state (eg after contacting NIQS)
	// Any existing state is erased, new state is saved.
	// Caller should not free structure
	virtual void SetTokenDescriptions( T_FrTokenType* in_pNewTokenDescriptions ) = 0 ;

} ;

#endif