// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

////////////////////////////////////////////////////////////////////////////////
// IWallet.h
////////////////////////////////////////////////////////////////////////////////

#ifndef _IWALLET_H
#define _IWALLET_H

#include <string>
#include <client.h>

// Interface
///////////////////////////////////////////////////////////////////////////////////////////////////

class IWallet
{
public:
	///////////////////////////////////////////////////////////////////////////////////////////////
	// PURPOSE: In order for Freedom to fill in forms, it must understand what belongs in each form
	//			element.  Real names are mapped to Zk names (#defined in WalletConsts.h).  An
	//			example of a Zk name is FE_NAME_FIRST.
	//			This function returns a map of all the real names -> Zk names.
	//
	// REQUIRE: Nothing
	//
	// PROMISE:	To clear out_realNameLwr2ZkNameMap before inserting mappings.
	//			To return true unless the domain is screwy. If no mappings are found for this
	//			out_realNameLwr2ZkNameMap will be empty.

	virtual bool GetScriptMappings(	const ZKSTD::string& in_strDomain, 
									String2StringMap& out_realNameLwr2ZkNameMap) PURE ;


	///////////////////////////////////////////////////////////////////////////////////////////////
	// PURPOSE: To write a script to disk.  This method will overwrite any existing script for
	//			the specified domain.
	//
	// REQUIRE: in_strDomain != NULL
	//			in_strFile != NULL
	//
	// PROMISE:	To write in_nBytes beginnning at in_strFile to disk.

	virtual bool WriteScriptToDisk(	const char* in_strDomain, 
									const char* in_strFile, 
									unsigned int in_nBytes) PURE ;

#ifdef INTERNAL_BUILD
	///////////////////////////////////////////////////////////////////////////////////////////////
	// PURPOSE: To find out if the specified real form element name -> ZK name is in the default
	//			script file.  Mappings that are in the default script file are used for all 
	//			domains.  This is to shrink the total size of the scripts and also enable the
	//			FormFiller to work on some sites we have never documented.
	//
	//			For example, if we find that addr_street1 (real form element name) often maps to 
	//			Street1 (ZK form elemenet name),
	//			We will put this in the default script file and not in any other.
	//			If, per chance, on some random site, Street1 maps to Name_Last, including this
	//			mapping in the script for that domain will override the one in the default!
	//
	// REQUIRE: This should only be called (indirectly) by the Form Teacher.  For normal use of
	//			wallet, no other class has to understand that there is a default script.
	//
	// PROMISE:	

	virtual bool IsMappingInDefaultScript(const ZKSTD::string& in_strRealName, 
										  const ZKSTD::string& in_strZkName) PURE ;
#endif //INTERNAL_BUILD

	virtual void SelfTest() PURE;
};

#endif //_IWALLET_H