// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom(R) code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#ifndef IWORDSCAN_H
#define IWORDSCAN_H

// This interface provides the ability to 
// scan for "offensive" data in a nym file

#include <client.h>
#include <vector>
#include <string>

class IWordScan
{
public:
	////////////////////////////////////////////////////////////////////////////
	// Offensive Words/Word Scan
	////////////////////////////////////////////////////////////////////////////

	virtual bool WordScan(
						unsigned char* in_pBuf, 
						size_t	in_pBufSiz,
						ZKSTD::vector<ZKSTD::string>& out_vecstrOffensiveWordsFound) PURE;
	
	// This variant allows you to specify which nym is "Acting"  For example if nym A is active
	// and Nym B is sending a mail then Nym B is the originating nym
	virtual bool WordScan(
						unsigned char* in_pBuf, 
						size_t	in_pBufSiz,
						ZKSTD::vector<ZKSTD::string>& out_vecstrOffensiveWordsFound,
						size_t indexOriginatingNym) PURE;

	////////////////////////////////////////////////////////////////
	// PURPOSE:	To sanitize a string
	//				
	// REQUIRE: Nada.
	//						
	// PROMISE: All instances of a nym name will be replaced with "NymX",
	//			where X in an integer between 1 and (numNyms - 1) inclusive.
	//			This method is not smart.  If a nym is named "down", and
	//			io_strToSanitize == "download", the string will be sanitized to
	//			"nymXoad"

	virtual ZKSTD::string Sanitize(const ZKSTD::string& in_strToSanitize) const throw() PURE;
} ;

#endif