// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RAdBlocker.cpp


#include "StdAfx.h"
#include <sys/stat.h>
#include "RAdBlocker.h"
#include "resource.h"
#include <fstream>
#include <ZKFileConfiguration.h>
#include <string_hlp.h>
#include <IScriptServer.h>
#include <ZkService.h>
#include <IScriptServer.h>
#include <IZkPopup.h>
static const char g_pGifResponse[] =		"HTTP/1.1 200 Lose weight, eat less spam\r\n"
											"Server: Freedom HTTP Proxy\r\n"
											"Content-Type: image/gif\r\n"
											"Accept-Ranges: bytes\r\n"
											"Content-Length: 44\r\n\r\n"
											"\x047\x049\x046\x038\x039\x061\x001\x000"
											"\x001\x000\x080\x000\x000\x0FF\x033\x0CC"
											"\x000\x000\x000\x021\x0F9\x004\x001\x000"
											"\x000\x000\x000\x02C\x000\x000\x000\x000"
											"\x001\x000\x001\x000\x040\x002\x002\x084"
											"\x051\x000\x03B\x000";

AdDescription::AdDescription(void)
		: m_bException(false)
		, m_bUpload(false)
		, m_bBadData(false)
{
}

AdDescription::AdDescription(const ZKSTD::string& in_strFriendlyName, const ZKSTD::string& in_strURL,
							bool in_bException)
							: m_bException(in_bException)
							, m_strURL(in_strURL)
							, m_strFriendlyName(in_strFriendlyName)
							, m_bUpload(false)
							, m_bBadData(false)
{
}

AdDescription::AdDescription(const ZKSTD::string& in_strFriendlyName, const ZKSTD::string& in_strURL)
							: m_strURL(in_strURL)
							, m_strFriendlyName(in_strFriendlyName)
							, m_bUpload(false)
							, m_bException(false)
							, m_bBadData(false)
{
	if(m_strURL[0] == '*')
	{
		m_bUpload = true;
		m_strURL.erase(0, 1);
	}

	switch(m_strURL[0])
	{
	case '!':
		m_strURL.erase(0, 1);
		ZKASSERT(0);
		break;
	case '@':
		m_strURL.erase(0, 1);
		m_bException = true;
		break;
	case '~':
		m_strURL.erase(0, 1);
		break;
	default:
		break;
	}
}

AdDescription::AdDescription(const ZKSTD::string& in_strRule)
		: m_bException(false)
		, m_bUpload(false)
		, m_bBadData(false)
{
	ZKSTD::string strTemp(in_strRule);

	if(!strTemp.length())
	{
		return;
	}

	if(strTemp[0] == '*')
	{
		m_bUpload = true;
		strTemp.erase(0, 1);
	}
	
	switch(strTemp[0])
	{
	case '!':
		ZKASSERT(0);
		break;
	case '@':
		m_bException = true;
		break;
	case '~':
		break;
	default:
		ZKASSERT(0);
		break;
	}
	strTemp.erase(0, 1);

	size_t i = strTemp.find_first_of('<');
	if(i != ZKSTD::string::npos)
	{
		m_strFriendlyName = strTemp.substr(0, i);
		m_strURL = strTemp.substr(i + 1);
	}
	else
	{
		m_strURL = strTemp;
	}
}

void AdDescription::GetRuleString(ZKSTD::string& out_strRule)
{
	
	if(m_bUpload)
	{
		out_strRule = '*';
	}
	if(m_bException)
	{
		out_strRule += '@';
	}
	else
	{
		out_strRule += '~';
	}
	if(m_strFriendlyName.length())
	{
		out_strRule += m_strFriendlyName + '<';
	}
	out_strRule += m_strURL;
}

RAdBlocker::RAdBlocker()
			: m_AdsBlocked(0)
{

	m_strBlockListPath = ZKFileConfiguration::GetFreedomCommonDataDir() ;
	m_strBlockListPath += BLOCK_LIST_FILE_NAME;

	LoadBlockList();
}

RAdBlocker::~RAdBlocker()
{
}

void RAdBlocker::GetBogusResponse(const char **out_ppBogusResponse, size_t *out_pBogusResponseLength)
{
	if(!out_ppBogusResponse || !out_pBogusResponseLength)
	{	// Ouch! Null pointer.
		ZKASSERT(0);
	}
	else
	{
		*out_ppBogusResponse = g_pGifResponse;
		*out_pBogusResponseLength = sizeof(g_pGifResponse) - 1;
	}
}

void RAdBlocker::GetBlockList(ZKSTD::vector<AdDescription> &out_vecAdList)
{
	out_vecAdList = m_vecBlockedURLs;
}

void RAdBlocker::SetBlockList(ZKSTD::vector<AdDescription> &in_vecAdList, bool bSaveToDisk)
{
	m_vecBlockedURLs = in_vecAdList;
	if(bSaveToDisk)
	{
		SaveToDisk();
	}
}

void RAdBlocker::AddEntry(const ZKSTD::string& in_strRule, bool bSaveToDisk)
{
	if(!in_strRule.length())
	{

	}
	else if(in_strRule[0] == '!')
	{
		ZKSTD::string strTemp = in_strRule;
		strTemp.erase(0, 1);
		RemoveEntry(strTemp, bSaveToDisk);
	}
	else
	{
		AdDescription Rule(in_strRule);

		// Bug 8833... avoid bad data 
		if(!Rule.IsBadData() &&
			Rule.GetURL().length())
		{
			ZKSTD::vector<AdDescription>::iterator it;
			for(it = m_vecBlockedURLs.begin(); it != m_vecBlockedURLs.end(); it++)
			{
				if(it->GetURL().find(Rule.GetURL()) != ZKSTD::string::npos
					&& it->IsException() == Rule.IsException())
				{
					m_vecBlockedURLs.erase(it);
					break;
				}
			}

			m_vecBlockedURLs.push_back(Rule);
			if(bSaveToDisk)
			{
				SaveToDisk();
			}
		}
	}
}

void RAdBlocker::RemoveEntry(const ZKSTD::string& in_strURL, bool bSaveToDisk)
{
	ZKSTD::vector<AdDescription>::iterator it;
	for(it = m_vecBlockedURLs.begin(); it != m_vecBlockedURLs.end(); it++)
	{
		if(it->GetURL() == in_strURL)
		{
			m_vecBlockedURLs.erase(it);
			break;
		}
	}
	if(bSaveToDisk)
	{
		SaveToDisk();
	}
}

void RAdBlocker::ReloadBlocklist()
{
	LoadBlockList();
}

void RAdBlocker::SaveToDisk()
{
	ZKSTD::ofstream BlockListFile;

	BlockListFile.open(m_strBlockListPath.c_str());
	if(BlockListFile.is_open())
	{
		ZKSTD::vector<AdDescription>::iterator it;
		for(it = m_vecBlockedURLs.begin(); it != m_vecBlockedURLs.end(); it++)
		{
			// Bug 8833... avoid bad data ( don't save)
			if ( !( it->IsBadData() ) )
			{
				ZKSTD::string strRule;
				it->GetRuleString(strRule);
				BlockListFile << strRule << "\n";
			}
		}
		BlockListFile.close();
	}

}

void RAdBlocker::LoadBlockList()
{
	ZKSTD::ifstream BlockListFile;
	char TempBuf[512] = {0};

	m_vecBlockedURLs.clear();

	BlockListFile.open(m_strBlockListPath.c_str(), ZKSTD::ios::in);

	if(BlockListFile.is_open())
	{
		while(!BlockListFile.eof())
		{
			BlockListFile.getline(TempBuf, sizeof(TempBuf));

			ZKSTD::string strLine(TempBuf);
			ZKS::parse(strLine, "\n");
			ZKS::trim(strLine);
			ZKS::trimLeadingWhite(strLine);
			if(!strLine.length() || strLine[0] == '#')
			{
				// do nothing.
			}
			else
			{
				// bug 8833... skip lines we couldn't parse into adDescriptions
				AdDescription desc( strLine );
				if( !(desc.IsBadData()) )
				{
					m_vecBlockedURLs.push_back(desc);				
				}
			}
		}
		BlockListFile.close();
	}
}

bool RAdBlocker::UriIsBlocked(const char *strUri)
{
	bool bBlock = false;

	ZKSTD::vector<AdDescription>::iterator it;
	// Don't handle exceptions first pass
	for(it = m_vecBlockedURLs.begin(); it != m_vecBlockedURLs.end(); it++)
	{
		if(!it->IsBadData() &&  // Bug 8833... avoid bad data
			!it->IsException())
		{
			if(strstr(strUri, it->GetURL().c_str()) )
			{
				bBlock = true;
				break;
			}
		}
	}

	// Second pass, deal with exceptions
	if(bBlock)
	{
		for(it = m_vecBlockedURLs.begin(); it != m_vecBlockedURLs.end(); it++)
		{
			if(!it->IsBadData() &&  // Bug 8833... avoid bad data
				it->IsException())
			{
				if(strstr(strUri, it->GetURL().c_str()) )
				{
					bBlock = false;
					break;
				}
			}
		}
	}

	if(bBlock)
	{
		m_AdsBlocked++;
		ZKTRACE2("Ads Blocked: %ld Apx. Bandwidth Saved: %ld\r\n", GetNumberOfAdsBlocked(), GetEstimatedSavedBandwidth());
	}
	return bBlock;
}

#ifdef INTERNAL_BUILD
void RAdBlocker::UploadToScriptServer()
{
	ZKSTD::vector<AdDescription>::iterator it;
	ZKSTD::string strUpdateList;

	for(it = m_vecBlockedURLs.begin(); it != m_vecBlockedURLs.end(); it++)
	{
		if(it->IsMarkedForUpload())
		{
			ZKSTD::string strRule;
			it->GetRuleString(strRule);
			strUpdateList += ZKS::EscapeString(strRule) + "\n";
		}
	}
	if(strUpdateList.length())
	{
		if(ZkService::GetIScriptServer()->UploadAdBlockerRules(strUpdateList))
		{
			IZkPopup::MessageBox("Successfully updated the scriptserver.");
			for(it = m_vecBlockedURLs.begin(); it != m_vecBlockedURLs.end(); it++)
			{
				if(it->IsMarkedForUpload())
				{
					it->MarkForUpload(false);
				}
			}
			SaveToDisk();
		}
		else
		{
			IZkPopup::MessageBox("Failed to update the scriptserver.\n"
								"Make sure you've entered the proper scriptserver "
								"name in the registry.");
		}
	}
}
#endif

void RAdBlocker::SelfTest()
{
	// If your block list file is greater than 8K, executing this test
	// will leave you with a chopped file.
	FILE *pFile;

	char tempFileBuf[8096] = {0};
	size_t tempFileSize = 0;
	
	pFile = fopen(m_strBlockListPath.c_str(), "r");

	ZKTEST(pFile);

	if(!pFile)
	{
		return;
	}
	tempFileSize = fread(tempFileBuf, 1, 8096, pFile);
	ZKASSERT(tempFileSize && tempFileSize < 8096);

	fclose(pFile);

	pFile = fopen(m_strBlockListPath.c_str(), "w");

	ZKTEST(pFile);
	if(!pFile)
	{
		return;
	}

	fprintf(pFile,	"# This is a comment\n\n\n"
					"Block Doubleclick Ads<doubleclick.net\n"
					"# hello\n"
					"!Allow Access To Doubleclick's Website<www.doubleclick.net");
	fclose(pFile);

	LoadBlockList();
	ZKTEST(m_vecBlockedURLs[0].GetURL() == "doubleclick.net");
	ZKTEST(m_vecBlockedURLs[1].GetURL() == "www.doubleclick.net");
	ZKTEST(m_vecBlockedURLs[1].GetFriendlyName() == "Allow Access To Doubleclick's Website");
	ZKTEST(m_vecBlockedURLs[1].IsException());
	ZKTEST(m_vecBlockedURLs.size() == 2);

	ZKTEST( UriIsBlocked("ads.doubleclick.net/goddamnmarketing.gif"));
	ZKTEST(!UriIsBlocked("www.doubleclick.net/whywesuck.html"));
	
	pFile = fopen(m_strBlockListPath.c_str(), "w");

	ZKTEST(pFile);
	
	if(!pFile)
	{
		return;
	}

	fwrite(tempFileBuf, 1, tempFileSize, pFile);
	fclose(pFile);

	LoadBlockList();

}
