// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RAdBlocker.h
// RAdBlocker is a concrete implementation ("_R_ealization") of the
// IAdBlocker interface

#ifndef _RAdBlocker_H
#define _RAdBlocker_H

#include "IAdBlocker.h"
#include <string>
#include <vector>

//#pragma warning (disable : 4786)

#define AVG_IMG_SIZE 7168

class RAdBlocker : public IAdBlocker
{
public:
	// For docs, see IAdBlocker.h
	RAdBlocker();
    virtual ~RAdBlocker();

	virtual bool	UriIsBlocked(const char *strUri);
	virtual void	GetBogusResponse(const char **out_ppBogusResponse, size_t *out_pBogusResponseLength);
	virtual void	ReloadBlocklist();
	virtual long	GetNumberOfAdsBlocked() { return m_AdsBlocked ;}
	virtual void	SetNumberOfAdsBlocked(long in_AdsBlocked) { m_AdsBlocked = in_AdsBlocked; }
	virtual long	GetEstimatedSavedBandwidth() { return m_AdsBlocked * AVG_IMG_SIZE ;}
	virtual void	GetBlockList(ZKSTD::vector<AdDescription>& out_vecAdList);
	virtual void	SetBlockList(ZKSTD::vector<AdDescription>& in_vecAdList, bool bSaveToDisk = true);
	virtual void	AddEntry(const ZKSTD::string &in_strRule, bool bSaveToDisk = true);
	virtual void	RemoveEntry(const ZKSTD::string &in_strURL, bool bSaveToDisk = true);
	virtual void	SaveToDisk();
	
#ifdef INTERNAL_BUILD
	virtual void UploadToScriptServer();
#endif
	// Test code, only called from Internal Build
	virtual void	SelfTest()  ;
private:
	
	// These aren't defined
	RAdBlocker(const RAdBlocker& rhs);
	RAdBlocker& operator=(const RAdBlocker& rhs);

	// Load the block list from disk and store it
	// in m_vecBlockedURLs
	void LoadBlockList();

	long m_AdsBlocked;
	ZKSTD::string m_strBlockListPath;
	ZKSTD::vector<AdDescription> m_vecBlockedURLs;
};


#endif
