// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RClientCentral.h
//

#ifndef _RClientCentral_H
#define _RClientCentral_H

#ifdef _WIN32
#pragma warning(disable: 4786) // Harmless template truncated to 255 character warning
#endif

#include "IClientCentral.h"
#include "RTokenService.h"
#include <frNym.h>
#include <frNymClient.h>
#include <ErrorHandling.h>
#include <zkList.h>
#include <DZkError.h>
#include <WRemoteNym.h>
#include <ZKNymName.h>
#include <ZKChunk.h>
#include <WSyncTypes.h>
#include <time.h>

class ZKNym ;
struct T_ZkDateTime ;

class RClientCentral : public IClientCentral
{

public:
	RClientCentral();
	virtual ~RClientCentral();
	

	virtual void SetPreference(unsigned long in_dwPrefSet, unsigned long in_dwProperty, bool in_bNewState) ;
	virtual bool GetPreference(unsigned long in_dwPrefSet, unsigned long in_dwProperty) const ;

	virtual void	SetReturnEmailAddress( const ZKSTD::string& ) throw(RZkException);  ;
	virtual void	GetReturnEmailAddress( ZKSTD::string& ) const ;

    virtual void    SetMailRelay( const ZKSTD::string& ) ;
    virtual void    GetMailRelay( ZKSTD::string& ) const ;

	// Low level access to the state of RClientCentral - for
	// usage by persistence

	virtual void Save( WMemBuf& io_Buffer ) throw(RZkException);
	virtual void Load( WReadBuffer& io_Buffer ) throw(RZkException);

	// Review - might want to seperate into a different
	// interface to avoid cluttering this interface

	virtual bool InitializeFromBuffer_Obsolete( const unsigned char* in_pBuffer, size_t in_cntBuffer) ;
	virtual bool InitializePreferencesFromBuffer_Obsolete( const unsigned char* in_pBuffer, size_t in_cntBuffer) ;

	virtual bool GetBuffer_Obsolete( unsigned char** out_ppBuffer, size_t* cntBuffer ) const ;
	virtual bool GetPreferenceBuffer_Obsolete( unsigned char** out_ppBuffer, size_t* cntBuffer ) const;

	// Provide access to the Token Service (not part of interface)
	ITokenService* GetTokenService() ;

	virtual void SetIsWalletDisabledForDomain(
		const ZKSTD::string& in_strDomain, 
		bool in_bDisable);
	
	virtual bool IsWalletDisabledForDomain(
		const ZKSTD::string& in_strDomain) const;

	virtual void SetDomainsForWhichWalletIsDisabled(
		const ZKSTD::set<ZKSTD::string>& in_strSetOfDomains);

	virtual void GetDomainsForWhichWalletIsDisabled(
		ZKSTD::set<ZKSTD::string>& out_strSetOfDomains) const;

	virtual bool IsPasswordRequired();
	virtual void CorrectPasswordEntered();
	virtual int  GetNumMinutesUntilReAskPassword();
	virtual void SetNumMinutesUntilReAskPassword(const int in_nMinutes);

private:
	RClientCentral(const RClientCentral& rhs);
	RClientCentral& operator=(const RClientCentral& rhs);

	void SaveRemoteNymChunk( WMemBuf& io_Buffer ) throw(RZkException);
	void SaveAdBlockerChunk( WMemBuf& io_Buffer ) throw(RZkException);
	void SavePrefChunk( WMemBuf& io_Buffer ) throw(RZkException);
	void SaveWalletDisabledDomainsChunk( WMemBuf& io_Buffer ) throw(RZkException);
	
	void LoadAdBlockerChunk( ZKChunkReader& io_ChunkReader ) throw(RZkException);
	void LoadRemoteNymChunk( ZKChunkReader& io_ChunkReader ) throw(RZkException);
	void LoadPrefChunk( ZKChunkReader& io_ChunkReader ) throw(RZkException);
	void LoadWalletDisabledDomainsChunk( ZKChunkReader& io_ChunkReader ) throw(RZkException);

	// ctor member init
	mutable DZkError				m_ZkError;

	WRemoteNym						m_RemoteNym ;
	RTokenService                   m_TokenService ;
	ZKSTD::set<ZKSTD::string>		m_setWalletDisabledDomains;

	unsigned long					m_aPrefs[ZK_NUM_PREFERENCE_SETS];	// Client preferences
	mutable WZkMutex				m_csWalletDisabledDomains;

	
	void							InitDefaultPreferenceValues() ;

	clock_t							m_clockLastInteraction;
	mutable	WZkMutex				m_csClock;
	int								m_nMinutesUntilReAskPassword;
    ZKSTD::string                   m_strMailRelay;

	//DebugTools
	void DumpStateToTraceWindow();
};

#endif 
