// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RClientPersist Implements IClientPersist interface


#ifndef _CLIENTPERSIST_H
#define _CLIENTPERSIST_H

#include "IClientPersist.h"
#include "INym.h"
#include <zkThread.h>
#include <WSyncTypes.h>

class RClientPersist : public IClientPersist
{
public:	
	RClientPersist() ;
	virtual ~RClientPersist() ;
	
	virtual bool IsFirstTimeRun() ;
	virtual void LoadClientState(const std::string& in_SuggestedPassPhrase)  throw(RZkException);
	virtual void SaveClientState()  throw(RZkException);
	virtual void GetActiveFilePath( std::string& out_FileWithPath ) throw(RZkException);
	virtual void SetDirty() ;
	virtual bool IsDirty() ;

private:

	void	GenerateAndEncryptTestStream( void** out_ppStream, size_t& out_cntStream ) ;
	void	GenerateAndEncryptDataStream( void** out_ppStream, size_t& out_cntStream ) ;


	void	DecryptBuffer( const void * in_pEncryptedBuffer, 
							size_t in_cntEncryptedSize,
							void ** out_ppBuffer,
							size_t& out_cntBuffer )  throw(RZkException);

	void	EncryptBuffer( const void * in_pClearBuffer, 
							size_t in_cntClearSize,
							void ** out_ppBuffer,
							size_t& out_cntBuffer )  throw(RZkException);

	void    LoadClientState(
				const std::string& in_SuggestedPassPhrase,
				const std::string& in_strFileToLoad )  throw(RZkException);

	void    SaveClientState(const char * in_szFilePath)  throw(RZkException);

	void    TestPassPhraseAgainstFile( 
				const uint8_t * in_pTestBuffer,
				size_t in_cntTestBuffer )  throw(RZkException);

	void	DecryptAndInitializeClientState( 
				const uint8_t * in_pBuffer,
				size_t in_cntBuffer )  throw(RZkException);

	static void ZK_ITIMER_CALLBACK PeriodicSaveCallback(
				void *p, 
				unsigned int i1, 
				unsigned int i2, 
				unsigned long time) ;

	std::string  m_strLoadFilePath ;	// File that we save into
	bool         m_bNewFormat ;			// Format that we load.  Old format refers to V1.X versions
										// that are loaded by RClientPersistOldWinVer.cpp

	bool         m_bDirtyFlag ;			// Notify that a save is needed

	ZkITimer			m_pZkTimer ;
	WZkMutex			m_cs ;			// Protect access to ClientPersist (only 1 save or load can happen at a time)

	mutable DZkError	m_err ;
} ;

#endif