// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RClientPersistOldWinVer.h

// Load and save the Windows v1.1 format files.
// Load code is necessary on Windows Client until all clients
// have guaranteed to have upgrade -> ie perhaps we might need
// to keep this around for ever.
// Other Platform don't even need to compile this at all

// Implements IClientPersist

#ifndef _CLIENTPERSISTOLDWINVER_H
#define _CLIENTPERSISTOLDWINVER_H

#include "IClientPersist.h"
#include "INym.h"


// UpgradePersistV1XHelper

// Most of the logic for converting the client into the new version 
// is bundled in this class, to try to keep 
// the main code line of logic as clear as possible.

// It acts as a bridge between the old, basically untouched,
// RClientPersit implementation
// (now called RClientPersistOldWinVer()) to a new implementation.

// This code needs to exist until we are sure that all 
// existing clients have updated or restored their persistence.
// In effect this might need to be around forever.

class UpgradePersistV1XHelper
{
public:
	static void GetDataFileToLoad( 
					std::string& out_strFilePath, 
					bool& out_bIsNewFormat ) ;

	static void LoadWithOldPersistenceCode( 
					const std::string& in_strPassphrase, 
					const std::string& in_strLoadCandidate )  throw(RZkException);
private:
	static bool IsDocFile( const std::string& in_strCandidateFile ) ;
	static void MakePreUpgradeBackup( const std::string& in_strLoadCandidate ) ;
} ;


// RClientPersistOldWinVer

class RClientPersistOldWinVer : public IClientPersist
{
public:	
	RClientPersistOldWinVer() ;
	virtual ~RClientPersistOldWinVer() ;
	
	virtual bool IsFirstTimeRun() ;
	virtual void LoadClientState(const std::string& in_SuggestedPassPhrase)  throw(RZkException);
	virtual void SaveClientState()  throw(RZkException);
	virtual void GetActiveFilePath( std::string& out_FileWithPath ) throw(RZkException);


	virtual void LoadClientState(
					const std::string& in_SuggestedPassPhrase,
					const std::string& in_FileToLoad )  throw(RZkException);


	virtual void SetDirty() {  ; } // not implemented in old persistence
	virtual bool IsDirty() { return false ; }

private:


	void	GetDefaultFileLocation();
	void    SaveClientState(WCHAR * in_wszFilePath);
	void    WriteTestStreamToFile(LPSTORAGE pFile);
	HRESULT TestPassPhraseAgainstFile(LPSTORAGE pFile);
	HRESULT GetBufferFromStream(LPSTREAM pStream, unsigned char** ppBuffer, unsigned long & ulBufferSize);
	void    WriteBufferToStream(LPSTREAM pStream, unsigned char* pBuffer, unsigned long ulBufferSize);
	HRESULT LoadNym(LPSTORAGE pFile,const size_t nIndex);
	
	void    SaveNym(LPSTORAGE pFile, const size_t in_nNymIndex, const size_t in_nStreamDigitName, INym * in_pNymDebugging);

	void    EncryptAndSaveBuffer(LPSTORAGE pFile, 
							     LPCWSTR wszStreamName, 
								 unsigned char* pBuffer, 
								 size_t cntSize );
	HRESULT LoadAndDecryptBuffer(LPSTORAGE pFile, 
							     LPCWSTR wszStreamName, 
								 unsigned char** ppBuffer, 
								 unsigned long & cntSize );

	void    TestAndThrowSaveHRESULT(unsigned int in_BaseErrorString, HRESULT in_hr);
	void    TestAndThrowLoadHRESULT(unsigned int in_BaseErrorString, HRESULT in_hr);

	std::string  m_strFilePath ;	 // File name that we load/save
} ;

#endif