// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RCookieManager.h
// This is an implementation ("realization") of the
// ICookieManager interface

#ifndef _RCookieManager_H
#define _RCookieManager_H

#include "ICookieManager.h"
#include <DZkError.h>

#ifdef _WIN32
#pragma warning (disable : 4786)
#endif

class RCookieManager : public ICookieManager
{
public:
	RCookieManager();
	virtual ~RCookieManager();

	// Add a new cookie
	// First argument is the string version of the cookie as sent
	// back from the web site.  Second is the domain which recieves it.
	// If there is a domain inside the cookie text we use that instead of the 
	// second argument.

	virtual void		AddCookie( 
							const ZKSTD::string& in_cookie, 
							const ZKSTD::string& in_domain ) ;

	// ELW: Build the appropriate cookie-field for an outgoing request based on the domain name.
	virtual bool		BuildCookieField(const ZKSTD::string&, ZKSTD::string&);	

	// Counter of Cookie Events so that we can show cookie activity
	// on the main UI
	// TDB: In the future we can keep seperate counters and have different
	// UI indication for incoming and outgoing cookies
	unsigned long *				GetCookieCounter() { return &m_cntCookieReadsAndWrites ; } ;

	// Used by Nyms to notify that they have updated their cookie
	// data or sent out cookies
	virtual void		SignalCookieEvent() { m_cntCookieReadsAndWrites++ ; } 

	// Test code, only called from Internal Build
	virtual void		SelfTest()  ;


private:
	
	RCookieManager(const RCookieManager& rhs);
	RCookieManager& operator=(const RCookieManager& rhs);

	DZkError					m_ZkError;

	unsigned long						m_cntCookieReadsAndWrites ; // Counter of operations for UI
};


#endif
