// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RKeyQry.h: declaration for the CKeyQry class.
// Clients should always access the functionality of this
// class through its interface IKeyQry

#if !defined(CORESERVERCONNECT_H__INCLUDED_)
#define CORESERVERCONNECT_H__INCLUDED_

#include "ICoreServerConnect.h"
#include <WNetInfoDatum.h>
#include <WSyncTypes.h>

typedef ZKSTD::map<FrEntIdType, uint32_t>				EntIdType2ServerIndex;

class RCoreServerConnect : public ICoreServerConnect
{
	

public:

	RCoreServerConnect();	
	virtual ~RCoreServerConnect();

	virtual void SmartConnectThroughCloud(	
			const FrEntIdType& in_entityType,
			WZkipClient& out_client,
			const int& in_nTimeoutPerConnect,
			const size_t& in_nAttempts = UINT32_MAX) throw(RZkException);

	virtual void ConnectToNiqs(
			WZkipClient& out_client,
			const int& in_nTimeoutPerConnect,
			const size_t& in_nAttempts = UINT32_MAX) const throw(RZkException) ;

private:
	bool ConnectToServer(	const sockaddr_in& in_addrServer, 
							const int& in_nTimeout, 
							DZkError& out_err,
							WZkipClient& out_client) const throw();

	void ConnectThroughCloudFailed(const WNetInfoDatum& in_server,
								   const DZkError& in_err) const throw();

	size_t								m_nAttempts;
	int									m_nConnectTimeout;
	ZKSTD::map<FrEntIdType, uint32_t>	m_mapEntIdType2NextServerIndex;
	mutable	WZkMutex					m_csMapEntIdType2NextServerIndex;
	mutable DZkError					m_err;
};

#endif // !defined(CORESERVERCONNECT_H__INCLUDED_)
