// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RCryptoService.h: interface for the RCryptoService class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CryptoService_H__EE293AE1_D6E5_11D2_8097_00104B6B4FA4__INCLUDED_)
#define AFX_CryptoService_H__EE293AE1_D6E5_11D2_8097_00104B6B4FA4__INCLUDED_

#ifdef WIN32
#  if _MSC_VER > 1000
#    pragma once
#  endif // _MSC_VER > 1000
#  pragma warning(disable: 4786)
#endif

#include "ICrypto.h"
#include <zks.h>
#include <zkError.h>
#include <frSec.h>
#include <DZkError.h>
#include <fr_entid.h>

class RCryptoService : public ICrypto
{

public:
	RCryptoService();
	virtual ~RCryptoService();

	void SetNewPassphraseAndSalt(IN const ZKSTD::string& strpass, IN void* salt, IN size_t saltsiz) throw(RZkException);
	void ChangePassphrase( const ZKSTD::string& strOld, 
						   const ZKSTD::string& strNew) throw(RZkException) ;

	bool GetSalt( void ** out_ppSalt, size_t& out_szSalt ) ;
	bool GetPassphrase(ZKSTD::string& strPass) ;
	void GetDefaultPassphrase(ZKSTD::string& strPass);
	bool SymDecryptBuf(const void* src, const size_t ssz, void** dst, size_t& dsz);
	bool SymEncryptBuf(const void* src, const size_t ssz, void** dst, size_t& dsz);
	void KeyCacheFromFile(const ZKSTD::string& strFile);
	void GetKeyCacheIntoFile(const ZKSTD::string& strFile);
	bool ZkDecryptFile(const ZKSTD::string& strFile, void*& pClearBuf, size_t& szClear);
	bool ZkEncryptFile(const ZKSTD::string& strFile, void* buf, size_t len);

	// CheckAndRemoveSignature
	// Given a signed buffer this function attempts to verify the signature.
	// Currently it does not fail if it can't verify the signature
	// 
	// This function should only throw if there was an unexpected 
	// corrupt buffer or otherwise invalid input

	virtual SigCheckResult CheckAndRemoveSignature( 
					unsigned char * io_pBuffer,
					size_t&         io_szBuffer,
					ZKSTD::string&    out_SigningEntity,
					FrEntIdType&	out_Entity) const throw(RZkException);

	// Checks a signature but does not remove it 
	virtual SigCheckResult CheckSignature( 
					const unsigned char * in_pBuffer,
					size_t				  in_szBuffer,
					ZKSTD::string&		  out_SigningEntity,
					FrEntIdType&		  out_Entity) const throw(RZkException);


	void NewCryptKey(
		const int eKeyVersion,
		const int eKeySize,
		const ZkClock crypt_expiration,	
		FrCryptPubKey& CryptPubKey,
		fr_entid_exp* ent) const throw(RZkException);

	void NewSigKey(
		const int sKeyVersion,
		const int sKeySize,
		const ZkClock sign_expiration,	
		FrCryptPubKey& SigPubKey,
		fr_entid_exp* ent) const throw(RZkException);

	int PutKeyInFile(
			DZkError& out_Err, 
			FrCryptPubKey key, 
			FILE *fp);

	int SymmetricKeyToBuffer(
			DZkError& out_Err, 
			const FrCryptSym in_sym, 
			char* out_buf, 
			const size_t in_bs);

private:

	// Current sym key
	FrCryptSym	m_symkey;
	
	mutable DZkError	m_err;

	ZKSTD::string		m_strSeedFile ;	// Warning: Yarrow will reference this string directly, so we must keep it
										// around for the entire duration of the frCryptInit/Final
	ZKSTD::string		m_pass;
	
	void*				m_salt;		// we'll want to save the new SALT in our Freedom.dat
	size_t				m_saltsiz;	// the crypto subsystem will keep a sym key around for us to use quickly

	RCryptoService& operator=(RCryptoService&);
		
	void newKey(
		const FrPubKeyType kt, 
		const int kv, 
		const int ks, 
		const ZkClock ke, 
		FrCryptPubKey& pk,
		fr_entid_exp* ent) const throw(RZkException);

	void SetNewPassphraseCreateSalt(IN const ZKSTD::string& strpass) throw(RZkException);

};

#endif // !defined(AFX_CryptoService_H__EE293AE1_D6E5_11D2_8097_00104B6B4FA4__INCLUDED_)
