// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RNym.h: interface for the RNym class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ZKNym_H__D4253842_D0B6_11D2_8097_00104B6B4FA4__INCLUDED_)
#define AFX_ZKNym_H__D4253842_D0B6_11D2_8097_00104B6B4FA4__INCLUDED_

#ifdef WIN32

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#pragma warning(disable: 4786)

#endif

#include "INym.h"
#include <frNym.h>
#include <frNymClient.h>
#include <zkList.h>
#include <DZkError.h>
#include <WRemoteNym.h>
#include <ZKNymName.h>
#include <WNymIconInfo.h>

struct T_ZkDateTime ;

class RZkException;
class RNymMail ;
class RNymWebCacheMgr ;
class RNymWallet;
class RNymCookieMgr;
class RNymRoute ;
class ZKChunkReader ;

class RNym : public INym 
{
public:
	RNym();
	virtual ~RNym();

	virtual void	QueryInterface(uint32_t in_iid, void** out_ppv) throw(RZkException);

	virtual void	CreateYourself(ZKNymName& name, const std::string& token) ;
	virtual void    ExtendYourself() throw(RZkException) ;

	virtual void	DisableNymOnNymServer() throw(RZkException);
	virtual void	ModifyExistingNymOnNymServer() throw(RZkException);
	virtual void	AddTokenToNymOnNymServer(const std::string& token) throw(RZkException);

	virtual void	SetUserName(const std::string&);
	
	virtual void	GetNymIconInfo( WNymIconInfo& out_NymIcon ) ;
	virtual void	SetNymIconInfo( const WNymIconInfo& in_NymIcon ) ;

	virtual void	SetNymDescription(std::string&) ;
	virtual void	GetNymDescription(std::string&) ;

	virtual void	SetNymPrivateNotes(std::string&) ;
	virtual void	GetNymPrivateNotes(std::string&) ;
	virtual void	SetNymPublicNotes(std::string&) ;
	virtual void	GetNymPublicNotes(std::string&) ;

	virtual bool	GetCookiesEnabled();
	virtual void	SetCookiesEnabled(bool);
	virtual bool	GetHideRefererField();
	virtual void	SetHideRefererField(bool);

	virtual void	GetNymPreferencesBitfield(unsigned long&) ;
	virtual void	SetNymPreferencesBitfield(unsigned long, unsigned long) ;
	virtual bool	IsNymPreferenceSet( unsigned long in_PrefToTest ) ;

	virtual bool	GetNymName(std::string&);
	virtual bool	GetNymEmailDomain(std::string&);	
	
	virtual void	SetRouteRulesOfNym(const MapRules&);
	virtual void	GetRouteRulesOfNym(MapRules&) throw(RZkException);

	virtual void	SetSecuritySettings(const SecuritySettings&) throw(RZkException);
	virtual void	GetSecuritySettings(SecuritySettings&) throw(RZkException);
	virtual size_t  GetNumberOfHops() ;

	virtual void	GetNymBirthDate(std::string&) throw(RZkException);
	virtual void	GetNymRetireDate(std::string&) throw(RZkException);
	virtual void	GetNymDeathDate(std::string&) throw(RZkException);

	virtual void	GetNymBirthDate(time_t&) throw(RZkException);
	virtual void	GetNymRetireDate(time_t&) throw(RZkException);
	virtual void	GetNymDeathDate(time_t&) throw(RZkException);

	virtual void    GetSignatureKey(fr_crypt_pubkey**)throw(RZkException);
	virtual void    GetCryptoKey(fr_crypt_pubkey**)throw(RZkException);

	virtual void	GetYourLifeStage(unsigned long& out_LifeStage) throw(RZkException);
	virtual void	SetYourLifeStage(unsigned long in_LifeStage ) ;


	virtual bool	GetOffensiveWordList(std::vector<std::string>&) ;
	virtual bool	SetOffensiveWordList(const std::vector<std::string>&);

	// Serialize the state into a chunked buffer
	virtual void Save( WMemBuf& io_Buffer ) throw(RZkException) ;

	// Any existing state is completely lost and a new state is read from the
	// stream
	virtual void Load( WReadBuffer& io_Buffer ) throw(RZkException) ;

	virtual bool DoesNymUseCloud() ;
	virtual bool DoesNymUseMail() ;
	virtual bool DoesNymAutoExtend() ;

	// Not so nice to reveal the internal structure
	// use only with caution and aim to replace with specific
	// methods to control access.
	// Thats why it is _not_ part of the interface.

	virtual WRemoteNym& RemoteNym() { return m_RemoteNym ; } ;

private:
	RNym(const RNym& rhs);
	RNym& operator=(const RNym& rhs);
	
	// ctor member init
	mutable DZkError			m_ZkError;

	WRemoteNym					m_RemoteNym ;
	NymServerResponse			m_NymServerResponse;

	// An instance of the nym mail object takes care of 
	// mail related tasks for the nym
	RNymMail *					m_pNymMail ;
	
	// An instance of the nym wallet object which stores 
	// and retrieves wallet data such as usernames and
	// passwords, addresses, credit card info, etc.
	RNymWallet*					m_pNymWallet;

	RNymCookieMgr*              m_pNymCookieMgr ;

	RNymRoute*					m_pNymRoute ;
	RNymWebCacheMgr*			m_pNymWebCacheMgr;

	// Althought these two vars don't belong as member
	// vars, I had to move them here so they could be
	// free'd in the DTOR (see SetNymCryptoAndSigningKeys()
	FrCryptPubKey				m_key1;
	FrCryptPubKey				m_key2;

	unsigned long				m_LifeStage ;
	WNymIconInfo				m_NymIconInfo ;

	virtual void	CreateNymOnNymServer(ZKNymName& NymName, const std::string& token) throw(RZkException);

	virtual void	SetNymCryptoAndSigningKeys() throw(RZkException);
	
	virtual void	MapRulesToBinary(const MapRules& mapRules, unsigned char**, size_t*);
	virtual void	MapRulesFromBinary(MapRules&, unsigned char*, size_t);
	
			void	TestAndThrowServerResponse(int);

 			void    GetNymStringParameter( enum FrRemoteNymParameter, std::string & out_string ) ;
			void    SetNymStringParameter( enum FrRemoteNymParameter, std::string & in_string ) ;

	// Help working with dates

			void	SecuritySettingsToBinary(const SecuritySettings&, unsigned char**, size_t*) ;
			void	SecuritySettingsFromBinary(SecuritySettings&, const unsigned char*, size_t) ;

	// Persistence
			void	SaveRemoteNymChunk( WMemBuf& io_Buffer ) throw(RZkException) ;
			void	LoadRemoteNymChunk( ZKChunkReader& io_ChunkReader ) throw(RZkException) ;

	// Debugging output
			void	DumpRouteRulesOfNym(const MapRules&);

			virtual FrNymCapability GetNymTypeCapability();

	friend class RNymMail ;

	FrNymCapability m_nymCapability;
};

#endif // !defined(AFX_ZKNym_H__D4253842_D0B6_11D2_8097_00104B6B4FA4__INCLUDED_)
