// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RNymCentral.h
//

#ifndef _NYMCENTRAL_H
#define _NYMCENTRAL_H


#include "INymCentral.h"



#ifdef WIN32
#  pragma warning(disable: 4786) // Harmless template truncated to 255 character warning
#endif

#include <WSyncTypes.h>



#include <frNym.h>
#include <frNymClient.h>
#include <ErrorHandling.h>
#include <zkList.h>
#include <DZkError.h>
#include <ZKNymName.h>

class INym ;
class RNym ;
struct T_ZkDateTime ;

class RNymCentral : public INymCentral
{

public:
	RNymCentral();
	virtual ~RNymCentral();
	
	virtual void CreateNym( ZKNymName& NymName, const ZKSTD::string& token ) throw(RZkException);
	virtual void CreateUser( ) ;

	virtual void MarkNymAsDead(	const size_t nIndex ) ;

	virtual bool GetActiveNym(uint32_t in_iid, void** ppv);
	virtual size_t GetActiveNymIndex();
	virtual bool SetActiveNym(const size_t nIndex) ;
	virtual bool IsCurrentNymUser() const;

	virtual bool GetNym(
		uint32_t in_iid, 
		void** ppv, 
		const ZKSTD::string& strNymName) ;

	virtual bool GetNym(
		uint32_t in_iid, 
		void** ppv, 
		const size_t nIndex) ;

	virtual void GetNymIndex( const ZKSTD::string& in_strNymName, size_t& out_indexNym );

	virtual void GetNymName( size_t in_indexNym, ZKSTD::string& out_strNymName ) ;
	virtual void GetActiveNymName( ZKSTD::string& out_strNymName ) ;

	virtual ZKSTD::string GetNoNymSelectedDisplayName() ;

	virtual void InitNymLifeCycle() throw(RZkException);
	virtual void HandleNymsLifeCycleIssues( size_t in_nIndex, ZKSTD::string& out_strWarning ) throw(RZkException);

	size_t GetNumberOfNyms() ;

	virtual void DecryptMessage( WCypherMailMsg&  io_MailMsg ) throw(RZkException);
	
	// Low level access to the state of RNymCentral - for
	// usage by persistence

	// Erase all nyms, including User nym.  No Network Access involved
	virtual void FlushNyms() ;

	// Add a nym to nym central
	// used when loading up from a persisted state
	// Caller is responsible for memory management of the buffer (ie
	// it is not "freed" inside AddNym
	virtual bool AddNym_Obsolete( unsigned char* in_pBuffer, unsigned long in_ulBufferSize ) ;

	virtual bool GetNymAsBuffer_Obsolete( size_t in_WhichNym, unsigned char** out_ppBuffer, size_t* cntBuffer ) ;
	
	// Persistence related
	virtual void Save( WMemBuf& io_Buffer ) throw(RZkException);
	virtual void Load( WReadBuffer& io_Buffer ) throw(RZkException);

	
	virtual bool DoesNymUseCloud(const size_t in_indexNym) ;
	virtual bool DoesNymUseMail(const size_t in_indexNym) ;

	virtual bool AtLeastOneNymUseMail();
	virtual bool AtLeastOneNymUseCloud();


private:
	RNymCentral(const RNymCentral& rhs);
	RNymCentral& operator=(const RNymCentral& rhs);
	
	// ctor member init
	size_t							m_ActiveNym;

	DZkError						m_ZkError;	
	WZkMutex						m_cs;
	ZKSTD::vector<RNym*>			m_vecNymPointers;


#ifdef WIN32
	void AttemptDecryptWithNym( WCypherMailMsg&  io_MailMsg, ZKSTD::string in_strNym ) throw(RZkException);
#endif

	// Create a new nym and read its settings out of the stream
	// Used to load up existing nyms, not to create new nyms from scratch
	virtual void AddNym( WReadBuffer& io_Buffer ) ;


	//DebugTools
	void DumpStateToTraceWindow();

	void HandleAutoExtendNym( 
			INym* in_pNym, 
			const ZKSTD::string& in_strNym, 
			int in_DaysRemaining, 
			ZKSTD::string& out_strMsgToUser ) ;
};

#endif 
