// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RNymCookieMgr implements INymCookieMgr

#ifndef _RNYMCOOKIEMGR_H
#define _RNYMCOOKIEMGR_H

#include <WSyncTypes.h>
#include "ICookieManager.h"
#include "INymCookieMgr.h"

#ifdef _WIN32
#pragma warning (disable : 4786)
#endif

class RNym ;

class RNymCookieMgr : public INymCookieMgr
{
public:
	RNymCookieMgr(RNym* pParent);
	virtual ~RNymCookieMgr();

	// Add a new cookie
	// First argument is the string version of the cookie as sent
	// back from the web site.  Second is the domain which recieves it.
	// If there is a domain inside the cookie text we use that instead of the 
	// second argument.

	virtual void    AddCookie( 
						const std::string& in_cookie, 
						const std::string& in_domain ) ;

	// ELW: Build the appropriate cookie-field for an outgoing request based on the domain name.
	virtual bool		BuildCookieField(const std::string&, std::string&);
	
	// Freedom Gui interface
	virtual bool		GetNymCookieMap(COOKIEJARMAP&);
	virtual bool        SetNymCookieMap(COOKIEJARMAP&);

	virtual void Save( WMemBuf& io_Buffer );

	virtual void Load( ZKChunkReader& io_ChunkReader );


	virtual void LoadOldV1XFormat() ;

	// Test code, only called from Internal Build
	virtual void		SelfTest()  ;

private:
	
	RNymCookieMgr(const RNymCookieMgr& rhs);
	RNymCookieMgr& operator=(const RNymCookieMgr& rhs);

	mutable	WZkMutex			m_cs;
	DZkError					m_ZkError;
	COOKIEJARMAP				m_NymCookieMap;

	RNym*						m_pParent;


	// PURPOSE:	I moved this here from DHttpClient.h
	// REQUIRE:	There were no comments in that header
	// PROMISE:	I'll try to document more as I learn more about	
	// THROWS:	
	bool		ParseCookie(
						const std::string& in_cookie, 
						const std::string& in_domain,
						std::string& out_namevalue, 
						std::string& out_domain,
						std::string& out_expires);

	bool		SetNymCookieMapEntry(const std::string&, const std::string&, const std::string&);

	void		DumpCookie(const DomainAndDate&);
	void		DumpCookie2(COOKIEJARMAP::iterator&);

	// This function returns true if the input string is actually an string
	// version of an IP address, eg 180.108.109.

	bool		DomainIsIP(const std::string&);

	////////////////////////////////////////////////////////////////////////////////////
	// RNymCookieMgr::DomainMatch checks to see if lhs domain-matches rhs
	// (From RFC 2109)
	//   Host A's  name domain-matches host B's if
	//
	//   * both host names are IP addresses and their host name strings match
	//     exactly; or
	//
	//   * both host names are FQDN strings and their host name strings match
	//     exactly; or
	//
	//   * A is a FQDN string and has the form NB, where N is a non-empty name
	//     string, B has the form .B', and B' is a FQDN string.  (So, x.y.com
	//     domain-matches .y.com but not y.com.)
	//	
	//   Note that domain-match is not a commutative operation: a.b.c.com
	//   domain-matches .c.com, but not the reverse.
	bool		DomainMatch(const std::string&, const std::string&);

	// Determines whether an expiry string represents an expiry string
	bool		IsSessionCookie( const std::string & in_ExpiryString ) ;

	void SaveCookie( WMemBuf& io_Buffer, 
					const std::string& in_strNameValue, 
					const DomainAndDate& in_DomainAndDate );

	// Loads a cookie and adds it directly to the data
	void LoadCookie( ZKChunkReader& io_ChunkReader );

	// Old persistence stuff...
	void		GetV1XCookieFilePath( std::string& out_strNymFileName ) ;

	bool		LoadNymCookieMapFromV1XFileFormat(
					const std::string& in_strNymFileName,  
					COOKIEJARMAP& ) ;
};


#endif
