// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "StdAfx.h"
#include "RNymRoute.h"
#include <ZKChunk.h>
#include <RZkException.h>

#define NYM_ROUTE_CHUNK_ID_VER 1

RNymRoute::RNymRoute(RNym* pParent)
{
}

RNymRoute::~RNymRoute()
{
}

void 
RNymRoute::GetLastSuccessfulRoute( WRoute& out_LastRoute )
{
	out_LastRoute = m_LastRoute ;
}

void 
RNymRoute::SetSuccessfulRoute( const WRoute& in_Route )
{
	// In theory you can want to reset the 
	// last nym route by calling this with empty 
	// route - but why would you want to?
	ZKASSERT( in_Route.GetSize() != 0 ) ;

	m_LastRoute = in_Route ;
}

void 
RNymRoute::Save( WMemBuf& io_Buffer ) throw(RZkException)
{
	// Only save if we actually have some state
	// for example free nyms may not have anything interesting
	// if they never create routes

	if ( m_LastRoute.GetSize() > 0 )
	{
		ZKChunkWriter cw( io_Buffer, NYM_ROUTE_CHUNK_ID, NYM_ROUTE_CHUNK_ID_VER ) ;

		// Route is always less than 255 hops!
		cw.WriteUInt8( (uint8_t) m_LastRoute.GetSize() ) ;

		for ( size_t i = 0 ; i <  m_LastRoute.GetSize() ; i++ )
		{
			cw.WriteUInt32( m_LastRoute.GetHop( i ) ) ;
		}			
	}
}

void 
RNymRoute::Load( ZKChunkReader& io_ChunkReader ) throw(RZkException)
{
	m_LastRoute.Clear() ;

	uint8_t cntHops = io_ChunkReader.ReadUInt8() ;

	for ( uint8_t cnt = 0 ; cnt < cntHops ; cnt++ )
	{
		m_LastRoute.AddHop( io_ChunkReader.ReadUInt32() ) ;
	}
}
