// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RNymWallet.cpp
//
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "RNymWallet.h"
#include <ZkService.h>
#include <INymCentral.h>
#include <ZKFileConfiguration.h>
#include <ZKChunk.h>
#include <TraceControl.h>
#include <RZkException.h>
#include <string_hlp.h>
#include <lockguard.h>
#include <sstream>

#define NYMWALLET_PERSIST_VERSION 1

static const char* EMAIL_SUPER_DOMAIN[] = { "com", "net", "org", "ca" };
static const int EMAIL_SUPER_DOMAIN_NUM = sizeof(EMAIL_SUPER_DOMAIN) 
										/ sizeof(*EMAIL_SUPER_DOMAIN);

static const char* STREET_SUFFIXES[] = { "Ave", "Circle", "Court", "Cres", "St" };
static const int STREET_SUFFIXES_NUM = sizeof(STREET_SUFFIXES) 
										/ sizeof(*STREET_SUFFIXES);

typedef struct StateZip
{
	ZKSTD::string	strState;
	unsigned int	nZipLowerBound;
	unsigned int	nZipUpperBound;
	StateZip(	const ZKSTD::string& in_strState, 
					const unsigned int in_nZipLowerBound,
					const unsigned int in_nZipUpperBound)
		: strState(in_strState)
		, nZipLowerBound(in_nZipLowerBound)
		, nZipUpperBound(in_nZipUpperBound)
	{};
} StateZip;

// From www.usps.com
static const StateZip STATE_AND_ZIPS[] = { 
	StateZip("NY",  10001, 10048),
	StateZip("CA", 94301, 94310),
	StateZip("AK", 99501, 99524),
	StateZip("WA", 98101, 98112),
	StateZip("FL", 33124, 33170),
	StateZip("CA", 90001, 90099),
	StateZip("LA", 70801, 70827),

};

static const int STATE_AND_ZIPS_NUM = sizeof(STATE_AND_ZIPS) / sizeof(*STATE_AND_ZIPS);

static const unsigned int NUM_LETTERS_IN_ALPHABET = 26;

static const float PROB_LETTER_STARTS_WORD[NUM_LETTERS_IN_ALPHABET] = 
{ (float)0.106078, (float)0.048491, (float)0.059260, (float)0.053567, (float)0.048444, 
  (float)0.085846, (float)0.031781, (float)0.059408, (float)0.047870, (float)0.011799, 
  (float)0.006159, (float)0.037147, (float)0.040902, (float)0.028452, (float)0.038707, 
  (float)0.044364, (float)0.003973, (float)0.036665, (float)0.058843, (float)0.068447, 
  (float)0.018892, (float)0.009836, (float)0.023292, (float)0.013311, (float)0.010595, 
  (float)0.007870
};

static const float COND_PROBS[NUM_LETTERS_IN_ALPHABET][NUM_LETTERS_IN_ALPHABET] = 
{
  {(float)0.004820, (float)0.032978, (float)0.070508, (float)0.039715, (float)0.005197, 
  (float)0.011125, (float)0.029945, (float)0.001547, (float)0.030294, (float)0.001794, 
  (float)0.007042, (float)0.106127, (float)0.045999, (float)0.143244, (float)0.001870, 
  (float)0.029006, (float)0.002848, (float)0.116296, (float)0.070482, (float)0.161961, 
  (float)0.015169, (float)0.024972, (float)0.004193, (float)0.006868, (float)0.033323, 
  (float)0.002678},

  {(float)0.097446, (float)0.121993, (float)0.031791, (float)0.067339, (float)0.145221, 
  (float)0.066292, (float)0.006516, (float)0.001749, (float)0.072705, (float)0.027054, 
  (float)0.004637, (float)0.076727, (float)0.018043, (float)0.004209, (float)0.071000, 
  (float)0.003400, (float)0.000363, (float)0.059781, (float)0.033857, (float)0.004321, 
  (float)0.037852, (float)0.001203, (float)0.001979, (float)0.000838, (float)0.038953, 
  (float)0.004731},

  {(float)0.111289, (float)0.007852, (float)0.028822, (float)0.012391, (float)0.130796, 
  (float)0.026211, (float)0.003527, (float)0.124903, (float)0.052679, (float)0.001371, 
  (float)0.031304, (float)0.034795, (float)0.009892, (float)0.005009, (float)0.185056, 
  (float)0.006256, (float)0.000774, (float)0.030312, (float)0.043133, (float)0.097144, 
  (float)0.039974, (float)0.002397, (float)0.001416, (float)0.000360, (float)0.010830, 
  (float)0.001509},


  {(float)0.081015, (float)0.040324, (float)0.025587, (float)0.115260, (float)0.207800, 
  (float)0.055497, (float)0.010717, (float)0.002084, (float)0.108108, (float)0.003515, 
  (float)0.001868, (float)0.064361, (float)0.012583, (float)0.007144, (float)0.061854, 
  (float)0.004284, (float)0.000713, (float)0.037851, (float)0.023909, (float)0.033144, 
  (float)0.077161, (float)0.004875, (float)0.003685, (float)0.002268, (float)0.011809, 
  (float)0.002585},

  {(float)0.043143, (float)0.008809, (float)0.052371, (float)0.081828, (float)0.046717, 
  (float)0.107053, (float)0.013308, (float)0.001403, (float)0.010653, (float)0.001306, 
  (float)0.002188, (float)0.041118, (float)0.031101, (float)0.101528, (float)0.007393, 
  (float)0.019204, (float)0.006536, (float)0.164187, (float)0.110628, (float)0.077228, 
  (float)0.003898, (float)0.011782, (float)0.019021, (float)0.031170, (float)0.005310, 
  (float)0.001116},

  {(float)0.036528, (float)0.044030, (float)0.078575, (float)0.038593, (float)0.108700, 
  (float)0.314716, (float)0.005691, (float)0.004953, (float)0.093679, (float)0.008141, 
  (float)0.001058, (float)0.011436, (float)0.005705, (float)0.005022, (float)0.119043, 
  (float)0.003322, (float)0.005267, (float)0.038547, (float)0.011229, (float)0.032854, 
  (float)0.017207, (float)0.003927, (float)0.000813, (float)0.000547, (float)0.004034, 
  (float)0.006383},


  {(float)0.066958, (float)0.024517, (float)0.005915, (float)0.008964, (float)0.214508, 
  (float)0.003414, (float)0.009398, (float)0.039087, (float)0.099194, (float)0.000826, 
  (float)0.001153, (float)0.031822, (float)0.032700, (float)0.036335, (float)0.111179, 
  (float)0.009182, (float)0.000564, (float)0.125733, (float)0.077982, (float)0.021017, 
  (float)0.033124, (float)0.001629, (float)0.003090, (float)0.001021, (float)0.014889, 
  (float)0.025800},

  {(float)0.095438, (float)0.002562, (float)0.003439, (float)0.002990, (float)0.316684, 
  (float)0.001433, (float)0.000789, (float)0.003392, (float)0.094245, (float)0.000491, 
  (float)0.001070, (float)0.004998, (float)0.004332, (float)0.009935, (float)0.080129, 
  (float)0.004253, (float)0.006138, (float)0.190308, (float)0.005801, (float)0.135895, 
  (float)0.014124, (float)0.000572, (float)0.001511, (float)0.000579, (float)0.018159, 
  (float)0.000736},

  {(float)0.029120, (float)0.024107, (float)0.081015, (float)0.032218, (float)0.040996, 
  (float)0.033523, (float)0.024493, (float)0.001320, (float)0.006202, (float)0.000770, 
  (float)0.005239, (float)0.053440, (float)0.036262, (float)0.244425, (float)0.090816, 
  (float)0.019238, (float)0.001543, (float)0.026503, (float)0.094906, (float)0.101933, 
  (float)0.004224, (float)0.030653, (float)0.000512, (float)0.005803, (float)0.000407, 
  (float)0.010331},

  {(float)0.162421, (float)0.012827, (float)0.014566, (float)0.012012, (float)0.275545, 
  (float)0.010164, (float)0.017447, (float)0.012302, (float)0.026560, (float)0.008606, 
  (float)0.006033, (float)0.006649, (float)0.010309, (float)0.007953, (float)0.126277, 
  (float)0.054279, (float)0.005453, (float)0.012899, (float)0.039097, (float)0.009204, 
  (float)0.131078, (float)0.006504, (float)0.006830, (float)0.003315, (float)0.007446, 
  (float)0.014222},

  {(float)0.122945, (float)0.042430, (float)0.008831, (float)0.007266, (float)0.315425, 
  (float)0.005920, (float)0.008266, (float)0.008633, (float)0.130369, (float)0.002732, 
  (float)0.008009, (float)0.028224, (float)0.013998, (float)0.042539, (float)0.041401, 
  (float)0.006435, (float)0.008286, (float)0.014493, (float)0.111223, (float)0.017631, 
  (float)0.023888, (float)0.003019, (float)0.006494, (float)0.003217, (float)0.016493, 
  (float)0.001831},

  {(float)0.103816, (float)0.004645, (float)0.008871, (float)0.026769, (float)0.190492, 
  (float)0.005330, (float)0.003573, (float)0.002644, (float)0.272083, (float)0.000847, 
  (float)0.003463, (float)0.094003, (float)0.004920, (float)0.005753, (float)0.095950, 
  (float)0.010477, (float)0.000356, (float)0.002842, (float)0.028464, (float)0.041761, 
  (float)0.035794, (float)0.004890, (float)0.001573, (float)0.000787, (float)0.045725, 
  (float)0.004173},

  {(float)0.232838, (float)0.031421, (float)0.007416, (float)0.004611, (float)0.201546, 
  (float)0.002344, (float)0.016624, (float)0.001513, (float)0.089441, (float)0.000809, 
  (float)0.002007, (float)0.083118, (float)0.029350, (float)0.004787, (float)0.089875, 
  (float)0.083683, (float)0.000457, (float)0.003023, (float)0.031694, (float)0.019535, 
  (float)0.032715, (float)0.001422, (float)0.000947, (float)0.019146, (float)0.009290, 
  (float)0.000389},

  {(float)0.075494, (float)0.003474, (float)0.056181, (float)0.129181, (float)0.149284, 
  (float)0.035697, (float)0.098974, (float)0.002456, (float)0.062447, (float)0.001072, 
  (float)0.011155, (float)0.013603, (float)0.005896, (float)0.015766, (float)0.056210, 
  (float)0.005418, (float)0.000527, (float)0.004659, (float)0.069501, (float)0.159620, 
  (float)0.022433, (float)0.006734, (float)0.001807, (float)0.000594, (float)0.010052, 
  (float)0.001767},

  {(float)0.006185, (float)0.019240, (float)0.038401, (float)0.029275, (float)0.004302, 
  (float)0.067694, (float)0.022466, (float)0.003031, (float)0.007772, (float)0.003248, 
  (float)0.012436, (float)0.059019, (float)0.078568, (float)0.196503, (float)0.018413, 
  (float)0.042840, (float)0.000255, (float)0.171566, (float)0.036472, (float)0.051007, 
  (float)0.068595, (float)0.033083, (float)0.022502, (float)0.004584, (float)0.001600, 
  (float)0.000944},


  {(float)0.125688, (float)0.003363, (float)0.012821, (float)0.009535, (float)0.131464, 
  (float)0.003927, (float)0.006911, (float)0.074222, (float)0.032001, (float)0.000861, 
  (float)0.001508, (float)0.085135, (float)0.008922, (float)0.002343, (float)0.092661, 
  (float)0.040224, (float)0.000511, (float)0.151859, (float)0.060464, (float)0.045754, 
  (float)0.097296, (float)0.002391, (float)0.000991, (float)0.001742, (float)0.006873, 
  (float)0.000531},

  {(float)0.011418, (float)0.006774, (float)0.011655, (float)0.006952, (float)0.007809, 
  (float)0.008934, (float)0.005975, (float)0.004408, (float)0.008312, (float)0.008283, 
  (float)0.005946, (float)0.013933, (float)0.013400, (float)0.005473, (float)0.003786, 
  (float)0.022334, (float)0.003639, (float)0.028102, (float)0.022009, (float)0.012779, 
  (float)0.648514, (float)0.008194, (float)0.003639, (float)0.114598, (float)0.006952, 
  (float)0.006183},

  {(float)0.089066, (float)0.003682, (float)0.048931, (float)0.019617, (float)0.302908, 
  (float)0.017310, (float)0.022770, (float)0.002329, (float)0.078692, (float)0.000657, 
  (float)0.016363, (float)0.012192, (float)0.031015, (float)0.025262, (float)0.108799, 
  (float)0.006830, (float)0.000258, (float)0.014727, (float)0.055650, (float)0.044253, 
  (float)0.014946, (float)0.022419, (float)0.025724, (float)0.000617, (float)0.034245, 
  (float)0.000738},


  {(float)0.063423, (float)0.003328, (float)0.052725, (float)0.013532, (float)0.199795, 
  (float)0.013750, (float)0.006253, (float)0.029757, (float)0.093970, (float)0.000812, 
  (float)0.009159, (float)0.010439, (float)0.011266, (float)0.005666, (float)0.058879, 
  (float)0.039872, (float)0.001944, (float)0.017854, (float)0.065119, (float)0.215967, 
  (float)0.049289, (float)0.001997, (float)0.009514, (float)0.000418, (float)0.024379, 
  (float)0.000893},

  {(float)0.077804, (float)0.003819, (float)0.008271, (float)0.002480, (float)0.150233, 
  (float)0.004868, (float)0.001822, (float)0.172230, (float)0.142515, (float)0.000366, 
  (float)0.004529, (float)0.033281, (float)0.041468, (float)0.002434, (float)0.103877, 
  (float)0.032383, (float)0.000188, (float)0.059589, (float)0.046807, (float)0.035407, 
  (float)0.024127, (float)0.001167, (float)0.012116, (float)0.011890, (float)0.024972, 
  (float)0.001358},

  {(float)0.024908, (float)0.084711, (float)0.043460, (float)0.023855, (float)0.048736, 
  (float)0.005827, (float)0.020294, (float)0.001471, (float)0.029431, (float)0.001139, 
  (float)0.018465, (float)0.090083, (float)0.072961, (float)0.113232, (float)0.004228, 
  (float)0.044497, (float)0.000384, (float)0.110235, (float)0.143049, (float)0.103056, 
  (float)0.006041, (float)0.001771, (float)0.002257, (float)0.003621, (float)0.001029, 
  (float)0.001259},


  {(float)0.122930, (float)0.001717, (float)0.003618, (float)0.003280, (float)0.549510, 
  (float)0.002796, (float)0.001633, (float)0.001229, (float)0.201980, (float)0.001211, 
  (float)0.000934, (float)0.003730, (float)0.013503, (float)0.003711, (float)0.053488, 
  (float)0.002318, (float)0.000591, (float)0.003622, (float)0.008305, (float)0.004532, 
  (float)0.004242, (float)0.001234, (float)0.000929, (float)0.002510, (float)0.004514, 
  (float)0.001933},

  {(float)0.113405, (float)0.002024, (float)0.004844, (float)0.003031, (float)0.143762, 
  (float)0.005299, (float)0.002877, (float)0.082324, (float)0.191275, (float)0.001085, 
  (float)0.002783, (float)0.009374, (float)0.002718, (float)0.028139, (float)0.105209,
  (float)0.005648, (float)0.000488, (float)0.021109, (float)0.065347, (float)0.005360, 
  (float)0.014762, (float)0.001491, (float)0.170190, (float)0.014758, (float)0.001963, 
  (float)0.000735},

  {(float)0.045647, (float)0.029863, (float)0.082097, (float)0.008134, (float)0.047399, 
  (float)0.010417, (float)0.003373, (float)0.009122, (float)0.055477, (float)0.002106, 
  (float)0.001556, (float)0.010398, (float)0.094862, (float)0.012355, (float)0.011293, 
  (float)0.078128, (float)0.002236, (float)0.046644, (float)0.012392, (float)0.297110, 
  (float)0.006569, (float)0.003354, (float)0.003885, (float)0.008954, (float)0.115361, 
  (float)0.001267},


  {(float)0.030549, (float)0.016587, (float)0.030715, (float)0.015598, (float)0.052818, 
  (float)0.004306, (float)0.013842, (float)0.005812, (float)0.022667, (float)0.003428, 
  (float)0.004426, (float)0.037895, (float)0.043319, (float)0.076169, (float)0.153096, 
  (float)0.171826, (float)0.001691, (float)0.027989, (float)0.197671, (float)0.056394, 
  (float)0.007161, (float)0.002800, (float)0.010950, (float)0.003465, (float)0.003567, 
  (float)0.005258},

  {(float)0.228657, (float)0.008899, (float)0.008752, (float)0.007163, (float)0.228046, 
  (float)0.005867, (float)0.008165, (float)0.074980, (float)0.191350, (float)0.010243, 
  (float)0.007628, (float)0.013495, (float)0.009388, (float)0.011955, (float)0.060996, 
  (float)0.006332, (float)0.002983, (float)0.008703, (float)0.009339, (float)0.009363, 
  (float)0.018360, (float)0.007432, (float)0.005305, (float)0.003472, (float)0.026672, 
  (float)0.026452}
};

RNymWallet::RNymWallet(RNym * pParent)
	: m_pParent(pParent)
	, m_bLastRandomFieldWasStateOrZip(false)
	, m_uiStateAndZip(0)

{
	::srand((unsigned)time(NULL));
}

RNymWallet::~RNymWallet()
{
}

bool RNymWallet::AddLoginInfo(const LoginInfo& in_loginInfo)
{
	bool bRet = false;

	if ((in_loginInfo.strUsername == "") 
		|| (in_loginInfo.strPassword == ""))
	{
		ZKASSERT(false);
		return bRet;
	}

	ZKS::Lock_Guard<WZkMutex> gate(m_csLoginInfo);
	
	ZKSTD::string strNymName;
	m_pParent->GetNymName(strNymName);
	if (strNymName != in_loginInfo.strNym)
	{
		// Bad strNym field in LoginInfo.
		// It should match the nym that own this NymWallet
	}
	else
	{
		ZKSTD::string strDomain = ZKS::ExtractDomain(in_loginInfo.strDomain);
		ZKS::MakeLower(strDomain);

		LoginInfo loginInfo(in_loginInfo);
		loginInfo.strDomain = strDomain;

		bool bFound = false;
		//If this loginInfo is already in the map, don't insert it again
		for (Domain2LoginInfoMap::iterator it = m_domain2LoginInfoMap.equal_range(strDomain).first;
			it != m_domain2LoginInfoMap.equal_range(strDomain).second;
			++it)
		{
			ZKASSERT((*it).second.strDomain == (*it).first);
			ZKASSERT((*it).second.strDomain == strDomain);

			if ((*it).second == loginInfo)
			{
				bFound = true;
				if ((*it).second.strPassword != loginInfo.strPassword)
				{
					bRet = true;
					(*it).second.strPassword = loginInfo.strPassword;
					break;
				}
			}
		}
		if (!bFound)
		{
			m_domain2LoginInfoMap.insert(
				Domain2LoginInfoMap::value_type(loginInfo.strDomain, loginInfo));
			bRet = true;
		}
	}
	return bRet;
}

void RNymWallet::GetLoginInfo(	const ZKSTD::string in_strDomain,
								ZKSTD::vector<LoginInfo>& out_loginInfoVector)
{
	ZKS::Lock_Guard<WZkMutex> gate(m_csLoginInfo);
	
	out_loginInfoVector.clear();

	ZKSTD::string strDomain = ZKS::ExtractDomain(in_strDomain);
	ZKS::MakeLower(strDomain);

	for (Domain2LoginInfoMap::const_iterator it 
		= m_domain2LoginInfoMap.equal_range(strDomain).first;
		it != m_domain2LoginInfoMap.equal_range(strDomain).second;
		++it)
	{
		ZKASSERT((*it).second.strDomain == strDomain);
		out_loginInfoVector.push_back((*it).second);
	}
}
								
void RNymWallet::GetAllLoginInfo(ZKSTD::vector<LoginInfo>& out_loginInfoVector)
{
	ZKS::Lock_Guard<WZkMutex> gate(m_csLoginInfo);
	
	out_loginInfoVector.clear();
	
	for (Domain2LoginInfoMap::const_iterator it = m_domain2LoginInfoMap.begin();
		it != m_domain2LoginInfoMap.end();
		++it)
	{
		out_loginInfoVector.push_back((*it).second);
	}
}

bool RNymWallet::DeleteLoginInfo(const LoginInfo& in_loginInfo)
{
	ZKS::Lock_Guard<WZkMutex> gate(m_csLoginInfo);
	bool bRet = false;

	ZKSTD::string strDomain = ZKS::ExtractDomain(in_loginInfo.strDomain);
	ZKS::MakeLower(strDomain);
		
	LoginInfo loginInfo(in_loginInfo);
	loginInfo.strDomain = strDomain;

	for (Domain2LoginInfoMap::iterator it = m_domain2LoginInfoMap.equal_range(strDomain).first;
		it != m_domain2LoginInfoMap.equal_range(strDomain).second;
		++it)
	{
		ZKASSERT((*it).second.strDomain == (*it).first);
		ZKASSERT((*it).second.strDomain == strDomain);

		if ((*it).second == loginInfo)
		{
			m_domain2LoginInfoMap.erase(it);
			bRet = true;
			break;
		}
	}
	return bRet;
}

ZKSTD::string RNymWallet::GetField(const int in_field)
{
	ZKS::Lock_Guard<WZkMutex> gate(m_csPersonalInfo);

	Field2ValueMap::const_iterator it = m_field2ValueMap.find(in_field);
	if (it != m_field2ValueMap.end())
	{
		return (*it).second;
	}
	return "";
}
	
void RNymWallet::SetField(	const int in_field, 
							const ZKSTD::string& in_strValue)
{
	ZKS::Lock_Guard<WZkMutex> gate(m_csPersonalInfo);

	if (in_strValue == "")
	{
		m_field2ValueMap.erase(in_field);
	}
	else
	{
		m_field2ValueMap[in_field] = in_strValue;
	}
}

void 
RNymWallet::Load( ZKChunkReader& io_ChunkReader ) 
{
	// Flush all existing state
	m_domain2LoginInfoMap.clear();
	m_field2ValueMap.clear();

	// We demand that our chunk be sent to us
	ZKASSERT( io_ChunkReader.GetChunkID() == WALLET_CHUNK_ID ) ;

	// Process and nested wallet chunks that we find
	while ( !io_ChunkReader.IsEndOfChunk() )
	{
		ZKChunkReader cr( io_ChunkReader.GetReadBuffer() );

		switch( cr.GetChunkID() )
		{
			case( WALLET_PERSONALINFO_CHUNK_ID ) :
				LoadPersonalInfoChunk( cr ) ;
				break ;
				
			case( WALLET_LOGININFO_CHUNK_ID ) :
				LoadLoginInfoChunk( cr ) ;
				break ;

			default :
				// Future chunks - just ignore them
				ZKASSERT( false ) ;
				break ;
		}
	}
}

void 
RNymWallet::LoadLoginInfoChunk( ZKChunkReader& io_ChunkReader ) 
{
	ZKASSERT( io_ChunkReader.GetChunkID() == WALLET_LOGININFO_CHUNK_ID ) ; 
	
	ZKSTD::string strNymName;
	m_pParent->GetNymName(strNymName);

	while( !io_ChunkReader.IsEndOfChunk() )
	{
		ZKSTD::string strUsername, strPassword, strDomain ;

		io_ChunkReader.ReadString( strUsername ) ;
		io_ChunkReader.ReadString( strPassword ) ;
		io_ChunkReader.ReadNonUIString( strDomain ) ;

#if defined( TRACE_LOAD_NYM_CENTRAL )
		ZKTRACE2( "Loading Wallet Login - Nym %s, Domain %s", strNymName.c_str(), strDomain.c_str() ) ;				
		ZKTRACE2( "Username %s, Password %s\n", strUsername.c_str(), strPassword.c_str() ) ;				
#endif

		LoginInfo loginInfo(strNymName, strUsername, strPassword, strDomain);
		m_domain2LoginInfoMap.insert(Domain2LoginInfoMap::value_type(strDomain, loginInfo));
	}
}

void 
RNymWallet::LoadPersonalInfoChunk( ZKChunkReader& io_ChunkReader ) 
{
	ZKASSERT( io_ChunkReader.GetChunkID() == WALLET_PERSONALINFO_CHUNK_ID ) ; 
	
	while( !io_ChunkReader.IsEndOfChunk() )
	{
		int nField = io_ChunkReader.ReadInt32();
		ZKSTD::string strValue;
		io_ChunkReader.ReadString( strValue ) ;

		m_field2ValueMap.insert(Field2ValueMap::value_type(nField, strValue));
	}
}

void 
RNymWallet::Save( WMemBuf& io_Buffer ) throw(RZkException)
{
	ZKChunkWriter cw( io_Buffer, WALLET_CHUNK_ID, NYMWALLET_PERSIST_VERSION ) ;

	if ( m_domain2LoginInfoMap.size() > 0 )
	{
		SaveLoginInfoChunk( io_Buffer ) ;
	}
	if ( m_field2ValueMap.size() > 0)
	{
		SavePersonalInfoChunk( io_Buffer ) ;
	}
}

void 
RNymWallet::SavePersonalInfoChunk( WMemBuf& io_Buffer ) throw(RZkException)
{
	ZKChunkWriter cw( io_Buffer, WALLET_PERSONALINFO_CHUNK_ID, 1 ) ;

	unsigned int nCount = 0;
	for (Field2ValueMap::const_iterator it = m_field2ValueMap.begin(); 
		it != m_field2ValueMap.end(); ++it)
	{
		++nCount;
		cw.WriteInt32( (*it).first ) ;
		cw.WriteString( (*it).second ) ;
	}
	ZKASSERT(nCount == m_field2ValueMap.size());	
}


void 
RNymWallet::SaveLoginInfoChunk( WMemBuf& io_Buffer ) throw(RZkException)
{
	ZKChunkWriter cw( io_Buffer, WALLET_LOGININFO_CHUNK_ID, 1 ) ;

	unsigned int nCount = 0;
	for (Domain2LoginInfoMap::const_iterator it = m_domain2LoginInfoMap.begin(); 
		it != m_domain2LoginInfoMap.end(); ++it)
	{
		++nCount;
		cw.WriteString( (*it).second.strUsername ) ;
		cw.WriteString( (*it).second.strPassword ) ;
		cw.WriteNonUIString( (*it).second.strDomain ) ;
	}
	ZKASSERT(nCount == m_domain2LoginInfoMap.size());	
}

void
RNymWallet::PopulatePersonalInfoWithRandomFields(bool in_bPseudonymous)
{
	SetField(WALLET_NAME_TITLE, GetRandomField(WALLET_NAME_TITLE, in_bPseudonymous));
	SetField(WALLET_NAME_FIRST, GetRandomField(WALLET_NAME_FIRST, in_bPseudonymous));
	SetField(WALLET_NAME_MIDDLE, GetRandomField(WALLET_NAME_MIDDLE, in_bPseudonymous));
	SetField(WALLET_NAME_LAST, GetRandomField(WALLET_NAME_LAST, in_bPseudonymous));
	SetField(WALLET_NAME_SUFFIX, GetRandomField(WALLET_NAME_SUFFIX, in_bPseudonymous));

	SetField(WALLET_BIRTH_DAY, GetRandomField(WALLET_BIRTH_DAY, in_bPseudonymous));
	SetField(WALLET_BIRTH_MONTH, GetRandomField(WALLET_BIRTH_MONTH, in_bPseudonymous));
	SetField(WALLET_BIRTH_YEAR, GetRandomField(WALLET_BIRTH_YEAR, in_bPseudonymous));
	
	SetField(WALLET_GENDER, GetRandomField(WALLET_GENDER, in_bPseudonymous));
	
	for(int i = 0; i < WALLET_NUM_ADDR; ++i)
	{
		for (int j = 1; j < WALLET_ADDR_END; ++j)
		{
			SetField(WALLET_ADDR_START[i] + j, GetRandomField(WALLET_ADDR_START[i] + j, in_bPseudonymous));
		}
	}
}

ZKSTD::string 
RNymWallet::GetRandomField(const int in_field, bool in_bPseudonymous)
{
	bool bThisFieldIsStateOrZip = false;
	ZKSTD::ostringstream oss;

	bool bFoundField = true;
	if (in_field == WALLET_NAME_TITLE)
	{
		oss << WALLET_NAME_TITLES[GetRandomUInt(0, WALLET_NAME_TITLES_NUM - 1)];
	}
	else if (in_field == WALLET_NAME_FIRST)
	{
		if (!in_bPseudonymous
			|| IsThisTheUserNym()
			|| (!m_pParent->DoesNymUseCloud() && !m_pParent->DoesNymUseMail()))
		{
			oss << GetRandomWord(4, 10);
		}
		else
		{
			ZKSTD::string strNymName;
			ZKVERIFY(m_pParent->GetNymName(strNymName));
			oss << strNymName;
		}
	}
	else if (in_field == WALLET_NAME_MIDDLE)
	{
		oss << GetRandomWord(1, 1);
	}
	else if (in_field == WALLET_NAME_LAST)
	{
		if (!in_bPseudonymous
			|| IsThisTheUserNym()
			|| (!m_pParent->DoesNymUseCloud() && !m_pParent->DoesNymUseMail()))
		{
			oss << GetRandomWord(4, 10);
		}
		else
		{
			oss << "Freedom";
		}
	}
	else if (in_field == WALLET_NAME_SUFFIX)
	{
		oss << WALLET_NAME_SUFFIXES[GetRandomUInt(0, WALLET_NAME_SUFFIXES_NUM - 1)];
	}
	else if (in_field == WALLET_BIRTH_DAY)
	{
		oss << GetRandomUInt(1, 28);
	}
	else if (in_field == WALLET_BIRTH_MONTH)
	{
		oss << GetRandomUInt(1, 12);;
	}
	else if (in_field == WALLET_BIRTH_YEAR)
	{
		oss << GetRandomUInt(1920, 1980);
	}
	else if (in_field == WALLET_GENDER)
	{
		oss << WALLET_GENDERS[GetRandomUInt(0, WALLET_GENDERS_NUM - 1)];
	}
	else
	{
		bFoundField = false;
	}

	if (!bFoundField)
	{
		for (int i = 0; i < WALLET_NUM_ADDR; ++i)
		{
			bFoundField = true;

			if (in_field == (WALLET_ADDR_START[i] + WALLET_STREET_LINE1_OFFSET))
			{
				oss << GetRandomUInt(1, 1000)
					<< " "
					<< GetRandomWord(5, 10)
					<< " "
					<< STREET_SUFFIXES[GetRandomUInt(0, STREET_SUFFIXES_NUM - 1)];
			}
			else if (in_field == (WALLET_ADDR_START[i] + WALLET_STREET_LINE2_OFFSET))
			{
				// NOTHING?
			}
			else if (in_field == WALLET_ADDR_START[i] + WALLET_STATEPROV_OFFSET)
			{
				if (!m_bLastRandomFieldWasStateOrZip)
				{
					m_uiStateAndZip = GetRandomUInt(0, STATE_AND_ZIPS_NUM - 1);
				}
				oss << STATE_AND_ZIPS[m_uiStateAndZip].strState;
				bThisFieldIsStateOrZip = true;
			}
			else if (in_field == WALLET_ADDR_START[i] + WALLET_POSTALCODE_OFFSET)
			{				
				if (!m_bLastRandomFieldWasStateOrZip)
				{
					m_uiStateAndZip = GetRandomUInt(0, STATE_AND_ZIPS_NUM - 1);
				}
				oss << GetRandomUInt(	STATE_AND_ZIPS[m_uiStateAndZip].nZipLowerBound,
										STATE_AND_ZIPS[m_uiStateAndZip].nZipUpperBound);
				bThisFieldIsStateOrZip = true;
			}
			else if (in_field == (WALLET_ADDR_START[i] + WALLET_COUNTRY_OFFSET))
			{
				oss << "UNITED STATES";
			}
			else if ((in_field == (WALLET_ADDR_START[i] + WALLET_TELEPHONE_INT_CODE_OFFSET))
				|| (in_field == (WALLET_ADDR_START[i] + WALLET_FAX_INT_CODE_OFFSET)))
			{
				//Nothing
			}
			else if ((in_field == (WALLET_ADDR_START[i] + WALLET_TELEPHONE_LOC_CODE_OFFSET))
				|| (in_field == (WALLET_ADDR_START[i] + WALLET_FAX_LOC_CODE_OFFSET)))
			{
				oss << GetRandomUInt(200, 999);
			}
			else if ((in_field == (WALLET_ADDR_START[i] + WALLET_TELEPHONE_PREFIX_OFFSET))
				|| (in_field == (WALLET_ADDR_START[i] + WALLET_FAX_PREFIX_OFFSET)))
			{
				oss << GetRandomUInt(200, 999);
			}
			else if ((in_field == (WALLET_ADDR_START[i] + WALLET_TELEPHONE_SUFFIX_OFFSET))
				|| (in_field == (WALLET_ADDR_START[i] + WALLET_FAX_SUFFIX_OFFSET)))
			{
				oss << GetRandomUInt(1000, 9999);
			}
			else if (in_field == (WALLET_ADDR_START[i] + WALLET_TELEPHONE_EXTENSION_OFFSET))
			{
				oss << GetRandomUInt(100, 9999);
			}
			else if (in_field == (WALLET_ADDR_START[i] + WALLET_EMAIL_OFFSET))
			{
				if (!in_bPseudonymous
					|| IsThisTheUserNym()
					|| !m_pParent->DoesNymUseMail())
				{
					ZKSTD::string str = GetRandomWord(4, 7);
					ZKS::MakeLower(str);
					oss << str
						<< '@';
					str = GetRandomWord(4, 7);
					ZKS::MakeLower(str);
					oss << str
						<< '.'
						<< EMAIL_SUPER_DOMAIN[GetRandomUInt(0, EMAIL_SUPER_DOMAIN_NUM - 1)];
				}
				else
				{
					ZKASSERT(m_pParent->DoesNymUseMail());

					ZKSTD::string strNymName;
					ZKVERIFY(m_pParent->GetNymName(strNymName));

					ZKSTD::string strDomain;
					ZKVERIFY(m_pParent->GetNymEmailDomain(strDomain));

					oss << strNymName << '@' << strDomain;
				}
			}
			else
			{
				bFoundField = false;
			}
			if (bFoundField)
			{
				break;
			}
		}
	}
	if (!bFoundField)
	{
		//case(WALLET_ADDR_START[i] + WALLET_CITY_OFFSET):
		oss << GetRandomWord(4, 10);
	}
	m_bLastRandomFieldWasStateOrZip = bThisFieldIsStateOrZip;

	return oss.str();
}

bool
RNymWallet::IsThisTheUserNym()
{
	ZKSTD::string strThisNymName, strUserNymName;

	ZKVERIFY(m_pParent->GetNymName(strThisNymName));
	ZkService::GetINymCentral()->GetNymName(USER_NYM_INDEX, strUserNymName);

	return (strThisNymName == strUserNymName);
}

ZKSTD::string
RNymWallet::GetRandomWord(const unsigned int in_nMinLength, const unsigned int in_nMaxLength)
{
	const unsigned int nLength = GetRandomUInt(in_nMinLength, in_nMaxLength);
	ZKSTD::string strRandomWord(nLength, ' ');

	if (nLength > 0)
	{
		float nProb = GetRandomFloat(0, 1);
		unsigned int nLastLetter = 0;
		float nTotal = 0;
		for (unsigned int i = 0; i < NUM_LETTERS_IN_ALPHABET; ++i)
		{
			nTotal += PROB_LETTER_STARTS_WORD[i];
			if (nProb < nTotal)
			{
				nLastLetter = i;
				break;
			}
		}
		strRandomWord.at(0) = 'A' + nLastLetter;

		for (unsigned int j = 1; j < nLength; ++j)
		{
			nProb = GetRandomFloat(0, 1);
			
			nTotal = 0;
			for (unsigned int k = 0; k < NUM_LETTERS_IN_ALPHABET; ++k)
			{
				nTotal += COND_PROBS[nLastLetter][k];
				if (nProb < nTotal)
				{
					nLastLetter = k;
					break;
				}
			}
			strRandomWord.at(j) = 'a' + nLastLetter;
		}
	}
	return strRandomWord;
}

float
RNymWallet::GetRandomFloat(const float in_nLowerBound, const float in_nUpperBound)
{
	ZKASSERT(in_nUpperBound >= in_nLowerBound);
	
	return ((float) rand() / RAND_MAX) * (in_nUpperBound - in_nLowerBound) + in_nLowerBound;
}

unsigned int
RNymWallet::GetRandomUInt(const unsigned int in_nLowerBound, const unsigned int in_nUpperBound)
{
	ZKASSERT(in_nUpperBound >= in_nLowerBound);
	
	return rand() % (in_nUpperBound - in_nLowerBound + 1) + in_nLowerBound;
}

void RNymWallet::SelfTest()
{
#ifdef INTERNAL_BUILD

	unsigned int uiRand = GetRandomUInt(0, 1);
	ZKTEST((uiRand == 0) || (uiRand == 1));

	uiRand = GetRandomUInt(10, 10);
	ZKTEST(uiRand == 10);

	float dRand = GetRandomFloat(10, 20);
	ZKTEST((dRand >= 10) && (dRand <= 20));

	uiRand = GetRandomUInt(200, 200);
	ZKTEST(uiRand == 200);

	ZKTEST(GetRandomWord(7, 7).size() == 7);
	ZKSTD::string str = GetRandomWord(5, 7);
	ZKTEST((str.size() >= 5) && (str.size() <= 7));

	GetRandomField(WALLET_ADDR_START[0] + WALLET_STATEPROV_OFFSET, true);
	ZKTEST(m_bLastRandomFieldWasStateOrZip == true);
	unsigned int uiTemp = m_uiStateAndZip;

	GetRandomField(WALLET_ADDR_START[0] + WALLET_POSTALCODE_OFFSET, true);
	ZKTEST(m_bLastRandomFieldWasStateOrZip == true);
	ZKTEST(uiTemp == m_uiStateAndZip);

	GetRandomField(WALLET_ADDR_START[0] + WALLET_FAX_SUFFIX_OFFSET, true);
	ZKTEST(m_bLastRandomFieldWasStateOrZip == false);
#endif
}
