// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RWallet.h


#if !defined(AFX_RNYMWALLET_H_INCLUDED_)
#define AFX_RNYMWALLET_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifdef _WIN32
#pragma warning(disable: 4786)
#endif

#include "INymWallet.h"
#include <DZkError.h>
#include <WSyncTypes.h>
#include "RNym.h"


typedef ZKSTD::multimap<ZKSTD::string, LoginInfo> Domain2LoginInfoMap;

class ZKChunkReader ;

class RNymWallet : public INymWallet
{
public:
	RNymWallet(RNym* pParent);
	virtual ~RNymWallet();

	/////////////////////////////////////////////////////////////////////////
	// Login Info
	/////////////////////////////////////////////////////////////////////////

	virtual bool AddLoginInfo(const LoginInfo& in_loginInfo);

	virtual void GetLoginInfo(	const ZKSTD::string in_strDomain,
								ZKSTD::vector<LoginInfo>& out_loginInfoVector);

	virtual void GetAllLoginInfo(ZKSTD::vector<LoginInfo>& out_loginInfoVector);
								
	virtual bool DeleteLoginInfo(const LoginInfo& in_loginInfo);

	/////////////////////////////////////////////////////////////////////////
	// Personal Info
	/////////////////////////////////////////////////////////////////////////
	
	virtual ZKSTD::string GetField(			const int in_field);
	
	virtual void SetField(					const int in_field, 
											const ZKSTD::string& in_strValue);

	virtual ZKSTD::string GetRandomField(	const int in_field, 
											bool in_bPseudonymous);

	virtual void PopulatePersonalInfoWithRandomFields(bool in_bPseudonymous);

	/////////////////////////////////////////////////////////////////////////
	// Persistence
	/////////////////////////////////////////////////////////////////////////

	virtual void Save( WMemBuf& io_Buffer ) throw(RZkException) ;

	virtual void Load( ZKChunkReader& io_ChunkReader ) throw(RZkException) ;

private:
	RNymWallet(const RNymWallet& rhs);
	RNymWallet& operator=(const RNymWallet& rhs);

	void LoadLoginInfoChunk( ZKChunkReader& io_ChunkReader ) throw(RZkException) ;
	void LoadPersonalInfoChunk( ZKChunkReader& io_ChunkReader ) throw(RZkException) ;

	void SaveLoginInfoChunk( WMemBuf& io_Buffer ) throw(RZkException) ;
	void SavePersonalInfoChunk( WMemBuf& io_Buffer ) throw(RZkException) ;

	ZKSTD::string GetRandomWord(const unsigned int in_nMinLength, const unsigned int in_nMaxLength);
	float GetRandomFloat(const float in_nLowerBound, const float in_nUpperBound);
	unsigned int GetRandomUInt(const unsigned int in_nLowerBound, const unsigned int in_nUpperBound);

	bool IsThisTheUserNym();

private:
	Domain2LoginInfoMap			m_domain2LoginInfoMap;
	Field2ValueMap				m_field2ValueMap;

	RNym*						m_pParent;
	mutable	WZkMutex			m_csLoginInfo;
	mutable WZkMutex			m_csPersonalInfo;

	bool						m_bLastRandomFieldWasStateOrZip;
	unsigned int				m_uiStateAndZip;
	
public:
	virtual void SelfTest();
};

#endif // !defined(AFX_RWALLET_H_INCLUDED_)
