/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef _RNYMWEBCACHEMGR_H
#define _RNYMWEBCACHEMGR_H

#include <WSyncTypes.h>
#include <INymWebCacheMgr.h>

#define NYM_WEB_CACHE_MGR_CHUNK_VERSION 1

class WebCacheInfo
{
public:
	WebCacheInfo()
		: m_lastAccessed(0)
		, m_lastModified(0)
	{
	}

	WebCacheInfo(const ZKSTD::string& in_strUrl, const time_t in_lastAccessed,
		const time_t in_lastModified, const ZKSTD::string in_strEtag)
		: m_strUrl(in_strUrl)
		, m_lastAccessed(in_lastAccessed)
		, m_lastModified(in_lastModified)
		, m_strEtag(in_strEtag)
	{
	}

	WebCacheInfo(const ZKSTD::string& in_strUrl,
		const time_t in_lastModified, const ZKSTD::string in_strEtag)
		: m_strUrl(in_strUrl)
		, m_lastAccessed(0)
		, m_lastModified(in_lastModified)
		, m_strEtag(in_strEtag)
	{
		m_lastAccessed = time(NULL);
	}

	WebCacheInfo(const ZKSTD::string& in_strUrl,
		const ZKSTD::string in_strEtag)
		: m_strUrl(in_strUrl)
		, m_lastAccessed(0)
		, m_lastModified(0)
		, m_strEtag(in_strEtag)
	{
		m_lastAccessed = time(NULL);
	}

	WebCacheInfo(const ZKSTD::string& in_strUrl,
		const time_t in_lastModified)
		: m_strUrl(in_strUrl)
		, m_lastAccessed(0)
		, m_lastModified(in_lastModified)
	{
		m_lastAccessed = time(NULL);
	}

	ZKSTD::string m_strUrl;
	time_t m_lastAccessed;

	time_t m_lastModified;
	ZKSTD::string m_strEtag;
};

typedef ZKSTD::multimap<ZKSTD::string, WebCacheInfo> WebCacheMap;

class RNymWebCacheMgr : public INymWebCacheMgr
{
public:
	RNymWebCacheMgr();
	virtual ~RNymWebCacheMgr();

	virtual void ProcessIncomingETag(const ZKSTD::string& in_strUrl,
								const ZKSTD::string& in_strETag);

	virtual void ProcessIncomingLastDateModified(const ZKSTD::string& in_strUrl,
								const ZKSTD::string& in_strLastDateModified);

	virtual bool GetOutgoingTagForUrl(const ZKSTD::string& in_strUrl, ZKSTD::string& out_strField, ZKSTD::string& out_strVal);

	virtual bool GetETagToGiveToBrowser(const ZKSTD::string& in_strUrl, ZKSTD::string& out_strTag);
	virtual bool GetLastModifiedTagToGiveToBrowser(const ZKSTD::string& in_strUrl, ZKSTD::string& out_strTag);

	virtual void	Save( WMemBuf& io_Buffer );
	virtual void	Load( ZKChunkReader& io_ChunkReader );
	
private:
	void PruneCacheInfo();

	RNymWebCacheMgr(const RNymWebCacheMgr& rhs);
	RNymWebCacheMgr& operator=(const RNymWebCacheMgr& rhs);

	WebCacheMap m_mapCacheInfo;

	mutable	WZkMutex			m_cs;

};

#endif // #ifndef _RNYMWEBCACHEMGR_H