// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RScriptServer.h

#if !defined(AFX_RSCRIPTSERVER_H_INCLUDED_)
#define AFX_RSCRIPTSERVER_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifdef _WIN32
#pragma warning(disable: 4786)
#endif

#include "IScriptServer.h"
#include <DZkError.h>
#include "RNym.h"
#include <zkThread.h>

class RScriptServer : public IScriptServer
{
public:
	RScriptServer();
	virtual ~RScriptServer();

	virtual bool UpdateAllInNonBlockingThread();
	virtual void AbortIfStillUpdating();

	virtual bool Update(bool in_bUpdateWalletScripts = true, 
						bool in_bUpdateAdBlockerRules = true);

	virtual void UpdateAllWithRetries();

#ifdef INTERNAL_BUILD
	virtual bool AddScriptMappings(	const ZKSTD::string& in_strDomain, 
									const String2StringMap& in_realName2ZkNameMap, 
									bool in_bOverwrite);

	virtual bool UploadAdBlockerRules(const ZKSTD::string& in_strRuleList);
#endif //INTERNAL_BUILD

private:
	RScriptServer(const RScriptServer& rhs);
	RScriptServer& operator=(const RScriptServer& rhs);
	
	static void * ZK_THREAD_CALLBACK UpdateAllThread(void * in_pThis);

	bool GetValue(const char* in_strResponse, const char* in_strParam, ZKSTD::string& out_strValue);
	bool ProcessWalletScriptsUpdate(const char* in_strResponse);
	bool ProcessAdBlockerUpdate(const char* in_strResponse);
	bool Decompress(	char* in_strCompressed, 
						unsigned int in_nCompressedLen, 
						char** out_pStrDecompressed);
	bool CheckSignature(char* in_strDataAndSignature, 
						const unsigned int in_nDataAndSignatureLen,
						unsigned int& out_nDataLen);
	ZKSTD::string GetNonce();

#ifdef INTERNAL_BUILD
	void DealWithConflicts(const ZKSTD::string& in_strDomain, const char* in_strResponse);
	ZKSTD::string m_strScriptServerPassword;
#endif //INTERNAL_BUILD

#if defined(INTERNAL_BUILD) && defined(WIN32)
	bool PromptForPassword();
#endif

	ZkThread		m_pUpdateThread;
	ZkEvent			m_abortUpdateEvent;
	bool			m_bUpdateThreadExists;
	bool			m_bAbortUpdate;
	ZKSTD::string	m_strScriptServerHost;

public:
	virtual void SelfTest();
};

#endif // !defined(AFX_RSCRIPTSERVER_H_INCLUDED_)
