// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RTokenService.h

#ifndef _TOKENSERVICE_H
#define _TOKENSERVICE_H

#include "ITokenService.h"
#include <DZkError.h>
#include <frTokenClient.h>
#include <frTokenCache.h>
#include <frNymClient.h>
#include <zkList.h>
#include <ErrorHandling.h>
#include <ZKConfiguration.h>

class WRemoteNym;

class RTokenService : public ITokenService
{

public:
	RTokenService(WRemoteNym& RemoteNym);
	virtual ~RTokenService();

	virtual bool				GetDescriptionOfTokenX(const unsigned int, WTokenDescriptions&) ;
	virtual unsigned int		GetTotalNumberOfTokens() ;
		
	// Use these to spend a token
	// 1. Get a Token in BASE64, pass to server
	virtual bool				GetTokenXinBase64(const unsigned int, ZKSTD::string&);

	// 2. If successful, remove the Token from the Array
	virtual bool				RemoveTokenXFromArray(const unsigned int);
	virtual bool				RemoveTokenFromArray(const ZKSTD::string&);


	// Serial number setup and token retrieval

	virtual	void				GetSerialNumberCapabilities(
									WSerialNumber& io_SerialNumber ) throw(RZkException);

	virtual void				RedeemSerialNumberForTokens( 
									const WSerialNumber& in_SerialNumber ) throw(RZkException);

	virtual void				RedeemSerialNumberForTokens( 
									const WSerialNumber& in_SerialNumber, 
									ZKSTD::vector<tstring>& out_vStrTokens) throw(RZkException);

	virtual void				GetFreeNymCreateSerialNumberAndRedeemIt( ) throw(RZkException);
	virtual void				GetFreeNymExtendSerialNumberAndRedeemIt( ZKSTD::string& out_token ) throw(RZkException);

	virtual void				GetTokensGroupedByTypeID(ZKSTD::vector<TokensOfType>& out_tokenTypes);

	virtual unsigned int		GetNumberOfDifferentTokenTypeID();

	void GetInterface( ITokenService** ) ;

	// Determine the token capability to use the cloud
	virtual bool DoesTokenUseCloud(const unsigned int);
	virtual bool DoesTokenUseCloud(const ZKSTD::string&);

	// Determine the token capability to use the mail
	virtual bool DoesTokenUseMail(const unsigned int);
	virtual bool DoesTokenUseMail(const ZKSTD::string&);

private:
	RTokenService(const RTokenService& rhs);
	RTokenService& operator=(const RTokenService& rhs);

	void 
	TestAndThrowServerResponse(
		int bStatus, 
		char* in_strOperation) throw (RZkException) ;

	// find the index of the token string
	bool FindIndex( const ZKSTD::string& in_strToken, unsigned int& out_index );

	TokensOfType & FindItemForTokenDesc(const WTokenDescriptions & in_Desc,
										ZKSTD::vector<TokensOfType>& io_tokenTypes );

	void                GetSerialNumberAndRedeemIt( 
								SerialNumberTypes in_type, 
								ZKSTD::vector<tstring>& out_vStrTokens) throw(RZkException);

	// ctor member init
	FrTokenCache			m_FrTokenCache;
	WRemoteNym&				m_RemoteNym ;   // Shared remote nym object

	DZkError				m_ZkError;
	TokenServerResponse		m_TokenServerResponse;	

	virtual bool SaveTokenCache();
	virtual bool LoadTokenCache();
} ;

#endif 
