/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef RWORDSCAN_H
#define RWORDSCAN_H

#include "IWordScan.h"
#include <WSyncTypes.h>

class RWordScan : public IWordScan
{
public:

	RWordScan() ;
	virtual ~RWordScan() ;

	////////////////////////////////////////////////////////////////////////////
	// Offensive Words/Word Scan
	////////////////////////////////////////////////////////////////////////////

	virtual bool WordScan(
						unsigned char* in_pBuf, 
						size_t	in_pBufSiz,
						ZKSTD::vector<ZKSTD::string>& out_vecstrOffensiveWordsFound) ;
	
	// This variant allows you to specify which nym is "Acting"  For example if nym A is active
	// and Nym B is sending a mail then Nym B is the originating nym
	virtual bool WordScan(
						unsigned char* in_pBuf, 
						size_t	in_pBufSiz,
						ZKSTD::vector<ZKSTD::string>& out_vecstrOffensiveWordsFound,
						size_t indexOriginatingNym) ;

	virtual ZKSTD::string Sanitize(const ZKSTD::string& in_strToSanitize) const throw();

private:

	bool IsBinaryValueInBuffer(
			unsigned char* in_pBuf, 
			size_t	in_pBufSiz,
			const ZKSTD::vector< uint32_t > & in_vecValuesToSearchFor ) ;

	// Add the http encoded versions of any phrases in the word scan list that
	// contain a space.  For example for "big daddy" we add "big+daddy" and "big%20daddy"
	void AddWebQueryStringSpaceEquivs( ZKSTD::vector<ZKSTD::string>& io_vecstrWordScanWords ) ;

	WZkMutex						m_cs;
} ;

#endif