// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RKeyQry.h: declaration for the CKeyQry class.
// Clients should always access the functionality of this
// class through its interface IKeyQry

#if !defined(AFX_KeyQry_H__353DF902_E0FC_11D2_8E3D_00104B6AF5F7__INCLUDED_)
#define AFX_KeyQry_H__353DF902_E0FC_11D2_8E3D_00104B6AF5F7__INCLUDED_

#if defined(WIN32)
#	if _MSC_VER > 1000
#	pragma once
#	endif // _MSC_VER > 1000

#	include <winsock2.h>
#elif defined(ZK_LINUX)
#	include <sys/types.h>
#	include <sys/socket.h>
#	include <clientlinux.h>
#else
#error "This source only compiles on Linux or Win32, sorry"
#endif

#include "IKeyQry.h"
#include <DZkError.h>
#include <fr_keyqry.h>
#include <ErrorHandling.h>

#include <WSyncTypes.h>

class RKeyQry : public IKeyQry
{
	

public:

	RKeyQry();	
	virtual		~RKeyQry();

	virtual bool GetKey(
				DZkError&			err, 
				const FrEntIdType	in_entity, 
				const int			nTimeout, 
				const char*			ccEntityName, 
				const unsigned int	uKeyVersion, 
				const FrPubKeyType  in_keytype, 
				OUT FrCryptPubKey*	outkey ) throw(RZkException);

	virtual bool GetKeyFromCache( 
					DZkError& err, 
					const FrEntIdType	in_entity, 
					const char*			ccEntityName, 
					const unsigned int	uKeyVersion, 
					const FrPubKeyType  in_keytype, 
					OUT FrCryptPubKey*	outkey ) ;

	virtual void GetMultipleKeys(
					FrEntIdType			 in_EntityType, 
					const WNetInfoDatumVec& in_EntitiesToLookup, 
					FrPubKeyType			 in_keytype            // Type of key to lookup
					 ) throw(RZkException) ;

	FrKeyQry	GetKeyCache();
	ZkSocketDescriptor	GetKeyQuerySocket() ;

	// PURPOSE: For all the Aips download their keys if it isn't in our cache
	//			Keys of type FRCRYPT_KEYTYPE_CRYPTION and FRCRYPT_KEYTYPE_SIGNING are retrieved.
	//			Also downloads keys for all FRENT_TYPE_FMG entities.
	// PROMISE: To Save the latest state of the Key cache before returning
	//			To Get the FRENT_TYPE_AIP and FRENT_TYPE_FMG entities from the INiqs interface.
	// REQUIRE: That The INiqs return some entities for any downloading to occur
	//			That StartKeyQueryListeningSocket() be called previously
	void		DownLoadKeysIfRequired() throw(RZkException);


private:

	RKeyQry& operator=(const RKeyQry&);
	bool		StartKeyQueryListeningSocket();
	ZKSTD::string GetStringFromKeyType( const FrPubKeyType in_keytype ) const ;

	//CTOR member init
	ZkSocket			m_Socket;
	FrKeyQry			m_kq;

	DZkError			m_Error;

    WZkMutex            m_keymutex;
};

#endif // !defined(AFX_KeyQry_H__353DF902_E0FC_11D2_8E3D_00104B6AF5F7__INCLUDED_)
