// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "StdAfx.h"

#include "clientdefs.h"
#ifdef INTERNAL_BUILD

#include "RouteTester.h"
#include "INiqs.h"
#include "INym.h"
#include "IRouteManager.h"
#include "INymCentral.h"
#include "ICoreServerConnect.h"
#include "ZkService.h"

#include <test_commands.h>
#include <IZkConsoleMsg.h>
#include <fr_entid.h>
#include <client.h>
#include <ZKConfiguration.h>
#include <zksocket.h>
#include <algorithm>
#include <sstream>
#include <ZKConvert.h>
#include <WZkipClient.h>
#include <RZkException.h>

// Just a guess
#define	TIME_TO_WAIT_FOR_ROUTE 	45000
#define CONNECT_TIMEOUT			90

#ifdef WIN32

#include <clientwin32.h>	// for FRWINDOWNAME

#include <clientwin32WM.h>

#else

#	define SOCKET_ERROR -1

#endif


#ifndef ZK_LINUX
TestUnauthenticatedRoutesUnitTest g_testUnauthenticatedRoutesUnitTest;
TestCurrentNymSecuritySettingsUnitTest g_testCurrentNymSecuritySettingsUnitTest;
TestAuthenticatedRoutesWithKnownDownAipUnitTest g_testAuthenticatedRoutesWithKnownDownAipUnitTest;
SimulateAuthRouteDestroyUnitTest g_simulateAuthRouteDestroyUnitTest;
BringUpAllUnauthenticatedRoutesUnitTest g_bringUpAllUnauthenticatedRoutesUnitTest;
#endif

typedef struct ServerInfo 
{
	const FrEntIdType entityType;
	const int sockType;
	const int sockProtocol;
	const char* description;
	const bool bRouteBuildsIndirectly;

	ServerInfo(	FrEntIdType entType, 
				char* desc,
				bool bTheRouteBuildsIndirectly = true,
				int socType = SOCK_STREAM, 
				int socProtocol = IPPROTO_TCP) : 
			entityType(entType), 
			sockType(socType),
			sockProtocol(socProtocol),
			description(desc),
			bRouteBuildsIndirectly(bTheRouteBuildsIndirectly) { };

} ServerInfo;

ServerInfo serverInfo[] = {
								ServerInfo(FRENT_TYPE_NYM_SRV, "NYM server", false),
								ServerInfo(FRENT_TYPE_NIQS, "NIQS"),
								ServerInfo(FRENT_TYPE_KEYQRY_SRV, "KEY QRY server", true, SOCK_DGRAM, IPPROTO_UDP),
								ServerInfo(FRENT_TYPE_TOK_SRV, "TOKEN server", false),
								ServerInfo(FRENT_TYPE_POP, "POP Server"),
								ServerInfo(FRENT_TYPE_NMTA, "NMTA server")
								
						};

long TestCurrentNymSecuritySettingsUnitTest::Execute( void* )
{
	TestCurrentNymSecuritySettings();

	return 1;
}

void TestCurrentNymSecuritySettingsUnitTest::TestCurrentNymSecuritySettings()
{
	Port2AipIds mmapAuth;
	size_t numAuthRoutes;
	ZkService::GetIRouteManager()->GetAuthenticatedRoutes(mmapAuth, numAuthRoutes);
	
	if (ZkService::GetINymCentral()->IsCurrentNymUser()) {
		ZKASSERT(numAuthRoutes == 0);
	}
	else {
		ZKASSERT(numAuthRoutes == 1);

		INym* pNym;
		ZkService::GetINymCentral()->GetActiveNym( IID_INYM, (void**)&pNym ) ;
		ZKASSERT( pNym->GetNumberOfHops() == static_cast<int>(mmapAuth.size()) );
	}
}



long TestAuthenticatedRoutesWithKnownDownAipUnitTest::Execute( void* )
{
	TestAuthenticatedRoutesWithKnownDownAip();

	return 1;
}

void TestAuthenticatedRoutesWithKnownDownAipUnitTest::TestAuthenticatedRoutesWithKnownDownAip()
{
	const unsigned long dwNetwork = ZKConfiguration::GetChosenNetwork() ; 
	if(ZKConfiguration::PrayFreedomNetwork != dwNetwork){
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, "Need to be on religious network to run TestAuthenticatedRoutesWithKnownDownAip" );
		return;
	}
	size_t iNym = ZkService::GetINymCentral()->GetActiveNymIndex();
	if (iNym == USER_NYM_INDEX) {
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, "Need a nym selected to run TestAuthenticatedRoutesWithKnownDownAip test" );
		return;
	}
	else {
		RouteTesterHelper::TestAuthenticatedRoutesWithKnownDownAip(iNym, RULE_FIRSTHOP);
		RouteTesterHelper::TestAuthenticatedRoutesWithKnownDownAip(iNym, RULE_EXITFUNNEL);
		RouteTesterHelper::TestAuthenticatedRoutesWithKnownDownAip(iNym, RULE_AVOID);
	}		
}



long SimulateAuthRouteDestroyUnitTest::Execute( void* )
{
	SimulateAuthRouteDestroy();

	return 1;
}

void SimulateAuthRouteDestroyUnitTest::SimulateAuthRouteDestroy()
{
	if (ZkService::GetINymCentral()->IsCurrentNymUser()) 
	{
		IZkConsoleMsg::ShowMsgForceUpdate( 
						CONS_DEBUG, 
						CONSIMAGE_QUESTION, 
						IDS_CONS_JUST_STRING, 
						"Need a nym selected to run SimulateAuthRouteDestroy test" );		
		return;
	}

	ZkService::GetIRouteManager()->SimulateAuthenticatedRouteDestroy() ;

}

void RouteTesterHelper::TestAuthenticatedRoutesWithKnownDownAip(const size_t in_nNym, const RULEZ in_Rule)
{
	// Let's ensure we are on the right network
	const unsigned long dwNetwork = ZKConfiguration::GetChosenNetwork() ; 
	if(ZKConfiguration::PrayFreedomNetwork != dwNetwork){ 
		ZKASSERT(0); // tests only run on religious network
		return;
	}

	// Let's ensure a valid nym index was used
	const size_t nTotalNyms = ZkService::GetINymCentral()->GetNumberOfNyms();
	ZKTRACE2("You have [%d] Nyms (including User), we will use nym [%d] to do authenticated route tests\n", nTotalNyms, in_nNym);
	if(in_nNym > nTotalNyms-1 || in_nNym < 1){
		ZKASSERT(0); // You specified an invalid nym index
		return;
	}

	WNetInfoDatum Aip;
	
	ZKVERIFY(ApplyRuleToRandomAipForNym(in_nNym, Aip, in_Rule));
	ZKVERIFY(MarkAipUnavailable(Aip)); // assertion may indicate failure to connect to WEBNIQSTOOL, did you open 51125?

	// We know there shouldn't be a permanent route to the niqs, so we're asserting this assumption
	ZKVERIFY(!ZkService::GetIRouteManager()->GetUnauthenticatedRoute(FR_DFLT_NIQS_SERVER_PORT) );
	ZkService::GetINiqs()->UpdateNiqsInformation() ;

	// We know there shouldn't be a permanent route to the niqs, so we're asserting this assumption
	ZKVERIFY(!ZkService::GetIRouteManager()->GetUnauthenticatedRoute(FR_DFLT_NIQS_SERVER_PORT) );


	if(RULE_AVOID == in_Rule)
		MakeNymActive(in_nNym, BLOCKING);	
	else
		MakeNymActive(in_nNym, NONBLOCKING);	


	TestResultsOfRoutes(in_nNym, in_Rule, Aip);	
		

	ZKVERIFY(MarkAipAvailable(Aip)); // assertion may indicate failure to connect to WEBNIQSTOOL
}

void RouteTesterHelper::sleep( size_t in_milliTimeToSleep )
{
#ifdef WIN32
		Sleep(in_milliTimeToSleep); 
#else
        usleep(in_milliTimeToSleep / 10);
#endif
}

void RouteTesterHelper::TestResultsOfRoutes(const size_t in_nNym, const RULEZ in_Rule, const WNetInfoDatum& in_OriginalFirstAip)

{


#ifdef WIN32
	if(RULE_FIRSTHOP == in_Rule){	// Test 3.a 	
		sleep(2500);
		EnsureRandomRouteWindowIsActiveAndDismiss(IDYES);
		sleep(TIME_TO_WAIT_FOR_ROUTE);
		ZKASSERT( !ZkService::GetINymCentral()->IsCurrentNymUser() );	// if you assert, try selecting in_nNym normally & see if he's actually working
		if( !ZkService::GetINymCentral()->IsCurrentNymUser() )
			VerifyRegions(in_OriginalFirstAip);
	}

	if(RULE_EXITFUNNEL == in_Rule){	// Test 3.d 
		sleep(2500);
		EnsureRandomRouteWindowIsActiveAndDismiss(IDYES);
		sleep(TIME_TO_WAIT_FOR_ROUTE);
		ZKASSERT( !ZkService::GetINymCentral()->IsCurrentNymUser() );
	}

	if(RULE_AVOID == in_Rule)		// Test 3.g
		ZKASSERT( !ZkService::GetINymCentral()->IsCurrentNymUser() ); // if you assert, try selecting in_nNym normally & see if he's actually working

#else

	ZKASSERT( false ) ; // Not ported

#endif
}

void RouteTesterHelper::EnsureRandomRouteWindowIsActiveAndDismiss(const int in_nDismissMode)

{
#ifdef WIN32
	HWND hUseRandomRouteWindow = GetLastActivePopup ( FindWindow(NULL, FRWINDOWNAME) );

	ZKASSERT(hUseRandomRouteWindow);

	if(hUseRandomRouteWindow){
		::SendMessage( hUseRandomRouteWindow, WM_COMMAND, in_nDismissMode, 0);
	}

#else

	ZKASSERT( false ) ;

#endif
}

void RouteTesterHelper::VerifyRegions(const WNetInfoDatum& in_FirstAip)
{
	// Get the region from the current first hop (if the nym selection worked)
	Port2AipIds mmapAuth;
	size_t numAuthRoutes = 0;
	ZkService::GetIRouteManager()->GetAuthenticatedRoutes(mmapAuth, numAuthRoutes);
	ZKASSERT(1 == numAuthRoutes);
	Cit_Port2AipIds it = mmapAuth.begin();
	WNetInfoDatum CurrentFirstAip;
	ZKVERIFY(ZkService::GetINiqs()->GetEntityById((*it).second, CurrentFirstAip));
	ZKVERIFY( CheckIfAipIsInRegionAsRules(in_FirstAip, CurrentFirstAip) );
}

bool RouteTesterHelper::CheckIfAipIsInRegionAsRules(const WNetInfoDatum& in_FirstAip, const WNetInfoDatum& in_CurrentFirstAip)
{
	ZKSTD::string strCountryCode;
	in_FirstAip.GetCountryCode(strCountryCode);
	ZKSTD::string strProvinceCode;
	in_FirstAip.GetProvinceCode(strProvinceCode);

	ZKSTD::string strCurrentCountryCode;
	in_FirstAip.GetCountryCode(strCurrentCountryCode);
	ZKSTD::string strCurrentProvinceCode;
	in_FirstAip.GetProvinceCode(strCurrentProvinceCode);

	return ( (strCountryCode == strCurrentCountryCode) && (strProvinceCode == strCurrentProvinceCode) );
}


bool RouteTesterHelper::ApplyRuleToRandomAipForNym(const unsigned int in_nymIndex, WNetInfoDatum& out_aipInfo, const RULEZ in_RuleToApply) 
{
	bool bRet = false;

	INym* pNym = NULL;
	ZkService::GetINymCentral()->GetNym(IID_INYM, (void**)&pNym, in_nymIndex) ;

	if (pNym) {
	
		WNetInfoDatumVec allAvailableAips;
		ZkService::GetINiqs()->GetServersOfType( FRENT_TYPE_AIP, allAvailableAips, false ) ;

		// Remove the niqs' from the vector
		// (we don't want to bring down those aips since we're using the niqs for some tests)
		// use the ip to match up aips with Niqs
		WNetInfoDatumVec allNiqs;
		ZkService::GetINiqs()->GetServersOfType( FRENT_TYPE_NIQS, allNiqs, false ) ;
		for(WNetInfoDatumVec::const_iterator itNiqs = allNiqs.begin(); itNiqs != allNiqs.end(); ++itNiqs){

			ZKSTD::string	strName;
			pNym->GetNymName(strName);

			ZKSTD::string	strNiqsName;
			(*itNiqs).GetEntityName(strNiqsName);

			ZKTRACE3("For Nym [%s] looking for aip corresponding to niqs [%s] [%d]\n", strName.c_str(), strNiqsName.c_str(), (*itNiqs).GetId() );
			
			SOCKADDR_IN NiqsIP = (*itNiqs).GetSockAddr();
			bool foundCorrespondingAip = false;

			for(WNetInfoDatumVec::iterator itAips = allAvailableAips.begin(); itAips != allAvailableAips.end(); ++itAips){
				
				SOCKADDR_IN AipIP  = (*itAips).GetSockAddr();

				if(0 == memcmp(&AipIP.sin_addr, &NiqsIP.sin_addr, 4) ){
					ZKSTD::string strAipName;
					(*itAips).GetEntityName(strAipName);
					ZKTRACE2("Found it, not gonna down Niqs aip [%s] [%d]\n", strAipName.c_str(), (*itAips).GetId() );
					allAvailableAips.erase(itAips);
					foundCorrespondingAip = true;
					break;
				}
			}
			ZKASSERT(foundCorrespondingAip);
		}

		ZKSTD::random_shuffle(allAvailableAips.begin(), allAvailableAips.end());
		WNetInfoDatumVec::iterator it = allAvailableAips.begin();
		size_t randomAipID = (*it).GetId();
		out_aipInfo = (*it);

		MapRules rules;
		rules.insert( MapRules::value_type(randomAipID, in_RuleToApply) );

		pNym->SetRouteRulesOfNym(rules);
		bRet = true;
	}
	
	return bRet ;
}

bool RouteTesterHelper::MarkAipUnavailable(const WNetInfoDatum& in_aipInfo)
{
	ZKSTD::list<ZKSTD::string> aipsToSetAvailable;
	ZKSTD::list<ZKSTD::string> aipsToSetUnavailable;

	ZKSTD::string aipName;
	in_aipInfo.GetEntityName(aipName);
	aipsToSetUnavailable.push_back(aipName);

	ZKSTD::ostringstream ostr;
	ostr << "Marking aip " << aipName << " as unavailable";

	IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, ostr.str().c_str() );
	
	return ChangeAvailabiliyOfAips(aipsToSetAvailable, aipsToSetUnavailable);
}

bool RouteTesterHelper::MarkAipAvailable(const WNetInfoDatum& in_aipInfo)
{
	ZKSTD::list<ZKSTD::string> aipsToSetAvailable;
	ZKSTD::list<ZKSTD::string> aipsToSetUnavailable;

	ZKSTD::string aipName;
	in_aipInfo.GetEntityName(aipName);
	aipsToSetAvailable.push_back(aipName);

	ZKSTD::ostringstream ostr;
	ostr << "Marking aip " << aipName << " as available";

	IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, ostr.str().c_str() );

	
	return ChangeAvailabiliyOfAips(aipsToSetAvailable, aipsToSetUnavailable);
}

bool RouteTesterHelper::MakeNymActive(unsigned int in_nymIndex, const BLOCKING_MODE in_Blocking)
{
	bool bRet = true;
	if (in_nymIndex >= ZkService::GetINymCentral()->GetNumberOfNyms()) {
		bRet = false;
	}
	else {

#ifdef WIN32
		HWND hWnd = ::FindWindow( NULL, "FreedomTestFramework" );	

		ZKASSERT(hWnd);
		if(hWnd){
			if(NONBLOCKING == in_Blocking)
				ZKVERIFY(::PostMessage(hWnd, WM_TF_RUN_TEST, PRIV_SELECT_NYM, in_nymIndex ));
			else
				ZKVERIFY(::SendMessage(hWnd, WM_TF_RUN_TEST, PRIV_SELECT_NYM, in_nymIndex  ));
		}
#else
		// Must port this test
		ZKASSERT( 0 ) ;
#endif
	}
	ZKASSERT(bRet); // nym out of range
	return bRet;
}

bool RouteTesterHelper::ChangeAvailabiliyOfAips(
	const ZKSTD::list<ZKSTD::string>& in_aipsToSetAvailable, 
	const ZKSTD::list<ZKSTD::string>& in_aipsToSetUnavailable )
{
	bool bReturn = false;

	//Requests from outside zks.net will be refused by web server

	ZkSocket sock = NULL;
	ZKVERIFY( ::zkSocketNew( NULL, AF_INET, SOCK_STREAM, IPPROTO_TCP, &sock ) );
	ZKASSERT( sock );

	const int bufLen = 1024;
	// Copy ZKSTD::string into temporary buffer
	char szSendBuffer[bufLen] = { 0	};
	const ZKSTD::string strRequest = "GET /fradmin/cgi-bin/niqstool.cgi?" +
		CreateStringUsingParameter( "setAipAvailable", in_aipsToSetAvailable ) +
		CreateStringUsingParameter( "setAipUnavailable", in_aipsToSetUnavailable ) +
		" HTTP/1.0\r\n\r\n";
	const ZKSTD::string::size_type nCopied = strRequest.copy( szSendBuffer, bufLen );
	ZKASSERT( nCopied == strRequest.size() );

	ZKTRACE1( "Sending %s to WEBNIQSTOOL\n", szSendBuffer );

	char szReceptionBuf[bufLen] = { 0 };
	uint32_t numBytesWritten = 0;
	uint32_t numBytesReceived = 0;

	IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, 
										CONSIMAGE_QUESTION, 
										IDS_CONS_JUST_STRING, 
										"Connecting to WEBNIQSTOOL" );

	if( !::zkSocketConnect( NULL, sock, ( ZKSTD::string( WEBNIQSTOOL ) + ":51125" ).c_str() ) )
	{
		ZKASSERT( 0 );
	}
	else if( !::zkSocketWrite( NULL, sock, szSendBuffer, ::strlen( szSendBuffer ), &numBytesWritten ) )
	{
		ZKASSERT( 0 );
	}
	else if( !::zkSocketReadAll( NULL, sock, szReceptionBuf, bufLen ) )
	{
		// Following may assert if getting a response is longer than the above recv timeout of 69 seconds
		ZKASSERT( 0 );
	}
	else 
	{
		const char* pOK = strstr( szReceptionBuf, "200" );
		bReturn = ( pOK != NULL ) && ( numBytesReceived > 10 );
	}

	::zkSocketFree( NULL, &sock );

	return bReturn;
}

ZKSTD::string RouteTesterHelper::CreateStringUsingParameter(
							const ZKSTD::string& in_parameter, 
							const ZKSTD::list<ZKSTD::string>& in_aips)
{
	ZKSTD::string retStr;
	
	for(ZKSTD::list<ZKSTD::string>::const_iterator it = in_aips.begin();
		it != in_aips.end(); 
		++it) {

		retStr += in_parameter + "=" + (*it) + "&";
	}

	ZKTRACE1("RouteTesterHelper::CreateStringUsingParameter returning [%s]\n", retStr.c_str());

	return retStr;
}

void
RouteTesterHelper::BuildUnauthenticatedRouteWithCoreServerConnect( int in_indexServerInfo )
{
	IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, 
										  serverInfo[in_indexServerInfo].description );

	if (ZkService::GetIRouteManager()->GetUnauthenticatedRoute(
			ZKConvert::GetPortFromEntityType(serverInfo[in_indexServerInfo].entityType)))
	{
		// We already have a route, this test doesn't really work
		ZKTEST( false ) ; 
	}
	else
	{
		try
		{
			WZkipClient client;
			// This throws on failure
			ZkService::GetICoreServerConnect()->SmartConnectThroughCloud(
				serverInfo[in_indexServerInfo].entityType, client, CONNECT_TIMEOUT);
		
			ZKTEST(ZkService::GetIRouteManager()->GetUnauthenticatedRoute(
				ZKConvert::GetPortFromEntityType(serverInfo[in_indexServerInfo].entityType)));
		}
		catch (IZkException&)
		{
			ZKTEST(false);
		}
	}
}

void 
RouteTesterHelper::BuildUnauthenticatedRouteIndirectly( int in_indexServerInfo )
{
	
	SOCKET sock = NULL ;

	unsigned int routeID = INVALID_ROUTE_ID;

	WNetInfoDatum server;
	ZkService::GetINiqs()->GetServer(serverInfo[in_indexServerInfo].entityType, server);

	SOCKADDR_IN sockAddr = server.GetSockAddr();

	if (ZkService::GetIRouteManager()->GetUnauthenticatedRoute(ntohs(sockAddr.sin_port), &routeID)) 
	{
		ZKASSERT( routeID != INVALID_ROUTE_ID);
		// We already have a route, this test doesn't really work
		ZKTEST( false ) ; 
		return ;
	}

	ZKASSERT(routeID == INVALID_ROUTE_ID);

	//Do a connnect  -> This should create the unauth route 
	sock = ::socket(
				AF_INET, 
				serverInfo[in_indexServerInfo].sockType, 
				serverInfo[in_indexServerInfo].sockProtocol);

	if (sock == INVALID_SOCKET)
	{
		ZKTEST(false);
		return ;
	}	

	IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, 
										  serverInfo[in_indexServerInfo].description );

	if (serverInfo[in_indexServerInfo].sockType == SOCK_DGRAM) 
	{
		const int bufLen = 10;
		char buf[bufLen] = {0};

		if (SOCKET_ERROR == ::sendto(sock, buf, bufLen, 0, (const struct sockaddr*)&sockAddr, sizeof(sockAddr))) 
		{
			ZKTEST(false);
		}
		else 
		{
			sleep( TIME_TO_WAIT_FOR_ROUTE ) ;
			
			// Following may assert if you bypassed loading the driver (or you don't yet have a
			// driver for your os)
			ZKVERIFY(ZkService::GetIRouteManager()->GetUnauthenticatedRoute(
														ntohs(sockAddr.sin_port)));
		}
	}
	else if (serverInfo[in_indexServerInfo].sockType == SOCK_STREAM) 
	{
		if (SOCKET_ERROR == ::connect(sock, (const struct sockaddr*)&sockAddr, sizeof(sockAddr))) 
		{
			// Following may assert if there are no routes available, bug 1979 fixed?
			//
			// Also, you need enough aips to make this work, here's a real-life example
			// Faith	<exit>
			// Truth
			// Temperence
			// Jesus	<down>
			//
			// Each one of these aips needs to be cross-connected for these tests to work. For example,
			// With Faith as <exit> and Jesus as <down> there are no routes (server side bug)
			// *BUT* on the other hand with Truth or Temperence as <exit> there is one route!

			//I replaced the assert with a trace because the
			//assert really screws up the timing of the code and route create
			//failures are a reality of life
			//ZKTEST(false);
			ZKTRACE0( "RouteTester Unit test - connect failed, possibly because we weren't able to build route\n" );
		}
		else 
		{

#ifdef WIN32
			ZKVERIFY(shutdown(sock, SD_BOTH) != SOCKET_ERROR);

#else

			// What is the purpose of the call above?

			ZKASSERT( false ) ;

#endif
			sleep(2000);
			// Following may assert if you bypassed loading the driver (or you don't yet have a
			// driver for your os)
			ZKVERIFY(ZkService::GetIRouteManager()->GetUnauthenticatedRoute(
														ntohs(sockAddr.sin_port)));
		}
	}


#ifdef WIN32

	SAFECLOSESOCKET(sock);

#else

	::close(sock);		

#endif


}

void 
RouteTesterHelper::BuildUnauthenticatedRouteDirectly( int in_indexServerInfo )
{
	unsigned int routeID = INVALID_ROUTE_ID;

	WNetInfoDatum server;
	ZkService::GetINiqs()->GetServer(serverInfo[in_indexServerInfo].entityType, server);

	SOCKADDR_IN sockAddr = server.GetSockAddr();

	if (ZkService::GetIRouteManager()->GetUnauthenticatedRoute(ntohs(sockAddr.sin_port), &routeID)) 
	{
		ZKASSERT( routeID != INVALID_ROUTE_ID);
		// We already have a route, this test doesn't really work
		return ;
	}

	if (ZkService::GetIRouteManager()->StartUnauthRoute( ntohs( sockAddr.sin_port ) ) )
	{
		ZKVERIFY(ZkService::GetIRouteManager()->GetUnauthenticatedRoute(
												ntohs(sockAddr.sin_port)));
	}
	else
	{	
		// Route Creation can fail.  Its a fact of life.  The network
		// may be busy, etc etc.  However if this
		// fails often there could be something wrong, like a network
		// poorly configured.

//		ZKTEST( false ) ;
		ZKTRACE0( "RouteTester Unit test - Build Unauth Route Failed\n" );
	}
}


// Helper function - it is not a failure if the route doesn't
// actually work
void
RouteTesterHelper::TearDownUnauthenticatedRoute( int in_indexServerInfo )
{
	WNetInfoDatum server;
	ZkService::GetINiqs()->GetServer(serverInfo[in_indexServerInfo].entityType, server);

	SOCKADDR_IN sockAddr = server.GetSockAddr();

	
	unsigned int routeID = INVALID_ROUTE_ID;
	if ( ZkService::GetIRouteManager()->GetUnauthenticatedRoute(
											ntohs(sockAddr.sin_port), 
											&routeID) ) 
	{
		// There is a route of this type
		ZKASSERT( routeID != INVALID_ROUTE_ID ) ;

		ZkService::GetIRouteManager()->TerminateUnauthRoute( ntohs(sockAddr.sin_port) ) ;

		// Verify that it is really gone
		ZKTEST( !ZkService::GetIRouteManager()->GetUnauthenticatedRoute(
											ntohs(sockAddr.sin_port)) );
	}
}



long BringUpAllUnauthenticatedRoutesUnitTest::Execute( void* )
{
	IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, "Attempting to build all unauthenticated routes" );
	BringUpAllUnauthenticatedRoutes();
	IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, "Completed building of unauthenticated routes" );

	return 1;
}

void
BringUpAllUnauthenticatedRoutesUnitTest::BringUpAllUnauthenticatedRoutes()
{
	try
	{
		for (unsigned int i = 0; i < (sizeof(serverInfo) / sizeof(serverInfo[0])); ++i) 
		{		
			RouteTesterHelper::BuildUnauthenticatedRouteDirectly( i ) ;
		}	
	}
	catch( IZkException& )
	{
		ZKTEST( false ) ;
	}
}



long TestUnauthenticatedRoutesUnitTest::Execute( void* )
{
	TestUnauthenticatedRoutes();

	return 1;
}

void
TestUnauthenticatedRoutesUnitTest::TestUnauthenticatedRoutes()
{
	try
	{
		for (unsigned int i = 0; i < (sizeof(serverInfo) / sizeof(serverInfo[0])); ++i) 
		{		
			// Tear down any existing route

			RouteTesterHelper::TearDownUnauthenticatedRoute( i ) ;

			RouteTesterHelper::BuildUnauthenticatedRouteDirectly( i ) ;

			RouteTesterHelper::TearDownUnauthenticatedRoute( i ) ;
		}


		for (i = 0; i < (sizeof(serverInfo) / sizeof(serverInfo[0])); ++i) 
		{
			if (serverInfo[i].bRouteBuildsIndirectly)
			{
				RouteTesterHelper::BuildUnauthenticatedRouteIndirectly( i ) ;
			}
			else
			{
				RouteTesterHelper::BuildUnauthenticatedRouteWithCoreServerConnect( i );
				// Corey- 01/16/2001: This sleep is required because disconnecting a socket
				// can cause packetsto be sent on that socket even after the disconnect
				// returns. Weird, eh?  Talk to Oliver to learn why.
				RouteTesterHelper::sleep(1500);
			}
			RouteTesterHelper::TearDownUnauthenticatedRoute( i ) ;
		}

	}
	catch( IZkException& )
	{
		ZKTEST( false ) ;
	}
}


#endif	// INTERNAL_BUILD
