// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// ZkServiceUnitTest.cpp

#include "StdAfx.h"
#include "clientdefs.h"
#include "ZKServiceUnitTest.h"

#include "IWordScan.h"
#include "IFirewall.h"
#include "IClientCentral.h"
#include "ICookieManager.h"
#include "INymMail.h"
#include "INymWallet.h"
#include "INymCookieMgr.h"
#include "INymCentral.h"
#include "INym.h"
#include "INiqs.h"
#include "IKeyQry.h"
#include "IScriptServer.h"
#include "ZkService.h"
#include "ICrypto.h"
#include "ITokenService.h"
#include "IClientPersist.h"
#include "IAdBlocker.h"
#include "RActiveRouteTable.h"
#include "INetworkPerformance.h"

// Marcp: this need to be fixed
#ifdef WIN32
#include "../win/ZKBaseUI.h"
#endif

//#include <file_hlp.h>
#include <IZkRegistry.h>
#include <IZkFileHelp.h>
#include <ZKFileConfiguration.h>
#include <ZkErrorHandler.h>
#include <ZKTestScalper.h>
#include <IZKErrorReporter.h>
#include <RZkException.h>
#include <IZkConsoleMsg.h>
#include <WMaipRb.h>
#include <test_registryKeys.h>
#include <ZKConvert.h> 
#include <frLocalAip.h>
#include <WNymIconInfo.h>
#include <zkDNS.h>

#include <fstream>
#include <sstream>
#include <set>
#include <math.h>

#ifdef ZK_LINUX
#include <netdb.h> // for hostent
#endif

// We don't include test code in our externally released binaries
#ifdef INTERNAL_BUILD

//#define SHOW_OFFENSIVE_WORD_DIALOGS	


#ifndef ZK_LINUX

// These globals force the unit tests objects to be created
// and hense registered.  Unfortunately we're having trouble
// with linking on linux (the objects are never created) so
// these defines have been moved to the freedomclient project.
// This isn't so elegant so for the moment 
// we leave them here for other platforms. (Andrew June 2000)
ZkServiceUnitTest g_zkServiceUnitTest;
RedeemAndCreateNymsUnitTest g_redeemAndCreateNymsUnitTest;
FreedomSetupUnitTest g_freedomSetupUnitTest;
CreateNymUnitTest g_createNymUnitTest;
FirewallUnitTest g_firewallUnitTest;
PersistStressUnitTest g_PersistStressTest;


#endif

long ZkServiceUnitTest::Execute( void* )
{
	size_t nIndex = ZkService::GetINymCentral()->GetActiveNymIndex() ;

	if ( nIndex == USER_NYM_INDEX ) 
	{
		ZkServiceUnitTest::TestsWithUserNymActive() ;
	}
	else 
	{
		ZkServiceUnitTest::TestsWithNymActive() ;
	}
	return 1;
}

void 
ZkServiceUnitTest::TestsWithUserNymActive()
{
	// First make sure Freedom is in correct situation for
	// these tests 

	if ( !ZkService::GetINymCentral()->IsCurrentNymUser() ) 
	{
		ZKTEST( false ) ; // Freedom is not in correct state for this test
		return ;
	}
	
	TestINymWallet();
	TestINiqs();
	TestITokenService() ;

	TestRActiveRouteTable() ;

	TestRNetworkPerformance() ;

	TestIClientCentral();

	TestIScriptServer();

	TestIWordScan();
}

void ZkServiceUnitTest::TestsWithNymActive()
{	
	// First make sure Freedom is in correct situation for
	// these tests 
	
	if ( ZkService::GetINymCentral()->IsCurrentNymUser() ) 
	{
		ZKTEST( false ) ; // Freedom is not in correct state for this test
		return ;
	}

	TestINymWallet();
	TestICookiesManager() ;
	TestINymCookieMgr() ;
	TestINiqs() ;
	TestINym() ;
	TestINymCentral() ;

//  Reply blocks are being phased out, so no use running this code with each unit test
//	TestINymMail() ;

	TestIAdBlocker();

	TestRNetworkPerformance() ;

	TestIClientCentral();

	TestIScriptServer();

	TestIWordScan();
}

void ZkServiceUnitTest::TestIAdBlocker()
{
#ifndef ZK_LINUX
	ZkService::GetIAdBlocker()->SelfTest();
#else
#warning "Adblocker unit test not called until Adblocker completely ported"
#endif
}


void ZkServiceUnitTest::TestICookiesManager()
{	
	/*
	if ( ZkService::GetINymCentral()->IsCurrentNymUser() ) 
	{
		ZKTEST( false ) ; // Freedom is not in correct state for this test
		return ;
	}
	*/
	  
	INym* pNym = NULL ;
	ZKTEST( ZkService::GetINymCentral()->GetActiveNym( IID_INYM, (void**) &pNym ) ) ;

	ZKSTD::string strNymName ;
	ZKTEST( pNym->GetNymName( strNymName ) );
	
	ZKTEST( strNymName.length() > 0 ) ;

	bool AreCookiesEnabledForThisNym = pNym->GetCookiesEnabled() ;

	if ( !AreCookiesEnabledForThisNym ) 
	{
		// We should clearly define what this 
		// state means in terms of the expected behavior of the nym and
		// then verify those in this test

		return ;
	}

	ZkService::GetICookieManager()->SelfTest() ;

	// Most accurate testing can occurs at the INymCookieMgr
	// level.  This interface at the moment is straightforward
}

void
ZkServiceUnitTest::TestINymCookieMgr()
{
	INymCookieMgr* pNymCookieMgr  = NULL;
	INymCookieMgr* pNymCookieMgr2 = NULL;

	INym* pNym = NULL;

	ZkService::GetINymCentral()->GetActiveNym(IID_INYMCOOKIEMGR, (void**)&pNymCookieMgr);
	ZkService::GetINymCentral()->GetActiveNym(IID_INYM, (void**)&pNym);

	ZKTEST( pNym != NULL ) ;
	ZKTEST( pNymCookieMgr != NULL ) ;

	pNym->QueryInterface(IID_INYMCOOKIEMGR, (void**)&pNymCookieMgr2) ;

	ZKTEST( pNymCookieMgr2 == pNymCookieMgr ) ;


	// User Interface Part of the API

	COOKIEJARMAP Cookies ;

	// Flush any existing cookies with empty list
	ZKTEST(pNymCookieMgr->SetNymCookieMap( Cookies ) );

	// Verify empty
	ZKTEST( pNymCookieMgr->GetNymCookieMap( Cookies ) ) ;
	ZKTEST( Cookies.size() == 0 ) ;


	// Test data goes into map

	DomainAndDate DandD ;

	time_t lCUTTime ;

	time(&lCUTTime);
	ZKSTD::string DateRecd (ctime(&lCUTTime)); // changes to use local time

	time_t lExpiryTime = lCUTTime + 1000 ;
	ZKSTD::string DateExpire (ctime(&lExpiryTime)); // changes to use local time

	DandD.DateRecd = DateRecd.substr(0, 24); // ditch the '\n\0' ;
	DandD.DateExpire = DateExpire.substr(0, 24);;
	DandD.SecondsExpire = lExpiryTime ;		
	
	DandD.Domain = "the_matrix.com" ;

	Cookies.insert(COOKIEJARMAP::value_type(ZKSTD::string( "Test Item=Test Data" ), DandD ) ) ;

	ZKTEST( pNymCookieMgr->SetNymCookieMap( Cookies ) );

	Cookies.clear() ;
	ZKTEST( pNymCookieMgr->GetNymCookieMap( Cookies ) );

	ZKTEST( Cookies.size() == 1 ) ;

	// Now test HTTP interface

	const ZKSTD::string Cookie1 = "id=xxx;path=/;domain=test.domain.com;expires=Wednesday, 09-Nov-2003 23:12:40 GMT";
	const ZKSTD::string Cookie2 = "id2=yo;path=/;domain=zkstestdomain.com;expires=Tue, 21 Aug 2017 09:20:21 GMT";
	const ZKSTD::string SessionCookie = "some_id=434xxsgdsdx;path=/;domain=zkstestdomain.com";

	// TODO: verify #1599 by pumping in a ctime(&lExpiryTime) value like above

	const ZKSTD::string strExpiry1 = "Wednesday, 09-Nov-2003 23:12:40 GMT" ;
	const ZKSTD::string TestDomain( "zkstestdomain.com" ) ; 
	const ZKSTD::string DomainInsideCookie( "test.domain.com" ) ; 

	pNymCookieMgr->AddCookie( 
					Cookie1,
					DomainInsideCookie ) ;

	pNymCookieMgr->AddCookie( 
					Cookie2,
					TestDomain ) ;

	pNymCookieMgr->AddCookie( 
					SessionCookie,
					TestDomain ) ;

	ZKSTD::string strCookiesForTestDomain ;

	// Now ask for those cookies we just set back
	ZKTEST( pNymCookieMgr->BuildCookieField( TestDomain, strCookiesForTestDomain ) );

	ZKTEST( !strCookiesForTestDomain.empty() ) ;
	
	strCookiesForTestDomain.clear() ;
	ZKTEST( pNymCookieMgr->BuildCookieField( DomainInsideCookie, strCookiesForTestDomain ) );
	ZKTEST( !strCookiesForTestDomain.empty() ) ;

	// Tell ICookieManager to test itself

	pNymCookieMgr->SelfTest() ;

}


void ZkServiceUnitTest::TestINymMail()
{
	// This was testing reply block stuff
}

void 
ZkServiceUnitTest::TestINym()
{
	if ( ZkService::GetINymCentral()->IsCurrentNymUser() ) 
	{
		ZKTEST( false ) ; // Freedom is not in correct state for this test
		return ;
	}
	
	try 
	{
		INym* pNym = NULL ;
		ZKTEST( ZkService::GetINymCentral()->GetActiveNym( IID_INYM, (void**) &pNym ) ) ;

#if 0
		// ROUTE RULES ARE REMOVED FROM THIS VERSION, CLEAN THIS CODE

		MapRules Rules ;

		// Reset any rules
		pNym->SetRouteRulesOfNym( Rules ) ;

		pNym->GetRouteRulesOfNym( Rules ) ;
	
		ZKTEST( Rules.empty() ) ;

		// now fill in some rules - we need some valid Aip ids

		WNetInfoDatumVec VectorOfAips ;
		ZkService::GetINiqs()->GetServersOfType( FRENT_TYPE_AIP, VectorOfAips, false ) ;

		WNetInfoDatumVec::iterator it = VectorOfAips.begin();
		WNetInfoDatum data ;
	
		uint32_t FirstHopId = (*it).GetId() ;
		++it ;
		uint32_t AvoidHopId = (*it).GetId() ;

		Rules.insert( MapRules::value_type( FirstHopId, RULE_FIRSTHOP ) ) ;
		Rules.insert( MapRules::value_type( AvoidHopId, RULE_AVOID ) ) ;
				
		pNym->SetRouteRulesOfNym( Rules ) ;

		Rules.clear() ;

		pNym->GetRouteRulesOfNym( Rules ) ;

		int cntFoundRules = 0;
		for(MapRules::const_iterator itMap = Rules.begin(); itMap != Rules.end(); ++itMap)
		{
			// We shouldn't care or assume about what order we get the map back in
			if ( (*itMap).first == FirstHopId ) 
			{
				ZKASSERT( (*itMap).second == RULE_FIRSTHOP ) ;
				cntFoundRules++ ;
			}
			else if ( (*itMap).first == AvoidHopId ) 
			{
				ZKASSERT( (*itMap).second == RULE_AVOID ) ;
				cntFoundRules++ ;
			}
			else 
			{
				// What else did we find?
				ZKASSERT( false ) ;
			}			
		}

		// Make sure we found all the rules that we set
		ZKASSERT( cntFoundRules == 2 ) ;
#endif

	}
	catch( IZkException& ex ) 
	{
		// Something went wrong
		ZKTEST( ZK_FALSE ) ;

		ZKTRACE0( "INym test failed\n" ) ;
		ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
	}
}

void 
ZkServiceUnitTest::TestINiqs()
{
		
	// Examine the main servers

	// Retrieving information for all the servers makes sure
	// that we actually have information for all of them

	try 
	{
		ZKTRACE0 ( "\n\n\n\n BEGIN NIQS SERVICE TEST CODE \n\n\n\n" ) ;

		WNetInfoDatumVec vecAips ;
		ZkService::GetINiqs()->GetServersOfType( FRENT_TYPE_AIP, vecAips, true ) ;

		ZKTEST( vecAips.size() > 0 ) ;


		ZKTRACE0( "\n\n===============================AIP LIST BEGINS==========================\n\n " ) ;

		WNetInfoDatumVec::const_iterator iter;

		for (iter = vecAips.begin();
			iter != vecAips.end();
			++iter)
		{
			const WNetInfoDatum& Info = *iter ;
			Info.DumpToDebugOutput() ;
		}

		
		// When there are multiple servers we can get
		// a random one from this call.  Difficult to know how exactly to test this
		WNetInfoDatum server ;

		ZkService::GetINiqs()->GetServer( FRENT_TYPE_NYM_SRV, server ) ;
		ZkService::GetINiqs()->GetServer( FRENT_TYPE_TOK_SRV, server ) ;
		ZkService::GetINiqs()->GetServer( FRENT_TYPE_NIQS, server ) ;
		ZkService::GetINiqs()->GetServer( FRENT_TYPE_POP, server ) ;


	}
	catch( IZkException& ex ) 
	{
		ZKTEST( ZK_FALSE ) ;
		ZKTRACE0( "NIQS test failed\n" ) ;
		ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
	}
}

void ZkServiceUnitTest::TestINymCentral()
{
	//
}

void ZkServiceUnitTest::TestIWordScan()
{
	ZKSTD::string strPreSanitize("");
	ZKSTD::string strPostSanitize;
	strPostSanitize = ZkService::GetIWordScan()->Sanitize(strPreSanitize);
	ZKTEST(strPreSanitize == strPostSanitize);

	// Ensure if nothing should be replaced, nothing is.
	strPreSanitize = "There is hopefully not a nym name in this sentence.";
	strPostSanitize = ZkService::GetIWordScan()->Sanitize(strPreSanitize);
	ZKTEST(strPreSanitize == strPostSanitize);

	// Ensure the user nym name is not replaced
	ZKSTD::string strNymName;
	ZkService::GetINymCentral()->GetNymName(USER_NYM_INDEX, strNymName);
	strPreSanitize = strNymName + "boo";	
	strPostSanitize = ZkService::GetIWordScan()->Sanitize(strPreSanitize);
	ZKTEST(strPreSanitize == strPostSanitize);

	const char STR_NYM_REPLACEMENT[] = "nym";
	if (ZkService::GetINymCentral()->GetNumberOfNyms() > 1)
	{
		size_t nIndex = 1;
		ZkService::GetINymCentral()->GetNymName(nIndex, strNymName);

		// Ensure the nym name is replaced if it appears first
		strPreSanitize = strNymName + "boo ";
		strPostSanitize = ZkService::GetIWordScan()->Sanitize(strPreSanitize);
		ZKSTD::ostringstream oss;
		oss << STR_NYM_REPLACEMENT << nIndex << "boo ";
		ZKTEST(strPostSanitize == oss.str());

		// Ensure the nym name is replaced if it appears last
		strPreSanitize = ZKSTD::string("boo ") + strNymName;
		strPostSanitize = ZkService::GetIWordScan()->Sanitize(strPreSanitize);
		oss.rdbuf()->str("");
		oss << "boo " << STR_NYM_REPLACEMENT << nIndex;
		ZKTEST(strPostSanitize == oss.str());

		// Ensure the nym name is replaced if it appears in the middle
		strPreSanitize = "boo " + strNymName + "you";
		strPostSanitize = ZkService::GetIWordScan()->Sanitize(strPreSanitize);
		oss.rdbuf()->str("");
		oss << "boo " << STR_NYM_REPLACEMENT << nIndex << "you";
		ZKTEST(strPostSanitize == oss.str());

		// Ensure the nym name is replaced if it appears multiple times
		strPreSanitize = "boo " + strNymName + strNymName + "you" + strNymName;
		strPostSanitize = ZkService::GetIWordScan()->Sanitize(strPreSanitize);
		oss.rdbuf()->str("");
		oss << "boo " << STR_NYM_REPLACEMENT << nIndex  
			<< STR_NYM_REPLACEMENT << nIndex << "you" << STR_NYM_REPLACEMENT << nIndex;
		ZKTEST(strPostSanitize == oss.str());
	}

	if (ZkService::GetINymCentral()->GetNumberOfNyms() > 2)
	{
		size_t nNym1 = 1;
		size_t nNym2 = 2;
		ZKSTD::string strNymName1;
		ZKSTD::string strNymName2;
		ZkService::GetINymCentral()->GetNymName(nNym1, strNymName1);
		ZkService::GetINymCentral()->GetNymName(nNym2, strNymName2);

		// Ensure the nym names are replaced if multiple nym names appear
		strPreSanitize = strNymName1 + "boo " + strNymName2;
		strPostSanitize = ZkService::GetIWordScan()->Sanitize(strPreSanitize);
		ZKSTD::ostringstream oss;
		oss << STR_NYM_REPLACEMENT << nNym1 << "boo " << STR_NYM_REPLACEMENT << nNym2;
		ZKTEST(strPostSanitize == oss.str());

	}

	if (USER_NYM_INDEX == ZkService::GetINymCentral()->GetActiveNymIndex())
	{
		// The below tests were written to run with a nym active.
		return;
	}
	ZKSTD::vector<ZKSTD::string> vecFound ;
	WordScanSummary          scanResult ;
	scanResult.pFoundWords = &vecFound ;

	try 
	{
		// Word Scanning feature

		// Remember old state
		bool bOldScanNymPref = ZkService::GetIClientCentral()->GetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_NYMNAMES ) ;
		bool bOldScanIPPref = ZkService::GetIClientCentral()->GetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_IP ) ;
		bool bOldScanEmailPref = ZkService::GetIClientCentral()->GetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_EMAIL_ADDR ) ;
		ZKSTD::vector<ZKSTD::string> vecOldOffensiveStrings ;

		ZKSTD::vector<ZKSTD::string> vecOffensiveStrings ;

		INym* pNym = NULL;
		ZkService::GetINymCentral()->GetNym(IID_INYM, (void**) &pNym, USER_NYM_INDEX);

		ZKTEST( pNym->GetOffensiveWordList( vecOldOffensiveStrings ) ) ;

		// Set up the word scan list with known contents.

		ZKSTD::string szBadWord1( "nym rod" ) ;
		ZKSTD::string szBadWord2( "chemical" ) ;

		vecOffensiveStrings.push_back( szBadWord1 ) ;
		vecOffensiveStrings.push_back( szBadWord2 ) ;

		ZKTEST( pNym->SetOffensiveWordList(vecOffensiveStrings) ) ;

		// Test 1 
		char szMsg1[] = "cheMIcal reaction" ;

		ZKTEST( ZkService::GetIWordScan()->WordScan( (unsigned char*) szMsg1, strlen( szMsg1 ), vecFound ) ) ;
		ZKTEST( vecFound.size() == 1 ) ;
		ZKTEST( vecFound[0].compare( szBadWord2 ) == 0 ) ;

		scanResult.pScannedBuffer = (unsigned char*) szMsg1 ;
		scanResult.cntBuffer = strlen( szMsg1 ) ;

		ShowOffensiveResults( &scanResult ) ;
		vecFound.clear() ;

		// Test 2
		char szMsg2[] = "caution-chemicals leaking" ;

		ZKTEST( ZkService::GetIWordScan()->WordScan( (unsigned char*) szMsg2, strlen( szMsg2 ), vecFound ) ) ;
		ZKTEST( vecFound.size() == 1 ) ;
		ZKTEST( vecFound[0].compare( szBadWord2 ) == 0 ) ;

		scanResult.pScannedBuffer = (unsigned char*) szMsg2 ;
		scanResult.cntBuffer = strlen( szMsg2 ) ;

		ShowOffensiveResults( &scanResult ) ;
		vecFound.clear() ;
			
		// Test 3

		char szMsg3[] = "Normon is a Nym ROD" ;

		ZKTEST( ZkService::GetIWordScan()->WordScan( (unsigned char*) szMsg3, strlen( szMsg3 ), vecFound ) ) ;
		ZKTEST( vecFound.size() == 1 ) ;
		ZKTEST( vecFound[0].compare( szBadWord1 ) == 0 ) ;

		scanResult.pScannedBuffer = (unsigned char*) szMsg3 ;
		scanResult.cntBuffer = strlen( szMsg3 ) ;

		ShowOffensiveResults( &scanResult ) ;
		vecFound.clear() ;

		// Test 4 - Look for escape characters and + equivalent to space (as in http )

		char szMsg4[] = "Normon+is+a+Nym+ROD Nym Rod  Nym%20ROD" ;

		ZKTEST( ZkService::GetIWordScan()->WordScan( (unsigned char*) szMsg4, strlen( szMsg4 ), vecFound ) ) ;
		ZKTEST( vecFound.size() == 3 ) ;

		scanResult.pScannedBuffer = (unsigned char*) szMsg4 ;
		scanResult.cntBuffer = strlen( szMsg4 ) ;

		ShowOffensiveResults( &scanResult ) ;
		vecFound.clear() ;

		// Test Email find feature
		ZKSTD::string strReturnAddr ;
		ZkService::GetIClientCentral()->GetReturnEmailAddress( strReturnAddr ) ;

		if ( !strReturnAddr.empty() ) 
		{			
			ZkService::GetIClientCentral()->SetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_EMAIL_ADDR, ZK_TRUE ) ;

			ZKSTD::string szMsg4( "Someone is trying to send out the address " ) ;
			szMsg4 += strReturnAddr ; 

			ZKTEST( ZkService::GetIWordScan()->WordScan( (unsigned char*) szMsg4.c_str(), szMsg4.size(), vecFound ) ) ;

			ZKTEST( vecFound.size() == 1 ) ;
			ZKTEST( vecFound[0].compare( strReturnAddr ) == 0 ) ;

			scanResult.pScannedBuffer = (unsigned char*) szMsg4.c_str() ;
			scanResult.cntBuffer = szMsg4.size() ;

			ShowOffensiveResults( &scanResult ) ;
			vecFound.clear() ;
		}
		
		// Test nym find feature

		size_t cntNyms = ZkService::GetINymCentral()->GetNumberOfNyms() ;

		if ( cntNyms > 2 ) 
		{

			// Test looking for other nyms can only happen if there are other nyms
			// to look for

			ZkService::GetIClientCentral()->SetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_NYMNAMES, ZK_TRUE ) ;

			size_t indexActive = ZkService::GetINymCentral()->GetActiveNymIndex( ) ;
			size_t anotherNym = 0 ;

			if ( indexActive > 1 ) 
			{
				anotherNym = 1 ;
			}
			else 
			{
				anotherNym = indexActive + 1 ;
			}

			ZKSTD::string strAnotherNym ;
			ZkService::GetINymCentral()->GetNymName( anotherNym, strAnotherNym );

			ZKSTD::string strMsg( "I'm specifically looking for " ) ;
			strMsg.append( strAnotherNym ) ;

			ZKTEST( ZkService::GetIWordScan()->WordScan( (unsigned char*) strMsg.c_str(), strMsg.size(), vecFound ) ) ;
			ZKTEST( vecFound.size() == 1 ) ;
			ZKTEST( vecFound[0].compare( strAnotherNym ) == 0 ) ;
			scanResult.pScannedBuffer =  (unsigned char*) strMsg.c_str() ;
			scanResult.cntBuffer = strMsg.size() ;
			ShowOffensiveResults( &scanResult ) ;

			vecFound.clear() ;
		}		

		// Look for an IP
		ZkService::GetIClientCentral()->SetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_IP, ZK_TRUE ) ;

		char szBuf[1024];  //MAXGETHOSTSTRUCT
		struct hostent* pHostEntry = NULL;
		
		gethostname(szBuf, 1024-1); // used to retrieve name of local machine, is the only standard database function that does not block

		if (zkGetHostByName(NULL, szBuf, &pHostEntry) // always blocks no matter what type of socket used
			&& pHostEntry->h_addr_list[0]) 
		{
			struct in_addr * pAddr = (struct in_addr *) pHostEntry->h_addr_list[0] ;

			ZKSTD::string IPmsg( "This is my IP: " ) ;
			ZKSTD::string MyIP( inet_ntoa( *pAddr )) ;
			IPmsg.append( MyIP ) ;

			ZKTEST( ZkService::GetIWordScan()->WordScan( (unsigned char*) IPmsg.c_str(), IPmsg.size(), vecFound ) ) ;
			ZKTEST( vecFound.size() == 1 ) ;
			ZKTEST( vecFound[0].compare( MyIP ) == 0 ) ;

			scanResult.pScannedBuffer =  (unsigned char*) IPmsg.c_str() ;
			scanResult.cntBuffer = IPmsg.size() ;
			ShowOffensiveResults( &scanResult ) ;

			vecFound.clear() ;

			// Now look for IP in a larger buffer

			unsigned char BinaryBuf[ 256 ] ;

			for ( int i = 0 ; i < 256 ; i++ ) 
			{
				// Fill in some distictive values
				BinaryBuf[i] = i ;
			}

			// Fill in an IP somewhere in the middle

			unsigned long * pIPpos = (unsigned long*) BinaryBuf + 62 ;
			*pIPpos = pAddr->s_addr ;

			ZKTEST( ZkService::GetIWordScan()->WordScan( BinaryBuf, 256, vecFound ) ) ;
			ZKTEST( vecFound.size() == 1 ) ;

			scanResult.pScannedBuffer =  BinaryBuf ;
			scanResult.cntBuffer = 256 ;

			ShowOffensiveResults( &scanResult ) ;

			vecFound.clear() ;

			unsigned char BigBuf[ 8000 ] ;

			for ( i = 0 ; i < 8000 ; i++ ) 
			{
				// Fill in some distictive values
				BigBuf[i] = rand() % 256 ;
			}

			pIPpos = (unsigned long*) BigBuf + 1000 ;
			*pIPpos = htonl( pAddr->s_addr ) ;


			ZKTEST( ZkService::GetIWordScan()->WordScan( BigBuf, 8000, vecFound ) ) ;
			ZKTEST( vecFound.size() == 1 ) ;

			scanResult.pScannedBuffer = BigBuf ;
			scanResult.cntBuffer = 8000 ;

			ShowOffensiveResults( &scanResult ) ;

			vecFound.clear() ;
		}

		// Restore Word Scanning List
		ZkService::GetIClientCentral()->SetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_NYMNAMES, bOldScanNymPref ) ;
		ZkService::GetIClientCentral()->SetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_IP, bOldScanIPPref ) ;
		ZkService::GetIClientCentral()->SetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_EMAIL_ADDR, bOldScanEmailPref ) ;
		ZKTEST( pNym->SetOffensiveWordList(vecOldOffensiveStrings) ) ;

	}
	catch ( IZkException& ex ) 
	{
		// Something went wrong
		ZKTEST( ZK_FALSE ) ;
		ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
	} 
}

void ZkServiceUnitTest::ShowOffensiveResults( WordScanSummary* in_pFoundWords )
{
	ZKTEST( in_pFoundWords->cntBuffer > 0 ) ;
	ZKTEST( in_pFoundWords->pFoundWords != NULL ) ;
	ZKTEST( in_pFoundWords->pScannedBuffer != NULL ) ;

	// This pops up a dialog which is offensive to 
	// automated tests and testing UI code, not ZkService - hence temporary
#ifdef SHOW_OFFENSIVE_WORD_DIALOGS
	HWND hFreedom = ::FindWindow(NULL, FRWINDOWNAME );	
	::SendMessage(hFreedom , WM_WARN_WORD_SCAN, (WPARAM)PROTOCOL_HTTP, (LPARAM)in_pFoundWords );
#endif
}



long FreedomSetupUnitTest::Execute( void* )
{
	return TestFreedomSetup() ? 1 : 0;
}

bool FreedomSetupUnitTest::TestFreedomSetup()
{
	// get the filename
	ZKSTD::string str_file = "";
	bool bValExists = IZkRegistry::GetString( SETUP_WIZARD_PARAM_FILE_KEY, str_file );
	if( ! bValExists )
	{
		str_file = "testparams.dat";
	}
	if( str_file.find('\\') == ZKSTD::string::npos )
	{
		// append the directory
		ZKSTD::string str_path = ZKFileConfiguration::GetFreedomTestDataDir();
		str_file = str_path + str_file;
	}

	ZKSTD::ifstream fileStream( str_file.c_str() ) ;
	bool bReturn = true;
	char bufSerialNum[80] = {0};
	char bufPassphrase[80] = {0};
	char bufEmailAddress[80] = {0};

	if ( fileStream ) {
		fileStream.getline(bufSerialNum, 80);
		fileStream.getline(bufPassphrase, 80);
		fileStream.getline(bufEmailAddress, 80);

		ZKSTD::string strSerialNum(bufSerialNum);
		ZKSTD::string strPassphrase(bufPassphrase);
		ZKSTD::string strEmailAddress(bufEmailAddress);

		if( strEmailAddress.empty() || strPassphrase.length() < MIN_PASSPHRASE)
		{
			bReturn = false;
		}
		else
		{
			ZKSTD::string old;
			ZkService::GetICrypto()->GetPassphrase(old);
			ZKTEST(old.empty());
			ZkService::GetICrypto()->ChangePassphrase(old, strPassphrase) ;
			ZKSTD::string ConsoleMessage = "Setup Test: Set Password To " + strPassphrase;
			IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, ConsoleMessage.c_str());
			
			ZkService::GetIClientCentral()->SetReturnEmailAddress( strEmailAddress ) ;
			ConsoleMessage = "Setup Test: Set Email To " + strEmailAddress;
			IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, ConsoleMessage.c_str());
		}

		if(bReturn == false)
		{
		}
		else if(strSerialNum.empty())
		{
			bReturn = false;
		}

		WSerialNumber SerialNumber( strSerialNum.c_str() ) ;

		try
		{
			ZkService::GetITokenService()->RedeemSerialNumberForTokens(SerialNumber);
		}
		catch( IZkException& )
		{
			bReturn = false;
			ZKTEST(0);
		}
		
		if(bReturn)
		{
			ZkService::GetIClientPersist()->SaveClientState();
			ZKTEST(ZkService::GetINymCentral()->GetNumberOfNyms() == 1);
			CreateNymUnitTest::TestNymCreation();
			ZKTEST(ZkService::GetINymCentral()->GetNumberOfNyms() == 2);
			CreateNymUnitTest::TestNymCreation();
			ZKTEST(ZkService::GetINymCentral()->GetNumberOfNyms() == 3);
			ZkService::GetIClientPersist()->SaveClientState();
		}
	}
	else
	{
		bReturn = false;
	}

	// delete the file for the next time a user start freedom without a nym file
	fileStream.close();
	IZkFileHelp::BackupAndErase(str_file);

	return bReturn;
}


long PersistStressUnitTest::Execute( void* )
{
	BashFileIO() ;
	SaveManyTimes();
	return 1;
}

void PersistStressUnitTest::BashFileIO()
{
	// Basic file io operations should never fail assuming
	// disk is accessable and has space
	// This mimics what happens at the low level of the 
	// Client Persistence operation.

	for ( size_t i = 0 ; i < 100 ; i++ )
	{
		SaveAndValidateBuffer( 100 ) ;
		SaveAndValidateBuffer( 1000 ) ;
		SaveAndValidateBuffer( 10000 ) ;
		SaveAndValidateBuffer( 640 ) ;
		SaveAndValidateBuffer( 128 ) ;
		SaveAndValidateBuffer( 1024 ) ;
		SaveAndValidateBuffer( 2048 ) ;
		SaveAndValidateBuffer( 127 ) ;
		SaveAndValidateBuffer( 1023 ) ;
		SaveAndValidateBuffer( 2047 ) ;
		SaveAndValidateBuffer( 2049 ) ;
		SaveAndValidateBuffer( 1024 * 512 ) ;
	}

	ZKTRACE0( "Bash File IO test complete\n" ) ;
}

void PersistStressUnitTest::SaveAndValidateBuffer( size_t in_cntBuffer ) 
{
	ZKTRACE1( "Attempting to saving and copy %d bytes\n", in_cntBuffer ) ;

	WAutoBuffer buf1 ;
	WAutoBuffer buf2 ;

	buf1.Allocate( in_cntBuffer ) ;
	
	for ( size_t i = 0 ; i < in_cntBuffer ; i++ )
	{
		// Fill in known values
		buf1[ i ] = i % 256 ;
	}

    ZKSTD::string strFile1 = ZKFileConfiguration::GetFreedomUserDataDir();
	strFile1 += "temp1.dat" ;

	ZKSTD::string strFile2 = ZKFileConfiguration::GetFreedomUserDataDir();
	strFile2 += "temp2.dat" ;

	ZKTEST( IZkFileHelp::WriteBinaryFile( strFile1.c_str(), buf1, buf1.Size() ) );
	ZKTEST( IZkFileHelp::FileCopy( strFile2.c_str(), strFile1.c_str() ) );
	ZKTEST( IZkFileHelp::ReadBinaryFile( strFile2.c_str(), &buf2, buf2.GetSizeRef() ) );

	if ( buf2.Size() != in_cntBuffer )
	{
		ZKTEST( "File is not correct size" && false ) ;
	}
	else if ( memcmp( buf1, buf2, in_cntBuffer ) != 0 )
	{
		ZKTEST( "File is corrupted" && false ) ;
	}
}

void PersistStressUnitTest::SaveManyTimes()
{
	// Save in a tight loop.  Doing other things at the
	// same time will give opportunities to see if we can 
	// cause corruptions or deadlocks
    for ( int i = 0 ; i < 10000 ; i++ )
    {
      ZKTRACE1( "Saving Nym File %d\n", i ) ; 

		ZkService::GetIClientPersist()->SaveClientState() ;

#ifdef WIN32
		::Sleep(1000);
#else
		usleep(100);
#endif

	}
}

long CreateNymUnitTest::Execute( void* )
{
	TestNymCreation();
	return 1;
}

void CreateNymUnitTest::TestNymCreation(const char* in_strToken)
{
	ZKFunctionEntry( "ZkService/CreateNymUnitTest::TestNymCreation" );
	if(ZkService::GetITokenService()->GetTotalNumberOfTokens() < 2)
	{
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, "Can't run the Nym Creation Test. Get some tokens first.");
		return;
	}


	//Steps include:
	//-Select a token -> should be trivial, assuming there is at least one
	//-Select number of hops
	//-Select a first hop (easy -> see the route rule unit test)
	//-Select a name - probably a name based on machine name + current time

	INym* pINym = NULL;
	INymMail* pINymMail = NULL;
	MapRules			mapRules;
	SecuritySettings	SecuritySettings = {0};

	bool bCreatedNym = false;
	bool bStatus;
	char tempBuf[256];
        char tempBuf2[16];
	time_t lTime;
	ZKSTD::string strToken;
	ZKSTD::string strNymName;
	ZKASSERT(ZkService::GetITokenService()->GetTotalNumberOfTokens());
	
	if( ! in_strToken )
	{
		ZkService::GetITokenService()->GetTokenXinBase64(0, strToken);
	}
	else
	{
		strToken = in_strToken;
	}

	try 
	{
		strNymName = "&%!#";
		ZKNymName theName(strNymName);// Create a syntactically invalid name
		ZkService::GetINymCentral()->CreateNym( theName, strToken) ;
		ZKTEST(0); // This should never execute
	}
	catch(IZkException&)
	{
		// good, this should fail
	}

	gethostname(tempBuf, 255);
	time(&lTime);
	strNymName = tempBuf;
	snprintf( tempBuf2, 16, "%d", lTime%10000 ) ; 

	strNymName += tempBuf2 ;

	try 
	{
		SecuritySettings.nReserved = sizeof(SecuritySettings);
		SecuritySettings.nHops = MAX_HOPS;

		time_t time1,time2;
		time( &time1 );

		ZKNymName theName(strNymName);
		ZkService::GetINymCentral()->CreateNym( theName, strToken) ;

		
		bStatus = ZkService::GetINymCentral()->GetNym(IID_INYM, (void**)&pINym, strNymName.c_str());
		ZKTestAndThrow(bStatus);
		bCreatedNym = true;

		WNymIconInfo info ;
		info.SetIconIndex( DEFAULT_NYM_ICON_INDEX ) ;
		pINym->SetNymIconInfo( info );
		
		ZKTEST(ZkService::GetITokenService()->RemoveTokenFromArray( strToken ) );

				
		pINym->SetSecuritySettings(SecuritySettings);
		ZkService::GetIClientPersist()->SaveClientState();


		ZkService::GetINymCentral()->GetNym(IID_IMAIL, (void**)&pINymMail, strNymName.c_str());
		ZKSTD::string ConsoleMessage = "Create Nym Test: Created Nym " + strNymName;
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, ConsoleMessage.c_str());

		time( &time2 );
		char strMsg[1024] = {0};
		snprintf(strMsg,sizeof(strMsg),"Creating Nym took [%d] seconds", time2 - time1);
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_WARNING, IDS_CONS_JUST_STRING, strMsg );
	}
	catch(IZkException& ex)
	{
		ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
	}

	try
	{
		ZKSTD::string newStrNymName;
		
		gethostname(tempBuf, 255);
		time(&lTime);
		newStrNymName = tempBuf;
	        snprintf( tempBuf2, 16, "%d", lTime ) ; 
	        newStrNymName += tempBuf2 ;

		// Use a new name, but old token, to be sure
		// that it fails because of the token
		ZKNymName theName(newStrNymName);
		ZkService::GetINymCentral()->CreateNym( theName, strToken) ;
		ZKTEST(0); // This should never execute
	}
	catch(IZkException&)
	{
		//Good, this should fail
	}
	
	ZKNymName theSameName(strNymName);
	try
	{
		ZKASSERT(ZkService::GetITokenService()->GetTotalNumberOfTokens());
		ZkService::GetITokenService()->GetTokenXinBase64(0, strToken);
		// Try this with the old name, but new token, since we
		// want to be sure it fails because of the name being taken
		ZkService::GetINymCentral()->CreateNym( theSameName, strToken) ;
		ZKTEST(0); // This should never execute
	}
	catch(IZkException&)
	{
		//Good, this should fail
		ZKTEST(theSameName.Status() == ZKNNStatusNameUnavailable);
	}
}


void ZkServiceUnitTest::TestITokenService()
{
	try
	{
		IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, "Token Unit Test");


//		ZkService::GetITokenService()->GetTheTokenTypeDescriptions() ;

		// Serial number stuff - verify what happens with bad serial numbers


		WSerialNumber FakeSerial( "Z96Y64LQ4PP77A" ) ;        // Spent or not in database
		WSerialNumber FakeSerial2( "TGXMTWLJL8HXU9" ) ;       // Spent or not in database
		WSerialNumber FakeSerialBadFormat( "----123$#*@&" ) ;

		ZKTEST( FakeSerial.IsWellFormed() ) ;
		ZKTEST( FakeSerial2.IsWellFormed() ) ;
		ZKTEST( !FakeSerialBadFormat.IsWellFormed() ) ;

		bool bFailed = false ;

		// Verify that GetSerialNumberCapabilities can take garbage input

		try 
		{
			ZkService::GetITokenService()->GetSerialNumberCapabilities( FakeSerialBadFormat ) ;
		}
		catch( IZkException& )
		{
			bFailed = true ;
		}
		ZKTEST( bFailed ) ;

		// Network down so we can't determine if serial is spent or not etc

		ZkService::GetINiqs()->SetNetworkFailureDebuggingFlag( true ) ;

		try 
		{
			bFailed = false ;
			ZkService::GetITokenService()->GetSerialNumberCapabilities( FakeSerial ) ;
		}
		catch( IZkException& )
		{
			bFailed = true ;
		}
		ZKTEST( bFailed ) ;

		ZkService::GetINiqs()->SetNetworkFailureDebuggingFlag( false ) ;

		// Network is hopefully up but serial number is spent

		try 
		{
			bFailed = false ;
			ZkService::GetITokenService()->GetSerialNumberCapabilities( FakeSerial2 ) ;
		}
		catch( IZkException& )
		{
			bFailed = true ;
		}
		ZKTEST( bFailed ) ;

		// ... TODO ... try with a real serial number


		// Try redeeming

		try 
		{
			ZkService::GetITokenService()->RedeemSerialNumberForTokens( FakeSerialBadFormat ) ;
		}
		catch( IZkException& )
		{
			bFailed = true ;
		}
		ZKTEST( bFailed ) ;

		// Network down so we can't determine if serial is spent or not etc

		ZkService::GetINiqs()->SetNetworkFailureDebuggingFlag( true ) ;

		try 
		{
			bFailed = false ;
			ZkService::GetITokenService()->RedeemSerialNumberForTokens( FakeSerial ) ;
		}
		catch( IZkException& )
		{
			bFailed = true ;
		}
		ZKTEST( bFailed ) ;

		ZkService::GetINiqs()->SetNetworkFailureDebuggingFlag( false ) ;

		// Network is hopefully up but serial number is spent

		try 
		{
			bFailed = false ;
			ZkService::GetITokenService()->RedeemSerialNumberForTokens( FakeSerial2 ) ;
		}
		catch(IZkException& )
		{
			bFailed = true ;
		}
		ZKTEST( bFailed ) ;

		// ... TODO ... try with a real serial number


		ZkService::GetITokenService()->GetTotalNumberOfTokens() ;

	}
	catch(IZkException& ex)
	{
		ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
	}
}

void 
ZkServiceUnitTest::TestINymWallet()
{
	INym* pNym = NULL;
	INymWallet* pNymWallet = NULL;
	
	ZkService::GetINymCentral()->GetActiveNym(IID_INYM, (void**)&pNym);

	if (!pNym)
	{
		ZKTEST(false);
	}
	else
	{
		pNym->QueryInterface( IID_INYMWALLET, (void**) &pNymWallet);
		if (!pNymWallet)
		{
			ZKTEST(false);
		}
		else
		{
			ZKSTD::string strNymName;
			pNym->GetNymName(strNymName);
			
			ZKSTD::vector<LoginInfo> loginInfoVector;

			pNymWallet->GetLoginInfo("test_domain.com", loginInfoVector);
			ZKTEST(loginInfoVector.size() == 0);

			pNymWallet->GetLoginInfo("test_domain.com", loginInfoVector);
			ZKTEST(loginInfoVector.size() == 0);

			LoginInfo loginInfo;
			loginInfo = LoginInfo(strNymName, "username", "password", "test_domain.com");
			pNymWallet->AddLoginInfo(loginInfo);
			
			pNymWallet->GetLoginInfo("test_domain.com", loginInfoVector);
			ZKTEST(loginInfoVector.size() == 1);

			
			loginInfo.strDomain = "computer_name.TEST_DOMAIN.COM";
			ZKTEST(pNymWallet->DeleteLoginInfo(loginInfo));

			pNymWallet->GetLoginInfo("test_domain.com", loginInfoVector);
			ZKTEST(loginInfoVector.size() == 0);
			
			
			LoginInfo loginInfo2(strNymName, "username2", "password", "comp1.comp2.test_domaiN.com");
			pNymWallet->AddLoginInfo(loginInfo);
			pNymWallet->AddLoginInfo(loginInfo2);

			pNymWallet->GetLoginInfo("comp0.comp1.comp2.test_domaiN.com:8080", loginInfoVector);
			ZKTEST(loginInfoVector.size() == 2);

			ZKTEST(pNymWallet->DeleteLoginInfo(loginInfo));
			ZKTEST(pNymWallet->DeleteLoginInfo(loginInfo2));

			pNymWallet->GetLoginInfo("test_domain.com", loginInfoVector);
			ZKTEST(loginInfoVector.size() == 0);


			loginInfo = LoginInfo(strNymName, "username1", "password1", "test_domain.com");
			pNymWallet->AddLoginInfo(loginInfo);
			pNymWallet->AddLoginInfo(loginInfo);

			pNymWallet->GetLoginInfo(loginInfo.strDomain, loginInfoVector);
			ZKTEST(loginInfoVector.size() == 1);
			ZKTEST(loginInfoVector[0] == loginInfo);
			ZKTEST(pNymWallet->DeleteLoginInfo(loginInfo));
			pNymWallet->GetLoginInfo(loginInfo.strDomain, loginInfoVector);
			ZKTEST(loginInfoVector.size() == 0);


			pNymWallet->GetAllLoginInfo(loginInfoVector);
			unsigned int size = loginInfoVector.size();

			pNymWallet->AddLoginInfo(loginInfo);
			pNymWallet->GetAllLoginInfo(loginInfoVector);
			ZKTEST((size + 1) == loginInfoVector.size());
			ZKTEST(pNymWallet->DeleteLoginInfo(loginInfo));
			pNymWallet->GetAllLoginInfo(loginInfoVector);
			ZKTEST(size == loginInfoVector.size());

			ZKSTD::string strOriginalField = pNymWallet->GetField(WALLET_NAME_FIRST);
			pNymWallet->SetField(WALLET_NAME_FIRST, "");
			ZKTEST(pNymWallet->GetField(WALLET_NAME_FIRST) == "");

			pNymWallet->SetField(WALLET_NAME_FIRST, "Hi");
			ZKTEST(pNymWallet->GetField(WALLET_NAME_FIRST) == "Hi");

			pNymWallet->SetField(WALLET_NAME_FIRST, strOriginalField);
			ZKTEST(pNymWallet->GetField(WALLET_NAME_FIRST) == strOriginalField);

			pNymWallet->GetRandomField(WALLET_NAME_FIRST, true);
			pNymWallet->GetRandomField(WALLET_NAME_FIRST, false);

			// Launch self test
			pNymWallet->SelfTest();
		}
	}
}


long FirewallUnitTest::Execute( void * )
{
	size_t nIndex = ZkService::GetINymCentral()->GetActiveNymIndex( ) ;

	if ( nIndex == USER_NYM_INDEX ) 
	{
		TestIFirewallWithUserNymActive() ;
	}	
	else 
	{
		TestIFirewallWithNymActive() ;
	}

	return 1;
}

void 
FirewallUnitTest::TestIFirewallWithNymActive()
{
	IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, "Launching Firewall Unit Test" );
	ZkService::GetIFirewall()->TestsWithNymActive() ;
}
void 
FirewallUnitTest::TestIFirewallWithUserNymActive()
{
	IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, "Launching Firewall Unit Test" );
	ZkService::GetIFirewall()->TestsWithUserNymActive()  ;
}

void 
ZkServiceUnitTest::TestRActiveRouteTable()
{

	DZkError err ;
	FrLocalAipCtx Context = NULL ;
	ZKTEST(frLocalAipCtxNew(&err, &Context, FRPKT_DATA_VERSION_2, FRPKT_ROUTE_VERSION_2, MAX_ROUTES, FRPKT_NUM_ROUTES_MAX, WINDOWS_TCP_AND_UDP_PORT));	
	
	try
	{

		ZKTEST(frLocalAipCtxSetKeyCache(&err, Context, ZkService::GetIKeyQry()->GetKeyCache()));
		ZKTEST(frLocalAipCtxSetNetInfo(&err, Context, ZkService::GetINiqs()->GetNetInfo()));


		RActiveRouteTable m_Table ;
		m_Table.Init( Context ) ;

		WNetInfoDatumVec lst; // list of aips	
		ZkService::GetINiqs()->GetServersOfType( FRENT_TYPE_AIP, lst, false ) ;

		WNetInfoDatum nymserver ;
		ZkService::GetINiqs()->GetServer( FRENT_TYPE_NYM_SRV, nymserver ) ;

		if ( lst.size() < 3 )
		{
			// Can't perform test
			return ;
		}

		// Topology might not actually allow these routes, however we aren't
		// really going to create them

		uint32_t aHops[3] ;

		for ( int i = 0 ; i < 3 ; i++ )
		{
			aHops[i] = lst[i].GetId() ;
		}

		WRoute NymServerRoute ;

		NymServerRoute.AddHop( lst[0].GetId() ) ;
		NymServerRoute.AddHop( lst[1].GetId() ) ;
		NymServerRoute.AddHop( nymserver.GetId() ) ;

		sockaddr_in nymserverAddr = nymserver.GetSockAddr() ;
		ZkACI newRoute = m_Table.InitNewUnauthenticatedRoute( 
									ntohs( nymserverAddr.sin_port ),
									100, 
									NymServerRoute ) ;

		ZKTEST( newRoute != INVALID_ROUTE_ID ) ;
		ZKTEST( !m_Table.IsRouteActive( newRoute ) ) ;

		// Pretend that we really created the route
		m_Table.RouteIsActive( newRoute ) ;
		ZKTEST( m_Table.IsRouteActive( newRoute ) ) ;

		// Confirm that the first hop really is in there
		sockaddr_in addr ;
		m_Table.GetFirstAipAddr( newRoute, addr ) ;

		sockaddr_in first = lst[ 0 ].GetSockAddr() ;
		ZKTEST( 0 == memcmp( &addr, &first, sizeof( sockaddr_in) ) );

		// Confirm that the last hop really is what we expect
		sockaddr_in exitAddr ;
		m_Table.GetExitAipAddr( newRoute, exitAddr ) ;


		ZKTEST( 0 == memcmp( &exitAddr, &nymserverAddr, sizeof( sockaddr_in) ) );

		ZKTEST( ntohs(exitAddr.sin_port) == FR_DFLT_NYM_SERVER_PORT ) ;
		ZKTEST( FRENT_TYPE_NYM_SRV == ZKConvert::GetEntityTypeFromPort( ntohs( exitAddr.sin_port ) ) );

		// We should recognize this route to the nym server
		ZkACI FoundRoute = INVALID_ROUTE_ID;
		ZKTEST( m_Table.IsRouteToServer( FRENT_TYPE_NYM_SRV, &FoundRoute ) ) ;
		ZKTEST( FoundRoute == newRoute ) ;

		ZKTEST( m_Table.IsIPFirstHopOfRoute( first.sin_addr.s_addr, &FoundRoute ) ) ;
		ZKTEST( FoundRoute == newRoute ) ;

		if ( ZkService::GetINymCentral()->GetNumberOfNyms() > 1 )
		{
			// Create a pretend route to Nym 1
			WRoute route ;
			route.AddHop( aHops[ 0 ] ) ;
			route.AddHop( aHops[ 1 ] ) ; 
			route.AddHop( aHops[ 2 ] ) ;

			ZkACI anotherRoute = m_Table.InitNewAuthenticatedRouteForNym( 100, route, 1 ) ;	
		}

		m_Table.DeleteDeadRoute( newRoute ) ;
		
	}
	catch( IZkException& ex ) 
	{
		ZKTEST( false ) ;
		ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
	}

	ZKTEST(frLocalAipCtxFree(&err, &Context));
}


long RedeemAndCreateNymsUnitTest::Execute( void* in_type)
{
	int value = 0;
	if( in_type != 0 )
	{
		value = *((int*)in_type);
	}
	SerialNumberTypes sntEquivalent;

	switch( value )
	{
		case 500 : sntEquivalent = NYM_TYPE1_CREATE; break;
		case 501 : sntEquivalent = NYM_TYPE2_CREATE; break;
		case 502 : sntEquivalent = NYM_TYPE3_CREATE; break;
		default: sntEquivalent = NYM_TYPE1_CREATE; break;
	}
	return RedeemAndCreateNyms( sntEquivalent );
}

long RedeemAndCreateNymsUnitTest::RedeemAndCreateNyms(SerialNumberTypes in_nymType)
{
	bool bReturn = true;
	tstring strSerialNum;

	ZKTestScalper::GetSerialNumber(strSerialNum, in_nymType);

	WSerialNumber serialNumber( strSerialNum.c_str() ) ;

	time_t timeStarted;
	time( &timeStarted );

	ZKSTD::vector<tstring> vTokens;

	if( !serialNumber.IsWellFormed() )
		bReturn = false;
	else 
	{
		try 
		{
			ZkService::GetITokenService()->RedeemSerialNumberForTokens( serialNumber, vTokens);
		}
		catch( IZkException& )
		{
			bReturn = false;
			ZKTEST( 0 );
		}
	}
		
	time_t timeRetire;
	time( &timeRetire );
	
	char strMsg[1024] = { 0 };
	::snprintf( strMsg, sizeof(strMsg), "Redeeming serial number took [%d] seconds", (timeRetire - timeStarted) );
	IZkConsoleMsg::ShowMsgForceUpdate( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, strMsg );
	
	if( bReturn )
	{
		ZKTEST( ZkService::GetITokenService()->GetTotalNumberOfTokens() > 0 ) ;

		ZkService::GetIClientPersist()->SaveClientState();
		// create the nyms...depending on how much tokens we got
		for( unsigned int i = 0 ; i < vTokens.size()-1 ; ++i)
		{
			CreateNymUnitTest::TestNymCreation(vTokens[i].c_str());
		}
		ZkService::GetIClientPersist()->SaveClientState();
	}

	return bReturn;
}

void ZkServiceUnitTest::TestRNetworkPerformance()
{
	ZkService::GetINetworkPerformance()->SelfTest() ;
}

void ZkServiceUnitTest::TestIClientCentral()
{
	int nMinutes = ZkService::GetIClientCentral()->GetNumMinutesUntilReAskPassword();

	ZkService::GetIClientCentral()->SetNumMinutesUntilReAskPassword(0);
	ZKTEST(ZkService::GetIClientCentral()->IsPasswordRequired());

	ZkService::GetIClientCentral()->SetNumMinutesUntilReAskPassword(
		ZK_CLIENT_NEVER_RE_ASK_PASSWORD);
	ZKTEST(!ZkService::GetIClientCentral()->IsPasswordRequired());

	ZkService::GetIClientCentral()->SetNumMinutesUntilReAskPassword(1);
	ZKTEST(!ZkService::GetIClientCentral()->IsPasswordRequired());

	ZkService::GetIClientCentral()->SetNumMinutesUntilReAskPassword(nMinutes);
	ZKTEST(nMinutes == ZkService::GetIClientCentral()->GetNumMinutesUntilReAskPassword());

	ZKSTD::set<ZKSTD::string> setOfDomains, setOfDomains2;

	ZkService::GetIClientCentral()->GetDomainsForWhichWalletIsDisabled(setOfDomains);

	ZKTEST(!ZkService::GetIClientCentral()->IsWalletDisabledForDomain("www.bogusDomain.com"));
	ZkService::GetIClientCentral()->SetIsWalletDisabledForDomain("www.bogusDomain.com", true);
	ZKTEST(ZkService::GetIClientCentral()->IsWalletDisabledForDomain("www.bogusDomain.com"));

	ZkService::GetIClientCentral()->SetIsWalletDisabledForDomain("www.bogusDomain2.com", true);
	ZkService::GetIClientCentral()->SetIsWalletDisabledForDomain("www.bogusDomain2.com", false);

	ZkService::GetIClientCentral()->GetDomainsForWhichWalletIsDisabled(setOfDomains2);
	ZKTEST(setOfDomains2.find("www.bogusdomain.com") != setOfDomains2.end());
	ZKTEST(setOfDomains2.find("bogusdomain.com") == setOfDomains2.end());
	ZKTEST(setOfDomains2.find("www.bogusdomain2.com") == setOfDomains2.end());

	ZkService::GetIClientCentral()->SetDomainsForWhichWalletIsDisabled(setOfDomains);
	ZkService::GetIClientCentral()->GetDomainsForWhichWalletIsDisabled(setOfDomains2);
	ZKTEST(setOfDomains2.find("www.bogusdomain.com") == setOfDomains2.end());
}

void ZkServiceUnitTest::TestIScriptServer()
{
	ZkService::GetIScriptServer()->SelfTest();
}

#endif // INTERNAL_BUILD
