/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef _ZKSERVICEUNITTEST_H
#define _ZKSERVICEUNITTEST_H

#ifdef INTERNAL_BUILD

#include "ZkServiceInc.h"
#include <fr_entid.h>
#include <zkbase/libzktest/ZkTestBase.h>
#include <ZKConfiguration.h>

#include <test_registryKeys.h>	// Name of Freedom setup unit test (also used in FreedomBar.cpp)

class WordScanSummary ;

class ZkService_API ZkServiceUnitTest : public ZkTestBase
{
public:
	ZkServiceUnitTest()
		: ZkTestBase( "zkcentral/zkservices/ZkServiceUnitTest",
						"Unit tests for ZkService" )
	{}

	~ZkServiceUnitTest()
	{}

	virtual long Execute( void* );

	// Run through Setup Wizard, creating
	// nyms and other useful functionality
	
//	static bool TestFreedomSetup();
//	static void TestNymCreation();

	// Once a nym is active this tests functionality on the active nym
	// One useful possibility it to test this with one nym, then
	// switch and try again
//	static void TestsWithNymActive() ;

	// Some functionality is available on the user nym, make
	// sure no nym is selected and then call this
//	static void TestsWithUserNymActive() ;

//	static void TestIFirewallWithNymActive() ;
//	static void TestIFirewallWithUserNymActive() ;

	// This test takes a long time
	// It is only valid with a nym active

//	static void TestINymMailRollback() ;

private:

	// Once a nym is active this tests functionality on the active nym
	// One useful possibility it to test this with one nym, then
	// switch and try again
	static void TestsWithNymActive() ;

	// Some functionality is available on the user nym, make
	// sure no nym is selected and then call this
	static void TestsWithUserNymActive() ;

	// These are the actual unit tests on interfaces
	static void TestINym() ;
	static void TestINymMail() ;
	static void TestINymWallet() ;
	static void TestINymCookieMgr() ;
	static void TestINymCentral() ;
	static void TestIWordScan() ;
	static void TestIClientPersist() ;
	static void TestIClientCentral() ;
	static void TestICookiesManager() ;
	static void TestINiqs() ;
	static void TestITokenService() ;
	static void TestIAdBlocker();
	static void ShowOffensiveResults( WordScanSummary* in_FoundWords ) ;
	static void TestRActiveRouteTable() ;
	static void TestRNetworkPerformance() ;
	static void TestIScriptServer();
} ;


class PersistStressUnitTest : public ZkTestBase
{
public:
	PersistStressUnitTest()
		: ZkTestBase( "zkcentral/zkservices/PersistStress",
						"Persistence Stress Test" )
	{}

	~PersistStressUnitTest()
	{}

	virtual long Execute( void* );

	static void SaveManyTimes();
	static void BashFileIO() ;
	static void SaveAndValidateBuffer( size_t in_cntBuffer ) ;
};


class CreateNymUnitTest : public ZkTestBase
{
public:
	CreateNymUnitTest()
		: ZkTestBase( "zkcentral/zkservices/nymstools/CreateNymUnitTest",
						"Nym Creation Test" )
	{}

	~CreateNymUnitTest()
	{}

	virtual long Execute( void* );

	static void TestNymCreation(const char* in_strToken = 0);
};

class FreedomSetupUnitTest : public ZkTestBase
{
public:
	FreedomSetupUnitTest()
		: ZkTestBase( FREEDOM_SETUP_UNIT_TEST_NAME,
						"Freedom Setup Test" )
	{}

	~FreedomSetupUnitTest()
	{}

	virtual long Execute( void* );

private:
	static bool TestFreedomSetup();
};

class FirewallUnitTest : public ZkTestBase
{
public:
	FirewallUnitTest()
		: ZkTestBase( "zkcentral/zkservices/FirewallUnitTest",
						"Firewall Unit Test" )
	{}

	~FirewallUnitTest()
	{}

	virtual long Execute( void* );

private:
	static void TestIFirewallWithNymActive() ;
	static void TestIFirewallWithUserNymActive() ;
};

class RedeemAndCreateNymsUnitTest : public ZkTestBase
{
public:
	RedeemAndCreateNymsUnitTest()
		: ZkTestBase( "zkcentral/zkservices/RedeemAndCreateNymsUnitTest", 
						"Redeeming serial number and creating 3 nyms" )
	{}

	~RedeemAndCreateNymsUnitTest()
	{}

	virtual long Execute( void* );

private:
	static long RedeemAndCreateNyms(SerialNumberTypes in_nymType);

};



#endif // INTERNAL_BUILD

#endif
