// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// ZkService.cpp : 
//
#include "StdAfx.h"
#include "ZkService.h"
#include "Rkeyqry.h"
#include "RCryptoService.h"
#include "RNiqsService.h"
#include "RNymCentral.h"
#include "RClientCentral.h"
#include "RClientPersist.h"
#include "RTokenService.h"
#include "RCookieManager.h"
#include "RAdBlocker.h"
#include "RFirewall.h"
#include "RRouteManager.h"
#include "RRoutePicker.h"
#include "RNetworkPerformance.h"
#include "RScriptServer.h"
#include "RWallet.h"
#include "RCoreServerConnect.h"
#include "RTokenDesc.h"
#include "RWordScan.h"
#include "RClientInfo.h"
#include <sstream>
#include <IZkRegistry.h>

#ifdef WIN32
#include "RClientPersistOldWinVer.h"		// Temporary
HMODULE g_hModule = NULL ;   // DLL module handle
#endif

RKeyQry*				ZkService::m_pKeyQryInstance = NULL ;
RCryptoService*			ZkService::m_pCryptoInstance = NULL ;
RNiqsService*			ZkService::m_pNiqsInstance	 = NULL ;
RNymCentral*			ZkService::m_pNymCentral	 = NULL ;
RClientCentral*			ZkService::m_pClientCentral	 = NULL ;
RClientPersist*			ZkService::m_pClientPersist  = NULL ;
RCookieManager*			ZkService::m_pCookieManager  = NULL ;
RAdBlocker*				ZkService::m_pAdBlocker		 = NULL ;
RFirewall*				ZkService::m_pFirewall       = NULL ;
RRouteManager*			ZkService::m_pRouteManager   = NULL ;
RRoutePicker*			ZkService::m_pRoutePicker    = NULL ;
RNetworkPerformance*	ZkService::m_pNetworkPerformance   = NULL ;
RScriptServer*			ZkService::m_pScriptServer	 = NULL ;
RCoreServerConnect*		ZkService::m_pCoreServerConnect = NULL ;
RWallet*				ZkService::m_pWallet		 = NULL ;
RTokenDesc*				ZkService::m_pTokenDesc      = NULL ;
RWordScan*				ZkService::m_pWordScan       = NULL ;
RClientInfo*			ZkService::m_pClientInfo     = NULL ;
#ifdef WIN32
RClientPersistOldWinVer* ZkService::m_pClientPersistOldWinVer  = NULL ;
#endif

void 
ZkService::Init()
{
	ZKASSERT( m_pKeyQryInstance == NULL ) ;  // Calling init() twice?  Idiot?
	ZKASSERT( m_pCryptoInstance == NULL ) ;
	ZKASSERT( m_pNiqsInstance == NULL ) ;
	ZKASSERT( m_pNymCentral == NULL ) ;
	ZKASSERT( m_pClientCentral == NULL ) ;
	ZKASSERT( m_pClientPersist == NULL ) ;
	ZKASSERT( m_pCookieManager == NULL ) ;
	ZKASSERT( m_pAdBlocker == NULL ) ;
	ZKASSERT( m_pFirewall == NULL ) ;
	ZKASSERT( m_pRouteManager == NULL ) ;
	ZKASSERT( m_pNetworkPerformance == NULL ) ;
	ZKASSERT( m_pRoutePicker == NULL ) ;
	ZKASSERT( m_pScriptServer == NULL ) ;
	ZKASSERT( m_pWallet == NULL ) ;
	ZKASSERT( m_pCoreServerConnect == NULL ) ;
	ZKASSERT( m_pTokenDesc == NULL ) ;
	ZKASSERT( m_pWordScan == NULL ) ;
	ZKASSERT( m_pClientInfo == NULL ) ;

#ifdef WIN32
	ZKASSERT( m_pClientPersistOldWinVer == NULL ) ;
#endif

	// Although in general we must avoid any
	// dependency on the order of creation,
	// the nature of the frCrypt library is
	// such that we should create this first 
	// and free it last for safety
	m_pCryptoInstance		= new RCryptoService;

	m_pKeyQryInstance		= new RKeyQry;
	m_pNiqsInstance			= new RNiqsService;
	m_pNymCentral			= new RNymCentral;
	m_pClientCentral		= new RClientCentral;
	m_pCookieManager		= new RCookieManager;
	m_pAdBlocker			= new RAdBlocker;
	m_pFirewall				= new RFirewall;
	m_pRouteManager			= new RRouteManager;
	m_pNetworkPerformance	= new RNetworkPerformance ;
	m_pRoutePicker          = new RRoutePicker ;
	m_pScriptServer			= new RScriptServer ;
	m_pWallet				= new RWallet ;
	m_pCoreServerConnect	= new RCoreServerConnect ;
	m_pTokenDesc			= new RTokenDesc ;
	m_pWordScan				= new RWordScan ;
	m_pClientInfo           = new RClientInfo ;

#ifdef WIN32
	if ( IZkRegistry::GetFlag( "OldFileFormat" ) )
	{
		m_pClientPersistOldWinVer   = new RClientPersistOldWinVer;
	}
	else
	{
		m_pClientPersist   = new RClientPersist;
	}
#else
		m_pClientPersist   = new RClientPersist;
#endif
}

void 
ZkService::Term()
{
	SAFEDELETE( m_pKeyQryInstance );
	SAFEDELETE( m_pNiqsInstance );
	SAFEDELETE( m_pNymCentral ) ;
	SAFEDELETE( m_pClientCentral ) ;
	SAFEDELETE( m_pClientPersist ) ;
	SAFEDELETE( m_pCookieManager ) ;
	SAFEDELETE( m_pAdBlocker ) ;
	SAFEDELETE( m_pFirewall ) ;
	SAFEDELETE( m_pRouteManager ) ;
	SAFEDELETE( m_pRoutePicker ) ;
	SAFEDELETE( m_pNetworkPerformance ) ;
	SAFEDELETE( m_pScriptServer ) ;
	SAFEDELETE( m_pWallet ) ;
	SAFEDELETE( m_pCoreServerConnect );
	SAFEDELETE( m_pTokenDesc ) ;
	SAFEDELETE( m_pClientInfo ) ;
#ifdef WIN32
	SAFEDELETE( m_pClientPersistOldWinVer ) ;
#endif

	// Intentionally freeing last
	SAFEDELETE( m_pCryptoInstance );
}

IKeyQry* 
ZkService::GetIKeyQry()
{
	ZKASSERT( m_pKeyQryInstance ) ;
	return m_pKeyQryInstance ;
}

ICrypto* 
ZkService::GetICrypto()
{
	ZKASSERT( m_pCryptoInstance ) ;
	return m_pCryptoInstance ;
}

INiqs* 
ZkService::GetINiqs()
{
	ZKASSERT( m_pNiqsInstance ) ;
	return m_pNiqsInstance ;
}

INymCentral* 
ZkService::GetINymCentral()
{
	ZKASSERT( m_pNymCentral ) ;
	return m_pNymCentral ;
}

IClientCentral* 
ZkService::GetIClientCentral()
{
	ZKASSERT( m_pClientCentral ) ;
	return m_pClientCentral ;
}

ITokenService* 
ZkService::GetITokenService()
{
	ZKASSERT( m_pClientCentral ) ;
	return m_pClientCentral->GetTokenService() ;
}

ITokenDesc*
ZkService::GetITokenDesc()
{
	ZKASSERT( m_pTokenDesc ) ;
	return m_pTokenDesc ;
}

IClientPersist* 
ZkService::GetIClientPersist()
{
#ifdef WIN32
	if ( m_pClientPersist ) 
	{
		ZKASSERT( m_pClientPersist ) ;
		return m_pClientPersist ;
	}
	else
	{
		ZKASSERT( m_pClientPersistOldWinVer ) ;
		return m_pClientPersistOldWinVer ;
	}
#else
    ZKASSERT( m_pClientPersist ) ;
    return m_pClientPersist ;
#endif
}

ICookieManager* 
ZkService::GetICookieManager()
{
	ZKASSERT( m_pCookieManager ) ;
	return m_pCookieManager ;
}

IAdBlocker* 
ZkService::GetIAdBlocker()
{
	ZKASSERT( m_pAdBlocker ) ;
	return m_pAdBlocker  ;
}

IFirewall* 
ZkService::GetIFirewall()
{
	ZKASSERT( m_pFirewall ) ;
	return m_pFirewall ;
}

IPacketFirewall*
ZkService::GetIPacketFirewall()
{
	ZKASSERT( m_pFirewall ) ;
	return m_pFirewall ;
}

IRouteManager* 
ZkService::GetIRouteManager()
{
	ZKASSERT( m_pRouteManager ) ;
	return m_pRouteManager ;
}

IRoutePacket*
ZkService::GetIRoutePacket()
{
	ZKASSERT( m_pRouteManager ) ;
	return m_pRouteManager->GetRoutePacketHandler() ;
}

IRoutePicker*
ZkService::GetIRoutePicker()
{
	ZKASSERT( m_pRoutePicker ) ;
	return m_pRoutePicker ;
}


INetworkPerformance* 
ZkService::GetINetworkPerformance()
{
	ZKASSERT( m_pNetworkPerformance ) ;
	return m_pNetworkPerformance ;
}

IScriptServer*
ZkService::GetIScriptServer()
{
	ZKASSERT( m_pScriptServer ) ;
	return m_pScriptServer ;
}

IWallet*
ZkService::GetIWallet()
{
	ZKASSERT( m_pWallet ) ;
	return m_pWallet ;
}

ICoreServerConnect*
ZkService::GetICoreServerConnect()
{
	ZKASSERT( m_pCoreServerConnect ) ;
	return m_pCoreServerConnect ;
}

IWordScan*
ZkService::GetIWordScan()
{
	ZKASSERT( m_pWordScan ) ;
	return m_pWordScan ;
}

IClientInfo*
ZkService::GetIClientInfo()
{
	ZKASSERT( m_pClientInfo ) ;
	return m_pClientInfo ;
}


#ifdef WIN32
///////////////////////////////////////////////////////////
//
// DLL module information
//
//
// Only reason this is here is to remember a handle so that 
// we can load string out of the RC file.

int APIENTRY 
DllMain(
	HANDLE hModule,
    unsigned long dwReason,
    void* lpReserved)
{
	if (dwReason == DLL_PROCESS_ATTACH)
	{
		g_hModule = (HINSTANCE)hModule ;		
	}
	return ZK_TRUE ;
}

#endif
