// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// ZkService.h
// 
// This object provides access to various central
// services useful for the client UI and other higher
// level components.
//
// Example of usage:
// To get a key call ZkService::GetIKeyQry()->GetKey()
//
// All access to these services is through interfaces,
// caller does not need to create any objects
//
// To access a interface method from a CPP file
// add #include <ZkService/ZkService.h>
// and the specific interface file for example <ZkService/ICrypto.h>

#ifndef _ZKSERVICE_H
#define _ZKSERVICE_H


#include "ZkServiceInc.h"

// Forward declaration of Interfaces
class IKeyQry ;
class ICrypto ;
class INiqs ;
class INymCentral ;
class IClientCentral ;
class ITokenService ;
class ITokenDesc ;
class IClientPersist ;
class ICookieManager ;
class IAdBlocker ;
class IFirewall ;
class IPacketFirewall ;
class IRouteManager ;
class IRoutePacket ;
class INetworkPerformance ;
class IRoutePicker ;
class IScriptServer ;
class IWallet ;
class ICoreServerConnect ;
class IWordScan ;
class IClientInfo ;

// Forward declaration of Classes
class RKeyQry ;
class RCryptoService ;
class RNiqsService ;
class RNymCentral ;
class RClientCentral ;
class RCookieManager ;
class RAdBlocker ;
class RFirewall ;
class RRouteManager ;
class RClientPersist ;
class RClientPersistOldWinVer ;
class RNetworkPerformance ;
class RRoutePicker ;
class RScriptServer ;
class RWallet ;
class RCoreServerConnect ;
class RTokenDesc ;
class RWordScan ;
class RClientInfo ;

class ZkService_API ZkService 
{
public:
	// Call this function at the launch of your
	// application which uses ZkService
	static void Init() ;
	
	// Call this function at the termination of
	// your application, to make sure that
	// all resources are deallocated
	static void Term() ;

	// Call these functions to access the various services
	// These functions guarantee a valid interface pointer

	static IKeyQry*				GetIKeyQry() ;
	static ICrypto*				GetICrypto() ;
	static INiqs*				GetINiqs() ;
	static INymCentral*			GetINymCentral() ;
	static IClientCentral*		GetIClientCentral() ;
	static ITokenService*		GetITokenService() ;
	static ITokenDesc*			GetITokenDesc() ;
	static IClientPersist*		GetIClientPersist() ;
	static ICookieManager*		GetICookieManager() ;
	static IAdBlocker*			GetIAdBlocker() ;
	static IFirewall*			GetIFirewall() ;
	static IPacketFirewall*		GetIPacketFirewall() ;
	static IRouteManager*		GetIRouteManager() ;
	static IRoutePacket*		GetIRoutePacket() ;
	static IRoutePicker*		GetIRoutePicker() ;
	static INetworkPerformance* GetINetworkPerformance() ;
	static IScriptServer*		GetIScriptServer() ;
	static IWallet*				GetIWallet() ;
	static ICoreServerConnect*  GetICoreServerConnect() ;
	static IWordScan*           GetIWordScan() ;
	static IClientInfo*         GetIClientInfo() ;
private:
	static RKeyQry*		   m_pKeyQryInstance ;
	static RCryptoService* m_pCryptoInstance ;
	static RNiqsService*   m_pNiqsInstance ;
	static RNymCentral*    m_pNymCentral ;
	static RClientCentral* m_pClientCentral ;
	static RClientPersist* m_pClientPersist ;
	static RCookieManager* m_pCookieManager ;
	static RAdBlocker*	   m_pAdBlocker ;
	static RFirewall*      m_pFirewall ;
	static RRouteManager*  m_pRouteManager ;
	static RRoutePicker*   m_pRoutePicker ;
	static RClientPersistOldWinVer* m_pClientPersistOldWinVer ;
	static RNetworkPerformance* m_pNetworkPerformance ;
	static RScriptServer*  m_pScriptServer;
	static RWallet*		   m_pWallet;
	static RCoreServerConnect* m_pCoreServerConnect;
	static RTokenDesc*     m_pTokenDesc ;
	static RWordScan*      m_pWordScan ;
	static RClientInfo*    m_pClientInfo ;
} ;

#endif 
