// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// ZKService.cpp : 
//

#ifdef WIN32 // XXX Temporary (bootstrap trfilter on linux) -- ol
#  include "StdAfx.h"
#  include "ZkService.h"
#  include "Rkeyqry.h"
#  include "RCryptoService.h"
#  include "RNiqsService.h"
#  include "RNymCentral.h"
#  include "RClientCentral.h"
#  include "RClientPersist.h"
#  include "RClientPersistOldWinVer.h"		// Temporary
#  include "RTokenService.h"
#  include "RCookieManager.h"
#  include "RAdBlocker.h"
#  include <sstream>
#  include <ZKRegistry.h>
#else
#  include "ZkServiceLinux.h"
#  include "Rkeyqry.h"
#  include "RNiqsService.h"
#  include "RFirewall.h"
#  include "RRouteManager.h"
#  include <sstream>
#endif

RKeyQry*        ZkService::m_pKeyQryInstance = NULL ;
RNiqsService*   ZkService::m_pNiqsInstance	 = NULL ;
RFirewall*      ZkService::m_pFirewall       = NULL ;
RRouteManager*  ZkService::m_pRouteManager   = NULL ;

#ifdef WIN32 
RCryptoService* ZkService::m_pCryptoInstance = NULL ;
RNymCentral*    ZkService::m_pNymCentral	 = NULL ;
RClientCentral* ZkService::m_pClientCentral	 = NULL ;
RClientPersist* ZkService::m_pClientPersist  = NULL ;
RCookieManager* ZkService::m_pCookieManager  = NULL ;
RAdBlocker*		ZkService::m_pAdBlocker		 = NULL ;

RClientPersistOldWinVer* ZkService::m_pClientPersistOldWinVer  = NULL ;
#endif

void 
ZkService::Init()
{
	ZKASSERT( m_pKeyQryInstance == NULL ) ;  // Calling init() twice?  Idiot?
	ZKASSERT( m_pNiqsInstance == NULL ) ;
	ZKASSERT( m_pFirewall == NULL ) ;
	ZKASSERT( m_pRouteManager == NULL ) ;

#ifdef WIN32
	ZKASSERT( m_pCryptoInstance == NULL ) ;
	ZKASSERT( m_pNymCentral == NULL ) ;
	ZKASSERT( m_pClientCentral == NULL ) ;
	ZKASSERT( m_pClientPersist == NULL ) ;
	ZKASSERT( m_pClientPersistOldWinVer == NULL ) ;
	ZKASSERT( m_pCookieManager == NULL ) ;
	ZKASSERT( m_pAdBlocker == NULL ) ;
#endif

	m_pKeyQryInstance  = new RKeyQry;
	m_pNiqsInstance    = new RNiqsService;
	m_pFirewall        = new RFirewall;
	m_pRouteManager    = new RRouteManager;

#ifdef WIN32
	m_pCryptoInstance  = new RCryptoService;
	m_pNymCentral	   = new RNymCentral;
	m_pClientCentral   = new RClientCentral;
	m_pCookieManager   = new RCookieManager;
	m_pAdBlocker	   = new RAdBlocker;

	if ( ZKRegistry::GetFlag( "OldFileFormat" ) )
	{
		m_pClientPersistOldWinVer   = new RClientPersistOldWinVer;
	}
	else
	{
		m_pClientPersist   = new RClientPersist;
	}
#endif
}

void 
ZkService::Term()
{
	SAFEDELETE( m_pKeyQryInstance );
	SAFEDELETE( m_pNiqsInstance );
	SAFEDELETE( m_pFirewall ) ;
	SAFEDELETE( m_pRouteManager ) ;

#ifdef WIN32
	SAFEDELETE( m_pCryptoInstance );
	SAFEDELETE( m_pNymCentral ) ;
	SAFEDELETE( m_pClientCentral ) ;
	SAFEDELETE( m_pClientPersist ) ;
	SAFEDELETE( m_pClientPersistOldWinVer ) ;
	SAFEDELETE( m_pCookieManager ) ;
	SAFEDELETE( m_pAdBlocker ) ;
#endif
}

IKeyQry* 
ZkService::GetIKeyQry()
{
	ZKASSERT( m_pKeyQryInstance ) ;
	return m_pKeyQryInstance ;
}

INiqs* 
ZkService::GetINiqs()
{
	ZKASSERT( m_pNiqsInstance ) ;
	return m_pNiqsInstance ;
}

IFirewall* 
ZkService::GetIFirewall()
{
	ZKASSERT( m_pFirewall ) ;
	return m_pFirewall ;
}

IRouteManager* 
ZkService::GetIRouteManager()
{
	ZKASSERT( m_pRouteManager ) ;
	return m_pRouteManager ;
}


#ifdef WIN32
ICrypto* 
ZkService::GetICrypto()
{
	ZKASSERT( m_pCryptoInstance ) ;
	return m_pCryptoInstance ;
}


INymCentral* 
ZkService::GetINymCentral()
{
	ZKASSERT( m_pNymCentral ) ;
	return m_pNymCentral ;
}

IClientCentral* 
ZkService::GetIClientCentral()
{
	ZKASSERT( m_pClientCentral ) ;
	return m_pClientCentral ;
}

ITokenService* 
ZkService::GetITokenService()
{
	ZKASSERT( m_pClientCentral ) ;
	return m_pClientCentral->GetTokenService() ;
}

IClientPersist* 
ZkService::GetIClientPersist()
{
	if ( m_pClientPersist ) 
	{
		ZKASSERT( m_pClientPersist ) ;
		return m_pClientPersist ;
	}
	else
	{
		ZKASSERT( m_pClientPersistOldWinVer ) ;
		return m_pClientPersistOldWinVer ;
	}
}

ICookieManager* 
ZkService::GetICookieManager()
{
	ZKASSERT( m_pCookieManager ) ;
	return m_pCookieManager ;
}

IAdBlocker* 
ZkService::GetIAdBlocker()
{
	ZKASSERT( m_pAdBlocker ) ;
	return m_pAdBlocker  ;
}

#endif

