// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// ZKService.h
// 
// This object provides access to various central
// services useful for the client UI and other higher
// level components.
//
// Example of usage:
// To get a key call ZkService::IKeyQry()->GetKey()
//
// All access to these services is through interfaces,
// caller does not need to create any objects
//
// To access a interface method from a CPP file
// add #include <ZkService/ZkService.h>
// and the specific interface file for example <ZkService/ICrypto.h>

#ifndef _ZKSERVICE_H
#define _ZKSERVICE_H


#include "ZkServiceInc.h"

// Forward declaration of Interfaces
class IKeyQry ;
class INiqs ;
class IFirewall ;
class IRouteManager ;
class IRoutePacket ;

#ifdef WIN32
class ICrypto ;
class INymCentral ;
class IClientCentral ;
class ITokenService ;
class IClientPersist ;
class ICookieManager ;
class IAdBlocker ;
class IFirewall ;
#endif

// Forward declaration of Classes
class RKeyQry ;
class RNiqsService ;
class RFirewall ;
class RRouteManager ;

#ifdef WIN32
class RCryptoService ;
class RNymCentral ;
class RClientCentral ;
class RCookieManager ;
class RAdBlocker ;
class RRouteManager ;
class RClientPersist ;
class RClientPersistOldWinVer ;
#endif

class ZkService_API ZkService 
{
public:
	// Call this function at the launch of your
	// application which uses ZkService
	static void Init() ;
	
	// Call this function at the termination of
	// your application, to make sure that
	// all resources are deallocated
	static void Term() ;

	// Call these functions to access the various services
	// These functions guarantee a valid interface pointer

	static IKeyQry*		   GetIKeyQry() ;
	static INiqs*		   GetINiqs() ;
	static IFirewall*      GetIFirewall() ;
	static IRouteManager*  GetIRouteManager() ;
	static IRoutePacket*   GetIRoutePacket() ;

#ifdef WIN32
	static ICrypto*		   GetICrypto() ;
	static INymCentral*    GetINymCentral() ;
	static IClientCentral* GetIClientCentral() ;
	static ITokenService*  GetITokenService() ;
	static IClientPersist* GetIClientPersist() ;
	static ICookieManager* GetICookieManager() ;
	static IAdBlocker*	   GetIAdBlocker() ;
#endif
private:
	static RKeyQry*		   m_pKeyQryInstance ;
	static RNiqsService*   m_pNiqsInstance ;
	static RFirewall*      m_pFirewall ;
	static RRouteManager*  m_pRouteManager ;

#ifdef WIN32
	static RCryptoService* m_pCryptoInstance ;
	static RNymCentral*    m_pNymCentral ;
	static RClientCentral* m_pClientCentral ;
	static RClientPersist* m_pClientPersist ;
	static RCookieManager* m_pCookieManager ;
	static RAdBlocker*	   m_pAdBlocker ;
	static RClientPersistOldWinVer* m_pClientPersistOldWinVer ;
#endif
} ;

#endif 
