#!/bin/bash
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

FREEDOM_HOME=/usr/share/freedom
FREEDOM_USER=$1
IFCONFIG=ifconfig
LDCONFIG=ldconfig
LSMOD=lsmod
IPCHAINS=ipchains
ROUTE=route
KERNELVERSION=`uname -r`
LIBERROR=0

#bug 8293 stat not present on Caldera or SuSE
if [ -f /usr/bin/stat ]
then
	FILECHECK=stat
else
	FILECHECK='ls -l'
fi

function check_lib() {
	LIBEXIST=`grep $1 /tmp/ldconfig.tmp` 
	if [ $? -ne 0 ] ; then
	  echo $1 not found
          LIBERROR=1
	else
          LIBEXIST=`grep $2 /tmp/ldconfig.tmp`
	  if [ $? -ne 0 ] ; then
	    echo $1 found, wrong version $LIBEXIST
          fi
	fi
}

if [ -z "$1" ]; then
  echo "Usage: $0 <freedom user>"
  exit
fi

$FREEDOM_HOME/freedom --version 2>&1
echo

uname -a
cat /proc/version 2>&1
echo

$IFCONFIG
echo

$ROUTE
echo

$LDCONFIG -v
$LDCONFIG -v >> /tmp/ldconfig.tmp
echo

$LSMOD
echo

$FILECHECK /dev/clientshim 2>&1
echo
$FILECHECK /lib/modules/$KERNELVERSION/fs/zkshim.o 2>&1
echo
$FILECHECK /usr/share/freedom/zkshim.o 2>&1
echo
echo

ls -Lla `grep "^$FREEDOM_USER:" /etc/passwd | awk -F : '{print $6}'`/.freedom 2>&1
echo

ls -Lla /usr/share/freedom 2>&1
echo

rpm -qa 2>&1
echo

cat /proc/ksyms 2>&1
echo
cat /usr/src/linux/.config 2>&1
echo

su - $FREEDOM_USER -c "/usr/share/freedom/zkreged -l"
echo

$IPCHAINS -L
echo

check_lib libgnomeui libgnomeui.so.32
check_lib libart_lgpl libart_lgpl.so.2
check_lib libgdk_imlib libgdk_imlib.so.1
check_lib libSM libSM.so.6
check_lib libICE libICE.so.6
check_lib libgtk libgtk-1.2.so.0
check_lib libgdk libgdk-1.2.so.0
check_lib libgmodule libgmodule-1.2.so.0
check_lib libXi libXi.so.6
check_lib libXext libXext.so.6
check_lib libX11 libX11.so.6
check_lib libgnome libgnome.so.32
check_lib libgnomesupport libgnomesupport.so.0
check_lib libesd libesd.so.0
check_lib libaudiofile libaudiofile.so.0
check_lib libm libm.so.6
check_lib libdb libdb.so.2
check_lib libglib libglib-1.2.so.0
check_lib libdl libdl.so.2
check_lib libgthread libgthread-1.2.so.0
check_lib libpthread libpthread.so.0
check_lib libc libc.so.6
check_lib libtiff libtiff.so.3
check_lib libjpeg libjpeg.so.62
check_lib libpng libpng.so.2
check_lib libxml libxml.so.1
check_lib libz libz.so.1
check_lib ld-linux ld-linux.so.2

if [ $LIBERROR -eq 0 ]; then
  echo No problem with libraries
  echo
fi

rm -f /tmp/ldconfig.tmp

