// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RZkContext.cpp

#include "RZkContext.h"
#ifndef WIN32
#include <netdb.h>
#endif

#include <zkDNS.h>

RZkContext::RZkContext()
		: m_Action( ZKPROXY_DO_NOTHING )
		, m_ByteBufferLength( 0 )
		, m_MemBuf(NULL)
		, m_bCloseConnection(false)
{
	memset( &m_RemoteAddress, 0, sizeof( m_RemoteAddress ) );
}


void RZkContext::ResetContext		()
{
	m_Action = ZKPROXY_DO_NOTHING;
	m_ByteBufferLength = 0;
	if(m_MemBuf)
	{
		zkMemBufFree(GetZkError(), &m_MemBuf);
		m_MemBuf = NULL;
	}
	m_ProtocolOutput = NULL;
	m_ProtocolOutputLength = 0;


}


void RZkContext::SetRemoteAddress( ZkSocketAddress& in_RemoteAddress )
{
	memcpy( &m_RemoteAddress, &in_RemoteAddress, sizeof( ZkSocketAddress ) );
    m_strRemoteDomain = "";
}


const std::string& RZkContext::GetRemoteDomainName	()
{
	if( m_strRemoteDomain.length() > 0 )
	{
		return m_strRemoteDomain;
	}

	if( ((struct sockaddr_in*)&m_RemoteAddress )->sin_family != AF_INET )
	{
		ZKASSERT(0);
		return m_strRemoteDomain;		//should be empty
	}

	// Do DNS lookup
	in_addr		iaHost = {0};	

	iaHost = ( (struct sockaddr_in*)&m_RemoteAddress )->sin_addr;

    const hostent* pHostEntry = gethostbyaddr((const char *)&iaHost, sizeof(in_addr), AF_INET); /* audit: ok */

	if(!pHostEntry)
    {
		//failed to get a domain name, so using IP address.
		m_strRemoteDomain = inet_ntoa(iaHost);
	}
	else
	{
		ZKASSERT(pHostEntry);
        m_strRemoteDomain = pHostEntry->h_name;
	}
	
	return m_strRemoteDomain;
}


void RZkContext::VerifyState() const
{
	ZKASSERT(	GetProxyAction() == ZKPROXY_DO_NOTHING ||
				GetProxyAction() == ZKPROXY_SEND_TO_CLIENT ||
				GetProxyAction() == ZKPROXY_SEND_TO_SERVER ||
				GetProxyAction() == ZKPROXY_CONNECT_TO_SERVER);
	
	ZKASSERT( GetByteBufferLength() <= ByteBufferSize() );
}
