// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RZkContext.h

#ifndef _RZKCONTEXT_H
#define _RZKCONTEXT_H


#include <DZkError.h>
#include "IZkContext.h"
#include <zksocket.h>
#include "ZkProxiesInc.h"

#define BYTE_BUFFER_SIZE 2048

class RZkContext : public IZkContext
{
public:
	RZkContext();
	
 	void*		ByteBuffer			() const { return (void *)m_ByteBuffer; }
 	size_t		ByteBufferSize		() const { return BYTE_BUFFER_SIZE; }
 	size_t		GetByteBufferLength	() const { return m_ByteBufferLength; }
 	void		SetByteBufferLength	(size_t in_BufferLength) { m_ByteBufferLength = in_BufferLength; }

	void*		GetProtocolOutput	() const { return m_ProtocolOutput; }
	void		SetProtocolOutput	(void* in_ProtocolOutput) { m_ProtocolOutput = in_ProtocolOutput; }

	size_t		GetProtocolOutputLength() const { return m_ProtocolOutputLength; }
	void		SetProtocolOutputLength(size_t in_ProtocolOutputLength) { m_ProtocolOutputLength = in_ProtocolOutputLength; }

	void		SetCloseConnection(bool in_bClose) { m_bCloseConnection = in_bClose; }
	bool		GetCloseConnection() const { return m_bCloseConnection; }

	ZkMemBuf&	GetZkMemBuf			() { return m_MemBuf; }

	int			GetProxyAction		() const { return m_Action; }
	void		SetProxyAction		(int in_Action) { m_Action = in_Action; }
	void		ResetContext		();
	void		SetRemoteAddress( ZkSocketAddress& );
	ZkSocketAddress& GetRemoteAddress( )  { return m_RemoteAddress; }
	const ZKSTD::string& GetRemoteDomainName ();
	void		SetRemoteDomainName	(const ZKSTD::string& in_strRemoteDomain) { m_strRemoteDomain = in_strRemoteDomain; }
	virtual DZkError*	GetZkError	() { return &m_err; }
	void		VerifyState	() const;
	
private:
	bool					m_bCloseConnection;
	char					m_ByteBuffer[BYTE_BUFFER_SIZE];
	int						m_ByteBufferLength;
	int						m_Action;
	ZKSTD::string			m_strRemoteDomain;
	ZkSocketAddress			m_RemoteAddress;
	DZkError				m_err;
	ZkMemBuf				m_MemBuf;
	void*					m_ProtocolOutput;
	int						m_ProtocolOutputLength;
};

#endif //_RZKCONTEXT_H
