// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RZkHttpProtocol.h

#ifndef _RZKHTTPPROTOCOL_H
#define _RZKHTTPPROTOCOL_H

#include <zkMIME.h>
#include "IZkProtocol.h"
#include <string>
#include <clientstl.h>
#include <list>

class RZkHttpProtocol : public IZkProtocol
{
public:
	RZkHttpProtocol();

    bool ConnectFailed              (IZkContext&);
	bool ClientReady				(IZkContext&);
	bool FreedomizeClientRequest	(IZkContext&);
	bool FreedomizeServerResponse	(IZkContext&);
	bool InitProtocolResources		(IZkContext&);
	bool FreeProtocolResources		(IZkContext&);
private:
	bool ResetRequestHeader			(IZkContext& in_Context);
	bool ProcessRequest				(IZkContext& in_Context);
	bool ManageOutgoingCookies		(IZkContext& in_Context);
	bool PlaceBogusResponseInContext(IZkContext& in_Context);
	bool ShouldBlockRequest			(IZkContext& in_Context);
	bool PlaceFreedomizedRequestInContext(IZkContext& in_Context);

	void GetRequestUri(ZKSTD::string& out_strUri, IZkContext& in_Context);
	int FindHeaderEnd(const char *in_pData, const size_t in_len);
	int FindLineEnd(const char *in_pData, const size_t in_len);
	void SanitizeIncomingHeader(IZkContext& in_Context, ZKSTD::string& io_strHdr);
	void SanitizeResponseBody(char *in_pBody, size_t in_lenBody);
	void AppendToFreedomizedResponse(IZkContext& in_Context, const char *in_pData, const size_t in_len);
	int GetResponseStatusCode(ZKSTD::string& in_strHdr);
	void DoneWithOneResponse(IZkContext& in_Context);

	ZKSTD::string m_strDebugSuffix;
	void TraceIncomingResponse(IZkContext& in_Context);
	void TraceIncomingRequest(IZkContext& in_Context);
	void TraceOutgoing(IZkContext& in_Context);
	void AppendWithCommentary(const char *in_pData, const size_t in_len);
	void AppendWithCommentary(const char *in_strData);
	void AppendWithCommentary(const ZKSTD::string& in_strData);

	enum {
		READING_RESPONSE_HEADER,
		
		READING_CHUNK_HEADER,
		READING_CHUNK_BODY,
		READING_FOOTER,

		READING_RESPONSE_WITH_CONTENT_LENGTH,

		READING_UNTERMINATED_RESPONSE
	};

	int m_ResponseState;
	size_t m_lenLeftToRead;

	ZkHTTPHeader	m_RequestHeader;

	ZKSTD::string m_strCurPart;

	ZKSTD::list<ZKSTD::string> m_listUris;

};

#endif // _RZKHTTPPROTOCOL_H
