// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#ifdef WIN32
#pragma warning (disable: 4251)
#endif

#include "RZkIrcProtocol.h"
#include <vector>
#include <ZkService.h>
#include <IWordScan.h>
#include <INymCentral.h>
#include <INym.h>
#include <ZkProxyDialogs.h>
#include <IZkConsoleMsg.h>

bool RZkIrcProtocol::ConnectFailed(IZkContext& in_Context)
{
    in_Context.SetCloseConnection(true);
    return true;
}

bool RZkIrcProtocol::ClientReady(IZkContext& in_Context)
{
	in_Context.SetProxyAction( IZkContext::ZKPROXY_CONNECT_TO_SERVER );

	ZKSTD::string strIP(inet_ntoa(in_Context.GetRemoteAddress().m_inet.sa.sin_addr));
	INym *pNym;
	ZkService::GetINymCentral()->GetActiveNym(IID_INYM, (void**)&pNym);
	if ( ZkService::GetINymCentral()->IsCurrentNymUser() 
		|| !pNym->DoesNymUseCloud()) 
	{
		IZkConsoleMsg::ShowMsg( IDS_CONS_IRC_NON_ANON_CONNECT, strIP.c_str() );
	}
	else
	{
		IZkConsoleMsg::ShowMsg( IDS_CONS_IRC_ANON_CONNECT, strIP.c_str() );
	}

	return true;
}

bool RZkIrcProtocol::FreedomizeClientRequest	(IZkContext& in_Context)
{
	ZKSTD::vector<ZKSTD::string> vecstrOffensiveWordsFound;

	in_Context.SetProxyAction( IZkContext::ZKPROXY_SEND_TO_SERVER);

	in_Context.SetProtocolOutput(in_Context.ByteBuffer());
	in_Context.SetProtocolOutputLength(in_Context.GetByteBufferLength());
	
	// Check for Freedom offensive material in the body of the message											
	if( ZkService::GetIWordScan()->WordScan(
			(unsigned char*)in_Context.GetProtocolOutput(),
			in_Context.GetProtocolOutputLength(),
			vecstrOffensiveWordsFound) 
			)
	{
					
		WordScanSummary Result ;
		Result.pScannedBuffer = (unsigned char*)in_Context.GetProtocolOutput();
		Result.cntBuffer = in_Context.GetProtocolOutputLength();
		Result.pFoundWords = &vecstrOffensiveWordsFound ;
		Result.strDomainName = in_Context.GetRemoteDomainName();
		
		Result.m_SockAddr = *( (sockaddr*)(&in_Context.GetRemoteAddress().m_inet.sa) );

		// Ask the user if they want to send this data out

        if( !ZkProxyDialogs::LaunchWordScanMsgDlg( PROTOCOL_IRC, &Result ) )
        {
            in_Context.SetProxyAction(IZkContext::ZKPROXY_DO_NOTHING);
        }

	}
	return true;
}

bool RZkIrcProtocol::FreedomizeServerResponse	(IZkContext& in_Context)
{
	in_Context.SetProxyAction( IZkContext::ZKPROXY_SEND_TO_CLIENT );

	in_Context.SetProtocolOutput(in_Context.ByteBuffer());
	in_Context.SetProtocolOutputLength(in_Context.GetByteBufferLength());
	return true;
}
