// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RZkNntpProtocol.h

#ifndef _RZKNNTPPROTOCOL_H
#define _RZKNNTPPROTOCOL_H


#include "IZkProtocol.h"
#include <zkMIME.h>
#include <DZkError.h>
#include <WSyncTypes.h>
#include <vector>
#include <string>

class RZkNntpProtocol: public IZkProtocol
{
public:
	RZkNntpProtocol();

    bool ConnectFailed              (IZkContext&);
	bool ClientReady				(IZkContext&);
	bool FreedomizeClientRequest	(IZkContext&);
	bool FreedomizeServerResponse	(IZkContext&);
	bool InitProtocolResources		(IZkContext&);
	bool FreeProtocolResources		(IZkContext&);
private:
	// used for the wordscan function
	ZkEmail m_Email;
	int		m_State;
	bool	ResetState(IZkContext& in_Context);
	void	SetResponse(IZkContext& in_Context, const char* strResponse);
	void	ErrorResponse (IZkContext& in_Context);
	void	HandlePostCommand(IZkContext& in_Context);
	void	HandleMoreData(IZkContext& in_Context);
	void	HandleCompletePost(IZkContext& in_Context);
	void	SendPostCommandToServer(IZkContext& in_Context);
	void	SendMessageBodyToServer(IZkContext& in_Context);
	void	PassResponseThroughToClient(IZkContext& in_Context);

	void	PostAsUser(IZkContext& in_Context, const ZKSTD::vector<ZKSTD::string>& in_vecGroups);
	void	PostAsNym(IZkContext& in_Context, const ZKSTD::vector<ZKSTD::string>& in_vecGroups, size_t iOriginatingNym);

	bool	ScanPostForOffensiveWords(IZkContext& in_Context, size_t in_NymIndex);

	void NotifyConsoleOfPost(const ZKSTD::vector<ZKSTD::string>& in_vecGroups, const size_t in_iNym);
	
	void	TraceOutgoingStream(IZkContext& in_Context);


	static WZkMutex			m_gMutex;

	enum STATES
	{
		FRESH_STATE,
		RECEIVING_POST_DATA,
		SENT_POST_COMMAND,
		SENT_MESSAGE_BODY
	};
};

#endif // _RZKNNTPPROTOCOL_H
