// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RZkProxy.h


#ifndef _RZKPROXY_H
#define _RZKPROXY_H
#pragma warning (disable : 4251)

#include "IZkProxy.h"
#include "IZkProtocol.h"
#include <zksocket.h>
#include "RZkContext.h"
#include "ZkProxiesInc.h"

class ZkProxies_API RZkProxy : public IZkProxy
{
public:
	RZkProxy(ZkSocket in_LocalSocket, IZkProxy::PROXY_PROTOCOLS in_WhichProtocol);
	~RZkProxy();
	virtual bool RunProxy();

	static int GetSignalFD();
	static void SetSignalFD(int fd);

private:
	bool ReceiveRemoteAddress();
	bool ConnectToRemoteServer();
	bool RecvRealRequest();
	bool RecvRealResponse();

	bool SendFreedomizedData();
	bool SendFreedomizedRequest();
	bool SendFreedomizedResponse();

	// CTOR member init
	ZkSocket		m_LocalSocket;
	ZkSocket		m_RemoteSocket;
	IZkProtocol *	m_ZkProtocol;
	bool			m_bOpenedRemoteSocket;
	ZkList			m_listSockets; // This list holds m_LocalSocket first, and then m_RemoteSocket. The order is important !!! zkSocketWaitReadableOrClosed() is called and the order in ZkList is assumed.
	bool			m_bFailedOpenRemoteSocket;
	IZkProxy::PROXY_PROTOCOLS 				m_nProtocol;

	ZkSocketAddress m_RemoteAddress;
	RZkContext		m_ZkContext;
	static int		m_signalFD;

	RZkProxy() {};
	RZkProxy(RZkProxy&) {};
};

#endif // _RZKPROXY_H
