// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RZkSmtpProtocol.h

#ifndef _RZKSMTPPROTOCOL_H
#define _RZKSMTPPROTOCOL_H

#include <zkMIME.h>
#include <zkEmailUtil.h>
#include <zkMemBuf.h>
#include <string>
#include <vector>
#include <clientwin32WM.h>
#include <client.h>

#include "IZkProtocol.h"

class RZkSmtpProtocol : public IZkProtocol
{
public:
	RZkSmtpProtocol();
    
    bool ConnectFailed              (IZkContext&);
	bool FreedomizeClientRequest	(IZkContext&);
	bool FreedomizeServerResponse	(IZkContext&);
	bool InitProtocolResources		(IZkContext&);
	bool FreeProtocolResources		(IZkContext&);
	bool ClientReady				(IZkContext&);

private:

	int		ParseCommand(IZkContext& in_Context);

	void	SetResponse(IZkContext& in_Context, const char* strResponse);
	void	ErrorResponse (IZkContext& in_Context);
	bool	ResetState(IZkContext& in_Context);

	void	HandleMailCommand(IZkContext& in_Context);
	void	HandleRcptCommand(IZkContext& in_Context);
	void	HandleDataCommand(IZkContext& in_Context);
	void	HandleMoreData(IZkContext& in_Context);
	void	HandleCompleteMail(IZkContext& in_Context);

	bool	GetSmtpCommandArguments(IZkContext& in_Context);

	void	SendMailAsNym(IZkContext& in_Context, size_t in_NymIndex );
	void	SendMailAsUser(IZkContext& in_Context);
	static bool	IsFreedomRecipient(const ZKSTD::string& in_strSenderDomain, const ZKSTD::string& in_strRcptDomain);

	void	SendHeloCommandToServer(IZkContext& in_Context);
	void	SendMailCommandToServer(IZkContext& in_Context);
	void	SendRcptCommandToServer(IZkContext& in_Context);
	void	SendDataCommandToServer(IZkContext& in_Context);
	void	SendRsetCommandToServer(IZkContext& in_Context);
	void	SendMailBodyToServer(IZkContext& in_Context);
	void	SendQuitCommandToServer(IZkContext& in_Context);

	bool	ScanEmailForOffensiveWords(IZkContext& in_Context, size_t in_NymIndex);
	bool	ChangeFromFieldToFreedom(IZkContext& in_Context, size_t in_NymIndex);
	bool	ScanForRealSignature(IZkContext& in_Context);
        bool	CheckMailCount( IZkContext& in_Context, size_t in_NymIndex);
	void	NotifyConsoleOfMail(const size_t in_iNym);
	void	TraceOutgoingStream(IZkContext& in_Context);
	bool	MsgIsReadReceipt(IZkContext& in_Context);

	ZkEmail m_Email;
	ZkMemBuf m_MemBuf;
	ZKSTD::string m_strMailFromArg;
	ZKSTD::vector<ZKSTD::string> m_vecRecipients;
	int m_State;
	bool m_bAlreadyConnected;
	bool m_bLinuxRelayHack;
	ZKSTD::string m_strLine;
	enum STATES
	{
		FRESH_STATE,
		RECEIVED_MAIL_CMD_STATE,
		RECEIVED_RCPT_CMD_STATE,
		RECEIVING_DATA_STATE,

		CONNECTING_TO_SERVER,
		SENT_HELO_CMD,
		SENT_MAIL_FROM_CMD,
		SENT_RCPT_TO_CMD,
		SENT_DATA_CMD,
		SENT_MAIL_BODY,
		SENT_RSET_CMD,

		SENT_QUIT_CMD
	};

	enum COMMANDS
	{
		HELO_CMD,
		EHLO_CMD,
		MAIL_CMD,
		RCPT_CMD,
		DATA_CMD,
		VRFY_CMD,
		EXPN_CMD,
		QUIT_CMD,
		RSET_CMD,
		NOOP_CMD,
		HELP_CMD,
		OTHER_CMD
	};
};

#endif // _RZKSMTPPROTOCOL_H
