// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#pragma warning (disable:4251)

#include "ZkEmailHelp.h"
#include <string_hlp.h>
#include <ZkService.h>
#include <INymCentral.h>
#include <IKeyQry.h>
#include <frSec.h>
#include <INym.h>
#include <INymMail.h>
#include <IClientInfo.h>
#include <INiqs.h>
#include <frmailmsg.h>
#include <zkBase64.h>
#include <sstream>
#include <EsmtpClient.h>
#include <zkCStr.h>


#define ZKTestAndThrowEncryption(zkerr, strRecip, bStatus) { if(!( bStatus)) { throw RZkException(zkerr, __function,"Unexpected Failure",IDS_FAILED_TO_ENCRYPT_MAIL_FOR_RECIPIENT, strRecip); }}

void ZkEmailHelp::GenerateBounceMessage(const char *in_strOrigMsg,
										ZKSTD::string strRecipientAddress,
										const size_t in_iSendingNym,
										ZKSTD::string& out_strBounceMsg)
{
	ZKSTD::ostringstream oss;

	ZKSTD::string strNymName;
	ZkService::GetINymCentral()->GetNymName(in_iSendingNym, strNymName);
	INymMail * pNymMail;
	ZkService::GetINymCentral()->GetNym(IID_IMAIL,(void**)&pNymMail, in_iSendingNym);
	INym *pNym;
	ZkService::GetINymCentral()->GetNym(IID_INYM,(void**)&pNym, in_iSendingNym);

	ZKSTD::string strEmailDomain;
	pNym->GetNymEmailDomain(strEmailDomain);


	ZKSTD::string strSubject;
	ZKSTD::string strDate;
	if(!ZKS::extractHeaderField(in_strOrigMsg, strlen(in_strOrigMsg), "Subject", strSubject))
	{
		strSubject = "[Subject Unknown]";
	}
	if(!ZKS::extractHeaderField(in_strOrigMsg, strlen(in_strOrigMsg), "Date", strDate))
	{
		strDate = "[Date Unknown]";
	}

	const char *strMsgBody = strstr(in_strOrigMsg, "\r\n\r\n");
	const char *strEndOfDigest;
	if(!strMsgBody)
	{
		strEndOfDigest = in_strOrigMsg + strlen(in_strOrigMsg) - 1;
	}
	else
	{
		strEndOfDigest = strMsgBody + 4;
		while(*strEndOfDigest == '\r' || *strEndOfDigest == '\n')
		{
			strEndOfDigest++;
		}

		for(int i = 0; i < 5; i++)
		{
			const char *temp = strchr(strEndOfDigest, '\n');
			if(!temp)
			{
				break;
			}
			strEndOfDigest = temp + 1;
		}
	}

	ZKSTD::string strMsgDigest(in_strOrigMsg, (strEndOfDigest - in_strOrigMsg));

	oss << "To: " << strNymName << '@' << strEmailDomain << "\r\n"
		<< "From: support@freedom.net\r\n"
		<< "Subject: Message delivery failure\r\n\r\n"

		<< "The message titled \"" << strSubject << "\" you sent on "
		<< strDate << " could not be delivered to " << strRecipientAddress
		<< ".\r\n\r\n"
        << "This may have occurred because the recipient is temporarily unable to recieve "
        << "mail or because of temporary problems with the mail system.\r\n\r\n"
        << "Please verify the recipient and try re-sending the message.\r\n\r\n"

		<< "   ----- Original message -----\r\n"
		<< strMsgDigest;
	out_strBounceMsg = oss.str();
	ZKTRACE1("Bounce Msg:\r\n%s\r\n", out_strBounceMsg.c_str());
}

bool ZkEmailHelp::SendEmailESMTP(DZkError* in_perr,
								ZKSTD::vector<ZKSTD::string>& in_vecInetRecipients,
								ZKSTD::vector<ZKSTD::string>& in_vecFreedomRecipients,
								const char* in_strOrigMsg,
								const size_t in_iSendingNym)
{
	ZKASSERT(in_perr);

	bool ReturnValue = false;

	FrCryptPubKey PrivSigningKey = NULL;
	INym * pNym = NULL ;
	INymMail *pNymMail = NULL;

	ZKSTD::string strSenderAddress;
	ZKSTD::string strNymFreedomDomain;

	ZkService::GetINymCentral()->GetNym( IID_INYM, (void**) &pNym, in_iSendingNym ) ;
	pNym->GetSignatureKey( &PrivSigningKey );

	pNym->GetNymEmailDomain(strNymFreedomDomain);
	pNym->GetNymName(strSenderAddress);
	
	strSenderAddress += '@' + strNymFreedomDomain;

	ZKSTD::string strAuthSecretCert;
	pNym->QueryInterface(IID_IMAIL, (void **)&pNymMail);
	pNymMail->GetMailCertificate(strAuthSecretCert);

	if(!strAuthSecretCert.length())
	{
		if(pNymMail->UpdateMailCertificate())
		{
			pNymMail->GetMailCertificate(strAuthSecretCert);
		}
	}

	if(!strAuthSecretCert.length())
	{
		ReturnValue = false;
		ZKASSERT(0);
	}
	else
	{
		bool bRetry = false;
		do
		{
			try
			{
				ZkSocketAddress RemoteAddr;
				SOCKADDR_IN sa;

				ZkService::GetINiqs()->GetNmtaAddr(&sa);
				memcpy( &RemoteAddr, &sa, sizeof (SOCKADDR_IN) );
				
				ZKSTD::string strFreedomDomain; // May be different from the nym freedom domain

				ZkService::GetIClientInfo()->GetFreedomDomain(strFreedomDomain);

				EsmtpClient Mailer(strFreedomDomain, strSenderAddress, strAuthSecretCert,
									RemoteAddr,PrivSigningKey);
				size_t i;

				if(in_vecInetRecipients.size())
				{
					// Inet Recipients
					for(i = 0; i < in_vecInetRecipients.size(); i++)
					{
						Mailer.AddRecipient(in_vecInetRecipients[i]);
					}
					Mailer.SendMsgBody(in_strOrigMsg, strlen(in_strOrigMsg));
				}
				
				if(in_vecFreedomRecipients.size())
				{
					// Freedom Recipients
					for(i = 0; i < in_vecFreedomRecipients.size(); i++)
					{
						char *pCryptedMsg = NULL;
						size_t lenCryptedMsg = 0;
						Mailer.AddRecipient(in_vecFreedomRecipients[i]);
						EncryptMessageForFreedomRecipient(&pCryptedMsg, &lenCryptedMsg,
												in_strOrigMsg, in_vecFreedomRecipients[i],
													in_iSendingNym);
						
						// Since encrypted message has bad line termination (see bug 5515),
						//	it is necessary to correct line termination here...

						ZKSTD::ostrstream strCorrectedCryptedMsg;
						if( !CorrectLineTermination(	strCorrectedCryptedMsg, 
														pCryptedMsg, 
														lenCryptedMsg ) ) {
							ZKASSERT( 0 );
						}

						char *pCorrectedCryptedMsg = strCorrectedCryptedMsg.str();
						size_t lenCorrectedCryptedMsg = strCorrectedCryptedMsg.pcount();

						Mailer.SendMsgBody(pCorrectedCryptedMsg, lenCorrectedCryptedMsg);

						ZKVERIFY(zkMemFree(in_perr, (void**)&pCryptedMsg));
					}
				}
				ReturnValue = true;
			}
			catch(DMailAuthException)
			{
				if(bRetry)
				{// Already had a second shot, and failed. I give up!
					bRetry = false;
					
					in_perr->setdetail( "Freedom Mail Server rejected the Nym Mail Certificate (even after a cert update!)" );
				}
				else
				{
					if(pNymMail->UpdateMailCertificate())
					{
						pNymMail->GetMailCertificate(strAuthSecretCert);
						bRetry = true;
					}
					else
					{
						in_perr->setdetail( "Freedom Mail Server rejected the Nym Mail Certificate (and cert update failed!)" );
					}
				}
			}
			catch(RZkException ex)
			{
				in_perr->setdetail(ex.GetFormattedString().c_str() );
				
				bRetry = false;
			}
		}while(!ReturnValue && bRetry);
	}
	if(PrivSigningKey)
	{
		frCryptPubKeyFree(NULL,&PrivSigningKey);
	}
	return ReturnValue;
}


bool ZkEmailHelp::CorrectLineTermination(	ZKSTD::ostrstream& strOut, 
											char* pIn, 
											size_t in_len )
{

	// Okay, so strOut is NULL, will be the message we'll send to the user.
	// pIn is the message we need to go through character by character...
	char c;
	bool inheader = true;
	char last = '\0';
	char lf = '\n';
	const char* crlf = "\r\n";
	unsigned long line_len = 0;


	while( in_len-- )
	{
		c = *(pIn++);

		// Sanity check:
		if( c == '\0' ) {
			ZKASSERT( 0 );
			return false;
		}

		// Are we still in the header?
		if( c == lf ) {
			line_len = 0;
			if( last == lf ) {
				// We reached end of header
				inheader = false;
			}
		}
		last = c;

		// Wrap lines to comply with RFC 822
		if( line_len == 70 )
		{
			if( inheader ) {
				// In the header, wrapped lines should (or can) be prefixed by a space.
				strOut << crlf << ' ' << c;
				line_len = 2;
			} else {
				// In the body, wrapped lines are just more lines; no prefix!
				strOut << crlf << c;
				line_len = 1;
			}
		}
		else {
			if( c == lf ) {
				strOut << crlf;
			} else {
				strOut << c;
				line_len++;
			}
		}
	}

	return true;
}



void ZkEmailHelp::EncryptMessageForFreedomRecipient(char** out_ppCryptedMsg,
													size_t* out_lenCryptedMsg,
													const char *in_strOrigMsg,
													const ZKSTD::string& in_strRecipient,
													const size_t in_iSendingNym) throw (RZkException)
{
	ZKFunctionEntry( "ZkEmailHelp::EncryptMessageForFreedomRecipient" );

	ZKSTD::string strRecipientName = in_strRecipient.substr(0, in_strRecipient.find('@'));

	ZKS::MakeLower(strRecipientName);

	FrCryptCrypt ReceivingNymCryptKey = NULL;
	FrCryptCrypt SendingNymPublicCryptKey = NULL;
	FrCryptPubKey PrivSigningKey = NULL;

	FrMailMsg MailMsg = NULL;

	char *strCryptedBounceMsg = NULL;
	size_t lenCryptedBounceMsg = 0;

	INym * pNym = NULL ;
	ZKSTD::string strSenderName;
	ZKSTD::string strSenderAddress;
	ZKSTD::string strFreedomDomain;

	ZkService::GetINymCentral()->GetNym( IID_INYM, (void**) &pNym, in_iSendingNym ) ;
	pNym->GetSignatureKey( &PrivSigningKey );

	pNym->GetNymEmailDomain(strFreedomDomain);
	pNym->GetNymName(strSenderName);
	strSenderAddress = strSenderName + '@' + strFreedomDomain;


	DZkError derr;

	ZKSTD::string strBounceMsg;
	
	GenerateBounceMessage(in_strOrigMsg, in_strRecipient, in_iSendingNym, strBounceMsg);
	
	if(!ZkService::GetIKeyQry()->GetKey(
		derr,
		FRENT_TYPE_NYM, 
		3, // timeout
		strSenderName.c_str(),
		0, // Version of the key to look up
		FRCRYPT_KEYTYPE_CRYPTION,
		&SendingNymPublicCryptKey))
	{
		ZKASSERT(0);
		throw RZkException(
				derr,
				"ZkEmailHelp::EncryptMessageForFreedomRecipient()",
				"IKeyQry::GetKey() failed", 
				IDS_FAILED_TO_QUERY_NYMS_PUBLIC_KEY_WITH_REASON,
				strSenderAddress.c_str(),
				derr.getdetail().c_str()
				);
	}
	
	if(!ZkService::GetIKeyQry()->GetKey(
		derr,
		FRENT_TYPE_NYM,		
		3, // timeout
		strRecipientName.c_str(), 
		0, // Version of the key to look up
		FRCRYPT_KEYTYPE_CRYPTION, 
		&ReceivingNymCryptKey))
	{
		throw RZkException(
				derr,
				"ZkEmailHelp::EncryptMessageForFreedomRecipient()",
				"IKeyQry::GetKey() failed", 
				IDS_FAILED_TO_QUERY_NYMS_PUBLIC_KEY_WITH_REASON,
				in_strRecipient.c_str(),
				derr.getdetail().c_str()
				);
	}
	
	try
	{
		ZKTestAndThrowEncryption(derr, strSenderAddress.c_str(),
						frMailMsgNew(&derr, &MailMsg));


		ZKTestAndThrowEncryption(derr, strSenderAddress.c_str(),
						frMailMsgCreateWithSig(&derr, MailMsg,
										SendingNymPublicCryptKey, PrivSigningKey, 0, 0,
										strBounceMsg.c_str(), strBounceMsg.length()));

		ZKTestAndThrowEncryption(derr, strSenderAddress.c_str(),
						frMailMsgGive(&derr, MailMsg, &strCryptedBounceMsg,
										&lenCryptedBounceMsg));
	
		ZKTestAndThrowEncryption(derr, strSenderAddress.c_str(),
						frMailMsgFree(&derr, &MailMsg));

		ZKTestAndThrowEncryption(derr, strSenderAddress.c_str(),
						frMailMsgNew(&derr, &MailMsg));

		ZKTestAndThrowEncryption(derr, strSenderAddress.c_str(),
						frMailMsgCreateWithSig(&derr, MailMsg,
										ReceivingNymCryptKey, PrivSigningKey,
										strCryptedBounceMsg, lenCryptedBounceMsg,
										in_strOrigMsg, strlen(in_strOrigMsg)));

		ZKTestAndThrowEncryption(derr, strSenderAddress.c_str(),
						frMailMsgGive(&derr, MailMsg, out_ppCryptedMsg,
										out_lenCryptedMsg));

		ZKTestAndThrowEncryption(derr, strSenderAddress.c_str(),
						frMailMsgFree(&derr, &MailMsg));

		ZKTestAndThrowEncryption(derr, strSenderAddress.c_str(),
			frCryptPubKeyFree(NULL,&PrivSigningKey));
	}
	catch(RZkException)
	{
		if (PrivSigningKey) 
		{
			frCryptPubKeyFree(NULL,&PrivSigningKey);
		}
		if(MailMsg)
		{
			frMailMsgFree(&derr, &MailMsg);
		}
		throw;
	}
}
