// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// ClientControl.cpp
#pragma warning (disable: 4786)
#include "stdafx.h"
#include "ClientControl.h"
#include <lockguard.h>

unsigned long	DClientControl::m_gdwProxyCount= 0;
ZkEvent	DClientControl::m_gNoProxiesEvent = NULL;
ZKCriticalSection DClientControl::m_cs;
DZkError DClientControl::m_err;

//---------------------------------------------------------------------------
// The CTOR
//
DClientControl::DClientControl()
{
	IncrementClientCount() ;
}

//---------------------------------------------------------------------------
//
// The DTOR
DClientControl::~DClientControl()
{
	DecrementClientCount() ;
}


//---------------------------------------------------------------------------
//
//static
ZkEvent DClientControl::InitClientCount()
{
	ZKASSERT( m_gdwProxyCount== 0 );
	ZKASSERT( m_gNoProxiesEvent == 0 );

	// Create the "no clients" signal event object
	ZKVERIFY( zkEventNew(	&m_err,
							&m_gNoProxiesEvent,	
							true ));	// Manual reset
						
	ZKVERIFY( zkEventSet( &m_err, m_gNoProxiesEvent ));

	return m_gNoProxiesEvent;
}


//---------------------------------------------------------------------------
//
void DClientControl::IncrementClientCount()
{
	// Not thread safe (Reset/Set Event)
	ZKS::Lock_Guard<ZKCriticalSection> gate(m_cs);

	m_gdwProxyCount++;
	ZKVERIFY( zkEventReset( &m_err, m_gNoProxiesEvent ));
}


//---------------------------------------------------------------------------
//
void DClientControl::DecrementClientCount()
{
	// Not thread safe (Reset/Set Event)
	ZKS::Lock_Guard<ZKCriticalSection> gate(m_cs);

	ZKASSERT( m_gdwProxyCount > 0 );
	
	if (m_gdwProxyCount > 0) 
	{
		m_gdwProxyCount--;
	}
	if (m_gdwProxyCount < 1) 
	{
		ZKVERIFY( zkEventSet( &m_err, m_gNoProxiesEvent));
	}
}

//---------------------------------------------------------------------------
//
//static
void DClientControl::DeleteClientCount()
{
	ZKVERIFY( zkEventFree( &m_err, &m_gNoProxiesEvent));
}
