// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom(R) code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// DInetD.cpp
//

// Winsock version requested (Spi.cpp reports 2.2 so why be less carefull?)
#define MAJOR_VERSION_REQUIRED 2
#define MINOR_VERSION_REQUIRED 2

#ifdef WIN32
#define PROXIES_CALL_WSASTARTUP
#endif

#include "DInetDInc.h" // Includes StdAfx.h on WIN32, and some C++ stuff.
#include "DInetD.h"
#include <zks.h>
#include <ZkProxyDialogs.h>
#include <ZkService.h>
#include <IClientCentral.h>
#include <INymCentral.h>
#include <IZkRegistry.h>
#include <IZkConsoleMsg.h>
#include <zkList.h>
#include <RZkException.h>
#include <ZkErrorHandler.h>
#include <IZKErrorReporter.h>
#include <lockguard.h>
// Marcp: this need to be fixed
#include <zkrand.h>
#include <sstream>
#include <WalletRegistryKeys.h>
#include <IZkPopup.h>

#ifdef WIN32
#include "../win/ZKStatusAVI.h"
#include <DIpcHttpServer.h>
#else
#include <sys/time.h>           // for struct timeout (for zkSocketWaitReadable or closed)
#include <netdb.h>
#include <ZkStatus.h>
#endif




// The proxies
#include <RZkProxy.h>
#include <RZkPopProxy.h>

#ifdef FREEDOM_VERSION_22
#include <RZkSslProxy.h>
#endif

// Timeouts to allow for the proxy threads to finish up before exiting.
#define WAIT_ON_PROXIES 8000
#define WAIT_ON_LISTEN_THREAD (WAIT_ON_PROXIES+2000)

//The _itoa, _i64toa, and _ui64toa function convert the digits of the given value argument
//to a null-terminated character string and stores the result (up to 33 bytes) in string. 
#define ITOABUFSIZ	34

// Error handling code for proxies
#ifdef WIN32
#define HANDLE_PROXY_THREAD_RUNTIME_EXCEPTION( thread, proxy )\
	ZKASSERT(0);\
\
	unsigned long regval = 0;\
	if(IZkRegistry::GetDWORD( "VerboseExceptions", regval) )\
		ExceptionFilter::ShowCPPUnhandledExceptionDialogThenExit(thread);\
\
	IZkConsoleMsg::ShowImportantMsg( CONSIMAGE_SERIOUS, IDS_CONS_THREAD_DIED_FREEDOM_EXCEPT, proxy, ZkErrorHandler::GetIZkErrorReporter()->GetErrorString(ex) ) ;


#define HANDLE_PROXY_THREAD_STD_EXCEPTION( thread, proxy )\
	ZKASSERT(0);\
\
	unsigned long regval = 0;\
	if(IZkRegistry::GetDWORD( "VerboseExceptions", regval) )\
		ExceptionFilter::ShowCPPUnhandledExceptionDialogThenExit(thread);\
\
	IZkConsoleMsg::ShowImportantMsg( CONSIMAGE_SERIOUS, IDS_CONS_THREAD_DIED, proxy) ;
#else
#define HANDLE_PROXY_THREAD_RUNTIME_EXCEPTION( thread, proxy)
#define HANDLE_PROXY_THREAD_STD_EXCEPTION( thread, proxy)
#endif

////////////////////////////////////////////////////////
// The singleton
CDInetDApp& CDInetDApp::theCDInetDApp()
{
	static CDInetDApp p;
	return p;
}

CDInetDApp::CDInetDApp()
	: m_bQuit(false)
{
	// Set up the signal file descriptor for linux
	RZkProxy::SetSignalFD(GetSignalFD());
}



CDInetDApp::~CDInetDApp()
{
	// Tear down the signal file descriptor for linux
	RZkProxy::SetSignalFD(-1);
}



void CDInetDApp::StartDInetD()
{
	// Set shared secret key in the registry.
	// The BHO uses this key to communicate with Freedom.
	// This limits apps that can talk to Freedom to ones that can read from the registry.
	// This eliminates standard javascript from communicating with Freedom.
#ifdef INTERNAL_BUILD
	ZKSTD::string strOldWalletKey;
	IZkRegistry::GetString(WALLET_SECRET_KEY, strOldWalletKey);
	if (strOldWalletKey != "ignore")
#endif //INTERNAL_BUILD
	{
		uint32_t rn = 0 ;
		ZKVERIFY(zkRandGetUInt32InRange(NULL, 0, -1, &rn));
		ZKSTD::ostringstream oss;
		oss << rn;
		IZkRegistry::SetString(WALLET_SECRET_KEY, oss.str().c_str());
	}

	// Assume Freedom has initialized Winsock2 properly
	// so don't bother checking Winsock2 is installed

#ifdef WIN32
	bool bContinueWithoutLsp = false;

#ifdef INTERNAL_BUILD
	if(IZkRegistry::GetFlag("AskLSP"))
	{
		if(AfxMessageBox("Continue even with our LSP not installed?", MB_YESNO | MB_ICONQUESTION | MB_SYSTEMMODAL) == IDYES)
			bContinueWithoutLsp = true;
	}
#endif
	
	// We need to make sure Freedom only runs if our LSP is first in the chain	
	if(!EnsureOurSPIisFirst() && !bContinueWithoutLsp)
	{
		throw RZkException("CDInetDApp::StartDInetD()", 
							"EnsureOurSPIisFirst() returned false", 
							IDS_FREEDOM_LSP_NOT_FIRST);
	}

	// We need to abort immediately if our LSP is not functionning correctly
	// To find out we need to try and create a socket.
	// Winsock stack doesn't call WSPStartup until it knows which service provider
	// to talk to.  And it won't know that until (generally) a "socket" call is made
	SOCKET sTmp = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if(INVALID_SOCKET == sTmp) 
		throw RZkException("CDInetDApp::StartDInetD()",
							"INVALID_SOCKET returned in socket test",
							IDS_SOCKET_FAILURE);
	
	SAFECLOSESOCKET(sTmp);

#else
#warning are there any config issues we need to check before running DInetD on Linux?
#endif


	//////////////////////////////////////////////////////////////////////
	// Create and bind the member var array of SOCKETs
	
	InitProtocol("Pop3", LOCAL_POP3_PORT);

	InitProtocol("SMTP", LOCAL_SMTP_PORT);
		
	InitProtocol("HTTP", LOCAL_HTTP_PORT);

	InitProtocol("LOCAL_NNTP_PORT", LOCAL_NNTP_PORT);

	InitProtocol("IRC", LOCAL_IRC_PORT);

	InitProtocol("TELNET", LOCAL_TELNET_PORT);

	InitProtocol("SSL", LOCAL_SSL_PORT);

	InitProtocol("IPC_HTTP_SERVER", LOCAL_IPC_HTTP_SERVER_PORT);

	
	///////////////////////////////////////////
	// The thread to wait for connections 
	if( !zkThreadCreateDetached(&m_err, &ListenThread, NULL) )
	{
		ZKASSERT(false);		

		FreeSockets();

		throw RZkException("CDInetDApp::StartDInetD()", "Main DInetD listening thread failed to start", IDS_PROXY_THREAD_FAILED);
	}
}

void CDInetDApp::CloseSockets()
{
	ZKS::Lock_Guard<WZkMutex> gate(m_Mutex);

	ZKSTD::vector<ZkSocket>::iterator it(m_vecListenSockets.begin());
	for( ; it != m_vecListenSockets.end(); ++it)
	{
		SAFECLOSESOCKET((*it)->descriptor);
	}

	it = m_vecClientSockets.begin();
	for( ; it != m_vecClientSockets.end(); ++it)
	{
		SAFECLOSESOCKET((*it)->descriptor);
	}
}

void CDInetDApp::FreeOneSocket( ZkSocket in_Socket )
{
	ZKS::Lock_Guard<WZkMutex> gate(m_Mutex);
	
	ZKSTD::vector<ZkSocket>::iterator it(m_vecListenSockets.begin());

	for( ; it != m_vecListenSockets.end(); ++it)
	{
		if(*it == in_Socket)
		{
			m_vecListenSockets.erase(it);
			zkSocketFree(&m_err, &in_Socket);
			return;
		}
	}

	it = m_vecClientSockets.begin();
	for( ; it != m_vecClientSockets.end(); ++it)
	{
		if(*it == in_Socket)
		{
			m_vecClientSockets.erase(it);
			ZKVERIFY(zkSocketFree(&m_err, &in_Socket));
			return;
		}
	}
}

void CDInetDApp::FreeSockets()
{
	ZKS::Lock_Guard<WZkMutex> gate(m_Mutex);

	while(!m_vecListenSockets.empty())
	{
		zkSocketFree(&m_err, &m_vecListenSockets.back());
		m_vecListenSockets.pop_back();
	}
	while(!m_vecClientSockets.empty())
	{
		zkSocketFree(&m_err, &m_vecClientSockets.back());
		m_vecClientSockets.pop_back();
	}
}

void CDInetDApp::InitProtocol(const char* in_strProtocol, const uint16_t in_defaultPort)
{
	bool bSuccess = false;
	char tempBuffer[10] = {0};

	uint16_t listenPort		= in_defaultPort;
	ZkSocket listenSocket	= NULL;

	IZkRegistry::GetDWORD( in_strProtocol, (unsigned long&)listenPort);
	ZKSTD::string strListenPath("127.0.0.1:");
	snprintf(tempBuffer, sizeof(tempBuffer), "%d", listenPort);
	strListenPath += tempBuffer;

	if( !zkSocketNew(&m_err, AF_INET, SOCK_STREAM, IPPROTO_TCP, &listenSocket))
	{

	}
	else if( !zkSocketSetOption(&m_err, listenSocket, ZKSOCKET_OPTION_QueueDepth, 50) )
	{

	}
    // Need to SO_REUSEADDR so that we can restart freedom without it complaining about 
    // ports in use.
	else if( !zkSocketSetOption(&m_err, listenSocket, ZKSOCKET_OPTION_ReuseAddress, 50) )
	{

	}
	else if( !zkSocketBind(&m_err, listenSocket, strListenPath.c_str()) )
	{

	}
	else if( !zkSocketListen(&m_err, listenSocket) )
	{

	}
	else
	{
		ZKS::Lock_Guard<WZkMutex> gate(m_Mutex);

		bSuccess = true;
		m_vecListenSockets.push_back(listenSocket);
	}

	if(!bSuccess)
	{
		if(listenSocket)
		{
			zkSocketFree(&m_err, &listenSocket);
			ZKTRACE1("CDInetDApp::SetToListen() error, %d\n", XP_LAST_SOCKET_ERR);
		}

		throw RZkException("CDInetDApp::SetToListen()",
						  "A proxy failed to start",
					       IDS_PROXY_FAILURE,
						   listenPort);
	}
}


bool CDInetDApp::StopDInetD()
{
	// Next time through the loop, listen thread will exit.
	m_bQuit = true;
	bool ReturnValue = true;

	// Causes ListenThread and all proxies to break out of Select calls
	CloseSockets();

#ifdef ZK_LINUX
    // Kick any blocking IO that's sitiing in WaitReadableOrClosed
    SignalShutdown();
#endif

	//Wait for all proxies (and listenthread) to exit
	ReturnValue = m_ThreadSemaphore.WaitUntilCountIsZero(WAIT_ON_PROXIES);
	ZKASSERT(ReturnValue);
	FreeSockets();

	ZKTRACE0("CDInetDApp::StopDInetD(): Local DInetD server stopped.\n");
	
	return ReturnValue;
}

void* ZK_THREAD_CALLBACK CDInetDApp::ListenThread(void *pVoid)
	// The ListenThread sits and waits for connections
{
	// Existance of this object will ensure that the client doesn't
	// go away while the thread is alive
	ZKInverseSemaphoreLock gate(theCDInetDApp().m_ThreadSemaphore);

#ifdef PROXIES_CALL_WSASTARTUP
	unsigned short		wVersionRequired = MAKEWORD(MAJOR_VERSION_REQUIRED, MINOR_VERSION_REQUIRED) ;
 	WSADATA				stWSAData;

	if(0 != ::WSAStartup( wVersionRequired, &stWSAData ))
	{
		ZKTRACE0("Is it possible Winsock isn't installed? WSAStartup() failed miserably\n");
		IZkPopup::MessageBox(IDS_NOWINSOCK);
		return 0;
	}
#endif

	try{

		ZkList listSockets = NULL;

		if( !zkListNew(&theCDInetDApp().m_err, &listSockets) )
		{

		}
		else if( !zkListSetDataFreeFunction(&theCDInetDApp().m_err, listSockets, NULL, NULL) )
		{

		}
		else
		{
			theCDInetDApp().m_Mutex.Lock();

			ZKSTD::vector<ZkSocket>::iterator it(theCDInetDApp().m_vecListenSockets.begin());
			for( 0; it != theCDInetDApp().m_vecListenSockets.end(); it++)
			{
				if(!zkListInsert(&theCDInetDApp().m_err, listSockets, *it, LIST_TAIL))
				{
					break;
				}
			}
			if(it != theCDInetDApp().m_vecListenSockets.end() )
			{
				theCDInetDApp().m_Mutex.Unlock();
				// An error ocurred inserting a socket....
			}
			else
			{
				theCDInetDApp().m_Mutex.Unlock();

				
				while( !theCDInetDApp().m_bQuit )
				{
					int iSocket = -1;
					
					struct timeval timeout = { 0 };
					timeout.tv_sec = 45;
	
					// Go to sleep
					if(!zkSocketWaitReadableOrClosed(&theCDInetDApp().m_err, listSockets,
						&timeout, &iSocket, theCDInetDApp().GetSignalFD()) )
					{
						// If the accept fails, don't exit this loop. That would
						// Cause DInetD to quit, and proxies stop working.
					}
					else if(iSocket >= 0)
					{
						if(!theCDInetDApp().HandleConnections(iSocket))
						{
							// What to do?
						}
					}

				} //while
			}
		}

		if(listSockets)
		{
			zkListFree(&theCDInetDApp().m_err, &listSockets);
		}
	}
	catch(IZkException& ex){
		HANDLE_PROXY_THREAD_RUNTIME_EXCEPTION( "ListenThread", "LISTEN" );
	}
	catch(ZKSTD::exception&){
		HANDLE_PROXY_THREAD_STD_EXCEPTION( "ListenThread", "LISTEN" );
	}

#ifdef PROXIES_CALL_WSASTARTUP
	::WSACleanup();
#endif
	return 0;
}

bool CDInetDApp::HandleConnections(const int ClientIndicator)
	// ClientIndicator -- the return value from WaitForMultipleObjects()
	// in ListenThread()
{
	bool ReturnValue = true;
	
	ZkSocket socketClient = NULL;
	
	if(!zkSocketAccept(&m_err, m_vecListenSockets[ClientIndicator], &socketClient) )
	{
		ZKTRACE1("ListenThread() failed, %d.\n", XP_LAST_SOCKET_ERR);
		ReturnValue = false;
	}

#ifdef _DEBUG
	if(socketClient)
	{
		{
			sockaddr_in name = { 0 };
			int namelen = sizeof(name);
			ZKASSERT( (SOCKET_ERROR != getsockname(socketClient->descriptor, (sockaddr*)&name, &namelen)) );
			ZKTRACE2("Local proxy accepting new socket [%d] on local port [%d]\n", socketClient->descriptor, ntohs(name.sin_port) );
		}

		{
			sockaddr_in name = { 0 };
			int namelen = sizeof(name);
			ZKASSERT( (SOCKET_ERROR != getsockname(m_vecListenSockets[ClientIndicator]->descriptor, (sockaddr*)&name, &namelen)) );
			ZKTRACE2("It was from [%d] on local port [%d]\n", m_vecListenSockets[ClientIndicator]->descriptor, ntohs(name.sin_port) );
		}
	}
#endif
	
	if(socketClient)
	{

		char temp[4] = {127, 0, 0, 1};

		if ( 0 != memcmp( &((sockaddr_in*)&socketClient->remote_addr)->sin_addr, temp, 4 ))
		{
			// close
			zkSocketFree(&m_err, &socketClient);
		}
		else
		{

			// Start a client thread to handle this transaction.

			ZKASSERT(ClientIndicator < CDInetDApp::NUM_SOCKETS);

// bug 6711
#ifndef _WIN32
			ZKVERIFY( zkSocketSetOption(&m_err, socketClient,ZKSOCKET_OPTION_SendTimeout, 15 * 1000) );
			ZKVERIFY( zkSocketSetOption(&m_err, socketClient,ZKSOCKET_OPTION_RecvTimeout, 45 * 1000) );
#endif

			if(ClientIndicator == 0)
			{ // Pop
				ZKVERIFY( zkThreadCreateDetached(&m_err,&Pop3ClientThread, (void*) socketClient) );
			}
			else if(ClientIndicator == 1)
			{ // Smtp
				ZKVERIFY( zkThreadCreateDetached(&m_err,&SmtpClientThread, (void*) socketClient) );
			}
			else if(ClientIndicator == 2)
			{ // Http
				ZKVERIFY( zkThreadCreateDetached(&m_err,&HttpClientThread, (void*) socketClient) );
			}
			else if(ClientIndicator == 3)
			{ // Nntp
				ZKVERIFY( zkThreadCreateDetached(&m_err,&NntpClientThread, (void*) socketClient) );
			}
			else if(ClientIndicator == 4)
			{ // Irc
				ZKVERIFY( zkThreadCreateDetached(&m_err,&IrcClientThread, (void*) socketClient) );
			}
			else if(ClientIndicator == 5)
			{ // Telnet
				ZKVERIFY( zkThreadCreateDetached(&m_err,&TelnetClientThread, (void*) socketClient) );
			}
			else if(ClientIndicator == 6)
			{ // SSL
				ZKVERIFY( zkThreadCreateDetached(&m_err,&SslClientThread, (void*) socketClient) );
			}
			else if(ClientIndicator == 7)
			{ // IPC HTTP server
	#ifdef WIN32
				ZKVERIFY( zkThreadCreateDetached(&m_err,&IpcHttpServerThread, (void*) socketClient) );
	#else
				// No Wallet on Linux
				ZKASSERT(0);
	#endif
			}
			{
				ZKS::Lock_Guard<WZkMutex> gate(m_Mutex);
 				m_vecClientSockets.push_back(socketClient);
			}
		}

	}
		
	return ReturnValue;
}

void* ZK_THREAD_CALLBACK CDInetDApp::NntpClientThread(void *pVoid)
{	
#ifdef PROXIES_CALL_WSASTARTUP
	unsigned short		wVersionRequired = MAKEWORD(MAJOR_VERSION_REQUIRED, MINOR_VERSION_REQUIRED) ;
 	WSADATA				stWSAData;

	if(0 != ::WSAStartup( wVersionRequired, &stWSAData ))
	{
		ZKTRACE0("Is it possible Winsock isn't installed? WSAStartup() failed miserably\n");
		IZkPopup::MessageBox(IDS_NOWINSOCK);
		return 0;
	}
#endif

	ZkSocket socketClient = ( ZkSocket ) pVoid ;
	ZKASSERT( socketClient != NULL ) ;

	IZkConsoleMsg::ShowMsg( CONSIMAGE_MAIL, IDS_CONS_PROXY_NNTP_START );

#ifdef WIN32	
	ZKStatusAVI BusyWithNews( ZK_STATUS_NEWS_CONNECTION ) ;
#else
	ZkStatus BusyWithNews( ZK_STATUS_NEWS_CONNECTION ) ;
#endif

	try{

		// Existance of this object will ensure that the client doesn't
		// go away while the thread is alive
		ZKInverseSemaphoreLock gate(theCDInetDApp().m_ThreadSemaphore);

        bool bAllowConnection = true;
        
        if ( !ZkService::GetIClientCentral()->GetPreference(
            ZK_SHOW_AGAIN_PREF_SET, 
            PREF_NO_SHOW_NEWS_OPTIONS))
        {
            size_t nActiveNym = ZkService::GetINymCentral()->GetActiveNymIndex();
            if ( !ZkService::GetINymCentral()->IsCurrentNymUser()
                 && (ZkService::GetINymCentral()->DoesNymUseCloud(nActiveNym)
                     || ZkService::GetINymCentral()->DoesNymUseMail(nActiveNym)))
            {
                if (!ZkProxyDialogs::LaunchWarnNNTPConnectionDlg())
                {
                    bAllowConnection = false;
                }
            }
        }
        
        if (bAllowConnection)
        {
            ZKTRACE0("NNTP ClientThread started...\n");
		
            RZkProxy NNTPProx( socketClient, IZkProxy::NNTP_PROTOCOL );
            NNTPProx.RunProxy();
		
            ZKTRACE0("Leaving a NNTP client thread\n");
        }
	}
	catch(IZkException& ex){
		HANDLE_PROXY_THREAD_RUNTIME_EXCEPTION("NntpThread", "NNTP");
	}
	catch(ZKSTD::exception&){
		HANDLE_PROXY_THREAD_STD_EXCEPTION("NntpThread", "NNTP");
	}
	
	CDInetDApp::theCDInetDApp().FreeOneSocket( socketClient );

	IZkConsoleMsg::ShowMsg( CONSIMAGE_MAIL, IDS_CONS_PROXY_NNTP_STOP );

#ifdef PROXIES_CALL_WSASTARTUP
	::WSACleanup();
#endif

	return 0;
}

void* ZK_THREAD_CALLBACK CDInetDApp::IrcClientThread(void *pVoid)
{
#ifdef PROXIES_CALL_WSASTARTUP
	unsigned short		wVersionRequired = MAKEWORD(MAJOR_VERSION_REQUIRED, MINOR_VERSION_REQUIRED) ;
 	WSADATA				stWSAData;

	if(0 != ::WSAStartup( wVersionRequired, &stWSAData ))
	{
		ZKTRACE0("Is it possible Winsock isn't installed? WSAStartup() failed miserably\n");
		IZkPopup::MessageBox(IDS_NOWINSOCK);
		return 0;
	}
#endif

#ifdef WIN32
	ZKStatusAVI Connection( ZK_STATUS_IRC_CONNECTION ) ;
#else
	ZkStatus Connection( ZK_STATUS_IRC_CONNECTION ) ;
#endif

	ZkSocket socketClient = ( ZkSocket ) pVoid ;
	ZKASSERT( socketClient != NULL ) ;
	
	try{
		// Existance of this object will ensure that the client doesn't
		// go away while the thread is alive
		ZKInverseSemaphoreLock gate(theCDInetDApp().m_ThreadSemaphore);

		ZKTRACE0("Started a IRC client thread\n");

		RZkProxy IRCProx( socketClient, IZkProxy::IRC_PROTOCOL );
		IRCProx.RunProxy();

		ZKTRACE0("Leaving the IRC client thread\n");

	}
	catch(IZkException& ex){
		HANDLE_PROXY_THREAD_RUNTIME_EXCEPTION("IrcThread", "IRC");
	}
	catch(ZKSTD::exception&){
		HANDLE_PROXY_THREAD_STD_EXCEPTION("IrcThread", "IRC");
	}
	
	CDInetDApp::theCDInetDApp().FreeOneSocket( socketClient );

#ifdef PROXIES_CALL_WSASTARTUP
	::WSACleanup();
#endif
	return 0;
}


void* ZK_THREAD_CALLBACK CDInetDApp::SslClientThread(void *pVoid)
{	
#ifdef PROXIES_CALL_WSASTARTUP
	unsigned short		wVersionRequired = MAKEWORD(MAJOR_VERSION_REQUIRED, MINOR_VERSION_REQUIRED) ;
 	WSADATA				stWSAData;

	if(0 != ::WSAStartup( wVersionRequired, &stWSAData ))
	{
		ZKTRACE0("Is it possible Winsock isn't installed? WSAStartup() failed miserably\n");
		IZkPopup::MessageBox(IDS_NOWINSOCK);
		return 0;
	}
#endif

#ifdef WIN32
	ZKStatusAVI Connection( ZK_STATUS_SSL_CONNECTION ) ;
#else
	ZkStatus Connection( ZK_STATUS_SSL_CONNECTION ) ;
#endif

	ZkSocket socketClient = ( ZkSocket ) pVoid ;
	ZKASSERT( socketClient != NULL ) ;

	try{
		// Existance of this object will ensure that the client doesn't
		// go away while the thread is alive
		ZKInverseSemaphoreLock gate(theCDInetDApp().m_ThreadSemaphore);

#ifndef FREEDOM_VERSION_22

        bool bAllowConnection = true;
        if ( !ZkService::GetIClientCentral()->GetPreference(
            ZK_SHOW_AGAIN_PREF_SET, 
            PREF_NO_SHOW_SSL_WARNING))
        {
            if (!ZkProxyDialogs::LaunchWarnSSLConnectionDlg())
            {
                bAllowConnection = false;
            }
        }
        
        if (bAllowConnection)
        {
            ZKTRACE0("Started a SSL client thread\n");

            RZkProxy SSLProx( socketClient, IZkProxy::SSL_PROTOCOL );
            SSLProx.RunProxy();

            ZKTRACE0("Leaving the SSL client thread\n");
        }
#else
		RZkSslProxy	SSLProxy( socketClient );
		SSLProxy.Init(true, "\\t2.pem", "\\lib3rd\\openssl\\certs");
		SSLProx.RunProxy();
#endif

	}
	catch(IZkException& ex){
		HANDLE_PROXY_THREAD_RUNTIME_EXCEPTION("SslThread", "SSL");
	}
	catch(ZKSTD::exception&){
		HANDLE_PROXY_THREAD_STD_EXCEPTION("SslThread", "SSL");
	}

	CDInetDApp::theCDInetDApp().FreeOneSocket( socketClient );

#ifdef PROXIES_CALL_WSASTARTUP
	::WSACleanup();
#endif
	return 0;
}


void* ZK_THREAD_CALLBACK  CDInetDApp::TelnetClientThread(void *pVoid)
{	
	IZkConsoleMsg::ShowMsg( CONSIMAGE_COMP_TERM, IDS_CONS_PROXY_TELNET_START );

#ifdef PROXIES_CALL_WSASTARTUP
	unsigned short		wVersionRequired = MAKEWORD(MAJOR_VERSION_REQUIRED, MINOR_VERSION_REQUIRED) ;
 	WSADATA				stWSAData;

	if(0 != ::WSAStartup( wVersionRequired, &stWSAData ))
	{
		ZKTRACE0("Is it possible Winsock isn't installed? WSAStartup() failed miserably\n");
		IZkPopup::MessageBox(IDS_NOWINSOCK);
		return 0;
	}
#endif

#ifdef WIN32
	ZKStatusAVI Connection( ZK_STATUS_TELNET_CONNECTION ) ;
#else
	ZkStatus Connection( ZK_STATUS_TELNET_CONNECTION ) ;
#endif

	ZkSocket socketClient = ( ZkSocket ) pVoid ;
	ZKASSERT( socketClient != NULL ) ;

	try{

		// Existance of this object will ensure that the client doesn't
		// go away while the thread is alive
		ZKInverseSemaphoreLock gate(theCDInetDApp().m_ThreadSemaphore);

		ZKTRACE0("Started a TELNET client thread\n");
		
		RZkProxy TelnetProx( socketClient, IZkProxy::TELNET_PROTOCOL );
		TelnetProx.RunProxy();

		ZKTRACE0("Leaving the TELNET client thread\n");

	}
	catch(IZkException& ex){
		HANDLE_PROXY_THREAD_RUNTIME_EXCEPTION("TelnetThread", "TELNET");
	}
	catch(ZKSTD::exception&){
		HANDLE_PROXY_THREAD_STD_EXCEPTION("TelnetThread", "TELNET");
	}

	CDInetDApp::theCDInetDApp().FreeOneSocket( socketClient );

	IZkConsoleMsg::ShowMsg( CONSIMAGE_COMP_TERM, IDS_CONS_PROXY_TELNET_STOP );

#ifdef PROXIES_CALL_WSASTARTUP
	::WSACleanup();
#endif
	return 0;
}


void* ZK_THREAD_CALLBACK  CDInetDApp::Pop3ClientThread(void *pVoid)
{		
#ifdef PROXIES_CALL_WSASTARTUP
	unsigned short		wVersionRequired = MAKEWORD(MAJOR_VERSION_REQUIRED, MINOR_VERSION_REQUIRED) ;
 	WSADATA				stWSAData;

	if(0 != ::WSAStartup( wVersionRequired, &stWSAData ))
	{
		ZKTRACE0("Is it possible Winsock isn't installed? WSAStartup() failed miserably\n");
		IZkPopup::MessageBox(IDS_NOWINSOCK);
		return 0;
	}
#endif

#ifdef WIN32
	ZKStatusAVI BusyWithMail( ZK_STATUS_MAIL_CONNECTION );
#else
	ZkStatus BusyWithMail( ZK_STATUS_MAIL_CONNECTION );
#endif
	IZkConsoleMsg::ShowMsg(CONSIMAGE_MAIL, IDS_CONS_PROXY_POP_START);


	ZkSocket socketClient = ( ZkSocket ) pVoid ;
	ZKASSERT( socketClient != NULL ) ;


	try{

		// Existance of this object will ensure that the client doesn't
		// go away while the thread is alive
		ZKInverseSemaphoreLock gate(theCDInetDApp().m_ThreadSemaphore);

		ZKTRACE0("Started a POP3 client thread\n");

		RZkPopProxy PopProx(socketClient);
		PopProx.RunProxy();

		ZKTRACE0("Leaving the POP3 client thread\n");

	}
	catch(IZkException& ex){
		HANDLE_PROXY_THREAD_RUNTIME_EXCEPTION("Pop3Thread", "POP3");
	}
	catch(ZKSTD::exception&){
		HANDLE_PROXY_THREAD_STD_EXCEPTION("Pop3Thread", "POP3");
	}

	CDInetDApp::theCDInetDApp().FreeOneSocket( socketClient );

	IZkConsoleMsg::ShowMsg(CONSIMAGE_MAIL, IDS_CONS_PROXY_POP_STOP);

#ifdef PROXIES_CALL_WSASTARTUP
	::WSACleanup();
#endif
	return 0;
}


void* ZK_THREAD_CALLBACK  CDInetDApp::SmtpClientThread(void *pVoid)
{	
#ifdef PROXIES_CALL_WSASTARTUP
	unsigned short		wVersionRequired = MAKEWORD(MAJOR_VERSION_REQUIRED, MINOR_VERSION_REQUIRED) ;
 	WSADATA				stWSAData;

	if(0 != ::WSAStartup( wVersionRequired, &stWSAData ))
	{
		ZKTRACE0("Is it possible Winsock isn't installed? WSAStartup() failed miserably\n");
		IZkPopup::MessageBox(IDS_NOWINSOCK);
		return 0;
	}
#endif

	ZkSocket socketClient = ( ZkSocket ) pVoid ;
	ZKASSERT( socketClient != NULL ) ;

	IZkConsoleMsg::ShowMsg(CONSIMAGE_MAIL, IDS_CONS_PROXY_SMTP_START);

#ifdef WIN32
	ZKStatusAVI BusyWithMail( ZK_STATUS_MAIL_CONNECTION ) ;
#else
	ZkStatus BusyWithMail( ZK_STATUS_MAIL_CONNECTION ) ;
#endif

	try{

		// Existance of this object will ensure that the client doesn't
		// go away while the thread is alive
		ZKInverseSemaphoreLock gate(theCDInetDApp().m_ThreadSemaphore);

		ZKTRACE0("SMTP ClientThread started...\n");

		RZkProxy SMTPProx( socketClient, IZkProxy::SMTP_PROTOCOL );
		SMTPProx.RunProxy();
		
		ZKTRACE0("Leaving the SMTP client thread\n");

	}
	catch(IZkException& ex){
		HANDLE_PROXY_THREAD_RUNTIME_EXCEPTION("SmtpThread", "SMTP");
	}
	catch(ZKSTD::exception&){
		HANDLE_PROXY_THREAD_STD_EXCEPTION("SmtpThread", "SMTP");
	}

	CDInetDApp::theCDInetDApp().FreeOneSocket( socketClient );

	IZkConsoleMsg::ShowMsg(CONSIMAGE_MAIL, IDS_CONS_PROXY_SMTP_STOP);

#ifdef PROXIES_CALL_WSASTARTUP
	::WSACleanup();
#endif
	return 0;
}


void* ZK_THREAD_CALLBACK  CDInetDApp::HttpClientThread(void *pVoid)
{
#ifdef PROXIES_CALL_WSASTARTUP
	unsigned short		wVersionRequired = MAKEWORD(MAJOR_VERSION_REQUIRED, MINOR_VERSION_REQUIRED) ;
 	WSADATA				stWSAData;

	if(0 != ::WSAStartup( wVersionRequired, &stWSAData ))
	{
		ZKTRACE0("Is it possible Winsock isn't installed? WSAStartup() failed miserably\n");
		IZkPopup::MessageBox(IDS_NOWINSOCK);
		return 0;
	}
#endif

#ifdef WIN32
	ZKStatusAVI Connection( ZK_STATUS_WEB_CONNECTION ) ;
#else
	ZkStatus Connection( ZK_STATUS_WEB_CONNECTION ) ;
#endif

	ZkSocket socketClient = ( ZkSocket ) pVoid ;
	ZKASSERT( socketClient != NULL ) ;

	try{

		// Existance of this object will ensure that the client doesn't
		// go away while the thread is alive
		ZKInverseSemaphoreLock gate(theCDInetDApp().m_ThreadSemaphore);

		RZkProxy HTTPProx( socketClient, IZkProxy::HTTP_PROTOCOL );
		HTTPProx.RunProxy();


	}
	catch(IZkException& ex){
		HANDLE_PROXY_THREAD_RUNTIME_EXCEPTION("HttpThread", "HTTP" );
	}
	catch(ZKSTD::exception&){
		HANDLE_PROXY_THREAD_STD_EXCEPTION("HttpThread", "HTTP" );
	}

	CDInetDApp::theCDInetDApp().FreeOneSocket( socketClient );

#ifdef PROXIES_CALL_WSASTARTUP
	::WSACleanup();
#endif
	return 0;
}

#ifdef WIN32
bool CDInetDApp::EnsureOurSPIisFirst()
{

	bool					bRet = false;
	LPWSAPROTOCOL_INFOW		ProtocolInfoBuff = NULL;
    unsigned long					ProtocolInfoBuffSize = 0;
	int						ErrorCode = 0;
    int						EnumResult = 0;

	::WSCEnumProtocols(
                NULL,								// lpiProtocols
                ProtocolInfoBuff,					// lpProtocolBuffer
                & ProtocolInfoBuffSize,				// lpdwBufferLength
                & ErrorCode);      // lpErrno

	ProtocolInfoBuff = (LPWSAPROTOCOL_INFOW)malloc(ProtocolInfoBuffSize);

	if(ProtocolInfoBuff){
		
		EnumResult = WSCEnumProtocols(			// WSAEnumProtocols() only returns information on base protocols and protocol chains, so we use WSCEnumProtocols()
				    NULL,						// lpiProtocols
					ProtocolInfoBuff,			// lpProtocolBuffer
					& ProtocolInfoBuffSize,		// lpdwBufferLength
					& ErrorCode);

        if(SOCKET_ERROR != EnumResult){
			bRet = (0 == wcsnicmp(L"ZKS", ProtocolInfoBuff[0].szProtocol, 3));

		} // if
		free(ProtocolInfoBuff);
	}
	return bRet;
}
#endif

#ifdef WIN32 // no wallet on linux

void* ZK_THREAD_CALLBACK CDInetDApp::IpcHttpServerThread(void *pVoid)
{
	ZkSocket socketClient = ( ZkSocket ) pVoid ;
	ZKASSERT( socketClient != NULL ) ;

	try{

		// Existance of this object will ensure that the client doesn't
		// go away while the thread is alive
		ZKInverseSemaphoreLock gate(theCDInetDApp().m_ThreadSemaphore);

		ZKTRACE0("Started Freedom wallet thread\n");

		DIpcHttpServer ipcHttpServer(socketClient);
		ipcHttpServer.RunIpcHttpServer();
		
		ZKTRACE0("Leaving the IpcHttpServer thread\n");

	}
	catch(IZkException& ex){
		HANDLE_PROXY_THREAD_RUNTIME_EXCEPTION("IPCHTTPServer", "IPCHTTPServer" );
	}
	catch(ZKSTD::exception&){
		HANDLE_PROXY_THREAD_STD_EXCEPTION("IPCHTTPServer", "IPCHTTPServer" );
	}

	CDInetDApp::theCDInetDApp().FreeOneSocket( socketClient );

	return 0;
}

#endif

#ifdef WIN32
#ifdef INTERNAL_BUILD
void CDInetDApp::TestCrash(ZkCrashType in_PreferredDeath){

	ZKASSERT(in_PreferredDeath != ZKUNDEFINED);

	DoCrash(in_PreferredDeath);
}
#endif // #ifdef INTERNAL_BUILD
#endif // WIN32


//---------------------------------------------------------------------------
//
#ifdef ZK_LINUX

int CDInetDApp::GetSignalFD()
{
    return m_shutdownSignalPipe.GetRead();
}

void CDInetDApp::SignalShutdown()
{
    char c = '~';
    write( m_shutdownSignalPipe.GetWrite(), &c, 1 );
}

#else // ZK_LINUX
int CDInetDApp::GetSignalFD()
{
    // This is just a NOOP. We don't want to have to put ifdefs everywhere
    return -1;
}

#endif
