// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// DInetD.h : main header file for the DINETD DLL
//

#if !defined(AFX_DINETD_H__A96FDC49_59EC_11D2_8097_00104B6B4FA4__INCLUDED_)
#define AFX_DINETD_H__A96FDC49_59EC_11D2_8097_00104B6B4FA4__INCLUDED_

#ifdef WIN32
#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#ifdef DINETD_IMPL
   #define DINETD_DLL __declspec( dllexport ) 
#else
   #define DINETD_DLL __declspec( dllimport ) 
#endif
#else // we're not on win
#define DINETD_DLL
#endif // WIN32

#include <DZkError.h>
#include <WSyncTypes.h>

#ifdef WIN32
#include <ExceptionFilter.h>
#else
#warning Do we need something like ExceptionFilter on Linux?
#endif

#include <vector>
#include <zksocket.h>
#include <zkThread.h>
#include <ZKInverseSemaphore.h>

#ifdef ZK_LINUX
#  include <../linux/ZkPipe.h>
#endif

class RZkException;
class DIpcHttpServer;

class DINETD_DLL CDInetDApp
{
public:
	~CDInetDApp();
	static CDInetDApp& theCDInetDApp();

	void				StartDInetD() throw(RZkException);
	bool				StopDInetD();

#ifdef WIN32
#ifdef INTERNAL_BUILD
	void				TestCrash(ZkCrashType in_PreferredDeath);	
#endif
#endif

    // File descriptor used to signal shutdown on linux
    int GetSignalFD();
#ifdef ZK_LINUX
    // Cause shutdown FD pipe to be writen to and break all
    // syscalls waiting on new connections etc.
    void SignalShutdown();
#endif

protected:
	CDInetDApp(const CDInetDApp& rhs);
	CDInetDApp operator=(const CDInetDApp& rhs);
	CDInetDApp();

private:
	bool				HandleConnections(const int ClientIndicator);
	void				FreeOneSocket( ZkSocket in_Socket );

#ifdef WIN32
	// No Wallet on Linux
	static void* ZK_THREAD_CALLBACK IpcHttpServerThread(void*
							    in_pVoid);
#endif

	static void* ZK_THREAD_CALLBACK ListenThread(void *pVoid);
	static void* ZK_THREAD_CALLBACK Pop3ClientThread(void *pVoid);
	static void* ZK_THREAD_CALLBACK SmtpClientThread(void *pVoid);
	static void* ZK_THREAD_CALLBACK HttpClientThread(void *pVoid);
	static void* ZK_THREAD_CALLBACK NntpClientThread(void *pVoid);
	static void* ZK_THREAD_CALLBACK IrcClientThread(void *pVoid);
	static void* ZK_THREAD_CALLBACK TelnetClientThread(void *pVoid);
	static void* ZK_THREAD_CALLBACK SslClientThread(void *pVoid);

	WZkMutex	m_Mutex;
	ZKInverseSemaphore m_ThreadSemaphore;

	bool   m_bQuit;

	enum { NUM_SOCKETS = 8 };
	
	ZKSTD::vector<ZkSocket>	m_vecListenSockets;
	ZKSTD::vector<ZkSocket>	m_vecClientSockets;

	DZkError m_err;
	void FreeSockets();
	void CloseSockets();
	void InitProtocol(const char* in_strProtocol, const uint16_t in_defaultPort);
#ifdef WIN32
	bool EnsureOurSPIisFirst();
#endif
#ifdef ZK_LINUX
    ZkPipe m_shutdownSignalPipe;
#endif
};

#endif // !defined(AFX_DINETD_H__A96FDC49_59EC_11D2_8097_00104B6B4FA4__INCLUDED_)
