// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// DIpcHttpServer.cpp
//

#pragma warning (disable : 4786)

#include "stdafx.h"
#include "DIpcHttpServer.h"
#include "Wallet.h"
#include "WalletConsts.h"
#include <sstream>


#ifdef INTERNAL_BUILD
ZkCrashType DIpcHttpServer::m_ThreadCrash = ZKUNDEFINED;
#endif


DIpcHttpServer::DIpcHttpServer(ZkSocket in_socket)
	: m_hEvent(::WSACreateEvent())
	, m_socket(in_socket)
{
}

DIpcHttpServer::~DIpcHttpServer()
{
	SAFECLOSEWSAEVENT(m_hEvent);
}

bool DIpcHttpServer::RunIpcHttpServer()
{

	bool bRet = true;
	
	ZKSTD::string strRequestedPage;

	if(!(RecvEntireRequest(strRequestedPage)))
	{
		bRet = false;
	}
	else 
	{
		Wallet wallet(strRequestedPage, m_socket);
		ZKSTD::string strResponse;
		if (!wallet.Run(strResponse))
		{
			ZKASSERT(false);
			bRet = false;
		}
		else if (strResponse != "")
		{
			SendResponse(strResponse.c_str());
		}
	}
	
	DoCrash(m_ThreadCrash);
	return bRet;
}

bool DIpcHttpServer::SendResponse(const char* in_strBody)
{ 
	bool bRet = true;

	ZKSTD::ostringstream oss;
	oss << "HTTP/1.1 200\r\n"
		<< "Server: Freedom Local HTTP server\r\n"
		<< "Content-Type: text/html\r\n"
		<< CONTENT_LENGTH
		<< " "
		<< strlen(in_strBody)
		<< "\r\n\r\n"
		<< in_strBody;

	if(!(zkSocketWriteAll(&m_err, m_socket, oss.str().c_str(), oss.str().size())) )
	{
		bRet = false;
	}

	return bRet;
}

bool DIpcHttpServer::RecvEntireRequest(ZKSTD::string& out_strRequestedPage)
{
	bool bRet = true;

	const unsigned int bufLen = 1024;
	char buf[bufLen] = { 0 };
	unsigned int nBytesRead;

	out_strRequestedPage = "";
	///////////////////////////////////////////////
	// Get the entire REQUEST	
	while (true)
	{
		if( !zkSocketRead(
			&m_err, 
			m_socket,
			buf, 
			bufLen,
			&nBytesRead))
		{
			bRet = false;
			break;
		}

		ZKTRACE2("\nRecvRealClientRequest():: #Bytes:%d, zkHTTPHeaderAddText:\n%s\n", bufLen, CString(buf).Left(std::min<int>(bufLen, 400)));
		buf[nBytesRead] = '\0';

		if ((nBytesRead > 0) && (nBytesRead <= bufLen))
		{
			out_strRequestedPage.append(buf, nBytesRead);
		}	
		
		std::string::size_type pos1, pos2;
		if ((pos2 = out_strRequestedPage.find(" HTTP")) != out_strRequestedPage.npos)
		{

			if ((pos1 = out_strRequestedPage.find('/')) == out_strRequestedPage.npos)
			{	
				ZKASSERT(false);
				bRet = false;
				break;
			}
			out_strRequestedPage = out_strRequestedPage.substr(pos1, pos2 - pos1);
			break;
		}
	}

	return bRet;
}


#ifdef INTERNAL_BUILD
void DIpcHttpServer::TestCrash(const ZkCrashType in_PreferredDeath){

	switch(in_PreferredDeath){
	case ZKDIVBYZERO:
		m_ThreadCrash = ZKDIVBYZERO;
		break;
	case ZKUNHANDLED_EXCEPTION:
		m_ThreadCrash = ZKUNHANDLED_EXCEPTION;
		break;
	default:
		ZKASSERT(0);
	};
}
#endif // #ifdef INTERNAL_BUILD
