// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// DIpcHttpServer.h
//

#if !defined(_DIPCHTTPSERVER_H_INCLUDED_)
#define _DIPCHTTPSERVER_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifdef DIPCHTTPSERVER_IMPL
   #define DIPCHTTPSERVER_DLL __declspec( dllexport ) 
#else
   #define DIPCHTTPSERVER_DLL __declspec( dllimport ) 
#endif


#include <ExceptionFilter.h>
#include <INymWallet.h> //For LoginInfo
#include <ZkNetHelp.h>
#include <dzkerror.h>


class Wallet;

class DIPCHTTPSERVER_DLL DIpcHttpServer AUTO_INSTANCE_COUNTING1(DIpcHttpServer)
{
public:

	///////////////////////////////////////////////////////////////////////////////////////////////
	// PURPOSE:	Constructs a DIpcHttpServer
	//
	// REQUIRE: Nothing
	//
	// PROMISE:	Nothing

	DIpcHttpServer(ZkSocket in_socket);


	///////////////////////////////////////////////////////////////////////////////////////////////
	// PURPOSE: Destructs the DIpcHttpServer 
	//
	// REQUIRE: Nothing
	//
	// PROMISE:	Closes the socket if it is still open.

	~DIpcHttpServer();


	///////////////////////////////////////////////////////////////////////////////////////////////
	// PURPOSE: Reads _ONE_ request (and only one which is how this class differs from the
	//			proxies - the class may change later to read and process requests until the
	//			socket is closed) from the socket, constructs a Wallet object, calls Wallet::Run(), 
	//			and sends a response back if required.  The Run() method may wait for user input
	//			from the UI, so this should be considered a "blocking" call.
	//
	// REQUIRE: The request must be a valid HTTP GET request. Otherwise this function returns 
	//			false.
	//
	// PROMISE:	Returns false if no request is received, an invalid http request is received, 
	//			or Wallet::Run() returns false.

	bool RunIpcHttpServer();

private:
	DIpcHttpServer(const DIpcHttpServer& rhs);
	DIpcHttpServer& operator=(const DIpcHttpServer& rhs);

	bool		RecvEntireRequest(ZKSTD::string& out_strRequestedPage);
	bool		SendResponse(const char* in_strBody);

private:
	HANDLE		m_hEvent;
	DZkError	m_err;
	ZkSocket	m_socket;

#ifdef INTERNAL_BUILD
public:
	static void				TestCrash(const ZkCrashType in_PreferredDeath);
	static ZkCrashType	    m_ThreadCrash;
#endif //INTERNAL_BUILD
};

#endif //_DIPCHTTPSERVER_H_INCLUDED_