// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// DPop3ProxUnitTest.cpp

#include "stdafx.h"
#include "DIpcHttpServer.h"
#include "DIpcHttpServerUnitTest.h"
#include <zkDNS.h>
// We don't include test code in our externally released binaries
#ifdef INTERNAL_BUILD

#include "Wallet.h"
#include "WalletConsts.h"
#include <IZkRegistry.h>
#include <WalletRegistryKeys.h>
#include <ZkService.h>
#include <INymCentral.h>
#include <INym.h>
#include <INymWallet.h>
#include <clientwin32.h>	// for LOCAL_IPC_HTTP_SERVER_PORT
#include <string_hlp.h>
#include <sstream>

DIpcHttpServerUnitTest g_dIpcHttpServerUnitTest;

long DIpcHttpServerUnitTest::Execute( void* )
{
	RunUnitTest();

	return 1;
}

void DIpcHttpServerUnitTest::RunUnitTest()
{
	ZkSocket bogusSock;

	Wallet wlt("", bogusSock);
	wlt.SelfTest();

	// Test WALLET_CMD_ADD_LOGIN_INFO
	INym* pNym = NULL;
	ZkService::GetINymCentral()->GetActiveNym( IID_INYM, (void far**)&pNym );
	ZKTEST(pNym);

	ZKSTD::string strActiveNym;
	pNym->GetNymName(strActiveNym);

	LoginInfo loginInfo(strActiveNym, "my username*&(9", "myPassword", "www.testerDomain.net");

	INymWallet* pNymWallet = NULL;

	pNym->QueryInterface( IID_INYMWALLET, (void far**) &pNymWallet ) ;

	ZKTEST(pNymWallet);

	ZKSTD::vector<LoginInfo> loginInfoVector;
	pNymWallet->GetLoginInfo(loginInfo.strDomain, loginInfoVector);
	ZKTEST(loginInfoVector.size() == 0);


	ZKSTD::string strWalletKey;
	ZKTEST(IZkRegistry::GetString(WALLET_SECRET_KEY, strWalletKey));

	ZKSTD::ostringstream ossRequest;
	ossRequest << WALLET_CMD_ADD_LOGIN_INFO << "?" 
		<< WALLET_PARAM_SECRET_KEY << "=" << ZKS::EscapeString(strWalletKey) << "&"
		<< WALLET_PARAM_DOMAIN << "=" << ZKS::EscapeString(loginInfo.strDomain) << "&" 
		<< WALLET_PARAM_USERNAME << "=" << ZKS::EscapeString(loginInfo.strUsername) << "&"
		<< WALLET_PARAM_PASSWORD << "=" << ZKS::EscapeString(loginInfo.strPassword) << "&";

	ZkSocket sock = ConnectAndSend(ossRequest.str());
	DZkError err;
	ZKTEST(zkSocketFree(&err, &sock));	

	::Sleep(500);
	pNymWallet->GetLoginInfo(loginInfo.strDomain, loginInfoVector);
	ZKTEST(loginInfoVector.size() == 1);

	ZKTEST(pNymWallet->DeleteLoginInfo(loginInfo));


	// Test WALLET_CMD_FILL_IN_FORM & WALLET_CMD_KILL_WINDOW
	
	ZKSTD::ostringstream ossRequest2;
	ossRequest2 << WALLET_CMD_FILL_IN_FORM << "?"
		<< WALLET_PARAM_SECRET_KEY << "=" << ZKS::EscapeString(strWalletKey) << "&"
		<< WALLET_PARAM_DOMAIN << "=" << "www.NoLoginInfoForThisDomain.com" << "&" 
		<< WALLET_FORMELEM_IDENTIFIER	<< WALLET_FORMELEM_TEXT << "=" << "uname" << "&"
		<< WALLET_FORMELEM_IDENTIFIER	<< WALLET_FORMELEM_PASSWORD << "=" << "pswd" << "&";

	sock = ConnectAndSend(ossRequest2.str());
	::Sleep(1000);

	ZKSTD::ostringstream ossRequest3;
	ossRequest3 << WALLET_CMD_KILL_WINDOW << "?" 
		<< WALLET_PARAM_SECRET_KEY << "=" << ZKS::EscapeString(strWalletKey);


	ZkSocket sock2 = ConnectAndSend(ossRequest3.str());
	ZKTEST(zkSocketFree(&err, &sock2));	

	ZKSTD::string strResponse;
	Read(sock, strResponse);
	ZKTEST(zkSocketFree(&err, &sock));	

	ZKTEST(strResponse == "");
}


ZkSocket DIpcHttpServerUnitTest::ConnectAndSend(const ZKSTD::string& in_strRequest)
{
	DZkError err;

	ZkSocket sock;
	ZKTEST(zkSocketNew(&err, AF_INET, SOCK_STREAM, IPPROTO_TCP, &sock));
	ZKTEST(zkSocketSetOption(&err, sock, ZKSOCKET_OPTION_SendTimeout, 15 * 1000));
	ZKTEST(zkSocketSetOption(&err, sock, ZKSOCKET_OPTION_RecvTimeout, 45 * 1000));

	LPHOSTENT lpHostEntry = NULL;
    ZKTEST(zkGetHostByName(NULL, "localhost", &lpHostEntry));
    ZKTEST(lpHostEntry);
	ZkSocketAddress sockAddr;
	sockAddr.m_inet.sa.sin_family = AF_INET;
	sockAddr.m_inet.sa.sin_port   = htons(LOCAL_IPC_HTTP_SERVER_PORT);
	sockAddr.m_inet.sa.sin_addr   = *((LPIN_ADDR)*lpHostEntry->h_addr_list);

	ZKTEST(zkSocketConnectAddr(&err, sock, sockAddr));

	ZKSTD::string strRequest = "GET /" + in_strRequest + " HTTP/1.0\r\n\r\n";
	ZKTEST(zkSocketWriteAll(&err, sock, strRequest.c_str(), strRequest.size()));

	return sock;
}

bool DIpcHttpServerUnitTest::Read(ZkSocket& sock, ZKSTD::string&  out_strResponse)
{
	bool bRet = false;

	DZkError err;

	const unsigned int nBufLen = 1000;
	char recvBuf[nBufLen] = { 0 };
	unsigned int nBytesRead = 0;

	unsigned long numBytesReceived = 0;

	out_strResponse.clear();
	if (zkSocketRead(&err, sock, recvBuf, nBufLen, &nBytesRead))
	{
		bRet = true;
		if ((nBytesRead > 0) && (nBytesRead <= nBufLen))
		{
			out_strResponse.append(recvBuf, nBytesRead);
		}
	}
	
	return bRet;	
}

#endif // INTERNAL_BUILD
