// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

/////////////////////////////////////////////////////////////////////////////////
//
// Wallet.cpp

#include "stdafx.h"
#include "Wallet.h"
#include "WalletConsts.h"
#include <IZkConsoleMsg.h>
#include <ZkService.h>
#include <INymCentral.h>
#include <INym.h>
#include <INymWallet.h>
#include <IClientCentral.h>
#include <WalletRegistryKeys.h>
// Marcp: this need to be fixed
#include "../win/ZKBaseUI.h"
#include <string_hlp.h>
#include <lockguard.h>
#include <IZkRegistry.h>
#include <IZkPopup.h>
#include <ZKKeybrd.h>
#include <IscriptServer.h>
#include <IZkAppState.h>
#include <ZkStringMgr.h>

//static members
CDialog* Wallet::s_pDialog = NULL;
ZKCriticalSection Wallet::s_cs;
bool Wallet::s_bGeneralFormSeen = false;

Wallet::Wallet(const ZKSTD::string& in_strRequest, const ZkSocket& in_sock)
	: m_hDialogClosedEvent(NULL)
	, m_strRequest(in_strRequest)
	, m_pStrDomain(NULL)
	, m_bFillInForm(false)
	, m_sock(in_sock)
{
}

Wallet::~Wallet()
{
	SAFEDELETE(m_pStrDomain);
	SAFECLOSEHANDLE(m_hDialogClosedEvent);
}

bool Wallet::Run(ZKSTD::string& out_strResponse)
{
	bool bRet = false;
	
	if (!IZkAppState::IsWalletReady())
	{
		ZKASSERT(false);
	}
	else if (!GetCommand())
	{
		ZKASSERT(false);
	}
	else if (!GetParameters())
	{
		ZKASSERT(false);
	}
	else if (!IsAuthenticated())
	{
		IZkConsoleMsg::ShowMsg(IDS_CONS_WALLET_UNAUTHENTICATED_ATTEMPT);
		ZKASSERT(false);
	}
	else if (!ProcessCommand())
	{
		ZKASSERT(false);
	}
	else
	{
		bRet = true;
	}
	
	out_strResponse = m_strResponse;

	return bRet;
}

bool Wallet::IsAuthenticated()
{
	bool bRet = false;

#ifdef INTERNAL_BUILD
	// For debugging - allow shared secret to be bypassed
	ZKSTD::string strWalletKey2;
	if ((IZkRegistry::GetString(WALLET_SECRET_KEY, strWalletKey2))
		&& (strWalletKey2 == "ignore"))
	{
		return true;
	}
#endif //INTERNAL_BUILD

	String2StringMap::const_iterator itWalletKey = m_param2ValueMap.find(WALLET_PARAM_SECRET_KEY);
	ZKSTD::string strWalletKey;

	if ((itWalletKey != m_param2ValueMap.end())
		&& (IZkRegistry::GetString(WALLET_SECRET_KEY, strWalletKey)))
	{
		bRet = ((*itWalletKey).second == strWalletKey);
	}
	return bRet;
}

bool Wallet::GetCommand()
{
	bool bRet = false;

	m_strCommand.clear();

	ZKSTD::string::size_type pos1;

	if (m_strRequest == "/")
	{
		ZKASSERT(false);
	}
	else if ((pos1 = m_strRequest.find('/')) != 0)
	{
		ZKASSERT(false);
	}
	else {
		ZKSTD::string::size_type pos2 = m_strRequest.find_first_of("? ", pos1);

		m_strCommand = m_strRequest.substr(pos1 + 1, 
			(pos2 == m_strRequest.npos)
			? m_strRequest.npos
			: pos2 - pos1 - 1);

		m_strCommand = ZKS::UnEscapeString(m_strCommand);
//		ZKS::MakeLower(m_strCommand);

		bRet = true;
	}

	return bRet;
}

bool Wallet::GetParameters()
{
	bool bRet = true;

	m_formElemVect.clear();
	m_param2ValueMap.clear();

	ZKSTD::string::size_type pos1 = m_strRequest.find('?');
	//If no '?' is found, there are no parameters.

	if (pos1 == m_strRequest.size() - 1)
	{
		//No parameters, so don't choke
		pos1 = m_strRequest.npos;
	}

	while (pos1 != m_strRequest.npos)
	{
		ZKSTD::string::size_type pos2 = m_strRequest.find('=', pos1);
		if (pos2 == m_strRequest.npos)
		{
			ZKASSERT(false);
			bRet = false;
			break;
		}
		ZKSTD::string::size_type pos3 = m_strRequest.find('&', pos2);
		
		ZKSTD::string strParam 
			= ZKS::UnEscapeString(m_strRequest.substr(pos1 + 1, pos2 - pos1 - 1));
		//ZKS::MakeLower(strParam);

		ZKSTD::string strValue;
		if (pos3 == m_strRequest.npos)
		{
			strValue = m_strRequest.substr(pos2 + 1, m_strRequest.npos);
		}
		else
		{
			strValue = m_strRequest.substr(pos2 + 1, pos3 - pos2 - 1 );
		}
		strValue = ZKS::UnEscapeString(strValue);

		if (strParam.find(WALLET_FORMELEM_IDENTIFIER) == 0)
		{
			//This is a form Element
			m_formElemVect.push_back(
				HtmlFormElem(strValue, strParam.substr(WALLET_FORMELEM_IDENTIFIER.size())));
		}
		else
		{
			//This is some other parameter
			m_param2ValueMap[strParam] = strValue;
			if (strParam == WALLET_PARAM_DOMAIN)
			{
				ZKASSERT(!m_pStrDomain);
				m_pStrDomain = new ZKSTD::string(strValue);
			}
		}

		if (pos3 == (m_strRequest.size() - 1))
		{
			// case where request ends in "&" eg Command?param1=value1&param2=value2&
			// We won't choke if this happens
			break;
		}
		pos1 = pos3;
	}

	return bRet;
}

bool Wallet::ProcessCommand()
{
	bool bRet = true;

	if (m_pStrDomain
		&& ZkService::GetIClientCentral()->IsWalletDisabledForDomain(*m_pStrDomain))
	{
		IZkConsoleMsg::ShowMsg(
			IDS_CONS_WALLET_DISABLED_FOR_DOMAIN,
			m_pStrDomain->c_str());

	}
	else if (m_strCommand == WALLET_CMD_FILL_IN_FORM)
	{
		CmdFillInForm();
	}
	else if (m_strCommand == WALLET_CMD_KILL_WINDOW)
	{
		CmdKillWindow();
	}
	else if (m_strCommand == WALLET_CMD_ADD_LOGIN_INFO)
	{
		CmdAddLoginInfo();
	}
#ifdef INTERNAL_BUILD
	else if (m_strCommand == WALLET_CMD_TEACH_FORM)
	{
		if (CmdTeachForm())
		{
			m_strResponse = WALLET_SERVER_ACK;
		}
		else
		{
			m_strResponse = WALLET_SERVER_NACK;
		}
	}
	else if (m_strCommand == WALLET_CMD_KNOW_FORM)
	{
		if (CmdKnowForm() > 0)
		{
			m_strResponse = "";

			for (unsigned int i = 0; i < m_formElemVect.size(); ++i)
			{
				String2StringMap::const_iterator it 
					= m_realNameLwr2ZkNameMap.find( m_formElemVect[i].strNameLwr );

				if (it != m_realNameLwr2ZkNameMap.end())
				{
					ZKASSERT( (*it).first == m_formElemVect[i].strNameLwr );
					const ZKSTD::string strZkName = (*it).second;
					m_strResponse += m_formElemVect[i].strName + '\t' + strZkName + '\n';
				}
			}
		}
		else
		{
			m_strResponse = WALLET_SERVER_NACK;
		}
	}
#endif //INTERNAL_BUILD
	else
	{
		bRet = false;
		ZKASSERT(false); //Unrecogized command;
	}

	return bRet;
}

bool Wallet::CmdAddLoginInfo()
{
	bool bRet = false;

	String2StringMap::const_iterator itUsername = m_param2ValueMap.find(WALLET_PARAM_USERNAME);
	String2StringMap::const_iterator itPassword = m_param2ValueMap.find(WALLET_PARAM_PASSWORD);

	if ((itUsername == m_param2ValueMap.end())
		|| (itPassword == m_param2ValueMap.end())
		|| (!m_pStrDomain))
	{
		ZKASSERT(false);
	}
	else {
		INym* pNym = NULL;
		ZkService::GetINymCentral()->GetActiveNym( IID_INYM, (void far**)&pNym );

		if (pNym == NULL)
		{
			ZKASSERT(false);
		}
		else
		{
			ZKSTD::string strActiveNym;
			pNym->GetNymName(strActiveNym);

			INymWallet* pNymWallet = NULL ;
			pNym->QueryInterface( IID_INYMWALLET, (void far**) &pNymWallet ) ;

			if (pNymWallet == NULL)
			{
				ZKASSERT(false);
			}
			else
			{
				LoginInfo loginInfo(strActiveNym, 
									(*itUsername).second, 
									(*itPassword).second, 
									*m_pStrDomain);
				if (pNymWallet->AddLoginInfo(loginInfo))
				{
					IZkConsoleMsg::ShowMsg(
						IDS_CONS_WALLET_ADDED_LOGIN_INFO,
						loginInfo.strUsername.c_str(),
						loginInfo.strDomain.c_str(),
						loginInfo.strNym.c_str());
				}
				bRet = true;
			}
		}
	}
	return bRet;
}

#ifdef INTERNAL_BUILD
bool Wallet::CmdTeachForm()
{
	bool bRet = false;

	String2StringMap::const_iterator it;
	for (it = m_param2ValueMap.begin(); it != m_param2ValueMap.end(); ++it)
	{
		if (((*it).first != WALLET_PARAM_DOMAIN)
			&& ((*it).first != WALLET_PARAM_SECRET_KEY))
		{
			m_realNameLwr2ZkNameMap.insert(*it);
		}
	}

	if (!m_pStrDomain)
	{
		ZKASSERT(false); //missing parameter
	}
	else if (!ZkService::GetIScriptServer()->AddScriptMappings(*m_pStrDomain, m_realNameLwr2ZkNameMap))
	{
		IZkPopup::MessageBox("Failed adding scripts\n\n"
							 "You must be\n - behind the Zero firewall, and\n"
							 "- have no nym selected\nto add scripts.",
							  ZK_MESSAGE,
							  OVER_CENTER);
		//Domain was likely screwy
	}
	else
	{
		IZkPopup::MessageBox("Successfully added scripts!\n"
							 "Hit back, then forward to try it out.",
							 ZK_MESSAGE,
							 OVER_CENTER);
		bRet = true;
	}
	return bRet;
}
#endif //INTERNAL_BUILD

int Wallet::CmdKnowForm()
{
	int nFormElemsKnown = 0;

	if (!m_pStrDomain)
	{
		ZKASSERT(false); //missing parameter
	}
	else if (!ZkService::GetIWallet()->GetScriptMappings(*m_pStrDomain, m_realNameLwr2ZkNameMap))
	{
		//Faulty file or faulty domain
	}
	else
	{
		for (unsigned int i = 0; i < m_formElemVect.size(); ++i)
		{
			if (m_realNameLwr2ZkNameMap.find( m_formElemVect[i].strNameLwr ) != m_realNameLwr2ZkNameMap.end())
			{
				++nFormElemsKnown;
			}
		}
	}
	return nFormElemsKnown;
}

bool Wallet::OneClickFillForm()
{
	bool bRet = true;

	CmdKillWindow();
	
	String2StringMap zkName2ValueMap;
	String2StringMultiMap zkName2RealNameMultiMap;

	for (unsigned int i = 0; i < m_formElemVect.size(); ++i)
	{
		String2StringMap::const_iterator it = m_realNameLwr2ZkNameMap.find(m_formElemVect[i].strNameLwr);
		if (it != m_realNameLwr2ZkNameMap.end())
		{
			const ZKSTD::string& strZkName = (*it).second;
			zkName2ValueMap[strZkName] = "";
			zkName2RealNameMultiMap.insert(
				String2StringMultiMap::value_type(strZkName, m_formElemVect[i].strName));
		}
	}
	ZKASSERT(zkName2ValueMap.size() > 0);

	m_hDialogClosedEvent = ::CreateEvent(NULL, TRUE, FALSE, NULL);
	ZKASSERT(m_hDialogClosedEvent);

	::PostMessage(	ZKBaseUI::GetFreedomBarHwnd(), 
					WM_WALLET_ONE_CLICK_FORM_FILL,	
					(WPARAM) &zkName2ValueMap,
					(LPARAM) this);

	::WaitForSingleObject(m_hDialogClosedEvent, INFINITE);

	if (m_bFillInForm)
	{
		ZKASSERT(m_strResponse == "");

		for (String2StringMap::const_iterator it = zkName2ValueMap.begin(); 
			it != zkName2ValueMap.end(); ++it)
		{
			const ZKSTD::string& strValue = (*it).second;
			if (strValue != "")
			{
				const ZKSTD::string& strZkName = (*it).first;

				ZKASSERT(zkName2RealNameMultiMap.lower_bound(strZkName) 
					!= zkName2RealNameMultiMap.end());
				
				for (String2StringMultiMap::const_iterator it2 
					= zkName2RealNameMultiMap.equal_range(strZkName).first;
					it2 != zkName2RealNameMultiMap.equal_range(strZkName).second;
					++it2)
				{
					const ZKSTD::string& strRealName = (*it2).second;
					m_strResponse += strRealName + '\n';
					m_strResponse += strValue + '\n';
				}
			}
		}

		if (!m_pStrDomain)
		{
			ZKASSERT(false);
		}
		else
		{
			ZKSTD::string strActiveNymName;
			ZkService::GetINymCentral()->GetActiveNymName(strActiveNymName);
			IZkConsoleMsg::ShowMsg(
				IDS_CONS_WALLET_FILLED_IN_FORM,
				m_pStrDomain->c_str(),
				strActiveNymName.c_str());
		}
	}
	else
	{
		m_strResponse = "";
	}
	return bRet;
}


bool Wallet::CmdFillInForm()
{
	unsigned int numPasswordElems = 0;
	unsigned int numTextElems = 0;
	for (unsigned int i = 0; i < m_formElemVect.size(); ++i)
	{
		if (m_formElemVect[i].strType == WALLET_FORMELEM_PASSWORD)
		{
			++numPasswordElems;
		}
		else if (m_formElemVect[i].strType == WALLET_FORMELEM_TEXT)
		{
			++numTextElems;
		}
	}

	if ((numPasswordElems == 1) && (numTextElems > 0))
	{
		return FillInLoginForm();
	}
	else if ((numPasswordElems == 1) && (numTextElems == 0))
	{
		return (FillInLoginForm(true)); //true -> re-login form
	}
	else if (CmdKnowForm() >= MIN_KNOWN_FORM_ELEMS_TO_POP_ONE_CLICK_FILL)
	{
		return OneClickFillForm();
	}
	else
	{	
		if (numTextElems >= MIN_TEXT_FIELDS_TO_INFORM_OF_DANDD)
		{
			return InformOfDragAndDrop();
		}
	}
	return true;
}

bool Wallet::FillInLoginForm(bool in_isReloginForm)
{
	bool bRet;

	if (!m_pStrDomain)
	{
		ZKASSERT(false); //missing parameter
		bRet = false;
	}
	else 
	{
		ZKSTD::vector<LoginInfo> loginInfoVector;
		unsigned int numNyms = ZkService::GetINymCentral()->GetNumberOfNyms();
		INymWallet* pNymWallet = NULL;
		bool bFoundLoginInfoForThisDomain = false;

		for (unsigned int i = 0; i < numNyms; ++i)
		{
			ZkService::GetINymCentral()->GetNym(IID_INYMWALLET, (LPVOID*)&pNymWallet, i);

			if (!pNymWallet)
			{
				ZKASSERT(false);
			}
			else
			{
				pNymWallet->GetLoginInfo(*m_pStrDomain, loginInfoVector);
				if (loginInfoVector.size() > 0)
				{
					bFoundLoginInfoForThisDomain = true;
					break;
				}
			}
		}
		//Kill window if there is one
		CmdKillWindow();

		m_hDialogClosedEvent = ::CreateEvent(NULL, TRUE, FALSE, NULL);
		ZKASSERT(m_hDialogClosedEvent);
		
		if (!in_isReloginForm && !bFoundLoginInfoForThisDomain
			&& !ZkService::GetIClientCentral()->GetPreference(
					ZK_SHOW_AGAIN_PREF_SET, 
					PREF_NO_SHOW_LOGIN_NORMALLY))
		{
			SAFECLOSESOCKET(m_sock->descriptor);
			::PostMessage(	ZKBaseUI::GetFreedomBarHwnd(), 
							WM_WALLET_LOGIN_NORMALLY,	
							(WPARAM)0, 
							(LPARAM)this);
		}
		else if (bFoundLoginInfoForThisDomain)
		{
			::PostMessage(	ZKBaseUI::GetFreedomBarHwnd(), 
							WM_WALLET_SELECT_LOGIN_INFO,	
							(WPARAM)0, 
							(LPARAM)this);
		}
		//The PostMessage returns before the dialog is popped up.
		//Upon creation, the dialog will callback to CallBackInformOfMyDialog
		//to give us a CDialog* so we can close it if we receive the KillWindow cmd.
		//Just before the dialog is closed (by us or the user), it calls
		//CallBackInformThatWindowClosed which signals m_hDialogClosedEvent

		::WaitForSingleObject(m_hDialogClosedEvent, INFINITE);

		if (m_bFillInForm)
		{
			ZKASSERT(m_loginInfo.strUsername != "");
			ZKASSERT(m_loginInfo.strPassword != "");

			for (unsigned int i = 0; i < m_formElemVect.size(); ++i)
			{
				if (m_formElemVect[i].strType == WALLET_FORMELEM_PASSWORD)
				{
					m_strResponse += m_formElemVect[i].strName + "\n";
					m_strResponse += m_loginInfo.strPassword + '\n';
				}
				else if (m_formElemVect[i].strType == WALLET_FORMELEM_TEXT)
				{
					m_strResponse += m_formElemVect[i].strName + "\n";
					m_strResponse += m_loginInfo.strUsername + '\n';
				}
			}
			ZKSTD::string strActiveNymName;
			ZkService::GetINymCentral()->GetActiveNymName(strActiveNymName);
			IZkConsoleMsg::ShowMsg(
				IDS_CONS_WALLET_FILLED_IN_FORM,
				m_pStrDomain->c_str(),
				strActiveNymName.c_str());
		}
	}
	return true;
}

bool Wallet::InformOfDragAndDrop()
{
	SAFECLOSESOCKET(m_sock->descriptor);
	//Kill window if there is one
	CmdKillWindow();

	if (!s_bGeneralFormSeen)
	{
		s_bGeneralFormSeen = true;
		if (!ZkService::GetIClientCentral()->GetPreference(
			ZK_SHOW_AGAIN_PREF_SET, 
			PREF_NO_INFORM_DRAG_AND_DROP))
		{
			ZKSTD::string strHotKey;
			ZKKeybrd::GetHotKeyAsString(ZK_SHOW_WALLET_HK_REGKEY, strHotKey);

			ZKSTD::string strMsg;
			ZkStringMgr::Load(strMsg, IDS_WALLET_HOW_TO_DRAG_AND_DROP, strHotKey.c_str());

			if (IZkPopup::MessageCheckBox(tstring(strMsg),
											IDS_DONT_SHOW_AGAIN_DEFAULT,
											false,
											ZK_INFORMATION,
											OVER_LAST_ACTIVE))
			{
				ZkService::GetIClientCentral()->SetPreference(
					ZK_SHOW_AGAIN_PREF_SET, 
					PREF_NO_INFORM_DRAG_AND_DROP,
					true);
			}
		}
	}
	return true;
}


bool Wallet::CmdKillWindow()
{
	ZKS::Lock_Guard<ZKCriticalSection> gate(s_cs);

	if (s_pDialog)
	{
		::PostMessage(s_pDialog->m_hWnd, WM_WALLET_CANCEL_DIALOG, (WPARAM) 0, (LPARAM) 0);
	}
	return true;
}

void Wallet::CallBackInformThatWindowClosed(const LoginInfo& in_loginInfo)
{
	m_loginInfo = in_loginInfo;
	
	CallBackInformThatWindowClosed(true);
}

void Wallet::CallBackInformThatWindowClosed(bool in_bFillInForm)
{
	m_bFillInForm = in_bFillInForm;
	
	if (m_hDialogClosedEvent)
	{
		::SetEvent(m_hDialogClosedEvent);
	}

	//Dangerous, but can't use lock in here because the lock is already set in CmdKillWindow
	s_pDialog = NULL;
}
void Wallet::CallBackInformOfMyDialog(CDialog* in_pDialog)
{
	ZKS::Lock_Guard<ZKCriticalSection> gate(s_cs);

	ZKASSERT(s_pDialog == NULL);
	s_pDialog = in_pDialog;
}

#ifdef INTERNAL_BUILD

#include <sstream>
#include <IZkFileHelp.h>
#include <ZKFileConfiguration.h>

void Wallet::SelfTest()
{
	const unsigned int nParams = 3;
	char pStrParam[nParams][20] = {"sfgGg&96", "shfbkSDjs", "kjhf3732(*&"};
	char pStrValue[nParams][20] = {"hre@#%^#", "(^%FS", "sDSGdah"};
	char strCommand[] = "someCommand";

	ZKSTD::ostringstream oss;

	oss << "/" << ZKS::EscapeString(strCommand) << "?";
	for (unsigned int i = 0; i < nParams; ++i)
	{
		oss << ZKS::EscapeString(pStrParam[i]) << "=" << ZKS::EscapeString(pStrValue[i]) << "&";
	}

	m_strRequest = oss.str();

	ZKTEST(GetParameters());

	ZKTEST(GetCommand());

	ZKTEST(m_strCommand == strCommand);

	ZKTEST(m_param2ValueMap.size() == nParams);

	for (i = 0; i < nParams; ++i)
	{		
		ZKSTD::string strParam = pStrParam[i];

		String2StringMap::const_iterator itParam = m_param2ValueMap.find(strParam);
		ZKTEST(itParam != m_param2ValueMap.end());
		ZKTEST((*itParam).second == pStrValue[i]);
	}

	//////////////////////////////////////

	ClearAllMembers();
	m_pStrDomain = new ZKSTD::string("domain_for_unit_test.com");

	m_formElemVect.push_back(HtmlFormElem("rastaunite1", "text"));
	m_formElemVect.push_back(HtmlFormElem("RastaUnite2", "text"));
	m_formElemVect.push_back(HtmlFormElem("RASTAUNITE3", "text"));

	ZKSTD::string strFile = ZKFileConfiguration::GetWalletScriptPath(*m_pStrDomain);
	{
		ZKSTD::ofstream of(strFile.c_str());
		for (unsigned int i = 0; i < m_formElemVect.size(); ++i)
		{
			of << m_formElemVect[i].strName << '\t' << "The_Zk_Name\n";
		}
	}
	ZKTEST(IZkFileHelp::DoesFileOrDirExist( strFile.c_str() ));
	ZKTEST(CmdKnowForm() == (int) m_formElemVect.size());
}

void Wallet::ClearAllMembers()
{
	s_pDialog = NULL;
	SAFECLOSEHANDLE(m_hDialogClosedEvent);
	ZKASSERT(!m_hDialogClosedEvent);
	
	m_loginInfo = LoginInfo();
	m_strCommand.erase();
	m_param2ValueMap.clear();
	m_strRequest.erase();
	m_formElemVect.clear();
	m_strResponse.erase();
	SAFEDELETE(m_pStrDomain);
}

#endif //INTERNAL_BUILD