// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// WalletConsts.h

#ifndef __WALLET_CONSTS_H__
#define __WALLET_CONSTS_H__

#include <client.h> //for ZKSTD
#include <clientdefs.h>


#define MIN_TEXT_FIELDS_TO_INVOKE_WALLET			2
#define MIN_TEXT_FIELDS_TO_INFORM_OF_DANDD			5
#define MIN_KNOWN_FORM_ELEMS_TO_POP_ONE_CLICK_FILL	2

///////////////////////////////////////////////////////////////////////////////////////////////////
// Wallet Commands accepted by DIpcHttpServer

static const char WALLET_CMD_KILL_WINDOW[]		= "killwindow";
static const char WALLET_CMD_ADD_LOGIN_INFO[]	= "addlogininfo";
static const char WALLET_CMD_FILL_IN_FORM[]		= "fillinform";
#ifdef	INTERNAL_BUILD
static const char WALLET_CMD_TEACH_FORM[]		= "teachform";
static const char WALLET_CMD_KNOW_FORM[]		= "knowform";
#endif	//INTERNAL_BUILD

///////////////////////////////////////////////////////////////////////////////////////////////////
// Wallet Parameters accepted by DIpcHttpServer

static const char WALLET_PARAM_DOMAIN[]					= "domain";
static const char WALLET_PARAM_USERNAME[]				= "username";
static const char WALLET_PARAM_PASSWORD[]				= "password";
static const char WALLET_PARAM_ECML[]					= "ecml";
static const char WALLET_PARAM_SECRET_KEY[]				= "key";

static const ZKSTD::string WALLET_FORMELEM_IDENTIFIER	= "zkfe";
static const char WALLET_FORMELEM_TEXT[]				= "text";
static const char WALLET_FORMELEM_PASSWORD[]			= "pwd";
static const char WALLET_FORMELEM_CHECKBOX[]			= "cb";
static const char WALLET_FORMELEM_SELECT[]				= "slt";
static const char WALLET_FORMELEM_RADIO[]				= "rb";

///////////////////////////////////////////////////////////////////////////////////////////////////
// These constants are used by the scripts to map real form input elements names to ZK names
// These are all of the ZK names.
// For names, addresses, phone, and fax, ZK names can be formed by appending a FE_* to a FE_PRE*
// For example, City, ShipTo_City, BillTo_City, and ReceiptTo_City are all valid.

static const ZKSTD::string FE_PRE_SHIPTO	= "ShipTo_";
static const ZKSTD::string FE_PRE_BILLTO	= "BillTo_";
static const ZKSTD::string FE_PRE_RECEIPTTO	= "ReceiptTo_";

static const char FE_NAME_FULL[]			= "Name_Full";
static const char FE_NAME_PREFIX[]			= "Name_Prefix";
static const char FE_NAME_FIRST[]			= "Name_First";
static const char FE_NAME_MIDDLE[]			= "Name_Middle";
static const char FE_NAME_LAST[]			= "Name_Last";
static const char FE_NAME_SUFFIX[]			= "Name_Suffix";

static const char FE_STREET1[]				= "Street1";
static const char FE_STREET2[]				= "Street2";
static const char FE_CITY[]					= "City";
static const char FE_STATE[]				= "State";
static const char FE_ZIP_CODE[]				= "Zip_Code";
static const char FE_COUNTRY[]				= "Country";
static const char FE_EMAIL[]				= "Email";

static const char FE_PHONE_FULL[]			= "Phone_Full";
static const char FE_PHONE_INT_CODE[]		= "Phone_Int_Code";
static const char FE_PHONE_LOC_CODE[]		= "Phone_Loc_Code";
static const char FE_PHONE_PREFIX[]			= "Phone_Prefix";
static const char FE_PHONE_SUFFIX[]			= "Phone_Suffix";
static const char FE_PHONE_EXTENSION[]		= "Phone_Extension";

static const char FE_FAX_FULL[]				= "Fax_Full";
static const char FE_FAX_INT_CODE[]			= "Fax_Int_Code";
static const char FE_FAX_LOC_CODE[]			= "Fax_Loc_Code";
static const char FE_FAX_PREFIX[]			= "Fax_Prefix";
static const char FE_FAX_SUFFIX[]			= "Fax_Suffix";

static const char FE_CCARD_NAME_ON_CARD[]	= "CCard_Name_On_Card";
static const char FE_CCARD_TYPE[]			= "CCard_Type";
static const char FE_CCARD_NUMBER[]			= "CCard_Number";
static const char FE_CCARD_EXPDATE_MONTH[]	= "CCard_ExpDate_Month";
static const char FE_CCARD_EXPDATE_YEAR[]	= "CCard_ExpDate_Year";
static const char FE_CCARD_EXPDATE_MMYY[]	= "CCard_ExpDate_MMYY";

static const char FE_BIRTH_DAY[]			= "Birth_Day";
static const char FE_BIRTH_MONTH[]			= "Birth_Month";
static const char FE_BIRTH_YEAR[]			= "Birth_Year";

static const char FE_GENDER[]				= "Gender";
static const char FE_ADD_TO_SPAM_LIST[]		= "Add_To_Spam_List";
static const char FE_ERASE_FIELD[]			= "ZK_Erase";


///////////////////////////////////////////////////////////////////////////////////////////////////
// These are responses from the DIpcHtppServer

static const char WALLET_YES[]					= "zk_yes";
static const char WALLET_NO[]					= "zk_no";

#ifdef INTERNAL_BUILD
static const char WALLET_SERVER_NACK[]			= "Failure";
static const char WALLET_SERVER_ACK[]			= "Success";
#endif	//INTERNAL_BUILD

static const char CONTENT_LENGTH[]				= "Content-Length:";

///////////////////////////////////////////////////////////////////////////////////////////////////
// All US states & Canadian provinces and their 2 letter codes

class StateOrProvince
{
public:
	StateOrProvince() {};
	StateOrProvince(const ZKSTD::string in_strName, const ZKSTD::string in_strAbbrev)
		: strName(in_strName)
		, strAbbrev(in_strAbbrev)
	{}
	const StateOrProvince& operator=(const StateOrProvince& in_rhs)
	{
		if (this == &in_rhs) return *this; // address equality
		strName = in_rhs.strName;
		strAbbrev = in_rhs.strAbbrev;
		return *this;
	}
	ZKSTD::string strName;
	ZKSTD::string strAbbrev;
};

static const StateOrProvince WALLET_STATES[] = 
{
	StateOrProvince("alabama", "al"),
	StateOrProvince("alaska", "ak"),
	StateOrProvince("american samoa", "as"),
	StateOrProvince("arizona", "az"),
	StateOrProvince("arkansas", "ar"),
	StateOrProvince("california", "ca"),
	StateOrProvince("colorado", "co"),
	StateOrProvince("connecticut", "ct"),
	StateOrProvince("delaware", "de"),
	StateOrProvince("district of columbia", "dc"),
	StateOrProvince("federated states of micronesia", "fm"),
	StateOrProvince("florida", "fl"),
	StateOrProvince("georgia", "ga"),
	StateOrProvince("guam", "gu"),
	StateOrProvince("hawaii", "hi"),
	StateOrProvince("idaho", "id"),
	StateOrProvince("illinois", "il"),
	StateOrProvince("indiana", "in"),
	StateOrProvince("iowa", "ia"),
	StateOrProvince("kansas", "ks"),
	StateOrProvince("kentucky", "ky"),
	StateOrProvince("louisianna", "la"),
	StateOrProvince("maine", "me"),
	StateOrProvince("marshall islands", "mh"),
	StateOrProvince("maryland", "md"),
	StateOrProvince("massachusetts", "ma"),
	StateOrProvince("michigan", "mi"),
	StateOrProvince("minnesota", "mn"),
	StateOrProvince("mississippi", "ms"),
	StateOrProvince("missouri", "mo"),
	StateOrProvince("montana", "mt"),
	StateOrProvince("nebraska", "ne"),
	StateOrProvince("new hampshire", "nh"), 
	StateOrProvince("new jersey", "nj"), 
	StateOrProvince("new mexico", "nm"), 
	StateOrProvince("new york", "ny"), 
	StateOrProvince("north carolina", "nc"), 
	StateOrProvince("north dakota", "nd"), 
	StateOrProvince("northern mariana islands", "mp"), 
	StateOrProvince("ohio", "oh"), 
	StateOrProvince("oklahoma", "ok"), 
	StateOrProvince("oregon", "or"), 
	StateOrProvince("palau", "pw"), 
	StateOrProvince("pennsylvania", "pa"), 
	StateOrProvince("puerto rico", "pr"), 
	StateOrProvince("rhode island", "ri"), 
	StateOrProvince("south carolina", "sc"), 
	StateOrProvince("south dakota", "sd"), 
	StateOrProvince("tennessee", "tn"), 
	StateOrProvince("texax", "tx"), 
	StateOrProvince("utah", "ut"), 
	StateOrProvince("vermont", "vt"), 
	StateOrProvince("virgin islands", "vi"), 
	StateOrProvince("virginia", "va"), 
	StateOrProvince("washinton", "wa"), 
	StateOrProvince("west virginia", "wv"), 
	StateOrProvince("wisconsin", "wi"), 
	StateOrProvince("wyoming", "wy"), 
	
	StateOrProvince("newfoundland", "nf"), 
	StateOrProvince("prince edward island", "pe"), 
	StateOrProvince("nova scotia", "ns"), 
	StateOrProvince("new brunswick", "nb"), 
	StateOrProvince("duebec", "qc"), 
	StateOrProvince("ontario", "on"), 
	StateOrProvince("manitoba", "mb"), 
	StateOrProvince("saskatchewan", "sk"), 
	StateOrProvince("alberta", "ab"), 
	StateOrProvince("british columbia", "bc"), 
	StateOrProvince("yukon", "yt"), 
	StateOrProvince("northwest territories", "nt")
};

static const unsigned int WALLET_STATES_SIZE = sizeof(WALLET_STATES) / sizeof(StateOrProvince);


///////////////////////////////////////////////////////////////////////////////////////////////////
// All countries and their 2 letter codes as defined by the ISO3166 standard

class CountryNameAndCode{
public:
	CountryNameAndCode(const char* in_strCountry, const char* in_strCode)
		: strCountry(in_strCountry)
		, strCode(in_strCode)
	{};
	const char* strCountry;
	const char* strCode;
};

static const CountryNameAndCode WALLET_COUNTRIES[] = {

#define COUNTRY_US_INDEX 0
	CountryNameAndCode("United States", "US"),
#define COUNTRY_CANADA_INDEX 1
	CountryNameAndCode("Canada", "CA"),
	CountryNameAndCode("Afghanistan", "AF"),
    CountryNameAndCode("Albania", "AL"), 
    CountryNameAndCode("Algeria", "DZ"),
    CountryNameAndCode("American Samoa", "AS"),
    CountryNameAndCode("Andorra", "AD"),
    CountryNameAndCode("Angola", "AO"),
    CountryNameAndCode("Anguilla", "AI"),
    CountryNameAndCode("Antarctica", "AQ"),
    CountryNameAndCode("Antigua And Barbuda", "AG"),
    CountryNameAndCode("Argentina", "AR"),
    CountryNameAndCode("Armenia", "AM"),
    CountryNameAndCode("Aruba", "AW"),
    CountryNameAndCode("Australia", "AU"),
    CountryNameAndCode("Austria", "AT"),
    CountryNameAndCode("Azerbaijan", "AZ"),
    CountryNameAndCode("Bahamas", "BS"),
    CountryNameAndCode("Bahrain", "BH"),
    CountryNameAndCode("Bangladesh", "BD"),
    CountryNameAndCode("Barbados", "BB"),
    CountryNameAndCode("Belarus", "BY"),
    CountryNameAndCode("Belgium", "BE"),
    CountryNameAndCode("Belize", "BZ"),
    CountryNameAndCode("Benin", "BJ"),
    CountryNameAndCode("Bermuda", "BM"),
    CountryNameAndCode("Bhutan", "BT"),
    CountryNameAndCode("Bolivia", "BO"),
    CountryNameAndCode("Bosnia and Herzegovina", "BA"),
    CountryNameAndCode("Botswana", "BW"),
    CountryNameAndCode("Bouvet Island", "BV"),
    CountryNameAndCode("Brazil", "BR"),
    CountryNameAndCode("British Indian Ocean Territory", "IO"),
    CountryNameAndCode("Brunei Darussalam", "BN"),
    CountryNameAndCode("Bulgaria", "BG"),
    CountryNameAndCode("Burkina Faso", "BF"),
    CountryNameAndCode("Burundi", "BI"),
    CountryNameAndCode("Cambodia", "KH"),
    CountryNameAndCode("Cameroon", "CM"),
    CountryNameAndCode("Cape Verde", "CV"),
    CountryNameAndCode("Cayman Islands", "KY"),
    CountryNameAndCode("Central African Republic", "CF"),
    CountryNameAndCode("Chad", "TD"),
    CountryNameAndCode("Chile", "CL"),
    CountryNameAndCode("China", "CN"),
    CountryNameAndCode("Christmas Island", "CX"),
    CountryNameAndCode("Cocos (Keeling) Islands", "CC"),
    CountryNameAndCode("Colombia", "CO"),
    CountryNameAndCode("Comoros", "KM"),
    CountryNameAndCode("Congo", "CG"),
    CountryNameAndCode("Congo, The Democratic Republic of the", "CD"),
    CountryNameAndCode("Cook Islands", "CK"),
    CountryNameAndCode("Costa Rica", "CR"),
    CountryNameAndCode("Cte D'Ivoire", "CI"),
    CountryNameAndCode("Croatia", "HR"),
    CountryNameAndCode("Cuba", "CU"),
    CountryNameAndCode("Cyprus", "CY"),
    CountryNameAndCode("Czech Republic", "CZ"),
    CountryNameAndCode("Denmark", "DK"),
    CountryNameAndCode("Djibouti", "DJ"),
    CountryNameAndCode("Dominica", "DM"),
    CountryNameAndCode("Dominican Republic", "DO"),
    CountryNameAndCode("East Timor", "TP"),
    CountryNameAndCode("Ecuador", "EC"),
    CountryNameAndCode("Egypt", "EG"),
    CountryNameAndCode("El Salvador", "SV"),
    CountryNameAndCode("Equatorial Guinea", "GQ"),
    CountryNameAndCode("Eritrea", "ER"),
    CountryNameAndCode("Estonia", "EE"),
    CountryNameAndCode("Ethiopia", "ET"),
    CountryNameAndCode("Falkland Islands (Malvinas)", "FK"),
    CountryNameAndCode("Faroe Islands", "FO"),
    CountryNameAndCode("Fiji", "FJ"),
    CountryNameAndCode("Finland", "FI"),
    CountryNameAndCode("France", "FR"),
    CountryNameAndCode("French Guiana", "GF"),
    CountryNameAndCode("French Polynesia", "PF"),
    CountryNameAndCode("French Southern Territories", "TF"),
    CountryNameAndCode("Gabon", "GA"),
    CountryNameAndCode("Gambia", "GM"),
    CountryNameAndCode("Georgia", "GE"),
    CountryNameAndCode("Germany", "DE"),
    CountryNameAndCode("Ghana", "GH"),
    CountryNameAndCode("Gibraltar", "GI"),
    CountryNameAndCode("Greece", "GR"),
    CountryNameAndCode("Greenland", "GL"),
    CountryNameAndCode("Grenada", "GD"),
    CountryNameAndCode("Guadeloupe", "GP"),
    CountryNameAndCode("Guam", "GU"),
    CountryNameAndCode("Guatemala", "GT"),
    CountryNameAndCode("Guinea", "GN"),
    CountryNameAndCode("Guinea-Bissau", "GW"),
    CountryNameAndCode("Guyana", "GY"),
    CountryNameAndCode("Haiti", "HT"),
    CountryNameAndCode("Heard Island and McDonald Islands", "HM"),
    CountryNameAndCode("Holy See (Vatican City State)", "VA"),
    CountryNameAndCode("Honduras", "HN"),
    CountryNameAndCode("Hong Kong", "HK"),
    CountryNameAndCode("Hungary", "HU"),
    CountryNameAndCode("Iceland", "IS"),
    CountryNameAndCode("India", "IN"),
    CountryNameAndCode("Indonesia", "ID"),
    CountryNameAndCode("Iran, Islamic Republic of", "IR"),
    CountryNameAndCode("Iraq", "IQ"),
    CountryNameAndCode("Ireland", "IE"),
    CountryNameAndCode("Israel", "IL"),
    CountryNameAndCode("Italy", "IT"),
    CountryNameAndCode("Jamaica", "JM"),
    CountryNameAndCode("Japan", "JP"),
    CountryNameAndCode("Jordan", "JO"),
    CountryNameAndCode("Kazakstan", "KZ"),
    CountryNameAndCode("Kenya", "KE"),
    CountryNameAndCode("Kiribati", "KI"),
    CountryNameAndCode("Korea, Democratic People's Republic of", "KP"),
    CountryNameAndCode("Korea, Republic Of", "KR"),
    CountryNameAndCode("Kuwait", "KW"),
    CountryNameAndCode("Kyrgyzstan", "KG"),
    CountryNameAndCode("Lao People'S Democratic Republic", "LA"),
    CountryNameAndCode("Latvia", "LV"),
    CountryNameAndCode("Lebanon", "LB"),
    CountryNameAndCode("Lesotho", "LS"),
    CountryNameAndCode("Liberia", "LR"),
    CountryNameAndCode("Libyan Arab Jamahiriya", "LY"),
    CountryNameAndCode("Liechtenstein", "LI"),
    CountryNameAndCode("Lithuania", "LT"),
    CountryNameAndCode("Luxembourg", "LU"),
    CountryNameAndCode("Macau", "MO"),
    CountryNameAndCode("Macedonia, The Former Yugoslav Republic of", "MK"),
    CountryNameAndCode("Madagascar", "MG"),
    CountryNameAndCode("Malawi", "MW"),
    CountryNameAndCode("Malaysia", "MY"),
    CountryNameAndCode("Maldives", "MV"),
    CountryNameAndCode("Mali", "ML"),
    CountryNameAndCode("Malta", "MT"),
    CountryNameAndCode("Marshall Islands", "MH"),
    CountryNameAndCode("Martinique", "MQ"),
    CountryNameAndCode("Mauritania", "MR"),
    CountryNameAndCode("Mauritius", "MU"),
    CountryNameAndCode("Mayotte", "YT"),
    CountryNameAndCode("Mexico", "MX"),
    CountryNameAndCode("Micronesia, Federated States of", "FM"),
    CountryNameAndCode("Moldova, Republic Of", "MD"),
    CountryNameAndCode("Monaco", "MC"),
    CountryNameAndCode("Mongolia", "MN"),
    CountryNameAndCode("Montserrat", "MS"),
    CountryNameAndCode("Morocco", "MA"),
    CountryNameAndCode("Mozambique", "MZ"),
    CountryNameAndCode("Myanmar", "MM"),
    CountryNameAndCode("Namibia", "NA"),
    CountryNameAndCode("Nauru", "NR"),
    CountryNameAndCode("Nepal", "NP"),
    CountryNameAndCode("Netherlands", "NL"),
    CountryNameAndCode("Netherlands Antilles", "AN"),
    CountryNameAndCode("New Caledonia", "NC"),
    CountryNameAndCode("New Zealand", "NZ"),
    CountryNameAndCode("Nicaragua", "NI"),
    CountryNameAndCode("Niger", "NE"),
    CountryNameAndCode("Nigeria", "NG"),
    CountryNameAndCode("Niue", "NU"),
    CountryNameAndCode("Norfolk Island", "NF"),
    CountryNameAndCode("Northern Mariana Islands", "MP"),
    CountryNameAndCode("Norway", "NO"),
    CountryNameAndCode("Oman", "OM"),
    CountryNameAndCode("Pakistan", "PK"),
    CountryNameAndCode("Palau", "PW"),
    CountryNameAndCode("Palestinian Territory, Occupied", "PS"),
    CountryNameAndCode("Panama", "PA"),
    CountryNameAndCode("Papua New Guinea", "PG"),
    CountryNameAndCode("Paraguay", "PY"),
    CountryNameAndCode("Peru", "PE"),
    CountryNameAndCode("Philippines", "PH"),
    CountryNameAndCode("Pitcairn", "PN"),
    CountryNameAndCode("Poland", "PL"),
    CountryNameAndCode("Portugal", "PT"),
    CountryNameAndCode("Puerto Rico", "PR"),
    CountryNameAndCode("Qatar", "QA"),
    CountryNameAndCode("Runion", "RE"),
    CountryNameAndCode("Romania", "RO"),
    CountryNameAndCode("Russian Federation", "RU"),
    CountryNameAndCode("Rwanda", "RW"),
    CountryNameAndCode("Saint Helena", "SH"),
    CountryNameAndCode("Saint Kitts and Nevis", "KN"),
    CountryNameAndCode("Saint Lucia", "LC"),
    CountryNameAndCode("Saint Pierre and Miquelon", "PM"),
    CountryNameAndCode("Saint Vincent and the Grenadines", "VC"),
    CountryNameAndCode("Samoa", "WS"),
    CountryNameAndCode("San Marino", "SM"),
    CountryNameAndCode("Sao Tome and Principe", "ST"),
    CountryNameAndCode("Saudi Arabia", "SA"),
    CountryNameAndCode("Senegal", "SN"),
    CountryNameAndCode("Seychelles", "SC"),
    CountryNameAndCode("Sierra Leone", "SL"),
    CountryNameAndCode("Singapore", "SG"),
    CountryNameAndCode("Slovakia", "SK"),
    CountryNameAndCode("Slovenia", "SI"),
    CountryNameAndCode("Solomon Islands", "SB"),
    CountryNameAndCode("Somalia", "SO"),
    CountryNameAndCode("South Africa", "ZA"),
    CountryNameAndCode("South Georgia and the South Sandwich Islands", "GS"),
    CountryNameAndCode("Spain", "ES"),
    CountryNameAndCode("Sri Lanka", "LK"),
    CountryNameAndCode("Sudan", "SD"),
    CountryNameAndCode("Suriname", "SR"),
    CountryNameAndCode("Svalbard and Jan Mayen", "SJ"),
    CountryNameAndCode("Swaziland", "SZ"),
    CountryNameAndCode("Sweden", "SE"),
    CountryNameAndCode("Switzerland", "CH"),
    CountryNameAndCode("Syrian Arab Republic", "SY"),
    CountryNameAndCode("Taiwan, Province Of China", "TW"),
    CountryNameAndCode("Tajikistan", "TJ"),
    CountryNameAndCode("Tanzania, United Republic of", "TZ"),
    CountryNameAndCode("Thailand", "TH"),
    CountryNameAndCode("Togo", "TG"),
    CountryNameAndCode("Tokelau", "TK"),
    CountryNameAndCode("Tonga", "TO"),
    CountryNameAndCode("Trinidad And Tobago", "TT"),
    CountryNameAndCode("Tunisia", "TN"),
    CountryNameAndCode("Turkey", "TR"),
    CountryNameAndCode("Turkmenistan", "TM"),
    CountryNameAndCode("Turks and Caicos Islands", "TC"),
    CountryNameAndCode("Tuvalu", "TV"),
    CountryNameAndCode("Uganda", "UG"),
    CountryNameAndCode("Ukraine", "UA"),
    CountryNameAndCode("United Arab Emirates", "AE"),
    CountryNameAndCode("United Kingdom", "GB"),
    CountryNameAndCode("United States Minor Outlying Islands", "UM"),
    CountryNameAndCode("Uruguay", "UY"),
    CountryNameAndCode("Uzbekistan", "UZ"),
    CountryNameAndCode("Vanuatu", "VU"),
    CountryNameAndCode("Venezuela", "VE"),
    CountryNameAndCode("Vietnam", "VN"),
    CountryNameAndCode("Virgin Islands, British", "VG"),
    CountryNameAndCode("Virgin Islands, U.S.", "VI"),
    CountryNameAndCode("Wallis And Futuna", "WF"),
    CountryNameAndCode("Western Sahara", "EH"),
    CountryNameAndCode("Yemen", "YE"),
    CountryNameAndCode("Yugoslavia", "YU"),
    CountryNameAndCode("Zambia", "ZM"),
    CountryNameAndCode("Zimbabwe", "ZW")
};

static const unsigned int WALLET_COUNTRIES_SIZE = sizeof(WALLET_COUNTRIES) / sizeof(CountryNameAndCode);

#endif //__WALLET_CONSTS_H__