/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

 */

/*******************************************************************************
; Public definitions for the ZKSHIM driver
; Written by Marc Boucher <marc@mbsi.ca>
*******************************************************************************/

/******************************************************************************
 * Extensions for Linux client by Oliver Lavery <oliver@zks.net>
 ******************************************************************************/

#ifndef _ZKSHIM_H
#define _ZKSHIM_H

#ifdef ZK_SOLARIS
#include <sys/ioccom.h>
#endif

#ifdef ZK_LINUX
#include <linux/types.h>
#include <linux/ioctl.h>

#if defined(__KERNEL_STRICT_NAMES) && !defined(_ZKINTTYPES_H)
typedef         __u8            uint8_t;
typedef         __u16           uint16_t;
typedef         __u32           uint32_t;
#endif

#define ZKSHIM_MAJOR 10
#define ZKSHIM_MINOR 194
#endif

#ifdef  __cplusplus
extern "C" {
#endif

#define ZKSHIM_NAME		"clientshim"
#define ZKSHIM_DEVNAME		"/dev/"ZKSHIM_NAME
	


/* Client shim is a minor version upgrade for now. */
#define ZKSHIM_VERSION		"0.8"
#define ZKSHIM_PATCHLEV		"1"

/* Shim Action constants
 *	pretty obvious
 */
#  define ZSA_ALLOW 0

/*	this should implement a firewall style deny (no ICMP) */
#  define ZSA_DENY 1

/*	
 *	Redirect. In the case of a filtered packet it should go to userland. In the
 * 	case of a connect() call it should be redirected to a local proxy service 
 */
#  define ZSA_REDIRECT 2

/*	Redirect with IP transmission.
 *	This is used by freedom to get original dst IPs for connect() calls.
 *	should be equivilent to ZSA_REDIRECT for filtered packets.
 */
#  define ZSA_REDIRECT_TXIP 3



/* Rule Type constants */

/*	'Socket' rule -- one that is applied to local connect() calls
 */
#  define ZRU_SOCKET 0

/*	'Filter' rule -- one that is applied to packets being transfered to userspace
 */
#  define ZRU_FILTER 1




typedef struct ZkShimRule {
	uint8_t		zru_ip_tos;
	uint8_t		zru_ip_p;
	uint32_t	zru_ip_src, zru_ip_src_mask;
	uint32_t	zru_ip_dst, zru_ip_dst_mask;
	uint16_t	zru_sport;
	uint16_t	zru_dport;
	uid_t		zru_uid;
	pid_t		zru_pid;
	pid_t		zru_sid;
	/* Client extensions. These are really only useful for zru_type == ZRU_SOCKET */
	uint8_t		zru_type;
	uint8_t		zru_action;
	uint32_t	zru_ip_arg;
	uint16_t	zru_port_arg;
} ZkShimRule;

	
#define ZKSHIM_COOKIE_SIZE 64

typedef struct ZkShimSecret {

	char     cookie[ZKSHIM_COOKIE_SIZE];
	/* a predefine size seems to be safer, do we need to change the size at runtime?
	uint32_t cookie_len;
	*/
} ZkShimSecret;
	
#define ZKSHIM_IOCTL_ID	'Z'

#define ZKIOC_ADD_RULE _IOW (ZKSHIM_IOCTL_ID, 0x1, ZkShimRule)
#define ZKIOC_DEL_RULE _IOW (ZKSHIM_IOCTL_ID, 0x2, ZkShimRule)
#define ZKIOC_CLEAR_RULES _IO (ZKSHIM_IOCTL_ID, 0x3)
#define ZKIOC_ADD_SECRET _IOW (ZKSHIM_IOCTL_ID, 0x4, ZkShimSecret)

	

#ifdef  __cplusplus
}
#endif

#endif /* _ZKSHIM_H */
