#!/bin/bash
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

module_name="clientshim.o"
freedom_home="/usr/share/freedom"
bin=""
etc=""
kernel_bin="kernel"

release_name=""
release_version=""
kernel_version=`uname -r`

# this is built by the sourceball bundled with freedom.
source_driver_name="zkshim"

running_csum=`cat /proc/ksyms | grep -v '\[' | sum | awk -- '{print $1}'`

DEBUG=0

function debug_echo() {
	if [ 1 -eq $DEBUG ]; then
		echo -e "\033[33m"DEBUG: $@"\033[m"
	fi
}

function stderr_echo() {
    echo "$@" 1>&2
}

function get_distro() {
    if [ -e /etc/slackware-version ]; then 
	release_name='slackware'
	release_version=`cat /etc/slackware-version | sed -e 's/[^0-9]*\([0-9]\+\).\([0-9]\+\).\([0-9]*\)[^0-9]*/\1\2\3/'`
	return 0
# must be before redhat since redhat-release and mandrake release exist on mandrake
    elif [ -e /etc/mandrake-release ]; then
	release_name='mandrake'
	release_version=`cat /etc/mandrake-release | sed -e 's/[^0-9]*\([0-9]\+\).\([0-9]\+\).\([0-9]*\)[^0-9]*/\1\2\3/'`
	return 0
#    exist on corel ??
#    elif [ -e /etc/debian_version ]; then
#	release_name='debian'
#	release_version=`cat /etc/debian_version | sed -e 's/[^0-9]*\([0-9]\+\).\([0-9]\+\).\([0-9]*\)[^0-9]*/\1\2\3/'`
#	return 0
    elif [ -e /etc/SuSE-release ]; then
	release_name='suse'
	release_version=`cat /etc/SuSE-release | grep -i linux | sed -e 's/[^0-9]*\([0-9]\+\).\([0-9]\+\).\([0-9]*\).*/\1\2\3/'`
	return 0
    elif [ -e /etc/turbolinux-release ]; then
	release_name='turbo'
	release_version=`cat /etc/turbolinux-release | sed -e 's/[^0-9]*\([0-9]\+\).\([0-9]\+\).\([0-9]*\)[^0-9]*/\1\2\3/'`
	return 0
    
    elif [ -e /etc/redhat-release ]; then
	release_name='redhat'
	release_version=`cat /etc/redhat-release | sed -e 's/[^0-9]*\([0-9]\+\).\([0-9]\+\).\([0-9]*\)[^0-9]*/\1\2\3/'`
	return 0
    fi
    if [ -e /etc/issue ]; then
	debug_echo "Looking at /etc/issue"
	if cat /etc/issue | grep -i 'red[ ]*hat'; then
	    release_name='redhat'
	    release_version=`cat /etc/issue | grep -i release | sed -e 's/[^0-9]*\([0-9]\+\).\([0-9]\+\).\([0-9]*\)[^0-9]*/\1\2\3/'`
	    return 0
	elif cat /etc/issue | grep -i 'caldera'; then
	    release_name='caldera'
	    release_version=`cat /etc/issue | grep -i Version | sed -e 's/[^0-9]*\([0-9]\+\).\([0-9]\+\).\([0-9]*\)[^0-9]*/\1\2\3/'`
	    return 0
	elif cat /etc/issue | grep -i 'corel'; then
	    release_name='corel'
	    release_version=`cat /etc/issue | grep -i Corel | sed -e 's/[^0-9]*\([0-9]\+\).\([0-9]\+\).\([0-9]*\)[^0-9]*/\1\2\3/'`
	    return 0
	elif cat /etc/issue | grep -i 'debian'; then
	    release_name='debian'
	    release_version=`cat /etc/issue | grep -i release | sed -e 's/[^0-9]*\([0-9]\+\).\([0-9]\+\).\([0-9]*\)[^0-9]*/\1\2\3/'`
	    return 0
	elif cat /etc/issue | grep -i 'suse'; then
	    release_name='suse'
	    release_version=`cat /etc/issue | grep -i release | sed -e 's/[^0-9]*\([0-9]\+\).\([0-9]\+\).\([0-9]*\)[^0-9]*/\1\2\3/'`
	    return 0
	elif cat /etc/issue | grep -i 'turbo'; then
	    release_name='turbo'
	    release_version=`cat /etc/issue | grep -i release | sed -e 's/[^0-9]*\([0-9]\+\).\([0-9]\+\).\([0-9]*\)[^0-9]*/\1\2\3/'`
	    return 0
	elif cat /etc/issue | grep -i 'mandrake'; then
	    release_name='mandrake'
	    release_version=`cat /etc/issue | grep -i release | sed -e 's/[^0-9]*\([0-9]\+\).\([0-9]\+\).\([0-9]*\)[^0-9]*/\1\2\3/'`
	    return 0
	fi
    fi
    return 1
}


debug_echo "Running Args = $@"

case "$1" in
	start)
	    # load binary for systems where it's been built
	    stderr_echo "Loading Zero-Knowledge FREEDOM(tm) kernel module..."
	    if [ -f "$freedom_home/$source_driver_name.o" ]; then
		if /sbin/insmod "$freedom_home/$source_driver_name.o" > /dev/null 2>&1; then
		    debug_echo "Driver successfully loaded."
		    exit 0
		fi
		exit 1
	    fi
	    # load binary driver for sane sstems with stock kernels
	    if get_distro; then	   
		debug_echo looking for "clientshim_$release_name"-"$release_version"_"$kernel_version".o
		    if [ -f "$freedom_home/$kernel_bin/""clientshim_$release_name"-"$release_version"_"$kernel_version".o ]; then
			if /sbin/insmod "$freedom_home/$kernel_bin/""clientshim_$release_name"-"$release_version"_"$kernel_version".o > /dev/null 2>&1 ; then
			    debug_echo "Driver successfully loaded."
			    exit 0
			fi
		    fi
	    fi
	    debug_echo "Couldn't determine system release, or our binary wouldn't load"

	    # Ok, if the running kernel is mod_versioned we can get kinda wacky
	    if cat /proc/ksyms | grep  _R[0-9a-z]* > /dev/null 2>&1; then
		# try to load anything that matches the kernel version
		debug_echo "Will try more modules."
		for i in `ls $freedom_home/$kernel_bin/clientshim_*_$kernel_version.o`; do
		    debug_echo "Trying to load $i"
		    if /sbin/insmod $i > /dev/null 2>&1 ; then
			debug_echo "Driver successfully loaded"
			exit 0
		    fi
		done
	    fi
            stderr_echo "Unable to load FREEDOM(tm) kernel module. Please see /usr/share/freedom/doc/INSTALL.DRIVER for asistance." 
	    exit 1
	    ;;
	stop)
	    stderr_echo "Unloading Zero-Knowledge FREEDOM(tm) kernel module..."
	    if [ -f "$freedom_home/$source_driver_name.o" ]; then
		if /sbin/rmmod "$source_driver_name"  > /dev/null 2>&1; then
		    debug_echo "Kernel module $source_driver_name unloaded"
		    exit 0
		fi
		debug_echo "Hand built driver unload failed. Trying binaries"
	    fi

	    for i in `/sbin/lsmod | grep clientshim | cut -f 1 -d ' '`; do
		debug_echo "Unloading kernel module $i..."
		/sbin/rmmod $i > /dev/null 2>&1
		debug_echo "Kernel module $i unloaded"
	    done
	    exit 0
	    ;;
	restart)
	    "$0" stop && "$0" start
	    ;;

	*)
	    echo Usage: "`basename "$0"` {start|stop|restart}"
	    exit 0
	    ;;
esac
exit 1

