/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

 */


#define ZKSHIM_FRAG_INTERVAL	30		/* 30 seconds */
#define ZKSHIM_FRAG_TIMEOUT		120		/* 2 minutes */

typedef struct ZkShimFragStateKey {
	uint16_t zfk_ip_id;
	uint8_t zfk_ip_p;
	uint32_t zfk_ip_src;
	uint32_t zfk_ip_dst;
} ZkShimFragStateKey;

#define ZK_FRAGKEYS_EQUAL(k1, k2) (((k1)->zfk_ip_id == (k2)->zfk_ip_id) && \
							((k1)->zfk_ip_p == (k2)->zfk_ip_p) && \
							((k1)->zfk_ip_src == (k2)->zfk_ip_src) && \
							((k1)->zfk_ip_dst == (k2)->zfk_ip_dst))

typedef struct ZkShimFragState {
	struct list_head zfs_list;
	ZkShimFragStateKey zfs_fragkey;
	uint16_t zfs_sport;
	uint16_t zfs_dport;
	struct list_head zfs_packetlist;
	unsigned long zfs_time;
} ZkShimFragState;

typedef struct ZkShimFragPacket {
	struct list_head zpe_list;
	ZKMB_T *zpe_mb;
	iphdr_t zpe_ip;
	int zpe_linkhdrlen;
	void (*zpe_divertproc) (ZkShimUnit *zun, ZKMB_T *mp, int lhl);
	void (*zpe_ifoutputproc) (void *, void *);
	void (*zpe_ifdiscardproc) (void *, void *);
	void *zpe_ifunitdesc, *zpe_ifpacketdesc;
	unsigned long zpe_time;
} ZkShimFragPacket;

void
zkShimFragInit(void);

void
zkShimFragCleanup(void);

boolean_t
zkShimProcessOutgoing(ZKMB_T *mp, iphdr_t *ip,
		uint16_t sport, uint16_t dport,
		void (*pdivertproc) (ZkShimUnit *, ZKMB_T *, int),
		void (*pifoutputproc) (void *, void *),
		void (*pifdiscardproc) (void *, void *),
		void *ifunitdesc,
		int linkhdrlen);

/*
boolean_t
zkShimProcessOutgoing(ZKMB_T *mp, iphdr_t *ip, uint16_t sport, uint16_t dport,
		void (*pdivertproc) (ZkShimUnit *, ZKMB_T *, int),
		void (*pifoutputproc) (void *, void *),
		void (*pifdiscardproc) (void *, void *),
		void *ifunitdesc,
		int linkhdrlen);
*/
