/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

 */

#ifndef __ZKSHIM_OS_INCLUDED
#define __ZKSHIM_OS_INCLUDED

#ifndef ZKSHIM_MAIN
#define __NO_VERSION__
#endif

#include <linux/config.h>

#if CONFIG_MODVERSIONS == 1 & !defined(MODVERSIONS)
#define MODVERSIONS
#include <linux/modversions.h>
#endif

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/file.h>
#include <linux/malloc.h>
#include <linux/miscdevice.h>
#include <linux/poll.h>
#include <linux/skbuff.h>
#include <linux/firewall.h>
#include <linux/in.h>
#include <linux/ip.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <net/protocol.h>
#include <linux/miscdevice.h>

#if (LINUX_VERSION_CODE == KERNEL_VERSION(2,2,2)) && defined(CONFIG_FILTER)
#  warning "undefining CONFIG_FILTER due to linux-2.2.2 header file bugs"
#  undef CONFIG_FILTER
#endif



#include <net/tcp.h>
#include <net/udp.h>
#include <asm/spinlock.h>


#include <stdarg.h>

#if defined(CONFIG_PROC_FS) && !defined(PROC_NET_ZKSHIM)
#define PROC_NET_ZKSHIM (PROC_NET_LAST + ZKSHIM_IOCTL_ID)
#define PROC_NET_ZKSHIM_TCP (PROC_NET_LAST + ZKSHIM_IOCTL_ID + 1)
#define PROC_NET_ZKSHIM_DGRAM (PROC_NET_LAST + ZKSHIM_IOCTL_ID + 2)
#endif

#ifdef MODULE
/* tasklist_lock is not exported by kernel, won't get resolved when compiling with CONFIG_SMP on */
#if defined(ZK_CALDERA) & ZK_CALDERA ==24 
extern rwlock_t tasklist_lock; 
#endif
/* because the kernel doesn't export the init_task symbol 
   (required by for_each_task), we have to determine it at run-time */ 
extern struct task_struct *zkInitTaskPtr;
#define ZK_FOR_EACH_TASK(p) \
        for (p = zkInitTaskPtr ; (p = p->next_task) != zkInitTaskPtr ; )

/* because the kernel doesn't export the pidhash symbol 
   (required by find_task_by_pid), we have to do an exhaustive search */
extern __inline__ struct task_struct *ZK_FIND_TASK_BY_PID(int pid)
{
        struct task_struct *p;

	ZK_FOR_EACH_TASK(p) {
		if(p->pid == pid)
			return p;
	}

	return NULL;
}
#else
#define ZK_FOR_EACH_TASK(p) for_each_task(p)
#define ZK_FIND_TASK_BY_PID(pid) find_task_by_pid(pid)
#endif /* MODULE */

#define CE_CONT	KERN_DEBUG
#define CE_NOTE KERN_NOTICE
#define CE_WARN KERN_WARNING
#define CE_PANIC KERN_EMERG

#define cmn_err(lev, fmt, arg...) printk(lev fmt,##arg)

#define INADDRLFMT		"l"

#define KRWLOCK_T rwlock_t
#define KRWLOCK_DECL(rwl) KRWLOCK_T rwl = RW_LOCK_UNLOCKED

#define INIT_LOCK(plck, str)    memset(plck, '\0', sizeof(*(plck)))
#define DESTROY_LOCK(plck)      

#define READ_LOCK(x)		read_lock(x)
#define WRITE_LOCK(x,f)		write_lock_irqsave(x,f)
#define READ_UNLOCK(x)		read_unlock(x)
#define WRITE_UNLOCK(x,f)	write_unlock_irqrestore(x,f)

#define ZKMB_T		struct sk_buff

#define ZKTIMER_T	struct timer_list
#define ZKTIMERFUNCARG_T	unsigned long
#define ZK_TIME_CURRENT()	(jiffies/HZ)
#define ZK_SET_TIMER(t, o, f, a) { (t)->next = (t)->prev = NULL; (t)->expires = jiffies+(o)*HZ; (t)->data = a; (t)->function = f; add_timer(t); }
#define ZK_CANCEL_TIMER(t)	del_timer(t)

#define bcmp(a,b,c)	memcmp(a,b,c)
#define bcopy(a,b,c)	memcpy(b,a,c)
#define bzero(a,c)	memset(a,0,c)

#define KMALLOC(s)	kmalloc(s, GFP_ATOMIC)
#define KMFREE(p)	kfree(p)

typedef enum { B_FALSE, B_TRUE } boolean_t;

typedef struct sk_buff mblk_t;

typedef struct iphdr iphdr_t;
#define ip_v version
#define ip_hl ihl
#define ip_tos tos
#define ip_len tot_len
#define ip_id id
#define ip_off frag_off
#define ip_ttl ttl
#define ip_p protocol
#define ip_sum check
#define ip_saddr saddr
#define ip_daddr daddr

ZKMB_T *
zkShimCloneMb(ZKMB_T *skb);


#ifdef ZKSHIM_DEBUG
extern int zkshim_debug;

#define ZKDBG(l, fmt, args...)  if(zkshim_debug >= l) printk( KERN_ERR __FILE__ ":" __FUNCTION__ " " fmt, ##args)

#else
#define ZKDBG(x...)

#endif

#endif /* __ZKSHIM_OS_INCLUDED */
