/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

 */

#ifndef __ZKSHIM_PRIV_INCLUDED
#define __ZKSHIM_PRIV_INCLUDED

/*******************************************************************************
; zkshim driver
; Written by Marc Boucher <marc@mbsi.ca>
*******************************************************************************/

#if defined(DEBUG) && !defined(ZKSHIM_DEBUG)
#define ZKSHIM_DEBUG
#endif


#include <zkshim_os.h>
#include <zkshim.h>

#define ZKSHIM_MODINFO	"ZKS SHIM Driver v"ZKSHIM_VERSION" pl"ZKSHIM_PATCHLEV




typedef struct ZkShimUnit {
	struct ZkShimUnit *zun_nextp;
	int zun_minordev;
#ifdef ZK_SOLARIS
	queue_t *zun_rdqueue;
	cred_t zun_cred;
#endif /* ZK_SOLARIS */
#ifdef ZK_LINUX
	struct file *zun_file;
	struct wait_queue *zun_waitq;
	struct fasync_struct *zun_fasync;
	struct sk_buff_head zun_readq;

#endif /* ZK_LINUX */
	struct ZkShimRuleEntry *zun_rules;
	struct ZkShimSecret *zun_secret;
	
} ZkShimUnit;

//#include "zkshim_util.h"
#include "zkshim_rules.h"
#include "zkshim_frag.h"
#include "zkshim_dgram_cache.h"
#include "zkshim_tcp_cache.h"

extern KRWLOCK_T zkShimLock; /* Lock protecting all global data */
extern ZkShimUnit	*zkShimUnitList; /* List of open units */

#define ZK_IS_UDPTCP(p)	((p == IPPROTO_TCP) || (p == IPPROTO_UDP))

#endif /* __ZKSHIM_PRIV_INCLUDED */
