/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <poll.h>
#ifdef ZK_SOLARIS
#include <stropts.h>
#endif
#include <ctype.h>
#include <errno.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/udp.h>

#include <arpa/inet.h>
#include <netdb.h>

#include <../../../libzkpacket/include/zkPacket.h>
#include "zkshim.h"

#define UDP_PORT 3981

#ifndef INFTIM
#define INFTIM -1
#endif


#define TYPE_OF_SERVICE          0
#define PROTOCOL                 1
#define SOURCE_ADDRESS           2
#define SOURCE_ADDRESS_MASK      3
#define DESTINATION_ADDRESS      4
#define DESTINATION_ADDRESS_MASK 5
#define SOURCE_PORT              6
#define DESTINATION_PORT         7
#define USER_ID                  8
#define PROCESS_ID               9
#define SESSION_ID              10
#define CLEARDEF                11
#define ADDRULE                 12
#define DELRULE                 13
#define SENDSOCK                14
#define SENDSHIM                15
#define HOST                    16
#define PORT                    17
#define BINDSOCK                18
#define CONNECTSOCK             19

#ifdef CLIENT_SHIM
#  define RULE_TYPE		20
#  define RULE_ACTION		21
#  define ACTION_ADDRESS	22
#  define ACTION_PORT		23
#  define CLEARRULES		24
#  define NKEYWORD		25
#else
#  define NKEYWORD		20
#endif

#define TYPE_OF_SERVICE_STRING          "TYPE_OF_SERVICE"
#define PROTOCOL_STRING                 "PROTOCOL"
#define SOURCE_ADDRESS_STRING           "SOURCE_ADDRESS"
#define SOURCE_ADDRESS_MASK_STRING      "SOURCE_ADDRESS_MASK"
#define DESTINATION_ADDRESS_STRING      "DESTINATION_ADDRESS"
#define DESTINATION_ADDRESS_MASK_STRING "DESTINATION_ADDRESS_MASK"
#define SOURCE_PORT_STRING              "SOURCE_PORT"
#define DESTINATION_PORT_STRING         "DESTINATION_PORT"
#define USER_ID_STRING                  "USER_ID"
#define PROCESS_ID_STRING               "PROCESS_ID"
#define SESSION_ID_STRING               "SESSION_ID"
#define CLEARDEF_STRING                 "CLEARDEF"
#define ADDRULE_STRING                  "ADDRULE"
#define DELRULE_STRING                  "DELRULE"
#define SENDSOCK_STRING                 "SENDSOCK"
#define SENDSHIM_STRING                 "SENDSHIM"
#define HOST_STRING                     "HOST"
#define PORT_STRING                     "PORT"
#define BINDSOCK_STRING                 "BINDSOCK"
#define CONNECTSOCK_STRING              "CONNECTSOCK"

#ifdef CLIENT_SHIM
#  define RULE_TYPE_STRING		"RULE_TYPE"
#  define RULE_ACTION_STRING		"RULE_ACTION"
#  define ACTION_ADDRESS_STRING		"ACTION_ADDRESS"
#  define ACTION_PORT_STRING		"ACTION_PORT"
#  define CLEARRULES_STRING		"CLEARRULES"
#endif

char keyword_array[NKEYWORD][100] = { {TYPE_OF_SERVICE_STRING},
                                      {PROTOCOL_STRING},
			              {SOURCE_ADDRESS_STRING},
                                      {SOURCE_ADDRESS_MASK_STRING},
                                      {DESTINATION_ADDRESS_STRING},
                                      {DESTINATION_ADDRESS_MASK_STRING},
                                      {SOURCE_PORT_STRING},
                                      {DESTINATION_PORT_STRING},
                                      {USER_ID_STRING},
                                      {PROCESS_ID_STRING},
                                      {SESSION_ID_STRING}, 
                                      {CLEARDEF_STRING}, 
                                      {ADDRULE_STRING}, 
                                      {DELRULE_STRING}, 
                                      {SENDSOCK_STRING}, 
                                      {SENDSHIM_STRING}, 
                                      {HOST_STRING}, 
                                      {PORT_STRING}, 
                                      {BINDSOCK_STRING}, 
                                      {CONNECTSOCK_STRING}
#ifdef CLIENT_SHIM
					,
                                      {RULE_TYPE_STRING},
                                      {RULE_ACTION_STRING},
                                      {ACTION_ADDRESS_STRING},
                                      {ACTION_PORT_STRING},
				      {CLEARRULES_STRING}
#endif				      
                                    };


int
bindsock(char *dhost, u_short port)
{
	int s;
	struct sockaddr_in lsin;
	struct hostent *entry;

	memset(&lsin, '\0', sizeof(lsin));

	lsin.sin_family = AF_INET;

	lsin.sin_port = htons(port);
	s = socket(AF_INET, SOCK_DGRAM, 0);
	if(s < 0) {
		fprintf(stderr, "Cannot create UDP socket: %s\n", strerror(errno));
		return -1;
	}

	if(bind(s, (struct sockaddr *)&lsin, sizeof(lsin)) < 0) {
		fprintf(stderr, "Cannot bind UDP socket to local port %d: %s\n", port, strerror(errno));
		return -1;
	}

	return s;
}

int
connectsock(char *dhost, u_short port)
{
	int s;
	struct sockaddr_in lsin;
	struct sockaddr_in sin;
	struct hostent *entry;

	memset(&lsin, '\0', sizeof(lsin));

	lsin.sin_family = AF_INET;

	memcpy(&sin, &lsin, sizeof(sin));

	lsin.sin_port = htons(INADDR_ANY);
	sin.sin_port = htons(port);
	s = socket(AF_INET, SOCK_DGRAM, 0);
	if(s < 0) {
		fprintf(stderr, "Cannot create UDP socket: %s\n", strerror(errno));
		return -1;
	}

	if(bind(s, (struct sockaddr *)&lsin, sizeof(lsin)) < 0) {
		fprintf(stderr, "Cannot bind UDP socket to local port %d: %s\n", port, strerror(errno));
		return -1;
	}

	if(dhost && *dhost) {

		sin.sin_addr.s_addr = inet_addr(dhost);
		if (sin.sin_addr.s_addr == -1) {
			entry = gethostbyname(dhost);
			if (entry && entry->h_addr && (entry->h_addrtype == AF_INET)) {
 				memcpy(&sin.sin_addr.s_addr, entry->h_addr, entry->h_length);
			} else {
				fprintf(stderr, "Unknown hostname or address - \"%s\"\n", dhost);
				return -1;
			}
		}
	
		if(connect(s, &sin, sizeof(sin)) < 0) {
			fprintf(stderr, "Cannot connect UDP socket to destination host/port: %s\n", strerror(errno));
			return -1;
		}
	}

	return s;
}

#ifdef ZK_SOLARIS
int
strioctl(int fd, int cmd, void *ptr, int len)
{
    struct strioctl sioc;
	int rc;

    sioc.ic_cmd = cmd;
    sioc.ic_timout = _INFTIM;
    sioc.ic_len = len;
    sioc.ic_dp = ptr;
	rc = ioctl(fd, I_STR, &sioc);
    if (rc < 0)
		return rc;
	else
	    return sioc.ic_len;
}
#else
#define strioctl(fd, cmd, ptr, len) ioctl(fd, cmd, ptr)
#endif

int
setup_shim(void)
{
	int shim_fd;

	shim_fd = open(ZKSHIM_DEVNAME, O_RDWR);
	if(shim_fd < 0) {
		perror(ZKSHIM_DEVNAME);
		return -1;
	}

#ifdef ZK_SOLARIS
	/* read mode, message non-discard mode */
	if(ioctl(shim_fd, I_SRDOPT, RMSGN) < 0) {
		perror("ioctl I_SRDOPT");
		close(shim_fd);
		return -1;
	}
#endif
        return shim_fd;
}

void add_rule(int shim_fd, ZkShimRule rule1)
{
	/* add a rule */
	if(strioctl(shim_fd, ZKIOC_ADD_RULE, &rule1, sizeof(rule1)) < 0) {
		perror("ioctl ZKIOC_ADD_RULE");
	}
        else {
          printf(" rule added successfully \n");
	}
}

int del_rule(int shim_fd, ZkShimRule rule1)
{
	/* test: delete rule */
	if(strioctl(shim_fd, ZKIOC_DEL_RULE, &rule1, sizeof(rule1)) < 0) {
		perror("ioctl ZKIOC_DEL_RULE");
	}
        else {
          printf(" rule deleted successfully \n");
	}
}

#ifdef CLIENT_SHIM
int clear_rules(int shim_fd)
{
	/* test: delete rule */
	if(strioctl(shim_fd, ZKIOC_CLEAR_RULES, NULL, 0) < 0) {
		perror("ioctl ZKIOC_CLEAR_RULES");
	}
        else {
          printf(" rules deleted successfully \n");
	}
}
#endif

/* Convert byte to two ascii-hex characters */
static void
ctohex(register char *buf, register short c)
{
	static char     hex[] = "0123456789abcdef";

	*buf++ = hex[(c & 0xf0) >> 4];
	*buf = hex[c & 0x0f];
}

/*
 * Print a buffer up to 16 bytes long in formatted hex with ascii
 * translation, e.g., 0000: 30 31 32 33 34 35 36 37 38 39 3a 3b 3c 3d 3e 3f
 * 0123456789:;<=>?
 */
static void
fmtline(short addr, const u_char * buf, short len)
{
	char            line[80];
	register char  *aptr, *cptr;
	register u_char c;

	memset(line, ' ', sizeof(line));
	ctohex(line, (short) ((addr & 0xff00) >> 8));
	ctohex(line + 2, (short) (addr & 0xff));
	aptr = &line[6];
	cptr = &line[55];
	while (len-- != 0) {
		c = *buf++;
		ctohex(aptr, (short) (c));
		aptr += 3;
		c &= 0x7f;
		*cptr++ = isprint((c)) ? c : '.';
	}
	*cptr++ = '\n';
	fwrite(line, 1, (unsigned) (cptr - line), stdout);
}

void
hex_print(register const u_char * bp, register u_int length)
{
	unsigned short  address = 0;
	u_int more;

	if(length > 256) {
		more = length - 256;
		length = 256;
	} else
		more = 0;

	while (length > 0) {
		int             n;

		n = (length > 16) ? 16 : length;
		fmtline(address, bp, n);
		length -= n;
		address += n;
		bp += n;
	}
	if(more)
		printf("[%d more bytes]\n", more);
	printf("\n");
}

void fixbuf( char* buf, int isz, ZkShimRule rule1)
{ 
        struct ip *ip;

        static int npkt = 0;

        int p;
        int ibsz;

	struct udphdr *udp;

	ip = (struct ip *)buf;
	memset(ip, '\0', sizeof(struct ip));
			
	ip->ip_hl = sizeof(struct ip) >> 2;
	ip->ip_v = IPVERSION;
	ip->ip_len = htons(sizeof(struct ip) + sizeof(struct udphdr) + sizeof(int));
	ip->ip_ttl = MAXTTL;
	ip->ip_ttl = 0x40;
	ip->ip_p   = ZK_IPPROTO_UDP;
	ip->ip_src.s_addr = rule1.zru_ip_src;
	ip->ip_dst.s_addr = rule1.zru_ip_dst;
	
	udp = (struct udphdr *)(buf+sizeof(struct ip));
	memset(udp, '\0', sizeof(struct udphdr));
			
        udp->source = htons(1024);
        udp->dest = rule1.zru_dport;
        udp->len = htons(sizeof( struct udphdr) + sizeof(int));

	*((int *)(udp + 1)) = htonl(npkt);
        npkt++;

	//zkIPHeaderDump((ZkIPHeader *) buf, stderr);
	//zkUDPHeaderDump((ZkUDPHeader *) buf, stderr);
        //fprintf(stderr, "-------------------------------\n");

        ibsz = isz;
          /* 
           * At this point we have a full IP packet.
           * Compute the checksum for the IP header
           */
        zkIPHeaderChecksum((ZkIPHeader *) buf);

        switch (((ZkIPHeader *) buf)->Protocol) {
          case ZK_IPPROTO_UDP:
            /* Compute the UDP header checksum */
            zkUDPHeaderChecksum((ZkUDPHeader *) buf, ibsz);
            
            break;

          case ZK_IPPROTO_TCP:
            /* Compute the TCP header checksum */
            zkTCPHeaderChecksum((ZkTCPHeader *) buf);

            break;

          default:
            break;
          }

	  //zkIPHeaderDump((ZkIPHeader *) buf, stderr);
	  //zkUDPHeaderDump((ZkUDPHeader *) buf, stderr);
	  //fprintf(stderr, "-------------------------------\n");

}

void sendsocket( int s, ZkShimRule rule1)
{ 
	unsigned char buf[10*1024];
        int p;

        int sz = sizeof(struct ip) + sizeof( struct udphdr) + sizeof(int);

        fixbuf( buf, sz, rule1);

        printf(" sending on socket bytes %d\n",sizeof(struct ip) + sizeof( struct udphdr) + sizeof(int));
	//hex_print(buf, sizeof(struct ip) + sizeof( struct udphdr) + sizeof(int));
	p = send(s, (const char *)buf, sz, 0);
	if(p < 0) {
	   perror("send");
	   close(s);
	}

}

void sendshim( int shim_fd, ZkShimRule rule1)
{
   unsigned char buf[10*1024];
   int n;
   static int npkt = 0;

   int sz = sizeof(struct ip) + sizeof( struct udphdr) + sizeof(int);

   fixbuf( buf, sz, rule1);

   printf(" sending %d bytes to shim \n", sz);

   n = write(shim_fd, buf, sz);
   if(n < 0) {
     perror("write");
   }
   npkt++;
}
          
void parse_line( char* line, int* key, int* val, char* string)
{
   char* keyt;
   char* pe;
   int lk;
   int i;

   for (i = 0; i < NKEYWORD; i++)
   {
     keyt = keyword_array[i];
     lk = strlen(keyt);
     if (strncmp( line, keyt, lk) == 0 && (line[lk] == '\0' || line[lk] == '=')) {
       //printf(" match line %s key %s \n",line,keyt);
       *key = i;
       *val = -1;
       strcpy(string,"");
       pe = strstr(line,"=");
       if (pe) {
         pe++;
         if (i == PROTOCOL) {
           if (strcmp(pe,"UDP") == 0) *val = IPPROTO_UDP;
           if (strcmp(pe,"TCP") == 0) *val = IPPROTO_TCP;
	 }
#ifdef CLIENT_SHIM
	 if (i == RULE_TYPE) {
           if (strcmp(pe,"SOCKET") == 0) *val = ZRU_SOCKET;
           if (strcmp(pe,"FILTER") == 0) *val = ZRU_FILTER;
	 }
	 if (i == RULE_ACTION) {
           if (strcmp(pe,"ALLOW") == 0) *val = ZSA_ALLOW;
           if (strcmp(pe,"DENY") == 0) *val = ZSA_DENY;
           if (strcmp(pe,"REDIRECT") == 0) *val = ZSA_REDIRECT;
           if (strcmp(pe,"REDIRECT_TXIP") == 0) *val = ZSA_REDIRECT_TXIP;
	 }
#endif	 
         if (i == SOURCE_ADDRESS      || i == SOURCE_ADDRESS_MASK     ||
             i == DESTINATION_ADDRESS || i == DESTINATION_ADDRESS_MASK
#ifdef CLIENT_SHIM
		|| i == ACTION_ADDRESS
#endif				
	     ) {
           *val = inet_addr(pe);
         }
         if (i == SOURCE_PORT || i == DESTINATION_PORT || i == USER_ID ||
             i == PROCESS_ID  || i == SESSION_ID       || i == PORT
#ifdef CLIENT_SHIM
		|| i == ACTION_PORT
#endif		
	     ) {
           *val = atoi(pe);
         }
         if (i == HOST) {
           strcpy( string, pe);
         }
         //printf(" val %x string %s \n",*val,string);
       } 
     }
   }
}
  
void parse_file(char* filename, int shim_fd, int *s, ZkShimRule *rule)
{
  FILE* f;
  char ch;
  char line[256];
  char string[256];
  char host[256];
  int  port;

  int   key;
  int   val;
  int   p;

  ZkShimRule rule1;

  int c;

  memset(&rule1, '\0', sizeof(rule1));

  p =0;

  f = fopen(filename,"r");
  if (f) {
    while (!feof(f)) {
      if (fread( &ch, 1, 1, f)) {
        if (ch == '\n' && p > 0) {
          line[p] = 0;          
          parse_line(line, &key, &val, string);
          if (key == TYPE_OF_SERVICE) {
	    rule1.zru_ip_tos = val;
          }
          else if (key == PROTOCOL) {
	    rule1.zru_ip_p = val;
          }
          else if (key == SOURCE_ADDRESS) {
            rule1.zru_ip_src = val;
          }
          else if (key == SOURCE_ADDRESS_MASK) {
            rule1.zru_ip_src_mask = val;
          }
          else if (key == DESTINATION_ADDRESS) {
            rule1.zru_ip_dst = val;
          }
          else if (key == DESTINATION_ADDRESS_MASK) {
            rule1.zru_ip_dst_mask = val;
          }
          else if (key == SOURCE_PORT) {
            rule1.zru_sport =  htons(val);
          }
          else if (key == DESTINATION_PORT) {
            rule1.zru_dport = htons(val);
          }
          else if (key == USER_ID) {
            rule1.zru_uid = val;
          }
          else if (key == PROCESS_ID) {
            if (val==-1) val = getpid();
            rule1.zru_pid = val;

          }
          else if (key == SESSION_ID) {
            rule1.zru_sid = val;
          }
          else if (key == CLEARDEF) {
            memset(&rule1, '\0', sizeof(rule1));
          }
          else if (key == ADDRULE) {
            add_rule(shim_fd, rule1);
          }
          else if (key == DELRULE) {
            del_rule(shim_fd, rule1);
          }
          else if (key == SENDSOCK) {
            sendsocket( *s, rule1);
	  }
          else if (key == SENDSHIM) {
            sendshim( shim_fd, rule1);
          }
          else if (key == HOST) {
            strcpy( host, string);
          }
          else if (key == PORT) {
            port = val;
          }
          else if (key == BINDSOCK) {
            *s = bindsock( host, port);
          }
          else if (key == CONNECTSOCK) {
            *s = connectsock( host, port);
          }
#ifdef CLIENT_SHIM
          else if (key == RULE_TYPE) {
            rule1.zru_type = val;
	    printf("rule type:%d\n", val);
          }
          else if (key == RULE_ACTION) {
            rule1.zru_action = val;
	    printf("rule action:%d\n", val);
          }
          else if (key == ACTION_ADDRESS) {
            rule1.zru_ip_arg = val;
          }
          else if (key == ACTION_PORT) {
            rule1.zru_port_arg = htons(val);
          } else if (key == CLEARRULES) {
	    clear_rules(shim_fd);
	  }
#endif
          p = 0;
        }
        else if (!(ch == '\n')){
          line[p] = ch;
          p++;
        }
      }
    }
    fclose(f);
  }
  memcpy( rule, &rule1, sizeof(rule1));

};

int
main(int argc, char **argv)
{
	int exitcode=0, s, shim_fd, p, i;
	int al;
	ssize_t n;
	struct sockaddr_in rsin;
	struct pollfd pfds[2];
	unsigned char buf[10*1024];
	char *commandfile = NULL;
        char default_commandfile[] = "test_zkshim.in";
        ZkShimRule rule;
        int c;
        char ch;
        char* pbuf;

        if((argc < 1) || (argc > 2)) {
		fprintf(stderr, "Usage: %s [commandfile]\n", argv[0]);
		return 1;

	}

	if(argc >= 2)
	  commandfile = argv[1];
        else
          commandfile = default_commandfile;

        printf(" pid %d uid %d sid %d \n",getpid(),getuid(),getsid(getpid()));

	shim_fd = setup_shim();
	if(shim_fd < 0) {
	   close(s);
	   return 3;
	}

        parse_file( commandfile, shim_fd, &s, &rule);

        pfds[0].fd = s;
        pfds[0].events = POLLIN;

        pfds[1].fd = shim_fd;
        pfds[1].events = POLLIN;

	while(1) {

		p = poll(pfds, 2, INFTIM);
		if(p < 0) {
			perror("poll");
			exitcode = 4;
			break;
		}

		if ((pfds[0].revents&POLLIN) != 0) { /* data from socket available */
			al = sizeof(rsin);
			n = recvfrom(s, (char *)buf, sizeof(buf), 0, (struct sockaddr *)&rsin, &al);
			if(n < 0) {
				perror("recvfrom");
				exitcode = 5;
				break;
			}
			fprintf(stderr, "received %d bytes from %s on UDP socket\n", n,
				inet_ntoa(rsin.sin_addr));
			hex_print(buf, n);
                        
                        /*fixbuf( buf, n, rule);

			n = write(shim_fd, buf, n);
			if(n < 0) {
				perror("write");
				exitcode = 6;
				break;
				}*/
		}

		if ((pfds[1].revents&POLLIN) != 0) { /* data from shim available */
			n = read(shim_fd, buf, sizeof(buf));
			if(n < 0) {
				perror("read");
				exitcode = 6;
				break;
			}
			fprintf(stderr, "received %d bytes from shim\n", n);
                        pbuf = buf;
			hex_print(pbuf, n);
		}
	}

	close(shim_fd);
	close(s);

	fprintf(stderr, "%s: Terminating\n", argv[0]);

	return(exitcode);
}
 
    
