// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

/*


Automatic testing of clientshim main entries
Basic tests.
  
*/
// TODO: review of coding standards stuff

#include "ClientshimUnitTest.h" 

#include <stdio.h>
#include <iostream>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <fcntl.h>   /* for open()*/
#include <unistd.h>  /* for close() and write() */
#include <error.h>   /* for perror()*/
#include <netinet/ip.h> /* IPVERSION and MAXTTL*/
#include <netinet/in.h> /* htons(), ntohs(), ... */
#include <sys/ioctl.h>
#include <sys/wait.h>   /* wait and WIFEXITED */
#include <netdb.h>
#include <arpa/inet.h>
/* move the include paths to the make file */


#include <ZkTestBase.h>


/* define a flag in the makefile to switch from standalone/zktest module */
#define  LINK_TEST_MODULE 1


int GenerateUDPPacket(uint32_t SrcAddress,
	uint32_t DestAddress,
	uint16_t SrcPort,
	uint16_t DestPort,
	char *Buffer,
	uint32_t Size);


int GenerateUDPPacket(uint32_t SrcAddress,
	uint32_t DestAddress,
	uint16_t SrcPort,
	uint16_t DestPort,
	char *Buffer,        // CHECK
	uint32_t Size        // Length of the bufer
	){

	struct ZkIPHeader  *pipheader;
	struct ZkUDPHeader *pudpheader;
	//struct ZkNetFrame  *pframe;
	char *data;
	// CLEANUP
	int   SizeOfBuffer=sizeof(ZkIPHeader) 
	                    + sizeof(ZkUDPHeader)
	                    + sizeof(int)+16;

		
	if (!Buffer || !Size)
		return -1;
	if (Size < (sizeof(ZkIPHeader)+ sizeof(ZkUDPHeader)))
		return -1;

	//pframe = (ZkNetFrame *)Buffer;
	//tempBuffer=Buffer + sizeof (ZkDIXHeader);
	pipheader = (ZkIPHeader *)Buffer;
	memset(pipheader,'\0',sizeof(ZkIPHeader));
	pudpheader= (ZkUDPHeader *)Buffer;
	memset(pudpheader,'\0',sizeof(ZkUDPHeader));
	data= (char *)Buffer + sizeof(ZkUDPHeader);

	// fill up the IP header stuff
	pipheader->ihl=sizeof(ZkIPHeader) >> 2;
	pipheader->version=IPVERSION;
	pipheader->TimeToLive=MAXTTL;
	pipheader->Protocol=ZK_IPPROTO_UDP;
	//zkIPHeaderChecksum(pipheader);
	// no need to convert to network byte order inet_addr was used before
	pipheader->SrcIpAddress = SrcAddress;
	pipheader->DstIpAddress = DestAddress; 
	pipheader->TotalLength = htons( sizeof(ZkIPHeader)+ sizeof(ZkUDPHeader)+sizeof(int));
	// Fill up the UDP header
	pudpheader->IP = (* pipheader); 
	pudpheader->SrcPort= htons(SrcPort);
	pudpheader->DstPort= htons(DestPort);
	pudpheader->UdpLength=htons(sizeof  (ZkUDPHeader)+sizeof (int));
       	//UDP checksum
       	zkUDPHeaderChecksum(pudpheader, SizeOfBuffer);
      	zkIPHeaderDump(pipheader,stderr);
	zkUDPHeaderDump(pudpheader,stderr);


	return 0;
}	



Test::Test(void)
{
	Device = -1;
};

Test::~Test(){

};

int Test::OpenDevice(){


	
	printf("Open Device test    \n");
	Device = open(ZKSHIM_DEVNAME, O_RDWR);
	if(Device < 0) {
		perror(ZKSHIM_DEVNAME);
		return -1;
	}
	// success 
	return 0;
};

int  Test::CloseDevice(){

int ret=-1;
	printf("Close Device  test  \n");
	if ((ret = close(Device)) < 0)
		perror(ZKSHIM_DEVNAME);
	return ret;
};




TestUtil::TestUtil(void)
{
	// for the time being put these stuff here
	SrcAddress = inet_addr("127.0.0.1");
	DestAddress= inet_addr("127.0.0.1");
	SrcPort = 1024;
	DestPort = 1024;
	Type=UDP;
	
	//default rule
	rule.zru_ip_tos=0;
	rule.zru_ip_p=0;
	rule.zru_ip_src=inet_addr("0.0.0.0");
	rule.zru_ip_src_mask=inet_addr("0.0.0.0") ;
	rule.zru_ip_dst=inet_addr("0.0.0.0");
	rule.zru_ip_dst_mask=inet_addr("0.0.0.0");
	rule.zru_sport=ntohs(0);
	rule.zru_dport=ntohs(0);
	rule.zru_uid=0;
	rule.zru_pid=0;
	rule.zru_sid=0;
	rule.zru_type=ZRU_FILTER;
	rule.zru_action=ZSA_REDIRECT;
	rule.zru_ip_arg=0;
	rule.zru_port_arg=0;

};

TestUtil::~TestUtil(){

};

void TestUtil::SetRule(int protocol, 
		     char * srcaddr,char * srcmask, 
		     char * destaddr, char* destmask,
		     uint16_t srcport,
		     uint16_t destport,
		     int type,
		     int action,
		     char * ip_arg,
		     uint16_t port_arg){

	// CHECKS TO BE ADDED?
	rule.zru_ip_tos=0;
	// valid ranges UDP TCP */
	rule.zru_ip_p=protocol;
	rule.zru_ip_src=inet_addr(srcaddr);
	rule.zru_ip_src_mask=inet_addr(srcmask) ;
	rule.zru_ip_dst=inet_addr(destaddr);
	rule.zru_ip_dst_mask=inet_addr(destmask);
	rule.zru_sport=ntohs(srcport);
	rule.zru_dport=ntohs(destport);
	rule.zru_uid=0;
	rule.zru_pid=0;
	rule.zru_sid=0;
	rule.zru_type=type;
	rule.zru_action=action;
	rule.zru_ip_arg=inet_addr(ip_arg);
	rule.zru_port_arg=ntohs(port_arg);
	return;

}


// utility functions move them to another class
int TestUtil::MakePacket(char * pBuf,int * pSize){

	if (Type == UDP){
		if (!GenerateUDPPacket(SrcAddress,DestAddress,SrcPort,DestPort,pBuf,BUF_SIZE)){
			*pSize=BUF_SIZE;
			//printf("GenerateUDPPacketSucceed Address of buffer:08%X Size:%d  \n",pBuf,*pSize );
			return 0;
		}
		printf("GenerateUDPPacketfailed \n");	  

	}
	else
		printf("Don't know how to make the packet type \n");

	return -1;

};



int Test::WriteToDevice(char *Buffer ,size_t numofbytes){

	size_t count;
	char devname[]=ZKSHIM_DEVNAME; 

	printf("Entering Write to device \n");

	// Q: How to get the device name from the device number?
	// Do not check the input parameters so the test programs can abuse
	// the device just make sure that you are abusing the correct device

	count = write(Device,Buffer,numofbytes);
	printf("Wrote: %d \n ",count);

	if (count ==  numofbytes)
		return 0;
	if( count<=0 ){
		perror(devname);
		return -1;
	}
	/* is shim sensitive to the size ? */
	return -1;
};

int Test::ReadFromDevice(char * Buffer,size_t BufferSize){

int count = -1;
	printf("Entering read from device ... \n");
	//read and print intercepted packets
	count=read(Device,Buffer,BufferSize);
	printf("Number of bytes read: %d\n", count);
	return count;
	
};

int Test::SetDeviceOption(int option, ZkShimRule *rule){

	printf("EnteringSetDeviceOption  \n");
	if(ioctl(Device,option, rule, sizeof(rule)) < 0) {
	  perror("ioctl ");
	  return -1;
	}
	printf("SetDeviceOption successful \n");
	return 0;
	
};

void Test::GetDeviceOption(){

};

//This has to be modified later when the test agent is ready.
//ClientshimStartup g_clientshimStartup;

#if LINK_TEST_MODULE
ZKTEST_START( ClientshimStartup )
#else
int main()
#endif
{
char Buffer[BUFFER_SIZE];
size_t BufferSize=BUFFER_SIZE;
//int Size; 
int ret,i=-1;
int numofbytes=-1;
pid_t pid;
//int stat;
Test TestInstance;
TestUtil UtilInstance;
struct hostent *hostinfo;
bool child_terminated;
	
    memset(Buffer,'\0',BUFFER_SIZE);

	// open close functionality
	ret=TestInstance.OpenDevice();
	if (ret < 0)
		return -1;
	TestInstance.CloseDevice();



	// open  set rule read  write close 
	ret=TestInstance.OpenDevice();
	if (ret < 0)
		return -1;
   TST_TRACE("Try to add a rule ....\n ");
	ret=TestInstance.SetDeviceOption(ZKIOC_ADD_RULE,UtilInstance.GetRule());
	if (ret < 0){
		TST_TRACE("Set Device Option returned error. \n");
		TestInstance.CloseDevice();
		return -1;
	}

	// read function blocks until a packet is captured
	// so feed a packet between read and write
	TST_TRACE("will fork another process....\n");
	pid=fork();
	switch(pid){
	case -1:
	  TST_TRACE("ERROR fork failed \n");
	  //delete rule and close the device
	  break;
	case 0:
	  TST_TRACE("process will be blocked on reading from the devicee \n");
	  numofbytes=TestInstance.ReadFromDevice(Buffer,BufferSize);
	  // dump the buffer
	  zkIPHeaderDump( (ZkIPHeader *)Buffer,stderr);

	  TestInstance.WriteToDevice(Buffer,numofbytes);
	  TST_TRACE("Try to  delete a rule \n");
	  ret=TestInstance.SetDeviceOption(ZKIOC_DEL_RULE,UtilInstance.GetRule());
	  TestInstance.CloseDevice(); // if read is not successful?   
	  break;
	default:
	  TST_TRACE("parent process will send a packet .. \n");
	  // send traffic that will be  captured by the driver
	  // wait until the traffic is captured by the child process
	  //system("ping -c 5 localhost");
	  // Should send a UDP packet for the DNS request 
	  hostinfo=gethostbyname("www.zeroknowledge.com");
	  if(!hostinfo){
	    TST_TRACE("no host  \n");
	  }
	  child_terminated=false;
	  // wait a little bit and check the status of the child
	  for(i=3;i>0;i--)
	  {
		  
		  if ((ret=waitpid(-1, (int *)0 , WNOHANG))==0){
			  sleep(1);
			  continue;
		  }
		  else{
			  child_terminated=true;
			  break;
		  }
	  }
	  if(!child_terminated)
		  if(!kill(pid,SIGKILL)){
			  TST_TRACE("Waiting too long to capture a packet. Give up, test not successful");}
			  else
				  TST_TRACE("Problem! a Zombie process");
		  
#if 0
	if (UtilInstance.MakePacket(Buffer,&Size) < 0 )
	{	
		TST_TRACE("Make Packet returned error \n");
		return -1;
	}
	TST_TRACE("Size of buffer %d ,Buffer: %s \n",Size,Buffer);
	TestInstance.WriteToDevice(Buffer,Size);
#endif
	break;
	  
	}
	  
	/* returning -1 means that all the tests passed  */
	return -1;
	
}
#if LINK_TEST_MODULE
ZKTEST_END
#else 
;
#endif
	







