// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom(R) code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include "InTacLinuxStub.h"
#include <netdb.h>
#include <frAips.h>

#define TOKEN_THROUGH_CLOUD
#define NYM_THROUGH_CLOUD
#define DNS_THROUGH_CLOUD
#define KEYUPDT_THROUGH_CLOUD
#define NIQS_THROUGH_CLOUD
#define KEYQRY_THROUGH_CLOUD
#define FMGMAIP_THROUGH_CLOUD


InTacLinuxStub::InTacLinuxStub()
	:m_dnsport(53) // http://support.microsoft.com/support/kb/articles/Q158/4/74.asp (Default registry port value, what above api probably uses...)
	,m_dwFirewallMode(BlockAllPorts)
	,m_cntClearPackets(0)
	,m_cntCloudPackets(0)
	,m_cntBlockedPackets(0)
	,m_bLoaded(ZK_FALSE)
	,m_UnauthRoute(0)
{
}

bool InTacLinuxStub::Init(){

	// DNS
	const struct servent* pservice = getservbyname("domain",  "udp"); // A user could change the port value in the registry from the default of 53 to something else
	if(pservice) {
		m_dnsport = ntohs(pservice->s_port);
	}

	InitFirewallMode() ;

	return true;
}


void InTacLinuxStub::RouteIDisDead(const unsigned int nRoute) const{
	ZKTRACE1("InTacLinuxStub::RouteIDisDead() [%d] is dead\n", nRoute);
}


bool InTacLinuxStub::InTAC(
	const uint32_t in_DestIP, 
	const uint16_t in_DestPort, 
	unsigned int& out_nRouteID, 
	const int in_nType) 
{

	bool bRet = false;	// let packet out as per usual	

	out_nRouteID = BLOCK_PACKET;

	// Save very everybody down the line the trouble of converting the port into little endian
	const uint16_t DestPort = ntohs(in_DestPort);

	if(!m_bLoaded){
		ZKTRACE0("Freedom is loading, allowing normal traffic to leave, probably connecting to ISP and getting aip keys along with NIQS info\n");	
	}
	else {
		
		PORT_SETTING PacketAction = PacketFirewall(in_DestIP, DestPort, in_nType) ;
		
		if ( PacketAction == ANON ) {
			// Use the cloud, this requires a route
			RouteManager(in_DestIP, DestPort, out_nRouteID, in_nType);
			m_cntCloudPackets++;
			bRet = true;
		}
		else if ( PacketAction == BLOCK )  {
			m_cntBlockedPackets++;
			bRet = true;
		}
		else {

			ZKASSERT( PacketAction == OPEN ) ;

			// As long as it is not going to our Aip then it is a packet going out in the clear		
			if ( DestPort != FR_DFLT_AIP_UDP_PORT ) {
				m_cntClearPackets++;
			}			
		}
	
	}

	return bRet;
}

// Reset our Firewall settings from the registry
// We can't afford to look each time.
void InTacLinuxStub::InitFirewallMode()
{
	ZKASSERT( m_dwFirewallMode != UndefinedPortAction ) ;
}

PORT_SETTING InTacLinuxStub::GetActionForUnsupportedPortWhenNymSelected( uint16_t in_port ) const
{
	ZKASSERT( m_dwFirewallMode != UndefinedPortAction ) ;
	
	PORT_SETTING ReturnValue = BLOCK;

	if ( m_dwFirewallMode == OpenAllPorts ) {
		ReturnValue = OPEN ;
	}
	else if ( m_dwFirewallMode == OpenSelectPorts ) {
		ZKVERIFY(m_dwFirewallMode != OpenSelectPorts);
	}

	return ReturnValue ;
}


PORT_SETTING InTacLinuxStub::PacketFirewall(const unsigned long DestIP, const uint16_t in_DestPort, const int nType) const{

	ZKASSERT(nType == IPPROTO_TCP || nType == IPPROTO_UDP);	

	ZKTRACE1("in_DestPort [%d] \n", in_DestPort);

	PORT_SETTING Action = OPEN ; // Default action: let packet out as per usual


		// By Default we block if a nym is selected
//	Action = GetActionForUnsupportedPortWhenNymSelected( in_DestPort ) ;

	if (nType == IPPROTO_TCP) {
		switch( in_DestPort )
		{
			case( 22 ) :	// SSH
			case( 23 ) :	// Telnet
			case( 80 ) :	// HTTP
			case( 81 ) :
			case( 88 ) :
			case( 8000 ) :
			case( 8008 ) :
			case( 8080 ) :
			case( 8088 ) :
			case( 8800 ) :
			case( 8808 ) :
			case( 8880 ) :
			case( 8888 ) :
			case( 443 ) :	// SSL
			case( 6660 ) :  // IRC
			case( 6661 ) :
			case( 6662 ) :
			case( 6663 ) :
			case( 6664 ) :
			case( 6665 ) :
			case( 6666 ) :
			case( 6667 ) :
			case( 6668 ) :
			case( 6669 ) :
			case( 6670 ) :
			case( 7000 ) :
				Action = ANON;
				break ;
			// Certain ports (NNTP, POP and SMTP connect directly
			// even with a nym
			case( 119 ) :		// NNTP
			case( 25 ) :		// SMTP
			case( 110 ) :		// POP3
				Action = OPEN;
				break ;
		}
	}

	if( (in_DestPort == FR_DFLT_AIP_TCP_PORT && nType == IPPROTO_TCP) ||	// D-H
		(in_DestPort == FR_DFLT_AIP_UDP_PORT && nType == IPPROTO_UDP) ){	// Data

		if(1) {
			ZKTRACE1("Encrypting a UDP packet to first hop, or doing D-H. Port is [%d]\n", in_DestPort);
			Action = OPEN ;
		}
		else{
			ZKTRACE0("Someone is trying to hack us!\n");
			Action = BLOCK;			
		}

#ifdef DNS_THROUGH_CLOUD
	}else if(in_DestPort == m_dnsport && IPPROTO_UDP == nType){
		ZKTRACE0("Use the cloud for DNS if a Nym is selected\n");
		Action = ANON ;
#endif

#ifdef TOKEN_THROUGH_CLOUD
	}else if( in_DestPort == FR_DFLT_TOKEN_SERVER_PORT ) {
		if ( nType == IPPROTO_TCP ) {
			Action = ANON;	// Must use the cloud for token server, say yes to TCP traffic, package it into a UDP packet
		}
		else {
			Action = BLOCK ;
		}
#endif

#ifdef KEYQRY_THROUGH_CLOUD
	}else if( in_DestPort == FR_DFLT_KEYQRY_SERVER_PORT ){
		if ( nType == IPPROTO_UDP ) {
			Action = ANON;	// Must use the cloud for key query server, // We may still need keys later on for Emailing and initial queries may have failed
		}
		else {
			Action = BLOCK ;
		}
#endif

#ifdef FMGMAIP_THROUGH_CLOUD
	}else if( in_DestPort == FR_DFLT_FMGMAIP_SERVER_PORT ){				
		if ( nType == IPPROTO_TCP ) {
			Action = ANON;
		}
		else {
			Action = BLOCK ;
		}
#endif

#ifdef NYM_THROUGH_CLOUD
	}else if(in_DestPort == FR_DFLT_NYM_SERVER_PORT){
		if ( nType == IPPROTO_TCP ) {
			Action = ANON;
		}
		else {
			Action = BLOCK ;
		}
#endif

#ifdef KEYUPDT_THROUGH_CLOUD
	}else if( in_DestPort == FR_DFLT_KEYUPD_SERVER_PORT ){
		// Must use the cloud for the key update server
		if ( nType == IPPROTO_TCP ) {
			Action = ANON;
		}
		else {
			Action = BLOCK ;
		}
#endif

#ifdef NIQS_THROUGH_CLOUD
	}else if(in_DestPort == FR_DFLT_NIQS_SERVER_PORT){
		// Must use the cloud for niqs info 
		// (except for the first time when we launch and Firewall isn't established yet)
		if ( nType == IPPROTO_TCP ) {
			Action = ANON;
		}
		else {
			Action = BLOCK ;
		}
#endif

	}

	return Action;
}

// We've already ascertained that this must go through the cloud
void InTacLinuxStub::RouteManager(const uint32_t DestIP, const uint16_t DestPort, unsigned int& nRouteID, const int nType) const{

	ZKASSERT( FR_DFLT_AIP_TCP_PORT != DestPort ) ; // Bug#1566	

	ZKTRACE1("We have ascertained that this packet [%d] should use the cloud, using existing route or building a new one if necessary\n", DestPort);
	ZKASSERT(nType == IPPROTO_TCP || nType == IPPROTO_UDP);

	// XXX ToDo need to set the out param to be a valid route
	nRouteID = m_UnauthRoute;
}

