/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef _8B9F9CA0_A591_11d2_8E3C_00104B6AF5F7__H
#define _8B9F9CA0_A591_11d2_8E3C_00104B6AF5F7__H

#include <zks.h>
#include <vector>
#include <InTac.h>
#include <frNetworkDefaults.h>

class InTacLinuxStub : public InTac {	

public:
	InTacLinuxStub();
	virtual				~InTacLinuxStub(){}

	virtual bool		InTAC(
					const uint32_t in_DestIP, 
					const uint16_t in_DestPort, 
					unsigned int& out_nRouteID, 
					const int in_nType) ;	

	virtual void		RouteIDisDead(const unsigned int) const;

	// Initialize MyInTac with the main UI object
	bool				Init();

	// Access to counters
	unsigned long * GetClearPacketCounter() { return &m_cntClearPackets ; } ;
	unsigned long * GetCloudPacketCounter() { return &m_cntCloudPackets ; } ;
	unsigned long * GetBlockedPacketCounter() { return &m_cntBlockedPackets ; } ;

	void                InitFirewallMode() ;
	bool 			isLoaded() const {return m_bLoaded ; } ;
	void			setLoaded(const bool b) { m_bLoaded = b ; } ;

	unsigned int 	getRoute() { return m_UnauthRoute; } ;
	void		   setRoute(unsigned int r) { m_UnauthRoute = r; } ;
private:

	short				  m_dnsport;	// DNS destination port number in host (little endian) order. When udp traffic goes out on that port, we redirect it to where <frnetworkdefaults.h> or GUI overrides tells us

	bool					m_bLoaded;
	unsigned int 			    m_UnauthRoute;
		
	std::vector<uint16_t> m_vecOpenPorts ;
	unsigned long				  m_dwFirewallMode ;

	// Statistic counters
	unsigned long			  m_cntClearPackets ;
	unsigned long			  m_cntCloudPackets ;
	unsigned long			  m_cntBlockedPackets ;

	PORT_SETTING PacketFirewall(const u_long, const uint16_t, const int) const;
	void RouteManager(const uint32_t, const uint16_t, unsigned int&, const int) const;
	PORT_SETTING GetActionForUnsupportedPortWhenNymSelected( uint16_t in_port ) const ;
};
#endif
