// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom(R) code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <LinuxDnsFilter.h>
#include <RDnsMessage.h>


//===========================================================================
// CLASS LinuxDnsFilter
//===========================================================================

//---------------------------------------------------------------------------
//
LinuxDnsFilter::LinuxDnsFilter( )
{
}

//---------------------------------------------------------------------------
//
LinuxDnsFilter::~LinuxDnsFilter( )
{
}


//---------------------------------------------------------------------------
//
bool LinuxDnsFilter::ProcessDnsRequest( ZkBuffer *pbuf )
{

    // 8 is the length of a udp header. (sizeof doesn't work 'cause the struct is f*cked) --ol
    // This looks scary but it isn't. We scan the buffer for any ocurances of ALL or MX which are both DNS question types
    // that can cause MX RRs to be returned (so we need to hijack those bastards)
    ZKASSERT( pbuf->Udp.SrcPort == htons(DNS_PORT) || pbuf->Udp.DstPort == htons(DNS_PORT));
    if (
        RDnsMessage::ScanBufferForShort( RDnsMessage::ALL, reinterpret_cast<char*>(pbuf) + sizeof(ZkUDPHeader), ntohs(pbuf->Udp.UdpLength) - 8 )
        || RDnsMessage::ScanBufferForShort( RDnsMessage::MX, reinterpret_cast<char*>(pbuf) + sizeof(ZkUDPHeader), ntohs(pbuf->Udp.UdpLength) - 8 )
    ) {
        ZKTRACE0("Got a DNS packet with a MX or ALL lookup in it (I think).\n");
        // Turn it into a usable data structure.
        RDnsMessage DnsData(reinterpret_cast<char*>(pbuf) + sizeof(ZkUDPHeader), ntohs(pbuf->Udp.UdpLength) - 8);
        RDnsMessage::DnsQuestion *q;
        bool found = false;
        size_t i;
        
        for ( i = 0; i < DnsData.GetHeader().GetQDCount(); i++ ) {
            q = DnsData.GetQuestion(i);
            if ( q && ( q->GetType() == RDnsMessage::ALL || q->GetType() == RDnsMessage::MX ) ) {                    
                // Got 'em
                ZKTRACE0("Got a DNS packet with a MX or ALL lookup in it (I'm certain).\n");
                found = 1;
                break;
            }                
        }
        if ( found && q->GetClass() == RDnsMessage::INTERNET ) {
            // Now this starts to get phun.
            RDnsMessage DnsReply;
            // Setup the general DNS header.
            RDnsMessage::DnsHeader &hresponse =  DnsReply.GetHeader();
            RDnsMessage::DnsHeader &hrequest =  DnsData.GetHeader();
            
            hresponse.SetID( hrequest.GetID() ); // Copy ID
            hresponse.SetQR( 1 ); // 1 == response 0 == question
            hresponse.SetOpcode( hrequest.GetOpcode() ); // Copy opcode
            hresponse.SetAA( 0 ); // We are not authoratative
            hresponse.SetTC( 0 ); // no truncation
            hresponse.SetRD( hrequest.GetRD() );
            hresponse.SetRA( 0 ); // We do not support recursion
            hresponse.z = 0 ;
            hresponse.SetRCode( 0 ); // Green light status.
               
            DnsReply.AddQuestion( *q ); // We're responding to q
            
            RDnsMessage::DnsResourceRecord rr = RDnsMessage::DnsResourceRecord();
                
            rr.SetName( q->GetName() );
            rr.SetClass( RDnsMessage::INTERNET );
            rr.SetTtl( 0 );                 
            
            if ( q->GetType() == RDnsMessage::ALL ) {
                // If the request is type ALL we need to send back some more info.
                // one way to do this better might be to just include an MX and a authorative nameserver
                // entry for the wormhole. We'll see.
                rr.SetType( RDnsMessage::NS );
                ((RDnsMessage::NSResourceData *)rr.GetResourceData())->SetNameServer("freedomized.safedns");
                
                DnsReply.AddAnswer( rr );
                
                rr.SetType( RDnsMessage::A );
                ((RDnsMessage::AResourceData *)rr.GetResourceData())->SetAddress( inet_addr("127.0.0.1") );
                
                DnsReply.AddAnswer( rr );
            }
            // Add MX response
            rr.SetType( RDnsMessage::MX );
            ((RDnsMessage::MXResourceData *)rr.GetResourceData())->SetPreference(10); // 10 is arbitrary, we just need a number
            ((RDnsMessage::MXResourceData *)rr.GetResourceData())->SetExchange("freedomized.safedns");
            
            DnsReply.AddAnswer( rr );
            
            // Add additional section for our phony freedomized.safedns address.
            rr.SetName( "freedomized.safedns" );
            rr.SetType( RDnsMessage::A );                
            ((RDnsMessage::AResourceData *)rr.GetResourceData())->SetAddress( inet_addr("127.0.0.1") );
            
            DnsReply.AddAdditional( rr );
            
            // Ok. It's built. Now we need to TCP/IPify the data.
            
            ZkBuffer OutputBuffer;
            uint32_t tmp32;
            uint16_t tmp16;
            size_t length;
            // Slap on the old header.
            memcpy(&OutputBuffer, pbuf, sizeof(ZkUDPHeader));
            
            // swap src and dst IPs
            tmp32 = OutputBuffer.Udp.IP.SrcIpAddress;
            OutputBuffer.Udp.IP.SrcIpAddress = OutputBuffer.Udp.IP.DstIpAddress;
            OutputBuffer.Udp.IP.DstIpAddress = tmp32;
            
            length = DnsReply.Serialize( ((char*)&OutputBuffer) + sizeof(ZkUDPHeader), ZKBUFFER_LEN );
            
            OutputBuffer.Udp.IP.TotalLength = htons( length + sizeof(ZkUDPHeader) );
            OutputBuffer.Udp.UdpLength = htons( length + 8 ); // 8 is length of UDP header only
            
            // swap src and dst ports
            tmp16 = OutputBuffer.Udp.SrcPort;
            OutputBuffer.Udp.SrcPort = OutputBuffer.Udp.DstPort;
            OutputBuffer.Udp.DstPort = tmp16;
            
            // calc csums
            OutputBuffer.Udp.UdpCheckSum = 0;
            OutputBuffer.Udp.UdpCheckSum = zkUDPHeaderChecksum(&OutputBuffer.Udp, length + 8 );
            OutputBuffer.Udp.IP.HeaderChkSum = zkIPHeaderChecksum(&OutputBuffer.Udp.IP);
            
            // cocked, locked, and ready to rock.
            ZKASSERT("We have a parent" && GetOwner());
            GetOwner()->InjectPacketIntoProtocolStack(OutputBuffer, 0);
            
            return ZK_FALSE;
        }
        
    } else if ( 
        RDnsMessage::ScanBufferForString("freedomized", reinterpret_cast<char*>(pbuf) + sizeof(ZkUDPHeader), ntohs(pbuf->Udp.UdpLength) - 8  ) 
        && RDnsMessage::ScanBufferForShort( RDnsMessage::A, reinterpret_cast<char*>(pbuf) + sizeof(ZkUDPHeader), ntohs(pbuf->Udp.UdpLength) - 8 )
    ) {
        // Turn it into a usable data structure.
        RDnsMessage DnsData(reinterpret_cast<char*>(pbuf) + sizeof(ZkUDPHeader), ntohs(pbuf->Udp.UdpLength) - 8);
        RDnsMessage::DnsQuestion *q;
        bool found = false;
        size_t i;
        
        for ( i = 0; i < DnsData.GetHeader().GetQDCount(); i++ ) {
            q = DnsData.GetQuestion(i);
            if ( q && ( q->GetType() == RDnsMessage::A ) ) {                    
                // Got 'em
                ZKTRACE0("Got a DNS packet with an A lookup in it for freedomized.safedns (I'm certain).\n");
                found = 1;
                break;
            }                
        }
        if ( found && q->GetClass() == RDnsMessage::INTERNET ) {
            // Now this starts to get phun.
            RDnsMessage DnsReply;
            // Setup the general DNS header.
            RDnsMessage::DnsHeader &hresponse =  DnsReply.GetHeader();
            RDnsMessage::DnsHeader &hrequest =  DnsData.GetHeader();
            
            hresponse.SetID( hrequest.GetID() ); // Copy ID
            hresponse.SetQR( 1 ); // 1 == response 0 == question
            hresponse.SetOpcode( hrequest.GetOpcode() ); // Copy opcode
            hresponse.SetAA( 0 ); // We are not authoratative
            hresponse.SetTC( 0 ); // no truncation
            hresponse.SetRD( hrequest.GetRD() );
            hresponse.SetRA( 0 ); // We do not support recursion
            hresponse.z = 0 ;
            hresponse.SetRCode( 0 ); // Green light status.
               
            DnsReply.AddQuestion( *q ); // We're responding to q
            
            RDnsMessage::DnsResourceRecord rr = RDnsMessage::DnsResourceRecord();
            
            rr.SetName( q->GetName() );
            rr.SetClass( RDnsMessage::INTERNET );
            rr.SetTtl( 0 );                 
            rr.SetType( RDnsMessage::A );                
            ((RDnsMessage::AResourceData *)rr.GetResourceData())->SetAddress( inet_addr("127.0.0.1") );
            
            DnsReply.AddAnswer( rr );
            
            // Ok. It's built. Now we need to TCP/IPify the data.
            
            ZkBuffer OutputBuffer;
            uint32_t tmp32;
            uint16_t tmp16;
            size_t length;
            // Slap on the old header.
            memcpy(&OutputBuffer, pbuf, sizeof(ZkUDPHeader));
            
            // swap src and dst IPs
            tmp32 = OutputBuffer.Udp.IP.SrcIpAddress;
            OutputBuffer.Udp.IP.SrcIpAddress = OutputBuffer.Udp.IP.DstIpAddress;
            OutputBuffer.Udp.IP.DstIpAddress = tmp32;
            
            length = DnsReply.Serialize( ((char*)&OutputBuffer) + sizeof(ZkUDPHeader), ZKBUFFER_LEN );
            
            OutputBuffer.Udp.IP.TotalLength = htons( length + sizeof(ZkUDPHeader) );
            OutputBuffer.Udp.UdpLength = htons( length + 8 ); // 8 is length of UDP header only
            
            // swap src and dst ports
            tmp16 = OutputBuffer.Udp.SrcPort;
            OutputBuffer.Udp.SrcPort = OutputBuffer.Udp.DstPort;
            OutputBuffer.Udp.DstPort = tmp16;

            // calc csums
            OutputBuffer.Udp.UdpCheckSum = 0;
            OutputBuffer.Udp.UdpCheckSum = zkUDPHeaderChecksum(&OutputBuffer.Udp, length + 8 );
            OutputBuffer.Udp.IP.HeaderChkSum = zkIPHeaderChecksum(&OutputBuffer.Udp.IP);
            
            // cocked, locked, and ready to rock.
            ZKASSERT("We have a parent" && GetOwner());
            GetOwner()->InjectPacketIntoProtocolStack(OutputBuffer, 0);
            
            return ZK_FALSE;
        }
    }
    return DnsFilter::ProcessDnsRequest( pbuf );
}

