/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#if !defined(__LINUX_PACKET_HANDLER_INCLUDED)
#define __LINUX_PACKET_HANDLER_INCLUDED

#include <stdexcept>
#include <zkThread.h>
#include <../linux/RZkException.h>
#include <DPacket.h>
#include <../linux/ZkPipe.h>
#include "LinuxDnsFilter.h"
//#define	FRAME_LENGTH 1514 
// Windoze makes the assumption that the biggest MTU will ever see is 1500 bytes.
// On linux the largest MTU I've found in a device driver is 10240byte, for the ap1000 bif (?!)
// device. It would probably be wise to make this dynamic in the future.
// (The driver for CTC ESCON actually has an MTU that in the 32000 ballpark, but I don't think
// this is availble for anything but S/390 ... and I doubt there will be a lot of users running 
// freedom on mainframes)
#define FRAME_LENGTH 10240

class RLinuxPacketHandler : public DPacket
{
private:
	ZkThread thread;
	static bool bRunning;
#ifdef POLL_SHUTDOWN
    ZkPipe m_pipe;
#else
    static void signalHandler(int);
#endif
	
    LinuxDnsFilter m_DnsFilter;

	RLinuxPacketHandler(const RLinuxPacketHandler&);
	RLinuxPacketHandler& operator=(const RLinuxPacketHandler&);

public:
	RLinuxPacketHandler();
	virtual ~RLinuxPacketHandler();

	virtual int Init() throw( RZkException );
	virtual void Terminate();

    virtual bool InjectPacketIntoProtocolStack(const ZkBuffer &buf, 
                                               const unsigned int &nDevice);

    virtual bool RLinuxPacketHandler::HandleOutGoingUDPPacket( const int nDevice, ZkBuffer *pbuf );

	static ZK_THREAD_CALLBACK void *PacketThread(void *);


	virtual DnsFilter& GetDnsFilter();

};

#endif // __LINUX_PACKET_HANDLER_INCLUDED
