/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef _RSHIMINTERFACE_INCLUDED
#define _RSHIMINTERFACE_INCLUDED

#include <string>
#include <zkshim.h>
#include <zkDefs.h>
#include <clientstl.h>
#include <../linux/RZkException.h>
#include <zksocket.h>


#include "RLinuxPacketHandler.h" // For friend func

// Temporary defines for ports. These should become dynamically configurable.

#define REMOTE_POP3_PORT            110
#define REMOTE_SMTP_PORT            25
#define REMOTE_HTTP_PORT            80
#define REMOTE_HTTP_PORT2           81
#define REMOTE_HTTP_PORT3           88
#define REMOTE_HTTP_PORT4           8000
#define REMOTE_HTTP_PORT5           8008
#define REMOTE_HTTP_PORT6           8080
#define REMOTE_HTTP_PORT7           8088
#define REMOTE_HTTP_PORT8           8800
#define REMOTE_HTTP_PORT9           8808
#define REMOTE_HTTP_PORT10          8880
#define REMOTE_HTTP_PORT11          8888
#define REMOTE_NNTP_PORT            119
#define REMOTE_SSH_PORT             22
#define REMOTE_TELNET_PORT          23
#define REMOTE_SSL_PORT             443
#define REMOTE_IRC_PORT             6660
#define REMOTE_IRC_PORT2            6661
#define REMOTE_IRC_PORT3            6662
#define REMOTE_IRC_PORT4            6663
#define REMOTE_IRC_PORT5            6664
#define REMOTE_IRC_PORT6            6665
#define REMOTE_IRC_PORT7            6666
#define REMOTE_IRC_PORT8            6667
#define REMOTE_IRC_PORT9            6668
#define REMOTE_IRC_PORT10           6669
#define REMOTE_IRC_PORT11           6670
#define REMOTE_IRC_PORT12           7000


extern const ZKSTD::string strZKSHIM_DEVNAME;

// forward decls
class RShimRule;
class RShimRuleSet;

class RShimInterface
{
public:
	// Obtain reference to shim
	static RShimInterface &getShimInterface();	

	// Initialize shim
	void Init(const ZKSTD::string &strName = strZKSHIM_DEVNAME) throw( RZkException );

	// Shutdown shim
	void Shutdown();
	
	// Add a rule 
	void AddShimRule(const RShimRule &rule) throw( RZkException );	

	// Remove a rule 
	void DeleteShimRule(const RShimRule &rule) throw( RZkException );	

	// Add a rule set
	void AddShimRuleSet(const RShimRuleSet &rule_set, bool reverse = ZK_TRUE) throw( RZkException );	

	// Remove a rule set
	void DeleteShimRuleSet(const RShimRuleSet &rule_set) throw( RZkException );	

	// Clear all rules
	void ClearShimRules() throw( RZkException );	
	
	// Write a packet into the network stack
	ssize_t WritePacket(unsigned char *buf, ssize_t len) throw( RZkException );
	
	// Read a packet from the network stack
	ssize_t ReadPacket(unsigned char *buf, ssize_t buflen) throw( RZkException );

    // Validate an inbound proxy connection. RShimInterface encapsulates all communication
    // with the kernel ... including it's LSP fn()s. This will also validate the secret cookie
    // information sent to the kernel at init time. 
    void VerifySocketConnection(ZkSocket ZkSock, uint32_t &out_addr, uint16_t &out_port) throw ( RZkException );
	
    // Insert driver rules used when freedom is running in user mode
    void UserMode() throw( RZkException );

    // Insert driver rules used when freedom is running in nym-selected mode
	void NymMode() throw( RZkException );

    // Insert driver rules used when freedom is starting up.
    void InitMode() throw( RZkException );

private:
	RShimInterface();
	RShimInterface(RShimInterface &);
	RShimInterface& operator=(const RShimInterface&);

    void InitializeSecret() throw ( RZkException );

//    void AddAllProtocolProxyRules(int protocol, uint16_t local_port, ZKSTD::string regkey, ZKSTD::vector<uint16_t> default);

    void AddAllProtocolProxyRules();

    void AddAllPacketInterceptionRules( bool AddDefaultRule );

    void AddProtocolProxyRule(int protocol, uint16_t local_port, 
                              uint16_t remote_port )  throw ( RZkException );

    int GetShimFD() { return shim_fd; };
    friend void *RLinuxPacketHandler::PacketThread(void *);

	int shim_fd;
	bool bInitialized;

    static pid_t m_globalPid;

    ZkShimSecret m_secret;
};


#endif //_RSHIMINTERFACE_INCLUDED
