// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#ifndef _RSHIMRULE_INCLUDED
#define _RSHIMRULE_INCLUDED
#include <zkIntTypes.h>
#include <zkIntLimits.h>
#include <zkshim.h>
#include <vector>
#include <string.h>

// Class wrapper for struct ZKShimRule
class RShimRule {
public:
	// Ctor
	RShimRule() { memset(&rule_struct, 0, sizeof(rule_struct)); };

	// Dtor
	~RShimRule() { };

	// Copy (default)
//	RShimRule(const RShimRule &copy);

	// Assign (default)
//	RShimRule &operator=(const RShimRule &copy);

	// Modifiers
	void setTOS(const uint8_t tos) 
		{ rule_struct.zru_ip_tos = tos; };

	void setProto(const uint8_t proto) 
		{ rule_struct.zru_ip_p = proto; };
	
	// it should be pretty obvious that byte order doesn't matter for the mask below...
	void setSourceIP(const uint32_t ip, const uint32_t mask = 0xFFFFFFFF)
	 	{ rule_struct.zru_ip_src = ip; rule_struct.zru_ip_src_mask = mask; };

	void setDestIP(const uint32_t ip, const uint32_t mask = 0xFFFFFFFF)
	 	{ rule_struct.zru_ip_dst = ip; rule_struct.zru_ip_dst_mask = mask; };

	void setSourcePort(const uint16_t sport) 
		{ rule_struct.zru_sport = sport; };

	void setDestPort(const uint16_t dport) 
		{ rule_struct.zru_dport = dport; };

	void setUID(const uid_t uid) 
		{ rule_struct.zru_uid = uid; };

	void setPID(const pid_t pid) 
		{ rule_struct.zru_pid = pid; };

	void setSID(const pid_t sid) 
		{ rule_struct.zru_sid = sid; };

	void setType(const uint8_t type) 
		{ rule_struct.zru_type = type; };

	void setAction(const uint8_t action) 
		{ rule_struct.zru_action = action; };

	void setIPArg(const uint32_t ipArg) 
		{ rule_struct.zru_ip_arg = ipArg; };

	void setPortArg(const uint16_t pArg) 
		{ rule_struct.zru_port_arg = pArg; };
		
	// Selectors		
	uint8_t getTOS() const
		{ return rule_struct.zru_ip_tos; };

	uint8_t getProto() const 
		{ return rule_struct.zru_ip_p; };

	uint32_t getSourceIP() const
	 	{ return rule_struct.zru_ip_src; }

	uint32_t getSourceIPMask() const
		{ return rule_struct.zru_ip_src_mask; };

	uint32_t getDestIP() const
	 	{ return rule_struct.zru_ip_dst; };

	uint32_t getDestIPMask() const
	 	{ return rule_struct.zru_ip_dst_mask; };

	uint16_t getSourcePort() const
		{ return rule_struct.zru_sport; };

	uint16_t getDestPort() const 
		{ return rule_struct.zru_dport; };

	uid_t getUID() const 
		{ return rule_struct.zru_uid; };

	pid_t getPID() const
		{ return rule_struct.zru_pid; };

	pid_t setSID() const
		{ return rule_struct.zru_pid; };

	uint8_t setType() const
		{ return rule_struct.zru_type; };

	uint8_t getAction() const
		{ return rule_struct.zru_action; };

	uint32_t getIPArg() const
		{ return rule_struct.zru_ip_arg; };

	uint16_t getPortArg() const
		{ return rule_struct.zru_port_arg; };

private:
	ZkShimRule rule_struct;
	const ZkShimRule *getRuleStruct() const
		{ return &rule_struct; };
		
	// these need to pass the address of struct_rule to the shim via an ioctl
	friend void RShimInterface::AddShimRule(const RShimRule &rule);
	friend void RShimInterface::DeleteShimRule(const RShimRule &rule);
};

class RShimRuleSet : public ZKSTD::vector<RShimRule>
{
public:
	// public ctors from <vector>
	RShimRuleSet() : ZKSTD::vector<RShimRule>() {};
	RShimRuleSet(const RShimRuleSet &cp) : ZKSTD::vector<RShimRule>(cp) {};
	RShimRuleSet(const_iterator first,
		    const_iterator last
		    ) : ZKSTD::vector<RShimRule>(first, last) {};
	explicit RShimRuleSet(ZKSTD::vector<RShimRule>::size_type n, RShimRule t = RShimRule()) : ZKSTD::vector<RShimRule>(n,t) {};
};

#endif //_RSHIMRULE_INCLUDED
