/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef DNSFILTER_INCLUDED__
#define DNSFILTER_INCLUDED__
#include <client.h>
#include <clientstl.h>
#include <map>
#include <zkPacket.h>
#include <zkThread.h>
#include <DZkError.h>
#define DNSKEY_TTL 5

//---------------------------------------------------------------------------
//
class DnsKey {
public:

	DnsKey();

	DnsKey(uint16_t dnsId, uint16_t SourcePort, unsigned short Ttl = DNSKEY_TTL);
	~DnsKey();

	void SetDnsId(const uint16_t dnsId) 
		{ m_DnsId = dnsId; };

	uint16_t GetDnsId() const 
		{ return m_DnsId; };

	void SetSourcePort(const uint16_t sPort) 
		{ m_SourcePort = sPort; };

	uint16_t GetSourcePort() const 
		{ return m_SourcePort; };

	void SetTtl(const unsigned short Ttl) 
		{ m_Ttl = Ttl; };

	unsigned short GetTtl() const 
		{ return m_Ttl; };


	// Comparison functor for STL sorted container templates
	struct lessthan
	{
		bool operator()(const DnsKey& c1, const DnsKey& c2) const;
	};
	
	operator uint32_t() const
	{
		return ( ( (uint32_t)GetDnsId() ) << 16 ) | ( (uint32_t)GetSourcePort() );
	}

private:
	uint16_t m_DnsId;
	uint16_t m_SourcePort;
	unsigned short m_Ttl;
};


//---------------------------------------------------------------------------
//
class IDnsFilterOwner {
public:
	virtual bool InjectPacketIntoProtocolStack(const ZkBuffer &buf, const unsigned int &nDevice) = 0;
};


//---------------------------------------------------------------------------
//
class DnsFilter {
public:
	DnsFilter();
	virtual ~DnsFilter();

	virtual bool ProcessDnsRequest( ZkBuffer *pdnsBuffer );
	virtual bool ProcessDnsResponse( ZkBuffer *pdnsBuffer );

    IDnsFilterOwner *GetOwner() const
    { return m_Owner; };

    void SetOwner( IDnsFilterOwner *owner)
    { m_Owner = owner; };

	static void ZK_ITIMER_CALLBACK CleanupTimer(void*, unsigned int, unsigned int, unsigned long);

private:

	// Prohibited funcs - not implemented.
	DnsFilter( const DnsFilter& );
	const DnsFilter& operator=( const DnsFilter& );

	typedef ZKSTD::map< DnsKey, uint32_t > DnsMapType;

	DnsMapType m_DnsMap;
	IDnsFilterOwner *m_Owner;
	ZkMutex m_mutex;
	DZkError m_err;
};



#endif DNSFILTER_INCLUDED__
