// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// RCloudSocket.h

#ifndef _RCLOUDSOCKET_H
#define _RCLOUDSOCKET_H

#ifdef WIN32
#pragma warning(disable: 4290)
#else
#	include <netinet/in.h>
#	include <arpa/inet.h>
#endif

#include <client.h>
#include <zksocket.h>

#include <zkThread.h>

#include "TrafficFilterInc.h"

class DPacket ;
class RZkException ;
class IRoutePacket ;
struct T_ZkFragCache ;


class TRAFFICFILTER_API RCloudSocket
{
public:
	void    Init( DPacket* in_pPacket ) throw(RZkException) ;
	void    Terminate() ;

	void	SetRoutePacketHandler(IRoutePacket* in_pRoutePacket);

	bool	SendTo(const char *, const int, const SOCKADDR_IN&, int nWait = 0) const;
	static	RCloudSocket& theRCloudSocket();		// doesn't clutter global namespace
	virtual ~RCloudSocket();

private:


	RCloudSocket();							// Can't be used as base classes, nor can it be embedded inside other objects. 
	RCloudSocket(const RCloudSocket&);			// never allow copying
	RCloudSocket& operator=(const RCloudSocket&); // never allow assignment
	bool CheckMTU(const ZkSocketDescriptor, const int) const;

	static void * ZK_THREAD_CALLBACK UDPServer(void * pThis);

	ZkSocketDescriptor  m_UDPThreadSocket;
	DPacket*			m_pDPacket ;
	T_ZkFragCache *		m_fcache ;
	ZkThread			m_pUdpThread;
	IRoutePacket*		m_pRoutePacket ;
} ;

#endif
