// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// TrafficFilter.cpp: TrafficFilter of the TrafficFilter class.
//
//////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

#include "TrafficFilter.h"
#include "RCloudSocket.h"
#include "DPacket.h"

#ifdef WIN32
#	include "../win9x/RWin9xPacketHandler.h"
#	include "../win2k/RWin2kPacketHandler.h"
#	include <../win/ZKBaseUI.h>
#else
#   include <RLinuxPacketHandler.h>
#endif
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TrafficFilter::TrafficFilter() : 	m_pPacket(NULL)
{
	
}

TrafficFilter::~TrafficFilter()
{
	
}

TrafficFilter&
TrafficFilter::theTrafficFilter()
{
	static TrafficFilter sTF; 
	return sTF ;
}


bool 
TrafficFilter::Init(IPacketFirewall* in_pFirewall, IRoutePacket* in_pRoutePacket ) throw(RZkException, ZKSTD::bad_alloc)
{
#ifdef WIN32
	if(ZKBaseUI::RunningUnderNt() )
	{
		m_pPacket = new RWin2kPacketHandler;		
	}
	else
	{
		m_pPacket = new RWin9xPacketHandler;
	}
#elif defined(ZK_LINUX)
	m_pPacket = new RLinuxPacketHandler;
#endif

	NEW_FIX(m_pPacket);

	m_pPacket->Init();

	m_pPacket->SetFirewallPtr(in_pFirewall);
	m_pPacket->SetRoutePacketHandler( in_pRoutePacket );

	RCloudSocket::theRCloudSocket().Init( m_pPacket ) ;
	RCloudSocket::theRCloudSocket().SetRoutePacketHandler( in_pRoutePacket ) ;

	return true;
}

void TrafficFilter::Terminate()
{	
	RCloudSocket::theRCloudSocket().Terminate() ;

	m_pPacket->Terminate();
	SAFEDELETE(m_pPacket);
}

#ifdef WIN32
void TrafficFilter::AllowNetBiosInTheClear(const bool in_bAllowInTheClear)
{
	
#ifdef INTERNAL_BUILD
 	if(getenv("BYPASS_DRIVER"))
		return;
#endif

	((RWinPacketHandler*)m_pPacket)->AllowNetBiosInTheClear( in_bAllowInTheClear ) ; 
}

void TrafficFilter::AllowRpcInTheClear(const bool in_bAllowInTheClear)
{
	
#ifdef INTERNAL_BUILD
 	if(getenv("BYPASS_DRIVER"))
		return;
#endif
	((RWinPacketHandler*)m_pPacket)->AllowRpcInTheClear( in_bAllowInTheClear ) ; 
}

void TrafficFilter::AllowDhcpInTheClear(const bool in_bAllowInTheClear)
{
	
#ifdef INTERNAL_BUILD
 	if(getenv("BYPASS_DRIVER"))
		return;
#endif
	((RWinPacketHandler*)m_pPacket)->AllowDhcpInTheClear( in_bAllowInTheClear ) ; 
}


// See bug 6933
void
TrafficFilter::SetRulesForFirstHopsInRoutes( const ZKSTD::vector<unsigned int>& in_nboIPs )
{
#ifdef INTERNAL_BUILD
 	if(getenv("BYPASS_DRIVER"))
		return;
#endif
	((RWinPacketHandler*)m_pPacket)->SetRulesForFirstHopsInRoutes( in_nboIPs ) ; 
}

// see bug 7139
void
TrafficFilter::CaptureAllIncomingPackets( const bool in_bAllow )
{
#ifdef INTERNAL_BUILD
 	if(getenv("BYPASS_DRIVER"))
		return;
#endif
	((RWinPacketHandler*)m_pPacket)->CaptureAllIncomingPackets( in_bAllow ) ; 
}

// See bug 7668
void 
TrafficFilter::FlushAllRules() throw()
{
#ifdef INTERNAL_BUILD
 	if(getenv("BYPASS_DRIVER"))
		return;
#endif
	((RWinPacketHandler*)m_pPacket)->FlushAllRules() ; 
}

#endif // WIN32

#ifdef INTERNAL_BUILD
#ifdef WIN32
void
TrafficFilter::TestCrash( ZkCrashType in_PreferredDeath )
{ 
	ZKASSERT( in_PreferredDeath != ZKUNDEFINED ) ;
	DoCrash( in_PreferredDeath ) ;
}
#endif // WIN32
#endif // INTERNAL_BUILD
