// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// Main singleton for the traffic filter
// Provides useful functionality of a general nature
// Used to be called "Exported Class"

#if !defined _TRAFFICFILTER_H
#define _TRAFFICFILTER_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TrafficFilterInc.h"

#include <client.h>

#include <zks.h>
#include <ErrorHandling.h>
//#include <ExceptionFilter.h> // For ZKCrashType
#include <zkThread.h>
#include <vector>

#ifdef WIN32
#	include "../win/ExceptionFilter.h"
#	include <winsock2.h>		// WSADATA
#endif


class RZkException ;
class DPacket ;
class IPacketFirewall;
class IRoutePacket ;



class TRAFFICFILTER_API TrafficFilter  
{
public:
	static TrafficFilter& theTrafficFilter();		// doesn't clutter global namespace
	virtual ~TrafficFilter();

//!italic: Interfaces available for use by clients
	
	// Returns true is winsock 2 is installed, and also fills in WSADATA structure.
	//!italic: Must be the first interface used, even before Init()
	//!italic: The reason I don't call it from Init() is to allow better granularity of error handling
	//!italic: so that you can warn the user about the winsock problem.
	//!italic: You can freely download Winsock 2.0 from http://www.microsoft.com/windows/downloads/contents/Updates/W95Sockets2/default.asp

	bool Init(IN IPacketFirewall*, IRoutePacket* in_pRoutePacket ) throw(RZkException, ZKSTD::bad_alloc);
	// Starts two threads, the thread listening to driver notifications
	// the second thread is the UDPServer thread, listening on port WINDOWS_TCP_AND_UDP_PORT
	// It will query the cryption and signing keys for the aips, so please specify the key query server
	// if the key query server name is empty, then it ill not attempt to retrieved the keys

	void Terminate() ;
	// terminates threads started by Init()
	// Only call this if Init() returned true;
	// It also kills any remaining routes so you don't have to

		
#ifdef WIN32
	// PURPOSE:	Bug # 318 workaround
	//			This tells the driver to hand us netbios traffic or not
	//
	//			Note. Netbios affects incoming AND outgoing traffic such that
	//			you will *NEVER* see netbios in the firewall logs
	// REQUIRES:Init() to have succeeded previously	
	void AllowNetBiosInTheClear(const bool in_bIntercept) ;

	// PURPOSE: This affects only outgoing RPC packets
	//			Incoming Rpc packets are handled by the firewall
	void AllowRpcInTheClear(const bool in_bIntercept) ;

	// PURPOSE: This affects only outgoing Dhcp packets
	//			Incoming Rpc packets are handled by the firewall
	void AllowDhcpInTheClear(const bool in_bIntercept) ;

	// PURPOSE: See bug 6933
	//			Inform the driver that packets should be allowed out on the UDP Cloud
	//          port (51101) to the following IPs.  This is a performance optimization,
	//			the client runs fine without it.  Linux does not need this optimization
	//			because it already allows all UDP 51101 packets sent from freedom PID out.
	void SetRulesForFirstHopsInRoutes( const ZKSTD::vector<unsigned int>& in_nboIPs ) ;

	// PURPOSE: See bug 7139
	void
	CaptureAllIncomingPackets( const bool in_bAllow );

	void 
	FlushAllRules() throw();

#endif

#ifdef INTERNAL_BUILD
#ifdef WIN32
	void TestCrash( ZkCrashType in_PreferredDeath);
#endif
#endif

private:
	// CTOR member init
	DPacket*		m_pPacket;

	TrafficFilter();								// Can't be used as base classes, nor can it be embedded inside other objects.
	TrafficFilter(const TrafficFilter&);			// never allow copying
	TrafficFilter& operator=(const TrafficFilter&); // never allow assignment
};

#endif
