/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UICONSOLEWINDOW
#define ZKS_INCL_UICONSOLEWINDOW

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#ifndef ZKS_INCL_IZKCONSOLEMSG
#include <IZkConsoleMsg.h>
#endif

#ifndef ZKS_INCL_ZKOBSERVER
#include <ZkObserver.h>
#endif

#ifndef ZKS_INCL_UITHREADSAFEDLG
#include <UiThreadSafeDlg.h>
#endif

#include <clientstl.h>
#include <memory>

#ifndef __GTK_H__
#include <gtk/gtk.h>
#endif

#ifndef __UTILITY__
#include <utility>
#endif

#ifndef __STRING__
#include <string>
#endif

#include <queue>

class ZkHint;


/*===========================================================================
 * CLASS UiConsoleWindow
 *=========================================================================*/

/*
  This class provides a top level window in which messages from the IZkConsoleMsg will get
  displayed, along with an icon corresponding to their type.
  To open a console window, simply instanciate one and open it.
  You will also need to connect it 
*/

class UiConsoleWindow 
    : public ZkObserver
    , public UiThreadSafeDlg
{
  
public: 
  
    /*----- member functions -----*/

    UiConsoleWindow();
    ~UiConsoleWindow();

    void Show();
    void Hide();

	// Purpose:  We unfortunately need this function so we can hook ourselves
    //           to the "hide" signal of the Console.
    GtkWidget* getTopContainer() const;

protected:

    /*----- member functions -----*/

    virtual void UiThreadCallback();


private: 

    /*----- types and enumerations ----*/

    typedef ZKSTD::pair< GdkPixmap*, GdkPixmap* > pixmask;


    /*----- static member functions -----*/

    static gint DeleteEventCb( GtkWidget*, GdkEvent*, UiConsoleWindow* );
    static void ConfigureCb( GtkWidget*, GdkEventConfigure*, 
       UiConsoleWindow*);
    void RegisterHints();
  


    /*----- member functions -----*/

    void AddMessage( 
        const ZKSTD::string& msg,
        ConsImage img = CONSIMAGE_NORMAL
    );

    void LoadPixmaps();
    void AddPixmap( int index, const char* filename );

	// Purpose:  Called by ZkSubject to notify  a change through a ZkHint.    
    //           (From the ZkObserver api)
	// Requires: This function expects a ZkHint of type ZkConsoleHInt.
    virtual void update( ZkHint* );

    /*----- data members -----*/

    GtkWidget* m_window;
    GtkWidget* m_list;

    int m_msgCount;
    gint m_window_x;
    gint m_window_y;
    gint m_window_height;
    gint m_window_width;
	bool m_verbose;

    // Table of pixmaps corresponding to the image enum in ZKConsoleMsg.h
    pixmask m_pixmaps[ CONSIMAGE_COUNT ];


    // Woah, there goes portability. Don't worry, this is platform specific code
    // to the bone.
    guint32 m_xid;

    // This varaible is used as a bridge between any background thread
    // and the UI thread. The background thread will fill up the string,
    // then it will write to the file descriptor to tell the ui thread
    // there is a message to display. The lock provided by the base
    // class UiThreadSafeDlg is used to protect this string during read/write 
    // ops.
//    ZKSTD::auto_ptr< ZkHint > m_pHint;
    ZKSTD::queue< ZkHint* > m_HintQueue;
};

//---------------------------------------------------------------------------
//
inline GtkWidget* UiConsoleWindow::getTopContainer() const
{
    return m_window;
}

#endif
