/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UIICONDISPLAY
#define ZKS_INCL_UIICONDISPLAY

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include <ZkObserver.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gtk/gtk.h>

#include <IZkException.h>


/*===========================================================================
 * CLASS UiIconDisplay
 *=========================================================================*/

/*
 * This class observes ZkStatusSubject and displays the animation or icon
 * when it gets busy/not-busy signals, respectively.
 */

class UiIconDisplay : public ZkObserver  {

public: 

    /*----- member functions -----*/

    UiIconDisplay( GtkWidget* icon ) throw(IZkException);
    virtual ~UiIconDisplay();


    void UpdateIcon( const char* filename );

protected:

    /*----- member functions -----*/

    // Purpose:  This function receives ZkStatusHint notices from the 
    //           ZkStatusSubject singleton and turns the busy animation 
    //           on or off accordingly
    virtual void update( ZkHint* );

private:

    /*----- static member functions -----*/

    static gint NextBusyFrame( gpointer data );


    /*----- member functions -----*/

    // Purpose: Renders the given GdkPixbuf into the m_icon GtkPixmap.
    //          If pixbuf == NULL, hide m_icon instead.
    //          If overlap == TRUE, render over top of the old pixbuf instead of destroying
    void Render( GdkPixbuf *pixbuf, bool overlap );

    // Purpose: Self explanatory (I hope..)
    //          Those function are private, since we want the animation to be
    //          controlled by status hints.
    //          The update() function processes hints sent by a global status
    //          subject.
    void StartAnimation();
    void StopAnimation();


    /*----- data members -----*/

    GtkWidget* m_icon; // GtkPixmap
    GdkPixbuf* m_pixbuf; // Last icon to be shown (already scaled)

    // Animation support
    GdkPixbufAnimation* m_anim;
    GList *m_frames;
    GList *m_curframe;

    gint m_busy_refcount;
    gint m_timeout_tag;
};

#endif
