// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):


//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiMainWindowImp.h>
#include <ZkGladeLoader.h>
#include <ZkGtkUtils.h>

#include <ZkTest.h>

#include <UiNIQSUpdate.h>
#include <UiStatusDisplay.h>
#include <UiThreadBase.h>
#include <UiIconDisplay.h>

#include <UiPrefInit.h>
#include <UiPrefDlg.h>
#include <UiMessages.h>
#include <WzSetupDlg.h>
#include <ZkDialogsStrings.h>

#include <ZkProxyDialogs.h> // for test purposes

#include <UiThreadId.h>

#include <INym.h>
#include <INymCentral.h>
#include <IRouteManager.h>
#include <IZkConsoleMsg.h>
#include <IZkException.h>
#include <ZkService.h>
#include <IClientCentral.h>
#include <IClientPersist.h>
#include <TrafficFilter.h>
#include <ZKFileConfiguration.h>
#include <ZkStringMgr.h>
#include <ZkStatus.h>
#include <ZkStatusSubject.h>

// For access to NIQS update method --ol
#include <ZkService.h>
#include <IClientInfo.h>

#include <IZkRegistry.h>
#include <IZKErrorReporter.h>
#include <ZkErrorHandler.h>
#include <client.h>

#include <UiThreadId.h>
#include <WNymIconInfo.h>

#include <gdk/gdkkeysyms.h>
#include <glade/glade.h>

#include <zkThread.h>

// FIXME - temporary test , remove.
#include <UiPrefBase.h>
#include <memory>

#include "UiMainWindow.h"

#include <gdk/gdkx.h>
#include <X11/Xlib.h> // for iconify

const char NYM_INDEX[] = { "nym_index" };  // index of a nym in zkcentral
const char MENU_INDEX[] = { "menu_index" };// index of a nym in the GTK menu.
const int  MW_NEW_NYM   = -1;

// FIXME - this should probably be defined in xome xp header, since 
//         the definitions are the same as for ../win
const char MAINDLG_RECT_LEFT_KEY[] = { "Main Dlg Rect Left" };
const char MAINDLG_RECT_TOP_KEY[] = { "Main Dlg Rect Top" };
const char MAINDLG_EXTENDED_KEY[] = { "Main Dlg Extended" };
const char CONSOLE_VISIBLE_KEY[] = { "Console Visible" };

//===========================================================================
// LOCAL FUNCTIONS
//===========================================================================

#ifdef INTERNAL_BUILD
// Simple little thread function for running unit tests. --ol
static void * ZK_THREAD_CALLBACK testThreadProc(void * arg)
{
    char strbuf[1024];

    GdkEventClient* event = (GdkEventClient *)gdk_event_copy( (GdkEvent*)arg );
    snprintf(strbuf, sizeof(strbuf), "Running Test %s", ZkTest::GetTestID(static_cast<unsigned int>(event->data.l[0]))->GetName());
	IZkConsoleMsg::ShowMsg( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, strbuf);
    ZkTest::RunTestID( static_cast<unsigned int>(event->data.l[0]), reinterpret_cast<void*>(event->data.l[1]));
    snprintf(strbuf, sizeof(strbuf), "Completed Test %s",  ZkTest::GetTestID(static_cast<unsigned int>(event->data.l[0]))->GetName());

    // XXX FIXME this sleep gets rid of an XLib async reply and SEGV. Remove when the console
    // becomes thread safe --ol
    sleep(1);
	IZkConsoleMsg::ShowMsg( CONS_DEBUG, CONSIMAGE_QUESTION, IDS_CONS_JUST_STRING, strbuf);

    gdk_event_free( (GdkEvent*)event );

    // ret val is never used
    return NULL;
}
#endif


//---------------------------------------------------------------------------
//
// This is a handler for client events. Client events are used by other apps to
// have freedom perform certain actions.
//
gboolean ClientEvent(
    GtkWidget* /*widget*/,
    GdkEventClient* event,
    gpointer /*user_data*/
)
{
#ifdef INTERNAL_BUILD
    // FIXME - message_type is hard code
    // this message_type is ExecuteUnitTest Id == l[0]
    if(event->message_type == 12345L)
    {
        ZkError err;
        ZKVERIFY( zkErrorNew(&err) );
        zkThreadCreateDetached (err, testThreadProc, (void *)event);
        return TRUE;
    }
#endif
    return FALSE;
}

//===========================================================================
// LOCAL CLASSES
//===========================================================================

namespace {

//===========================================================================
// CLASS UiStartAuthRoute
//===========================================================================

class UiStartAuthRoute : public UiThreadBase
{
public:
    UiStartAuthRoute( int index ): m_index( index ), m_built( false ) {};
    ~UiStartAuthRoute(){};

    bool IsRouteBuilt() const
    {
        return ( WasSuccessful() && m_built );
    }

protected:
    virtual void OnThreadStart()
    {
        UiMainWindow::get().SetDisabled( true );
        ZkStatusSubject::UpdateStatus( ZK_STATUS_NETWORK_BUSY, true, true );
    }

    virtual void OnThreadEnd()
    {
        ZkStatusSubject::UpdateStatus(ZK_STATUS_NETWORK_BUSY, false, true);
        UiMainWindow::get().SetDisabled( false );
    }

    virtual void OnThreadExecute()
    {
        m_built = ZkService::GetIRouteManager()->StartAuthenticatedRoute( m_index );
    }
    
private:

    int  m_index;
    bool m_built;
};


//===========================================================================
// CLASS UiLightTest
//===========================================================================

class UiLightTest : public UiThreadBase
{
public:
    UiLightTest( ){};
    ~UiLightTest(){};


protected:
    virtual void OnThreadStart()
    {
        ZKTRACE0( "Starting Light Test ******************\n" );
    }

    virtual void OnThreadEnd()
    {
        ZKTRACE0( "End Light Test ***********************\n" );
    }

    virtual void OnThreadExecute()
    {
        for( int i = 0; i < 5; ++i )
        {
            // Test the status lights.
            ZKTRACE0( "Lights ON\n" );
            ZkStatusSubject::UpdateStatus(ZK_STATUS_MAIL_CONNECTION  , true, true ); 
            ZkStatusSubject::UpdateStatus(ZK_STATUS_SSL_CONNECTION   , true, true ); 
            ZkStatusSubject::UpdateStatus(ZK_STATUS_NEWS_CONNECTION  , true, true ); 
            ZkStatusSubject::UpdateStatus(ZK_STATUS_IRC_CONNECTION   , true, true ); 
            ZkStatusSubject::UpdateStatus(ZK_STATUS_TELNET_CONNECTION, true, true ); 
            
            {
                ZkStatus status( ZK_STATUS_WEB_CONNECTION );
                sleep( 1 );
            }
            
            // Test the status lights.
            ZKTRACE0( "Lights OFF\n" );
            ZkStatusSubject::UpdateStatus(ZK_STATUS_MAIL_CONNECTION  , false, true ); 
            ZkStatusSubject::UpdateStatus(ZK_STATUS_SSL_CONNECTION   , false, true ); 
            ZkStatusSubject::UpdateStatus(ZK_STATUS_NEWS_CONNECTION  , false, true ); 
            ZkStatusSubject::UpdateStatus(ZK_STATUS_IRC_CONNECTION   , false, true ); 
            ZkStatusSubject::UpdateStatus(ZK_STATUS_TELNET_CONNECTION, false, true ); 

            sleep( 1 );
        }
    }
};




}

//===========================================================================
// PUBLIC FUNCTIONS
//===========================================================================

//===========================================================================
// CLASS UiMainWindowImp
//===========================================================================

//---------------------------------------------------------------------------
//
UiMainWindowImp::UiMainWindowImp() 
    : m_window( 0 )
    , m_lockPixmap( 0 )
    , m_lockBitmapMask( 0 )
    , m_bExtended( false )
    , m_dragging( false ) 
{
    // Load small footprint layout.
    ZkGladeLoader loader( "main.xml", "MainWindow" );

    m_window  = loader.GetWidget( "MainWindow" );
    m_info    = loader.GetWidget( "console" );
    m_prefs   = loader.GetWidget( "prefs" );
    m_nyms    = loader.GetWidget( "nyms" );
    m_nym_ct  = loader.GetWidget( "nym_ct" );
    m_ebox    = loader.GetWidget( "icon_box" );
    m_layout  = loader.GetWidget( "layout" );
    m_extend  = loader.GetWidget( "extend" );
    m_titlebar= loader.GetWidget( "titlebar" );
    m_minimize= loader.GetWidget( "minimize" );
    m_close   = loader.GetWidget( "close" );
    m_mail    = loader.GetWidget( "mail" );
    m_help    = loader.GetWidget( "help" );
    m_cookie  = loader.GetWidget( "cookie" );
    m_adblock = loader.GetWidget( "adblock" );
    m_icon    = loader.GetWidget( "icon" );

    gtk_widget_realize( m_window );
    gdk_window_set_decorations( m_window->window, (GdkWMDecoration)0 );

    m_dragCursor = gdk_cursor_new( GDK_FLEUR );

    try {
        m_icondisplay = new UiIconDisplay( m_icon );
    }
    catch (IZkException& ex) {
        ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
    }

    gtk_signal_connect( GTK_OBJECT( m_window ), "delete_event", 
                        GTK_SIGNAL_FUNC( DeleteCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_window ), "configure_event", 
                        GTK_SIGNAL_FUNC( ConfigureCb ), this);
    gtk_signal_connect( GTK_OBJECT( m_window ), "client_event", 
                        GTK_SIGNAL_FUNC( ClientEvent ), this );
    gtk_signal_connect( GTK_OBJECT( m_ebox ), "button_press_event", 
                        GTK_SIGNAL_FUNC( IconClickedCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_info ), "toggled", 
                        GTK_SIGNAL_FUNC( ConsoleCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_extend ), "clicked", 
                        GTK_SIGNAL_FUNC( ExtendCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_prefs ), "clicked", 
                        GTK_SIGNAL_FUNC( PrefsCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_minimize ), "clicked", 
                        GTK_SIGNAL_FUNC( MinimizeCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_console.getTopContainer() ), "hide", 
                        GTK_SIGNAL_FUNC( HideCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_mail ), "clicked", 
                        GTK_SIGNAL_FUNC( MailCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_help ), "clicked", 
                        GTK_SIGNAL_FUNC( HelpCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_cookie ), "clicked", 
                        GTK_SIGNAL_FUNC( CookieCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_adblock ), "clicked", 
                        GTK_SIGNAL_FUNC( AdBlockCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_close ), "clicked", 
                        GTK_SIGNAL_FUNC( CloseCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_titlebar ), "button-press-event", 
                        GTK_SIGNAL_FUNC( TitlebarPressCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_titlebar ), "button-release-event", 
                        GTK_SIGNAL_FUNC( TitlebarReleaseCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_titlebar ), "motion-notify-event", 
                        GTK_SIGNAL_FUNC( TitlebarMoveCb ), this );


    // Load "extended" window layout;
    ZkGladeLoader loader2( "main.xml", "layout_ext" );

    m_layout_ext   = loader2.GetWidget( "layout_ext" );
    m_collapse     = loader2.GetWidget( "collapse" );
    m_info_ext     = loader2.GetWidget( "console" );
    m_prefs_ext    = loader2.GetWidget( "prefs" );
    m_nym_ct_ext   = loader2.GetWidget( "nym_ct" );
    m_ebox_ext     = loader2.GetWidget( "icon_box" );
    m_titlebar_ext = loader2.GetWidget( "titlebar" );
    m_minimize_ext = loader2.GetWidget( "minimize" );
    m_close_ext    = loader2.GetWidget( "close" );
    m_mail_ext     = loader2.GetWidget( "mail" );
    m_help_ext     = loader2.GetWidget( "help" );
    m_cookie_ext   = loader2.GetWidget( "cookie" );
    m_adblock_ext  = loader2.GetWidget( "adblock" );

    gtk_widget_set_events( m_titlebar, 
                           GDK_FOCUS_CHANGE_MASK | 
                           GDK_BUTTON_MOTION_MASK | 
                           GDK_BUTTON_PRESS_MASK | 
                           GDK_BUTTON_RELEASE_MASK);
 
    gtk_signal_connect( GTK_OBJECT( m_collapse ), "clicked", 
                        GTK_SIGNAL_FUNC( CollapseCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_prefs_ext ), "clicked", 
                        GTK_SIGNAL_FUNC( PrefsCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_ebox_ext ), "button_press_event", 
                        GTK_SIGNAL_FUNC( IconClickedCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_info_ext ), "toggled", 
                        GTK_SIGNAL_FUNC( ConsoleCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_minimize_ext ), "clicked", 
                        GTK_SIGNAL_FUNC( MinimizeCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_close_ext ), "clicked", 
                        GTK_SIGNAL_FUNC( CloseCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_mail_ext ), "clicked", 
                        GTK_SIGNAL_FUNC( MailCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_help_ext ), "clicked", 
                        GTK_SIGNAL_FUNC( HelpCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_cookie_ext ), "clicked", 
                        GTK_SIGNAL_FUNC( CookieCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_adblock_ext ), "clicked", 
                        GTK_SIGNAL_FUNC( AdBlockCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_titlebar_ext ), "button-press-event", 
                        GTK_SIGNAL_FUNC( TitlebarPressCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_titlebar_ext ), "button-release-event", 
                        GTK_SIGNAL_FUNC( TitlebarReleaseCb ), this );
    gtk_signal_connect( GTK_OBJECT( m_titlebar_ext ), "motion-notify-event", 
                        GTK_SIGNAL_FUNC( TitlebarMoveCb ), this );


    // Initialize the indicators.
    m_status[0] = new UiStatusDisplay( loader2.GetWidget( "status_mail" ), ZK_STATUS_MAIL_CONNECTION );
    m_status[1] = new UiStatusDisplay( loader2.GetWidget( "status_web" ),  ZK_STATUS_WEB_CONNECTION );
    m_status[2] = new UiStatusDisplay( loader2.GetWidget( "status_ssl" ),  ZK_STATUS_SSL_CONNECTION );
    m_status[3] = new UiStatusDisplay( loader2.GetWidget( "status_news" ), ZK_STATUS_NEWS_CONNECTION );
    m_status[4] = new UiStatusDisplay( loader2.GetWidget( "status_irc" ),  ZK_STATUS_IRC_CONNECTION );
    m_status[5] = new UiStatusDisplay( loader2.GetWidget( "status_tlnt" ), ZK_STATUS_TELNET_CONNECTION );

    // In order to be able to connect the console window with the actual
    // console subject, the RZkConsoleMsg should already have been initialized.
    IZkConsoleMsg::get().connect( &m_console );

    // Make sure we keep a reference to our main layout in case we extend the 
    // window.
    gtk_widget_ref( m_layout );
    gtk_widget_ref( m_layout_ext );

    gtk_widget_ref( m_nyms );
    gtk_widget_ref( m_icon );
}

//---------------------------------------------------------------------------
//
UiMainWindowImp::~UiMainWindowImp()
{
    gdk_cursor_destroy( m_dragCursor );
    gtk_widget_destroy( m_window );
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::Initialize() 
{
    LoadPixmaps();
            
    // set the cursor over the nym icon        
    GdkCursor * cursor = gdk_cursor_new( GDK_HAND2 );
    gtk_widget_realize( m_icon );
    gdk_window_set_cursor( m_icon->window, cursor );
    
    // This is done here because the Registry singleton does not exist
    // when the main window gets created.
    unsigned long dwPosX, dwPosY;
	if( (IZkRegistry::GetDWORD(MAINDLG_RECT_LEFT_KEY, dwPosX)) &&
        (IZkRegistry::GetDWORD(MAINDLG_RECT_TOP_KEY, dwPosY)) )
    {
        gtk_widget_set_uposition( m_window, dwPosX, dwPosY );
    }

    bool consoleWasUp = IZkRegistry::GetFlag( CONSOLE_VISIBLE_KEY );
    if( consoleWasUp )
    {
        m_console.Show();
        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_info ),
            true );
        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_info_ext ),
            true );
    }
    
    bool extended = IZkRegistry::GetFlag( MAINDLG_EXTENDED_KEY );
    if( extended )
    {
        ExtendCb( NULL, this );
    }

    SetTopPref();    
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::Finalize() 
{
    // We have to do the work here, otherwise the lifescope of the registry 
    // singleton may end before the main window lifescope.
	IZkRegistry::SetDWORD(MAINDLG_RECT_LEFT_KEY, m_window_x);
	IZkRegistry::SetDWORD(MAINDLG_RECT_TOP_KEY, m_window_y);
	IZkRegistry::SetFlag(MAINDLG_EXTENDED_KEY, m_bExtended );
        IZkRegistry::SetFlag(CONSOLE_VISIBLE_KEY, 
           gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( m_info )));
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::Show()
{
    UiThreadGuard beefeater;
    gtk_widget_show( GTK_WIDGET( m_window ) );
    gtk_key_snooper_install( KeyCb , this );                        
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::Hide() 
{
    UiThreadGuard thor;
    gtk_widget_hide( GTK_WIDGET( m_window ) );
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::LoadPixmaps()
{
    // we load the image only one time
    if( m_lockPixmap == 0 )
    {
        // Load the lock icon for the drop down menu
        ZKSTD::string fPath = ZKFileConfiguration::GetFreedomPixmapDir();
        fPath += "lock.xpm";

        GdkColormap *colormap = gdk_colormap_get_system();

        // white
        GdkColor transColor;
        transColor.red = 0xffff;
        transColor.green = 0xffff;
        transColor.blue = 0xffff;
        
        m_lockPixmap = gdk_pixmap_colormap_create_from_xpm( NULL,
                                                            colormap,  
                                                            &m_lockBitmapMask,
                                                            &transColor,
                                                            fPath.c_str() );
    }
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::SetTopPref()
{
   ZkGtkUtils::GetAndSetTopPref( GTK_WIDGET( m_window ));
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::OpenPreferences( ZKSTD::string & pageName ) 
{

    UiPrefDlg::get().Open( pageName );    
    InitializeNymList();
    SetTopPref();

    // saving the file even though the config may not have
    // been changed - there is no "dirty logic" on the linux
    // client yet. -- bug 6768.
    try {
       ZkService::GetIClientPersist()->SaveClientState();
    }
    catch(IZkException& ex){
       ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
    }
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::ConfigureCb( 
    GtkWidget*, 
    GdkEventConfigure*,
    UiMainWindowImp* inst 
)
{
    gdk_window_get_deskrelative_origin( inst->m_window->window, 
                                        &(inst->m_window_x), &(inst->m_window_y) );
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::IconClickedCb( 
    GtkWidget*, 
    GdkEventKey *,
    UiMainWindowImp* inst 
)
{
    ZKSTD::string nymName;
	INym * pNym = NULL;

    ZkService::GetINymCentral()->GetActiveNym( IID_INYM, (void**)&pNym );
    pNym->GetNymName( nymName );
    
    ZKSTD::string pageName( "nym-" );
    pageName += nymName;

    if( nymName == "User" )
    {
        ZKSTD::string strNymEmail; 
        ZkStringMgr::Load( strNymEmail, IDS_NYMMAIL_TITLE );
        pageName += strNymEmail;
    }

    inst->OpenPreferences( pageName );    
}

//---------------------------------------------------------------------------
//
gint UiMainWindowImp::KeyCb( 
    GtkWidget *widget,
    GdkEventKey *event,
    UiMainWindowImp* inst
)
{
#warning This is all test code and doesnt belong here.
    // FIXME - all events are of type PRESS !?! , so open up the dialog only
    //         on a count of 1.
    static int counter_fix = 0;

    if( (widget == inst->m_window) && 
        (event->type == GDK_KEY_PRESS) ){

        counter_fix++;
        if( counter_fix < 2 ) {
            return FALSE;
        }
        
        counter_fix = 0;

        switch( ((GdkEventKey*)event)->keyval )
        {
#ifdef INTERNAL_BUILD
            case GDK_a:
                ZkStatusSubject::UpdateStatus( ZK_STATUS_NETWORK_BUSY, true, true );
                break;

            case GDK_A:
                ZkStatusSubject::UpdateStatus(ZK_STATUS_NETWORK_BUSY, false, true);
                break;

            case GDK_l:
                {
                    UiLightTest lightTest;
                    lightTest.StartThread();
                }
                break;

            case GDK_m:
                {
                    ZKSTD::vector< ZKSTD::string > vecAddresses ;
                    vecAddresses.push_back( "Win32@zks.net" ) ;
                    vecAddresses.push_back( "differ@zks.net" ) ;
                    vecAddresses.push_back( "Windows@zks.net" ) ;
                    vecAddresses.push_back( "in several@zks.net" ) ;
                    vecAddresses.push_back( "uniform@zks.net" ) ;
                    vecAddresses.push_back( "ways@zks.net" ) ;
                    vecAddresses.push_back( "API@zks.net" ) ;
                    vecAddresses.push_back( "Pion@zks.net" ) ;
                    vecAddresses.push_back( "Eric@zks.net" ) ;
                    vecAddresses.push_back( "Marty@zks.net" ) ; 
                    ZkProxyDialogs::LaunchMailSentCountDlg( 999999999,
                                                            & vecAddresses);
                }
                break;

            case GDK_n:
            case GDK_o:
                {
                    ZKSTD::vector< ZKSTD::string > vecAddresses ;
                    vecAddresses.push_back( "Win32@zks.net" ) ;
                    vecAddresses.push_back( "differ@zks.net" ) ;
                    vecAddresses.push_back( "Windows@zks.net" ) ;
                    vecAddresses.push_back( "in several@zks.net" ) ;
                    vecAddresses.push_back( "uniform@zks.net" ) ;
                    vecAddresses.push_back( "ways@zks.net" ) ;
                    vecAddresses.push_back( "API@zks.net" ) ;
                    vecAddresses.push_back( "Pion@zks.net" ) ;
                    vecAddresses.push_back( "Eric@zks.net" ) ;
                    vecAddresses.push_back( "Marty@zks.net" ) ; 
                    
                    ZkProxyDialogs::LaunchOutgoingMsgDlg( &vecAddresses, 
                                                          ((GdkEventKey*)event)->keyval == GDK_n );
                }
                break;

            case GDK_p:
                for( int i = 0; i < 2000; i++ )
                {
                    IZkConsoleMsg::ShowMsg( IDS_FREEDOM );
                }
                break;

            case GDK_q:
                ZKTRACE0( "Light Status:\n" );
                ZKTRACE1( "  Mail  : %d\n", inst->m_status[0]->GetState() );
                ZKTRACE1( "  Web   : %d\n", inst->m_status[1]->GetState() );
                ZKTRACE1( "  SSL   : %d\n", inst->m_status[2]->GetState() );
                ZKTRACE1( "  News  : %d\n", inst->m_status[3]->GetState() );
                ZKTRACE1( "  IRC   : %d\n", inst->m_status[4]->GetState() );
                ZKTRACE1( "  Telnet: %d\n", inst->m_status[5]->GetState() );
                break;

            case GDK_r:
                inst->ReselectUserNym();
                break;

            case GDK_s:
                ZkProxyDialogs::LaunchWarnSignedMailDlg();
                break;

            case GDK_u:
                {
                    UiNIQSUpdate niqsUpdate;
                    niqsUpdate.StartThread();
                }
                break;

            case GDK_w:
                {
                    WordScanSummary input ;

                    char msg[] = "Test lifted from ZKDialogsUnitTest. Should probably be moved there eventually." ;

                    ZKSTD::vector< ZKSTD::string > vecFound ;
                    vecFound.push_back( "Win32" ) ;
                    vecFound.push_back( "differ" ) ;
                    vecFound.push_back( "Windows" ) ;
                    vecFound.push_back( "in several" ) ;
                    vecFound.push_back( "uniform" ) ;
                    vecFound.push_back( "ways" ) ;
                    vecFound.push_back( "API" ) ;
                    vecFound.push_back( "Andrew" ) ;
                    vecFound.push_back( "Eric" ) ;
                    vecFound.push_back( "Marty" ) ;
                    vecFound.push_back( "Whoever else" ) ;
                    vecFound.push_back( "Is the scroll bar there yet ?" ) ;
                    
                    input.pScannedBuffer = (unsigned char *) msg ;						  
                    input.cntBuffer = strlen( msg ) ;
                    input.pFoundWords = &vecFound ;
                    input.strDomainName = "www.zeroknowledge.com" ;
                    
                    ZkProxyDialogs::LaunchWordScanMsgDlg( PROTOCOL_IRC, &input ) ;
                    break;
                }

#endif // INTERNAL_BUILD
            default:
                break;
        }
    }

    return FALSE;
}


//---------------------------------------------------------------------------
//
void UiMainWindowImp::HideCb( GtkWidget*, UiMainWindowImp* inst )
{
    // The Console is getting hidden, make sure we reset our toggle button.
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( inst->m_info ), FALSE ) ;
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( inst->m_info_ext ), FALSE ) ;
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::CloseCb( GtkWidget*, UiMainWindowImp* )
{
    gtk_main_quit();
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::PrefsCb( GtkWidget*, UiMainWindowImp* inst )
{
    // Re-opends to the last active page.
    ZKSTD::string empty;
    inst->OpenPreferences( empty );    
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::ExtendCb( GtkWidget*, UiMainWindowImp* inst )
{
    gtk_container_remove( GTK_CONTAINER( inst->m_window ), inst->m_layout );
    gtk_container_add( GTK_CONTAINER( inst->m_window ), inst->m_layout_ext );

    // Reparent the icon    
    gtk_container_remove( GTK_CONTAINER( inst->m_ebox ), inst->m_icon );
    gtk_container_add( GTK_CONTAINER( inst->m_ebox_ext ), inst->m_icon );

    // set the cursor over the icon    
    GdkCursor * cursor = gdk_cursor_new( GDK_HAND2 );
    gtk_widget_realize( inst->m_icon );
    gdk_window_set_cursor( inst->m_icon->window, cursor );

    // Reparent the nym list..
    gtk_container_remove( GTK_CONTAINER( inst->m_nym_ct ),  inst->m_nyms );
    gtk_container_add( GTK_CONTAINER( inst->m_nym_ct_ext ), inst->m_nyms );

    inst->m_bExtended = true;
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::CollapseCb( GtkWidget*, UiMainWindowImp* inst )
{
    gtk_container_remove( GTK_CONTAINER( inst->m_window ), inst->m_layout_ext );
    gtk_container_add( GTK_CONTAINER( inst->m_window ), inst->m_layout );

    // Reparent the icon
    gtk_container_remove( GTK_CONTAINER( inst->m_ebox_ext ), inst->m_icon );
    gtk_container_add( GTK_CONTAINER( inst->m_ebox ), inst->m_icon );

    // set the cursor over the icon    
    GdkCursor * cursor = gdk_cursor_new( GDK_HAND2 );
    gtk_widget_realize( inst->m_icon );
    gdk_window_set_cursor( inst->m_icon->window, cursor );
    
    // Reparent the nym list..
    gtk_container_remove( GTK_CONTAINER( inst->m_nym_ct_ext ),  inst->m_nyms );
    gtk_container_add( GTK_CONTAINER( inst->m_nym_ct ), inst->m_nyms );

    inst->m_bExtended = false;
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::MinimizeCb( GtkWidget*, UiMainWindowImp* inst )
{
    Window xwindow = GDK_WINDOW_XWINDOW( inst->m_window->window );
    XIconifyWindow( GDK_DISPLAY(), xwindow, DefaultScreen( GDK_DISPLAY() ));
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::ConsoleCb( GtkWidget* w, UiMainWindowImp* inst )
{
    if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( w ) ) ) 
    {
        inst->m_console.Show();
        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( inst->m_info ), TRUE ) ;
        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( inst->m_info_ext ), TRUE ) ;
    }
    else
    {
        inst->m_console.Hide();
        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( inst->m_info ), FALSE ) ;
        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( inst->m_info_ext ), FALSE ) ;
    }
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::HelpCb( GtkWidget*, UiMainWindowImp* )
{
    // FIXME - get a define for this index file name.
    ZKSTD::string helpFile( ZKFileConfiguration::GetFreedomHelpDir() );
    helpFile += "html/index.htm";

    gnome_url_show( helpFile.c_str() );

}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::MailCb( GtkWidget*, UiMainWindowImp* inst )
{
    ZKSTD::string nymName;
    ZKSTD::string pageName;
    ZKSTD::string strNymEmail; 
    ZkStringMgr::Load( strNymEmail, IDS_NYMMAIL_TITLE );
	INym * pNym = NULL;

    // Fix for bug 7400 - when User nym is selected, brung us to the main
    //                    Mail page, not the User nym main page.
    if( ZkService::GetINymCentral()->IsCurrentNymUser() )
    {
        pageName = "freedommail";
    }
    else
    {
        ZkService::GetINymCentral()->GetActiveNym( IID_INYM, (void**)&pNym );
        pNym->GetNymName( nymName );
    
        pageName = "nym-" + nymName + strNymEmail;
    }

    inst->OpenPreferences( pageName );    
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::CookieCb( GtkWidget*, UiMainWindowImp* inst )
{
    ZKSTD::string nymName;
    ZKSTD::string strCookieJar;
    ZkStringMgr::Load( strCookieJar, IDS_COOKIEJAR_TITLE );
	INym * pNym = NULL;

    ZkService::GetINymCentral()->GetActiveNym( IID_INYM, (void**)&pNym );
    pNym->GetNymName( nymName );
    
    ZKSTD::string pageName;
    pageName = "nym-" + nymName + strCookieJar;

    inst->OpenPreferences( pageName );    
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::AdBlockCb( GtkWidget*, UiMainWindowImp* inst )
{
    ZKSTD::string nymName;
	INym * pNym = NULL;

    ZkService::GetINymCentral()->GetActiveNym( IID_INYM, (void**)&pNym );
    pNym->GetNymName( nymName );
    
    ZKSTD::string pageName( "ad" );

    inst->OpenPreferences( pageName );    
}

//---------------------------------------------------------------------------
//
void  UiMainWindowImp::TitlebarPressCb( 
    GtkWidget* widget,
    GdkEventButton*,
    UiMainWindowImp* inst
)
{    
    gdk_window_raise( inst->m_window->window );
	gdk_window_get_pointer(  NULL, 
                             &(inst->m_offset_x), 
                             &(inst->m_offset_y), 
                             NULL );
    inst->m_dragging = true;

    gdk_pointer_grab( widget->window, TRUE, 
		     GDK_BUTTON_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, 
		     0, inst->m_dragCursor, GDK_CURRENT_TIME);

}


//---------------------------------------------------------------------------
//
void UiMainWindowImp::TitlebarMoveCb( 
    GtkWidget*,
    GdkEventMotion*,
    UiMainWindowImp* inst
)
{
	XSync( GDK_DISPLAY(), False );

    if( inst->m_dragging )
    {
        gint x,y; 
        
        gdk_window_get_pointer(  NULL, 
                                 &x, 
                                 &y, 
                                 NULL );

        gint move_x = x - inst->m_offset_x;
        gint move_y = y - inst->m_offset_y;

        inst->m_window_x +=  move_x;
        inst->m_window_y +=  move_y;
        gtk_widget_set_uposition( inst->m_window, 
                                  inst->m_window_x, 
                                  inst->m_window_y);

        

        inst->m_offset_x = x;
        inst->m_offset_y = y;

        gdk_flush();
    }
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::TitlebarReleaseCb( 
    GtkWidget*,
    GdkEventButton*,
    UiMainWindowImp* inst
)
{
    gint clk_x, clk_y;
	gdk_window_get_pointer(  inst->m_window->window, &clk_x, &clk_y, NULL );

    inst->m_dragging = false;

	gdk_pointer_ungrab( GDK_CURRENT_TIME );
}

//---------------------------------------------------------------------------
//
gint UiMainWindowImp::DeleteCb( GtkWidget*, UiMainWindowImp* /*inst*/ )
{
// Whether or not the user should confirm exit remains to be seen. (see Cooper)
// For now being asked all the time is just too annoying.
#if 0 
    GtkWidget* dialog = gnome_ok_cancel_dialog_modal_parented(
        "You are about to put your privacy at risk. \nContinue ?",
        0, 0, GTK_WINDOW( inst->m_window )
    );

    int choice = gnome_dialog_run_and_close( GNOME_DIALOG( dialog ) );

    if( !choice ) {
        gtk_main_quit();
    }

    return choice;
#endif

    gtk_main_quit();
    return FALSE;
}


//---------------------------------------------------------------------------
//
void UiMainWindowImp::NymSelectCb( GtkWidget* item, UiMainWindowImp* inst )
{
    // FIXME - make sure we are initialized.
	ZKSTD::string strNymName;

    /* The index may actually have been set through a hot key, in which
       case gtk doesn't reset the position of the menu. 
       this is a fix for bug 7047 */
    GtkWidget *nymMenu = gtk_option_menu_get_menu(GTK_OPTION_MENU(inst->m_nyms));
    GtkWidget* currentW = gtk_menu_get_active( GTK_MENU( nymMenu ) );
    
    if( item != currentW )
    {
        int menu_index  = (int)gtk_object_get_data( GTK_OBJECT( item ), MENU_INDEX );

        gtk_menu_set_active( GTK_MENU( nymMenu ), menu_index );

        // FIXME - remove this if gtk fixes the menu update problem.
        //         we do this mainly to force the menu to redraw and show the 
        //         new selection.
        gtk_widget_ref( nymMenu );
        gtk_option_menu_remove_menu( GTK_OPTION_MENU( inst->m_nyms ) );
        gtk_option_menu_set_menu( GTK_OPTION_MENU( inst->m_nyms ), nymMenu );
        gtk_widget_unref( nymMenu );
    }
    // end fix 7047

    // Extract the nym index from the item.
    int index  = (int)gtk_object_get_data( GTK_OBJECT( item ), NYM_INDEX );

    switch( index )
    {
        case USER_NYM_INDEX:
            inst->SetActiveNym( index );
            break;

        case MW_NEW_NYM:
            {
                WzSetupDlg nymWizard( true );
                nymWizard.Run();
                inst->InitializeNymList();
            }
            break;
            
        default:
            inst->SetActiveNym( index );
            break;
    }

}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::InitializeNymList()
{
	INym * pNym = NULL;

    // Get the index rightaway so we can set it in the menu as we fill it.
	size_t nActiveNym = ZkService::GetINymCentral()->GetActiveNymIndex();


    // We need thread protection here --ol
    // (cerebus was the dog that guarded hades ... bored)
    // Sure ol' , whatever. -eb
    { // lifescope for the guard, so we let it go before calling SetActiveNym
    UiThreadGuard cerebus;

    // First get rid of everything in this menu.
    gtk_option_menu_remove_menu( GTK_OPTION_MENU( m_nyms ) );
    
    // Now get a new menu, fill it, and put it back.
    GtkWidget* newMenu    = gtk_menu_new();

	int cntNyms = ZkService::GetINymCentral()->GetNumberOfNyms() ;

	ZKSTD::string strNymName;

    // First item in the list is the "no selection" option.
    ZKSTD::string strNoNym = ZkService::GetINymCentral()->GetNoNymSelectedDisplayName().c_str();
    gtk_menu_append( GTK_MENU( newMenu ), GetMenuItem( strNoNym, USER_NYM_INDEX, 0 ));
                                     
    unsigned long dwLifeStage = NYM_LIFE_STAGE_UNDEFINED ;

    // just to make sure the for loop is always valid.
    ZKASSERT( USER_NYM_INDEX == 0); 

    // Count the items inserted in the menu so we can set the index of
    // the active nym related to the menu entries. (they do not necessarily
    // correspond to nym indices if some nyms are inactive or deleted).
    int itemCnt = 0;
    int activeItem = 0;

    for( int i = 1; i < cntNyms; i++ )
    {					 
		ZkService::GetINymCentral()->GetNym( IID_INYM, (void**)&pNym, i );

        pNym->GetNymName( strNymName );
        pNym->GetYourLifeStage( dwLifeStage ) ;
	
        if ( dwLifeStage == NYM_LIFE_STAGE_ACTIVE ) 
        {
            ++itemCnt;
            if( i == static_cast< int >( nActiveNym ) ) 
            {
                activeItem = itemCnt;
            }

            bool bShowLockIcon = pNym->DoesNymUseMail() && pNym->DoesNymUseCloud();
            gtk_menu_append( GTK_MENU( newMenu ), GetMenuItem( strNymName, i, itemCnt, bShowLockIcon ));
        }			
    }

	// Add create nym as an extra option
    ZKSTD::string text;
    ZkStringMgr::Load( text, UIMSG_CREATE_NYM_ENTRY );

    gtk_menu_append( GTK_MENU( newMenu ), GetMenuItem( text, MW_NEW_NYM, itemCnt+1 ));
                     
    gtk_menu_set_active( GTK_MENU( newMenu ), activeItem );
    gtk_option_menu_set_menu( GTK_OPTION_MENU( m_nyms ), newMenu );

    } // thread guard lifescope.

	// Select the active nym
    // this cannot be done at this stage because the networking is not ye
    // initialized.
//	SetActiveNym( nActiveNym );
    UpdateNymIcon();
}

//---------------------------------------------------------------------------
//
GtkWidget* UiMainWindowImp::GetMenuItem( 
    const ZKSTD::string& label, 
    int nym_index,
    int menu_index,
    bool in_bShowLockIcon)
{
    GtkWidget *hbox;
    GtkWidget *item;
    GtkWidget *accel_label;

    // Create the Menu Item
    item = gtk_menu_item_new ();
    
    // Create the box contaning the pixmap and Label
    hbox = gtk_hbox_new (FALSE, 3);
    gtk_container_add (GTK_CONTAINER (item), hbox);
    gtk_widget_show (hbox);
    
    // Create the Label and Attach it
    accel_label = gtk_accel_label_new (label.c_str());
    gtk_misc_set_alignment (GTK_MISC (accel_label), 0.0, 0.5);
    gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (accel_label), item);
    gtk_widget_show (accel_label);
    gtk_box_pack_start (GTK_BOX (hbox), accel_label, TRUE, TRUE, 0);

    if( in_bShowLockIcon )
    {
        ZKASSERT( m_lockPixmap && m_lockBitmapMask);
        if( m_lockPixmap && m_lockBitmapMask )
        {
             GtkWidget* pixmap = gtk_pixmap_new (m_lockPixmap,m_lockBitmapMask);
             gtk_widget_show (pixmap);
             gtk_box_pack_start (GTK_BOX (hbox), pixmap, FALSE, FALSE, 0);
        }
    }

    // Set the given index as the item data.
    gtk_object_set_data( GTK_OBJECT( item ), NYM_INDEX, (void*)nym_index ); 

    // Also set the widget's index in the popdown menu.
    // This can only work if we rebuild the entire list everytime/
    gtk_object_set_data( GTK_OBJECT( item ), MENU_INDEX, (void*)menu_index ); 

    // Connect the "select" callback.
    gtk_signal_connect( GTK_OBJECT( item ), 
                        "activate",
                        GTK_SIGNAL_FUNC( NymSelectCb ), 
                        this );

    // show element
    gtk_widget_show( item );    
    
    return item;
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::SetDisabled( bool disabled )
{
    UiThreadGuard sentinel;

    ZKASSERT(m_window != NULL);
    gtk_widget_set_sensitive( m_ebox, !disabled );
    gtk_widget_set_sensitive( m_nyms, !disabled );
    gtk_widget_set_sensitive( m_prefs, !disabled );
    gtk_widget_set_sensitive( m_mail, !disabled );    
    gtk_widget_set_sensitive( m_cookie, !disabled );  
    gtk_widget_set_sensitive( m_adblock, !disabled ); 
    gtk_widget_set_sensitive( m_ebox_ext, !disabled );    
    gtk_widget_set_sensitive( m_mail_ext, !disabled );    
    gtk_widget_set_sensitive( m_cookie_ext, !disabled );  
    gtk_widget_set_sensitive( m_adblock_ext, !disabled ); 
    gtk_widget_set_sensitive( m_prefs_ext, !disabled ); 

    // Force update of the option menu 
    GtkWidget *nymMenu = gtk_option_menu_get_menu(GTK_OPTION_MENU(m_nyms));
    gtk_widget_set_sensitive( nymMenu, !disabled );
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::ReselectUserNym()
{
    
    {
        UiThreadGuard sentinel;
    
        GtkWidget *nymMenu = gtk_option_menu_get_menu(GTK_OPTION_MENU(m_nyms));
        gtk_menu_set_active(GTK_MENU(nymMenu), USER_NYM_INDEX);
        
        // FIXME - force update of the option menu (leave out for now)
        gtk_widget_ref( nymMenu );
        gtk_option_menu_remove_menu( GTK_OPTION_MENU( m_nyms ) );
        gtk_option_menu_set_menu( GTK_OPTION_MENU( m_nyms ), nymMenu );
        gtk_widget_unref( nymMenu );
    }
    // Could be improved... this is a recursive call, since ReselectUserNym is 
    // called from SetActiveNym
    SetActiveNym( USER_NYM_INDEX );
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::SetActiveNym( int iNymSelected )
{
    ZKTRACE2( "SetActiveNym to %d, currently %d\n", 
              iNymSelected, ZkService::GetINymCentral()->GetActiveNymIndex() );

    if( iNymSelected == ZkService::GetINymCentral()->GetActiveNymIndex() )
    {
        return;
    }

    // Switch the Active Nym to the selection
    ZKVERIFY( ZkService::GetINymCentral()->SetActiveNym( iNymSelected ) );

	// Only one authenticated route at a time!	
    ZkService::GetIRouteManager()->KillAuthenticatedRoute(); 

    if ( iNymSelected == USER_NYM_INDEX ) // 'user' nym selected
    {
        IZkConsoleMsg::ShowMsgForceUpdate( IDS_CONS_USER_NYM_ACTIVE ) ;
    }
    else 
    {
        bool bBuiltRoute = false ;
        bool bFreeNym = false;

        try 
        {
            INym* pNym = 0;
            ZkService::GetINymCentral()->GetNym( IID_INYM, (void**)&pNym, iNymSelected);
        
            bool bAccess = ZkService::GetINymCentral()->DoesNymUseCloud(iNymSelected);
        
            if (!bAccess)
            {
                // do not create a route for a free nym
                bFreeNym = true;
            }
            else 
            {
                UiStartAuthRoute authenticate( iNymSelected );
                authenticate.StartThread();
                bBuiltRoute = authenticate.IsRouteBuilt();
            }
        }
        catch (IZkException& ex)
        {
            ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
            bBuiltRoute = false;
        }


        if( !bBuiltRoute && !bFreeNym ) // route not built for some reason
        {
            // This causes a recursive call in SetActiveNym.
            ReselectUserNym();
            return;
        }
        else
        {
            // Show the messagr to the console
            INym* pNym = 0;
            ZKSTD::string strName;
            ZkService::GetINymCentral()->GetActiveNym( IID_INYM, (void**)&pNym );
            ZKVERIFY( pNym->GetNymName( strName ) );
            
            IZkConsoleMsg::ShowMsgForceUpdate( CONS_IMPORTANT, 
                                               CONSIMAGE_NORMAL, 
                                               IDS_CONS_NYMACTIVE_STATUS, 
                                               strName.c_str() ) ;
        }
    }

    UpdateNymIcon();
}

//---------------------------------------------------------------------------
//
void UiMainWindowImp::UpdateNymIcon( )
{
    UiThreadGuard sentinel;

    INym* pNym = 0;
    ZkService::GetINymCentral()->GetActiveNym( IID_INYM, (void**)&pNym );

    const char *filename = NULL;

    if( pNym ) 
    {
        WNymIconInfo info ;

        pNym->GetNymIconInfo( info ) ;
        // GetFilePath asserts that IconType != NymIconTypeUndefined
        if( info.GetIconType() != NymIconTypeUndefined ) 
        {
            filename = info.GetFilePath().c_str();
        }
    }
    
    ZKTRACE1( "Updating Nym icon to : %s\n", filename );
    m_icondisplay->UpdateIcon( filename );
}

