/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UIMAINWINDOWIMP
#define ZKS_INCL_UIMAINWINDOWIMP

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#ifndef ZKS_INCL_UICONSOLEWINDOW
#include <UiConsoleWindow.h>
#endif

#include <gtk/gtk.h>


/*----- forward declaration ------*/

class UiStatusDisplay;
class UiIconDisplay;
class INym;

/*===========================================================================
 * CLASS UiMainWindowImp
 *=========================================================================*/

class UiMainWindowImp 
{

public: 

    /*----- member functions -----*/
      
    UiMainWindowImp();
    ~UiMainWindowImp();    

    void Initialize();
    void Finalize();

    void Show();
    void Hide();
    void SetDisabled(bool disabled);

    void InitializeNymList();

    // This is called when a route goes down, or in other circumstances
    // in which we need freedom to revert back to the user nym
    void ReselectUserNym();

private: 

    // Load the pixmaps use in the drop down    
    void LoadPixmaps();
        
    void SetTopPref();
    void OpenPreferences( ZKSTD::string & );

    /*----- static member functions (callbacks) -----*/

    // Will have to come up with better way of opening the dialog box here.
    // For now we click on the Icon.
    static void IconClickedCb( GtkWidget*, GdkEventKey *, UiMainWindowImp* );
    static void ConsoleCb( GtkWidget*, UiMainWindowImp* );
    static void HideCb( GtkWidget*, UiMainWindowImp* );
    static gint DeleteCb( GtkWidget*, UiMainWindowImp* );
    static void NymSelectCb( GtkWidget*, UiMainWindowImp* );
    static gint UiMainWindowImp::KeyCb( GtkWidget *, GdkEventKey *, UiMainWindowImp* );
    static void ExtendCb( GtkWidget*, UiMainWindowImp* );
    static void CollapseCb( GtkWidget*, UiMainWindowImp* );
    static void MinimizeCb( GtkWidget*, UiMainWindowImp* );
    static void CloseCb( GtkWidget*, UiMainWindowImp* );
    static void PrefsCb( GtkWidget*, UiMainWindowImp* );

    static void HelpCb( GtkWidget*, UiMainWindowImp* );
    static void MailCb( GtkWidget*, UiMainWindowImp* );
    static void CookieCb( GtkWidget*, UiMainWindowImp* );
    static void AdBlockCb( GtkWidget*, UiMainWindowImp* );

    static void ConfigureCb( GtkWidget *widget,
                             GdkEventConfigure *event,
                             UiMainWindowImp* );
    static void TitlebarPressCb( GtkWidget *widget,
                                 GdkEventButton *event,
                                 UiMainWindowImp* );
    static void TitlebarReleaseCb( GtkWidget *widget,
                                   GdkEventButton *event,
                                   UiMainWindowImp* );
    static void TitlebarMoveCb( GtkWidget *widget,
                               GdkEventMotion *event,
                               UiMainWindowImp* inst );

    /*----- member functions -----*/

    void SetActiveNym( int iNymSelected );
    void UpdateNymIcon( );
        
    GtkWidget* GetMenuItem (
        const ZKSTD::string& name,
        int nym_index,
        int menu_index,
        bool in_bShowLockIcon = false );

    /*----- data members -----*/

    GtkWidget* m_window;
    GtkWidget* m_info;
    GtkWidget* m_prefs;
    GtkWidget* m_nym_ct;
    GtkWidget* m_nyms;
    GtkWidget* m_ebox;
    GtkWidget* m_icon;
    GtkWidget* m_layout;
    GtkWidget* m_extend;
    GtkWidget* m_collapse;
    GtkWidget* m_titlebar;
    GtkWidget* m_minimize;
    GtkWidget* m_close;
    GtkWidget* m_mail;
    GtkWidget* m_help;
    GtkWidget* m_cookie;
    GtkWidget* m_adblock;

    GtkWidget* m_layout_ext;
    GtkWidget* m_info_ext;
    GtkWidget* m_prefs_ext;
    GtkWidget* m_nym_ct_ext;
    GtkWidget* m_ebox_ext;
    GtkWidget* m_titlebar_ext;
    GtkWidget* m_minimize_ext;
    GtkWidget* m_close_ext;
    GtkWidget* m_help_ext;
    GtkWidget* m_mail_ext;
    GtkWidget* m_cookie_ext;
    GtkWidget* m_adblock_ext;

    GdkPixmap* m_lockPixmap;
    GdkBitmap* m_lockBitmapMask;

    GdkCursor* m_dragCursor;
    
    UiConsoleWindow m_console;

    UiStatusDisplay* m_status[6];
    UiIconDisplay*   m_icondisplay;

    bool m_bExtended;
    
    // members for window dragging.
    bool m_dragging;
    gint m_offset_x;
    gint m_offset_y;

    gint m_window_x;
    gint m_window_y;
};

#endif
