// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiMsgDisplay.h>
#include <IZkConsoleMsg.h>
#include <ZkConsoleHint.h>
#include <UiThreadId.h>
#include <lockguard.h>

//===========================================================================
// CLASS UiMsgDisplay
//===========================================================================

//---------------------------------------------------------------------------
//
UiMsgDisplay::UiMsgDisplay( GtkLabel* w )
    : m_label( w )
{
    ZKASSERT( m_label != 0 );

    IZkConsoleMsg::get().connect( this );

    // Just make sure we keep our own reference to the widget while we exist,
    // otherwise we may have lifescope issues if the UI gets destroyed before
    // the message display.
    gtk_widget_ref( GTK_WIDGET( m_label ) );

    GtkStyle* wStyle = gtk_widget_get_style( GTK_WIDGET( m_label ) );
    GtkStyle* newStyle = gtk_style_copy( wStyle );

    GdkFont* newFt = gdk_font_load( "-*-helvetica-medium-r-normal-*-10-*-*-*-*-*-*-*" );
    newStyle->font = newFt;

    gtk_widget_set_style( GTK_WIDGET( m_label ), newStyle );

}

//---------------------------------------------------------------------------
//
UiMsgDisplay::~UiMsgDisplay( )
{
    gtk_widget_unref( GTK_WIDGET( m_label ) );
}

//---------------------------------------------------------------------------
//
void UiMsgDisplay::update( ZkHint* hint )
{
    // For mow just clear the previous message and put the new one.
    // Could eventually have a timeout on the displayed message.

    ZkConsoleHint* cHint = dynamic_cast< ZkConsoleHint* >( hint );

    // First acquire the lock on the data member.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    if( cHint && (cHint->getSeverity() != CONS_UI_NOT_CONSOLE) )
    {
        m_HintQueue.push( new ZkConsoleHint( *cHint ) );

        // Now if this is a background thread, defer the work otherwise 
        // do it rightaway. Ask not to wait on the UI thread.
        NotifyUi( false );
    }
}

//---------------------------------------------------------------------------
//
void UiMsgDisplay::UiThreadCallback()
{

    // FIXME - hate to do this dynamic_cast buziness on both sides of the 
    //         descriptor. Will have to generalize the copy constructor
    //         of ZkHint.

    // First acquire the lock on the data member.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    ZkHint* pHint = m_HintQueue.front();
    m_HintQueue.pop();

    if( pHint != 0 )
    {
        ZkConsoleHint* cHint = dynamic_cast< ZkConsoleHint* >( pHint );

        if( cHint != 0 ) 
        {
            gtk_label_set_text( m_label, cHint->getMessage().c_str() );
        }
        delete pHint;
    }
}



