/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UIMSGDISPLAY
#define ZKS_INCL_UIMSGDISPLAY

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#ifndef ZKS_INCL_UITHREADSAFEDLG
#include <UiThreadSafeDlg.h>
#endif

#ifndef ZKS_INCL_ZKOBSERVER
#include <ZkObserver.h>
#endif

#include <gnome.h>
#include <memory>
#include <queue>

class ZkHint;

/*===========================================================================
 * CLASS UiMsgDisplay
 *=========================================================================*/

/*
  This class handles the display of messages in the Main Window. It is simply an observer
  of the IZkConsoleMsg subject.
  It can be configured to display only certain types of messages.

  Being an observer of a ZkSingleton, and given the fact the connection is actually made inside 
  the constructor, this class requires the IZkConsoleMSg singleton to have been created prior
  to this object.
*/

class UiMsgDisplay 
    : public ZkObserver 
    , public UiThreadSafeDlg
{

public: 

    /*----- member functions -----*/

    UiMsgDisplay( GtkLabel* w );
    virtual ~UiMsgDisplay();
      
protected: 

    /*----- member functions -----*/

	// Prohibited funcs - not implemented.
	UiMsgDisplay( const UiMsgDisplay& );
	const UiMsgDisplay& operator=( const UiMsgDisplay& );

	// Purpose:  Called by a ZkSubject to send a hint notification.
    //           In this case the expected hint is of type ZkConsoleHint.
    virtual void update( ZkHint* );

    // FIXME - create a thread safe dialog and rename this function to 
    //         something more appropriate. This function services a descriptor
    //         bieng woken up on the ui thread side.
    virtual void UiThreadCallback();

private: 

    /*----- data members -----*/

    GtkLabel* m_label;
    ZKSTD::queue< ZkHint * > m_HintQueue;
    
};
#endif
