// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiStatusDisplay.h>
#include <ZkStatusSubject.h>
#include <ZKFileConfiguration.h>
#include <UiThreadId.h>

#include <clientdebug.h>
#include <client.h>
#include <string>

#define FREEDOM_LIGHT_ON "green_light.xpm"


//===========================================================================
// CLASS UiStatusDisplay
//===========================================================================


//---------------------------------------------------------------------------
//
UiStatusDisplay::UiStatusDisplay( GtkWidget* light, ZkStatusId status )
    : m_light( light )
    , m_status( status ) 
    , m_on( 0 )
{
    ZKASSERT( m_light != 0 );

    ZkStatusSubject::get().connect( this );
}

//---------------------------------------------------------------------------
//
UiStatusDisplay::~UiStatusDisplay( )
{
}

//---------------------------------------------------------------------------
//
void UiStatusDisplay::update( ZkHint* hint )
{

    ZkStatusHint* cHint = dynamic_cast< ZkStatusHint* >( hint );

    if( cHint != 0 ) 
    {
        if( cHint->GetStatusId() == m_status ) {
            UiThreadGuard guard;
            
            cHint->GetStatus() ? ++m_on : --m_on;
            if ( m_on < 0 ) m_on = 0;

            gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_light ), m_on != 0 );
        }
    }
}

//---------------------------------------------------------------------------
//
int UiStatusDisplay::GetState() const
{
    return m_on;
}
