/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UISTATUSDISPLAY
#define ZKS_INCL_UISTATUSDISPLAY

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include <ZkObserver.h>
#include <ZkStatusId.h>
#include <gnome.h>

/*===========================================================================
 * CLASS UiStatusDisplay
 *=========================================================================*/

/*
  This class takes care of observing the ZkStatusSubject and lighting some indicators
  acording to the current status.
  It takes care of reference counting status, so that a state getting set multiple times
  will require it to get unset the same amount of times in order to set the light off.
  (I guess I should rephrase that)

  Note that for now this is highly dependant on the bitmap representing the actual lights.

*/

class UiStatusDisplay : public ZkObserver  {

public: 

    /*----- member functions -----*/

    UiStatusDisplay( GtkWidget* w, ZkStatusId status );
    virtual ~UiStatusDisplay();

    int GetState() const;
      
protected: 

    /*----- member functions -----*/

	// Prohibited funcs - not implemented.
	UiStatusDisplay( const UiStatusDisplay& );
	const UiStatusDisplay& operator=( const UiStatusDisplay& );

	// Purpose:  Called by a ZkSubject to send a hint notification.
    //           In this case the expected hint is of type FIXME 
    virtual void update( ZkHint* );


private: 

    /*----- data members -----*/

    GtkWidget* m_light; // the light to light (...)
    ZkStatusId m_status;       // the status we are interested in.
    int        m_on;
};

#endif
