// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include <UiFreedomManager.h>
#include <ZKConfiguration.h>
#include <ZKFileConfiguration.h>

#include <sys/types.h> // for umask
#include <sys/stat.h>

#include <errno.h>
#include <client.h>

#include <gnome.h>
#include <glade/glade.h>

#include <iostream>

#include <unistd.h>

//---------------------------------------------------------------------------
//
int main(int argc, char *argv[]) 
{
    // Set a secure umask for all files created by freedom.
    // some core lib functins verify that files are umask 066
    // and will fail if the file is not secure, so changing this
    // may break some functions (see fileIsecure() ) --ol
    umask(066);

    // We need to give ZkFileConfiguration the contents of argv[0] such
    // that it can be retrieved later.
    ZKFileConfiguration::SetFreedomExeFilePath(argv[0]);
    g_thread_init( NULL );
    gnome_init("Freedom", ZKConfiguration::GetFreedomProductVersion().c_str(), argc, argv);
    glade_gnome_init();

    // Just make sure we dont produce stupid sounds when user clicks buttons...
    // bug 7040
    gnome_sound_shutdown();

    // It seems there is a bug in older version of gnome where the lock just isnt
    // released. Although it works fine on Helix, this is left there for
    // backward compatibility.
    GDK_THREADS_LEAVE();


    ZKSTD::string datadir = ZKFileConfiguration::GetFreedomPixmapDir();
    datadir += "freedomrc";
    gtk_rc_parse( datadir.c_str() );
    
    
    UiFreedomManager theManager;
    
    if ( ! theManager.Initialize() )
    {
        exit(-1);
    }
    
    theManager.Run();    
    theManager.Shutdown();

    return 0;    
}
