
// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom(R) code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

  


//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include "UiAssertReporter.h"

#include <UiThreadId.h>
#include <lockguard.h>
//#include <ZkGtkUtils.h>

#include <IZkException.h>
#include <../linux/RZKErrorReporter.h>

#include <gnome.h>
#include <strstream>
#include <string_hlp.h>

//===========================================================================
// CLASS UiAssertReporter
//===========================================================================

//---------------------------------------------------------------------------
//
UiAssertReporter::UiAssertReporter( )
{
}

//---------------------------------------------------------------------------
//
UiAssertReporter::~UiAssertReporter( )
{
}

//---------------------------------------------------------------------------
//
void UiAssertReporter::OnDisplayMsg( 
    const char* msg, 
    const char* fileName, 
    int   lineNumber 
)
{
    ZKSTD::ostrstream s;

    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );
   
    m_msg        = msg;
    m_fileName   = fileName;
    m_lineNumber = lineNumber;

    // Now if this is a background thread, defer the work otherwise 
    // open the dialog rightaway.
    NotifyUi();
}


//---------------------------------------------------------------------------
//
void UiAssertReporter::UiThreadCallback()
{
    const gchar * strButtons[] = 
       { "Abort (dump core)", "Ignore", NULL };

    ZKSTD::ostrstream ost;

    ost << "ASSERTION FAILED at: " << m_fileName << ":" << m_lineNumber
        << ZKSTD::endl << m_msg << '\0';

    ZKSTD::string dlgMsg( ost.str() );

    m_dialog = gnome_message_box_newv( dlgMsg.c_str(),
                                       GNOME_MESSAGE_BOX_WARNING, 
                                       strButtons );

    gtk_window_set_position( GTK_WINDOW( m_dialog ), GTK_WIN_POS_CENTER );

//       ZkGtkUtils::GtkWidgetShow( m_dialog );
    gtk_widget_show( m_dialog );
    int answer = gnome_dialog_run( GNOME_DIALOG( m_dialog ) );

    switch( answer ){
        case 0:
            abort();
            break;
  
        case 1:
        default:
            /* User closed dialog with window manager,
             * assume "ignore"
             */
            break;
    }

}
