// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiBackupBase.h>

#include <clientdebug.h>
#include <ZkErrorHandler.h>
#include <IZkPopup.h>
#include <IZKErrorReporter.h>
#include <IZkException.h>
#include <RZkException.h>
#include <INymCentral.h>
#include <IClientPersist.h>
#include <ZkService.h>
#include <ZKFileConfiguration.h>
#include <IZkFileHelp.h>
#include <ZkDialogsStrings.h>


#include <gnome.h>

//===========================================================================
// CLASS UiBackupBase
//===========================================================================

//---------------------------------------------------------------------------
//
bool UiBackupBase::DoBackup( GtkWidget* entry )
{
    // Get the file name.
    
    char *strBuf = gnome_file_entry_get_full_path( GNOME_FILE_ENTRY( entry ), FALSE );
    if( !strBuf )  {
		IZkPopup::MessageBox( IDS_NEED_DESTINATION, ZK_INFORMATION );
        ZKTRACE0("backup path Empty\n");
        return false;    
    }
    
    ZKSTD::string strDest( strBuf );

	try {
        
		// Make a complete save of our data if anything has changed
		// since we got on this dialog
        ZkService::GetIClientPersist()->SaveClientState();

		// Verify that the directory entered actually exists
		if ( !IZkFileHelp::DoesFileOrDirExist( strDest.c_str() ) )
		{
			throw RZkException( "UiPrefBackup::BackupCb", 
								"Backup can't create directories",
								IDS_BACKUP_DIR_NOT_EXIST ) ;
		}


		// Determine the file paths
		std::string strActiveFile; // File (with path) to backup
		ZkService::GetIClientPersist()->GetActiveFilePath( strActiveFile );

        ZKTRACE1( "Backup - active file path is : %s\n", strActiveFile.c_str() );
		ZKASSERT( !strActiveFile.empty() );

        // Make sure the path contains a trailing / so we can append the
        // backup file name.
		if( strDest[ strDest.length() - 1 ] != '/' ) {
			strDest += "/";
		}

		strDest += ZKFileConfiguration::GetFreedomDataBackupFileName().c_str() ;

        ZKTRACE1( "Backup - destination: %s\n", strDest.c_str() );
        
		// Warn user about any existing backup that will be destroyed
		if (IZkFileHelp::DoesFileOrDirExist( strDest.c_str() )) {

			if(! IZkPopup::QuestionBox( IDS_BACKUP_EXISTS_BLOW_AWAY,ZK_YESNO ))
            {			
				return false;
			}
		}

		// Perform the backup - file should be pretty small
		// we could do a progress bar if necessary		

		if( !IZkFileHelp::FileCopyWithVerification(strActiveFile.c_str(), strDest.c_str(), false) )
		{
            throw RZkException( "BackupPage::PerformBackup", 
                                "IZkFileHelp::FileCopyWithVerification() failed",
                                IDS_WRITE_CANT_COPY,
                                strActiveFile.c_str(),
                                strDest.c_str() ) ;
		}

		IZkPopup::MessageBox( IDS_BACKUP_SUCCESSFUL );
	}
	catch( IZkException& ex)
	{
 		ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
        return false;
	}

    return true;
}





