// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include "UiCheckBoxPopup.h"

#include <UiThreadId.h>
#include <ZkGladeLoader.h>
#include <ZkGtkUtils.h>
#include <lockguard.h>
#include <clientdebug.h>
#include <zkCStr.h>

#include <gnome.h>


//===========================================================================
// CLASS UiCheckBoxPopup.cpp
//===========================================================================

//---------------------------------------------------------------------------
//
UiCheckBoxPopup::UiCheckBoxPopup( )
    : m_bDoNotShowAgainIsChecked(false)
{
    // Call that a flok legend if you want, but it seems X really doesn't
    // appreciate top level windows being created in threads.
    // (which makes sense, given the fact X is not really thread safe)
    ZKASSERT( UiThreadId::IsUiThread() );
    
    ZkGladeLoader loader( "check_box_popup.xml", "CheckBoxPopup_window" );

    m_window = loader.GetWidget( "CheckBoxPopup_window" );
    m_check  = loader.GetWidget( "CheckBoxPopup_dontshow" );
    m_label = loader.GetWidget( "CheckBoxPopup_msg" );
    
    gtk_widget_hide( GTK_WIDGET( m_window ) );
}

//---------------------------------------------------------------------------
//
UiCheckBoxPopup::~UiCheckBoxPopup( )
{
}

//---------------------------------------------------------------------------
//
bool UiCheckBoxPopup::Run( const tstring& in_strMsg,
                           const tstring& in_strCheckMsgId,
                           bool	  in_bDefaultChecked, 
                           IconType  /*in_iconType*/ )
{
    // TO DO: add an icon to the dialog.

    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    m_bDoNotShowAgainIsChecked = in_bDefaultChecked;
    m_strLabel = in_strMsg;
    m_strDoNotShowAgainLabel = in_strCheckMsgId;

    // Now if this is a background thread, defer the work otherwise 
    // open the dialog rightaway.
    NotifyUi();

    return m_bDoNotShowAgainIsChecked;
}

//---------------------------------------------------------------------------
//
void UiCheckBoxPopup::UiThreadCallback()
{
    gtk_label_set_text( GTK_LABEL(m_label), (const gchar*) m_strLabel.c_str() );

    GtkBin* pToggleButton = GTK_BIN(m_check);

    if (pToggleButton)
    {
        gtk_label_set_text( GTK_LABEL(pToggleButton->child), 
                            (const gchar*) m_strDoNotShowAgainLabel.c_str() );
    }
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_check ), 
                                  m_bDoNotShowAgainIsChecked);

    ZkGtkUtils::GtkWidgetShowAll( m_window ); 
    gnome_dialog_run_and_close( GNOME_DIALOG( m_window ) );
    
    m_bDoNotShowAgainIsChecked = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( m_check ));
}


