// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiEnterSerialNumber.h>
#include <UiThreadId.h>
#include <ZkDialogsStrings.h>
#include <ZkGtkUtils.h>

#include <ZkGladeLoader.h>
#include <lockguard.h>

#include <IClientCentral.h>
#include <ITokenService.h>
#include <IZkPopup.h>
#include <ZkService.h>
#include <clientdebug.h>

#include <gnome.h>

#include <cstring>

//===========================================================================
// CLASS UiEnterSerialNumber.cpp
//===========================================================================

//---------------------------------------------------------------------------
//
UiEnterSerialNumber::UiEnterSerialNumber( )
    : m_answer( false )
{
    // Call that a flok legend if you want, but it seems X really doesn't
    // appreciate top level windows being created in threads.
    // (which makes sense, given the fact X is not really thread safe)
    ZKASSERT( UiThreadId::IsUiThread() );

    ZkGladeLoader loader( "enterSerialNumber.xml", "SerialNumber_window" );

    m_window = loader.GetWidget( "SerialNumber_window" );
    m_okButton  = loader.GetWidget( "SerialNumber_ok");    
    m_serialNumber  = loader.GetWidget( "serial_number");


    gtk_signal_connect(GTK_OBJECT(m_serialNumber), "changed",
                        GTK_SIGNAL_FUNC(UiEnterSerialNumber::SerialChangedCb),
                        this);
       
    gtk_widget_set_sensitive( m_okButton, false);

    gtk_widget_hide( GTK_WIDGET( m_window ) );
}

//---------------------------------------------------------------------------
//
UiEnterSerialNumber::~UiEnterSerialNumber( )
{
}

//---------------------------------------------------------------------------
//
void UiEnterSerialNumber::SerialChangedCb( GtkWidget *,
                                           UiEnterSerialNumber* in_inst )
{
    gchar* text =
        gtk_editable_get_chars( GTK_EDITABLE( in_inst->m_serialNumber ),
                                0, -1 );
    size_t len = strlen( text );

    gtk_widget_set_sensitive( in_inst->m_okButton,
                              ( len == ZKSERIAL_NUMBER_LENGTH ) );

    ZkGtkUtils::ToUpper( in_inst->m_serialNumber );
}

//---------------------------------------------------------------------------
//
bool UiEnterSerialNumber::Run( )
{
    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    // Now if this is a background thread, defer the work otherwise 
    // open the dialog rightaway.
    NotifyUi();

    return m_answer;
}

//---------------------------------------------------------------------------
//
void UiEnterSerialNumber::UiThreadCallback()
{
    ZkGtkUtils::GtkWidgetShowAll( m_window );
    int answer = gnome_dialog_run( GNOME_DIALOG( m_window ) );

    m_strSerialNumber = (char*)gtk_entry_get_text( GTK_ENTRY( m_serialNumber ) );
    
    m_answer = false;   // default value 
    
    switch( answer )
    {
        case 0: /* OK Message */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            m_answer = true;
            break;
            
        case 1: /* Cancel Message */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            break;
            
        case -1: /* ESC or wm close */
            break;                
    }  
}

//---------------------------------------------------------------------------
//
tstring UiEnterSerialNumber::GetSerialNumber() const
{
    return m_strSerialNumber;
}
