/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UIERRORREPORTER
#define ZKS_INCL_UIERRORREPORTER

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include <ZkAbstractReporter.h>
#include <UiThreadSafeDlg.h>

/*===========================================================================
 * CLASS UiErrorReporter
 *=========================================================================*/

/*
  This class is the mirror of the low level ZkErrorReporter.
  It simply observes the ZkErrorReporter (a ZkSubject under Linux) and
  popups up a dialog box whenever an error occurs.
  
  Main reason for this separation is that we could a UI less error reporter 
  that simply logs the errors to a file.
*/
  
class UiErrorReporter 
    : public ZkAbstractReporter
    , public UiThreadSafeDlg
{

public: 

    /*----- member functions -----*/

    UiErrorReporter();
    virtual ~UiErrorReporter();
      
	// Purpose:  The object creates a copy of itself and returns it.
    //           Allows the factory to create new instances from a prototype.
	// Promise:  Should return an exact copy of the concrete reporter.
    virtual ZkAbstractReporter* clone();

	// Purpose:  Called by RZKErrorReporter to have the error reported.
    //           It is up to the concrete reporter to handle the error.
	virtual void ShowError( const IZkException& in_except ) throw();

protected:

    /*----- member functions -----*/

    virtual void UiThreadCallback();

private:

    /*----- data members -----*/

    GtkWidget*    m_dialog;
    int           m_answer;  

    ZKSTD::string m_strCallerFunctionName;
    ZKSTD::string m_strDetail;
    ZKSTD::string m_strDescription;
    unsigned long m_errNum;

};
#endif
