/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UIPOPUP
#define ZKS_INCL_UIPOPUP

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include <../linux/RZkPopup.h>
#include "UiThreadSafeDlg.h"
#include "UiCheckBoxPopup.h"
#include "UiQuestionCheckBoxPopup.h"

/*===========================================================================
 * CLASS UiPopup
 *=========================================================================*/

class UiPopup 
    : public ZkPopupHandler 
    , public UiThreadSafeDlg
{

public:

    /*----- member functions -----*/

    UiPopup();
    virtual ~UiPopup();

	virtual void openMessageBox( 
        const tstring&, 
        const tstring&, 
        IconType );

	virtual bool openQuestionBox( 
        const tstring&, 
        BtnType, 
        IconType, 
        bool );

	virtual bool openMessageCheckBox(	const tstring&,
										const tstring& 	in_strCheckMsg,
										bool		in_bDefaultChecked, 
										IconType);

    virtual bool openQuestionCheckBox( const tstring& in_strMsg,
                                       bool&       out_bChecked,
                                       uint32_t    in_btn1textId,
                                       uint32_t    in_btn2textId,
                                       uint32_t    in_checkMsgId,    
                                       bool        in_bDefaultChecked,
                                       IconType    in_iconType );

	virtual tstring openRequestStringBox(	const tstring& in_strTitle,
                                            const tstring& in_strPrompt,
                                            const tstring& in_strInitialValue,
											int in_nMinLength,
                                            PopupPosition in_popupPos);

protected:

    /*----- member functions -----*/

    virtual void UiThreadCallback();

private:

    /*----- data members -----*/

    GtkWidget*    m_dialog;
    int           m_answer;

    ZKSTD::auto_ptr< UiCheckBoxPopup > m_messageCheckPopupDlg;
    ZKSTD::auto_ptr< UiQuestionCheckBoxPopup > m_questionCheckBoxPopupDlg;
};

#endif
