// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include "UiQuestionCheckBoxPopup.h"

#include <UiThreadId.h>
#include <ZkGladeLoader.h>
#include <ZkGtkUtils.h>
#include <lockguard.h>
#include <clientdebug.h>
#include <zkCStr.h>
#include <ZkCatalogRegistry.h>
#include <ZkStringMgr.h>

#include <gnome.h>


//===========================================================================
// CLASS UiQuestionCheckBoxPopup.cpp
//===========================================================================

//---------------------------------------------------------------------------
//
UiQuestionCheckBoxPopup::UiQuestionCheckBoxPopup( )
    :       m_answer(false),
            m_bDoNotShowAgainIsChecked(false)
{
    // Call that a flok legend if you want, but it seems X really doesn't
    // appreciate top level windows being created in threads.
    // (which makes sense, given the fact X is not really thread safe)
    ZKASSERT( UiThreadId::IsUiThread() );
    
    ZkGladeLoader loader( "question_check_box.xml", "QuestionCheckBoxPopup_window" );

    m_window = loader.GetWidget( "QuestionCheckBoxPopup_window" );
    m_check  = loader.GetWidget( "QuestionCheckBoxPopup_dontshow" );
    m_label = loader.GetWidget( "QuestionCheckBoxPopup_msg" );
    m_true = loader.GetWidget( "QuestionCheckBoxPopup_true" );
    m_false = loader.GetWidget( "QuestionCheckBoxPopup_false" );

    gtk_widget_hide( GTK_WIDGET( m_window ) );
}

//---------------------------------------------------------------------------
//
UiQuestionCheckBoxPopup::~UiQuestionCheckBoxPopup( )
{
}

//---------------------------------------------------------------------------
//
bool UiQuestionCheckBoxPopup::Run( const tstring& in_strMsg,
                                   bool&        out_bChecked,
                                   uint32_t     in_btn1textId,
                                   uint32_t     in_btn2textId,
                                   uint32_t     in_checkMsgId,
                                   bool         in_bDefaultChecked, 
                                   IconType     /*in_iconType*/ )
{
    // TO DO: add an icon to the dialog.(right now there's a warning about icontype)
    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    m_bDoNotShowAgainIsChecked = in_bDefaultChecked;
    m_strLabel = in_strMsg;
    m_strDoNotShowAgainLabel = ZkCatalogRegistry::GetString( in_checkMsgId );
    ZkStringMgr::Load( m_strTrueButtonLabel, in_btn1textId );
    ZkStringMgr::Load( m_strFalseButtonLabel, in_btn2textId );

    // Now if this is a background thread, defer the work otherwise 
    // open the dialog rightaway.
    NotifyUi();

    out_bChecked = m_bDoNotShowAgainIsChecked;
    return !m_answer;

}

//---------------------------------------------------------------------------
//
void UiQuestionCheckBoxPopup::UiThreadCallback()
{
    gtk_label_set_text( GTK_LABEL(m_label), (const gchar*) m_strLabel.c_str() );
    GtkLabel* label = GTK_LABEL( GTK_BIN( m_true )->child );
    gtk_label_set_text( label, (const gchar*) m_strTrueButtonLabel.c_str()  );    
    label = GTK_LABEL( GTK_BIN( m_false )->child );
    gtk_label_set_text( label, (const gchar*) m_strFalseButtonLabel.c_str()  );    
    label = GTK_LABEL( GTK_BIN( m_check )->child );
    gtk_label_set_text( label, (const gchar*) m_strDoNotShowAgainLabel.c_str() );

    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_check ), 
                                  m_bDoNotShowAgainIsChecked);

    ZkGtkUtils::GtkWidgetShowAll( m_window ); 
    m_answer = gnome_dialog_run_and_close( GNOME_DIALOG( m_window ) );
    
    m_bDoNotShowAgainIsChecked = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( m_check ));
}


