// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiSimpleMessage.h>
#include <ZkStringMgr.h>
#include <ZkGtkUtils.h>

//===========================================================================
// CLASS UiSimpleMessage
//===========================================================================



//---------------------------------------------------------------------------
//
UiSimpleMessage::UiSimpleMessage( )
{
    m_window = gtk_window_new( GTK_WINDOW_TOPLEVEL );

    gtk_window_set_policy( GTK_WINDOW( m_window ), FALSE, FALSE, FALSE );
    gtk_window_set_position( GTK_WINDOW( m_window ), GTK_WIN_POS_CENTER );

    // FIXME - hardcoded string should go into catalog.
    gtk_window_set_title( GTK_WINDOW( m_window ), "One moment please..." );

    GtkWidget* vbox = gtk_vbox_new( FALSE, 0 );    
    gtk_container_set_border_width( GTK_CONTAINER( vbox ), 20 );

    m_label = gtk_label_new( "" );
    
    gtk_container_add( GTK_CONTAINER( m_window ), vbox );
    gtk_container_add( GTK_CONTAINER( vbox ), m_label );

    gtk_signal_connect (GTK_OBJECT( m_window ), "delete_event",
                        GTK_SIGNAL_FUNC( CloseCb ), NULL );
    gtk_signal_connect (GTK_OBJECT( m_window ), "destroy",
                        GTK_SIGNAL_FUNC( DestroyCb ), NULL );
}

//---------------------------------------------------------------------------
//
UiSimpleMessage::~UiSimpleMessage( )
{
    gtk_widget_hide( GTK_WIDGET( m_window ) );
    gtk_widget_destroy( m_window );
}

//---------------------------------------------------------------------------
//
void UiSimpleMessage::Open( uint32_t msgId )
{
    ZKSTD::string msg;
    ZkStringMgr::Load( msg, msgId );

    gtk_label_set_text( GTK_LABEL( m_label ), msg.c_str() );

    ZkGtkUtils::GtkWidgetShowAll( GTK_WIDGET( m_window ) );
}

//---------------------------------------------------------------------------
//
void UiSimpleMessage::Close()
{
    gtk_widget_hide( m_window );
}

//---------------------------------------------------------------------------
//
gint UiSimpleMessage::CloseCb( 
    GtkWidget */*widget*/,
    gpointer   /*data*/ )
{
    return( TRUE ); 
}


//---------------------------------------------------------------------------
//
void UiSimpleMessage::DestroyCb( 
    GtkWidget */*widget*/,
    gpointer   /*data*/ )
{
}


