/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */
  


//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiThreadBase.h>

#include <UiThreadId.h>
#include <UiThreadSafeMessage.h>
#include <UiSimpleMessage.h>
#include <IZkException.h>
#include <ZkErrorHandler.h>
#include <IZKErrorReporter.h>

//===========================================================================
// CLASS UiThreadBase
//===========================================================================

//---------------------------------------------------------------------------
//
UiThreadBase::UiThreadBase( )
    : m_bSuccess( false )
    , m_thread( 0 )
{
}

//---------------------------------------------------------------------------
//
UiThreadBase::~UiThreadBase( )
{
    
}


//---------------------------------------------------------------------------
//
void UiThreadBase::StartThread( )
{
    OnThreadStart();
    
    ZkError err;
    zkErrorNew(&err);
    ZKVERIFY( zkThreadCreate(err, &m_thread, UiThreadBaseThread::Execute, this ) );
    zkErrorFree(&err);

    // Start the message loop from the main ui thread.
    UiThreadSafeMessage::NotifyUi( (void*)&UiThreadBase::MessageLoop, 
                                   0, true );

    m_thread = 0;
}

//---------------------------------------------------------------------------
//
void UiThreadBase::JoinThread()
{
    if( m_thread )
    {
        void *rval;

        ZkError err;
        zkErrorNew( &err );
		ZKVERIFY( ZK_TRUE == zkThreadJoin( err, &m_thread, &rval ) ); 
        m_thread = 0;
        zkErrorFree( &err );
    }
}

//---------------------------------------------------------------------------
//
void UiThreadBase::EndThread( void* klass )
{
    UiThreadBase* inst = static_cast< UiThreadBase* >( klass );
    gtk_main_quit();

    inst->OnThreadEnd();
}

//---------------------------------------------------------------------------
//
void UiThreadBase::MessageLoop( void* )
{
    gtk_main();
}

//---------------------------------------------------------------------------
//
void UiThreadBase::DoThreadWork()
{
    m_bSuccess = true;

    try {
        OnThreadExecute();
    }
    catch( IZkException& ex ) 
    {
        ZkErrorHandler::GetIZkErrorReporter()->ShowError( ex );        
        m_bSuccess = false;
    }
}
      
//---------------------------------------------------------------------------
//
void UiThreadBase::OnThreadStart(  )
{
}

//---------------------------------------------------------------------------
//
void UiThreadBase::OnThreadExecute()
{
    /*
      Be careful when you override this function, It gets called by a 
      background thread.
     */
}

//---------------------------------------------------------------------------
//
void UiThreadBase::OnThreadEnd()
{
}


//===========================================================================
// CLASS UiThreadBaseThread
//===========================================================================

//---------------------------------------------------------------------------
//
void* ZK_THREAD_CALLBACK UiThreadBaseThread::Execute( void* klass )
{
    UiThreadBase *inst = static_cast< UiThreadBase* >( klass );
    
    inst->DoThreadWork( );

    UiThreadSafeMessage::NotifyUi( (void*)&UiThreadBase::EndThread, 
                                   klass, false );
    
    return NULL;
}



