// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiThreadId.h>
#include <gdk/gdk.h>


//===========================================================================
// CLASS UiThreadGuard
//===========================================================================

//---------------------------------------------------------------------------
//
UiThreadGuard::UiThreadGuard( )
{
    UiThreadId::Lock();
}

//---------------------------------------------------------------------------
//
UiThreadGuard::~UiThreadGuard( )
{
    UiThreadId::Unlock();
}


//===========================================================================
// CLASS UiThreadUnguard
//===========================================================================

//---------------------------------------------------------------------------
//
UiThreadUnguard::UiThreadUnguard( )
{
    UiThreadId::Unlock();
}

//---------------------------------------------------------------------------
//
UiThreadUnguard::~UiThreadUnguard( )
{
    UiThreadId::Lock();
}




//===========================================================================
// CLASS UiThreadId
//===========================================================================

UiThreadId* ZkSingleton<UiThreadId>::m_instance = 0;

//---------------------------------------------------------------------------
//
UiThreadId::UiThreadId( )
    : m_thread_id( pthread_self() )
{
}

//---------------------------------------------------------------------------
//
UiThreadId::~UiThreadId( )
{
}

//---------------------------------------------------------------------------
//
void UiThreadId::Lock()
{
    if( !IsUiThread() ) {
        GDK_THREADS_ENTER();
    }
}

//---------------------------------------------------------------------------
//
void UiThreadId::Unlock()
{
    if( !IsUiThread() ) {
        GDK_THREADS_LEAVE();
    }
}
                        
//---------------------------------------------------------------------------
//
bool UiThreadId::IsUiThread() 
{
    return pthread_equal( get().m_thread_id, pthread_self() );
}
