/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UITHREADID
#define ZKS_INCL_UITHREADID

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include <ZkSingleton.h>
#include <pthread.h>


/*===========================================================================
 * CLASS UiThreadGuard
 *=========================================================================*/

/*
  This object should be used to created a scoped acquisition of the ui thread
  lock.
*/

class UiThreadGuard {
public:

    UiThreadGuard();
    ~UiThreadGuard();
};

/*===========================================================================
 * CLASS UiThreadUnguard
 *=========================================================================*/

/*
  This object should be used to created a scoped release of the ui thread
  lock.
*/

class UiThreadUnguard {
public:

    UiThreadUnguard();
    ~UiThreadUnguard();
};


/*===========================================================================
 * CLASS UiThreadId
 *=========================================================================*/

/*
  Unless we can find a better approach to the problem, this class is to be used
  to identify the UI thread and make sure we do not try to acquire the
  gtk thread lock from ... the ui thread.

  It uses a thread id to check who is calling gdk_threads_enter and avoid to 
  reacquire the lock if the thread is identified as the ui thread.

  Those calls should be used instead of the calls provided by gdk.
*/

class UiThreadId : public ZkSingleton< UiThreadId > {

public: 

    /*----- static member functions -----*/

    static void Lock();
    static void Unlock();

    static bool IsUiThread();

    /*----- member functions -----*/

    // The thread id is noted at creation, so make sure you create this
    // object from the ui thread otherwise you will suffer a few hours of
    // intense and frustrating debugging.
    UiThreadId();
    virtual ~UiThreadId();
      

private: 

    /*----- data members -----*/

    pthread_t m_thread_id;

};
#endif
