// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiThreadSafeDlg.h>
#include <UiThreadId.h>

#include <clientdebug.h>
#include <unistd.h>

//===========================================================================
// CLASS UiThreadSafeDlg
//===========================================================================

//---------------------------------------------------------------------------
//
UiThreadSafeDlg::UiThreadSafeDlg( )
{
    ZKVERIFY( "UiThreadSafeDlg - create pipe" && pipe( m_descr ) == 0 );

    m_input = gdk_input_add( m_descr[0],
                             GDK_INPUT_READ,
                             Dispatch,
                             this );
}

//---------------------------------------------------------------------------
//
UiThreadSafeDlg::~UiThreadSafeDlg( )
{
    gdk_input_remove( m_input );
    close(m_descr[0]);
    close(m_descr[1]);
}

//---------------------------------------------------------------------------
//
void UiThreadSafeDlg::Dispatch(
    gpointer data,
    gint /*source*/,
    GdkInputCondition /*cond*/
)
{
    ZKASSERT( "This is the UI thread." && UiThreadId::IsUiThread() );

    UiThreadSafeDlg* dlg = (UiThreadSafeDlg*)data;

    // Here we clear the file descriptor, otherwise we enter 
    // the message loop again in the dialog box and we end up dispatching again.
    // Guess the code to clear the descriptor is somewhere after the 
    // dispatch call.(which makes sense...)
    char theBytes[1];

    // Made this EINTR aware.
    // if we get a signal while doing this read bad things could happen
    // like multiple calls to UihreadCallback. --ol
 retry:
    if ( -1 == read( dlg->m_descr[0], theBytes, 1 ) && errno == EINTR ) goto retry;
    // Now we ask the derived class to pop whatever it needs.
    GDK_THREADS_ENTER();
    dlg->UiThreadCallback();
    GDK_THREADS_LEAVE();

    // Unblock background thread waiting for the popup to close.
    dlg->m_event.Set();
}

//---------------------------------------------------------------------------
//
void UiThreadSafeDlg::NotifyUi( bool blockAndWait )
{
    // This is just for debugging purposes, incrementing this value after each write
    // let's you peek at the sequence messages are dispatched in.
    static char theByte = 0;
    if( UiThreadId::IsUiThread() ) 
    {
        UiThreadCallback();
    }
    else 
    {
        m_event.Reset();
        
        ZKVERIFY( write( m_descr[1], &theByte,  1 ) != -1 );
        theByte++;
        if ( blockAndWait )
            m_event.Wait();
    }
}
