// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include "UiTokenCtrl.h"
#include <ZkCommonUIStrings.h>

#include <ITokenService.h>
#include <ZkService.h>
#include <ZKFileConfiguration.h>
#include <ZkStringMgr.h>
#include <MiscMessages.h>


// I just use this value to determine what token color to use
// This is not the correct way to determine whether a token is trial or not
#define TRIALTOKEN_DURATION 45

#define NUMBER_COLUMN_TEXT		"# Remaining"
#define TYPE_COLUMN_TEXT		"Nym Type"
#define LIFETIME_COLUMN_TEXT    "Duration"


//===========================================================================
// LOCAL DECLARATIONS
//===========================================================================

//===========================================================================
// CLASS UiPopup
//===========================================================================

//---------------------------------------------------------------------------
//
UiTokenCtrl::UiTokenCtrl(GtkCList* in_clist, GtkWidget* in_window)
    : m_clist(in_clist)
    , m_window(in_window)
    , m_curSelection( -1 )
{
    // initialize the object
    // remark, the number of column must be exactly 3
    
    gtk_clist_set_column_title( m_clist, NUMBER_COLUMN, NUMBER_COLUMN_TEXT);
    gtk_clist_set_column_title( m_clist, TYPE_COLUMN, TYPE_COLUMN_TEXT);
    gtk_clist_set_column_title( m_clist, LIFETIME_COLUMN, LIFETIME_COLUMN_TEXT);

    // desactive the buttons
    gtk_clist_column_titles_passive( m_clist );
    
    gtk_signal_connect(GTK_OBJECT(m_clist), "select_row",
                        GTK_SIGNAL_FUNC( SelectCb ),
                        this);
    gtk_signal_connect(GTK_OBJECT(m_clist), "unselect_row",
                        GTK_SIGNAL_FUNC( UnselectCb ),
                        this);
}


//---------------------------------------------------------------------------
//
UiTokenCtrl::~UiTokenCtrl()
{
    
}

//---------------------------------------------------------------------------
//
void UiTokenCtrl::SelectCb(  GtkWidget *,
                             gint row,
                             gint, 
                             GdkEventButton *,
                             UiTokenCtrl* inst)
{
   inst->m_curSelection = row;
} 

//---------------------------------------------------------------------------
//
void UiTokenCtrl::UnselectCb(  GtkWidget *,
                               gint row,
                               gint, 
                               GdkEventButton *,
                               UiTokenCtrl* inst)
{
    if( inst->m_curSelection == row )
    {
        inst->m_curSelection = -1;
    }
} 

//---------------------------------------------------------------------------
//
void UiTokenCtrl::Init()
{
    LoadPixmaps();
    BuildTokenList();
}

//---------------------------------------------------------------------------
//
void UiTokenCtrl::SetSensitive( bool in_enable)
{
    gtk_widget_set_sensitive( GTK_WIDGET( m_clist ), (gboolean)in_enable);
}

//---------------------------------------------------------------------------
//
void UiTokenCtrl::BuildTokenList()
{
    // note the current selection.
    int curSel = m_curSelection;

   	// Empty any existing state
    m_TokenTypes.clear() ;
    gtk_clist_clear( m_clist );
    
	ZkService::GetITokenService()->GetTokensGroupedByTypeID( m_TokenTypes );

	// Display the tokens by type

	for ( size_t j = 0 ; j < m_TokenTypes.size() ; j++ )
	{
		TokensOfType & refItemData = m_TokenTypes[ j ];

        ZKSTD::string strDuration;
        
        if( refItemData.Tokens.size() > 0 &&
            ! ZkService::GetITokenService()->DoesTokenUseCloud( refItemData.Tokens[0] ) &&
            ! ZkService::GetITokenService()->DoesTokenUseMail( refItemData.Tokens[0] ))
        {
            ZkStringMgr::Load( strDuration, IDS_TOKEN_UNLIMITED);
        }
        else
        {
            // Only show number of days if we know a value, zero days looks bad!
	        if ( refItemData.Desc.GetDurationInDays() > 0 ) 
            {
		        ZkStringMgr::Load(strDuration, IDS_TOKEN_DURATION, refItemData.Desc.GetDurationInDays());
	        }
        }                
                
		InsertItem( refItemData.Tokens.size(), 
	      		    refItemData.Desc.GetTokenLabels().c_str(), 
        		    strDuration.c_str()
    			    ,refItemData.Desc.GetDurationInDays() <= TRIALTOKEN_DURATION ? 1 : 0 ) ;
	}

    // Select first available token by default.
    if( m_TokenTypes.size() == 0 ) 
    {
        curSel = -1;
    }
    else if( curSel < 0 || curSel >= m_TokenTypes.size() )
    { 
        curSel = 0;
    }

    // This call triggers a call to the SelectCb which sets m_curSelection.
    gtk_clist_select_row( m_clist, curSel, -1 );
}


//---------------------------------------------------------------------------
//
// Customized Insert Item feature
int UiTokenCtrl::InsertItem( int iNumber, const char* in_pszDesc, const char* in_strDuration, int )
{
	ZKASSERT(in_pszDesc);

	if (!strlen( in_pszDesc ) ) // Even without an Niqs ZKTokenSubSys::GetDescriptionOfTokenX() should of returned some description
	{
		ZKASSERT(FALSE);
		return -1 ;
	}
	
	char strNum[16] = {0};

	snprintf(strNum,16,"%d", iNumber );
    
    gchar* item[3];
    
    item[NUMBER_COLUMN] = const_cast<gchar*>(strNum);
    item[TYPE_COLUMN] = const_cast<gchar*>(in_pszDesc);
    item[LIFETIME_COLUMN] = const_cast<gchar*>(in_strDuration);
        
    int iAddedPos;

    gtk_clist_freeze( m_clist );
    iAddedPos = gtk_clist_append( m_clist , item );
/*    gtk_clist_set_pixmap( m_clist ,
                          ind, 0,
                          m_pixmaps[t].first, m_pixmaps[t].second );
                          
  */
          /*
    gtk_clist_set_pixtext(   m_clist,
                             iAddedPos,
                             NUMBER_COLUMN,
                             item[0],
                             16,
                             m_pixmaps[TOKEN_GOLD].first,
                             m_pixmaps[TOKEN_GOLD].second );                            
            */              
    gtk_clist_thaw( m_clist );
    gtk_clist_moveto( m_clist , iAddedPos, 0, 0, 0 );

	return iAddedPos;
}

//---------------------------------------------------------------------------
//
void UiTokenCtrl::LoadPixmaps()
{
    
    AddPixmap( TOKEN_GOLD,    "token_gold.xpm"  );
    //AddPixmap( TOKEN_SILVER,   "token_gray.xpm" );
    
}

//---------------------------------------------------------------------------
//
void UiTokenCtrl::AddPixmap( int index, const char* filename )
{
    GdkPixmap* pixmap;
    GdkPixmap* mask;

    ZKSTD::string fPath = ZKFileConfiguration::GetFreedomPixmapDir();
    fPath += filename;
/*
    GtkStyle*  style;
    pixmap  = gdk_pixmap_colormap_create_from_xpm_d(GdkWindow *window,
                                              GdkColormap *colormap,
                                              GdkBitmap **mask,
                                              GdkColor *transparent_color,
                                              const gchar *filename)
            
                                             (GdkWindow *window,
                                              GdkBitmap **mask,
                                              GdkColor *transparent_color,
                                              const gchar *filename);
  */   /*       
    gtk_widget_realize( m_window );
    pixmap = gdk_pixmap_create_from_xpm(  m_window->window,
                                          &mask,
                                          &style->bg[ GTK_STATE_NORMAL ],
                                          fPath.c_str() );
*/
    m_pixmaps[ index ] = Pixmaps( pixmap, mask );    

    // FIXME - check this out, what about leaks here ?
    // gdk_pixmap_unref( pixmap );
}




//---------------------------------------------------------------------------
//
// returns false if user has no selection
bool UiTokenCtrl::GetSelectedToken(ZKSTD::string & out_strTokenInBase64 )
{	
	out_strTokenInBase64.erase() ;

	int i = m_curSelection;

	ZKASSERT( i < (int) m_TokenTypes.size() ) ;

	if ( i != -1 ) 
    {

		TokensOfType & refItemData = m_TokenTypes[ i ] ;
		
		const size_t cntTokens = refItemData.Tokens.size() ;

		if(0 == cntTokens)
        {
			ZKTRACE0("ZKTokenCtrl::GetSelectedToken() returning false, would of divided by zero!\n");
			return false;
		}

		// Determine what token to display
		// Find a random token out of the available nyms of that type

		srand( (unsigned)time( NULL ) );
		size_t randomPosition = (rand() % cntTokens );

		ZKASSERT( randomPosition < cntTokens ) ;

		size_t TokenIndex = refItemData.Tokens[ randomPosition ] ;

		ZKVERIFY(ZkService::GetITokenService()->GetTokenXinBase64(
			TokenIndex ,
			out_strTokenInBase64));

		return true ;
	}

    return false ;

}

//---------------------------------------------------------------------------
//
int UiTokenCtrl::GetNumTokenTypes()
{
	return m_TokenTypes.size() ;	
}

//---------------------------------------------------------------------------
//
bool UiTokenCtrl::GetSelectedTokenDescription( WTokenDescriptions** ppTokenDescription )
{
	ZKASSERT( ppTokenDescription != NULL ) ;

	int i = m_curSelection;

	ZKASSERT( i < (int) m_TokenTypes.size() ) ;

	if ( i != -1 ) 
    {
		TokensOfType & refItemData = m_TokenTypes[ i ] ;
		*ppTokenDescription = &refItemData.Desc;
		return true ;
	}

    return false ;
}



