/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UITOKENCTRL
#define ZKS_INCL_UITOKENCTRL

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include <gnome.h>
#include <gdk/gdk.h>
#include <gtk/gtkvbox.h>

#include <WTokenDescriptions.h>

#define NUMBER_COLUMN   0
#define TYPE_COLUMN     1
#define LIFETIME_COLUMN 2

#define TOKEN_IMAGE_COUNT 2

enum TokenImage { TOKEN_GOLD, TOKEN_SILVER }; 

#include <vector>

class TokensOfType;


/*===========================================================================
 * CLASS UiTokenCtrl
 *=========================================================================*/

class UiTokenCtrl
{
public:
        
    /*----- member functions -----*/
        
    UiTokenCtrl(GtkCList* in_clist, GtkWidget* in_window);
    virtual ~UiTokenCtrl();
    
    // initialization , need ZkService Up 
    // intialize graphics
    // note : this function call BuildTokenList
    void Init();    
    // simply rebuild the token list
    void BuildTokenList();

    // enable of disable the control
    void SetSensitive( bool in_enable);
    
    // return the number of differents token types
    int  GetNumTokenTypes();
    
    // get function to have the tokens and the description
    bool GetSelectedToken(ZKSTD::string & out_strTokenInBase64 );
    bool GetSelectedTokenDescription( WTokenDescriptions** ppTokenDescription );

            
protected:
        
    /*----- member functions -----*/        
        
    int InsertItem( int iNumber, const char* in_pszDesc, const char* in_strDuration, int );
    void LoadPixmaps();
    void AddPixmap( int index, const char* filename );
    
private:   
         
    /*----- static member functions -----*/
        
    static void SelectCb( GtkWidget      *clist,
                          gint            row,
                          gint            column,
                          GdkEventButton *event,
                          UiTokenCtrl*    inst );
    
    static void UnselectCb( GtkWidget      *clist,
                          gint            row,
                          gint            column,
                          GdkEventButton *event,
                          UiTokenCtrl*    inst );
    
    /*----- data members -----*/
        
    typedef ZKSTD::pair< GdkPixmap*, GdkPixmap* > Pixmaps;

    // the tokens represented in their types
   	ZKSTD::vector< TokensOfType > m_TokenTypes;

    // the representation of the object
    GtkCList*  m_clist;
    GtkWidget* m_window;
    
    // Table of pixmaps corresponding to the image enum in ZKConsoleMsg.h
    Pixmaps m_pixmaps[ TOKEN_IMAGE_COUNT ];
    
    int m_curSelection;
    
};


#endif
