/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_ZKGLADELOADER
#define ZKS_INCL_ZKGLADELOADER

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include <glib.h>
#include <glade/glade.h>

#define class _c_class
#define new   _c_new
#include <glade/glade-build.h>
#undef new
#undef class


/*===========================================================================
 * CLASS ZkGladeLoaderEx
 *=========================================================================*/

class ZkGladeLoaderEx{};
class ZkXMLFileNotFoundEx : public ZkGladeLoaderEx {};
class ZkXMLWidgetNotFoundEx : public ZkGladeLoaderEx {};

/*===========================================================================
 * CLASS ZkGladeLoader
 *=========================================================================*/

/*
  This is a helper class that takes care of finding the applications XML
  files and loading them. The lifescope of the object also take care of
  loading and getting rid of the XML file.
*/

class ZkGladeLoader {

public: 

    /*----- member functions -----*/

    ZkGladeLoader();

    ZkGladeLoader( 
        const char* in_xmlfile, 
        const char* in_component 
    ) throw( ZkGladeLoaderEx );

    ~ZkGladeLoader();

    void LoadFile( 
        const char* in_xmlfile, 
        const char* in_component 
    )  throw( ZkGladeLoaderEx );

    GtkWidget* GetWidget( const char* in_widget ) const;

private: 

    /*----- member functions -----*/

    /*----- static data members -----*/

    static bool m_initialized;

    static GladeWidgetBuildData m_data;

    /*----- data members -----*/

    GladeXML* m_xml;

};
#endif
