// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):





//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <ZkGtkUtils.h>
#include <clientdebug.h>
#include <gnome.h>
#include <IClientCentral.h>
#include <ZkService.h>

#include <cstring>

//===========================================================================
// LOCAL DECLARATIONS
//===========================================================================

#define TITLE_FONT "-*-helvetica-bold-r-normal-*-18-*-*-*-*-*-*-*"


//===========================================================================
// FUNCTIONS
//===========================================================================

//---------------------------------------------------------------------------
//
void ZkGtkUtils::UseTitleFont( GtkWidget* w )
{
    GdkColor darkgray  = { 0, 0x0005, 0x0005, 0x0005 };
    gdk_color_alloc( gdk_colormap_get_system(), &darkgray );
        
    GtkStyle* wStyle   = gtk_widget_get_style( w );
    GtkStyle* newStyle = gtk_style_copy( wStyle );
        
    GdkFont* newFt = gdk_font_load( TITLE_FONT );
        
    newStyle->fg[ GTK_STATE_NORMAL ]   = darkgray;
    newStyle->text[ GTK_STATE_NORMAL ] = darkgray;
    if( newFt )
    {
        newStyle->font = newFt;
    }
        
    gtk_widget_set_style( w,  newStyle );

    // FIXME - check for leaks here, not sure I should get rid of 
    //         the GtkStyle.    
}

//---------------------------------------------------------------------------
//
void ZkGtkUtils::ToUpper( GtkWidget* w )
{
    gchar* text = gtk_editable_get_chars( GTK_EDITABLE( w ), 0, -1 );

    size_t len = strlen( text );
        
    // Convert the text to upper case
    bool changed = false;

    for( size_t i = 0; i < len; i++ )
    {
        if ( islower( text[i] ) )
        {
            text[i] = toupper( text[i] );
            changed = true;
        }
    }

    if ( changed )
    {
        gint pos = gtk_editable_get_position( GTK_EDITABLE( w ) );
        gtk_editable_delete_text( GTK_EDITABLE( w ), 0, -1 );
                                  
        gint foo = 0;
        gtk_editable_insert_text( GTK_EDITABLE( w ), text, len, &foo ); 
        gtk_editable_set_position( GTK_EDITABLE( w ), pos );
    }

    g_free(text);    
}

void ZkGtkUtils::GetAndSetTopPref( GtkWidget * in_ct ) 
{
    // IT is possible that we want to display an error prior to ZkService's 
    // startup. This will ASSERT in debug builds but we will at least display
    // the error correctly.
    if (!ZkService::GetIClientCentral()) 
    {
        gnome_win_hints_set_layer( in_ct, WIN_LAYER_NORMAL );        
    } 
    else
    {        
        bool top = ZkService::GetIClientCentral()->GetPreference( ZK_TOOLBAR_PREF_SET,
                                                              PREF_NOT_ALWAYS_ON_TOP );
        // if they want it on top, top = 0, and vice-versa
        if( ! top )
        {
            gnome_win_hints_set_hints( in_ct, WIN_HINTS_DO_NOT_COVER );
            gnome_win_hints_set_layer( in_ct, WIN_LAYER_ONTOP );
            gdk_window_raise( in_ct->window );
        }
        else
        {
            gnome_win_hints_set_layer( in_ct, WIN_LAYER_NORMAL );
        }
    }
}

void ZkGtkUtils::GtkWidgetShow( GtkWidget * in_widget )
{
   gtk_widget_show( in_widget );
   ZkGtkUtils::GetAndSetTopPref( in_widget );
}

void ZkGtkUtils::GtkWidgetShowAll( GtkWidget * in_widget )
{
   gtk_widget_show_all( in_widget );
   ZkGtkUtils::GetAndSetTopPref( in_widget );
}
