// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiExtendNym.h>
#include <UiThreadId.h>
#include <ZkDialogsStrings.h>
#include <ZkGtkUtils.h>

#include <ZkGladeLoader.h>
#include <lockguard.h>

#include <IClientCentral.h>
#include <ITokenService.h>
#include <IClientPersist.h>
#include <INym.h>
#include <INymCentral.h>
#include <IZkPopup.h>
#include <IZkConsoleMsg.h>
#include <ZkService.h>
#include <ZKConfiguration.h>
#include <clientdebug.h>

#include <ZkErrorHandler.h>
#include <IZKErrorReporter.h>
#include <RZkException.h>

#include <UiTokenCtrl.h>
#include <UiEnterSerialNumber.h>
// FIXME : remove that dependency
#include <UiMainWindow.h>

#include <gnome.h>

//===========================================================================
// CLASS UiExtendNym.cpp
//===========================================================================

//---------------------------------------------------------------------------
//
UiExtendNym::UiExtendNym( INym* in_pNym )
    : m_result( false )
    , m_pNym( in_pNym ) 
{
    ZKASSERT( UiThreadId::IsUiThread() );
    
    ZkGladeLoader loader( "extendnym.xml", "ExtendNym_window" );

    m_dialog = loader.GetWidget( "ExtendNym_window" );
    GtkWidget* tokenlist = loader.GetWidget( "ExtendNym_tokenlist" );

    m_pTokenCtrl = new UiTokenCtrl(GTK_CLIST(tokenlist), m_dialog);
    m_pTokenCtrl->Init();
    
    m_get = loader.GetWidget( "ExtendNym_get" );    
    gtk_signal_connect( GTK_OBJECT( m_get ), "clicked",
                        GTK_SIGNAL_FUNC( GetTokensCb ), this );    
        
    gtk_widget_hide( GTK_WIDGET( m_dialog ) );
}

//---------------------------------------------------------------------------
//
UiExtendNym::~UiExtendNym( )
{
}


//---------------------------------------------------------------------------
//
bool UiExtendNym::Run( )
{
    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    // Now if this is a background thread, defer the work otherwise 
    // open the dialog rightaway.
    NotifyUi();

    return m_result;
}

//---------------------------------------------------------------------------
//
void UiExtendNym::UiThreadCallback()
{
    m_result = false;
    
    bool cancelled = false;

    while( !cancelled && !m_result ) 
    {
        ZkGtkUtils::GtkWidgetShow( m_dialog );
        int answer = gnome_dialog_run( GNOME_DIALOG (m_dialog) );

        switch( answer )
        {
            case 0: /* OK */
                m_result = OnOK( );
                if( m_result ) 
                {
                    gnome_dialog_close( GNOME_DIALOG (m_dialog) );
                }
                break;
                
            case 1: /* Cancel */
                gnome_dialog_close( GNOME_DIALOG (m_dialog) );
                cancelled = true;
                break;
                
            case -1: /* ESC or wm close */
                cancelled = true;                
                break;                
        }        
    }
}
    
//---------------------------------------------------------------------------
//
bool UiExtendNym::OnOK() 
{	
    bool bSuccess = false;
	ZKSTD::string szToken ;

	bool bSelectedToken = m_pTokenCtrl->GetSelectedToken( szToken );

	if ( !bSelectedToken || szToken.empty() ) 
	{		
		IZkPopup::MessageBox( IDS_SELECT_TOKEN_PLEASE );
		return false;
	}	

	ZKSTD::string strNymName;
	m_pNym->GetNymName( strNymName ) ;
 
	try 
    {
		// Extend the Nym
		//CWaitCursor	TheseArentTheDroidsDude; 
		//ZKBusyAVI busy;
		IZkConsoleMsg::ShowMsgForceUpdate(IDS_CONS_EXTEND_NYM);

		m_pNym->AddTokenToNymOnNymServer(szToken);

		bSuccess = true ;

		// After successful update we will no longer be expired
		m_pNym->SetYourLifeStage( NYM_LIFE_STAGE_ACTIVE ) ;

		ZKVERIFY(ZkService::GetITokenService()->RemoveTokenFromArray( szToken ) );
		ZkService::GetIClientPersist()->SaveClientState();
	}
	catch( IZkException& ex)
    {
        // FIXME - use a more reliable way of recognizing error.
        //         use a dedicated exception type after mac port.
        if( ex.GetErrorStringId() == IDS_TOKEN_ALREADY_SPENT )
        {
            IZkPopup::MessageBox( IDS_TOKEN_ALREADY_SPENT_STATEMENT );
            
            ZkService::GetITokenService()->RemoveTokenFromArray( szToken );
            m_pTokenCtrl->Init();
        }
        else
        {
            ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
        }
	}
    
    return bSuccess;
}

//---------------------------------------------------------------------------
//
void UiExtendNym::GetTokensCb( GtkWidget *, UiExtendNym* in_inst  )
{
	if( ! ZkService::GetINymCentral()->IsCurrentNymUser() )
	{
	    if( IZkPopup::QuestionBox( IDS_CHANGE_TO_USER_NYM_QUESTION, ZK_YESNO ) )
	    {
            UiMainWindow::get().ReselectUserNym();
	    }
    }        
    
    // the web integretion is not implemented on linux
    gnome_url_show( ZKConfiguration::GetSerialNumberUrl(NYM_TYPE1_EXTEND).c_str() );
    
    UiEnterSerialNumber serialNumberDlg;
    if(serialNumberDlg.Run())
    {
        in_inst->GetTokens( serialNumberDlg.GetSerialNumber() );
    }
}

//---------------------------------------------------------------------------
//
bool UiExtendNym::GetTokens(const tstring in_strSerialNumber)
{
	bool bRetVal = false;

    // FIXME - put a buzy AVI here
    
	// UI Should only activate button if there is
	// a valid sized string
	ZKASSERT( in_strSerialNumber.size() == ZKSERIAL_NUMBER_LENGTH ) ;

	WSerialNumber serialNumber( in_strSerialNumber.c_str() ) ;

	try
	{
		ZkService::GetITokenService()->GetSerialNumberCapabilities( serialNumber ) ;

		IZkConsoleMsg::ShowMsgForceUpdate(IDS_CONS_REDEEM_SERIAL);
		
		ZkService::GetITokenService()->RedeemSerialNumberForTokens( serialNumber ) ;

		// Save right now so that user doesn't lose the new tokens
		try 
		{
			ZkService::GetIClientPersist()->SaveClientState();
		}
		catch( IZkException&  )
		{
			// Not much positive we can say if this fails
		}

		//Refresh token list because, if successful, we now have tokens
		m_pTokenCtrl->BuildTokenList() ;
		bRetVal = true;
	}
	catch( IZkException& ex)
	{
		ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
		bRetVal = false ;
	}

	return bRetVal;
}
