/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UILOGINDLG
#define ZKS_INCL_UILOGINDLG

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

/*----- forward declarations -----*/

class UiLoginDlgImp;



/*===========================================================================
 * CLASS UiLoginDlg
 *=========================================================================*/

class UiLoginDlg {
  
public: 

    /*----- member functions -----*/

    // Purpose:  Scopes the lifetime of the login dialog ui.
    // Promise:  Will cleanup any UI component that was built.
    // Requires: Nothing, it is self contained.    
    UiLoginDlg();

    ~UiLoginDlg();
    
	// Purpose:  Initializes the dialog box and displays it.
    //           Validates the entered pasphrase and warns if invalid.
	// Promises: True means a valid passphrase was entered and OK was clicked.
	// Throws:   FIXME - will eventually throw something.
    bool getPassphrase();

	// Purpose:  DEBUGGING ONLY - sort of breaks encapsulation, you know..
	// Promise:  Will kick your butt if you use it.
    UiLoginDlgImp* getImplementation() const;
    
private: 

    /*----- member functions -----*/

    // Prohibited funcs - not implemented.
    UiLoginDlg( const UiLoginDlg& );
    const UiLoginDlg& operator=( const UiLoginDlg& );


    /*----- data members -----*/

    // Opaque pointer to an implementation class tha does the actual work.
    // Avoids external dependencies on gtk.
    UiLoginDlgImp* m_priv;
};

//---------------------------------------------------------------------------
//
inline UiLoginDlgImp* UiLoginDlg::getImplementation() const
{
    return m_priv;
}

#endif

