// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):


//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiLoginDlgImp.h>


#include <ZkGladeLoader.h>
#include <IZkPopup.h>

#include <glib.h>
#include <glade/glade.h>

#include <cstring>
#include <cstdio>

#include <ExceptionMessages.h>
#include <ZkService.h>
#include <IClientPersist.h>
#include <ICrypto.h>
#include <ZkErrorHandler.h>
#include <IZKErrorReporter.h>
#include <IZkException.h>
#include <ZkGtkUtils.h>


//===========================================================================
// CLASS UiLoginDlgImp
//===========================================================================

//---------------------------------------------------------------------------
//
UiLoginDlgImp::UiLoginDlgImp() 
    : m_result( false )
   ,  m_dialog( 0 )
   ,  m_txt( 0 ) 
{
    ZkGladeLoader loader( "login.xml", "LoginDialog" );

    m_dialog = loader.GetWidget( "LoginDialog");   
    m_txt    = loader.GetWidget( "Login_passwd" );

    // Make sure users can't turn the *** off and see the atcual text...
    gtk_entry_set_visibility( GTK_ENTRY( m_txt ), FALSE );
    
    // Initialize the dialog.
    gnome_dialog_set_default( GNOME_DIALOG( m_dialog ), 0 );
    gnome_dialog_editable_enters( GNOME_DIALOG( m_dialog ), 
                                  GTK_EDITABLE( m_txt ) );

    // Make sure the keyboard input will get to our text field by default.
    // (when the mouse pointer moves into the login dialog, that is)
    gtk_widget_grab_focus( m_txt );
    gtk_signal_connect( GTK_OBJECT( m_txt ), "insert_text",
                        GTK_SIGNAL_FUNC( UiLoginDlgImp::ValidateInputCb ),
                        this);
 }

//---------------------------------------------------------------------------
//
UiLoginDlgImp::~UiLoginDlgImp()
{
}

//---------------------------------------------------------------------------
//
bool UiLoginDlgImp::getPassphrase()
{
    // First try to open the dialog using the default passphrase.
    // Success would mean the user has chosen the "No passphrase" option.    
    if( TryDefault() )
    {
        return true;
    }

    m_result = false;
    bool cancelled = false;

    while( !cancelled && !m_result ) {
        ZkGtkUtils::GtkWidgetShow( m_dialog );
        int answer = gnome_dialog_run( GNOME_DIALOG (m_dialog) );

        switch( answer )
        {
            case 0: /* OK */
                m_result = ValidatePasswd( );
                if( m_result ) {
                    gnome_dialog_close( GNOME_DIALOG (m_dialog) );
                }
                break;
                
            case 1: /* Cancel */
                gnome_dialog_close( GNOME_DIALOG (m_dialog) );
                cancelled = true;
                break;
                
            case -1: /* ESC or wm close */
                cancelled = true;                
                break;                
        }        
    }
        
    return m_result;
}


//---------------------------------------------------------------------------
//
bool UiLoginDlgImp::ValidatePasswd( )
{
    try {
        ZkService::GetIClientPersist()->LoadClientState( gtk_entry_get_text( GTK_ENTRY( m_txt ) ) );
        return true;
    }
    catch ( IZkException & ex ) 
    {
        ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
        return false;
    }
}


//---------------------------------------------------------------------------
//
bool UiLoginDlgImp::TryDefault()
{
    bool success = true;

	try
	{
		ZKSTD::string strDefaultPassphrase;
		ZkService::GetICrypto()->GetDefaultPassphrase( strDefaultPassphrase );

        // This line throws if strPassphrase is invalid 
        ZkService::GetIClientPersist()->LoadClientState( strDefaultPassphrase );
	}
	catch(IZkException&)
	{
        success = false;
	}    

    return success;
}

//----------------------------------------------------------------------------
//
void UiLoginDlgImp::ValidateInputCb( 
    GtkEditable* /*editable*/,
    gchar* new_text,
    gint /* new_text_length */,
    gint* /* position */,
    UiLoginDlgImp * inst )

{
   // '\r' turns out to be the enter key on the numeric keypad
   // it wasn't getting caught before, but now it behaves
   // just like the enter key.
   // bug 5941

   if( new_text[ 0 ] == '\r' ) 
   {
      new_text[ 0 ] = '\0';
      inst->m_result = inst->ValidatePasswd( );
      if( inst->m_result ) 
      {
         gnome_dialog_close( GNOME_DIALOG (inst->m_dialog) );
      }
   }
}
