// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiMailFirstCheck.h>
#include <UiThreadId.h>
#include <ZkDialogsStrings.h>

#include <ZkStringMgr.h>
#include <ZkGladeLoader.h>
#include <lockguard.h>

#include <ZkService.h>
#include <INym.h>
#include <INymCentral.h>
#include <IClientCentral.h>
#include <ZkGtkUtils.h>

#include <clientdebug.h>
#include <vector>

#include <gnome.h>
//===========================================================================
// CLASS UiMailFirstCheck.cpp
//===========================================================================

//---------------------------------------------------------------------------
//
UiMailFirstCheck::UiMailFirstCheck( )
    : m_answer( 0 )
{
    // Call that a folk legend if you want, but it seems X really doesn't
    // appreciate top level windows being created in threads.
    // (which makes sense, given the fact X is not really thread safe)
    ZKASSERT( UiThreadId::IsUiThread() );
    
    ZkGladeLoader loader( "first_mail_check.xml", "window" );

    m_window     = loader.GetWidget( "window" );
    
    gtk_widget_hide( GTK_WIDGET( m_window ) );
}

//---------------------------------------------------------------------------
//
UiMailFirstCheck::~UiMailFirstCheck( )
{
}

//---------------------------------------------------------------------------
//
long UiMailFirstCheck::Run( )
{
    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    // Now if this is a background thread, defer the work otherwise 
    // open the dialog rightaway.
    NotifyUi();

    return m_answer;
}

//---------------------------------------------------------------------------
//
void UiMailFirstCheck::UiThreadCallback()
{
    ZkGtkUtils::GtkWidgetShowAll( m_window );

    int answer = gnome_dialog_run( GNOME_DIALOG( m_window ) );

    switch( answer )
    {
        case 0: /* Cancel FALLTHROUGH TO CASE -1 */
        case -1: /* Pushed dialog close button */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            m_answer = ZK_MAIL_CHECK_BLOCK;
            break;
            
        case 1: /* Active Nym Only */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            m_answer = ZK_MAIL_CHECK_ACTIVE;
            break;
            
        case 2: /* Active Nym Only */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            m_answer = ZK_MAIL_CHECK_ALL;
            break;
                
        default: /* hmm... */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            ZKASSERT("UiMailFirstCheck::UiThreadCallback() : Valid choice." && 0 );
            m_answer = ZK_MAIL_CHECK_BLOCK;
            break;
    }        
}
