// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiMailSentCount.h>
#include <UiThreadId.h>
#include <ZkDialogsStrings.h>

#include <ZkStringMgr.h>
#include <ZkGladeLoader.h>
#include <lockguard.h>
#include <ZkGtkUtils.h>

#include <clientdebug.h>
#include <vector>

#include <gnome.h>

//===========================================================================
// LOCAL DECLARATIONS
//===========================================================================

//===========================================================================
// CLASS UiMailSentCount.cpp
//===========================================================================

//---------------------------------------------------------------------------
//
UiMailSentCount::UiMailSentCount( )
    : m_resetTime( 0 )
    , m_answer( 0 )
    , m_pvecRecipients( 0 )
{
    // Call that a flok legend if you want, but it seems X really doesn't
    // appreciate top level windows being created in threads.
    // (which makes sense, given the fact X is not really thread safe)
    ZKASSERT( UiThreadId::IsUiThread() );
    
    ZkGladeLoader loader( "mail_sent_count.xml", "MailSentCount_window" );

    m_window  = loader.GetWidget( "MailSentCount_window" );
    m_recipients = loader.GetWidget( "MailSentCount_recipients" );
    m_header  = loader.GetWidget( "MailSentCount_msg" );

    gtk_widget_hide( GTK_WIDGET( m_window ) );
}

//---------------------------------------------------------------------------
//
UiMailSentCount::~UiMailSentCount( )
{
}

//---------------------------------------------------------------------------
//
bool UiMailSentCount::Run( time_t resetTime,
                           ZKSTD::vector<ZKSTD::string> * pvecRecipients )
{
    m_resetTime = resetTime;
    m_pvecRecipients = pvecRecipients;

    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    // Now if this is a background thread, defer the work otherwise 
    // open the dialog rightaway.
    NotifyUi();

    return m_answer;
}

//---------------------------------------------------------------------------
//
void UiMailSentCount::UiThreadCallback()
{
    // if anything is already in the recipients box, get rid of it.
    gtk_list_clear_items( GTK_LIST( m_recipients ), 0, -1 );
    GtkWidget* li;

    ZKSTD::vector<ZKSTD::string>::const_iterator it;

    // load the list of intended recipients
    for( it = m_pvecRecipients->begin();
         it != m_pvecRecipients->end();
         ++it )
    {
        li = gtk_list_item_new_with_label (const_cast<gchar *>((*it).c_str()));
        gtk_widget_show( li );
        gtk_container_add( GTK_CONTAINER( m_recipients ), li );
    }

    ZKSTD::string msg;

    struct tm * resetTimeTm = localtime( & m_resetTime ); 
    char timeString[ MAX_TIME_STRING ] = ""; 
    strftime( timeString, MAX_TIME_STRING, "%H:%M:%S on %A, %B %d", resetTimeTm );
    
    ZkStringMgr::Load( msg, IDS_MAIL_SENT_COUNT, 
                       timeString);
    gtk_label_set( GTK_LABEL( m_header ), msg.c_str());
 
    ZkGtkUtils::GtkWidgetShowAll( m_window ); 
    gnome_dialog_run( GNOME_DIALOG( m_window )); 
    gnome_dialog_close( GNOME_DIALOG( m_window ));
   
    // this is really the only way the user can go. 
    m_answer = false;
}


