// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):


//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPasswordConfirm.h>
#include <UiThreadId.h>
#include <ExceptionMessages.h>
#include <ZkGtkUtils.h>

#include <IZkPopup.h>
#include <ZkGladeLoader.h>
#include <lockguard.h>

#include <ICrypto.h>
#include <ZkService.h>
#include <clientdebug.h>

#include <gnome.h>

//===========================================================================
// CLASS UiPasswordConfirm.cpp
//===========================================================================

//---------------------------------------------------------------------------
//
UiPasswordConfirm::UiPasswordConfirm( )
    : m_answer( false )
{
    // Call that a flok legend if you want, but it seems X really doesn't
    // appreciate top level windows being created in threads.
    // (which makes sense, given the fact X is not really thread safe)
    ZKASSERT( UiThreadId::IsUiThread() );
    
    ZkGladeLoader loader( "password_confirm.xml", "window" );

    m_window = loader.GetWidget( "window" );
    m_passwd = loader.GetWidget( "passwd" );

    // Make sure users can't turn the *** off and see the atcual text...
    gtk_entry_set_visibility( GTK_ENTRY( m_passwd ), FALSE );

    gnome_dialog_set_default( GNOME_DIALOG( m_window ), 0 );
    gnome_dialog_editable_enters( GNOME_DIALOG( m_window ), 
                                  GTK_EDITABLE( m_passwd ) );

    gtk_widget_hide( GTK_WIDGET( m_window ) );
}

//---------------------------------------------------------------------------
//
UiPasswordConfirm::~UiPasswordConfirm( )
{
}

//---------------------------------------------------------------------------
//
bool UiPasswordConfirm::Run( )
{
    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    // Now if this is a background thread, defer the work otherwise 
    // open the dialog rightaway.
    NotifyUi();

    return m_answer;
}

//---------------------------------------------------------------------------
//
void UiPasswordConfirm::UiThreadCallback()
{
    ZkGtkUtils::GtkWidgetShowAll( m_window );

    bool close = false;

    while( !close && !m_answer )
    {
        int answer = gnome_dialog_run( GNOME_DIALOG( m_window ) );

        switch( answer )
        {
            case 0: /* OK */
                {
                    ZKSTD::string passwd( gtk_entry_get_text( GTK_ENTRY( m_passwd ) ));
                    ZKSTD::string currentPP;
                    ZkService::GetICrypto()->GetPassphrase( currentPP );
                    if( passwd == currentPP )
                    {
                        gnome_dialog_close( GNOME_DIALOG( m_window ) );
                        m_answer = true;
                    }
                    else
                    {
                        IZkPopup::MessageBox( IDS_CANT_LOAD_PASSWORD_INVALID );
                        m_answer = false;
                    }
                }

                break;
            
            case 1: /* Cancel */
                gnome_dialog_close( GNOME_DIALOG( m_window ) );
                m_answer = false;
                close = true;
                break;
            
            case -1: /* ESC or wm close */
                m_answer = false;
                close = true;
                break;                
        }  
    }
}



