// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPrefAbout.h>
#include <UiMessages.h>
#include <ZkGladeLoader.h>
#include <IZkPopup.h>
#include <zkVerInfo.h>

#include <ZkWebIntegration.h>
#include <ZKConfiguration.h>
#include <clientdebug.h>

//===========================================================================
// CLASS UiPrefAbout
//===========================================================================

//---------------------------------------------------------------------------
//
UiPrefAbout::UiPrefAbout( 
    const ZKSTD::string& name,
    const ZKSTD::string& desc 
) 
    : UiPrefBase( name, desc )
{
    ZkGladeLoader loader( "pref_about.xml", "topct" );

    m_team    = loader.GetWidget( "team" );
    m_home    = loader.GetWidget( "homepage" );
    m_support = loader.GetWidget( "support" );
    m_buildId = loader.GetWidget( "labelBuildId" );
    m_buildVersion = loader.GetWidget( "labelBuildVersion" );
    m_buildDate = loader.GetWidget( "labelBuildDate" );

    gnome_href_set_url( GNOME_HREF( m_team ), 
                        ZkWebIntegration::GetURL( URL_TEAM ).c_str() );
    gnome_href_set_url( GNOME_HREF( m_home ), 
                        ZkWebIntegration::GetURL( URL_HOMEPAGE_ID ).c_str() );
    gnome_href_set_url( GNOME_HREF( m_support ), 
                        ZkWebIntegration::GetURL( URL_SUPPORT_ID ).c_str() );


    const ZkVerInfo * verInfo = zkVerInfoGetStruct();
    
    ZKSTD::string period( "." );

    ZKSTD::string buildId( "Build Id: " );
    buildId += verInfo->buildnum;
    gtk_label_set_text( GTK_LABEL( m_buildId ), buildId.c_str() );

    ZKSTD::string buildVersion( "Build Version: " );
    buildVersion += verInfo->major + period 
                  + verInfo->minor ;
    gtk_label_set_text( GTK_LABEL( m_buildVersion ), buildVersion.c_str() );

    ZKSTD::string buildDate( "Build Date: " );
    buildDate += verInfo->date;
    gtk_label_set_text( GTK_LABEL( m_buildDate ), buildDate.c_str() );

    SetTopContainer( loader.GetWidget( "topct" ) );
}

//---------------------------------------------------------------------------
//
UiPrefAbout::~UiPrefAbout( )
{
}
