// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPrefAd.h>
#include <UiMessages.h>
#include <ZkGladeLoader.h>
#include <IZkPopup.h>

#include <INym.h>
#include <INymCentral.h>
#include <ZkService.h>
#include <IClientPersist.h>
#include <IClientCentral.h>

#include <clientdebug.h>
#include <string_hlp.h>

//===========================================================================
// LOCAL DECLARATIONS
//===========================================================================

bool operator==( const AdDescription& desc1, const AdDescription& desc2 )
{
    return(( desc1.GetFriendlyName() == desc2.GetFriendlyName() ) &&
           ( desc1.GetURL() == desc2.GetURL() ));
}

//===========================================================================
// CLASS UiPrefAd
//===========================================================================

//---------------------------------------------------------------------------
//
UiPrefAd::UiPrefAd( 
    const ZKSTD::string& name,
    const ZKSTD::string& desc
) 
    : UiPrefBase( name, desc ),
      m_vecRules( 0 ),
      m_bAscendingSort( true ),
      m_nCurrentSortColumn( 0 )
{
    ZkGladeLoader loader( "pref_ad.xml", "topct" );

    SetTopContainer( loader.GetWidget( "topct" ) );

    m_delete     = loader.GetWidget( "delete" );
    m_add        = loader.GetWidget( "add" );
    m_clist      = loader.GetWidget( "list" );
    m_provider   = loader.GetWidget( "provider" );
    m_url        = loader.GetWidget( "URL" );
    m_enableTog  = loader.GetWidget( "enable" );
    m_adsblocked = loader.GetWidget( "blocked" );
    m_bytessaved = loader.GetWidget( "saved" );

    gtk_signal_connect (GTK_OBJECT( m_url ), "activate",
                        GTK_SIGNAL_FUNC( AddCb ), this );
    gtk_signal_connect (GTK_OBJECT( m_delete ), "clicked",
                        GTK_SIGNAL_FUNC( DeleteCb ), this);
    gtk_signal_connect (GTK_OBJECT( m_add ), "clicked",
                        GTK_SIGNAL_FUNC( AddCb ), this);
    gtk_signal_connect (GTK_OBJECT( m_clist), "click-column",
                        GTK_SIGNAL_FUNC( ListClickCb ), this );

    gtk_clist_set_compare_func( GTK_CLIST( m_clist), 
       CaseInsensitiveCompareFunc );
    gtk_clist_set_auto_sort( GTK_CLIST( m_clist ), true );
}

//---------------------------------------------------------------------------
//
UiPrefAd::~UiPrefAd( )
{
}

//---------------------------------------------------------------------------
//
void UiPrefAd::Initialize()
{
    m_modified = false;

    bool flag;
    flag = ZkService::GetIClientCentral()->GetPreference(ZK_CLIENT_PREF_SET, 
                                                         PREF_NO_BLOCK_ADS );
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_enableTog ), !flag );

    gchar buffer[16];
	
	snprintf(buffer, sizeof(buffer), "%ld", ZkService::GetIAdBlocker()->GetNumberOfAdsBlocked());
    gtk_label_set_text(GTK_LABEL(m_adsblocked), buffer);

	snprintf(buffer, sizeof(buffer), "%ld", ZkService::GetIAdBlocker()->GetEstimatedSavedBandwidth());
	gtk_label_set_text(GTK_LABEL(m_bytessaved), buffer);

    // Get the rules from ZkService, so we can modify them locally.
    ZkService::GetIAdBlocker()->GetBlockList( m_vecRules );

    FillListBox( );
}

//---------------------------------------------------------------------------
//
bool UiPrefAd::Finalize()
{
	bool bDisable = !gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( m_enableTog ));

	if( bDisable != ZkService::GetIClientCentral()->GetPreference( ZK_CLIENT_PREF_SET, PREF_NO_BLOCK_ADS ) ){
		ZkService::GetIClientCentral()->SetPreference( ZK_CLIENT_PREF_SET, PREF_NO_BLOCK_ADS, bDisable ) ;
	}

	if( m_modified )
	{
		ZkService::GetIAdBlocker()->SetBlockList( m_vecRules );
        m_modified = false;
	}	

    return true;
}

//---------------------------------------------------------------------------
//
void UiPrefAd::Cancel()
{
    m_modified = false;    
}

//---------------------------------------------------------------------------
//
void UiPrefAd::AddCb( GtkWidget*, UiPrefAd* inst )
{
    inst->AddRule();
}

//---------------------------------------------------------------------------

//
void UiPrefAd::AddRule()
{
    ZKSTD::string newProvider;
    ZKSTD::string newURL;

    gchar *strNewProvider;
    gchar *strNewURL;

    strNewProvider = gtk_entry_get_text( GTK_ENTRY( m_provider ) );
    strNewURL      = gtk_entry_get_text( GTK_ENTRY( m_url ) );

    if( strNewProvider ) {
        newProvider = strNewProvider;
    }
    if( strNewURL ) {
        newURL = strNewURL;
    }

	ZKS::trim(newURL);
	ZKS::trimLeadingWhite(newURL);

	ZKSTD::string strLowerUrl(newURL);

	ZKS::MakeLower(strLowerUrl);

	if(strLowerUrl.find("http://") != strLowerUrl.npos)
	{
		newURL.erase(strLowerUrl.find("http://"), strlen("http://"));
	}

    if( newURL.length() != 0 )
    {
		AdDescription newRule = AdDescription( newProvider, newURL ); 
        
        // FIXME - missing an INTERNAL_BUILD MarkForUpload here.
        
        ZKSTD::vector<AdDescription>::iterator iter;
        iter = ZKSTD::find( m_vecRules.begin(), m_vecRules.end(), newRule );
        if( iter == m_vecRules.end() )
        {
            ZKTRACE0( "Entry not Found, adding it\n" );
            m_vecRules.push_back( newRule );
            FillListBox();
        }
        else 
        {
            ZKTRACE0( "Already in the list.\n" );
        }
    }

    m_modified = true;
}

//----------------------------------------------------------------------------
//

void UiPrefAd::ListClickCb( GtkWidget*,
                                 gint column,
                                 UiPrefAd* inst )
{
   
   // if they already sorted this column one way, then flip
   // the direction and sort it the other way.
   // otherwise (implicitly) maintain the sort direction
   if( inst->m_nCurrentSortColumn == column ) {
      inst->m_bAscendingSort = ! inst->m_bAscendingSort;
   }
   inst->m_nCurrentSortColumn = column;

   if( inst->m_bAscendingSort )
   {
      gtk_clist_set_sort_type( GTK_CLIST( inst->m_clist),
         GTK_SORT_ASCENDING );
   }
   else
   {
      gtk_clist_set_sort_type( GTK_CLIST( inst->m_clist),
         GTK_SORT_DESCENDING );
   }
   gtk_clist_set_sort_column( GTK_CLIST( inst->m_clist), column );
   gtk_clist_sort( GTK_CLIST( inst->m_clist ));
}


//---------------------------------------------------------------------------
//
void UiPrefAd::DeleteCb( GtkWidget*, UiPrefAd* inst )
{
    inst->DeleteSelection();
}

//---------------------------------------------------------------------------
//
void UiPrefAd::DeleteSelection()
{
    GList   *dlist;

    dlist = GTK_CLIST( m_clist )->selection;
    
    if( !dlist ) 
    {
        // Nothing selected, just return.
        return;
    }


    // Now remove those items from the ZKSTD::set.
    // (this is arguably a very inefficient way of doing it)

    while (dlist) {
        AdDescription *rule = (AdDescription*)gtk_clist_get_row_data( GTK_CLIST( m_clist ), (int)(dlist->data) );

        ZKSTD::vector<AdDescription>::iterator iter;
        iter = ZKSTD::find( m_vecRules.begin(), m_vecRules.end(), *rule );

        if( iter != m_vecRules.end() )
        {
            m_vecRules.erase(iter);            
        }

        dlist = dlist->next;
    }

    m_modified = true;
    FillListBox();
}

//---------------------------------------------------------------------------
//
void UiPrefAd::FillListBox()
{
    gchar* cells[ 2 ];
    gtk_clist_freeze( GTK_CLIST( m_clist ) );

    gtk_clist_clear(  GTK_CLIST( m_clist ) );

    ZKSTD::vector<AdDescription>::iterator iter;

    int row;

	for( iter = m_vecRules.begin(); 
         iter != m_vecRules.end(); 
         iter++ )
	{
        cells[0] = const_cast<gchar*>( iter->GetFriendlyName().c_str() );
        cells[1] = const_cast<gchar*>( iter->GetURL().c_str());

        row = gtk_clist_append( GTK_CLIST( m_clist ), cells );
        gtk_clist_set_row_data( GTK_CLIST( m_clist ), row, (void*)&(*iter));
	}

    gtk_clist_thaw( GTK_CLIST( m_clist ) );
}

gint UiPrefAd::CaseInsensitiveCompareFunc( GtkCList *clist,
                                           gconstpointer ptr1,
                                           gconstpointer ptr2 )
{

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  char * text1 = (GTK_CELL_TEXT (row1->cell[clist->sort_column])->text);
  char * text2 = (GTK_CELL_TEXT (row2->cell[clist->sort_column])->text);

  if (!text2) {
    return (text1 != NULL);
  } else if (!text1) {
    return -1;
  } else {
    return strnicmp( text1, text2, MIN( strlen( text1), strlen( text2 )));
  }
}
