// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPrefBackup.h>
#include <UiMessages.h>
#include <ZkGladeLoader.h>
#include <UiBackupBase.h>
#include <ZkGtkUtils.h>

//===========================================================================
// CLASS UiPrefBackup
//===========================================================================

//---------------------------------------------------------------------------
//
UiPrefBackup::UiPrefBackup( 
    const ZKSTD::string& name,
    const ZKSTD::string& desc 
) 
    : UiPrefBase( name, desc )
{
    ZkGladeLoader loader( "pref_backup.xml", "topct" );

    SetTopContainer( loader.GetWidget( "topct" ) );

    m_backup = loader.GetWidget( "backup" );
    m_entry  = loader.GetWidget( "entry" );

    gtk_signal_connect (GTK_OBJECT( m_backup ), "clicked",
                        GTK_SIGNAL_FUNC( BackupCb ), this);
    gtk_signal_connect (GTK_OBJECT( m_entry ), "browse_clicked",
                        GTK_SIGNAL_FUNC( PickFileCb ), this );
}

//---------------------------------------------------------------------------
//
void UiPrefBackup::BrowseDialogOk( GtkWidget *, gpointer data )
{
        GtkFileSelection *fs;
        GnomeFileEntry *fentry;
        GtkWidget *entry;

        fs = GTK_FILE_SELECTION (data);
        fentry = GNOME_FILE_ENTRY (gtk_object_get_user_data (GTK_OBJECT (fs)));
        entry = gnome_file_entry_gtk_entry (fentry);

        gtk_entry_set_text (GTK_ENTRY (entry),
                            gtk_file_selection_get_filename (fs));
        gtk_signal_emit_by_name (GTK_OBJECT (entry), "activate");
        gtk_widget_destroy (GTK_WIDGET (fs));
}

//---------------------------------------------------------------------------
//
void UiPrefBackup::BrowseDialogKill (GtkWidget *, gpointer data)
{
        GnomeFileEntry *fentry;
        fentry = GNOME_FILE_ENTRY (data);
        fentry->fsw = NULL;
}


//---------------------------------------------------------------------------
//
void UiPrefBackup::PickFileCb( GtkWidget *, UiPrefBackup * inst )
{
        GnomeFileEntry * fentry = (GnomeFileEntry *)inst->m_entry;
        GtkWidget *fsw;
        GtkFileSelection *fs;
        GtkWidget *parent;
        char *p;

        fentry->fsw = 0;
        fsw = gtk_file_selection_new ( "Select file" );
        parent = gtk_widget_get_toplevel (GTK_WIDGET (fentry));
        gtk_window_set_transient_for (GTK_WINDOW(fsw), GTK_WINDOW (parent));

        gtk_object_set_user_data (GTK_OBJECT (fsw), fentry);

        fs = GTK_FILE_SELECTION (fsw);
        gtk_widget_set_sensitive(fs->file_list,
                                 !fentry->directory_entry);

        p = gtk_entry_get_text (GTK_ENTRY (gnome_file_entry_gtk_entry (fentry)))
;
        if(p && *p!='/' && fentry->default_path) {
                p = g_concat_dir_and_file (fentry->default_path, p);
                gtk_file_selection_set_filename (fs, p);
                g_free(p);
        } else
                gtk_file_selection_set_filename (fs, p);

        gtk_signal_connect (GTK_OBJECT (fs->ok_button), "clicked",
                            GTK_SIGNAL_FUNC( BrowseDialogOk ),
                            fs);
        gtk_signal_connect (GTK_OBJECT (fsw), "destroy",
                            GTK_SIGNAL_FUNC(BrowseDialogKill),
                            fentry);
        gtk_signal_connect_object (GTK_OBJECT (fs->cancel_button), "clicked",
                                   GTK_SIGNAL_FUNC(gtk_widget_destroy),
                                   GTK_OBJECT(fsw));

        if (gtk_grab_get_current ())
                gtk_grab_add (fsw);

        ZkGtkUtils::GtkWidgetShow( fsw );
        fentry->fsw = fsw;
}


//---------------------------------------------------------------------------
//
UiPrefBackup::~UiPrefBackup( )
{
}

//---------------------------------------------------------------------------
//
void UiPrefBackup::Initialize()
{
    // FIXME - if the past used path was saved in the registry we could use 
    //         it to offer the same directory from one session to the next.

    return;
}

//---------------------------------------------------------------------------
//
bool UiPrefBackup::Finalize()
{
    // FIXME - we could save the last used path in here so that the user would
    //         get the same backup directory from one session to the other.
    //         The value in the registry could be used for a restore op.

    return true;
}

//---------------------------------------------------------------------------
//
void UiPrefBackup::BackupCb( GtkWidget*, UiPrefBackup* inst )
{
    UiBackupBase::DoBackup( inst->m_entry );
}



