// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPrefConsole.h>
#include <ZkGladeLoader.h>

#include <ZkService.h>
#include <IClientCentral.h>
#include <IZkRegistry.h>
#include <clientdebug.h>
#include <IZkConsoleMsg.h>


//===========================================================================
// CLASS UiPrefConsole
//===========================================================================

//---------------------------------------------------------------------------
//
UiPrefConsole::UiPrefConsole( 
    const ZKSTD::string& name,
    const ZKSTD::string& desc 
) 
    : UiPrefBase( name, desc )
{
    ZkGladeLoader loader( "pref_console.xml", "topct" );

    SetTopContainer( loader.GetWidget( "topct" ));

    m_openLogDirectory = loader.GetWidget( "open_log_directory" );

    m_verboseMode    = GTK_TOGGLE_BUTTON( loader.GetWidget( "verbose_mode" ));
    m_copyLogToDisk  = GTK_TOGGLE_BUTTON( loader.GetWidget( "copy_log_to_disk" ));

    gtk_signal_connect (GTK_OBJECT( m_openLogDirectory ), "clicked",
                        GTK_SIGNAL_FUNC( OpenLogDirectoryCb ), this);
}

//---------------------------------------------------------------------------
//
UiPrefConsole::~UiPrefConsole( )
{
}

//---------------------------------------------------------------------------
//
void UiPrefConsole::Initialize()
{
    bool flag;

	flag = IZkRegistry::GetFlag(CONSOLE_VERBOSE_REGISTRY_KEY);
    gtk_toggle_button_set_active( m_verboseMode, flag );

  	flag = IZkRegistry::GetFlag(CONSOLE_LOG_REGKEY);
    gtk_toggle_button_set_active( m_copyLogToDisk, flag );
}

//---------------------------------------------------------------------------
//
bool UiPrefConsole::Finalize()
{
    bool bFlag = gtk_toggle_button_get_active( m_verboseMode );
	if ( IZkRegistry::GetFlag(CONSOLE_VERBOSE_REGISTRY_KEY) != bFlag ) 
    {
		IZkRegistry::SetFlag(CONSOLE_VERBOSE_REGISTRY_KEY, bFlag );
	}

    bFlag = gtk_toggle_button_get_active( m_copyLogToDisk );
	if ( IZkRegistry::GetFlag(CONSOLE_LOG_REGKEY) != bFlag ) 
    {
		IZkRegistry::SetFlag(CONSOLE_LOG_REGKEY, bFlag );
	}
    
    return true;
}

//---------------------------------------------------------------------------
//
void UiPrefConsole::OpenLogDirectoryCb( GtkWidget*, UiPrefConsole* inst )
{

}

