// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPrefCookies.h>
#include <UiMessages.h>
#include <ZkGladeLoader.h>
#include <IZkPopup.h>
#include <ZkStringMgr.h>
#include <ZkDialogsStrings.h>
#include <ICookieManager.h>

#include <INymCookieMgr.h>
#include <INym.h>
#include <INymCentral.h>
#include <IClientCentral.h>
#include <ZkService.h>

#include <clientdebug.h>

//===========================================================================
// CLASS UiPrefCookies
//===========================================================================

//---------------------------------------------------------------------------
//
UiPrefCookies::UiPrefCookies( 
    const ZKSTD::string& name,
    const ZKSTD::string& desc,
    INym* pNym,
    bool isUserNym
) 
    : UiPrefBase( name, desc )
    , m_selectall( 0 )
    , m_delete( 0 )
    , m_clist( 0 )
    , m_txt( 0 )
    , m_passthru( 0 )
    , m_block( 0 )
    , m_enable( 0 )
    , m_pNym( pNym )
    , m_isUserNym( isUserNym )
    , m_bAscendingSort( true )
    , m_nCurrentSortColumn( 0 )
{
    ZkGladeLoader loader;

    if( m_isUserNym )
    {
        loader.LoadFile( "pref_cookies_u.xml", "topct" );
        m_enable   = GTK_TOGGLE_BUTTON( loader.GetWidget( "radio_enable" ));
        m_block    = GTK_TOGGLE_BUTTON( loader.GetWidget( "radio_block" ));
        m_passthru = GTK_TOGGLE_BUTTON( loader.GetWidget( "radio_browser" ) );

        gtk_signal_connect (GTK_OBJECT( m_passthru ), "toggled",
                            GTK_SIGNAL_FUNC( ToggleCb ), this);
    }
    else
    {
        loader.LoadFile( "pref_cookies.xml", "topct" );
        m_block   = GTK_TOGGLE_BUTTON( loader.GetWidget( "block" ));
        m_txt     = loader.GetWidget( "txt" );
    }

    SetTopContainer( loader.GetWidget( "topct" ) );

    m_selectall = loader.GetWidget( "selectall" );
    m_delete    = loader.GetWidget( "delete" );
    m_clist     = loader.GetWidget( "clist" );

    gtk_signal_connect (GTK_OBJECT( m_selectall ), "clicked",
                        GTK_SIGNAL_FUNC( SelectAllCb ), this);
    gtk_signal_connect (GTK_OBJECT( m_delete ), "clicked",
                        GTK_SIGNAL_FUNC( DeleteCb ), this);
    gtk_signal_connect (GTK_OBJECT( m_clist ), "click-column",
                        GTK_SIGNAL_FUNC( ListClickCb), this );

}

//---------------------------------------------------------------------------
//
UiPrefCookies::~UiPrefCookies( )
{
}


//---------------------------------------------------------------------------
//
void UiPrefCookies::Initialize()
{
    ZKASSERT( m_pNym != 0 );

    SetModified( false );

	INymCookieMgr* pNymCookieMgr = NULL ;
	m_pNym->QueryInterface( IID_INYMCOOKIEMGR, (void**)&pNymCookieMgr ) ;
	pNymCookieMgr->GetNymCookieMap( m_CookieMap ); 

    UpdateCookieList();

    bool bNoCookieMgr, bCookiesEnabled;

    bNoCookieMgr = ZkService::GetIClientCentral()->GetPreference( ZK_CLIENT_PREF_SET, 
                                                                PREF_NO_USER_COOKIE_MANAGEMENT ); 
    bCookiesEnabled = m_pNym->GetCookiesEnabled();

    // Also reset the block toggle button.
    if( m_isUserNym )
    {
        if ( bNoCookieMgr )
        {
            gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_passthru ), true );
        }
        else
        {
            if( bCookiesEnabled )
                gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_enable ), true );
            else
                gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_block ), true );
        }
    }
    else
    {
        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_block ), 
                                      !bCookiesEnabled );

        ZKSTD::string msg;
        ZKSTD::string name;
        m_pNym->GetNymName( name );

        ZkStringMgr::Load( msg, IDS_COOKIES_OF_NYM, name.c_str() );
        gtk_label_set_text( GTK_LABEL( m_txt ), msg.c_str() );
    }
    
}

//---------------------------------------------------------------------------
//
bool UiPrefCookies::Finalize()
{
    if( WasModified() )
    {
        INymCookieMgr* pNymCookieMgr = NULL ;
        m_pNym->QueryInterface( IID_INYMCOOKIEMGR, (void**)&pNymCookieMgr ) ;
        pNymCookieMgr->SetNymCookieMap( m_CookieMap ); 
    }

    if( m_isUserNym )
    {
		bool bOldNoCookieMgr    = ZkService::GetIClientCentral()->GetPreference(ZK_CLIENT_PREF_SET, PREF_NO_USER_COOKIE_MANAGEMENT);
		bool bOldCookiesEnabled = m_pNym->GetCookiesEnabled();

        bool bNoCookieMgr    = gtk_toggle_button_get_active( m_passthru );
        bool bCookiesEnabled = false;

        if( !bNoCookieMgr );
		{
            bCookiesEnabled = gtk_toggle_button_get_active( m_enable );
		}

		if(bCookiesEnabled != bOldCookiesEnabled ||
           bNoCookieMgr != bOldNoCookieMgr)
		{
			m_pNym->SetCookiesEnabled( bCookiesEnabled );
			ZkService::GetIClientCentral()->SetPreference(
                ZK_CLIENT_PREF_SET, 
                PREF_NO_USER_COOKIE_MANAGEMENT, bNoCookieMgr );
		}
    }
    else
    {
		bool bCookiesEnabled = !gtk_toggle_button_get_active( m_block );

		if( bCookiesEnabled != m_pNym->GetCookiesEnabled() )
		{
			m_pNym->SetCookiesEnabled( bCookiesEnabled );
		}
    }

    return true;
}

//---------------------------------------------------------------------------
//
void UiPrefCookies::SelectAllCb( GtkWidget*, UiPrefCookies* inst )
{
    gtk_clist_select_all( GTK_CLIST( inst->m_clist ) );
}

//---------------------------------------------------------------------------
//
void UiPrefCookies::DeleteCb( GtkWidget*, UiPrefCookies* inst )
{
    inst->DeleteSelection();
}

//---------------------------------------------------------------------------
//
void UiPrefCookies::ListClickCb( GtkWidget*, 
                                 gint column,
                                 UiPrefCookies* inst )
{
   if( inst->m_nCurrentSortColumn == column ) 
   {
      inst->m_bAscendingSort = ! inst->m_bAscendingSort;
   }

   inst->m_nCurrentSortColumn = column;

   if( inst->m_bAscendingSort )
   {
      gtk_clist_set_sort_type( GTK_CLIST( inst->m_clist), 
         GTK_SORT_ASCENDING );
   }
   else
   {
      gtk_clist_set_sort_type( GTK_CLIST( inst->m_clist), 
         GTK_SORT_DESCENDING );
   }
   gtk_clist_set_sort_column( GTK_CLIST( inst->m_clist), column );
   gtk_clist_sort( GTK_CLIST( inst->m_clist ));
}


//---------------------------------------------------------------------------
//
void UiPrefCookies::ToggleCb( GtkToggleButton* widget, 
                              UiPrefCookies* inst)
{
    ZKASSERT( inst->m_isUserNym );
    bool bCookieActive = gtk_toggle_button_get_active( widget );
    gtk_widget_set_sensitive( inst->m_clist, !bCookieActive );
    gtk_widget_set_sensitive( inst->m_selectall, !bCookieActive );
    gtk_widget_set_sensitive( inst->m_delete, !bCookieActive );
}


//---------------------------------------------------------------------------
//
void UiPrefCookies::DeleteSelection()
{

    GList   *dlist;

    // Fetch the doubly linked list of selected items
    // of the List, remember to treat this as read-only!
    dlist = GTK_CLIST( m_clist )->selection;
    
    if( !dlist ) 
    {
        // Nothing selected, just return.
        // FIXME - we should probably just have disabled the delete button.
        return;
    }


    // Now remove those items from the ZKSTD::set.
    // (this is arguably a very inefficient way of doing it)
    int row;
    gchar* strVal;
    gchar* strDom;

    while (dlist) {
        row = GPOINTER_TO_INT( dlist->data );

        gtk_clist_get_text( GTK_CLIST( m_clist ), row, 0, &strDom );
        gtk_clist_get_text( GTK_CLIST( m_clist ), row, 3, &strVal );
        EraseCookie( strVal, strDom );

        dlist = dlist->next;
    }

    UpdateCookieList();
}

//---------------------------------------------------------------------------
//
void UiPrefCookies::UpdateCookieList()
{
	COOKIEJARMAP::iterator iter;

    gchar* cells[ 4 ];
    gtk_clist_freeze( GTK_CLIST( m_clist ) );

    gtk_clist_clear(  GTK_CLIST( m_clist ) );

    int row;

	for( iter = m_CookieMap.begin(); 
         iter != m_CookieMap.end(); 
         iter++ )
	{
        cells[0] = const_cast<gchar*>( (*iter).second.Domain.c_str() );
        cells[1] = const_cast<gchar*>( (*iter).second.DateRecd.c_str());
        cells[2] = const_cast<gchar*>( (*iter).second.DateExpire.c_str());
        cells[3] = const_cast<gchar*>( (*iter).first.c_str() );

        row = gtk_clist_append( GTK_CLIST( m_clist ), cells );
	}

    gtk_clist_thaw( GTK_CLIST( m_clist ) );
}

//---------------------------------------------------------------------------
//
void UiPrefCookies::EraseCookie( const ZKSTD::string& val, const ZKSTD::string& dom )
{
    COOKIEJARMAP::iterator theIterator;

    ZKSTD::pair<COOKIEJARMAP::iterator, COOKIEJARMAP::iterator> pos;

    // Look for range of cookies with the same value.
    // FIXME - why not look through the list ? is this faster ?
    // FIXME - and why is this data arranged in such a strange way ?.....
    for( pos = m_CookieMap.equal_range( val ) ; 
         pos.first != pos.second; 
         ++pos.first ){

        // Then look for the given domain.
        theIterator = pos.first;
			
        if( (*theIterator).second.Domain == dom ){
            m_CookieMap.erase( theIterator );
            SetModified( true );
            break;
        }
    }
}
