/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UIPREFCOOKIES
#define ZKS_INCL_UIPREFCOOKIES

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#ifndef ZKS_INCL_UIPREFBASE
#include <UiPrefBase.h>
#endif

#include <INym.h>
#include <INymCookieMgr.h>

/*===========================================================================
 * CLASS UiPrefCookies
 *=========================================================================*/

class UiPrefCookies : public UiPrefBase {

public: 

    /*----- member functions -----*/

    // Note: apparently there is no way of determining if a nym is teh actual 
    //       "user" nym ,so specify it in the constructor.
    UiPrefCookies( 
        const ZKSTD::string& name,
        const ZKSTD::string& desc,
        INym* pNym,
        bool isUserNym = false
    );

    virtual ~UiPrefCookies();

    virtual void Initialize();
    virtual bool Finalize();


private: 

    /*----- static member functions -----*/

    static void SelectAllCb( GtkWidget*, UiPrefCookies* );
    static void DeleteCb( GtkWidget*, UiPrefCookies* );
    static void ListClickCb( GtkWidget*, gint, UiPrefCookies*);
    static void ToggleCb( GtkToggleButton*, UiPrefCookies*);

    /*----- member functions -----*/


    void EraseCookie( const ZKSTD::string& val, const ZKSTD::string& dom ) ;
    void UpdateCookieList();

    void DeleteSelection();

    /*----- static data members -----*/

    /*----- data members -----*/

    GtkWidget*   m_selectall;
    GtkWidget*   m_delete;
    GtkWidget*   m_clist;
    GtkWidget*   m_txt;
    GtkToggleButton* m_passthru;
    GtkToggleButton* m_block;
    GtkToggleButton* m_enable; 

    COOKIEJARMAP m_CookieMap;

    INym* m_pNym;
    bool  m_isUserNym;
    bool  m_bAscendingSort;
    gint  m_nCurrentSortColumn;
};
#endif
