/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UIPREFDLG
#define ZKS_INCL_UIPREFDLG

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#ifndef ZKS_INCL_ZKSINGLETON
#include <ZkSingleton.h>
#endif

#include <clientstl.h>
#include <memory>
#include <string>

/*----- forward declarations -----*/

class UiPrefBase;
class UiPrefDlgImp;


/*===========================================================================
 * CLASS UiPrefDlg
 *=========================================================================*/

/*
  This class is the placeholder for all preferences pages.
  It contains the page tree on the left and the different pages on 
  the right. 
  Selecting an entry in the tree displays the corresponding page.
*/

class UiPrefDlg : public ZkSingleton< UiPrefDlg > {

public: 

    /*----- member functions -----*/

    UiPrefDlg();
    ~UiPrefDlg();
      
    static void Open( const ZKSTD::string& page = "" );
  
	// Purpose:  Will add the dialog to the tree and give the tree entry the
    //           specified label. If a node name is given, it is looked for 
    //           in the tree and the dialog gets added as a sub-node, otherwise
    //           it just gets added to the root node.
	// Promise:  Will steal your pointer through the auto_ptr copy so that
    //           you can't shoot yourself in the foot.
    // FIXME:    Throw something if the given node is not found.
    // FIXME:    Use standard string here.
    static void AddDialog( 
        const ZKSTD::string& label,
        ZKSTD::auto_ptr<UiPrefBase> dialog, 
        const ZKSTD::string& nodeName = ""
    );

	// Purpose:  Sets the currently displayed dialog and selects the 
    //           corresponding node in the tree.
	// Throws:   FIXME - throw if not in the tree.
    static void SetActiveDialog( const ZKSTD::string& name );

    // Purpose:  Disables the given page along with all its children.
    static void DisablePage( const ZKSTD::string& name );
    
    // Purpose:  Deletes all the child pages of the given page.
    //           Used to remove all nym pages from the "Identities" node.
    static void DeleteSubPages( const ZKSTD::string& pageName );

private: 

    /*----- member functions -----*/

    // prohibited funcs
    UiPrefDlg( const UiPrefDlg& );
    const UiPrefDlg& operator=( const UiPrefDlg& );

    /*----- data members -----*/

    UiPrefDlgImp* m_impl;

};
#endif
