/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UIPREFDLGIMP
#define ZKS_INCL_UIPREFDLGIMP

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include <clientstl.h>

#include <gnome.h>
#include <memory>
#include <string>

#include <map>

/*----- forward declarations -----*/

class UiPrefBase;


/*===========================================================================
 * CLASS UiPrefDlgImp
 *=========================================================================*/

/*
  Contains gnome specific part of the Preferences dialogs box.
  For more details on this dialog please see ZkPrefDlg.h
*/

class UiPrefDlgImp {

public: 

    /*----- member functions -----*/

    UiPrefDlgImp();
    ~UiPrefDlgImp();

    void Open( const ZKSTD::string& page = "" );

	// Purpose:  Will add the dialog to the tree and give the tree entry the
    //           specified label. If a node name is given, it is looked for 
    //           in the tree and the dialog gets added as a sub-node, otherwise
    //           it just gets added to the root node.
	// Promise:  Will steal your pointer through the auto_ptr copy so that
    //           you can't shoot yourself in the foot.
    void AddDialog( 
        const ZKSTD::string& label,
        ZKSTD::auto_ptr<UiPrefBase> dialog, 
        const ZKSTD::string& nodeName
    );

	// Purpose:  Sets the currently displayed dialog and selects the 
    //           corresponding node in the tree.
    void SetActiveDialog( const ZKSTD::string& name );

    // Purpose:  Disables the given page along with all its children.
    void DisablePage( const ZKSTD::string& name );

    // Purpose:  Looks for a page by its name, returns 0 if not found.
    GtkWidget* FindPage( const ZKSTD::string& pageName );
    
    void DeleteSubPages( const ZKSTD::string& name );

private: 

    /*----- typedefs -----*/

    typedef ZKSTD::pair< GtkWidget*, UiPrefBase* > TreePair;
    typedef ZKSTD::map< ZKSTD::string, TreePair > TreeMap;

    /*----- static member functions -----*/

    static gint CloseCb( GtkWidget *, gpointer );

    static void DestroyCb( GtkWidget *, UiPrefDlgImp*  );

    static void SelectCb (
        GtkWidget*    child, 
        UiPrefDlgImp* inst
    );


    /*----- member functions -----*/

    void SetCurrentDialog( UiPrefBase* );

    void InitializeAll();
    bool FinalizeAll();    
    void CancelAll();

    /*----- data members -----*/

    GtkWidget* m_ct;
    GtkWidget* m_tree;
    GtkWidget* m_dialog;

    UiPrefBase* m_currentDialog;

    int m_max_w;
    int m_max_h;

    TreeMap m_treeMap;
};


#endif
