// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPrefFreedomMail.h>
#include <UiMessages.h>
#include <ZkGladeLoader.h>
#include <IZkPopup.h>
#include <ZkStringMgr.h>
#include <ZkDialogsStrings.h>

#include <INym.h>
#include <INymCentral.h>
#include <ZkService.h>
#include <IClientPersist.h>
#include <IClientCentral.h>
#include <zksocket.h>
#include <clientdebug.h>

//===========================================================================
// CLASS UiPrefFreedomMail
//===========================================================================

//---------------------------------------------------------------------------
//
UiPrefFreedomMail::UiPrefFreedomMail( 
    const ZKSTD::string& name,
    const ZKSTD::string& desc
) 
    : UiPrefBase( name, desc )
{
    ZkGladeLoader loader( "pref_freedom_mail.xml", "topct" );

    SetTopContainer( loader.GetWidget( "topct" ) );
    m_checkall    = GTK_TOGGLE_BUTTON( loader.GetWidget( "radio_checkall" ));
    m_checkactive = GTK_TOGGLE_BUTTON( loader.GetWidget( "radio_checkactive" ));
    m_mailrelay   = GTK_ENTRY( loader.GetWidget( "entry_mailrelay" ));
}

//---------------------------------------------------------------------------
//
UiPrefFreedomMail::~UiPrefFreedomMail( )
{
}

//---------------------------------------------------------------------------
//
void UiPrefFreedomMail::Initialize()
{
    if( ZkService::GetIClientCentral()->GetPreference(ZK_CLIENT_PREF_SET, 
                                                      PREF_CHECK_MAIL_FOR_ACTIVE_NYM_ONLY ))
    {
        gtk_toggle_button_set_active( m_checkactive, true );
    }
    else
    {
        gtk_toggle_button_set_active( m_checkall, true );
    }
    ZKSTD::string strRelay;
    ZkService::GetIClientCentral()->GetMailRelay(strRelay);
    gtk_entry_set_text( m_mailrelay, strRelay.c_str() );
}

//---------------------------------------------------------------------------
//
bool UiPrefFreedomMail::Finalize()
{
    bool ReturnValue = true;
    gchar *strMailRelay = gtk_entry_get_text( m_mailrelay );
    ZKSTD::string strOldRelay;
    ZkService::GetIClientCentral()->GetMailRelay( strOldRelay );
    if( strOldRelay != strMailRelay )
    {
        T_ZkInetAddr    zkInetAddr;
        if(!parse_inet_addr( NULL, strMailRelay, &zkInetAddr ))
        {
            ZKSTD::string strMsg;
            ZkStringMgr::Load( strMsg, IDS_INVALID_MAIL_RELAY );
            ReturnValue = IZkPopup::QuestionBox( strMsg.c_str(), 
                                                 ZK_OKCANCEL, 
                                                 ZK_QUESTION, 
                                                 false );
        }
      
        if( ReturnValue )
        {
            ZkService::GetIClientCentral()->SetMailRelay( strMailRelay );
        }
    }

    if( ReturnValue )
    {
        bool bCheckActive = gtk_toggle_button_get_active( m_checkactive );

        if( bCheckActive != ZkService::GetIClientCentral()->GetPreference( ZK_CLIENT_PREF_SET, PREF_CHECK_MAIL_FOR_ACTIVE_NYM_ONLY ) ){
            ZkService::GetIClientCentral()->SetPreference( ZK_CLIENT_PREF_SET, 
                                                           PREF_CHECK_MAIL_FOR_ACTIVE_NYM_ONLY, 
                                                           bCheckActive ) ;
        }
    }

    return ReturnValue;
}
