// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPrefInit.h>

#include <UiPrefDlg.h>
#include <UiPrefBase.h>
#include <UiPrefMain.h>
#include <UiPrefWordScan.h>
#include <UiPrefPassphrase.h>
#include <UiPrefBackup.h>
#include <UiPrefConsole.h>
#include <UiPrefAbout.h>
#include <UiPrefSecurity.h>
#include <UiPrefSpam.h>
#include <UiPrefProNymAd.h>
#include <UiPrefCookies.h>
#include <UiPrefUser.h>
#include <UiPrefNym.h>
#include <UiPrefAd.h>
#include <UiPrefFreedomMail.h>

#include <UiThreadId.h>

#include <INym.h>
#include <INymCentral.h>
#include <ZkService.h>
#include <ZkStringMgr.h>
#include <ZKFileConfiguration.h>
#include <ZkDialogsStrings.h>
#include <clientstl.h> // for the ... namespace
#include <memory> // auto_ptr


//===========================================================================
// Namespace UiPrefInit
//===========================================================================

namespace UiPrefInit 
{

//---------------------------------------------------------------------------
//
void AddPrefsPages()
{
    ZKSTD::auto_ptr< UiPrefBase > dlg1( new UiPrefMain( "preferences", "Preferences" ) );
    ZKSTD::auto_ptr< UiPrefBase > dlg2( new UiPrefWordScan( "wordscan", "Keyword Alert" ) );
    ZKSTD::auto_ptr< UiPrefBase > dlg3( new UiPrefPassphrase( "passphrase", "Passphrase" ) );
    ZKSTD::auto_ptr< UiPrefBase > dlg4( new UiPrefFreedomMail( "freedommail", "Freedom Email" ) );

    ZKSTD::auto_ptr< UiPrefBase > dlg5( new UiPrefBackup( "backup", "Backup" ) );
    ZKSTD::auto_ptr< UiPrefBase > dlg6( new UiPrefAd( "ad", "Ad Manager" ) );
    ZKSTD::auto_ptr< UiPrefBase > dlg7( new UiPrefAbout( "about", "About Freedom" ) );
    ZKSTD::auto_ptr< UiPrefBase > dlg8( new UiPrefConsole( "console", "Console" ) );
    ZKSTD::auto_ptr< UiPrefBase > dlg10( new UiPrefBase( "Identities", "Identities" ) );


    UiPrefDlg::AddDialog( "Preferences",  dlg1 );
    UiPrefDlg::AddDialog( "Keyword Alert",    dlg2);
    UiPrefDlg::AddDialog( "Passphrase",   dlg3 );
    UiPrefDlg::AddDialog( "Freedom Email",dlg4 );
    UiPrefDlg::AddDialog( "Backup",       dlg5 );
    UiPrefDlg::AddDialog( "Ad Manager",   dlg6 );
    UiPrefDlg::AddDialog( "About",        dlg7 );
    UiPrefDlg::AddDialog( "Console",      dlg8 );
    UiPrefDlg::AddDialog( "Identities",   dlg10 );

    UiPrefDlg::SetActiveDialog( "preferences" );
}

//---------------------------------------------------------------------------
//
void AddNymPages( )
{
    UiThreadGuard dog;
    ZKSTD::string strSecuritySettings; 
    ZKSTD::string strNymEmail; 
    ZKSTD::string strCookieJar;
    ZKSTD::string strTrueIdentity;
    ZKSTD::string strUser( "nym-User" );

    ZkStringMgr::Load( strSecuritySettings, IDS_SECSET_TITLE );
    ZkStringMgr::Load( strNymEmail, IDS_NYMMAIL_TITLE );
    ZkStringMgr::Load( strCookieJar, IDS_COOKIEJAR_TITLE );
    ZkStringMgr::Load( strTrueIdentity, IDS_TRUEID_TITLE );


	INym * pNym = 0;
    ZKSTD::string nymName;    

    // "user" pages
    ZkService::GetINymCentral()->GetNym( IID_INYM, (void**)&pNym, 0 );
    ZKSTD::auto_ptr< UiPrefBase > trueDlg( new UiPrefUser( strUser+strNymEmail, strTrueIdentity.c_str() ) );
    ZKSTD::auto_ptr< UiPrefBase > cookieDlg( new UiPrefCookies( strUser+strCookieJar, strCookieJar.c_str(), pNym, true ) );

    UiPrefDlg::AddDialog( strTrueIdentity.c_str(), trueDlg  , "Identities" );
    UiPrefDlg::AddDialog( strCookieJar.c_str(), cookieDlg, strUser+strNymEmail );    

	int cntNyms = ZkService::GetINymCentral()->GetNumberOfNyms() ;

        
    // Load the lock icon for the top right image
    ZKSTD::string fPath = ZKFileConfiguration::GetFreedomPixmapDir();
    fPath += "lock.xpm";

    GdkPixmap* lockPixmap;
    GdkBitmap* lockBitmapMask;
    
    GdkColormap *colormap = gdk_colormap_get_system();
    
    lockPixmap = gdk_pixmap_colormap_create_from_xpm( NULL,
                                                      colormap,  
                                                      &lockBitmapMask,
                                                      NULL,
                                                      fPath.c_str() );
    
    for( int i = 0; i < cntNyms; i++ )
    {					 
		ZkService::GetINymCentral()->GetNym( IID_INYM, (void**)&pNym, i );

		unsigned long dwLifeStage = NYM_LIFE_STAGE_UNDEFINED ;

		if( i != USER_NYM_INDEX ) 
        {
            pNym->GetNymName( nymName );
			pNym->GetYourLifeStage( dwLifeStage ) ;
	
            ZKSTD::string pageName( "nym-" );
            pageName += nymName;
            
            if ( dwLifeStage == NYM_LIFE_STAGE_ACTIVE )
            {
                ZKSTD::string pageNameCookie;
                pageNameCookie = nymName + " - " + strCookieJar;

                ZKSTD::string pageNameSec;
                pageNameSec = nymName + " - " + strSecuritySettings;

                ZKSTD::string pageNameMail;
                pageNameMail = nymName + " - " +  strNymEmail;


                // Creates the set of preferences pages corresponding to a nym.
                ZKSTD::auto_ptr< UiPrefBase > nymDlg( 
                    new UiPrefNym( pageName, nymName, pNym ) );
                ZKSTD::auto_ptr< UiPrefBase > nymCookie( 
                    new UiPrefCookies( pageName+strCookieJar, pageNameCookie, pNym ) );
                ZKSTD::auto_ptr< UiPrefBase > nymSec( 
                    new UiPrefSecurity( pageName+strSecuritySettings, pageNameSec, pNym ) );

                // add a page depending of the kind of nym
                ZKSTD::auto_ptr< UiPrefBase > nymMail;
                if( pNym->DoesNymUseMail() )
                {
                    // this is the mail page for a nym that support mail
                    nymMail = new UiPrefSpam( pageName+strNymEmail, pageNameMail, pNym );
                }
                else
                {
                    // here we show publicity for Pro Nyms
                    nymMail = new UiPrefProNymAd( pageName+strNymEmail, pageNameMail, pNym );
                }

                // set the lock pixmap if the nym is pro
                if( pNym->DoesNymUseMail() && pNym->DoesNymUseCloud() )
                {
                    nymDlg->SetPixmap( lockPixmap , lockBitmapMask);
                    nymCookie->SetPixmap( lockPixmap , lockBitmapMask);
                    nymSec->SetPixmap( lockPixmap , lockBitmapMask);
                    nymMail->SetPixmap( lockPixmap , lockBitmapMask);
                }                                
                
                UiPrefDlg::AddDialog( nymName.c_str(), nymDlg, "Identities" );
                UiPrefDlg::AddDialog( strSecuritySettings.c_str(), 
                   nymSec, pageName.c_str() );
                UiPrefDlg::AddDialog( strCookieJar.c_str(),  
                   nymCookie, pageName.c_str() );    
                UiPrefDlg::AddDialog( strNymEmail.c_str(),  
                   nymMail, pageName.c_str() );    

                if( !pNym->DoesNymUseCloud() )
                {
                    UiPrefDlg::DisablePage( pageName+strSecuritySettings );
                }
            }
            else if( dwLifeStage == NYM_LIFE_STAGE_EXPIRED )
            {
                ZKSTD::auto_ptr< UiPrefBase > nymDlg( 
                    new UiPrefNym( pageName, nymName, pNym ) );
                
                UiPrefDlg::AddDialog( nymName.c_str(), nymDlg, "Identities" );
            }
		}
    }
}

//---------------------------------------------------------------------------
//
void RemoveNymPages( )
{
    UiPrefDlg::DeleteSubPages( "Identities" );
}

}

