// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPrefMain.h>
#include <ZkGladeLoader.h>

#include <ZkService.h>
#include <IClientCentral.h>
#include <clientdebug.h>

//===========================================================================
// CLASS UiPrefMain
//===========================================================================

//---------------------------------------------------------------------------
//
UiPrefMain::UiPrefMain( 
    const ZKSTD::string& name,
    const ZKSTD::string& desc 
) 
    : UiPrefBase( name, desc )
{
    ZkGladeLoader loader( "prefmain.xml", "topct" );

    SetTopContainer( loader.GetWidget( "topct" ));

    m_restore     = loader.GetWidget( "restore" );
    m_onTopTog    = GTK_TOGGLE_BUTTON( loader.GetWidget( "on_top" ));
    m_newsWarnTog = GTK_TOGGLE_BUTTON( loader.GetWidget( "news_warning" ));
    m_mailTog     = GTK_TOGGLE_BUTTON( loader.GetWidget( "outgoing_mail" ));
    m_newsTog     = GTK_TOGGLE_BUTTON( loader.GetWidget( "outgoing_news" ));
    m_signedTog   = GTK_TOGGLE_BUTTON( loader.GetWidget( "outgoing_signed" ));
    m_sslWarnTog  = GTK_TOGGLE_BUTTON( loader.GetWidget( "ssl_warning"));
    m_asNymTog    = GTK_TOGGLE_BUTTON( loader.GetWidget( "send_as_nym_warning" ));

    gtk_signal_connect (GTK_OBJECT( m_restore ), "clicked",
                        GTK_SIGNAL_FUNC( RestoreCb ), this);
}

//---------------------------------------------------------------------------
//
UiPrefMain::~UiPrefMain( )
{
}

//---------------------------------------------------------------------------
//
void UiPrefMain::Initialize()
{
    bool flag;

	flag = ZkService::GetIClientCentral()->GetPreference( ZK_TOOLBAR_PREF_SET, 
                                                          PREF_NOT_ALWAYS_ON_TOP );
    gtk_toggle_button_set_active( m_onTopTog, !flag );

  	flag = ZkService::GetIClientCentral()->GetPreference( ZK_SHOW_AGAIN_PREF_SET, 
                                                          PREF_NO_SHOW_NEWS_OPTIONS );
    gtk_toggle_button_set_active( m_newsWarnTog, flag );

	flag = ZkService::GetIClientCentral()->GetPreference(ZK_SHOW_AGAIN_PREF_SET, 
                                                         PREF_NO_SHOW_OUTGOING_MAIL );
    gtk_toggle_button_set_active( m_mailTog, flag );

	flag = ZkService::GetIClientCentral()->GetPreference(ZK_SHOW_AGAIN_PREF_SET, 
                                                         PREF_NO_SHOW_OUTGOING_NEWS );
    gtk_toggle_button_set_active( m_newsTog, flag );

	flag = ZkService::GetIClientCentral()->GetPreference(ZK_SHOW_AGAIN_PREF_SET, 
                                                         PREF_NO_WARN_SIGNED_MAIL );
    gtk_toggle_button_set_active( m_signedTog, flag );

	flag = ZkService::GetIClientCentral()->GetPreference(ZK_SHOW_AGAIN_PREF_SET, 
                                                         PREF_NO_SHOW_SSL_WARNING );
    gtk_toggle_button_set_active( m_sslWarnTog, flag );

	flag = ZkService::GetIClientCentral()->GetPreference(ZK_SHOW_AGAIN_PREF_SET, 
                                                         PREF_NO_SHOW_MAIL_SENT_AS_ACTIVE_NYM );
    gtk_toggle_button_set_active( m_asNymTog, flag );
}

//---------------------------------------------------------------------------
//
bool UiPrefMain::Finalize()
{
    // Get the state from each toggle and set it in the preferences.
    bool flag;

    flag = gtk_toggle_button_get_active( m_onTopTog );
	SetIfChanged( ZK_TOOLBAR_PREF_SET, PREF_NOT_ALWAYS_ON_TOP, !flag ) ;	

    flag = gtk_toggle_button_get_active( m_mailTog );
	SetIfChanged( ZK_SHOW_AGAIN_PREF_SET, PREF_NO_SHOW_OUTGOING_MAIL, flag) ;	

    flag = gtk_toggle_button_get_active( m_newsTog );
	SetIfChanged( ZK_SHOW_AGAIN_PREF_SET, PREF_NO_SHOW_OUTGOING_NEWS, flag) ;	

    flag = gtk_toggle_button_get_active( m_signedTog );
	SetIfChanged( ZK_SHOW_AGAIN_PREF_SET, PREF_NO_WARN_SIGNED_MAIL, flag) ;	

    flag = gtk_toggle_button_get_active( m_newsWarnTog );
	SetIfChanged( ZK_SHOW_AGAIN_PREF_SET, PREF_NO_SHOW_NEWS_OPTIONS, flag) ;	

    flag = gtk_toggle_button_get_active( m_sslWarnTog );
	SetIfChanged( ZK_SHOW_AGAIN_PREF_SET, PREF_NO_SHOW_SSL_WARNING, flag) ;	

    flag = gtk_toggle_button_get_active( m_asNymTog );
	SetIfChanged( ZK_SHOW_AGAIN_PREF_SET, PREF_NO_SHOW_MAIL_SENT_AS_ACTIVE_NYM, flag) ;	

    return true;
}

//---------------------------------------------------------------------------
//
void UiPrefMain::RestoreCb( GtkWidget*, UiPrefMain* inst )
{
    inst->RestoreDefaults();
}

//---------------------------------------------------------------------------
//
void UiPrefMain::RestoreDefaults()
{
    gtk_toggle_button_set_active( m_onTopTog, true );
    gtk_toggle_button_set_active( m_newsWarnTog, false );
    gtk_toggle_button_set_active( m_mailTog, false );
    gtk_toggle_button_set_active( m_newsTog, false );
    gtk_toggle_button_set_active( m_signedTog, false );
    gtk_toggle_button_set_active( m_sslWarnTog, false );
    gtk_toggle_button_set_active( m_asNymTog, false );
}

//---------------------------------------------------------------------------
//
void UiPrefMain::SetIfChanged( 
    long in_dwPrefSet, 
    long in_dwProperty, 
    bool in_bNewValue 
)
{
	bool bOldValue = ZkService::GetIClientCentral()->GetPreference(in_dwPrefSet, in_dwProperty);

	if ( bOldValue != in_bNewValue ) {
		ZkService::GetIClientCentral()->SetPreference(in_dwPrefSet, in_dwProperty, in_bNewValue );
	}
}
