/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UIPREFMAIN
#define ZKS_INCL_UIPREFMAIN

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#ifndef ZKS_INCL_UIPREFBASE
#include <UiPrefBase.h>
#endif

/*===========================================================================
 * CLASS UiPrefMain
 *=========================================================================*/

class UiPrefMain : public UiPrefBase {

public: 

    /*----- member functions -----*/

    UiPrefMain( 
        const ZKSTD::string& name,
        const ZKSTD::string& desc 
    );

    virtual ~UiPrefMain();

	// Purpose:  Reads the user preferences from the refitry and sets the state of
    //           the toggle buttons accordingly.
    virtual void Initialize();

    virtual bool Finalize();


private: 

    /*----- static member functions -----*/

    static void ToggleCb( GtkWidget*, UiPrefMain* );    
    static void RestoreCb( GtkWidget*, UiPrefMain* );

    /*----- member functions -----*/


	// Purpose:  Checks a flag in the registry against a toggle state in the ui and sets
    //           the registry flag to the toggle state if they differ.
    void SetIfChanged( 
        long in_dwPrefSet, 
        long in_dwProperty, 
        bool in_bNewValue 
    );

	// Purpose:  Resets the toggle buttons to their state when Freedom first started.
    void RestoreDefaults();

    /*----- data members -----*/

    GtkWidget*       m_restore;
    GtkToggleButton* m_onTopTog;
    GtkToggleButton* m_newsWarnTog;
    GtkToggleButton* m_mailTog;
    GtkToggleButton* m_newsTog; 
    GtkToggleButton* m_signedTog; 
    GtkToggleButton* m_sslWarnTog;
    GtkToggleButton* m_asNymTog;
};
#endif
