// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):




//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPrefNym.h>

#include <clientdebug.h>
#include <clientdefs.h>
#include <INym.h>
#include <INymCentral.h>
#include <ITokenService.h>
#include <IZkPopup.h>
#include <IClientPersist.h>
#include <IZkConsoleMsg.h>
#include <ZkService.h>
#include <IZkPopup.h>
#include <IZkConsoleMsg.h>
#include <ZKConfiguration.h>
#include <WNymIconInfo.h>
#include <ZkDialogsStrings.h>
#include <ZkErrorHandler.h>
#include <IZKErrorReporter.h>
#include <RZkException.h>
#include <ZkWebIntegration.h>
#include <ZkStringMgr.h>
#include <ZkGladeLoader.h>
#include <UiEnterSerialNumber.h>
#include <UiExtendNym.h>
#include <UiMainWindow.h>
#include <ZkWebIntegration.h>
#include <ZkGtkUtils.h>


//===========================================================================
// CLASS UiPrefNym
//===========================================================================

//---------------------------------------------------------------------------
//
UiPrefNym::UiPrefNym( 
    const ZKSTD::string& name,
    const ZKSTD::string& desc,
    INym* pNym
) 
    : UiPrefBase( name, desc )
    , m_pNym( pNym )
{
    ZkGladeLoader loader( "pref_nymmain.xml", "PrefNym_topct" );

    SetTopContainer( loader.GetWidget( "PrefNym_topct" ));

    m_frameDetails  = loader.GetWidget( "frameDetails" );    
    m_frameDesc     = loader.GetWidget( "frameDesc" );
    m_nymTypeDesc   = loader.GetWidget( "nymTypeDesc" );
    m_hrefSubscribe = loader.GetWidget( "hrefSubscribe" );
    m_icon          = loader.GetWidget( "Icon" );
    m_extend        = loader.GetWidget( "Extend" );
    m_delete        = loader.GetWidget( "Delete" );
    m_notes         = loader.GetWidget( "Notes" );
    m_birth         = loader.GetWidget( "Birth" );
    m_retire        = loader.GetWidget( "Retire" );

    gtk_signal_connect (GTK_OBJECT( m_extend ), "clicked",
                        GTK_SIGNAL_FUNC( ExtendCb ), this);
    gtk_signal_connect (GTK_OBJECT( m_delete ), "clicked",
                        GTK_SIGNAL_FUNC( DeleteCb ), this);
    gtk_signal_connect (GTK_OBJECT(((GnomeIconEntry*)m_icon)->pickbutton ), "clicked",
                        GTK_SIGNAL_FUNC( IconPickCb), this );
 
    gnome_href_set_url( GNOME_HREF( m_hrefSubscribe ), 
                        ZkWebIntegration::GetURL( URL_PREMIUM_INFO_ID ).c_str() );    

}

//---------------------------------------------------------------------------
//
UiPrefNym::~UiPrefNym( )
{
}

//---------------------------------------------------------------------------
//
void UiPrefNym::Initialize()
{
    ReadNymInfo( );
}

//---------------------------------------------------------------------------
//
bool UiPrefNym::Finalize()
{
    // Check whether or not the icon has changed.
    gchar* iconPathPtr = gnome_icon_entry_get_filename(
        GNOME_ICON_ENTRY( m_icon ) );

    WNymIconInfo iconInfo;
    if( iconPathPtr != 0 ) 
    {
        ZKSTD::string iconPath( iconPathPtr );
            
        if( m_current_icon != iconPath ) 
        {
            ZKTRACE1( "Changing icon to: %s\n", iconPath.c_str() );
            iconInfo.SetIconFilePath( iconPath );            
            m_pNym->SetNymIconInfo( iconInfo );
        }
    }
    else 
    {
        // if the path was never set, or if it is invalid, clear it.
        iconInfo.SetIconFilePath( "" );
        m_pNym->SetNymIconInfo( iconInfo );
    }

    // Check notes as well.
    ZKSTD::string notes = gtk_editable_get_chars( GTK_EDITABLE( m_notes ),
                                                  0, -1 );
    if( m_current_notes != notes ) 
    {
        ZKTRACE1( "Changing notes to: %s\n", notes.c_str() );        
        m_pNym->SetNymPrivateNotes( notes );
    }


    return true;
}


//---------------------------------------------------------------------------
//
void UiPrefNym::ExtendCb( GtkWidget*, UiPrefNym* in_inst)
{
  	bool bSuccess = false;

 	if( ! in_inst->m_pNym->DoesNymUseCloud() && ! in_inst->m_pNym->DoesNymUseMail() )
	{
		// Free nym
		// We send the user to a Web Page and we ask him to enter a serial number
		// that he can get from the web page
 
	    ZkWebIntegration::LaunchWebSite( URL_BUY_SERIAL_NUM );

	    // show serial Number dialog
        UiEnterSerialNumber serialNumberDlg;
        if(serialNumberDlg.Run())
        {
    	    bSuccess = in_inst->GetTokensAndUseIt(serialNumberDlg.GetSerialNumber());
        }
    }
    else
    {
 		// Mail nym or full nym - Type 2 or Type 1
		// we show existing tokens
		// and we ask him to choose from or to get new ones
	    // show serial Number dialog
                
        UiExtendNym extendNymDlg(in_inst->m_pNym);
        bSuccess = extendNymDlg.Run();
    }
        
    if( bSuccess )
	{
        in_inst->UpdateRetireDate();
	}
}

//---------------------------------------------------------------------------
//
void UiPrefNym::IconPickCb( GtkWidget*, UiPrefNym* inst )
{
    ZkGtkUtils::GetAndSetTopPref( ((GnomeIconEntry*)inst->m_icon)->fentry);
}

//---------------------------------------------------------------------------
//
void UiPrefNym::DeleteCb( GtkWidget*, UiPrefNym* inst )
{
#ifdef _DEBUG
    ZKSTD::string strName;
    inst->m_pNym->GetNymName(strName);
    ZKTRACE1("[%s] is getting smoked\n", strName.c_str());
//    ZKTRACE2("[%s] which is [%d] is getting smoked\n", strName.c_str(), m_iNym);
#endif

	if( IZkPopup::QuestionBox( IDS_RETIRE_ARE_YOU_SURE, ZK_YESNO ))
	{
		ZKASSERT( inst->m_pNym != 0 );
		
		try 
		{
			// Retire the Nym
// FIMXME - look buzy
/*
			CWaitCursor	TheseArentTheDroidsDude; 
			ZKBusyAVI busy;
*/
			IZkConsoleMsg::ShowMsgForceUpdate( IDS_CONS_RETIRE_NYM );

			inst->m_pNym->DisableNymOnNymServer();

            // Well... convoluted but does the job, given the api.
            size_t iNym;
            ZKSTD::string nymName;
            ZKSTD::string nymNameToSet = "";
            inst->m_pNym->GetNymName( nymName );
            ZkStringMgr::Load( nymNameToSet, IDS_NYM_DELETED,
               nymName.c_str());
            inst->SetTitle( nymNameToSet.c_str() );

            inst->m_pNym->GetNymName( nymName );           
            ZkService::GetINymCentral()->GetNymIndex( nymName, iNym );
            
		    // we first check if the nym is currently selected, if so, we select the 
		    // user nym. Because deleting the current nym cause problems.
		    // bug 7900
		    size_t activeNymIndex = ZkService::GetINymCentral()->GetActiveNymIndex();
		    if(iNym == activeNymIndex)
		    {
			    // change to user nym
                UiMainWindow::get().ReselectUserNym();
		    }
            
			ZkService::GetINymCentral()->MarkNymAsDead( iNym );
			ZkService::GetIClientPersist()->SaveClientState();

            // First disable the page. It will be removed from the tree when we 
            // exit the dialog.
            // This call also disables the sub-nodes in the tree.
            inst->MarkForDelete();

			IZkConsoleMsg::ShowImportantMsg( IDS_CONS_NYM_DELETED ) ;
		}
		catch( IZkException& ex){
			ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
		}
	}
}

//---------------------------------------------------------------------------
//
bool UiPrefNym::GetTokensAndUseIt(const tstring in_strSerialNumber)
{        
	bool bRetVal = false;
    
	//CWaitCursor	TheseArentTheDroidsDude; 
	//ZKBusyAVI   busy;

	// UI Should only activate button if there is
	// a valid sized string
	ZKASSERT( in_strSerialNumber.size() == ZKSERIAL_NUMBER_LENGTH ) ;

	WSerialNumber serialNumber( in_strSerialNumber.c_str() ) ;

	try
	{
		ZkService::GetITokenService()->GetSerialNumberCapabilities( serialNumber ) ;

		IZkConsoleMsg::ShowMsgForceUpdate(IDS_CONS_REDEEM_SERIAL);
		
		ZKSTD::vector<tstring> vNewTokens;
		ZkService::GetITokenService()->RedeemSerialNumberForTokens( serialNumber, vNewTokens );

		// Save right now so that user doesn't lose the new tokens
		try 
		{
			ZkService::GetIClientPersist()->SaveClientState();
		}
		catch( IZkException&  )
		{
			// Not much positive we can say if this fails
		}

		// server should return one token, if he return more than one we take the first anyway
		ZKASSERT( vNewTokens.size() == serialNumber.GetTokenCount() == 1);
		
		// use that token
		if( vNewTokens.size() <= 0 )
		{
			// redeem failed
			throw RZkException("NymMainPage::GetTokensAndUseIt",
					"No tokens got from the activation code",
					IDS_DEFAULT_ERROR);
		}

		ZKSTD::string& szToken = vNewTokens[0];

		ZKSTD::string strNymName;
		m_pNym->GetNymName( strNymName ) ;

		// Extend the Nym
/*			if ( m_dwLifeStage == NYM_LIFE_STAGE_EXPIRED ) 
		{
			IZkConsoleMsg::ShowMsgForceUpdate(IDS_CONS_RENEW_NYM);
		}
		else 
		{
			IZkConsoleMsg::ShowMsgForceUpdate(IDS_CONS_EXTEND_NYM);
		}
*/
		m_pNym->AddTokenToNymOnNymServer(szToken);

		// After successful update we will no longer be expired
		m_pNym->SetYourLifeStage( NYM_LIFE_STAGE_ACTIVE ) ;

		ZKVERIFY(ZkService::GetITokenService()->RemoveTokenFromArray( szToken ) );
		ZkService::GetIClientPersist()->SaveClientState();

		bRetVal = true;
	}
	catch( IZkException& ex)
	{
		ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
	}

	return bRetVal;
}

//---------------------------------------------------------------------------
//
void UiPrefNym::ReadNymInfo()
{
    ZKASSERT( m_pNym != 0 );
    ZKSTD::string nymName;

    unsigned long lifeStage;
    m_pNym->GetYourLifeStage( lifeStage ) ;

    m_pNym->GetNymName( nymName );

    if ( lifeStage == NYM_LIFE_STAGE_EXPIRED ) {

        // Added "expired" to the nym 
        ZKSTD::string windowText( nymName ) ;
        ZKSTD::string text;
        ZkStringMgr::Load( text, IDS_EXPIRED_POSTFIX );
        windowText += text;

        SetTitle( windowText.c_str() );

        // User will renew the nym rather than extending it.
        text.clear();
        ZkStringMgr::Load( text, IDS_RENEW_BUTTON_TEXT );
        GtkLabel* label = GTK_LABEL( GTK_BIN( m_extend )->child );
        gtk_label_set_text( label, text.c_str() );
    }
    else {
        SetTitle( nymName.c_str() ) ;
    }	

    UpdateNotes();
    UpdateIcon();
    UpdateBirthDate();
    UpdateRetireDate();
    UpdateDescription();
}

       
//---------------------------------------------------------------------------
//
void UiPrefNym::UpdateBirthDate()
{
    ZKSTD::string strBirthDate;

    // Get nym dates
    try {
        m_pNym->GetNymBirthDate( strBirthDate );
    }
    catch( IZkException& ex) {
        ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
    }

    gtk_label_set_text( GTK_LABEL( m_birth ), strBirthDate.c_str() ); 
}

//---------------------------------------------------------------------------
//
void UiPrefNym::UpdateRetireDate()
{
    ZKSTD::string strRetireDate;

    // Get nym dates
    try {
        m_pNym->GetNymRetireDate( strRetireDate );
    }
    catch( IZkException& ex) {
        ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
    }

    gtk_label_set_text( GTK_LABEL( m_retire ), strRetireDate.c_str() ); 
}

//---------------------------------------------------------------------------
//
void UiPrefNym::UpdateIcon()
{
    WNymIconInfo info ;
    m_pNym->GetNymIconInfo( info ) ;

    if( info.GetIconType() == NymIconFilePath ) {
        m_current_icon = info.GetFilePath();
        gnome_icon_entry_set_icon( GNOME_ICON_ENTRY( m_icon ), 
                                   m_current_icon.c_str() );
    }
}

//---------------------------------------------------------------------------
//
void UiPrefNym::UpdateNotes()
{
    ZKSTD::string strPrivateNotes;
    m_pNym->GetNymPrivateNotes( strPrivateNotes );
	
    int pos = 0;
    
    gtk_editable_delete_text( GTK_EDITABLE( m_notes ), 0, -1 );
    if ( strPrivateNotes.length() > 0 ) {
        gtk_editable_insert_text( GTK_EDITABLE( m_notes ), 
                                  strPrivateNotes.c_str(),
                                  strPrivateNotes.length(), 
                                  &pos );
    }
    
    m_current_notes = strPrivateNotes;
}

//---------------------------------------------------------------------------
//
void UiPrefNym::UpdateDescription()
{
    ZKSTD::string strDesc;
    ZKSTD::string strFrameText;
    
    if( m_pNym->DoesNymUseMail() && m_pNym->DoesNymUseCloud() )
    {
        ZkStringMgr::Load( strDesc, IDS_PRO_NYM_DESCRIPTION );
        ZkStringMgr::Load( strFrameText, IDS_PRO_NYM_SELECTED );
        gtk_widget_hide( m_hrefSubscribe );
    }
    else
    {
        ZkStringMgr::Load( strDesc, IDS_FREE_NYM_DESCRIPTION );
        ZkStringMgr::Load( strFrameText, IDS_FREE_NYM_SELECTED );
        gtk_widget_hide( m_frameDetails );
    }
    
    gtk_frame_set_label( GTK_FRAME( m_frameDesc ), strFrameText.c_str());
    gtk_label_set_text( GTK_LABEL( m_nymTypeDesc ), strDesc.c_str());
}



