/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UIPREFNYM
#define ZKS_INCL_UIPREFNYM

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#ifndef ZKS_INCL_UIPREFBASE
#include <UiPrefBase.h>
#endif

#include <client.h>
#include <memory>

class INym;
class UiEnterSerialNumber;
class UiExtendNym;

/*===========================================================================
 * CLASS UiPrefNym
 *=========================================================================*/

class UiPrefNym : public UiPrefBase {

public: 

    /*----- member functions -----*/

    UiPrefNym( 
        const ZKSTD::string& name,
        const ZKSTD::string& desc,
        INym* pNym
    );

    virtual ~UiPrefNym();

    virtual void Initialize();
    virtual bool Finalize();


private: 

    /*----- static member functions -----*/

    static void ExtendCb( GtkWidget*, UiPrefNym* );    
    static void DeleteCb( GtkWidget*, UiPrefNym* );
    static void SecurityCb( GtkWidget*, UiPrefNym* );
    static void IconPickCb( GtkWidget*, UiPrefNym* );

    /*----- member functions -----*/

    bool GetTokensAndUseIt(const tstring in_strSerialNumber);

    void ReadNymInfo();
    void UpdateDescription();
    void UpdateBirthDate();
    void UpdateRetireDate();
    void UpdateIcon();
    void UpdateNotes();


    /*----- data members -----*/

    GtkWidget* m_frameDetails;
    GtkWidget* m_frameDesc;
    GtkWidget* m_nymTypeDesc;
    GtkWidget* m_hrefSubscribe;
    GtkWidget* m_icon;
    GtkWidget* m_extend;
    GtkWidget* m_delete;
    GtkWidget* m_notes;
    GtkWidget* m_birth;
    GtkWidget* m_retire;

    INym* m_pNym;

    size_t        m_current_security;
    ZKSTD::string m_current_notes;
    ZKSTD::string m_current_icon;
};
#endif
