// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom(R) code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPrefPorts.h>
#include <ZkGladeLoader.h>

#include <clientdebug.h>

//===========================================================================
// LOCAL DECLARATIONS
//===========================================================================


#define DATA_KEY "key_value"


//===========================================================================
// CLASS UiPrefPorts
//===========================================================================

//---------------------------------------------------------------------------
//
UiPrefPorts::UiPrefPorts( 
    const ZKSTD::string& name,
    const ZKSTD::string& desc 
) 
    : UiPrefBase( name, desc )
{
    ZkGladeLoader loader( "prefports.xml", "PrefPorts_topct" );

    SetTopContainer( loader.GetWidget( "PrefPorts_topct" ));

    m_add      = loader.GetWidget( "PrefPorts_add" );
    m_delete   = loader.GetWidget( "PrefPorts_delete" );
    m_table    = loader.GetWidget( "PrefPorts_table" );

    m_enabled  = loader.GetWidget( "PrefPorts_enable" );
    m_disabled = loader.GetWidget( "PrefPorts_disable" );
    m_selected = loader.GetWidget( "PrefPorts_selected" );

    m_newport  = loader.GetWidget( "PrefPorts_newport" );
    m_ports    = loader.GetWidget( "PrefPorts_ports" );

    GtkWidget* lbl = loader.GetWidget( "PrefPorts_descr" );
    gtk_label_set_line_wrap( GTK_LABEL( lbl ), TRUE );

    gtk_signal_connect (GTK_OBJECT( m_enabled ), "toggled",
                        GTK_SIGNAL_FUNC( ToggleCb ), this);
    gtk_signal_connect (GTK_OBJECT( m_disabled ), "toggled",
                        GTK_SIGNAL_FUNC( ToggleCb ), this);
    gtk_signal_connect (GTK_OBJECT( m_selected ), "toggled",
                        GTK_SIGNAL_FUNC( ToggleCb ), this);

    gtk_signal_connect (GTK_OBJECT( m_add ), "clicked",
                        GTK_SIGNAL_FUNC( AddCb ), this);
    gtk_signal_connect (GTK_OBJECT( m_delete ), "clicked",
                        GTK_SIGNAL_FUNC( DeleteCb ), this);
}

//---------------------------------------------------------------------------
//
UiPrefPorts::~UiPrefPorts( )
{
}

//---------------------------------------------------------------------------
//
void UiPrefPorts::OnOK()
{
    // Set the new list of ports and new Firewall option
}

//---------------------------------------------------------------------------
//
void UiPrefPorts::OnReset()
{

}


//---------------------------------------------------------------------------
//
void UiPrefPorts::ToggleCb( GtkWidget* w, UiPrefPorts* inst )
{
    inst->Toggle( w );
}

//---------------------------------------------------------------------------
//
void UiPrefPorts::Toggle( GtkWidget* w )
{
    // Only take care of 'active' notifications.
    GtkToggleButton* btn = GTK_TOGGLE_BUTTON( w );
    if( btn->active ) 
    {
        if( w == m_enabled )
        {
            
        }
        else if( w == m_disabled )
        {

        }
        else if( w == m_selected )
        {
            gtk_widget_set_sensitive( m_table, TRUE );
        }
    }    
    else if( w == m_selected )
    {
        gtk_widget_set_sensitive( m_table, FALSE );
    }
}

//---------------------------------------------------------------------------
//
void UiPrefPorts::AddCb( GtkWidget* w, UiPrefPorts* inst )
{
    inst->Add( w );
}

//---------------------------------------------------------------------------
//
void UiPrefPorts::Add( GtkWidget* )
{
    int val = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( m_newport ) );

    // Check whether or not it is in the list so we save a list update
    // if it is already there.
    ZKSTD::pair< PortList::iterator, bool > result;
    result = m_portList.insert( val );
    
    if( result.second ) {

        // Now get the inserted port index and add it to our port list.
        int index = 0;
        ZKSTD::distance( m_portList.begin(), result.first, index );

        // Since it is only for numbers, we don't have to worry about i18n issues here.
        // I hope.
        gchar* value = g_strdup_printf( "%d", val );

        GtkWidget* item = gtk_list_item_new_with_label( value );
        gtk_widget_show( item );

        GList* items = 0;
        items = g_list_append( items, item );
        
        // Set the iterator in the object for future ref.
        // Insertions and deletions in a set so not mess with iterators.

        gtk_object_set_data( GTK_OBJECT( m_ports ),
                             DATA_KEY,
                             GINT_TO_POINTER( val ) );
        gtk_list_insert_items( GTK_LIST( m_ports ), items, index );

        g_free( value );
    }
}

//---------------------------------------------------------------------------
//
void UiPrefPorts::DeleteCb( GtkWidget* w, UiPrefPorts* inst )
{
    inst->Del( w );
}

//---------------------------------------------------------------------------
//
void UiPrefPorts::Del( GtkWidget* )
{
    GList   *dlist;

    
    // Fetch the doubly linked list of selected items
    // of the List, remember to treat this as read-only!
    dlist = GTK_LIST( m_ports )->selection;
    
    if( !dlist ) 
    {
        // Nothing selected, just return.
        // FIXME - we should probably just disable the delete button.
        return;
    }


    // Now remove those items from the ZKSTD::set.
    // (this is arguably a very inefficient way of doing it)
    while (dlist) {
        GtkObject  *list_item;
        int val;
	
        list_item = GTK_OBJECT( dlist->data );
        val = GPOINTER_TO_INT( gtk_object_get_data( list_item, DATA_KEY ));
	
        dlist = dlist->next;

        m_portList.erase( val );
    }

    dlist = GTK_LIST( m_ports )->selection;
    // We need a local copy, since removing the items will unselect them as well..
    GList* dlistremove = g_list_copy( dlist );
    gtk_list_remove_items( GTK_LIST( m_ports ), dlistremove );
}
