/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UIPREFPORTS
#define ZKS_INCL_UIPREFPORTS

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#ifndef ZKS_INCL_UIPREFBASE
#include <UiPrefBase.h>
#endif

#include <set>

/*===========================================================================
 * CLASS UiPrefPorts
 *=========================================================================*/

class UiPrefPorts : public UiPrefBase {

public: 

    /*----- member functions -----*/

    UiPrefPorts( 
        const ZKSTD::string& name,
        const ZKSTD::string& desc 
    );

    virtual ~UiPrefPorts();



private: 

    /*----- typedefs and enums -----*/

    typedef ZKSTD::set< int > PortList;

    /*----- static member functions -----*/

    static void ToggleCb( GtkWidget*, UiPrefPorts* );    
    static void AddCb( GtkWidget*, UiPrefPorts* );    
    static void DeleteCb( GtkWidget*, UiPrefPorts* );    


    /*----- member functions -----*/

    // Non-static function corresponding to the callbacks.
    // Mainly to avoid constant use of an instance pointer..
    void Toggle( GtkWidget* );    
    void Add( GtkWidget* );       
    void Del( GtkWidget* );    


	// Purpose:  For overriding. Called when parent dialog OK button gets clicked.
    //           Will get called only if the modified flag is set.
	// Requires: The modified flag must be set for the function to get called.
    virtual void OnOK();

	// Purpose:  For overriding. Called when parent dialog Reset button gets clicked.
    //           Will get called only if the modified flag is set.
	// Requires: The modified flag must be set for the function to get called.
    virtual void OnReset();


    /*----- data members -----*/

    GtkWidget* m_table;
    GtkWidget* m_add;
    GtkWidget* m_delete;
               
    GtkWidget* m_enabled;
    GtkWidget* m_disabled;
    GtkWidget* m_selected;
               
    GtkWidget* m_newport;
    GtkWidget* m_ports;

    PortList   m_portList;

};
#endif
