// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):




//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPrefSecurity.h>

#include <clientdebug.h>
#include <clientdefs.h>
#include <INym.h>
#include <INymCentral.h>
#include <ITokenService.h>
#include <IZkPopup.h>
#include <IClientPersist.h>
#include <IZkConsoleMsg.h>
#include <ZkService.h>
#include <IZkPopup.h>
#include <IZkConsoleMsg.h>
#include <ZKConfiguration.h>
#include <WNymIconInfo.h>
#include <ZkDialogsStrings.h>
#include <ZkErrorHandler.h>
#include <IZKErrorReporter.h>
#include <RZkException.h>
#include <ZkStringMgr.h>
#include <ZkGladeLoader.h>

//===========================================================================
// CLASS UiPrefSecurity
//===========================================================================

//---------------------------------------------------------------------------
//
UiPrefSecurity::UiPrefSecurity( 
    const ZKSTD::string& name,
    const ZKSTD::string& desc,
    INym* pNym
) 
    : UiPrefBase( name, desc )
    , m_pNym( pNym )
{
    ZkGladeLoader loader( "pref_security.xml", "topct" );

    SetTopContainer( loader.GetWidget( "topct" ));

    m_checkbox = loader.GetWidget( "select" );
    m_security = loader.GetWidget( "security" );
    m_seclabel = loader.GetWidget( "security_txt" );

    m_adjust = gtk_range_get_adjustment( GTK_RANGE( m_security ) );
    gtk_signal_connect (GTK_OBJECT( m_adjust ), "value_changed",
                        GTK_SIGNAL_FUNC( SecurityCb ), this);


}

//---------------------------------------------------------------------------
//
UiPrefSecurity::~UiPrefSecurity( )
{
}

//---------------------------------------------------------------------------
//
void UiPrefSecurity::Initialize()
{
    ZKASSERT( m_pNym != 0 );

    UpdateSecurity();
    SecurityCb( NULL, this );
}

//---------------------------------------------------------------------------
//
bool UiPrefSecurity::Finalize()
{
    // Get the new security setting if any.
    size_t val =  (size_t)(m_adjust->value);
    if( val != m_current_security )
    {
        ZKTRACE2( "Changing security from %d to %d\n", m_current_security, val );
        SecuritySettings settings;
        settings.nHops = val;
        settings.nReserved = 8; // ? define is right in the RNym.cpp ..

        m_pNym->SetSecuritySettings( settings );

        // Warn the user he will have to reselect his nym if for the change
        // to take effect if this nym is currently active.
        size_t iActiveNym = ZkService::GetINymCentral()->GetActiveNymIndex( ) ;
        ZKSTD::string nymName;
        size_t iNym;
        m_pNym->GetNymName( nymName );           
        ZkService::GetINymCentral()->GetNymIndex( nymName, iNym );

        if ( iNym == iActiveNym ) {
            ZKSTD::string msg;
            ZkStringMgr::Load( msg, IDS_SECURITY_APPLIED_LATER, nymName.c_str() );
            IZkPopup::MessageBox( msg, ZK_MESSAGE );
        }
    }

    val = GTK_TOGGLE_BUTTON(m_checkbox)->active;
    ZKTRACE1("Setting interactive route to %d\n", val);
    m_pNym->SetNymPreferencesBitfield( (val ? PREF_INTERACTIVE_ROUTES : 0), PREF_INTERACTIVE_ROUTES ) ;

    return true;
}

//---------------------------------------------------------------------------
//
void UiPrefSecurity::SecurityCb( GtkWidget*, UiPrefSecurity* inst )
{
    int iPos = static_cast< int >( inst->m_adjust->value );

    ZKSTD::string text;

	switch ( iPos )
	{
		case( 3 ) :
            ZkStringMgr::Load( text, IDS_SECURITY_3_HOP_DLG );
			break ;
		case( 2 ) :
            ZkStringMgr::Load( text, IDS_SECURITY_2_HOP_DLG );
			break ;
		case( 1 ) :
            ZkStringMgr::Load( text, IDS_SECURITY_1_HOP_DLG );
			break ;	
		default:
			NODEFAULT ;
	}

    gtk_label_set_text( GTK_LABEL( inst->m_seclabel ), text.c_str() );    
}

//---------------------------------------------------------------------------
//
void UiPrefSecurity::UpdateSecurity()
{
    // Security Setting Stuff
    SecuritySettings ss;
    size_t nHops = MAX_HOPS; // Default is MAX_HOPS
    try {
        m_pNym->GetSecuritySettings(ss); // may throw
        ZKTRACE1("This Nym had a setting of [%d] hops when surfing\n",
		 ss.nHops);
        nHops = ss.nHops==-1?MAX_HOPS:ss.nHops; // Default is MAX_HOPS
    }

#ifdef NDEBUG
    catch(IZkException&){}
#else
    catch(IZkException& e)
    { 
        ZKTRACE1("Bad problem [%s]\n", 
                 ZkErrorHandler::GetIZkErrorReporter()->GetErrorString(e).c_str() 
        ); 
    }
#endif

/*
	m_SecSlider.SetRange(MIN_HOPS, MAX_HOPS);
	m_SecSlider.SetPos(nHops); // Default is MAX_HOPS
*/

    gtk_adjustment_set_value( GTK_ADJUSTMENT( m_adjust ), nHops );
    m_current_security = nHops;

    ZKTRACE1("Interactive route is %d\n", m_pNym->IsNymPreferenceSet( PREF_INTERACTIVE_ROUTES ) );
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(m_checkbox),
				  m_pNym->IsNymPreferenceSet( PREF_INTERACTIVE_ROUTES ) );
}
