// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPrefSpam.h>
#include <ZkGladeLoader.h>
#include <ZkDialogsStrings.h>

#include <INym.h>
#include <INymMail.h>
#include <INymCentral.h>
#include <ZkService.h>
#include <ZkErrorHandler.h>
#include <ZkWebIntegration.h>
#include <IZkConsoleMsg.h>
#include <IZkPopup.h>
#include <IZKErrorReporter.h>
#include <IZkException.h>
#include <clientdebug.h>
#include <ZkStringMgr.h>
#include <INiqs.h>

//===========================================================================
// LOCAL DECLARATIONS
//===========================================================================


#define DATA_KEY "key_value"


//===========================================================================
// CLASS UiPrefSpam
//===========================================================================

//---------------------------------------------------------------------------
//
UiPrefSpam::UiPrefSpam( 
    const ZKSTD::string& name,
    const ZKSTD::string& desc,
    INym* pNym
) 
    : UiPrefBase( name, desc )
{
    ZKASSERT( pNym != 0 );

    ZkGladeLoader loader( "pref_spam.xml", "topct" );

    SetTopContainer( loader.GetWidget( "topct" ));

    m_tag    = loader.GetWidget( "tagline" );
    m_add    = loader.GetWidget( "add" );
    m_delete = loader.GetWidget( "delete" );
    m_block  = loader.GetWidget( "block" );
    m_new    = loader.GetWidget( "new_address" );
    m_list   = loader.GetWidget( "list" );
    m_msg    = loader.GetWidget( "msg" );
    m_address = loader.GetWidget( "address" );

    gtk_signal_connect (GTK_OBJECT( m_new ), "activate",
                        GTK_SIGNAL_FUNC( AddCb ), this );
    gtk_signal_connect (GTK_OBJECT( m_add ), "clicked",
                        GTK_SIGNAL_FUNC( AddCb ), this);
    gtk_signal_connect (GTK_OBJECT( m_delete ), "clicked",
                        GTK_SIGNAL_FUNC( DeleteCb ), this);

    ZKSTD::string nymName;
    ZKSTD::string msg;

    pNym->GetNymName( nymName );
    pNym->GetNymEmailDomain( m_strFreedomDomain );

    ZkService::GetINymCentral()->GetNym( IID_IMAIL, (void**) &m_pMail, nymName );

    if( nymName.find('@') == nymName.npos ) {
       nymName = nymName +  "@" + m_strFreedomDomain;
    }

    ZkStringMgr::Load( msg, IDS_BLOCK_LIST_TEXT, nymName.c_str() );
    gtk_label_set_text( GTK_LABEL( m_msg ), msg.c_str() );
    gtk_label_set_text( GTK_LABEL( m_address), nymName.c_str());
    ZKASSERT( m_pMail != 0 );
}

//---------------------------------------------------------------------------
//
UiPrefSpam::~UiPrefSpam( )
{
}

//---------------------------------------------------------------------------
//
void UiPrefSpam::Initialize()
{
	ZKSTD::vector<ZKSTD::string> vecAddresses ;

    m_pMail->GetEmailAddressBlockList( vecAddresses ) ;

    // Populate List box with these items
    ZKSTD::vector<ZKSTD::string>::iterator iter;
	
    for(iter = vecAddresses.begin(); iter != vecAddresses.end(); ++iter)
    {
        m_spam.insert( *iter );
    }

    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_block ),
                                  m_pMail->GetSpamPreference() );
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_tag ),
                                  m_pMail->GetTaglinePreference() );

    gtk_widget_set_sensitive( m_tag, m_pMail->NymHasTaglineChoice() );
    
	SetModified( false );
    FillListBox();
}

//---------------------------------------------------------------------------
//
bool UiPrefSpam::Finalize()
{
    try
    {
        bool bNewBlockSpam = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( m_block ) );

        if ( bNewBlockSpam != m_pMail->GetSpamPreference() )
        {
            m_pMail->SetSpamPreference( bNewBlockSpam );
        }
		
        bool bNewTagline = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( m_tag ) );

        if ( bNewTagline != m_pMail->GetTaglinePreference())
        {
            m_pMail->SetTaglinePreference( bNewTagline );
        }

        // FIXME - this could probably be done better by some stl copy function.
        ZKSTD::vector<ZKSTD::string> vecAddresses ;
        SpamList::iterator iter;
        for( iter = m_spam.begin(); iter != m_spam.end(); ++iter )
        {
            vecAddresses.push_back( *iter );
        }

        if ( WasModified() ) {
            // Show busy AVI during update of server
// FIXME - will need separate thread.
//            ZKBusyAVI busy;
            IZkConsoleMsg::ShowMsgForceUpdate( IDS_CONS_BLOCKLIST_TO_SERVER, TRUE);

            m_pMail->SetEmailAddressBlockList( vecAddresses) ;

            IZkConsoleMsg::ShowMsg( IDS_CONS_BLOCKLIST_UPDATE_SUCCESSFUL ) ;
        }
    }
    catch( IZkException& ex){
        ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);

        // We couldn't commit the changes onto the server or the
        // disk.  The user should know about this can have the chance
        // to cancel out or try again
        return false;
    }

    return true;
}

//---------------------------------------------------------------------------
//
void UiPrefSpam::AddCb( GtkWidget* w, UiPrefSpam* inst )
{
    inst->Add( w );
}

//---------------------------------------------------------------------------
//
void UiPrefSpam::Add( GtkWidget* )
{
    ZKSTD::string txt( gtk_entry_get_text( GTK_ENTRY( m_new ) ) );

    if( txt.empty() )
    {
        return;
    }
	
    if( txt.find('@') == txt.npos ) {
        // complete the name with @*.freedom.net if there is no @ in the string.
        txt = txt +  "@" + m_strFreedomDomain;
    }

    // Check whether or not it is in the list so we save a list update
    // if it is already there.
    ZKSTD::pair< SpamList::iterator, bool > result;
    result = m_spam.insert( txt );
    
    if( result.second ) 
    {
        // Now get the inserted address and add it to our gtk list
        int index = 0;
        ZKSTD::distance( m_spam.begin(), result.first, index );

        GtkWidget* item = gtk_list_item_new_with_label( txt.c_str() );
        gtk_widget_show( item );

        GList* items = 0;
        items = g_list_append( items, item );
        
        gtk_list_insert_items( GTK_LIST( m_list ), items, index );

        // Finally, clear the text entry to be ready for the next one.
        gtk_entry_set_text( GTK_ENTRY( m_new ), "" );

        SetModified( true );
    }
    else
    {
        // FIXME - re-using string from Word Scan, get a new one.
        IZkPopup::MessageBox( IDS_WORD_IN_LIST );        
    }
}

//---------------------------------------------------------------------------
//
void UiPrefSpam::DeleteCb( GtkWidget* w, UiPrefSpam* inst )
{
    inst->Del( w );
}

//---------------------------------------------------------------------------
//
void UiPrefSpam::Del( GtkWidget* )
{
    GList   *dlist;

    
    // Fetch the doubly linked list of selected items
    // of the List, remember to treat this as read-only!
    dlist = GTK_LIST( m_list )->selection;
    
    if( !dlist ) 
    {
        // Nothing selected, just return.
        // FIXME - we should probably just disable the delete button.
        return;
    }


    // Now remove those items from the ZKSTD::set.
    // (this is arguably a very inefficient way of doing it)
    while (dlist) {
        GtkBin  *list_item;
	
        list_item = GTK_BIN( dlist->data );
        
        // Get the text.
        char* label;
        gtk_label_get( GTK_LABEL( list_item->child ), &label );
        ZKSTD::string val( label );
        m_spam.erase( val );

        dlist = dlist->next;

        SetModified( true );
    }

    FillListBox();
}

//---------------------------------------------------------------------------
//
void UiPrefSpam::FillListBox()
{
    gtk_list_clear_items( GTK_LIST( m_list ), 0, -1 );

    SpamList::iterator iter;

    GtkWidget* item;
    GList* items = 0;

    for( iter = m_spam.begin(); iter != m_spam.end(); ++iter )
    {
        item = gtk_list_item_new_with_label( (*iter).c_str() );
        gtk_widget_show( item );
        items = g_list_append( items, item );
    }

    gtk_list_append_items( GTK_LIST( m_list ), items );
}

