// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiPrefWordScan.h>
#include <ZkGladeLoader.h>
#include <ZkDialogsStrings.h>

#include <IZkPopup.h>
#include <ZkService.h>
#include <INym.h>
#include <INymCentral.h>
#include <IClientCentral.h>
#include <clientdebug.h>

#define DATA_KEY "key_word"

#include <algorithm>
#include <iterator>

//===========================================================================
// CLASS UiPrefWordScan
//===========================================================================

//---------------------------------------------------------------------------
//
UiPrefWordScan::UiPrefWordScan( 
    const ZKSTD::string& name,
    const ZKSTD::string& desc 
) 
    : UiPrefBase( name, desc )
{
    ZkGladeLoader loader( "prefwordscan.xml", "topct" );

    SetTopContainer( loader.GetWidget( "topct" ) );

    m_add      = loader.GetWidget( "add" );
    m_delete   = loader.GetWidget( "delete" );

    m_newword  = loader.GetWidget( "newword" );
    m_words    = loader.GetWidget( "words" );
    
    m_ipTog    = GTK_TOGGLE_BUTTON( loader.GetWidget( "top_ip" ) );
    m_nymTog   = GTK_TOGGLE_BUTTON( loader.GetWidget( "top_nym" ) );
    m_emailTog = GTK_TOGGLE_BUTTON( loader.GetWidget( "top_email" ) );


    gtk_signal_connect (GTK_OBJECT( m_newword ), "activate",
                        GTK_SIGNAL_FUNC( EnterCb ), this );
    gtk_signal_connect (GTK_OBJECT( m_add ), "clicked",
                        GTK_SIGNAL_FUNC( AddCb ), this);
    gtk_signal_connect (GTK_OBJECT( m_delete ), "clicked",
                        GTK_SIGNAL_FUNC( DeleteCb ), this);
}

//---------------------------------------------------------------------------
//
UiPrefWordScan::~UiPrefWordScan( )
{
}

//---------------------------------------------------------------------------
//
void UiPrefWordScan::Initialize()
{
    SetModified( false );

    // First make sure we cleanup.
    m_wordList.clear();
    gtk_list_clear_items( GTK_LIST( m_words ), 0, -1 );

    bool flag;
	flag = ZkService::GetIClientCentral()->GetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_NYMNAMES );
    gtk_toggle_button_set_active( m_nymTog, flag );

	flag = ZkService::GetIClientCentral()->GetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_IP );
    gtk_toggle_button_set_active( m_ipTog, flag );

	flag = ZkService::GetIClientCentral()->GetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_EMAIL_ADDR );
    gtk_toggle_button_set_active( m_emailTog, flag );

    // Now fill the Word Scan list.
	ZKSTD::vector<ZKSTD::string> vecWords;
	ZKSTD::vector<ZKSTD::string>::iterator iter;

	INym* pNym = NULL;
	ZkService::GetINymCentral()->GetNym(IID_INYM, (void**) &pNym, USER_NYM_INDEX);
	ZKVERIFY( pNym->GetOffensiveWordList( vecWords ) ) ;
	
	for( iter = vecWords.begin(); iter != vecWords.end(); ++iter )
	{
        AddWord( *iter );
	}
}

//---------------------------------------------------------------------------
//
bool UiPrefWordScan::Finalize()
{
	bool bScanIp    = gtk_toggle_button_get_active( m_ipTog );
	bool bScanNym   = gtk_toggle_button_get_active( m_nymTog );
	bool bScanEmail = gtk_toggle_button_get_active( m_emailTog );

	if( bScanIp != ZkService::GetIClientCentral()->GetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_IP ) ){
		ZkService::GetIClientCentral()->SetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_IP, bScanIp ) ;
	}

	if( bScanNym != ZkService::GetIClientCentral()->GetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_NYMNAMES ) ){
		ZkService::GetIClientCentral()->SetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_NYMNAMES, bScanNym ) ;
	}

	if( bScanEmail != ZkService::GetIClientCentral()->GetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_EMAIL_ADDR ) ){
		ZkService::GetIClientCentral()->SetPreference( ZK_CLIENT_PREF_SET, PREF_SCAN_EMAIL_ADDR, bScanEmail ) ;
	}


 	if( WasModified() ) 
    {  

        // Altough it will probably change the words order, just use the set we 
        // already have and replace the vector.
        ZKSTD::vector<ZKSTD::string> vecWords;
        ZKSTD::insert_iterator< ZKSTD::vector<ZKSTD::string> > outIter( vecWords, vecWords.begin() );
        
        ZKSTD::copy( m_wordList.begin(), m_wordList.end(), outIter );
        
        INym* pNym = NULL;
        ZkService::GetINymCentral()->GetNym( IID_INYM, (void**) &pNym, USER_NYM_INDEX );
        ZKVERIFY( pNym->SetOffensiveWordList( vecWords ) ) ;
	}    

    return true;
}


//---------------------------------------------------------------------------
//
void UiPrefWordScan::EnterCb( GtkWidget*, UiPrefWordScan* inst )
{
    inst->Add();
}

//---------------------------------------------------------------------------
//
void UiPrefWordScan::AddCb( GtkWidget*, UiPrefWordScan* inst )
{
    inst->Add( );
}

//---------------------------------------------------------------------------
//
void UiPrefWordScan::Add(  )
{
    ZKSTD::string word = gtk_entry_get_text( GTK_ENTRY( m_newword ) );

    AddWord( word );
    
    gtk_entry_set_text( GTK_ENTRY( m_newword ), "" );
}

//---------------------------------------------------------------------------
//
void UiPrefWordScan::DeleteCb( GtkWidget*, UiPrefWordScan* inst )
{
    inst->Del( );
}

//---------------------------------------------------------------------------
//
void UiPrefWordScan::Del(  )
{
    GList   *dlist;

    
    // Fetch the doubly linked list of selected items
    // of the List, remember to treat this as read-only!
    dlist = GTK_LIST( m_words )->selection;
    
    if( !dlist ) 
    {
        // Nothing selected, just return.
        // FIXME - we should probably just disable the delete button.
        return;
    }


    // Now remove those items from the ZKSTD::set.
    // (this is arguably a very inefficient way of doing it)
    while (dlist) {
        GtkObject  *list_item;
	
        list_item = GTK_OBJECT( dlist->data );
        char* pWord = (char *)gtk_object_get_data( list_item, DATA_KEY );

        ZKSTD::string word = pWord;
        ZKTRACE1( "Erasing: %s\n", pWord );
        m_wordList.erase( word );
        SetModified( true );
	
        dlist = dlist->next;
    }

    // We need a local copy, since removing the items will unselect them as well..
    dlist = GTK_LIST( m_words )->selection;
    GList* dlistremove = g_list_copy( dlist );
    gtk_list_remove_items( GTK_LIST( m_words ), dlistremove );
}



//---------------------------------------------------------------------------
//
void UiPrefWordScan::AddWord( const ZKSTD::string& word )
{
    if( word.empty() )
    {
        return;
    }

    // Make sure this is not only spaces.
    if( word.find_first_not_of( ' ' ) >= word.size() )
    {
        return;
    }


    // Check whether or not it is in the list so we save a list update
    // if it is already there.
    ZKSTD::pair< WordList::iterator, bool > result;
    result = m_wordList.insert( word );
    
    if( result.second ) {
        SetModified( true );

        // Now get the inserted port index and add it to our port list.
        int index = 0;
        ZKSTD::distance( m_wordList.begin(), result.first, index );

        // This assume that we have some sort of correspondance between
        // the ZKSTD::set and the GtkList (which should be true).
        char* pWord = const_cast< char *>((*(result.first)).c_str()) ;

        GtkWidget* item = gtk_list_item_new_with_label( pWord );
        gtk_widget_show( item );

        GList* items = 0;
        items = g_list_append( items, item );
        
        ZKTRACE1( "Storing: <%s>\n", pWord );
        gtk_object_set_data( GTK_OBJECT( item ),
                             DATA_KEY,
                             (gpointer)pWord );
        gtk_list_insert_items( GTK_LIST( m_words ), items, index );
    }   
    else
    {
        IZkPopup::MessageBox( IDS_WORD_IN_LIST );        
    }
}

