// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiProxyDialogs.h>
#include <UiOutgoingMail.h>
#include <UiWordScan.h>
#include <UiSignedMail.h>
#include <UiMailFirstCheck.h>
#include <UiMailSentCount.h>
#include <UiWarnNNTPConnection.h>
#include <UiWarnSSLConnection.h>

#include <ZkService.h>
#include <IClientCentral.h>

//===========================================================================
// CLASS UiProxyDialogs
//===========================================================================

//---------------------------------------------------------------------------
//
UiProxyDialogs::UiProxyDialogs( )
   : m_outgoingMailDlg( new UiOutgoingMail() )
   , m_wordScanDlg( new UiWordScan() )
   , m_signedMailDlg( new UiSignedMail() )
   , m_firstMailCheckDlg( new UiMailFirstCheck() )
   , m_mailSentCountDlg( new UiMailSentCount())
   , m_warnNNTPConnectionDlg( new UiWarnNNTPConnection() ) 
   , m_warnSSLConnectionDlg( new UiWarnSSLConnection() ) 
{
}

//---------------------------------------------------------------------------
//
UiProxyDialogs::~UiProxyDialogs( )
{
}


//---------------------------------------------------------------------------
//
long UiProxyDialogs::LaunchOutgoingMsgDlg( 
    ZKSTD::vector<ZKSTD::string>* pVecAddresses, 
    bool in_fSMTP = true,
	const ZKSTD::string* in_pStrUserEmailAddr)
{
    return m_outgoingMailDlg->Run( pVecAddresses, 
                                   in_fSMTP, 
                                   in_pStrUserEmailAddr );
}
//---------------------------------------------------------------------------
//

long UiProxyDialogs::LaunchFirstMailCheckDlg( )
{
	// To be filled in at a later date.
	return m_firstMailCheckDlg->Run();
}

//---------------------------------------------------------------------------
//
bool UiProxyDialogs::LaunchWordScanMsgDlg( 
    unsigned int in_Protocol, 
    WordScanSummary* in_pScanResults 
)
{
    return m_wordScanDlg->Run( in_Protocol, in_pScanResults );
}

//---------------------------------------------------------------------------
//
bool UiProxyDialogs::LaunchWarnSignedMailDlg()
{
    return m_signedMailDlg->Run();
}

//---------------------------------------------------------------------------
//
void UiProxyDialogs::LaunchMailSentCountDlg( time_t resetTime,
                                             ZKSTD::vector< ZKSTD::string> * pvecRecipients ) 
{
    /* return */ m_mailSentCountDlg->Run( resetTime, pvecRecipients );
}

//---------------------------------------------------------------------------
//
bool UiProxyDialogs::LaunchWarnNNTPConnectionDlg()
{
	ZKS::Lock_Guard<ZKCriticalSection> gate(m_NNTPConnectionCs);

	// the default value is Allow
	bool bRes = true;

	if( !ZkService::GetIClientCentral()->GetPreference(
            ZK_SHOW_AGAIN_PREF_SET, 
            PREF_NO_SHOW_NEWS_OPTIONS))
	{
        bRes = m_warnNNTPConnectionDlg->Run();
	}
    return bRes;
}

//---------------------------------------------------------------------------
//
bool UiProxyDialogs::LaunchWarnSSLConnectionDlg()
{
  	ZKS::Lock_Guard<ZKCriticalSection> gate(m_SSLConnectionCs);

	// the default value is Allow
	bool bRes = true;

	if( !ZkService::GetIClientCentral()->GetPreference(
            ZK_SHOW_AGAIN_PREF_SET, 
            PREF_NO_SHOW_SSL_WARNING))
	{
        bRes = m_warnSSLConnectionDlg->Run();
	}
    return bRes;
}
