/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_UIPROXYDIALOGS
#define ZKS_INCL_UIPROXYDIALOGS

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include <ZkProxyDialogs.h>

class UiOutgoingMail;
class UiWordScan;
class UiSignedMail;
class UiMailFirstCheck;
class UiMailSentCount;
class UiWarnNNTPConnection;
class UiWarnSSLConnection;

/*===========================================================================
 * CLASS UiProxyDialogs
 *=========================================================================*/

class UiProxyDialogs
    : public IProxyDialogs {

public: 

    /*----- member functions -----*/

    UiProxyDialogs();
    virtual ~UiProxyDialogs();
      
	// Purpose:  Opens up the Word Scan dialog box, warning the user
    //           about a word found in outgoig traffic that corresponds to an
    //           entry in the Word Scan list.
    // Returns:  False if user chooses to block outgoing message.
    //           True  if the message is to be sent anyway.
	virtual long LaunchOutgoingMsgDlg( 
        ZKSTD::vector<ZKSTD::string>* pVecAddresses, 
        bool in_fSMTP = true,
		const ZKSTD::string* in_pStrUserEmailAddr = NULL);

	// Purpose:  Opens up the Word Scan dialog box, warning the user
    //           about a word found in outgoig traffic that corresponds to an
    //           entry in the Word Scan list.
    // Returns:  False if user chooses to block outgoing message.
    //           True  if the message is to be sent anyway.
	virtual bool LaunchWordScanMsgDlg( 
        unsigned int in_Protocol, 
        WordScanSummary* in_pScanResults );

	// Purpose:  Opens the Signed Mail warning dialog box, explaining to the 
    //           user that the email about to be sent contain a digital signature.
    // Returns:  False if user chooses to block outgoing message.
    //           True  if the message is to be sent anyway.
	virtual bool LaunchWarnSignedMailDlg();

	// Purpose: Opens the "First Mail Check Dialog", explaining briefly the possible
	//          Mail Correlation Attack. Gives the user the opportunity to 1) Cancel the mailcheck,
	//          2) Check mail only for the active nym or 3) check mail for all nyms.
	// Returns: 
	//		ZK_MAIL_CHECK_ALL 1
	//		ZK_MAIL_CHECK_ACTIVE 2
	//		ZK_MAIL_CHECK_BLOCK 3

	virtual long LaunchFirstMailCheckDlg();

	virtual void LaunchMailSentCountDlg( time_t resetTime,
          				     ZKSTD::vector< ZKSTD::string> * pvecRecipients );
      
    virtual bool LaunchWarnNNTPConnectionDlg();

    virtual bool LaunchWarnSSLConnectionDlg();

private: 

    /*----- data members -----*/

    ZKSTD::auto_ptr< UiOutgoingMail > m_outgoingMailDlg;
    ZKSTD::auto_ptr< UiWordScan >     m_wordScanDlg;
    ZKSTD::auto_ptr< UiSignedMail >   m_signedMailDlg;
    ZKSTD::auto_ptr< UiMailFirstCheck >   m_firstMailCheckDlg;
    ZKSTD::auto_ptr< UiMailSentCount >    m_mailSentCountDlg;
    ZKSTD::auto_ptr< UiWarnNNTPConnection > m_warnNNTPConnectionDlg;
    ZKSTD::auto_ptr< UiWarnSSLConnection > m_warnSSLConnectionDlg;
    
	ZKCriticalSection m_SSLConnectionCs;
	ZKCriticalSection m_NNTPConnectionCs;

};
#endif
