// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiSignedMail.h>
#include <UiThreadId.h>
#include <ZkDialogsStrings.h>
#include <ZkGtkUtils.h>

#include <ZkGladeLoader.h>
#include <lockguard.h>

#include <IClientCentral.h>
#include <ZkService.h>
#include <clientdebug.h>

#include <gnome.h>

//===========================================================================
// CLASS UiSignedMail.cpp
//===========================================================================

//---------------------------------------------------------------------------
//
UiSignedMail::UiSignedMail( )
    : m_answer( false )
{
    // Call that a flok legend if you want, but it seems X really doesn't
    // appreciate top level windows being created in threads.
    // (which makes sense, given the fact X is not really thread safe)
    ZKASSERT( UiThreadId::IsUiThread() );
    
    ZkGladeLoader loader( "signed_mail.xml", "SignedMail_window" );

    m_window = loader.GetWidget( "SignedMail_window" );
    m_check  = loader.GetWidget( "SignedMail_dontshow" );

    gtk_widget_hide( GTK_WIDGET( m_window ) );
}

//---------------------------------------------------------------------------
//
UiSignedMail::~UiSignedMail( )
{
}

//---------------------------------------------------------------------------
//
bool UiSignedMail::Run( )
{
    if ( ZkService::GetIClientCentral()->GetPreference(
        ZK_SHOW_AGAIN_PREF_SET, 
        PREF_NO_WARN_SIGNED_MAIL)
    )
	{
		return true;
	}



    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    // Now if this is a background thread, defer the work otherwise 
    // open the dialog rightaway.
    NotifyUi();

    return m_answer;
}

//---------------------------------------------------------------------------
//
void UiSignedMail::UiThreadCallback()
{
    ZkGtkUtils::GtkWidgetShowAll( m_window );
    int answer = gnome_dialog_run( GNOME_DIALOG( m_window ) );

    // Alway reset the toggle, if we dgot here it means the option is off.
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_check ), FALSE );

    switch( answer )
    {
        case 0: /* Send Message */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            m_answer = true;
            break;
            
        case 1: /* Block Message */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            m_answer = false;
            break;
            
        case -1: /* ESC or wm close */
            m_answer = false;
            break;                
    }  

    bool noshow = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( m_check ));

    if( noshow ) 
    {
        ZkService::GetIClientCentral()->SetPreference(
            ZK_SHOW_AGAIN_PREF_SET, 
            PREF_NO_WARN_SIGNED_MAIL, 
            true );
    }
}


