// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiUpgradeNotification.h>
#include <UiThreadId.h>
#include <ZkDialogsStrings.h>
#include <ZkGtkUtils.h>

#include <ZkGladeLoader.h>
#include <lockguard.h>

#include <clientdebug.h>

#include <ZkStringMgr.h>
#include <ZkCatalogRegistry.h>

#include <gnome.h>

#include <ZkService.h>
#include <IClientInfo.h>

//===========================================================================
// CLASS UiUpgradeNotification.cpp
//===========================================================================

//---------------------------------------------------------------------------
//
UiUpgradeNotification::UiUpgradeNotification( )
{
    ZKASSERT( UiThreadId::IsUiThread() );
    
    ZkGladeLoader loader( "expiry_notification.xml", "expiry_notify_dialog" );

    m_window = loader.GetWidget( "expiry_notify_dialog" );
    m_label  = loader.GetWidget( "notify_label" );
    m_href_button = loader.GetWidget( "href_button" );

    gtk_widget_hide( GTK_WIDGET( m_window ) );
}

//---------------------------------------------------------------------------
//
UiUpgradeNotification::~UiUpgradeNotification( )
{
}

//---------------------------------------------------------------------------
//
bool UiUpgradeNotification::Run( long secsUntilExpiry )
{

    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    m_secsUntilExpiry = secsUntilExpiry;

    // Now if this is a background thread, defer the work otherwise 
    // open the dialog rightaway.
    NotifyUi();

    return secsUntilExpiry > SECS_IN_A_DAY;
}

//---------------------------------------------------------------------------
//
void UiUpgradeNotification::UiThreadCallback()
{
    ZKSTD::string buf;

    gnome_href_set_url( GNOME_HREF( m_href_button ), ZkService::GetIClientInfo()->GetUpgradeUrl().c_str() );

    if (m_secsUntilExpiry <= SECS_IN_A_DAY) 
    {
        ZkStringMgr::Load( buf, IDS_FREEDOM_EXPIRED );
    } 
    else 
    {
        ZkStringMgr::Load( buf, IDS_FREEDOM_WILLEXPIRE, (unsigned int)(m_secsUntilExpiry /60 /60 /24) );
    }

    ZKASSERT("Have a string" && !buf.empty() );
    gtk_label_set_text( GTK_LABEL(m_label), (gchar *)buf.c_str());
    ZkGtkUtils::GtkWidgetShowAll( m_window );
    gtk_label_set_text( GTK_LABEL(m_label), buf.c_str() );

    gnome_dialog_run( GNOME_DIALOG( m_window ) );

    gtk_widget_hide( m_window );
}

