// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiWarnNNTPConnection.h>
#include <UiThreadId.h>
#include <ZkDialogsStrings.h>
#include <ZkGtkUtils.h>

#include <ZkGladeLoader.h>
#include <lockguard.h>

#include <IClientCentral.h>
#include <ZkService.h>
#include <clientdebug.h>
#include <ZkStringMgr.h>

#include <gnome.h>

//===========================================================================
// CLASS UiWarnNNTPConnection.cpp
//===========================================================================

//---------------------------------------------------------------------------
//
UiWarnNNTPConnection::UiWarnNNTPConnection( )
    : m_answer( false )
{
    // Call that a flok legend if you want, but it seems X really doesn't
    // appreciate top level windows being created in threads.
    // (which makes sense, given the fact X is not really thread safe)
    ZKASSERT( UiThreadId::IsUiThread() );
    
    ZkGladeLoader loader("warn_nntp_connection.xml", "WarnNNTP_window" );

    m_window = loader.GetWidget( "WarnNNTP_window" );
    m_check  = loader.GetWidget( "WarnNNTP_dontshow" );

    m_msg1 = loader.GetWidget( "WarnNNTP_msg1" );
    m_msg2 = loader.GetWidget( "WarnNNTP_msg2" );

    m_linkToKC = loader.GetWidget( "WarnNNTP_linkToKC" );

    gtk_signal_connect (GTK_OBJECT( m_linkToKC ), "clicked",
                        GTK_SIGNAL_FUNC( LinkToKCCb ), this);

    gtk_widget_hide( GTK_WIDGET( m_window ) );

}

//---------------------------------------------------------------------------
//
UiWarnNNTPConnection::~UiWarnNNTPConnection( )
{
}

//---------------------------------------------------------------------------
//
void UiWarnNNTPConnection::LinkToKCCb( GtkWidget*, UiWarnNNTPConnection* inst )
{
    inst->LinkToKCClicked();
}

//---------------------------------------------------------------------------
//
void UiWarnNNTPConnection::LinkToKCClicked()
{
    gnome_dialog_close( GNOME_DIALOG( m_window ) );
    m_answer = false;
}

//---------------------------------------------------------------------------
//
bool UiWarnNNTPConnection::Run( )
{
    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    // Now if this is a background thread, defer the work otherwise 
    // open the dialog rightaway.
    NotifyUi();

    return m_answer;
}

//---------------------------------------------------------------------------
//
void UiWarnNNTPConnection::UiThreadCallback()
{
    // Alway reset the toggle, if we dgot here it means the option is off.
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_check ), FALSE );

    ZKSTD::string strMsg;
    ZkStringMgr::Load( strMsg, IDS_NNTP_WARNING_MSG1 );
    gtk_label_set_text(GTK_LABEL( m_msg1 ), strMsg.c_str() );

    ZkStringMgr::Load( strMsg, IDS_NNTP_WARNING_MSG2 );
    gtk_label_set_text(GTK_LABEL( m_msg2 ), strMsg.c_str() );

    ZkGtkUtils::GtkWidgetShowAll( m_window );
    int answer = gnome_dialog_run( GNOME_DIALOG( m_window ) );

    switch( answer )
    {
        case 0: /* Allow Connection */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            m_answer = true;
            break;
            
        case 1: /* Block Connection */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            m_answer = false;
            break;
            
        case -1: /* ESC or wm close */
            m_answer = false;
            break;                
    }  

    if (m_answer == true)
    {
        bool noshow = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( m_check ));

        if( noshow ) 
        {
            ZkService::GetIClientCentral()->SetPreference(
                ZK_SHOW_AGAIN_PREF_SET, 
                PREF_NO_SHOW_NEWS_OPTIONS, 
                true );
        }
    }
}


