// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiWarnSSLConnection.h>
#include <UiThreadId.h>
#include <ZkDialogsStrings.h>
#include <ZkGtkUtils.h>

#include <ZkGladeLoader.h>
#include <lockguard.h>

#include <IClientCentral.h>
#include <ZkService.h>
#include <clientdebug.h>
#include <ZkStringMgr.h>
#include <ZkWebIntegration.h>

#include <gnome.h>

//===========================================================================
// CLASS UiWarnSSLConnection.cpp
//===========================================================================

//---------------------------------------------------------------------------
//
UiWarnSSLConnection::UiWarnSSLConnection( )
    : m_answer( false )
{
    // Call that a flok legend if you want, but it seems X really doesn't
    // appreciate top level windows being created in threads.
    // (which makes sense, given the fact X is not really thread safe)
    ZKASSERT( UiThreadId::IsUiThread() );
    
    ZkGladeLoader loader("warn_ssl_connection.xml", "WarnSSL_window" );

    m_window = loader.GetWidget( "WarnSSL_window" );
    m_check  = loader.GetWidget( "WarnSSL_dontshow" );

    m_msg1 = loader.GetWidget( "WarnSSL_msg1" );
    m_msg2 = loader.GetWidget( "WarnSSL_msg2" );
    
    GtkWidget* link = loader.GetWidget( "WarnSSL_linkToKC" );
    gnome_href_set_url( GNOME_HREF( link ), 
                        ZkWebIntegration::GetURL( URL_KB_SSL_LOGIN_PROBLEMS ).c_str() );
        
    gtk_widget_hide( GTK_WIDGET( m_window ) );
}

//---------------------------------------------------------------------------
//
UiWarnSSLConnection::~UiWarnSSLConnection( )
{
}

//---------------------------------------------------------------------------
//
bool UiWarnSSLConnection::Run( )
{
    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    // Now if this is a background thread, defer the work otherwise 
    // open the dialog rightaway.
    NotifyUi();

    return m_answer;
}

//---------------------------------------------------------------------------
//
void UiWarnSSLConnection::UiThreadCallback()
{
    // Alway reset the toggle, if we dgot here it means the option is off.
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( m_check ), FALSE );

    ZKSTD::string strMsg1;
    ZkStringMgr::Load( strMsg1, IDS_SSL_WARNING_MSG1 );
    gtk_label_set_text(GTK_LABEL( m_msg1 ), strMsg1.c_str() );

    ZKSTD::string strMsg2;
    ZkStringMgr::Load( strMsg2, IDS_SSL_WARNING_MSG2 );
    gtk_label_set_text(GTK_LABEL( m_msg2 ), strMsg2.c_str() );

    ZkGtkUtils::GtkWidgetShowAll( m_window ); 
    int answer = gnome_dialog_run( GNOME_DIALOG( m_window ) );

    switch( answer )
    {
        case 0: /* Allow Connection */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            m_answer = true;
            break;
            
        case 1: /* Block Connection */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            m_answer = false;
            break;
            
        case -1: /* ESC or wm close */
            m_answer = false;
            break;                
    }  

    if (m_answer == true)
    {
        bool noshow = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( m_check ));

        if( noshow ) 
        {
            ZkService::GetIClientCentral()->SetPreference(
                ZK_SHOW_AGAIN_PREF_SET, 
                PREF_NO_SHOW_SSL_WARNING, 
                true );
        }
    }
}


