// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <UiWordScan.h>
#include <UiThreadId.h>
#include <ZkDialogsStrings.h>
#include <ZkGtkUtils.h>

#include <ZkStringMgr.h>
#include <ZkGladeLoader.h>
#include <lockguard.h>

#include <INymCentral.h> // for WordScanSummary ...
#include <clientdebug.h>
#include <vector>

#include <gnome.h>

// FIXME - put this in a catalog.
static const char g_strYourTrueIdentity[] = "Your true identity";

//===========================================================================
// LOCAL DECLARATIONS
//===========================================================================

namespace {

//---------------------------------------------------------------------------
//
void ProtocolToString( unsigned int p, ZKSTD::string& inout_str )
{
    inout_str.erase();

	switch( p )
	{
        case( PROTOCOL_HTTP ):
            inout_str = "Web";
            break;
            
        case( PROTOCOL_SMTP ):
            inout_str = "Mail";
            break;
            
        case( PROTOCOL_IRC ):
            inout_str = "IRC";
            break;
            
        case( PROTOCOL_NNTP1 ):
            inout_str = "News";
            break;
            
        default:
            inout_str = "Internet";
            break;
	}
}

}

//===========================================================================
// CLASS UiWordScan.cpp
//===========================================================================

//---------------------------------------------------------------------------
//
UiWordScan::UiWordScan( )
    : m_pScanResult( 0 )
    , m_protocol( 0 )
    , m_answer( false )
{
    // Call that a flok legend if you want, but it seems X really doesn't
    // appreciate top level windows being created in threads.
    // (which makes sense, given the fact X is not really thread safe)
    ZKASSERT( UiThreadId::IsUiThread() );
    
    ZkGladeLoader loader( "word_scan.xml", "WordScan_window" );

    m_window  = loader.GetWidget( "WordScan_window" );
    m_matches = loader.GetWidget( "WordScan_matches" );
    m_header  = loader.GetWidget( "WordScan_msg" );

    gtk_widget_hide( GTK_WIDGET( m_window ) );
}

//---------------------------------------------------------------------------
//
UiWordScan::~UiWordScan( )
{
}

//---------------------------------------------------------------------------
//
bool UiWordScan::Run( 
    unsigned int in_protocol, 
    WordScanSummary* in_pScanResult 
)
{
    m_pScanResult = in_pScanResult;
    m_protocol    = in_protocol;

	ZKASSERT( m_pScanResult );
	ZKASSERT( m_pScanResult->pFoundWords != NULL ) ;
	ZKASSERT( m_pScanResult->pScannedBuffer != NULL ) ;
	ZKASSERT( m_pScanResult->cntBuffer > 0 ) ;

    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    // Now if this is a background thread, defer the work otherwise 
    // open the dialog rightaway.
    NotifyUi();

    return m_answer;
}

//---------------------------------------------------------------------------
//
void UiWordScan::UiThreadCallback()
{
    // Display word scan matches.
    // Empty current list.
    gtk_list_clear_items( GTK_LIST( m_matches ), 0, -1 );
    GtkWidget* li;

    ZKSTD::vector<ZKSTD::string>::const_iterator it;

    for( it = m_pScanResult->pFoundWords->begin();
         it != m_pScanResult->pFoundWords->end();
         ++it )
    {
        li = gtk_list_item_new_with_label (const_cast<gchar *>((*it).c_str()));
        gtk_widget_show( li );
        gtk_container_add( GTK_CONTAINER( m_matches ), li );
    }

    ZKSTD::string msg;
    ZKSTD::string dest;
    ZKSTD::string msgHeader;

    // Set dialog box title.
	if(m_pScanResult->strDomainName.length()){
        ZkStringMgr::Load( msg, IDS_OFFENDING_DEST, m_pScanResult->strDomainName.c_str() );
	}
	else if(m_pScanResult->m_SockAddr.sa_family == AF_INET)
	{
#warning Is this portable ? check later...
/*
		szDest.FormatMessage( IDS_OFFENDING_DEST, inet_ntoa(((struct sockaddr_in*)&m_pScanResult->m_SockAddr)->sin_addr));
		if(m_hAsyncTask == NULL)
		{
			m_hAsyncTask = WSAAsyncGetHostByAddr(this->GetSafeHwnd(), WM_WSAASYNCGETHOSTBYADDR_COMPLETE,
				(const char *)&((struct sockaddr_in*)&m_pScanResult->m_SockAddr)->sin_addr, sizeof(struct in_addr),
				AF_INET, pHostEnt, MAXGETHOSTSTRUCT);
		}
*/
	}

    ProtocolToString( m_protocol, dest );
    ZkStringMgr::Load( msgHeader, IDS_OFFENSIVE_HEADER, dest.c_str());

    gtk_label_set( GTK_LABEL( m_header ), msgHeader.c_str() );

    ZkGtkUtils::GtkWidgetShowAll( m_window ); 
    int answer = gnome_dialog_run( GNOME_DIALOG( m_window ) );

    switch( answer )
    {
        case 0: /* Send Message */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            m_answer = true;
            break;
            
        case 1: /* Block Message */
            gnome_dialog_close( GNOME_DIALOG( m_window ) );
            m_answer = false;
            break;
            
        case -1: /* ESC or wm close */
            m_answer = false;
            break;                
    }        
}


