// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#pragma warning (disable:4251)

//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include "ZkDialogsHelpers.h"
#include "ZkDialogsStrings.h"
#include <ZkStringMgr.h>

#include <ZkService.h>
#include <INym.h>
#include <INymCentral.h>
#include <IClientCentral.h>
#include <IZkPopup.h>

#include <clientdebug.h>


//===========================================================================
// CLASS ZkDialogsHelpers.cpp
//===========================================================================

//---------------------------------------------------------------------------
//
size_t ZkDialogsHelpers::GetNumNymsThatUseMail( )
{
	INym* pNym = NULL ;
    size_t cntNyms = ZkService::GetINymCentral()->GetNumberOfNyms();
    size_t cntNymsThatUseMail = 0;

	for (size_t i = 1; i < cntNyms; i++)
	{					 
		ZkService::GetINymCentral()->GetNym( IID_INYM, (void**)&pNym, i );

		unsigned long dwLifeStage = NYM_LIFE_STAGE_UNDEFINED ;
		pNym->GetYourLifeStage( dwLifeStage ) ;

		if ( ( dwLifeStage == NYM_LIFE_STAGE_ACTIVE ) &&
             ( pNym->DoesNymUseMail()) )
        {
            ++cntNymsThatUseMail;
        }
	}
    return cntNymsThatUseMail;
}


//---------------------------------------------------------------------------
//
bool ZkDialogsHelpers::ShouldShowOutgoingMailDlg(bool in_bUsingBogusMailServer,
                                                 bool in_bSMTP,
                                                 long& out_nSendFromNymIndex,
                                                 bool& out_bShowCheckBox)
{ 
    bool bShowDlg = false;

    out_bShowCheckBox = true;
    out_nSendFromNymIndex  =-1;

    int pref;
    if( in_bSMTP ) 
    {
        pref = PREF_NO_SHOW_OUTGOING_MAIL;
    }
    else 
    {
        pref = PREF_NO_SHOW_OUTGOING_NEWS;
    }

	if( !in_bSMTP && (GetNumNymsThatUseMail() == 0) ) {
		// it's news, so we don't care about the true ID's mail address 
		// (bug 7963)
		out_nSendFromNymIndex = USER_NYM_INDEX;
	}
    else if (in_bUsingBogusMailServer && (GetNumNymsThatUseMail() == 0) )
    {
        // Using mail.freedom.net & no mail-worthy nyms 
		IZkPopup::MessageBox( IDS_NO_NYM_CAN_SEND_MAIL, ZK_ERROR, OVER_CENTER );
		out_nSendFromNymIndex = -1;
	}
    else if (!in_bUsingBogusMailServer
             && (GetNumNymsThatUseMail() == 0))
    {
        // Not using mail.freedom.net & no mail-worthy nyms 
        out_nSendFromNymIndex =  USER_NYM_INDEX;
    }
	// Catch bug 6919: if user is 'checking only for active nym', prevent correlation...
	else if ( ZkService::GetIClientCentral()->GetPreference(ZK_CLIENT_PREF_SET, 
							PREF_CHECK_MAIL_FOR_ACTIVE_NYM_ONLY) )
    {
        // User wants the mail to go out as the active nym, and we're going to make
		// sure that happens, or the message gets blocked (prevent choice of other nym)

        size_t nActiveNym = ZkService::GetINymCentral()->GetActiveNymIndex();
        if (((USER_NYM_INDEX == nActiveNym) && !in_bUsingBogusMailServer)
			// This next line is fix for bug 8105 -- if it's news, we don't care if you
			// have a real mail account or not
			|| ((USER_NYM_INDEX == nActiveNym) && !in_bSMTP )
            || ((USER_NYM_INDEX != nActiveNym)
                && (ZkService::GetINymCentral()->DoesNymUseMail(nActiveNym))) )
        {
			// OK, active nym is mail-capable.

			if( !ZkService::GetIClientCentral()->GetPreference(ZK_SHOW_AGAIN_PREF_SET, 
							PREF_NO_SHOW_MAIL_SENT_AS_ACTIVE_NYM ) )
			{
				// Warn user that mail will go out as active nym
				bool bChecked = false;
				if( IZkPopup::QuestionCheckBox( IDS_MAIL_WILL_BE_SENT_AS_NYM, bChecked, IDS_STOCK_OK, IDS_STOCK_CANCEL ) )
				{	// ok
					out_nSendFromNymIndex =  nActiveNym;
				}
				else
				{	// cancel
					out_nSendFromNymIndex = -1;
				}
				if( bChecked ) {
					ZkService::GetIClientCentral()->SetPreference( ZK_SHOW_AGAIN_PREF_SET, 
							PREF_NO_SHOW_MAIL_SENT_AS_ACTIVE_NYM, true );
				}
			}
			else
			{	// go ahead and send it as the active nym
				out_nSendFromNymIndex = nActiveNym;
			}
        }
        else
        {
            // The active nym can't send mail

			// bug 6919: prevent user from sending if he's 'checking for active nym only'
			// Pop up a message box; user has to retry with a premium nym selected
			IZkPopup::MessageBox( IDS_MAIL_PREVENT_CORRELATION );

			out_nSendFromNymIndex = -1;
        }
    }
	// Now that the bug 6919 condition is out of the way, deal with the outgoingmsg dialog
    else if ( ZkService::GetIClientCentral()->GetPreference(ZK_SHOW_AGAIN_PREF_SET, pref) )
	{
        // User wants the mail to go out as the active nym

        size_t nActiveNym = ZkService::GetINymCentral()->GetActiveNymIndex();
        if (((USER_NYM_INDEX == nActiveNym) && !in_bUsingBogusMailServer)
			// This next line is fix for bug 8105 -- if it's news, we don't care if you
			// have a real mail account or not
			|| ((USER_NYM_INDEX == nActiveNym) && !in_bSMTP )
            || ((USER_NYM_INDEX != nActiveNym)
                && (ZkService::GetINymCentral()->DoesNymUseMail(nActiveNym))) )
        {
			// OK, active nym is mail-capable.
			out_nSendFromNymIndex = nActiveNym;
		}
        else
        {
            // The active nym can't send mail
			// do not show the "Do not show again" check box
			out_bShowCheckBox = false;
			bShowDlg = true;
		}

	}
    else
    {
        bShowDlg = true;
    }

    return bShowDlg;
}
