// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <WzBasePage.h>
#include <WzBase.h>
#include <ZkGladeLoader.h>
#include <ZkGtkUtils.h>

#include <clientdebug.h>


//===========================================================================
// CLASS WzBasePage
//===========================================================================

//---------------------------------------------------------------------------
//
WzBasePage::WzBasePage( const ZKSTD::string& in_name )
    : m_parent( 0 )
    , m_skip( false )
    , m_topCt( 0 )
    , m_subCt( 0 )
    , m_name( in_name )
    , m_default( 0 )
{
    ZkGladeLoader loader( "wizard_base.xml", "Wizard_ct" );
    
    // The only reason to have two containers here is for future expansion.
    // We may put some standard basic widgets such as a label on each page
    // in the future, at which point ct will be used to hold the entire
    // page while subCt will be used to put the derived classes UI.
    m_topCt = loader.GetWidget( "Wizard_ct" );
    m_subCt = loader.GetWidget( "Wizard_subCt" );
    m_pixmap = loader.GetWidget( "pixmap" );
    
    // Keep our own instance so we control the container life scope.
    // Remove it from its current parent since we got it from the XML spec.
    // (the real parent will be the WzSetupDlg when we add the page to the
    // wizard)
    gtk_widget_ref( m_topCt );
    gtk_widget_unparent( m_topCt );

}

//---------------------------------------------------------------------------
//
WzBasePage::~WzBasePage( )
{
    // Unparent the container.
    // We got our own ref in order to control the lifescope. Otherwise 
    // unparenting would have destroyed the container.
    gtk_widget_unparent( m_topCt );

    // Now destroy it.
    gtk_widget_destroy( m_topCt );    
}

//---------------------------------------------------------------------------
//
void
WzBasePage::SetParent( WzBase* dlg, GtkWidget* parent )
{
    ZKASSERT( dlg != 0 );
    ZKASSERT( parent != 0 );
    ZKASSERT( GTK_IS_CONTAINER( parent ) );

    m_parent = dlg;
    gtk_container_add( GTK_CONTAINER( parent ), m_topCt );
}

//---------------------------------------------------------------------------
//
void
WzBasePage::UnParent( )
{
    m_parent = NULL;
    // FIXME: ->parent is not beautiful
    gtk_container_remove( GTK_CONTAINER( m_topCt->parent ), m_topCt );
}

//---------------------------------------------------------------------------
//
void
WzBasePage::AddPageUi( GtkWidget* ct )
{
    gtk_widget_ref( ct );
    gtk_widget_unparent( ct );
    
    gtk_box_pack_start( GTK_BOX( m_subCt ), ct, TRUE, TRUE, 0 );
    gtk_widget_show( ct );

    gtk_widget_unref( ct );
}

//---------------------------------------------------------------------------
//
void
WzBasePage::Hide()
{
    ZKASSERT( m_topCt );
    gtk_widget_hide( m_topCt );
}

//---------------------------------------------------------------------------
//
void
WzBasePage::Show()
{
    ZKASSERT( m_topCt );
    gtk_widget_show( m_topCt );
   

    // Check whether or not a widget should have the focus by default.
    if( m_default != 0 )
    {
        gtk_widget_grab_focus( m_default );
    }
}

//---------------------------------------------------------------------------
//
void
WzBasePage::NextPageCb( GtkWidget*,  WzBasePage* dlg )
{
    ZKASSERT( dlg );

    // This function in turn will have the effect of calling NextPage() 
    // on the page.
    if( dlg->GetParent() != 0 ) 
    {
        dlg->GetParent()->NextPage();
    }
}
