
// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom(R) code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

  


//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <WzLicense.h>
#include <WzSetupDlgImp.h>

#include <ZkGladeLoader.h>
#include <ZkGtkUtils.h>
#include <ZKFileConfiguration.h>
#include <IZkFileHelp.h>

//===========================================================================
// CLASS WzLicense
//===========================================================================

//---------------------------------------------------------------------------
//
WzLicense::WzLicense( )
    : WzSetupBasePage( "LicenseAgreement" )
{
    ZkGladeLoader loader( "wizard_license.xml", "Wizard_ct" );

    // Get the label and make it bigger.
    GtkWidget* title = loader.GetWidget( "Wizard_title" );
    ZkGtkUtils::UseTitleFont( title );

    // Parent this UI in the base class container.
    AddPageUi( loader.GetWidget( "Wizard_ct" ) );

    m_agreement = loader.GetWidget( "Agreement" );

    ZKSTD::string licenseFile( ZKFileConfiguration::GetFreedomHelpDir() );
    licenseFile += "LICENSE";

    ZKSTD::string licenseText;
    bool bSuccess = IZkFileHelp::ReadTextFile( licenseFile, licenseText );

    if( bSuccess )
    {
        gtk_label_set_text( GTK_LABEL( m_agreement ), licenseText.c_str() );
    }
}

//---------------------------------------------------------------------------
//
WzLicense::~WzLicense( )
{
}

//---------------------------------------------------------------------------
//
bool WzLicense::NextPage()
{
    // User agreed, so remove the license page and start normally.
    GetMainDialog()->StartMode( );
    
    // Set the current page.
    // Interestingly enough, we have to do this twice, first to make the
    // current page different from zero, then to set it back to zero.
    // This is due to the fact the wizard considers the page index to
    // determine the active page. When we remove the license page, the
    // index of the Welcome page becomes zero, which would be the active page.
    GetMainDialog()->SetActivePage( "SetupWelcome" );

    // Just make sure we do nothing after this call, since it is equivalent
    // to self-destruct.
    GetMainDialog()->RemovePage( GetName() );    

    // Set the current page.
    GetMainDialog()->SetActivePage( "SetupWelcome" );
    

    return true;
}

