// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include "WzRoute.h"
#include "WzRoutePage.h"

#include <ZkFreedomWizardsStrings.h>

#include <ZkGladeLoader.h>
#include <ZkGtkUtils.h>
#include <ZkStringMgr.h>

#include <ZkService.h>
#include <INiqs.h>
#include <INetworkPerformance.h>
#include <IZkException.h>
#include <IZKErrorReporter.h>
#include <ZkErrorHandler.h>
#include <IZkRegistry.h>

#include <IZkPopup.h>

#include <gtk/gtk.h>

const int HOP_INDEX_FIRST = 0;
const int HOP_INDEX_SECOND = 1;
const int HOP_INDEX_THIRD = 2;

//===========================================================================
// CLASS WzRoutePage
//===========================================================================

//---------------------------------------------------------------------------
//
WzRoutePage::WzRoutePage( guint hop )
    : WzBasePage( "RouteSelect" )
    , m_sort_type( GTK_SORT_ASCENDING )
    , m_last_column( -1 )
    , m_indexHop( hop )
	, m_bShowDownAips( false )
{
    ZkGladeLoader loader( "wizard_route.xml", "Wizard_route_ct" );

    m_clist = loader.GetWidget( "clist" );
    m_HopLabel = loader.GetWidget( "Wizard_text" );

    ZKSTD::string hopInstructionString;
    ZkStringMgr::Load( hopInstructionString,
                       IDS_INSTRUCTION_DEFAULT_HOP );
    switch( m_indexHop ) {
        case( HOP_INDEX_FIRST ):
           ZkStringMgr::Load( hopInstructionString,
                              IDS_INSTRUCTION_FIRST_HOP );
           break;
        case( HOP_INDEX_SECOND ):
           ZkStringMgr::Load( hopInstructionString,
                              IDS_INSTRUCTION_SECOND_HOP );
           break;
        case( HOP_INDEX_THIRD ):
           ZkStringMgr::Load( hopInstructionString,
                              IDS_INSTRUCTION_THIRD_HOP );
           break;
        default:
           break;
    }
    gtk_label_set_text( GTK_LABEL( m_HopLabel ),
                        hopInstructionString.c_str());

    // Get events when the user clicks on the clist title buttons
    gtk_clist_column_titles_active( GTK_CLIST(m_clist) );
    gtk_signal_connect( GTK_OBJECT(m_clist), "click_column",
			GTK_SIGNAL_FUNC(SortClist), this );

    gtk_signal_connect( GTK_OBJECT(m_clist), "select_row",
			GTK_SIGNAL_FUNC( SelectHop ), this );

    gtk_signal_connect( GTK_OBJECT(m_clist), "unselect_row",
			GTK_SIGNAL_FUNC( UnselectHop ), this );

    gtk_object_set_data( GTK_OBJECT(m_clist), "imp", (gpointer)this );

    // Parent this UI in the base class container.
    AddPageUi( loader.GetWidget( "Wizard_route_ct" ) );

    // Hide the pixmap on the left
    HidePixmap();

	// Normally only NOC or testers would ever want to try to create
	// a route through a down server, thats why functionality is only available through
	// registry
	m_bShowDownAips = IZkRegistry::GetFlag( "RouteWizardShowDownAips" ) ;

    // FIXME: fixup instructions depending on hop
}

//---------------------------------------------------------------------------
//
WzRoutePage::~WzRoutePage()
{
}

//---------------------------------------------------------------------------
// The sort function used when sorting by the ping time
//
gint
WzRoutePage::SortByPing( GtkCList *clist, gconstpointer ptr1,
			     gconstpointer ptr2 )
{
    gpointer klass = gtk_object_get_data( GTK_OBJECT(clist), "imp" );
    WzRoutePage *imp = (WzRoutePage *)klass;

    gint a = 0, b = 0;
    guint i;

    for ( i = 0; i < 6; i++ )
    {
	if ( static_cast<WzRoute *>(imp->GetParent())->GetRatings(i) == ptr1 )
	    a = i;
	if ( static_cast<WzRoute *>(imp->GetParent())->GetRatings(i) == ptr2 )
	    b = i;
	if ( a && b )
	    break;
    }

    if ( a < b )
	return -1;
    else if ( a == b )
	return 0;
    else
	return 1;
}

//---------------------------------------------------------------------------
//
void
WzRoutePage::SortClist( GtkWidget *clist, gint column, gpointer klass )
{
    WzRoutePage *imp = (WzRoutePage *)klass;

    if ( column == imp->GetLastColumn() )
    {
	imp->SetSortType( imp->GetSortType() == GTK_SORT_ASCENDING ?
			  GTK_SORT_DESCENDING : GTK_SORT_ASCENDING );
	gtk_clist_set_sort_type( GTK_CLIST(clist), imp->GetSortType() );
    }

    if ( column == 0 )
    {
	gtk_clist_set_compare_func( GTK_CLIST(clist), SortByPing );
	gtk_clist_set_sort_column( GTK_CLIST(clist), 0 );
	gtk_clist_sort( GTK_CLIST(clist) );
	gtk_clist_set_compare_func( GTK_CLIST(clist), NULL );
    }
    else
    {
	gtk_clist_set_sort_column( GTK_CLIST(clist), column );
	gtk_clist_sort( GTK_CLIST(clist) );
    }

    imp->SetLastColumn( column );
}

//---------------------------------------------------------------------------
//
gint WzRoutePage::SelectHop( 
    GtkWidget *, 
    gint row, 
    gint,
    GdkEventButton *, 
    WzRoutePage* inst
)
{
    WzRoute *wiz = static_cast<WzRoute *>(inst->GetParent());

    WNetInfoDatum* pInfo = NULL;
    inst->GetItemInfo( row, &pInfo );

    if ( pInfo == NULL )
    {
        return true;
    }


    if (( wiz->GetRoute().GetSize() > inst->m_indexHop ) &&
        ( wiz->GetRoute().GetHop( inst->m_indexHop ) ==
          pInfo->GetId() ))
    {
        // we have the same selection now as we did last time we came
        // through so just leave m_Route as it is.  This
        // preserves selections on pages later in the wizard, if any
        ZKTRACE0( " ** Same Selection ** \n " );
    } 
    else 
    {
        // Make sure we don't have any extra hops before we add
        // this one
        while ( wiz->GetRoute().GetSize() > inst->m_indexHop )
        {
            wiz->GetRoute().PopHop();
        }

        ZKTRACE1( " ** AddHop( %d ) ** \n", pInfo->GetId() );
        wiz->GetRoute().AddHop( pInfo->GetId() );
    }

    return true;
}

//---------------------------------------------------------------------------
//
gint WzRoutePage::UnselectHop( 
    GtkWidget *, 
    gint, 
    gint,
    GdkEventButton *, 
    WzRoutePage* inst
)
{
    WzRoute *wiz = static_cast<WzRoute *>(inst->GetParent());

    ZKTRACE0( "** Unselecting.  **\n" );

    // Erase any existing entity (user has unselected)
    while ( wiz->GetRoute().GetSize() > inst->m_indexHop )
    {
        wiz->GetRoute().PopHop();
    }

    return true;
}

//---------------------------------------------------------------------------
//
void WzRoutePage::GetConnectedAips( 
    gint in_idPreviousHop,
    WNetInfoDatumVec& out_lst 
)
{
    out_lst.clear() ;

    WNetInfoDatum PreviousAip ;

    // User is supposed to have already chosen the previous hop
    ZKASSERT( in_idPreviousHop != 0 ) ;

    if( ZkService::GetINiqs()->GetEntityById( in_idPreviousHop, PreviousAip ) )
    {			
        // We look at all the Aips connected to this aips and add the valid
        // ones to the list
        guint cntAipConnections = PreviousAip.GetCntConnections() ;
        guint j;

        for (j = 0; j < cntAipConnections; j++) {
            WNetInfoDatum ConnectedAip ;
            gint idConnectedAip = PreviousAip.GetConnectionX( j ) ;

            if( !ZkService::GetINiqs()->GetEntityById( idConnectedAip, ConnectedAip ) )
            {
                // NIQS doesn't know about this, must be
                // a corrupt NIQS database
                continue ;
            }

            if ( ConnectedAip.IsCoreAip() )
            {
                // Nyms never use Core Aips so don't show them
                continue ;
            }
			
            if ( !m_bShowDownAips && !ConnectedAip.IsServerUp() )
            {
                // Don't show Down aips - we want the
                // selected route to work
                continue ;
            }

            // Verify that we don't already have a particular Aip in
            // the list - ( zks1, zks2, zks1 is not a valid route! )

            if ( static_cast<WzRoute *>(GetParent())->GetRoute().IsHopInRoute( idConnectedAip ) )
            {
                continue ;
            }
	    
            out_lst.push_back( ConnectedAip ) ;
        }								
    }

    if ( out_lst.empty() ) {
        // Generally this should very rarely happen - presumably
        // all the aips are down

        IZkPopup::MessageBox( IDS_NO_HOPS_FROM_AIP ) ;
    }
}

//---------------------------------------------------------------------------
//
gint WzRoutePage::InsertAip( const WNetInfoDatum *in_pInfo )
{
    if (in_pInfo == NULL) {
	ZKASSERT(FALSE);
	return -1;
    }
    // Get the entity name
    ZKSTD::string strName ;
    in_pInfo->GetEntityName( strName ) ;

    // Format the actual item

    // We use our own performance value now instead of the rating coming back
    // from the server this is because the rating will no longer be useful
    // information with new NIQS

    gint score = ZkService::GetINetworkPerformance()->GetEntityPerformanceResult( in_pInfo->GetId() ) ;

    // Convert between 0-255 and 1-5 with special case for score == 255
    gint iImageIndex = 1 + score / 51;

    if ( iImageIndex > 5 ) {
	iImageIndex = 5;
    }

    // We'll use 1 as a minimum image. Eg: A rating of 0 means 1 green bar icon
    ZKASSERT(iImageIndex >= 1 && iImageIndex <= 5);

    // Except if it is really down, then we'll use image 0, a red icon
    if (0 == in_pInfo->GetStatus()) {
	ZKTRACE1("We have a downed aip [%s], using red icon image\n",
		 strName.c_str());
	iImageIndex = 0;
    }

    ZKSTD::string strRegion ;
    in_pInfo->GetRegion( strRegion ) ;

    ZKSTD::string strOperator ;
    in_pInfo->GetOperator( strOperator ) ;

    const char *foo[5] = { NULL, // image
			   strName.c_str(),
			   strRegion.c_str(),
			   strOperator.c_str(),
			   inet_ntoa(in_pInfo->GetSockAddr().sin_addr) };

    gint iAddedPos = gtk_clist_append( GTK_CLIST(m_clist),
				       const_cast<gchar **>(foo) );

    gtk_clist_set_pixmap( GTK_CLIST(m_clist), iAddedPos, 0,
			  static_cast<WzRoute *>(GetParent())->GetRatings(iImageIndex), NULL);

    // Remember pointer back to the data.
    // Note, however, that we don't copy the data ourself
    gtk_clist_set_row_data( GTK_CLIST(m_clist), iAddedPos,
			    (gpointer)const_cast<WNetInfoDatum *>(in_pInfo) );

    return iAddedPos;
}

//---------------------------------------------------------------------------
// Returns the selected row of the clist, or -1 for none
gint WzRoutePage::GetCurrentSelection( )
{
    ZKASSERT(m_clist);

    if ( !GTK_CLIST(m_clist)->selection )
	return -1;

    return (gint)( GTK_CLIST(m_clist)->selection->data );
}

//---------------------------------------------------------------------------
// Returns the selected row of the clist, or -1 for none
//
void WzRoutePage::GetItemInfo( gint row, WNetInfoDatum** pInfo )
{
    *pInfo = (WNetInfoDatum *)gtk_clist_get_row_data( GTK_CLIST(m_clist), row );
        
    ZKASSERT( *pInfo );
}

//---------------------------------------------------------------------------
//
void
WzRoutePage::SetParent( WzBase *imp, GtkWidget *widget )
{
    WzBasePage::SetParent( imp, widget );

    // Verify that we're being inserted in a Route wizard
    ZKASSERT( "Valid parent type" && dynamic_cast<WzRoute *>(GetParent()) );
}

//---------------------------------------------------------------------------
//
void
WzRoutePage::Show( )
{
    WzBasePage::Show( );

    if ( static_cast<WzRoute *>(GetParent())->GetRoute().GetSize() > m_indexHop )
    {	
	// We don't refresh the list if we already have established
	// a selection for this page.  This allows user to scroll
	// through the changes without losing his/her choices

	return;
    }

    if ( m_indexHop > 0 )
    {
	// Previous pages should have already established hops
	// at least up to this index
	ZKASSERT( static_cast<WzRoute *>(GetParent())->GetRoute().GetSize() >
		  m_indexHop - 1 ) ;
    }

    // Rebuild our list
    gtk_clist_freeze( GTK_CLIST(m_clist) );
    m_lst.clear() ;
    gtk_clist_clear( GTK_CLIST(m_clist) );

    // Fill in the UI with this information
    // We don't show the down servers

    try
    {
	// Fill the m_lst array with the aips we want to show on this page
	if ( m_indexHop == 0 )
	{	
	    // First page we show all aips		
	    ZkService::GetINiqs()->GetListOfAips( 
		true,			// Include non-core aips
		false,			// Exclude core aips
		m_bShowDownAips,
		m_lst ) ;

	    // Bad network conditions?  Usually Freedom won't run if
	    // if no Aips
	    ZKASSERT(m_lst.size() > 0);
	}
	else
	{
	    // Later pages we only show aips connected to the previous item
	    gint idPreviousHop =
		static_cast<WzRoute *>(GetParent())->GetRoute().GetHop( m_indexHop - 1 ) ;
	    GetConnectedAips( idPreviousHop, m_lst ) ;
	}

	// Fill the display based on the aips in the list
	for (WNetInfoDatumVec::const_iterator it = m_lst.begin();
	     it != m_lst.end(); ++it)
	{
	    const WNetInfoDatum& Info = *it ;
	    ZKASSERT(Info.GetId() > 0);

	    // We don't show down servers on this dialog
	    ZKASSERT( m_bShowDownAips || Info.IsServerUp() ) ;
	    InsertAip( &Info );
	}
    }
    catch( IZkException& ex )
    {
	ZkErrorHandler::GetIZkErrorReporter()->ShowError( ex );
    }

    gtk_clist_columns_autosize( GTK_CLIST(m_clist) );
    gtk_clist_thaw( GTK_CLIST(m_clist) );
}

//---------------------------------------------------------------------------
//
bool
WzRoutePage::Finish()
{
    return NextPage();
}

//---------------------------------------------------------------------------
//
bool
WzRoutePage::NextPage()
{
    if ( static_cast<WzRoute *>(GetParent())->GetRoute().GetSize() <= m_indexHop )
    {
	// We have no selection
	ZKASSERT( GetCurrentSelection() == -1 );
	IZkPopup::MessageBox( "Please select a server from the list to be the next hop" ) ;
	return true;
    }

    return false;
}
