// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include "WzRoute.h"
#include "WzRoutePageFirst.h"

#include <ZkGladeLoader.h>
#include <ZkGtkUtils.h>

#include <WRouteRules.h>

#include <ZkService.h>
#include <INymRoute.h>
#include <IRouteManager.h>

#include <guids.h>
#include <INymCentral.h>

/*===========================================================================
 * CLASS WzRoutePageFirst
 *=========================================================================*/

//---------------------------------------------------------------------------
//
WzRoutePageFirst::WzRoutePageFirst( )
    : WzBasePage( "RouteSelect" )
{
    ZkGladeLoader loader( "wizard_route.xml", "Wizard_first_ct" );

    m_auto_button  = loader.GetWidget( "auto_button" );
    m_dont_show    = loader.GetWidget( "dont_show" );
    m_interactive  = loader.GetWidget( "interactive" );
    m_last_used    = loader.GetWidget( "last_used" );
    m_server_names = loader.GetWidget( "server_names" );

    gtk_signal_connect( GTK_OBJECT(m_auto_button), "toggled",
			GTK_SIGNAL_FUNC(OnAutoRouteCb), this );
    gtk_signal_connect( GTK_OBJECT(m_interactive), "toggled",
			GTK_SIGNAL_FUNC(OnUserSelectedRouteCb), this );
    gtk_signal_connect( GTK_OBJECT(m_last_used), "toggled",
			GTK_SIGNAL_FUNC(OnUseLastRouteCb), this );

    // Parent this UI in the base class container.
    AddPageUi( loader.GetWidget( "Wizard_first_ct" ) );
}

WzRoutePageFirst::~WzRoutePageFirst()
{
}

//---------------------------------------------------------------------------
//
void
WzRoutePageFirst::OnAutoRoute( )
{
    if ( !GTK_TOGGLE_BUTTON(m_auto_button)->active )
	return;

    static_cast<WzRoute *>(GetParent())->RemoveHopPagesFromWizard();

    // Re-enable ``Don't Show Again'' option
    gtk_widget_set_sensitive( m_dont_show, TRUE );
}

//---------------------------------------------------------------------------
//
void 
WzRoutePageFirst::OnUserSelectedRoute( )
{
    if ( !GTK_TOGGLE_BUTTON(m_interactive)->active )
	return;

    static_cast<WzRoute *>(GetParent())->AddHopPagesToWizard() ;

    // Disable Don't Show Again option
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(m_dont_show), FALSE );
    gtk_widget_set_sensitive( m_dont_show, FALSE );

}

//---------------------------------------------------------------------------
//
void
WzRoutePageFirst::OnUseLastRoute( )
{
    if ( !GTK_TOGGLE_BUTTON(m_last_used)->active )
	return;

    static_cast<WzRoute *>(GetParent())->RemoveHopPagesFromWizard();

    // Disable Don't Show Again option
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(m_dont_show), FALSE );
    gtk_widget_set_sensitive( m_dont_show, FALSE );
}

//---------------------------------------------------------------------------
//
void
WzRoutePageFirst::OnAutoRouteCb( GtkWidget *, gpointer klass )
{
    WzRoutePageFirst *imp = (WzRoutePageFirst *)klass;

    imp->OnAutoRoute();
}

//---------------------------------------------------------------------------
//
void
WzRoutePageFirst::OnUserSelectedRouteCb( GtkWidget *, gpointer klass )
{
    WzRoutePageFirst *imp = (WzRoutePageFirst *)klass;

    imp->OnUserSelectedRoute();
}

//---------------------------------------------------------------------------
//
void
WzRoutePageFirst::OnUseLastRouteCb( GtkWidget *, gpointer klass )
{
    WzRoutePageFirst *imp = (WzRoutePageFirst *)klass;

    imp->OnUseLastRoute();
}

//---------------------------------------------------------------------------
//
void
WzRoutePageFirst::SetParent( WzBase *imp, GtkWidget *widget )
{
    WzBasePage::SetParent( imp, widget );

    // Verify that we're being inserted in a Route wizard
    WzRoute * wiz = dynamic_cast<WzRoute *>(GetParent());
    ZKASSERT( "Valid parent type" && wiz );

    if ( wiz->GetLastRoute().GetSize() ==
	 wiz->GetRouteRules().GetHopCount() )
    {
	ZKSTD::string lastlabel( "( " ) ;
	lastlabel += ZkService::GetIRouteManager()->GetRouteDesc( wiz->GetLastRoute() ).c_str();
	lastlabel += " )" ;

	gtk_label_set_text( GTK_LABEL(m_server_names), lastlabel.c_str() );
    }
    else
    {
	// Option is not available

	gtk_widget_hide( m_last_used );
	gtk_widget_hide( m_server_names );
    }

    // By default select auto
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(m_auto_button), TRUE );
}
