/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_WZSETUPDLG
#define ZKS_INCL_WZSETUPDLG

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include <clientstl.h>

#include <string>
#include <memory>

class WzSetupDlgImp;
class WzSetupBasePage;

/*===========================================================================
 * CLASS WzSetupDlg
 *=========================================================================*/

/*
  Main dialog to build Wizards.
  This is just the wrapper class that allows to manipulate a wizard dialog
  without any dependency on gtk+. For the implementation see WzSetupDlgImp.h
*/

class WzSetupDlg {

public: 

    /*----- member functions -----*/
      
    /*
      FIXME - The flag here tells the setup wizard whether to build a Setup
              wizard, or a Nym Creation wizard. It is really the same object,
              it just ends up having different pages.
              There is a fixme because the addition of the pages really 
              shouldn't be the problem of the object containing the pages,
              be some other higher level object.

              Same thing for the IsFreeNym(). It doesn't belong here.
              Corners are cut round these days.

              ericb
     */
    WzSetupDlg( bool NymWizard = false );
    ~WzSetupDlg();

    bool Run();

    void AddPage( ZKSTD::auto_ptr< WzSetupBasePage > page );

	// Purpose:  Sets the currently displayed page.
	// Throws:   FIXME - throw if not in the wizard.
    void SetActivePage( const ZKSTD::string& name );

	// Purpose:  Indicates whether or not we are using the free or "pro"
    //           version. Influences the content of some pages.
    bool IsFreeNym() const;

private: 

    /*----- member functions -----*/

    // prohibited funcs
    WzSetupDlg( const WzSetupDlg& );
    const WzSetupDlg& operator=( const WzSetupDlg& );

    /*----- data members -----*/

    WzSetupDlgImp* m_impl;
};



#endif
