// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <WzSetupDlgImp.h>
#include <WzSetupBasePage.h>
#include <ZkFreedomWizardsStrings.h>
#include <WzSetupInit.h>

#include <UiThreadId.h>
#include <lockguard.h>
#include <ZKFileConfiguration.h>
#include <ZkGladeLoader.h>
#include <ZkCatalogRegistry.h>
#include <IZkPopup.h>
#include <ZkGtkUtils.h>
#include <ZkWebIntegration.h>

#include <clientdebug.h>

#include <glade/glade.h>

#include <algorithm>

//===========================================================================
// LOCAL DECLARATIONS
//===========================================================================

#define V_SPACING 0
#define H_SPACING 3
#define PADDING   2
#define BORDER    4
#define PADDING_H 30 // we add this height to make space for the bottom buttons.

//===========================================================================
// CLASS WzSetupDlgImp
//===========================================================================

//---------------------------------------------------------------------------
//
WzSetupDlgImp::WzSetupDlgImp( bool NymWizard )
    : m_currentPage( 0 )
   , m_mode( START )
   , m_max_w( 0 )
   , m_max_h( 0 )
   , m_done( false )
   , m_freeVersion( true )
   , m_nymWizard( NymWizard )
   , m_built( false )
   , m_startTestDrive( true )
{
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::BuildUI()
{
   /*
      Well. Glade XML is very nice, but here I think we will just have to 
      do it the hard way. (maybe we can migrate this to an XML def later)
      We create our own sort of dialog box to contain wizard pages.
      Decided not to use druids because of many uneeded features, also cannot 
      use a GnomeDialog because we need to be able to add and remove buttons
      in the 'action area'.
    */
//    UiThreadGuard();

    if ( !m_built ) 
    {
        m_window = gtk_window_new( GTK_WINDOW_DIALOG );

        gtk_window_set_modal( GTK_WINDOW( m_window ), TRUE );
        gtk_window_set_policy( GTK_WINDOW( m_window ), FALSE, FALSE, FALSE );
        gtk_window_set_position( GTK_WINDOW( m_window ), GTK_WIN_POS_CENTER );

        m_vbox   = gtk_vbox_new( FALSE, V_SPACING );
        m_hbox   = gtk_hbox_new( TRUE, 0 );
        m_pageCt = gtk_hbox_new( FALSE, 0 );
        m_btnbox_left  = gtk_hbutton_box_new();
        m_btnbox_right = gtk_hbutton_box_new();
    
        gtk_container_set_border_width( GTK_CONTAINER( m_btnbox_right ), BORDER);
        gtk_container_set_border_width( GTK_CONTAINER( m_btnbox_left ),  BORDER);

        gtk_button_box_set_spacing( GTK_BUTTON_BOX( m_btnbox_right ), H_SPACING );
        gtk_button_box_set_spacing( GTK_BUTTON_BOX( m_btnbox_left ),  H_SPACING );
        gtk_button_box_set_child_ipadding( GTK_BUTTON_BOX( m_btnbox_right ), PADDING, PADDING );
        gtk_button_box_set_child_ipadding( GTK_BUTTON_BOX( m_btnbox_left ),  PADDING, PADDING );

        // Assemble those containers.
        // Window, with vertical box. Button box at the bottom with page
        // container at the top.
        gtk_container_add( GTK_CONTAINER( m_window ), m_vbox );
        gtk_box_pack_start( GTK_BOX( m_vbox ), m_pageCt, TRUE, TRUE, 0 );    
        gtk_box_pack_start( GTK_BOX( m_vbox ), gtk_hseparator_new(), FALSE, FALSE, 0 );
        gtk_box_pack_start( GTK_BOX( m_vbox ), m_hbox, FALSE, FALSE, 0 );

        gtk_box_pack_start( GTK_BOX( m_hbox ), m_btnbox_left,  FALSE, FALSE, 0 );
        gtk_box_pack_start( GTK_BOX( m_hbox ), m_btnbox_right, FALSE, FALSE, 0 );


        gtk_button_box_set_layout( GTK_BUTTON_BOX( m_btnbox_left ),  GTK_BUTTONBOX_SPREAD );
        gtk_button_box_set_layout( GTK_BUTTON_BOX( m_btnbox_right ), GTK_BUTTONBOX_SPREAD );


    // Then create all buttons and reference them so that we can remove them 
    // from the button box without loosing them.

        // FIXME - remove this test when we derive from WzBase.
        if( m_nymWizard ) {
            m_start    = gnome_stock_or_ordinary_button( GNOME_STOCK_BUTTON_NEXT );

        }
        else {
            m_start    = gnome_stock_or_ordinary_button( 
                ZkCatalogRegistry::GetString( IDS_START ));
        }

        m_finish   = gnome_stock_or_ordinary_button( 
            ZkCatalogRegistry::GetString( IDS_FINISH ));
        m_previous = gnome_stock_or_ordinary_button( GNOME_STOCK_BUTTON_PREV );
        m_next     = gnome_stock_or_ordinary_button( GNOME_STOCK_BUTTON_NEXT );
        m_help     = gnome_stock_or_ordinary_button( GNOME_STOCK_BUTTON_HELP );
        m_cancel   = gnome_stock_or_ordinary_button( GNOME_STOCK_BUTTON_CANCEL );
        m_no       = gnome_stock_or_ordinary_button( GNOME_STOCK_BUTTON_NO );
        m_yes      = gnome_stock_or_ordinary_button( GNOME_STOCK_BUTTON_YES );
    
        gtk_widget_ref( m_start );                               
        gtk_widget_ref( m_finish ); 
        gtk_widget_ref( m_previous );
        gtk_widget_ref( m_next );
        gtk_widget_ref( m_help );
        gtk_widget_ref( m_cancel );
        gtk_widget_ref( m_no );
        gtk_widget_ref( m_yes );

        gtk_widget_show( m_start );                               
        gtk_widget_show( m_finish ); 
        gtk_widget_show( m_previous );
        gtk_widget_show( m_next );
        gtk_widget_show( m_help );
        gtk_widget_show( m_cancel );
        gtk_widget_show( m_no );
        gtk_widget_show( m_yes );
        gtk_widget_show_all( m_vbox );

        // Hook functions.
        gtk_signal_connect (GTK_OBJECT( m_start ), "clicked",
                            GTK_SIGNAL_FUNC( WzSetupDlgImp::StartCb ), this );
        gtk_signal_connect (GTK_OBJECT( m_finish ), "clicked",
                            GTK_SIGNAL_FUNC( WzSetupDlgImp::FinishCb ), this );
        gtk_signal_connect (GTK_OBJECT( m_previous ), "clicked",
                            GTK_SIGNAL_FUNC( WzSetupDlgImp::PreviousCb ), this );
        gtk_signal_connect (GTK_OBJECT( m_next ), "clicked",
                            GTK_SIGNAL_FUNC( WzSetupDlgImp::NextCb ), this );
        gtk_signal_connect (GTK_OBJECT( m_help ), "clicked",
                            GTK_SIGNAL_FUNC( WzSetupDlgImp::HelpCb ), this );
        gtk_signal_connect (GTK_OBJECT( m_cancel ), "clicked",
                            GTK_SIGNAL_FUNC( WzSetupDlgImp::CancelCb ), this );
    
        gtk_signal_connect (GTK_OBJECT( m_yes ), "clicked",
                            GTK_SIGNAL_FUNC( WzSetupDlgImp::NextCb ), this );
        gtk_signal_connect (GTK_OBJECT( m_no ), "clicked",
                            GTK_SIGNAL_FUNC( WzSetupDlgImp::CancelCb ), this );

    
        gtk_signal_connect (GTK_OBJECT( m_window ), "delete_event",
                            GTK_SIGNAL_FUNC( WzSetupDlgImp::CloseCb ), NULL );


        // Initial mode is START
        gtk_box_pack_end( GTK_BOX( m_btnbox_left ), m_start, FALSE, FALSE, 0 );

        gtk_box_pack_start( GTK_BOX( m_btnbox_right ), m_cancel, FALSE, FALSE, 0 );
        gtk_box_pack_start( GTK_BOX( m_btnbox_right ), m_help, FALSE, FALSE, 0 );

        if( m_nymWizard ) {
            WzSetupInit::AddNymCreatePages( this );
        }
        else {
            WzSetupInit::AddSetupPages( this );
        }

        m_built = true;
    }
}

//---------------------------------------------------------------------------
//
WzSetupDlgImp::~WzSetupDlgImp()
{
    if ( m_built )
    {
        gtk_widget_unref( m_start );                               
        gtk_widget_unref( m_finish ); 
        gtk_widget_unref( m_previous );
        gtk_widget_unref( m_next );
        gtk_widget_unref( m_help );
        gtk_widget_unref( m_cancel );
        m_built = false;
    }
}

//---------------------------------------------------------------------------
//
bool WzSetupDlgImp::Run( )
{
    // First acquire the lock on the data members so we can safely wait
    // in a thread while the ui does its job.
    ZKS::Lock_Guard< WZkMutex > gate( m_lock );

    // Have the UiThreadCallback function called from the proper thread.
    NotifyUi();

    // FIXME - To be moved when we split Nym Creation and Setup Wizard.
    if( m_done && !m_nymWizard && m_startTestDrive )
    {
        if( IsFreeVersion() )
        {
            ZkWebIntegration::LaunchWebSite( URL_TESTDRIVE_ID );
        }
        else
        {
            ZkWebIntegration::LaunchWebSite( URL_TESTDRIVE_STD_NYM_ID );
        }
    }

    return m_done;
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::UiThreadCallback() 
{
    // Build ui from the UI thread, X is not exactely thread safe.
    BuildUI();

    ZkGtkUtils::GtkWidgetShow( m_window );
    gtk_main();
    gtk_widget_hide( m_window );
}

//---------------------------------------------------------------------------
//
gint WzSetupDlgImp::CloseCb( 
    GtkWidget */*widget*/,
    gpointer   /*data*/ )
{
    if( IZkPopup::QuestionBox( IDS_CLOSE_ARE_YOU_SURE, ZK_YESNO ) ) 
    {
        // Get out of here and return false.
        gtk_main_quit();
    }
    else
    {
        return TRUE;
    }

    return FALSE;
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::AddPage( 
    ZKSTD::auto_ptr< WzSetupBasePage > page
)
{
    ZKASSERT( page.get() != NULL );

    WzSetupBasePage* dlg = page.release();

    dlg->SetParent( this, m_pageCt );
    m_pages.push_back( dlg );

    // First page is shown, others get hidden.
    if( m_pages.size() == 1 ) 
    {
        dlg->Show();
    }
    else 
    {
        dlg->Hide();
    }

    GtkRequisition size_requisition;

    // In an attempt to get this thing to look good, calculate max page size
    // so that we dont jump from one size to the other when we click 
    // next/previous.
    gtk_widget_size_request( 
        GTK_WIDGET( dlg->GetContainer() ), 
        &size_requisition );

    m_max_w = MAX( m_max_w, size_requisition.width );
    m_max_h = MAX( m_max_h, size_requisition.height );
         
    // Now set the size hint for our window.
    gtk_widget_set_usize( GTK_WIDGET( m_pageCt ), m_max_w, m_max_h );    
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::RemovePage( const ZKSTD::string& name )
{
    /*
      NOTE: This function is mainly used to remove the license agreement from 
            the setup wizard. It does not update the Start/Finish mode properly
            so it should be used with care. In the case of the license agreement,
            the page itself sets the state to start mode.
    */

    ZKASSERT( !name.empty() );

    // Find the page by name.
    PageList::iterator iter;

    for( iter = m_pages.begin(); iter != m_pages.end(); ++iter ) {
        if( (*iter)->GetName() == name ) {            
            break;
        }
    }

    ZKASSERT( iter != m_pages.end() );
    if( iter != m_pages.end() )
    {
        WzSetupBasePage* dlg = (*iter);
        dlg->Hide();

        // Unparent the page from the wizard.
        GtkWidget* ct = dlg->GetContainer();
        gtk_container_remove( GTK_CONTAINER( m_pageCt ), ct );

        m_pages.erase( iter );

        // Finally, get rid of the window.
        delete dlg;
    }
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::SetActivePage( const ZKSTD::string& name )
{
    ZKASSERT( !name.empty() );

    // Find the page by name.
    PageList::size_type size = m_pages.size();
    PageList::size_type index = 0;

    for( ; index < size; index++ ) {
        if( m_pages[ index ]->GetName() == name ) {
            SetCurrentPage( index );
            break;
        }
    }

    ZKASSERT( index < size );
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::SkipPage( const ZKSTD::string& name, bool skip )
{
    ZKASSERT( !name.empty() );

    // Find the page by name.
    PageList::size_type size = m_pages.size();
    PageList::size_type index = 0;

    for( ; index < size; index++ ) {
        if( m_pages[ index ]->GetName() == name ) {
            m_pages[ index ]->SkipPage( skip );
            break;
        }
    }

    ZKASSERT( index < size );
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::SetCurrentPage( int index )
{
    ZKASSERT( (index >= 0) && (index < static_cast< int >( m_pages.size() )) );

    if( index == m_currentPage ) {
        return;
    }

    m_pages[ m_currentPage ]->Hide();
    m_currentPage = index;
    m_pages[ m_currentPage ]->Show();
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::StartMode()
{
    if( m_mode == START )
    {
        return;
    }
    
    if( m_mode == NORMAL )
    {
        gtk_container_remove( GTK_CONTAINER( m_btnbox_left ), m_previous );
        gtk_container_remove( GTK_CONTAINER( m_btnbox_left ), m_next );
    }
    else if( m_mode == FINISH )
    {
        gtk_container_remove( GTK_CONTAINER( m_btnbox_left ), m_previous );
        gtk_container_remove( GTK_CONTAINER( m_btnbox_left ), m_finish );
    }
    else if( m_mode == LICENSE )
    {
        gtk_container_remove( GTK_CONTAINER( m_btnbox_left ), m_no );
        gtk_container_remove( GTK_CONTAINER( m_btnbox_right ), m_yes );

        gtk_box_pack_start( GTK_BOX( m_btnbox_right ), m_cancel, FALSE, FALSE, 0 );
        gtk_box_pack_start( GTK_BOX( m_btnbox_right ), m_help, FALSE, FALSE, 0 );
    }

    gtk_box_pack_start( GTK_BOX( m_btnbox_left ), m_start, FALSE, FALSE, 0 );
    m_mode = START;
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::LicenseMode()
{
    ZKASSERT( m_mode == START );

    gtk_container_remove( GTK_CONTAINER( m_btnbox_left ), m_start );
    gtk_container_remove( GTK_CONTAINER( m_btnbox_right ), m_cancel );
    gtk_container_remove( GTK_CONTAINER( m_btnbox_right ), m_help  );

    gtk_box_pack_start( GTK_BOX( m_btnbox_left ),  m_no, FALSE, FALSE, 0 );
    gtk_box_pack_start( GTK_BOX( m_btnbox_right ), m_yes, FALSE, FALSE, 0 );
    m_mode = LICENSE;
}


//---------------------------------------------------------------------------
//
void WzSetupDlgImp::NormalMode()
{
    ZKASSERT( m_mode != LICENSE );

    if( m_mode == NORMAL )
    {
        return;
    }
    
    if( m_mode == START )
    {
        gtk_container_remove( GTK_CONTAINER( m_btnbox_left ), m_start );
    }
    else if( m_mode == FINISH )
    {
        gtk_container_remove( GTK_CONTAINER( m_btnbox_left ), m_previous );
        gtk_container_remove( GTK_CONTAINER( m_btnbox_left ), m_finish );
    }

    gtk_box_pack_start( GTK_BOX( m_btnbox_left ), m_previous, FALSE, FALSE, 0 );
    gtk_box_pack_start( GTK_BOX( m_btnbox_left ), m_next, FALSE, FALSE, 0 );
    m_mode = NORMAL;
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::FinishMode()
{
    ZKASSERT( m_mode != LICENSE );

    if( m_mode == FINISH )
    {
        return;
    }
    
    if( m_mode == START )
    {
        gtk_container_remove( GTK_CONTAINER( m_btnbox_left ), m_start );
    }
    else if( m_mode == NORMAL )
    {
        gtk_container_remove( GTK_CONTAINER( m_btnbox_left ), m_next );
        gtk_container_remove( GTK_CONTAINER( m_btnbox_left ), m_previous );
    }

    gtk_box_pack_start( GTK_BOX( m_btnbox_left ), m_previous, FALSE, FALSE, 0 );
    gtk_box_pack_start( GTK_BOX( m_btnbox_left ), m_finish, FALSE, FALSE, 0 );
    m_mode = FINISH;
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::StartCb( GtkWidget *, WzSetupDlgImp* inst )
{
    inst->NextPage();
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::FinishCb( GtkWidget *,  WzSetupDlgImp* inst  )
{
    // Get all information, commit user choices.
    // To Be Defined.

    if( !inst->m_pages[ inst->m_currentPage ]->Finish() ) 
    {
        inst->m_done = true;
        gtk_main_quit();
    }
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::PreviousCb( GtkWidget *,  WzSetupDlgImp* inst  )
{
    inst->PrevPage();
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::NextCb( GtkWidget *,  WzSetupDlgImp* inst  )
{
    inst->NextPage();
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::CancelCb( GtkWidget *,  WzSetupDlgImp* /*inst*/  )
{
    if( IZkPopup::QuestionBox( IDS_CLOSE_ARE_YOU_SURE, ZK_YESNO ) ) 
    {
        // Get out of here and return false.
        gtk_main_quit();
    }
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::HelpCb( GtkWidget *,  WzSetupDlgImp* inst )
{
    ZKSTD::string helpFile;
    if( inst->m_nymWizard ) {
        helpFile = ZKFileConfiguration::GetFreedomNymWizardHelp();
    }
    else {
        helpFile = ZKFileConfiguration::GetFreedomSetupWizardHelp();
    }

    gnome_url_show( helpFile.c_str() );
}


//---------------------------------------------------------------------------
//
void WzSetupDlgImp::NextPage()
{
    bool processed;
    processed = m_pages[ m_currentPage ]->NextPage();
        
    if( !processed )
    {
        int size = static_cast< int >( m_pages.size() );
        int current = m_currentPage+1;

        // go to the next page not marked as skipped.
        for( ; current < size && m_pages[ current ]->SkipIt(); current++ );

        // Make sure at least the last page is not skippable.
        ZKASSERT( current < size );

        if( current == size-1 ) 
        {
            FinishMode();
        }
        else 
        {
            NormalMode();
        }
        
        SetCurrentPage( current );
    }
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::PrevPage()
{
    bool processed;
    processed = m_pages[ m_currentPage ]->PreviousPage();

    if( !processed )
    {
        int current = m_currentPage-1;

        // go to the next page not marked as skipped.
        for( ; current >= 0 && m_pages[ current ]->SkipIt(); current-- );
       
        if( current == 0 ) 
        {
            StartMode();
        }
        else 
        {
            NormalMode();
        }
        
        SetCurrentPage( current );
    }
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::SetSensitive( bool s, int button = WZ_START | 
                    WZ_PREVIOUS | WZ_NEXT | WZ_CANCEL | WZ_FINISH )
{
    gtk_widget_set_sensitive( m_pageCt, s );

    ButtonSetActive( button, s );
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::ButtonSetActive( int btn, bool state )
{
    if( btn & WZ_START )
    {
        gtk_widget_set_sensitive( m_start, state );    
    }

    if( btn & WZ_PREVIOUS )
    {
        gtk_widget_set_sensitive( m_previous, state );    
    }

    if( btn & WZ_NEXT )
    {
        gtk_widget_set_sensitive( m_next, state );    
    }

    if( btn & WZ_CANCEL )
    {
        gtk_widget_set_sensitive( m_cancel, state );    
    }

    if( btn & WZ_FINISH )
    {
        gtk_widget_set_sensitive( m_finish, state );    
    }


}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::SetFreeFlag( bool flag )
{
    m_freeVersion = flag;
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::SetTestDriveFlag( bool flag )
{
    m_startTestDrive = flag;
}

//---------------------------------------------------------------------------
//
bool WzSetupDlgImp::IsFreeVersion() const
{
    return m_freeVersion;
}

//---------------------------------------------------------------------------
//
tstring WzSetupDlgImp::GetToken() const
{
    return m_token;
}

//---------------------------------------------------------------------------
//
void WzSetupDlgImp::SetToken( const tstring& in_token )
{
    m_token = in_token;    
}

// FIXME - remove code below.

//---------------------------------------------------------------------------
//
bool WzSetupDlgImp::IsNymWizard() const
{
    return m_nymWizard;
}

//---------------------------------------------------------------------------
//
const ZKSTD::string& WzSetupDlgImp::GetNymName() const
{
    return m_nymName;
}
 
//---------------------------------------------------------------------------
//
void WzSetupDlgImp::SetNymName( const ZKSTD::string& name )
{
    m_nymName = name;
}
