/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_WZSETUPDLGIMP
#define ZKS_INCL_WZSETUPDLGIMP

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#include <UiThreadSafeDlg.h>

#include <clientstl.h>

#include <gnome.h>
#include <vector>
#include <memory>
#include <string>

class WzSetupBasePage;



/*===========================================================================
 * TYPE WzSetupBtnId
 *=========================================================================*/

typedef enum {
    WZ_START    = 1 << 0,
    WZ_PREVIOUS = 1 << 1,
    WZ_NEXT     = 1 << 2,
    WZ_CANCEL   = 1 << 3,
    WZ_FINISH   = 1 << 4
} WzSetupBtnId;

/*===========================================================================
 * CLASS WzSetupDlgImp
 *=========================================================================*/

class WzSetupDlgImp 
    : public UiThreadSafeDlg {

public: 


    /*----- member functions -----*/
      
    /*
      FIXME - The flag here tells the setup wizard whether to build a Setup
              wizard, or a Nym Creation wizard. It is really the same object,
              it just ends up having different pages.
              There is a fixme because the addition of the pages really 
              shouldn't be the problem of the object containing the pages,
              be some other higher level object.

              Same thing for the IsFreeNym(). It doesn't belong here.
              Corners are cut round these days.

              ericb
     */
    WzSetupDlgImp( bool NymWizard = false );
    ~WzSetupDlgImp();

    bool Run();

    void AddPage( ZKSTD::auto_ptr<WzSetupBasePage> page );
    void RemovePage( const ZKSTD::string& name );

    // Purpose:  Sets the currently displayed page.
    // Throws:   FIXME - throw if not in the wizard.
    void SetActivePage( const ZKSTD::string& name );
    
    // Purpose:  This function can be used ti=o indicate to the wizard that 
    //           some pages should be skipped.
    void SkipPage( const ZKSTD::string& name, bool skip );
    
    // Purpose: This function enables or disables the entire wizard.
    void SetSensitive( bool, int btn = WZ_START | WZ_NEXT |
                                       WZ_PREVIOUS | WZ_CANCEL |
                                       WZ_FINISH );

    void ButtonSetActive( int btn, bool state ); 

    void NextPage();
    void PrevPage();

    void BuildUI();

    // Purpose:  Indicates whether or not we are using the free or "pro"
    //           version. Influences the content of some pages.
    bool IsFreeVersion() const;

    // FIXME - remove as soon as we startusing Phil's new classes.
    // Then use dynamic cast to figure it out, parent will be something like
    // WzSetupWizard or WzNymCreationWizard
    bool IsNymWizard() const;
    const ZKSTD::string& GetNymName() const;
    void SetNymName( const ZKSTD::string& );

	// Purpose:  Well... sets the flag indicating whther or not we are
    //           doing Setup or Creating a nym for th efree version.
    void SetFreeFlag( bool );
    
    // Purpose: These two function set/get the token, to communicate 
    //          between pages.
    tstring GetToken() const;
    void SetToken( const tstring& in_token );

    void StartMode();
    void NormalMode();
    void FinishMode();

    // Purpose: See note below near the yes/no buttons for explanation of this 
    //          "fix".
    void LicenseMode();

	// Purpose:  Well... sets the flag indicating whther or not we should
    //           start the tes drive on exit.
    //           FIXME - remove when we derive from WzBase.
    void SetTestDriveFlag( bool );
    

protected:

    /*----- member functions -----*/

    virtual void UiThreadCallback();

    
private: 

    /*----- types and enumerations ----*/

    typedef ZKSTD::vector< WzSetupBasePage* > PageList;

    typedef enum  {
        START,
        NORMAL,
        FINISH,
        LICENSE
    } Mode;


    /*----- static member functions -----*/

    static gint CloseCb( GtkWidget *, gpointer   );

    static void StartCb( GtkWidget *,  WzSetupDlgImp* inst  );
    static void FinishCb( GtkWidget *,  WzSetupDlgImp* inst  );
    static void PreviousCb( GtkWidget *,  WzSetupDlgImp* inst  );
    static void NextCb( GtkWidget *,  WzSetupDlgImp* inst  );
    static void CancelCb( GtkWidget *,  WzSetupDlgImp* inst  );
    static void HelpCb( GtkWidget *,  WzSetupDlgImp* inst  );


    /*----- member functions -----*/

    // prohibited funcs
    WzSetupDlgImp( const WzSetupDlgImp& );
    const WzSetupDlgImp& operator=( const WzSetupDlgImp& );

    void SetCurrentPage( int dlg );

    /*----- data members -----*/

    GtkWidget* m_window;
    GtkWidget* m_pageCt;     // Main container for pages.
    GtkWidget* m_vbox;
    GtkWidget* m_hbox;
    GtkWidget* m_btnbox_left;
    GtkWidget* m_btnbox_right;
    GtkWidget* m_start;
    GtkWidget* m_finish;
    GtkWidget* m_previous;
    GtkWidget* m_next;
    GtkWidget* m_help;
    GtkWidget* m_cancel;

    // This is really an ugly add on to an already ugly class.
    // Those buttons are needed for the setup wizard when we display the 
    // license agreement.
    // Post 2.0, this class will be removed and every wizard will start using
    // WzBase, at which point this hack will be fixed properly.
    // ericb - 2000/11/08
    GtkWidget* m_no;
    GtkWidget* m_yes;

    PageList   m_pages;
    int        m_currentPage;

    Mode m_mode;

    int        m_max_w;
    int        m_max_h;

    bool       m_done;
    bool       m_freeVersion;
    bool       m_nymWizard;
    bool       m_built;
    tstring    m_token;

    // This field will have to go in each wizard class where it belongs
    ZKSTD::string m_nymName;

    // November 22, 2000 - ericb
    // Yet another ugly patch to determine whether or not to start the test drive.
    // This happens only for the SetupWizard, and will have to follow an
    // eventual SetupWizard class.
    // This class is slated for removal, to be replaced by two new classes
    // derived from WzBase. 
    bool       m_startTestDrive;
};
#endif
