// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <WzSetupFinish.h>
#include <WzSetupDlgImp.h>

#include <ZkFreedomWizardsStrings.h>
#include <ZkGladeLoader.h>
#include <ZkGtkUtils.h>
#include <ZkStringMgr.h>

#include <WNymIconInfo.h>
#include <INym.h>
#include <INymCentral.h>
#include <ZkService.h>


#include <RZkException.h>
#include <ZkErrorHandler.h>
#include <IZKErrorReporter.h>

//===========================================================================
// CLASS WzSetupFinish
//===========================================================================

//---------------------------------------------------------------------------
//
WzSetupFinish::WzSetupFinish( )
    : WzSetupBasePage( "WizardFinish" )
    , m_bInitialized( false )
{
    ZkGladeLoader loader( "wizard_finish.xml", "topct" );

    // Get the label and make it bigger.
    GtkWidget* title = loader.GetWidget( "title" );
    ZkGtkUtils::UseTitleFont( title );

    m_testDriveCt = loader.GetWidget( "test_drive_ct" );
    m_testDrive   = loader.GetWidget( "test_drive" );
    m_nymName     = loader.GetWidget( "nym_name" );
    m_nymIcon     = loader.GetWidget( "nym_icon" );
    m_expDate     = loader.GetWidget( "exp_date" );
    m_expDateLabel	= loader.GetWidget( "expiryDateLabel" );   
    m_launch      = loader.GetWidget( "launch" );

    // Parent this UI in the base class container.
    AddPageUi( loader.GetWidget( "topct" ) );

    gtk_signal_connect (GTK_OBJECT( m_launch ), "toggled",
                        GTK_SIGNAL_FUNC( BackupCb ), this);

}


//---------------------------------------------------------------------------
//
WzSetupFinish::~WzSetupFinish( )
{
}

//---------------------------------------------------------------------------
//
bool WzSetupFinish::NextPage()
{
    OnNext();
    return false;
}

//---------------------------------------------------------------------------
//
bool WzSetupFinish::Finish()
{
    OnNext();
    return false;
}

//---------------------------------------------------------------------------
//
void WzSetupFinish::OnNext()
{
    WzSetupDlgImp* pSetupWz = GetMainDialog();
    ZKASSERT( pSetupWz );
    if( pSetupWz )
    {
        bool td = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( m_testDrive ) );
        pSetupWz->SetTestDriveFlag( td );
    }    
}

//---------------------------------------------------------------------------
//
void WzSetupFinish::MapPage()
{
    // reinitialie the buttons
    BackupCb(NULL, this);

    // Once you get to that page, there is no going back.
    GetMainDialog()->ButtonSetActive( WZ_PREVIOUS, false );
    
    if( GetMainDialog()->IsFreeVersion() || GetMainDialog()->IsNymWizard() )
    {
        gtk_widget_hide( m_testDrive );
    }
    else 
    {
        gtk_widget_show( m_testDrive );
    }


    // Although I dont like this sort of flag, Previous/Next should not 
    // reinitialize the page.
    if( m_bInitialized )
    {
        return;
    }

    m_bInitialized = true;

    ZKSTD::string nymName = GetMainDialog()->GetNymName( );

    INym* pINym = NULL;
    ZkService::GetINymCentral()->GetNym(IID_INYM, (void**)&pINym, nymName.c_str() );

    if( GetMainDialog()->IsNymWizard( ) )
    {
        gtk_widget_hide( m_testDriveCt );
    }
    
    // Nym name.

    char* fmt;
    gtk_label_get( GTK_LABEL( m_nymName ), &fmt );
    char* newText = g_strdup_printf( fmt, nymName.c_str() );
    
    ZKSTD::string txt;

    // FIXME - use this string when we have resolved the problem with %1 %2 in strings.
//    ZkStringMgr::Load( txt, IDS_NYMCREATECONCLUDE_TXT, nymName ) ;
//    gtk_label_set( GTK_LABEL( m_nymName ), txt.c_str() );

    gtk_label_set( GTK_LABEL( m_nymName ), newText );


    // Nym Icon
    // FIXME - use gdk_pixbuf (?)

    WNymIconInfo info ;
    pINym->GetNymIconInfo( info ) ;

    if( info.GetIconType() == NymIconFilePath ) 
    {
        txt = info.GetFilePath();
        if( !txt.empty() ) 
        {
            gtk_widget_show( m_nymIcon );
            gnome_pixmap_load_file_at_size( GNOME_PIXMAP( m_nymIcon ),
                                            txt.c_str(),
                                            48, 48 );
        }
        else
        {
            gtk_widget_hide( m_nymIcon );
        }
    }


    // Expity Date.
    // don't show the expiry date if they are a fre nym.

    if( pINym->DoesNymUseMail() &&
        pINym->DoesNymUseCloud()) 
    {
        try {
            pINym->GetNymRetireDate( txt );
        }
        catch( IZkException& ex) {
            ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
        }

        gtk_label_set_text( GTK_LABEL( m_expDate ), txt.c_str() ); 
        gtk_widget_show( GTK_WIDGET( m_expDate));
        gtk_widget_show( GTK_WIDGET( m_expDateLabel));
    }
    else
    {
        gtk_widget_hide( GTK_WIDGET( m_expDate));
        gtk_widget_hide( GTK_WIDGET( m_expDateLabel));
    }
}
//---------------------------------------------------------------------------
//
void WzSetupFinish::BackupCb( GtkWidget*, WzSetupFinish* inst )
{
    if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( inst->m_launch ) ))
    {
        // Change the buttons in the dialog, it is no longer the Finish page.
        inst->GetMainDialog()->NormalMode();
    }
    else
    {
        inst->GetMainDialog()->FinishMode();
    }

}
