// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):


//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <WzSetupDlgImp.h>
#include <WzSetupWelcome.h>
#include <WzSetupPassphrase.h>
#include <WzSetupNym.h>
#include <WzSetupSerial.h>
#include <WzSetupFinish.h>
#include <WzSetupSelectToken.h>
#include <WzBackup.h>
#include <WzLicense.h>

#include <UiThreadId.h>

#include <clientstl.h> // for the ... namespace
#include <memory> // auto_ptr


//===========================================================================
// Namespace WzSetupInit
//===========================================================================

namespace WzSetupInit 
{

//---------------------------------------------------------------------------
//
void AddSetupPages( WzSetupDlgImp* wizard )
{
    ZKSTD::auto_ptr< WzSetupBasePage > licensePage( new WzLicense() );
    ZKSTD::auto_ptr< WzSetupBasePage > welcomePage( new WzSetupWelcome() );
    ZKSTD::auto_ptr< WzSetupBasePage > passphrasePage( new WzSetupPassphrase() );
    ZKSTD::auto_ptr< WzSetupBasePage > serialPage( new WzSetupSerial() );
    ZKSTD::auto_ptr< WzSetupBasePage > nymPage( new WzSetupNym() );
    ZKSTD::auto_ptr< WzSetupBasePage > finishPage( new WzSetupFinish() );
    ZKSTD::auto_ptr< WzSetupBasePage > backupPage( new WzBackup() );
    
    wizard->AddPage( licensePage );
    wizard->AddPage( welcomePage );
    wizard->AddPage( passphrasePage );
    wizard->AddPage( serialPage );
    wizard->AddPage( nymPage );
    wizard->AddPage( finishPage );    
    wizard->AddPage( backupPage );        

    wizard->LicenseMode();
}

//---------------------------------------------------------------------------
//
void AddNymCreatePages( WzSetupDlgImp* wizard )
{
    ZKSTD::auto_ptr< WzSetupBasePage > selectTokenPage( new WzSetupSelectToken() );
    ZKSTD::auto_ptr< WzSetupBasePage > nymPage( new WzSetupNym() );
    ZKSTD::auto_ptr< WzSetupBasePage > finishPage( new WzSetupFinish() );
    ZKSTD::auto_ptr< WzSetupBasePage > backupPage( new WzBackup() );
            
    wizard->AddPage( selectTokenPage );
    wizard->AddPage( nymPage );
    wizard->AddPage( finishPage );    
    wizard->AddPage( backupPage );    
    
}

}
