// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <WzSetupNym.h>
#include <ZkFreedomWizardsStrings.h>

#include <WzSetupSelectToken.h>

#include <ZkStringMgr.h>
#include <ZkGladeLoader.h>
#include <ZkGtkUtils.h>

#include <UiThreadId.h>

#include <IZkPopup.h>
#include <IZkConsoleMsg.h>
#include <INiqs.h>
#include <INym.h>
#include <INymMail.h>
#include <IClientPersist.h>
#include <IClientCentral.h>
#include <INymCentral.h>
#include <ITokenService.h>
#include <ZkService.h>
#include <ZkErrorHandler.h>
#include <IZKErrorReporter.h>
#include <RZkException.h>
#include <UiThreadSafeMessage.h>
#include <ZKBusyAVI.h>
#include <ZkStatusSubject.h>

#include <WTokenDescriptions.h>
#include <WNymIconInfo.h>

#include <ctype.h>

// temporary patch/
#include <gdk/gdkx.h>
#include <X11/Xlib.h> // for iconify

#include <unistd.h> // for a test



//===========================================================================
// CLASS WzSetupNym
//===========================================================================

//---------------------------------------------------------------------------
//
WzSetupNym::WzSetupNym( )
    : WzSetupBasePage( "SetupNym" )
   , m_ct( 0 )
   , m_name( 0 )
   , m_icon( 0 )
   , m_security( 0 )
   , m_list( 0 )
   , m_status( false )
   , m_createdNym( false )
   , m_threadAction( -1 )
   , m_NymName( NULL )
{
    ZkGladeLoader loader( "wizard_nym.xml", "topct" );

    // Get the label and make it bigger.
    GtkWidget* title = loader.GetWidget( "title" );
    ZkGtkUtils::UseTitleFont( title );

    m_ct          = loader.GetWidget( "topct");
    m_name        = loader.GetWidget( "name" );
    m_icon        = loader.GetWidget( "icon" ); 
    m_icon_ct     = loader.GetWidget( "icon_ct" ); 
    m_security    = loader.GetWidget( "security" );
    m_securityBox = loader.GetWidget( "security_box" );
    m_securityTxt = loader.GetWidget( "security_text" );

    m_adjust = gtk_range_get_adjustment( GTK_RANGE( m_security ) );

    gtk_widget_ref( m_securityBox );
    gtk_object_ref( GTK_OBJECT( m_adjust ) );  // Fix for 7181.

    // call the SecurityCb as a way to initialize the tex
    // under the slider/range control dynamically
    SecurityCb( NULL, this );

    // Parent this UI in the base class container.
    AddPageUi( m_ct );

    // Hook the 'return' key to the NextPage function in the confirm box.
    gtk_signal_connect (GTK_OBJECT( m_name ), "activate",
                        GTK_SIGNAL_FUNC( WzSetupBasePage::NextPageCb ), this );

    // Hook an input validator to the widget name
    gtk_signal_connect (GTK_OBJECT( m_name ), "insert_text",
                        GTK_SIGNAL_FUNC( WzSetupNym::ValidateInputCb ), this );

    gtk_signal_connect (GTK_OBJECT( m_adjust ), "value_changed",
                        GTK_SIGNAL_FUNC( SecurityCb ), this);

    // Hook the icon button to a callback to safely raise the window
    gtk_signal_connect (GTK_OBJECT(((GnomeIconEntry*)m_icon)->pickbutton ), 
                        "clicked",
                        GTK_SIGNAL_FUNC( PickIconCb ), this);
}

//---------------------------------------------------------------------------
//
WzSetupNym::~WzSetupNym( )
{
    if ( m_NymName ) {
        delete m_NymName;
    }

    gtk_widget_unref( m_securityBox );
    gtk_object_unref( GTK_OBJECT( m_adjust ) );
}

//---------------------------------------------------------------------------
//
void WzSetupNym::DeleteTokenCallback( gpointer klass )
{

    WzSetupNym * inst = (WzSetupNym*)klass;

    ZkService::GetITokenService()->RemoveTokenFromArray( inst->m_token );
             
    // FIXME ? is the select token page always going to be PrevPage()?
    // take the user back to the select token page
    inst->GetMainDialog()->PrevPage();
}


//---------------------------------------------------------------------------
//
void WzSetupNym::ChangeToAltNymNamePage( )
{
    gtk_widget_ref( m_icon );
    gtk_container_remove( GTK_CONTAINER( m_icon_ct ), m_icon );

    // First get rid of the current content of the window.
    gtk_widget_hide( m_ct );
    gtk_container_remove( GTK_CONTAINER( GetSubContainer() ), m_ct );

    ZkGladeLoader loader( "wizard_alter.xml", "topct" );

    // Get the label and make it bigger.
    GtkWidget* title = loader.GetWidget( "title" );
    ZkGtkUtils::UseTitleFont( title );

    m_ct      = loader.GetWidget( "topct");
    m_name    = loader.GetWidget( "name" );
    m_list    = loader.GetWidget( "list" );
    m_icon_ct = loader.GetWidget( "icon_ct" ); 

    gtk_container_add( GTK_CONTAINER( m_icon_ct ), m_icon );
    gtk_widget_unref( m_icon );

    // Parent this UI in the base class container.
    AddPageUi( m_ct );

    gtk_signal_connect( GTK_OBJECT( m_list ),
                        "select_child",
                        GTK_SIGNAL_FUNC( NameSelected ),
                        this );

    // Hook an input validator to the widget name
    gtk_signal_connect (GTK_OBJECT( m_name ), "insert_text",
                        GTK_SIGNAL_FUNC( WzSetupNym::ValidateInputCb ), this );

    // Hook the 'return' key to the NextPage function in the confirm box.
    gtk_signal_connect (GTK_OBJECT( m_name ), "activate",
                        GTK_SIGNAL_FUNC( WzSetupBasePage::NextPageCb ), this );

    ZKSTD::vector <ZKSTD::string>& names = m_NymName->GetVectorOfValidNames();
    ZKSTD::vector <ZKSTD::string>::iterator iter;

    GtkWidget* item;

    for( iter = names.begin(); iter != names.end(); ++iter )
    {
        ZKTRACE1( "Adding <%s> to the list.\n", (*iter).c_str() );
        
        item = gtk_list_item_new_with_label( (*iter).c_str() ); 

        gtk_container_add( GTK_CONTAINER( m_list ), item );
        gtk_widget_show( item );
    }
}

//---------------------------------------------------------------------------
//
void  WzSetupNym::NameSelected( GtkList */*list*/,
                                GtkWidget *widget,
                                gpointer user_data)
{
    ZKASSERT( user_data != 0 );
    WzSetupNym* inst = (WzSetupNym*)user_data;
    
    GtkWidget* label = GTK_BIN( widget )->child;
    char* name; 
    gtk_label_get( GTK_LABEL( label), &name );

    gtk_entry_set_text( GTK_ENTRY( inst->m_name ), name );
}

//---------------------------------------------------------------------------
//
bool WzSetupNym::NextPage()
{
    ZKASSERT( UiThreadId::IsUiThread() );

    // If the nym has already been successfully created, just skip.
    if( m_createdNym ) 
    {
        return false;
    }

    m_status = false;

    ZKSTD::string strNymName( gtk_entry_get_text( GTK_ENTRY( m_name ) ));

    if( strNymName.empty() )
    {
		IZkPopup::MessageBox( IDS_EMPTY_NYM_NAME );
        return true;
    }

//   CWaitCursor	TheseArentTheDroidsDude; 

    IZkConsoleMsg::ShowMsgForceUpdate( IDS_CONS_CREATE_NYM );

    // Will stay in this function as long as the thread is running.
    // EndCreateThread will cause this function to return.
    m_threadAction = THREAD_CREATE_NYM;
    StartThread();

    return m_status;
}

//---------------------------------------------------------------------------
//
void WzSetupNym::MapPage()
{
    // if the nym has already been created, just return;
    if( m_createdNym )
    {
        return;
    }


    // The Nym Creation wizard provides a token to WzSetupDlgImp before 
    // mapping this page, and it is then up to us to determine whether or
    // not the token is for a paid nym.
    m_token = GetMainDialog()->GetToken();
    if( !m_token.empty() )
    {
        if( ZkService::GetITokenService()->DoesTokenUseCloud( m_token ) )
            gtk_widget_show( m_securityBox );
        else
            gtk_widget_hide( m_securityBox );
    }
    else
    {
        // On the other hand, if no token has been provided to us, we are
        // in the Setup Wizard in which case the user has chosen which type
        // of nym to create right from the first page.
        if( GetMainDialog()->IsFreeVersion() )
        {
            // First check whether or not we have free tokens, and if not
            // get a few.
            ZKSTD::string free_token = FindToken( false );
            
            if( free_token.empty() )
            {
                // In the case of the free version we have to get some tokens in the background since the
                // user did not go through the Echange Serial Number page.
                m_threadAction = THREAD_EXCHANGE_SERIAL_NB;
                StartThread();
            }

            gtk_widget_hide( m_securityBox );
        }
        else
        {
            gtk_widget_show( m_securityBox );
        }

        // Since the user may have changed his mind and gone back to change
        // free to paid and vice-versa, we may have two types of tokens in the
        // list. So first find the token type corresponding to the selected type,
        // then use a random token among them.
        m_token = FindToken( !GetMainDialog()->IsFreeVersion() );
        ZKASSERT( !m_token.empty() && GetMainDialog() );
        GetMainDialog()->SetToken( m_token );
	}
}

//---------------------------------------------------------------------------
//
// GTK+ signal handler to valdate input in nym name text box
// nym names can not include uppercase characters #5752
void WzSetupNym::ValidateInputCb ( 
    GtkEditable* /*editable*/,
    gchar* new_text,
    gint new_text_length,
    gint* /*position*/,
    gpointer /*user_data*/ )
{
    gint i;
    for ( i=0; i < new_text_length; i++ )
    {
        if ( isalpha( new_text[i] ) ||
             isdigit( new_text[i] ) ||
             new_text[i] == '.' ||
             new_text[i] == '-' ||
             new_text[i] == '_' )
        {
            if ( isupper( new_text[i] ) )
            {
                new_text[i] = tolower( new_text[i] );
            }
        }
        else{
            new_text[i] = '\0';
        }
    }    
}

//---------------------------------------------------------------------------
//
void WzSetupNym::SecurityCb( GtkWidget*, WzSetupNym* inst )
{
    int iPos = static_cast< int >( inst->m_adjust->value );

    ZKSTD::string text;

	switch ( iPos )
	{
		case( 3 ) :
            ZkStringMgr::Load( text, IDS_SECURITY_3_HOP );
			break ;
		case( 2 ) :
            ZkStringMgr::Load( text, IDS_SECURITY_2_HOP );
			break ;
		case( 1 ) :
            ZkStringMgr::Load( text, IDS_SECURITY_1_HOP );
			break ;	
		default:
			NODEFAULT ;
	}

    gtk_label_set_text( GTK_LABEL( inst->m_securityTxt ), text.c_str() );
}

//---------------------------------------------------------------------------
//
void WzSetupNym::AltPageCallback( gpointer klass )
{
    WzSetupNym *inst = static_cast<WzSetupNym *>(klass);

    inst->ChangeToAltNymNamePage( );
}

//---------------------------------------------------------------------------
//
void WzSetupNym::PickIconCb( GtkWidget*, WzSetupNym* inst )
{
    ZkGtkUtils::GetAndSetTopPref(((GnomeIconEntry*)inst->m_icon)->pick_dialog );
}

//---------------------------------------------------------------------------
//
void WzSetupNym::OnThreadStart()
{
    GetMainDialog()->SetSensitive( false, m_ButtonsToAble );
    ZkStatusSubject::UpdateStatus( ZK_STATUS_NETWORK_BUSY, true, true );

    if( m_threadAction == THREAD_CREATE_NYM )
        m_messageBox.Open( IDS_CONS_CREATE_NYM );
// need a new message here.... "Getting free nym from ZK" or something
/*
    else if( m_threadAction == THREAD_EXCHANGE_SERIAL_NB )
        m_messageBox.Open( IDS_CONS_CREATE_NYM );
*/
}

//---------------------------------------------------------------------------
//
void WzSetupNym::OnThreadEnd( )
{
    if ( m_createdNym && WasSuccessful() ) 
    {
        // Disable field in case user presses back/next.
        gtk_widget_set_sensitive( GTK_WIDGET( m_name ), FALSE );

        if( m_list != 0 ) 
        {
            gtk_widget_set_sensitive( GTK_WIDGET( m_list ), FALSE );
        }
    }

    // Stop the animation.
    ZkStatusSubject::UpdateStatus(ZK_STATUS_NETWORK_BUSY, false, true);
    GetMainDialog()->SetSensitive( true, m_ButtonsToAble );    
    m_messageBox.Close();
}

//---------------------------------------------------------------------------
//
void WzSetupNym::OnThreadExecute( )
{

    switch( m_threadAction )
    {
        case THREAD_CREATE_NYM:
            CreateNym();
            break;

        case THREAD_EXCHANGE_SERIAL_NB:
            ExchangeFreeSerialNb();
            break;

        default:
            ZKASSERT( "Valid thread action in WzSetupNym" && 0 );
            break;
    }

}


//---------------------------------------------------------------------------
//
void WzSetupNym::CreateNym( )
{
    ZKSTD::string strNymName( gtk_entry_get_text( GTK_ENTRY( m_name ) ));
    m_NymName = new ZKNymName( strNymName );

    try {

        // We should have tokens at this point, they were fetched in MapPage
        // if we did not have any when we got to this page.
        ZKASSERT(ZkService::GetITokenService()->GetTotalNumberOfTokens());

        ZkService::GetINymCentral()->CreateNym( *m_NymName, m_token ) ;

        // Figure out whether or not a nym was really created
        INym* pINym = NULL;
        bool retval = ZkService::GetINymCentral()->GetNym(IID_INYM,
                                                          (void**)&pINym,
                                                          strNymName.c_str());

        ZKASSERT( retval == true );

        m_createdNym = true;

        ZKSTD::string iconPath;
        gchar* iconFile = gnome_icon_entry_get_filename(
            GNOME_ICON_ENTRY( m_icon ) );
        
        if( iconFile != 0 )
        {
            iconPath = iconFile;
        }

        WNymIconInfo iconInfo;
        iconInfo.SetIconFilePath( iconPath );

        pINym->SetNymIconInfo( iconInfo );

        ZKVERIFY(ZkService::GetITokenService()->RemoveTokenFromArray( m_token ) );

        {
            // Default route rules.
            WNetInfoDatumVec  vecAips ;
            ZkService::GetINiqs()->GetServersOfType( FRENT_TYPE_AIP, vecAips,
                                                     false ) ;
            if(vecAips.size() == 0)
            {
                // FIXME - put this in a catalog, and be nicer to the user.
                IZkPopup::MessageBox( "Can't create the nym route. No AIPS available." );
                abort();
            }

            MapRules mapRules;
            mapRules.clear();
            mapRules[vecAips[0].GetId()] = RULE_FIRSTHOP;

            pINym->SetRouteRulesOfNym(mapRules);
        }

        {
            SecuritySettings SecuritySettings = {0,1};
            SecuritySettings.nReserved = sizeof( SecuritySettings );
            if( !pINym->DoesNymUseCloud() )
            {
                SecuritySettings.nHops = MIN_HOPS;
            }
            else
            {
                size_t val = (size_t)(m_adjust->value);
                if( val < MIN_HOPS ) val = MIN_HOPS;
                if( val > MAX_HOPS ) val = MAX_HOPS;
            
                SecuritySettings.nHops = val;
            }

            ZKTRACE0( "Calling: SetSecuritySettings\n" );
            pINym->SetSecuritySettings( SecuritySettings );
        }

        ZkService::GetIClientPersist()->SaveClientState();
    }
    catch (IZkException& ex) 
    {
        m_status = true;

        // FIXME - well.. this may not be the best solution, but if the string
        //         id is the "Nym exists" error, just go to the next page.
        if( ex.GetErrorStringId() == IDS_NYM_NAME_ALREADY_EXISTS )
        {
            UiThreadSafeMessage::NotifyUi( (gpointer)&WzSetupNym::AltPageCallback,
                                           (gpointer)this, false );
            return;
        }
        else if( ex.GetErrorStringId() == IDS_TOKEN_ALREADY_SPENT )
        {
            IZkPopup::MessageBox( IDS_TOKEN_ALREADY_SPENT_STATEMENT );
            UiThreadSafeMessage::NotifyUi( (gpointer)&WzSetupNym::DeleteTokenCallback,
                                           (gpointer)this, false );
            return;
        }
        else if( ex.GetErrorStringId() == IDS_ALTNAMEFAILED )
        {
             // this leaves the user up the creek, but if the list is empty
             // there is nothing more we can do.
             IZkPopup::MessageBox( IDS_ALTNAMEFAILED );
             return;
        }
        else
        {
            ZkErrorHandler::GetIZkErrorReporter()->ShowError( ex );
        }
    }
//	m_StatusNetwork.SetState( ZK_CONDITION_OFF ) ;

//	if (!m_bShowingAlternate ) {
    // Return to default text

//		GetDlgItem(IDC_NYMNAME_TXT_TITLE)->SetWindowText(m_strOldTitle);
//		GetDlgItem(IDC_NYMNAME_TXT)->SetWindowText(m_strOldText);
//	}

    if ( m_createdNym ) {
        // FIXME - TEMPORARY: Set the name in the wizard main dialog so it 
        //                    can be used by other pages.
        GetMainDialog()->SetNymName( strNymName );
    }
}

//---------------------------------------------------------------------------
//
void WzSetupNym::ExchangeFreeSerialNb()
{
    try 
    {
        ZkService::GetITokenService()->GetFreeNymCreateSerialNumberAndRedeemIt();
    }
    catch( IZkException& ex ) 
    {
        // Failed token server access or invalid serial number
        ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
        
        // FIXME - find a better state to fallback to.
        abort();
    }
}

//---------------------------------------------------------------------------
//
ZKSTD::string WzSetupNym::FindToken( bool bUsesCloud )
{
    unsigned int ntokens;
    ZKSTD::string token;

    ntokens = ZkService::GetITokenService()->GetTotalNumberOfTokens();
    if( ntokens == 0 )
    {
        // no tokens available, return empty token string.
        return token;
    }

    ZKSTD::vector< TokensOfType > tokenTypes;
    ZkService::GetITokenService()->GetTokensGroupedByTypeID( tokenTypes );

    for ( size_t j = 0 ; j < tokenTypes.size() ; j++ )
    {
        TokensOfType& refItemData = tokenTypes[ j ];
        
        ntokens = refItemData.Tokens.size();
        if( ntokens != 0 )
        {
            size_t tokIndex;

            // Right type, so choose a token randomly from the list.
            srand( (unsigned)time( NULL ) );
            size_t randomPos = (rand() % ntokens );
            
            ZKASSERT( randomPos < ntokens ) ;
            
            tokIndex = refItemData.Tokens[ randomPos ] ;

            // Look for the required capabilities.
            if( bUsesCloud == ZkService::GetITokenService()->DoesTokenUseCloud( tokIndex ) )
            {                    
                ZKVERIFY(ZkService::GetITokenService()->GetTokenXinBase64(
                    tokIndex ,
                    token));

                break;
            }
        }
    }

    return token;
}
