/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_WZSETUPNYM
#define ZKS_INCL_WZSETUPNYM

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#ifndef ZKS_INCL_WZSETUPBASE
#include <WzSetupBasePage.h>
#endif

#ifndef ZKS_INCL_UITHREADBASE
#include <UiThreadBase.h>
#endif

#ifndef ZKS_INCL_UISIMPLEMESSAGE
#include <UiSimpleMessage.h>
#endif

#include <gnome.h>
#include <string>
#include <WzSetupDlgImp.h>

class ZKNymName;

/*===========================================================================
 * CLASS WzSetupNym
 *=========================================================================*/

class WzSetupNym : 
    public WzSetupBasePage, 
    public UiThreadBase
{

    // Provides access to control function and allows us to prevent thread calls 
    // from unauthorized partys.
    friend class WzSetupNymThread;

public: 

    /*----- static member functions -----*/

    static void WzSetupNym::FinishedCreate( gpointer klass );
    static void WzSetupNym::AltPageCallback( gpointer klass );
    static void WzSetupNym::DeleteTokenCallback( gpointer klass );

    /*----- member functions -----*/

    WzSetupNym();
    virtual ~WzSetupNym();

    bool NextPage();

protected:

    /*----- member functions -----*/

    virtual void MapPage();

    /*----- UiThreadBase overrides -----*/

    virtual void OnThreadStart();
    virtual void OnThreadEnd();
    virtual void OnThreadExecute();
    

      
private: 

    /*----- typedefs and enumerations -----*/

    enum {
        THREAD_CREATE_NYM,
        THREAD_EXCHANGE_SERIAL_NB
    };

    /*----- static member functions -----*/

    static void NameSelected( GtkList *list,
                              GtkWidget *widget,
                              gpointer user_data );


    // GTK+ signal handler to valdate input in nym name text box
    // nym names can not include uppercase characters #5752
    static void ValidateInputCb ( GtkEditable * editable,
                                  gchar *new_text,
                                  gint new_text_length,
                                  gint *position,
                                  gpointer user_data );

    static void SecurityCb( GtkWidget*, WzSetupNym* inst );
    static void PickIconCb( GtkWidget*, WzSetupNym* inst );

    /*----- member functions -----*/

	// Prohibited funcs - not implemented.
	WzSetupNym( const WzSetupNym& );
	const WzSetupNym& operator=( const WzSetupNym& );

    void GetToken( ZKSTD::string& ) ;
    void ChangeToAltNymNamePage( );

    // Purpose: Starts the nym create thread and sets states needed
    //          during nym creation. (animation, disabled UI)
    //          This function enters a local message loop and returns only 
    //          when the nym creation thread has finished,
    void StartCreateThread();

    // Purpose: Called at the end of the create thread to reset the states and
    //          finalize nym creation.
    void EndCreateThread( gpointer klass );

    // Purpose: Does the actual job of creating a Nym from the thread.
    void DoThreadWork( );

    // Purpose: Called by DoThreadWork to create the nym.
    void CreateNym( );

    // Purpose: Called by DoThreadWork to get free nym tokens automatically.
    void ExchangeFreeSerialNb();

    // Purpose: Finds a token of the specified type (paid or not) in the token
    //          list.
    ZKSTD::string FindToken( bool bUsesCloud );


    /*----- data members -----*/

    GtkWidget* m_ct;
    GtkWidget* m_name;
    GtkWidget* m_icon;
    GtkWidget* m_icon_ct;
    GtkWidget* m_security;
    GtkWidget* m_list;
    GtkWidget* m_securityBox;
    GtkWidget* m_securityTxt;

    GtkAdjustment* m_adjust;

    bool m_status;
    bool m_createdNym;

    int  m_threadAction;
    static const int m_ButtonsToAble = WZ_NEXT | WZ_PREVIOUS |
       WZ_FINISH | WZ_CANCEL;

    UiSimpleMessage m_messageBox;

    ZKSTD::string m_token;
    ZKNymName * m_NymName;
};
#endif
