// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <WzSetupPassphrase.h>
#include <ZkFreedomWizardsStrings.h>

#include <ZkGladeLoader.h>
#include <ZkGtkUtils.h>
#include <IZkPopup.h>

#include <ICrypto.h>
#include <ZkService.h>
#include <ZkErrorHandler.h>
#include <IZKErrorReporter.h>
#include <IZkException.h>
#include <INym.h>
#include <INymCentral.h>

//===========================================================================
// CLASS WzSetupPassphrase
//===========================================================================

//---------------------------------------------------------------------------
//
WzSetupPassphrase::WzSetupPassphrase( )
    : WzSetupBasePage( "SetupPassphrase"  )
    , m_bProtect( true )
{
    ZkGladeLoader loader( "wizard_passphrase.xml", "ct" );

    m_frame      = loader.GetWidget( "frame" );
    m_toggle     = loader.GetWidget( "remember" );
    m_new        = GTK_ENTRY( loader.GetWidget( "new" ));
    m_confirm    = GTK_ENTRY( loader.GetWidget( "confirm" ));
    m_new_ct     = loader.GetWidget( "new_ct" );
    m_confirm_ct = loader.GetWidget( "confirm_ct" );

    gtk_signal_connect (GTK_OBJECT( m_toggle ), "toggled",
                        GTK_SIGNAL_FUNC( ToggleCb ), this);

    // Get the label and make it bigger.
    GtkWidget* title = loader.GetWidget( "title" );
    ZkGtkUtils::UseTitleFont( title );

    // FIXME - work around for a small libglade omission.
    GtkWidget* text = loader.GetWidget( "text" );
    gtk_label_set_line_wrap( GTK_LABEL( text ), TRUE );

    // Make sure users can't turn the *** off and see the actual text...
    gtk_entry_set_visibility( m_new,     FALSE );
    gtk_entry_set_visibility( m_confirm, FALSE );

    // Hook the 'return' key to the NextPage function in the confirm box.
    // FIXME - try to see if we can hook the return key in 'new' to get to the
    //         next text field.
    gtk_signal_connect (GTK_OBJECT( m_confirm ), "activate",
                        GTK_SIGNAL_FUNC( WzSetupBasePage::NextPageCb ), this );


    SetDefaultWidget( GTK_WIDGET( m_new ) );

    // Finally, last but not least parent this UI in the base class container.
    AddPageUi( loader.GetWidget( "ct" ) );
}

//---------------------------------------------------------------------------
//
WzSetupPassphrase::~WzSetupPassphrase( )
{
}

//---------------------------------------------------------------------------
//
bool WzSetupPassphrase::NextPage()
{
    ZKSTD::string passwd;

    if( m_bProtect )
    {
        gchar* new_pwd     = gtk_entry_get_text( m_new );
        gchar* confirm_pwd = gtk_entry_get_text( m_confirm );
        
        if( strlen( new_pwd ) < MIN_PASSPHRASE )
        {
            IZkPopup::MessageBox( IDS_PASSPHRASE_ERR1, ZK_WARNING );
            return true;
        }
        
        // verify the password was entered the same way in the verify field.
        if( strcmp( new_pwd, confirm_pwd ) )
        {
            IZkPopup::MessageBox( IDS_NEW_PASSPHRASE_CONFIRM_ERROR_WZ, ZK_ERROR );
            return true;
        }

        passwd = new_pwd;
    }
    else 
    {
		ZkService::GetICrypto()->GetDefaultPassphrase( passwd );        
    }

	ZKSTD::string old;
	try
    {
		// Just to support coming backwards through the wizard....
		ZkService::GetICrypto()->GetPassphrase( old );
		ZkService::GetICrypto()->ChangePassphrase( old, passwd ) ;
	} 
    catch(IZkException& ex)
    {
		ZkErrorHandler::GetIZkErrorReporter()->ShowError( ex );
    }

    return false;
}

//---------------------------------------------------------------------------
//
bool WzSetupPassphrase::PreviousPage()
{
    return false;
}

//---------------------------------------------------------------------------
//
bool WzSetupPassphrase::Finish()
{
    return false;
}


//---------------------------------------------------------------------------
//
void WzSetupPassphrase::ToggleCb( GtkWidget*, WzSetupPassphrase* inst )
{
    inst->m_bProtect = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( inst->m_toggle ) );

    if( inst->m_bProtect )
    {
        gtk_widget_show( inst->m_new_ct );
        gtk_widget_show( inst->m_confirm_ct );
    }
    else
    {
        gtk_widget_hide( inst->m_new_ct );
        gtk_widget_hide( inst->m_confirm_ct );
    }
}
