// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================


#include <WzSetupSelectToken.h>
#include <WzSetupDlgImp.h>
#include <ZkGladeLoader.h>
#include <ZkGtkUtils.h>
#include <UiTokenCtrl.h>
#include <ZkStringMgr.h>
#include <ZkFreedomWizardsStrings.h>
#include <UiEnterSerialNumber.h>
#include <ZkService.h>
#include <ITokenService.h>
#include <IClientPersist.h>
#include <INymCentral.h>
#include <IZkPopup.h>
#include <IZkConsoleMsg.h>
#include <ZkErrorHandler.h>
#include <IZKErrorReporter.h>
#include <IZkException.h>
#include <ZkWebIntegration.h>
#include <ZKConfiguration.h>
// FIXME : remove that dependency
#include <UiMainWindow.h>

//===========================================================================
// LOCAL DECLARATIONS
//===========================================================================


//===========================================================================
// CLASS WzSetupSelectToken
//===========================================================================

//---------------------------------------------------------------------------
//
WzSetupSelectToken::WzSetupSelectToken( )
    : WzSetupBasePage( "SetupSerial" )
{
    ZkGladeLoader loader( "wizard_selecttoken.xml", "Wizard_ct" );

    // Get the label and make it bigger.
    GtkWidget* title = loader.GetWidget( "Wizard_title" );
    ZkGtkUtils::UseTitleFont( title );

    // Parent this UI in the base class container.
    GtkWidget* window = loader.GetWidget( "Wizard_ct" );
    AddPageUi( window );

    GtkWidget* tokenlist = loader.GetWidget( "Wizard_tokenlist" );
    m_pTokenCtrl = new UiTokenCtrl(GTK_CLIST(tokenlist), window);

    GtkWidget * wizText = loader.GetWidget( "Wizard_text" );
    ZKSTD::string strStandard;
    ZkStringMgr::Load( strStandard, IDS_CREATE_STANDARD_DESC );
    ZKSTD::string strPremium;
    ZkStringMgr::Load( strPremium, IDS_CREATE_PRO_DESC );
    strStandard = strStandard + "\n\n" + strPremium;
    gtk_label_set_text( GTK_LABEL(wizText), strStandard.c_str() );
    
    m_get = loader.GetWidget( "Wizard_get" );    
    gtk_signal_connect( GTK_OBJECT( m_get ), "clicked",
                        GTK_SIGNAL_FUNC( GetTokensCb ), this );    
    m_strTopText = loader.GetWidget( "labelTopText" );
}

//---------------------------------------------------------------------------
//
void WzSetupSelectToken::MapPage()
{
    // this rebuils the token list
    m_pTokenCtrl->Init();
    QueryAndEnable();
}

//---------------------------------------------------------------------------
//
void WzSetupSelectToken::QueryAndEnable()
{
    ZKSTD::string strLabelText;

    if( m_pTokenCtrl->GetNumTokenTypes() == 0  )
    {
        m_pTokenCtrl->SetSensitive( false );
        GetMainDialog()->ButtonSetActive( WZ_START, false );
        ZkStringMgr::Load( strLabelText, IDS_SELECT_NYM_TEXT_NO_NYMS );
        gtk_label_set_text( GTK_LABEL( m_strTopText ), strLabelText.c_str());
    }
    else
    {
        m_pTokenCtrl->SetSensitive( true );
        GetMainDialog()->ButtonSetActive( WZ_START, true );
        ZkStringMgr::Load( strLabelText, IDS_SELECT_NYM_TEXT_HAS_NYMS );
        gtk_label_set_text( GTK_LABEL( m_strTopText ), strLabelText.c_str());
    }
}

//---------------------------------------------------------------------------
//
WzSetupSelectToken::~WzSetupSelectToken( )
{
}

//---------------------------------------------------------------------------
//
void WzSetupSelectToken::GetTokensCb( GtkWidget *, WzSetupSelectToken* in_inst  )
{
	if( ! ZkService::GetINymCentral()->IsCurrentNymUser() )
	{
 	    if( IZkPopup::QuestionBox( IDS_CHANGE_TO_USER_NYM_QUESTION, ZK_YESNO) )
	    {
            UiMainWindow::get().ReselectUserNym();
	    }
    }
   
    // the web integretion is not implemented on linux
    ZkWebIntegration::LaunchWebSite( URL_BUY_SERIAL_NUM );
    
    in_inst->m_serialNumberDlg = new UiEnterSerialNumber();
    if(in_inst->m_serialNumberDlg->Run())
    {
        in_inst->GetTokens( in_inst->m_serialNumberDlg->GetSerialNumber() );
    }
    in_inst->QueryAndEnable();
}

//---------------------------------------------------------------------------
//
bool WzSetupSelectToken::GetTokens(const tstring in_strSerialNumber)
{
	bool bRetVal = false;

    // FIXME - put a buzy AVI here
    
	// UI Should only activate button if there is
	// a valid sized string
	ZKASSERT( in_strSerialNumber.size() == ZKSERIAL_NUMBER_LENGTH ) ;

	WSerialNumber serialNumber( in_strSerialNumber.c_str() ) ;

	try
	{
		ZkService::GetITokenService()->GetSerialNumberCapabilities( serialNumber ) ;

		IZkConsoleMsg::ShowMsgForceUpdate(IDS_CONS_REDEEM_SERIAL);
		
		ZkService::GetITokenService()->RedeemSerialNumberForTokens( serialNumber ) ;

		// Save right now so that user doesn't lose the new tokens
		try 
		{
			ZkService::GetIClientPersist()->SaveClientState();
		}
		catch( IZkException&  )
		{
			// Not much positive we can say if this fails
		}

		//Refresh token list because, if successful, we now have tokens
		m_pTokenCtrl->BuildTokenList() ;
		bRetVal = true;
	}
	catch( IZkException& ex)
	{
		ZkErrorHandler::GetIZkErrorReporter()->ShowError(ex);
		bRetVal = false ;
	}

	return bRetVal;
}
        
//---------------------------------------------------------------------------
//
bool WzSetupSelectToken::NextPage()
{
  
    // look to see if they have tokens. if not,
    // all these other calls will assert.
    // FIXME: pick a better error message, although it shouldn't ever come
    // up.
    if( m_pTokenCtrl->GetNumTokenTypes() == 0  )
    {    
	IZkPopup::MessageBox( IDS_BUY_NYM_PLEASE );
        return true;
    }

    tstring token;    
    m_pTokenCtrl->GetSelectedToken( token );


    // Always set the token, so if the user clicks "Previous" to come back 
    // to this screen, then unselects the token and clicks "Next" again he wil get
    // an error.
    WzSetupDlgImp* pSetupWz = GetMainDialog();
    pSetupWz->SetToken( token );

    if( token == "" )
    {
	IZkPopup::MessageBox( IDS_SEL_TOKEN_PLEASE );
        return true;
    }

    return false;
}
