// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <WzSetupSerial.h>
#include <ZkFreedomWizardsStrings.h>
#include <WzSetupDlgImp.h>

#include <ZkGtkUtils.h>
#include <UiThreadId.h>
#include <IZkPopup.h>

#include <ZkGladeLoader.h>
#include <ZkGtkUtils.h>
#include <ZkStatusSubject.h>

#include <ZKConfiguration.h>
#include <INym.h>
#include <INymCentral.h>
#include <ZkWebIntegration.h>
#include <ITokenService.h>
#include <ZkService.h>
#include <ZkErrorHandler.h>
#include <IZKErrorReporter.h>
#include <RZkException.h>
#include <IClientPersist.h>
#include <MiscMessages.h>



//===========================================================================
// CLASS WzSetupSerial
//===========================================================================

//---------------------------------------------------------------------------
//
WzSetupSerial::WzSetupSerial( )
    : WzSetupBasePage( "SetupSerial" )
{
    ZkGladeLoader loader( "wizard_serial.xml", "Wizard_ct" );

    // Get the label and make it bigger.
    GtkWidget* title = loader.GetWidget( "Wizard_title" );
    ZkGtkUtils::UseTitleFont( title );

    // Parent this UI in the base class container.
    AddPageUi( loader.GetWidget( "Wizard_ct" ) );

    m_get    = loader.GetWidget( "Wizard_get" );
    m_serial = loader.GetWidget( "Wizard_serial" );

    gtk_signal_connect( GTK_OBJECT( m_get ), "clicked",
                        GTK_SIGNAL_FUNC( GetSerialCb ), this );    

    // Hook the 'return' key to the NextPage function in the confirm box.
    gtk_signal_connect( GTK_OBJECT( m_serial ), "activate",
                        GTK_SIGNAL_FUNC( NextPageCb ), this );

    gtk_signal_connect( GTK_OBJECT( m_serial ), "changed",
                        GTK_SIGNAL_FUNC( SerialChangedCb ), this);
                        
}

//---------------------------------------------------------------------------
//
WzSetupSerial::~WzSetupSerial( )
{
}

//---------------------------------------------------------------------------
//
void WzSetupSerial::GetSerialCb( GtkWidget *, WzSetupSerial* /*inst*/ )
{
	// we used to go to seperate URL depending
	// whether Trial or not, but trial has been removed
	ZkWebIntegration::LaunchWebSite( URL_BUY_SERIAL_NUM_DURING_SETUP ) ;
}

//---------------------------------------------------------------------------
//
void WzSetupSerial::SerialChangedCb( 
    GtkWidget *,
    WzSetupSerial* in_inst 
)
{
    ZkGtkUtils::ToUpper( in_inst->m_serial );
}

//---------------------------------------------------------------------------
//
bool WzSetupSerial::NextPage()
{
    ZKASSERT( UiThreadId::IsUiThread() );
    ZKSTD::string strSerial( gtk_entry_get_text( GTK_ENTRY( m_serial ) ) );

	if( strSerial.empty() ) 
	{
		IZkPopup::MessageBox( IDS_NO_SERIAL_NUMBER, ZK_ERROR );
		return true;
	}
	else if( strSerial.length() < ZKSERIAL_NUMBER_LENGTH )
	{
		IZkPopup::MessageBox( IDS_INVALID_SERIAL_NUMBER, ZK_ERROR );
		return true;
	}
    else if( strSerial == m_exchanged ) {
        // If we already just exchanged this serial number successfuly,
        // just skip to the next page.
        return false;
    }

    StartThread();

    if( !WasSuccessful() )
    {
        // make sure we stay on the same page.
        return true;
    }
    else
    {
        // Disable field in case user presses back/next.
        gtk_widget_set_sensitive( GTK_WIDGET( m_serial ), FALSE );
        gtk_widget_set_sensitive( GTK_WIDGET( m_get ), FALSE );
    }

    return false;
}

//---------------------------------------------------------------------------
//
void WzSetupSerial::OnThreadStart()
{
    GetMainDialog()->SetSensitive( false );
    ZkStatusSubject::UpdateStatus( ZK_STATUS_NETWORK_BUSY, true, true );

    m_messageBox.Open(  IDS_REDEEMING_SERIAL );
}

//---------------------------------------------------------------------------
//
void WzSetupSerial::OnThreadEnd(  )
{
    // Stop the animation.
    ZkStatusSubject::UpdateStatus(ZK_STATUS_NETWORK_BUSY, false, true);
    GetMainDialog()->SetSensitive( true );    

    m_messageBox.Close();
}

//---------------------------------------------------------------------------
//
void WzSetupSerial::OnThreadExecute( )
{
    ZKSTD::string strSerial( gtk_entry_get_text( GTK_ENTRY( m_serial ) ) );
    WSerialNumber serialNumber( strSerial.c_str() );
    ZkService::GetITokenService()->RedeemSerialNumberForTokens( serialNumber );
    
    // persist the client file. they just paid money.
    ZkService::GetIClientPersist()->SaveClientState();
    
    m_exchanged = strSerial;
}




