/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_INCL_WZSETUPSERIAL
#define ZKS_INCL_WZSETUPSERIAL

/*===========================================================================
 * EXTERNAL DECLARATIONS
 *=========================================================================*/

#ifndef ZKS_INCL_WZSETUPBASE
#include <WzSetupBasePage.h>
#endif

#ifndef ZKS_INCL_UITHREADBASE
#include <UiThreadBase.h>
#endif

#ifndef ZKS_INCL_UISIMPLEMESSAGE
#include <UiSimpleMessage.h>
#endif

#include <gnome.h>
#include <string>

/*===========================================================================
 * CLASS WzSetupSerial
 *=========================================================================*/

class WzSetupSerial : 
    public WzSetupBasePage,
    public UiThreadBase
{

public: 

    /*----- member functions -----*/

    WzSetupSerial();
    virtual ~WzSetupSerial();

    virtual bool NextPage();
      
private: 

    /*----- static member functions -----*/

    static void GetSerialCb( GtkWidget *, WzSetupSerial* inst );
    static void SerialChangedCb( GtkWidget *, WzSetupSerial* in_inst  );

    /*----- member functions -----*/

	// Prohibited funcs - not implemented.
	WzSetupSerial( const WzSetupSerial& );
	const WzSetupSerial& operator=( const WzSetupSerial& );

    // Thread support functions.
    // Purpose: Starts the token exchange thread and sets states needed
    //          during the exchange. (animation, disabled UI)
    //          This function enters a local message loop and returns only 
    //          when the exchange thread has finished,
    void OnThreadStart();

    // Purpose: Called at the end of the exchange thread to reset the states and
    //          finalize serial number redemption.
    void OnThreadEnd( );

    // Purpose: Does the actual job of exchanging the tokens from the thread.
    void OnThreadExecute( );



    /*----- data members -----*/

    GtkWidget* m_get;
    GtkWidget* m_serial;

    ZKSTD::string m_exchanged;

    UiSimpleMessage m_messageBox;
};
#endif
