// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):



//===========================================================================
// EXTERNAL DECLARATIONS
//===========================================================================

#include <WzSetupWelcome.h>
#include <ZkFreedomWizardsStrings.h>

#include <WzSetupDlgImp.h>

#include <UiThreadId.h>

#include <ZkGladeLoader.h>
#include <ZkGtkUtils.h>
#include <ZkService.h>
#include <IClientInfo.h>

//===========================================================================
// CLASS WzSetupWelcome
//===========================================================================

//---------------------------------------------------------------------------
//
WzSetupWelcome::WzSetupWelcome( )
    : WzSetupBasePage( "SetupWelcome" )
{
    ZkGladeLoader loader( "wizard_welcome.xml", "Wizard_ct" );

    // Get the label and make it bigger.
    GtkWidget* title = loader.GetWidget( "Wizard_title" );
    ZkGtkUtils::UseTitleFont( title );

    // Parent this UI in the base class container.
    AddPageUi( loader.GetWidget( "Wizard_ct" ) );

    m_free = loader.GetWidget( "Wizard_free" );
    m_pro  = loader.GetWidget( "Wizard_pro" );

    if( !ZkService::GetIClientInfo()->IsFreeVersionAvailable())
    {
        // If you change this value, make sure you change the flag's default
        // value in WzSetupDlgImp as well.
        // Unfortunately we cannot set it here since the wizard way not even
        // exist at this point.
        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(m_pro), TRUE);

        // remove the instructions and radio button
        gtk_widget_hide(loader.GetWidget( "Wizard_text2" ));
        gtk_widget_hide(loader.GetWidget( "Wizard_frame" ));
    }        
    else
    {
        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(m_free), TRUE);
    }

    gtk_signal_connect (GTK_OBJECT( m_free ), "toggled",
                        GTK_SIGNAL_FUNC( ToggleCb ), this);    
    
}

//---------------------------------------------------------------------------
//
WzSetupWelcome::~WzSetupWelcome( )
{
}

//---------------------------------------------------------------------------
//
bool WzSetupWelcome::NextPage()
{
    ZKASSERT( UiThreadId::IsUiThread() );

    bool isFree = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( m_free ) );

    GetMainDialog()->SkipPage( "SetupSerial", isFree );
    GetMainDialog()->SetFreeFlag( isFree );


    return false;
}

//---------------------------------------------------------------------------
//
void WzSetupWelcome::ToggleCb( GtkWidget*, WzSetupWelcome* inst )
{
    bool basic = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( inst->m_free ) );

    inst->GetMainDialog()->SetFreeFlag( basic );

    // When this option is changed, make sure we clear any previously selected
    // token. This situation can occur if the user goes all the way to the nym 
    // page, clicks Previous all the way to this page and changes nym types.
    inst->GetMainDialog()->SetToken( "" );
    
}
