/*
 * $Id: zkSockets.c,v 1.8.32.1 2000/10/11 13:17:37 phil Exp $
 */

static const char * module_id_ = "$Id: zkSockets.c,v 1.8.32.1 2000/10/11 13:17:37 phil Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include "zkDNS.h"
#include "frAips.h"

#ifndef INADDR_NONE
#define INADDR_NONE -1
#endif

/**
 */
int zkSocketsFdSetopt(ZkError err, int fd, int opt)
{
  long flags;
  
  (void) module_id_;

  zkErrorClear(err);
  
  if ((flags = fcntl(fd, F_GETFL)) == -1) {
    close(fd);
    return 0;
  }

  if (fcntl(fd,  F_SETFL, flags | opt) == -1) {
    close(fd);
    return 0;
  }
  return 1;
}

/**
 * 
 */
int zkSocketsSetSockaddrIn(ZkError err, const char *hostname, 
                           const char *service, const char *protocol, 
                           struct sockaddr_in *addr)
{
  struct hostent *phe;
  struct servent *pse;
  size_t port;

  memset(addr, 0, sizeof *addr);
  addr->sin_family = AF_INET;
    
  if ((pse = getservbyname(service, protocol))) {
    /* Found in /etc/services */
    addr->sin_port = pse->s_port;
  } else {
    addr->sin_port = htons(strtol(service, (char **)NULL, 10));
    port = ntohs(addr->sin_port);
    if (port > 65536) {
      zkErrorSet(err, "setSockaddrIn", ZKERR, ZKMAIP, ZKEINVAL, 
                 "No service entry");
      return 0;
    }
  }

  if (zkGetHostByName(err, hostname, &phe)) {
    memcpy(&(addr->sin_addr), phe->h_addr, phe->h_length);
  } else if ((addr->sin_addr.s_addr = inet_addr(hostname)) == INADDR_NONE) {
    zkErrorSet(err, "setSockaddrIn", ZKERR, ZKMAIP, ZKEINVAL, "No hostname");
    return 0;
  }
  return 1;
}

int zkSocketsSetSockaddrIn2(ZkError err, const char *hostname, 
                            unsigned short port, struct sockaddr_in *addr)
{
  struct hostent *phe = NULL;

  memset(addr, 0, sizeof *addr);
  addr->sin_family = AF_INET;
    
  addr->sin_port = htons(port);

  if ((addr->sin_addr.s_addr = inet_addr(hostname)) == INADDR_NONE) {
    if (!zkGetHostByName(err, hostname, &phe)) {
      return 0;
    } else {
      memcpy(&(addr->sin_addr), phe->h_addr, phe->h_length);
    }
#if 0
  } else {
    zkErrorSet(err, "setSockaddrIn", ZKERR, ZKMAIP, ZKEINVAL, "No hostname");
    return 0;
#endif
  }

  return 1;
}
