/*
 * $Id: frAmtpClient.c,v 1.15.10.4 2000/12/14 17:52:36 jean Exp $
 */

static const char * module_id_ = "$Id: frAmtpClient.c,v 1.15.10.4 2000/12/14 17:52:36 jean Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <zkError.h>
#include "zkCStr.h"
#include "zkFile.h"
#include <zkMemBuf.h>
#include <ZKIP.h>
#include <zkipClient.h>
#include <zkMIME.h>
#include <frAmtpClient.h>

enum FrAmtpTarget {
  amtpTargetNym ,
  amtpTargetReplyBlock ,
  amtpTargetInternetUser ,
};

static int frAmtpSendMessage ( ZkError , MaipServer * , MaipServerResponse * , ZkEmailHeader , const char * , size_t , enum FrAmtpTarget , const void *,size_t );

DLLExport int
amtpHeaderDump (
  ZkError err ,
  void * data ,
  const unsigned char * text ,
  int size )
{
  ZkMemBuf membuf = (ZkMemBuf) data;

  if (!zkMemBufAppend(err,membuf,text,(size_t) size)) {
    return 0;
  }
  return 1;
}

static int
frAmtpSendMessage ( 
  ZkError err ,
  MaipServer * maip ,
  MaipServerResponse * msresp ,
  ZkEmailHeader header ,
  const char * mesg ,
  size_t mesg_sz ,
  enum FrAmtpTarget target_type ,
  const void * target ,
  size_t target_sz )
{
#define XXCOMMAND_SZ 512
  int return_code = 0;
  ZkipClient client = NULL;
  struct ZkipCommandResponse * response = NULL;
  char command[XXCOMMAND_SZ];
  char * extra_data = NULL;
  ZkMemBuf membuf = NULL;
  char * header_text = NULL;
  size_t hsz = 0;
  int fd = 0;
  int fConnected = 0; /* flag to know if we should send a quit, bug # 1983 */

  (void) module_id_; /* Get rid of compile warning */
  msresp->code = 0;
  zkCStrlcpy(msresp->message,"Did not connect to MAIP server", sizeof msresp->message);

  /*
   * First, we need to make a connection to a remote MAIP server
   */
  
  if (!(zkipClientNew(err,&client) &&
        zkipClientMakeBlocking(err,client,maip->timeout))) {
    goto finish;
  }

  if ((maip->maip[0] == (char) 0) || (maip->port == 0)) { 
    if (!zkipClientConnectToAddress(err,client,maip->addr)) {
      goto finish;
    }
  } else if (!zkipClientConnectToHost(err,client,maip->maip,maip->port)) {
    goto finish;
  }
  if (!zkipClientFd(err,client,&fd)) {
    goto finish;
  }

  /*
   * Try to get the welcome banner.  we should be greeted with a 220
   * (SERVICE_READY) 
   */

  if (!zkipClientReadResponse(err,client,&response,&extra_data)) {
    zkCStrlcpy(msresp->message,"Failed to read welcome banner", sizeof msresp->message);
    goto finish;
  }
  if (response->response != _SERVICE_READY) {
    msresp->code = response->response;
    strncpy(msresp->message,response->message,MSRMESG_SZ);    
    msresp->message[MSRMESG_SZ] = (char) 0;
    zkErrorSet(err,"frAmtpSendMessage",ZKERR,ZKMAMTP,ZKERROR,response->message);
    goto finish;
  } 
  else {
    fConnected = 1;
  }


  /*
   * First, we need to convert the header to a data buffer...
   */

  if (!(zkMemBufNew(err,&membuf) &&
        zkEmailHeaderDump(err,header,amtpHeaderDump,membuf) &&
        zkMemBufAppend(err,membuf,"\0",1) &&
        zkMemBufGetData(err,membuf, (void *) &header_text,&hsz))) {
    goto finish;
  }

  /*
   * Now we send over the HEADER command and the header text...
   */

  snprintf(command,sizeof command,"1 2.1.0 header rfc822header binary %d\015\012",hsz - 1);
  if (!zkipClientWrite(err,client,command,strlen(command))) {
    zkCStrlcpy(msresp->message,"Failed to write header command to MAIP server",sizeof msresp->message);
    msresp->code = 0;
    goto finish;
  } else if (!zkipClientReadResponse(err,client,&response,&extra_data)) {
    zkCStrlcpy(msresp->message,"Failed to read header command response from MAIP server",sizeof msresp->message);
    msresp->code = 0;
    goto finish;
  } else if (response->response != _DATA_STARTINPUT) {
    msresp->code = response->response;
    strncpy(msresp->message,response->message,MSRMESG_SZ);   
    msresp->message[MSRMESG_SZ] = (char) 0;
    goto finish;
  } else if (!(zkipClientWrite(err,client,header_text,hsz - 1) &&
               zkipClientWrite(err,client,"\015\012.\015\012",5))) {
    zkCStrlcpy(msresp->message,"Failed to write header data to MAIP server",sizeof msresp->message);
    goto finish;
  } else if (!zkipClientReadResponse(err,client,&response,&extra_data)) {
    zkCStrlcpy(msresp->message,"Failed to read header data response from MAIP server",sizeof msresp->message);
    msresp->code = 0;
    goto finish;
  } else if (response->response != _SERVICE_READY) {
    msresp->code = response->response;
    strncpy(msresp->message,response->message,MSRMESG_SZ);   
    msresp->message[MSRMESG_SZ] = (char) 0;
    goto finish; 
  }

  /* 
   * Now we send the message body 
   */

  snprintf(command,sizeof command,"1 2.1.0 message text binary %d\015\012",mesg_sz);
  if (!zkipClientWrite(err,client,command,strlen(command))) {
    zkCStrlcpy(msresp->message,"Failed to write message command to MAIP server",sizeof msresp->message);
    msresp->code = 0;
    goto finish;
  } else if (!zkipClientReadResponse(err,client,&response,&extra_data)) {
    zkCStrlcpy(msresp->message,"Failed to read message command response from MAIP server",sizeof msresp->message);
    msresp->code = 0;
    goto finish;
  } else if (response->response != _DATA_STARTINPUT) {
    msresp->code = response->response;
    strncpy(msresp->message,response->message,MSRMESG_SZ);   
    msresp->message[MSRMESG_SZ] = (char) 0;
    goto finish;
  } else if (!(zkipClientWrite(err,client,mesg,mesg_sz) &&
               zkipClientWrite(err,client,"\015\012.\015\012",5))) {
    zkCStrlcpy(msresp->message,"Failed to write message body to MAIP server",sizeof msresp->message);
    goto finish;
  } else if (!zkipClientReadResponse(err,client,&response,&extra_data)) {
    zkCStrlcpy(msresp->message,"Failed to read message body response from MAIP server",sizeof msresp->message);
    msresp->code = 0;
    goto finish;
  } else if (response->response != _SERVICE_READY) {
    msresp->code = response->response;
    strncpy(msresp->message,response->message,MSRMESG_SZ);   
    msresp->message[MSRMESG_SZ] = (char) 0;
    goto finish; 
  }

  /*
   * If we have alternate text to send, send it now.
   */

  if (maip->alttext && (maip->alttext_sz != 0)) {
    snprintf(command,sizeof command,"1 2.1.0 alttext text binary %d\015\012",maip->alttext_sz);
    if (!zkipClientWrite(err,client,command,strlen(command))) {
      zkCStrlcpy(msresp->message,"Failed to write alternate text command to MAIP server",sizeof msresp->message);
      msresp->code = 0;
      goto finish;
    } else if (!zkipClientReadResponse(err,client,&response,&extra_data)) {
      zkCStrlcpy(msresp->message,"Failed to read alternate text command response from MAIP server",sizeof msresp->message);
      msresp->code = 0;
      goto finish;
    } else if (response->response != _DATA_STARTINPUT) {
      msresp->code = response->response;
      strncpy(msresp->message,response->message,MSRMESG_SZ);   
      msresp->message[MSRMESG_SZ] = (char) 0;
      goto finish;
    } else if (!(zkipClientWrite(err,client,maip->alttext,maip->alttext_sz) &&
                 zkipClientWrite(err,client,"\015\012.\015\012",5))) {
      zkCStrlcpy(msresp->message,"Failed to write alternate text data to MAIP server",sizeof msresp->message);
      goto finish;
    } else if (!zkipClientReadResponse(err,client,&response,&extra_data)) {
      zkCStrlcpy(msresp->message,"Failed to read alternate text data response from MAIP server",sizeof msresp->message);
      msresp->code = 0;
      goto finish;
    } else if (response->response != _SERVICE_READY) {
      msresp->code = response->response;
      strncpy(msresp->message,response->message,MSRMESG_SZ);   
      msresp->message[MSRMESG_SZ] = (char) 0;
      goto finish; 
    }
  }

  /*
   * Convert the target to a data buffer
   */


  /*
   * Now, we send the target...
   */


  if (target_type == amtpTargetNym) {
    snprintf(command,sizeof command,"1 2.1.0 target target binary %d NYM\015\012",target_sz);
  } else if (target_type == amtpTargetReplyBlock) {
    snprintf(command,sizeof command,
            "1 2.1.0 target target binary %d REPLYBLOCK\015\012",target_sz);
  } else if (target_type == amtpTargetInternetUser) {
    snprintf(command,sizeof command,
            "1 2.1.0 target target binary %d INTERNET\015\012",target_sz);
  }
  if (!zkipClientWrite(err,client,command,strlen(command))) {
    zkCStrlcpy(msresp->message,"Failed to write target command to MAIP server",sizeof msresp->message);
    msresp->code = 0;
    goto finish;
  } else if (!zkipClientReadResponse(err,client,&response,&extra_data)) {
    zkCStrlcpy(msresp->message,"Failed to read target command response from MAIP server",sizeof msresp->message);
    msresp->code = 0;
    goto finish;
  } else if (response->response != _DATA_STARTINPUT) {
    msresp->code = response->response;
    strncpy(msresp->message,response->message,MSRMESG_SZ);   
    msresp->message[MSRMESG_SZ] = (char) 0;
    goto finish;
  } else if (!(zkipClientWrite(err,client,target,target_sz) &&
               zkipClientWrite(err,client,"\015\012.\015\012",5))) {
    zkCStrlcpy(msresp->message,"Failed to write target data to MAIP server",sizeof msresp->message);
    goto finish;
  } else if (!zkipClientReadResponse(err,client,&response,&extra_data)) {
    zkCStrlcpy(msresp->message,"Failed to read target data response from MAIP server",sizeof msresp->message);
    msresp->code = 0;
    goto finish;
  } else if (response->response != _SERVICE_READY) {
    msresp->code = response->response;
    strncpy(msresp->message,response->message,MSRMESG_SZ);   
    msresp->message[MSRMESG_SZ] = (char) 0;
    goto finish; 
  } else {
    return_code = 1;
  }

  /*
   * Terminate the connection
   */

finish:
#define QUIT_COMMAND "1 0.1.0 quit nodata none 0\015\012"
  if(fConnected) {
    zkipClientWrite(NULL,client,QUIT_COMMAND,strlen(QUIT_COMMAND));
    zkipClientReadResponse(NULL,client,&response,&extra_data);
  }
  if (client) {
    zkipClientDisconnect(NULL,client);
    zkipClientFree(NULL,&client);
  }
  if (membuf) {
    zkMemBufFree(NULL,&membuf);
  }

  return return_code;
}

DLLExport int
frAmtpSendMessageToNym (
  ZkError err ,
  MaipServer * maip ,
  MaipServerResponse * msresp ,
  ZkEmailHeader header ,
  const char * mesg ,
  size_t mesg_sz ,
  const char * nym ) 
{
  zkErrorClear(err);
  if (maip && msresp && mesg && nym) {
    if (!frAmtpSendMessage(err,maip,msresp,header,mesg,mesg_sz,
                           amtpTargetNym,nym,(size_t)strlen(nym))) {
      return 0;
    }
    return 1;
  }
  zkErrorSet(err,"frAmtpSendMessageToNym",ZKERR,ZKMAMTP,ZKENULLPTR,NULL);
  return 0;
}

DLLExport int
frAmtpSendMessageToReplyBlock (
  ZkError err ,
  MaipServer * maip ,
  MaipServerResponse * msresp ,
  ZkEmailHeader header ,
  const char * mesg ,
  size_t mesg_sz ,
  void * rb ,
  size_t rbsz )
{
  zkErrorClear(err);
  if (maip && msresp && mesg && rb) {
    if (!frAmtpSendMessage(err,maip,msresp,header,mesg,mesg_sz,
                           amtpTargetReplyBlock,rb,rbsz)) {
      return 0;
    }
    return 1;
  }
  zkErrorSet(err,"frAmtpSendMessageToReplyBlock",ZKERR,ZKMAMTP,ZKENULLPTR,NULL);
  return 0;
}

/*
 * Mario made me do it ;-)
 */

DLLExport int
frAmtpSendMessageToInternetUsersOrMail2NewsGateway (
  ZkError err ,
  MaipServer * maip,
  MaipServerResponse * msresp ,
  ZkEmailHeader header ,
  const char * mesg ,
  size_t mesg_sz ,
  ZkList iusers )
{
  int return_code = 0;
  size_t recipients = 0;
  char * target = NULL;
  size_t target_sz = 0;
  char * addr = NULL; /* Individual email addresses */
  ZkMemBuf membuf = NULL;
  int addcomma = 0;

  zkErrorClear(err);
  if (maip && msresp && header && mesg && iusers) {
    if (!zkMemBufNew(err,&membuf)) {
      goto finish;
    }
    zkListSize(err,iusers,&recipients);
    if (recipients < 1) {
      zkErrorSet(err,"frAmtpSendMessageToInternetUsersOrMail2NewsGateway",
                 ZKERR,ZKMAMTP,ZKEDATABAD,
                 "You must have at least one recipient in the list");
      goto finish;
    }
    zkListMoveCursor(err,iusers,LIST_HEAD);
    while (zkListGet(err,iusers,(void **) &addr,LIST_FORWARDS)) {
      if (addcomma) {
        if (!zkMemBufAppend(err,membuf,", ",2)) {
          goto finish;
        }
      } else {
        addcomma = 1;
      }
      if (!zkMemBufAppend(err,membuf,addr,(size_t) strlen(addr))) {
        goto finish;
      }
    }
    if (!zkMemBufGetData(err,membuf,(void *)&target,&target_sz)) {
      goto finish;
    }

    return_code = frAmtpSendMessage(err,maip,msresp,header,mesg,mesg_sz,
                           amtpTargetInternetUser,target,target_sz);

finish:
    if (membuf) {
      zkMemBufFree(NULL,&membuf);
    }
    return return_code;
  }
  zkErrorSet(err,"frAmtpSendMessageToInternetUsers",ZKERR,ZKMAMTP,ZKENULLPTR,NULL);
  return 0;
}

