/*
 *  $Id: fr_gencrypt.h,v 1.5.26.2 2000/10/11 13:17:46 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FR_GENCRYPT_H_
#define FR_GENCRYPT_H_


/*
 *
 *
 *                      General macros, typedefs, etc.
 *                      and prototypes for frCrypt initialization
 *
 *
 */


#include <limits.h>


/*
 *  The size of an initialization vector used in symmetric cryption (CBC
 *  mode).
 *  
 *  Any random data substituting for an IV should have the same size.
 */
#define FRCRYPT_IVEC_SIZE 8
#define FRCRYPT_RAND_SIZE FRCRYPT_IVEC_SIZE

/*
 *  All symmetric cryption must be done on blocks of data having the following
 *  size.
 */
#define FRCRYPT_SIZE_MULT 8

/*
 *  The maximum size of a symmetric key (bignum).
 *  
 *  bug: should hide this value.
 */
#define FRCRYPT_SYM_MAX_KEY_LEN 20

/*
 *  The amount of space needed to store a compact, portable representation of
 *  an FrCyptSym object.
 *  
 *  bug: should hide this value.
 */
#define FRCRYPT_SYM_PORTABLE_SIZE (1 + 3 * FRCRYPT_SYM_MAX_KEY_LEN)

/*
 *  Return the largest number of bytes of data, less than or equal to n, that
 *  may be encrypted.  That is, the number must be a multiple of
 *  FRCRYPT_SIZE_MULT.
 */
#define FRCRYPT_SIZE_FLOOR(n) ((n) - ((n) % FRCRYPT_SIZE_MULT))

/*
 *  Return the smallest number of bytes of data, greater than or equal to n,
 *  that may be encrypted.  That is, the number must be a multiple of
 *  FRCRYPT_SIZE_MULT.
 *  
 *  bug: using & and ~ is ugly.
 */
#define FRCRYPT_SIZE_CEIL(n) (((n) + (FRCRYPT_SIZE_MULT - 1)) & ~(FRCRYPT_SIZE_MULT - 1))


/*
 *  Convert between a number of bits and the corresponding number of bytes.
 */

#define BITS_TO_BYTES(n) (((n) + CHAR_BIT - 1) / CHAR_BIT)
#define BYTES_TO_BITS(n) ((n) * CHAR_BIT)


typedef struct fr_crypt_sym *FrCryptSym;

/* The number of key types that are defined */
#define FRCRYPT_NUM_KEYTYPES  2

typedef enum {
  FRCRYPT_KEYTYPE_UNUSED_,
  FRCRYPT_KEYTYPE_CRYPTION,
  FRCRYPT_KEYTYPE_SIGNING
} FrPubKeyType;

typedef enum {
  FRCRYPT_PUBALG_UNUSED_,
  FRCRYPT_PUBALG_DSA,
  FRCRYPT_PUBALG_ELGAMAL
} FrCryptPubAlg;

#if 0
typedef enum {
  FRCRYPT_KEYPARM_UNUSED_,
  FRCRYPT_KEYPARM_ALL,
  FRCRYPT_KEYPARM_PRIVONLY,
  FRCRYPT_KEYPARM_PUBONLY
} FrCryptPubPart;
#endif

/*
 *  bug: _NONE and XOR should be disallowed in production code.
 */
typedef enum {
  /** It's an error if this occurs. */
  FRCRYPT_SYMALG_UNUSED_ = 0,
  
#ifndef FREEDOM_PRODUCTION_BUILD

  /** No encryption is performed. */
  FRCRYPT_SYMALG_NONE = 1,
  /** Exclusive-or each byte of data with 0x55. */
  FRCRYPT_SYMALG_XOR = 2,
  
#endif
  
  /** Blowfish with a 128 bit key. */
  FRCRYPT_SYMALG_BLOWFISH = 3,
  /** Triple-DES with two keys. */
  FRCRYPT_SYMALG_DES3 = 4,
  /** DESX */
  FRCRYPT_SYMALG_DESX = 5
} FrCryptSymAlg;



#include "zks.h"
#include "zkError.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

DLLExport extern int frCryptInit(ZkError err, char *entFile);
DLLExport extern int frCryptFinal(ZkError err, char *entFile);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_GENCRYPT_H_ */
