/*
 *  $Id: fr_pk.h,v 1.19.8.3 2000/12/14 17:52:39 jean Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRPK_H_
#define FRPK_H_

/*
 *
 *
 *                       General public key routines.
 *
 *
 */

#include "zks.h"

#include <stdio.h>
#include "frEnt.h"
#include "zkError.h"
#include "zkIntTypes.h"
#include "zkClock.h"
#include "fr_gencrypt.h"
#include "openssl/bn.h"


typedef struct fr_crypt_pubkey *FrCryptPubKey;


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  extern DLLExport int frCryptPubKeyNew(ZkError err, FrCryptPubKey *key);
  extern DLLExport int frCryptPubKeyNewFromFp(ZkError err, FrCryptPubKey *key, FILE *fp);
  extern DLLExport int frCryptPubKeyNewFromFile(ZkError err, FrCryptPubKey *key, const char *file);
  extern DLLExport int frCryptPubKeyDup(ZkError err, FrCryptPubKey *copy, FrCryptPubKey key);
  extern DLLExport int frCryptPubKeyFree(ZkError err, FrCryptPubKey *key);

  extern DLLExport int frCryptPubKeyGetVersion(ZkError err, FrCryptPubKey key, unsigned int *vers);
  extern DLLExport int frCryptPubKeySetVersion(ZkError err, FrCryptPubKey key, unsigned int vers);
  extern DLLExport unsigned int frCryptPubKeyMinVersion(void);
  extern DLLExport int frCryptPubKeyNextVersion(ZkError err, FrCryptPubKey key, unsigned int *vers);

  extern DLLExport int frCryptPubKeyGetType(ZkError err, FrCryptPubKey key, FrPubKeyType *type);
  extern DLLExport int frCryptPubKeySetType(ZkError err, FrCryptPubKey key, FrPubKeyType type);

  extern DLLExport int frCryptPubKeyGetEntity(ZkError err, FrCryptPubKey key, FrEntIdExp *ent);
  extern DLLExport int frCryptPubKeySetEntity(ZkError err, FrCryptPubKey key, FrEntIdExp *ent);

  extern DLLExport int frCryptPubKeyGetSize(ZkError err, FrCryptPubKey key, unsigned int *nbits);
  extern DLLExport int frCryptPubKeySetSize(ZkError err, FrCryptPubKey key, unsigned int nbits);

  extern DLLExport int frCryptPubKeyGetExpiration(ZkError err, FrCryptPubKey key, ZkClock *exp);
  extern DLLExport int frCryptPubKeySetExpiration(ZkError err, FrCryptPubKey key, ZkClock exp);

  extern DLLExport int frCryptPubKeyGetParm(ZkError err, FrCryptPubKey key, size_t n, BIGNUM **bn, int *willFree, int *willSave);
  extern DLLExport int frCryptPubKeySetParm(ZkError err, FrCryptPubKey key, size_t n, BIGNUM *bn, int freeIt, int saveIt);

  extern DLLExport int frCryptPubKeyGetFreeMask(ZkError err, FrCryptPubKey key, uint8_t *mask);
  extern DLLExport int frCryptPubKeySetFreeMask(ZkError err, FrCryptPubKey key, uint8_t mask);
  extern DLLExport int frCryptPubKeyGetSaveMask(ZkError err, FrCryptPubKey key, uint8_t *mask);
  extern DLLExport int frCryptPubKeySetSaveMask(ZkError err, FrCryptPubKey key, uint8_t mask);

  extern DLLExport int frCryptPubKeyStorageSize(ZkError err, FrCryptPubKey key, size_t *n);

  extern DLLExport int frCryptPubKeyReadFromBuf(ZkError err, FrCryptPubKey key, char *b, size_t *bsz);
  extern DLLExport int frCryptPubKeyWriteToBuf(ZkError err, FrCryptPubKey key, char *b, size_t *bsz);
  extern DLLExport int frCryptPubKeyWriteToMem(ZkError err, FrCryptPubKey key, void **mem, size_t *msz);

  extern DLLExport int frCryptPubKeyBufGetVersion(ZkError err, void *kbuf, size_t ksz, unsigned int *vers);
  extern DLLExport int frCryptPubKeyBufGetType(ZkError err, void *kbuf, size_t ksz, FrPubKeyType *type);
  extern DLLExport int frCryptPubKeyBufGetExpiration(ZkError err, void *kbuf, size_t ksz, ZkClock *exp);
  extern DLLExport int frCryptPubKeyBufGetEntity(ZkError err, void *kbuf, size_t ksz, FrEntIdExp *ent);
  
  extern DLLExport int frCryptPubKeyEqual(ZkError err, FrCryptPubKey k0, FrCryptPubKey k1, int *equal);
  extern DLLExport int frCryptPubKeyMaskEqual(ZkError err, FrCryptPubKey k0, FrCryptPubKey k1, uint8_t mask, int *equal);

  extern DLLExport const char *frCryptPubKeyTypeStr(FrPubKeyType type);
  extern DLLExport const char *frCryptPubKeyAlgStr(FrCryptPubAlg alg);
  extern DLLExport int frCryptPubKeyPrint(ZkError err, FrCryptPubKey key, FILE *fp);
  extern DLLExport char *frCryptPubKeyBufIdSnprint(ZkError err, void *k, size_t ksz, char *s, size_t ssz);
  extern DLLExport int frCryptPubKeyShortPrint(ZkError err, FrCryptPubKey key, int indent, FILE *fp);
  extern DLLExport int frCryptPubKeyToCString(ZkError err, FrCryptPubKey key, char *aname, uint8_t mask, char **str);
  extern DLLExport int frCryptPubKeyPrintAsC(ZkError err, FrCryptPubKey key, uint8_t mask, char *name, FILE *fp);

  extern DLLExport int frCryptPubKeySetSharedParms(ZkError err, FrCryptPubKey key);
  extern DLLExport int frCryptPubKeyPublicValidate(ZkError err, FrCryptPubKey key);
  extern DLLExport int frCryptPubKeyBufPublicValidate(ZkError err, void *buf, size_t bsz);
  extern DLLExport int frCryptPubKeySetBlinding(ZkError err, FrCryptPubKey key, int boolean);

  extern DLLExport int frCryptPubKeySetCanCreateRoutes(ZkError err, FrCryptPubKey key, int canCreateRoutes);
  extern DLLExport int frCryptPubKeyGetCanCreateRoutes(ZkError err, FrCryptPubKey key, int * canCreateRoutes);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRPK_H_ */
