/*
 *  $Id: fr_rand.h,v 1.11.8.5 2000/12/14 17:52:39 jean Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FR_RAND_H_
#define FR_RAND_H_

#include "openssl/crypto.h"
#ifndef HAVE_YARROW
# define HAVE_YARROW
#endif
#if OPENSSL_VERSION_NUMBER < 0x0090581fL
# undef HAVE_YARROW
#endif

#ifdef linux
# undef HAVE_YARROW
# define USE_DEVRANDOM
#endif

/*
 *
 *
 *                           Randomness routines.
 *
 *
 */

#include "zks.h"
#include "zkError.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */
  
  DLLExport extern int frCryptAddEntropy(ZkError err, void *data, size_t dsz);
  DLLExport int frCryptAddEntropyFromSource(ZkError err, int source_id,
					    void *data, size_t dsz,
					    unsigned int bits);
  /* *enough is set to 1 if the PRNG has enough entropy */
  DLLExport int frCryptGetEntropyCount(ZkError err, int *enough,
				       int *num_sources, unsigned *source_id,
				       size_t *entropy_bits,
				       size_t *entropy_max);
  DLLExport int frCryptNewSource(ZkError err, unsigned *source);
  DLLExport extern int frCryptSetRandomData(ZkError err, void *data, size_t sz);
  DLLExport int frCryptRandomUInt32InRange(ZkError err, 
					   uint32_t min, uint32_t max,
					   uint32_t * rn);
  DLLExport void frRand_Debug(void); /* to be called in test programs only */

  /* caller asserts that /dev/urandom has entropy and it is not
     necessary to read from /dev/random in blocking mode */
  DLLExport void frRand_DontBlock(void);


  /*
   * Internal Functions */
  
  extern int fr_CryptRandInit(ZkError err, const char *entropyfile);
  extern int fr_CryptRandFinal(ZkError err);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_RAND_H_ */
