/*
 *  $Id: fr_sym.h,v 1.6.8.2 2000/12/14 17:52:39 jean Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FR_SYM_H_
#define FR_SYM_H_

/*
 *  
 *  
 *                           Symmetric Cryption.
 *  
 *  
 */
  
#include "zks.h"
#include "zkError.h"
#include "zkvheap.h"


/*
 *  Symmetric keys should be generated with this many bytes of seed material.
 */
#define FRCRYPT_SYMKEY_SEED_SIZE 24


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  DLLExport extern int frCryptSymNameToAlg(ZkError err, const char *name, FrCryptSymAlg *alg);

  DLLExport extern int frCryptSymDecrypt(ZkError err, FrCryptSym sym, void *ivec, void *data, size_t len);
  DLLExport extern int frCryptSymDecryptToBuf(ZkError err, FrCryptSym sym, void *ivec, void *data, void *output, size_t len);
  DLLExport extern int frCryptSymEncrypt(ZkError err, FrCryptSym sym, void *ivec, void *data, size_t len);
  DLLExport extern int frCryptSymEncryptToBuf(ZkError err, FrCryptSym sym, void *ivec, void *data, void *output, size_t len);
  DLLExport extern int frCryptSymFree(ZkError err, FrCryptSym *sym);
  DLLExport extern int frCryptSymGetAlg(ZkError err, FrCryptSym sym, FrCryptSymAlg *alg);
  DLLExport extern int frCryptSymKeyFromData(ZkError err, FrCryptSym sym, void *data, size_t len);
  DLLExport extern int frCryptSymLoad(ZkError err, FrCryptSym sym, void *data, size_t len);
  DLLExport extern int frCryptSymNew(ZkError err, FrCryptSym *sym);

#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )
  DLLExport extern int frCryptSymVHeapNew(ZkError err, FrCryptSym *sym, ZkVHeap vheap);
  DLLExport extern int frCryptSymVHeapFree(ZkError err, FrCryptSym *sym, ZkVHeap vheap);
#endif

  DLLExport extern int frCryptSymPortableSize(ZkError err, FrCryptSym sym, size_t *sz);
  DLLExport extern int frCryptSymCopy(ZkError err, FrCryptSym key, FrCryptSym *copy);
  DLLExport extern int frCryptSymGetKey(ZkError err, FrCryptSym sym, void *kbuf, size_t kbsz);
  DLLExport extern int frCryptSymGetKeySize(ZkError err, FrCryptSym sym, size_t *sz);
  DLLExport extern int frCryptSymSetKeySize(ZkError err, FrCryptSym sym, size_t sz);
  DLLExport extern int frCryptSymRandomKey(ZkError err, FrCryptSym sym);
  DLLExport extern int frCryptSymSetAlg(ZkError err, FrCryptSym sym, FrCryptSymAlg alg);
  DLLExport extern int frCryptSymSizeMultiple(ZkError err, FrCryptSym sym, size_t *sz);
  DLLExport extern int frCryptSymStore(ZkError err, FrCryptSym sym, void *data, size_t *len);
  DLLExport extern int frCryptSymDebugDump(ZkError err, FrCryptSym sym, char *buffer, uint16_t buffersz);
  
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_SYM_H_ */
