/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRBF_H_
#define FRBF_H_

#include "openssl/blowfish.h"
#include "fr_gencrypt.h"
#include "fr_sym.h"
#include "zkError.h"


/** The maximum key size. */
#define BLOWFISH_KEY_SIZE_MAX 72

/** The default size, in bytes, of the Blowfish key. */
#define BLOWFISH_DFL_KEY_SIZE 16

/*
 *  Blowfish keys can vary in size, and can be up to 448 bits in length.  For
 *  version 1 of our packets we'll set the value to 128 bits (16 bytes).
 */
struct blowfish_v1 {
  /** The length, in bytes, of the key. */
  size_t keySize;
  /** The original key.  Size based on SSLeay code. */
  unsigned char key[72];
  /** The expanded version of the key, used for efficiency. */
  BF_KEY expKey;
};


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  extern int fr_BlowfishInit(ZkError err, FrCryptSym sym);
  extern int fr_BlowfishGetKey(ZkError err, FrCryptSym sym, void *kbuf, size_t kbsz);
  extern int fr_BlowfishGetKeySize(ZkError err, FrCryptSym sym, size_t *sz);
  extern int fr_BlowfishSetKeySize(ZkError err, FrCryptSym sym, size_t sz);
  extern int fr_BlowfishRandomKey(ZkError err, FrCryptSym sym);
  extern int fr_BlowfishDecrypt(ZkError err, FrCryptSym sym, void *iv, void *data, size_t sz);
  extern int fr_BlowfishDecryptToBuf(ZkError err, FrCryptSym sym, void *iv, void *data, void *output, size_t sz);
  extern int fr_BlowfishEncrypt(ZkError err, FrCryptSym sym, void *iv, void *data, size_t sz);
  extern int fr_BlowfishEncryptToBuf(ZkError err, FrCryptSym sym, void *iv, void *data, void *output, size_t sz);

  extern int fr_BlowfishLoad(ZkError err, FrCryptSym sym, void *data, size_t sz);
  extern int fr_BlowfishStore(ZkError err, FrCryptSym sym, void *data, size_t *sz);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRBF_H_ */
