/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRDES3_H_
#define FRDES3_H_

#include "zks.h"
#include "openssl/des.h"
#include "fr_gencrypt.h"
#include "fr_sym.h"
#include "zkError.h"


/*
 *  Once the keys are generated, only the key schedules are needed for
 *  cryption, but we keep the original keys around in case we need to load or
 *  store them.
 */
struct des3 {
  des_cblock key1;
  des_cblock key2;
  des_cblock key3;
  des_key_schedule ks1;
  des_key_schedule ks2;
  des_key_schedule ks3;
};


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  extern int fr_Des3Init(ZkError err, FrCryptSym sym);
  extern int fr_Des3GetKey(ZkError err, FrCryptSym sym, void *kbuf, size_t kbsz);
  extern int fr_Des3GetKeySize(ZkError err, FrCryptSym sym, size_t *sz);
  extern int fr_Des3SetKeySize(ZkError err, FrCryptSym sym, size_t sz);
  extern int fr_Des3RandomKey(ZkError err, FrCryptSym sym);
  extern int fr_Des3Decrypt(ZkError err, FrCryptSym sym, void *iv, void *data, size_t sz);
  extern int fr_Des3DecryptToBuf(ZkError err, FrCryptSym sym, void *iv, void *data, void *output, size_t sz);
  extern int fr_Des3Encrypt(ZkError err, FrCryptSym sym, void *iv, void *data, size_t sz);
  extern int fr_Des3EncryptToBuf(ZkError err, FrCryptSym sym, void *iv, void *data, void *output, size_t sz);

  extern int fr_Des3Load(ZkError err, FrCryptSym sym, void *data, size_t sz);
  extern int fr_Des3Store(ZkError err, FrCryptSym sym, void *data, size_t *sz);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRDES3_H_ */
