/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include <string.h>
#include "fr_gencrypt.h"
#include "fr_misc.h"


/*
 *  Verify that the `dsz' characters in `data' all have the appropriate
 *  "cryptographic" padding character.
 */
DLLExport int fr_CryptCheckPad(void *data, size_t dsz)
{
  char *d = data;
  char c = FRCRYPT_SIZE_MULT - (dsz % FRCRYPT_SIZE_MULT);
  size_t i;

  for (i = 0; i < dsz; i++) {
    if (d[i] != c) {
      return 0;
    }
  }

  return 1;
}


/*
 *  Set the `dsz' bytes in `data' to the appropriate character for
 *  "cryptographic" padding.  That is, the value 8 - dsz mod 8.
 */
DLLExport void fr_CryptSetPad(void *data, size_t dsz)
{
  char c = FRCRYPT_SIZE_MULT - (dsz % FRCRYPT_SIZE_MULT);

  memset(data, c, dsz);
}
