/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef PUBKEY_H_
#define PUBKEY_H_


#include <limits.h>
#include "frCrypt.h"
#include "zkIntTypes.h"


/*
 *  This value should be the number of bits in the mask field of the
 *  fr_pubkey_info structure.
 */
#define NUM_BN_MAX (sizeof (uint8_t) * CHAR_BIT)


struct fr_pubkey_info {
  unsigned int vers; /* key version */
  FrPubKeyType type;
  FrCryptPubAlg alg;
  uint8_t mask; /* bitmask of set values in bn field */
  int keySize;
  BIGNUM *bn[NUM_BN_MAX];
};


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  extern DLLExport int fr_CryptPubKeyLoad(ZkError err, struct fr_pubkey_info *pki, void *data, size_t *dsz);
  extern DLLExport int fr_CryptPubKeyStore(ZkError err, struct fr_pubkey_info *pki, void *data, size_t *dsz);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* PUBKEY_H_ */
