/*
 * Freedom defined database object types
 *
 * $Id: frDBOTypes.h,v 1.43.26.3 2000/12/14 17:52:43 jean Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zeroknowledge.com>
 * Louis Beaumier <louis@zeroknowledge.com>

 *
 *
 *
 */

#ifndef FRDBOTYPES_H_
#define FRDBOTYPES_H_

#include "zks.h"
#include "zkIntTypes.h"
#include "zk_fr_stat_defs.h"
#include "zkDBO.h"
#include "zkDBODefs.h"
#include "zkDBOTypes.h"
#include "frEnt.h"
#include "frDBOFuncs.h"
#include "frDBOTypeNidbObj.h"
#include "zk_fr_stat_defs.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


  typedef uint32_t FrEntIdent;
  
  /***********/
  /* UserDef */
  /***********/

  struct T_FrDBOTypeUserDef {
    uint32_t regID;	/* The registration ID */
    uint32_t dsSize;    /* The data structure real size */
    char dboData[1];    /* The DBO data */
  };
  
  typedef struct FrDBOTypeUserDef_T *FrDBOTypeUserDef;
  
  /*********/
  /* ENTDB */
  /*********/

  struct T_FrDBOTypeEntity {
    uint32_t regID; /* MUST BE THE FIRST */
    FrEntIdExp ent;
    
  };

  struct FrDBOTypeEntityUpdates_T {
    uint32_t   regID;             /* MUST BE THE FIRST */
    uint32_t   _ds_size;          /* Internal DS size */
    FrEntIdent ent_ident;         /* Ent identifier */
    time_t     last_update_ts;    /* The last topo change made */
    time_t     status_change_ts;  /* The last status change made */
  };
  
  typedef struct FrDBOTypeEntityUpdates_T *FrDBOTypeEntityUpdates;

  struct T_FrDBOTypeEntityID {
    uint32_t regID;          /* MUST BE THE FIRST */
    uint32_t entid;
    time_t last_update;    /* Used in transmission to track update */
    FrEntIdExp ent;
  };

  typedef struct T_FrDBOTypeEntityID *FrDBOTypeEntityID;

  /********/
  /* NTDB */
  /********/

  struct T_FrDBOTypeNtdbAip {
    uint32_t regID;         /* MUST BE THE FIRST */
    uint32_t aip_entid;     /* Aip ent identifier */
    uint32_t aip_port;      /* The aip port number */
    uint32_t protocol_vers; /* The aip protocol version number */
    uint32_t dh_port;       /* DH Port number */
    uint32_t dh_timeout;    /* DH Timeout */
    uint32_t dh_negotiation;/* DH Negotiation */
    uint32_t host_entid;    /* The host upon which this aip resides */
                            /* This may have to change for a cluster */
  };

  typedef struct T_FrDBOTypeNtdbAip *FrDBOTypeNtdbAip;

  struct T_FrDBOTypeNtdbNode {
    uint32_t regID;        /* MUST BE THE FIRST */
    uint32_t node_entid;   /* This entity's ent-ident */
    uint32_t node_port;    /* The port this entity runs on */
    uint32_t num_hosts;    /* The number of hosts this entity resides on.
			    * Currently, this is set to one. In anticipation of
			    * clusters */
    uint32_t host_entid;   /* The host upon which this entity resides */
    uint32_t spare1;       /* Spare */
    uint32_t spare2;       /* Spare */
  };

  typedef struct T_FrDBOTypeNtdbNode *FrDBOTypeNtdbNode;

  struct T_FrDBOTypeNtdbHost {
    uint32_t regID;        /* MUST BE THE FIRST */
    uint32_t host_entid;   /* The host ent-identifier */
    uint32_t policy;       /* Private or public */
    uint32_t num_ipaddrs;  /* Place holder for now */
    uint32_t ipaddr_nbo;   /* in nbo - should be an array */
    char hostname[64];     /* FQN of this host */
    char region[64];       /* Bug: workaround for limitation in zkDBOTypeStr */
    char servoperator[64];     /* Bug: workaround for limitation in zkDBOTypeStr */
  };

  typedef struct T_FrDBOTypeNtdbHost *FrDBOTypeNtdbHost;

  /*
   * A link is defined as to end nodes
   * For searching purposes, aip1 < aip2
   */
  struct T_FrDBOTypeNtdbLink {
    uint32_t regID;        /* MUST BE THE FIRST */
    uint32_t link_entid;   /* The Link's ent-identifier */
    uint32_t node1_entid;  /* Low order node ent-identifier */
    uint32_t node2_entid;  /* High order node ent-identifier */
    uint32_t ts_delay;     /* For this link */
    uint32_t ts_pkts;      /* For this link */
    uint32_t ct_on_flag;   /* Enable cover traffic */
    uint32_t spare1;    
    uint32_t spare2;    
    
  };

  typedef struct T_FrDBOTypeNtdbLink *FrDBOTypeNtdbLink;

  
  struct T_FrDBOTypeNtdbNeighbours {
    uint32_t regID;         /* MUST BE THE FIRST */
    uint32_t num_neighbours;/* Used for size */
    uint32_t node_entid;    /* Remote node identifier */
    uint32_t neighbour[1];  /* Aip EntId's of neighbours 
			     * HACK: actually num neighbours */
  };

  typedef struct T_FrDBOTypeNtdbNeighbours *FrDBOTypeNtdbNeighbours;

  struct neibr_detail {
    FrEntIdExp ent;
    uint32_t ipaddr;           /* In NBO */
    uint32_t dh_port;
    uint32_t aip_port;
    uint32_t dh_timeout;
    uint32_t dh_negotiation;
    uint32_t ts_delay;
    uint32_t ts_pkts;
    uint32_t ct_on_flag;
    uint32_t spare1;
    uint32_t spare2;
  };
    
  struct T_FrDBOTypeNtdbDetailedNeighbours {
    uint32_t regID;                     /* MUST BE THE FIRST */
    uint32_t num_neighbours;            /* Used for size */
    uint32_t dh_port;
    uint32_t aip_port;
    FrEntIdExp ent;                     /* The requesting node's entity  */
    struct neibr_detail neighbour[1];   /* HACK: actually num neighbours */
  };

  typedef struct T_FrDBOTypeNtdbDetailedNeighbours *FrDBOTypeNtdbDetailedNeighbours;

  /********/
  /* NSDB */
  /********/
  struct T_FrDBOTypeNsdbAip {
    uint32_t regID;		/* The DBO Registration ID     */
    uint32_t entid;             /* Entity identifier           */
    uint32_t reference_date;    /* The reference date for info */
    uint32_t last_state_date;   /* The last date the server went up or down */
    uint32_t last_update_date;  /* The time of the last update */
    uint32_t unavailable_flag;  /* Entity is unavailable       */
    uint32_t up_flag;           /* Entity is up                */
    UINT64_T ttl;               
    uint32_t uptime;            /* Time up since reference     */
    uint32_t downtime;          /* Time down since reference  +*/
    UINT64_T msgs_rcvd;         
    UINT64_T msgs_sent;         
    UINT64_T msgs_rcvd_1;       
    UINT64_T msgs_sent_1;       
    UINT64_T msgs_rcvd_2;       
    UINT64_T msgs_sent_2;       
    uint32_t num_updates;       
    uint32_t entity_rating;     
    uint32_t security_level;    
    UINT64_T OR_entity_rating;  
    UINT64_T OR_up_flag;        
    UINT64_T msgs_real_sent;      
    UINT64_T msgs_real_sent_1;  
    UINT64_T msgs_real_sent_2;  
    UINT64_T spare_1;           
    UINT64_T spare_2;
    UINT64_T spare_3;
    UINT64_T spare_4;
    UINT64_T spare_5;
    UINT64_T total_bytes_in;
    UINT64_T total_bytes_out;
    UINT64_T connections;       
    char release[FRSTATUPD_RELEASE_STRING_SZ];/* The release strings reported */
    /* Optimization - data not stored */
    /* FrEntIdExp entity; */
  };

  typedef struct T_FrDBOTypeNsdbAip  * FrDBOTypeNsdbAip;
  
  struct T_FrDBOTypeAipStatus {
    uint32_t regID; /* MUST BE THE FIRST */
    uint32_t status;
  };

  typedef struct T_FrDBOTypeAipStatus *FrDBOTypeAipStatus;
  
  typedef struct T_FrDBOTypeNsdbAipObj * FrDBOTypeNsdbAipObj;
  typedef struct T_FrDBOTypeNsdbMaipObj * FrDBOTypeNsdbMaipObj;
  typedef struct T_FrDBOTypeNsdbLinkObj * FrDBOTypeNsdbLinkObj;

  typedef struct T_FrDBOTypeNsdbObj  * FrDBOTypeNsdbObj;

  struct T_frDBOTypeNsdbAipObj {

    uint32_t new_cnx;        	/* Varies according to context */
    uint32_t current_cnx;	/* Current connections */
    uint32_t new_streams;
    uint32_t curr_streams;
    uint32_t new_ports;
    uint32_t curr_ports;

    uint32_t bytes_rcvd;        /* Client bytes and packets */
    uint32_t bytes_sent;         
    uint32_t bytes_dropped;
    
    uint32_t pkts_rcvd;         
    uint32_t pkts_sent;         
    uint32_t pkts_dropped;

    uint32_t wh_configured_thput;
    UINT64_T wh_bytes_rcvd;
    UINT64_T wh_bytes_sent;

    uint32_t wh_update;         /* indacate if wormhole data has been updated: 0=no, 1=yes */

    /* Computed values */

    UINT64_T total_cnx;
    UINT64_T total_streams;

    UINT64_T total_bytes_rcvd;
    UINT64_T total_bytes_sent;
    UINT64_T total_bytes_dropped;

    UINT64_T total_pkts_rcvd;
    UINT64_T total_pkts_sent;
    UINT64_T total_pkts_dropped;

    UINT64_T total_wh_bytes_rcvd;
    UINT64_T total_wh_bytes_sent;

    uint32_t num_neighbours;

  };


  struct T_frDBOTypeNsdbMaipObj {

    uint32_t msgs_rcvd;         
    uint32_t msgs_sent;         
    uint32_t msgs_dropped;
    uint32_t msgs_queued;

    uint32_t bytes_rcvd;         
    uint32_t bytes_sent;         
    uint32_t bytes_dropped;
    
    UINT64_T total_msgs_rcvd;
    UINT64_T total_msgs_sent;
    UINT64_T total_msgs_dropped;

    UINT64_T total_bytes_rcvd;
    UINT64_T total_bytes_sent;
    UINT64_T total_bytes_dropped;

  };

  struct T_frDBOTypeNsdbLinkObj {
    uint32_t linkend;      /* which end is filled 0 - Low end, 1 - high end , 2 - both ends */
    UINT64_T link_ents_1;
    UINT64_T link_ents_2;

    UINT64_T link_pkts_lost_1;
    UINT64_T link_pkts_lost_2;

    UINT64_T link_pkts_dropped_1;
    UINT64_T link_pkts_dropped_2;
    UINT64_T link_pkts_rcvd_1;
    UINT64_T link_pkts_rcvd_2;
    UINT64_T link_pkts_sent_1;
    UINT64_T link_pkts_sent_2;

    UINT64_T link_bytes_dropped_1;
    UINT64_T link_bytes_dropped_2;
    UINT64_T link_bytes_rcvd_1;
    UINT64_T link_bytes_rcvd_2;
    UINT64_T link_bytes_sent_1;
    UINT64_T link_bytes_sent_2;

    UINT64_T total_bytes_rcvd;
    UINT64_T total_bytes_sent;
    UINT64_T total_bytes_dropped;

    UINT64_T total_pkts_rcvd;
    UINT64_T total_pkts_sent;
    UINT64_T total_pkts_dropped;

    UINT64_T total_link_pkts_rcvd_1;
    UINT64_T total_link_pkts_rcvd_2;
    UINT64_T total_link_pkts_sent_1;
    UINT64_T total_link_pkts_sent_2;
    UINT64_T total_link_pkts_dropped_1;
    UINT64_T total_link_pkts_dropped_2;

  };

  typedef enum {
    NsdbAip,
    NsdbMaip,
    NsdbLink
  } E_frNsdbObjType;
  
  struct T_FrDBOTypeNsdbObj {
    uint32_t entid;             /* Entity identifier MUST be first */

    FrEntIdExp entity;

    E_frNsdbObjType type;
    uint32_t last_seqno;        /* Last sequence number        */
    
    time_t   reference_date;    /* The reference date for info */
    time_t   launch_date;       /* Might not need this. */
    time_t   last_update_date;  /* The time of the last update */
    time_t   last_state_date;   /* The last date the server went up or down */
    
    uint32_t up_flag;           /* Entity is up                */
    uint32_t unavailable_flag;  /* Entity is unavailable       */
    uint32_t num_updates;       /* Number reports recv'd      +*/
    uint32_t entity_rating;     /* Entity Rating              +*/
    uint32_t security_level;    /* Entity Security Level      +*/

    char release[FRSTATUPD_RELEASE_STRING_SZ];/* The release strings reported */

    uint32_t OR_entity_rating;  /* Override Entity Rating      */
    uint32_t OR_up_flag;        /* Override Entity is up       */
    time_t   uptime;            /* Time up since reference     */
    time_t   downtime;          /* Time down since reference   */

    union {
      struct T_frDBOTypeNsdbAipObj   aip;
      struct T_frDBOTypeNsdbMaipObj  maip;
      struct T_frDBOTypeNsdbLinkObj  link;
    } u;
  };

    struct T_FrDBOTypeNsdbCDV {
	uint32_t regID;         /* MUST BE THE FIRST */
	uint32_t DBOversion;    /* version control */
	uint32_t cdv;           /* Cache Data Version */
    };

    typedef struct T_FrDBOTypeNsdbCDV *FrDBOTypeNsdbCDV;

  /****************/
  /* NIDB QUERIES */
  /****************/

  struct FrDBOTypeNidbQRespList_T {
    uint32_t regID;         	/* MUST BE THE FIRST */
    uint32_t dboSize;		/* The DBO size */
    uint32_t retcode;           /* The return code from the server */
    uint32_t num_answers;	/* The number of answers */
    struct T_FrDBOTypeEntityID answer[1];  /* The answers */
  };

  typedef struct FrDBOTypeNidbQRespList_T *FrDBOTypeNidbQRespList;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif

