/*
 * Initialize all DBO Drivers
 *
 * $Id: frDBODriverInit.c,v 1.13.28.4 2000/12/14 17:52:44 jean Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zeroknowledge.com>

 * 

 */

#include "zks.h"

#include <zkError.h>
#include <zkDebug.h>
#include <zkDBO.h>
#include <frDBOTypes.h>
#include <frDBOFuncs.h>
#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB

static int frDBODriverInitialized = 0; /* Bug: Not thread safe */

DLLExport
int frDBODriverInitAll(ZkError err)
{

  if ( frDBODriverIsInitialized(err) )
    return 1;
  
#ifndef WIN32
  if ( ! frDBODriverInitEntityID(err) || 
       ! frDBODriverInitEntity(err) || 
       ! frDBODriverInitEntityUpdates(err) || 
       ! frDBODriverInitNtdbAip(err) ||
       ! frDBODriverInitNsdbAip(err) ||
       ! frDBODriverInitNsdbCDV(err) ||
       ! frDBODriverInitNtdbLink(err) ||
       ! frDBODriverInitNtdbDetailedNeighbours(err) ||
       ! frDBODriverInitNtdbNeighbours(err) ||
       ! frDBODriverInitNtdbNode(err) ||
       ! frDBODriverInitNtdbHost(err) ||
       ! frDBODriverInitNidbObj(err) ||
       (0)) {
    return 0;
  }
#else
#pragma message(Reminder "Trimming the of DBO lib for win32 needs to be checked")
  if ( ! frDBODriverInitEntityID(err) || 
       ! frDBODriverInitEntity(err) || 
       ! frDBODriverInitNtdbNeighbours(err) ||
       ! frDBODriverInitNsdbCDV(err) ||
       ! frDBODriverInitNidbObj(err) ||
       (0)) {
    return 0;
  }
#endif
  
#ifndef NDEBUG
  if ( ! zkDBOTestAll(err) ) {
    return 0;
  }
#endif  

  frDBODriverInitialized = 1;
  return 1;
  
}

DLLExport
int frDBODriverClearAll(ZkError err)
{

  if ( ! frDBODriverIsInitialized(err) ) {
    return 0;
  }
  if ( ! zkDBODescUnRegisterAll(err) ) {
    return 0;
  }
  frDBODriverInitialized = 0;
  
  return 1;
}

DLLExport
int frDBODriverIsInitialized (ZkError err)
{
  (void) err;
  return (frDBODriverInitialized);
}
