/*
 * NTDBaip  DBO Driver routines
 * $Id: frDBOTypeNtdbAip.c,v 1.8.28.1 2000/10/11 13:17:53 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zeroknowledge.com>

 *
 */
#ifndef WIN32

#include "zks.h"

#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>
#include "zkError.h"
#include "zkLog.h"
#include "zkDefs.h"
#include "zkDump.h"
#include "zkDebug.h"
#include "frDBOTypes.h"

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNDB

#define DBODS_REGID ZKDBO_REG_FRNTDBAIP
#define DBODS_T T_FrDBOTypeNtdbAip
#define DBODS_VERS  3


typedef struct DBODS_T *DBODS;

static int DSnew (ZkError err, uint32_t version, void **ds, size_t *dsz)
{
  static const char *funcname = "DSnew";

  DBODS d;

  (void) version;

  if (!dsz || !ds) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,
	      		  "Null pointers passed to function");
    return 0;
  }
  
  *dsz = sizeof(struct DBODS_T);
  zkMALLOC (d, DBODS, sizeof(struct DBODS_T));

  d->regID = DBODS_REGID /*  || (DBOS_VERS << 0xffff) - later */ ;

  *ds = (void *)d;
  return 1;
}

static int DSfree (ZkError err, void **ds, size_t *dsz)
{
  static const char *funcname = "DSfree";
  
  if (!dsz || !ds || !*ds) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,
	      		  "Null pointers passed to function");
    return 0;
  }
  
  memset (*ds, 0, *dsz);
  free (*ds);
  *ds = NULL;
  *dsz = 0;
  return 1;
}

/**
 * Passing a null pointer to *pobj will create the buffer you need.
 * On failure, you'll have to take care of cleaning it up.
 */
static int DSpack (ZkError err, void *ds, size_t dsz, void **pds, size_t *psz)
{
  static const char *funcname = "DSpack";
  DBODS d;
  size_t pos;
  
  if ( !ds || !dsz || !pds || !psz ) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,
	      		  "Null pointers or values passed to function");
    return 0;
  }
  d = ds;

  if (!zkDBOPackedInit(err, d->regID, pds, psz, &pos))
    return 0;

#define DBO_PACK(x) \
  if (!zkDBOPackField(err, d->regID, #x, (void *) & d->x, \
		      sizeof(d->x), pds, psz, &pos)) { \
    return 0; \
  }

  DBO_PACK(regID);
  DBO_PACK(aip_entid);
  DBO_PACK(aip_port);
  DBO_PACK(protocol_vers);
  DBO_PACK(dh_port);
  DBO_PACK(dh_timeout);
  DBO_PACK(dh_negotiation);
  DBO_PACK(host_entid);

  return 1;
}

/**
 * Passing a null pointer to *ds will create the buffer you need.
 * On failure, you'll have to take care of cleaning it up.
 */
static int DSunpack(ZkError err, void **ds, size_t *dsz, void *pds, size_t psz)
{
  static const char *funcname = "DSunpack";
  size_t s, pos=0;
  int regID;
  void *pd = NULL;
  struct DBODS_T *d;
  
  if ( !ds || !dsz || !pds || !psz) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,
		  "Null pointers or values passed to function");
    return 0;
  }

  if (!zkDBOUnpackedInit(err, &regID, ds, dsz, pds, psz, &pos))
    return 0;

  d = *ds;

#define DBO_UNPACK(x) \
  s = sizeof(d->x); \
  pd = &d->x; \
  if (!zkDBOUnpackField(err, regID, #x, &pd, \
		 &s, pds, psz, &pos)) \
    return 0;

  DBO_UNPACK(regID);
  DBO_UNPACK(aip_entid);
  DBO_UNPACK(aip_port);
  DBO_UNPACK(protocol_vers);
  DBO_UNPACK(dh_port);
  DBO_UNPACK(dh_timeout);
  DBO_UNPACK(dh_negotiation);
  DBO_UNPACK(host_entid);

  return 1;
}

#ifndef DSdump
static int DSdump (ZkError err, void *ds)
{
  static const char *funcname = "DSdump";
  DBODS d;
  
  if ( !ds ) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,
	      		  "Null pointers or values passed to function");
    return 0;
  }
  d = (DBODS) ds;
  zkDebug("**Dumping Registered Datastructure for AipStatus***");
  zkDebug("regID    : %lu", (unsigned long) d->regID);
  zkDebug("aip_entid: %lu", (unsigned long) d->aip_entid);
  zkDebug("aip_port : %lu", (unsigned long) d->aip_port);
  zkDebug("proto_ver: %lu", (unsigned long) d->protocol_vers);
  zkDebug("dh_port  : %lu", (unsigned long) d->dh_port);
  zkDebug("dh_timout: %lu", (unsigned long) d->dh_timeout);
  zkDebug("dh_negot : %lu", (unsigned long) d->dh_negotiation);
  zkDebug("hostentid: %lu", (unsigned long) d->host_entid);

  return 1;
}

static int DStest (ZkError err)
{
  DBODS d1 = NULL;
  DBODS d2 = NULL;
  void *pds = NULL , *ds2_p = NULL;
  size_t dsz1 = 0 , dsz2 = 0 , pdsz = 0 ;
  int retcode = 0;

  dsz1 = sizeof(struct DBODS_T);
  zkDBONew(err, DBODS_REGID, (void **) &d1, &dsz1);
  zkDBONew(err, DBODS_REGID, (void **) &d2, &dsz2);

  /*
   * Do any special initialization here
   */
  d1->aip_entid = 16;
  d1->aip_port = 0x0FFF;
  d1->protocol_vers = 4;
  d1->dh_port = 0x1111;
  d1->dh_timeout = 0x1111;
  d1->dh_negotiation = 0x1111;
  d1->host_entid = 3333;

  /*
   * The rest should be the same
   */
  DSdump(err, d1);
  	 
  zkDebug("Packing datastructure");
  if (!zkDBOPack (err, d1, dsz1, &pds, &pdsz))
    goto fail;

  dsz2 = sizeof(struct DBODS_T);
  ds2_p = (void *) d2;

  zkDebug("Unpacking datastructure");
  if (!zkDBOUnpack (err, &ds2_p, &dsz2, pds, pdsz))
    goto fail;

  if (dsz1 != dsz2) {
    zkDebug("Data structure sizes have changed (%u -> %u)", dsz1, dsz2);
    if (zkDebugIsOn())
      zkDumpBuffer(err,(void *)d1,dsz1,zkDumpToStream,stderr);
    if (zkDebugIsOn())
      zkDumpBuffer(err,(void *)d2,dsz2,zkDumpToStream,stderr);
    goto fail;

  }

  if (memcmp(d1, d2, dsz1) != 0) {
    zkDebug("Data structure improperly copied");
    if (zkDebugIsOn())
      zkDumpBuffer(err,(void *)d1,dsz1,zkDumpToStream,stderr);
    if (zkDebugIsOn())
      zkDumpBuffer(err,(void *)d2,dsz2,zkDumpToStream,stderr);
    goto fail;
  }

  retcode = 1;
  
 fail:
  if (retcode) {
    zkDebug("Self test passed");
  } else {
    zkDebug("Self test FAILED");
  }
  FREEIT (d1);
  FREEIT (d2);
  FREEIT (pds);
  return retcode;
  
}

#else
#define DSdump NULL
#define DStest NULL
#endif


DLLExport
int frDBODriverInitNtdbAip(ZkError err)
{
  ZkDBODesc dboDesc;
  ZkDBODriver dboDrv;

  size_t sz, psz;

#define NUMFIELDS 8

  if (!zkDBODescNew(err, NUMFIELDS, &dboDesc))
    return 0;

  /* Bug: Fix the userdefs */
  /* Bug: Problem with recording sizes of BUF and STR */
  if (!zkDBODescSetField(err, dboDesc, 0, "regID", FT_UINT32, &sz, &psz)
      || !zkDBODescSetField(err, dboDesc, 1, "aip_entid", FT_UINT32, &sz, &psz)
      || !zkDBODescSetField(err, dboDesc, 2, "aip_port", FT_UINT32, &sz, &psz)
      || !zkDBODescSetField(err, dboDesc, 3, "protocol_vers",FT_UINT32, &sz, &psz)
      || !zkDBODescSetField(err, dboDesc, 4, "dh_port", FT_UINT32, &sz, &psz)
      || !zkDBODescSetField(err, dboDesc, 5, "dh_timeout", FT_UINT32, &sz, &psz)
      || !zkDBODescSetField(err, dboDesc, 6, "dh_negotiation",FT_UINT32, &sz, &psz)
      || !zkDBODescSetField(err, dboDesc, 7, "host_entid",FT_UINT32, &sz, &psz)
      )
    return 0;

  if (!zkDBODriverNew (err, &dboDrv))
    return 0;

  if (!zkDBODriverInit (err, DBODS_REGID, dboDrv,
			DSnew, DSfree, DSpack, DSunpack,
			NULL, NULL, DSdump, DStest))
    return 0;

  if (!zkDBODescRegister(err, dboDesc, dboDrv))
    return 0;

# ifndef NDEBUG
  if (!zkDBODescDump(err, dboDesc))
    return 0;
# endif  
  return 1;
}

#endif
