#ifndef FR_ENTID_H_
#define FR_ENTID_H_


/*
 *  $Id: fr_entid.h,v 1.19.10.2 2000/10/11 13:17:58 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Entities.
 *  
 *  bug: for now we expose the entity ID structure, as it's simple and we're
 *  pressed for time...
 */


#include "zks.h"

#include <stdio.h>
#include "zkError.h"

#include "frentids.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  /* Basic type manipulation */ 

  extern DLLExport FrEntIdType frEntTypeFromStr(const char * s);
  extern DLLExport const char * frEntTypeGetStr(FrEntIdType type);

  /* Create and release entity structures */

  extern DLLExport int frEntNew(ZkError err, FrEntIdExp **e);
  extern DLLExport int frEntFree(ZkError err, FrEntIdExp **e);
  
  /* setting, accessing and comparing */

  extern DLLExport int frEntSet(ZkError err, FrEntIdExp *ent, FrEntIdType entType, const char *entNameStr);
  extern DLLExport int frEntSetFromStr2(ZkError err, FrEntIdExp *ent, const char *entTypeStr, const char *entNameStr);

  extern DLLExport int frEntCopy(ZkError err, FrEntIdExp *dst, FrEntIdExp *src);

  extern DLLExport int frEntGetTypeStr(ZkError err, FrEntIdExp *ent, const char **str);

  extern DLLExport int frEntEqual(ZkError err, FrEntIdExp *e0, FrEntIdExp *e1, int *equal);
  extern DLLExport int frEntEqualAllowUndef(ZkError err, FrEntIdExp *e0, FrEntIdExp *e1, int *equal);

  /* Validation */

  extern DLLExport int frEntTypeIsValid(ZkError err, FrEntIdType e, int *valid);
  extern DLLExport int frEntIsValid(ZkError err, FrEntIdExp *ent, int *valid);

  /* Two pairs of functions for serializing entities.
   * I don't know why there are two.
   */

  extern DLLExport void fr_EntFromPacked(FrEntIdExp *ent, void *buf);
  extern DLLExport void fr_EntToPacked(FrEntIdExp *ent, void *buf);

  extern DLLExport int frEntStore(ZkError err, FrEntIdExp *ent, void *buf, size_t bsz);
  extern DLLExport int frEntLoad(ZkError err, FrEntIdExp *ent, void *buf, size_t bsz);

  /* 
   * key hierarchy relationship functions
   */

  extern DLLExport int frEntMayRevokeKey(ZkError err, FrEntIdExp *revoker, FrEntIdExp *victim, int *allow);
  extern DLLExport int frEntGetSignerEntIdType(ZkError err, FrEntIdType *s, int isCryptKey, FrEntIdType eid);
  extern DLLExport int frEntIsKeySigningParent(ZkError err, FrEntIdExp *child, int isCryptKey, FrEntIdExp *parent);

  /* Debugging stuff */

  extern DLLExport void fr_EntSetRandom(FrEntIdExp *ent);

  extern DLLExport int frEntBufSnprint(ZkError err, void *ebuf, size_t esz, char *buf, size_t sz);
  extern DLLExport int frEntSnprint(ZkError err, char *buf, int sz, FrEntIdExp *ent);
  extern DLLExport int frEntPrint(ZkError err, FILE *fp, FrEntIdExp *ent);
  extern DLLExport int frEntBufPrint(ZkError err, FILE *fp, void *ebuf, size_t esz);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_ENTID_H_ */
