/*
 * $Id: fr_keyqry_fetchkey.h,v 1.6.20.4 2000/12/14 17:52:47 jean Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */
#ifndef FR_KEYQRY_FETCHKEY_H_
#define FR_KEYQRY_FETCHKEY_H_

#include <sys/types.h>

#ifndef WIN32
#include <sys/socket.h>
#include <netinet/in.h>
#else
#include <winsock2.h>
#endif

#include <string.h>
#include <errno.h>
#include "zks.h"
#include "zkNet.h"
#include "frCrypt.h"
#include "frEnt.h"
#include "zkError.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct fr_keyqry_host {
  int fd;
  struct sockaddr_in host;
} FrKeyQryHost;

/** 
 * Fills in a FrKeyQryQueryExp structure and tries to retrieve the key for
 * that query from the key query server.  This function will "block" until the
 * key is found and authenticated, key is not found, or the timeout (in
 * seconds).
 *
 * @param err
 * @param kq                    Key cache to store the keys
 * @param keyVers               Version of the key to look up
 * @param keyType               Type of key (signing or cryption)
 * @param entid                 Entity of the key to look up
 * @param keyHost               A sockaddr_in and the descriptor of 
 *                              the open unconnected UDP socket
 * @param retries               Number of retries
 * @param timeout               Max time to wait for all keys
 * @param key                   Public key (if found)
 * @return                      1 on success, 0 on failure
 */
DLLExport 
extern int frKeyQryFetchPubKey(ZkError err, FrKeyQry kq, unsigned int keyVers,
                               FrPubKeyType keyType, FrEntIdExp entid, 
                               FrKeyQryHost keyHost, int retries, int timeout, 
                               FrCryptPubKey *key);
/**  
 * The following function performs the same operation as above except that it provides
 * an interface that distinguishes between a key not found condition and an error. When
 * a key is not found, 1 is returned (instead of 0 in the above function) and found is 
 * set 0.
 */
DLLExport 
extern int frKeyQryFetchPubKeyWithStatus(ZkError err, FrKeyQry kq, unsigned int keyVers,
                                         FrPubKeyType keyType, FrEntIdExp entid, 
                                         FrKeyQryHost keyHost, int retries, int timeout, 
                                         FrCryptPubKey *key, ZkBool *found, ZkBool *valid);

DLLExport
extern int frKeyQryMultiFetch(ZkError err, FrKeyQry kq, FrKeyQryHost keyHost,
                              unsigned int retries, struct timeval timeout,
                              FrKeyQryQueryExp *qry, size_t nqrys, FrCryptPubKey **key);

DLLExport
extern int frKeyQryMultiFetch2(ZkError err, FrKeyQry kq, FrKeyQryHost keyHost,
                               unsigned int retries, struct timeval timeout,
                               FrKeyQryQueryExp *qry, size_t nqrys, FrCryptPubKey **key);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_KEYQRY_FETCHKEY_H_ */
