/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef WIN32
#  include <unistd.h>
#  include <sys/socket.h>
#  include <netinet/in.h>
#  include <arpa/inet.h>
#  include <netdb.h>
#  include <sys/time.h>
#  include <net/if.h>
#  include <stdio.h>
#  include <sys/ioctl.h>
#  include <stdio.h>
#  include <string.h>
#  ifdef ZK_SOLARIS
#    include <sys/sockio.h>
#    ifndef INADDR_NONE
#      define INADDR_NONE (unsigned long int) -1
#    endif
#  endif
#else
#  include <winsock2.h>
#  include <time.h>
#endif

#include "frLocalAipGetIP.h"

/* Don't want to add dependancy for true / false */
#define ZK_FALSE 0
#define ZK_TRUE 1

#ifndef WIN32

#define _PROC_NET_ROUTE_FNAME "/proc/net/route"

/* Fetch the interface configuration from the kernel. */

/* this is a tiny subset of a function from nettools ... your source for linux net-majik
 * This uses the largely undocumented socket ioctls for pulling interface config.
 *.--ol
 */
static int frLocalAipFetchIFAddr(struct ifreq *ifr, char *ifname)
{
    int fd;
    int rval = ZK_FALSE;

    /* IPv4 addresses */
    fd = socket(AF_INET, SOCK_STREAM, 0);

    if (fd >= 0) {
        strncpy(ifr->ifr_name, ifname, sizeof(ifr->ifr_name));
        ifr->ifr_addr.sa_family = AF_INET;
        if (ioctl(fd, SIOCGIFADDR, ifr) == 0) {
            rval = ZK_TRUE;
        } 
        close (fd);
	} 
    return rval;
}

/* returns the interface on which the default route resides 
 * returns ZK_TRUE for success, ZK_FALSE for failure 
 */
static int frLocalAipGetDefaultRouteIF( char *ifbuf, size_t ifbuflen )
{
    char buf[1024];
    FILE *fp;
    char ifname[128];
    char rtaddr[128];
    char gwaddr[128];
    int flags;
    int num;
    unsigned int i;
    fp = fopen(_PROC_NET_ROUTE_FNAME, "r");
    if (fp) {
        while ( fgets(buf, 1023, fp) ) 
        {
            /* This should be safe, we limit our buffer sizes */
            num = sscanf(buf, "%128s %128s %128s %X", ifname, rtaddr, gwaddr, &flags);
            if ( num < 4 ) continue;
            /* well Ok, this is a bit dumbass, we don't check the mask
             * or look at the address's value as anything other than a string
             * still, it's all that should be necessary --ol
             */
            if ( !(flags & 1) ) continue;
            for ( i = 0; i < strlen(rtaddr) && rtaddr[i] == '0'; i++ );
            if (i >= strlen(rtaddr) - 1) {
                strncpy(ifbuf, ifname, ifbuflen - 1);
                fclose(fp);
                return ZK_TRUE;
            }            
        }
    } 
    fclose(fp);
    return ZK_FALSE;
}

#endif /* WIN32 */

/**
 * Get my own IP address and port.
 *
 * @param host                  sockaddr_in to fill
 * @param port                  port number to store in sockaddr_in
 * @return                      ZK_TRUE on success, ZK_FALSE on failure
 */ 
int frLocalAipGetIP(struct sockaddr_in *host, int port)
{
    int rval = ZK_FALSE;

#ifdef WIN32
	char szBuffy[255];
	LPHOSTENT pHostEntry = NULL;
	gethostname(szBuffy, 254);
	pHostEntry = gethostbyname(szBuffy); /* audit: ok */
	memcpy (&host->sin_addr, pHostEntry->h_addr_list[0], 4);
    rval = ZK_TRUE;    
#else

    struct ifreq ifr;
    char ifBuf[128];

    if ( !frLocalAipGetDefaultRouteIF(ifBuf, 128) ) {
        rval = ZK_FALSE;
        goto try2;
    }

    if ( !frLocalAipFetchIFAddr(&ifr, ifBuf) ) {
        rval = ZK_FALSE;
        goto try2;
    } 
    host->sin_addr.s_addr = ((struct sockaddr_in*)&ifr.ifr_addr)->sin_addr.s_addr;
    rval = ZK_TRUE;
    /* we got the address from route table */
    goto out;

 try2:
    {
        struct hostent *phe;
        char myName[BUFSIZ];
    
        if (gethostname(myName, sizeof(myName) - 1) == -1) {
            rval = ZK_FALSE;
            goto out;
        }
        
        if ((phe = gethostbyname(myName))) { /* audit: ok */
            memcpy(&(host->sin_addr), phe->h_addr, phe->h_length);        
        } else if ((host->sin_addr.s_addr = inet_addr(myName)) == INADDR_NONE) {
            rval = ZK_FALSE;
            goto out;
        }
    }
    rval = ZK_TRUE;
 out:
#endif /* WIN32 */
    host->sin_port = htons(port);
    return rval;
}
