/* $Id: localAipStructs.h,v 1.18.10.3 2000/12/15 02:21:47 jean Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include <frLocalAip.h>

typedef
struct T_RouterInfo {
  struct sockaddr_in host;      /* Host IP address and port */
  FrCryptSym forwSymKey;        /* Forward symmetric key */
  FrCryptSym backSymKey;        /* Backward symmetric key */
  FrEntIdExp aipEntId;          /* AIPs Entity ID (for key lookups) */
  uint32_t aipId;               /* AIP ID number */
} *RouterInfo;

struct T_DH {
  FrPktDifHel dh;               /* Diffie-Hellman Key Exchange */
  int send;                     /* Send state, Send = 1, Recv = 0 */
};

/* 
 * modified Feb. 2000, Frank Foster
 * - move aciCtx from LocalAipCtx to FirstHop object
 */ 
typedef
struct T_FirstHops {
  FrCryptSym symKey[2];
  struct sockaddr_in host;
  int valid;
  size_t used;
  ZkACIContext aciCtx;          /* ACIs available for use */
} *FirstHops;

typedef 
struct T_RouterTableEntry {
  ZkACI aciIn;                  /* Incoming ACI */
  FrCryptMacKey macKey;         /* MAC for incoming/outgoing data */
  FrCryptSym symKey[2];         /* Link Encryption key to first router */
  int current;                  /* Current version of the symKey */
  uint32_t expiry;              /* Expiry time */
  int valid;                    /* Valid bit */
  ZkACI aciOut;                 /* Outgoing ACI */

  unsigned int prio;            /* Priority of the packets */

                                /* Router List */
  size_t numRouters;
  size_t maxRouters;
  RouterInfo router;            /* Array of routes */
  struct T_DH dh;               /* Diffie-Hellman stuff */

  FrCryptSig fhSigKey;          /* First Hop's Public Signing Key */

  struct fr_nym_auth_exp na;    /* Nym Authentication information */
  ZkACI aciExit;                /* The ACI chosen for the wormhole at the 
                                   exit funnel */
} *RouterTableEntry;

typedef
struct T_RouterTable {
  size_t curRoute;              /* Number of used slots */
  size_t maxRoutes;             /* Maximum size of table */
  RouterTableEntry route;       /* Routes */
  FirstHops fh;                 /* First hop addr and symKey */
} *RouterTable;

struct T_FrLocalAipCtx {
  struct sockaddr_in me;        /* My IP Address & Port */

  struct {                      /* Data handling */
    struct {
      FrPktAD   handler;        /* Packet handler */
      size_t    max_size;       /* Maximum possible data packet size */
    } packet;

    struct {
      unsigned long version;    /* Current data packet's protocol version */    
    } protocol;
  } data;

  struct {                      /* Route handling */
    RouterTable context;        /* Router Table Context */

    struct {
      FrPktRC   handler;        /* Packet handler */
      size_t    max_size;       /* Maximum possible rte create packet size */
    } packet;                   

    struct {
      unsigned long version;    /* Current route create packet's protocol version */    
    } protocol;
  } route;

                                /**
                                 * Pointer to key query cache.  Do not
                                 * initialize or free (read-only)
                                 */ 
  FrKeyQry kq;
  FrNetInfo ni;                 /* 
                                 * Network information object. Do not
                                 * initialize or free (read-only)
                                 */  

  time_t freedomDelta;          /* Delta Time to the Freedom network time */
};
