/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

# include "frMailAccount.h"
# include <stdio.h>
# include <stdlib.h>
# include <unistd.h>

# define QMAIL_SUCCESS_CONTINUE        (0)
# define QMAIL_SUCCESS_ABORT           (99)
# define QMAIL_FAILURE_PERMANENT       (100)
# define QMAIL_FAILURE_TEMPORARY       (111)

/*
 * Converts a string to a number, returning whether or not the input
 * string was valid.
 */

static int
validate_number(
    const char * str,
    uint32_t *   num )
{
    char *   ptr   = NULL;
    *num = strtoul(str,&ptr,10);

    return (*ptr == '\0');
}

/*
 * The program main line
 */

int main(int argc, char * argv[])
{
    uint32_t     dflt_limit = 0;
    uint32_t     num_rcpts  = 1;
    ZkError      error      = NULL;
    const char * prog       = strrchr(argv[0], '/');
    char *       home       = NULL;
    ZkBool       is_allowed = ZK_FALSE;
    int          ret_code   = QMAIL_FAILURE_TEMPORARY;

    prog = ((prog == NULL) ? argv[0] : ++prog);
    
    if ((argc < 3)
        || (argc > 4)
        || (!validate_number(argv[2], &dflt_limit))
        || ((argc == 4) && (!validate_number(argv[3], &num_rcpts))))
    {
        fprintf( stderr,
                 "%s: pass account directory, default limit, and (optional) # of recipients\n",
                 prog );
        goto cleanup;
    }

    home = argv[1];

    if (home[0] != '/')
    {
        fprintf( stderr,
                 "%s: account path is not absolute!\n",
                 prog );
        goto cleanup;
    }

    if (!zkErrorNew(&error))
    {
        fprintf(stderr, "%s: cannot allocate error object\n", prog);
        goto cleanup;
    }

    if (!frMailAccount_AllowedToSend( error,
                                      home,
                                      dflt_limit,
                                      num_rcpts,
                                      &is_allowed))
    {
        char message[1024];
        zkErrorMessage(error, message, sizeof(message)-1);
        fprintf(stderr, "%s: %s\n", prog, message);
        goto cleanup;
    }


    ret_code = is_allowed ? QMAIL_SUCCESS_CONTINUE : QMAIL_SUCCESS_ABORT;

cleanup:

    if (error)
    {
        zkErrorFree(&error);
    }
    
    return (ret_code);
}
