/* ==========================================================================
 * $Id: frMailAccount_Tagline.c,v 1.1.10.1 2000/10/11 13:18:20 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *
 * Copyright (c) 2000 Zero-Knowledge Systems.  All rights reserved.
 */

# include "frMailAccount.h"
# include "frMailAccount_utils.h"
# include "zkFile.h"
# include "zkAlloc.h"
# include <stdio.h>
# include <errno.h>

# define DEFAULT_TAGLINE_OPTION    (ZK_TRUE)
# define TAGLINE_SETTINGS_FILE     ".tagline"


/* --------------------------------------------------------------------------
 *
 * Looks in the specefied account to see if the system should add taglines
 * to internet mail sent from the account.
 *
 */
int
frMailAccount_HasTagline(
    ZK_INOUT ZkError   error,
    const char *       account_path,
    ZkBool *           has_tagline )
{
    const char * const func          = "frMailAccount_HasTagline";
    char *             file_name     = NULL;
    size_t             file_name_len = 0;
    const char *       seperator     = "";
    char *             setting       = NULL;
    size_t             setting_size  = 0;
    int                result        = ZK_FAILURE_CODE;

    ZK_CHECK_CONSTRAINT( account_path != NULL );
    ZK_CHECK_CONSTRAINT( has_tagline != NULL );

    (*has_tagline) = DEFAULT_TAGLINE_OPTION;

    /*
     * Figure out how long the path to the tagline options file is then
     * allocate space for it (on the stack).
     */

    file_name_len = strlen( account_path ) + strlen( TAGLINE_SETTINGS_FILE );

    if (account_path[file_name_len-1] != '/')
    {
        seperator = "/";
        ++file_name_len;
    }

    file_name = alloca( file_name_len + 1 );

    ZK_CHECK_CONSTRAINT( file_name != NULL );

    /*
     * Construct the path to the settings file
     */

    if (-1 == snprintf( file_name,
                        file_name_len + 1,
                        "%s%s%s",
                        account_path,
                        seperator,
                        TAGLINE_SETTINGS_FILE))
    {
        zkErrorSet(error, func, ZKERR, ZKMMAILACCOUNT, ZKERROR,
                   "failed to construct tagline settings file path");
        goto ZK_ATTEMPT_FAILED;
    }

    /*
     * Load the tagline settings file.  If the file is not there,
     * then continue with the default tagline setting.
     */

    if (!zkFileLoad( error, file_name, &setting, &setting_size ))
    {
        if (zkErrorCode(error) == ENOENT)
        {
            zkErrorClear(error);
            result = ZK_SUCCESS_CODE;
        }

        goto ZK_ATTEMPT_FAILED;
    }

    /*
     * The file was present, so check to see what it says
     */

    if ( (0 == strncasecmp("off", setting, 3))
         || (0 == strncasecmp("no", setting, 2))
         || (0 == strncasecmp("false", setting, 5))
         || (0 == strncasecmp("0", setting, 1)) )
    {
        (*has_tagline) = ZK_FALSE;
    }
    else
    {
        (*has_tagline) = ZK_TRUE;
    }

    result = ZK_SUCCESS_CODE;

ZK_ATTEMPT_FAILED:

    /*
     * Cleanup time ...
     */

    if (setting != NULL)
    {
        zkMemFree(NULL, (void**)&setting);
    }

    return (result);
}


