/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
 *  
 *  
 *                  Cacophonix's Mail Authentication library
 *  
 *  
 */

#ifndef FRMAILAUTH_H_
#define FRMAILAUTH_H_
  
#include "zks.h"
#include "zkError.h"
#include "frCrypt.h"
#include "frEnt.h"
#include "zkClock.h"


typedef struct fr_mail_auth *FrMailAuth;


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  DLLExport extern size_t frMailAuthMacLength(void);
  DLLExport extern size_t frMailAuthCteSize(void);
  DLLExport extern int frMailAuthSize(ZkError err, FrMailAuth cert, size_t *csz);
  
  DLLExport extern int frMailAuthNew(ZkError err, FrMailAuth *cert);
  DLLExport extern int frMailAuthNewFromBuf(ZkError err, FrMailAuth *cert, const void *buf, size_t bsz);
  DLLExport extern int frMailAuthFree(ZkError err, FrMailAuth *cert);
  
  DLLExport extern int frMailAuthGetEnt(ZkError err, FrMailAuth cert, FrEntIdExp *ent);
  DLLExport extern int frMailAuthSetEnt(ZkError err, FrMailAuth cert, FrEntIdExp *ent);
  DLLExport extern int frMailAuthGetEntSize(ZkError err, FrMailAuth cert, size_t *esz);

  DLLExport extern int frMailAuthGetVersion(ZkError err, FrMailAuth cert, uint16_t *vers);
  DLLExport extern int frMailAuthSetVersion(ZkError err, FrMailAuth cert, uint16_t vers);

  DLLExport extern int frMailAuthGetExpiration(ZkError err, FrMailAuth cert, ZkClock *exp);
  DLLExport extern int frMailAuthSetExpiration(ZkError err, FrMailAuth cert, ZkClock exp);

  DLLExport extern int frMailAuthGetNymType(ZkError err, FrMailAuth cert, uint8_t *type);
  DLLExport extern int frMailAuthSetNymType(ZkError err, FrMailAuth cert, uint8_t type);

  DLLExport extern int frMailAuthGetMailLimit(ZkError err, FrMailAuth cert, uint32_t *limit);
  DLLExport extern int frMailAuthSetMailLimit(ZkError err, FrMailAuth cert, uint32_t limit);

  DLLExport extern int frMailAuthGetVolLimit(ZkError err, FrMailAuth cert, uint32_t *limit);
  DLLExport extern int frMailAuthSetVolLimit(ZkError err, FrMailAuth cert, uint32_t limit);

  DLLExport extern int frMailAuthGetSizeLimit(ZkError err, FrMailAuth cert, uint32_t *limit);
  DLLExport extern int frMailAuthSetSizeLimit(ZkError err, FrMailAuth cert, uint32_t limit);

  DLLExport extern int frMailAuthGetCrossLimit(ZkError err, FrMailAuth cert, uint32_t *limit);
  DLLExport extern int frMailAuthSetCrossLimit(ZkError err, FrMailAuth cert, uint32_t limit);

  DLLExport extern int frMailAuthToBuf(ZkError err, FrMailAuth cert, void **buf, size_t *bsz);
  
  DLLExport extern int frMailAuthMac(ZkError err, FrMailAuth cert, FrCryptMacKey key);
  DLLExport extern int frMailAuthCheckMac(ZkError err, FrMailAuth cert, FrCryptMacKey key, int *ok);
  DLLExport extern int frMailAuthCheckOwner(ZkError err, FrMailAuth cert, FrEntIdExp *ent, int *ok);
  DLLExport extern int frMailAuthCheckExpiration(ZkError err, FrMailAuth cert, ZkClock clk, int *ok);
  DLLExport extern int frMailAuthCheckOwnerAndMac(ZkError err, FrMailAuth cert, FrEntIdExp *ent, FrCryptMacKey key, int *ok);
    
  DLLExport extern int frGenSharedNymMailMacKey(ZkError err, void *secret, size_t ssz, FrEntIdExp *nym, FrCryptMacKey mkey);

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* FRMAILAUTH_H_ */
