/* $Id: auth-common.cpp,v 1.7.4.7 2000/12/14 17:52:52 jean Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include <unistd.h>

#include <iostream>
#include <strstream>
#include <fstream>
#include <string>

#include <frMailAccount.h>

#include <zkFile.h>

#include "auth-common.hh"

using namespace std;



const char* qmailCtlDirDefault = "/freedom/qmail/control";
const char* qmailBinDirDefault = "/freedom/qmail/bin";



bool legal(string &str) {
  const string legals("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_");
  for (size_t i = 0; i < str.size(); ++i) {
    if (legals.rfind(str[i]) == string::npos) return false;
  }
  return true;
}

string guid(string &nym) {
  ostrstream s;
  char buf[1024];
  memset(buf,0,1024);
  gethostname(buf,1023);
  static int i = 0;
  s << hex << time(NULL) << '-' << string(buf) << '-' 
    << nym << '-' << getpid() << '-' << ++i << '\0';
  return s.str();
}

string slurpConfigFile(const char *file) {

  /* Note: this function will return a string containing everything up
     to but not including the first whitespace in file. */

  ifstream cfg(file);
  if (!cfg) { 
    string death("failed reading config file: ");
    death += file;
    die(const_cast<char *>(death.c_str()));
  }
  string tmp;
  cfg >> tmp;
  return tmp;
}

void die(const char *msg) {
    // LOG AUDIT: necessary?
    cerr << "-ERR " << msg << endl;
    _exit(1);
}

void dietmp(const char *msg) {
    // LOG AUDIT: necessary?
    cerr << "-ERR (temporary) " << msg << endl;
    _exit(111);
}

void dieOnError(ZkError err_, const char *msg) {
    if (zkErrorType(err_)) {
        // LOG AUDIT: are those 3 lines necessary?
        char buf[1024]; 
        zkErrorMessage(err_,buf,1024);
        cerr << "-ERR " << msg << " # " << buf << endl;
        _exit(1);
    }
}

void loadMacKey(ZkError err_, FrCryptMacKey &mkey, const char* file) {

  size_t key_size;
  char *key = 0;

  zkFileLoad(err_, file, &key, &key_size);
  dieOnError(err_, "failed loading mac file");

  frCryptMacKeyNew(err_, &mkey); 
  dieOnError(err_, "failed building mac key");

  frCryptMacKeyLoad(err_, mkey, key, key_size);
  dieOnError(err_, "loading mac key");
}

void nym2Entity(ZkError err_, const string &nym, FrEntIdExp &ent) {
  frEntSetFromStr2(err_, &ent, "nym", nym.c_str()); 
  dieOnError(err_, "setting up entity");
}

void Base64(ZkError err_, string &str) {
  unsigned char result[((str.size() + 2) / 3) * 4 + 1];
  zkConvertToBase64(err_,(const unsigned char *)(str.c_str()),result,(size_t)(str.size()));
  dieOnError(err_, "failed to base64 encode data");
  str = string((const char *)result);
}

void unBase64(ZkError err_, string &str) {
  unsigned char result[str.size()];
  size_t len;
  zkConvertFromBase64(err_,(const unsigned char *)(str.c_str()),result,&len);
  dieOnError(err_, "failed to decode base64 data");
  str = string((const char *)result, len);
}


string calculateUserDir(ZkError err_, string &root, string &user) {
  size_t sz = root.size() + user.size() + 512; 
  char buf[sz];
  memset(buf,0,sz);
  frMailAccount_GetPath(err_, root.c_str(), user.c_str(),buf,sz);
  dieOnError(err_,"failed to lookup nym account");
  return string(buf);  
}
