/*
 * $Id: checkpassword.cpp,v 1.6.4.11 2000/12/14 17:52:52 jean Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *
 * simple checkpassword replacement based on lookup-less shared secret mac 
 *
 * Copyright (c) 2000 Zero-Knowledge Systems Inc.
 */

#include <string>
#include <iostream>
#include <stdio.h>
#include <pwd.h>
#include <errno.h>
#include <unistd.h>
#include <grp.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stdlib.h>

#include <zkTypes.h>
#include <zkIntTypes.h>
#include <zkErrdef.h>
#include <zkError.h>
#include <frMailAuth.h>
#include "frMailBlockList_server.h"
#include <zkBase64.h>
#include "zkCStr.h"

#include "auth-common.hh"

#define REVOCATION_LIST_PATH       "/freedom/qmail/control/revoked.cdb"

using namespace std;

static ZkError        err_; // this is a straight-line program 
extern int            errno;
extern char       **  environ;  // "const" clashes with unistd.h

static int prot_uid(const int uid)
{
    return setuid(uid); /* audit: ok */
}

static int prot_gid(const unsigned int gid)
{
    // _should_ be redundant, but on some systems it isn't 

    if (setgroups(1,&gid) == -1) return -1;
    return setgid(gid);  /* audit: ok */
}


static void getInput(string &nym, string &pass, string &timestamp)
{
    // input consists of an already-decoded username, a null,
    // a base64-encoded cert, a null, and a timstamp.
    char c;
    while ((read(3,&c,1) == 1) && c != '\0') nym += c;
    while ((read(3,&c,1) == 1) && c != '\0') pass += c;
    while ((read(3,&c,1) == 1) && c != '\0') timestamp += c;
    close(3);
    unBase64(err_, pass);
}

static FrMailAuth
checkCert(
    const string &pass,
    FrEntIdExp &ent,
    FrCryptMacKey &mkey)
{
    FrMailAuth cert;
    ZkClock clk;
    frMailAuthNewFromBuf(err_, &cert, const_cast<char *>(pass.c_str()), pass.size()); 
    dieOnError(err_, "failed to build mail auth from buffer");
    int valid = 0;
    int exp = 0;
  
    /* check validity */
    frMailAuthCheckOwnerAndMac(err_, cert, &ent, mkey, &valid); 
    dieOnError(err_, "failed while checking validity of authentication token");

    /* check expiration */  
    clk = zkClockNow();
    frMailAuthCheckExpiration(err_, cert, clk, &exp); 
    dieOnError(err_, "failed while checking expiration of authentication token");
    frCryptMacKeyFree(err_, &mkey); 
    dieOnError(err_, "failed to free MAC key");
    if ( ! (valid && exp) ) _exit(1);
    return cert;
}


static int
createVariableStr(
    const char **    buffer,
    const string &   variable,
    uint32_t         value,
    const string &   modifier = "" )
{
    char * new_buffer = (char*) malloc( 64 );
    
    if (new_buffer == NULL)
    {
        die(string(variable + " could not be allocated").c_str());
    }
    
    *buffer = new_buffer;
    
    return snprintf( new_buffer, 64,
                     "%s=%u%s",
                     variable.c_str(),
                     value,
                     modifier.c_str() );
}
                         

static void
ensureUserExec(
    const char*  path,
    const char*  nym,
    FrMailAuth   cert )
{   
    uint8_t      type;
    uint32_t     limit;
    int          numenv = 0;
    const char** newenv = NULL;
    int          i      = 0;
    const char * cmd    = strrchr(path, '/');

    cmd =  (cmd == NULL) ? path : (cmd + 1);

    if (path[0] != '/')
    {
        die("must specify absolute path to ensure-user");
    }

    /*
     * The number of new variables to add to the environment.
     *
     * IMPORTANT: if you add more variables you must modify this value.
     */

    enum { NEW_ENV_VAR_CNT = 4 };
    
    /*
     * Count the current number of environment variables
     */

    while (environ[numenv])
    {
        ++numenv;
    }
    
    /*
     * Allocate a new environment table, copying the current variables
     * over.
     */

    newenv = (const char**) malloc( (numenv + NEW_ENV_VAR_CNT + 1)
                                    * sizeof(char*) );

    if (!newenv)
    {
        _exit(111);
    }
    
    for(i = 0; i < numenv; i++) {
        newenv[i] = environ[i];
    }

    /*
     *  Add the nym type
     */
    
    frMailAuthGetNymType(err_, cert, &type);
    dieOnError(err_, "failed fetching nym type");
    createVariableStr(&newenv[i++], FRMAILENV_NYM_TYPE, type);

    /*
     * Add the limit for the number of mails sent
     */

    frMailAuthGetMailLimit(err_, cert, &limit);
    dieOnError(err_, "failed fetching mail limit");
    createVariableStr(&newenv[i++],FRMAILENV_SEND_LIMIT,limit);

    /*
     * Add the crosspost limit
     */

    frMailAuthGetCrossLimit(err_, cert, &limit);
    dieOnError(err_, "failed fetching crosspost limit");
    createVariableStr(&newenv[i++],FRMAILENV_CROSSPOST_LIMIT,limit);

    /*
     * Add the limit for the volume of sent mail in kilobytes
     */

    frMailAuthGetVolLimit(err_, cert, &limit);
    dieOnError(err_, "failed fetching volume limit");
    createVariableStr(&newenv[i++],FRMAILENV_VOLUME_LIMIT,limit);

    /*
     * Add the limit for the volume of stored mail (quota) in kilobytes
     */

    frMailAuthGetSizeLimit(err_, cert, &limit);
    dieOnError(err_, "failed fetching mailbox quota");
    createVariableStr(&newenv[i++],FRMAILENV_STORAGE_QUOTA,limit);

    /*
     * Replace the old environment with the new one
     */

    newenv[i] = 0;
    environ = newenv;

    /*
     * Run ensure-user
     */

    execl(path, cmd, nym, NULL);
    perror("-ERR (temporary) Failed to exec ensure-user: ");
    _exit(111);
}


static void
ensureUserFork(
    const char*  cmd,
    const char*  nym,
    FrMailAuth  cert)
{
    pid_t res = fork();
    int status;
  
    switch(res)
    {

    case -1:
        /*
         * Error
         */
        die("fork failed");
        break;

    case 0:
        /*
         * Child Process
         */
        ensureUserExec(cmd, nym, cert);
        break;

    default:
        /*
         * Parent Process
         */
        if(wait(&status) == -1)
        {
            perror("-ERR child exited abnormally # ");
            _exit(1);
        }
    
    
        if(WIFEXITED(status) == 0) {
            perror("-ERR child exited abnormally # ");
            _exit(1);
        }
   
        if(WEXITSTATUS(status) != 0) {      
            perror("-ERR ensure-user failed # ");
        } 
        break;
    }

    return;
}


void checkIfBlocked(const string & nym)
{
    FrMailBlockList blocklist  = NULL;
    ZkBool          is_blocked = ZK_TRUE;

    if (!frMailBlockList_Open( err_,
                               &blocklist,
                               REVOCATION_LIST_PATH ))
    {
        /*
         * The file not being there isn't a problem!
         */
        if (zkErrorCode(err_) == ZKENOTFOUND)
        {
            zkErrorClear(err_); 
            return;
        }
    }

    dieOnError(err_, "failed to open revocation list");

    frMailBlockList_LookupSender(err_, blocklist, nym.c_str(), &is_blocked);
    dieOnError(err_, "failed to look up nym");
    
    frMailBlockList_Close(NULL, &blocklist);

    if (is_blocked)
    {
        
        die("access has been revoked");
    }
}


int main(int argc, char **argv)
{
 
    FrCryptMacKey mkey;
    FrEntIdExp ent;
    string nym,pass,timestamp;
    FrMailAuth cert = NULL;
    
    
    char* qmailCtlDir = (char*) malloc(sizeof(char) * (strlen(qmailCtlDirDefault)+1));
    zkCStrlcpy(qmailCtlDir, qmailCtlDirDefault, strlen(qmailCtlDirDefault)+1);
    
    
    zkErrorNew (&err_);
    getInput(nym,pass,timestamp);
    
    string mailMacKeyFile(string(qmailCtlDir)+ "/mailmackey");
    
    loadMacKey(err_, mkey, mailMacKeyFile.c_str());   
    nym2Entity(err_, nym,ent);
    cert = checkCert(pass,ent,mkey); 

    checkIfBlocked(nym);
     
    if (argc == 1) return 0;
    
    // else we have a subprocess to run:
    
    // now enter into the nym's selected environment
    // if (!legal(nym)) die("illegal nym submitted");
    
    
    ensureUserFork("/freedom/qmail/bin/ensure-user", nym.c_str(), cert);

    string uname("alias"); // all mailboxes owned by 1 uid

    string mailrootConfigFile(string(qmailCtlDir) + "/mailroot");
    string mailroot(slurpConfigFile(mailrootConfigFile.c_str())); // based on a single mail root

    string home = calculateUserDir(err_, mailroot, nym); // in some weird subdirectory
    string shell("/nonexistent"); // and they can't login

    struct passwd *pw = getpwnam(uname.c_str());
    if (!pw) {
        if (errno == ETXTBSY) _exit(111);
        _exit(1);
    }

    if (prot_gid((int) pw->pw_gid) == -1) _exit(1);
    if (prot_uid((int) pw->pw_uid) == -1) _exit(1);
    if (chdir(home.c_str()) == -1) _exit(111);

    /* augment environment and exec given program */
    int numenv = 0;
    while (environ[numenv]) ++numenv;
    const char **newenv = (const char **) malloc((numenv + 4) * sizeof(char *));
    if (!newenv) _exit(111);

    for (int i = 0;i < numenv;++i) newenv[i] = environ[i];

    string env_user = string("USER=") + pw->pw_name;
    string env_home = string("HOME=") + home;
    string env_shell = string("SHELL=") + shell;

    newenv[numenv++] =  env_user.c_str();
    newenv[numenv++] =  env_home.c_str();
    newenv[numenv++] =  env_shell.c_str();
    newenv[numenv] = NULL;
    environ = newenv;

    execv(argv[1],argv + 1);

    fprintf(stderr, "-ERR could not execute `%s'\n", argv[1]);
    fflush(stderr);

    _exit(111);
}
 
