//
// $Id: genMacKey.cpp,v 1.3.10.4 2000/12/14 17:52:52 jean Exp $
// 
// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom(R) code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

// 
//
// Generates the shared Mac Key, given a secret and a nym name
//

#include <string>
#include <stdio.h>
#include <fstream>
#include <streambuf>
#include <iostream>
#include <zkError.h>
#include "frMailAuth.h"

using namespace std;

#define ensure(x) if(!x) { cerr << "failed in library call\n"; exit(1);}

static ZkError err_; // this is a straight-line program 

int main() 
{
  string nymname, secret;
  FrCryptMacKey mkey;
  FrEntIdExp ent;
  char mkeybuf[FRCRYPT_MACKEY_SIZE];
  
  zkErrorNew(&err_);
  
  // reading the inputs (nym name on fd3, secret on fd4)
  char c;
  istream nym(new std::filebuf(3,"nym",ios_base::in));
  istream scrt(new std::filebuf(4,"secret",ios_base::in));
  while(nym.get(c)) nymname += c;
  while(scrt.get(c)) secret += c;
  
  if ( ! strcmp(secret.c_str(),"")) {
    cerr << "empty secret\n";
    exit(1);
  }
  if ( ! strcmp(nymname.c_str(),"")) {
    cerr << "empty nymname\n";
    exit(1);
  }
  
  ensure(frCryptMacKeyNew(err_, &mkey));
  ensure(frEntSetFromStr2(err_, &ent, "nym", nymname.c_str()));
  ensure(frGenSharedNymMailMacKey(err_, const_cast<char *>(secret.c_str()), secret.size(), &ent, mkey));
  ensure(frCryptMacKeyStore(err_, mkey, mkeybuf, FRCRYPT_MACKEY_SIZE));
  
  cout << string(mkeybuf,FRCRYPT_MACKEY_SIZE);
  
  ensure(frCryptMacKeyFree(err_, &mkey));
  ensure(zkErrorFree(&err_));
  
  exit(0);
}
