/*
 *  $Id: frmailmsg.h,v 1.6.12.2 2000/10/11 13:18:27 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRMAILMSG_H_
#define FRMAILMSG_H_

#include "zkError.h"
#include "frCrypt.h"
#include "frEnt.h"


typedef struct fr_mailmsg *FrMailMsg;


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  extern DLLExport int frMailMsgNew(ZkError err, FrMailMsg *msg);
  extern DLLExport int frMailMsgFree(ZkError err, FrMailMsg *msg);
  extern DLLExport int frMailMsgGetAuthHeaderName(ZkError err, const char **name);
  extern DLLExport int frMailMsgGetEnvSigHeaderName(ZkError err, const char **name);
  extern DLLExport int frMailMsgGetContentHeaderName(ZkError err, const char **name);
  extern DLLExport int frMailMsgGetContentBodyName(ZkError err, const char **name);
  extern DLLExport int frMailMsgGetContentBounceName(ZkError err, const char **name);

  extern DLLExport int frMailMsgSetSenderEmail(ZkError err, FrMailMsg m, const char *email, size_t emailsz);
  extern DLLExport int frMailMsgGetSenderEmail(ZkError err, FrMailMsg m, char **email, size_t *emailsz);
  extern DLLExport int frMailMsgCreateWithMac(ZkError err, FrMailMsg m, FrCryptCrypt enc, FrCryptMacKey mac, FrEntIdExp *sender, const char *bounce, size_t bsz, const char *msgtxt, size_t msz);
  extern DLLExport int frMailMsgCreateWithSig(ZkError err, FrMailMsg m, FrCryptCrypt enc, FrCryptSig sig, const char *bounce, size_t bsz, const char *msgtxt, size_t msz);
  extern DLLExport int frMailMsgGive(ZkError err, FrMailMsg m, char **msgtxt, size_t *mlen);
  extern DLLExport int frMailMsgSetFromHeader(ZkError err, FrMailMsg m, FrCryptCrypt enc, void *hdr, size_t hsz);
  extern DLLExport int frMailMsgGetSender(ZkError err, FrMailMsg m, FrEntIdExp *sender);
  extern DLLExport int frMailMsgMacVerify(ZkError err, FrMailMsg m, FrCryptMacKey mac, int *valid);
  extern DLLExport int frMailMsgSigVerify(ZkError err, FrMailMsg m, FrCryptSig sig, int *valid);
  extern DLLExport int frMailMsgHashVerify(ZkError err, FrMailMsg m, const char *msgtxt, size_t mtsz, int *valid);
  extern DLLExport int frMailMsgDecrypt(ZkError err, FrMailMsg m, void *encmsg, size_t emsz, void **origmsg, size_t *omsz);

#define FRMAIL_ENV_SIG_SIZE 1024

  extern DLLExport int frMailEnvelopeAddrSign(ZkError err, 
					      FrCryptSig sigKey,
					      const char* env_sender, 
					      const char* env_recip, 
					      char sig[FRMAIL_ENV_SIG_SIZE]);
  
  extern DLLExport int frMailEnvelopeAddrSigVerify(ZkError err, 
						   FrCryptSig sigKey,
						   const char* env_sender, 
						   const char* env_recip, 
						   char hdr_sig[FRMAIL_ENV_SIG_SIZE],
						   int* ok);

  extern DLLExport int frMailEnvelopeGetSigVersion(ZkError err, 
						   const char *hdr_sig, 
						   unsigned int *vers);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRMAILMSG_H_ */

