// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include <string>
#include <zkBase64.h>
#include <iostream>

using namespace std;

static ZkError err_; // this is a straight-line program 

static inline void Base64(string &str) {
  unsigned char result[((str.size() + 2) / 3) * 4 + 1];
  zkConvertToBase64(err_,(const unsigned char *)(str.c_str()),result,(size_t)(str.size()));
  str = string((const char *)result);
}

static inline void unBase64(string &str) {
  unsigned char result[str.size()];
  size_t len;
  zkConvertFromBase64(err_,(const unsigned char *)(str.c_str()),result,&len);
  str = string((const char *)result, len);
}


// 
// this is a simple program which reads a normalized rfc 822 message on stdin,
// scans backwards from the end of the message until it hits a blank
// crlf-terminated line, then outputs the base64-decoding of the string between
// the blank line and the end of input. this extracted string is the
// self-encrypted "bounce message" attached to nym-sent encrypted emails,
// containing a failure delivery status notification only the sending nym can
// read.
//


int main() {
  zkErrorNew (&err_);
  string str, sep("\r\n\r\n");
  char c;
  while (cin.get(c)) {str += c;}
  string::size_type pos = str.rfind(sep);
  if (pos == string::npos) {
    cout << str;
  } else {
    pos += sep.size();
    string bounce(str.substr(pos, str.size() - pos));
    unBase64(bounce);
    cout << bounce;  
  }
}
