// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include <string>
#include <iostream>

using namespace std;

//
// this program performs a simple string transformation on its input. it assumes
// it is being fed email messages composed of lf-terminated header lines which
// are "too long" for rfc822, followed by a line containing only an lf, followed
// by lf-terminated body lines which are also too long for 822. it spits out
// header lines wrapped to 72 characters, with lines following the first header
// indented by one space, as specified by 822, then the blank lf-terminated
// line, then message body lines also wrapped to 72 columns but not
// indented. i/o is just stdin/stdout.
//
//
// NOTE: this deviates _slightly_ from 822 insofar as it treats lf as a line
// terminator rather than the "correct" crlf. this is because qmail makes the
// assumption that all unix text processing is done strictly in terms of
// lf-terminated lines, and that the qmail suite itself ought to be responsible
// for performing the crlf correction when speaking smtp or pop3. so much so
// that qmail doesn't even check to see if you happen to have done the
// correction yourself, so it winds up spitting out lines ending with crcrlf
// (crlf-delimited line, assumed lf-terminator removed, crlf appended) when you
// try to pick up your mail thru pop. obviously this is wrong, so we're playing
// dumb and pretending we're a normal unix text utility.
//

int main() {
  char c, last = '\0';
  bool inheader = true;
  char lf = '\n';
  unsigned long len = 0;
  while(cin.get(c)) {    

    // decide if we're still in a header
    if (c == lf) {
      len = 0;
      if (last == lf) inheader = false;
    }
    last = c;

    // wrap lines
    if (len == 72) {
      if (inheader) {
	cout << lf << ' ' << c; len = 2;
      } else {
	cout << lf << c; len = 1;
      }
    } else {
      cout.put(c); len += (c == lf ? 0 : 1); 
    }
  }
}

