/*
 * $Id: frMaipUtil.c,v 1.7.32.3 2000/12/14 17:52:57 jean Exp $
 */


static const char * module_id_ = "$Id: frMaipUtil.c,v 1.7.32.3 2000/12/14 17:52:57 jean Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <zkError.h>
#include <zkBase64.h>
#include <frCrypt.h>
#include <frSec.h>
#include "frNetInfo.h"
#include "frMaip.h"

/*
 * This is quite incomplete at this time
 */

DLLExport int
frMaipGenerateMessageID (
  ZkError err ,
  unsigned char * buf ,
  size_t bsz )
{
  static const char * digit = 
        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-.";

  zkErrorClear(err);
  if (buf) {
                /* 0       1       2       3       4       5       6        */
                /* 01234567012345670123456701234567012345670123456701234567 */
    if (zkRandGetData(err,buf,bsz - 1)) {
      size_t n = 0;
      for (n = 0; n < (bsz - 1); n++) {
        buf[n] = digit[buf[n] % 64];
      }
      buf[bsz -1 ] = (char) 0;
    } else {
      return 0;
    }

    return 1;
  }
  zkErrorSet(err,"frMaipGenerateMessageID",ZKERR,ZKMFRMAIP,ZKENULLPTR,NULL);
  return 0;
}

/*
 * 
 */

DLLExport int
frMaipProcessMessageID (
  ZkError err ,
  char * raw,  /* Raw message ID */
  size_t rsz,  /* Size of the message ID */
  char ** buf ,   /* Returns a NUL terminated allocated string buffer */
  FrCryptCrypt crypt ,
  FrCryptSig sig )
{
#define MESGID_SZ 64
  const char * func = "frMaipProcessMessageID";
  char * encr = NULL;
  char * sig1 = NULL;
  char * sig2 = NULL;
  size_t esz = 0;
  size_t ssz1 = 0;
  size_t ssz2 = 0;
  int return_code = 0;

  (void) module_id_; /* Get rid of the compile warning */
  zkErrorClear(err);
  if (buf && raw) {

    if (crypt && sig) {
      if (!frSecPubKeyEncryptData(err,crypt,FRCRYPT_SYMALG_BLOWFISH,raw,rsz,
                                  (void **) &encr,&esz)) {
        goto finish;
      }
      if (!frSecSignData(err,sig,encr,esz,
                         (void **) &sig1, &ssz1)) {
        goto finish;
      }
      if (!frSecSignData(err,sig,sig1,ssz1,
                         (void **) &sig2, &ssz2)) {
        goto finish;
      }
      
      *buf = (char *) malloc(sizeof(char) * ((ssz2 * 4) / 3 + 5));
      if (*buf == NULL) {
        zkErrorSet(err,func,ZKERR,ZKMFRMAIP,ZKENOMEM,"malloc");
        goto finish;
      }

      if (!zkConvertToBase64(err,sig2,*buf,ssz2)) {
        goto finish;
      } 
      return_code = 1; 

    } else {
      *buf = (char *) malloc(sizeof(char) * ((rsz * 4) / 3 + 5));
      if (*buf == NULL) {
        zkErrorSet(err,func,ZKERR,ZKMFRMAIP,ZKENOMEM,"malloc");
        goto finish;
      }
      if (!zkConvertToBase64(err,raw,*buf,rsz)) {
        goto finish;
      } 
      return_code = 1;
    }

finish:
    if (encr) {
      free(encr);
    }
    if (sig1) {
      free(sig1);
    }
    if (sig2) {
      free(sig2);
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMFRMAIP,ZKENULLPTR,NULL);
  return 0;
}

