#ifndef FR_NETINFO_H_
#define FR_NETINFO_H_
/* $Id: frNetInfo.h,v 1.36.8.8 2000/12/14 17:52:59 jean Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */



#include "zks.h"

#include "zkipClient.h"
#include "frKeyQry.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkList.h"
#include "zkclientconsole.h"
#include "frEnt.h"
#include "frNiqs.h"
#include "frTokenType.h"


/**
 * Network information object.  Holds all information about the Freedom
 * network entities.  This includes key servers, AIPs, MAIPs, ...
 */
typedef struct T_FrNetInfo *FrNetInfo;

#ifndef WIN32
#include "netinfo_child.h"
#endif

/**
 * Network information data object.  For a given entity information like
 * IP address and port numbers of the servers, ID, entity type/name, ...  
 * See the FrNetInfoDatumGet...() functions.
 */
typedef struct T_FrNetInfoDatum *FrNetInfoDatum;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  /**
   * Create a new FrNetInfo object.
   */
  DLLExport extern 
  int frNetInfoNew(ZkError err,
                   FrNetInfo *ni); /* NetInfo Object */

  /**
   * Free the data from the FrNetInfo object.
   */
  DLLExport extern
  int frNetInfoFree(ZkError err,
                    FrNetInfo *nic);

/*
** PATCH: add the OLD fucntion for beta testing of the new client prior to 
** NIQS update
*/
DLLExport int 
OLD_frNetInfoClientInitSequence(
    ZK_INOUT ZkError            err,          /* error structure */ 
    ZK_INOUT FrNetInfo          ni,           /* NetInfo object */
    ZK_IN    const char *       zkipHost ,    /* host address */
    ZK_IN    uint16_t           zkipPort ,    /* port address */
    ZK_IN    uint16_t           zkipTimeout,  /* connection timeout */
    ZK_IN    FrKeyQry           kqCache,      /* key cache  -- was kq */
    ZK_IN    const char *       clientId,     /* client string identifier */
    ZK_OUT   FrNiqsClientInfo * pClientInfo,  /* pointer to client info structure */
    ZK_OUT   FrTokenType *      pTokenTypes   /* pointer to token type structure */
    );


  /**
   * This is the generic function used to populate a FrNetInfo object
   */
  DLLExport
  int frNetInfoPopulateSequence(
    ZK_INOUT ZkError            err,          /* error structure */ 
    ZK_INOUT FrNetInfo          ni,           /* NetInfo object */
    ZK_IN    const char *       defaultHost , /* host address */
    ZK_IN    uint16_t           defaultPort , /* port address */
    ZK_IN    uint16_t           zkipTimeout,  /* connection timeout */
    ZK_IN    uint16_t           numListIter,  /* number of cache list lookup */
    ZK_IN    FrKeyQry           kqCache       /* key cache */
    );

  /**
   * This is the generic function used to partially  populate a FrNetInfo object
   * It works like frNetInfoPopulateSequence, escept that it gets info only for 
   * the specified entity types
   * The list of entity types is specified be host_ent, an array of entity types
   *  of size host_ent_sz
   */
  DLLExport
  int frNetInfoSelectivePopulateSequence(
    ZK_INOUT ZkError            err,          /* error structure */ 
    ZK_INOUT FrNetInfo          ni,           /* NetInfo object */
    ZK_IN    const char *       defaultHost , /* host address */
    ZK_IN    uint16_t           defaultPort , /* port address */
    ZK_IN    uint16_t           zkipTimeout,  /* connection timeout */
    ZK_IN    uint16_t           numListIter,  /* number of cache list lookup */
    ZK_IN    FrKeyQry           kqCache,      /* key cache */
    ZK_IN    FrEntIdType *      host_ent,     /* array of entity types to query */
    ZK_IN    unsigned int       host_ent_sz   /* size of the array */
    );

	/*
	*  Given a netinfo object, populate it with information about servers in the
	*  Freedom network. 
	*  
	*  Besides modifying the netinfo object, it will also insert some keys into
	*  the key query object.
	*
	*  Supply list of entities to populate the netinfo DB with.
	*/
	DLLExport
	int frNetInfoSelectivePopulate(
	ZK_INOUT ZkError err,
	ZK_INOUT FrNetInfo ni,
	ZK_INOUT ZkipClient cli,
	ZK_IN    FrKeyQry kq,
	ZK_IN    FrEntIdType *host_ent,
	ZK_IN    unsigned int host_ent_sz) ;

  /**
   * This is the function that the client calls at startup
   * It populates a FrNetInfo object and gets clientinfo and tokeninfo
   */
  DLLExport int 
  frNetInfoClientInitSequence(
    ZK_INOUT ZkError            err,          /* error structure */ 
    ZK_INOUT FrNetInfo          ni,           /* NetInfo object */
    ZK_IN    const char *       zkipHost ,    /* host address */
    ZK_IN    uint16_t           zkipPort ,    /* port address */
    ZK_IN    uint16_t           zkipTimeout,  /* connection timeout */
    ZK_IN    uint16_t           numListIter,  /* number of cache list lookup */
    ZK_IN    FrKeyQry           kqCache,      /* key cache  -- was kq */
    ZK_IN    const char *       clientId,     /* client string identifier */
    ZK_OUT   FrNiqsClientInfo * pClientInfo,  /* pointer to client info structure */
    ZK_OUT   FrTokenType *      pTokenTypes   /* pointer to token type structure */
    );

  /**
   * This is the function that the client calls at periodical updates
   * It only populates a FrNetInfo object
   */
  DLLExport int 
  frNetInfoClientUpdateSequence(
    ZK_INOUT ZkError            err,          /* error structure */ 
    ZK_INOUT FrNetInfo          ni,           /* NetInfo object */
    ZK_IN    const char *       defaultHost , /* host address */
    ZK_IN    uint16_t           defaultPort , /* port address */
    ZK_IN    uint16_t           zkipTimeout,  /* connection timeout */
    ZK_IN    uint16_t           numListIter,  /* number of cache list lookup */
    ZK_IN    FrKeyQry           kqCache       /* key cache  -- was kq */
    );


  /**
   * This is the function that core servers/tools call at startup
   * It populates a FrNetInfo object and gets tokeninfo
   */
  DLLExport int 
  frNetInfoCoreInitSequence(
    ZK_INOUT ZkError            err,          /* error structure */ 
    ZK_INOUT FrNetInfo          ni,           /* NetInfo object */
    ZK_IN    const char *       defaultHost , /* host address */
    ZK_IN    uint16_t           defaultPort , /* port address */
    ZK_IN    uint16_t           zkipTimeout,  /* connection timeout */
    ZK_IN    uint16_t           numListIter,  /* number of cache list lookup */
    ZK_IN    FrKeyQry           kqCache,      /* key cache */
    ZK_OUT   FrTokenType *      pTokenTypes   /* pointer to token type structure */
    );

  /**
   * This is the generic function that everyone should call.
   * frNetInfoCoreInitSequence and frNetInfoClientInitSequence should no longer be used.
   */
  DLLExport int
  frNetInfoQuerySequence(
    ZK_INOUT ZkError            err,           /* error structure */
    ZK_INOUT ZkLog              log,           /* log object */
    ZK_INOUT FrNetInfo          ni,            /* NetInfo object */
    ZK_IN    const char *       defaultHost ,  /* host address */
    ZK_IN    uint16_t           defaultPort ,  /* port address */
    ZK_IN    uint16_t           zkipTimeout,   /* connection timeout */
    ZK_IN    uint16_t           numListIter,   /* number of cache list lookup */
    ZK_IN    FrKeyQry           kqCache,       /* key cache */

    ZK_IN    FrEntIdType      * host_ent,      /* array of entity type */
    ZK_IN    uint32_t           he_sz,         /* size of the array */

    ZK_IN    int                getClientInfo, /* flag to get ClientInfo */
    ZK_IN    const char *       clientId,      /* client string identifier */
    ZK_OUT   FrNiqsClientInfo * pClientInfo,   /* pointer to client info structure */

    ZK_IN    int                getTokenInfo,  /* flag to get TokenInfo */
    ZK_OUT   FrTokenType *      pTokenTypes    /* pointer to token type structure */
  );


  /**
   *  Update the netinfo object with any changes `since' a given time.
   */
  DLLExport extern
  int frNetInfoUpdateSince(ZkError err,
                           FrNetInfo ni,
                           ZkipClient cli,
                           FrKeyQry kq,
                           time_t since);
  
  /**
   * Given a populated netinfo object, update the status and rating fields
   * with a query to the niqs. New objects will be ignored
   */
  DLLExport extern
  int frNetInfoDoStatusUpdateSince(
                                   ZkError err,
                                   FrNetInfo ni,
                                   ZkipClient cli,
                                   FrKeyQry kq,
                                   time_t since);
  
  /**
   * Insert a FrNetInfoDatum object into the FrNetInfo object
   */
  DLLExport extern 
  int frNetInfoInsert(ZkError err, 
                      FrNetInfo nic,
                      FrNetInfoDatum data);


  /**
   * Get a ZkList of type entities from the FrNetInfo object.  The
   * ZkList consists of FrNetInfoDatum objects which can be retrieved
   * using zkListGet() on this list.
   */
  DLLExport extern
  int frNetInfoGetListOf(ZkError err,
                         FrNetInfo nic,
                         ZkList *l,
                         FrEntIdType type);

  /**
   * Randomize the order in which the objects are stored in the list
   * of the given entity type. Using this function will destroy the 
   * current list object for the given type. This means all current 
   * references to such lists obtained with frNetInfoGetListOf() will
   * become invalid.
   * This function assumes frCrypt has been initialized, and that
   * zkRandGetUInt32InRange is functional.
   */
  DLLExport extern
  int frNetInfoRandomizeList(ZkError err,
                             FrNetInfo nic,
                             FrEntIdType type);

  /**
   * Round robin the order in which the objects are stored in the list
   * of the given entity type, by taking the current list head and putting 
   * it at the tail. Using this function will destroy the current list 
   * object for the given type. This means all current references to such 
   * lists obtained with frNetInfoGetListOf() will become invalid.
   */
  DLLExport extern
  int frNetInfoRoundRobinList(ZkError err,
                              FrNetInfo nic,
                              FrEntIdType type);

  /**
   * Given an ID (associated with all entities), retrieve the FrNetInfoDatum
   * object.
   */
  DLLExport extern
  int frNetInfoSearchById(ZkError err,
                          FrNetInfo ni,
                          uint32_t id,
                          FrNetInfoDatum *data);

  /**
   * Given the entity type and name, get the FrNetInfoDatum object.
   */
  DLLExport extern
  int frNetInfoSearchByEntity(ZkError err,
                              FrNetInfo ni, 
                              FrEntIdExp entid,
                              FrNetInfoDatum *data);

  /**
   * Given the ID, entity type and name, get the FrNetInfoDatum object.
   */
  DLLExport extern 
  int frNetInfoSearchByEntityId(ZkError err, 
                                FrNetInfo ni, 
                                uint32_t id,
                                FrEntIdExp entid,
                                FrNetInfoDatum *data);
  /**
   * Given a sockaddr_in, get the FrNetInfoDatum object for it
   */
  DLLExport extern
  int frNetInfoSearchBySockaddr_in (ZkError err,
                                    FrNetInfo ni,
                                    struct sockaddr_in *saddr,
                                    FrNetInfoDatum *data);
  /**
   * Given an ip address and port, get the ent_ident for it. Return NULL
   * if it can't be found.
   */
  DLLExport extern
  int frNetInfoEntIdentFromIPandPort (ZkError err,
                                    FrNetInfo ni,
                                    struct in_addr iaddr,
                                    uint16_t port,
                                    uint32_t *ent_ident);
  /**
   * Delete the information about the ID from the network information object.
   */ 
  DLLExport extern 
  int frNetInfoDeleteById(ZkError err, 
                          FrNetInfo ni, 
                          uint32_t id);

  /**
   * Delete the information about the entid from the network 
   * information object.
   */ 
  DLLExport extern 
  int frNetInfoDeleteByEntity(ZkError err, 
                              FrNetInfo ni, 
                              FrEntIdExp entid);

  /**
   * Delete the information about the ID and entid from the network 
   * information object.
   */ 
  DLLExport extern
  int frNetInfoDeleteByEntityId(ZkError err, 
                                FrNetInfo ni, 
                                uint32_t id,
                                FrEntIdExp entid);


  /*
   *  Replace a datum based on its ID.  It's not an error if the datum doesn't
   *  exist.
   */
  DLLExport extern
  int frNetInfoReplaceById(ZkError err,
                           FrNetInfo ni,
                           FrNetInfoDatum d);

  
  /*
   * FrNetInfoDatumObject functions.
   */

  /**
   * Create a FrNetInfoDatum object.
   */
  DLLExport extern 
  int frNetInfoDatumNew(ZkError err,
                        FrNetInfoDatum *data);

  /**
   * Free a FrNetInfoDatum object.
   */
  DLLExport extern 
  int frNetInfoDatumFree(ZkError err,
                         FrNetInfoDatum *data);

  /**
   * Set the signed bufferfor the data object.
   */
  DLLExport extern 
  int frNetInfoDatumSetSignedBuffer(ZkError err, 
                              FrNetInfoDatum data, 
                              void * sbuf,
                              size_t sbsz);

  /**
   * Set the ID number for the data object.
   */
  DLLExport extern 
  int frNetInfoDatumSetId(ZkError err, 
                          FrNetInfoDatum data, 
                          uint32_t id);

  /**
   * Set the entity (inlcudes the type and the name) for the data object.
   */
  DLLExport extern 
  int frNetInfoDatumSetEntity(ZkError err, 
                              FrNetInfoDatum data, 
                              FrEntIdExp entity);

  /**
   * Set the status (currently 0 = down, 1 = up) for the data object.
   */
  DLLExport extern
  int frNetInfoDatumSetStatus(ZkError err, 
                             FrNetInfoDatum data, 
                             int status);

  /**
   * Set the rating for the data object.
   */
  DLLExport extern
  int frNetInfoDatumSetRating(ZkError err,
                             FrNetInfoDatum data, 
                             int rating);

  /**
   * Set the region where this data object resides
   */
  DLLExport int frNetInfoDatumSetRegion(ZkError err, FrNetInfoDatum data,
                                     const char *region);
  /**
   * Set the region where this data object resides
   */
  DLLExport int frNetInfoDatumSetRegionCode(ZkError err, FrNetInfoDatum data,
                                     const char *region_code);
  
  /**
   * Set the servoperator where this data object resides
   */
  DLLExport int frNetInfoDatumSetOperator(ZkError err, FrNetInfoDatum data,
                                     const char *servoperator);
  /**
   * Set the IP address and port in struct sockaddr_in format 
   * for the data object.
   */
  DLLExport extern 
  int frNetInfoDatumSetSockaddr(ZkError err,
                                FrNetInfoDatum data, 
                                struct sockaddr_in addr);

  /*
   *  Set/Get the port on which the Diffie-Hellman key agreement 
   *  takes place (AIPs only).
   *  
   *  The `port' should be in host byte order.
   */
  DLLExport extern
  int frNetInfoDatumSetDifHelPort(ZkError err, FrNetInfoDatum data, 
                                  uint16_t port);
  
  DLLExport extern
  int frNetInfoDatumGetDifHelPort(ZkError err, FrNetInfoDatum data, 
				  uint16_t *pPort);

  /**
   * Get the signed bufferfor the data object.
   */
  DLLExport extern 
  int frNetInfoDatumGetSignedBuffer(ZkError err, 
                              FrNetInfoDatum data, 
                              void ** sbuf,
                              size_t * sbsz);

/**
   * Get the ID number for the data object.
   */
  DLLExport extern 
  int frNetInfoDatumGetId(ZkError err, 
                          FrNetInfoDatum data, 
                          uint32_t *id);

  /**
   * Get the entity (inlcudes the type and the name) for the data object.
   */
  DLLExport extern 
  int frNetInfoDatumGetEntity(ZkError err, 
                              FrNetInfoDatum data, 
                              FrEntIdExp *entity);

  /**
   * Get the status (currently 0 = down, 1 = up) for the data object.
   */
  DLLExport extern
  int frNetInfoDatumGetStatus(ZkError err, 
                             FrNetInfoDatum data, 
                             int *status);

  /**
   * Get the rating for the data object.
   */
  DLLExport extern
  int frNetInfoDatumGetRating(ZkError err,
                             FrNetInfoDatum data, 
                             int *rating);

  /**
   * Get the region where this data object resides
   */
  DLLExport int frNetInfoDatumGetRegion(ZkError err, FrNetInfoDatum data,
                                     const char **region);
  
  /**
   * Get the region where this data object resides
   */
  DLLExport int frNetInfoDatumGetRegionCode(ZkError err, FrNetInfoDatum data,
                                     const char **region_code);
  
  /**
   * Get the servoperator where this data object resides
   */
  DLLExport int frNetInfoDatumGetOperator(ZkError err, FrNetInfoDatum data,
                                     const char **servoperator);
  /**
   * Get the IP address and port in struct sockaddr_in format 
   * for the data object.
   */
  DLLExport extern 
  int frNetInfoDatumGetSockaddr(ZkError err,
                               FrNetInfoDatum data, 
                               struct sockaddr_in *addr);

  /*
   * The following functions only have significance for entity types
   * FRENT_TYPE_AIP.
   */

  /**
   * Add a link between this entity and the neighbour whose ID is nbid. 
   */
  DLLExport extern 
  int frNetInfoDatumAddNb(ZkError err, 
                          FrNetInfoDatum data, 
                          uint32_t nbid);

  /**
   * For an AIP entity, get the number of neighbours that this entity has.
   * Used in conjunction with the following function.
   */
  DLLExport extern 
  int frNetInfoDatumGetNumNb(ZkError err,
                             FrNetInfoDatum data, 
                             size_t *nb);

  /**
   * For the AIP entity, get the ID of neighbour nb.  
   */
  DLLExport extern 
  int frNetInfoDatumGetNbX(ZkError err, 
                           FrNetInfoDatum data, 
                           size_t nb, 
                           uint32_t *nbid);

  /**
   *  Convert the datum to a binary buffer.  The buffer is allocated and 
   *  returned by the function.
   */

  DLLExport extern
  int frNetInfoDatumStore (ZkError err ,
                           FrNetInfoDatum data ,
                           void ** buf ,
                           size_t * bsz );

   /**
    * Load an FrNetInfoDatum structure from a binary buffer.  You must first
    * create an FrNetInfoDatum object using the frNetInfoDatumNew() function.
    */

  DLLExport extern 
  int frNetInfoDatumLoad (ZkError err ,
                          FrNetInfoDatum data ,
                          void * buf ,
                          size_t bsz );

   /**
    * Verify the contents of a FrNetInfoDatum structure corresponds to the one
    * found in its signed buffer. 
    */

  DLLExport extern 
  int frNetInfoDatumVerifyContents (ZkError err ,
                                    FrNetInfoDatum data ,
                                    int * contents_identical_flag);

  DLLExport extern 
  int frNetInfoValidateHeader(ZkError err,
			      uint8_t  *buf,
			      size_t    bsz,
			      time_t   *pUpdateTime,
			      uint32_t *pCacheDataVersion);

  DLLExport extern 
  int frNetInfoGetSignedBuffer(ZkError err,
			       FrNetInfo fni,
			       uint8_t ** pcbuf, size_t * pbsz);

  DLLExport extern 
  int frNetInfoLoadNoSig (ZkError err ,
			  FrNetInfo fni ,
			  uint8_t * buf ,
			  size_t bsz );

  DLLExport extern 
  int frNetInfoLoad (ZkError err ,
                     FrNetInfo fni ,
                     FrKeyQry kq ,
                     void * buf ,
                     size_t bsz );

  DLLExport extern
  int frNetInfoStore (ZkError err ,
                      FrNetInfo fni ,
                      void ** buf ,
                      size_t * bsz );

  /**
   * Return the time that the FrNetInfo object was last updated from the 
   * NIQS server.
   */

  DLLExport extern 
  int frNetInfoLastUpdate (ZkError err , 
                           FrNetInfo fni ,
                           time_t * lastUpdate );

  DLLExport extern 
  int frNetInfoCacheDataVersion(ZkError    err, 
				FrNetInfo  fni,
				uint32_t  *pCacheDataVersion);



  /**
   * Delete all entries in the frnetinfo structure.
   */

  DLLExport extern
  int frNetInfoDeleteAll (
    ZkError err ,
    FrNetInfo fni );

  /**
   * Given the list of currently available entities fromt the NIQS, remove
   * all entities from the frNetInfo which are not in the list.  This
   * function is used by the update and updateSince functions in order
   * to clean the netinfo cache of entities which are no longer needed.  This
   * is done automatically
   */

  DLLExport extern
  int frNetInfoCullEntities (
    ZkError err ,          /* Returns the error */
    FrNetInfo fni ,        /* The net info object */
    uint32_t * entid ,     /* The array of entity ID's */
    size_t entid_sz );     /* The number of entries in the array */

  /*
   * Set the display callback so that the frnetinfo object can write messages
   * to the Freedom client console.  Calling this function is optional.  If the
   * callback is not set, FrNetInfo will just do its job silently.
   */

  DLLExport int frNetInfoSetClientConsoleCallback (
    ZK_INOUT ZkError err ,
    ZK_INOUT FrNetInfo nic ,
    ZK_IN    ZkClientConsoleCallback clicons ,
    ZK_IN    void * clicons_data );

  /*
   * Set and retrieve the nextUpdate time (this is an absolute time, not
   * an offset).  If it is set to 0, the FrNetInfo will not ever force a 
   * complete reload of the NIQS information unless explicitly directed
   * to do so
   */

  DLLExport int frNetInfoNextUpdate (
    ZK_INOUT ZkError err ,
    ZK_IN    FrNetInfo fni ,
    ZK_OUT   time_t * nextUpdate );

  DLLExport int frNetInfoSetNextUpdate (
    ZK_INOUT ZkError err ,
    ZK_INOUT FrNetInfo fni ,
    ZK_IN    time_t nextUpdate );

  /*
   *  If you set the update offset without setting an explicit next update
   *  time using frNetInfoSetNextUpdate(), the object will set the next
   *  update time to the current time + the specified offset.
   */

  DLLExport int frNetInfoSetUpdateOffset (
    ZK_INOUT ZkError err ,
    ZK_INOUT FrNetInfo fni ,
    ZK_IN    time_t offset );

  DLLExport int frNetInfoUpdateOffset(
    ZK_INOUT ZkError err ,
    ZK_IN    FrNetInfo fni ,
    ZK_OUT   time_t * offset );

  /*
   * update the frnetinfo structure with only the current AIP status.
   */

  DLLExport int frNetInfoUpdateAipStatus (
    ZK_INOUT ZkError err,
    ZK_INOUT FrNetInfo ni,
    ZK_IN    ZkipClient cli,
    ZK_IN    FrKeyQry kq);


  DLLExport int frNetInfoCountRecords (
    ZkError err ,
    FrNetInfo fni ,
    size_t * c );   /* Returns the number of entries in the NetInfo */

#ifdef FREEDOM_DEVELOPMENT_BUILD

  /*
   * Dumps the netinfo object for specific entity types, to a file pointer.
   */
  DLLExport int frNetInfoDump(ZkError err,
                              FrNetInfo nic,
                              FrEntIdType entityType,
                              FILE * outFile);

  /*
   * Dumps the netinfo object for specific entity types, to a log.
   */
  DLLExport int frNetInfoDumpInLog(ZkError err,
                                   FrNetInfo nic,
                                   FrEntIdType entityType,
                                   ZkLog log);

#endif /* FREEDOM_DEVELOPMENT_BUILD */




  /**
   *  Initially populate the netinfo object with information about servers in
   *  the Freedom network.
   */
  DLLExport extern
  int frNetInfoPopulate(ZkError err,
                        FrNetInfo ni,
                        ZkipClient cli,
                        FrKeyQry kq);



#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif/* FR_NETINFO_H_ */
