/* 
 * $Id: netinfo_data.c,v 1.26.8.3 2000/12/14 17:53:00 jean Exp $
 */

static const char * module_id_ = "$Id: netinfo_data.c,v 1.26.8.3 2000/12/14 17:53:00 jean Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include "niobj.h"
#include "netinfo_data.h"
#include "string.h"
#include "frsec_sig.h"
#include "frNidbObj.h"
#include "zkAlloc.h"
#include "zkCStr.h"

/**
 * Create a new FrNetInfoDatum object.
 */
DLLExport int frNetInfoDatumNew(ZkError err, FrNetInfoDatum *data)
{
  const char *func = "frNetInfoDatumNew";

  (void) module_id_; /* Get rid of compile warning */

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  *data = malloc(sizeof(struct T_FrNetInfoDatum));
  if (!*data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENOMEM, "No more memory");
    return 0;
  }

  (*data)->signed_buffer = NULL;
  (*data)->signed_buffer_sz = 0;
  (*data)->id = 0;
  memset(&((*data)->entid), 0, sizeof(FrEntIdExp));
  (*data)->status = 0;
  (*data)->rating = 0;
  (*data)->region = NULL;
  (*data)->region_code = NULL;
  (*data)->servoperator = NULL;
  
  memset(&((*data)->addr), 0, sizeof(struct sockaddr_in));

  (*data)->dhport = 0;

  (*data)->nb.nbid = NULL;
  (*data)->nb.numnb = 0;

  (*data)->valid = 1;
  (*data)->cull = 0;

  return 1;
}

/**
 * Free the FrNetInfoDatum object.
 */
DLLExport int frNetInfoDatumFree(ZkError err, FrNetInfoDatum *data)
{
  const char *func = "frNetInfoDatumFree";


  if (!data || !*data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  if ((*data)->signed_buffer) {
    free((*data)->signed_buffer);
    (*data)->signed_buffer = NULL;
  }

  if ((*data)->nb.nbid) {
    free((*data)->nb.nbid);
    (*data)->nb.nbid = NULL;
  }

  if ((*data)->servoperator) {
    free((*data)->servoperator);
    (*data)->servoperator = NULL;
  }
  if ((*data)->region) {
    free((*data)->region);
    (*data)->region = NULL;
  }
  if ((*data)->region_code) {
    free((*data)->region_code);
    (*data)->region_code = NULL;
  }

  free(*data);
  *data = NULL;
  
  return 1;
}



/**
 * Set the signed buffer of the datum.
 */
DLLExport int frNetInfoDatumSetSignedBuffer(ZkError err, 
                                            FrNetInfoDatum data, 
                                            void * sbuf,
                                            size_t sbsz)
{
  const char *func = "frNetInfoDatumSetSignedBuffer";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  if (data->signed_buffer) {
    free (data->signed_buffer);
    data->signed_buffer = NULL;
    data->signed_buffer_sz = 0;
  }

  if (!zkMemDup(err, (void **)&(data->signed_buffer), sbuf, sbsz)) {
    return 0;
  }

  data->signed_buffer_sz = sbsz;

  return 1;
}

/**
 * Set the ID of the datum
 */
DLLExport int frNetInfoDatumSetId(ZkError err, FrNetInfoDatum data, 
                                  uint32_t id)
{
  const char *func = "frNetInfoDatumSetId";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  data->id = id;

  return 1;
}

/**
 * Set the entity of the datum
 */
DLLExport int frNetInfoDatumSetEntity(ZkError err, FrNetInfoDatum data, 
                                      FrEntIdExp entity)
{
  const char *func = "frNetInfoDatumSetEntity";
  int valid = 0;
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  if (!frEntIsValid(err, &entity, &valid)) {
    return 0;
  }

  if (!valid) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKEINVAL, "Entity is invalid");
    return 0;
  }

  memcpy(&data->entid, &entity, sizeof(FrEntIdExp));

  return 1;
}

/**
 * Set the status of the datum.
 */
DLLExport int frNetInfoDatumSetStatus(ZkError err, FrNetInfoDatum data, 
                                     int status)
{
  const char *func = "frNetInfoDatumSetStatus";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  data->status = status;

  return 1;
}

/**
 * Set the rating of the datum.
 */
DLLExport int frNetInfoDatumSetRating(ZkError err, FrNetInfoDatum data,
                                     int rating)
{
  const char *func = "frNetInfoDatumSetRating";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  data->rating = rating;

  return 1;
}

/**
 * Set the region string for the entity
 */
DLLExport int frNetInfoDatumSetRegion(ZkError err, FrNetInfoDatum data,
                                     const char *region)
{
  const char *func = "frNetInfoDatumSetRegion";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  if ((data->region = zkCStrDup(region)) == NULL) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENOMEM, "strdup() failure");
    return 0;
  }

  return 1;
}

/**
 * Set the region string for the entity
 */
DLLExport int frNetInfoDatumSetRegionCode(ZkError err, FrNetInfoDatum data,
                                     const char *region_code)
{
  const char *func = "frNetInfoDatumSetRegionCode";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  if ((data->region_code = zkCStrDup(region_code)) == NULL) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENOMEM, "strdup() failure");
    return 0;
  }

  return 1;
}

/**
 * Set the operator of the entity.
 */
DLLExport int frNetInfoDatumSetOperator(ZkError err, FrNetInfoDatum data,
                                        const char *servoperator)
{
  const char *func = "frNetInfoDatumSetOperator";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  if ((data->servoperator = zkCStrDup(servoperator)) == NULL) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENOMEM, "strdup() failure");
    return 0;
  }

  return 1;
}

/**
 * Set the struct sockaddr_in of the datum.
 */
DLLExport int frNetInfoDatumSetSockaddr(ZkError err, FrNetInfoDatum data, 
                                       struct sockaddr_in addr)
{
  const char *func = "frNetInfoDatumSetSockaddr";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  memcpy(&data->addr, &addr, sizeof(struct sockaddr_in));

  return 1;
}

/*
 *  Set the port on which the Diffie-Hellman key agreement takes place. (For
 *  AIPs only.)
 *  
 *  The port should be in host byte order.
 */
DLLExport int frNetInfoDatumSetDifHelPort(ZkError err, FrNetInfoDatum data, 
                                          uint16_t port)
{
  const char *func = "frNetInfoDatumSetDifHelPort";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  data->dhport = port;

  return 1;
}

/**
 * Set the cull field of the datum
 */

DLLExport int frNetInfoDatumSetCull(ZkError err, FrNetInfoDatum data,
                                     int cull)
{
  const char *func = "frNetInfoDatumSetCull";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  data->cull = cull;

  return 1;
}

/**
 * Set the valid bit of the datum.
 */
DLLExport int frNetInfoDatumSetValid(ZkError err, FrNetInfoDatum data,
                                     int valid)
{
  const char *func = "frNetInfoDatumSetValid";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  data->valid = valid;

  return 1;
}

/**
 * Get the signed buffer from the datum.
 */
DLLExport int frNetInfoDatumGetSignedBuffer(ZkError err, 
                                            FrNetInfoDatum data, 
                                            void ** sbuf,
                                            size_t * sbsz)
{
  const char *func = "frNetInfoDatumGetSignedBuffer";
  

  if (!data || !sbuf || !sbsz) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  if (data->signed_buffer) {
    if (!zkMemDup(err, sbuf, data->signed_buffer, data->signed_buffer_sz)) {
      return 0;
    }

    *sbsz = data->signed_buffer_sz;
  }

  else {
    *sbuf = NULL;
    *sbsz = 0;
  }

  return 1;
}

/**
 * Get the ID of the datum
 */
DLLExport int frNetInfoDatumGetId(ZkError err, FrNetInfoDatum data,
                                  uint32_t *id)
{
  const char *func = "frNetInfoDatumGetId";
  

  if (!data || !id) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  *id = data->id;

  return 1;
}

/**
 * Get the entity of the datum
 */
DLLExport int frNetInfoDatumGetEntity(ZkError err, FrNetInfoDatum data, 
                                      FrEntIdExp *entity)
{
  const char *func = "frNetInfoDatumGetEntity";
  

  if (!data || !entity) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
   return 0;
  }


  memcpy(entity, &data->entid, sizeof(FrEntIdExp));

  return 1;
}


/*
 *  Return the port on which the Diffie-Hellman key agreement takes place. 
 *  (For AIPs only.)
 *  
 *  The port should be in host byte order.
 */
DLLExport int frNetInfoDatumGetDifHelPort(
  ZkError         err, 
  FrNetInfoDatum  data, 
  uint16_t       *pPort)
{
  const char *func = "frNetInfoDatumGetDifHelPort";
  
  zkErrorClear(err);
  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  *pPort = data->dhport;

  return 1;
}


/**
 * Get the status of the datum.
 */
DLLExport int frNetInfoDatumGetStatus(ZkError err, FrNetInfoDatum data,
                                     int *status)
{
  const char *func = "frNetInfoDatumGetStatus";
  

  if (!data || !status) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  *status = data->status;

  return 1;
}

/**
 * Get the rating of the datum.
 */
DLLExport int frNetInfoDatumGetRating(ZkError err, FrNetInfoDatum data, 
                                     int *rating)
{
  const char *func = "frNetInfoDatumGetRating";
  

  if (!data || !rating) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  *rating = data->rating;

  return 1;
}

DLLExport int frNetInfoDatumGetRegion(ZkError err, FrNetInfoDatum data,
                                      const char **region)
{
  static const char *func = "frNetInfoDatumGetRegion";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  *region = data->region;

  return 1;
}

DLLExport int frNetInfoDatumGetOperator(ZkError err, FrNetInfoDatum data,
                                        const char **servoperator)
{
  static const char *func = "frNetInfoDatumGetOperator";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  *servoperator = data->servoperator;

  return 1;
}

DLLExport int frNetInfoDatumGetRegionCode(ZkError err, FrNetInfoDatum data,
                                      const char **region_code)
{
  static const char *func = "frNetInfoDatumGetRegionCode";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  *region_code = data->region_code;

  return 1;
}

/**
 * Get the struct sockaddr_in of the datum.
 */

DLLExport int frNetInfoDatumGetSockaddr(ZkError err, FrNetInfoDatum data, 
                                       struct sockaddr_in *addr)
{
  const char *func = "frNetInfoDatumGetSockaddr";
  

  if (!data || !addr) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  memcpy(addr, &data->addr, sizeof(struct sockaddr_in));

  return 1;
}

/**
 * Get the cull field of the datum.
 */
DLLExport int frNetInfoDatumGetCull(ZkError err, FrNetInfoDatum data,
                                     int *cull)
{
  const char *func = "frNetInfoDatumGetCull";
  

  if (!data || !cull) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  *cull = data->cull;

  return 1;
}

/**
 * Get the valid bit of the datum.
 */
DLLExport int frNetInfoDatumGetValid(ZkError err, FrNetInfoDatum data,
                                     int *valid)
{
  const char *func = "frNetInfoDatumGetValid";
  

  if (!data || !valid) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  *valid = data->valid;

  return 1;
}

/**
 * Get the number of neighbours.
 */
DLLExport int frNetInfoDatumGetNumNb(ZkError err, FrNetInfoDatum data, 
                                     size_t *nb)
{
  const char *func = "frNetInfoDatumGetNumNb";
  

  if (!data || !nb) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  *nb = data->nb.numnb;
  return 1;
}

/*
 * Get neighbour X ID.
 */
DLLExport int frNetInfoDatumGetNbX(ZkError err, FrNetInfoDatum data, 
                                   size_t nb, uint32_t *nbid)
{
  const char *func = "frNetInfoDatumGetNbX";
  

  if (!data || !nbid) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  if (nb > data->nb.numnb) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKEPARMBAD, "No neighbour");    
    return 0;
  }

  *nbid = data->nb.nbid[nb];

  return 1;
}

/*
 * Add a new neighobur to the data object.  Allocate more memory if needed.
 */
DLLExport int frNetInfoDatumAddNb(ZkError err, FrNetInfoDatum data, 
                                  uint32_t nbid)
{
  const char *func = "frNetInfoDatumGetNbX";
  

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  if( (data->entid.type!=FRENT_TYPE_AIP) && 
      (data->entid.type!=FRENT_TYPE_COREAIP) ){
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKEPARMBAD, "Nb for AIPs or COREAIPs only");
    return 0;
  }

  data->nb.numnb++;
  if (!data->nb.nbid) {
    if ((data->nb.nbid = calloc(1, data->nb.numnb * sizeof(NbData))) == NULL) {
      zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENOMEM, "calloc() failed");
      return 0;
    }
  } else {
    if (!zkMemRealloc(err, (void *) &data->nb.nbid, 
                   data->nb.numnb * sizeof(NbData))) {
      return 0;
    }
  }
  data->nb.nbid[data->nb.numnb - 1] = nbid;

  return 1;
}

/*
 *
 */

DLLExport int
frNetInfoDatumLoad (
  ZkError err ,
  FrNetInfoDatum data ,
  void * buf ,
  size_t bsz )
{
  static const char * func = "frNetInfoDatumLoad";
  unsigned char * b = NULL;
  size_t n = 0;
  size_t os = 0;
  size_t str_sz = 0; /* For computing string lengths in the binary buffer */
  size_t bsz2 = 0;
  int ret = 0;
  
  if (!data) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "FrNetInfoDataLoad object (data) is NULL");
  } else if (!buf) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "Data buffer pointer (buf) is NULL");
  } else {
    b = (unsigned char *) buf;
    bsz2 = b[0] * 0x100 + b[1];
    if (bsz2 > bsz) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEINCOMPLETE,
                 "FrNetInfoDatum buffer is not complete");
    } else if (!frEntLoad(err,&(data->entid),&(b[6]),FRENT_PORTABLE_SIZE)) {
 
    } else {

      /*
       * The entity ID number
       */

      data->id = b[2] * 0x1000000 + b[3] * 0x10000 + b[4] * 0x100 + b[5];

      /*
       * Skip over the entity ID which we've alreay processed and get the
       * status, 0 for down, non-zero for up.
       */

      os += FRENT_PORTABLE_SIZE + 6;
      if (b[os]) {
        data->status = 1;
      } else {
        data->status = 0;
      }
 
      /*
       * Get the entity's rating
       */

      os += 1;
      data->rating = b[os] * 0x1000000 + b[os + 1] * 0x10000 + 
                     b[os + 2] * 0x100 + b[os + 3];
    
      /*
       * Get the Entity's port and IP address
       */

      os += 4;
      data->addr.sin_family = AF_INET;
      data->addr.sin_port = ntohs(b[os] * 0x100 + b[os + 1]);
      os += 2;
      data->addr.sin_addr.s_addr = ntohl(b[os] * 0x1000000 + 
                                         b[os + 1] * 0x10000 + 
                                         b[os + 2] * 0x100 + b[os + 3]);

      /*
       * Load the Diffie Helman port
       */

      os += 4;
      data->dhport = b[os] * 0x100 + b[os + 1];

      /*
       * Get the number of neighbours and their ID codes
       */

      os += 2;
      data->nb.numnb = b[os] * 0x1000000 + b[os + 1] * 0x10000 + 
                       b[os + 2] * 0x100 + b[os + 3];
      if (data->nb.numnb > 0) {
        data->nb.nbid = (uint32_t *) malloc(sizeof(uint32_t) * data->nb.numnb);
      } else {
        data->nb.nbid = NULL;
        zkErrorSet_va(err,func,ZKWARN,ZKMFRNETINFO,ZKERROR,
                      "Warning, entity %d has no neighbours", data->id);
      }

      os += 4;
      for (n = 0; n < data->nb.numnb; n++) {
        data->nb.nbid[n] = b[os] * 0x1000000 + b[os + 1] * 0x10000 + 
                           b[os + 2] * 0x100 + b[os + 3];
        os += 4;
      }

      /*
       * Now get the region string
       */

      while (((os + str_sz) < bsz2) && (b[os + str_sz] != 0)) str_sz++;
      data->region_code = strdup((char *) &(b[os])); 
      os += str_sz + 1;
      str_sz = 0;

      /*
       * Now get the region string
       */

      while (((os + str_sz) < bsz2) && (b[os + str_sz] != 0)) str_sz++;
      data->region = strdup((char *) &(b[os])); 
      os += str_sz + 1;
      str_sz = 0;
   
      /*
       * Get the server operator string
       */

      while (((os + str_sz) < bsz2) && (b[os + str_sz] != 0)) str_sz++;
      data->servoperator = strdup((char *) &(b[os])); 
      os += str_sz + 1;
      str_sz = 0;

     
      /*
       * Get the signed buffer string
       */
      data->signed_buffer_sz = b[os] * 0x1000000 + b[os + 1] * 0x10000 + 
                               b[os + 2] * 0x100 + b[os + 3];
      os += 4;

      /* 
      ** HACK: could be no signed buffer
      */
      if (0 == data->signed_buffer_sz ||
          frNetInfoDatumSetSignedBuffer(err, data, &(b[os]), data->signed_buffer_sz)) { 
        ret = 1;
      }
    }
  }
  return ret;
}

/*
 * 
 */

DLLExport int
frNetInfoDatumStore (
  ZkError err ,
  FrNetInfoDatum data ,
  void ** buf ,
  size_t * bsz )
{
  static const char * func = "frNetInfoDatumStore";
  unsigned char * b = NULL;      /* Assembly space for the buffer */
  size_t nb_sz = 0;   /* Length of the neighbour list */
  size_t r_sz = 0;    /* Length of the region string list */
  size_t rc_sz = 0;    /* Length of the region code string list */
  size_t so_sz = 0;   /* Length of the server operator string */
  size_t os = 0;      /* Offset */
  size_t n = 0;
  int ret = 0;

  if (!data) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "FrNetInfoDataLoad object (data) is NULL");
  } else if (!buf) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "Data buffer pointer pointer (buf) is NULL");
  } else if (!bsz) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "Data buffer size (bsz) is NULL");
  } else if (data->valid) {
    nb_sz = data->nb.numnb * 4 + 4 ;
    so_sz = strlen(data->servoperator ? data->servoperator : "");
    r_sz = strlen(data->region ? data->region : "");
    rc_sz = strlen(data->region_code ? data->region_code : "");
    *bsz = nb_sz + 1 + so_sz + 1 + r_sz + 1 + rc_sz + 1 +
           4 + FRENT_PORTABLE_SIZE + 1 + 4 + 6 + 2 + 1 + 2 +
           data->signed_buffer_sz + 4;
    b = (unsigned char *) malloc(*bsz);
    if (!b) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENOMEM,
                 "Failed to allocate space for FrNetInfoDatum buffer");
    /*
     * Now store the parts that require storage functions.  We begin with the
     * datum's entity type and name.
     */

    } else if (!frEntStore(err,&(data->entid),&(b[6]),FRENT_PORTABLE_SIZE)) {
    

    /*
     * Everything else is just copying bytes into the right places in the
     * buffer.
     */

    } else {
      /* Store the size of the buffer, as always, MSB first */

      b[os++] = *bsz / 0x100;
      b[os++] = *bsz % 0x100;

      /* Store the entity ID number of the Datum */

      b[os++] = (data->id & 0xFF000000) / 0x1000000; 
      b[os++] = (data->id & 0xFF0000) / 0x10000;
      b[os++] = (data->id & 0xFF00) / 0x100;
      b[os++] = (data->id & 0xFF);

      /* Skip over the space which stores the FrNetInfoDatum's entity */
   
      os += FRENT_PORTABLE_SIZE;

      /* Entity status, 0 = down, non-zero = up */
      
      b[os++] = data->status ? 1 : 0;
     
      /* The rating which should really be a uint32_t */

      b[os++] = (data->rating & 0xFF000000) / 0x1000000;
      b[os++] = (data->rating & 0xFF0000) / 0x10000;
      b[os++] = (data->rating & 0xFF00) / 0x100;
      b[os++] = (data->rating & 0xFF);
      
      /* Then entity's address and port */
   
      {
        uint16_t foo_port = (uint16_t) htons(data->addr.sin_port);
        b[os++] = (foo_port & 0xFF00) / 0x100;
        b[os++] = (foo_port & 0xFF);
      }

      /* */
     
      {
        uint32_t foo_addr = (uint32_t) htonl(data->addr.sin_addr.s_addr); 
        b[os++] = (foo_addr & 0xFF000000) / 0x1000000;
        b[os++] = (foo_addr & 0xFF0000) / 0x10000;
        b[os++] = (foo_addr & 0xFF00) / 0x100;
        b[os++] = (foo_addr & 0xFF);
      }

      /* The entity's Diffie Helman port */
      
      b[os++] = (data->dhport & 0xFF00) / 0x100;
      b[os++] = (data->dhport & 0xFF);
    
      /* Store the neighbour list (first we store the size, then we store the
       * list itself
       */

      b[os++] = (data->nb.numnb & 0xFF000000) / 0x1000000;
      b[os++] = (data->nb.numnb & 0xFF0000) / 0x10000;
      b[os++] = (data->nb.numnb & 0xFF00) / 0x100;
      b[os++] = (data->nb.numnb & 0xFF);
      for (n = 0; n < data->nb.numnb; n++) {
        b[os++]     = (data->nb.nbid[n] & 0xFF000000) / 0x1000000;
        b[os++] = (data->nb.nbid[n] & 0xFF0000) / 0x10000;
        b[os++] = (data->nb.nbid[n] & 0xFF00) / 0x100;
        b[os++] = (data->nb.nbid[n] & 0xFF);
      }

      /* Store the region string */

      strncpy(&(b[os]),data->region_code,rc_sz);
      os += rc_sz; 
      b[os++] = (unsigned char) 0;

      /* Store the region string */

      strncpy(&(b[os]),data->region,r_sz);
      os += r_sz; 
      b[os++] = (unsigned char) 0;

      /* Store the server operator string */

      strncpy(&(b[os]),data->servoperator,so_sz);
      os += so_sz;
      b[os++] = (unsigned char) 0;

      /* Store the signed buffer */

      b[os++] = (data->signed_buffer_sz & 0xFF000000) / 0x1000000;
      b[os++] = (data->signed_buffer_sz & 0xFF0000) / 0x10000;
      b[os++] = (data->signed_buffer_sz & 0xFF00) / 0x100;
      b[os++] = (data->signed_buffer_sz & 0xFF);

      memcpy(&(b[os]), data->signed_buffer, data->signed_buffer_sz);
      os += data->signed_buffer_sz;

      *buf = b;
      ret = 1;
    }
  }
  return ret;
}


/*
 * 
 */

DLLExport int 
frNetInfoDatumVerifyContents (
  ZkError err ,
  FrNetInfoDatum data ,
  int * contents_identical_flag)
{
  static const char * func = "frNetInfoDatumVerifyContents";
  char *buf = NULL;
  size_t bsz = 0;
  FrNidbObj obj = NULL;
  FrEntIdExp datum_entity;
  FrEntIdExp buffer_entity;
  FrEntIdent ent_id;
  FrNidbObjType type;
  const char *region_code = 0;
  char *region = 0;
  const char *servop = 0;
  size_t i;
  size_t nbs; /* the number of an AIPs neighbours */
  struct in_addr iaddr; /* server address */
  uint16_t dhport; /* AIP Diffie-Hellman port */
  uint16_t port; /* main server port */
  uint32_t nbid;
  uint32_t rating; /* latest rating */
  uint32_t status; /* latest status of the aip */
  int neigb_list_is_ok = 1;
  int ret = 1;

  if (!data) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "FrNetInfoDatum object (data) is NULL");
    ret = 0;
  }

  else if (!contents_identical_flag) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "int object (contents_identical_flag) is NULL");
    ret = 0;
  }

  if (( ! frDBODriverIsInitialized(err)) && ( ! frDBODriverInitAll(err))) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKERROR,
               "Could not initialize DBO driver");
    ret = 0;
  }

  else {
    *contents_identical_flag = 0;

    /* Get the signed buffer */
    if (!frNetInfoDatumGetSignedBuffer(err, data, (void **)&buf, &bsz)) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKERROR,
                 "Could not get signed buffer for object");
    }

    /* Remove signature */
    else if (!frSecSigRemoveInSitu(err, buf, &bsz)) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKERROR,
                 "Could not remove signature from buffer");
    }

    /* Get the datum entity */
    else if (!frNetInfoDatumGetEntity(err, data, &datum_entity)) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                 "Could not get entity from datum");
    }

    /* Extract NidbObj from buffer */
    
    else if ( ! (frNidbObjUnpack(err, buf, bsz, &obj) &&
                 frNidbObjGetEntity(err, obj, &buffer_entity))) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                 "Could not extract NidbObj from buffer");
    }

    /* Validate entity */

    else if (datum_entity.type != buffer_entity.type ||
             (strcmp(datum_entity.name, buffer_entity.name) != 0)) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                 "Entity differ between signed buffer and datum");
    }

    else if  (!frNidbObjGetEntIdent (err, obj, &ent_id)) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                 "Could not get entity id from NidbObj");
    }

    else if  (ent_id != data->id) {
      zkErrorSet_va(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                    "Entity id differ for %s/%d (%d != %d)",
                    datum_entity.name,datum_entity.type,ent_id,data->id);
    }

    /* Validate specific data */

    else if (!frNidbObjGetType(err, obj, &type)) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                 "Could not get type from NidbObj");
    }

    else if ( (datum_entity.type==FRENT_TYPE_AIP)||(datum_entity.type==FRENT_TYPE_COREAIP) ) {
 
      if (type != FRNIDBOBJ_AIP) {
        zkErrorSet_va(err,func,ZKERR,ZKMNIQS,ZKEDATABAD,
                      "Improper type (%u) in NidbObj", type);
      }

      else if (!frNidbObjGetAipParams(err, obj, &iaddr, &dhport,
                                      &port, &status, &rating,
                                      &region_code, &region, &servop, 
                                      &nbs)) {
        zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                   "Could not get AIP params from NidbObj");
      }
        
      else if (data->addr.sin_port != htons(port)) {
        zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                   "Ports differ");
      }

      else if (memcmp (&(data->addr.sin_addr), &iaddr, sizeof (struct in_addr)) 
               != 0) {
        zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                   "Addresses differ");
      }

      else if (data->dhport != dhport) {
        zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                   "DifHel ports differ");
      }

      else if (data->rating != (int)rating) {
        zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                   "Rating differs");
      }

      else if (data->status != (int)status) {
        zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                   "Status differs");
      }

      else if (strcmp (data->region_code, region_code) != 0) {
        zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                   "Region code differs");
      }

      else if (strcmp (data->region, region) != 0) {
        zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                   "Region differs");
      }

      else if (strcmp (data->servoperator, servop) != 0) {
        zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                   "Server operator differs");
      }

      else if (data->nb.numnb != nbs) {
        zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                   "Number of neighbors differs");
      }

      /* Verify the neighbours list */
      for (i = 0; i < nbs; i++) {
        if (!frNidbObjGetAipNeighbour(err, obj, i, &nbid)) {
          zkErrorSet_va(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                     "Could not get neighbor %d in NidbObj", i);
          neigb_list_is_ok = 0;
          break;
        }
        
        else if ( data->nb.nbid[i] != nbid) {
          zkErrorSet_va(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                     "Neighbor %d in list differs", i);
          neigb_list_is_ok = 0;
          break;
        }
      }
      
      if (neigb_list_is_ok) {
        *contents_identical_flag = 1;
      }
    }

    else if (type != FRNIDBOBJ_SERVER) {
      zkErrorSet_va(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                    "Improper type (%u) in NidbObj", type);
    }

    else if (!frNidbObjGetServerParams(err, obj, &iaddr,
                                       &port, &status, &rating,
                                       &region_code, &region, &servop)) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                 "Could not get server params from NidbObj");
    }
        
    else if (data->addr.sin_port != htons(port)) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                 "Ports differ");
    }

    else if (memcmp (&(data->addr.sin_addr), &iaddr, sizeof (struct in_addr)) 
             != 0) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                   "Addresses differ");
    }

    else if (data->rating != (int)rating) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                 "Rating differs");
    }

    else if (data->status != (int)status) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                 "Status differs");
    }

    else if (strcmp (data->region_code, region_code) != 0) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                 "Region code differs");
    }

    else if (strcmp (data->region, region) != 0) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                 "Region differs");
    }

    else if (strcmp (data->servoperator, servop) != 0) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEDATABAD,
                 "Server operator differs");
    }

    else {
      *contents_identical_flag = 1;
    }

    if (region) {
      free (region);
      region = NULL;
    }

    if (buf) {
      free(buf);
      buf = NULL;
    }

    if (obj) {
      free(obj); 
      obj = NULL;
    }
  }

  return ret;
}




