/* $Id: netinfo_object.c,v 1.38.8.8 2000/12/14 17:53:00 jean Exp $ */

static const char * module_id_ = "$Id: netinfo_object.c,v 1.38.8.8 2000/12/14 17:53:00 jean Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#define ZKMOD ZKMFRNETINFO

#include "zks.h"

#include "niobj.h"
#include "nihash.h"
#include "netinfo_data.h"
#include "netinfo_object.h"
#include "zkList.h"
#include "frsec_sig.h"
#include <zkDebug.h>
#include "frNidbObj.h"


/**
 * Free the datum from the ZkList object.  This is a callback function from
 * ZkList.  Clean up the FrNetInfoDatum object and free any allocated memory.
 */
static int frNetInfoDataFree(ZkError err, ZkList l, void *data, void *userData)
{
  const char *func = "frNetInfoDataFree";
  FrNetInfoDatum d = (FrNetInfoDatum) data;
  FrNetInfo ni = (FrNetInfo) userData;

  (void) module_id_; /* Get rid of compile warning */


  if (!d || !ni || !l) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (!frNetInfoDatumFree(err, &d)) {
    return 0;
  }

  return 1;
}


/*
 *
 */

DLLExport int 
frNetInfoCountRecords (
  ZkError err ,
  FrNetInfo fni ,
  size_t * c )
{
  int ret = 1;         /* Function return value */
  int i = 0;           /* Loop index */
  size_t list_sz = 0;  /* Number of entries in each entity list */
  const char * func = "frNetInfoCountRecords";

  if (!fni) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "FrNetInfo pointer is NULL");
    ret = 0;
  } else if (!c) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "count pointer is NULL");
    ret = 0;
  } else {
    *c = 0;
    for (i = 0; i < FRENT_TYPE_MAX; i++) {
      list_sz = 0;
      if (zkListSize(NULL,fni->ent_typeList[i],&list_sz)) {
        *c += list_sz;
      }
    } 
  }
    
  return ret;
}

/**
 * Create a new NetInfo object.
 */
DLLExport int frNetInfoNew(ZkError err, FrNetInfo *nic)
{
  const char *func = "frNetInfoNew";
  int i;


  if (!nic) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  *nic = (FrNetInfo) malloc(sizeof(struct T_FrNetInfo));
  if (!*nic) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENOMEM, "malloc() failed");
    return 0;
  }
  
  /* Create the hash table */
  if (!zkHashNew(err, &(*nic)->h)) {
    return 0;
  }

  /* Set the key freeing function */
  if (!zkHashSetKeyFreeFn(err, (*nic)->h, niHashKeyFree)) {
    return 0;
  }
  if (!zkHashSetKeySize(err, (*nic)->h, niHashKeySize())) {
    return 0;
  }
  if (!zkHashSetTableSize(err, (*nic)->h, 1000)) {
    return 0;
  }  
  if (!zkHashSetUserData(err, (*nic)->h, NULL)) {
    return 0;
  }

  /*
   * Object has not been loaded yet, so we'll set the last update code to 0
   * and its cache data version to 0 as well (this is ordinarily an error 
   * value) the cacheDataVersion will be set when we do a populate.
   */

  (*nic)->lastUpdate = 0;
  (*nic)->nextUpdate = 0;
  (*nic)->updateOffset = 0;
  (*nic)->cacheDataVersion = 0;
  (*nic)->clicons = NULL;
  (*nic)->clicons_data = NULL;
  (*nic)->mxquery_signed_buffer = NULL;
  (*nic)->mxquery_signed_buffer_sz = 0;

  /* Do not create the lists of types yet.  Only when they exist. */
  for (i = 0; i < FRENT_TYPE_MAX; i++) {
    (*nic)->ent_typeList[i] = NULL;
  }

  return 1;
}

/** 
 *  Provide a callback to the frnetinfo object so that it can communicate with
 *  the system console.
 */

DLLExport int
frNetInfoSetClientConsoleCallback (
  ZK_INOUT ZkError err ,
  ZK_INOUT FrNetInfo nic ,
  ZK_IN    ZkClientConsoleCallback clicons ,
  ZK_IN    void * clicons_data )
{
  static const char * func = "frNetInfoSetClientConsoleCallback";
  int ret = 1;

  if (!nic) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,"FrNetInfo is NULL");
    ret = 0;
  } else {
    nic->clicons = clicons;
    nic->clicons_data = clicons_data;
  }

  return ret;
}

/**
 * Free the FrNetInfo object and all of its elements.
 */
DLLExport int frNetInfoFree(ZkError err, FrNetInfo *nic)
{
  const char *func = "frNetInfoFree";
  int i;


  if (!nic || !*nic) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  for (i = 0; i < FRENT_TYPE_MAX; i++) {
    if ((*nic)->ent_typeList[i]) {
      (void) zkListFree(err, &(*nic)->ent_typeList[i]);
    }
  }

  if ((*nic)->h) {
    (void) zkHashFree(err, &(*nic)->h);
  }

  if ((*nic)->mxquery_signed_buffer) {
    free((*nic)->mxquery_signed_buffer);
    (*nic)->mxquery_signed_buffer = NULL;
    (*nic)->mxquery_signed_buffer_sz = 0;
  }

  free(*nic);
  *nic = NULL;

  return 1;
}

/**
 * Insert a new datum into the frNetInfo.  A check will be performed to see
 * if it already exists in the data structure.  Otherwise, insert all
 * corresponding keys to it and add it to the ZkList for that entity type. 
 */
DLLExport int frNetInfoInsert(ZkError err, FrNetInfo nic, FrNetInfoDatum data)
{
  const char *func = "frNetInfoInsert";
  int found;
  FrEntIdExp entity;
  uint32_t id;


  if (!nic || !data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (!niHashInTable(err, nic->h, data, &found)) {
    return 0;
  }

  if (found) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKEEXIST, 
               "hash Key already in stored, Entity exists in hash table, not a bug, removing/updating from table");
  
    if (!(frNetInfoDatumGetId(err, data, &id) &&
          frNetInfoDeleteById(err, nic, id))) {
      zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKERROR, 
               "Unable to delete old record from the hash table");
      return 0;
    }
  }

  if (!niHashInsert(err, nic->h, data)) {
    return 0;
  }

  if (!frNetInfoDatumGetEntity(err, data, &entity)) {
    return 0;
  }
  
  if (!nic->ent_typeList[entity.type]) {
    if (!zkListNew(err, &nic->ent_typeList[entity.type])) {
      return 0;
    }

    if (!zkListSetDataFreeFunction(err, nic->ent_typeList[entity.type], 
                                   frNetInfoDataFree, nic)) {
      return 0;
    }

  }
  if (!zkListInsert(err, nic->ent_typeList[entity.type], data, LIST_TAIL)) {
    return 0;
  }

  return 1;
}


/**
 * Returns a ZkList of entity type.
 */
DLLExport int frNetInfoGetListOf(ZkError err, FrNetInfo nic, ZkList *l,
                                 FrEntIdType type)
{
  const char *func = "frNetInfoGetListOf";
  int valid = 0;



  if (!nic || !l) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  if (!frEntTypeIsValid(err, type, &valid)) {
    return 0;
  }

  if (!valid) {
    zkErrorSet_va(err, func, ZKERR, ZKMFRNETINFO, ZKEPARMBAD,
                  "Invalid type %d", type);
    return 0;
  }

  *l = nic->ent_typeList[type];

  return 1;
}


/**
 * Randomize the order in which the objects are stored in the list
 * of the given entity type. Using this function will destroy the 
 * current list object for the given type. This means all current 
 * references to such lists obtained with frNetInfoGetListOf() will
 * become invalid. 
 * This function assumes frCrypt has been initialized, and that
 * zkRandGetUInt32InRange is functional.
 */
DLLExport
int frNetInfoRandomizeList (
  ZkError err,
  FrNetInfo nic,
  FrEntIdType type)
{
  static const char * func = "frNetInfoRandomizeList";
  int retVal = 1;
  ZkList origList = NULL;
  ZkList newList = NULL;
  void **elemArray = NULL;
  void *curElem = NULL;
  size_t numElem = 0;
  unsigned int remainingElem = 0;
  unsigned int newOffset = 0;
  unsigned int tableOffset = 0;
  unsigned int emptySlotOffset = 0;
  unsigned int curIndex = 0;
  ZkListDataFreeFn freeFunc = 0;
  void *           userData = NULL;

  if (!nic) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "FrNetInfo object (nic) is NULL");
    retVal = 0;
  } 
  
  else if (type >= FRENT_TYPE_MAX) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEPARMBAD,
               "Entity type is invalid");
    retVal = 0;
  }

  else if (nic->ent_typeList[type] != NULL) {
    origList = nic->ent_typeList[type];
    
    if (!zkListSize (err, origList, &numElem)) {
      retVal = 0;
    }

    else if (numElem > 1) { /* No need to shuffle if there is only one element */
      if ( !(elemArray = malloc(sizeof (void *) * numElem)) ) {
        zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENOMEM,
               "Could not allocate space for pointer array");
        retVal = 0;
      }

      else if ( !(memset (elemArray, 0, sizeof (void *) * numElem) &&
                  zkListNew(err, &newList) &&
                  zkListGetDataFreeFunction(err, origList, &freeFunc, &userData) &&
                  zkListSetDataFreeFunction(err, newList, freeFunc, userData)) ) {
        retVal = 0;
      }

      else if (!zkListMoveCursor (err, origList, LIST_HEAD)){
        retVal = 0;
      }

      else {
        remainingElem = numElem;

        while ((remainingElem > 0) && retVal) {
          /* Get current list item in original list */
          if (!zkListGet (err, origList, &curElem, LIST_FORWARDS)) {
            retVal = 0;
          }

          /* Get a random value for the new offset */
          else if (!zkRandGetUInt32InRange(err, 0, remainingElem - 1, &newOffset)) {
			/* Call above can fail on the first run of the client because
			   there might not be any entropy yet.  Because random selection of
			   servers is more a question of load balancing instead of 
			   security, and this only will fail the first time client is run,
			   AdamB has given permission for this careful use of rand().
			   See 5811 */
			srand( (unsigned)time( NULL ) );
			newOffset = (rand() % remainingElem );
          }

          tableOffset = 0;
          emptySlotOffset = 0;

          while ((emptySlotOffset < newOffset) || (elemArray[tableOffset] != NULL)) {
            if (elemArray[tableOffset] == NULL) {
              emptySlotOffset++;
            } 
            tableOffset++;
          }

          elemArray[tableOffset] = curElem;
          remainingElem--;
        }

        if (!retVal) {
          zkListFree(err, &newList);
        }
        else {
          for (curIndex = 0; curIndex < numElem; curIndex++) {
            /* Add current item to new list */
            if (!zkListInsert (err, newList, elemArray[curIndex], LIST_TAIL)) {
              retVal = 0;
            }
          }

          if (!retVal) {
            if (zkListSetDataFreeFunction(err, newList, NULL, NULL)) {
              zkListFree(err, &newList);
            }
          }

          else {
            nic->ent_typeList[type] = newList;
             
            if ( !(zkListSetDataFreeFunction(err, origList, NULL, NULL) &&
                   zkListFree(err, &origList)) ) {
              retVal = 0;
            }
          }
        }
      }
    }
  }

  if (elemArray != NULL) {
    free (elemArray);
    elemArray = NULL;
  }

  return retVal;
}



/**
 * Round robin the order in which the objects are stored in the list
 * of the given entity type, by taking the current list head and putting 
 * it at the tail. Using this function will destroy the current list 
 * object for the given type. This means all current references to such 
 * lists obtained with frNetInfoGetListOf() will become invalid.
 */
DLLExport
int frNetInfoRoundRobinList (
  ZkError err,
  FrNetInfo nic,
  FrEntIdType type)
{
  static const char * func = "frNetInfoRoundRobinList";
  int retVal = 1;
  ZkList origList = NULL;
  ZkList newList = NULL;
  void *firstElem = NULL;
  void *curElem = NULL;
  size_t numElem = 0;
  unsigned int remainingElem = 0;
  ZkListDataFreeFn freeFunc = 0;
  void *           userData = NULL;

  if (!nic) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "FrNetInfo object (nic) is NULL");
    retVal = 0;
  } 
  
  else if (type >= FRENT_TYPE_MAX) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKEPARMBAD,
               "Entity type is invalid");
    retVal = 0;
  }

  else if (nic->ent_typeList[type] != NULL) {
    origList = nic->ent_typeList[type];
    
    if (!zkListSize (err, origList, &numElem)) {
      retVal = 0;
    }

    else if (numElem > 1) { /* No need to round robin if there is only one element */
      if (!zkListMoveCursor (err, origList, LIST_HEAD)){
        retVal = 0;
      }

      /* Get first list item in original list */
      else if (!zkListGet (err, origList, &firstElem, LIST_FORWARDS)) {
        retVal = 0;
      }

      else if ( !(zkListNew(err, &newList) &&
                  zkListGetDataFreeFunction(err, origList, &freeFunc, &userData) &&
                  zkListSetDataFreeFunction(err, newList, freeFunc, userData) ) ) {        
        retVal = 0;
      }

      else {
        remainingElem = numElem - 1;

        while ((remainingElem > 0) && retVal) {
          /* Get current list item in original list */
          if (!zkListGet (err, origList, &curElem, LIST_FORWARDS)) {
            retVal = 0;
          }

          /* Add current item to new list */
          if (!zkListInsert (err, newList, curElem, LIST_TAIL)) {
            retVal = 0;
          }

          remainingElem--;
        }

        if (!retVal) {
          if (zkListSetDataFreeFunction(err, newList, NULL, NULL)) {
            zkListFree(err, &newList);
          }
        }
        else {
          /* Add original first item to new list */
          if (!zkListInsert (err, newList, firstElem, LIST_TAIL)) {
            retVal = 0;
          }

          if (!retVal) {
            if (zkListSetDataFreeFunction(err, newList, NULL, NULL)) {
              zkListFree(err, &newList);
            }
          }

          else {
            nic->ent_typeList[type] = newList;
             
            if ( !(zkListSetDataFreeFunction(err, origList, NULL, NULL) &&
                   zkListFree(err, &origList)) ) {
              retVal = 0;
            }
          }
        }
      }
    }
  }

  return retVal;
}



/*
 *
 */
DLLExport int
frNetInfoStore (
  ZkError err ,
  FrNetInfo fni ,
  void ** buf ,
  size_t * bsz )
{
  static const char * func = "frNetInfoStore";
  unsigned char *tmpbuf = NULL;
  int ret = 0;

  if (!fni) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "FrNetInfo object (fni) is NULL");
  } else if (!buf) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "Data buffer pointer pointer (buf) is NULL");
  } else if (!bsz) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "Data buffer size pointer (bsz) is NULL");
  } else {
    *bsz =  fni->mxquery_signed_buffer_sz + 14;
    if (NULL == (tmpbuf = (unsigned char *) malloc(*bsz))) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENOMEM,
                 "Unable to allocate buffer to hold FrNetInfo");
    } else {
      /*
       * Set the version of the buffer
       */
      tmpbuf[0] = NET_INFO_VERSION_MAJOR;
      tmpbuf[1] = NET_INFO_VERSION_MINOR;

      /*
       * Set the size of the buffer
       */
      tmpbuf[2] = (*bsz & 0xFF000000) >> 24;
      tmpbuf[3] = (*bsz & 0xFF0000) >> 16;
      tmpbuf[4] = (*bsz & 0xFF00) >> 8;
      tmpbuf[5] = (*bsz & 0xFF);

      /*
       * Set the last update time  (time_t)
       */
      tmpbuf[6] = (fni->lastUpdate & 0xFF000000) >> 24;
      tmpbuf[7] = (fni->lastUpdate & 0xFF0000) >> 16;
      tmpbuf[8] = (fni->lastUpdate & 0xFF00) >> 8;
      tmpbuf[9] = (fni->lastUpdate & 0xFF);

      /*
       * Store the cache data version number (a uint32_t)
       */
      tmpbuf[10] = (fni->cacheDataVersion & 0xFF000000) >> 24;
      tmpbuf[11] = (fni->cacheDataVersion & 0xFF0000) >> 16;
      tmpbuf[12] = (fni->cacheDataVersion & 0xFF00) >> 8;
      tmpbuf[13] = (fni->cacheDataVersion & 0xFF);

      /*
       * Store the MxQuery signed buffer
       */
      memcpy(&tmpbuf[14], fni->mxquery_signed_buffer, fni->mxquery_signed_buffer_sz);

      *buf = tmpbuf;
      ret = 1;
    }
  }

  /*
   * Cleanup
   */
  if (!ret) {
    /*
     * An error has occurred
     */
    if (tmpbuf) {
      free(tmpbuf);
    }
    *buf = NULL;
    *bsz = 0;
  }

  return ret;
}



/*
 *  Return pointer to signed buffer and its corresponding size.  Should
 *  only be manipulated with the frSec interface.
 */
DLLExport int
frNetInfoGetSignedBuffer(
  ZkError err,
  FrNetInfo fni,
  uint8_t ** pcbuf,
  size_t * pbsz)
{
  static const char *func = "frNetInfoGetSignedBuffer";
  int nRetCode = 0;

  /*
   *  Validate input pointers
   */

  if (!fni) {
    zkErrorSet(err,func, ZKERR, ZKMFRNETINFO, ZKENULLPTR,
               "FrNetInfo object (fni) is NULL");
  } 
  else if (!pcbuf) {
    zkErrorSet(err,func, ZKERR, ZKMFRNETINFO, ZKENULLPTR,
               "Data buffer pointer (pcbuf) is NULL");
  }
  else if (!pbsz) {
    zkErrorSet(err,func, ZKERR, ZKMFRNETINFO, ZKENULLPTR,
               "Pointer to buffer size is NULL.");
  }
  else if (!(fni->mxquery_signed_buffer && (fni->mxquery_signed_buffer_sz > 0))) {
    zkErrorSet(err,func, ZKERR, ZKMFRNETINFO, ZKEINCOMPLETE,
               "Bad Netinfo signed buffer.");
  }
  else {
    *pcbuf = fni->mxquery_signed_buffer;
    *pbsz  = fni->mxquery_signed_buffer_sz;

    nRetCode = 1;
  }


  return nRetCode;
}


#define  FR_NI_HEADER_SIZE  (14)


/*
 *  Return the update time and cache data version from a raw input buffer
 */

DLLExport int 
frNetInfoValidateHeader(
  ZkError   err,
  uint8_t  *buf,
  size_t    bsz,
  time_t   *pUpdateTime,
  uint32_t *pCacheDataVersion)
{
  static const char *func = "frNetInfoValidateHeader";
  int nRetCode = 0;

  /*
   *  Validate input pointers
   */

  if (!buf) {
    zkErrorSet(err, func,
	       ZKERR, ZKMFRNETINFO, ZKENULLPTR,
               "Data buffer (buf) is NULL");
  }
  else if (bsz < FR_NI_HEADER_SIZE) {
    zkErrorSet(err, func,
	       ZKERR, ZKMFRNETINFO, ZKEINCOMPLETE,
               "Buffer is too small");
  } 
  else if (!pUpdateTime || !pCacheDataVersion) {
    zkErrorSet(err, func,
	       ZKERR, ZKMFRNETINFO, ZKENULLPTR,
               "Output pointer(s) NULL");
  }
  else  if (buf[0] != NET_INFO_VERSION_MAJOR || buf[1] != NET_INFO_VERSION_MINOR) {
    zkErrorSet(err, func,
	       ZKERR, ZKMFRNETINFO, ZKEDATABAD,
	       "Incorrect net info version");
  } 
  else if (bsz != (size_t) ((buf[2] << 24) + (buf[3] << 16) + (buf[4] << 8) + buf[5])) {
    zkErrorSet(err, func,
	       ZKERR, ZKMFRNETINFO, ZKEINCOMPLETE,
	       "Buffer size does not match");
  }
  else {
    /* Success! */
    *pUpdateTime       = (buf[6] << 24) + (buf[7] << 16) + (buf[8] << 8) + buf[9];
    *pCacheDataVersion = (buf[10] << 24) + (buf[11] << 16) + (buf[12] << 8) + buf[13];

    nRetCode = 1;
  }

  return nRetCode;
}




/*
 *  Decompresses and loads the frnetinfo data, but does not verify the NIQS
 *  signature (this is left to the caller).  
 *
 *  N.B.:  Caller should invoke frNetInfoDeleteAll() on failure.
 */


DLLExport int
frNetInfoLoadNoSig (
  ZkError err,
  FrNetInfo fni,
  uint8_t * cbuf,
  size_t bsz)
{
  static const char *func = "frNetInfoLoadNoSig";
  int      nRetCode = 0;
  time_t   lastUpdate;
  uint32_t cacheDataVersion;


  /*
   *  Copy and uncompress the buffer payload.  Fill the netinfo structure from
   *  the uncompressed payload.  Save a duplicate copy of the original, 
   *  compressed payload in the netinfo structure (for signature verification).
   *
   *  Note that the netinfo structure is left in an inconsistent state if
   *  any of these operations fails.  Accordingly, it should be deleted on
   *  failure by the caller.
   *
   *  Resources are allocated and stored within the netinfo structure.  Other
   *  tempoary resources are freed.  The input buffer is unmodified on return.
   */

  if (!fni) {
    zkErrorSet(err, func,
	       ZKERR, ZKMFRNETINFO, ZKENULLPTR,
               "FrNetInfo object (fni) is NULL");
  } 
  else   if (!frNetInfoValidateHeader(err, cbuf, bsz, &lastUpdate, &cacheDataVersion)) {
    zkErrorSet(err, func, 
	       ZKERR, ZKMFRNETINFO, ZKENULLPTR,
               "Buffer header is invalid.");
  }
  else {
    char   *payload = NULL;
    size_t  payloadSize = bsz - FR_NI_HEADER_SIZE;
    ZkLog   log = NULL;


    /* From this point on, input netinfo structure is modified */
    fni->lastUpdate = lastUpdate;
    fni->cacheDataVersion = cacheDataVersion;
      
    /*       * Allocate memory for the MxQuery buffer
     */
    if (NULL == (fni->mxquery_signed_buffer = (unsigned char *)
		 realloc(fni->mxquery_signed_buffer, payloadSize))) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENOMEM,
		 "Unable to allocate buffer to hold mxquery_signed_buffer");
      goto done;
    }

    fni->mxquery_signed_buffer_sz = payloadSize;
    memcpy(fni->mxquery_signed_buffer, &cbuf[FR_NI_HEADER_SIZE], payloadSize);

    /*
     *  Make another copy for decompression
     */
    if (NULL == (payload = (unsigned char *)malloc(payloadSize))) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENOMEM,
		 "Unable to allocate buffer to hold mxquery_signed_buffer");
    }
    else { /* Successful allocation of temporary payload buffer */
      memcpy(payload, &cbuf[FR_NI_HEADER_SIZE], payloadSize);

      if (!frSecSigRemoveInSitu(err, payload, &payloadSize)) {
	zkLogWriteContext(log, err, func, 
			  "call to frSecSigRemoveInSitu failed");
      }
      else {
	/*
	** uncompress the stored buffer
	*/
	char *raw_payload = NULL;

	if (!frNiqsUncompress(err, payload, &raw_payload, &payloadSize) ){
	  zkLogWriteContext(log, err, func, 
			    "call to frNiqsUncompress failed");
	}
	else {
	  /*
	   * Fill the NetInfo structure from the MxQuery buffer
	   */
	  if (!(zkErrorLog(err, &log) && 
		frNetInfoFillNIFromMxQueryBuffer(err,log,fni, raw_payload, payloadSize))) {
	    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKERROR,
		       "Call to frNetInfoFillNIFromMxQueryBuffer failed");
	  }
	  else {
	    /* Success at last! */
	    nRetCode = 1;
	  }

	  free(raw_payload);
	}
      }

      free(payload);
    }
  }


 done:
  /*
   * Cleanup
   */
    
  return nRetCode;
}



/*
 *
 */
DLLExport int
frNetInfoLoad (
  ZkError err ,
  FrNetInfo fni ,
  FrKeyQry kq,
  void * buf ,
  size_t bsz )
{
  static const char * func = "frNetInfoLoad";
  int nRetCode = 0;
  FrKeyQryHost keyhost;

  keyhost.fd = -1;
  

  if (!frNetInfoLoadNoSig(err, fni, (uint8_t *) buf, bsz)) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKERROR,
	       "Could not load net info object");
  }
  else {
    /*
     * Find key query server in net info object
     */
    ZkList currentlist;

    if (!frNetInfoGetListOf(err, fni, &currentlist, 
			    FRENT_TYPE_KEYQRY_SRV)) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKERROR,
		 "Could not get key query server list in net info object");
      goto done;
    }
    else if (!zkListMoveCursor(err, currentlist, LIST_HEAD)) {
      zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKERROR,
		 "Problem with key query server list");
      goto done;
    }
    else {
      FrNetInfoDatum data = NULL;

      if (!zkListGet(err, currentlist, (void **) &data,LIST_STAY)) {
	zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKERROR,
		   "Key query server list is empty");
	goto done;
      }

      /*
       * Prepare key host object for signature checks
       */
      else if (!frNetInfoDatumGetSockaddr(err, data, &(keyhost.host))) {
	zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKERROR,
		   "Could not get key server socket address from record");
	goto done;
      }
      else if ((keyhost.fd = socket(AF_INET,SOCK_DGRAM,0)) < 0) {
	zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKERROR,
		   "Could not get file descriptor for key server socket");
	goto done;
      }
      
      /*
       * Verify the signature of the MxQuery response
       */
      else if (!frNetInfoCheckSig(err, keyhost, kq, fni->mxquery_signed_buffer, 
				  fni->mxquery_signed_buffer_sz)) {
	zkErrorSet(err,func,ZKERR, ZKMFRNETINFO, ZKERROR,
		   "Sig verification failure for MxQuery buffer");
	goto done;
      }
      else {
	nRetCode = 1;
      }
    }
  }

 done:
  /*
   * Cleanup
   */
#ifdef WIN32
  if (-1 != keyhost.fd) closesocket(keyhost.fd);
#else
  if (-1 != keyhost.fd) close(keyhost.fd);
#endif

  
  if (!nRetCode) {
    /*
     * An error has occurred
     */
    frNetInfoDeleteAll(NULL, fni);
  }
  
  return nRetCode;
}


/** 
 *  Return the last time the FrNetInfo object was updated from the NIQS
 *
 */

DLLExport int
frNetInfoLastUpdate (
  ZkError err ,
  FrNetInfo fni ,
  time_t * lastUpdate )
{
  static const char * func = "frNetInfoLastUpdate";
  int ret = 0;

  if (!fni) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "FrNetInfo object is NULL");
  } else if (!lastUpdate) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "lastUpdate pointer (time_t) is NULL");
  } else {
    *lastUpdate = fni->lastUpdate;
    ret = 1;
  }
  return ret;
}


/** 
 *  Return the NIQS cache data version of the FrNetInfo object
 *
 */

DLLExport int
frNetInfoCacheDataVersion(
  ZkError    err,
  FrNetInfo  fni ,
  uint32_t  *pCacheDataVersion)
{
  static const char *func = "frNetInfoCacheDataVersion";
  int ret = 0;

  if (!fni) {
    zkErrorSet(err, func, 
               ZKERR, ZKMFRNETINFO, ZKENULLPTR,
               "FrNetInfo object is NULL");
  } 
  else if (!pCacheDataVersion) {
    zkErrorSet(err, func,
               ZKERR, ZKMFRNETINFO, ZKENULLPTR,
               "cacheDataVersion pointer is NULL");
  } 
  else {
    *pCacheDataVersion = fni->cacheDataVersion;
    ret = 1;
  }

  return ret;
}


/*
 *
 */

DLLExport int
frNetInfoUpdateOffset(
  ZK_INOUT ZkError err ,
  ZK_IN    FrNetInfo fni ,
  ZK_OUT   time_t * offset )
{
  static const char * func = "frNetInfoUpdateOffset";
  int ret = 0;

  if (!fni) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "FrNetInfo object is NULL");
  } else if (!offset) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "offset pointer (time_t) is NULL");
  } else {
    *offset = fni->updateOffset;
    ret = 1;
  }

  return ret;
}

/*
 *
 */

DLLExport int
frNetInfoSetUpdateOffset (
  ZK_INOUT ZkError err ,
  ZK_INOUT FrNetInfo fni ,
  ZK_IN    time_t offset ) 
{
  static const char * func = "frNetInfoSetUpdateOffset";
  int ret = 0;

  if (!fni) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "FrNetInfo object is NULL");
  } else {
    fni->updateOffset = offset;
    if (fni->nextUpdate == 0) {
      fni->nextUpdate = time(NULL) + offset;
    }
    ret = 1;
  }
  
  return ret;
}

/** 
 *
 */

DLLExport int
frNetInfoNextUpdate (
  ZK_INOUT ZkError err ,
  ZK_IN    FrNetInfo fni ,
  ZK_OUT   time_t * nextUpdate )
{
  static const char * func = "frNetInfoNextUpdate";
  int ret = 0;

  if (!fni) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "FrNetInfo object is NULL");
  } else if (!nextUpdate) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "nextUpdate pointer (time_t) is NULL");
  } else {
    *nextUpdate = fni->nextUpdate;
    ret = 1;
  }
  return ret;
} 


/*
 * Set the absolute time at which the next full set of NIQS information
 * will be downloaded. 
 */

DLLExport int
frNetInfoSetNextUpdate (
  ZK_INOUT ZkError err ,
  ZK_INOUT FrNetInfo fni ,
  ZK_IN    time_t nextUpdate )
{
  static const char * func = "frNetInfoSetNextUpdate";
  int ret = 0;

  if (!fni) {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,
               "FrNetInfo object is NULL");
  } else {
    fni->nextUpdate = nextUpdate;
    ret = 1;
  }

  return ret;
}

/*
 *
 */

DLLExport int
frNetInfoCullEntities (
  ZkError err ,
  FrNetInfo fni ,
  uint32_t * entid ,
  size_t entid_sz )
{
#define WINMSG_SZ 128
  static const char * func = "frNetInfoCullEntities";
  FrNetInfoDatum data = NULL; 
  int cull = 0;               /* Should we cull the entity? */
  size_t n = 0;               /* Loop index for all of the lists */
  int ret = 1;                /* Function return code */
  uint32_t entity_id = 0;     /* Entity ID of a datum we are analyzing */
  char winmsg[WINMSG_SZ];     /* For windows tracing messages */

  if (fni && entid) {
    
    /*
     * First we mark all entities as culled.
     */

    for (n = 0; n < FRENT_TYPE_MAX; n++) {
      if (fni->ent_typeList[n]) {
        zkListMoveCursor(err,fni->ent_typeList[n],LIST_HEAD);
        while (zkListGet(err,fni->ent_typeList[n],
                         (void **) &data,LIST_FORWARDS)) {
          frNetInfoDatumSetCull(err,data,1);
        }
      }
    }

    /*
     * Now go through the entity list and turn off the cull field in the
     * matching entries in the frNetInfo object.  Not all entities in the
     * available list from the NIQS will be found in the frNetInfo object.
     */

    for (n = 0; n < entid_sz; n++) {
      if (frNetInfoSearchById(err,fni,entid[n],&data)) {
        frNetInfoDatumSetCull(err,data,0);
      }
    }

    /*
     * Now go through the entity lists again and delete the ones which
     * are still marked as cull.  They were not listed in the available
     * list of entities which was returned from the NIQS
     */

    for (n = 0; n < FRENT_TYPE_MAX; n++) {
      if (fni->ent_typeList[n]) {
        zkListMoveCursor(err,fni->ent_typeList[n],LIST_HEAD);
        while (zkListGet(err,fni->ent_typeList[n],
                         (void **) &data,LIST_FORWARDS)) {
          frNetInfoDatumGetCull(err,data,&cull);
          if (cull) {
            if (frNetInfoDatumGetId(err,data,&entity_id)) {
              zkDebug("***************************** DELETING %u",entity_id);
              snprintf(winmsg,WINMSG_SZ,"FRNETINFO: Deleting entity ID %d",
                       entity_id);
              ClientTrace(func,winmsg);
              CONSOLE_MSG(fni,winmsg);
              frNetInfoDeleteById(err,fni,entity_id);
            } else {
			  ClientTrace(func,
                        "FRNETINFO: Attempted to delete entity with no id");
              CONSOLE_MSG(fni,"Attempted to delete entity with no id");
            }
          } 
        }
      }
    }

  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,NULL);
    ret = 0;
  }
  return ret;
}

/*
 * This function will delete all entries in the frNetInfo object.  It is
 * essentially the code from frNetInfoFree and frNetInfoNew smooshed 
 * together.  You cant just destroy the object and make a new one because
 * the code which calls the frNetInfoPopulate() function must not know
 * that this has happened. (The populate functions are the only ones which
 * call this function)
 */

DLLExport int
frNetInfoDeleteAll (
  ZkError err ,
  FrNetInfo fni )
{
  static const char * func = "frNetInfoDeleteAll";
  int ret = 0;
  int i = 0;

  if (fni) {
 
    /* Destroy the lists */

    for (i = 0; i < FRENT_TYPE_MAX; i++) {
      if (fni->ent_typeList[i]) {
        (void) zkListFree(err, &(fni->ent_typeList[i]));
      }
    }

    /* Destroy the hash table */

    if (fni->h) {
      (void) zkHashFree(err, &(fni->h));
    }

    /* Now try to recreate all of the internal structures... */
   
    /* Create the hash table */
    if (!zkHashNew(err, &(fni->h))) {
      return 0;
    } 
    /* Set the key freeing function */
    if (!zkHashSetKeyFreeFn(err, (fni->h), niHashKeyFree)) {
      return 0;
    }
    if (!zkHashSetKeySize(err, (fni->h), niHashKeySize())) {
      return 0;
    }
    if (!zkHashSetTableSize(err, (fni->h), 1000)) {
      return 0;
    } 
    if (!zkHashSetUserData(err, (fni->h), NULL)) {
      return 0;
    }

    fni->lastUpdate = 0;
    fni->cacheDataVersion = 0;

    /* Do not create the lists of types yet.  Only when they exist. */
    for (i = 0; i < FRENT_TYPE_MAX; i++) {
      fni->ent_typeList[i] = NULL;
    }

    if (fni->mxquery_signed_buffer) {
      free(fni->mxquery_signed_buffer);
      fni->mxquery_signed_buffer = NULL;
    }
    fni->mxquery_signed_buffer_sz = 0;

    ret = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRNETINFO,ZKENULLPTR,NULL);
  }

  return ret;
}


/*
 *  Convert a struct in_addr, with a port, to a struct sockaddr_in.  The
 *  `port' is assumed to be in host byte order.
 */
static int sAddrInFromInAddr(struct sockaddr_in *saddr, struct in_addr *iaddr, uint16_t port)
{
  memset(saddr, 0, sizeof *saddr);

  saddr->sin_family = AF_INET;
  saddr->sin_port = htons(port);
  saddr->sin_addr = *iaddr;

  return 1;
}


/*
** Fill the NetInfo structure with data from the MxQuery buffer
*/
DLLExport int
frNetInfoFillNIFromMxQueryBuffer(ZkError err, ZkLog log, FrNetInfo ni, void *lbuf, size_t lbsz)
{
#define WINMSG_SZ 128
  static const char * fn = "frNetInfoFillNIFromMxQueryBuffer";

  FrNetInfoDatum nid = 0;

  FrNidbObj obj = 0;
  FrNidbObjType type;

  FrNidbObj listObj;
  uint32_t  nbDesc;
  size_t    idx;

  FrEntIdent eid;
  FrEntIdExp entity;

  char *region = 0;         /* This needs to be freed. Because of the goto's I have
		               to change the scope of this pointer, to be able to do a
		               free at the end */
  const char *region_code = 0;
  const char *servop = 0;

  size_t nbs;               /* the number of an AIPs neighbours */
  struct in_addr iaddr;     /* server address */
  struct sockaddr_in saddr; /* datum uses this, rather than struct in_addr */
  uint16_t dhport;          /* AIP Diffie-Hellman port */
  uint16_t port;            /* main server port */
  uint32_t nbid;
  uint32_t rating;          /* latest rating */
  uint32_t status;          /* latest status of the aip */
  ZkList srvlist = 0;

  char winmsg[WINMSG_SZ];   /* For creating windows tracing messages */
  size_t i;
  int ret = 0;


  /*
  ** This seems a safe place to do the DBO subsystem initialization
  ** (this function is called from other functions that do not int the DB...)
  */
  if( ! frDBODriverIsInitialized(err) && ! frDBODriverInitAll(err)) {
      zkLogWriteContext(log, err, fn, "call to frDBODriverInitAll failed");
  }

  /*
  ** extract the NidbObj from the buffer
  */
  if ( ! frNidbObjUnpack(err, lbuf, lbsz, &obj)) {
    snprintf(winmsg,WINMSG_SZ,
	     "FRNETINFO: Failed to unpack DBO");
    ClientTrace(fn,winmsg);
    CONSOLE_MSG(ni,winmsg);
    zkLogWriteContext(log, err, fn, "call to frNidbObjUnpack failed");
    goto done;
  }

  /*
  ** make sure we received a list
  */
  if ( ! frNidbObjGetType(err, obj, &type) || type != FRNIDBOBJ_LIST_RESP){
    snprintf(winmsg,WINMSG_SZ, "Improper reponse type (%d)", type);
    ClientTrace(fn,winmsg);
    CONSOLE_MSG(ni,winmsg);
    goto done;
  }

  /*
  ** get number of element in list
  */
  if ( ! frNidbObjGetListParams(err,obj, &nbDesc)){
    ClientTrace(fn,"FRNETINFO: Failed to retrieve info from NidbObj");
    CONSOLE_MSG(ni,"Failed to retrieve info from NidbObj");
    zkLogWriteContext(log, err, fn, "call to frNidbObjGetListParams failed");
    goto done;
  }
  
  for (idx=0; idx < nbDesc; idx++) {
    /*
    ** position a pointer on a list element
    */
    if( ! frNidbObjGetListRespAtIdx(err,obj, idx, &listObj)){
      ClientTrace(fn,"FRNETINFO: Failed to retrieve info from NidbObj ");
      CONSOLE_MSG(ni,"Failed to retrieve info from NidbObj");
      zkLogWriteContext(log, err, fn, "call to frNidbObjGetListRespAtIdx failed");
      goto done;
    }

    /*
    **  Extract the information we need
    */
    if ( ! frNidbObjGetEntIdent(err, listObj, &eid)  ||
	 ! frNidbObjGetEntity(err, listObj, &entity) ||
	 ! frNidbObjGetType(err, listObj, &type)) {
      ClientTrace(fn,"FRNETINFO: Failed to retrieve info from NidbObj ");
      CONSOLE_MSG(ni,"Failed to retrieve info from NidbObj");
      goto done;
    }

    /*
    ** copying it to a new FrNetInfoDatum,
    ** which is entered into the netinfo object.
    */
    if ( ! frNetInfoDatumNew(err, &nid) ||
	      ! frNetInfoDatumSetId(err, nid, eid) ||
	      ! frNetInfoDatumSetEntity(err, nid, entity)) {
      ClientTrace(fn,"FRNETINFO: Failed to create a new FrNetInfoDatum structure");
      CONSOLE_MSG(ni,"Failed to create a new FrNetInfoDatum structure");
      goto done;
    }
    
    /*
    ** information to extract depends of the entity type
    */
    if ((entity.type==FRENT_TYPE_AIP)||(entity.type==FRENT_TYPE_COREAIP) ) {
      if ( ! (frNidbObjGetAipParams(err, listObj, &iaddr, &dhport,
				    &port, &status, &rating,
				    &region_code,
				    &region, &servop, &nbs) &&
	      sAddrInFromInAddr(&saddr, &iaddr, port) &&
	      frNetInfoDatumSetSockaddr(err, nid, saddr) &&
	      frNetInfoDatumSetDifHelPort(err, nid, dhport) &&
	      frNetInfoDatumSetRating(err, nid, rating) &&
	      frNetInfoDatumSetStatus(err, nid, status) &&
	      frNetInfoDatumSetRegionCode(err, nid, region_code) &&
	      frNetInfoDatumSetRegion(err, nid, region) &&
	      frNetInfoDatumSetOperator(err, nid, servop) )) {
	goto done;
      }
      
      if(region) {
	FREEIT(region);
      }
      
      /* Get the neighbours into the list too */
      for (i = 0; i < nbs; i++) {
	if (!(frNidbObjGetAipNeighbour(err, listObj, i, &nbid) &&
	      frNetInfoDatumAddNb(err, nid, nbid))) {
	  goto done;
	}
      }
    }
    else if (type == FRNIDBOBJ_SERVER) {
      if ( ! (frNidbObjGetServerParams(err, listObj, &iaddr, &port,
				       &status, &rating, &region_code,
				       &region, &servop) &&
	      sAddrInFromInAddr(&saddr, &iaddr, port) &&
	      frNetInfoDatumSetSockaddr(err, nid, saddr) &&
	      frNetInfoDatumSetRating(err, nid, rating) &&
	      frNetInfoDatumSetStatus(err, nid, status) &&
	      frNetInfoDatumSetRegionCode(err, nid, region_code) &&
	      frNetInfoDatumSetRegion(err, nid, region) &&
	      frNetInfoDatumSetOperator(err, nid, servop) )) {
	goto done;
      }
      if(region) {
	FREEIT(region);
      }
    }
    else {
      ClientTrace(fn,"FRNETINFO: Received info for an unsupported type of entity");
      CONSOLE_MSG(ni,"Received info for an unsupported type of entity");
      goto done;
    }
    
    /*
    ** insert/replace data in ni object
    */
    snprintf(winmsg,WINMSG_SZ,"Inserting/replacing entity %d/%s in FrNetInfo",
	     entity.type,entity.name);
    ClientTrace(fn,winmsg);
    CONSOLE_MSG(ni,winmsg); 
    
    if ( ! frNetInfoInsert(err, ni, nid)) {
      ClientTrace(fn,"FRNETINFO: Failed to insert/replace the entity");
      CONSOLE_MSG(ni,"Failed to insert/replace the entity"); 
      goto done;
    }
    
    nid = 0; /* it's been inserted... */
  }

  /*
   *  bug: we could check the internal consistency of our object, here.  That
   *  is, check that all AIP neighbours are in our table, and that if A is a
   *  neighbour of B then B is a neighbour of A, etc.
   */
  ret = 1;
  ni->lastUpdate = time(NULL); 

done:
  
  /* Cleanup */
  if (nid) { 
    frNetInfoDatumFree(0, &nid); 
  }
  if (srvlist) { 
    zkListFree(0, &srvlist); 
  }
  if (obj) {
    frNidbObjListFree(err, &obj);
  }

  return ret;
}

/*
 *  Verify a buffer produced by frSecSignData(), making any necessary key
 *  queries.  We check that the signer is the NIQS.
 */
DLLExport int frNetInfoCheckSig(ZkError err, FrKeyQryHost kqh, FrKeyQry kq, void *buf, 
                    size_t bsz)
{
  FrEntIdExp signer;
  FrCryptSig sigkey = 0;
  const char *fn = "frNetInfoCheckSig";
  int ret = 0;
  unsigned int kv; /* signature key version */
  
  if ( ! (frSecSigGetEntity(err, buf, bsz, &signer) &&
          frSecSigGetVersion(err, buf, bsz, &kv))) {

  } else if (signer.type != FRENT_TYPE_NIQS) {
      zkErrorSet_va(err, fn, ZKERR, ZKMOD, ZKEDATABAD,
                    "unexpected enity type (%d) in NIQS buffer",
                    signer.type);

  } else {

#ifndef WIN32
#warning "Hardcoded retries and timeouts. Is this correct?"
#endif
    
    int retries = 5, timeout = 3;
    int valid;

    if ( ! (frKeyQryFetchPubKey(err, kq, kv, FRCRYPT_KEYTYPE_SIGNING,
                                signer, kqh, retries, timeout, &sigkey) &&
            frSecSigVerify(err, sigkey, buf, bsz, &valid))) {
    } else if ( ! valid) {
      zkErrorSet_va(err, fn, ZKERR, ZKMOD, ZKEDATABAD,
                    "invalid signature on buffer from NIQS");
    } else {
      ret = 1;
    }
  }

  return ret;
}


/*
**------------------------------------------------------------------------------
** private functions
**------------------------------------------------------------------------------
*/

/*
** delete from lists the entity corresponding to the given entId
** 
** NOTE: this is not an optimal way of dooing it, but we miss the type from the datum...
*/
int
frNetInfoListDeleteById(
    ZkError err,
    FrNetInfo ni,
    uint32_t entId
)
{
    const char *func = "frNetInfoListDeleteById";
    int ret = 1;

    ZkLog log;

    ZkList list = NULL;

    size_t n;
    size_t sz;
    size_t i;

    FrNetInfoDatum data;
    uint32_t id;
    int found = 0;


    /* get the log pointer */
    if( ! zkErrorLog(err, &log )) {
        zkErrorSet(err,func,ZKERR,ZKMOD,ZKERROR, "call to zkErrorLog failed");
        ret = 0;
    }
    else {

        for (n = 0; (ret==1) && (n < FRENT_TYPE_MAX) ; n++) {

            /* the list could be empty for that type; just skip it */
            if( ni->ent_typeList[n] == NULL ) {
                continue;
            }

            list = ni->ent_typeList[n];

            /* 
            ** scan the list, searching for entID
            */

            if (!zkListMoveCursor(err, list, LIST_HEAD)) {
                zkLogWriteContext(log, err, func, "call to zkListMoveCursor failed");
                ret = 0;
            }
            else if (!zkListSize(err, list, &sz)) {
                zkLogWriteContext(log, err, func, "call to zkListSize failed");
                ret = 0;
            }
            else {

                for (i = 0; (ret==1) && (i < sz) ; i++) {

                    if (!zkListGet(err, list, (void *) &data, LIST_STAY)) {
                        zkLogWriteContext(log, err, func, "call to zkListGet failed");
                        /*
                        ** it could fails because we're at list's end,
                        ** don't set ret = 0, let's continue
                        */
                    }
                    else if (!frNetInfoDatumGetId(err, data, &id)) {
                        zkLogWriteContext(log, err, func, "call to frNetInfoDatumGetId failed");
                        ret = 0;
                    }
                    else if ( id == entId) {
                        if (!zkListDelete(err, list, LIST_FORWARDS)) {
                            zkLogWriteContext(log, err, func, "call to zkListDelete failed");
                            ret = 0;
                        }
                        else {
                            /* we can stop since we found it */
                            found = 1;
                            break;
                        }
                    } 
                    else {
                        if (!zkListMoveCursor(err, list, LIST_FORWARDS)) {
                            zkLogWriteContext(log, err, func, "call to zkListMoveCursor failed");
                            ret = 0;
                        }
                    }
                }
            }

            if( found ) {
                break;
            }
        }
    }

    return ret;
}


/*
**------------------------------------------------------------------------------
** internal test functions
**------------------------------------------------------------------------------
*/

#ifdef FREEDOM_DEVELOPMENT_BUILD

/**
 * Dump the netinfo object for specific entity types, to a file pointer.
 */
DLLExport int frNetInfoDump(ZkError err,
                            FrNetInfo nic,
                            FrEntIdType entityType,
                            FILE * outFile)
{
  const char *func = "frNetInfoDump";
  FrNetInfoDatum data = NULL;


  if (!nic || !outFile || !(nic->ent_typeList[entityType])) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  fprintf(outFile, "Dumping frNetInfo\n");

  zkListMoveCursor(err, nic->ent_typeList[entityType], LIST_HEAD);

  while (zkListGet(err,
                   nic->ent_typeList[entityType],
                   (void *) &data,
                   LIST_FORWARDS)) {

    fprintf(outFile,
            "\nEntity\n"
            "Entity id:     %ld\n"
            "Entity type:   %ld\n"
            "Entity name:   %s\n"
            "Status:        %s\n"
            "Rating:        %d\n"
            "Region:        %s\n"
            "Cull:          %d\n"
            "Valid:         %d\n"
            ,
            (long) data->id,
            (long) data->entid.type,
            data->entid.name,
            data->status ? "up" : "down",
            data->rating,
            data->region,
            data->cull,
            data->valid
            );
      }

  return 1;
}

/**
 * Dump the netinfo object for specific entity types, to a log.
 */
DLLExport int frNetInfoDumpInLog(ZkError err,
                                 FrNetInfo nic,
                                 FrEntIdType entityType,
                                 ZkLog log)
{
  const char *func = "frNetInfoDumpInLog";
  FrNetInfoDatum data = NULL;


  if (!nic || !log || !(nic->ent_typeList[entityType])) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  zkLogWrite_va(log,"Dumping frNetInfo\n");

  zkListMoveCursor(err, nic->ent_typeList[entityType], LIST_HEAD);

  while (zkListGet(err,
                   nic->ent_typeList[entityType],
                   (void *) &data,
                   LIST_FORWARDS)) {

    zkLogWrite_va(log,
                  "\nEntity\n"
                  "Entity id:     %ld\n"
                  "Entity type:   %ld\n"
                  "Entity name:   %s\n"
                  "Status:        %s\n"
                  "Rating:        %d\n"
                  "Region:        %s\n"
                  "Cull:          %d\n"
                  "Valid:         %d\n"
                  ,
                  (long) data->id,
                  (long) data->entid.type,
                  data->entid.name,
                  data->status ? "up" : "down",
                  data->rating,
                  data->region,
                  data->cull,
                  data->valid
                  );
      }

  return 1;
}

#endif /* FREEDOM_DEVELOPMENT_BUILD */
