#ifndef NIOBJ_H_
#define NIOBJ_H_
/* $Id: niobj.h,v 1.22.8.1 2000/10/11 13:18:43 phil Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"
#include <time.h>

#ifndef WIN32
#include <netinet/in.h>
#include <sys/socket.h> 
#else
#include <winsock2.h>
#endif

#include "zkIntTypes.h"
#include "zkHash.h"
#include <sys/types.h>
#include "zkError.h"
#include "zkList.h"
#include "zkclientconsole.h"
#include "frEnt.h"
#include "frNetInfo.h"

#define NET_INFO_VERSION_MAJOR 0x01
#define NET_INFO_VERSION_MINOR 0x02

typedef 
enum { 
  FRNETINFO_NONE, 
  FRNETINFO_ID,                 /* Key equality on the ID */
  FRNETINFO_ENTITY,             /* Key equality on the entity */
  FRNETINFO_ALL,                /* Key equality on the entire key */
  FRNETINFO_SADDR,              /* Key equality on the ip and port numbers */
  FRNETINFO_MAX                 /* 
                                 * Place holder.  Do NOT delete from last 
                                 * position in this enumerated type.
                                 */ 
} FrNetInfoKeyType;

typedef 
struct T_FrNetInfoKey {
  FrNetInfoKeyType key;         /* Search on equality on this type */
  uint32_t id;                  /* Entity ID from NIQS */
  FrEntIdExp entid;             /* Entity */
  struct sockaddr_in saddr;     /* IP addr/Port of the entity */
} *FrNetInfoKey;

typedef
struct {
  size_t numnb;                 /* Number of neighbours */
  uint32_t *nbid;               /* Neighbour by ID */
} NbData;

struct T_FrNetInfoDatum {
  /* 
   * The key data is stored in the Datum too.  This way I can get access to 
   * all the keys from one search.
   */   
  char * signed_buffer;         /* Signed buffer containing entity received from NIQS */
  size_t signed_buffer_sz;      /* Size of signed buffer */
  
  uint32_t id;                  /* Entity ID */
  FrEntIdExp entid;             /* Entity */

  int status;                   /* Up/Down */
  int rating;                   /* Rating of usefullness */
  struct sockaddr_in addr;      /* IP addr/Port of the entity */
  char *region;                 /* The region of this entity */
  char *region_code;            /* The region_code [CCRR] of this entity */
  char *servoperator;           /* The operator of this entity */
  uint16_t dhport;              /* Diffie-Hellman port, for AIPs only */
  NbData nb;                    /* Neighbour data */
  
  int cull;                     /* Setting this to a non-zero value will cause
                                 * the datum to be deleted when updating the
                                 * frNetInfo with the list of currently
                                 * available entities.
                                 */

  int valid;                    /* 
                                 * Marks this entry as deleted from the hash
                                 * table, but needs to be deleted in the
                                 * ZkList.  This way I don't have to search
                                 * the list when making a deletion.
                                 */ 
};

struct T_FrNetInfo {
  time_t lastUpdate;            /* Last time the FrNetInfo object was updated */
  time_t updateOffset;          /* When a complete reload is forced, the 
                                 * nextUpdate field will be reset to the 
                                 * current time plus this value 
                                 */
  time_t nextUpdate;            /* Time after which a completely fresh set 
                                 * of data must be loaded (as opposed to an
                                 * incremental update)  If this value is 0,
                                 * this capability is disabled.
                                 */
  uint32_t cacheDataVersion;    /* Version number of the NIQS database */
  ZkHash h;                     /* Hash table based on ID/Entity */
  ZkList ent_typeList[FRENT_TYPE_MAX]; /* Entity type list of objects */
  ZkClientConsoleCallback clicons;  /* To talk to the client console */
  void * clicons_data;              /* Extra data pointer */

  char * mxquery_signed_buffer;     /* Signed buffer containing mxquery info received from NIQS */
  size_t mxquery_signed_buffer_sz;  /* Size of mxquery signed buffer */
};

/*
 * This macro is used to send message to the Freedom windows client message
 * console so that a user doesnt go nuts waiting without knowing what is
 * going on.  It takes a FrNetInfo pointer and a message string.
 */


#define CONSOLE_MSG(fni,msg) { \
  if (fni->clicons) { \
    (fni->clicons)(NULL,"NETINFO",ClientConsoleMessage,CONS_DEBUG, \
                   "",msg,NULL,fni->clicons_data); \
  } }

#define CONSOLE_MSG_IMPORTANT(fni,msg) { \
  if (fni->clicons) { \
    (fni->clicons)(NULL,"NETINFO",ClientConsoleMessage,CONS_IMPORTANT, \
                   "",msg,NULL,fni->clicons_data); \
  } }


/*
**------------------------------------------------------------------------------
** private function prototypes
**------------------------------------------------------------------------------
*/

/*
** delete from lists the entity corresponding to the given entId
*/
extern int
frNetInfoListDeleteById(ZkError err,
                        FrNetInfo ni,
                        uint32_t entId);


#endif /* NIOBJ_H_ */
