/*
 *  $Id: pop.c,v 1.47.8.14 2000/12/14 17:53:00 jean Exp $
 */

static const char * module_id_ = "$Id: pop.c,v 1.47.8.14 2000/12/14 17:53:00 jean Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#define ZKMOD ZKMFRNETINFO

#include "zks.h"

#include "niobj.h"
#include "nihash.h"
#include "netinfo_data.h"
#include "frKeyQry.h"
#include "frNetworkDefaults.h"
#include "frTokenType.h"
#include "frDBOFuncs.h"
#include "frNidbObj.h"
#include "frNiqs.h"
#include "frNiqsTokenInfo.h"
#include "frSec.h"
#include "zkError.h"
#include "zkList.h"
#include "zkipClient.h"
#include "netinfo_object.h"


static int frNetInfoDoSelectiveStatusUpdateSince( ZkError err, FrNetInfo ni, ZkipClient cli, FrKeyQry kq,
                                                  FrEntIdType *host_ent, unsigned int he_sz, time_t since);

static int getKeyQueryHostSocket (ZkError err, ZkipClient cli, FrNetInfo fni , FrKeyQryHost *kqh_p)
{
  static const char *fn = "getKeyQueryHostSocket";
  struct sockaddr saddr;
  int retcode = 0;
  int do_lookup = 1;
  
  if ( ! kqh_p ) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "khq_p is null");
  } else {    
    kqh_p->fd = -1;
    
    if ((kqh_p->fd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
      zkErrorSet_va(err, fn, ZKERR, ZKMOD, errno,
                  "can't create key query socket: %s", strerror(errno));
    } else { 
  
      /*
       * If we already know who the keyquery server is, dont bother asking the
       * NIQS
       */

      if (fni) {
        ZkList list = NULL;
        FrNetInfoDatum datum = NULL;
        if (frNetInfoGetListOf(err,fni,&list,FRENT_TYPE_KEYQRY_SRV) &&
            zkListMoveCursor(err,list,LIST_HEAD) &&
            zkListGet(err,list,(void **) &datum,LIST_FORWARDS) &&
            frNetInfoDatumGetSockaddr(err,datum,(struct sockaddr_in *)&saddr)) {
          do_lookup = 0;
          retcode = 1;
        }
      }

      if (do_lookup) {

        /*
         *  First, get the address of the key query server, so that we can do a
         *  lookup on the NIQS's key.
         */

        if ( frNiqsGetKeyQryAddr(err, cli, &saddr)) {
          retcode = 1;
        }
      }

      if (retcode) {
        memcpy(&(kqh_p->host), &saddr, sizeof (kqh_p->host));
      }
    }
  }
  return retcode;
}


#if 0

/*
** must be eliminated
*/

/*
** BUG - BUG - BUG
** CONSOLIDATION should clean that fct and make it call frNetInfoPopSince
** since it's been define as a subset of this function
*/
static int frNetInfoPopulateOrUpdateSince(
  ZkError err,
  FrNetInfo ni,
  ZkipClient cli,
  FrKeyQry kq,
  FrEntIdType *host_ent,
  unsigned int he_sz,
  time_t since,
  int update_flag)
{
#define WINMSG_SZ 128
  FrKeyQryHost kqh;
  FrNetInfoDatum nid = 0;
  FrNidbObj obj = 0;
  char *region = 0; /* This needs to be freed. Because of the goto's I have
                       to change the scope of this pointer, to be able to do a
                       free at the end */
  ZkList srvlist = 0;
  const char *fn = "frNetInfoPopulateOrUpdateSince";
  int ret = 0;
  size_t lbsz; /* amount of data in lbuf */
  void *srvbuf = 0;
  void *sigdbuf = 0;
  void *lbuf = 0; /* signed buffer listing all servers */
  uint32_t cacheDataVersion = 0;  /* From the NIQS */
  char winmsg[WINMSG_SZ];   /* For creating windows tracing messages */
  uint32_t * entid = NULL;   /* We may need to load the available list */
  size_t entid_sz = 0;       /* Holds the size of the list */
  


  if ( ! ni) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "ni is null");
    return 0;
  }
  if ( ! cli) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "cli is null");
    return 0;
  }
  if ( ! kq) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "kq is null");
    return 0;
  }
  /*
   * This seems a safe place to do the DBO subsystem initialization
   */
  if (( ! frDBODriverIsInitialized(err)) && ( ! frDBODriverInitAll(err))) {
    goto done;
  }

  if ( !  getKeyQueryHostSocket (err, cli, ni, &kqh) ) {
    return 0;
  }

  /*
   * Get the current cache data version from the NIQS.  If we are doing an
   * update and this number has changed, nuke the contents of the frNetInfo
   * and reload everything.  If the GetCacheDataVersion() command fails (likely
   * because the NIQS did not have the appropriate command, a side effect
   * of the server transitions during the beta period)  We'll force a complete
   * reload of the cache.  If the cause of the failure was a signature error,
   * the next buffer which is read from the server will trigger the
   * appropriate response.
   */

  if (!frNiqsGetCacheDataVersion(err,cli,kqh,kq,&cacheDataVersion)) {
    zkErrorSet(err,fn,ZKWARN,ZKMOD,ZKERROR,
               "WARNING:  Failed to get cache data version, flushing cache");
    ni->cacheDataVersion = 0;
    cacheDataVersion = 1;
  }

  /*
   * If we are updating the frNetInfo object, we'll first check the cache
   * versions.  If they do not match, we destroy the contents of the 
   * frNetInfo and load a new one.  If the versions do match, we'll try
   * to get the list of available entities from the NIQS and cull the
   * frNetInfo object of old (no longer in use) entities.  If the
   * command to get that list fails, we can keep on going -- either we talked
   * to an old server, or the signature check failed, in which case this
   * function will fail when it gets the update list. 
   */

  if (update_flag) {

    /*  Check the cache version and force a reload if required */

    if (cacheDataVersion != ni->cacheDataVersion) {
      ClientTrace(fn,"FRNETINFO: CacheDataVersion has changed, flushing NIQS cache");
      CONSOLE_MSG(ni,"CacheDataVersion has changed, flushing NIQS cache");
      zkErrorSet_va(err,fn,ZKWARN,ZKMOD,ZKERROR,
                    "NIQS Cache data version number has changed to %lu",
                    (unsigned long) cacheDataVersion);

      /* Force the complete reload */
      since = 0;
      update_flag = 0;
  
      /* Clean out the frNetInfo object */ 
      if (!frNetInfoDeleteAll(err,ni)) {
        return 0;
      }

    /* Get list of available entities and delete old ones from frnetinfo */
     
    } else if (frNiqsGetAvailable(err,cli,kqh,kq,host_ent,he_sz,
                                  &entid,&entid_sz)) {
      ClientTrace(fn,"FRNETINFO: Culling entities from FrNetInfo structure");
      CONSOLE_MSG(ni,"Culling entities from FrNetInfo structure");
      if (!frNetInfoCullEntities(err,ni,entid,entid_sz)) {
        return 0;
      }

    /* Could not get the list.  Issue a warning and continue */

    } else {  
      ClientTrace(fn,"FRNETINFO: Failed to get list of active entities");
      CONSOLE_MSG(ni,"Failed to get list of active entities");
      zkErrorSet(err,fn,ZKWARN,ZKMOD,ZKERROR,
                 "WARNING:  Failed to get list of active entities, unable to cull frnetinfo");
    }

  }
 
  ni->cacheDataVersion = cacheDataVersion;
  
  /*
   *  Get a list of all servers from the NIQS, verifying the signature over
   *  its contents.
   */

  if ( ! (frNiqsList(err, cli, since, host_ent, he_sz, &lbuf, &lbsz) &&
          frNetInfoCheckSig(err, kqh, kq, lbuf, lbsz) &&
          frSecSigRemoveInSitu(err, lbuf, &lbsz))) {
    ClientTrace(fn,"FRNETINFO: Sig verification failure for entity list buffer");
    CONSOLE_MSG(ni,"Sig verification failure for entity list buffer");
    goto done;
  }

  /*
   *  Convert the buffer into a list of structures, using it to populate the 
   *  netinfo object.  
   */ 

  if ( ! (frNiqsListFromBuffer(err, lbuf, lbsz, &srvlist) && 
          zkListMoveCursor(err, srvlist, LIST_HEAD))) { 
    ClientTrace(fn,"FRNETINFO: Failed to recreate the entity list");
    CONSOLE_MSG(ni,"Failed to recreate the entity list");
    goto done;
  } else {
    FrNidbObjType type;
    FrNiqsEntity *srv;
    FrEntIdExp entity;
    const char *region_code = 0;
    const char *servop = 0;
    size_t i;
    size_t nbs; /* the number of an AIPs neighbours */
    size_t sigdbsz; /* size of data in sigdbuf */
    size_t sbsz; /* size of data in srvbuf */
    struct in_addr iaddr; /* server address */
    struct sockaddr_in saddr; /* datum uses this, rather than struct in_addr */
    uint16_t dhport; /* AIP Diffie-Hellman port */
    uint16_t port; /* main server port */
    uint32_t nbid;
    uint32_t rating; /* latest rating */
    uint32_t status; /* latest status of the aip */
    
    /* bug? tolerate errors, bailing out only if too many occur? */
    
    while (zkListGet(err, srvlist, (void **)&srv, LIST_FORWARDS)) {
      if (sigdbuf) {
        free(sigdbuf);
        sigdbuf = NULL;
      }

      if (srvbuf) {
        free(srvbuf);
        srvbuf = NULL;
      }

      {
        snprintf(winmsg,WINMSG_SZ,"%d/%s",
                 (int) srv->entity.type, srv->entity.name);
        ClientTrace(fn,winmsg);
        CONSOLE_MSG_IMPORTANT(ni,winmsg);
      }
 
      if ( ! (frNiqsQuery(err, cli, FRNIQ_ATTR_DESCRIPTION,
                          &(srv->entity), &sigdbuf, &sigdbsz) &&
              frNetInfoCheckSig(err, kqh, kq, sigdbuf, sigdbsz) &&
              frSecSigRemove(err, sigdbuf, sigdbsz, &srvbuf, &sbsz))) {
        snprintf(winmsg,WINMSG_SZ,
                 "Failed to verify signature on description buffer for %d/%s",
                 srv->entity.type,srv->entity.name);
        ClientTrace(fn,winmsg);
        CONSOLE_MSG(ni,winmsg);
        goto done;
      }

      /*
       * Now, extract the NidbObj from the buffer
       */
      if ( ! (frNidbObjUnpack(err, srvbuf, sbsz, &obj) &&
              frNidbObjGetType(err, obj, &type))) {
        snprintf(winmsg,WINMSG_SZ,
                 "FRNETINFO: Failed to unpack DBO for %d/%s",
                 srv->entity.type,srv->entity.name);
        ClientTrace(fn,winmsg);
        CONSOLE_MSG(ni,winmsg);
        goto done;
      }

      /*
       *  Verify the data received corresponds to the requested entity
       */

      if (!frNidbObjGetEntity(err, obj, &entity) ||
          (entity.type != srv->entity.type) ||
          (strcmp(entity.name, srv->entity.name) != 0)) {
        snprintf(winmsg,WINMSG_SZ,
                 "FRNETINFO: Failed to validate entity for %d/%s",
                 srv->entity.type,srv->entity.name);
        ClientTrace(fn,winmsg);
        CONSOLE_MSG(ni,winmsg);
        goto done;
      }

      /*
       *  Extract the information we need, copying it to a new FrNetInfoDatum,
       *  which is entered into the netinfo object.
       */

      if ( ! (frNetInfoDatumNew(err, &nid) &&
              frNetInfoDatumSetSignedBuffer(err, nid, sigdbuf,sigdbsz) &&
              frNetInfoDatumSetEntity(err, nid, srv->entity) &&
              frNetInfoDatumSetId(err, nid, srv->id))) {
        ClientTrace(fn,"FRNETINFO: Failed to create a new FrNetInfoDatum structure");
        CONSOLE_MSG(ni,"Failed to create a new FrNetInfoDatum structure");
        goto done;
      }

      if( (srv->entity.type==FRENT_TYPE_AIP)||(srv->entity.type==FRENT_TYPE_COREAIP) ) {
        if (type != FRNIDBOBJ_AIP) {
          zkErrorSet_va(err,fn,ZKERR,ZKMNIQS,ZKEDATABAD,
                        "Improper reponse type (%u) from server", type);

        } else {
          if ( ! (frNidbObjGetAipParams(err, obj, &iaddr, &dhport,
                                        &port, &status, &rating,
                                        &region_code,
                                        &region, &servop, &nbs) &&
                  sAddrInFromInAddr(&saddr, &iaddr, port) &&
                  frNetInfoDatumSetSockaddr(err, nid, saddr) &&
                  frNetInfoDatumSetDifHelPort(err, nid, dhport) &&
                  frNetInfoDatumSetRating(err, nid, rating) &&
                  frNetInfoDatumSetStatus(err, nid, status) &&
                  frNetInfoDatumSetRegionCode(err, nid, region_code) &&
                  frNetInfoDatumSetRegion(err, nid, region) &&
                  frNetInfoDatumSetOperator(err, nid, servop) )) {
            goto done;
          }
          FREEIT(region);
          /* Get the neighbours into the list too */
          for (i = 0; i < nbs; i++) {
            if (!(frNidbObjGetAipNeighbour(err, obj, i, &nbid) &&
                  frNetInfoDatumAddNb(err, nid, nbid))) {
              goto done;
            }
          }
        }
      } else if (type != FRNIDBOBJ_SERVER) {
        zkErrorSet_va(err,fn,ZKERR,ZKMNIQS,ZKEDATABAD,
                      "Improper reponse type (%u) from server", type);
      } else if ( ! (frNidbObjGetServerParams(err, obj, &iaddr, &port,
                                              &status, &rating, &region_code,
                                              &region, &servop) &&
                     sAddrInFromInAddr(&saddr, &iaddr, port) &&
                     frNetInfoDatumSetSockaddr(err, nid, saddr) &&
                     frNetInfoDatumSetRating(err, nid, rating) &&
                     frNetInfoDatumSetStatus(err, nid, status) &&
                     frNetInfoDatumSetRegionCode(err, nid, region_code) &&
                     frNetInfoDatumSetRegion(err, nid, region) &&
                     frNetInfoDatumSetOperator(err, nid, servop) )) {
        goto done;
      } else {
        FREEIT(region);
      }

      if (obj) {
        free(obj); obj = 0;
      }

      /*
      ** frNetInfoInsert takes care of removing an entry (by calling frNetInfoDeleteById )
      ** if it 's already in the datum
      ** it is really doing an insert OR replace job...
      */
      snprintf(winmsg,WINMSG_SZ,"Inserting/Replacing entity %d/%s in FrNetInfo",
               srv->entity.type,srv->entity.name);
      ClientTrace(fn,winmsg);
      CONSOLE_MSG(ni,winmsg); 
      
      if ( ! frNetInfoInsert(err, ni, nid)) {
          ClientTrace(fn,"FRNETINFO: Failed to insert/replace the entity");
          CONSOLE_MSG(ni,"Failed to insert/replace the entity"); 
          goto done;
      }

      nid = 0; /* it's been inserted... */
    }
  }

  /*
   *  bug: we could check the internal consistency of our object, here.  That
   *  is, check that all AIP neighbours are in our table, and that if A is a
   *  neighbour of B then B is a neighbour of A, etc.
   */

  ret = 1;
  ni->lastUpdate = time(NULL); 

  if (entid) {
    size_t fni_sz = 0;
    if (!frNetInfoCountRecords(err,ni,&fni_sz)) {
      goto done;
    }
    if (fni_sz != entid_sz) {
      zkErrorSet(err,fn,ZKERR,ZKMFRNETINFO,ZKERROR,
                 "Local and remote entity count inconsistency");
      CONSOLE_MSG(ni,"Local and remote entity count inconsistency"); 
      ret = 0;
    }
  }
  
 done:

  /* Cleanup */
  if (entid) {
    free(entid);
  }
  if (nid) { 
    frNetInfoDatumFree(0, &nid); 
  }
  if (srvlist) { 
    zkListFree(0, &srvlist); 
  }
  if (lbuf) {
    free(lbuf);
  }
  if (kqh.fd > -1) { 
#ifdef WIN32
	  closesocket(kqh.fd);
#else
	  close(kqh.fd); 
#endif
  }
  if (sigdbuf) { free(sigdbuf); } /* bug? */
  if (srvbuf) { free(srvbuf); } /* bug? */
  if (obj) { free(obj); }
  
  return ret;
}

#endif



/*
 *  The entities for which we query, in order to initially populate the
 *  netinfo object.
 */
static FrEntIdType hostEnt[] = {
  FRENT_TYPE_AIP,
  FRENT_TYPE_NYM_SRV,
  FRENT_TYPE_SER_SRV,           
  FRENT_TYPE_TOK_SRV,
  FRENT_TYPE_KEYQRY_SRV,
  FRENT_TYPE_KEYUPD_SRV,
  FRENT_TYPE_NNTP_SRV,
  FRENT_TYPE_NIQS,
  FRENT_TYPE_NISS,
  FRENT_TYPE_POP,
  FRENT_TYPE_NMTA,
  FRENT_TYPE_IMEP,
  FRENT_TYPE_COREAIP
};

#define ASZ(a) (sizeof (a) / sizeof (a)[0])


/*
** this structure is used to hold the information retrieved from NIQS for
** a Client request
*/
typedef struct responsedata {

    uint32_t niqsCacheVersion;

    void   * mxqBuf; /* mxquery raw buffer */
    size_t   mxqBsz; /* mxquery raw buffer size */

    void   * ciBuf;  /* client info raw buffer */
    size_t   ciBsz;  /* client info raw buffer size */

    void   * tiBuf;  /* token info raw buffer */
    size_t   tiBsz;  /* token info raw buffer size */

} FrResponseData;


/*
**-----------------------------------------------------------------------------
** This function handle the connection to the NIQS servers
** - fisrt it tries the ip addresses found in the provided NetInfo Object
** the number of iteration done in that list is set by numListIter
**
** - if not connected, it tries a DNS lookup with the provided hostname and port
**
**-----------------------------------------------------------------------------
*/
static int
frNetInfoHandleNiqsConnection(
    ZK_IN    ZkError        err,
    ZK_IN    ZkLog          log,
    ZK_IN    FrNetInfo      ni,
    ZK_IN    uint16_t       numListIter,  /* # of list lookup */
    ZK_IN    const char *   defaultHost , /* host address */
    ZK_IN    uint16_t       defaultPort , /* port address */
    ZK_IN    uint16_t       zkipTimeout,  /* connection timeout */
    ZK_OUT   ZkipClient *   p_cli        /* pointer to ZKIP client object */
)
{
#define WINMSG_SZ 128

    static const char * fn = "frNetInfoHandleNiqsConnection";
    int ret = 0;

    int connected = 0;
    int iter = 0;

    ZkList         list = NULL;
    FrNetInfoDatum datum = NULL;
    struct sockaddr_in saddr;


	char winmsg[WINMSG_SZ];   /* For creating windows tracing messages */

    /*
    ** get the list of NIQS if a NetInfo object is provided
    */
    if( ni &&
        ! frNetInfoGetListOf(err, ni, &list, FRENT_TYPE_NIQS)) {
        zkLogWriteContext(log, err, fn, "call to frNetInfoGetListOf failed");
    }
    else if(list &&
            ! zkListMoveCursor(err,list,LIST_HEAD)) {
        zkLogWriteContext(log, err, fn, "call to zkListMoveCursor failed");
    }
    else if(list) {

        /*
        ** iterate over the NIQS list
        */
        while( ! connected &&
               iter < numListIter &&
               zkListGet(err,list,(void **) &datum,LIST_FORWARDS)) {

			snprintf(winmsg,WINMSG_SZ,"Attempting to connect to entity %s",
				   datum->entid.name);
			ClientTrace(fn,winmsg);
			CONSOLE_MSG(ni,winmsg); 

            if( ! frNetInfoDatumGetSockaddr(err,datum,(struct sockaddr_in *)&saddr)) {
                zkLogWriteContext(log, err, fn, "call to frNetInfoDatumGetSockaddr failed");
            }
            else if ( ! frNiqsConnect(err, p_cli,
                                      inet_ntoa( saddr.sin_addr ),
                                      defaultPort,
                                      zkipTimeout)) {
                zkLogWriteContext(log, err, fn, "call to frNiqsConnect failed");
            }
            else {
                /*
                ** connection success !
                */
                connected = 1;
                ret = 1;
            }
            
            iter++;
        }
    }

    /*
    ** if still not connected go through DNS
    */
    if( ! connected) {

		snprintf(winmsg,WINMSG_SZ,"Attempting to connect to %s",defaultHost);
		ClientTrace(fn,winmsg);
		CONSOLE_MSG(ni,winmsg); 

        if( ! frNiqsConnect(err, p_cli,
                            defaultHost,
                            defaultPort,
                            zkipTimeout)) {
            zkLogWriteContext(log, err, fn, "call to frNiqsConnect failed");
        }
        else {
            /*
            ** connection success !
            */
            connected = 1;
            ret = 1;
        }
    }

    return ret;
}


/*
**-----------------------------------------------------------------------------
**
** This function is used to:
** - get the key server address from NIQS
** - open a socket in order to "talk" with it
**
**-----------------------------------------------------------------------------
*/
static int
frNetInfoGetKeyServerInfo(
    ZK_INOUT ZkError            err,     /* error structure */
    ZK_INOUT ZkLog              log,     /* log structure */
    ZK_IN    ZkipClient         cli,     /* zkip client information */
    ZK_OUT   FrKeyQryHost     * pKqHost  /* pointer to key query host info */
)
{
    static const char *fn = "frNetInfoGetKeyServerInfo";
    int ret = 1;

    struct sockaddr saddr;


    /*
    ** sanity check
    */
    if ( ! cli || ! pKqHost ) {
        zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "null parameters passed");
        ret =  0;
    }
   
    /*
    ** get the address from NIQS
    */
    else if ( ! frNiqsGetKeyQryAddr(err, cli, &saddr)) {
        zkLogWriteContext(log, err, fn, "call to frNiqsGetKeyQryAddr failed");
        ret = 0;
    }

    /*
    ** fill FrKeyQryHost
    */
    else if ( ! memcpy(&(pKqHost->host), &saddr, sizeof (pKqHost->host)))  {
        zkErrorSet(err,fn,ZKERR,ZKMOD,ZKERROR, "call to memcpy() failed");
        ret = 0;
    }
    /* create a socket */
    else if ((pKqHost->fd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
        zkErrorSet_va(err, fn, ZKERR, ZKMOD, errno,
                      "can't create key query socket: %s", strerror(errno));
        ret = 0;
    }


    return ret;
}


/*
**-----------------------------------------------------------------------------
**
** This function populates a FrNetInfo object with the information contained
** in the FrResponseData object
** Prior to populating, a signature verification is done to make sure that the 
** information is valid.
**
**-----------------------------------------------------------------------------
*/
static int
frNetInfoPopulateFromBuffer(
    ZkError            err,
    ZkLog              log,
    FrNetInfo          ni,
    FrKeyQryHost       kqh,
    FrKeyQry           kq,    
    FrResponseData     respData
)
{
  static const char *fn = "frNetInfoPopulateFromBuffer";
  int nRetCode = 0;

  void *raw_buf = NULL;  /* uncompresses buffer */
  size_t raw_bsz = 0;    /* buffer size */


  /*
  ** sanity check
  */
  if (!ni) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "ni is null");
    return 0;
  }
  if (!kq) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "kq is null");
    return 0;
  }
      
  /*
  ** Verify the signature of the MxQuery response
  */
  if (!frNetInfoCheckSig(err, kqh, kq, respData.mxqBuf, respData.mxqBsz)) {
      ClientTrace(fn,"FRNETINFO: Sig verification failure for MxQuery buffer");
      CONSOLE_MSG(ni,"Sig verification failure for MxQuery buffer");
      zkLogWriteContext(log, err, fn, "call to frNetInfoCheckSig failed");
  }

  /*
  ** at this point we know it's good: delete the old info before populating with new data
  */
  else if (!frNetInfoDeleteAll(err,ni)) {
      zkLogWriteContext(log, err, fn, "call to frNetInfoDeleteAll failed");
  }

  /*
  ** copy original buffer to the frNetInfo structure
  */
  else if (! (ni->mxquery_signed_buffer = (unsigned char *) realloc(ni->mxquery_signed_buffer, respData.mxqBsz)) ||
           ! (memcpy(ni->mxquery_signed_buffer, respData.mxqBuf, respData.mxqBsz)) ||
           ! (ni->mxquery_signed_buffer_sz = respData.mxqBsz)) {
      ClientTrace(fn,"FRNETINFO: Unable to allocate buffer to hold MxQuery buffer");
      CONSOLE_MSG(ni,"Unable to allocate buffer to hold MxQuery buffer");
      zkLogWriteContext(log, err, fn, "Unable to allocate buffer to hold MxQuery buffer");
  }

  /*
  ** remove sig and uncompress data
  */
  else if (!frSecSigRemoveInSitu(err, respData.mxqBuf, &respData.mxqBsz)) {
      ClientTrace(fn,"FRNETINFO: Signature removal failure for MxQuery buffer");
      CONSOLE_MSG(ni,"Signature removal failure for MxQuery buffer");
      zkLogWriteContext(log, err, fn, "call to frSecSigRemoveInSitu failed");
  }
  else if( ! frNiqsUncompress(err, (const unsigned char *)respData.mxqBuf, (unsigned char **)&raw_buf, &raw_bsz) ){
      ClientTrace(fn,"FRNETINFO: Unable to uncompress NIQS response");
      CONSOLE_MSG(ni,"Unable to uncompress NIQS response");
      zkLogWriteContext(log, err, fn, "call to frNiqsUncompress failed");
  }
  /*
  ** Fill the NetInfo structure members with data from the MxQuery buffer
  */
  else if (!frNetInfoFillNIFromMxQueryBuffer(err, log, ni, raw_buf, raw_bsz)) {
      ClientTrace(fn,"FRNETINFO: Unable to fill the NetInfo structure");
      CONSOLE_MSG(ni,"Unable to fill the NetInfo strucure");
      zkLogWriteContext(log, err, fn, "call to frNetInfoFillNIFromMxQuery failed");
  }
  /*
  ** copy the new cache version value
  */
  else {
      ni->cacheDataVersion = respData.niqsCacheVersion;

      /* at this point it's a success */
      nRetCode = 1;
  }

  /*
  ** Cleanup
  */
  if (raw_buf) { free(raw_buf); }
  
  if (!nRetCode) {
    /*
    ** An error has occurred
    */
    if (ni->mxquery_signed_buffer) {
      free(ni->mxquery_signed_buffer);
      ni->mxquery_signed_buffer = NULL;
      ni->mxquery_signed_buffer_sz = 0;
    }
  }

  return nRetCode;
}

/*
**-----------------------------------------------------------------------------
**
** This function extracts ClientInfo from the information contained
** in the FrResponseData object
** Prior to that, a signature verification is done to make sure that the 
** information is valid.
**
**-----------------------------------------------------------------------------
*/
static int
frNetInfoClientInfoFromBuffer(
    ZkError            err,
    ZkLog              log,
    FrNetInfo          ni,
    FrKeyQryHost       kqh,
    FrKeyQry           kq,    
    FrResponseData     respData,
    const char       * clientId,
    FrNiqsClientInfo * pClientInfo
)
{
  static const char *fn = "frNetInfoClientInfoFromBuffer";
  int nRetCode = 0;


  /*
  ** sanity check
  */
  if (!ni) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "ni is null");
    return 0;
  }
  if (!kq) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "kq is null");
    return 0;
  }

  /*
  ** Verify the signature
  */
  if (!frNetInfoCheckSig(err, kqh, kq, respData.ciBuf, respData.ciBsz)) {
      ClientTrace(fn,"FRNETINFO: Sig verification failure for ClientInfo buffer");
      CONSOLE_MSG(ni,"Sig verification failure for ClientInfo buffer");
      zkLogWriteContext(log, err, fn, "call to frNetInfoCheckSig failed");
  }
  /*
  ** remove sig and uncompress data
  */
  else if (!frSecSigRemoveInSitu(err, respData.ciBuf, &respData.ciBsz)) {
      ClientTrace(fn,"FRNETINFO: Signature removal failure for ClientInfo buffer");
      CONSOLE_MSG(ni,"Signature removal failure for ClientInfo buffer");
      zkLogWriteContext(log, err, fn, "call to frSecSigRemoveInSitu failed");
  }
  /*
  ** retrieve info from buffer
  */
  else if( ! frNiqsClientInfoLoadFromBuffer(err,pClientInfo,clientId, respData.ciBuf, respData.ciBsz)) {
      ClientTrace(fn,"FRNETINFO: Info retrieval from ClientInfo buffer failed");
      CONSOLE_MSG(ni,"Info retrieval from ClientInfo buffer failed");
      zkLogWriteContext(log, err, fn, "call to frNiqsClientInfoLoadFromBuffer failed");
  }
  else {
      nRetCode = 1;
  }

  return nRetCode;
}


/*
**-----------------------------------------------------------------------------
**
** This function extracts TokenInfo from the information contained
** in the FrResponseData object
** Prior to that, a signature verification is done to make sure that the 
** information is valid.
**
**-----------------------------------------------------------------------------
*/
static int
frNetInfoTokenInfoFromBuffer(
    ZkError            err,
    ZkLog              log,
    FrNetInfo          ni,
    FrKeyQryHost       kqh,
    FrKeyQry           kq,    
    FrResponseData     respData,
    FrTokenType *      pTokenTypes
)
{
  static const char *fn = "frNetInfoTokenInfoFromBuffer";
  int nRetCode = 0;


  /*
  ** sanity check
  */
  if (!ni) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "ni is null");
    return 0;
  }
  if (!kq) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "kq is null");
    return 0;
  }


  /*
  ** Verify the signature
  */
  if (!frNetInfoCheckSig(err, kqh, kq, respData.tiBuf, respData.tiBsz)) {
      ClientTrace(fn,"FRNETINFO: Sig verification failure for TokenInfo buffer");
      CONSOLE_MSG(ni,"Sig verification failure for TokenInfo buffer");
      zkLogWriteContext(log, err, fn, "call to frNetInfoCheckSig failed");
  }
  /*
  ** remove sig and uncompress data
  */
  else if (!frSecSigRemoveInSitu(err, respData.tiBuf, &respData.tiBsz)) {
      ClientTrace(fn,"FRNETINFO: Signature removal failure for TokenInfo buffer");
      CONSOLE_MSG(ni,"Signature removal failure for TokenInfo buffer");
      zkLogWriteContext(log, err, fn, "call to frSecSigRemoveInSitu failed");
  }
  /*
  ** retrieve info from buffer
  */
  else if( ! frTokenTypeFromBinary(err,pTokenTypes, respData.tiBuf, respData.tiBsz)) {
      ClientTrace(fn,"FRNETINFO: Info retrieval from TokenInfo buffer failed");
      CONSOLE_MSG(ni,"Info retrieval from TokenInfo buffer failed");
      zkLogWriteContext(log, err, fn, "call to frTokenTypeFromBinary failed");
  }
  else {
      nRetCode = 1;
  }

  return nRetCode;
}

/*
**-----------------------------------------------------------------------------
**
** This function performs the necessary requests to gather information needed by 
** the various clients
**
** The key server address is ALWAYS queried directly from NIQS, this to make
** sure that we get a valid address. The signature verification is performed on
** the answer.
**
** Then the cacheDateVersion is also queried from NIQS, and the signature is 
** verified.
**
** If the cacheDataVersion has changed, then mxquery is performed. The 
** signature is NOT verified right away and the reponse buffer is saved as is
** in the FrResponseData object.
** 
** if corresponding flags are set then ClientInfo and/or TokenInfo are queried
** otherwise that information is not queried. Again the signatures are not
** verified and the buffers are stored in the FrResponseData object.
**
**-----------------------------------------------------------------------------
*/
static int 
frNetInfoQueryNIQS(
    ZK_INOUT ZkError            err,           /* error structure */
    ZK_INOUT ZkLog              log,           /* log structure */
    ZK_INOUT FrNetInfo          ni,            /* NetInfo object */
    ZK_IN    ZkipClient         cli,           /* zkip client information */
    ZK_IN    FrKeyQryHost       kqHost,        /* key query host info */
    ZK_IN    FrKeyQry           kqCache,       /* key cache  -- was kq */
    ZK_IN    FrEntIdType      * host_ent,      /* array of entity type */
    ZK_IN    uint32_t           he_sz,         /* size of the array */
    ZK_IN    int                getClientInfo, /* flag to get ClientInfo */
    ZK_IN    int                getTokenInfo,  /* flag to get TokenInfo */
    ZK_IN    const char       * clientId,      /* client string identifier */
    ZK_OUT   FrResponseData   * pClientData    /* object that holds retrieved raw data */
)
{
    static const char * fn = "frNetInfoQueryClientData";
    int ret = 0;


    /*
    ** ALWAYS get cache version
    */
    if ( ! frNiqsGetCacheDataVersion(err,cli,kqHost,kqCache,&(pClientData->niqsCacheVersion) )) {
        ClientTrace(fn,"FRNETINFO: could not get cacheDataVerison from NIQS");
        CONSOLE_MSG(ni,"Could not get cacheDataVerison from NIQS");
        zkLogWriteContext(log, err, fn, "call to frNiqsGetCacheDataVersion failed");
    }

    else {
        
        /*
        ** if client has no cache 
        ** OR doesn't has the latest from NIQS
        ** OR we couldn't get the cache version from NIQS *
        ** --> update
        */
        if( ni->cacheDataVersion != pClientData->niqsCacheVersion ) {

            zkLogWriteContext(log, NULL, fn, "[client init] cache is out of sync; full update");
            CONSOLE_MSG(ni,"Client's cache is out of sync; performing full update");

            /*
            ** Get a list of descriptions for all servers from NIQS
            */
            if (!frNiqsMxQuery(err, cli, FRNIQ_ATTR_DESCRIPTION, 0,
                               ZKIP_ENCODING_BZIP2, host_ent, he_sz,
                               &(pClientData->mxqBuf), &(pClientData->mxqBsz))) {

                ClientTrace(fn,"FRNETINFO: Query to NIQS failed");
                CONSOLE_MSG(ni,"Query to NIQS failed");
                zkLogWriteContext(log, err, fn, "call to frNiqsMxQuery failed");
            }

            else {
                /*
                ** it's a success !
                */
                ret = 1;
            }
        } else {
            /* clientCacheVersion is the same as niqsCacheversion */
            /* no need to update !!! */
            ret = 1;
        }
    }

    /*
    ** ONLY get  token info and client info if requested
    ** (and if previous call succeeded)
    */
    if (1 == ret) {
        if(getTokenInfo && 
           ! frNiqsTokenInfo (err, cli,
                              &(pClientData->tiBuf), &(pClientData->tiBsz))) {
            ClientTrace(fn,"FRNETINFO: failed to get TokenInfo from NIQS");
            CONSOLE_MSG(ni,"Failed to get TokenInfo from NIQS");
            zkLogWriteContext(log, err, fn, "call to frNiqsTokenInfo failed");
            ret = 0;
        }

        else if (getClientInfo && 
                 ! frNiqsClientInfo (err, cli, clientId,
                                     &(pClientData->ciBuf), &(pClientData->ciBsz))) {
            ClientTrace(fn,"FRNETINFO: failed to get ClientInfo from NIQS");
            CONSOLE_MSG(ni,"Failed to get ClientInfo from NIQS");
            zkLogWriteContext(log, err, fn, "call to frNiqsClientInfo failed");
            ret = 0;
        }    
    }

    /*
    ** cleanup
    */

    return ret;
}


/*
**-----------------------------------------------------------------------------
** Generic query sequence
** 
** This is the sequence:
** - connect to NIQS
** - get information from it
** - disconnect
** - validate daa and populate FrNetInfo object
**
** We disconnect from NIQS before validating the information in order to 
** minimize the connection time to NIQS
**
** The only information that is validate during the connection are:
**     - key server address
**     - cacheDataVersion
** and this is for obvious reasons... 
**
**-----------------------------------------------------------------------------
*/
DLLExport int 
frNetInfoQuerySequence(
    ZK_INOUT ZkError            err,           /* error structure */
    ZK_INOUT ZkLog              log,           /* log object */
    ZK_INOUT FrNetInfo          ni,            /* NetInfo object */
    ZK_IN    const char *       defaultHost ,  /* host address */
    ZK_IN    uint16_t           defaultPort ,  /* port address */
    ZK_IN    uint16_t           zkipTimeout,   /* connection timeout */
    ZK_IN    uint16_t           numListIter,   /* number of cache list lookup */
    ZK_IN    FrKeyQry           kqCache,       /* key cache */

    ZK_IN    FrEntIdType      * host_ent,      /* array of entity type */
    ZK_IN    uint32_t           he_sz,         /* size of the array */

    ZK_IN    int                getClientInfo, /* flag to get ClientInfo */
    ZK_IN    const char *       clientId,      /* client string identifier */
    ZK_OUT   FrNiqsClientInfo * pClientInfo,   /* pointer to client info structure */

    ZK_IN    int                getTokenInfo,  /* flag to get TokenInfo */
    ZK_OUT   FrTokenType *      pTokenTypes    /* pointer to token type structure */
)
{
    static const char * fn = "frNetInfoQuerySequence";
    int ret = 1;

    ZkipClient   cli = NULL;  /* zkip client information */

    FrKeyQryHost       kqHost;

    FrResponseData     respData;


    /* initialize respData */
    respData.ciBuf  = NULL;
    respData.ciBsz  = 0;
    respData.tiBuf  = NULL;
    respData.tiBsz  = 0;
    respData.mxqBuf = NULL;
    respData.mxqBsz = 0;

    /*
    ** This seems a safe place to do the DBO subsystem initialization
    */
    if( ! frDBODriverIsInitialized(err) && ! frDBODriverInitAll(err)) {
        zkLogWriteContext(log, err, fn, "call to frDBODriverInitAll failed");
        ret = 0;
    }

    /*
    ** connect
    */
    else if( ! frNetInfoHandleNiqsConnection(err, log, ni, numListIter, 
                                             defaultHost, defaultPort, zkipTimeout,
                                             &cli)) {
        zkLogWriteContext(log, err, fn, "call to frNetInfoHandleNiqsConnection failed");
        ret = 0;
    }

    /*
    ** get the key query host address (and open a socket)
    ** don't take the one in ni since it could have change
    */
    else if ( ! frNetInfoGetKeyServerInfo(err, log, cli, &kqHost) ){
        zkLogWriteContext(log, err, fn, "call to frNetInfoGetKeyServerInfo failed");
        ret = 0;
    }

    /*
    ** collect data from NIQS
    */
    else if ( ! frNetInfoQueryNIQS(err, log, ni,
                                   cli,
                                   kqHost, kqCache,
                                   host_ent, he_sz,
                                   getClientInfo, getTokenInfo,
                                   clientId,
                                   &respData)) {
        zkLogWriteContext(log, err, fn, "call to frNetInfoQueryClientData failed");
        ret = 0;
    }

    /*
    ** disconnect (must do it regardless of the result of previous command)
    */
    if ( cli && ( ! frNiqsDisconnect(err, &cli)) ) {
        zkLogWriteContext(log, err, fn, "call to frNiqsDisconnect failed");
        ret = 0;
    }
    /*
    ** validate collected data and fill the NetInfo object with it
    */
    else if(respData.mxqBuf &&
            ! frNetInfoPopulateFromBuffer(err, log, ni,
                                          kqHost, kqCache,
                                          respData) ){
        zkLogWriteContext(log, err, fn, "call to frNetInfoPopulateFromBuffer failed");
        ret = 0;
    }

    /*
    ** extract client info
    */
    else if(respData.ciBuf &&
            ! frNetInfoClientInfoFromBuffer(err, log, ni,
                                            kqHost, kqCache,
                                            respData,
                                            clientId, 
                                            pClientInfo) ){
        zkLogWriteContext(log, err, fn, "call to frNetInfoClientInfoFromBuffer failed");
        ret = 0;
    }
    /*
    ** extract token info
    */
    else if(respData.tiBuf &&
            ! frNetInfoTokenInfoFromBuffer(err, log, ni,
                                           kqHost, kqCache,
                                           respData,
                                           pTokenTypes) ){
        zkLogWriteContext(log, err, fn, "call to frNetInfoTokenInfoFromBuffer failed");
        ret = 0;
    }


    /*
    ** cleaning
    */

    /* free the FrResponseData object's buffers */
    if(respData.mxqBuf) free(respData.mxqBuf);
    if(respData.ciBuf)  free(respData.ciBuf);
    if(respData.tiBuf)  free(respData.tiBuf);

    /* close the socket */
    if (kqHost.fd > -1) { 
#ifdef WIN32
        closesocket(kqHost.fd);
#else
        close(kqHost.fd); 
#endif
    }

    return ret;
}


/*
**=============================================================================
** Public function section
**=============================================================================
*/

/*
**-----------------------------------------------------------------------------
** This is the generic function used to populate a FrNetInfo object
**-----------------------------------------------------------------------------
*/
DLLExport
int frNetInfoPopulateSequence(
    ZK_INOUT ZkError            err,          /* error structure */ 
    ZK_INOUT FrNetInfo          ni,           /* NetInfo object */
    ZK_IN    const char *       defaultHost , /* host address */
    ZK_IN    uint16_t           defaultPort , /* port address */
    ZK_IN    uint16_t           zkipTimeout,  /* connection timeout */
    ZK_IN    uint16_t           numListIter,  /* number of cache list lookup */
    ZK_IN    FrKeyQry           kqCache       /* key cache */
)
{
    static const char * fn = "frNetInfoClientUpdateSequence";
    int ret = 1;

    ZkLog log;

    /* parameter validation */
    if ( !ni || !defaultHost || !kqCache) {
        zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "null parameter passed");
        ret = 0;
    }

    /* get the log pointer from ZkError */
    else if( ! zkErrorLog(err, &log )) {
        zkErrorSet(err,fn,ZKERR,ZKMOD,ZKERROR, "call to zkErrorLog failed");
        ret = 0;
    }

    else if( ! frNetInfoQuerySequence(err, log, ni, 
                                      defaultHost, defaultPort, zkipTimeout, 
                                      numListIter, kqCache,
                                      hostEnt, ASZ(hostEnt),
                                      0, NULL, NULL,
                                      0, NULL)) {
        zkLogWriteContext(log, err, fn, "call to frNetInfoClientSequence failed");
        ret = 0;
    }

    return ret;
}

/*
**-----------------------------------------------------------------------------
**
** This is the generic function used to partially  populate a FrNetInfo object
** It works like frNetInfoPopulateSequence, escept that it gets info only for 
** the specified entity types
** The list of entity types is specified be host_ent, an array of entity types
**  of size host_ent_sz
**-----------------------------------------------------------------------------
*/
DLLExport
int frNetInfoSelectivePopulateSequence(
    ZK_INOUT ZkError            err,          /* error structure */ 
    ZK_INOUT FrNetInfo          ni,           /* NetInfo object */
    ZK_IN    const char *       defaultHost , /* host address */
    ZK_IN    uint16_t           defaultPort , /* port address */
    ZK_IN    uint16_t           zkipTimeout,  /* connection timeout */
    ZK_IN    uint16_t           numListIter,  /* number of cache list lookup */
    ZK_IN    FrKeyQry           kqCache,      /* key cache */
    ZK_IN    FrEntIdType *      host_ent,     /* array of entity types to query */
    ZK_IN    unsigned int       host_ent_sz   /* size of the array */
)
{
    static const char * fn = "frNetInfoClientUpdateSequence";
    int ret = 1;

    ZkLog log;

    /* parameter validation */
    if ( !ni || !defaultHost || !kqCache) {
        zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "null parameter passed");
        ret = 0;
    }

    /* get the log pointer from ZkError */
    else if( ! zkErrorLog(err, &log )) {
        zkErrorSet(err,fn,ZKERR,ZKMOD,ZKERROR, "call to zkErrorLog failed");
        ret = 0;
    }

    else if( ! frNetInfoQuerySequence(err, log, ni, 
                                      defaultHost, defaultPort, zkipTimeout, 
                                      numListIter, kqCache,
                                      host_ent, host_ent_sz,
                                      0, NULL, NULL,
                                      0, NULL)) {
        zkLogWriteContext(log, err, fn, "call to frNetInfoClientSequence failed");
        ret = 0;
    }

    return ret;
}


/*
**-----------------------------------------------------------------------------
** This is the function that the client calls at startup
** It populates a FrNetInfo object and gets clientinfo and tokeninfo
**-----------------------------------------------------------------------------
*/
DLLExport int 
frNetInfoClientInitSequence(
    ZK_INOUT ZkError            err,          /* error structure */ 
    ZK_INOUT FrNetInfo          ni,           /* NetInfo object */
    ZK_IN    const char *       defaultHost , /* host address */
    ZK_IN    uint16_t           defaultPort , /* port address */
    ZK_IN    uint16_t           zkipTimeout,  /* connection timeout */
    ZK_IN    uint16_t           numListIter,  /* number of cache list lookup */
    ZK_IN    FrKeyQry           kqCache,      /* key cache */
    ZK_IN    const char *       clientId,     /* client string identifier */
    ZK_OUT   FrNiqsClientInfo * pClientInfo,  /* pointer to client info structure */
    ZK_OUT   FrTokenType *      pTokenTypes   /* pointer to token type structure */
)
{
    static const char * fn = "frNetInfoClientInitSequence";
    int ret = 1;

    ZkLog log;

    /* parameter validation */
    if ( !ni || !defaultHost || !kqCache || !clientId || !pClientInfo || !pTokenTypes ) {
        zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "null parameter passed");
        ret = 0;
    }

    /* get the log pointer from ZkError */
    else if( ! zkErrorLog(err, &log )) {
        zkErrorSet(err,fn,ZKERR,ZKMOD,ZKERROR, "call to zkErrorLog failed");
        ret = 0;
    }

    else if( ! frNetInfoQuerySequence(err, log, ni, 
                                      defaultHost, defaultPort, zkipTimeout, 
                                      numListIter, kqCache,
                                      hostEnt, ASZ(hostEnt), 
                                      1, clientId, pClientInfo,
                                      1, pTokenTypes)) {
        zkLogWriteContext(log, err, fn, "call to frNetInfoClientSequence failed");
        ret = 0;
    }

    return ret;
}


/*
**-----------------------------------------------------------------------------
** This is the function that the client calls on periodical updates
** It only populates a FrNetInfo object
**-----------------------------------------------------------------------------
*/
DLLExport int 
frNetInfoClientUpdateSequence(
    ZK_INOUT ZkError            err,          /* error structure */ 
    ZK_INOUT FrNetInfo          ni,           /* NetInfo object */
    ZK_IN    const char *       defaultHost , /* host address */
    ZK_IN    uint16_t           defaultPort , /* port address */
    ZK_IN    uint16_t           zkipTimeout,  /* connection timeout */
    ZK_IN    uint16_t           numListIter,  /* number of cache list lookup */
    ZK_IN    FrKeyQry           kqCache       /* key cache */
)
{
    static const char * fn = "frNetInfoClientUpdateSequence";
    int ret = 1;

    ZkLog log;

    /* parameter validation */
    if ( !ni || !defaultHost || !kqCache) {
        zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "null parameter passed");
        ret = 0;
    }

    /* get the log pointer from ZkError */
    else if( ! zkErrorLog(err, &log )) {
        zkErrorSet(err,fn,ZKERR,ZKMOD,ZKERROR, "call to zkErrorLog failed");
        ret = 0;
    }

    else if( ! frNetInfoQuerySequence(err, log, ni, 
                                      defaultHost, defaultPort, zkipTimeout, 
                                      numListIter, kqCache,
                                      hostEnt, ASZ(hostEnt),
                                      0, NULL, NULL,
                                      0, NULL)) {
        zkLogWriteContext(log, err, fn, "call to frNetInfoClientSequence failed");
        ret = 0;
    }

    return ret;
}

/*
**-----------------------------------------------------------------------------
** This is the function that core servers/tools call at startup
** It populates a FrNetInfo object and gets tokeninfo
**-----------------------------------------------------------------------------
*/
DLLExport int 
frNetInfoCoreInitSequence(
    ZK_INOUT ZkError            err,          /* error structure */ 
    ZK_INOUT FrNetInfo          ni,           /* NetInfo object */
    ZK_IN    const char *       defaultHost , /* host address */
    ZK_IN    uint16_t           defaultPort , /* port address */
    ZK_IN    uint16_t           zkipTimeout,  /* connection timeout */
    ZK_IN    uint16_t           numListIter,  /* number of cache list lookup */
    ZK_IN    FrKeyQry           kqCache,      /* key cache */
    ZK_OUT   FrTokenType *      pTokenTypes   /* pointer to token type structure */
)
{
    static const char * fn = "frNetInfoNymtoolInitSequence";
    int ret = 1;

    ZkLog log;

    /* parameter validation */
    if ( !ni || !defaultHost || !kqCache || !pTokenTypes ) {
        zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "null parameter passed");
        ret = 0;
    }

    /* get the log pointer from ZkError */
    else if( ! zkErrorLog(err, &log )) {
        zkErrorSet(err,fn,ZKERR,ZKMOD,ZKERROR, "call to zkErrorLog failed");
        ret = 0;
    }

    else if( ! frNetInfoQuerySequence(err, log, ni, 
                                      defaultHost, defaultPort, zkipTimeout, 
                                      numListIter, kqCache,
                                      hostEnt, ASZ(hostEnt), 
                                      0, NULL, NULL,
                                      1, pTokenTypes)) {
        zkLogWriteContext(log, err, fn, "call to frNetInfoClientSequence failed");
        ret = 0;
    }

    return ret;
}


/*
**------------------------------------------------------------------------------
**------------------------------------------------------------------------------
**
** PATCH PATCH PATCH
** 
** from here to end of file
**
** those function have been re-introduce to allow beta-testing of client 2.0
** under prod-net before the releasing of the new NIQS
**
** they must be removed before deployment of client 2.0
**
** LBe 2000.07.26
**------------------------------------------------------------------------------
**------------------------------------------------------------------------------
*/

#ifndef WIN32
#warning must remove the following functions before releasing of client 2.0
#endif


/*
 *  Convert a struct in_addr, with a port, to a struct sockaddr_in.  The
 *  `port' is assumed to be in host byte order.
 */
static int sAddrInFromInAddr(struct sockaddr_in *saddr, struct in_addr *iaddr, uint16_t port)
{
  memset(saddr, 0, sizeof *saddr);

  saddr->sin_family = AF_INET;
  saddr->sin_port = htons(port);
  saddr->sin_addr = *iaddr;

  return 1;
}


/*
** BUG - BUG - BUG
** CONSOLIDATION should clean that fct and make it call frNetInfoPopSince
** since it's been define as a subset of this function
*/
static int frNetInfoPopulateOrUpdateSince(
  ZkError err,
  FrNetInfo ni,
  ZkipClient cli,
  FrKeyQry kq,
  FrEntIdType *host_ent,
  unsigned int he_sz,
  time_t since,
  int update_flag)
{
#define WINMSG_SZ 128
  FrKeyQryHost kqh;
  FrNetInfoDatum nid = 0;
  FrNidbObj obj = 0;
  char *region = 0; /* This needs to be freed. Because of the goto's I have
                       to change the scope of this pointer, to be able to do a
                       free at the end */
  ZkList srvlist = 0;
  const char *fn = "frNetInfoPopulateOrUpdateSince";
  int ret = 0;
  size_t lbsz; /* amount of data in lbuf */
  void *srvbuf = 0;
  void *sigdbuf = 0;
  void *lbuf = 0; /* signed buffer listing all servers */
  uint32_t cacheDataVersion = 0;  /* From the NIQS */
  char winmsg[WINMSG_SZ];   /* For creating windows tracing messages */
  uint32_t * entid = NULL;   /* We may need to load the available list */
  size_t entid_sz = 0;       /* Holds the size of the list */
  


  if ( ! ni) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "ni is null");
    return 0;
  }
  if ( ! cli) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "cli is null");
    return 0;
  }
  if ( ! kq) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "kq is null");
    return 0;
  }
  /*
   * This seems a safe place to do the DBO subsystem initialization
   */
  if (( ! frDBODriverIsInitialized(err)) && ( ! frDBODriverInitAll(err))) {
    goto done;
  }

  if ( !  getKeyQueryHostSocket (err, cli, ni, &kqh) ) {
    return 0;
  }

  /*
   * Get the current cache data version from the NIQS.  If we are doing an
   * update and this number has changed, nuke the contents of the frNetInfo
   * and reload everything.  If the GetCacheDataVersion() command fails (likely
   * because the NIQS did not have the appropriate command, a side effect
   * of the server transitions during the beta period)  We'll force a complete
   * reload of the cache.  If the cause of the failure was a signature error,
   * the next buffer which is read from the server will trigger the
   * appropriate response.
   */

  if (!frNiqsGetCacheDataVersion(err,cli,kqh,kq,&cacheDataVersion)) {
    zkErrorSet(err,fn,ZKWARN,ZKMOD,ZKERROR,
               "WARNING:  Failed to get cache data version, flushing cache");
    ni->cacheDataVersion = 0;
    cacheDataVersion = 1;
  }

  /*
   * If we are updating the frNetInfo object, we'll first check the cache
   * versions.  If they do not match, we destroy the contents of the 
   * frNetInfo and load a new one.  If the versions do match, we'll try
   * to get the list of available entities from the NIQS and cull the
   * frNetInfo object of old (no longer in use) entities.  If the
   * command to get that list fails, we can keep on going -- either we talked
   * to an old server, or the signature check failed, in which case this
   * function will fail when it gets the update list. 
   */

  if (update_flag) {

    /*  Check the cache version and force a reload if required */

    if (cacheDataVersion != ni->cacheDataVersion) {
      ClientTrace(fn,"FRNETINFO: CacheDataVersion has changed, flushing NIQS cache");
      CONSOLE_MSG(ni,"CacheDataVersion has changed, flushing NIQS cache");
      zkErrorSet_va(err,fn,ZKWARN,ZKMOD,ZKERROR,
                    "NIQS Cache data version number has changed to %lu",
                    (unsigned long) cacheDataVersion);

      /* Force the complete reload */
      since = 0;
      update_flag = 0;
  
      /* Clean out the frNetInfo object */ 
      if (!frNetInfoDeleteAll(err,ni)) {
        return 0;
      }

    /* Get list of available entities and delete old ones from frnetinfo */
     
    } else if (frNiqsGetAvailable(err,cli,kqh,kq,host_ent,he_sz,
                                  &entid,&entid_sz)) {
      ClientTrace(fn,"FRNETINFO: Culling entities from FrNetInfo structure");
      CONSOLE_MSG(ni,"Culling entities from FrNetInfo structure");
      if (!frNetInfoCullEntities(err,ni,entid,entid_sz)) {
        return 0;
      }

    /* Could not get the list.  Issue a warning and continue */

    } else {  
      ClientTrace(fn,"FRNETINFO: Failed to get list of active entities");
      CONSOLE_MSG(ni,"Failed to get list of active entities");
      zkErrorSet(err,fn,ZKWARN,ZKMOD,ZKERROR,
                 "WARNING:  Failed to get list of active entities, unable to cull frnetinfo");
    }

  }
 
  ni->cacheDataVersion = cacheDataVersion;
  
  /*
   *  Get a list of all servers from the NIQS, verifying the signature over
   *  its contents.
   */

  if ( ! (frNiqsList(err, cli, since, host_ent, he_sz, &lbuf, &lbsz) &&
          frNetInfoCheckSig(err, kqh, kq, lbuf, lbsz) &&
          frSecSigRemoveInSitu(err, lbuf, &lbsz))) {
    ClientTrace(fn,"FRNETINFO: Sig verification failure for entity list buffer");
    CONSOLE_MSG(ni,"Sig verification failure for entity list buffer");
    goto done;
  }

  /*
   *  Convert the buffer into a list of structures, using it to populate the 
   *  netinfo object.  
   */ 

  if ( ! (frNiqsListFromBuffer(err, lbuf, lbsz, &srvlist) && 
          zkListMoveCursor(err, srvlist, LIST_HEAD))) { 
    ClientTrace(fn,"FRNETINFO: Failed to recreate the entity list");
    CONSOLE_MSG(ni,"Failed to recreate the entity list");
    goto done;
  } else {
    FrNidbObjType type;
    FrNiqsEntity *srv;
    FrEntIdExp entity;
    const char *region_code = 0;
    const char *servop = 0;
    size_t i;
    size_t nbs; /* the number of an AIPs neighbours */
    size_t sigdbsz; /* size of data in sigdbuf */
    size_t sbsz; /* size of data in srvbuf */
    struct in_addr iaddr; /* server address */
    struct sockaddr_in saddr; /* datum uses this, rather than struct in_addr */
    uint16_t dhport; /* AIP Diffie-Hellman port */
    uint16_t port; /* main server port */
    uint32_t nbid;
    uint32_t rating; /* latest rating */
    uint32_t status; /* latest status of the aip */
    
    /* bug? tolerate errors, bailing out only if too many occur? */
    
    while (zkListGet(err, srvlist, (void **)&srv, LIST_FORWARDS)) {
      if (sigdbuf) {
        free(sigdbuf);
        sigdbuf = NULL;
      }

      if (srvbuf) {
        free(srvbuf);
        srvbuf = NULL;
      }

      {
        snprintf(winmsg,WINMSG_SZ,"%d/%s",
                 (int) srv->entity.type, srv->entity.name);
        ClientTrace(fn,winmsg);
        CONSOLE_MSG_IMPORTANT(ni,winmsg);
      }
 
      if ( ! (frNiqsQuery(err, cli, FRNIQ_ATTR_DESCRIPTION,
                          &(srv->entity), &sigdbuf, &sigdbsz) &&
              frNetInfoCheckSig(err, kqh, kq, sigdbuf, sigdbsz) &&
              frSecSigRemove(err, sigdbuf, sigdbsz, &srvbuf, &sbsz))) {
        snprintf(winmsg,WINMSG_SZ,
                 "Failed to verify signature on description buffer for %d/%s",
                 srv->entity.type,srv->entity.name);
        ClientTrace(fn,winmsg);
        CONSOLE_MSG(ni,winmsg);
        goto done;
      }

      /*
       * Now, extract the NidbObj from the buffer
       */
      if ( ! (frNidbObjUnpack(err, srvbuf, sbsz, &obj) &&
              frNidbObjGetType(err, obj, &type))) {
        snprintf(winmsg,WINMSG_SZ,
                 "FRNETINFO: Failed to unpack DBO for %d/%s",
                 srv->entity.type,srv->entity.name);
        ClientTrace(fn,winmsg);
        CONSOLE_MSG(ni,winmsg);
        goto done;
      }

      /*
       *  Verify the data received corresponds to the requested entity
       */

      if (!frNidbObjGetEntity(err, obj, &entity) ||
          (entity.type != srv->entity.type) ||
          (strcmp(entity.name, srv->entity.name) != 0)) {
        snprintf(winmsg,WINMSG_SZ,
                 "FRNETINFO: Failed to validate entity for %d/%s",
                 srv->entity.type,srv->entity.name);
        ClientTrace(fn,winmsg);
        CONSOLE_MSG(ni,winmsg);
        goto done;
      }

      /*
       *  Extract the information we need, copying it to a new FrNetInfoDatum,
       *  which is entered into the netinfo object.
       */

      if ( ! (frNetInfoDatumNew(err, &nid) &&
              frNetInfoDatumSetSignedBuffer(err, nid, sigdbuf,sigdbsz) &&
              frNetInfoDatumSetEntity(err, nid, srv->entity) &&
              frNetInfoDatumSetId(err, nid, srv->id))) {
        ClientTrace(fn,"FRNETINFO: Failed to create a new FrNetInfoDatum structure");
        CONSOLE_MSG(ni,"Failed to create a new FrNetInfoDatum structure");
        goto done;
      }

      if( (srv->entity.type==FRENT_TYPE_AIP)||(srv->entity.type==FRENT_TYPE_COREAIP) ) {
        if (type != FRNIDBOBJ_AIP) {
          zkErrorSet_va(err,fn,ZKERR,ZKMNIQS,ZKEDATABAD,
                        "Improper reponse type (%u) from server", type);

        } else {
          if ( ! (frNidbObjGetAipParams(err, obj, &iaddr, &dhport,
                                        &port, &status, &rating,
                                        &region_code,
                                        &region, &servop, &nbs) &&
                  sAddrInFromInAddr(&saddr, &iaddr, port) &&
                  frNetInfoDatumSetSockaddr(err, nid, saddr) &&
                  frNetInfoDatumSetDifHelPort(err, nid, dhport) &&
                  frNetInfoDatumSetRating(err, nid, rating) &&
                  frNetInfoDatumSetStatus(err, nid, status) &&
                  frNetInfoDatumSetRegionCode(err, nid, region_code) &&
                  frNetInfoDatumSetRegion(err, nid, region) &&
                  frNetInfoDatumSetOperator(err, nid, servop) )) {
            goto done;
          }
          FREEIT(region);
          /* Get the neighbours into the list too */
          for (i = 0; i < nbs; i++) {
            if (!(frNidbObjGetAipNeighbour(err, obj, i, &nbid) &&
                  frNetInfoDatumAddNb(err, nid, nbid))) {
              goto done;
            }
          }
        }
      } else if (type != FRNIDBOBJ_SERVER) {
        zkErrorSet_va(err,fn,ZKERR,ZKMNIQS,ZKEDATABAD,
                      "Improper reponse type (%u) from server", type);
      } else if ( ! (frNidbObjGetServerParams(err, obj, &iaddr, &port,
                                              &status, &rating, &region_code,
                                              &region, &servop) &&
                     sAddrInFromInAddr(&saddr, &iaddr, port) &&
                     frNetInfoDatumSetSockaddr(err, nid, saddr) &&
                     frNetInfoDatumSetRating(err, nid, rating) &&
                     frNetInfoDatumSetStatus(err, nid, status) &&
                     frNetInfoDatumSetRegionCode(err, nid, region_code) &&
                     frNetInfoDatumSetRegion(err, nid, region) &&
                     frNetInfoDatumSetOperator(err, nid, servop) )) {
        goto done;
      } else {
        FREEIT(region);
      }

      if (obj) {
        free(obj); obj = 0;
      }

      /*
      ** frNetInfoInsert takes care of removing an entry (by calling frNetInfoDeleteById )
      ** if it 's already in the datum
      ** it is really doing an insert OR replace job...
      */
      snprintf(winmsg,WINMSG_SZ,"Inserting/Replacing entity %d/%s in FrNetInfo",
               srv->entity.type,srv->entity.name);
      ClientTrace(fn,winmsg);
      CONSOLE_MSG(ni,winmsg); 
      
      if ( ! frNetInfoInsert(err, ni, nid)) {
          ClientTrace(fn,"FRNETINFO: Failed to insert/replace the entity");
          CONSOLE_MSG(ni,"Failed to insert/replace the entity"); 
          goto done;
      }

      nid = 0; /* it's been inserted... */
    }
  }

  /*
   *  bug: we could check the internal consistency of our object, here.  That
   *  is, check that all AIP neighbours are in our table, and that if A is a
   *  neighbour of B then B is a neighbour of A, etc.
   */

  ret = 1;
  ni->lastUpdate = time(NULL); 

  if (entid) {
    size_t fni_sz = 0;
    if (!frNetInfoCountRecords(err,ni,&fni_sz)) {
      goto done;
    }
    if (fni_sz != entid_sz) {
      zkErrorSet(err,fn,ZKERR,ZKMFRNETINFO,ZKERROR,
                 "Local and remote entity count inconsistency");
      CONSOLE_MSG(ni,"Local and remote entity count inconsistency"); 
      ret = 0;
    }
  }
  
 done:

  /* Cleanup */
  if (entid) {
    free(entid);
  }
  if (nid) { 
    frNetInfoDatumFree(0, &nid); 
  }
  if (srvlist) { 
    zkListFree(0, &srvlist); 
  }
  if (lbuf) {
    free(lbuf);
  }
  if (kqh.fd > -1) { 
#ifdef WIN32
	  closesocket(kqh.fd);
#else
	  close(kqh.fd); 
#endif
  }
  if (sigdbuf) { free(sigdbuf); } /* bug? */
  if (srvbuf) { free(srvbuf); } /* bug? */
  if (obj) { free(obj); }
  
  return ret;
}

DLLExport 
int frNetInfoDoStatusUpdateSince(
  ZkError err,
  FrNetInfo ni,
  ZkipClient cli,
  FrKeyQry kq,
  time_t since)
{
  static const char * func = "frNetInfoDoStatusUpdateSince";
  int ret = 0;
  int forceUpdate = 0;   /* Set to non-zero to force a complete reload */

  /*
   * Should we force the partial update to download a completely fresh
   * set of data from the NIQS?
   */

  if (ni->nextUpdate) {
    if (time(NULL) > ni->nextUpdate) {
      forceUpdate = 1;
      if (ni->updateOffset) {
        ni->nextUpdate = time(NULL) + ni->updateOffset;
      } else {
        ni->nextUpdate = 0;
      }
    }
  }

  if (!forceUpdate) {
    ret = frNetInfoDoSelectiveStatusUpdateSince(err, ni, cli, kq, 
                                                hostEnt, ASZ(hostEnt), since);
  }

  /*
   * If the status update fails, ditch the cache contents and try to reload
   * brand new data.
   */


  if (!ret) {
    FrNetInfo ni2 = NULL;
    void * ni2buf = NULL;
    size_t ni2buf_sz = 0;

    if (!forceUpdate) {
      zkErrorSet(err,func,ZKWARN,ZKMFRNETINFO,ZKERROR,
                 "Encountered error while updating NIQS information, "
                 "attempting to load clean cache");
      CONSOLE_MSG(ni,"NetInfo update failed, downloading fresh information");
    } else {
      zkErrorSet(err,func,ZKWARN,ZKMFRNETINFO,ZKERROR,
                 "Forcing clean reload of NIQS information"); 
      CONSOLE_MSG(ni,"Forcing clean reload of NIQS information");
    }

    if (!(frNetInfoNew(err,&ni2) &&
          frNetInfoSetClientConsoleCallback(err,ni2,
                                            ni->clicons,ni->clicons_data))) {
      ret = 0;
    } else if (!frNetInfoPopulateOrUpdateSince(err, ni2, cli, kq,
                                               hostEnt, ASZ(hostEnt), 0, 0)) {
      ret = 0;
    } else if (!frNetInfoStore(err,ni2,&ni2buf,&ni2buf_sz)) {
      ret = 0;
    } else {
      frNetInfoDeleteAll(err,ni);
      ret = frNetInfoLoad(err,ni,kq,ni2buf,ni2buf_sz);
    }

    if (ni2buf) {
      free(ni2buf);
    }
    if (ni2) {
      frNetInfoFree(NULL,&ni2);
    }
  }

  return ret;
}

/*
 *  Given a netinfo object, populate it with information about servers in the
 *  Freedom network. 
 *  
 *  Besides modifying the netinfo object, it will also insert some keys into
 *  the key query object.
 */
DLLExport
int frNetInfoPopulate(ZkError err, FrNetInfo ni, ZkipClient cli, FrKeyQry kq)
{

  return (frNetInfoPopulateOrUpdateSince(err, ni, cli, kq, hostEnt, ASZ(hostEnt), 0, 0));
}

/*
 *  Given a netinfo object, populate it with information about servers in the
 *  Freedom network. 
 *  
 *  Besides modifying the netinfo object, it will also insert some keys into
 *  the key query object.
 *
 *  Supply list of entities to populate the netinfo DB with.
 */
DLLExport
int frNetInfoSelectivePopulate(
  ZK_INOUT ZkError err,
  ZK_INOUT FrNetInfo ni,
  ZK_INOUT ZkipClient cli,
  ZK_IN    FrKeyQry kq,
  ZK_IN    FrEntIdType *host_ent,
  ZK_IN    unsigned int host_ent_sz) 
{

  return (frNetInfoPopulateOrUpdateSince(err, ni, cli, kq, host_ent, host_ent_sz, 0, 0));
}


/*
 *  Update the netinfo object with any changes `since' a given time.
 *  
 *  bug: almost identical to Populate().  Have Populate() merely call
 *  UpdateSince() with a time of 0?
 */
DLLExport
int frNetInfoUpdateSince(ZkError err, FrNetInfo ni, ZkipClient cli, FrKeyQry kq, time_t since)
{
  static const char * func = "frNetInfoUpdateSince";
  int ret = 0;        /* Function return code */
  int forceUpdate = 0;   /* Set to non-zero to force a complete reload */

  (void) module_id_;

  if (ni->nextUpdate) {
    if (time(NULL) > ni->nextUpdate) {
      forceUpdate = 1;
      if (ni->updateOffset) {
        ni->nextUpdate = time(NULL) + ni->updateOffset;
      } else {
        ni->nextUpdate = 0;
      }
    }
  }

  if (!forceUpdate) {
    ret = frNetInfoPopulateOrUpdateSince(err, ni, cli, kq, 
                                         hostEnt, ASZ(hostEnt), since, 1);
  }

  
  /*
   * If the update fails, delete everything in the cache and try to load it
   * fresh.  We probably had bad data in there somewhere which prevented it
   * from being updated.
   */

  if (!ret) {
    FrNetInfo ni2 = NULL;
    void * ni2buf = NULL;
    size_t ni2buf_sz = 0;
   
    if (!forceUpdate ) {
      zkErrorSet(err,func,ZKWARN,ZKMFRNETINFO,ZKERROR,
                 "Encountered error while updating NIQS information, "
                 "attempting to load clean cache");
      CONSOLE_MSG(ni,"NetInfo update failed, downloading fresh information");
    } else {
      zkErrorSet(err,func,ZKWARN,ZKMFRNETINFO,ZKERROR,
                 "Forcing clean reload of NIQS information"); 
      CONSOLE_MSG(ni,"Forcing clean reload of NIQS information");
    }
    if (!(frNetInfoNew(err,&ni2) &&
          frNetInfoSetClientConsoleCallback(err,ni2,
                                            ni->clicons,ni->clicons_data))) {
      ret = 0;
    } else if (!frNetInfoPopulateOrUpdateSince(err, ni2, cli, kq, 
                                               hostEnt, ASZ(hostEnt), 0, 0)) {
      ret = 0;
    } else if (!frNetInfoStore(err,ni2,&ni2buf,&ni2buf_sz)) {
      ret = 0;
    } else {
      frNetInfoDeleteAll(err,ni);
      ret = frNetInfoLoad(err,ni,kq,ni2buf,ni2buf_sz);
    }

    if (ni2buf) {
      free(ni2buf);
    }
    if (ni2) {
      frNetInfoFree(NULL,&ni2);
    }
  }
  return ret;
}


#if 1
/* not used anymore BUT must keep it till the OLD_* functions are in */
/**
 * Given a populated netinfo object, update the status and rating fields
 * with a query to the niqs. New objects will be ignored
 */

static 
int frNetInfoDoSelectiveStatusUpdateSince(
  ZkError err,
  FrNetInfo ni,
  ZkipClient cli,
  FrKeyQry kq,
  FrEntIdType *host_ent,
  unsigned int he_sz,
  time_t since)
{
  static const char *fn = "frNetInfoDoSelectiveStatusUpdateSince";
  FrKeyQryHost kqh;
  FrNetInfoDatum nid = 0;
  FrNidbObj obj = 0;
  int ret = 0;
  size_t lbsz; /* amount of data in lbuf */
  void *lbuf = 0; /* signed buffer listing all servers */
  


  if ( ! ni) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "ni is null");
    return 0;
  }
  if ( ! cli) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "cli is null");
    return 0;
  }
  if ( ! kq) {
    zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "kq is null");
    return 0;
  }

  /*
   * This seems a safe place to do the DBO subsystem initialization
   */
  if (( ! frDBODriverIsInitialized(err)) && ( ! frDBODriverInitAll(err))) {
    goto done;
  }

  if ( !  getKeyQueryHostSocket (err, cli, ni, &kqh) )
    return 0;
      
  /*
   *  Get a status update of all entities since the specified timem from the
   *  NIQS, verifying the signature over its contents.
   */

  CONSOLE_MSG(ni,"Downloading entity status list");
  if ( ! (frNiqsStatus(err, cli, since, host_ent, he_sz, &lbuf, &lbsz) &&
          frNetInfoCheckSig(err, kqh, kq, lbuf, lbsz) &&
          frSecSigRemoveInSitu(err, lbuf, &lbsz))) {
    goto done;
  } else {

    /* Now scan the reponse and update the nidb */

    FrNidbObjType type;
    size_t i, nresp = 0;
    uint32_t ent_ident; /* the entity identifier */
    uint32_t rating; /* latest rating */
    uint32_t status; /* latest status of the aip */
    
    /*
     * Now, extract the NidbObj from the buffer
     */
    if ( ! (frNidbObjUnpack(err, lbuf, lbsz, &obj) &&
            frNidbObjGetType(err, obj, &type))) {
      goto done;
    }
    if (type != FRNIDBOBJ_STATUS_RESP) {
      zkErrorSet_va(err,fn,ZKERR,ZKMNIQS,ZKEDATABAD,
                    "Improper reponse type (%u) from server", type);
      
    } else {
      if ( ! frNidbObjGetStatusParams(err, obj, &nresp) )
        goto done;
      
      /* Get the responses and update the netinfo structure  */
      for (i = 0; i < nresp; i++) {
        if ( ! frNidbObjGetStatusRespAtIdx (err, obj, i,
                                            &ent_ident, &status, &rating) ) {
          goto done;

        } else {
          if ( ! frNetInfoSearchById(err, ni, ent_ident, &nid) )
            goto done;
          if ( ! nid ) {
            zkErrorSet_va(err,fn,ZKWARN,ZKMOD,ZKENOTFOUND,
                    "Entry for (%u) not found", ent_ident);
          } else {
            if ( ! (frNetInfoDatumSetStatus(err, nid, status) &&
                    frNetInfoDatumSetRating(err, nid, rating)) ) 
              goto done;
          }
        }
      }
      
      if (obj) {
        free(obj); obj = 0;
      }

      nid = 0; /* it's been inserted... */
    }
  }

  ret = 1;
  
 done:

  if (nid) { frNetInfoDatumFree(0, &nid); }
  if (kqh.fd > -1) { 
#ifdef WIN32
	  closesocket(kqh.fd);
#else
	  close(kqh.fd); 
#endif
  }
  if (lbuf) { free(lbuf); } 
  if (obj) { free(obj); }
  
  return ret;
}

#endif



static FrEntIdType OLD_hostEnt[] = {
  FRENT_TYPE_AIP,
  FRENT_TYPE_FMG,   /* no more FMG in the next version */
  FRENT_TYPE_MAIP,  /* no more MAIP in the next version */
  FRENT_TYPE_NYM_SRV,
  FRENT_TYPE_SER_SRV,           
  FRENT_TYPE_TOK_SRV,
  FRENT_TYPE_KEYQRY_SRV,
  FRENT_TYPE_KEYUPD_SRV,
  FRENT_TYPE_NNTP_SRV,
  FRENT_TYPE_NIQS,
  FRENT_TYPE_NISS
};


/*
**-----------------------------------------------------------------------------
**
** (Modified version of frNetInfoPopulateOrUpdateSince)
** - NOT doing getKeyQueryHostSocket() nor frNiqsGetCacheDataVersion
**
**-----------------------------------------------------------------------------
*/

static int
OLD_frNetInfoPopSince(
    ZkError err,
    FrNetInfo ni,
    ZkipClient cli,
    FrKeyQryHost kqh,
    FrKeyQry kq,
    FrEntIdType *host_ent,
    unsigned int he_sz,
    uint32_t cacheDataVersion,  /* From the NIQS */
    time_t since
)
{
#define WINMSG_SZ 128

    static const char * fn = "frNetInfoPopSince";
    int ret = 0;

    FrNetInfoDatum nid = 0;
    FrNidbObj obj = 0;
    char *region = 0; /* This needs to be freed. Because of the goto's I have
                         to change the scope of this pointer, to be able to do a
                         free at the end */
    ZkList srvlist = 0;
    void *srvbuf = 0;
    void *sigdbuf = 0;

    void *lbuf = 0; /* signed buffer listing all servers */
    size_t lbsz; /* amount of data in lbuf */

    char winmsg[WINMSG_SZ];   /* For creating windows tracing messages */


    if ( ! ni) {
        zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "ni is null");
        return 0;
    }
    if ( ! cli) {
        zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "cli is null");
        return 0;
    }
    if ( ! kq) {
        zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "kq is null");
        return 0;
    }


    ni->cacheDataVersion = cacheDataVersion;
  
    /*
    **  Get a list of all servers from the NIQS, verifying the signature over
    **  its contents.
    */

    if ( ! (frNiqsList(err, cli, since, host_ent, he_sz, &lbuf, &lbsz) &&
            frNetInfoCheckSig(err, kqh, kq, lbuf, lbsz) &&
            frSecSigRemoveInSitu(err, lbuf, &lbsz))) {
        ClientTrace(fn,"FRNETINFO: Sig verification failure for entity list buffer");
        CONSOLE_MSG(ni,"Sig verification failure for entity list buffer");
        goto done;
    }

    /*
    ** Convert the buffer into a list of structures, using it to populate the 
    ** netinfo object.  
    */ 

    if ( ! (frNiqsListFromBuffer(err, lbuf, lbsz, &srvlist) && 
            zkListMoveCursor(err, srvlist, LIST_HEAD))) { 
        ClientTrace(fn,"FRNETINFO: Failed to recreate the entity list");
        CONSOLE_MSG(ni,"Failed to recreate the entity list");
        goto done;
    } else {
        FrNidbObjType type;
        FrNiqsEntity *srv;
        FrEntIdExp entity;
        const char *region_code = 0;
        const char *servop = 0;
        size_t i;
        size_t nbs;               /* the number of an AIPs neighbours */
        size_t sigdbsz;           /* size of data in sigdbuf */
        size_t sbsz;              /* size of data in srvbuf */
        struct in_addr iaddr;     /* server address */
        struct sockaddr_in saddr; /* datum uses this, rather than struct in_addr */
        uint16_t dhport;          /* AIP Diffie-Hellman port */
        uint16_t port;            /* main server port */
        uint32_t nbid;
        uint32_t rating;          /* latest rating */
        uint32_t status;          /* latest status of the aip */
    
 
        /* bug? tolerate errors, bailing out only if too many occur? */
        while (zkListGet(err, srvlist, (void **)&srv, LIST_FORWARDS)) {
            if (sigdbuf) {
                free(sigdbuf);
                sigdbuf = NULL;
            }

            if (srvbuf) {
                free(srvbuf);
                srvbuf = NULL;
            }

            {
                snprintf(winmsg,WINMSG_SZ,"%d/%s",
                         (int) srv->entity.type, srv->entity.name);
                ClientTrace(fn,winmsg);
                CONSOLE_MSG_IMPORTANT(ni,winmsg);
            }
 
            if ( ! (frNiqsQuery(err, cli, FRNIQ_ATTR_DESCRIPTION,
                                &(srv->entity), &sigdbuf, &sigdbsz) &&
                    frNetInfoCheckSig(err, kqh, kq, sigdbuf, sigdbsz) &&
                    frSecSigRemove(err, sigdbuf, sigdbsz, &srvbuf, &sbsz))) {
                snprintf(winmsg,WINMSG_SZ,
                         "Failed to verify signature on description buffer for %d/%s",
                         srv->entity.type,srv->entity.name);
                ClientTrace(fn,winmsg);
                CONSOLE_MSG(ni,winmsg);
                goto done;
            }

            /*
             * Now, extract the NidbObj from the buffer
             */
            if ( ! (frNidbObjUnpack(err, srvbuf, sbsz, &obj) &&
                    frNidbObjGetType(err, obj, &type))) {
                snprintf(winmsg,WINMSG_SZ,
                         "FRNETINFO: Failed to unpack DBO for %d/%s",
                         srv->entity.type,srv->entity.name);
                ClientTrace(fn,winmsg);
                CONSOLE_MSG(ni,winmsg);
                goto done;
            }

            /*
             *  Verify the data received corresponds to the requested entity
             */

            if (!frNidbObjGetEntity(err, obj, &entity) ||
                (entity.type != srv->entity.type) ||
                (strcmp(entity.name, srv->entity.name) != 0)) {
                snprintf(winmsg,WINMSG_SZ,
                         "FRNETINFO: Failed to validate entity for %d/%s",
                         srv->entity.type,srv->entity.name);
                ClientTrace(fn,winmsg);
                CONSOLE_MSG(ni,winmsg);
                goto done;
            }

            /*
             *  Extract the information we need, copying it to a new FrNetInfoDatum,
             *  which is entered into the netinfo object.
             */

            if ( ! (frNetInfoDatumNew(err, &nid) &&
                    frNetInfoDatumSetSignedBuffer(err, nid, sigdbuf,sigdbsz) &&
                    frNetInfoDatumSetEntity(err, nid, srv->entity) &&
                    frNetInfoDatumSetId(err, nid, srv->id))) {
                ClientTrace(fn,"FRNETINFO: Failed to create a new FrNetInfoDatum structure");
                CONSOLE_MSG(ni,"Failed to create a new FrNetInfoDatum structure");
                goto done;
            }

            if (srv->entity.type == FRENT_TYPE_AIP) {
                if (type != FRNIDBOBJ_AIP) {
                    zkErrorSet_va(err,fn,ZKERR,ZKMNIQS,ZKEDATABAD,
                                  "Improper reponse type (%u) from server", type);

                } else {
                    if ( ! (frNidbObjGetAipParams(err, obj, &iaddr, &dhport,
                                                  &port, &status, &rating,
                                                  &region_code,
                                                  &region, &servop, &nbs) &&
                            sAddrInFromInAddr(&saddr, &iaddr, port) &&
                            frNetInfoDatumSetSockaddr(err, nid, saddr) &&
                            frNetInfoDatumSetDifHelPort(err, nid, dhport) &&
                            frNetInfoDatumSetRating(err, nid, rating) &&
                            frNetInfoDatumSetStatus(err, nid, status) &&
                            frNetInfoDatumSetRegionCode(err, nid, region_code) &&
                            frNetInfoDatumSetRegion(err, nid, region) &&
                            frNetInfoDatumSetOperator(err, nid, servop) )) {
                        goto done;
                    }
                    FREEIT(region);
                    /* Get the neighbours into the list too */
                    for (i = 0; i < nbs; i++) {
                        if (!(frNidbObjGetAipNeighbour(err, obj, i, &nbid) &&
                              frNetInfoDatumAddNb(err, nid, nbid))) {
                            goto done;
                        }
                    }
                }
            } else if (type != FRNIDBOBJ_SERVER) {
                zkErrorSet_va(err,fn,ZKERR,ZKMNIQS,ZKEDATABAD,
                              "Improper reponse type (%u) from server", type);
            } else if ( ! (frNidbObjGetServerParams(err, obj, &iaddr, &port,
                                                    &status, &rating, &region_code,
                                                    &region, &servop) &&
                           sAddrInFromInAddr(&saddr, &iaddr, port) &&
                           frNetInfoDatumSetSockaddr(err, nid, saddr) &&
                           frNetInfoDatumSetRating(err, nid, rating) &&
                           frNetInfoDatumSetStatus(err, nid, status) &&
                           frNetInfoDatumSetRegionCode(err, nid, region_code) &&
                           frNetInfoDatumSetRegion(err, nid, region) &&
                           frNetInfoDatumSetOperator(err, nid, servop) )) {
                goto done;
            } else {
                FREEIT(region);
            }

            if (obj) {
                free(obj); obj = 0;
            }

            /*
            ** insert/replace data in ni object
            */
            snprintf(winmsg,WINMSG_SZ,"Inserting/replacing entity %d/%s in FrNetInfo",
                     srv->entity.type,srv->entity.name);
            ClientTrace(fn,winmsg);
            CONSOLE_MSG(ni,winmsg); 

            if ( ! frNetInfoInsert(err, ni, nid)) {
                ClientTrace(fn,"FRNETINFO: Failed to insert/replace the entity");
                CONSOLE_MSG(ni,"Failed to insert/replace the entity"); 
                goto done;
            }

            nid = 0; /* it's been inserted... */
        }
    }

    /*
     *  bug: we could check the internal consistency of our object, here.  That
     *  is, check that all AIP neighbours are in our table, and that if A is a
     *  neighbour of B then B is a neighbour of A, etc.
     */

    ret = 1;
    ni->lastUpdate = time(NULL); 

 done:

    /* Cleanup */
    if (nid) { 
        frNetInfoDatumFree(0, &nid); 
    }
    if (srvlist) { 
        zkListFree(0, &srvlist); 
    }
    if (lbuf) {
        free(lbuf);
    }
    if (kqh.fd > -1) { 
#ifdef WIN32
        closesocket(kqh.fd);
#else
        close(kqh.fd); 
#endif
    }
    if (sigdbuf) { free(sigdbuf); } /* bug? */
    if (srvbuf) { free(srvbuf); } /* bug? */
    if (obj) { free(obj); }
  
    return ret;
}

/*
**-----------------------------------------------------------------------------
** This is the function that performs the init sequence for the client
**-----------------------------------------------------------------------------
*/

static int 
OLD_frNetInfoRunClientInitSequence(
    ZK_INOUT ZkError            err,          /* error structure */
    ZK_INOUT ZkLog              log,          /* log structure */
    ZK_INOUT FrNetInfo          ni,           /* NetInfo object */
    ZK_IN    ZkipClient         cli,          /* zkip client information */
    ZK_IN    FrKeyQry           kqCache,      /* key cache  -- was kq */
    ZK_IN    const char *       clientId,     /* client string identifier */
    ZK_OUT   FrNiqsClientInfo * pClientInfo,  /* pointer to client info structure */
    ZK_OUT   FrTokenType *      pTokenTypes   /* pointer to token type structure */
)
{
    static const char * fn = "frNetInfoRunClientInitSequence";
    int ret = 0;

    struct sockaddr saddr;
    FrKeyQryHost kqHost;                /* key query host info */

    uint32_t  clientCacheVersion = ni->cacheDataVersion;
    uint32_t     niqsCacheVersion = 0;  /* From the NIQS */
    int doFullUpdate = 0;

    /*
     * This seems a safe place to do the DBO subsystem initialization
     */
    if ( ! frDBODriverIsInitialized(err) && ! frDBODriverInitAll(err)) {
        zkLogWriteContext(log, err, fn, "call to frDBODriverIsInitialized AND frDBODriverInitAll failed");
    }

    /*
    ** get the key query host address
    ** don't take the one in ni since it could have change
    */
    else if ((kqHost.fd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
        zkErrorSet_va(err, fn, ZKERR, ZKMOD, errno,
                      "can't create key query socket: %s", strerror(errno));
    }
    else if ( ! OLD_frNiqsGetKeyQryAddr(err, cli, &saddr)) {
        zkLogWriteContext(log, err, fn, "call to frNiqsGetKeyQryAddr failed");
    }
    else if ( ! memcpy(&(kqHost.host), &saddr, sizeof (kqHost.host)))  {
        zkErrorSet(err,fn,ZKERR,ZKMOD,ZKERROR, "call to memcpy() failed");
    }

    /*
    ** get cache version
    */
    else if ( ! frNiqsGetCacheDataVersion(err,cli,kqHost,kqCache,&niqsCacheVersion)) {
        zkLogWriteContext(log, err, fn, "call to frNiqsGetCacheDataVersion failed");
    }
    
    /*
    ** the processing to do will depend on cache version and age of the
    ** client's cache file
    */
    else {
        
        /*
        ** if client has no cache 
        ** OR doesn't has the latest from NIQS
        ** OR we couldn't get the cache version from NIQS *
        ** --> full update
        */
        if( clientCacheVersion != niqsCacheVersion ) {

            zkLogWriteContext(log, NULL, fn, "[client init] cache is out of sync; full update");
            CONSOLE_MSG(ni,"Client's cache is out of sync; performing full update");

            doFullUpdate = 1;
        }

        /*
        ** if neither a partial nor a full update is performed, we still need to update aip state
        */
        else {

            FrEntIdType aip = FRENT_TYPE_AIP;

            zkLogWriteContext(log, NULL, fn, "[client init] everything's fine; AIP update");
            CONSOLE_MSG(ni,"Updating AIP status");

            if ( ! frNetInfoDoSelectiveStatusUpdateSince(err, ni, cli, kqCache, &aip, 1, 0)) {
                zkLogWriteContext(log, err, fn, "call to frNetInfoDoSelectiveStatusUpdateSince failed");
                CONSOLE_MSG(ni,"Failed to get current AIP status");
                  }
            else {
                /* 
                ** it's a success ! 
                */
                ret = 1;
            }
        }

        /*
        ** if we need to, we do a full update here
        */
        if( doFullUpdate ) {
            /* Clean out the frNetInfo object */
            if (!frNetInfoDeleteAll(err,ni)) {
                zkLogWriteContext(log, err, fn, "call to frNetInfoDeleteAll failed");
            }
            /* update the FrNetInfo structure */
            else if( ! OLD_frNetInfoPopSince(err, ni, cli, kqHost, kqCache, OLD_hostEnt, ASZ(OLD_hostEnt),
                                         niqsCacheVersion, 0)) {
                zkLogWriteContext(log, err, fn, "call to frNetInfoPopSince failed");
            }
            else {
                /*
                ** it's a success !
                */
                ret = 1;
            }
        }

    }

    /*
    ** get token info and client info
    */
    if (1 == ret) {
        if ( ! frNiqsGetTokenInfo (err, cli, kqHost, kqCache, pTokenTypes)) {
            zkLogWriteContext(log, err, fn, "call to frNiqsGetTokenInfo failed");
            ret = 0;
        }

        else if ( ! frNiqsGetClientInfo(err, cli, kqHost, kqCache, clientId, pClientInfo)) {
            zkLogWriteContext(log, err, fn, "call to frNiqsGetClientInfo failed");
            ret = 0;
        }
    }


    /* close the socket */
    if (kqHost.fd > -1) { 
#ifdef WIN32
        closesocket(kqHost.fd);
#else
        close(kqHost.fd); 
#endif
    }

    return ret;
}



/*
**-----------------------------------------------------------------------------
** This is the function that the client calls at startup
**-----------------------------------------------------------------------------
*/
DLLExport int 
OLD_frNetInfoClientInitSequence(
    ZK_INOUT ZkError            err,          /* error structure */ 
    ZK_INOUT FrNetInfo          ni,           /* NetInfo object */
    ZK_IN    const char *       zkipHost ,    /* host address */
    ZK_IN    uint16_t           zkipPort ,    /* port address */
    ZK_IN    uint16_t           zkipTimeout,  /* connection timeout */
    ZK_IN    FrKeyQry           kqCache,      /* key cache  -- was kq */
    ZK_IN    const char *       clientId,     /* client string identifier */
    ZK_OUT   FrNiqsClientInfo * pClientInfo,  /* pointer to client info structure */
    ZK_OUT   FrTokenType *      pTokenTypes   /* pointer to token type structure */
)
{
    static const char * fn = "frNetInfoClientInitSequence";
    int ret = 1;

    ZkLog        log;
    ZkipClient   cli = NULL;  /* zkip client information */


    /* parameter validation */
    if ( !ni || !zkipHost || !kqCache || !clientId || !pClientInfo || !pTokenTypes ) {
        zkErrorSet(err, fn, ZKERR, ZKMOD, ZKENULLPTR, "null parameter passed");
        ret = 0;
    }

    /* get the log pointer from ZkError */
    else if( ! zkErrorLog(err, &log )) {
        zkErrorSet(err,fn,ZKERR,ZKMOD,ZKERROR, "call to zkErrorLog failed");
        ret = 0;
    }

    /*
    ** connect
    */
    else if( ! frNiqsConnect(err, &cli, zkipHost, zkipPort, zkipTimeout)) {
        zkLogWriteContext(log, err, fn, "call to frNiqsConnect failed");
        ret = 0;
    }
    /*
    ** perform the actual init sequence
    */
    else if ( ! OLD_frNetInfoRunClientInitSequence(err, log, ni, cli, kqCache, clientId, pClientInfo, pTokenTypes)) {
        zkLogWriteContext(log, err, fn, "call to frNetInfoRunClientInitSequence failed");
        ret = 0;
    }

    /*
    ** disconnect (must do it regardless of the result of previous command)
    */
    if ( cli && ( ! frNiqsDisconnect(err, &cli)) ) {
        zkLogWriteContext(log, err, fn, "call to frNiqsDisconnect failed");
        ret = 0;
    }

    return ret;
}
