/*
 * $Id: frNiqs.h,v 1.26.8.5 2000/12/14 17:53:03 jean Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRNIQ_H_
#define FRNIQ_H_

#include <zkList.h>

#ifndef WIN32
#include <sys/socket.h>
#else
#include <winsock2.h>
#endif

#ifndef _ZKIP_H
#include <ZKIP.h>
#endif /* _ZKIP_H */

#ifndef ZKIPCLIENT_H_
#include <zkipClient.h>
#endif /* ZKIPCLIENT_H_ */

#ifndef FRENT_H_
#include <frEnt.h>
#endif /* FRENT_H_ */

#ifndef ZKDATETIME_H_
#include <zkDateTime.h>
#endif /* ZKDATETIME_H_ */

#include "zkConfig.h"
#include "frKeyQry.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef enum {
  FRNIQ_ATTR_TOPOLOGY    = 0x1,
  FRNIQ_ATTR_DESCRIPTION = 0x2,
  FRNIQ_ATTR_LIST        = 0x3,
  FRNIQ_ATTR_STATUS      = 0x4,
  FRNIQ_ATTR_UNKNOWN     = 0x5
} FrNiqAttr;

/*
 * The functions frNiqsResponseToList() and frNiqsList() return a zkList where
 * each element is a pointer to a structure defined as follows
 */

struct T_FrNiqsEntity {
  FrEntIdExp entity;       /* The entity structure */
  uint32_t id;             /* The entity's NIQS assigned unique ID number */
  time_t update;           /* The time when server entry was last updated */
};

typedef struct T_FrNiqsEntity FrNiqsEntity;

/*
 *  Client info structure.  You allocate the space for it and load it with
 *  the functions frNiqsClientInfoLoadFromBuffer() if you already have
 *  a buffer, or with frNiqsGetClientInfo() if you just have a connection
 *  to the NIQS.
 */

#define CI_SNURL_SZ 256
#define CI_DOMAIN_SZ 128
#define CI_CI_SZ 32

/*
 * %%Note:
 *  The client info should in fact support a list of URLs and a list of
 *  product domains (so that we can simultenously support more than just
 *  freedom.net)
 */

struct T_FrNiqsClientInfo {
  char client_id[CI_CI_SZ];     /* The client identifier string */
  char domain[CI_DOMAIN_SZ];    /* The domain name (Usually freedom.net) */
  char sn_url[CI_SNURL_SZ];     /* The serial number URL 
                                 * (points to the scalper) 
                                 */ 
  char upgrade_url[CI_SNURL_SZ]; /* URL which points to client upgrade */
  int upgrade_ready;            /* Non-zero when there is an upgrade ready */
  ZkDateTime expiry ;           /* The expiry */
  time_t expire_in;             /* Number of seconds left until the client 
                                 * expires.  This value will be accurate to 
                                 * within a minute or so.
                                 */
  time_t current_UTC;           /* The current UTC time in seconds since 
                                 * 00:00:00 UTC, january 1, 1970
                                 */
  char * motd;                  /* Message of the day */
  size_t motd_sz;               /* Size of the MOTD string 
                                 * (not counting terminating NUL) 
                                 */
  int motd_urg;                 /* MOTD urgency (for future use, always 1) */
  int niqs_update;              /* NIQS Update Frequency (in seconds) */

};

typedef struct T_FrNiqsClientInfo FrNiqsClientInfo;

/*
 * Client info parameters.  You dont have to worry about these, they are
 * just informational at this point so that you'll know what is in the client
 * info buffer and there is an easy place to find them for whoever sets
 * up the client info file on the NIQS server
 */

#define NIQS_CLIENT_INFO_EXPIRY       "expiry"
#define NIQS_CLIENT_INFO_EXPIRE_IN    "expire-in"
#define NIQS_CLIENT_INFO_SN_URL       "sn-URL"
#define NIQS_CLIENT_INFO_UPGRADE_URL  "upgrade-URL"
#define NIQS_CLIENT_INFO_CURRENT_UTC  "current-UTC"
#define NIQS_CLIENT_INFO_MOTD         "motd"
#define NIQS_CLIENT_INFO_MOTD_URG     "motd-urg"
#define NIQS_CLIENT_INFO_DOMAIN       "domain"
#define NIQS_CLIENT_INFO_UPGRADE_READY "upgrade-ready"
#define NIQS_CLIENT_INFO_NIQS_UPDATE   "niqs-update"

/*
 *
 */

DLLExport int frNiqsConnect (
  ZkError ,           /* Returns the error if any occured */
  ZkipClient * ,      /* Allocates and returns a new ZKIP client struct */
  const char * ,      /* NIQS Server host name */
  uint16_t ,          /* NIQS server port number */
  uint16_t );         /* NIQS server connection timeout */

DLLExport int frNiqsDisconnect (
  ZkError ,           /* Returns the error if any occured */
  ZkipClient * );     /* The ZKIP client to disconnect (ptr reset to NULL)*/

/*
 *
 */

DLLExport int frNiqsClientInfo (
  ZkError ,           /* Return the error if any occured */
  ZkipClient  ,       /* The zkip client structure connected to the NIQS */
  const char * ,      /* A string identifying the client ID */
  void ** ,           /* Returns a signed data buffer parsable as a ZkConfig */
  size_t * );         /* Size of the signed response */

DLLExport int frNiqsClientInfoLoadFromBuffer (
  ZkError ,            /* Return the error if one occured */
  FrNiqsClientInfo * , /* CLient info structure to load */
  const char *  ,      /* Client ID string */
  void *  ,            /* The niqs response buffer (signature removed ) */
  size_t  );           /* Size of the buffer */


/*
 * Use this function to get the client information from the NIQS.  Use one
 * of the CLIENT_ID strings above to identify the client.
 */

DLLExport int frNiqsGetClientInfo (
  ZkError ,              /* Return the error if one occured */
  ZkipClient  ,          /* The zkip client connection */
  FrKeyQryHost ,       /* key server host addr and descriptor */
  FrKeyQry ,            /* The key query cache */
  const char * ,         /* Client ID string (see above) */
  FrNiqsClientInfo * );  /* structure to load */


DLLExport int frNiqsAccessCtrlList (
  ZkError ,           /* Return the error if any occured */
  ZkipClient  ,       /* The zkip client structure connected to the NIQS */
  void ** ,           /* Returns a signed data buffer parsable as a ZkConfig */
  size_t * );         /* Size of the signed response */

/*
 * Use this function to get the access control list from the NIQS.  
 */

DLLExport int frNiqsGetAccessCtrlList (
  ZkError ,           /* Return the error if one occured */
  ZkipClient  ,       /* The zkip client connection */
  FrKeyQryHost ,      /* key server host addr and descriptor */
  FrKeyQry ,          /* The key query cache */
  ZkConfig );         /* structure to load */


/*
 * These two functions are used to get the NIQS's current cache data version 
 * number.  The first function returns a signed buffer, the second number
 * uses's the client key query cache to verify the buffer and return the
 * number itself.  If this number does not match the client's frNetInfo
 * cache data version number, you must destroy the contents of the
 * frNetInfo data object and reload it from scratch.
 */

DLLExport int frNiqsCacheDataVersion (
  ZkError ,                  /* Return the error if one occured */
  ZkipClient ,               /* Zkip client connection */
  void ** ,                  /* Returns signed cache data version # buffer */
  size_t * );                /* Size of the buffer */

DLLExport int frNiqsGetCacheDataVersion (
  ZkError ,                  /* Return the error if one occured */
  ZkipClient ,               /* Zkip client connection */
  FrKeyQryHost  ,
  FrKeyQry  ,
  uint32_t * );               /* Returns the cache version number */

/*
 * The next two functions contain an NidbObject that can be exrtacted once
 * the signature has been verified and removed
 */

DLLExport int frNiqsQuery (
  ZkError ,           /* Return the error if any occured */
  ZkipClient ,        /* The ZKIP client structure connected to the NIQS */
  FrNiqAttr ,         /* The requested attribute */
  FrEntIdExp * ,      /* The requested entity */
  void ** ,           /* Allocates and returns a pointer to data */
  size_t * );         /* Returns the size of the data */

DLLExport int frNiqsStatus (
  ZkError ,           /* Returns the error if one occured */
  ZkipClient ,        /* The zkip client structure connected to the niqs */
  time_t ,            /* New data since this time (should always set to 0) */
  FrEntIdType * ,     /* (array) Type of entity for which we want a list */
  size_t ,            /* Number of entities in the array */
  void ** ,           /* Returns a buffer containing the response */
  size_t * );         /* Size of the buffer */


/*
 * Once you've checked the signature on this buffer, you can convert it to
 * a zkList using the frNiqsListFromBuffer() function 
 */

DLLExport int frNiqsList (
  ZkError ,           /* Returns the error if one occured */
  ZkipClient ,        /* The zkip client structure connected to the niqs */
  time_t ,            /* New data since this time (should always set to 0) */
  FrEntIdType * ,     /* (array) Type of entity for which we want a list */
  size_t ,            /* Number of entities in the array */
  void ** ,           /* Returns a buffer containing the list */
  size_t * );         /* Size of the buffer */

DLLExport int
frNiqsMxQuery (
    ZkError err,       /* Returns the error if one occured */
    ZkipClient client, /* The zkip client structure connected to the NIQS */
    FrNiqAttr attr,    /* The requested attribute */
    time_t since,      /* new data since this time (should always be set to 0) */
    uint16_t encoding,
    FrEntIdType *type, /* (array) Type of entity for which we want to query */
    size_t types,      /* the number of entity in the array */
    void **data,       /* Returns a buffer containing the list */
    size_t *sz         /* size of the buffer */
    );

/*
 * This function is used by frNiqsQuery() and frNiqsList().  You should not
 * call this yourself unless you know very well how to formulate ZKIP 
 * commands.
 */

DLLExport int frNiqsCommand (
  ZkError ,           /* Returns the error if any occured */
  ZkipClient ,        /* The ZKIP client structure connected to the NIQS */
  char * ,            /* The command string */
  void ** ,           /* Allocates and returns a pointer to the data */
  size_t * );         /* Returns the size of the data */

/*
 * For looking up attribute query tokens
 */

DLLExport int frNiqsTokenToAttribute (
  ZkError ,           /* Returns the error if one occured */
  const char * ,      /* The token to lookup */
  FrNiqAttr * );      /* Returns the attribute code */
 
DLLExport int frNiqsAttributeToToken (
  ZkError ,           /* Returns the error if one occured */
  FrNiqAttr  ,        /* Returns the attribute to lookup */
  const char ** );    /* returns the The token */

/*
 * For converting buffers returned by frNiqsList() to a ZkList object.
 * Be sure to check and remove the signature first.
 */

DLLExport int frNiqsListFromBuffer (
  ZkError ,           /* Returns the error if one occured */
  void * ,            /* The buffer */
  size_t ,            /* Size of the buffer */
  ZkList * );         /* Creates and returns a list of FrNiqsEntity structs */

DLLExport int frNiqsGetKeyQryAddr (
  ZkError ,            /* Return the error if one occured */
  ZkipClient ,         /* The NIQS client structure */
  struct sockaddr * ); /* Points to a sockaddr which will hold the key server */


/*
 * An NIQS client should never have to call this function.
 */

DLLExport int frNiqsListToBuffer (
  ZkError ,           /* Returns the error if one occured */
  ZkList  ,           /* List of FrNiqsEntity structures to convert */
  void ** ,           /* Allocates and returns the buffer */
  size_t * );         /* Returns the size of the buffer */

/*
 *
 */

DLLExport int frNiqsAvailable (
  ZkError ,           /* Returns the error if one occured */
  ZkipClient ,        /* The zkip client structure connected to the niqs */
  FrEntIdType *  ,    /* List of entity types to check for at server */
  size_t  ,           /* Number of entity types in the above array */
  void ** ,           /* Returns a signed buffer containing entity IDs */
  size_t * );         /* Size of the buffer */

DLLExport int frNiqsGetAvailable (
  ZkError ,           /* Returns the error if one occured */
  ZkipClient ,        /* The zkip client structure connected to the niqs */
  FrKeyQryHost kh ,
  FrKeyQry kq ,
  FrEntIdType * ,     /* (array) Type of entity for which we want a list */
  size_t ,            /* Number of entities in the array */
  uint32_t ** ,       /* Returns an array of entities (dont forget to free it)*/
  size_t * );         /* Returns the size of the array */

DLLExport int frNiqsClientInfoFree (
  ZkError err ,
  FrNiqsClientInfo * clientinfo ); /* clientinfo where MOTD will be free()d */




/*
 * THIS IS A HACK.  IT MUST BE REMOVED
 */

DLLExport int frNiqsSetListProtocol (
  ZkError ,
  int  );       /* 1 or 2 */




DLLExport int frNiqsCompress( 
  ZkError err,
  uint16_t encoding, 
  const unsigned char *raw, 
  size_t raw_sz,
  unsigned char **compressed,
  size_t *compressed_sz );

DLLExport int frNiqsUncompress( 
  ZkError err, 
  const unsigned char *compressed_buf, 
  unsigned char **raw,
  size_t *raw_sz );

/*
**------------------------------------------------------------------------------
**------------------------------------------------------------------------------
**
** PATCH PATCH PATCH
** 
** from here to end of file
**
** this function has bee nre-introduce to allow the NEW client to talk with the
** OLD NIQS in case something goes wrong in the migration and a roll back in
** needed.
**
** LBe 2000.11.28
**------------------------------------------------------------------------------
**------------------------------------------------------------------------------
*/

DLLExport int 
OLD_frNiqsGetKeyQryAddr (
  ZkError err,     
  ZkipClient client,
  struct sockaddr * addr);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRNIQ_H_ */
 
