
/*
 * $Id: frNiqsAccessCtrlList.c,v 1.2.32.2 2000/10/11 13:18:56 phil Exp $
 */

static const char * module_id_ = "$Id: frNiqsAccessCtrlList.c,v 1.2.32.2 2000/10/11 13:18:56 phil Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"
#include "frNiqs.h"
#include "zkConfig.h"
#include "frSec.h"


DLLExport int
frNiqsGetAccessCtrlList (
  ZkError err ,
  ZkipClient client ,
  FrKeyQryHost kh ,
  FrKeyQry kq ,
  ZkConfig accessctrllist )
{
  static const char * func = "frNiqsGetAccessCtrlList";
  int return_code = 0;
  FrEntIdExp signer;
  FrCryptPubKey sigkey = NULL;
  unsigned int kv = 0; /* Key version */
  void * buf = NULL;        /* Buffer from the NIQS */
  size_t sz = 0;
  int valid = 0;
#ifndef WIN32
#warning Dont hard code the timeout and retry values for the key query
#else
#pragma message(Reminder "Dont hard code the timeout and retry values for the key query")
#endif
  int retries = 5;
  int timeout = 3;


  if (client && kq && accessctrllist) {
    if (frNiqsAccessCtrlList(err,client,&buf,&sz) &&
        frSecSigGetEntity(err,buf,sz,&signer) &&
        frSecSigGetVersion(err,buf,sz,&kv) &&
        (signer.type == FRENT_TYPE_NIQS) &&
        frKeyQryFetchPubKey(err,kq,kv,FRCRYPT_KEYTYPE_SIGNING,
                            signer,kh,retries,timeout,&sigkey) &&
        frSecSigVerify(err,sigkey,buf,sz,&valid) &&
        (valid) &&
        frSecSigRemoveInSitu(err,buf,&sz) &&
        zkConfigLoadFromBuffer(err,accessctrllist,buf,sz)) {
      return_code = 1;
    }

    /* 
     * cleanup
     */

    if (buf) {
      free(buf);
      buf = NULL;
    }

  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}
  

