
/*
 * $Id: frNiqsClientInfo.c,v 1.18.8.5 2000/12/14 17:53:03 jean Exp $
 */

static const char * module_id_ = "$Id: frNiqsClientInfo.c,v 1.18.8.5 2000/12/14 17:53:03 jean Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */



#include "zks.h"
#include "zkIntTypes.h"
#include "frNetworkDefaults.h"
#include <stdio.h>
#include <stdlib.h>

#ifndef WIN32
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#else
#include <winsock2.h>
#endif

#include <string.h>
#include "zkError.h"
#include "frEnt.h"
#include "ZKIP.h"
#include "zkipClient.h"
#include "zkList.h"
#include "frNiqs.h"
#include "zkConfig.h"
#include "frCrypt.h"
#include "frSec.h"
#include "zkDateTime.h"
#include <zkBase64.h>
#include "frNidbObj.h"
#include "frDBOTypes.h"


/*
 *
 */

DLLExport int
frNiqsClientInfoFree (
  ZkError err ,
  FrNiqsClientInfo * clientinfo )
{
  static const char * func = "frNiqsClientInfoFree";
  int ret = 0;

  if (!clientinfo) {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,"Clientinfo pointer is NULL");
  } else {
    if (clientinfo->motd) {
      free(clientinfo->motd);
    }
    memset(clientinfo,0,sizeof(FrNiqsClientInfo)); 
    ret = 1;
  }

  return ret;
}



/*
 *
 */

DLLExport int
frNiqsClientInfoLoadFromBuffer (
  ZkError err ,
  FrNiqsClientInfo * clientinfo ,
  const char * clientid ,
  void * buf ,
  size_t sz )
{
  static const char * func = "frNiqsClientInfoLoadFromBuffer";
  int return_code = 0;
  char * sval = NULL;
  uint8_t bool = 0;  /* For reading boolean values from the client info */
  ZkConfig c = NULL;
  char *motd64_str = NULL;

  (void) module_id_; /* Get rid of compile time warning */


  if (clientinfo && buf && clientid) {
    memset(clientinfo,0,sizeof(struct T_FrNiqsClientInfo));

    /*
     * There may not be an MOTD so make sure this is all nice and clean
     */

    clientinfo->motd_sz = 0;
    clientinfo->motd_urg = 0;
    clientinfo->motd = NULL;

    if (zkConfigNew(err,&c) &&
        zkConfigLoadFromBuffer(err,c,buf,sz)) {

      /*
       * Set the client id string
       */

      snprintf(clientinfo->client_id,CI_CI_SZ,"%s",clientid);

      /*
       * Load the expiry date time struct and the number of seconds in
       * which the client will expire.  If an expiry date-time is specified,
       * there MUST be a matching expire-in parameter.  If the Date-Time is
       * not specified, the expire-in parameter is ignored.  
       */
    
      /*
       * If the expire_in is not defined, we dont want to default this
       * value to 0 (that would indicate that the client is expired) 
       * instead, I'm hardwiring it for 10 years so as not to trigger any
       * automatic expiry warnings in the client.
       */

      clientinfo->expire_in = 3600 * 24 * 365 * 10;  
      if (zkConfigGet(err,c,clientid,NIQS_CLIENT_INFO_EXPIRY,
                      (const char **) &sval)) {
        if (!zkDateTimeParse(err,&(clientinfo->expiry),sval)) {
          goto cleanup;
        }
        if (!zkConfigGet(err,c,clientid,NIQS_CLIENT_INFO_EXPIRE_IN,
                         (const char **) &sval)) {
          zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKEINCOMPLETE,
                     "Missing "NIQS_CLIENT_INFO_EXPIRE_IN
                     " parameter in clientinfo");
        } else if (sval) {
          clientinfo->expire_in = (time_t) atoi(sval);
        }
      }

      /*
       * Get the current UTC time.  This must ABSOLUTELY be present in the
       * clientinfo structure, otherwise, the client cannot synchronise its
       * time with the network's and it will not be able to create routes. 
       */

      if (zkConfigGet(err,c,clientid,NIQS_CLIENT_INFO_CURRENT_UTC,
                       (const char **) &sval)) {
        clientinfo->current_UTC = (time_t) strtoul(sval,NULL,10);
        

        if (clientinfo->current_UTC == (time_t) ULONG_MAX) {
          zkErrorSet_va(err,func,ZKERR,ZKMNIQS,ZKEDATABAD,
                        "Error converting current UTC time (%s)",sval);
          goto cleanup;

        /*
         * This number was the UTC time offset when I wrote the code.  Wont
         * catch all problems.
         */

        } else if (clientinfo->current_UTC < 926966675) {
          zkErrorSet_va(err,func,ZKERR,ZKMNIQS,ZKEDATABAD,
                        "UTC time offset %ld is in the past!!!",
                        clientinfo->current_UTC);
          goto cleanup;
        }
      } else {
        zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKEINCOMPLETE,
                   "Missing "NIQS_CLIENT_INFO_CURRENT_UTC
                   " parameter in clientinfo");
        goto cleanup;
      }
   
      /*
       * Load a serial number url 
       */

      if (zkConfigGet(err,c,clientid,NIQS_CLIENT_INFO_SN_URL,
                      (const char **) &sval)) {
        snprintf(clientinfo->sn_url,CI_SNURL_SZ,"%s",sval);
      }

      /*
       * Load an upgrade URL if one is available
       */

      if (zkConfigGet(err,c,clientid,NIQS_CLIENT_INFO_UPGRADE_URL,
                      (const char **) &sval)) {
        snprintf(clientinfo->upgrade_url,CI_SNURL_SZ,"%s",sval);
      }

      /*
       * Get the network domain name
       */

      if (zkConfigGet(err,c,clientid,NIQS_CLIENT_INFO_DOMAIN,
                      (const char **) &sval)) {
        snprintf(clientinfo->domain,CI_DOMAIN_SZ,"%s",sval);
      } else {
        zkErrorSet(err,func,ZKWARN,ZKMNIQS,ZKERROR,
                   "WARNING:  Setting domain to "FR_DFLT_FREEDOM_DOMAIN);    
        snprintf(clientinfo->domain,CI_DOMAIN_SZ,"%s",
                 FR_DFLT_FREEDOM_DOMAIN);
      }

      /*
       * Get the NIQS update frequency. If this is not specified, the
       * default is 1800 seconds (30 minutes).   I'm hardwiring this so that you cannot
       * specify an interval less than 15 minutes (900 seconds)
       *
       * BUG 5030
       * no lower limit anymore; only logging a warning if < 900
       * LBe 2000.08.09
       */
      clientinfo->niqs_update = 1800;
      if (zkConfigGet(err,c,clientid,NIQS_CLIENT_INFO_NIQS_UPDATE,
                      (const char **) &sval)) {
        clientinfo->niqs_update = (int) atoi(sval);
      } 
      if (clientinfo->niqs_update < 900) {
        zkErrorSet(err,func,ZKWARN,ZKMNIQS,ZKERROR,
                   "You should specify an "NIQS_CLIENT_INFO_NIQS_UPDATE
                   " of at least 900 seconds");
      }

      /*
       * Is the next upgrade available?
       */

      if (zkConfigGetParameter(err,c,TYPE_BOOLEAN,clientid,
                               NIQS_CLIENT_INFO_UPGRADE_READY, 
                               (void *) &bool)) {
        if (bool) {
          clientinfo->upgrade_ready = 1;
        } else {
          clientinfo->upgrade_ready = 0;
        } 
      }

      /*
       * Some optional bits, the MOTD and its urgency code
       */

      if (zkConfigGet(err,c,clientid,NIQS_CLIENT_INFO_MOTD,
                      (const char **) &sval)) {
        size_t dsz = 0;
	const char *urg_str = NULL;

	motd64_str = NULL;

	if (zkConfigGet(err,c,clientid,NIQS_CLIENT_INFO_MOTD_URG,&urg_str)) {
	  size_t motd64_sz = 0;
	  const char *motd64bis_str = NULL;

	  motd64bis_str = strstr( urg_str, " motd2 = " );
	  if( motd64bis_str ) {
	    motd64bis_str += strlen( " motd2 = " );
	    motd64_sz = strlen(sval) + strlen(motd64bis_str);
	    motd64_str = (char*)malloc( motd64_sz + 1 );
	    if( motd64_str == NULL ) {
	      zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENOMEM,
			 "Failed to allocate memory to decode MOTD");
	      goto cleanup;
	    }
	    memset( motd64_str, 0, motd64_sz );
	    snprintf( motd64_str, motd64_sz, "%s%s", sval, motd64bis_str );
	    clientinfo->motd_sz = motd64_sz * 3 / 4 + 6;
	  } else {
	    motd64_str = NULL;
	  }
	}

	if( !motd64_str ) {
	  motd64_str = strdup(sval);
	  if( !motd64_str ) {
	  zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENOMEM,
		     "Failed to allocate memory to decode MOTD");
	  goto cleanup;	    
	  }
	  clientinfo->motd_sz = strlen(sval) * 3 / 4 + 6;
	}

	clientinfo->motd = (char *) malloc(clientinfo->motd_sz);
	if (clientinfo->motd == NULL) {
	  zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENOMEM,
		     "Failed to allocate memory to decode MOTD");
	  goto cleanup;
	}
        if (!zkConvertFromBase64(err,motd64_str,clientinfo->motd,&dsz)) {
          goto cleanup; 
        }
        if (dsz >= clientinfo->motd_sz) {
          zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKERROR,
                     "Size mismatch when decoding MOTD");
          goto cleanup;
        } 
        clientinfo->motd_sz = dsz;
        clientinfo->motd[clientinfo->motd_sz] = (char) 0;
      }

      /*
       * The MOTD urgency code is for future use and should always be 1
       * for the moment.  If there is no MOTD then the urgency code is
       * set to 0
       */

      if (zkConfigGet(err,c,clientid,NIQS_CLIENT_INFO_MOTD_URG,
                      (const char **) &sval)) {
        clientinfo->motd_urg = (int) atoi(sval);
      } else if (clientinfo->motd_sz > 0) { 
        clientinfo->motd_urg = 1;
      } else {
        clientinfo->motd_urg = 0;
      }
        

      return_code =  1;
    }
   
    /*
     * Cleanup
     */

cleanup:
    if (c) {
      zkConfigFree(err,&c);
    }

    if( motd64_str ) {
      free(motd64_str);
      motd64_str = NULL;
    }

    if (return_code == 0) {
      if (clientinfo->motd) {
        free(clientinfo->motd);
        clientinfo->motd = NULL;
      }
      clientinfo->motd_sz = 0;
      clientinfo->motd_urg = 0;
    }
    
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}


DLLExport int
frNiqsGetClientInfo (
  ZkError err ,
  ZkipClient client ,
  FrKeyQryHost kh ,
  FrKeyQry kq ,
  const char * clientid ,
  FrNiqsClientInfo * clientinfo )
{
  static const char * func = "frNiqsGetClientInfo";
  int return_code = 0;
  FrEntIdExp signer;
  FrCryptPubKey sigkey = NULL;
  unsigned int kv = 0; /* Key version */
  void * buf = NULL;        /* Buffer from the NIQS */
  size_t sz = 0;
  int valid = 0;
#ifndef WIN32
#warning Dont hard code the timeout and retry values for the key query
#else
#pragma message(Reminder "Dont hard code the timeout and retry values for the key query")
#endif
  int retries = 5;
  int timeout = 3;


  if (client && kq && clientid && clientinfo) {
    if (frNiqsClientInfo(err,client,clientid,&buf,&sz) &&
        frSecSigGetEntity(err,buf,sz,&signer) &&
        frSecSigGetVersion(err,buf,sz,&kv) &&
        (signer.type == FRENT_TYPE_NIQS) &&
        frKeyQryFetchPubKey(err,kq,kv,FRCRYPT_KEYTYPE_SIGNING,
                            signer,kh,retries,timeout,&sigkey) &&
        frSecSigVerify(err,sigkey,buf,sz,&valid) &&
        (valid) &&
        frSecSigRemoveInSitu(err,buf,&sz) &&
        frNiqsClientInfoLoadFromBuffer(err,clientinfo,clientid,buf,sz)) {
      return_code = 1;
    }

    /* 
     * cleanup
     */

    if (buf) {
      free(buf);
    }

  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}
  

