/*
 * $Id: frNiqsCompress.c,v 1.3.8.5 2000/12/14 17:53:03 jean Exp $
 */
static const char * module_id_ = "$Id: frNiqsCompress.c,v 1.3.8.5 2000/12/14 17:53:03 jean Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */



#include "zks.h"
#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* WIN32 */
#include <string.h>
#include <zkError.h>
#include "zkAlloc.h"
#include <ZKIP.h>
#include <bzlib.h>
#include "frNiqs.h"

#define ZK_NIQS_COMPRESS_BLOCK_SIZE 1
#define ZK_NIQS_COMPRESS_WORK_FACTOR 30

#define ZK_NIQS_COMPRESS_HDR_VERSION 1

typedef struct FrNiqsCompressHdr_T {
    uint16_t frNiqsCompressHdr_version;
    uint16_t encoding;
    uint16_t encoding_major_version;
    uint16_t encoding_minor_version;
    uint32_t uncompressed_length;
    uint32_t compressed_length;
} FrNiqsCompressHdr;

#define FRNIQSCOMPRESS_HDR "frniqscompresshdr_version"
#define FRNIQSCOMPRESS_ENC "encoding"
#define FRNIQSCOMPRESS_ENC_MAJ "encoding_major_version"
#define FRNIQSCOMPRESS_ENC_MIN "encoding_minor_version"
#define FRNIQSCOMPRESS_UNC_LEN "uncompressed_length"
#define FRNIQSCOMPRESS_CMP_LEN "compressed_length"

static struct PackRec {
  const char *name;
  size_t size; /* in bytes */
} FrNiqsCompressHdrDef[] = {
    {FRNIQSCOMPRESS_HDR, 2},
    {FRNIQSCOMPRESS_ENC, 2},
    {FRNIQSCOMPRESS_ENC_MAJ, 2},
    {FRNIQSCOMPRESS_ENC_MIN, 2},
    {FRNIQSCOMPRESS_UNC_LEN, 4},
    {FRNIQSCOMPRESS_CMP_LEN, 4}
};

#define NUM_RECS_FRNIQSCOMPRESSHDR \
    sizeof(FrNiqsCompressHdrDef) / sizeof(struct PackRec)

static int packFrNiqsCompressHdr( ZkError err, 
				  const FrNiqsCompressHdr *hdr,
				  unsigned char **packed_hdr )
{
  const char * funcname = "packFrNiqsCompressHdr";
  int return_code = 0;
  unsigned short int shrt_buf = 0;
  unsigned long int long_buf = 0;
  unsigned char bytes[4];
  unsigned char *out = NULL;

  if( !hdr || !packed_hdr || !*packed_hdr ) {
    zkErrorSet( err,funcname,ZKERR,ZKMNIQS,ZKENULLPTR,"Null parameter passed");
    goto finish;
  }

  out = *packed_hdr;

  shrt_buf = htons( hdr->frNiqsCompressHdr_version );
  bytes[0] = (unsigned char)(shrt_buf >> 8);
  bytes[1] = (unsigned char)(shrt_buf >> 0);
  memcpy( out, bytes, 2 );
  out += 2;

  shrt_buf = htons( hdr->encoding );
  bytes[0] = (unsigned char)(shrt_buf >> 8);
  bytes[1] = (unsigned char)(shrt_buf >> 0);
  memcpy( out, bytes, 2 );
  out += 2;

  shrt_buf = htons( hdr->encoding_major_version );
  bytes[0] = (unsigned char)(shrt_buf >> 8);
  bytes[1] = (unsigned char)(shrt_buf >> 0);
  memcpy( out, bytes, 2 );
  out += 2;

  shrt_buf = htons( hdr->encoding_minor_version );
  bytes[0] = (unsigned char)(shrt_buf >> 8);
  bytes[1] = (unsigned char)(shrt_buf >> 0);
  memcpy( out, bytes, 2 );
  out += 2;

  long_buf = htonl( hdr->uncompressed_length );
  bytes[0] = (unsigned char)(long_buf >> 24);
  bytes[1] = (unsigned char)(long_buf >> 16);
  bytes[2] = (unsigned char)(long_buf >>  8);
  bytes[3] = (unsigned char)(long_buf >>  0);
  memcpy( out, bytes, 4 );
  out += 4;

  long_buf = htonl( hdr->compressed_length );
  bytes[0] = (unsigned char)(long_buf >> 24);
  bytes[1] = (unsigned char)(long_buf >> 16);
  bytes[2] = (unsigned char)(long_buf >>  8);
  bytes[3] = (unsigned char)(long_buf >>  0);
  memcpy( out, bytes, 4 );
  out += 4;

  return_code = 1;

 finish:
  
  return return_code;
}

static int unpackFrNiqsCompressHdr( ZkError err,
				    const unsigned char *packed_hdr,
				    FrNiqsCompressHdr *hdr )
{
  const char * funcname = "unpackFrNiqsCompressHdr";
  int return_code = 0;
  unsigned char bytes[4];
  const unsigned char *in = NULL;
  unsigned short int shrt_buf = 0;
  unsigned long int long_buf = 0;
  if( !packed_hdr || !hdr ) {
    zkErrorSet( err,funcname,ZKERR,ZKMNIQS,ZKENULLPTR,"Null parameter passed");
    goto finish;
  }

  in = packed_hdr;

  memcpy( bytes, in, 2 );
  shrt_buf = (unsigned short int)((bytes[0] << 8) +
                                  (bytes[1] << 0)  );
  hdr->frNiqsCompressHdr_version = ntohs( shrt_buf );
  in += 2;

  memcpy( bytes, in, 2 );
  shrt_buf = (unsigned short int)((bytes[0] << 8) +
                                  (bytes[1] << 0)  );
  hdr->encoding = ntohs( shrt_buf );
  in += 2;

  memcpy( bytes, in, 2 );
  shrt_buf = (unsigned short int)((bytes[0] << 8) +
                                  (bytes[1] << 0)  );
  hdr->encoding_major_version = ntohs( shrt_buf );
  in += 2;

  memcpy( bytes, in, 2 );
  shrt_buf = (unsigned short int)((bytes[0] << 8) +
                                  (bytes[1] << 0)  );
  hdr->encoding_minor_version = ntohs( shrt_buf );
  in += 2;

  memcpy( bytes, in, 4 );
  long_buf = (unsigned long int)((bytes[0] << 24) +
                                 (bytes[1] << 16) +
                                 (bytes[2] <<  8) +
                                 (bytes[3] <<  0)  );
  hdr->uncompressed_length = ntohl( long_buf );
  in += 4;

  memcpy( bytes, in, 4 );
  long_buf = (unsigned long int)((bytes[0] << 24) +
                                 (bytes[1] << 16) +
                                 (bytes[2] <<  8) +
                                 (bytes[3] <<  0)  );
  hdr->compressed_length = ntohl( long_buf );
  in += 4;

  return_code = 1;

 finish:
  
  return return_code;
}


static size_t GetSizeofPackedFrNiqsCompressHdr( void )
{
  unsigned short int i = 0;
  size_t size = 0;

  for( i=0; i<NUM_RECS_FRNIQSCOMPRESSHDR; i++ ) {
    size += FrNiqsCompressHdrDef[i].size;
  }

  return size;
}

static int CreateFrNiqsCompressBuffer( ZkError err, size_t compressed_sz, 
				       unsigned char **comp_buf, size_t *comp_buf_sz ) {
  const char * funcname = "CreateFrNiqsCompressBuffer";
  int return_code = 0;
  unsigned char *buf = NULL;
  size_t hdr_sz = 0;
  size_t buf_sz = 0;

  if( !comp_buf ) {
    zkErrorSet( err,funcname,ZKERR,ZKMNIQS,ZKENULLPTR,"Null parameter passed");
    goto finish;
  }

  hdr_sz = GetSizeofPackedFrNiqsCompressHdr();
  buf_sz = hdr_sz + compressed_sz;
  buf = (unsigned char *)malloc( buf_sz );
  if( !buf ) {
      zkErrorSet_va(err,funcname,ZKERR,ZKMNIQS,ZKERROR,
		    "Unable to malloc %d bytes for compression", buf_sz );
      goto finish;
  }

  *comp_buf = buf;
  if( comp_buf_sz ) *comp_buf_sz = buf_sz;

  return_code = 1;

 finish:
  if( !return_code ) {
    if( !buf ) free(buf);
    if( !comp_buf ) *comp_buf = NULL;
    if( !comp_buf_sz ) *comp_buf_sz = 0;
  }
  return return_code;
}

static int ReallocFrNiqsCompressBuffer( ZkError err, size_t compressed_sz, 
					unsigned char **comp_buf, size_t *comp_buf_sz ){
  const char * funcname = "ReallocFrNiqsCompressBuffer";
  int return_code = 0;
  size_t hdr_sz = 0;
  size_t buf_sz = 0;

  if( !comp_buf || !*comp_buf || !comp_buf_sz ) {
    zkErrorSet( err,funcname,ZKERR,ZKMNIQS,ZKENULLPTR,"Null parameter passed");
    goto finish;
  }

  hdr_sz = GetSizeofPackedFrNiqsCompressHdr();
  buf_sz = hdr_sz + compressed_sz;
  if( !zkMemRealloc( err, (void**)comp_buf, buf_sz ) ){
    zkErrorSet(err, funcname, ZKERR, ZKMNIQS, ZKERROR,
	"Error reallocating the compressed buffer" );
    goto finish;
  }
    
  *comp_buf_sz = buf_sz;
  return_code = 1;

 finish:
    
  return return_code;
}

static int GetCompressedFromPackedFrNiqsCompress( ZkError err, 
						  const unsigned char *packed,
						  const unsigned char **compressed ) {
  const char * funcname = "GetCompressedFromFrNiqsCompressBuffer";
  int return_code = 0;
  size_t hdr_sz;

  if( !packed || !compressed ) {
    zkErrorSet( err,funcname,ZKERR,ZKMNIQS,ZKENULLPTR,"Null parameter passed");
    goto finish;
  }

  hdr_sz = GetSizeofPackedFrNiqsCompressHdr();

  *compressed = packed + hdr_sz ;

  return_code = 1;

 finish:

  return return_code;
}

/* This function compresses a Niqs packed buffer.
   Compressed will always be allocated, caller should free it when done.
*/
DLLExport
int frNiqsCompress( ZkError err, uint16_t encoding, 
		    const unsigned char *raw_buf, size_t raw_sz,
		    unsigned char **compressed, size_t *compressed_sz )
{
  const char * funcname = "frNiqsCompress";
  int return_code = 0;
  unsigned int bzip2_sz = 0;
  FrNiqsCompressHdr hdr;
  unsigned char *buf_start = NULL;
  size_t buf_sz = 0;
  unsigned char *packed_hdr = NULL;
  unsigned char *comp_buf = NULL;

  (void) module_id_;

  if( !raw_buf || !compressed || !compressed_sz ) {
    zkErrorSet( err,funcname,ZKERR,ZKMNIQS,ZKENULLPTR,"Null parameter passed");
    goto finish;
  }

  if( encoding != ZKIP_ENCODING_BZIP2 ) {
    /* the only supported encoding is BZIP2 */
    /* fake a null encoding */
    if( !CreateFrNiqsCompressBuffer( err, raw_sz, &buf_start, &buf_sz) ) {
      zkErrorSet_va(err,funcname,ZKERR,ZKMNIQS,ZKERROR,
		    "Unable to malloc buffer for compression" );
      goto finish;
    }

    packed_hdr = buf_start;
    if( !GetCompressedFromPackedFrNiqsCompress(err, buf_start, (const unsigned char **)&comp_buf ) ) {
      zkErrorSet_va(err,funcname,ZKERR,ZKMNIQS,ZKERROR,
		    "Unable to get compression buffer" );
      goto finish;
    }

    hdr.frNiqsCompressHdr_version = ZK_NIQS_COMPRESS_HDR_VERSION;
    hdr.encoding = ZKIP_ENCODING_NONE;
    hdr.encoding_major_version = 0;
    hdr.encoding_minor_version = 0;
    hdr.uncompressed_length = raw_sz;
    hdr.compressed_length = raw_sz;
    if(!packFrNiqsCompressHdr( err, &hdr, &packed_hdr ) ) {
      zkErrorSet_va(err,funcname,ZKERR,ZKMNIQS,ZKERROR,
		    "Unable to pack FrNiqs compressed hdr" );
      goto finish;

    }
    
    memcpy( comp_buf, raw_buf, raw_sz );

    *compressed = buf_start;
    *compressed_sz = buf_sz;

    return_code = 1;

    goto finish;
  }
  
  /* figure maximal size, this is taken from bzip2's doc */
  bzip2_sz = (unsigned int)(1.01 * (double)raw_sz) + 600 + 0.5;
  if( !CreateFrNiqsCompressBuffer( err, bzip2_sz, &buf_start, &buf_sz) ) {
      zkErrorSet( err, funcname, ZKERR, ZKMNIQS, ZKERROR,
		  "Unable to malloc buffer for compression" );
      goto finish;
  }

  packed_hdr = buf_start;
  if( !GetCompressedFromPackedFrNiqsCompress(err, buf_start, (const unsigned char **)&comp_buf ) ) {
      zkErrorSet_va(err,funcname,ZKERR,ZKMNIQS,ZKERROR,
		    "Unable to get compression buffer" );
      goto finish;
  }

  hdr.frNiqsCompressHdr_version = ZK_NIQS_COMPRESS_HDR_VERSION;
  hdr.encoding = ZKIP_ENCODING_BZIP2;
  hdr.encoding_major_version = 2;
  hdr.encoding_minor_version = 1;
  hdr.uncompressed_length = raw_sz;

  if( BZ2_bzBuffToBuffCompress( comp_buf, &bzip2_sz,
				(char *)raw_buf, raw_sz,
				ZK_NIQS_COMPRESS_BLOCK_SIZE,
				0,
				ZK_NIQS_COMPRESS_WORK_FACTOR ) != BZ_OK ) {
    zkErrorSet(err, funcname, ZKERR, ZKMNIQS, ZKERROR,
	       "Error compressing buffer" );
    goto finish;
  }
  
  /* bzip2_size is now the real compressed length */
  hdr.compressed_length = bzip2_sz;

  if(!packFrNiqsCompressHdr( err, &hdr, &packed_hdr ) ) {
      zkErrorSet_va(err,funcname,ZKERR,ZKMNIQS,ZKERROR,
		    "Unable to pack FrNiqs compressed hdr" );
      goto finish;
  }

  /* shorten buffen to real used space */
  if( !ReallocFrNiqsCompressBuffer( err, bzip2_sz, &buf_start, &buf_sz ) ) {
    zkErrorSet(err, funcname, ZKERR, ZKMNIQS, ZKERROR,
	       "Error reallocating the compressed buffer");
    goto finish;
  }
 
  *compressed = (unsigned char*)buf_start;
  *compressed_sz = buf_sz;

  return_code = 1;

 finish:
  if( !return_code ) {
    if( buf_start ) free( buf_start );
    if( compressed ) *compressed = NULL;
    if( compressed_sz ) *compressed_sz = 0;
  }

  return return_code;
}

DLLExport
int frNiqsUncompress( ZkError err, const unsigned char *compressed_buf,
		      unsigned char **raw_buf, size_t *raw_sz  )
{
  const char * funcname = "frNiqsUncompress";
  int return_code = 0;
  FrNiqsCompressHdr hdr;
  const unsigned char *comp_buf = NULL;
  unsigned char *uncompressed = NULL;

  if( !compressed_buf || !raw_buf || !raw_sz ) {
    zkErrorSet( err,funcname,ZKERR,ZKMNIQS,ZKENULLPTR,"Null parameter passed");
    goto finish;
  }

  if( !unpackFrNiqsCompressHdr( err, compressed_buf, &hdr ) ){
    zkErrorSet( err, funcname, ZKERR, ZKMNIQS, ZKERROR,
		"Unable to unpack compressed data" );
    goto finish;
  }

  if( hdr.frNiqsCompressHdr_version != ZK_NIQS_COMPRESS_HDR_VERSION ) {
    zkErrorSet( err, funcname, ZKERR, ZKMNIQS, ZKERROR,
		"Illegal header version" );
    goto finish;
  }

  if( (hdr.encoding != ZKIP_ENCODING_BZIP2)
      && (hdr.encoding != ZKIP_ENCODING_NONE) ) {
    zkErrorSet( err, funcname, ZKERR,ZKMNIQS,ZKERROR,
		"Unsupported encoding" );
    goto finish;
  }

  if( !GetCompressedFromPackedFrNiqsCompress(err, compressed_buf, &comp_buf) ){
    zkErrorSet( err, funcname, ZKERR, ZKMNIQS, ZKERROR,
		"Unable to get compressed buffer" );
    goto finish;
  }

  uncompressed = (unsigned char*)malloc(hdr.uncompressed_length);
  if( !uncompressed ) {
    zkErrorSet( err, funcname, ZKERR,ZKMNIQS,ZKERROR,
		"Unable to allocate memory for uncompress buffer");
    goto finish;
  }

  if( hdr.encoding == ZKIP_ENCODING_NONE ) {
    /* fake null encoding */
    memcpy( uncompressed, comp_buf, hdr.compressed_length );
    
    *raw_sz = hdr.uncompressed_length;
    *raw_buf = uncompressed;

    return_code = 1;
    goto finish;
  }

  /* encoding is BZIP2 */
  if( (hdr.encoding_major_version > 2)
      || ( (hdr.encoding_major_version == 2) 
	   && (hdr.encoding_minor_version > 1) ) ) {
    zkErrorSet_va( err, funcname, ZKERR,ZKMNIQS,ZKERROR,
		   "Unsupported encoding BZIP2 version %d.%d",
		   hdr.encoding_major_version,
		   hdr.encoding_minor_version);
    goto finish;
  }

  if( BZ2_bzBuffToBuffDecompress( uncompressed, 
				   &hdr.uncompressed_length,
				   (char *)comp_buf,
				   hdr.compressed_length,
				   0, 0) != BZ_OK ) {
    zkErrorSet( err, funcname, ZKERR,ZKMNIQS,ZKERROR,
		"Error decompressing buffer" );
    goto finish;
  }

  *raw_sz = hdr.uncompressed_length;
  *raw_buf = uncompressed;

  return_code = 1;

 finish:
  if( !return_code ) {
      if( uncompressed ) free(uncompressed);
      if( raw_buf ) *raw_buf = NULL;
      if( raw_sz ) *raw_sz = 0;
  }

  return return_code;
}
