/*
 * $Id: frNiqsList.c,v 1.6.32.6 2000/12/14 17:53:03 jean Exp $
 */

static const char * module_id_ = "$Id: frNiqsList.c,v 1.6.32.6 2000/12/14 17:53:03 jean Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */



#include <zks.h>
#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* WIN32 */
#include <string.h>
#include <zkError.h>
#include <frEnt.h>
#include <ZKIP.h>
#include <zkipClient.h>
#include <zkList.h>
#include <frNiqs.h>


/*
 *
 */

DLLExport int
frNiqsListFromBuffer (
  ZkError err ,
  void * buf ,
  size_t sz ,
  ZkList * list )
{
  static const char * func = "frNiqsListToList";
  FrNiqsEntity * list_ent = NULL;
  int return_code = 0;
  unsigned char * b = (unsigned char *) buf;
  int base = 0;
  char ent_type[20]; 
  int update = 0;

  (void) module_id_; /* Gets rid of compile time warning */

  if (buf && list) {
    if (zkListNew(err,list) &&
        zkListSetDataFreeFunction(err,*list,zkListDataFree,NULL)) {
      return_code = 1;
     
      while (base < (int) sz) {
  
        /* 
         * Allocate a new entry for the list 
         */

        if ((list_ent = (FrNiqsEntity *) malloc(sizeof(FrNiqsEntity))) == NULL){
          zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENOMEM,"malloc");
          break;
        }
        memset(list_ent,0,sizeof(FrNiqsEntity)); 

        /*
         * Read data and convert where nescessary to fill in the new list
         * entry and add it to the list.
         */

        if (sscanf(&(b[base]),"%u %10s %63s %d",        /* audit: ok */
                   &(list_ent->id),ent_type,list_ent->entity.name,
                   &update) != 4) {
          zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKEDATABAD,
                     "Missing parameters in list entry");
          return_code = 0;
          break;
        } else if ((list_ent->entity.type = 
                    frEntTypeFromStr(ent_type)) == FRENT_TYPE_UNUSED_) {
          zkErrorSet_va(err,func,ZKERR,ZKMNIQS,ZKEDATABAD,
                        "Unrecognized entity type %s",ent_type);
          return_code = 0;
          break;
        } else if (!zkListInsert(err,*list,list_ent,LIST_HEAD)) {
          return_code = 0;
          break;
        } else {
          /* bug: is there a conversion/truncation problem here
             (signed -> unsigned) - luc */
          list_ent->update = (time_t) update;
        } 
     
        /*
         * Move the index to the next line in the buffer
         */

        while (base < (int) sz) {
          base++; 
          if (b[base - 1] == (char) '\012') {
            break;
          } 
        }
      }
    }

    if (return_code == 0) {
      if (*list) {
        zkListFree(err,list);
      }
      *list = NULL;
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 * 
 */

DLLExport int
frNiqsListToBuffer (
  ZkError err ,
  ZkList list ,
  void ** buf ,
  size_t * sz )
{
  static const char * func = "frNiqsListToBuffer";
  FrNiqsEntity * list_ent = NULL;
  int return_code = 0;
  size_t list_size = 0;
  unsigned char * b = NULL;
  int base  = 0;
  int bytes = 0;
  const char * ent_type = NULL;
  

  if (list && buf && sz) {
    *buf = NULL;
    *sz = 0;
    zkListSize(err,list,&list_size);
    return_code = 1;

    /*
     * Upper bound on the size of an entity name, 10 bytes for the ID, 
     * 10 bytes for the last update time and 10 bytes for the entity type.
     * Add to that 10 bytes of whitespace (includes CRLF).  This is a bit
     * of overkill and some memory is going to be wasted. 
     */

    *sz = list_size * (FRENT_PORTABLE_SIZE + 40); 
    b = (unsigned char *) malloc(sizeof(char) * (*sz));
    if (b == NULL) {
      zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENOMEM,"malloc");
      return_code = 0;
    } else while (zkListGet(err,list,(void **) &list_ent,LIST_FORWARDS)) {
      if (list_ent == NULL) {
        zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKERROR,
                   "List contained NULL element");
        return_code = 0;
        break;
      } else if ((ent_type = frEntTypeGetStr(list_ent->entity.type)) == NULL) {
        zkErrorSet_va(err,func,ZKERR,ZKMNIQS,ZKERROR,
                      "Unrecognized entity type %d",
                      list_ent->entity.type);
        return_code = 0;
        break;
      } else if ((bytes = snprintf(&(b[base]),*sz - base,
                                  "%d %s %s %d\015\012",list_ent->id,
                                  ent_type,list_ent->entity.name,
                                  ( int )list_ent->update)) < 0) {
        zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENOMEM,
                   "Ran out of buffer space to store list");
        return_code = 0;
        break;
      } else {
        base += bytes;
      }
    }

    if (return_code == 0) {
      if (b) {
        free(b);
      } 
      b = NULL;
      *sz = 0;
    }
    *buf = (void *) b;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}
