/*
 * $Id: frNiqsToken.c,v 1.5.32.2 2000/10/11 13:18:56 phil Exp $
 */

static const char * module_id_ = "$Id: frNiqsToken.c,v 1.5.32.2 2000/10/11 13:18:56 phil Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* WIN32 */
#include <string.h>
#include <zkError.h>
#include <frNiqs.h>

static const struct T_FrNiqToken {
  FrNiqAttr attr; 
  const char * token;
} frNiqToken[] = {
  { FRNIQ_ATTR_TOPOLOGY ,    "topology"    },
  { FRNIQ_ATTR_DESCRIPTION , "description" },
  { FRNIQ_ATTR_STATUS ,      "status" }
};

#define FRNIQ_TOKENS (sizeof(frNiqToken)/sizeof(struct T_FrNiqToken))

DLLExport int
frNiqsAttributeToToken (
  ZkError err , 
  FrNiqAttr attr ,
  const char ** token )
{
  static const char * func = "frNiqAttributeToToken";
  int return_code = 0;
  uint16_t n = 0;  /* Loop index */

  (void) module_id_;

  if (token && attr) {
    for (n = 0; n < FRNIQ_TOKENS; n++) {
      if (attr == frNiqToken[n].attr) {
        return_code = 1;
        *token = frNiqToken[n].token;
        break;
      }
    }
    if (return_code != 1) {
      zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENOTFOUND,
                 "Requested attribute does not have a token");
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}

DLLExport int
frNiqsTokenToAttribute (
  ZkError err ,
  const char * token ,
  FrNiqAttr * attr )
{
  static const char * func = "frNiqTokenToAttribute";
  int return_code = 0;
  uint16_t n = 0;  /* Loop index */


  if (token && attr) {
    for (n = 0; n < FRNIQ_TOKENS; n++) {
      if (strcasecmp(token,frNiqToken[n].token) == 0) {
        return_code = 1;
        *attr = frNiqToken[n].attr;
        break;
      }
    }
    if (return_code != 1) {
      zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENOTFOUND,
                 "Token is not an NIQ attribute");
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}
  
