/*
 * $Id: frNymClient.h,v 1.10.8.2 2001/01/15 19:27:30 jocelynm Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRNYMCLIENT_H_
#define FRNYMCLIENT_H_

#ifndef _ZKERROR_H
#include <zkError.h>
#endif /* _ZKERROR_H */

#ifndef FR_SIG_H_
#include <frCrypt.h>
#endif /* FR_SIG_H_ */

#ifndef ZKLIST_H_
#include "zkList.h"
#endif /* ! ZKLIST_H_ */

#ifndef ZKIPCLIENT_H_
#include <zkipClient.h>
#endif /* ZKIPCLIENT_H_ */

#include <frMailAuth.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */
  
#define NSHOST_SZ 64
struct T_NymServer {
  char nymserver[NSHOST_SZ + 1]; /* Name of the remote nym server host */
  unsigned short port;           /* Port on which the nymserver is running */
  int timeout;                   /* Timeout for trying to communicate */
  FrCryptSig sigkey;             /* Nymserver's public signature key */
                                 /* This key is used to verify incoming nyms */
};

/*
 * The response codes are described in ZKIP.h.  The 3 digit code is relevant,
 * the message string is informational only.  You could display it to the
 * user if you want.
 */

#define NSRMESG_SZ 256
struct T_NymServerResponse {
  char message[NSRMESG_SZ + 1]; /* The message string */
  int code;  /* The 3 digit code.  If 0, then no response came from server */
};

typedef struct T_NymServerResponse NymServerResponse;
typedef struct T_NymServer NymServer;

/* 
 * If these command functions return succesfully, you dont need to bother with
 * the response structure.  If the function fails, you have to determine 
 * what you will tell the user by looking at both the response structure
 * and the ZkError since quite a number of things can go wrong.
 */ 

/* Obsolete- please use frRemoteNymGetConnected() corey */
DLLExport int frRemoteNymGet (
  ZkError ,                 /* Returns the error if any occured */
  FrRemoteNym * ,           /* Returns the nym if one was retreived */
  char * ,                  /* Name of the nym to lookup */
  NymServer * ,             /* Nym server structure (see above) */
  NymServerResponse * );    /* Returns the nym server response */

/* The ZkipClient must be disconnected and free'd be the caller */
DLLExport int frRemoteNymGetConnected (
  ZkError ,                 /* Returns the error if any occured */
  FrRemoteNym * ,           /* Returns the nym if one was retreived */
  char * ,                  /* Name of the nym to lookup */
  ZkipClient ,              /* A ZkipClient that is already connected to a nym server */
  FrCryptSig ,				/* The sig key of the nym server that the ZkipClient is connected to*/
  NymServerResponse * );    /* Returns the nym server response */

/*							
 * On success, the NymServerResponse message will store the name of the nym.
 * This is important in the cases where the nymserver assigns a new 
 * nym name for numbered (non-vanity) nyms
 */

/* Obsolete- please use frRemoteNymAddTokenConnected() corey */
DLLExport int frRemoteNymAddToken (
  ZkError ,                 /* Returns the error if any occured */
  FrRemoteNym ,             /* Contains name + keys of nym to which to add */                               /* the upgrage token */
  char * ,                  /* Nym creation/update token to use */
  NymServer * ,             /* Target nym server */
  NymServerResponse * ,     /* Returns the nym server response */
  FrRemoteNym * );          /* Returns a pointer to a new nym as stored on NS */

/* The ZkipClient must be disconnected and free'd be the caller */
DLLExport int frRemoteNymAddTokenConnected (
  ZkError ,                 /* Returns the error if any occured */
  FrRemoteNym ,             /* Contains name + keys of nym to which to add */                               /* the upgrage token */
  char * ,                  /* Nym creation/update token to use */
  ZkipClient ,              /* A ZkipClient that is already connected to a nym server */
  FrCryptSig ,				/* The sig key of the nym server that the ZkipClient is connected to*/
  NymServerResponse * ,     /* Returns the nym server response */
  FrRemoteNym * );          /* Returns a pointer to a new nym as stored on NS */

/* Obsolete- please use frRemoteNymCreateConnected() corey */
DLLExport int frRemoteNymCreate (
  ZkError ,                 /* Returns the error if any occured */
  FrRemoteNym ,             /* The nym you wish to place in the nymserver */
  char * ,                  /* Name of the nym */
  char * ,                  /* Nym creation token to use */
  NymServer * ,             /* Target nym server */
  NymServerResponse * ,     /* Returns the nym server response */
  FrRemoteNym * );          /* Returns a pointer to a new nym as stored on NS */

/* The ZkipClient must be disconnected and free'd be the caller */
DLLExport int frRemoteNymCreateConnected (
  ZkError ,                 /* Returns the error if any occured */
  FrRemoteNym ,             /* The nym you wish to place in the nymserver */
  char * ,                  /* Name of the nym */
  char * ,                  /* Nym creation token to use */
  ZkipClient ,              /* A ZkipClient that is already connected to a nym server */
  FrCryptSig ,				/* The sig key of the nym server that the ZkipClient is connected to*/
  NymServerResponse * ,     /* Returns the nym server response */
  FrRemoteNym * );          /* Returns a pointer to a new nym as stored on NS */

/* Obsolete- please use frRemoteNymModifyConnected() corey */
DLLExport int frRemoteNymModify (
  ZkError ,                 /* Returns the error if any occured */
  FrRemoteNym ,             /* The new version of the nym */
  char * ,                  /* The name of the nym */
  NymServer * ,             /* Target nym server */
  NymServerResponse * ,     /* Returns the nym server response */
  FrRemoteNym * );          /* Returns a pointer to a new nym as stored on NS */

/* The ZkipClient must be disconnected and free'd be the caller */
DLLExport int frRemoteNymModifyConnected (
  ZkError ,                 /* Returns the error if any occured */
  FrRemoteNym ,             /* The new version of the nym */
  char * ,                  /* The name of the nym */
  ZkipClient ,              /* A ZkipClient that is already connected to a nym server */
  FrCryptSig ,				/* The sig key of the nym server that the ZkipClient is connected to*/
  NymServerResponse * ,     /* Returns the nym server response */
  FrRemoteNym * );          /* Returns a pointer to a new nym as stored on NS */

/* Obsolete- please use frRemoteNymDisableConnected() corey */
DLLExport int frRemoteNymDisable (
  ZkError ,                 /* Returns the error if any occured */
  FrRemoteNym ,             /* The new version of the nym */
  char * ,                  /* The name of the nym to disable */
  NymServer * ,             /* Target nym server */
  NymServerResponse * );    /* Returns the nym server response */

/* The ZkipClient must be disconnected and free'd be the caller */
DLLExport int frRemoteNymDisableConnected (
  ZkError ,                 /* Returns the error if any occured */
  FrRemoteNym ,             /* The new version of the nym */
  char * ,                  /* The name of the nym to disable */
  ZkipClient ,              /* A ZkipClient that is already connected to a nym server */
  FrCryptSig ,				/* The sig key of the nym server that the ZkipClient is connected to*/
  NymServerResponse * );    /* Returns the nym server response */

/* Obsolete- please use frRemoteNymAvailableNamesConnected() corey */
DLLExport int frRemoteNymAvailableNames (
  ZkError ,                 /* Returns the error if any occured */
  ZkList ,                  /* The list of names to lookup */
  NymServer * ,             /* Target nym server */
  NymServerResponse * ,     /* Returns the nym server response */
  ZkList * );               /* Returns a list of available names */

/* The ZkipClient must be disconnected and free'd be the caller */
DLLExport int frRemoteNymAvailableNamesConnected (
  ZkError ,                 /* Returns the error if any occured */
  ZkList ,                  /* The list of names to lookup */
  ZkipClient ,              /* A ZkipClient that is already connected to a nym server */
  FrCryptSig ,				/* The sig key of the nym server that the ZkipClient is connected to*/
  NymServerResponse * ,     /* Returns the nym server response */
  ZkList * );               /* Returns a list of available names */

/* Obsolete- please use frRemoteNymRbActivateConnected() corey */
DLLExport int frRemoteNymRbActivate (
  ZK_INOUT ZkError ,                 /* Returns the error if any occured */
  ZK_IN    FrRemoteNym ,             /* The nym who owns the reply block */
  ZK_IN    NymServer * ,             /* Target nym server */
  ZK_IN    const void * ,            /* reply block activation token */
  ZK_IN    size_t ,                  /* sizeof reply block activation token */
  ZK_OUT   NymServerResponse * );    /* Returns the nym server response */

/* The ZkipClient must be disconnected and free'd be the caller */
DLLExport int frRemoteNymRbActivateConnected (
  ZK_INOUT ZkError ,                 /* Returns the error if any occured */
  ZK_IN    FrRemoteNym ,             /* The nym who owns the reply block */
  ZK_INOUT ZkipClient ,              /* A ZkipClient that is already connected to a nym server */
  ZK_IN    FrCryptSig , 				/* The sig key of the nym server that the ZkipClient is connected to*/
  ZK_IN    const void * ,            /* reply block activation token */
  ZK_IN    size_t ,                  /* sizeof reply block activation token */
  ZK_OUT   NymServerResponse * );    /* Returns the nym server response */

/* Obsolete- please use frRemoteNymMailCertConnected() corey */
DLLExport int frRemoteNymMailCert(
  ZK_INOUT ZkError,                  /* Returns the error if any occured */
  ZK_IN    FrRemoteNym,              /* Nym requesting the certificate */
  ZK_IN    NymServer *,              /* Target Nym server */
  ZK_IN    uint16_t,                 /* Mail Certificate version */
  ZK_OUT   NymServerResponse *,      /* Returns the nym server response */
  ZK_OUT   FrMailAuth *,             /* Returns the Mail Certificate */
  ZK_OUT   FrCryptMacKey * );        /* Secret MAC key for mail authentication */ 

/* The ZkipClient must be disconnected and free'd be the caller */
DLLExport int frRemoteNymMailCertConnected(
  ZK_INOUT ZkError,                  /* Returns the error if any occured */
  ZK_IN    FrRemoteNym,              /* Nym requesting the certificate */
  ZK_INOUT ZkipClient ,              /* A ZkipClient that is already connected to a nym server */
  ZK_IN FrCryptSig ,                 /* The sig key of the nym server that the ZkipClient is connected to*/
  ZK_IN    uint16_t,                 /* Mail Certificate version */
  ZK_OUT   NymServerResponse *,      /* Returns the nym server response */
  ZK_OUT   FrMailAuth *,             /* Returns the Mail Certificate */
  ZK_OUT   FrCryptMacKey * );        /* Secret MAC key for mail authentication */ 

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRNYMCLIENT_H_ */
