/*
 * $Id: frNymUtil.h,v 1.10.32.1 2000/10/11 13:19:00 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRNYM_UTIL_H_
#define FRNYM_UTIL_H_

#ifndef FRNYM_H_
#include <frNym.h>
#endif /* FRNYM_H_ */

#ifndef ZKLIST_H_
#include <zkList.h>
#endif /* ZKLIST_H_ */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

DLLExport int frRemoteNymGetKeys (
  ZkError ,               /* Returns the error if any occured */
  FrRemoteNym ,           /* The nym whose key structures you want */
  FrCryptSig *  ,         /* Returns the nym's public signature key */
  FrCryptCrypt * );       /* Returns the nym's public encryption key */

DLLExport int frRemoteNymSetPubKeys (
  ZkError ,               /* Returns the error if any occured */
  FrRemoteNym ,           /* FrRemoteNym to which to add the keys */
  FrCryptPubKey ,         /* Nym's encryption key */
  FrCryptPubKey );        /* Nym's signature key */

DLLExport int frRemoteNymGetPrivateKeys (
  ZkError ,
  FrRemoteNym ,
  FrCryptPubKey * ,   /* Returns the private encryption key */
  FrCryptPubKey * );   /* Returns the private signature key */

DLLExport int frRemoteNymSetPrivateKeys (
  ZkError ,
  FrRemoteNym ,
  FrCryptPubKey , /* Encryption key */
  FrCryptPubKey );  /* Signature key */


DLLExport int frRemoteNymSetPublicKeys (
  ZkError ,              /* Returns the error if any occured */
  FrRemoteNym ,          /* The nym for which to set the keys */
  FrCryptSig ,           /* PRIVATE signature key */
  FrCryptCrypt ,         /* Public encryption key */
  FrCryptSig );          /* Public signature key */


/*
 * These are being phased out, dont use them
 */

DLLExport int frRemoteNymSetBlockList (
  ZkError ,              /* Returns the error if any occured */
  FrRemoteNym ,          /* The nym which will hold this blocklist */
  ZkList );              /* Contains the list of blocked email addresses */


DLLExport int frRemoteNymGetBlockList(
  ZkError ,              /* Return the error if any occured */
  FrRemoteNym ,          /* The nym from which to get the blocklist */
  ZkList * );            /* Returns a list of blocked email addresses */


/*
 * These functions expect and return ZkLists containing NUL terminated strings
 * of at most 254 characters.  In the case of the 
 * frRemoteNymGetStringListParameter() function, the zkList which is returned
 * will have a data freeing function set so that the strings which it contains
 * will be deallocated when you call zkListFree(). 
 */

DLLExport int frRemoteNymAddStringListParameter (
  ZkError ,              /* Returns the error if one occured */
  FrRemoteNym ,          /* The nym to which to add the list */
  enum FrRemoteNymParameter , /* The parameter for which to add the list */
  ZkList );              /* a ZkList containing NUL terminated strings */


DLLExport int frRemoteNymGetStringListParameter (
  ZkError ,              /* Return the error if one occured */
  FrRemoteNym ,          /* The nym from which to get the list */
  enum FrRemoteNymParameter , /* The requested nym parameter */
  ZkList * );            /* Creates and returns a list */


/*
 * The spam controls are toggled off or on for each nym using these functions.
 * You must OR the spam control flags together (they are listed in frNym.h).
 */

DLLExport int frRemoteNymGetSpamControls (
  ZkError ,             /* Return the error if one occured */
  FrRemoteNym ,         /* Nym to search */
  uint32_t * );         /* Returns the spam controls, ORred together */

DLLExport int frRemoteNymSetSpamControls (
  ZkError ,             /* Return the error if one occured */
  FrRemoteNym ,         /* Nym to which to add the spam controls */
  uint32_t );           /* Spam controls to enable (ORred together) */

/*
 * This function is for the nym server.  It allows the nym server to modify
 * the expiration date of a signing key based on the nym's own expiration date
 */

DLLExport int frRemoteNymSetKeyExpiry (
  ZkError ,             /* Return the error if one occured */
  FrRemoteNym );        /* The nym to modify */




#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRNYM_UTIL_H_ */
