/*
 *  $Id: fr_datapkt.h,v 1.4.10.2 2000/12/14 17:53:08 jean Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FR_DATAPKT_H_
#define FR_DATAPKT_H_


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  /*
   *
   *
   *  Routines for AIP data packets.
   *
   *
   */
  
  typedef struct fr_pkt_aip_data *FrPktAD;

  DLLExport extern size_t frPktAipDataBufferSize(void);
  DLLExport extern size_t frPktAipDataXmitSize(void);

  DLLExport extern int frPktAipDataNew(ZkError err, FrPktAD *pkt);
  DLLExport extern int frPktAipDataNewNoBuffer(ZkError err, FrPktAD *pkt);
  DLLExport extern int frPktAipDataNewUsingBuffer(ZkError err, FrPktAD *pkt, void *data, size_t len);
  DLLExport extern int frPktAipDataFree(ZkError err, FrPktAD *pkt);

  DLLExport extern int frPktAipDataSetAci(ZkError err, FrPktAD pkt, FrAci aci);
  DLLExport extern int frPktAipDataGetAci(ZkError err, FrPktAD pkt, FrAci *aci);

  extern DLLExport int frPktAipDataPayloadMaxSize(ZkError err, uint8_t vers, size_t *sz);
  extern DLLExport int frPktAipDataGetMaxSize(ZkError err, unsigned int vers, size_t *sz);
  extern DLLExport int frPktAipDataNew(ZkError err, FrPktAD *pkt);
  extern DLLExport int frPktAipDataFree(ZkError err, FrPktAD *pkt);
  extern DLLExport int frPktAipDataGetCheckSpec(ZkError err, FrPktAD pkt, int *boolean);
  extern DLLExport int frPktAipDataSetCheckSpec(ZkError err, FrPktAD pkt, int boolean);
  extern DLLExport int frPktAipDataGetSpec(ZkError err, FrPktAD pkt, void *buf, size_t *bsz);
  extern DLLExport int frPktAipDataSetSpec(ZkError err, FrPktAD pkt, void *buf, size_t bsz);
  extern DLLExport int frPktAipDataGetAci(ZkError err, FrPktAD pkt, FrAci *aci);
  extern DLLExport int frPktAipDataSetAci(ZkError err, FrPktAD pkt, FrAci aci);
  extern DLLExport int frPktAipDataGetType(ZkError err, FrPktAD pkt, FrPktType *type);
  extern DLLExport int frPktAipDataSetType(ZkError err, FrPktAD pkt);
  extern DLLExport int frPktAipDataGetVersion(ZkError err, FrPktAD pkt, unsigned int *vers);
  extern DLLExport int frPktAipDataSetVersion(ZkError err, FrPktAD pkt, unsigned int vers);
  extern DLLExport int frPktAipDataGetUseMac(ZkError err, FrPktAD pkt, int *boolean);
  extern DLLExport int frPktAipDataSetUseMac(ZkError err, FrPktAD pkt, int boolean);
  extern DLLExport int frPktAipDataGetPayload(ZkError err, FrPktAD pkt, FrCryptMacKey mk, void **pay, size_t *psz);
  extern DLLExport int frPktAipDataSetPayload(ZkError err, FrPktAD pkt, FrCryptMacKey mk, void *pay, size_t psz);
  extern DLLExport int frPktAipDataSetRandomFiller(ZkError err, FrPktAD pkt);
  extern DLLExport int frPktAipDataCreateInBuffer(ZkError err, FrPktAD pkt, void *buf, size_t bsz);
  extern DLLExport int frPktAipDataReceiveInBuffer(ZkError err, FrPktAD pkt, void *data, size_t dsz);
  extern DLLExport int frPktAipDataEncryptPayload(ZkError err, FrPktAD pkt, FrCryptSym sym);
  extern DLLExport int frPktAipDataDecryptPayload(ZkError err, FrPktAD pkt, FrCryptSym sym);
  extern DLLExport int frPktAipDataGetCurrentSize(ZkError err, FrPktAD pkt, size_t *sz);
  /*
   *  Internal library routines.  (I.e. for use within other parts of this
   *  library.)
   */

  extern DLLExport int frPktAipData_SetType(ZkError err, FrPktAD pkt, FrPktType type);

  /*
   *  Test routines.
   */
  
  extern DLLExport int frPktAipData_SetPayloadSize(ZkError err, FrPktAD pkt, size_t psz);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_DATAPKT_H_ */
