/*
 *  $Id: fr_rcpkt.h,v 1.9.20.2 2000/12/14 17:53:09 jean Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FR_RCPKTH_
#define FR_RCPKTH_


/*
 *
 *
 *  Routines manipulating route information.
 *
 *
 */

/*
 *  A route creation packet may contain up to this many routes.
 */

#define FRPKT_NUM_ROUTES_MAX 5

/*
 *  The bits in the flags field are interpeted as follows:
 *  
 *  bit 0: last route -- 0 intermediate hop; 1 this is the last hop.
 *  
 *  bit 1: exit ACI -- 0 ignore the exit ACI; 1 use given ACI at exit funnel.
 */
#define FRPKT_RTECRT_FL_LAST_RTE 0x01
#define FRPKT_RTECRT_FL_SPEC_ACI 0x02


/*
 *  bug: for expediency we expose (hence the trailing _exp) two structures
 *  containing all the information we need to pass back and forth.  Given
 *  more time we probably ought to hide it.
 */

#if 0
/* bug: make consistent with rte.h. */
typedef enum {
  FRHOSTADDR_UNUSED_,
  FRHOSTADDR_IPV4
} FrHostAddrType;

#define ADDR_LEN	16
typedef char FrHostAddr[ADDR_LEN];
typedef uint16_t FrHostPort;
#endif
typedef uint32_t FrTime;

/*
 *  Use this structure when generating route creation packets.
 */
  
struct fr_rte_set_exp {
  FrCryptSymAlg rteAlg; /* to encrypt trailing part of packet route field */
  FrCryptSym bckCrypt;
  FrCryptSym fwdCrypt;
  FrCryptMacKey macKey;
  FrEntIdExp hostEnt; /* identifies this hop's host */
  FrTime expTime;
  uint8_t flags;
  FrAci specAci;
};
  
/*
 *  Use this structure when extracting routes from route creation packets.
 */
  
struct fr_rte_get_exp {
  FrCryptSym bckCrypt;
  FrCryptSym fwdCrypt;
#if 0
  unsigned int cryptKeyVers;
#endif
  FrCryptMacKey macKey;
  FrEntIdExp hostEnt;
  FrTime expTime;
  uint8_t flags;
  FrAci specAci;
};
  

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  /*
   *
   *
   *  Routines for AIP route creation packets.
   *
   *
   */
  
  typedef struct fr_pkt_rte_crt *FrPktRC;

  DLLExport extern void fr_PktAipStructInfo(void);

  DLLExport extern size_t frPktAipRteCrtBufferSize(void);
  DLLExport extern size_t frPktAipRteCrtXmitSize(void);

  DLLExport extern int frPktAipRteCrtNew(ZkError err, FrPktRC *pkt);
  DLLExport extern int frPktAipRteCrtNewNoBuffer(ZkError err, FrPktRC *pkt);
  DLLExport extern int frPktAipRteCrtNewUsingBuffer(ZkError err, FrPktRC *pkt, void *data, size_t len);
  DLLExport extern int frPktAipRteCrtFree(ZkError err, FrPktRC *pkt);
  DLLExport extern int frPktAipRteCrtReset(ZkError err, FrPktRC pkt);

  DLLExport extern int frPktAipRteCrtSetAci(ZkError err, FrPktRC pkt, FrAci aci);
  DLLExport extern int frPktAipRteCrtGetAci(ZkError err, FrPktRC pkt, FrAci *aci);

  DLLExport extern int frPktAipRteCrtAppendRouteExp(ZkError err, FrPktRC pkt, FrCryptCrypt aipPubKey, struct fr_rte_set_exp *r);
  DLLExport extern int frPktAipRteCrtGetCryptKeyVersion(ZkError err, FrPktRC pkt, unsigned int *vers);
  DLLExport extern int frPktAipRteCrtGetNextRouteExp(ZkError err, FrPktRC pkt, FrCryptCrypt privKey, struct fr_rte_get_exp *r);

  DLLExport extern int frPktAipRteCrtGetBuffer(ZkError err, FrPktRC pkt, void **data, size_t *len);
  DLLExport extern int frPktAipRteCrtSetBuffer(ZkError err, FrPktRC pkt, void *data, size_t len);

  DLLExport extern int frPktAipRteCrtEncryptAllRoutes(ZkError err, FrPktRC pkt);

  extern DLLExport void fr_PktAipStructInfo(void);
  extern DLLExport int frPktAipRteCrtGetMaxSize(ZkError err, unsigned int vers, size_t *sz);
  extern DLLExport int frPktAipRteCrtNew(ZkError err, FrPktRC *pkt);
  extern DLLExport int frPktAipRteCrtFree(ZkError err, FrPktRC *pkt);
  extern DLLExport int frPktAipRteCrtReset(ZkError err, FrPktRC pkt);
  extern DLLExport int frPktAipRteCrtCreateInBuffer(ZkError err, FrPktRC pkt, void *buf, size_t bsz);
  extern DLLExport int frPktAipRteCrtReceiveInBuffer(ZkError err, FrPktRC pkt, void *buf, size_t bsz);
  extern DLLExport int frPktAipRteCrtSetAci(ZkError err, FrPktRC pkt, FrAci aci);
  extern DLLExport int frPktAipRteCrtGetAci(ZkError err, FrPktRC pkt, FrAci *aci);
  extern DLLExport int frPktAipRteCrtAppendRouteExp(ZkError err, FrPktRC pkt, FrCryptCrypt aipPubKey, struct fr_rte_set_exp *r);
  extern DLLExport int frPktAipRteCrtGetCryptKeyVersion(ZkError err, FrPktRC pkt, unsigned int *vers);
  extern DLLExport int frPktAipRteCrtGetNextRouteExp(ZkError err, FrPktRC pkt, FrCryptCrypt aipPrivKey, struct fr_rte_get_exp *r);
  extern DLLExport int frPktAipRteCrtAppendNymAuthExp(ZkError err, FrPktRC pkt, struct fr_nym_auth_exp *nymAuth);
  extern DLLExport int frPktAipRteCrtGetNymAuthExp(ZkError err, FrPktRC pkt, struct fr_nym_auth_exp *nymAuth);
  extern DLLExport int frPktAipRteCrtAuthenticateNym(ZkError err, struct fr_nym_auth_exp *nymAuth, int *isAuth);
  extern DLLExport int frPktAipRteCrtGetBuffer(ZkError err, FrPktRC pkt, void **data, size_t *len);
  extern DLLExport int frPktAipRteCrtEncryptAllRoutes(ZkError err, FrPktRC pkt);
  extern DLLExport int frPktAipRteCrtGetCurrentSize(ZkError err, FrPktRC pkt, size_t *sz);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_RCPKTH_ */
