/*
 *  $Id: pkt.h,v 1.4.32.1 2000/10/11 13:19:09 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef PKT_H_
#define PKT_H_

#include "zkIntTypes.h"
#include "zks.h"
#include "frPktFmt.h"


/*
 */

#define PKT_RAND_OFFSET        0
#define PKT_RAND_SIZE          FRCRYPT_RAND_SIZE

#define STD_HDR_VERSION_OFFSET (PKT_RAND_OFFSET + PKT_RAND_SIZE)
#define STD_HDR_VERSION_SIZE   1

#define STD_HDR_TYPE_OFFSET    (STD_HDR_VERSION_OFFSET + STD_HDR_VERSION_SIZE)
#define STD_HDR_TYPE_SIZE      1

#define STD_HDR_PRI_OFFSET     (STD_HDR_TYPE_OFFSET + STD_HDR_TYPE_SIZE)
#define STD_HDR_PRI_SIZE       1

#define STD_HDR_SPEC_OFFSET    (STD_HDR_PRI_OFFSET + STD_HDR_PRI_SIZE)
#define STD_HDR_SPEC_SIZE      3

#define STD_HDR_OFFSET         STD_HDR_VERSION_OFFSET
#define STD_HDR_SIZE           (STD_HDR_VERSION_SIZE + STD_HDR_TYPE_SIZE + \
                                STD_HDR_SPEC_SIZE)


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  /*
   *  Internal library routines.
   */
  
  extern DLLExport int fr_PktMemCheck(void *data, char value, size_t sz);

  
  /*
   *  General packet routines.
   */
  
#if 0
  extern int frPktNew(ZkError err, FrPkt *pkt);
  extern int frPktFree(ZkError err, FrPkt *pkt);

  extern int frPktGetHeader(ZkError err, FrPkt pkt,
                            unsigned int *version, unsigned int *type, size_t *len);
  extern int frPktSetHeader(ZkError err, FrPkt pkt,
                            unsigned int version, unsigned int type);

  extern int frPktInitEmptyPacket(ZkError err, FrPkt pkt, void *data, size_t len);
  extern int frPktInitFromFullPacket(ZkError err, FrPkt pkt, void *data, size_t len);

  extern int frPktAppendData(ZkError err, FrPkt pkt, void *data, size_t len);
  extern int frPktPadToMultipleFromOffset(ZkError err, FrPkt pkt, size_t mult, size_t off);

  extern int frPktSymEncrFromOffset(ZkError err, FrPkt pkt, FrCryptSym sym, size_t off);
  extern int frPktSymDecrFromOffset(ZkError err, FrPkt pkt, FrCryptSym sym);

#if 0
  /* bug: currently no length in packet field. */
  extern int frPktGetLength(ZkError err, FrPkt pkt, size_t *len);
#endif
#endif

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
