/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef __KEEPALIVEPKT_H
#define __KEEPALIVEPKT_H

#include "frPktFmt.h"
#include "frPktWrap.h"

class cKeepAlivePkt
{
private:
    FrPktAD  kaCtx;      // pkt buffer context
    uint8_t  *pktbuffer; //pkt buffer
    size_t   pktbufferlen;
    ZkBool   freePkt;    // indicates whether the pkt buffer must be freed
                         //  before destruction of the object

    //copy constructor (disabled)
    cKeepAlivePkt( const cKeepAlivePkt&) { assert(0); };

    //assignment operator (disabled)
    cKeepAlivePkt& operator=( const cKeepAlivePkt&) { assert(0); };

public:
    cKeepAlivePkt();         // constructor
    ~cKeepAlivePkt();        // destructor

    //
    // This member generates a new Keep Alive packet, including the link
    // layer header and trailer.  After a call to this member, the packet
    // is ready to be transmitted.
    // 
    // NOTE: After a call to this member, make sure destroy is called
    //       before the object is destroyed.
    //
    void create
    (
        uint32_t      version,    // packet version
        uint8_t       prio        // packet priority
    );
    
    //
    // This member is called to release the resources allocated during
    // the previous call to create.
    // 
    // NOTE: This MUST be called before the object is destroyed if create
    //       was used.
    //
    void destroy(void);
    
    //
    // This member returns a reference to the Keep Alive packet buffer
    // and its size, including the link layer header and trailer.
    // 
    // A packet must currently be attached for this member to work,
    // i.e. a previous call to create was made.
    // 
    void getBuffer
    (
        uint8_t ** buf,    // out: pointer to buffer
        size_t   * bufSize // out: buffer size
    );
};
#endif /* __KEEPALIVEPKT_H */
