// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom(R) code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#undef NDEBUG
#include <assert.h>
#include "zksexception.h"
#include "cRouteDestroyPkt.h"
#include <iostream>

cRouteDestroyPkt::cRouteDestroyPkt()
{
    rdCtx = NULL;
    pktbuffer = NULL;
    pktbufferlen = 0;
    freePkt = ZK_FALSE;
}

cRouteDestroyPkt::~cRouteDestroyPkt()
{
   if (freePkt) {
      destroy();

   }
}

void cRouteDestroyPkt::create(uint32_t version, uint8_t prio,
                              FrAci nextHopAci, uint32_t errcode,
                              FrEntIdExp entity)
{
    const char    * func = "cRouteDestroyPkt::create";
    CZKS::Exception e;

    /* Validate params */
    assert((version == FRPKT_DATA_VERSION_1) || (version == FRPKT_DATA_VERSION_2));
    assert(rdCtx == NULL);
    assert(!freePkt);
            
    try
    {
        uint8_t       * pkt = NULL;
        size_t          pktSize = 0;
        size_t          maxsize;

        if (version ==  FRPKT_DATA_VERSION_2) {
           if (frPktAipRteDestGetMaxSize(e.zkError(), version, &maxsize) == 
               ZKS_FAILURE) 
           {
              CZKS::Throw(e);
           }
	}
        else {
           if (frPktAipDataGetMaxSize(e.zkError(), version, &maxsize) ==
               ZKS_FAILURE) 
           {
              CZKS::Throw(e);
           }
	}

        pktbufferlen = frPktGetSize( maxsize);
        pktbuffer = new uint8_t[pktbufferlen+1]; 
      
        // Allocate Route Create ACK context
        if (frPktAipRteDestNew(e.zkError(), &rdCtx) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        else if (frPktSetType(e.zkError(), pktbuffer, pktbufferlen, 
                              FRPKTTYPE_AIP_ROUTE_DESTROY) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        else if (frPktSetVersion(e.zkError(), pktbuffer, pktbufferlen, 
                                 version) == ZKS_FAILURE) 
        {
            CZKS::Throw(e);
        }
        else if (frPktSetPriority(e.zkError(), pktbuffer, pktbufferlen, 
                                  prio) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        else if (frPktGetEmbeddedPacket(e.zkError(), 
                                        pktbuffer, pktbufferlen, 
                                        (void**)&pkt, &pktSize) == ZKS_FAILURE) 
        {
            CZKS::Throw(e);
        }
        else if (frPktAipRteDestCreateInBuffer(e.zkError(), rdCtx, pkt, 
                                               pktSize) == ZKS_FAILURE)
        {
           CZKS::Throw(e);
        }
        else if (frPktAipRteDestSetVersion(e.zkError(), rdCtx, 
                                           version) == ZKS_FAILURE) 
        {
           CZKS::Throw(e);
        }
        else if (version == FRPKT_DATA_VERSION_2) {
	
           if (frPktAipRteDestSetErrorCode(e.zkError(), rdCtx, errcode) ==
               ZKS_FAILURE)
           {
              CZKS::Throw(e);
           }
	}
           if (frPktAipRteDestSetIssuer(e.zkError(), rdCtx, entity) ==
                    ZKS_FAILURE)
           {
              CZKS::Throw(e);
           }
           else if (frPktAipRteDestSetAci(e.zkError(), rdCtx, nextHopAci) ==
                    ZKS_FAILURE)
           {
              CZKS::Throw(e);
           }
	
        freePkt = ZK_TRUE;
    }
    catch (...)
    {
        delete pktbuffer;
        pktbuffer = NULL;

 
        if (rdCtx != NULL)
        {
            frPktAipRteDestFree(e.zkError(), &rdCtx);
            rdCtx = NULL;
        }
        
        CZKS::Rethrow(func, "Failed");
    }
}

void cRouteDestroyPkt::destroy(void)
{
    const char * func = "cRouteDestroyPkt::destroy";

    assert(rdCtx != NULL);
    assert(freePkt);
    
    try
    {
        CZKS::Exception e;
        
        delete pktbuffer;
        pktbuffer = NULL;
 
        if (frPktAipRteDestFree(e.zkError(), &rdCtx) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        freePkt = ZK_FALSE;
        rdCtx = NULL;
    }
    
    catch (CZKS::Exception &e)
    {
        CZKS::Rethrow(func, "Failed");
    }
}

void cRouteDestroyPkt::attachPkt
(
    uint8_t * pkt,         /* buffer containing RC pkt */
    size_t    pktSize      /* pkt size */
)
{
    const char    * func = "cRouteDestroyPkt::attachPkt";
    CZKS::Exception e;

    /* Validate params */
    assert(pkt != NULL);
    assert(pktSize > 0);
    assert(rdCtx == NULL);

    try
    {
        uint8_t *epkt;
        size_t epktlen;        

        if (frPktAipRteDestNew(e.zkError(), &rdCtx) == ZKS_FAILURE) 
        {
            CZKS::Throw(e);
        }
        else if (frPktGetEmbeddedPacket(e.zkError(), 
                                        pkt, pktSize, 
                                        (void**)&epkt, &epktlen) == ZKS_FAILURE) 
        {
            CZKS::Throw(e);
        }
        /* Associate pkt with packet context */
        else if (frPktAipRteDestReceiveInBuffer(e.zkError(), rdCtx, 
                                                epkt, epktlen) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
	}
        //save pointer to pkt
        pktbuffer = pkt;
        pktbufferlen = pktSize;

    } /* end of try */

    catch (...)
    {
        if (rdCtx != NULL)
        {
            frPktAipRteDestFree(e.zkError(), &rdCtx);
        }
        
        CZKS::Rethrow(func, "failed");
    }
}

void cRouteDestroyPkt::detachPkt(void)
{
    const char * func = "cRouteDestroyPkt::detachPkt";

    /* Validate params */
    assert(rdCtx != NULL);

    try
    {
        CZKS::Exception e;
        
        if (frPktAipRteDestFree(e.zkError(), &rdCtx) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        else
        {
            rdCtx = NULL;
        }
    } /* end of try */

    catch(...)
    {
        CZKS::Rethrow(func, "failed");
    }
}

void cRouteDestroyPkt::formatNextHop( FrAci aci)
{
    const char    * func = "cRouteDestroyPkt::formatNextHop";
    CZKS::Exception e;

    /* Validate params */
    assert(rdCtx != NULL);
            
    try
    {
        if (!frPktAipRteDestSetAci(e.zkError(), rdCtx, aci))
        {
            CZKS::Throw(e);
        }
    }
    catch (...)
    {
        CZKS::Rethrow(func, "Failed");
    }
}

FrAci cRouteDestroyPkt::getAci()
{
    const char * func = "cRouteDestroyPkt::getAci";

    assert(rdCtx != NULL);

    FrAci aci;

    try
    {
        CZKS::Exception e;
        
        if (frPktAipRteDestGetAci(e.zkError(), rdCtx, &aci) == 
            ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
    } /* end of throw */

    catch (...)
    {
        CZKS::Rethrow(func, "cRouteCreatePkt failed");
    }

    return aci;

} 

uint32_t cRouteDestroyPkt::getErrcode()
{
    const char * func = "cRouteDestroyPkt::getErrcode";

    assert(rdCtx != NULL);

    uint32_t errcode;

    try
    {
        CZKS::Exception e;
        
        if (frPktAipRteDestGetErrorCode(e.zkError(), rdCtx, &errcode) == 
            ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
    } /* end of throw */

    catch (...)
    {
        CZKS::Rethrow(func, "cRouteCreatePkt failed");
    }

    return errcode;

} 

FrEntIdExp cRouteDestroyPkt::getEntity()
{
    const char * func = "cRouteDestroyPkt::getEntity";

    assert(rdCtx != NULL);

    FrEntIdExp entity;

    try
    {
        CZKS::Exception e;
        
        if (frPktAipRteDestGetIssuer(e.zkError(), rdCtx, &entity) == 
            ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
    } /* end of throw */

    catch (...)
    {
        CZKS::Rethrow(func, "cRouteCreatePkt failed");
    }

    return entity;

} 

void cRouteDestroyPkt::encrypt
(
    FrCryptSym cryptKey /* cryption key (for hop info decryption) */
)
{
    const char    * func = "cRouteCreatePkt::decrypt";
    CZKS::Exception e;

    /* Validate params */
    assert(cryptKey != NULL);
    assert(rdCtx != NULL);

    try
    {
        if (frPktAipRteDestEncrypt(e.zkError(), rdCtx, cryptKey) == 
            ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }

    } /* end of try */

    catch (...)
    {
        CZKS::Rethrow(func,"failed");
    }
}

void cRouteDestroyPkt::decrypt
(
    FrCryptSym cryptKey /* cryption key (for hop info decryption) */
)
{
    const char    * func = "cRouteCreatePkt::decrypt";
    CZKS::Exception e;

    /* Validate params */
    assert(cryptKey != NULL);
    assert(rdCtx != NULL);

    try
    {
        if (frPktAipRteDestDecrypt(e.zkError(), rdCtx, cryptKey) == 
            ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }

    } /* end of try */

    catch (...)
    {
        CZKS::Rethrow(func, "failed");
    }

}

void cRouteDestroyPkt::getBuffer
(
    uint8_t ** buf,    // out: pointer to buffer
    size_t   * bufSize // out: buffer size
)
{
    const char * func = "cRouteDestroyPkt::getBuffer";

    size_t pktsize;

    // Validate params
    assert(buf != NULL);
    assert(bufSize != NULL);

    // A packet MUST currently be attached
    assert(rdCtx != NULL);

    try
    {
        CZKS::Exception e;
        
        /* Get size of data packet. */
        if (frPktAipRteDestGetCurrentSize(e.zkError(), rdCtx, 
                                          &pktsize) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        *bufSize = frPktGetSize(pktsize);
        *buf = (uint8_t*)pktbuffer;
 
    }
    catch (CZKS::Exception &e)
    {
        CZKS::Rethrow(func, "Failed");
    }
}
