/*
 *  $Id: rand.c,v 1.2.32.3 2000/12/14 17:53:14 jean Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include <errno.h>
#include <limits.h>
#include <stdio.h>
#include <string.h>
#include "frsec_rand.h"
#include "frCrypt.h"
#include "zkError.h"


DLLExport int frSecSeedFromRandomSource(ZkError err, char *path, size_t nbits)
{
  FILE *fp = 0;
  int c, ret = 0;
  size_t n, nbytes = nbits / CHAR_BIT + 1;
  
  zkErrorClear(err);

  if ( ! path) {
    zkErrorSet(err, "frSecSeedFromRandomSource", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "path is null");
    return 0;
  }
  if (nbits == 0) {
    zkErrorSet(err, "frSecSeedFromRandomSource", ZKERR, ZKMFRSEC, ZKEPARMBAD,
               "nbits is 0");
    return 0;
  }
  
  if ( ! (fp = fopen(path, "rb"))) {
    zkErrorSet_va(err, "frSecSeedFromRandomSource", ZKERR, ZKMFRSEC, errno,
                  "can't open `%s' for reading: %s", path, strerror(errno));
    goto done;
  }

  for (n = 0; (c = fgetc(fp)) != EOF && n < nbytes; n++) {
    unsigned char uc = c;
    (void)zkRandEntropyAdd(err, &uc, sizeof uc);
  }

  if (c == EOF) {
    zkErrorSet_va(err, "frSecSeedFromRandomSource", ZKERR, ZKMFRSEC, errno,
                  "error reading from `%s': %s", path, strerror(errno));
    goto done;
  }
  
  ret = 1;

 done:

  if (fp) { fclose(fp); }

  return ret;
}
