/*
 * $Id: frToken.h,v 1.6.32.1 2000/10/11 13:19:25 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRTOKEN_H_
#define FRTOKEN_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifndef FRCRYPT_H_ 
#include <frCrypt.h>
#endif /* !FRCRYPT_H_ */

#include "frEnt.h"

typedef struct T_FrToken * FrToken;

#define TOKEN_SZ 128      /* Size of the token's random number (in bytes) */
#define RESERVED_SZ 20    /* Size of the reserved area */
#define TSNAME_SZ 64      /* Name of the signing token server */

/*
 * Raw (unsigned) number of bytes for binary version of token
 */

#define TOKEN_RAW_BINARY_SZ (2+1+2+TOKEN_SZ+RESERVED_SZ)

/*
 * Buffer size required to store the complete signed token
 */

#define TOKEN_BINARY_SZ (TOKEN_RAW_BINARY_SZ + 150)

/*
 * Suggested string size for a base64 token
 */
#define TOKEN_BASE64_SZ (TOKEN_BINARY_SZ * 4 / 3 + 5)

DLLExport int frTokenNew (
  ZkError ,
  FrToken * );

DLLExport int frTokenFree (
  ZkError ,
  FrToken * );

DLLExport int frTokenNumber (
  ZkError ,
  FrToken ,
  const unsigned char ** ,
  size_t * );

DLLExport int frTokenTypeId (
  ZkError ,
  FrToken ,
  unsigned short * );
 
DLLExport int frTokenSignerEntity (
  ZkError ,           /* Return the error if one occured */
  FrToken ,           /* The token structure */
  FrEntIdExp * );     /* Loads struct with the signer's entity */

DLLExport int frTokenSet (
  ZkError ,
  FrToken ,
  unsigned short ,
  FrEntIdExp * );

DLLExport int frTokenNewFromBinary (
  ZkError ,
  FrToken * ,
  unsigned char * ,
  size_t ,
  FrCryptSig );

DLLExport int frTokenStoreToBinary (
  ZkError ,
  FrToken ,
  unsigned char * ,
  size_t * ,
  FrCryptSig );

DLLExport int frTokenNewFromBase64 (
  ZkError ,
  FrToken * ,
  char * ,
  FrCryptSig );

DLLExport int frTokenStoreToBase64 (
  ZkError ,
  FrToken ,
  char * ,
  size_t * , /* Takes size of preallocated buffer, returns lenght of string */
  FrCryptSig );



DLLExport int frTokenEntityAndKeyVersionFromBinary (
  ZkError ,               /* Returns the error if one occured */
  void * ,                /* Binary version of the token */
  size_t ,                /* Size of the binary token buffer */
  FrEntIdExp * ,          /* Loads the struct with the token signer's entity */
  unsigned int *);        /* Returns the required signature key version */

DLLExport int frTokenEntityAndKeyVersionFromBase64 (
  ZkError ,               /* Returns the error if one occured */
  char * ,                /* The base 64 token */
  FrEntIdExp * ,          /* Loads an entity structure with the token signer */
  unsigned int * );       /* Returns the required signature key version */





#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRTOKEN_H_ */
