/*
 * $Id: frCapability.c,v 1.6.28.1 2000/10/11 13:19:26 phil Exp $
 */

static const char * module_id_ = "$Id: frCapability.c,v 1.6.28.1 2000/10/11 13:19:26 phil Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

 
#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <ctype.h>
#include <string.h>
#include <zkError.h>
#include <frCapability.h>

#define FRSNCAPABILITY_SZ     FRCAPCODE_V2_SZ /* Size of the capability code */

/*
 * These are defined in frSerial.c
 */

extern const char sn_digit[];
extern const unsigned char sn_value[]; 

#define FRCAPCODE_SZ      10  /* Version 1, no longer in active use */
#define FRCAPCODE_V2_SZ   20  /* Version 2 (current) */

DLLExport int
frCapabilityCodeString (
  ZK_INOUT ZkError err ,
  ZK_INOUT char * c ,
  ZK_IN    size_t size ,
  ZK_IN    size_t token_count ,
  ZK_IN    unsigned short token_type ,
  ZK_IN    unsigned short sn_type ,
  ZK_IN    time_t activate )
{
  (void) module_id_; /* Gets rid of compile time warning */
  zkErrorClear(err);
  if (c) {
    if (size < (FRCAPCODE_V2_SZ + 1)) {
      zkErrorSet(err,"frCapabilityCodeString",ZKERR,ZKMTOKEN,ZKEDATABAD,
                 "String is too small to hold capability code");
      return 0;
    } 

    c[0] = sn_digit[1];   /* Code ID, digit 1 */
    c[1] = sn_digit[5];   /* Code ID, digit 2 */
    c[2] = sn_digit[2];   /* Version 2 */
    c[3] = sn_digit[token_count % 32]; /* Number of tokens */
    c[4] = sn_digit[(token_type&0x7C00)>>10]; /* Token Type */
    c[5] = sn_digit[(token_type&0x03E0)>>5];  /* Token Type */
    c[6] = sn_digit[(token_type&0x001F)];     /* Token Type */

    c[7] = sn_digit[(sn_type&0x7C00)>>10]; /* serial number Type */
    c[8] = sn_digit[(sn_type&0x03E0)>>5];  /* serial number Type */
    c[9] = sn_digit[(sn_type&0x001F)];     /* serial number Type */

    c[10] = sn_digit[(activate&0x80000000)>>30]; /* SN activate time */
    c[11] = sn_digit[(activate&0x3E000000)>>25]; /* SN activate time */
    c[12] = sn_digit[(activate&0x01F00000)>>20]; /* SN activate time */
    c[13] = sn_digit[(activate&0x000F1000)>>15]; /* SN activate time */
    c[14] = sn_digit[(activate&0x00007C00)>>10]; /* SN activate time */
    c[15] = sn_digit[(activate&0x000003E0)>>5];  /* SN activate time */
    c[16] = sn_digit[(activate&0x0000001F)];     /* SN activate time */

    c[17] = sn_digit[11];
    c[18] = sn_digit[24];
    c[19] = sn_digit[9];
    c[20] = (char) 0;

    return 1;
  }
  zkErrorSet(err,"frCapabilityCodeString",ZKERR,ZKMTOKEN,ZKENULLPTR,NULL);
  return 0;
}

DLLExport int
frCapabilityCodeParse (
  ZkError err, 
  char * c ,
  size_t size ,
  size_t * token_count ,
  unsigned short * token_type ,
  unsigned short * sn_type ,
  time_t * activate)
{
  zkErrorClear(err);
  if (c) {
    if (size < FRCAPCODE_SZ) {
      zkErrorSet(err,"frCapabilityCodeParse",ZKERR,ZKMTOKEN,ZKEDATABAD,
                 "Capability code string is too short");
      return 0;

    /* Version 1 of the capability code */

    } else if ((sn_value[(int) c[0]] == 1) && (sn_value[(int) c[1]] == 5) &&
               (sn_value[(int) c[2]] == 1) && (sn_value[(int) c[7]] == 11) &&
               (sn_value[(int) c[8]] == 24) && (sn_value[(int) c[9]] == 9)) {
      if (token_count) {
        *token_count = sn_value[(int) c[3]];
      }
      if (token_type) {
        *token_type = sn_value[(int) c[4]]*1024 + sn_value[(int) c[5]]*32 + 
                      sn_value[(int) c[6]];
      }
      if (activate) {
        *activate = time(NULL);
      }
      if (sn_type) {
        *sn_type = OLD_STYLE_SERIAL_NUMBER;
      }

    /* Version 2 capability code is bigger */

    } else if (size < FRCAPCODE_V2_SZ) {
      zkErrorSet(err,"frCapabilityCodeParse",ZKERR,ZKMTOKEN,ZKEDATABAD,
                 "Capability code string is too short");
      return 0;

    /* Version 2 of the capability code */

    } else if ((sn_value[(int) c[0]] == 1) && (sn_value[(int) c[1]] == 5) &&
        (sn_value[(int) c[2]] == 2) && (sn_value[(int) c[17]] == 11) &&
        (sn_value[(int) c[18]] == 24) && (sn_value[(int) c[19]] == 9)) {

      if (token_count) {
        *token_count = sn_value[(int) c[3]];
      }
      if (token_type) {
        *token_type = sn_value[(int) c[4]]*1024 + sn_value[(int) c[5]]*32 + 
                      sn_value[(int) c[6]];
      }
      if (sn_type) {
        *sn_type = sn_value[(int) c[7]]*1024 + sn_value[(int) c[8]]*32 +
                   sn_value[(int) c[9]];
      }
      if (activate) {
        *activate = sn_value[(int) c[10]]*1073741824 +
                    sn_value[(int) c[11]]*33554432 + 
                    sn_value[(int) c[12]]*1048576 +
                    sn_value[(int) c[13]]*32768  + sn_value[(int) c[14]]*1024 + 
                    sn_value[(int) c[15]]*32 + sn_value[(int) c[16]];
      }

    /* Unrecognized capability code */

    } else {
      zkErrorSet(err,"frCapabilityCodeParse",ZKERR,ZKMTOKEN,ZKEDATABAD,
                 "Incorrect ID, version, or reserved values");
      return 0;
    } 
    return 1;
  }
  zkErrorSet(err,"frCapabilityCodeParse",ZKERR,ZKMTOKEN,
             ZKENULLPTR,NULL);
  return 0;
}
  
  
