/* 
 * $Id: frToken.c,v 1.9.32.3 2000/12/14 17:53:16 jean Exp $
 */

static const char * module_id_ = "$Id: frToken.c,v 1.9.32.3 2000/12/14 17:53:16 jean Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */



#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <zkError.h>
#include "frSec.h"
#include <zkBase64.h>
#include <frSerial.h>
#include <frCapability.h>
#include <frToken.h>

/*
 * Token version 1 uses the old style signature stuff
 * Token version 2 uses the frSec functions to sign the token.
 */

struct T_FrToken {
  unsigned char version;  /* Version number, for now should always be 1 or 2 */
  unsigned char token[TOKEN_SZ + 1]; /* The token number */
  unsigned short type_id; /* The token type ID */
  FrEntIdExp entity;         /* Entity which created the token */
};

DLLExport int
frTokenNew (
  ZkError err ,
  FrToken * token )
{
  (void) module_id_; /* Get rid of compile time warning */
  zkErrorClear(err);
  if (token) {
    if ((*token = (FrToken) malloc(sizeof(struct T_FrToken))) == NULL) {
      zkErrorSet(err,"frTokenNew",ZKERR,ZKMTOKEN,ZKENOMEM,"malloc");
      return 0;
    }
    memset(*token,0,sizeof(struct T_FrToken));
    (*token)->version = 2;
    return 1;
  }
  zkErrorSet(err,"frTokenNew",ZKERR,ZKMTOKEN,ZKENULLPTR,NULL);
  return 0;
}

DLLExport int
frTokenFree (
  ZkError err ,
  FrToken * token )
{
  zkErrorClear(err);
  if (token) {
    if (*token) {
      memset(*token,0,sizeof(struct T_FrToken));
      free(*token);
      *token = NULL;
      return 1;
    }
  }
  zkErrorSet(err,"frTokenFree",ZKERR,ZKMTOKEN,ZKENULLPTR,NULL);
  return 0;
}

DLLExport int
frTokenNumber (
  ZkError err ,
  FrToken token ,
  const unsigned char ** number ,
  size_t * size )
{
  zkErrorClear(err);
  if (token && number && size) {
    *number = token->token;
    *size = TOKEN_SZ;
    return 1;
  }
  zkErrorSet(err,"frTokenNumber",ZKERR,ZKMTOKEN,ZKENULLPTR,NULL);
  return 0;
}

DLLExport int
frTokenTypeId (
  ZkError err,
  FrToken token ,
  unsigned short * typeid )
{
  zkErrorClear(err);
  if (token && typeid) {
    *typeid = token->type_id;
    return 1;
  }
  zkErrorSet(err,"frTokenTypeId",ZKERR,ZKMTOKEN,ZKENULLPTR,NULL);
  return 0;
}

DLLExport int
frTokenSignerEntity (
  ZkError err ,
  FrToken token ,
  FrEntIdExp * entity)
{
  zkErrorClear(err);
  if (token && entity) {
    *entity = token->entity; 
    return 1;
  }
  zkErrorSet(err,"frTokenSignerName",ZKERR,ZKMTOKEN,ZKENULLPTR,NULL);
  return 0;
}

DLLExport int
frTokenSet (
  ZkError err ,
  FrToken token ,
  unsigned short type_id ,
  FrEntIdExp * entity )
{

  zkErrorClear(err);
  if (token && entity) {
    if (!zkRandGetData(err,token->token,TOKEN_SZ)) {
      return 0;
    }
    token->type_id = type_id;
    token->entity = *entity; 
    token->version = 2;    
    return 1;
  }
  zkErrorSet(err,"frTokenSet",ZKERR,ZKMTOKEN,ZKENULLPTR,NULL);
  return 0;
}

DLLExport int
frTokenNewFromBase64 (
  ZkError err ,
  FrToken * token ,
  char * base64 ,
  FrCryptSig sig )
{
  unsigned char * binary = NULL;
  int return_code = 0;
  size_t size = 0;

  zkErrorClear(err);
  if (token && base64) {
    size = strlen(base64);
    if ((binary = (unsigned char *) malloc(sizeof(char) * size)) == NULL) {
      zkErrorSet(err,"frTokenNewFromBase64",ZKERR,ZKMTOKEN,ZKENOMEM,"malloc");
      return 0;
    }
    if (zkConvertFromBase64(err,(unsigned char *)base64,binary,&size)) {
      if (frTokenNewFromBinary(err,token,binary,size,sig)) {
        return_code = 1;
      }
    }
    free(binary);
    return return_code;
  }
  zkErrorSet(err,"frTokenNewFromBase64",ZKERR,ZKMTOKEN,ZKENULLPTR,NULL);
  return 0;
}
  
/**
 * Create a new token structure from its binary representation.  If no 
 * signature is specified, the token will automatically be created if the
 * format is correct, otherwise, the token will only be created if the 
 * signature checks out fine.
 */

DLLExport int
frTokenNewFromBinary (
  ZkError err ,
  FrToken * token ,
  unsigned char * binary ,
  size_t size ,
  FrCryptSig sig )
{
  int valid = 0; /* Is the token data valid? */
  unsigned char * buf = NULL;

  zkErrorClear(err);
  if (token && binary) {
    if (size < TOKEN_RAW_BINARY_SZ) {
      zkErrorSet(err,"frTokenNewFromBinary",ZKERR,ZKMTOKEN,ZKEINCOMPLETE,
                 "Buffer is incomplete");
      return 0;
    }
    /*
     * Verify the token signature
     */
   
    if (sig) {  
#if 0
      if (!frCryptSigVerifyData(err,sig,binary,TOKEN_BINARY_SZ-FRCRYPT_SIG_SIZE,
                                &(binary[TOKEN_BINARY_SZ - FRCRYPT_SIG_SIZE]),
                                FRCRYPT_SIG_SIZE,&valid)) {
        zkErrorSet(err,"frCryptSigVerifyData",ZKERR,ZKMTOKEN,ZKERROR,
                   "Some sort of error");
        return 0;
#endif
   
      /*
       * Check the signature on the token 
       */

      if (!frSecSigVerify(err,sig,binary,size,&valid)) {
        return 0;
      } else if (!valid) {
        zkErrorSet(err,"frTokenNewFromBinary",ZKERR,ZKMTOKEN,ZKESIGCK,
                   "Signature verification failed");
        return 0;
      } 
    }

    if (!frTokenNew(err,token)) {
      return 0;
    }
   
    /*
     * Set the signing entity
     */

    if (!frSecSigGetEntity(err,binary,size,&((*token)->entity))) {
      return 0;
    }

    /*
     * Now remove the signature so we can reconstruct the rest of the token.
     */

    if (!frSecSigRemove(err,binary,size,(void **) &buf,&size)) {
      return 0;
    }

    if ((buf[0] != 0xCA) || (buf[1] != 0xFE) || (buf[2] != 0x02) ||
        (buf[3] != (TOKEN_RAW_BINARY_SZ/0x100)) || 
        (buf[4] != (TOKEN_RAW_BINARY_SZ%0x100))) {
      zkErrorSet(err,"frTokenNewFromBinary",ZKERR,ZKMTOKEN,ZKEPARSE,
                 "Token contains incorrect id, version, or size");
      return 0;
    }

    (*token)->version = 2;
    memcpy((*token)->token,&(buf[5]),TOKEN_SZ);
    (*token)->type_id = buf[5 + TOKEN_SZ]*0x100 + buf[6 + TOKEN_SZ];
  
    /*
     * Get rid of our temporary copy of the token 
     */

    if (buf) {
      free(buf);
    }

    return 1;
  }
  zkErrorSet(err,"frTokenNewFromBinary",ZKERR,ZKMTOKEN,ZKENULLPTR,NULL);
  return 0;
}

DLLExport int
frTokenStoreToBase64 (
  ZkError err ,
  FrToken token ,
  char * base64 ,
  size_t * size ,
  FrCryptSig sig )
{
  unsigned char * binary = NULL;
  int return_code = 0;
  size_t binary_sz = 0;

  zkErrorClear(err);
  if (token && base64 && size) {
    if (*size < (((TOKEN_BINARY_SZ-1)/3+2)*4 + 1)) {
      zkErrorSet(err,"frTokenStoreToBase64",ZKERR,ZKMTOKEN,ZKEDATABAD,
                 "Buffer insufficiently large to store base64 token");
      return 0;
    }
    binary_sz = *size;
    if ((binary = (unsigned char *) malloc(sizeof(char) * binary_sz)) == NULL) {
      zkErrorSet(err,"frTokenStoreToBase64",ZKERR,ZKMTOKEN,ZKENOMEM,"malloc");
      return 0;
    }
    if (frTokenStoreToBinary(err,token,binary,&binary_sz,sig)) {
      if (zkConvertToBase64(err,binary,(unsigned char *)base64,binary_sz)) {
        return_code = 1;
        *size = strlen(base64);
      }
    }
    free(binary);
    return return_code;
  }
  zkErrorSet(err,"frTokenStoreToBase64",ZKERR,ZKMTOKEN,ZKENULLPTR,NULL);
  return 0;
}

/**
 *
 * Dont forget to set the maximum size of the buffer and pass it through the
 * size parameter.  This parameter will be reset to reflect the new size of
 * the data.
 */


DLLExport int
frTokenStoreToBinary (
  ZkError err ,
  FrToken token ,
  unsigned char * binary ,
  size_t * size ,
  FrCryptSig sig )
{
  void * sbuf = NULL;
  size_t sbuf_sz = 0;

  zkErrorClear(err);
  if (token && binary && size) { 

    /*
     * The 150 is the space required to add the signature.  There isnt
     * a proper define for this anywhere unfortunatly.
     */

    if (*size < TOKEN_BINARY_SZ) {
      zkErrorSet_va(err,"frTokenStoreToBinary",ZKERR,ZKMTOKEN,ZKEDATABAD,
                    "Buffer too small to store token (should be %d)",
                    TOKEN_BINARY_SZ);
      *size = 0;
      return 0;
    } 
    binary[0] = 0xCA;
    binary[1] = 0xFE;
    binary[2] = (unsigned char) token->version;
    binary[3] = TOKEN_RAW_BINARY_SZ / 0x100;
    binary[4] = TOKEN_RAW_BINARY_SZ % 0x100;
    memcpy(&(binary[5]),token->token,TOKEN_SZ);
    binary[5 + TOKEN_SZ] = token->type_id / 0x100;
    binary[6 + TOKEN_SZ] = token->type_id % 0x100;
    memset(&(binary[7 + TOKEN_SZ]),0,RESERVED_SZ);
    binary[7 + TOKEN_SZ + RESERVED_SZ] = (char) 0;
#if 0
    if (!frCryptSigSignData(err,sig,binary,TOKEN_BINARY_SZ - FRCRYPT_SIG_SIZE,
                            &(binary[TOKEN_BINARY_SZ - FRCRYPT_SIG_SIZE]),
                            FRCRYPT_SIG_SIZE)) {
      memset(binary,0,TOKEN_BINARY_SZ);
      *size = 0;
      return 0;
    }
#endif
    if (!frSecSignData(err,sig,binary,TOKEN_RAW_BINARY_SZ,&sbuf,&sbuf_sz)) {
      memset(binary,0,TOKEN_BINARY_SZ);
      *size = 0;
      return 0;
    }
    memcpy(binary,sbuf,sbuf_sz);
    *size = sbuf_sz;
    free(sbuf);
    return 1;
  }
  zkErrorSet(err,"frTokenStoreToBinary",ZKERR,ZKMTOKEN,ZKENULLPTR,NULL);
  return 0;
}

/*
 * Pull out the signing entity and required key version from the binary
 * version of the token.  This allows you to do a key lookup before
 * converting the token from binary or base64.
 */

DLLExport int
frTokenEntityAndKeyVersionFromBinary (
  ZkError err ,
  void * binary ,
  size_t sz ,
  FrEntIdExp * signer ,
  unsigned int * kv )
{
  static const char * func = "frTokenEntityAndKeyVersionFromBinary";
  int return_code = 0;

  if (binary && signer && kv) {
    if (frSecSigGetEntity(err,binary,sz,signer) &&
        frSecSigGetVersion(err,binary,sz,kv)) {
      return_code = 1;
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMTOKEN,ZKENULLPTR,NULL);
  }
  return return_code;
} 

/*
 * Pull out the signing entity and required key version from the binary
 * version of the token.  This allows you to do a key lookup before
 * converting the token from binary or base64.
 */

DLLExport int
frTokenEntityAndKeyVersionFromBase64 (
  ZkError err ,
  char * base64 ,
  FrEntIdExp * signer ,
  unsigned int * kv )
{
  static const char * func = "frTokenEntityAndKeyVersionFromBase64";
  void * buf = NULL;
  int sz = 0;
  int size = 0; /* Size of the base64 token string */
  int return_code = 0;

  if (base64 && signer && kv) {
    size = strlen(base64);
    if ((buf = malloc(sizeof(char) * size)) == NULL) {
      zkErrorSet(err,"frTokenNewFromBase64",ZKERR,ZKMTOKEN,ZKENOMEM,"malloc");
    } else if (zkConvertFromBase64(err,(unsigned char *)base64,buf,&sz)) {
      return_code = frTokenEntityAndKeyVersionFromBinary(err,buf,sz,signer,kv);
    }
    if (buf) {
      free(buf);
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMTOKEN,ZKENULLPTR,NULL);
  }
  return return_code;
} 
