/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKDEFS_H_
#define ZKDEFS_H_

#include <string.h>

/*
 *  Macro definitions of general interest.
 */

#define ZK_MAX(a, b) ((a) > (b) ? (a) : (b))
#define ZK_MIN(a, b) ((a) < (b) ? (a) : (b))


/*
** arbitrary values associate with the ZkBool type 
*/
#ifndef ZK_FALSE
# define ZK_FALSE               0 /* ZERO mean false */
#endif
#ifndef ZK_TRUE
# define ZK_TRUE                1 /* consider that NON ZERO mean true */
#endif



/* funcname and ZKMOD must be defined */
#ifndef ZKMOD
#define ZKMOD ZKMOD_IS_NOT_DEFINED_FOR_ZKMALLOC
#endif
/*
 * p = (char *) malloc (sizeof(char))
 *
 * zkMALLOC(p,char*,(sizeof(char))
 *
 */
#define zkMALLOC(dest,cast,size) \
{ \
    if ((dest = (cast) malloc (size)) == NULL) { \
      zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENOMEM,"Can't malloc() memory"); \
      return 0; \
    } \
    memset(dest,0,size);\
}

# define NEW(x) (unsigned char *)malloc(x)


#define ZKERRDUMP(err) {\
  ZkLog log; \
  if (!(zkErrorLog (err, &log) && zkLogSetStream (log, stderr) \
      && zkLogEnable (log))) { \
    fprintf(stderr, "Can't setup error reporting system\n"); \
  } else {\
    zkLogError(log, err); \
    zkLogDisable(log); \
  } \
}

#define ZKERRFATAL(err) {\
  ZkLog log; \
  if (!(zkErrorLog (err, &log) && zkLogSetStream (log, stderr) \
      && zkLogEnable (log))) { \
    fprintf(stderr, "Can't setup error reporting system\n"); \
  } else { \
    zkLogError(log, err);\
    zkLogDisable(log); \
    exit (1);\
  } \
}



/*
 *  Define two macros to allow GNU C to do format argument checking.  That is,
 *  variable argument functions that take printf or scanf-like argument lists
 *  can have their arguments checked.
 *  
 *  The first integer argument is the position, counting from 1, of the format
 *  string; the second integer argument is the position of the first argument
 *  corresponding to a %-string in the format string.  Typically, the second
 *  argument will be the value of the first, plus 1.
 */

#ifdef __GNUC__

#define ZK_PRINTF_LIKE(fpos, apos) __attribute__ ((format (printf, fpos, apos)))
#define ZK_SCANF_LIKE(fpos, apos) __attribute__ ((format (scanf, fpos, apos)))

#else

#define ZK_PRINTF_LIKE(fpos, apos)
#define ZK_SCANFF_LIKE(fpos, apos)

#endif

#endif
