/*
 *  $Id: zkIntTypes.h,v 1.11.10.3 2001/01/17 18:02:05 corey Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  zkIntTypes.h
 */ 

#ifndef _ZKINTTYPES_H
#define _ZKINTTYPES_H

#ifndef _ZKINTLIMITS_H
#include <zkIntLimits.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#if defined(ZK_SOLARIS)
  
  #define HAVE_INTTYPES 1
  
#elif defined(__linux)

  /*
   *  RedHat 5.2 doesn't have inttypes.h, but 6.1 does.  Under Linux we'll
   *  assume its existence depends on the version of the GNU C compiler.
   *  
   *  bug: what's a better way to do this?
   */
  
  #ifndef __GNUC__
  #error "expecting Linux to use GNU C"
  #endif

  #if __GNUC__ >= 2 && __GNUC_MINOR__ >= 91
  #define HAVE_INTTYPES 1
  #else
  #define HAVE_INTTYPES 0
  #endif
  
#else
  
  #define HAVE_INTTYPES 0
  
#endif


#if 0

  /*
   *  Typedefs similar to the following appear in the new C standard, but
   *  currently don't exist on all systems.  Define the ones that are missing.
   *  
   *  This is sample code only; don't `#if 1' it.
   */
  
  typedef signed char int8_t;
  typedef unsigned char uint8_t;

  typedef short int int16_t;
  typedef unsigned short uint16_t;

  typedef int int32_t;
  typedef unsigned int uint32_t;

  typedef long long int int64_t;
  typedef unsigned long long uint64_t;
  
  typedef signed char int_least8_t;
  typedef unsigned char uint_least8_t;
  
  typedef short int int_least16_t;
  typedef unsigned short int uint_least16_t;
  
  typedef int int_least32_t;
  typedef unsigned int uint_least32_t;

  typedef long long int_least64_t;
  typedef unsigned long long uint_least64_t;

  typedef int64_t intmax_t;
  typedef uint64_t uintmax_t;
  
#endif

#if HAVE_INTTYPES
  /*
   *  This header file is required by the latest version of the C standard.
   */

#include <inttypes.h>

#define  UINT64_T uint64_t

#else
#ifdef WIN32
  
	/*
   *  Define unsigned types of specific sizes.  The #if's are used to ensure
   *  the code won't compile if the sizes are not what we're expecting.
   */
	#include <limits.h>
	#include <winsock2.h>

	typedef signed char int8_t;
	#if CHAR_MAX != 127
	#error "the size of this type is not what we expect!"
	#endif
	#if CHAR_BIT != 8
	#error "we expect chars to be eight bits!"
	#endif

	typedef UCHAR  uint8_t;
	#if UCHAR_MAX != 0xff
	#error "the size of this type is not what we expect!"
	#endif

	typedef SHORT int16_t;
	#if SHRT_MAX != 0x7fff
	#error "the size of this type is not what we expect!"
	#endif

	typedef USHORT uint16_t; 
	#if USHRT_MAX != 0xffff
	#error "the size of this type is not what we expect!"
	#endif

	typedef INT int32_t;
	#if INT_MAX != 0x7fffffff
	#error "the size of this type is not what we expect!"
	#endif

	typedef UINT uint32_t;
	#if UINT_MAX != 0xffffffff
	#error "the size of this type is not what we expect!"
	#endif

	// The following type is guaranteed to be signed and 64 bits wide.
	typedef LONG64 int64_t;


	// The following type is guaranteed to be unsigned and 64 bits wide.
	typedef ULONG64 UINT64_T;
	typedef ULONG64 uint64_t;


	#define LONG_LONG_FTM	"%I64d"
	#define LONG_LONG		__int64

	#define UINT32_MAX	UINT_MAX

#endif	/* WIN32 */

#if defined(__linux)
  
  /*
   *
   *
   *                                Linux
   *
   *
   */

#include <sys/types.h>

#define UINT64_T unsigned long long int

  /*
   *  In the new C standard, the following preprocessor definitions appear in
   *  inttypes.h.
   */
  
#define INT8_MAX        (127)
#define INT16_MAX       (32767)
#define INT32_MAX       (2147483647)
#define UINT8_MAX       (255U)
#define UINT16_MAX      (65535U)
#define UINT32_MAX      (4294967295U)

#ifdef __GNUC__
#define INT64_MAX       (9223372036854775807)
#define UINT64_MAX      (18446744073709551615U)
#endif

#ifdef INT64_MAX
#define INTMAX_MAX      INT64_MAX
#else
#define INTMAX_MAX      INT32_MAX
#endif

#ifdef UINT64_MAX
#define UINTMAX_MAX     UINT64_MAX
#else
#define UINTMAX_MAX     UINT32_MAX
#endif

#define INT_LEAST8_MAX  INT8_MAX
#define INT_LEAST16_MAX INT16_MAX
#define INT_LEAST32_MAX INT32_MAX
#ifdef INT64_MAX
#define INT_LEAST64_MAX INT64_MAX
#endif

/*
 * The following four defines are currently missing from the standard and
 * that is assumed to be an oversight.
 */
#define UINT_LEAST8_MAX UINT8_MAX
#define UINT_LEAST16_MAX UINT16_MAX
#define UINT_LEAST32_MAX UINT32_MAX
#ifdef UINT64_MAX
#define UINT_LEAST64_MAX UINT64_MAX
#endif

/*
 * The following 2 macros are provided for testing whether the types
 * intptr_t and uintptr_t (integers large enough to hold a void *) are
 * defined in this header. They are needed in case the architecture can't
 * represent a pointer in any standard integral type.
 */
#define INTPTR_MAX
#define UINTPTR_MAX

#define INT8_MIN        (-128)
#define INT16_MIN       (-32767-1)
#define INT32_MIN       (-2147483647-1)
#ifdef __GNUC__
#define INT64_MIN       (-9223372036854775807-1)
#endif

#ifdef INT64_MIN
#define INTMAX_MIN      INT64_MIN
#else
#define INTMAX_MIN      INT32_MIN
#endif

#define INT_LEAST8_MIN  INT8_MIN
#define INT_LEAST16_MIN INT16_MIN
#define INT_LEAST32_MIN INT32_MIN
#ifdef INT64_MIN
#define INT_LEAST64_MIN INT64_MIN
#endif

  
  typedef unsigned char uint8_t;

  typedef unsigned short uint16_t;

  typedef unsigned int uint32_t;

  typedef unsigned long long uint64_t;
  
  typedef signed char int_least8_t;
  typedef unsigned char uint_least8_t;
  
  typedef short int int_least16_t;
  typedef unsigned short int uint_least16_t;
  
  typedef int int_least32_t;
  typedef unsigned int uint_least32_t;

  typedef long long int_least64_t;
  typedef unsigned long long uint_least64_t;

  typedef int64_t intmax_t;
  typedef uint64_t uintmax_t;

#endif	/* #ifdef __linux */
#endif /* HAVE_INTTYPES */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _ZKINTTYPES_H */
