/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
 * zkrev.h by Nicolas Roy
 *
 * The purpose of this file is to create a persistant symbol into
 * the object file that will represent the RCS revision ID of the file
 *
 * Here is an example of how to use it.
 *
 * i.e:  At the top of my source file
 * 
 * #define   REV_ID "@(#) $Id: zkrev.h,v 1.1.2.1 2001/01/18 15:06:03 nicolas Exp $"
 * #include <zkrcs.h>
 *
 * Then use the command ident to dump the revision id:
 *
 * i.e: ident mybin
 *
 */
#ifndef ZKREVISION_ID_H
# define ZKREVISION_ID_H

# if defined( REV_ID )
	
/* define the persistant symbol for each platform */
#  if defined( WIN32 )
#   pragma optimize("", off)  /* turn off optimization */
#   pragma warning( push )
#   pragma warning( disable : 4101 ) /* remove unuse warninig */
static const char * __revision_id = REV_ID;
#   pragma warning( pop )     /* recover warning state */
#   pragma optimize("", on)	  /* recover original optimization state */
	
#  elif defined( ZK_LINUX )
static const __attribute((unused)) char *__revision_id = REV_ID;
	
#  else
static const char *__revision_id = REV_ID;
#  endif

#  undef REV_ID
# endif /* REV_ID */

#endif /* ZKREVISION_ID_H */

