/* $Id: zks.h,v 1.70.8.1 2000/10/11 13:19:55 phil Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_H_
#define ZKS_H_

/*======================= COMMON DEFINES ========================*/


/*
 *  These macros support the Zero-Knowledge coding guidelines.  They are used
 *  to indicate how function parameters are used.
 */

#define ZK_IN
#define ZK_INOUT
#define ZK_OUT

typedef enum ConsSeverity { 
  CONS_VERY_IMPORTANT	=0, /* Perhaps will provoke a beep, message box or other strong indication */
  CONS_IMPORTANT	=1, /* Appears on the main UI */
  CONS_NORMAL		=2, /* Appears in the console only */
  CONS_UI_NOT_CONSOLE   =3, /* A transient message which appears on status
                               bar but does not pollute the console 
                               used to update or build a message on the screen */
  CONS_VERBOSE		=4, /* Once we have a VERBOSE preference, these message
                               will only appears when it is set */
  CONS_DEBUG		=5  /* Appears only in internal version */
} ConsSeverity;


#ifdef WIN32
/*======================= WIN 32 Specific =======================*/
#ifdef __cplusplus

#ifndef _CPPUNWIND
#error("Error, you need to enable exception handling for Windows!")
#endif

#ifndef _CPPRTTI
#error("Error, you need to enable RTTI for Windows!")
#endif

#endif /* __cplusplus */

#define ZK_LITTLE_ENDIAN

#include <winsock2.h>
#include <windows.h>
#include <process.h>


/* These macros make using 
 #pragma message(Reminder "whatever")
 fun to use under MSVC
*/
#define Stringize( L )		#L
#define MakeString( M, L )	M(L)
#define $LINE	MakeString(Stringize, __LINE__)
#define Reminder __FILE__	"("$LINE") : Reminder: "



#ifndef __cplusplus /* This is a keyword in C++, please abstain from using it */
	typedef BOOL bool;
#endif

/*  Case insensitive */
#define strcasecmp	_stricmp
#define strncasecmp	_strnicmp
#define fchmod		_chmod
#define snprintf	_snprintf
#define vsnprintf	_vsnprintf
typedef	long		ssize_t;
typedef	long		uid_t;
typedef unsigned int mode_t;

#define strerror	zksstrerror
#define _strerror	zksstrerror


#define geteuid()	0

#define	S_IRUSR		S_IREAD
#define S_IWUSR		S_IWRITE
#define S_IRWXU		(S_IREAD|S_IWRITE|S_IEXEC)

// Read, write, and execute by group. Crypto routines fail when Unix bit is set, so for Win32 we never set them
#define S_IRWXG 0
// Read, write, and execute by others. Crypto routines fail when Unix bit is set, so for Win32 we never set them
#define S_IRWXO 0

#define __S_ISTYPE(mode, mask) (((mode) & S_IFMT) == (mask))
#define S_ISREG(mode) __S_ISTYPE((mode), S_IFREG)

/* 
For Windows 95, I'll return false, meaning it's not symbolic link
As a note for future reference, NTFS has hard links to files (not directories)
*/
#define S_ISLNK(filebitmask)	0

#define lstat	fstat

typedef struct sockaddr_un
{
	unsigned short sun_family;	/* AF_UNIX */
	char	sun_path[MAX_PATH];
} sockaddr_un;

#define SOL_TCP IPPROTO_TCP

// 4 octets uniquely identifying thread throughout the system
#define ZKGetCurrentThreadId	GetCurrentThreadId

// 4 octets value indicating number of milliseconds elapsed since system was started
#define ZKGetTickCount	GetTickCount

#ifdef _WINDLL
#define DLLExport __declspec(dllexport)
#else
#define DLLExport __declspec(dllimport)
#endif

#define DLLImport __declspec(dllimport)



#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*  inet_aton doesn't exist under Windows, I'll have to write it, see zks.c 
 * Returns 0 on error
 * Returns 1 on success
 */
DLLExport extern BOOL inet_aton(LPCSTR pStr, IN_ADDR* InetAddr);

/* gettimeofday doesn't exist under Windows */
DLLExport int gettimeofday(struct timeval*, void*);

/* Alot of the Unix DLLs print to stderr. This function can be used inside those DLLs, which aren't using MFC,
 * to trace to the debug window.
 */ 
DLLExport void ClientTrace(const char*, const char*);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#else /* WIN32 */
/*======================= Unix Specific ===============================*/
#define DLLExport
#define DLLImport
#define ClientTrace(a,b)
#define __TIMESTAMP__ __TIME__
typedef unsigned short USHORT;

#define U_LONG_LONG_FMT	"%Lu"
#define LONG_LONG_FMT	"%Ld"
/* The following is to support an old typo. */
#define LONG_LONG_FTM LONG_LONG_FMT
#define LONG_LONG		long long

#define stricmp strcasecmp
#define	strnicmp strncasecmp
#define _stricmp strcasecmp
#define	_strnicmp strncasecmp

#endif /* ! WIN32 */

/*======================= WIN 32 And Unix Stuff =======================*/

#define FR_WINDOWSCLIENT_VERSION "0.5"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
