/*
 *  $Id: zkClock.h,v 1.5.20.1 2000/10/11 13:19:59 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  A ZkClock is a quasi-opaque type, representing the number of 
 *  seconds elapsed since January 1, 1970, UTC.  The range of a 
 *  ZkClock is that of a signed 64-bit integer.
 *  
 *  Important: NEVER assign a time_t directly to a ZkClock or vice-
 *  versa.  The encoding of time_t is implementation-defined, so 
 *  such an assignment is inherently non-portable.  Use the 
 *  conversion functions zkClockToTime() and zkClockFromTime() 
 *  instead.
 */

#ifndef ZKCLOCK_H_
#define ZKCLOCK_H_


#include "zks.h"

#include <sys/types.h>
#include <time.h>
#include "zkError.h"
#include "zkIntTypes.h"


typedef struct {
  int64_t rep;
} ZkClock;


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* This macro can be used to statically initialise a ZkClock 
   object to a given number of seconds */
#define ZKCLOCK_INIT(secs)	{ ((int64_t) (secs)) }

  extern DLLExport ZkClock zkClockFromNbo(ZkClock n);
  extern DLLExport ZkClock zkClockToNbo(ZkClock n);
  extern DLLExport ZkClock zkClockNow(void);
  extern DLLExport ZkClock zkClockDiffSat(ZkClock a, ZkClock b);
  extern DLLExport ZkClock zkClockMaxPos(void);
  extern DLLExport size_t zkClockStrftime(char *buf, size_t bsz, const char *fmt, ZkClock n);
  extern DLLExport time_t zkClockToTime(ZkClock n);
  extern DLLExport ZkClock zkClockFromTime(time_t t);
  extern DLLExport ZkClock zkClockFromSeconds(int64_t secs);
  extern DLLExport int64_t zkClockToSeconds(ZkClock n);
  extern DLLExport int zkClockCmp(ZkClock c1, ZkClock c2);
  extern DLLExport ZkClock zkClockAdd(ZkClock a, ZkClock b);

#if defined(macintosh)
  extern DLLExport void zkClockFlushUTCCache(void);
#endif

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZKCLOCK_H_ */
