/*
 * zkConfFile.h
 * $Id: zkConfProto.h,v 1.2.32.1 2000/10/11 13:19:59 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 * 
 */

#ifndef ZKCONFPROTO_H_
#define ZKCONFPROTO_H_
#include <zkMemPool.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define ZKCP_REQ 0
#define ZKCP_OPT 1

#define ZKCP_SINGLE   2
#define ZKCP_MULTIPLE 3
  
  typedef int (* ZkConfCvtFn)(const char *, void *, ZkMemPool);  
  typedef int (* ZkConfAllocFn)(const char *, void *, void **, ZkMemPool);  
  
  struct T_ZkConfProtoSection {
    const char *name;
    ZkConfCvtFn cvtFunc;
    int optional;
    void *variable;
    const char *dflt;
  };
  
  typedef struct T_ZkConfProtoSection *ZkConfProtoSection;
  
  struct T_ZkConfProtoS {
    const char *name;
    ZkConfProtoSection section;
    int numsect;    /* The number of generic sections - set internally */
    ZkConfAllocFn alloc;
    int   type;
  };
  
  typedef struct T_ZkConfProtoS *ZkConfProtoS;
  
  extern int zkCvtBool(const char *, void *, ZkMemPool);
  extern int zkCvtInt(const char *, void *, ZkMemPool);
  extern int zkCvtStr(const char *, void *, ZkMemPool);
  extern int zkCvtZkTime(const char *s, void *n, ZkMemPool mpool);
  
  DLLExport extern int zkConfProtoLoadFromBuffer (ZkError,
					     char *,
					     size_t,
					     ZkConfProtoS,
					     ZkMemPool);
  
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZKCONFPROTO_H_ */
