/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef _ZKCONFIG_H
#define _ZKCONFIG_H

#ifndef _ZKINTTYPES_H
#include <zkIntTypes.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "zkIntTypes.h"
#ifndef _ZKLOG_H
#include <zkLog.h>
#endif /* _ZKLOG_H */

#include <zkIntTypes.h>

/*
 * Format
 * 
 * FFFFFFTT where TT is the type
 */

#define TYPE_UINT8    0x1           /* uint8_t */
#define TYPE_UINT16   0x2           /* uint16_t */
#define TYPE_UINT32   0x3           /* uint32_t */
#define TYPE_INT8     0x4           /* int8_t */
#define TYPE_INT16    0x5          /* int16_t */
#define TYPE_INT32    0x6          /* int32_t */
#define TYPE_BOOLEAN  0x7      /* parameter is of type uint8_t */
#define TYPE_STRING   0x8          /* const char * */

typedef struct T_ZkConfig * ZkConfig;

DLLExport int zkConfigNew (
  ZkError ,                           /* Returns the error if any occured */
  ZkConfig * );                       /* Returns a pointer to a new ZkConfig */

DLLExport int zkConfigFree (
  ZkError ,                           /* Returns the error if any occured */
  ZkConfig * );                       /* Destroys the ZkConfig */

DLLExport int zkConfigLoad (
  ZkError ,                           /* Returns the error if any occured */
  ZkConfig ,                          /* ZkConfig which will hold the file */
  const char * );                           /* Config file name to load */

DLLExport int zkConfigSave (
  ZkError ,                           /* Returns the error if any occured */
  ZkConfig ,                          /* ZkConfig to save as a file */
  const char * );                     /* Filename to save the configuration */

DLLExport int zkConfigSet (
  ZkError ,                           /* Returns the error if any occured */
  ZkConfig ,
  const char * ,                       /* Name of the section */
  const char * ,                       /* Comment for the section/parameter */
  const char * ,                       /* Name of the parameter */
  const char * );                      /* Value of the parameter */

DLLExport int zkConfigUnset (
  ZkError ,                           /* Returns the error if any occured */
  ZkConfig ,
  const char * ,                            /* Name of the section */
  const char * );                           /* Name of the parameter */

DLLExport int zkConfigReset (
  ZkError ,                           /* Returns the error if any occured */
  ZkConfig ,
  const char * ,                      /* Name of the section */
  const char * ,                      /* Name of the parameter */ 
  const char * );                     /* parameter value */


DLLExport int zkConfigGet (
  ZkError ,                           /* Returns the error if any occured */
  ZkConfig ,
  const char * ,                      /* Name of the section */
  const char * ,                      /* Name of the parameter */ 
  const char ** );                    /* Returns the parameter value */

DLLExport int zkConfigGetNext (
  ZkError err ,                       /* Returns the error if any occured */
  ZkConfig ,                          /* Configuration object to scan */
  const char ** );                    /* Returns string pointer to the value */

DLLExport int zkConfigSections (
  ZkError ,                           /* Returns the error if any occured */
  ZkConfig ,
  size_t * );                         /* Returns the number of sections */

DLLExport int zkConfigSection (
  ZkError ,                           /* Returns the error if any occured */
  ZkConfig ,
  size_t ,                            /* Section index */
  const char ** );                    /* Returns a section name */ 

DLLExport int zkConfigGetPairReset (
  ZkError ,
  ZkConfig ,
  const char * );

DLLExport int zkConfigLoadFromBuffer(
  ZkError ,
  ZkConfig ,
  const char * ,
  size_t );
              
DLLExport int zkConfigToBuffer (
  ZkError ,
  ZkConfig ,
  void ** ,
  size_t * );

DLLExport int zkConfigParseLine (
  ZkError ,
  ZkConfig ,
  char * ,
  char * ,              /* Section name */
  int );


DLLExport int zkConfigGetPair ( 
  ZkError ,
  ZkConfig ,
  const char * ,
  const char ** ,
  const char ** );


DLLExport int zkConfigSetCmdParms (
  ZkError ,                       /* Returns the error if one occured */
  ZkConfig ,                      /* ZkConfig object to update */
  int ,                           /* argc from main */
  const char ** );                /* argv from main */

/*
 *  Say you want to retrieve an integer from the configuration:
 *
 *  int count
 *
 *  zkConfigGetParameter(err,config,TYPE_INT16,"foo","count",&count)
 *
 */

DLLExport int zkConfigGetParameter (
  ZkError ,                     /* return the error if one occured */
  ZkConfig ,                    /* Configuration object to scan */
  uint32_t ,                    /* Flags and value types (see above) */
  const char * ,                /* Section name */
  const char *  ,               /* Parameter name */
  void * );                      /* value pointer */


DLLExport int
zkConfigSectionIndexByName (
  ZkError ,
  ZkConfig ,
  const char *  ,
  size_t * );

DLLExport int zkConfigSectionToBuffer (
  ZkError ,              /* Return the error if one occured */
  ZkConfig  ,            /* Configuration object to search */
  const char * ,         /* Requested section name */
  void **  ,             /* Allocates and returns a section text buffer */
  size_t * );            /* Size of the returned buffer */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _ZKCONFIG_H */
