/*
 * zkErrdef.h
 * Wednesday, June 24, 1998
 * $Id: zkErrdef.h,v 1.95.8.1 2000/10/11 13:19:59 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Philippe Boucher (philippe@zks.net)

 */

#ifndef _ZKERRDEF_H
#define  _ZKERRDEF_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * Error code format:
 *  
 * bits 31 - 28    Message type (4 bits)
 *          ZKERROR
 *                        ZKWARNING
 * bits 27 - 24    Reserved (4 bits)
 * bits 23 - 16    Module ID (8 bits)
 *         ZKLOG  
 *         ZKERROR
 *        ...
 * bits 15 - 00    Message ID (16 bits)
 *          ENOMEM
 *         EAGAIN
 *         ...
 */

/*
 * Message types
 */

#define ZKERR           0x01            /* Error message type */
#define ZKWARN          0x02            /* Warning message type */
#define ZKURG           0x03            /* Urgent error message */
#define ZKINFO          0x04            /* Informational message */

/*
 * Module / object numbers 
 */

#define ZKMLOG          0x01            /* zkLog (standard ZK library) */
#define ZKMERROR        0x02            /* zkError (standard ZK library) */
#define ZKMZKIP         0x03            /* ZKIP protocol engine */
#define ZKIP            ZKMZKIP         /* old version */
#define ZKMSTRING       0x04            /* zkString (standard ZK library) */
#define ZKMBASE64       0x05            /* Base64 (standard ZK library) */
#define ZKMAIP          0x06            /* AIP (standard ZK library) */
#define ZKMACI          0x07            /* ACI (standard ZK library) */
#define ZKMQUEUE        0x08            /* Queue (standard ZK library) */
#define ZKMHASH         0x09            /* Hash table (standard ZK library) */
#define ZKMCONFIG       0x0A            /* Configuration data (std ZK library) */
#define ZKMFRPKTFMT     0x0B            /* Freedom packet format library */
#define ZKMFRCRYPT      0x0C            /* Freedom crypto library */
#define	ZKMWORMHOLE     0x0D            /* Freedom wormhole */
#define	ZKMANONROUTE    0x0E            /* Freedom Anonymous Routing Module */
#define ZKMEMAIL        0x0F            /* ZkEmail... (MIME support library) */
#define ZKMFRLAIP       0x10            /* Freedom Local AIP library */
#define ZKMRNYM         0x11            /* Remote nym support (frNym lib) */
#define ZKMALLOC        0x12            /* memory allocation and manipulation */
#define ZKMMEMBUF       0x13            /* memory buffer */
#define ZKMDB           0x14            /* Zk Database */
#define ZKMFRAIPS       0x15            /* Freedom AIP library (LAIP, AIP) */
#define ZKMSECURE       0x16            /* Secure functions */
#define ZKMFRSEC        0x17            /* Freedom-specific security functions */
#define ZKMSHIM         0x18            /* Shim Library */
#define ZKMHTTP         0x19            /* ZkHTTP structs (zkMIME library) */
#define ZKMNDB          0x1A            /* Network Database */
#define ZKMNNP          0x1B            /* Nym Network Preferences */
#define ZKMDTSTAMP      0x1C            /* DTstamp functions in libzk */
#define ZKMLIST         0x1D            /* ZkList (standard zk library) */
#define ZKMFILE         0x1E            /* File operations */
#define ZKMKQD          0x1F            /* Freedom UDP key query daemon */
#define ZKMLAIP         0x20            /* The LocalAIP */
#define ZKMDUMP         0x21            /* Standard buffer dump functions */
#define ZKMKEY          0x22            /* Key update server client functions */
#define ZKMDNS          0x23            /* Name lookup functions (libzk) */
#define ZKMDATETIME     0x24            /* DateTime functions (libzk) */
#define ZKMKEYUTILS     0x25            /* Public key utility programs */
#define ZKMNIDB         0x26            /* Network information database */
#define ZKMFRKEYQRY     0x27            /* Public key queries */
#define ZKMDIR          0x28            /* Directory operations */
#define ZKMSHELL        0x29            /* Simple command line shell */
#define ZKMTOKEN        0x2A            /* Token and serial number library */
#define ZKMFRMAIP       0x2B            /* Freedom MAIP library */
#define ZKMMAIP         0x2C            /* Freedom MAIP */
#define ZKMRANDSRV      0x2D            /* Random data server */
#define ZKMAMTP         0x2E            /* AMTP client library */
#define ZKMFMG          0x2F            /* Freedom Mail Gateway filter */
#define ZKMM2N          0x30            /* Freedom Mail 2 News Filter */
#define ZKMNNTP         0x31            /* Freedom NNTP Newsgroup server */
#define ZKMMEMPOOL      0x32            /* Memory Pool library */
#define ZKMLIB          0x33            /* General UNIX functions */
#define ZKMFORK         0x34            /* General UNIX functions */
#define ZKMCODE         0x35            /* Functions that produce C code */
#define ZKMFRENT        0x36            /* Entity functions */
#define ZKMXFER         0x37            /* XFER library */
#define ZKMNET          0x38            /* General networking routines */
#define ZKMFRSTAT       0x39            /* Statistics update functions */
#define ZKMECHO         0x3A            /* UDP Echo server */
#define ZKMFRNETINFO    0x3B            /* Freedom NetInfo library */
#define ZKMFRNETROUTE   0x3C            /* Freedom NetRoute library */
#define ZKMGRAPH        0x3D            /* Graph library */
#define ZKMSTATD        0x40            /* Statistics update server */
#define ZKMNISS         0x40            /* Statistics update server-yup same */
#define ZKMNIQS         0x41            /* Network Info Query server */
#define ZKMTOKENCACHE   0x42            /* Token cache for managing tokens */
#define ZKMFRADMIN      0x43            /* Administrative programs, etc. */
#define ZKMQCD          0x44            /* Query Cache daemon */
#define ZKMFRQC         0x45            /* Query cache library */
#define ZKMNIDBTOOL     0x46            /* NIDB administration tool */
#define ZKMFRLAIPCLI    0x47            /* Local aip client library */
#define ZKMPNEWS        0x48            /* Spool/Post news programe */
#define ZKMVHEAP        0x49            /* virtual memory heap manipulator */
#define ZKMSHM          0x4A            /* Shared Memory */
#define ZKMSEM          0x4B            /* Semaphore handling */
#define ZKMIPC          0x4C            /* Inter Processus Communication routines */
#define ZKMAUDIT        0x4D            /* Audit Logging */
#define ZKMSHAREDOBJECT 0x4E            /* Shared Object */
#define ZKMPLUGIN       0x4F            /* Dynamic plugins */
#define ZKMAUTH         0x50            /* Authentication service */
#define ZKMLODE         0x51            /* Administration Lode Server */
#define ZKMALLOCATOR    0x52            /* Generic Memory Allocator */
#define ZKMXML          0x53            /* XML handling */
#define ZKMCLIENTCONSOLE 0x54           /* Client console message callbacks */
#define ZKMSTAT         0x55            /* Generic Stats Calls */
#define ZKMEXPIRYTABLE  0x56            /* Expiry table (libzk) */
#define ZKMNAMETABLE    0x57            /* Name table (libzk) */
#define ZKMFRNYMSDB     0x58            /* nym database access library */
#define ZKMFRPGNYM      0x59            /* Plugin for accessing nym databases */
#define ZKMFRPGKEY      0x5A            /* Plugin for accessing key databases */
#define ZKMDISPATCHER   0x5B            /* Dispatcher module code */
#define ZKMZKCP         0x5C            /* Command Protocol Library */
#define ZKMCMDSRV       0x5D            /* Command Server */
#define ZKMFRKINVPOOL   0x5E            /* To speed up DSA signing */
#define ZKMSOCKET       0x5F            /* Socket wrapper library */
#define ZKMREPLIC       0x60            /* Replicator module code */
#define ZKMDBUTIL       0x61            /* database utilities */
#define ZKMPERF         0x62            /* Performance library (libzkperf) */
#define ZKMNSCPD        0x63            /* nym server */
#define ZKMPTHREAD      0x64            /* pthread handler */
#define ZKMLOCK         0x65            /* Simple lock operations */
#define ZKMDBSRV        0x66            /* ZK Database Server */
#define ZKMMAILACCOUNT  0x67            /* mail account library */
#define ZKMMAILMSG      0x68            /* mail message library */
#define ZKMDBVERIFY     0x69            /* ZkDB verification utility */ 
#define ZKMMAILAUTH     0x70            /* ZK mail authentication library */
#define ZKMMAILBLOCKLIST  0x71          /* ZK mail block list library */
#define ZKMRAND         0x72            /* randomness library */


/*
 * Error codes taken from Linux and Solaris
 */

#define	ZKENOERROR        0x0000    /* Everything is okely dokely */
#define ZKE2BIG           0x0001    /* Arg list too long   */
#define ZKEACCES          0x0002    /* Permission denied */
#define ZKEADDRINUSE      0x0003    /* Address already in use */
#define ZKEADDRNOTAVAIL   0x0004    /* Can't assign requested address */
#define ZKEADV            0x0005    /* Advertise error */
#define ZKEAFNOSUPPORT    0x0006    /* Address family not supported by proto. */
#define ZKEAGAIN          0x0007    /* Resource temporarily unavailable */
#define ZKEALREADY        0x0008    /* operation already in progress */
#define ZKEBADE           0x0009    /* Invalid exchange */
#define ZKEBADF           0x000A    /* Bad file number */
#define ZKEBADFD          0x000B    /* File descriptor in bad state */
#define ZKEBADMSG         0x000C    /* trying to read unreadable message */
#define ZKEBADR           0x000D    /* Invalid request descriptor */
#define ZKEBADRQC         0x000E    /* Invalid request code */
#define ZKEBADSLT         0x000F    /* Invalid slot */
#define ZKEBFONT          0x0010    /* bad font file fmt   */
#define ZKEBUSY           0x0011    /* Mount device busy   */
#define ZKECANCELED       0x0012    /* Operation canceled   */
#define ZKECHILD          0x0013    /* No children    */
#define ZKECHRNG          0x0014    /* Channel number out of range */
#define ZKECOMM           0x0015    /* Communication error on send */
#define ZKECONNABORTED    0x0016    /* Software caused connection abort */
#define ZKECONNREFUSED    0x0017    /* Connection refused */
#define ZKECONNRESET      0x0018    /* Connection reset by peer */
#define ZKEDEADLK         0x0019    /* Resource deadlock would occur */
#define ZKEDEADLOCK       ZKEDEADLK
#define ZKEDESTADDRREQ    0x001A    /* Destination address required */
#define ZKEDOM            0x001B    /* Math argument out of domain of func */
#define ZKEDOTDOT         0x001C    /* RFS specific error */
#define ZKEDQUOT          0x001D    /* Disc quota exceeded   */
#define ZKEEXIST          0x001E    /* File exists */
#define ZKEFAULT          0x001F    /* Bad address */
#define ZKEFBIG           0x0020    /* File too large */
#define ZKEHOSTDOWN       0x0021    /* Host is down */
#define ZKEHOSTUNREACH    0x0022    /* No route to host */
#define ZKEIDRM           0x0023    /* Identifier removed */
#define ZKEILSEQ          0x0024    /* Illegal byte sequence */
#define ZKEINPROGRESS     0x0025    /* Operation now in progress */
#define ZKEINTR           0x0026    /* Interrupted system call */
#define ZKEINVAL          0x0027    /* Invalid argument */
#define ZKEIO             0x0028    /* I/O error */
#define ZKEISCONN         0x0029    /* Socket is already connected */
#define ZKEISDIR          0x002A    /* Is a directory */
#define ZKEISNAM          0x002B    /* Is a named type file */
#define ZKEL2HLT          0x002C    /* Level 2 halted */
#define ZKEL2NSYNC        0x002D    /* Level 2 not synchronized */
#define ZKEL3HLT          0x002E    /* Level 3 halted */
#define ZKEL3RST          0x002F    /* Level 3 reset */
#define ZKELIBACC         0x0030    /* Can not access a needed shared library */
#define ZKELIBBAD         0x0031    /* Accessing a corrupted shared library */
#define ZKELIBEXEC        0x0032    /* Cannot exec a shared library directly */
#define ZKELIBMAX         0x0033    /* Attempting to link in too many libs. */
#define ZKELIBSCN         0x0034    /* .lib section in a.out corrupted. */
#define ZKELNRNG          0x0035    /* Link number out of range */
#define ZKELOOP           0x0036    /* Too many symbolic links encountered */
#define ZKEMEDIUMTYPE     0x0037    /* Wrong medium type */
#define ZKEMFILE          0x0038    /* Too many open files */
#define ZKEMLINK          0x0039    /* Too many links */
#define ZKEMSGSIZE        0x003A    /* Message too long */
#define ZKEMULTIHOP       0x003B    /* Multihop attempted */
#define ZKENAMETOOLONG    0x003C    /* File name too long */
#define ZKENAVAIL         0x003D    /* No XENIX semaphores available */
#define ZKENETDOWN        0x003E    /* Network is down */
#define ZKENETRESET       0x003F    /* Net. dropped conn. because of reset */
#define ZKENETUNREACH     0x0040    /* Network is unreachable */
#define ZKENFILE          0x0041    /* File table overflow */
#define ZKENOANO          0x0042    /* No anode */
#define ZKENOBUFS         0x0043    /* No buffer space available */
#define ZKENOCSI          0x0044    /* No CSI structure available */
#define ZKENODATA         0x0045    /* No data available */
#define ZKENODEV          0x0046    /* No such device */
#define ZKENOENT          0x0047    /* No such file or directory */
#define ZKENOEXEC         0x0048    /* Exec format error */
#define ZKENOLCK          0x0049    /* No record locks available */
#define ZKENOLINK         0x004A    /* Link has been severed */
#define ZKENOMEDIUM       0x004B    /* No medium found */
#define ZKENOMEM          0x004C    /* Out of memory */
#define ZKENOMSG          0x004D    /* No message of desired type */
#define ZKENONET          0x004E    /* Machine is not on the network */
#define ZKENOPKG          0x004F    /* Package not installed */
#define ZKENOPROTOOPT     0x0050    /* Protocol not available */
#define ZKENOSPC          0x0052    /* No space left on device  */
#define ZKENOSR           0x0053    /* Out of streams resources */
#define ZKENOSTR          0x0054    /* Device not a stream */
#define ZKENOSYS          0x0055    /* Function not implemented */
#define ZKENOTBLK         0x0056    /* Block device required */
#define ZKENOTCONN        0x0057    /* Socket is not connected */
#define ZKENOTDIR         0x0058    /* Not a directory */
#define ZKENOTEMPTY       0x0059    /* Directory not empty */
#define ZKENOTNAM         0x005A    /* Not a XENIX named type file */
#define ZKENOTSOCK        0x005B    /* Socket operation on non-socket */
#define ZKENOTSUP         0x005C    /* Operation not supported  */
#define ZKENOTTY          0x005D    /* Not a typewriter */
#define ZKENOTUNIQ        0x005E    /* Name not unique on network */
#define ZKENXIO           0x005F    /* No such device or address */
#define ZKEOPNOTSUPP      0x0060    /* Operation not supported on socket */
#define ZKEOVERFLOW       0x0061    /* Value too large for defined data type */
#define ZKEPERM           0x0062    /* Operation not permitted */
#define ZKEPFNOSUPPORT    0x0063    /* Protocol family not supported */
#define ZKEPIPE           0x0064    /* Broken pipe */
#define ZKEPROTO          0x0065    /* Protocol error */
#define ZKEPROTONOSUPPORT 0x0066    /* Protocol not supported */
#define ZKEPROTOTYPE      0x0067    /* Protocol wrong type for socket */
#define ZKERANGE          0x0068    /* Math result not representable */
#define ZKEREMCHG         0x0069    /* Remote address changed */
#define ZKEREMOTE         0x006A    /* Object is remote */
#define ZKEREMOTEIO       0x006B    /* Remote I/O error */
#define ZKERESTART        0x006C    /* Int. sys. call should be restarted */
#define ZKEROFS           0x006D    /* Read-only file system */
#define ZKESHUTDOWN       0x006E    /* Can't send after socket shutdown */
#define ZKESOCKTNOSUPPORT 0x006F    /* Socket type not supported */
#define ZKESPIPE          0x0070    /* Illegal seek */
#define ZKESRCH           0x0071    /* No such process */
#define ZKESRMNT          0x0072    /* Srmount error */
#define ZKESTALE          0x0073    /* Stale NFS file handle */
#define ZKESTRPIPE        0x0074    /* Streams pipe error */
#define ZKETIME           0x0075    /* Timer expired */
#define ZKETIMEDOUT       0x0076    /* Connection timed out */
#define ZKETOOMANYREFS    0x0077    /* Too many references: cannot splice */
#define ZKETXTBSY         0x0078    /* Text file busy */
#define ZKEUCLEAN         0x0079    /* Structure needs cleaning */
#define ZKEUNATCH         0x007A    /* Protocol driver not attached */
#define ZKEUSERS          0x007B    /* Too many users */
#define ZKEWOULDBLOCK     ZKEAGAIN  /* Operation would block */
#define ZKEXDEV           0x007C    /* Cross-device link */
#define ZKEXFULL          0x007D    /* Exchange full */

/*
 * Zero Knowledge specific error codes 
 */


#define	ZKERROR         0x0A00          /* Generic error for generic problems */
#define ZKENULLPTR      0x0A03          /* Null pointer parameter */
#define ZKEPARSE        0x0A04          /* Parser error */
#define ZKENOTFOUND     0x0A05          /* requested item not found */
#define ZKEINCOMPLETE   0x0A06          /* Incomplete data received */ 
#define ZKETOOLONG      0x0A07          /* Too much data received */
#define ZKEREAD         0x0A08          /* Error reading from a descriptor */
#define ZKESTUB         ZKENOSYS        /* Function not implemented (stub) */
#define ZKEPARMRANGE    0x0A0B          /* Function parm out of range */
#define ZKEPARMBAD      0x0A0C          /* Generic invalid function parameter */
#define	ZKEFILEOPEN     0x0A0D          /* Generic error while opening a file */
#define ZKEBADSTATE     0x0A0E          /* inappropriate state for operation */
#define ZKESOCKET       0x0A0F          /* Generic socket error */
#define ZKEIOCTL        0x0A10          /* Generic ioctl() error */
#define ZKEWRITE        0x0A11          /* Generic write() error */
#define ZKEDATABAD      0x0A12          /* Data was determined to be invalid */
#define ZKEDUP          0x0A13          /* Duplicated parameters */
#define ZKEFULL         0x0A14          /* Data structure is full */
#define ZKECLOSE        0x0A15          /* Generic close() error */
#define ZKEUNLINK       0x0A16          /* Generic unlink() error */
#define ZKEBIND         0x0A17          /* Generic bind() error */
#define ZKEFREE         0x0A18          /* Generic free() error */
#define ZKEFORK         0x0A19          /* Generic fork() error */
#define ZKEEMPTY        0x0A1A          /* Data structure is empty */
#define ZKESIGCK        0x0A1B          /* Signature verification failed */
#define ZKENOADDR       0x0A1C          /* No address (for zkGetHostByName) */
#define ZKENORECOVERY   0x0A1D          /* No recovery (for zkGetHostByName) */
#define ZKEDEFUNCT      0x0A1E          /* The executable has expired */
#define ZKEVERSION      0x0A1F          /* There is a version mismatch */
#define ZKENUMPARAM     0x0A20          /* Wrong number of parameters */
#define ZKENYMNUKED     0x0A21          /* The nym is nuked */
#define ZKERECORDLOCK   0x0B01          /* Record lock. Can't proceed transaction */  
#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* _ZKERRDEF_H */
