/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 *  Philippe Boucher (philippe@zks.net)

 */

/*
 * $id$ 
 *
 *  zkError.h
 *  Wednesday, june 24, 1998
 */

#ifndef _ZKERROR_H
#define _ZKERROR_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <zks.h>
#include <zkIntTypes.h>
#include <zkTypes.h>

#ifndef _ZKERRDEF_H
#include <zkErrdef.h>
#endif /* _ZKERRDEF_H */

#include "zkDefs.h"

#ifndef T_ZKLOG
#define T_ZKLOG
typedef struct T_ZkLog * ZkLog;
#endif /* T_ZKLOG */

typedef struct T_ZkError * ZkError;

#define zkErrorCodeFromErr(err)         ((zkErrorCode(err)) ? zkErrorCode(err) : ZKERROR)

DLLExport int zkErrorNew (
  ZkError * );                          /* Returns a new zkError */

DLLExport int zkErrorFree (
  ZkError * );                          /* Destroys the zkError */

DLLExport int zkErrorClear (
  ZkError );                            /* zkError structure to clear */

DLLExport int zkErrorSet (
  ZkError,                              /* zkError structure to set */
  const char * ,                        /* Function name */
  const unsigned short ,                /* Message type */
  const unsigned short ,                /* Module id code */
  const unsigned short ,                /* Message id code */
  const char * );                       /* Small detail string */

DLLExport int zkErrorSet_va (
  ZkError,                              /* zkError structure to set */
  const char * ,                        /* Function name */
  const unsigned short ,                /* Message type */
  const unsigned short ,                /* Module id code */
  const unsigned short ,                /* Message id code */
  const char * ,                        /* Small detail string */
  ... ) ZK_PRINTF_LIKE(6, 7);           /* Optional arguments */

DLLExport int zkErrorAdd (
  ZkError,                              /* zkError structure to set */
  char * ,				/* Function name */
  unsigned short ,                      /* Message type */
  unsigned short ,                      /* Module id code */
  unsigned short ,                      /* Message id code */
  char * );				/* Small detail string */

DLLExport int zkErrorSystemSet (
  ZkError,                              /* A ZkError object */
  const char *,                         /* String name of the function reporting the error */
  const uint16_t,                       /* Either ZKWARNING or ZKERROR */
  const uint16_t,                       /* Module / library ID (ZKLOG, ZKERROR, ...) */
  const char* );                        /* Optional short detail string */

DLLExport int zkErrorMessage (
  ZkError ,				/* Error from which to get message */
  char * ,				/* Stores the error message */
  int );				/* Max size of the message string */

DLLExport unsigned short zkErrorModule (
  ZkError );				

DLLExport unsigned short zkErrorType (
  ZkError );

DLLExport unsigned short zkErrorCode (
  ZkError );

DLLExport const char * zkErrorFunction (
  ZkError );

DLLExport const char * zkErrorDetail (
  ZkError );

DLLExport int zkErrorSetDetail (
  ZkError ,                               /* Error structure to modify */
  const char * );                         /* Detail string */

DLLExport int zkErrorSetFunction (
  ZkError ,
  const char * );                         /* Name of the function */

DLLExport int zkErrorLog (
  ZkError ,
  ZkLog * );                              /* Returns log assigned to error */

DLLExport int zkErrorSetLoggedFlag (
  ZkError ,
  ZkBool );                               /* New state to be set */

DLLExport int zkErrorGetLoggedFlag (
  ZkError ,
  ZkBool * );                            /* return TRUE if flag on */


DLLExport void zkErrorCtxInit (
  ZkError err,
  ZkLog   log );

DLLExport void zkErrorCtxDestroy (
  ZkError err );

DLLExport void zkErrorCtxClear (
  ZkError err );


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _ZKERROR_H */
