/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKEXPIRYTABLE_H_
#define ZKEXPIRYTABLE_H_

#ifndef _ZKERROR_H
#include <zkError.h>
#endif /* _ZKERROR_H */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct T_ZkExpiryTable * ZkExpiryTable; /* The expiry table structure */

DLLExport int zkExpiryTableNew (
  ZkError ,               /* Returns the error if any occured */
  ZkExpiryTable * );      /* Returns a new zk expiry table */

DLLExport int zkExpiryTableFree (
  ZkError ,               /* Returns the error if any occured */
  ZkExpiryTable );        /* Expiry table to destroy */

DLLExport int zkExpiryTableAddElement (
  ZkError ,               /* Returns the error if any occured */
  ZkExpiryTable ,         /* Table to which we want to add an item */
  const char *,           /* Name of the item to add */ 
  time_t );               /* New item's expiry delay */

DLLExport int zkExpiryTableIsElement (
  ZkError ,               /* Returns the error if any occured */
  ZkExpiryTable ,         /* Table which we want to check for existing element */
  const char *,           /* Name of the item to verify */ 
  int * );                /* Boolean, verification result */

DLLExport int zkExpiryTableGetFirstExpired (
  ZkError ,               /* Returns the error if any occured */
  ZkExpiryTable ,         /* Expiry table to scan */
  char ** );              /* First expired name */

DLLExport int zkExpiryTableGetNextExpired (
  ZkError ,               /* Returns the error if any occured */
  ZkExpiryTable ,         /* Expiry table to scan */
  char ** );              /* Next expired name */

DLLExport int zkExpiryTableDeleteElement (
  ZkError ,               /* Returns the error if any occured */
  ZkExpiryTable ,         /* Expiry table from which we want to delete */
  const char * );         /* Name of element to delete */


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* ZKEXPIRYTABLE_H_ */
