/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef _ZKHASH_H
#define _ZKHASH_H


#include <stdlib.h>
#include "zkError.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  typedef struct zk_hash *ZkHash;

  /*
   *  The function that tests for key equality must return 0 when the two keys
   *  are the same.  It should return non-zero when they're unequal.
   *  
   *  This allows us to use memcmp(), strncmp(), etc. without another layer of
   *  wrapper functions.
   */
  typedef int ZkHashKeyEqualFn(const void *a, const void *b, size_t sz);
  typedef int ZkHashKeyFreeFn(ZkHash h, void *key, void *userData);
  typedef int ZkHashDataFreeFn(ZkHash h, void *data, void *userData);
  typedef int ZkHashTravFn(ZkError err, void *key, void *data, void *userData);
  typedef size_t ZkHashHashFn(ZkHash h, void *data, void *userData);


  DLLExport extern int zkHashDelete(ZkError err, ZkHash h, void *key);
  DLLExport extern int zkHashFree(ZkError err, ZkHash *h);
  DLLExport extern int zkHashInsert(ZkError err, ZkHash h, void *key, void *data);
  DLLExport extern int zkHashNew(ZkError err, ZkHash *h);
  DLLExport extern int zkHashSearch(ZkError err, ZkHash h, void *key, void **data);
  DLLExport extern int zkHashFind(ZkError err, ZkHash h, void *key, void **data, int *equal);
  DLLExport extern int zkHashSearchGivingKeyData(ZkError err, ZkHash h, void *key, void **kptr, void **data);
  DLLExport extern int zkHashSetDataFreeFn(ZkError err, ZkHash h, ZkHashDataFreeFn fn);
  DLLExport extern int zkHashSetKeyFreeFn(ZkError err, ZkHash h, ZkHashKeyFreeFn fn);
  DLLExport extern int zkHashSetHashFn(ZkError err, ZkHash h, ZkHashHashFn fn);
  DLLExport extern int zkHashSetKeyEqualFn(ZkError err, ZkHash h, ZkHashKeyEqualFn *fn);
  DLLExport extern int zkHashSetKeySize(ZkError err, ZkHash h, size_t sz);
  DLLExport extern int zkHashSetTableSize(ZkError err, ZkHash h, size_t sz);
  DLLExport extern int zkHashSetUserData(ZkError err, ZkHash h, void *userData);
  DLLExport extern int zkHashTraverse(ZkError err, ZkHash h, ZkHashTravFn cb, void *userData);


  typedef struct zk_hash_enum *ZkHashEnum;

  DLLExport extern int zkHashEnumFree(ZkError err, ZkHashEnum *e);
  DLLExport extern int zkHashEnumGetData(ZkError err, ZkHashEnum e, void **data);
  DLLExport extern int zkHashEnumGetKey(ZkError err, ZkHashEnum e, void **key);
  DLLExport extern int zkHashEnumInit(ZkError err, ZkHashEnum e);
  DLLExport extern int zkHashEnumNew(ZkError err, ZkHashEnum *e);
  DLLExport extern int zkHashEnumNext(ZkError err, ZkHashEnum e);
  DLLExport extern int zkHashEnumSetHash(ZkError err, ZkHashEnum e, ZkHash h);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _ZKHASH_H */
