/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKLIST_H_
#define ZKLIST_H_

#ifndef _ZKERROR_H
#include <zkError.h>
#endif /* _ZKERROR_H */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct T_ZkList * ZkList;              /* The list structure */
typedef struct T_ZkListNode * ZkListNode;      /* Nodes & cursor in list */

typedef int (* ZkListDataFreeFn)(ZkError , ZkList , void * , void *);  

#define LIST_HEAD    1
#define LIST_TAIL    2
#define LIST_BEFORECURSOR  3
#define LIST_AFTERCURSOR   4
#define LIST_BACKWARDS 3
#define LIST_FORWARDS  4
#define LIST_STAY      5

DLLExport int zkListNew (
  ZkError ,               /* Returns the error if any occured */
  ZkList * );             /* Returns a new zk linked list */

DLLExport int zkListFree (
  ZkError ,               /* Returns the error if any occured */
  ZkList * );             /* Pointer to linked list to destroy */

DLLExport int zkListDataFree (
  ZkError ,               /* Returns the error if any occured */
  ZkList ,                /* List which owns the node */
  void * ,                /* Data pointer to deallocate */
  void * );               /* Extra data pointer (optional, can be NULL) */


DLLExport int zkListSetDataFreeFunction (
  ZkError ,               /* Returns the error if any occured */
  ZkList ,
  ZkListDataFreeFn ,      /* The function to free the inserted data */
  void * );               /* Extra data pointer which will be passed as well */

DLLExport int zkListGetDataFreeFunction (
  ZkError ,             /* Returns the error if any occured */
  ZkList ,
  ZkListDataFreeFn * ,  /* The function to free the inserted data */
  void ** );            /* Extra data pointer which will be passed as well */

DLLExport int zkListCursor (
  ZkError ,               /* Returns the error if any occured */
  ZkList ,
  ZkListNode * );         /* Returns the current cursor pointer */

DLLExport int zkListSetCursor (
  ZkError ,               /* Returns the error if any occured */
  ZkList ,
  ZkListNode );           /* Set the cursor pointer */

DLLExport int zkListSize (
  ZkError ,               /* Returns the error if any occured */
  ZkList ,
  size_t * );             /* Returns the number of elements in the list */

DLLExport int zkListMoveCursor (
  ZkError ,               /* Returns the error if any occured */
  ZkList ,
  int );                  /* Where to set the cursor position */

     /* Note: zkListGet() does not modify the third arguments when it returns 0 */

DLLExport int zkListGet (
  ZkError ,               /* Returns the error if any occured */
  ZkList ,
  void ** ,               /* Returns data pointed to by cursor */
  int );                  /* Where to move the cursor next */

DLLExport int zkListSet (
  ZkError ,               /* Returns the error if any occured */
  ZkList ,                /* The list from which you wish to retreive data */
  void * );               /* Pointer to the data to be set */
 
DLLExport int zkListDelete (
  ZkError ,               /* Returns the error if any occured */
  ZkList ,
  int );                  /* Where to move the cursor after deletion */

DLLExport int zkListInsert (
  ZkError ,               /* Returns the error if any occured */
  ZkList ,
  void * ,                /* Data to insert in the list */
  int );                  /* Insertion point */

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* ZKLIST_H_ */
