/*
 *  zkLog.h
 *  $Id: zkLog.h,v 1.14.26.1 2000/10/11 13:19:59 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 *  Philippe Boucher (philippe@zks.net)

 *  Wednesday, june 24, 1998
 */

#ifndef _ZKLOG_H
#define	_ZKLOG_H
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifndef _ZKERROR_H
#include "zkError.h"
#endif /* _ZKERROR_H */

#ifndef T_ZKLOG
#define T_ZKLOG
typedef struct T_ZkLog * ZkLog;
#endif /* T_ZKLOG */

DLLExport int zkLogNew (
  ZkLog * );                    /* Returns a new zkLog structure */

DLLExport int zkLogError (
  ZkLog ,                       /* target zkLog structure */
  ZkError );                    /* zkError structure to log */

DLLExport int zkLogFree (
  ZkLog * );                    /* zkLog structure to destroy */

DLLExport int zkLogSetProgname (
  ZkLog ,                       /* zklog structure to modify */
  const char * );               /* Name of the program */

int zkLogSetFile (
  ZkLog ,                       /* zkLog structure to set */
  const char * );               /* filename for the log */

int
zkLogReopenFile (
  ZkLog zklog ,                 /* zkLog structure to set */
  const char * filename);       /* new filename for the log */
  
int zkLogStream (
  ZkLog ,
  FILE ** );                    /* Returns current stream of log object */

DLLExport int zkLogSetStream (
  ZkLog ,                       /* zkLog structure to set */
  FILE * );                     /* FILE stream for the log */

DLLExport int zkLogEnable (
  ZkLog );                      /* zkLog structure to enable */

DLLExport int zkLogDisable (
  ZkLog );                      /* zkLog structure to disable */

DLLExport int zkLogWrite_va (
  ZkLog ,                       /* zkLog to which to write a message */
  const char * ,                /* Message format string to send to log */
  ... ) ZK_PRINTF_LIKE(2, 3);   /* Optional arguments */

DLLExport int zkLogWrite (
  ZkLog ,                       /* zkLog to which to write a message */
  const char * );               /* Message string to send to log */

DLLExport int zkLogWriteContext (
  ZkLog ,                       /* ZkLog object to which to write the message*/
  ZkError ,                     /* current ZkError object */
  const char * ,                /* Name of the current function */
  const char * ,                /* A NULL terminated message string to send to the log */
  ... );                        /* elipse */


/*
 * Wrapper to solve an historical naming problem 
 */
#define  zkLogWriteContext_va   zkLogWriteContext


#define zkLogWritePtr(x,y) zkLogWrite(x, (const char *)y)



#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _ZKLOG_H */

