/*
 * $Id: zkShell.h,v 1.5.32.1 2000/10/11 13:19:59 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKSHELL_H_
#define ZKSHELL_H_

#ifdef __xplusplus 
extern "C" {
#endif /* __cplusplus */

typedef struct T_ZkShell * ZkShell;

#define ZKSHELL_PARMS_NONE         0x1
#define ZKSHELL_PARMS_1            0x2
#define ZKSHELL_PARMS_2            0x4
#define ZKSHELL_PARMS_3            0x8
#define ZKSHELL_PARMS_4           0x10
#define ZKSHELL_PARMS_5           0x20
#define ZKSHELL_PARMS_6           0x40
#define ZKSHELL_PARMS_7           0x80
#define ZKSHELL_PARMS_8          0x100
#define ZKSHELL_PARMS_9          0x200
#define ZKSHELL_PARMS_10         0x400
#define ZKSHELL_PARMS_11         0x800
#define ZKSHELL_PARMS_12        0x1000
#define ZKSHELL_PARMS_13        0x2000
#define ZKSHELL_PARMS_14        0x4000
#define ZKSHELL_PARMS_15        0x8000
#define ZKSHELL_PARMS_16       0x10000
#define ZKSHELL_PARMS_17       0x20000
#define ZKSHELL_PARMS_18       0x40000
#define ZKSHELL_PARMS_19       0x80000

/*
 * Default ZkShell values
 */

#define ZKSHELL_DEFAULT_INPUT            stdin
#define ZKSHELL_DEFAULT_OUTPUT          stdout
#define ZKSHELL_DEFAULT_PROMPT      "prompt> "
#define ZKSHELL_ECHO_CMD            "_echo_cmds" /* Variable for echoing commands */
#define ZKSHELL_NO_STR              "no" 
#define ZKSHELL_YES_STR             "yes" 
  
typedef int (*ZkShellCommandCallback) (
  ZkError err ,           /* To return the error if one occured */
  ZkShell ,               /* The ZkShell object */
  char ** ,               /* The NULL terminated list of token strings */
  size_t ,                /* The number of tokens */
  void * );               /* The shell command context pointer */

/*
 * Return codes for the command callbacks
 */

#define ZKSHELL_SESSION_QUIT           2
#define ZKSHELL_SESSION_FATAL          0
#define ZKSHELL_SESSION_CONTINUE       1

/*
 * Function prototypes
 */

DLLExport int zkShellNew (
  ZkError ,               /* Return the error */
  ZkShell * );            /* Returns a new zkShell structure */

DLLExport int zkShellFree (
  ZkError ,               /* Return the error */
  ZkShell * );            /* Pointer to zkShell to destroy */

DLLExport int zkShellSetPrompt (
  ZkError ,               /* Return the error if any occured */
  ZkShell ,               /* Shell object to modify */
  const char * );         /* Prompt to use for the shell */

DLLExport int zkShellSetStreams (
  ZkError ,               /* Return the error if any occured */
  ZkShell ,               /* Shell to modify */
  FILE * ,                /* Input file stream */
  FILE * );               /* Output file stream */

DLLExport int zkShellSetContext (
  ZkError ,               /* Return the error if any occured */
  ZkShell ,               /* The Shell to modify */
  void * );               /* Pointer to some context structure for commands */

DLLExport int zkShellWrite_va (
  ZkError ,               /* Return the error if any occured */
  ZkShell ,               /* The Shell to modify */
  const char * ,          /* Format string */
  ... );                  /* Format parameters */

DLLExport int zkShellVariableSet (
  ZkError ,               /* Return the error if any occured */
  ZkShell ,               /* Shell to which to add the variable */
  char * ,                /* Name of the variable */
  char * );              /* Value of the variable */

DLLExport int zkShellVariableGet (
  ZkError ,               /* Return the error if any occured */
  ZkShell ,               /* Shell from which to get the variable */
  char * ,                /* Name of the desired variable */
  char ** );              /* Returns the value string of the variable */

DLLExport int zkShellVariableUnset (
  ZkError ,               /* Return the error if any occured */
  ZkShell ,               /* Shell from which to delete the variable */
  char * );               /* Name of the variable to delete */



DLLExport int zkShellAddCommand (
  ZkError ,               /* Return the error if any occured */
  ZkShell ,               /* Shell to which to add the command */
  ZkShellCommandCallback , /* Command callback function */
  const char * ,          /* Shell command token string */
  const char * ,          /* Usage string */
  const char * ,          /* Help string */
  uint32_t );             /* Parameter mask */


DLLExport int zkShellRun (
  ZkError ,               /* Return the error if any occured */ 
  ZkShell );              /* Command shell to execute */

/*
 * Some builtin zkShell commands
 */

DLLExport int zkShellCommandSet (
  ZkError ,           /* Returns the error if any occured */
  ZkShell ,           /* The shell object which holds this command */
  char ** ,           /* The list of input tokens */
  size_t ,            /* The number of tokens on the input line */
  void * );           /* User specified shell context pointer */

DLLExport int zkShellCommandUnset (
  ZkError ,           /* Returns the error if any occured */
  ZkShell ,           /* The shell object which holds this command */
  char ** ,           /* The list of input tokens */
  size_t ,            /* The number of tokens on the input line */
  void * );           /* User specified shell context pointer */

DLLExport int zkShellCommandQuit (
  ZkError ,           /* Returns the error if any occured */
  ZkShell ,           /* The shell object which holds this command */
  char ** ,           /* The list of input tokens */
  size_t ,            /* The number of tokens on the input line */
  void * );           /* User specified shell context pointer */

DLLExport int zkShellCommandHelp (
  ZkError ,           /* Returns the error if any occured */
  ZkShell ,           /* The shell object which holds this command */
  char ** ,           /* The list of input tokens */
  size_t ,            /* The number of tokens on the input line */
  void * );           /* User specified shell context pointer */

DLLExport int zkShellCommandPrompt (
  ZkError ,           /* Returns the error if any occured */
  ZkShell ,           /* The shell object which holds this command */
  char ** ,           /* The list of input tokens */
  size_t ,            /* The number of tokens on the input line */
  void * );           /* User specified shell context pointer */

DLLExport int zkShellCommandEcho (
  ZkError ,           /* Returns the error if any occured */
  ZkShell ,           /* The shell object which holds this command */
  char ** ,           /* The list of input tokens */
  size_t ,            /* The number of tokens on the input line */
  void * );           /* User specified shell context pointer */

DLLExport int zkShellCommandSource (
  ZkError ,           /* Returns the error if any occured */
  ZkShell ,           /* The shell object which holds this command */
  char ** ,           /* The list of input tokens */
  size_t ,            /* The number of tokens on the input line */
  void * );           /* User specified shell context pointer */


#ifdef __xplusplus 
extern "C" {
#endif /* __cplusplus */

#endif /* ZKSHELL_H_ */
