#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )
/*
 * Semaphore manipulation
 *
 * $Id: zksem.h,v 1.4.8.1 2000/10/11 13:19:59 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Nicolas Roy <nicolas@zeroknowledge.com>

 */
#ifndef ZK_SEM_H_
# define ZK_SEM_H_


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "zks.h"
#include "zkIntTypes.h"
#include "zkTypes.h"
#include "zkError.h" 

#include "zkipc.h" 

/*
** name maximum length
*/
#define ZKSEM_MAXNAME_LENGTH                               ZKIPC_MAXNAME_LENGTH

/*
** read/write permission mode
*/
#define ZKSEM_NONE_PMODE                                   ZKIPC_NONE_PMODE            
#define ZKSEM_USER_READ_PMODE                              ZKIPC_USER_READ_PMODE        
#define ZKSEM_USER_WRITE_PMODE                             ZKIPC_USER_WRITE_PMODE       
#define ZKSEM_GROUP_READ_PMODE                             ZKIPC_GROUP_READ_PMODE       
#define ZKSEM_GROUP_WRITE_PMODE                            ZKIPC_GROUP_WRITE_PMODE      
#define ZKSEM_OTHER_READ_PMODE                             ZKIPC_OTHER_READ_PMODE       
#define ZKSEM_OTHER_WRITE_PMODE                            ZKIPC_OTHER_WRITE_PMODE      
#define ZKSEM_FULL_PERM_PMODE                              ZKIPC_FULL_PERM_PMODE

/*
** Other flags
*/
#define ZKSEM_PRIVATE_KEY                                   ZKIPC_PRIVATE_KEY

/*
** ZKSEM handle
*/
typedef struct S_zkSem * ZkSem;


/*
** Exportable calls
*/
DLLExport int zkSemNew
(
 ZkError     err,                        /* [in]   ZkError */                   
 ZkSem      *semP,                       /* [out]  pointer on a ZkSem */
 const char *name,                       /* [in]   unique string to identify the semaphore */
 uint16_t    value,                      /* [in]   original value of the semaphore */
 uint32_t    flags                       /* [in]   creation flag */
);

DLLExport int zkSemFree
(
 ZkError   err,                        /* [in]   ZkError */                   
 ZkSem    *semP,                       /* [out]  pointer on a ZkSem */
 ZkBool    undo                        /* [in]   if TRUE than release the semaphore before closing */
);

DLLExport int zkSemWait
(
 ZkError   err,                        /* [in]  ZkError */                   
 ZkSem     sem                         /* [in]  ZkSem */
);

DLLExport int zkSemTryWait
(
 ZkError   err,                        /* [in]     ZkError */                   
 ZkSem     sem,                        /* [in]     ZkSem */
 ZkBool   *successP                    /* [in/out] pointer on a ZkBool to represent if the waiting success */
);

DLLExport int zkSemRelease
(
 ZkError   err,                        /* [in]  ZkError */                   
 ZkSem     sem                         /* [in]  ZkSem */
);

DLLExport int zkSemSetHook
(
 ZkError    err,                       /* [in]   pointer on a ZkError */
 ZkSem      sem,                       /* [in]  ZkSem */
 char      *hook                       /* [in]  pointer the extra data (may be NULL) */
);

DLLExport int zkSemGetHook
(
 ZkError    err,                       /* [in]   pointer on a ZkError */
 ZkSem      sem,                       /* [in]   ZkSem */
 char     **hookP                      /* [out]  pointer the extra data */
);

DLLExport int zkSemCheckOwner
(
 ZkError    err,     /* [in]   pointer to a ZkError */
 ZkSem      sem,     /* [in]   ZkSem */
 ZkBool    *powner   /* [out]  ZK_TRUE if calling process is owner */
);


DLLExport int zkSemCleanup
(
 ZkError     err,                      /* [in]   ZkError */                   
 const char *name                      /* [in]   unique string to identify the semaphore */
);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZK_SEM_H_ */

#endif /* ZK_LINUX || ZK_SOLARIS */
