#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )
/*
 * Shared Memory manipulation
 *
 * $Id: zkshm.h,v 1.5.8.1 2000/10/11 13:19:59 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Nicolas Roy <nicolas@zeroknowledge.com>

 */
#ifndef ZK_SHM_H_
# define ZK_SHM_H_


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#include "zks.h"
#include "zkIntTypes.h"
#include "zkError.h" 

#include "zkipc.h" 


/*
** name maximum length
*/
#define ZKSHM_MAXNAME_LENGTH                               ZKIPC_MAXNAME_LENGTH


/*
** read/write permission mode
*/
#define ZKSHM_NONE_PMODE                                   ZKIPC_NONE_PMODE
#define ZKSHM_USER_READ_PMODE                              ZKIPC_USER_READ_PMODE
#define ZKSHM_USER_WRITE_PMODE                             ZKIPC_USER_WRITE_PMODE
#define ZKSHM_GROUP_READ_PMODE                             ZKIPC_GROUP_READ_PMODE
#define ZKSHM_GROUP_WRITE_PMODE                            ZKIPC_GROUP_WRITE_PMODE
#define ZKSHM_OTHER_READ_PMODE                             ZKIPC_OTHER_READ_PMODE
#define ZKSHM_OTHER_WRITE_PMODE                            ZKIPC_OTHER_WRITE_PMODE
#define ZKSHM_FULL_PERM_PMODE                              ZKIPC_FULL_PERM_PMODE

/*
** Other flag
*/
#define ZKSHM_PRIVATE_KEY                                   ZKIPC_PRIVATE_KEY
                                                           

/*
** ZkShm handle
*/
typedef struct S_zkShm * ZkShm;


/*
** Exportable calls
*/
DLLExport int zkShmNew
(
 ZkError     err,                                          /* [in]   pointer on a ZkError */
 ZkShm      *shmP,                                         /* [out]  pointer on a ZkShm */
 const char *name,                                         /* [in]   unique string to identify the shared memory */
 uint32_t    size,                                         /* [in]   requested size of the shared memory */
 uint32_t    flags                                         /* [in]   creation flag */
);

DLLExport int zkShmDetach
(
  ZkError  err,    /* [in]  pointer on a ZkError */
  ZkShm    shm     /* [in]  Shared memory from which to detach */
);


DLLExport int zkShmFree
(
 ZkError   err,                                            /* [in]   pointer on a ZkError */
 ZkShm    *shmP                                            /* [in]  Shared memory to be destroy */
);

DLLExport int zkShmAddr
(
 ZkError   err,                                            /* [in]   pointer on a ZkError */
 ZkShm     shm,                                            /* [in]  ZkShm */
 void    **addr                                            /* [out] pointer to the address pointer */
);

DLLExport int zkShmSize
(
 ZkError   err,                                            /* [in]   pointer on a ZkError */
 ZkShm     shm,                                            /* [in]  ZkShm */
 uint32_t *size                                            /* [out] pointer to the size */
);

DLLExport int zkShmName
(
 ZkError   err,                                            /* [in]   pointer on a ZkError */
 ZkShm     shm,                                            /* [in]  ZkShm */
 char     *name,                                           /* [out] pointer the name */
 uint32_t  namesz                                          /* [in]  length of the namesz buffer */
);

DLLExport int zkShmCheckOwner
(
 ZkError    err,     /* [in]   pointer to a ZkError */
 ZkShm      shm,     /* [in]   ZkShm */
 ZkBool    *powner   /* [out]  ZK_TRUE if calling process is owner */
);

DLLExport int zkShmSetHook
(
 ZkError    err,                                           /* [in]   pointer on a ZkError */
 ZkShm      shm,                                           /* [in]  ZkShm */
 char      *hook                                           /* [in]  pointer the extra data (may be NULL) */
);

DLLExport int zkShmGetHook
(
 ZkError    err,                                           /* [in]   pointer on a ZkError */
 ZkShm      shm,                                           /* [in]   ZkShm */
 char     **hookP                                          /* [out]  pointer the extra data */
);

DLLExport int zkShmCleanup
(
 ZkError     err,                                          /* [in]   ZkError */                   
 const char *name                                          /* [in]   unique string to identify the semaphore */
);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZK_SHM_H_ */

#endif /* ZK_LINUX || ZK_SOLARIS */
