/* **************************************************************************
** $Id: zksocket.h,v 1.13.10.2 2000/10/11 15:53:50 build Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file contains definitions and declarations for a socket wrapper
** object and its associated API.
**
** **************************************************************************
*/

# ifndef zksocket_h_INCLUDED
# define zksocket_h_INCLUDED

# include "zkError.h"
# include "zksockettypes.h"
# include <zkList.h>

typedef struct T_ZkSocket *     ZkSocket;

typedef enum T_ZkSocketOption   ZkSocketOption;

# if defined (WIN32)
# include <winsock2.h>
# endif

# ifdef __cplusplus
extern "C" {
# endif


DLLExport int
parse_inet_addr(
    ZK_INOUT ZkError               error,
    ZK_IN    const char *          path,
    ZK_OUT   struct T_ZkInetAddr *   addr );

DLLExport
int zkSocketNew(
    ZK_INOUT ZkError                error,
    ZK_IN    int                    family,  /* AF_INET, AF_LOCAL */
    ZK_IN    int                    type,    /* SOCK_STREAM, SOCK_DGRAM */
    ZK_IN    int                    protocol, /* IPPROTO_xxx */
    ZK_OUT   ZkSocket *             out_socket );

DLLExport
int zkSocketNewFromDescriptor(
    ZK_INOUT ZkError                error,
    ZK_IN    int                    family,  /* AF_INET, AF_LOCAL */
    ZK_IN    ZkSocketDescriptor     desriptor,
    ZK_OUT   ZkSocket *             out_socket );

DLLExport
int zkSocketFree(
    ZK_INOUT ZkError                error,
    ZK_INOUT ZkSocket *             out_socket );

DLLExport
int zkSocketGetRemoteAddress(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_OUT   const char * *         out_path );

int
zkSocketAddressCompare(
    ZK_INOUT ZkError                error,
    ZK_IN    uint32_t               flags,
    ZK_IN    const char *           path1,
    ZK_IN    const char *           path2,
    ZK_OUT   ZkBool *               is_same );

DLLExport
int zkSocketSetOption(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_IN    ZkSocketOption         option,
    ZK_IN    uint32_t               value );

DLLExport
int zkSocketBind(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_IN    const char *           path );

DLLExport
int zkSocketListen(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s );

DLLExport
int zkSocketAccept(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               in_socket,
    ZK_OUT   ZkSocket *             out_socket );

DLLExport
int zkSocketAcceptWithTimeoutUpdate(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               in_socket,
    ZK_INOUT uint32_t *             io_timeout_in_secs,
    ZK_OUT   ZkSocket *             out_socket );

DLLExport
int zkSocketConnect(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_IN    const char *           path );

DLLExport
int zkSocketConnectAddr(
	ZK_INOUT	ZkError					error,
	ZK_IN		ZkSocket				s,
	ZK_IN		const ZkSocketAddress	address);

DLLExport
int zkSocketWaitReadableOrClosed(
	ZK_INOUT ZkError				error,
	ZK_IN	 ZkList 				list_of_sockets,
	ZK_IN    struct timeval			*timeout,
	ZK_OUT   int					*whichSocket,
    ZK_IN    int                    signal_fd);

DLLExport
int zkSocketRead(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_INOUT void *                 target_buffer,
    ZK_IN    uint32_t               max_bytes_to_read,
    ZK_OUT   uint32_t *             out_bytes_read );

DLLExport
int zkSocketReadAll(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_INOUT void *                 target_buffer,
    ZK_IN    uint32_t               bytes_to_read );

DLLExport
int zkSocketWrite(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_INOUT const void *           source_buffer,
    ZK_IN    uint32_t               max_bytes_to_write,
    ZK_OUT   uint32_t *             out_bytes_written );

DLLExport
int zkSocketWriteAll(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_INOUT const void *           source_buffer,
    ZK_IN    uint32_t               bytes_to_write );

DLLExport
int zkSocketGetFD(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_OUT   ZkSocketDescriptor*    fd);

# ifdef __cplusplus
}
# endif

# endif /* zksocket_h_INCLUDED */
