#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )

/*
 * Timer manipulation
 *
 * $Id: zktimer.h,v 1.3.18.1 2000/10/11 13:20:00 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Nicolas Roy <nicolas@zeroknowledge.com>

 */

#include "zks.h"
#include "zkIntTypes.h"
#include "zkTypes.h"
#include "zkDefs.h"
#include "zkError.h"
#include "zkallocator.h"

typedef struct S_Timer * ZkTimer;


DLLExport int zkTimerNew
(
 ZkError     err,                     /* [in]   ZkError */                   
 ZkTimer    *timerP,                  /* [out]  pointer on a ZkTimer */
 ZkAllocator allocator
 );

DLLExport int zkTimerFree
(
 ZkError     err,                     /* [in]   ZkError */                   
 ZkTimer    *timerP                   /* [out]  pointer on a ZkTimer */
 );

DLLExport int zkTimerStart
(
 ZkError     err,                     /* [in]  ZkError */
 ZkTimer     timer                    /* [in]  ZkTimer */
 );

DLLExport int zkTimerStop
(
 ZkError     err,                     /* [in]  ZkError */
 ZkTimer     timer                    /* [in]  ZkTimer */
 );

DLLExport int zkTimerPause
(
 ZkError     err,                     /* [in]  ZkError */
 ZkTimer     timer                    /* [in]  ZkTimer */
 );

DLLExport int zkTimerResume
(
 ZkError     err,                     /* [in]  ZkError */
 ZkTimer     timer                    /* [in]  ZkTimer */
 );

DLLExport int zkTimerReset
(
 ZkError     err,                     /* [in]  ZkError */
 ZkTimer     timer                    /* [in]  ZkTimer */
 );

DLLExport int zkTimerProcessTimesGet
(
 ZkError  err,                       /* [in]  ZkError */
 ZkTimer  timer,                     /* [in]  ZkTimer */
 double  *realTimeP,                 /* [out] return Real elapsed time (uptional) */
 double  *userTimeP,                 /* [out] return User CPU elapsed time (uptional) */
 double  *sysTimeP                   /* [out] return System CPU elapsed time (uptional) */
 );

DLLExport int zkTimerProcessChildTimesGet
(
 ZkError  err,                       /* [in]  ZkError */
 ZkTimer  timer,                     /* [in]  ZkTimer */
 double  *realTimeP,                 /* [out] return Real elapsed time (uptional) */
 double  *userTimeP,                 /* [out] return User CPU elapsed time (uptional) */
 double  *sysTimeP                   /* [out] return System CPU elapsed time (uptional) */
 );

DLLExport int zkTimerPauseTimesGet
(
 ZkError  err,                       /* [in]  ZkError */
 ZkTimer  timer,                     /* [in]  ZkTimer */
 double  *realTimeP,                 /* [out] return Real elapsed time (uptional) */
 double  *userTimeP,                 /* [out] return User CPU elapsed time (uptional) */
 double  *sysTimeP                   /* [out] return System CPU elapsed time (uptional) */
 );
 
#endif /* ZK_LINUX || ZK_SOLARIS */
