/*
 *  $Id: zkCode.c,v 1.4.32.1 2000/10/11 13:20:01 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Routines that produce C source code.
 */


#include "zks.h"

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "zkAlloc.h"
#include "zkCStr.h"
#include "zkCode.h"
#include "zkError.h"


/*
 *  Create code to initialize an unsigned character array with the contents of
 *  `data'.
 *  
 *  Given a one byte data buffer, containing the value 5, we would produce the
 *  string similar to "{ 0x05 }".  The exact formatting may vary.
 *  
 *  bug: we assume characters are eight bits, and can thus be represented as
 *  two hexadecimal digits.
 */
DLLExport int zkCodeUCharArrayFromData(ZkError err, void *data, size_t dsz, char **code, size_t *codesz)
{
  char *c = 0;
  int ret = 0;
  size_t csz, i;

  zkErrorClear(err);

  if ( ! data) {
    zkErrorSet(err, "zkCodeUCharArrayFromData", ZKERR, ZKMCODE, ZKENULLPTR,
               "data is null");
    return 0;
  }
  if (dsz == 0) {
    zkErrorSet(err, "zkCodeUCharArrayFromData", ZKERR, ZKMCODE, ZKEPARMBAD,
               "dsz is 0");
    return 0;
  }
  if ( ! code) {
    zkErrorSet(err, "zkCodeUCharArrayFromData", ZKERR, ZKMCODE, ZKENULLPTR,
               "code is null");
    return 0;
  }
  if ( ! codesz) {
    zkErrorSet(err, "zkCodeUCharArrayFromData", ZKERR, ZKMCODE, ZKENULLPTR,
               "codesz is null");
    return 0;
  }

  if (zkCStrSprintf(&c, "{\n ") == -1) {
    zkErrorSet_va(err, "zkCodeUCharArrayFromData", ZKERR, ZKMCODE, errno,
                  "can't initialize string: %s", strerror(errno));
    return 0;
  }

  csz = strlen(c); /* buffer length, less 1 */

  for (i = 0; i < dsz; i++) {
    char s[64];
    int l;
    
    l = snprintf(s, sizeof s, " 0x%02x,%s",
                  (unsigned int)((unsigned char *)data)[i],
                  (i + 1) % 10 == 0 ? "\n " : "");
    if (l == -1) {
      zkErrorSet(err, "zkCodeUCharArrayFromData", ZKERR, ZKMCODE, ZKERROR,
                 "caught buffer overrun");
      goto done;
    }
    
    if ( ! zkMemAppend(err, (void **)&c, &csz, s, l)) {
      goto done;
    }
  }

  if (c[csz-1] == ',') {
    c[csz-1] = '\n';
  } else {
    c[csz-3] = '\n'; csz -= 2;
  }
  
  if ( ! zkMemAppend(err, (void **)&c, &csz, "}\0", 2)) {
    goto done;
  }

  *code = c; c = 0; /* the caller now owns it */
  *codesz = csz; /* includes terminating nul */
  
  ret = 1;

 done:

  if (c) { free(c); }

  return ret;
}
