/**
 * $Id: zkPFork.c,v 1.2.32.1 2000/10/11 13:20:02 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *
 * Facilitate setting up a pipe and forking a child
 */

#include "zks.h"

#include <zkError.h>
#include <zkPFork.h>
#ifndef WIN32
#include <unistd.h>
#endif
#include <errno.h>
#include <string.h>

/*  Stolen from Bill
 *
 *  Create a child.  Both parent and child receive input and output file
 *  descriptors for communicating with each other.
 *
 * Note that the return values are not the same as the ones used in the 
 * ZK libraries.  Return values are the same as fork().
 */
int zkPFork(ZkError err, int *fd0, int *fd1)
{
  const char *func = "zkPFork";
  int cp[2];
  int pc[2];
  int pid;

  if (pipe(cp) == -1) {
    zkErrorSet_va(err, func, ZKERR, ZKMFORK, errno, "pipe(%s) failed",
                  strerror(errno));
    return -1;
  }

  if (pipe(pc) == -1) {
    zkErrorSet_va(err, func, ZKERR, ZKMFORK, errno, "pipe(%s) failed",
                  strerror(errno));
    close(cp[0]); close(cp[1]);
    return -1;
  }

  switch (pid = fork()) {
  case -1:
    zkErrorSet_va(err, func, ZKERR, ZKMFORK, errno, "fork(%s) failed",
                  strerror(errno));
    close(cp[0]); close(cp[1]);
    close(pc[0]); close(pc[1]);
    return -1;

  case 0:                       /* child */
    *fd0 = pc[0];
    *fd1 = cp[1];
    close(pc[1]);
    close(cp[0]);
    return 0;

  default:                      /* parent */
    *fd0 = cp[0];
    *fd1 = pc[1];
    close(cp[1]);
    close(pc[0]);
    return pid;
  }
}
