#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )
/*
 * Inter Processus Communication routines
 *
 * $Id: zkipc.c,v 1.2.32.2 2000/10/11 13:20:03 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Nicolas Roy <nicolas@zeroknowledge.com>

 */

#include "zks.h"
#include "zkIntTypes.h"
#include "zkTypes.h"

#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include <sys/shm.h>
#include <sys/stat.h>

#include "zkipc.h"


/*
** System dependant file open mode
*/
#define SYS_FILE_APPEND_MODE                               ( O_APPEND )
#define SYS_FILE_RDONLY_MODE                               ( O_RDONLY )
#define SYS_FILE_WRONLY_MODE                               ( O_WRONLY )
#define SYS_FILE_RDWR_MODE                                 ( O_RDWR )
#define SYS_FILE_APPEND_MODE                               ( O_APPEND )
#define SYS_FILE_CREAT_MODE                                ( O_CREAT )
#define SYS_FILE_EXCL_MODE                                 ( O_EXCL )
#define SYS_FILE_TRUNC_MODE                                ( O_TRUNC )


/*
** System dependant file permission mode
*/
#define SYS_FILE_USR_R_PERM                                ( S_IREAD )
#define SYS_FILE_USR_W_PERM                                ( S_IWRITE )
#define SYS_FILE_GRP_R_PERM                                ( S_IRGRP )
#define SYS_FILE_GRP_W_PERM                                ( S_IWGRP )
#define SYS_FILE_OTH_R_PERM                                ( S_IROTH )
#define SYS_FILE_OTH_W_PERM                                ( S_IWOTH )



                                                                
/* 
** Create a new file given a path length
*/
DLLExport int zkPathNameCreate
(
 ZkError     err,                      /* [in]   pointer on a ZkError */
 const char *pathname,                 /* [in]   unique string to identify the semaphore */
 uint32_t    pmode                     /* [in]   read/write permission mode */
)
{
  const char *fn        = "zkPathNameCreate"; 
  int         sys_perms = 0;
  int         fd        = -1;

  zkErrorClear(err);

  /* 
  ** check input values 
  */
  if( !pathname )
    {
      zkErrorSet_va(err, fn, ZKERR, ZKMSHM, ZKENULLPTR, "pathname is null");
      goto error;
    }
   
  /*
  ** remap to system flags
  */
  sys_perms |= (pmode & ZKIPC_USER_READ_PMODE)   ? SYS_FILE_USR_R_PERM : 0;
  sys_perms |= (pmode & ZKIPC_USER_WRITE_PMODE)  ? SYS_FILE_USR_W_PERM : 0;
  sys_perms |= (pmode & ZKIPC_GROUP_READ_PMODE)  ? SYS_FILE_GRP_R_PERM : 0;
  sys_perms |= (pmode & ZKIPC_GROUP_WRITE_PMODE) ? SYS_FILE_GRP_W_PERM : 0;
  sys_perms |= (pmode & ZKIPC_OTHER_READ_PMODE)  ? SYS_FILE_OTH_R_PERM : 0;
  sys_perms |= (pmode & ZKIPC_OTHER_WRITE_PMODE) ? SYS_FILE_OTH_W_PERM : 0;

  if( (fd = open(pathname,
                 (SYS_FILE_WRONLY_MODE|SYS_FILE_CREAT_MODE|SYS_FILE_EXCL_MODE),
                 sys_perms)) == -1 )
    {
      zkErrorSet_va(err, fn, ZKERR, ZKMSHM, errno, "open fail");
      goto error;
    }
  else if( close(fd) == -1 )
    {
      zkErrorSet_va(err, fn, ZKERR, ZKMSHM, errno, "close fail");
      goto error;
    }
  return 1;
error:
 return 0;
}



/* 
** Create a new file given a path length
*/
DLLExport int zkPathNameRemove
(
 ZkError     err,                      /* [in]   pointer on a ZkError */
 const char *pathname                  /* [in]   unique string to identify the semaphore */
)
{
  const char *fn        = "zkPathNameRemove"; 

  zkErrorClear(err);

  /* 
  ** check input values 
  */
  if( !pathname )
    {
      zkErrorSet_va(err, fn, ZKERR, ZKMSHM, ZKENULLPTR, "pathname is null");
      goto error;
    }

  if( remove(pathname) == -1 )
    {
      zkErrorSet_va(err, fn, ZKERR, ZKMSHM, errno, "remove fail");
      goto error;
    }
  return 1;
error:
 return 0;
}


#endif /* ZK_LINUX || ZK_SOLARIS */
