/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

/*
We don't use the <inc/clientdebug.h> macros since those
are for C++ only, these are pure C Win32 functions.
( Or unix C sunctions, as the case may be )
*/
#ifdef WIN32
#include <CRTDBG.H>
#include <winsock2.h>
#include <time.h>
#include <winbase.h>
#include <TraceControl.h>

/* inet_aton doesn't exist under Windows, I'll have to write it here
 * This file belongs with the zklib.Dll project and it is exported from that DLL only
 * Returns FALSE on error
 * Returns TRUE on success
 */
DLLExport BOOL inet_aton(LPCSTR pStr, IN_ADDR* InetAddr){
	
	_ASSERT(pStr);
	_ASSERT( !IsBadWritePtr((LPVOID)InetAddr, sizeof(IN_ADDR)) );

	InetAddr->s_addr = inet_addr(pStr);

	return(InetAddr->s_addr != INADDR_NONE);
}

/* Returns -1 on error, as stated in Unix man pages
 * Should follow unix functionality as in
 * getting microseconds since Jan 1. 1970
 * Good until 2038
 */
DLLExport int gettimeofday(struct timeval* tp, void* v){
  if( tp ) {
    SYSTEMTIME st;
    struct tm  ts;
 
    /* Get current System UTC Time */
    GetSystemTime(&st);

    /* Convert it into calendar time to get the epoch delta value */
    ts.tm_sec   = st.wSecond;
    ts.tm_min   = st.wMinute;
    ts.tm_hour  = st.wHour;
    ts.tm_mday  = st.wDay;
    ts.tm_mon   = st.wMonth;
    ts.tm_year  = st.wYear-1900;
    ts.tm_wday  = 0;
    ts.tm_yday  = 0;
    ts.tm_isdst = 0;

    tp->tv_sec  = mktime(&ts);
    tp->tv_usec = st.wMilliseconds * 1000;
  }
	return 1;
}

DLLExport void ClientTrace(const char fnc[], const char detail[]){
#ifdef TRACE_UNIX_LIB
#if!defined( NDEBUG )
	CHAR buf[255];
	_snprintf(buf, 254, "ClientTrace() function [%s], message [%s]\n", fnc, strlen(detail)?detail:"no detail");
	OutputDebugString(buf);
#endif
#endif
}

TCHAR szMessage[MAX_PATH + 1];

DLLExport char* zksstrerror(int error){

	ZeroMemory(szMessage, sizeof(szMessage));

	FormatMessage(
            FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_SYSTEM,
            NULL,
            GetLastError(),
            MAKELANGID(LANG_NEUTRAL, SUBLANG_NEUTRAL),
            szMessage,
            MAX_PATH,
            NULL);

	return szMessage;
}

#else

/* Linux client support 
 */

#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>

/* This function doesn't exist on UNIX.
 * Implemented here it is  quite different from GetTickCount on win32
 * I'm not returning uptime, but execution time of the program. The reason is that
 * getting uptime from /proc/uptime, for example, is relatively time consuming, and
 * I've never seen this used for anything except measuring execution time. So
 * it's implemented here just as the time since ZKGetTickCount was fisrt called.
 * --ol
 */

unsigned long ZKGetTickCount( void )
{
    static int first_call = 1;
    unsigned long ret;
    static struct timeval first_tv;
    struct timeval this_tv;

    if ( first_call )
    {
        /* fail silently. This function not working shouldn't be fatal */
        if ( gettimeofday(&first_tv, NULL) ) perror("Unable to get system time! :");        
        else first_call = 0;
    }

    if ( gettimeofday(&this_tv, NULL) ) perror("Unable to get system time! :");

    /* We could wrap in here ... This function should not be used for anything
     * extremely important
     */
    ret = ((unsigned long)(this_tv.tv_sec - first_tv.tv_sec)) * 1000L;
    ret += ((unsigned long)(this_tv.tv_usec - first_tv.tv_usec)) / 1000L;
    
    return ret;
}

#endif 
