/* 
 * $Id: zkspid.c,v 1.1.4.1 2000/12/14 17:54:29 jean Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */
#include <zks.h>
#include <zkIntTypes.h>

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#if defined( ZK_LINUX )
# include <unistd.h>
# include <pthread.h>
#endif

#include "zkserr.h"
#include "zkspid.h"


/*
 *
 */
static ZkBool pidInit(T_ZksPid * pid)
{
#if defined( WIN32 )
  pid->pId = (ZksProcessId) GetCurrentProcessId();
  pid->tId = (ZksThreadId)  GetCurrentThreadId();

#elif defined( ZK_LINUX )
  pid->pId = (ZksProcessId) getpid();
  pid->tId = (ZksThreadId)  pthread_self();
#endif
  return ZKS_SUCCESS;
}




/*
 *
 */
ZkBool zksPidInit(T_ZksPid * pid)
{
  ZkBool  success = ZKS_SUCCESS;

  if( !pid ) { 
    success = ZKS_FAILURE;

  } else {
    pidInit(pid);
  }
  return success;  
}




/*
 *
 */
ZkBool zksPidEqual(const T_ZksPid * pidA, 
                   const T_ZksPid * pidB, 
                   ZkBool         * equalP)
{
  ZkBool  success = ZKS_SUCCESS;
  ZkBool  equal   = ZK_FALSE;

  if( !pidA || !pidB  ) { 
    success = ZKS_FAILURE;

  } else {
    equal = ( (pidA->pId == pidB->pId) && (pidA->tId == pidB->tId) ) ? ZK_TRUE: ZK_FALSE;
  }

  if( equalP ) { *equalP = equal; }

  return success;
}




/*
 *
 */
ZkBool zksPidAssign(T_ZksPid       * dstPid, 
                    const T_ZksPid * srcPid)
{
  ZkBool  success = ZKS_SUCCESS;

  if( !dstPid || !srcPid ) { 
    success = ZKS_FAILURE;

  } else {
    *dstPid = *srcPid;
  }
  return success;  
}





/*
 *
 */
ZkBool zksPidAsString(const T_ZksPid * pid, 
                      char           * buffer, 
                      uint32_t         buffersz)
{
  ZkBool  success = ZKS_SUCCESS;

  if( !pid ) { 
    success = ZKS_FAILURE;

  } else if( buffer && buffersz ){
    snprintf(buffer, buffersz, LONG_LONG_FTM ":" LONG_LONG_FTM, pid->pId, pid->tId);
  }
  return success;  
}
