/*
** **************************************************************************
** $Id: zkAuditHelper.h,v 1.2.30.1 2000/10/11 13:20:08 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

**
** Copyright (C) 1999 Zero-Knowledge Systems Inc.  All Rights Reserved
** **************************************************************************
*/

# ifndef ZKAUDITHELPER_H_
# define ZKAUDITHELPER_H_

# ifndef _ZKTYPES_H
# include <zkTypes.h>
# endif

# ifndef ZKERROR_H_
# include <zkError.h>
# endif

# ifndef ZKDATETIME_H_
# include <zkDateTime.h>
# endif

# ifndef ZKFILE_H_
# include <zkFile.h>
# endif

# ifdef __cplusplus
extern "C" {
# endif


/* --------------------------------------------------------------------------
 * Type definitions
 */

struct t_ZkAuditLog {
    const char *    audit_file;
    const char *    sid_file;          /* holds the last session id */
};

struct t_ZkAuditSession {
    ZkAuditLog      audit_log;    
    uint32_t        session_id;
    const char *    user;
    const char *    host;              /* let the function set this? */
    const char *    ticket_id;         /* may be NULL */
    const char *    client_id;         /* may be NULL */
    const char *    transaction_id;    /* may be NULL */
};

struct t_ZkAuditEntry {
    ZkDateTime      timestamp;
    const char *    user;
    const char *    host;
    uint32_t        session_id;
    const char *    service;
    const char *    request;
    const char *    comment;           /* may be NULL */
    const char *    ticket_id;         /* may be NULL */
    const char *    client_id;         /* may be NULL */
    const char *    transaction_id;    /* may be NULL */
    uint32_t        result_code;
    const char *    result_text;       /* may be NULL */
};

/* --------------------------------------------------------------------------
 * Function prototypes
 */

int
zkaudit_EntryCreate(
    ZK_INOUT  ZkError            err,
    ZK_IN     const ZkDateTime * timeStamp,
    ZK_IN     const char *       user,
    ZK_IN     const char *       host,
    ZK_IN     uint32_t           session_id,
    ZK_IN     const char *       service,
    ZK_IN     const char *       request,
    ZK_IN     const char *       comment,           /* may be NULL */
    ZK_IN     const char *       ticket_id,         /* may be NULL */
    ZK_IN     const char *       client_id,         /* may be NULL */
    ZK_IN     const char *       transaction_id,    /* may be NULL */
    ZK_IN     uint32_t           result_code,
    ZK_IN     const char *       result_text,       /* may be NULL */
    ZK_OUT    ZkAuditEntry *     audit_entry_ptr );

int
zkaudit_ParseQueryStr(
    ZK_INOUT  ZkError            err,
    ZK_IN     const char *       query_str,
    ZK_OUT    ZkAuditEntry *     audit_entry_ptr );

int
zkaudit_EntryCmp(
    ZK_INOUT  ZkError            err,
    ZK_IN     ZkAuditEntry       key,
    ZK_IN     ZkAuditEntry       entry,
    ZK_OUT    ZkBool*            match );

int
zkaudit_FopenLocked(
    ZK_INOUT  ZkError            err,
    ZK_IN     const char *       filename,
    ZK_IN     const char *       mode,
    ZK_OUT    FILE * *           file_ptr );

int
zkaudit_ConvertToBase64(
    ZK_INOUT  ZkError            err,
    ZK_IN     const char *       source,
    ZK_OUT    char **            dest );

int
zkaudit_ConvertFromBase64(
    ZK_INOUT  ZkError            err,
    ZK_IN     const char *       source,
    ZK_OUT    char **            dest );

int
zkaudit_WriteEntry(
    ZK_INOUT  ZkError            err,
    ZK_IN     ZkAuditSession     audit_session,
    ZK_OUT    ZkAuditEntry       audit_entry );

int
zkaudit_ReadEntry(
    ZK_INOUT  ZkError            err,
    ZK_IN     FILE *             fp,
    ZK_OUT    ZkAuditEntry *     audit_entry);

const char *
zkaudit_GetHostName(void); /* XXX - This function is not thread safe! */

int
zkaudit_GetNewSessionId(
    ZK_INOUT  ZkError            err,
    ZK_IN     ZkAuditLog         audit_log,
    ZK_OUT    uint32_t *         sid_ptr );

# ifdef __cplusplus
}
# endif

# endif /*  ZKAUDITHELPER_H_ */

