/*
 * $Id: zkauth.h,v 1.4.30.1 2000/10/11 13:20:11 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef _ZKAUTH_H
#define _ZKAUTH_H

#include "zks.h"

#include "zkIntTypes.h"
#include "zkTypes.h"

#include "zkConfig.h"
#include "zkError.h"
#include "zkLog.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define MAX_STR 100

/*
** token used in the passphrase/config file
*/
#define ZKAUTH_TOKEN_PASSPHRASE    "passphrase"

/*
** various error values
*/
#define ZKAUTH_ERR_NOACCESS -401


/*
 * Private structure
 */
typedef struct t_ZkAuthContext * ZkAuthContext;
typedef struct t_ZkAuthSession * ZkAuthSession;


/*
 * Public structure
 */
/* structure for user's personnal info */
typedef struct {
    char realName[MAX_STR];
    char title[MAX_STR];
    char dept[MAX_STR];
    char phone[MAX_STR];
    char email[MAX_STR];
} ZkAuthUserInfo;

/* entry structure */
typedef struct {
    char             user[MAX_STR];
    char             service[MAX_STR];
    char             operation[MAX_STR];
    ZkAuthUserInfo * userInfo;
} ZkAuthEntry;



/*
 * Init operation
 */
DLLExport int
zkAuthCreate(
	     ZkError          err,
	     const char *     dbFilename,
	     ZkAuthContext *  authContext
);

DLLExport int
zkAuthFree(
	   ZkError         err,
	   ZkAuthContext * authContext
);

/*
 * Authentication operation
 */
DLLExport int
zkAuthLogin(
	    ZkError         err,
	    ZkAuthContext   authContext,
	    const char *    userName,
	    const char *    userPassphrase,
	    ZkAuthSession * authSessionPtr
);

DLLExport int
zkAuthCheckAccess(
		  ZkError       err,
		  ZkAuthSession authSession,
		  const char *  reqService,
		  const char *  reqOperation,
		  ZkBool *      accessGranted
);

DLLExport int
zkAuthLogout(
	     ZkError         err,
	     ZkAuthSession * authSession
);


/*
 * Edition and query operation
 */
DLLExport int
zkAuthChangePassphrase(
		       ZkError        err,
		       ZkAuthSession  authSession,
		       char *	      newPassphrase
);

DLLExport int
zkAuthAdd(
	  ZkError        err,
	  ZkAuthSession	 authSession,
	  ZkAuthEntry *  authEntry
);

DLLExport int
zkAuthDelete(
	     ZkError        err,
	     ZkAuthSession  authSession,
	     ZkAuthEntry *  authEntry
);

DLLExport int
zkAuthQuery(
	    ZkError        err,
	    ZkAuthSession  authSession,
	    ZkAuthEntry *  authEntry
);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _ZKAUTH_H */
