/* **************************************************************************
** $Id: zkcpHash.c,v 1.3.22.1 2000/10/11 13:20:15 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file contains definitions and declarations for the Zero-Knowledge
** Command Protocol.
**
** **************************************************************************
*/

# include <string.h>
# include "zkcp_priv.h"

static const char * const
_module_id = "$Id: zkcpHash.c,v 1.3.22.1 2000/10/11 13:20:15 phil Exp $";


/* ------------------------------------------------------------------------
 *
 * pre-computes the initialization vectors for HMAC given the shared
 * secret (key)
 */
void
initialize_pad(
    LinkAuthPad *   pad,
    const void *    key,
    size_t          length )
{
    unsigned char truncated_key[ZKCP_HASH_SIZE];
    unsigned int  i;

    (void)_module_id; /* compiler complains about unreferenced variable */

    if (length > sizeof(pad->inner))
    {
        ZkcpHashContext hc;

        zkcpHash_Init(&hc);
        zkcpHash_Update(&hc,key,length);
        zkcpHash_Final(truncated_key, &hc);
        
        key    = truncated_key;
        length = ZKCP_HASH_SIZE;
    }

    memset(pad->inner, 0, sizeof(pad->inner));
    memset(pad->outer, 0, sizeof(pad->outer));

    memcpy(pad->inner, key, length);
    memcpy(pad->outer, key, length);

    for (i=0; i < sizeof(pad->inner); ++i)
    {
        pad->inner[i] ^= 0x36;
        pad->outer[i] ^= 0x5c;
    }
}


/* ------------------------------------------------------------------------
 *
 * This wrapper for SHA1_Update exists primarily so there is only one
 * occurence of the compiler warning that passing buffer to SHA1_Update
 * isn't const correct
 *
 */
void
zkcpHash_Update(
    ZkcpHashContext * hc,
    const void *      buffer,
    size_t            size )
{
    SHA1_Update(hc,buffer,size);
}
