/* **************************************************************************
** $Id: zkcpValue_Binary.c,v 1.6.22.1 2000/10/11 13:20:15 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file contains definitions and declarations for the Zero-Knowledge
** Command Protocol.
**
** **************************************************************************
*/

# include "zkcp_impl.h"
# include "zkcpValue_Binary.h"
# include "zkAlloc.h"
# include "zknbo.h"

static const char * const
_module_id = "$Id: zkcpValue_Binary.c,v 1.6.22.1 2000/10/11 13:20:15 phil Exp $";


/* ------------------------------------------------------------------------
 * Static and Member function prototypes
 */

static int value_type_is_Binary(
    ZK_IN    ZkcpValue             value);

static int constructor_type_is_Binary(
    ZK_IN    ZkcpValueConstructor   constructor);

static int zkcpValue_Binary_Destroy(
    ZK_INOUT ZkError                error,
    ZK_INOUT ZkcpValue *            value );

static int zkcpValue_Binary_GetTypeId(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValue              in_value,
    ZK_OUT   ZkcpTypeId *           out_type_id );

static int zkcpValue_Binary_GetSize(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValue              in_value,
    ZK_OUT   uint32_t *             out_size );

static int zkcpValue_Binary_WriteToBuffer(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValue              value,
    ZK_INOUT void *                 buffer,
    ZK_IN    size_t                 max_size,
    ZK_OUT   size_t *               bytes_written );

static int zkcpValueConstructor_Binary_Destroy(
    ZK_INOUT ZkError                error,
    ZK_INOUT ZkcpValueConstructor * constructor );

static int zkcpValueConstructor_Binary_GetTypeId(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValueConstructor   constructor,
    ZK_OUT   ZkcpTypeId *           out_type_id );

static int zkcpValueConstructor_Binary_FromBuffer(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValueConstructor   constructor,
    ZK_IN    const void *           in_buffer,
    ZK_IN    uint32_t               in_buffer_size,
    ZK_OUT   ZkcpValue *            out_new_value );

/* ------------------------------------------------------------------------
 * Type infomation and function table
 */

union T_PolyDatum {
    uint8_t  i8;
    uint16_t i16;
    uint32_t i32;
    uint64_t u64;
    float    f;
    double   d;
};


typedef struct T_ZkcpValue_Binary
{
    ZkcpValue_vtbl                  vtbl;
    size_t                          size;
    union T_PolyDatum               val[1];

} *
/* ---> */
/* ---> */ ZkcpValue_Binary;
/* ---> */


typedef ZkcpValueConstructor ZkcpValueConstructor_Binary;

static const struct
{
    ZkcpTypeId                                  type_id;
    const struct T_ZkcpValueConstructor_vtbl    constructor_vtbl;
    const struct T_ZkcpValue_vtbl               value_vtbl;
}
/* ---> */
/* ---> */ zkcpValue_Binary_TypeInfo =
/* ---> */
{
    ZKCP_TYPE_ID_Binary,
    {   zkcpValueConstructor_Binary_Destroy,
        zkcpValueConstructor_Binary_GetTypeId,
        zkcpValueConstructor_Binary_FromBuffer },
    {   zkcpValue_Binary_Destroy,
        zkcpValue_Binary_GetTypeId,
        zkcpValue_Binary_GetSize,
        zkcpValue_Binary_WriteToBuffer },
};



/* ------------------------------------------------------------------------
 *
 */
static int
value_type_is_Binary( ZkcpValue value )
{
    (void)_module_id; /* compiler complains about unreferenced variable */

    return (value->vtbl == &zkcpValue_Binary_TypeInfo.value_vtbl);
}


/* ------------------------------------------------------------------------
 *
 */
static int
constructor_type_is_Binary( ZkcpValueConstructor constructor )
{
    return (constructor->vtbl == &zkcpValue_Binary_TypeInfo.constructor_vtbl);
}



/* ------------------------------------------------------------------------
 *
 */
int
zkcpCommand_AddParamAsBinary(
    ZK_INOUT ZkError                    error,
    ZK_INOUT ZkcpCommand                command,
    ZK_IN    ZkcpParameterId            param_id,
    ZK_IN    const void *               buffer,
    ZK_IN    uint32_t                   size )
{
    const char * const func   = "zkcpCommand_AddParamAsBinary";
    ZkcpValue          value  = NULL;
    int                result =  0;

    if (command == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'command' is NULL");
    }
    else if ((buffer == NULL) && (size != 0))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'buffer' is NULL but 'size' isn't zero");
    }
    else if (zkcpValue_Binary_Create(error,buffer,size,&value)
             && zkcpCommand_AddParam(error,command,param_id,value))
    {
        value  = NULL;
        result = 1;
    }

    if (value != NULL)
    {
        zkcpValue_Destroy(NULL,&value);
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
int
zkcpCommand_GetParamAsBinary(
    ZK_INOUT ZkError                    error,
    ZK_INOUT ZkcpCommand                command,
    ZK_IN    ZkcpParameterId            param_id,
    ZK_OUT   const void * *             out_buffer,
    ZK_OUT   uint32_t *                 out_size )
{
    const char * const func   = "zkcpCommand_GetParamAsBinary";
    ZkcpValue          value  = NULL;
    int                result =  0;

    if (command == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'command' is NULL");
    }
    else if (out_buffer == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_number' is NULL");
    }
    else if (out_size == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_size' is NULL");
    }
    else if (zkcpCommand_GetParamById(error,command,param_id,&value)
             && zkcpValue_AsBinary(error,value,out_buffer,out_size))
    {
        result = 1;
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
int
zkcpValue_Binary_Create(
    ZK_INOUT ZkError                 error,
    ZK_IN    const void *            in_blob,
    ZK_IN    size_t                  size,
    ZK_OUT   ZkcpValue *             out_value )
{
    const char * const func   = "zkcpValue_Binary_Create";
    int                result =  0;

    if (out_value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_value' is NULL");
    }
    else if ((in_blob == NULL) && (size != 0))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'in_blob' is NULL but 'size' isn't zero");
    }
    else
    {
        ZkcpValue_Binary new_value = NULL;

        (*out_value) = NULL;

        if (zkMemAlloc( error,
                        (void**)&new_value,
                        sizeof(*new_value) + size ))
        {
            if (in_blob && size)
            {
                memcpy(new_value->val,in_blob, size);
            }

            new_value->vtbl = &zkcpValue_Binary_TypeInfo.value_vtbl;
            new_value->size = size;
            (*out_value)    = (ZkcpValue) new_value;
            result          = 1;
        }
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
int
zkcpValue_AsBinary(
    ZK_INOUT ZkError           error,
    ZK_IN    ZkcpValue         value,
    ZK_OUT   const void **     out_buffer,
    ZK_OUT   size_t *          out_size )
{
    const char * const func   = "zkcpValue_AsBinary";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (out_buffer == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_buffer' it NULL");
    }
    else if (!value_type_is_Binary(value))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "type mismatch");
    }
    else
    {
        ZkcpValue_Binary binary  = (ZkcpValue_Binary) value;

        (*out_buffer) = binary->size ? binary->val : NULL;
        (*out_size)   = binary->size;
        result        = 1;
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 */
static int zkcpValue_Binary_Destroy(
     ZK_INOUT ZkError       error,
     ZK_INOUT ZkcpValue *   value )
{
    const char * const func   = "zkcpValue_Binary_Destroy";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (*value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' doesn't point to anything");
    }
    else if (!value_type_is_Binary(*value))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "type mismatch (is vtbl set up properly?)");
    }
    else
    {
        zkMemFree(NULL,(void**)value);

        result = 1;
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValue_Binary_GetTypeId(
    ZkError       error,
    ZkcpValue     value,
    ZkcpTypeId *  out_type_id )
{
    const char * const func   = "zkcpValue_Binary_GetTypeId";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (out_type_id == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_type_id' is NULL");
    }
    else
    {
        if (!value_type_is_Binary(value))
        {
            zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                       "type mismatch (is vtbl set up properly?)");
        }
        else
        {
            (*out_type_id) = zkcpValue_Binary_TypeInfo.type_id;
            result         = 1;
        }
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValue_Binary_GetSize(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValue              value,
    ZK_OUT   uint32_t *             out_size )
{
    const char * const func   = "zkcpValue_Binary_GetSize";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (out_size == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_type_id' is NULL");
    }
    else
    {
        (*out_size) = 0;

        if (!value_type_is_Binary(value))
        {
            zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                       "type mismatch (is vtbl set up properly?)");
        }
        else
        {
            ZkcpValue_Binary p = (ZkcpValue_Binary) value;
            (*out_size)        = p->size;
            result             = 1;
        }
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValue_Binary_WriteToBuffer(
    ZK_INOUT ZkError            error,
    ZK_IN    ZkcpValue          value,
    ZK_INOUT void *             buffer,
    ZK_IN    size_t             max_size,
    ZK_OUT   size_t *           bytes_written )
{
    const char * const func   = "zkcpValue_Binary_WriteToBuffer";
    int                result = 0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (buffer == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'buffer' is NULL");
    }
    else if (bytes_written == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'bytes_written' is NULL");
    }
    else if (!value_type_is_Binary(value))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "type mismatch (is vtbl set up properly?)");
    }
    else
    {
        ZkcpValue_Binary data = (ZkcpValue_Binary) value;

        if (max_size < data->size)
        {
            zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                       "insufficient buffer size");
        }
        else
        {
            memcpy(buffer, data->val, data->size);

            (*bytes_written) = data->size;
            result           = 1;
        }
    }

    return (result);
}




/* ------------------------------------------------------------------------
 *
 */
int
zkcpValueConstructor_Binary_Create(
    ZK_INOUT ZkError                error,
    ZK_OUT   ZkcpValueConstructor * out_constructor )
{
    const char * const func   = "zkcpValueConstructor_Binary_Create";
    int                result =  0;

    if (out_constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_constructor' is NULL");
    }
    else
    {
        ZkcpValueConstructor_Binary new_constructor = NULL;

        (*out_constructor) = NULL;

        if (zkMemAlloc(error,(void**)&new_constructor,sizeof(*new_constructor)))
        {
            new_constructor->vtbl = &zkcpValue_Binary_TypeInfo.constructor_vtbl;
            (*out_constructor)    = (ZkcpValueConstructor) new_constructor;
            result                = 1;
        }
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValueConstructor_Binary_Destroy(
    ZK_INOUT ZkError                error,
    ZK_INOUT ZkcpValueConstructor * constructor )
{
    const char * const func   = "zkcpValueConstructor_Binary_Destroy";
    int                result =  0;

    if (constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' is NULL");
    }
    else if (*constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' doesn't point to anything");
    }
    else if (!constructor_type_is_Binary(*constructor))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "type mismatch (is vtbl set up properly?)");
    }
    else
    {
        result = zkMemFree(error,(void**)constructor);
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValueConstructor_Binary_GetTypeId(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValueConstructor   constructor,
    ZK_OUT   ZkcpTypeId *           out_type_id )
{
    const char * const func   = "zkcpValueConstructor_Binary_GetTypeId";
    int                result =  0;

    if (constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' is NULL");
    }
    else if (!constructor_type_is_Binary(constructor))
    {
      zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                 "type mismatch (is vtbl set up properly?)");
    }
    else
    {
      (*out_type_id) = zkcpValue_Binary_TypeInfo.type_id;
      result         = 1;
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValueConstructor_Binary_FromBuffer(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValueConstructor   constructor,
    ZK_IN    const void *           in_buffer,
    ZK_IN    size_t                 in_buffer_size,
    ZK_OUT   ZkcpValue *            out_new_value )
{
    const char * const func   = "zkcpValue_Binary_FromBuffer";
    int                result =  0;

    if (constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' is NULL");
    }
    else if ((in_buffer == NULL) && (in_buffer_size != 0))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'in_buffer' is NULL but 'in_buffer_size' isn't zero");
    }
    else if (out_new_value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_new_value' is NULL");
    }
    else 
    {
        result = zkcpValue_Binary_Create( error,
                                          in_buffer,
                                          in_buffer_size,
                                          out_new_value );
    }

    return (result);
}
