/* **************************************************************************
** $Id: zkcp_priv.h,v 1.4.12.1 2000/10/11 13:20:15 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file contains definitions and declarations for the Zero-Knowledge
** Command Protocol.
**
** **************************************************************************
*/

# ifndef zkcp_priv_h_INCLUDED
# define zkcp_priv_h_INCLUDED

# include "zkcp_impl.h"
# include "openssl/sha.h"

# define ZkcpHashContext                       SHA_CTX
# define ZKCP_HMAC_PAD_LENGTH                  (64)
# define ZKCP_HMAC_SIZE                        (14)
# define ZKCP_HASH_SIZE                        SHA_DIGEST_LENGTH

# define zkcpHash_Init                         SHA1_Init
# define zkcpHash_Final                        SHA1_Final


typedef struct {
    unsigned char          inner[ZKCP_HMAC_PAD_LENGTH];
    unsigned char          outer[ZKCP_HMAC_PAD_LENGTH];
} LinkAuthPad;

void initialize_pad(
    LinkAuthPad *                    pad,
    const void *                     key,
    size_t                           length );

void zkcpHash_Update(
    ZkcpHashContext *                hc,
    const void *                     buffer,
    size_t                           size );

int zkcpContext_LookupType(
    ZK_INOUT ZkError                 error,
    ZK_IN    ZkcpContext             context,
    ZK_IN    ZkcpTypeId              type_id,
    ZK_OUT   ZkcpValueConstructor *  constructor );

int zkcpConnection_Create(
    ZK_INOUT ZkError                 error,
    ZK_IN    ZkcpContext             context,
    ZK_IN    LinkAuthPad *           pad,
    ZK_IN    int                     current_pad,
    ZK_IN    void *                  io_context,
    ZK_IN    ZkcpCallback_Read       read_func,
    ZK_IN    ZkcpCallback_Write      write_func,
    ZK_OUT   ZkcpConnection *        out_connection );

# endif /* zkcp_priv_h_INCLUDED */
