/*
 * $Id: zkipCommand.c,v 1.4.32.1 2000/10/11 13:20:22 phil Exp $
 */

static const char * module_id_ = "$Id: zkipCommand.c,v 1.4.32.1 2000/10/11 13:20:22 phil Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <zkIntTypes.h>
#include <ZKIP.h>
#include "zkipCommand.h"

/**
 * 
 *
 *
 */

int
zkipCommandNew (
  ZkError err ,
  ZkipCommand * command )
{
  (void) module_id_;
  zkErrorClear(err);
  if (command) {
    *command = (ZkipCommand) malloc(sizeof(struct T_ZkipCommand));
    if (*command == NULL) {
      zkErrorSet(err,"zkipCommandNew",ZKERR,ZKIP,ZKENOMEM,NULL);
      return 0;
    }
    memset(*command,0,sizeof(struct T_ZkipCommand));
    return 1;
  }
  zkErrorSet(err,"zkipCommandNew",ZKERR,ZKIP,ZKENULLPTR,NULL);
  return 0;
}

/**
 * 
 *
 *
 */

int
zkipCommandFree (
  ZkError err ,
  ZkipCommand * command )
{
  ZkipCommand link;

  zkErrorClear(err);
  if (command) {
    if (*command) {
      do {
        link = (*command)->link;
        memset(*command,0,sizeof(struct T_ZkipCommand));
        free(*command);
        *command = link;
      } while (link != NULL);
      *command = NULL;
      return 1;
    }
  }
  zkErrorSet(err,"zkipCommandFree",ZKERR,ZKIP,ZKENULLPTR,NULL);
  return 0;
}

/**
 * 
 *
 *
 */

int
zkipCommandSet (
  ZkError err ,
  ZkipCommand command ,
  uint32_t cmdid ,
  ZkipCommandCallback callback ,
  const char * cmdstr ,
  uint16_t parms ,
  ZkipCommand link )
{
  zkErrorClear(err);
  command->callback = callback;
  command->cmdid = cmdid;
  command->parms = parms;
  strncpy(command->command,cmdstr,__COMMAND_SZ);
  command->command[__COMMAND_SZ] = (char) 0;
  command->link = link;
  return 1;
}

ZkipCommandCallback 
zkipCommandCallback (
  ZkError err ,
  ZkipCommand command )
{
  zkErrorClear(err);
  if (command) {
    return command->callback;
  }
  zkErrorSet(err,"zkipCommandCallback",ZKERR,ZKIP,ZKENULLPTR,NULL);
  return NULL;
}

const char * 
zkipCommandToken (
  ZkError err ,
  ZkipCommand command ) 
{
  zkErrorClear(err);
  if (command) {
    return command->command;
  }
  zkErrorSet(err,"zkipCommandToken",ZKERR,ZKIP,ZKENULLPTR,NULL);
  return NULL;
}

uint16_t
zkipCommandParms (
  ZkError err ,
  ZkipCommand command )
{
  zkErrorClear(err);
  if (command) {
    return command->parms;
  }
  zkErrorSet(err,"zkipCommandParms",ZKERR,ZKIP,ZKENULLPTR,NULL);
  return 0;
}

uint32_t
zkipCommandCmdid (
  ZkError err ,
  ZkipCommand command )
{
  zkErrorClear(err);
  if (command) {
    return command->cmdid;
  }
  zkErrorSet(err,"zkipCommandCmdid",ZKERR,ZKIP,ZKENULLPTR,NULL);
  return 0;
}


const ZkipCommand
zkipCommandNext (
  ZkError err ,
  ZkipCommand command )
{
  zkErrorClear(err);
  if (command) {
    return command->link;
  }
  zkErrorSet(err,"zkipCommandNext",ZKERR,ZKIP,ZKENULLPTR,NULL);
  return NULL;
}

