/*
 * $Id: zkipParse.c,v 1.7.8.1 2000/10/11 13:20:22 phil Exp $
 */

static const char * module_id_ = "$Id: zkipParse.c,v 1.7.8.1 2000/10/11 13:20:22 phil Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <zkIntTypes.h>
#include <ZKIP.h>
#include "zkipParse.h"

#if defined(ZK_LINUX) 
extern int strcasecmp( const char * , const char * );
#endif /* ZK_LINUX */


int
zkipParseInteger (
  ZkError err ,
  unsigned char * token ,
  uint32_t * value)
{
  int i = 0;

  (void) module_id_; 
  zkErrorClear(err);
  if ((token == NULL) || (value == NULL)) {
    zkErrorSet(err,"zkipParseInteger",ZKERR,ZKIP,ZKENULLPTR,NULL);
    return 0;
  }
  *value = 0;
  while (token[i] != CHAR_NUL) {
    if (isdigit(token[i])) {
      *value = (*value) * 10 + (token[i] - CHAR_ZERO);
    } else {
      *value = 0;
      zkErrorSet(err,"zkipParseInteger",ZKERR,ZKIP,ZKEPARSE,NULL);
      return 0;
    }
    i++;
  }
  return 1;
}
 
int
zkipParseCmdid (
  ZkError err ,
  unsigned char * token ,
  uint32_t *cmdid )
{
  int state = 0;
  int i = 0;
  uint32_t value = 0;

  zkErrorClear(err);
  if ((token == NULL) || (cmdid == NULL)) {
    zkErrorSet(err,"zkipParseCmdid",ZKERR,ZKIP,ZKENULLPTR,NULL);
    return 0;
  }
  while (state < 5) {
    switch(state) {
    case 0:
      if (isdigit(token[i])) {
        value = value * 10 + (token[i] - CHAR_ZERO);
      } else if (token[i] == CHAR_PERIOD) {
        *cmdid = (*cmdid) * 0x0100 + value;
        state = 1;
      } else {
        *cmdid = 0;
        state = 5;
      }
      break;
    case 1:
      if (isdigit(token[i])) {
        value = token[i] - CHAR_ZERO;
        state = 2;
      } else {
        *cmdid = 0;
        state = 5;
      }
      break;
    case 2:
      if (isdigit(token[i])) {
        value = value * 10 + (token[i] - CHAR_ZERO);
      } else if (token[i] == CHAR_PERIOD) {
        *cmdid = (*cmdid) * 0x0100 + value;
        state = 3;
      } else {
        *cmdid = 0;
        state = 5;
      }
      break;
    case 3:
      if (isdigit(token[i])) {
        value = token[i] - CHAR_ZERO;
        state = 4;
      } else {
        *cmdid = 0;
        state = 5;
      }
      break;
    case 4:
      if (isdigit(token[i])) {
        value = value * 10 + (token[i] - CHAR_ZERO);
      } else if (token[i] == CHAR_NUL) {
        *cmdid = (*cmdid) * 0x0100 + value;
        state = 6;
      } else {
        *cmdid = 0;
        state = 5;
      }
      break;
    }
    i += 1;
  }
  if (state == 5) {
    zkErrorSet(err,"zkipParseCmdid",ZKERR,ZKIP,ZKEPARSE,"CMDID");
    return 0;
  }
  return 1;
}

int
zkipParseEncoding (
  ZkError err ,
  unsigned char * token ,
  uint16_t * encoding)
{
  zkErrorClear(err);
  if ((token == NULL) || (encoding == NULL)) {
    zkErrorSet(err,"zkipParseEncoding",ZKERR,ZKIP,ZKENULLPTR,NULL);
    return 0;
  }

  if (strcasecmp((const char *)token,TOK_BASE64) == 0) {
    *encoding = ZKIP_ENCODING_BASE64; 
    return 1;
  } else if (strcasecmp((const char *)token,TOK_BINARY) == 0) { 
    *encoding = ZKIP_ENCODING_BINARY;
    return 1;
  } else if (strcasecmp((const char *)token,TOK_NONE) == 0) { 
    *encoding = ZKIP_ENCODING_NONE;
    return 1;
  } else if (strcasecmp((const char *)token,TOK_7BIT) == 0) { 
    *encoding = ZKIP_ENCODING_7BIT;
    return 1;
  } else if (strcasecmp((const char *)token,TOK_8BIT) == 0) { 
    *encoding = ZKIP_ENCODING_8BIT;
    return 1;
  } else if (strcasecmp((const char *)token,TOK_UNKNOWN) == 0) { 
    *encoding = ZKIP_ENCODING_UNKNOWN;
    return 1;
  } else if (strcasecmp((const char*)token,TOK_BZIP2) == 0) {
    *encoding = ZKIP_ENCODING_BZIP2;
    return 1;
  }
  zkErrorSet(err,"zkipParseEncoding",ZKERR,ZKIP,ZKENOTFOUND,
             (const char *)token);
  return 0;
}

