/*
 * $Id: zkEmailField.h,v 1.16.16.1 2000/10/11 13:20:27 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#ifndef ZKEMAILFIELD_H_
#define ZKEMAILFIELD_H_

#ifndef _ZKERROR_H
#include <zkError.h>
#endif /* _ZKERROR_H */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define FL_DELETE       0x1   /* Field which should be thrown away */
#define FL_RFC822       0x2   /* A Mail field described in RFC822 */
#define FL_MIME         0x4   /* A MIME field */
#define FL_ZKS_FREEDOM  0x8   /* A Zero-Knowledge Freedom product field */
#define FL_ZKS          0x10  /* Some other Zero-Knowledge mail field */
#define FL_UNKNOWN      0x20  /* Unknown email header field */
#define FL_RFC1327      0x40  /* A Mail extension field described in RFC1327 */
#define FL_EXTENSION    0x80  /* Some other extension field */
#define FL_RFC1945      0x100 /* An HTTP header field 
                               * (or http1.1, RFC 2068)
                               */
#define FL_COOKIE       0x200 /* HTTP cookie field (rfc2109) */
#define FL_RFC1036      0x400 /* USENET header fields */
#define FL_DONTSTRCMT   0x800 /* Dont strip the comments of this field */
#define FL_RFC2298      0x1000   /* A Message Disposition Notificaiton
                                  * (RFC2298) field)
                                  */

/*
 * Note on the FL_FONTSTRCMT flag.  This tells the mail parser to not
 * strip comments for particular fields.  It is currently only used for
 * Subject:
 */


#ifndef ZKEMAILFIELD_T
#define ZKEMAILFIELD_T
typedef struct T_ZkEmailField * ZkEmailField;
#endif /* ZKEMAILFIELD_T */

DLLExport void
zkMIMEresolveName (
  const unsigned char * name ,
  enum ZkMIMEFieldId * id,
  uint32_t * id_flags );

DLLExport int zkEmailFieldNew (
  ZkError ,
  ZkEmailField * );
 
DLLExport int zkEmailFieldFree (
  ZkError ,
  ZkEmailField * );

DLLExport int zkEmailFieldParse (
  ZkError ,
  ZkEmailField ,
  const unsigned char * );

DLLExport int zkEmailFieldAddParameter (
  ZkError ,
  ZkEmailField ,
  const unsigned char *,            /* Name of the parameter */
  const unsigned char * );          /* Value of the parameter */


/*
 * Accessor functions
 */

DLLExport int zkEmailFieldSetLink (
  ZkError ,
  ZkEmailField ,
  ZkEmailField );

DLLExport int zkEmailFieldLink (
  ZkError ,
  ZkEmailField ,
  ZkEmailField * );

DLLExport int zkEmailFieldDontStripComments (
  ZkError ,            /* Returns the error if any occured */
  ZkEmailField );      /* The field to modify */


DLLExport int zkEmailFieldDeleteById (
  ZkError ,             /* Returns the error if one occured */
  ZkEmailField * ,      /* Head of the field list */
  enum ZkMIMEFieldId ); /* Field ID to delete */

DLLExport int zkEmailFieldForceValue (
  ZkError ,                    /* Return the error if any occured */
  ZkEmailField ,               /* Initial field in the list to scan */
  enum ZkMIMEFieldId  ,        /* Field type whose value we wish to force */
  const char * ,               /* The string to which the value must be set */
  size_t );                    /* Size of the value string (not count NUL) */


DLLExport int zkEmailFieldSetName (
  ZkError ,
  ZkEmailField ,
  const unsigned char * );

DLLExport int zkEmailFieldName (
  ZkError ,
  ZkEmailField ,
  const unsigned char ** );

DLLExport int zkEmailFieldSetValue (
  ZkError ,
  ZkEmailField ,
  const unsigned char * );

DLLExport int zkEmailFieldValue (
  ZkError ,
  ZkEmailField ,
  const unsigned char ** );

DLLExport int zkEmailFieldId (
  ZkError ,
  ZkEmailField ,
  enum ZkMIMEFieldId * );

DLLExport int zkEmailFieldSetId (
  ZkError ,
  ZkEmailField ,
  enum ZkMIMEFieldId );

DLLExport int zkEmailFieldStrip (
  ZkError ,
  ZkEmailField * ,
  uint32_t );   /* The strip flags, see the zkMIME.h file for the list */

DLLExport int zkEmailFieldDelete (
  ZkError ,
  ZkEmailField * );

DLLExport int zkEmailFieldDump (
  ZkError ,
  ZkEmailField ,
  ZkEmailDumpCallback ,
  void * );

DLLExport int zkEmailFieldDumpCanonical (
  ZkError ,
  ZkEmailField ,
  ZkEmailDumpCallback ,
  void * );

DLLExport int zkEmailFieldGetParameter (
  ZkError ,                /* Returns the error if any occured */
  ZkEmailField ,           /* MIME field to analyze. */
  const unsigned char * ,  /* Name of the parameter */
  const unsigned char ** ); /* Returns a pointer to the value string */

DLLExport int zkEmailFieldDup (
  ZkError ,
  ZkEmailField ,
  ZkEmailField * );

DLLExport int zkEmailFieldIdFlags (
  ZkError ,
  ZkEmailField ,
  uint32_t * );


#ifdef __cplusplus
}
#endif /* __cplusplus */



#endif /* ZKEMAILFIELD_H_ */
