/*
 * $Id: zkHTTPHeader.h,v 1.9.10.1 2000/10/11 13:20:27 phil Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#ifndef ZKHTTPHEADER_H_
#define ZKHTTPHEADER_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define UNSPECIFIED_HEADER 0
#define RESPONSE_HEADER  1
#define REQUEST_HEADER 2

#ifndef ZKHTTPHEADER_T
#define ZKHTTPHEADER_T
typedef struct T_ZkEmailHeader * ZkHTTPHeader;
#endif /* ZKHTTPHEADER_T */


DLLExport int zkHTTPHeaderNew (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader * header );

DLLExport int zkHTTPHeaderFree (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader * );                   /* Returns a new HTTP header struct */

DLLExport int zkHTTPHeaderStatus (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* Header to scan */
  int * );                            /* Returns the parse status */

DLLExport int zkHTTPHeaderCookies (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* The header to analyze */
  int * );                            /* Returns number of cookies in header */

DLLExport int zkHTTPHeaderCookie (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* The header to analyze */
  int ,                               /* The cookie index (first = 0) */
  ZkHTTPField * );                    /* Returns the (Set-)Cookie field */

DLLExport int zkHTTPHeaderForceReferer (
  ZkError ,                           /* Return the error if any occured */
  ZkHTTPHeader ,                      /* Header to modify */
  const char * );                     /* New referer field value */


DLLExport int zkHTTPHeaderForceFieldValue (
  ZkError ,                        /* Returns the error if any occured */
  ZkHTTPHeader ,                   /* Header whose fields you wish to change*/
  enum ZkMIMEFieldId ,             /* Type of field to change */
  const char * ,                   /* Value to set these fields to */
  size_t );                        /* Size of the value string */


DLLExport int zkHTTPHeaderKeepBodyData (
  ZkError ,
  ZkHTTPHeader );

DLLExport int zkHTTPHeaderDontKeepBodyData (
  ZkError ,
  ZkHTTPHeader );

DLLExport int zkHTTPHeaderType (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* Header to scan */
  int * );                            /* Returns header type (see list above) */

DLLExport int zkHTTPHeaderSetType (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* Header to identify */
  int );                              /* Header type (see list above) */

DLLExport int zkHTTPHeaderAddText (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* Header to which to add the text */
  unsigned char * ,                   /* Text to add */
  size_t );                           /* Number of bytes to add */

DLLExport int zkHTTPHeaderResetFieldSearch (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader );                     /* Header to reset */

DLLExport int zkHTTPHeaderParse (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader );                     /* Header to parse */


DLLExport int zkHTTPHeaderAddField (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* HTTP header to modify */
  ZkHTTPField );                      /* Field to add to HTTP header */

DLLExport int zkHTTPHeaderStripField (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* HTTP header to strip */
  uint32_t );

DLLExport int zkHTTPHeaderStripFieldByID (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* HTTP header to strip */
  enum ZkMIMEFieldId );

DLLExport int zkHTTPHeaderAddFieldData (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* Header to which to add the field */
  const unsigned char * ,             /* Name of the field */
  const unsigned char * );            /* Value of the field */

DLLExport int zkHTTPHeaderFirstField (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* HTTP header to search */
  ZkHTTPField * );                    /* Returns the first field */

DLLExport int zkHTTPHeaderGetField (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* Header to search */
  enum ZkMIMEFieldId ,                /* ID of the field to get */
  ZkHTTPField * );                    /* Returns the field */

DLLExport int zkHTTPHeaderNextField (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* Header to search */
  ZkHTTPField * );                    /* Returns the field */

DLLExport int zkHTTPHeaderDump (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* Header to dump */
  ZkEmailDumpCallback ,               /* Dump call back */
  void * );                           /* Callback's extra data pointer */

DLLExport int zkHTTPBodyDump (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* Header to dump */
  ZkEmailDumpCallback ,               /* Dump call back */
  void * );                           /* Callback's extra data pointer */

DLLExport int zkHTTPHeaderDup (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* Header to duplicate */
  ZkHTTPHeader * );                   /* Returns the HTTP header copy */

DLLExport int zkHTTPHeaderSetRequest (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* Header for which to set request */
  const char * );                     /* Request or response test */

DLLExport int zkHTTPHeaderRequest (
  ZkError ,                           /* Returns the error if any occured */
  ZkHTTPHeader ,                      /* Header to analyze */
  const char ** );                    /* Returns pointer to request/response */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZKHTTPHEADER_H_ */

