/*
 * $Id: zkEmailField.c,v 1.34.16.5 2000/12/14 17:54:33 jean Exp $
 */

static const char * module_id_ = "$Id: zkEmailField.c,v 1.34.16.5 2000/12/14 17:54:33 jean Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */



#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>        /* ctype functions on solaris in stdlib.h */
#include <zkIntTypes.h>
#include <zkMIME.h>
#include <zkCStr.h>
#include "zkEmailParse.h"
#include "zk_mime.h"

/*
 * If you want everything converted to upper case, define this
 */

#if 0
#define CVRT_FLD_TO_UPPER_CASE 
#endif

/*
 * If you're going to be screwing around with these structs, a) dont remove
 * any fields.  b) if you add fields, make sure that you update the ...New()
 * ...Free(), and ...Dup() functions.  You should also add accessors for fields
 * that will be required outside of this particular file.
 */

struct T_ZkEmailFieldParameter {
  unsigned char * name;
  unsigned char * value;
  struct T_ZkEmailFieldParameter * link;
};

typedef struct T_ZkEmailFieldParameter * ZkEmailFieldParameter;

struct T_ZkEmailField {
  int flags;
  unsigned char * name;
  unsigned char * value;
  uint32_t id_flags; /* Copied from the FieldLabel array when id is set */
  enum ZkMIMEFieldId id;
  ZkEmailFieldParameter parm; 
  struct T_ZkEmailField * prev;
  struct T_ZkEmailField * link;
};

/*
 * Setting FL_DONTSTRCMT will prevent the comments from being automatically 
 * stripped from a field value
 */

/*
 * Yes I know this is not really the best way, but I dont have any time here
 */

struct T_FieldLabel {
  uint32_t flags;
  const char * name;
  enum ZkMIMEFieldId id;
} FieldLabel[] = {
  {(FL_DELETE | FL_UNKNOWN), "UNKNOWNFIELD", UnknownField },
  {(FL_DELETE | FL_EXTENSION), "X-", Extension },
  {(FL_DELETE | FL_RFC822), "Return-Path",Return_Path },
  {(FL_DELETE | FL_RFC1036) , "Path" , Path },
  {(FL_DELETE | FL_RFC822), "Received", Received },
  {(FL_DELETE | FL_RFC822 | FL_RFC1036), "Reply-To", Reply_To },
  {(FL_DELETE | FL_RFC822 | FL_RFC1036), "Sender", Sender },
  {(FL_RFC1036) , "Followup-To",Followup_To },
  {(FL_RFC822 | FL_RFC1945 | FL_RFC1036 ), "From", From },
  {(FL_DELETE | FL_RFC822), "Resent-From", Resent_From },
  {(FL_DELETE | FL_RFC822), "Resent-Sender", Resent_Sender },
  {(FL_DELETE | FL_RFC822 | FL_RFC1036 ), "Resent-Date", Resent_Date },
  {(FL_RFC822), "To", To },
  {(FL_DELETE | FL_RFC822), "Resent-To", Resent_To },
  {(FL_RFC822), "CC", cc },
  {(FL_DELETE | FL_RFC822), "Resent-CC", Resent_cc },
  {(FL_RFC822),"BCC", bcc },
  {(FL_DELETE | FL_RFC822), "Resent-BCC", Resent_bcc },
  {(FL_DELETE | FL_RFC822 | FL_RFC1036), "Message-ID", Message_ID },
  {(FL_DELETE | FL_RFC822), "Resent-Message-ID", Resent_Message_ID },
  {(FL_DELETE | FL_RFC822), "In-Reply-To", In_Reply_To },
  {(FL_RFC822 | FL_RFC1036), "References", References },
  {(FL_DELETE | FL_RFC822 | FL_RFC1036), "Keywords", Keywords },
  {(FL_RFC1036) , "Summary", Summary },
  {(FL_RFC1036) , "Approved", Approved },
  {(FL_RFC1036) , "Lines", Lines },
  {(FL_RFC1036) , "Xref", XRef },
  {(FL_RFC822 | FL_RFC1036 | FL_DONTSTRCMT), "Subject", Subject },
  {(FL_DELETE | FL_RFC822), "Comments", Comments },
  {(FL_DELETE | FL_RFC822), "Encrypted", Encrypted },
  {(FL_RFC1036) , "Control",Control }, /* Should I prevent these from going out? */
  {(FL_RFC1036) , "Distribution", Distribution },
  {(FL_DELETE | FL_RFC822), "Extension", Extension },
  {(FL_DELETE | FL_RFC1036) , "Organization", Organization },
  {(FL_MIME), "MIME-Version", MIME_Version },
  {(FL_MIME|FL_RFC1945), "Content-Type", Content_Type },
  {(FL_MIME), "Content-Transfer-Encoding", Content_Transfer_Encoding },
  {(FL_MIME), "Content-Disposition", Content_Disposition },
  {(FL_MIME), "Content-Base", Content_Base },
  {(FL_MIME), "Content-Conversion", Content_Conversion }, 
  {(FL_MIME), "Content-Description", Content_Description },
  {(FL_MIME), "Content-ID", Content_ID }, 
  {(FL_MIME | FL_RFC1945), "Content-Language", Content_Language }, 
  {(FL_MIME|FL_RFC1945), "Content-Length", Content_Length },        
  {(FL_MIME), "Content-Location", Content_Location },
  {(FL_MIME), "Content-MD5", Content_MD5 }, 
  {(FL_MIME), "Content-Return", Content_Return }, 
  {(FL_MIME), "Content-SGML-Entity", Content_SGML_Entity },
  {(FL_ZKS_FREEDOM), "X-ZKS-Freedom-Comment", X_ZKS_Freedom_Comment},
  {(FL_ZKS_FREEDOM), "X-ZKS-Freedom-MessageID", X_ZKS_Freedom_MessageID},
  {(FL_DELETE | FL_RFC1327), "Autoforwarded", Autoforwarded },
  {(FL_DELETE | FL_RFC1327), "Content-Identifier", Content_Identifier },
  {(FL_DELETE | FL_RFC1327), "Conversion", Conversion },
  {(FL_DELETE | FL_RFC1327), "Conversion-With-Loss", Conversion_With_Loss },  
  {(FL_DELETE | FL_RFC1327), "Deferred-Delivery", Deferred_Delivery },
  {(FL_DELETE | FL_RFC1327), "Delivery-Date", Delivery_Date },
  {(FL_DELETE | FL_RFC1327), "Discarded-X400-IPMS-Extensions", Discarded_X400_IPMS_Extensions },
  {(FL_DELETE | FL_RFC1327), "Discarded-X400-MTS-Extensions", Discarded_X400_MTS_Extensions },
  {(FL_DELETE | FL_RFC1327), "DL-Expansion-History", DL_Expansion_History },
  {(FL_DELETE | FL_RFC1327), "Expiry-Date", Expiry_Date },
  {(FL_DELETE | FL_RFC1327), "Importance", Importance },
  {(FL_DELETE | FL_RFC1327), "Incomplete-Copy", Incomplete_Copy },
  {(FL_DELETE | FL_RFC1327), "Language", Language },
  {(FL_DELETE | FL_RFC1327), "Latest-Delivery-Time", Latest_Delivery_Time },
  {(FL_DELETE | FL_RFC1327), "Message-Type", Message_Type },
  {(FL_DELETE | FL_RFC1327), "Obsoletes", Obsoletes },
  {(FL_DELETE | FL_RFC1327), "Original-Encoded-Information-Types", Original_Encoded_Information_Types },
  {(FL_DELETE | FL_RFC1327), "Originator-Return-Address", Originator_Return_Address },
  {(FL_DELETE | FL_RFC1327), "Priority", Priority },
  {(FL_DELETE | FL_RFC1327), "Reply-By", Reply_By },
  {(FL_DELETE | FL_RFC1327), "Requested-Delivery-Method", Requested_Delivery_Method },
  {(FL_DELETE | FL_RFC1327), "Sensitivity", Sensitivity },
  {(FL_DELETE | FL_RFC1327), "X400-Content-Type", X400_Content_Type },
  {(FL_DELETE | FL_RFC1327), "X400-MTS-Identifier", X400_MTS_Identifier },
  {(FL_DELETE | FL_RFC1327), "X400-Originator", X400_Originator },
  {(FL_DELETE | FL_RFC1327), "X400-Received", X400_Received },
  {(FL_DELETE | FL_RFC1327), "X400-Recipients", X400_Recipients  },
  {(FL_RFC1945), "Date" , Date },
  {(FL_RFC1945), "Pragma", Pragma },
  {(FL_RFC1945), "Authoriazation", Authorization },
  {(FL_DELETE | FL_RFC1945), "If-Modified-Since", If_Modified_Since },
  {(FL_DELETE | FL_RFC1945), "If-Unmodified-Since", If_Unmodified_Since },
  {(FL_RFC1945), "Referer", Referer },
  {(FL_RFC1945), "User-Agent", User_Agent },
  {(FL_RFC1945), "Location", Location },
  {(FL_RFC1945), "Server", Server },
  {(FL_RFC1945), "WWW-Authenticate", WWW_Authenticate },
  {(FL_RFC1945), "Allow", Allow },
  {(FL_RFC1945), "Content-Encoding", Content_Encoding },
  {(FL_RFC1945 | FL_RFC1036), "Expires", Expires }, 
  {(FL_RFC1945), "Last-Modified", Last_Modified },
  {(FL_RFC1945), "Accept", Accept },
  {(FL_RFC1945), "Accept-Charset", Accept_Charset },
  {(FL_RFC1945), "Accept-Encoding", Accept_Encoding },
  {(FL_RFC1945), "Accept-Language", Accept_Language },
  {(FL_RFC1945), "Host", Host },
  {(FL_RFC1945), "Link", Link },
  {(FL_RFC1945), "Retry-After", Retry_After },
  {(FL_RFC1945), "Title", Title },
  {(FL_RFC1036 | FL_DELETE), "Newsgroups", Newsgroups },
  {(FL_RFC1945), "URI", URI },
  {(FL_COOKIE | FL_DELETE) , "Cookie" , Cookie },
  {(FL_COOKIE | FL_DELETE) , "Set-Cookie" , Set_Cookie },
  {(FL_RFC1945 | FL_DELETE), "If-Match" , If_Match },
  {(FL_RFC1945 | FL_DELETE), "If-None-Match" , If_None_Match },
  /* E-mail notification fields */

  {(FL_DELETE | FL_RFC2298), "Disposition", Disposition },
  {(FL_DELETE | FL_RFC2298), "Disposition-Notification-To", Disposition_Notification_To },
  {(FL_DELETE | FL_RFC2298), "Disposition-Notification-Options", Disposition_Notification_Options },
  {(FL_DELETE | FL_RFC2298), "Reporting-UA", Reporting_UA },
  {(FL_DELETE | FL_RFC2298), "MDN-Gateway", MDN_Gateway },
  {(FL_DELETE | FL_RFC2298), "Original-Message-ID", Original_Message_ID },
  {(FL_DELETE | FL_RFC2298), "Original-Recipient", Original_Recipient },
  {(FL_DELETE | FL_RFC2298), "Final-Recipient", Final_Recipient },

  /* better add these at the end otherwise dire consequences might ensue */

  {(FL_ZKS_FREEDOM), "X-ZKS-Freedom-Hdr-Sig", X_ZKS_Freedom_Header_Sig},
  {(FL_ZKS_FREEDOM), "X-ZKS-Freedom-Auth-From", X_ZKS_Freedom_Auth_From},
  {(FL_ZKS_FREEDOM), "X-ZKS-Freedom-Auth-To", X_ZKS_Freedom_Auth_To},
  {(FL_ZKS_FREEDOM), "X-ZKS-Freedom-Auth-Cc", X_ZKS_Freedom_Auth_Cc},
  {(FL_ZKS_FREEDOM), "X-ZKS-Freedom-Auth-Newsgroups", X_ZKS_Freedom_Auth_Newsgroups},
  {(FL_ZKS_FREEDOM), "X-ZKS-Freedom-Auth-Date", X_ZKS_Freedom_Auth_Date}
};

/*
 * Internal function prototypes
 */

static int stripComments ( 
  unsigned char * );

static int zkEmailFieldParameterDup ( 
  ZkError , 
  ZkEmailFieldParameter ,
  ZkEmailFieldParameter * );


/*
 *  Convert a string name for a field to a field ID
 */

DLLExport void
zkMIMEresolveName (
  const unsigned char * name ,
  enum ZkMIMEFieldId * id,
  uint32_t * id_flags )
{
  int i = 0; /* Loop index */
  int labels = sizeof(FieldLabel) / sizeof(struct T_FieldLabel);

     
  for (i = 0; i < labels; i++) {
    if (toupper(name[0]) == toupper(FieldLabel[i].name[0])) {
      if (strcasecmp((const char *) name,FieldLabel[i].name) == 0) {
        *id = FieldLabel[i].id;
        *id_flags = FieldLabel[i].flags;
        return;
      }
    }
  }

  if ((name[0] == 'X') || (name[0] == 'x')) {
    if (name[1] == '-') {
      *id = Extension;
      *id_flags = (FL_DELETE|FL_EXTENSION);
      return;
    }
  }

  *id = UnknownField;
  *id_flags = (FL_DELETE | FL_UNKNOWN);
  return;
}


/*
 * Remove all of the comments from an RFC822 field.  This is essentially 
 * anything which is surrounded by parenthesis
 */

static int
stripComments (
  unsigned char * value )
{
  unsigned char * t = value;
  int ret = 1;
  int depth = 0;
  int quotemode = 0;

  while (*value != CHAR_NUL) {
    switch (*value) {
    case CHAR_SOLIDUS:
      if (depth == 0) {
        *t++ = *value;
      } if (value[1] != CHAR_NUL) {
        value++;
        if (depth == 0) {
          *t++ = *value;
        }
      }
      break;  
    case CHAR_DQUOTE:
      *t++ = *value;
      if (quotemode) {
        quotemode = 0;
      } else {
        quotemode = 1;
      }
      break;
    case CHAR_OPENPAREN:
      if (!quotemode) {
        depth++;
      } else {
        *t++ = *value;
      }
      break;
    case CHAR_CLOSEPAREN:
      if (!quotemode) {
        if ((--depth) < 0) {
          return 0;              /* Too many close parenthesis, error */
        }
      } else {
        *t++ = *value;
      }
      break;
    default:
      if (depth == 0) {
        *t++ = *value;
      }
      break;
    }
    value++; 
  }
  *t = CHAR_NUL;
  return ret;
}

/*
 * Create a new field parameter object.  This object is internal to this 
 * file.
 */

static int
zkEmailFieldParameterNew (
  ZkError err ,
  ZkEmailFieldParameter * parm )
{
  (void) module_id_;
  zkErrorClear(err);
  *parm = (ZkEmailFieldParameter) 
          malloc(sizeof(struct T_ZkEmailFieldParameter));
  if (*parm == NULL) {
    zkErrorSet(err,"zkEmailFieldParameterNew",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
    return 0;
  }
  memset(*parm,0,sizeof(struct T_ZkEmailFieldParameter));
  (*parm)->name = NULL;
  (*parm)->value = NULL;
  (*parm)->link = NULL;
  return 1;
}

/*
 * Destroy the zkEmailFieldParameter
 */

static int
zkEmailFieldParameterFree (
  ZkError err ,
  ZkEmailFieldParameter * parm )
{
  const char * const func = "zkEmailFieldParameterFree";

  if (parm == NULL) {
    zkErrorSet(err,func,ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
    return 0;
  }

  zkErrorClear(err);
  if ((*parm)->name != NULL) {  
    free((*parm)->name);
  }
  if ((*parm)->value != NULL) {
    free((*parm)->value);
  }
  memset(*parm,0,sizeof(struct T_ZkEmailFieldParameter));
  free(*parm);
  *parm = NULL;
  return 1;
}

static int
zkEmailFieldParameterName (
  ZkError err,
  ZkEmailFieldParameter parm,
  const unsigned char ** name )
{
  zkErrorClear(err);
  *name = parm->name;
  return 1;
}

static int
zkEmailFieldParameterValue (
  ZkError err,
  ZkEmailFieldParameter parm,
  const unsigned char ** value )
{
  zkErrorClear(err);
  *value = parm->value;
  return 1;
}


static int
zkEmailFieldParameterSetValue (
  ZkError err ,
  ZkEmailFieldParameter parm,
  const unsigned char * value )
{
  zkErrorClear(err);
  if (parm->value != NULL) {
    free(parm->value);
  }
  parm->value = (unsigned char *)strdup((const char *) value);
  if (parm->value == NULL) {
    zkErrorSet(err,"zkEmailFieldParameterSetValue",ZKERR,ZKMEMAIL,
               ZKENOMEM,"strdup");
    return 0;
  }
  return 1;
}

static int
zkEmailFieldParameterSetName (
  ZkError err ,
  ZkEmailFieldParameter parm,
  const unsigned char * name)
{
  zkErrorClear(err);
  if (parm->name != NULL) {
    free(parm->name);
  }
  parm->name = (unsigned char *)strdup((const char *) name);
  if (parm->name == NULL) {
    zkErrorSet(err,"zkEmailFieldParameterSetName",ZKERR,ZKMEMAIL,
               ZKENOMEM,"strdup");
    return 0;
  }
  return 1;
}


/**
 *  Delete the specified zkEmailField.  Return a new pointer so that the
 *  field list can be maintained.  You should not call this function
 *  yourself, call the zkEmailHeader..() field deletion functions instead
 *  because when a field is deleted from the header, some additional work
 *  needs to be done.
 *
 */

DLLExport int
zkEmailFieldDelete (
  ZkError err ,
  ZkEmailField * field )
{
  ZkEmailField newptr = NULL;
  zkErrorClear(err);
  if (field) {
    if (*field) {
      if ((*field)->prev != NULL) {
        newptr = (*field)->prev;
        newptr->prev = (*field)->prev;
      } else if ((*field)->link != NULL) {
        newptr = (*field)->link;
        newptr->prev = NULL; 
      } else {
        newptr = NULL;
      }
      (*field)->link = NULL;
      (*field)->prev = NULL;
      zkEmailFieldFree(err,field);
      *field = newptr;
      return 1;
    }
  }
  zkErrorSet(err,"zkEmailFieldDelete",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
  return 0;
} 
  
  
/**
 *  Create a new ZkEmailField structure.
 *
 *  @param err Returns the error if one occured
 *  @param field Returns a new ZkEmailField object
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldNew (
  ZkError err ,
  ZkEmailField * field )
{
  zkErrorClear(err);
  if (field) {
    *field = (ZkEmailField) malloc(sizeof(struct T_ZkEmailField));
    if (*field == NULL) {
      zkErrorSet(err,"zkEmailFieldNew",ZKERR,ZKMEMAIL,ZKENOMEM,"malloc");
      return 0;
    }
    memset(*field,0,sizeof(struct T_ZkEmailField));
    (*field)->flags = 0;
    (*field)->name = NULL;
    (*field)->value = NULL;
    (*field)->id = (enum ZkMIMEFieldId) UnknownField;
    (*field)->id_flags = FL_DELETE | FL_UNKNOWN;
    (*field)->prev = NULL;
    (*field)->link = NULL;
    (*field)->parm = NULL;
    return 1;
  }
  zkErrorSet(err,"zkEmailField",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
  return 0;
}

/** 
 *  Destroy a ZkEmailField structure.
 *
 *  @param err Returns the error if one occured
 *  @param field The ZkEmailField object to destroy.  The pointer will be
 *               reset to NULL on success
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldFree (
  ZkError err ,
  ZkEmailField * field )
{
  ZkEmailFieldParameter parm = NULL;

  zkErrorClear(err);
  if (field) {
    if (*field) {
      if ((*field)->name) {
        free((*field)->name);
      }
      if ((*field)->value) {
        free((*field)->value);
      }
      while ((*field)->parm != NULL) {
        parm = ((ZkEmailFieldParameter)(*field)->parm)->link;
        zkEmailFieldParameterFree(err,&((*field)->parm));
        (*field)->parm = parm;
      }
      memset(*field,0,sizeof(struct T_ZkEmailField));
      free(*field);
      *field = NULL;
      return 1;
    }
  }
  zkErrorSet(err,"zkEmailField",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
  return 0;
}

/**
 *  Dont automatically strip comments in the field value.
 *
 *  @param err Returns the error if one occured
 *  @param field The field to modify
 *  @return A non-zero value on success, a 0 on failure
 *  @version 1.0
 *  @author Philippe Boucher
 */

DLLExport int
zkEmailFieldDontStripComments (
  ZkError err ,
  ZkEmailField field )
{
  zkErrorClear(err);
  if (field == NULL) {
    zkErrorSet(err,"zkEmailFieldDontStripComments",ZKERR,ZKMEMAIL,
               ZKENULLPTR,NULL);
    return 0;
  }
  field->flags |= FL_DONTSTRCMT;
  return 1;
}

/**
 *  Retrieve a MIME parameter from the given field object.
 * 
 *  @param err Returns the error if one occured
 *  @param field The ZkEmailField to analyze
 *  @param name The name of the parameter you want (NUL terminated string)
 *  @param value Returns the value of that parameter (A pointer to a NUL
 *               terminated string contained within the ZkEmailField.  dont
 *               mess with it)
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success ( the parameter was found) or a 0
 *          on failure (the parameter was not found)
 */

DLLExport int
zkEmailFieldGetParameter (
  ZkError err ,
  ZkEmailField field, 
  const unsigned char * name ,
  const unsigned char ** value )
{
  ZkEmailFieldParameter parm = NULL;
  const unsigned char * pname = NULL;

  zkErrorClear(err);
  if ((field == NULL) || (name == NULL) || (value == NULL)) {
    zkErrorSet(err,"zkEmailFieldGetParameter",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
    return 0;
  }
  parm = field->parm;
  while (parm != NULL) {
    zkEmailFieldParameterName(err,parm,&pname);
    if (strcasecmp((const char *) name,(const char *) pname) == 0) {
      zkEmailFieldParameterValue(err,parm,(const unsigned char **) value);
      return 1;
    }
    parm = parm->link;
  }
  zkErrorSet(err,"zkEmailFieldGetParameter",ZKERR,ZKMEMAIL,ZKENOTFOUND,NULL);
  return 0;
}

/**
 *  This returns the link to the next field in a field list (ZkEmailFields are
 *  stored as lists).  Do not use this function directly.  In order to search
 *  fields in a mail header use the zkEmailHeaderFirstField(), 
 *  zkEmailHeaderNextField(), and zkEmailHeaderGetField() functions.
 * 
 *  @param err Returns the error if one occured
 *  @param field The field object to modify
 *  @param link Returns the pointer to the next field in the field list
 *  @version 1.0
 *  @author Philippe Boucher
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldLink (
  ZkError err ,
  ZkEmailField field ,
  ZkEmailField * link )
{
  zkErrorClear(err);
  if ((field) && (link)) {
    *link = field->link;
    return 1; 
  }
  zkErrorSet(err,"zkEmailFieldLink",ZKERR,ZKMEMAIL,ZKENULLPTR,
             "Null field or link pointer");
  return 0;
}

/**
 *  This function is used to set the link to the next field in the field list.
 *  Do not use this function directly, instead, to add a field to a mail
 *  header, use the zkEmailHeaderAddField() or zkEmailHeaderAddFieldData()
 *  functions.
 * 
 *  @param err Returns the error if one occured
 *  @param field The ZkEmailField to modify
 *  @param link The next email field in the list
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldSetLink (
  ZkError err ,
  ZkEmailField field ,
  ZkEmailField link )
{
  zkErrorClear(err);
  if (field) {
    field->link = link;
    if (link) {
      link->prev = field;
    }  
    return 1;
  }
  zkErrorSet(err,"zkEmailFieldSetLink",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
  return 0;
}



/**
 *  This function scans the field list for fields of the specified field
 *  id and replaces their values with the value string specified.  If the
 *  field add any parameters, those will be deleted.  Use this function when
 *  you wish to force the value of a field in a header only if that field
 *  is already there.  This function was written to provide a clean way
 *  of forcing the HTTP Referer field to be www.freedom.net when it is
 *  present in an HTTP transaction.
 *
 *  @param err Returns the error if any occured
 *  @param field The zkEmailField list to scan
 *  @param id The target field type
 *  @param value The value string
 *  @param size The size of the value string, not counting the string's 
 *              terminating NUL character.
 *  @version 1.0
 *  @author Philippe Boucher
 *  @return a zero value on failure, a non-zero value on success
 */

DLLExport int
zkEmailFieldForceValue (
  ZkError err ,
  ZkEmailField field ,
  enum ZkMIMEFieldId id ,
  const char * value ,
  size_t size )
{
  ZkEmailFieldParameter parm = NULL;

  zkErrorClear(err);
  if (field && value) {
   
    /*
     * Scan all of the fields
     */

    while (field) {
   
      /*
       * If this is a field we're targeting, nuke its value and replace it
       */

      if (field->id == id) {
        if (field->value) {
          free(field->value);
        }
        field->value = (unsigned char *) malloc(size + 1);
        if (field->value == NULL) {
          zkErrorSet(err,"zkEmailFieldForceValue",ZKERR,ZKMEMAIL,ZKENOMEM,
                     "malloc");
          return 0;
        }
        memcpy(field->value,value,size);
        field->value[size] = (char) 0;

        /*
         * Get rid of any field parameters
         */

        while (field->parm != NULL) {
          parm = ((ZkEmailFieldParameter)field->parm)->link;
          zkEmailFieldParameterFree(err,&(field->parm));
          field->parm = parm;
        }

      } 
      field = field->link;
    }
    return 1;
  }
  zkErrorSet(err,"zkEmailFieldForceValue",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
  return 0;
}
  

/**
 *  Remove fields which have been marked as potentially compromising.
 *  Use the function zkEmailHeaderStripField() instead in order to clean
 *  up email headers.  The flags can be set to 0, DONT_STRIP_UNKNOWN,
 *  or DONT_STRIP_NEWSGROUPS (for Mario)
 * 
 *  @param err Returns the error if one occured
 *  @param head The first field in the ZkEmailField list to strip
 *  @param flags Controls what gets stripped.  See zkMIME.h for the flags
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure 
 */


DLLExport int
zkEmailFieldStrip (
  ZkError err ,
  ZkEmailField * head,
  uint32_t flags )
{
  ZkEmailField field = NULL;
  ZkEmailField *prev = NULL;
  ZkEmailField link = NULL;

  zkErrorClear(err);
  if (head == NULL) {
    zkErrorSet(err,"zkEmailFieldStrip",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
    return 0;
  }
  field = *head; 
  prev = head;
  while (field) {
    zkEmailFieldLink(err,field,&link);
    if (field->id_flags & FL_DELETE) {
      if (!(((flags & DONT_STRIP_UNKNOWN) && (field->id == UnknownField)) ||
            ((flags & DONT_STRIP_NEWSGROUPS) && (field->id == Newsgroups)))) {
        *prev = link;
        zkEmailFieldFree(err,&field);
        field = link;
        continue;
      }
    }
    prev = &(field->link);
    field = *prev;
  }
  return 1;
}

/**
 * Delete fields in the field list based on their ID's
 *
 * 
 */

DLLExport int
zkEmailFieldDeleteById (
  ZkError err ,
  ZkEmailField * head ,
  enum ZkMIMEFieldId id )
{
  ZkEmailField field = NULL;
  ZkEmailField *prev = NULL;
  ZkEmailField link = NULL;
  
  zkErrorClear(err);
  if (head) {
    if (*head) {
      field = *head; 
      prev = head;
      while (field) {
        zkEmailFieldLink(err,field,&link);
        if (field->id == id) {
          *prev = link;
          zkEmailFieldFree(err,&field);
          field = link;
          continue;
        }
        prev = &(field->link);
        field = *prev;
      }
      return 1;
    }
  }
  zkErrorSet(err,"zkEmailFieldDeleteById",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
  return 0;
}


/**
 *  This function is used to set the name string of the field.  The name string
 *  must be NUL terminated.  When the name is set, the function will try to 
 *  determine the matching ID for the field from the list in zkMIME.h
 *
 *  @param err Returns the error if one occured
 *  @param field The ZkEmailField for which to set the name
 *  @param name The NUL terminated name string
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldSetName (
  ZkError err ,
  ZkEmailField field ,
  const unsigned char * name )
{
#if defined(CVRT_FLD_TO_UPPER_CASE)
  int i = 0;  /* Loop index for converting name to upper case */
#endif /* CVRT_FLD_TO_UPPER_CASE */
  int len = 0; /* Length of the field name */

  zkErrorClear(err);
  if (field) {
    if (name == NULL) {
      name = (const unsigned char *) "";
    }
    field->name = (unsigned char *) strdup((const char *) name);
    if (field->name == NULL) {
      zkErrorSet(err,"zkEmailFieldSetName",ZKERR,ZKMEMAIL,ZKENOMEM,"strdup");
      return 0;
    }
    len = strlen((char *) field->name);
#if defined(CVRT_FLD_TO_UPPER_CASE)
    for (i = 0; i < len; i++) {
      field->name[i] = toupper(field->name[i]);
    }
#endif /* CVRT_FLD_TO_UPPER_CASE */
    zkMIMEresolveName(field->name,&(field->id),&(field->id_flags));
    return 1;
  }
  zkErrorSet(err,"zkEmailFieldSetName",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
  return 0;
}

/**
 *  Returns the NUL terminated name string for the specified field.
 * 
 *  @param err Returns the error if one occured
 *  @param field The ZkEmailField for which you wish to have the name string
 *  @param name Returns a pointer to a NUL terminated string contained inside
 *              The field object.  Do not mess with this string.
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldName (
  ZkError err ,
  ZkEmailField field ,
  const unsigned char ** name )
{
  zkErrorClear(err);
  if ((field) && (name)) {
    *name = (const unsigned char *) field->name;
    return 1;
  }
  zkErrorSet(err,"zkEmailFieldName",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
  return 0;
}

/**
 *  Set the email field value.  If there are comments in the value string,
 *  they will be automatically stripped.  The value string must be NUL 
 *  terminated.
 * 
 *  @param err Returns the error if one occured
 *  @param field The ZkEmailField for which to set the value string.
 *  @param value The NUL terminated value string
 *  @version 1.0
 *  @author Philippe Boucher
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldSetValue (
  ZkError err ,
  ZkEmailField field ,
  const unsigned char * value )
{
  zkErrorClear(err);
  if (field) {
    if (value == NULL) {
      value = (const unsigned char *) "";
    }
    if (field->value != NULL) {
      free(field->value);
    }
    field->value = (unsigned char *) strdup((const char *) value);
    if (field->value == NULL) {
      zkErrorSet(err,"zkEmailFieldSetValue",ZKERR,ZKMEMAIL,ZKENOMEM,"strdup");
      return 0;
    }
    if (!(field->flags & FL_DONTSTRCMT) && !(field->id_flags & FL_DONTSTRCMT)) {
      if (stripComments(field->value) == 0) {
        zkErrorSet(err,"zkEmailFieldSetValue (stripComments)",ZKERR,ZKMEMAIL,
                   ZKEPARSE,(const char *) field->name);
        return 0;
      }
    }
    return 1;
  }
  zkErrorSet(err,"zkEmailFieldSetValue",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
  return 0;
}

/**
 *  Returns the NUL terminated value string for the field.
 * 
 *  @param err Returns the error if one occured
 *  @param field The ZkEmailField whose value string you want
 *  @param value Returns a pointer to a NUL terminated string which is 
 *               contained in the ZkEmailField object.  Dont mess with this
 *               string.
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldValue (
  ZkError err ,
  ZkEmailField field ,
  const unsigned char ** value )
{
  zkErrorClear(err);
  if ((field) && (value)) {
    *value = (const unsigned char *) field->value;
    return 1;
  }
  zkErrorSet(err,"zkEmailFieldValue",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
  return 0;
}

/**
 *  Return the field ID for the ZkEmailField.  This id is one of the elements
 *  in the list described in the file zkMIME.h  
 *
 *  @param err Returns the error if one occured
 *  @param field The ZkEmailField to analyze
 *  @param id Returns the field id
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldId (
  ZkError err ,
  ZkEmailField field ,
  enum ZkMIMEFieldId * id )
{
  zkErrorClear(err);
  if (field && id) {
    *id = field->id;
    return 1;
  }
  zkErrorSet(err,"zkEmailFieldId",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
  return 0;
}

/**
 *  This sets the field id.  You must use one of the elements from the list in 
 *  zkMIME.h.  When the field ID is set, the name string will be updated to 
 *  match the new id.
 *  
 *  @param err Returns the error if one occured.
 *  @param field The ZkEmailField to modify
 *  @param id The new id for the field object
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldSetId (
  ZkError err ,
  ZkEmailField field ,
  enum ZkMIMEFieldId id )
{
  int labels = sizeof(FieldLabel) / sizeof(struct T_FieldLabel);
  int i = 0; /* Loop index for finding the id */

  zkErrorClear(err);
  if (field) {
    field->id = id;
    for (i = 0; i < labels; i++) {
      if (FieldLabel[i].id == id) {
        if (field->name != NULL) {
          free(field->name);
        }
        field->name = (unsigned char *) strdup((const char *) FieldLabel[i].name);
        if (field->name == NULL) {
          zkErrorSet(err,"zkEmailFieldSetId",ZKERR,ZKMEMAIL,ZKENOMEM,"strdup");
          return 0;
        } 
        field->id_flags = FieldLabel[i].flags;
        return 1;
      }
    }
    zkErrorSet(err,"zkEmailFieldSetId",ZKERR,ZKMEMAIL,ZKENOTFOUND,NULL);
    return 0;
  }
  zkErrorSet(err,"zkEmailFieldSetId",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
  return 0;
}

/*
 *  Scan the value of an email field to see if it contains MIME parameters.
 *  (this is anything following a semicolon having the form NAME = VALUE).
 *  This function is called automatically as the system parses fields so you
 *  should not call it yourself.
 */

static int
zkEmailFieldParameterParse (
  ZkError err,
  ZkEmailField field )
{
  int quotemode = 0;
  int anglemode = 0;
  int parseparms = 0;
  int done = 0;
  unsigned char * name = NULL;
  unsigned char * value = NULL;
  unsigned char * sptr = NULL;

  zkErrorClear(err);
  if (field == NULL) {
    zkErrorSet(err,"zkEmailFieldParameterParse",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
    return 0;
  }

  /*
   * First, this loop will scan the field value to see if we have anything
   * that qualifies as a string containing MIME field parameters.
   */

  sptr = field->value;
  while (*sptr != (char) 0) {
    switch (*sptr) {
    case CHAR_DQUOTE:
      if (quotemode == 0) {
        quotemode = 1;
      } else { 
        quotemode = 0;
      }
      break;
    case CHAR_LT:
      if (quotemode == 0) {
        anglemode += 1;
      }
      break;
    case CHAR_GT:
      if (quotemode == 0) {
        anglemode -= 1;
      }
      break;
    case CHAR_SEMICOLON:
      if ((quotemode <= 0) && (anglemode <= 0)) {
        parseparms = 1;
        *sptr = (char) 0;
      }
      break;
    }
    sptr++;
    if (parseparms) {
      break;
    }
  } 
  if (!parseparms) {
    return 1;
  }
  while (*sptr != (char) 0) {

    /*
     * Get the name of the MIME field parameter 
     */

    name = sptr;
    while ((*sptr != (char) 0) && (*sptr != CHAR_EQUALS)) {
      sptr++;
    } 
    if (*sptr == (char) 0) {
      goto parse_error;
    }
   
    /*
     * Now get the value of the parameter.
     */

    *sptr++ = (char) 0;
    value = sptr;
    quotemode = 0;
    done = 0;
    while (*sptr != (char) 0) {
      switch (*sptr) {
      case CHAR_SOLIDUS:
        if (sptr[1] != (char) 0) {
          sptr++;
        }
        break;
      case CHAR_DQUOTE:
        if (quotemode) {
          quotemode = 0;
        } else {
          quotemode = 1;
        }
        break;
      case CHAR_SEMICOLON:
        if (!quotemode) {
          *sptr = (char) 0;
          done = 1;
        }
        break;
      } 
      sptr++; 
      if (done) { 
        break;
      }
    }
    name = trimWhiteSpace(name);
    value = trimWhiteSpace(value); 
    value = trimDoubleQuotes(value); 
    /*
     * Create a new EmailFieldParameter object and link it to the current
     * field.
     */
    if (!zkEmailFieldAddParameter(err,field,name,value)) {
      return 0;
    }
  } 
  return 1;

parse_error:
  zkErrorSet(err,"zkEmailFieldParameterParse",ZKERR,ZKMEMAIL,ZKEPARSE,NULL);
  return 0;
}

/**
 *  Add a parameter to the given email field.  You should only do this to MIME
 *  fields.
 *
 *  @param err Returns the error if one occured
 *  @param field The ZkEmailField to which to add the parameter
 *  @param name The NUL terminated string which is the name of the parameter
 *  @param value The NUL terminated string which is the value of the parameter
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldAddParameter (
  ZkError err ,
  ZkEmailField field ,
  const unsigned char * name,
  const unsigned char * value )
{
  ZkEmailFieldParameter parm = NULL;

  zkErrorClear(err);
  if ((field == NULL) || (name == NULL) || (value == NULL)) {
    zkErrorSet(err,"zkEmailFieldAddParameter",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
    return 0;
  }
  if (!zkEmailFieldParameterNew(err,&parm)) {
    return 0;
  }
  zkEmailFieldParameterSetName(err,parm,name);
  zkEmailFieldParameterSetValue(err,parm,value);
  parm->link = field->parm;
  field->parm = parm;
  return 1;
}

/**
 *  Parse the string into an RFC822/ MIME field.  You probably don need to 
 *  call this yourself as all of the parsing is handled by the zkEmailParse()
 *  function.
 * 
 *  @param err  Returns the error if one occured
 *  @param field The ZkEmailField object which will hold the result of the parse
 *  @param line The NUL terminated line of text which will be parsed in order 
 *              to load the ZkEmailField object.  Note that it is assumed here
 *              that the line has already been unfolded.
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldParse (
  ZkError err ,
  ZkEmailField field ,
  const unsigned char * line )
{
  unsigned char * linecopy = NULL;
  unsigned char * lineptr = NULL;             /* Loops within the line copy */
  unsigned char * name = NULL;                /* Name of the field */
  unsigned char * value = NULL;               /* Value of the field */

  zkErrorClear(err);
  if ((field == NULL) || (line == NULL)) {
    zkErrorSet(err,"zkEmailFieldParse",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
    return 0;
  }

  linecopy = (unsigned char *) strdup((const char *) line);
  if (linecopy == NULL) {
    zkErrorSet(err,"zkEmailFieldParse",ZKERR,ZKMEMAIL,ZKENOMEM,"strdup");
    return 0;
  }
  lineptr = linecopy;
  name = linecopy;
  while (*lineptr != CHAR_COLON) { 
    if (*lineptr == CHAR_NUL) {
      zkErrorSet(err,"zkEmailFieldParse",ZKERR,ZKMEMAIL,ZKEPARSE,
                 "No field value");
      free(linecopy);
      return 0;
    }
    lineptr++;
  }
  *lineptr++ = CHAR_NUL;
  value = lineptr;
  name = trimWhiteSpace(name);
  value = trimWhiteSpace(value); 

  if (!zkEmailFieldSetName(err,field,name)) {
    free(linecopy);
    return 0; 
  }
  if (!zkEmailFieldSetValue(err,field,value)) {
    free(linecopy);
    return 0;
  }
  if (field->id_flags & FL_MIME) {
    if (!zkEmailFieldParameterParse(err,field)) {
      free(linecopy);
      return 0;
    }
  }
  
  free(linecopy);
  return 1;
}

/**
 *  Dump the EmailField to the given callback.  See the zkEmailDump() function
 *  for more details
 * 
 *  @param err Returns the error if one occured
 *  @param field The ZkEmailField object to dump
 *  @param callback The callback function to use for dumping the field
 *  @param cdata The extra data pointer for the callback function
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldDump (
  ZkError err ,
  ZkEmailField field ,
  ZkEmailDumpCallback callback ,
  void * cdata )
{
  const unsigned char * str = NULL;
  ZkEmailFieldParameter parm = NULL;
  int splitValue = 0; /* Set this to 1 for special field values that need
                       * to be split
                       */

#define LINE_LEN 72         /* Ideal length of a line */
#define DUMP_TO_CALLBACK(_str) \
  if ((str = (const unsigned char *)(_str)) == NULL) \
    str = (const unsigned char *) ""; \
  if (!callback(err,cdata, str,strlen((const char *)str))) \
    return 0; 

  zkErrorClear(err);
  if ((field == NULL) || (callback == NULL)) {
    zkErrorSet(err,"zkEmailFieldDump",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
    return 0;
  }
  if (field->id == X_ZKS_Freedom_MessageID) {
    zkCStrStripWhiteSpace(field->value);
    splitValue = 1;
  }
  DUMP_TO_CALLBACK(field->name);
  DUMP_TO_CALLBACK(": ");
  if (splitValue) {
    int offset = 0;
    int len = 0;
    int vlen = 0;
    vlen = strlen(field->value);
    len = LINE_LEN - (strlen(field->name) + strlen(": "));
    while (offset < vlen) {
      if (len > (vlen - offset)) {
        if (!callback(err,cdata,&(field->value[offset]),vlen - offset)) { 
          return 0;
        }
      } else {
        if (!callback(err,cdata,&(field->value[offset]),len)) { 
          return 0;
        }
        DUMP_TO_CALLBACK("\015\012 ");
      }
      offset += len;
      len = LINE_LEN - 1;  /* -1 because we already have 1 space on new line */
    }
  } else {
    DUMP_TO_CALLBACK(field->value);
  }
  parm = field->parm;
  while (parm) {
    DUMP_TO_CALLBACK(";\015\012     ");
    DUMP_TO_CALLBACK(parm->name);
    DUMP_TO_CALLBACK(" = \"");
    DUMP_TO_CALLBACK(parm->value);
    DUMP_TO_CALLBACK("\" ");
    parm = parm->link;
  }
  DUMP_TO_CALLBACK("\015\012");
  return 1;

#undef DUMP_TO_CALLBACK
}
 
/**
 *  Canonicalise and Dump the EmailField to the given callback.  
 *  See the zkEmailDump() function  for more details
 * 
 *  @param err Returns the error if one occured
 *  @param field The ZkEmailField object to dump
 *  @param callback The callback function to use for dumping the field
 *  @param cdata The extra data pointer for the callback function
 *  @author Adam Back
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldDumpCanonical (
  ZkError err ,
  ZkEmailField field ,
  ZkEmailDumpCallback callback ,
  void * cdata )
{
  const unsigned char * str = NULL;
  ZkEmailFieldParameter parm = NULL;
  int splitValue = 0; /* Set this to 1 for special field values that need
                       * to be split
                       */

#define LINE_LEN 72         /* Ideal length of a line */
#define DUMP_TO_CALLBACK(_str) \
  if ((str = (const unsigned char *)(_str)) == NULL) \
    str = (const unsigned char *) ""; \
  if (!callback(err,cdata, str,strlen((const char *)str))) \
    return 0; 

  zkErrorClear(err);
  if ((field == NULL) || (callback == NULL)) {
    zkErrorSet(err,"zkEmailFieldDump",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
    return 0;
  }

  if (field->id != X_ZKS_Freedom_Header_Sig) {
    if ( !zkCStrToLowerCase( field->value ) ) {
      return 0;
    }
    if ( !zkCStrStripTrailingWhiteSpace( field->value ) ) {
      return 0;
    }
  }

  if ( !zkCStrToLowerCase( field->name ) ) {
    return 0;
  }
  if ( !zkCStrStripTrailingWhiteSpace( field->name ) ) {
    return 0;
  }

  if (field->id == X_ZKS_Freedom_MessageID) {
    zkCStrStripWhiteSpace(field->value);
    splitValue = 1;
  }
  DUMP_TO_CALLBACK(field->name);
  DUMP_TO_CALLBACK(": ");
  if (splitValue) {
    int offset = 0;
    int len = 0;
    int vlen = 0;
    vlen = strlen(field->value);
    len = LINE_LEN - (strlen(field->name) + strlen(": "));
    while (offset < vlen) {
      if (len > (vlen - offset)) {
        if (!callback(err,cdata,&(field->value[offset]),vlen - offset)) { 
          return 0;
        }
      } else {
        if (!callback(err,cdata,&(field->value[offset]),len)) { 
          return 0;
        }
        DUMP_TO_CALLBACK("\015\012 ");
      }
      offset += len;
      len = LINE_LEN - 1;  /* -1 because we already have 1 space on new line */
    }
  } else {
    DUMP_TO_CALLBACK(field->value);
  }
  parm = field->parm;
  while (parm) {
    DUMP_TO_CALLBACK(";\015\012     ");
    DUMP_TO_CALLBACK(parm->name);
    DUMP_TO_CALLBACK(" = \"");
    DUMP_TO_CALLBACK(parm->value);
    DUMP_TO_CALLBACK("\" ");
    parm = parm->link;
  }
  DUMP_TO_CALLBACK("\015\012");
  return 1;

#undef DUMP_TO_CALLBACK
}
 
static int
zkEmailFieldParameterDup (
  ZkError err,
  ZkEmailFieldParameter parm ,
  ZkEmailFieldParameter * copy )
{
  ZkEmailFieldParameter parm_t = NULL;

  zkErrorClear(err);
  if ((parm == NULL) || (copy == NULL)) {
    zkErrorSet(err,"zkEmailFieldParameterDup",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
    return 0;
  }
  *copy = NULL;
  while (parm) {
    if (!zkEmailFieldParameterNew(err,&parm_t)) {
      return 0;
    }
    if (parm->name) {
      if (!zkEmailFieldParameterSetName(err,parm_t,parm->name)) {
        return 0;
      } 
    } else {
      parm_t->name = NULL;
    }
    if (parm->value) {
      if (!zkEmailFieldParameterSetValue(err,parm_t,parm->value)) {
        return 0;
      }
    } else {
      parm_t->value = NULL;
    }
    parm_t->link = *copy;
    *copy = parm_t;
    parm = parm->link;
  }
  return 1;
}
  
/** 
 *  This function makes a complete copy of a ZkEmailField.  Note that if you
 *  wish to make a copy of a a specific field (recall that fields are actually
 *  lists) you should get the link of the field using the zkEmailFieldLink()
 *  function, then reset the link to NULL using a call like:
 *  zkEmailFieldSetLink(err,field,NULL)
 *  make a duplicate of the field using this function then reset the link of
 *  the original to what it previously was.
 *  Otherwise, the field list beginning with the specified field is copied
 *  completely.
 * 
 *  @param err Returns the error if one occured
 *  @param field The ZkEmailField to copy
 *  @param copy Returns a new copy of the field 
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldDup (
  ZkError err ,
  ZkEmailField field ,
  ZkEmailField * copy )
{
  ZkEmailField copy_t = NULL;

  zkErrorClear(err);
  if ((copy == NULL) || (field == NULL)) {
    zkErrorSet(err,"zkEmailFieldDup",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
    return 0;
  }
  *copy = NULL;
  while (field) {
    if (!zkEmailFieldNew(err,&copy_t)) {
      zkErrorSetDetail(err,"zkEmailFieldDup");
      return 0;
    }
    copy_t->flags = field->flags;
    if (field->name) {
      if (!zkEmailFieldSetName(err,copy_t,field->name)) {
        zkErrorSetDetail(err,"zkEmailFieldDup");
        return 0;
      }
    } else {
      copy_t->name = NULL;
    }
    if (field->value) {
      if (!zkEmailFieldSetValue(err,copy_t,field->value)) {
        zkErrorSetDetail(err,"zkEmailFieldDup");
        return 0;
      } 
    } else {
      copy_t->value = NULL;
    }
    /* id_flags and id were copied by zkEmailFieldSetName */
    if (field->parm) {
      if (!zkEmailFieldParameterDup(err,field->parm,&(copy_t->parm))) {
        zkErrorSetDetail(err,"zkEmailFieldDup");
        return 0;
      }
    } else {
      copy_t->parm = NULL;
    }
    if (!zkEmailFieldSetLink(err,copy_t,*copy)) {
      zkErrorSetDetail(err,"zkEmailFieldDup");
      return 0;
    }
    *copy = copy_t;
    field = field->link;
  }
  return 1;
}

/**
 * Returns the flags which identify the type of field.  These flags are
 * define in zkEmailField.h:
 *   FL_DELETE       Field which should be thrown away 
 *   FL_RFC822       A Mail field described in RFC822 
 *   FL_RFC1327      A Mail extension field described in RFC1327
 *   FL_MIME         A MIME field 
 *   FL_ZKS_FREEDOM  A Zero-Knowledge Freedom product field 
 *   FL_ZKS          Some other Zero-Knowledge mail field 
 *   FL_EXTENSION    Some other extension field 
 *   FL_UNKNOWN      Unknown email header field 
 *   FL_RFC1945      an HTTP 1.0 field
 *   FL_COOKIE       an HTTP cookie field (rfc2109)
 *
 *
 * @param err Returns the error if one occured
 * @param field The ZkEmailField to analyze
 * @param flags Returns the flags associated with this field
 * @author Philippe Boucher
 * @version 1.0
 * @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkEmailFieldIdFlags (
  ZkError err ,
  ZkEmailField field ,
  uint32_t * flags )
{
  zkErrorClear(err);
  if ((field == NULL) || (flags == NULL)) {
    zkErrorSet(err,"zkEmailFieldIdFlags",ZKERR,ZKMEMAIL,ZKENULLPTR,NULL);
    return 0;
  }
  *flags = field->id_flags;
  return 1;
}
  
