/*
 * $Id: zkEmailParse.c,v 1.7.32.1 2000/10/11 13:20:28 phil Exp $
 */

static const char * module_id_ = "$Id: zkEmailParse.c,v 1.7.32.1 2000/10/11 13:20:28 phil Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <zkIntTypes.h>
#include "zkEmailParse.h"

/**
 *  This internal function is used to strip leading and trailing whitespace
 */

unsigned char * 
trimWhiteSpace (
  unsigned char * s )
{
  unsigned char * t = NULL;

  (void) module_id_;
  if (s == NULL) {
    return NULL;
  }
  while (isspace((int) *s)) s++;
  if (*s == CHAR_NUL) return s;
  t = s;
  while (*t != CHAR_NUL) t++;
  t--;
  while (isspace((int) *t)) *t-- = CHAR_NUL;
  return s;
}


/**
 *  Unfold a line of text.
 *  If flags in non-zero, the whitespacec which follows the CR/LF will be
 *  deleted as well.
 */

unsigned char *
unfoldLine (
  unsigned char * s ,
  uint32_t flags )
{
  unsigned char * start = NULL;
  unsigned char * t = NULL;
  int nukespace = 0;

  start = s;
  t = s;

/*
 * This has been split into two loops to make the execution faster for the
 * mime library, since the 822 standard specifies that the whitespace
 * character following the CR/LF is kept.
 */

  if (flags) {
    while (*s != CHAR_NUL) {
      switch (*s) {
      case CHAR_CR:
      case CHAR_LF:
        nukespace = 1;
        break;
      case CHAR_SPACE:
      case CHAR_HTAB:
        if (nukespace) {
          nukespace = 0;
        } else {
         *t++ = *s;
        }
        break;
      default:
        *t++ = *s;
        break;
      }
      s++;
    }
  } else {
    while (*s != CHAR_NUL) {
      switch (*s) {
      case CHAR_CR:
      case CHAR_LF:
        break;
      default:
        *t++ = *s;
        break;
      }
      s++;
    }
  }
  *t = CHAR_NUL;
  return start;
} 


/**
 *  Remove the doubld quote characters from a string.
 */

unsigned char *
stripDoubleQuotes (
  unsigned char * s )
{
  unsigned char * start = NULL;
  unsigned char * t = NULL;
  
  if (s == NULL) {
    return NULL;
  }
  start = s;
  t = s;
 
  while (*s != (char) 0) {
    switch (*s) {
    case CHAR_DQUOTE:
      break;
    default:
      *t++ = *s;
      break;
    }
    s++;
  }
  *t = (char) 0;
  return start;
}

unsigned char * 
trimDoubleQuotes (
  unsigned char * s )
{
  unsigned char * t = NULL;

  if (s == NULL) {
    return NULL;
  }
  t = s;
  if (s[0] == (char) 0) {
    return s;
  }
  if (s[0] == CHAR_DQUOTE) {
    s++;
  }
  while (t[1] != (char) 0) {
    t++;
  }
  if (*t == CHAR_DQUOTE) {
    *t = (char) 0;
  }
  return s;
}


/**
 *  Return a pointer to the start (1st character) of the next line.  If there
 *  are no more lines, a NULL is returned.  The end of the line is identified
 *  by one of the following.  CRLF followed by anything, CR followed by anything
 *  other than LF, or LF.  The current line will become NUL terminated at
 *  the first CR or LF.
 */

unsigned char * 
startOfNextLine (
  unsigned char * s )
{
  while (*s != (char) 0) {
    if (*s == CHAR_CR) {
      *s = (char) 0;
      s++;
      if (*s == CHAR_LF) {
        s++;
      }
      return s;
    } else if (*s == CHAR_LF) {
      *s = (char) 0;
      s++;
      return s;
    }
    s++;
  }
  return NULL;
}


/*
 * Return the start of the next line.  If the text contains a line fold,
 * ie, a CRLF followed by a space or tab character, then the line will 
 * also include the folded text.
 */

unsigned char *
startOfNextFoldedLine (
  unsigned char * s )
{
  while (*s != (char) 0) {
    if (*s == CHAR_CR) {
      *s = (char) 0;
      if (s[1] == CHAR_LF) {
        if ((s[2] == CHAR_SPACE) || (s[2] == CHAR_HTAB)) {
          *s = CHAR_CR;
          s = &(s[3]);
          continue;
        }
        s++;
      } 
      s++;
      return s;
    } else if (*s == CHAR_LF) {
      *s = (char) 0;
      s++;
      return s;
    }
    s++;
  }
  return NULL;
}    
