/*
 * $Id: zkURL.c,v 1.4.32.5 2000/12/14 17:54:33 jean Exp $
 */

static const char * module_id_ = "$Id: zkURL.c,v 1.4.32.5 2000/12/14 17:54:33 jean Exp $";

/*
 * zkURL.c
 * 
 * $Id: zkURL.c,v 1.4.32.5 2000/12/14 17:54:33 jean Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 * 
 * Based on wget.c 
 *         (C) 1997 by Philippe Boucher <splinter@cs.mcgill.ca> 
 *
 * Permission to use this source code has been granted by the author
 * provided these comments remain with the source 
 * code. This code is provided as a public service.  No claims as to the
 * fitness of this source code is made, and the author assumes no 
 * responsibility for use or misuse.                              
 *                                                                
 * Adapted for zkURL by Luc Boulianne <luc@zks.net>
 */

#include "zks.h"

#include "zkDNS.h"
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#define __USE_GNU
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <ctype.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include <zkMemBuf.h>
#include <zkMIME.h>
#include <zkHTTPHeader.h>
#include <zkURL.h>
#include <zkDebug.h>
#include <zkDefs.h>
#include "zkCStr.h"

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMHTTP

#define REQUEST_SZ 256
#define BUFFER_SZ 1024

#define FLAG_STRIPHEADER 0x1 /* Remove the header before outputing the data */
#define FLAG_STRIPHTML   0x2 /* Strip the HTML tags from the text */

#define PROTO_HTTP    1

#define HTTP_HDR_DELIM   "\015\012\015\012" 

/* Bug: probably a buffer overrun problem here */
#define DATA_SZ      512 /* Amount of data allowed for a URL */
#define HOST_SZ       64 /* Number of characters allowed for a hostname */
struct T_URL {
  int proto;                   /* the protocol required */
  char host[HOST_SZ + 1];      /* The remote server name */
  unsigned short port;         /* The port */
  char data[DATA_SZ + 1];      /* The url data */
};


/*
 * Right now, all protocols are assumed to be HTTP
 * returns a 0 on failure
 */

static int
SplitURL (
	  struct T_URL * url ,
	  const char * s )
{
  const char * host = NULL;
  const char * port = NULL;
  const char * data = NULL;
  int end = 0; /* End offset for an extracted string from "data" */
  int i = 0; /* Loop index */

  (void) module_id_;
  if (url && s) {
    url->proto = PROTO_HTTP;
    url->port = 80;
    if ((host = strstr(s,"://")) == NULL) {
      return 0;
    } else {
      host += 3;
    }
    while (host[i] != (char) 0) {
      if (host[i] == ':') {
        end = i;
        i++;
        port = &(host[i]);
        i = 0;
        url->port = 0;
        while (port[i] != (char) 0) {
          if (port[i] == '/')  {
            data = &(port[i]);
            break;
          } else if (isdigit(port[i])) {
            /* Warning: This is not very portable.. fix this */
            url->port = url->port * 10 + (port[i] - '0');
          } else {
            /* character was not a digit */
            return 0;
          }
          i++;
        }
        break;
      } else if (host[i] == '/') {
        end = i;
        data = &(host[i]);
        break;
      } 
      i++;
      end = i;
    }
    if (end > HOST_SZ) {
      end = HOST_SZ;
    }
    strncpy(url->host,host,end);
    url->host[end] = (char) 0;
    if (data) {
      strncpy(url->data,data,DATA_SZ);
      url->data[DATA_SZ] = (char) 0;
    } else {
      zkCStrlcpy(url->data,"/",sizeof url->data);
    }
    return 1; 
  }
  return 0;
}
  

static int
StripHTML (
	   char * buffer ,
	   int * depth ,
	   int buflen ,
	   int * newlen )
{
  int i = 0;
  int tail = 0;

  if (depth && buffer && newlen) {
    for (i = 0; i < buflen; i++) { 
      if (buffer[i] == '<') {
        *depth += 1;
      }
      if (*depth == 0) {
        buffer[tail] = buffer[i];
        tail += 1;
      }
      if (buffer[i] == '>') {
        if (*depth > 0) {
          *depth -= 1;
        }
      }
    }
    *newlen = tail;
    return 1;
  }
  return 0;
}

static int writeBodyToMemBuf (ZkError err , void * cdata , const unsigned char * data ,  int dlen )
{
  
  if (cdata && data && (dlen > 0)) {
    zkMemBufAppend(err, (ZkMemBuf) cdata, (const void *) data, dlen);
  }
  return 1;
}


/**
 *  Get the body of an HTTP URL. 
 * 
 *  @param err      Returns the error if any occured.
 *  @param url_s    URL string (http://host/resource
 *  @param body     Pointer to the body. Caller must free the 
 *		    space allocated.
 *  @param bodysize The size of the body
 *  @version 1.0
 *  @author  Luc Boulianne
 *  @return a non-zero value on success, a 0 on failure.
 *  
 */

DLLExport
int zkURLGetBody (ZkError err, const char *url_s, char **body, uint32_t *bodysize)
{
  static const char *funcname = "zkURLGetBody";

  char buffer[BUFFER_SZ];
  char request[REQUEST_SZ];
  char *p;
  const char * version = "ZKS WebGet $Revision: 1.4.32.5 $";
  int bytes = 0;
  int fd = 0;
  int status = 0;

  struct T_URL url; /* Holds the parsed URL */
  struct hostent * hostent = NULL;
  struct sockaddr_in httpd;
  unsigned long flags = 0; /* Behaviour flags */
  ZkHTTPHeader header;
  ZkMemBuf membuf;
  
  flags |= FLAG_STRIPHEADER;

  /*
   * Create the HTTP header structure 
   */

  if (!zkHTTPHeaderNew(err,&header)) {
    return 0;
  }

  if (!zkHTTPHeaderSetType(err,header,RESPONSE_HEADER)) {
    zkHTTPHeaderFree(NULL,&header);
    return 0;
  }

  if (!zkHTTPHeaderKeepBodyData(err, header)){
    zkHTTPHeaderFree(NULL,&header);
    return 0;
  }  

  if (!zkMemBufNew(err, &membuf)) {
    zkHTTPHeaderFree(err,&header);
    return 0;
  }
  
  /* Get the URL */
  if (url_s == NULL) {
    zkErrorSet (err, funcname, ZKERR, ZKMOD, ZKENULLPTR, "No URL passed");
    zkHTTPHeaderFree(NULL,&header);
    zkMemBufFree(NULL,&membuf);
    return 0;
  } else {
    if (!SplitURL(&url,url_s)) {
      zkErrorSet_va(err, funcname, ZKERR, ZKMOD,
		    ZKEPARSE, "Couldn't make sense of the URL [%s]", url_s);
      zkHTTPHeaderFree(NULL,&header);
      zkMemBufFree(NULL,&membuf);
      return 0;
    }
  }

  /* Connect to the server; hopefully this will be with AHTTP */
  if (!zkGetHostByName(err, url.host, &hostent)) {
    return 0;
  }

  if (hostent == NULL) {
    zkErrorSet_va(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Couldn't get hostname: %s", strerror(errno));
    return 0;
  }

  httpd.sin_family = AF_INET;
  httpd.sin_port = htons(url.port);
  httpd.sin_addr.s_addr = *((unsigned int *) hostent->h_addr);

  if ((fd = socket(AF_INET,SOCK_STREAM,0)) < 0) {
    zkErrorSet_va(err, funcname, ZKERR, ZKMOD,
	       ZKESOCKET, "Couldn't get socket: %s", strerror(errno));
    close (fd);
    return 0;
  }
 
  if (connect(fd,&httpd,sizeof(httpd)) != 0) {
    zkErrorSet_va(err, funcname, ZKERR, ZKMOD,
	       ZKENOMEM, "Couldn't connect socket: %s", strerror(errno));
    close (fd);
    return 0;
  }

  /*
   * Construct a request. Quite possibly, I could have used
   * zkHTTP* for this, but this was faster.
   */
  snprintf(request,sizeof request,"GET %s HTTP/1.1\015\012"
	  "Host: %s:%d\015\012"
	  "Connection: close\015\012"
	  "User-Agent: %s\015\012"
	  "\015\012",
          url.data,url.host,url.port,version);

  if (write(fd,request,strlen(request)) < 0) {
    /* Bug: check the number of bytes written */
    zkErrorSet_va(err, funcname, ZKERR, ZKMOD,
	       ZKEWRITE, "Problem writing to socket: %s", strerror(errno));
    close (fd);
    return 0;
  }

  /* Read the response */
  for (;;) {
    bytes = read(fd,buffer,BUFFER_SZ);
    zkDebug_va ("Read [%d] bytes", bytes);
    if (bytes < 0) {
      if (errno == EINTR) {
        continue;
      } else {
	/* Bug: check the number of bytes read */
	zkErrorSet_va(err, funcname, ZKERR, ZKMOD,
		      ZKENOMEM, "Problem reading from socket: %s", strerror(errno));
	close (fd);
      }
    } else if (bytes == 0) {
      zkDebug_va ("************* Read 0 bytes...");
      zkErrorSet_va(err, funcname, ZKWARN, ZKMOD,
		      ZKENOMEM, "Problem reading from socket: %s", strerror(errno));
      break;
    }
    if (!zkHTTPHeaderAddText(err,header,buffer,bytes)) {
      close(fd);
      return 0;
    }
    if (!zkHTTPHeaderStatus(err,header,&status)) {
      close(fd);
      return 0;
    }  
    if (status == PARSE_COMPLETE) {
      zkDebug_va("************** encountered parse complete");
      break;
    } else if (status == PARSE_HEADER_COMPLETE) {
      zkDebug_va("************** encountered parse header complete");
    } else {
      zkDebug_va("************** status is %d!!!",status);
    }
  }

  close (fd);

  /* Parse is complete, now let's get the body */
  if (!zkHTTPBodyDump (err, header, writeBodyToMemBuf, membuf)) {
    return 0;
  }

  if (!zkHTTPHeaderFree(err,&header)) {
    return 0;
  }

#if NULL_TERMINATE_MEMBUF 
  if ( ! zkMemBufAppend (err, membuf, "\0", 1)) {
    return 0;
  }
#endif
    
  if ( ! zkMemBufGetData(err, membuf, (void **) &p, bodysize)) {
    return 0;
  }

  if ((*body = (char *)malloc(*bodysize)) == NULL) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENOMEM,NULL); 
    return 0; 
  } 

  memcpy(*body, p, *bodysize);

  if ( ! zkMemBufFree(err, &membuf)) {
    return 0;
  }

  return 1;
}
