/*
 * $Id: zkPacket.h,v 1.26.10.4 2001/01/10 20:01:26 mario Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef _ZKPACKET_H
#define	_ZKPACKET_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifndef _ZKINTTYPES_H
#include <zkIntTypes.h>
#endif /* _ZKINTTYPES_H */

#include <zks.h>
#include <stdio.h>

/*
 * These are the protocol ID numbers for the Protocol field of an IP header.
 */

#define ZK_IPPROTO_IP       0   /* Dummy protocol for TCP.  */
#define ZK_IPPROTO_ICMP     1   /* Internet Control Message Protocol.  */
#define ZK_IPPROTO_IGMP     2   /* Internet Group Management Protocol. */
#define ZK_IPPROTO_IPIP     4   /* IPIP tunnels (older KA9Q tunnels use 94).  */
#define ZK_IPPROTO_TCP      6   /* Transmission Control Protocol.  */
#define ZK_IPPROTO_EGP      8   /* Exterior Gateway Protocol.  */
#define ZK_IPPROTO_PUP     12   /* PUP protocol.  */
#define ZK_IPPROTO_UDP     17   /* User Datagram Protocol.  */
#define ZK_IPPROTO_IDP     22   /* XNS IDP protocol.  */
#define ZK_IPPROTO_IPV6    41   /* IPv6-in-IPv4 tunnelling.  */
#define ZK_IPPROTO_ICMPV6  58   /* ICMPv6.  */
#define ZK_IPPROTO_RAW    255   /* Raw IP packets.  */

/* ICMP message types */

#define ZK_ICMP_ECHOREPLY		0		/* Echo Reply */
#define ZK_ICMP_UNREACH			3		/* Destination Unreachable, (see message codes) below */
#define ZK_ICMP_SOURCEQUENCH	4		/* Source Quench */
#define ZK_ICMP_REDIRECT		5		/* Redirect (change route) */
#define ZK_ICMP_ECHO            8		/* Echo Request */
#define ZK_ICMP_ROUTERADVERT	9		/* Router advertisement */		
#define ZK_ICMP_TIMXCEED		11		/* Time Exceeded (see messages below) */
#define ZK_ICMP_PARAMPROB       12		/* Parameter Problem */
#define	ZK_ICMP_MAXTYPE         18		/* Maximum */


/*
 * ICMP message codes for unreachable destinations [Type == 3].  These are passed as values
 * for the code parameter of the zkICMPCreateDestinationUnreachable() 
 * function.
 */

#define ZK_ICMP_NETWORK_UNREACHABLE                     0	/* Network Unreachable */
#define ZK_ICMP_HOST_UNREACHABLE                        1	/* Host Unreachable */
#define ZK_ICMP_PROTOCOL_UNREACHABLE                    2	/* Protocol Unreachable */
#define ZK_ICMP_PORT_UNREACHABLE                        3	/* Port Unreachable */
#define ZK_ICMP_FRAGMENTATION_NEEDED                    4	/* Fragmentation Needed/DF set */
#define ZK_ICMP_SOURCE_ROUTE_FAILED                     5	/* Source Route failed */
#define ZK_ICMP_DESTINATION_NETWORK_UNKNOWN             6	/* Destination Network Unknown */
#define ZK_ICMP_DESTINATION_HOST_UNKNOWN                7	/* Destination Host Unknown */
#define ZK_ICMP_SOURCE_HOST_ISOLATED                    8	/* Source Host Isolated */
#define ZK_ICMP_NETWORK_COMMUNICATION_PROHIBITED        9	/* Communication w/destination net administrativly prohibited */
#define ZK_ICMP_HOST_COMMUNICATION_PROHIBITED			10	/* Communication w/destination host administrativly prohibited */
#define ZK_ICMP_NETWORK_UNREACHABLE_FOR_TOS				11	/* Network Unreachable for Type-of-Service */
#define ZK_ICMP_HOST_UNREACHABLE_FOR_TOS				12	/* Host Unreachable for Type-of-Service */
#define ZK_ICMP_UNREACH_PKTFILT							13	/* Packet filtered */
#define ZK_ICMP_UNREACH_PRECVIO							14	/* Precedence violation */
#define ZK_ICMP_UNREACH_PRECCUT							15	/* Precedence cut off */
#define ZK_ICMP_UNREACH_MAX								15	/* Maximum */


/*
 * ICMP message codes for time exceeded packets [Type == 11].  These are passed as values
 * for the code parameter of the zkICMPCreateTimeExceeded() function.
 */

#define ZK_ICMP_TTL_COUNT_EXCEEDED                      0	/* TTL count exceeded	in transit */
#define ZK_ICMP_FRAGMENT_REASSEMBLY_TIME_EXCEEDED       1	/* Fragment Reassembly time exceeded */


/*
 * Defines for TCP Options
 */
#define ZK_TCPOPT_EOL			0
#define ZK_TCPOPT_NOP			1
#define ZK_TCPOPT_MSS			2
#define ZK_TCPOPT_WINSCALEFACT	3
#define ZK_TCPOPT_TIMESTAMP		8


typedef struct ZkIPHeader ZkIPHeader;
typedef struct ZkICMPHeader ZkICMPHeader;
typedef struct ZkTCPHeader ZkTCPHeader;
typedef struct ZkUDPHeader ZkUDPHeader;

/* 20 bytes for the IP header */
#pragma pack(1)
struct ZkIPHeader {
#ifdef ZK_LITTLE_ENDIAN
  uint8_t ihl:4;
  uint8_t version:4;
#elif ZK_BIG_ENDIAN
  uint8_t version:4;
  uint8_t ihl:4;
#else
#error "You must specify -DZK_LITTLE_ENDIAN or -DZK_BIG_ENDIAN (set ZK_BYTEORDER using the zk_endian script)"
#endif
  uint8_t TypeOfSvr;
  uint16_t TotalLength;
  uint16_t Identification;
  uint16_t Flags_FrameOffs;
  uint8_t  TimeToLive;
  uint8_t  Protocol;
  uint16_t HeaderChkSum;
  uint32_t SrcIpAddress;
  uint32_t DstIpAddress;
};
#pragma pack()
/* 20 bytes for the TCP header unless options are present */
#pragma pack(1)
struct ZkTCPHeader {
  ZkIPHeader IP;
  uint16_t SrcPort;
  uint16_t DstPort;
  uint32_t SequenceNumber;
  uint32_t AckNumber;
#ifdef ZK_LITTLE_ENDIAN
  uint16_t res1:4;
  uint16_t doff:4;
  uint16_t fin:1;
  uint16_t syn:1;
  uint16_t rst:1;
  uint16_t psh:1;
  uint16_t ack:1;
  uint16_t urg:1;
  uint16_t res2:2;
#elif ZK_BIG_ENDIAN
  uint16_t doff:4;
  uint16_t res1:4;
  uint16_t res2:2;
  uint16_t urg:1;
  uint16_t ack:1;
  uint16_t psh:1;
  uint16_t rst:1;
  uint16_t syn:1;
  uint16_t fin:1;
#else
#error "You must specify -DZK_LITTLE_ENDIAN or -DZK_BIG_ENDIAN (set ZK_BYTEORDER using the zk_endian script)"
#endif 
  uint16_t Window;
  uint16_t TcpCheckSum;
  uint16_t UrgentPointer;
};
#pragma pack()

/* 8 bytes for the UDP header */
#pragma pack(1)
struct ZkUDPHeader {
  ZkIPHeader IP;
  uint16_t SrcPort;
  uint16_t DstPort;
  uint16_t UdpLength;
  uint16_t UdpCheckSum;
};
#pragma pack()

#pragma pack(1)
struct ZkICMPHeader {
  uint8_t Type;
  uint8_t code;
  uint16_t CheckSum;  
  union
	  {
	  struct ih_id_seq {
		uint16_t	icd_id;
		uint16_t	icd_seq;
	  } ih_id_seq;
	  struct mtu {       /* RFC1191, section 4 -- MTU discovery */
		uint16_t mtu_void;
		uint16_t NextHop_MTU;
	  } mtu;
  } icmp_payload;

#define icmp_NextHop_MTU		icmp_payload.mtu.NextHop_MTU
#define icmp_Identifier			icmp_payload.ih_id_seq.icd_id
#define icmp_SequenceNumber		icmp_payload.ih_id_seq.icd_seq
};
#pragma pack()

#ifndef WIN32
#define ZKBUFFER_LEN	4096
#else
#define ZKBUFFER_LEN	1500
#endif

#pragma pack(1)
struct ZkBuffer{
	union{
		ZkTCPHeader	Tcp;	/* use ZkBuffer.Tcp.IP.Protocol to determine which union member is valid */
		ZkUDPHeader	Udp;
#if !defined(WIN32) & !defined(FREEDOM_CLIENT)
	} pkt;
#else
	} ;
#endif /* WIN32 */
	unsigned char Data[ZKBUFFER_LEN];
};
#pragma pack()

#pragma pack(1)
typedef struct ZkDIXHeader {
  uint8_t	DstAddress[6];
  uint8_t	SrcAddress[6];
  uint16_t	Type;
} ZkDIXHeader;
#pragma pack()

#pragma pack(1)
typedef struct ZkNetFrame {
	ZkDIXHeader	DIX;	
	union{
		ZkTCPHeader	 TCP;	/* use ZkNetFrame.TCP.IP.Protocol to determine which union member is valid */
		ZkUDPHeader	 UDP;	/* this has the ZkIPHeader structure within it */
		ZkICMPHeader ICMP;  /* this does not have the ZkIPHeader structure within it */
	};
	unsigned char Data[ZKBUFFER_LEN];
} ZkNetFrame;
#pragma pack()


DLLExport uint16_t zkTCPHeaderChecksum (
  ZkTCPHeader * );

DLLExport uint16_t zkUDPHeaderChecksum (
  ZkUDPHeader * ,
  size_t );

/*
 * IP Header manipulation functions
 */

DLLExport uint16_t zkIPHeaderChecksum (
  ZkIPHeader * );

DLLExport int zkIPHeaderDump (
  ZkIPHeader * ,                        /* IP Header to dump */
  FILE * );                             /* destination file (eg stdout) */


DLLExport int zkIPHeaderSourceAddress (
  ZkIPHeader * ,
  uint32_t * );                          /* Returns source IP address */

DLLExport int zkIPHeaderDestinationAddress (
  ZkIPHeader * ,
  uint32_t * );                          /* Returns destination IP address */

DLLExport int zkIPHeaderSetSourceAddress (
  ZkIPHeader * ,
  uint32_t );                            /* source IP address */

DLLExport int zkIPHeaderSetDestinationAddress (
  ZkIPHeader * ,
  uint32_t );                            /* destination IP address */

DLLExport int zkIPHeaderStripOptions (
  ZkIPHeader * );                       /* IP header to strip */

/*
 * ICMP header functions 
 */

DLLExport uint16_t zkICMPHeaderChecksum (
  ZkICMPHeader * ,            /* Pointer to the ICMP header */
  size_t );                   /* Number of bytes over which to compute cksum */

DLLExport int zkICMPCreateDestinationUnreachable (
  ZkIPHeader * ,              /* Original (offending) IP packet */
  uint8_t ,                   /* Reason code (see list above) */
  unsigned char * ,           /* Buffer to hold the icmp packet */
  size_t * );                 /* original size of buffer, returns new size */

DLLExport int zkICMPCreateFragmentationNeeded (
  ZkIPHeader * ,              /* Original (offending) IP packet */
  unsigned char * ,           /* Buffer to hold ICMP response (128 bytes) */
  size_t * );                 /* original size of buffer, returns new size */

DLLExport int zkICMPCreateSourceQuench (
  ZkIPHeader * ,              /* Original (offending) IP packet */
  unsigned char * ,           /* Buffer to hold ICMP response (128 bytes) */
  size_t * );                 /* original size of buffer, returns new size */

DLLExport int zkICMPCreateTimeExceeded (
  ZkIPHeader * ,              /* Original (offending) IP packet */
  uint8_t ,                   /* Reason code (see list above) */
  unsigned char * ,           /* Buffer to hold the icmp packet */
  size_t * );                 /* original size of buffer, returns new size */


/*
 * TCP Header manipulation functions
 */

DLLExport int zkTCPHeaderDump (
  ZkTCPHeader * ,                       /* TCP packet to dump */
  FILE * );                             /* FILE to dump it to (eg stdout) */

DLLExport int zkTCPHeaderAcknowledgement (
  ZkTCPHeader * ,                       /* TCP header */
  uint32_t * );                         /* Returns the acknowledgement number */

DLLExport int zkTCPHeaderSequence (
  ZkTCPHeader * ,                       /* TCP header */
  uint32_t * );                         /* Return the sequence number */

DLLExport uint16_t zkTCPHeaderSourcePort (
  ZkTCPHeader * ,                       /* TCP Header */
  uint16_t * );                         /* Returns the TCP source port */

DLLExport uint16_t zkTCPHeaderDestinationPort (
  ZkTCPHeader * ,                       /* TCP Header */
  uint16_t * );                         /* Returns the TCP destination port */

DLLExport int zkTCPHeaderSetSourcePort (
  ZkTCPHeader * ,                       /* TCP Header */
  uint16_t );                           /* TCP source port */

DLLExport int zkTCPHeaderSetDestinationPort (
  ZkTCPHeader * ,                       /* TCP Header */
  uint16_t );                           /* TCP destination port */

DLLExport int zkTCPHeaderGetMSS (
  ZkTCPHeader * ,                       /* TCP header */
  uint16_t * );                         /* Returns the Maximum Segment Size */


DLLExport int zkTCPHeaderSetMSS (
  ZkTCPHeader * ,                       /* TCP Header */
  uint16_t );                           /* Maximum Segment Size */

DLLExport int zkTCPHeaderStripOptions (
  ZkTCPHeader * );                      /* TCP Header */


/*
 * UDP header manipulation functions
 */

DLLExport int zkUDPHeaderDump (
  ZkUDPHeader * ,
  FILE * );


DLLExport uint16_t zkUDPHeaderSourcePort (
  ZkUDPHeader * ,                       /* UDP Header */
  uint16_t * );                         /* Returns the UDP source port */

DLLExport uint16_t zkUDPHeaderDestinationPort (
  ZkUDPHeader * ,                       /* UDP Header */
  uint16_t * );                         /* Returns the UDP destination port */

DLLExport int zkUDPHeaderSetSourcePort (
  ZkUDPHeader * ,                       /* UDP Header */
  uint16_t );                           /* UDP source port */

DLLExport int zkUDPHeaderSetDestinationPort (
  ZkUDPHeader * ,                       /* UDP Header */
  uint16_t );                           /* UDP destination port */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _ZKPACKET_H */
