/*
 * $Id: cksum.c,v 1.14.16.1 2000/10/11 13:20:34 phil Exp $
 */

static const char * module_id_ = "$Id: cksum.c,v 1.14.16.1 2000/10/11 13:20:34 phil Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <stdio.h>
#include <stdlib.h>

#ifndef WIN32
#include <netinet/in.h> /* htons(), ntohs(), ... */
#include <arpa/inet.h>
#else
#include <winsock2.h>
#endif

#include <zkPacket.h>

#define __IPPACKET_SZ          20
#define	__UDPPACKET_SZ          8
#define __TCPPACKET_SZ         20



/**
 *  Compute a ZkTCPIPHeader checksum
 *
 *
 */

DLLExport uint16_t
zkTCPHeaderChecksum (
  ZkTCPHeader * tcph )
{
  unsigned long cksum = 0;
  unsigned short * data = NULL;
  ZkIPHeader * iph = NULL;
  size_t i = 0;
  size_t datalen = 0;

  (void) module_id_; /* Get rid of compile warning */
  if (!tcph) {
    return 0;
  }

  /*
   * First we compute a checksum on the TCP pseudo header (this consists of
   * the source and destination IP addresses, the lengths listed in the IP
   * packet and the IP protocol code (6).
   */

  iph = &(tcph->IP);
  tcph->TcpCheckSum = 0;

  datalen = ntohs(iph->TotalLength) - (iph->ihl * 4);
  data = (unsigned short *) &(iph->SrcIpAddress);
  cksum = data[0] + data[1];
  data = (unsigned short *) &(iph->DstIpAddress);
  cksum += data[0] + data[1];
  cksum += htons(6) + 
           htons((USHORT) (ntohs(iph->TotalLength) - (iph->ihl * 4)));
  /*
   * Possibly pad one extra byte at the end of the data so that we can
   * properly convert the data to an array of unsigned shorts.
   */

  if (datalen % 2) {
    ((char *)tcph)[datalen + (iph->ihl * 4)] = (char) 0;
    datalen += 1;
  }
  datalen >>= 1;

  /*
   * Compute the checksum
   */

  data = (unsigned short *) &(iph[1]);
  for (i = 0; i < datalen; i++) {
    cksum += *data++;
  } 

  /*
   * Handle the overflow
   */

  cksum = (cksum >> 16) + (cksum & 0xFFFF);
  cksum += (cksum >> 16);
  tcph->TcpCheckSum = ~cksum;

  zkIPHeaderChecksum(iph);

  return tcph->TcpCheckSum;
}

/**
 *  Compute a ZkUDPIPHeader checksum.
 *
 */

DLLExport uint16_t
zkUDPHeaderChecksum (
  ZkUDPHeader * udph ,
  size_t datalen )
{
  unsigned long cksum = 0;
  unsigned short * data = NULL;
  ZkIPHeader * iph = NULL;
  size_t i = 0;

  if (!udph) {
    return 0;
  }

  /*
   * First we compute a checksum on the TCP pseudo header (this consists of
   * the source and destination IP addresses, the lengths listed in the IP
   * packet and the IP protocol code (6).
   */

  iph = &(udph->IP);
  udph->UdpCheckSum = 0;

  datalen = ntohs(iph->TotalLength) - (iph->ihl * 4);
  data = (unsigned short *) &(iph->SrcIpAddress);
  cksum = data[0] + data[1];
  data = (unsigned short *) &(iph->DstIpAddress);
  cksum += data[0] + data[1];
  cksum += htons(ZK_IPPROTO_UDP) + 
           htons((USHORT) (ntohs(iph->TotalLength) - (iph->ihl * 4)));

  /*
   * Possibly pad one extra byte at the end of the data so that we can
   * properly convert the data to an array of unsigned shorts.
   */

  if (datalen % 2) {
    ((char *)udph)[datalen + (iph->ihl * 4)] = (char) 0;
    datalen += 1;
  }
  datalen >>= 1;

  /*
   * Compute the checksum
   */

  data = (unsigned short *) &(iph[1]);
  for (i = 0; i < datalen; i++) {
    cksum += *data++;
  }

  /*
   * Handle the overflow
   */

  cksum = (cksum >> 16) + (cksum & 0xFFFF);
  cksum += (cksum >> 16);
  udph->UdpCheckSum = ~cksum;

  zkIPHeaderChecksum(iph);

  return udph->UdpCheckSum;
}


