/*
 * $Id: zkIP.c,v 1.10.16.3 2000/12/14 17:54:34 jean Exp $
 */

static const char * module_id_ = "$Id: zkIP.c,v 1.10.16.3 2000/12/14 17:54:34 jean Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom(R) code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <stdio.h>
#include <stdlib.h>

#ifndef WIN32
#include <netinet/in.h> /* htons(), ntohs(), ... */
#else
#include <winsock2.h>
#endif

#include <zkPacket.h>

#define __IPPACKET_SZ          20
#define	__UDPPACKET_SZ          8
#define __TCPPACKET_SZ         20

/**
 *  Compute a ZkIPHeader checksum.
 *
 *
 */

uint16_t
zkIPHeaderChecksum (
  ZkIPHeader * iph )
{
  unsigned long cksum = 0;
  unsigned short * data = NULL;
  size_t datalen = 0;
  size_t i = 0;

  (void) module_id_; /* Get rid of compile warning */
  if (!iph) {
    return 0;
  }
  iph->HeaderChkSum = 0;
  data = (unsigned short *) iph;
  datalen = iph->ihl * 2;
  cksum = 0;
  for (i = 0; i < datalen; i++) {
    cksum += *data++;
  }
  cksum = (cksum >> 16) + (cksum & 0xFFFF);
  cksum += (cksum >> 16);
  iph->HeaderChkSum = ~cksum;

  return iph->HeaderChkSum;
}

static char *
zkIPAddrToStr (
  uint32_t addr )
{
  unsigned char * ptr = NULL;
  static char buffer[32];

  ptr = (unsigned char *) &addr;
  snprintf(buffer,sizeof buffer,"%d.%d.%d.%d",ptr[0],ptr[1],ptr[2],ptr[3]);
  return buffer;
}
  
int
zkIPHeaderDump (
  ZkIPHeader * iph ,
  FILE * stream )
{

  fprintf(stream,"IP Packet Header pointer %p\n", (void *)iph);
  if (iph == NULL) {
    return 0;
  }
  fprintf(stream,"  IP Packet Length        %d bytes\n",iph->ihl * 4);
  fprintf(stream,"  Version                 %d\n",iph->version);
  fprintf(stream,"  Type Of Service         %d\n",iph->TypeOfSvr);
  fprintf(stream,"  Total Length            %d\n",ntohs(iph->TotalLength));
  fprintf(stream,"  Identification          %d\n",ntohs(iph->Identification));
  fprintf(stream,"  Fragmentation Flags     0x%X\n",
          (ntohs(iph->Flags_FrameOffs)&0xE000)>>13);
  fprintf(stream,"  Fragmentation Offset    %d\n",
          ntohs(iph->Flags_FrameOffs)&0x1FFF);
  fprintf(stream,"  Time To Live            %d\n",iph->TimeToLive);
  fprintf(stream,"  Protocol                %d\n",iph->Protocol);
  fprintf(stream,"  Header Checksum         %d\n",iph->HeaderChkSum);
  fprintf(stream,"  Source IP Address       %s\n",
          zkIPAddrToStr(iph->SrcIpAddress));
  fprintf(stream,"  Destination IP Address  %s\n",
          zkIPAddrToStr(iph->DstIpAddress));

  return 1;
}

int
zkIPHeaderSourceAddress (
  ZkIPHeader * iph ,
  uint32_t * addr )
{
  *addr = iph->SrcIpAddress;
  return 1;
}

int
zkIPHeaderDestinationAddress (
  ZkIPHeader * iph ,
  uint32_t * addr )
{
  *addr = iph->DstIpAddress;
  return 1;
}

int
zkIPHeaderSetSourceAddress (
  ZkIPHeader * iph ,
  uint32_t addr )
{
  iph->SrcIpAddress = addr;
  return 1;
}
 
DLLExport int
zkIPHeaderSetDestinationAddress (
  ZkIPHeader * iph ,
  uint32_t addr )
{
  iph->DstIpAddress = addr;
  return 1;
}

/**
 *  Strip out all of the IP packet options.  This is fairly draconian but
 *  what we're doing either makes the IP options useless (in the case of
 *  source routing) or dangerous (in the case of route record).
 *  This function assumes that the data immediatly follows the IP header in
 *  memory.  You must recompute the IP checksum after striping the options 
 *  (this function will not recompute it for you).
 *  Note:  In order to avoid breaking things, I am copying the data portion
 *         of the packet overtop the IP options.  It may be more efficient to
 *         go the otherway, ie copy the header data forward to overwrite the
 *         options, however, this then makes the IP header pointer move.
 *  Note:  We may want to keep one option, StreamID, if it is preserved
 *         both ways, we might be able to use it to keep track of ACI <->
 *         ICMP message going through the wormhole.
 *
 *
 *  @param iph  The IP packet header pointer
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkIPHeaderStripOptions (
  ZkIPHeader * iph )
{
  int i = 0;
  int len = 0;
  char * cat = NULL;
  char * mouse = NULL;

  if (iph->ihl == 5) {
    return 1;
  } else if (iph->ihl < 5) {
    return 0;        /* Cant have a proper IP packet with length  < 5 */
  }
  cat = &(((char *)iph)[20]);
  mouse = &(((char *)iph)[iph->ihl * 4]);
  len = (int) ntohs(iph->TotalLength) - (iph->ihl * 4);
  for (i = 0; i < len; i++) {
    *cat++ = *mouse++; 
  }
  iph->TotalLength = htons(ntohs(iph->TotalLength) - (iph->ihl * 4 - 20));
  iph->ihl = 5;
  iph->HeaderChkSum = 0;
  return 1;
}
