/*
 * $Id: zkTCP.c,v 1.10.20.2 2001/01/09 22:25:26 mario Exp $
 */

static const char * module_id_ = "$Id: zkTCP.c,v 1.10.20.2 2001/01/09 22:25:26 mario Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <stdio.h>
#include <stdlib.h>

#ifndef WIN32
#include <netinet/in.h> /* htons(), ntohs(), ... */
#else
#include <winsock2.h>
#endif

#include <zkPacket.h>

DLLExport int
zkTCPHeaderDump (
  ZkTCPHeader * tcph ,
  FILE * stream )
{
  char yn[2] = { ' ', 'X' };

  (void) module_id_; /* Get rid of compile warning */
  fprintf(stream,"TCP Packet Header Pointer %p\n",(void *)tcph);
  if (tcph == NULL) {
    return 0;
  }
  fprintf(stream,"  Source Port             %d\n",ntohs(tcph->SrcPort));
  fprintf(stream,"  Destination Port        %d\n",ntohs(tcph->DstPort));
  fprintf(stream,"  Sequence Number         %lu\n",(unsigned long)ntohl(tcph->SequenceNumber));
  fprintf(stream,"  Acknowledgement number  %lu\n",(unsigned long)ntohl(tcph->AckNumber));
  fprintf(stream,"  Reserved1               %d\n",tcph->res1);
  fprintf(stream,"  Header Length           %d\n",tcph->doff * 4);
  fprintf(stream,"  FIN flag                %c\n",yn[tcph->fin]);
  fprintf(stream,"  SYN flag                %c\n",yn[tcph->syn]);
  fprintf(stream,"  RST flag                %c\n",yn[tcph->rst]);
  fprintf(stream,"  PSH flag                %c\n",yn[tcph->psh]);
  fprintf(stream,"  ACK flag                %c\n",yn[tcph->ack]);
  fprintf(stream,"  URG flag                %c\n",yn[tcph->urg]);
  fprintf(stream,"  Reserved2               %d\n",tcph->res2);
  fprintf(stream,"  Window                  %d\n",htons(tcph->Window));
  fprintf(stream,"  Header Checksum         %d\n",tcph->TcpCheckSum);
  fprintf(stream,"  Urgent Pointer          %d\n",ntohs(tcph->UrgentPointer));
  return 1;
}

DLLExport int
zkTCPHeaderAcknowledgement (
  ZkTCPHeader * tcph ,
  uint32_t * ack )
{
  if (tcph && ack) {
    *ack = ntohl(tcph->AckNumber);
    return 1;
  } 
  return 0;
} 

DLLExport int
zkTCPHeaderSequence (
  ZkTCPHeader * tcph ,
  uint32_t * seq )
{
  if (tcph && seq) {
    *seq = ntohl(tcph->SequenceNumber);
    return 1;
  }
  return 0;
}



uint16_t
zkTCPHeaderSourcePort (
  ZkTCPHeader * tcph ,
  uint16_t * port ) 
{
  *port = ntohs(tcph->SrcPort);
  return *port ;
}

uint16_t
zkTCPHeaderDestinationPort (
  ZkTCPHeader * tcph ,
  uint16_t * port )
{
  *port = ntohs(tcph->DstPort);
  return *port;
}

int
zkTCPHeaderSetSourcePort (
  ZkTCPHeader * tcph ,
  uint16_t port )
{  
  tcph->SrcPort = htons(port);
  return 1;
}

int
zkTCPHeaderSetDestinationPort (
  ZkTCPHeader * tcph ,
  uint16_t port )
{
  tcph->DstPort = htons(port);
  return 1;
}

/**
 *  Set the maximum segment size option on the TCP packet to the specified
 *  value.  This function will move the packet data payload forward 4 bytes
 *  in order to accomodate this TCP option.  The header lengths will be 
 *  updated accordingly but the header checksums will not.  You must be
 *  sure to recompute them yourself.
 * 
 *  @param tcph  A pointer to a TCP header structure
 *  @param mss   The new Maximum Segment Size option
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int 
zkTCPHeaderSetMSS (
  ZkTCPHeader * tcph ,
  uint16_t mss )
{
  int i = 0; /* Loop index for moving packet payload */
  int len = 0;  /* Number of bytes to move */
  int hlen = 0;  /* Length of combined headers */
  char * cat = NULL;
  char * mouse = NULL;
  unsigned char *mss_ptr = NULL;
  
  /*
   * Move the payload forward by 4 bytes
   */

  cat = &(((char *)tcph)[ntohs(tcph->IP.TotalLength) - 1]);
  mouse = &(((char *)tcph)[ntohs(tcph->IP.TotalLength) - 5]);
  hlen = (tcph->IP.ihl + tcph->doff) * 4;
  len = ntohs(tcph->IP.TotalLength) - hlen;
  for (i = 0; i < len; i++) {
    *cat-- = *mouse--;
  }
  
  /*
   * Append the MSS to the end of the option list
   */

  mss_ptr = (unsigned char *) &(((char *)tcph)[hlen]);
  mss_ptr[0] = 0x02; /* Set the MSS option code, 2 */
  mss_ptr[1] = 0x04; /* Set the MSS length, 4 */
  mss_ptr[2] = (unsigned char) (mss / 0x100);
  mss_ptr[3] = (unsigned char) (mss % 0x100);
#if 0
  mss_ptr[0] = 0x0402;   /* Set the MSS option code, kind 2, length 4 */
  mss_ptr[1] = htons(mss);  /* Set the new MSS value */
#endif

  /*
   * Update header lengths
   */

  tcph->doff += 1;
  tcph->IP.TotalLength = htons(ntohs(tcph->IP.TotalLength) + 4);
  tcph->TcpCheckSum = 0;
  tcph->IP.HeaderChkSum = 0;
  return 1;
}

/**
 *  This function will remove all of the TCP options from the TCP header.  It
 *  assumes that the accompanying packet data follows immediatly after in 
 *  memory and that the tcp header also has a proper IP header immediatly
 *  before.
 *
 *  @param tcph The TCP header to strip of options
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure.
 */

DLLExport int
zkTCPHeaderStripOptions (
  ZkTCPHeader * tcph )
{
  int i = 0; /* Loop index */
  int len = 0;
  char * cat = NULL;
  char * mouse = NULL;

  if (tcph->doff == 5) {
    return 1;
  } else if (tcph->doff < 5) {
    return 0;
  }
  cat = &(((char *)tcph)[tcph->IP.ihl * 4 + 20]);
  mouse = &(((char *)tcph)[(tcph->IP.ihl + tcph->doff) * 4]);
  len = ntohs(tcph->IP.TotalLength) - (tcph->IP.ihl + tcph->doff) * 4;
  for (i = 0; i < len; i++) {
    *cat++ = *mouse++;
  }
  tcph->IP.TotalLength = htons(ntohs(tcph->IP.TotalLength)
                               - (tcph->doff * 4 - 20) );
  tcph->doff = 5;
  tcph->TcpCheckSum = 0;
  tcph->IP.HeaderChkSum = 0;
  return 1;
}


/*
 *  Modify the existing MSS value.  If the MSS option cannot
 *  be found, the function fails and returns a 0.
 *
 *  @param out_tcph The TCP header structure to analyze.  It is assumed that the
 *              packet data follows immediatly after in memory.
 *  @param in_mss  New MSS value in host byte order HBO
 *  @author Mario Contestabile
 *  @version 1.0
 *  @return A non-zero value if the MSS option was found, 0 otherwise.
*/
DLLExport int
zkTCPHeaderModifyMSS (
  ZkTCPHeader * out_tcph ,
  const uint16_t in_mss )
{
  unsigned char * ptr = NULL;
  int len = 0;
  int i = 0; /* Loop index for scanning option data */

  if (out_tcph->doff <= 5) {
    return 0;      /* No options means no MSS specified */
  }
  len = out_tcph->doff * 4 - 20;  /* Number of bytes devoted to options */
  ptr = (unsigned char *) &(out_tcph[1]);
  for (i = 0; i < len; i++,ptr++) {
    switch (*ptr) {
    case ZK_TCPOPT_EOL:  /* End of option list, terminate... */
      i = len;
      break;
    case ZK_TCPOPT_NOP: /* Nop, skip it */
      break;
    case ZK_TCPOPT_MSS: /* This is the MSS option */
      i++;
      if (i >= len) {
        break;  /* Do we have enough data left in the packet for an MSS ? */
      } 
      ptr++;
      if (*ptr != 4) {   /* Is the length correct */
        i = len;
        break;
      }
      ptr++;

      /* There are two bytes where to store the mss value */
      ptr[0] = (unsigned char) (in_mss / 0x100);
      ptr[1] = (unsigned char) (in_mss % 0x100);
      return 1;
      break;
    default:
      i++; /* Skip the length byte */
      ptr++;
      break;
    }
  }
  return 0;
}

/**
 *  Analyze a TCP packet and return the MSS option.  If this option cannot
 *  be found, the function fails and returns a 0.
 *
 *  @param tcph The TCP header structure to analyze.  It is assumed that the
 *              packet data follows immediatly after in memory.
 *  @param mss  Returns the MSS (Maximum Segment Size for the connection)
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value if the MSS option was found, 0 otherwise.
 */

DLLExport int
zkTCPHeaderGetMSS (
  ZkTCPHeader * tcph ,
  uint16_t * mss )
{
  unsigned char * ptr = NULL;
  int len = 0;
  int i = 0; /* Loop index for scanning option data */

  if (tcph->doff <= 5) {
    return 0;      /* No options means no MSS specified */
  }
  len = tcph->doff * 4 - 20;  /* Number of bytes devoted to options */
  ptr = (unsigned char *) &(tcph[1]);
  for (i = 0; i < len; i++,ptr++) {
    switch (*ptr) {
    case 0:  /* End of option list, terminate... */
      i = len;
      break;
    case 1: /* Nop, skip it */
      break;
    case 2: /* This is the MSS option */
      i++;
      if (i >= len) {
        break;  /* Do we have enough data left in the packet for an MSS ? */
      } 
      ptr++;
      if (*ptr != 4) {   /* Is the length correct */
        i = len;
        break;
      }
      ptr++;
#if 0
      *mss = (uint16_t) ntohs(*((uint16_t *)ptr));  /* Get the MSS */
#else
      *mss = (uint16_t) ptr[0] * 0x100 + ptr[1];
#endif
      return 1;
      break;
    default:
      i++; /* Skip the length byte */
      ptr++;
      break;
    }
  }
  return 0;
}  

