#!/usr/bin/perl -w
# 
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
# 
# The Original Code is Freedom(R) code.
# 
# The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
# Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
# Zero-Knowledge Systems, Inc.  All Rights Reserved.
# 
# Contributor(s):


# Used to get the name of the OS
use Config;

sub CountWarnings {
    my ($localdir, $listfile, $lines, $warnings) = @_;
    my $module = undef;
    my %counts = ();
    my $directories = {};
    my $listfileOpen = 1;
    my $length = 0;
    my $newModule = 0;
    my $prevWarn = "";

    # Open the listfile for writting
    if (!open(LISTFILE, ">$listfile")) {
      print("Error: can't open '$listfile'\n");
      $listfileOpen = 0;
    } else {
	$listfileOpen = 1;
    }

    # Parse each line to report on
    foreach (@{$lines}) {
	# Found start of a new module (Win32 || Linux)
	if ((m,--------------------Configuration: (.+) - Win32 (.+)--------------------,) || (m,Entering directory \`$localdir/(.+)\',)) {
	    # If we're on Win32 and this is a new module, print a few blank lines for readability
	    if (($newModule == 1) && ($Config::Config{osname} eq "MSWin32")) {
		print(LISTFILE "\n\n");
	    }

	    # Reset the new module indicator
	    $newModule = 0;

	    # Save the module name for later use
	    $module = $1;

	    # Compute length used to display module name esthetically
	    $length = length($1) + 12;

	    # Initialize the warning counters
	    if (!exists($directories->{$module})) {
		$directories->{$module}->{P0} = 0;
		$directories->{$module}->{P1} = 0;
		$directories->{$module}->{P2} = 0;
		$directories->{$module}->{P3} = 0;
		$directories->{$module}->{P4} = 0;	
		$directories->{$module}->{P5} = 0;
	    }
	    next;
	}

	# Found end of module (Linux)
	if ((m,Leaving directory \`$localdir/(.+)\',) && ($newModule == 1)) {
	    print(LISTFILE "\n");
	    $module = $1;
	}

	# Found compilation warning (Win32 || Linux) or link warning (Win32)
        if ((m,^(.+\(\d+\)) : warning (C\d+: .+),) || (m,^(.+\d+): warning: (.+),) || (m,^(.*)LINK : warning (LNK\d+:.+$),)) {
	    # Check if this is the start of a new module
	    if (!$newModule) {
		$newModule = 1;

		# Display the module header in the listfile
		if (defined($listfileOpen)) {
		    print(LISTFILE "*" x $length, "\n");
		    print(LISTFILE "* Module: $module *\n");
		    print(LISTFILE "*" x $length, "\n");
		}
	    }

	    # Ignore repetitive warnings
    	    next if (($prevWarn eq $_) && ($newModule == 0));

	    my $file = $1;
	    my $warning = $2;

	    # Display the warning in the list file
	    print(LISTFILE "$_") if (defined($listfileOpen));

	    # Generalize the warning
            if ($Config::Config{osname} eq "MSWin32") {
		$warning =~ s,\'[^\']+\',\'XXX\',g;
	    	$warning =~ s,parameter \d+,parameter XXX,g;
		$warning=~ s,references to \"(.+)\" discarded,references to \"XXX\" discarded,g;
	    } else {
		$warning =~ s,\`[^\']+\',\`XXX\',g;
		$warning =~ s,arg \d+,arg XXX,g;
		$warning =~ s,\",\',g;
		$warning =~ s,argument \d+,argument XXX,g;
	    }

	    $file = "$module/$file" if ($file !~ m,^/,);

	    # Increment the associated warning counter
	    $counts{$warning} = exists($counts{$warning}) ? $counts{$warning}+1 : 1;

	    # Increment the warning count for the proper priority of the current module
	    if (exists($warnings{$warning}) && ($warnings{$warning}{priority} =~ m,^P[0-5]$,)) {
		$directories->{$module}->{$warnings{$warning}{priority}}++;
	    } else {
		$directories->{$module}->{P0}++;
	    }

	    # Save the warning so we can test for repetitiveness
	    $prevWarn = $_;
	}
    }

    return(\%counts, $directories, \$warnings);
}

sub main {
    my ($localdir, $warningfile, $logfile, $listfile, @files) = @_;
    my $file = undef;
    my @lines = ();
    my $counts = undef;
    my $directories = undef;
    my %total = (P0 => 0, P1 => 0, P2 => 0, P3 => 0, P4 => 0, P5 => 0);
    my $total = 0;
    my $p = undef;
    my $warnings = {};
    
    # Remove trailing /. at the end of $localdir
    $localdir =~ s,[\\/]+\.$,,;

    # Store each generalized warning line
    if (open(WARNFILE, $warningfile)) {
        # Store warning lines
        push(@warninglines, <WARNFILE>);
        close(WARNFILE);
    }

    # Create the hash table of warnings/priority
    while (@warninglines) {
        $_ = pop(@warninglines);
        (m,^\"(.+)\" => \"(P\d{1}),);
        $warnings{$1}{priority} = $2 if (!exists($warnings{$1}));
    }

    # Store each line of all files to report on
    while(@files) {
	$file = shift(@files);
	if (open(FILE, $file)) {
	    push(@lines, <FILE>);
	    close(FILE);
	} else {
	    print("Error: can't open '$file'\n");
	    exit(1);
	}
    }

    # Open the logfile for writting
    if (!open(OUTFILE, ">$logfile")) {
      print("Error: can't open '$logfile'\n");
      exit(1);
    }

    # Count the warnings
    ($counts, $directories, $warnings) = &CountWarnings($localdir, $listfile, \@lines, \$warnings);

    # Display the number of warnings sorted by module
    print(OUTFILE "***************************************\n");
    print(OUTFILE "* Number of warnings sorted by module *\n");
    print(OUTFILE "***************************************\n\n");
    print(OUTFILE "P0\tP1\tP2\tP3\tP4\tP5\tTotal\tModule\n");
    print(OUTFILE "=" x 80, "\n");

    foreach (sort(keys(%{$directories}))) {
	next if ($directories->{$_}->{P0} == 0 &&
		 $directories->{$_}->{P1} == 0 &&
		 $directories->{$_}->{P2} == 0 &&
		 $directories->{$_}->{P3} == 0 &&
		 $directories->{$_}->{P4} == 0 &&
		 $directories->{$_}->{P5} == 0);
	print(OUTFILE "$directories->{$_}->{P0}\t",
		"$directories->{$_}->{P1}\t",
	      	"$directories->{$_}->{P2}\t",
	      	"$directories->{$_}->{P3}\t",
	      	"$directories->{$_}->{P4}\t",
	      	"$directories->{$_}->{P5}\t");
	my $moduleTotal = $directories->{$_}->{P0}+
		$directories->{$_}->{P1}+
		$directories->{$_}->{P2}+
		$directories->{$_}->{P3}+
		$directories->{$_}->{P4}+
		$directories->{$_}->{P5};
	print(OUTFILE "$moduleTotal\t$_\n");
	foreach $p (keys(%total)) {
	    $total{$p} += $directories->{$_}->{$p};
	}
    }

    # Display the total number of warnings sorted by priority
    my $totalWarns = $total{P0}+$total{P1}+$total{P2}+$total{P3}+$total{P4}+$total{P5};
    print(OUTFILE "=" x 80, "\n");
    print(OUTFILE "$total{P0}\t$total{P1}\t$total{P2}\t$total{P3}\t$total{P4}\t$total{P5}\t$totalWarns\tAll modules\n\n\n");

    # Display the warnings sorted by reverse count
    print(OUTFILE "********************************\n");
    print(OUTFILE "* Warnings sorted by frequency *\n");
    print(OUTFILE "********************************\n\n");
    foreach (sort {$counts->{$a} <=> $counts->{$b}} keys(%{$counts})) {
	print(OUTFILE sprintf("%4d %s\n", $counts->{$_}, $_));
    }

    # Display the list of unprioritized general warnings and their frequency
    print(OUTFILE "\n\n**************************\n");
    print(OUTFILE "* Unprioritized warnings *\n");
    print(OUTFILE "**************************\n\n");
    foreach (sort(keys(%{$counts}))) {
	if (!exists($warnings{$_}) || $warnings{$_}{priority} eq "P0") {
	    print(OUTFILE "$counts->{$_}\t$_\n");
	    $total += $counts->{$_};
	}
    }

    # Display the total number of unprioritized warnings
    print(OUTFILE "=" x 80, "\n");
    print(OUTFILE "$total\tUnprioritized warnings\n");

    close(OUTFILE);
}

&main(@ARGV);
