#!/usr/bin/perl -w
# 
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
# 
# The Original Code is Freedom(R) code.
# 
# The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
# Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
# Zero-Knowledge Systems, Inc.  All Rights Reserved.
# 
# Contributor(s):


BEGIN {
  $GscriptDir = $0;
  $GscriptDir =~ s,\\,/,g;
  $GscriptDir =~ s,/?[^/]+$,,;
  $GscriptDir = "." if ($GscriptDir eq "");
  unshift(@INC, $GscriptDir);
}

use File::Basename;
use POSIX;

sub main {
  my ($dswFileName, $logFileName) = @_;
  my $baseDir = undef;
  my @dswLines = ();

  # Open the specified .DSW file
  if (open(DSWFILE, $dswFileName)) {
    unshift(@dswLines, <DSWFILE>);
    close(DSWFILE);
  } else {
    print("Error: can't open .DSW file '$dswFileName'\n");
    exit(1);
  }

  # Get the base diretory of the source tree
  $baseDir = &File::Basename::dirname($dswFileName);

  # Save the current working directory
  my $cwd = &POSIX::getcwd();
  
  # Change into the base directory of the source tree
  if (chdir($baseDir)) {
    # Overwrite the old log file before appending
    if (!open(LOGFILE, ">", $logFileName)) {
      print("Error: can't open .DSW file '$dswFileName'\n");
      exit(1);
    }
    close(LOGFILE);

    # Parse the .DSW file
    while (@dswLines) {
      $_ = shift(@dswLines);

      # Find out which .DSP files are in the .DSW file
      if (m,^Project: \"(.+)\"=(.+) - Package,) {
	my $project = $1;
	my $dspFileName = $2;
	my $length = length($project) + 4;

	# Get the project subdirectory
	my $projectSubdir = &File::Basename::dirname($dspFileName);
	
	# Open the specified log file for output
	if (!open(LOGFILE, ">>", $logFileName)) {
	  print("Error: can't open log file '$logFileName'\n");
	  exit(1);
	}

	# Display project header in log file
	print(LOGFILE "\n\n", "*" x $length, "\n");
	print(LOGFILE "* $project *\n");
	print(LOGFILE "*" x $length, "\n");
	close(LOGFILE);

	# Open the .DSP file
	if (open(DSPFILE, $dspFileName)) {
	  my $dspLines = ();
	  
	  # Store the lines from the .DSP file
	  unshift(@dspLines, <DSPFILE>);
	  close(DSPFILE);

	  while (@dspLines) {
	    $_ = shift(@dspLines);
	    # Find out which source files are in the .DSP file
	    if (m,^SOURCE=(.+\.)(c|cpp)$,) {
	      my $srcFile = "$1$2";

	      # Run ITS4 on the source file
	      if (system("its4 -W $projectSubdir\\$srcFile >> $logFileName")) {
		print("Error: 'its4 -W $projectSubdir\\$srcFile >> $logFileName' exited with an error code\n");
	      }
	    }
	  }
	} else {
	  # Couldn't open .DSP file, skip it
	  print("Warning: can't open .DSP file '$dspFileName', file skipped\n");
	}
      }
    }

    # Change back to previous working directory
    if (!chdir($cwd)) {
      print("Error: can't chdir back to '$cwd'\n");
      exit(1);
    }
  } else {
    print("Error: can't chdir to '$baseDir'\n");
    exit(1);
  }
}

&main(@ARGV);
