#!/usr/bin/perl -w
# 
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
# 
# The Original Code is Freedom(R) code.
# 
# The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
# Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
# Zero-Knowledge Systems, Inc.  All Rights Reserved.
# 
# Contributor(s):


use Config;

sub main {
  my $logfile = $ARGV[0];
  my $warningfile = $ARGV[1];
  my @loglines = ();
  my @newwarnings = ();
  my $warnings = {};
  my $warning = "";
  my $osname = $Config::Config{osname};

  # Open warning list file
  if (open(WARNFILE, $warningfile)) {
    # Store warning lines
    push(@warninglines, <WARNFILE>);
    close(WARNFILE);
  } else {
    print("Error: can't open '$warningfile'\n");
  }

  # Add the warnings to th ehash table
  while (@warninglines) {
    $_ = pop(@warninglines);
    (m,^\"(.+)\" => \"(P\d{1}),);
    if (!exists($warnings->{$1})) {
      $warnings->{$1}->{priority} = $2;
    }
  }

  # Open log file
  if (open(LOGFILE, $logfile)) {
    # Store log lines
    push(@loglines, <LOGFILE>);
    close(LOGFILE);
  } else {
    print("Error: can't open '$logfile'\n");
    exit(1);
  }

  # Parse every line in the log file
  while (@loglines) {
    $_ = pop(@loglines);

    # Compilation (Win32 || Linux) or link (Win32) warning found
    if ((m,^(.+\(\d+\)) : warning (C\d+: .+),) || (m,^(.+\d+): warning: (.+),) || (m,^(.*)LINK : warning (LNK\d+:.+$),)) {
      $warning = $2;

      #  Generalize the warning
      if ($osname eq "MSWin32") {
	$warning =~ s,parameter \d+,parameter XXX,g;
	$warning =~ s,\'[^\']+\',\'XXX\',g;
	$warning=~ s,references to \"(.+)\" discarded,references to \"XXX\" discarded,g;
      } else {
	$warning =~ s,\`[^\']+\',\`XXX\',g;
	$warning =~ s,arg \d+,arg YYY,g;
	$warning =~ s,\",\',g;
	$warning =~ s,argument \d+,argument XXX,g;
    }

      # Check if this is a new warning
      if (!exists($warnings->{$warning})) {
	$warnings->{$warning}->{priority} = "P0";
	push(@newwarnings, $warning);
      }
    }

    # Update the warnings list by adding the new ones
    if (open(WARNFILE, ">>$warningfile")) {
      while (@newwarnings) {
	$_ = pop(@newwarnings);
	my $newwarning = "\"$_\" => \"P0\"\n";
	print WARNFILE "$newwarning";
      }
    } else {
        print("Error: can't open '$warningfile' for writting\n");
    }
  }
}

&main(@ARGV);
