#
# CVS Utilities functions
#
#
use FileHandle;

@ISA=(Exporter);
@EXPORT=qw();
@EXPORT_OK=qw();

package ZKCvs;

sub GetRCSFilename {
    my ($cvsroot, $dir, $file) = @_;

    # Check dir and make sure not in Attic
    if (-f "$cvsroot/$dir/$file,v") {
	if (! -f "$cvsroot/$dir/Attic/$file,v") {
	    return("$cvsroot/$dir/$file,v");
	} else {
	    print("Duplicate: '$cvsroot/$dir/$file,v'\n");
	    return(undef);
	}
    } else {
	# Check for Attic
	if (-f "$cvsroot/$dir/Attic/$file,v") {
	    return("$cvsroot/$dir/Attic/$file,v");
	} else {
	    return(undef);
	}
    }
    return(undef);
}

sub GetRevisionBranch {
    my ($filename) = @_;
    my %symbols = ();
    my $fh = new FileHandle;

    # Read the RCS file until all symbols are processed
    if (defined($fh) && $fh->open("<$filename")) {
	my $inSymbols = 0;
	
	while(<$fh>) {
	    if ($inSymbols) {
		if (m,^\t([^:]+):\s*(\d+(\.\d+)+)\s*;?$,) {
		    my $symver = $2;
		    my $symname = $1;
		    
		    if ($symver =~ m,(.*\d+)\.0(\.\d+)$,) {
			$symver = "$1$2";
		    } else {
			# This is now a branch, ignore it
			next
		    }
		    $symbols{$symver} = $symname;
		    next;
		}
		last;
	    }
	    if (m,^symbols\s*$,) {
		$inSymbols = 1;
		next;
	    }
	}
	$fh->close();
    } else {
	print("Error: opening '$filename'\n");
    }
    return(\%symbols);
}

sub GetCurrentFileVersion {
    my ($localdir, $reppath, $info) = @_;
    my $status = 0;

    # Valid args
    if ( !defined($localdir)) {
	print("Error: missing directory name in GetCurrentTreeStatus\n");
	return(1);
    }

    if ( !defined($info) || (ref($info) ne "HASH")) {
	print("Error: invalid arg to GetCurrentTreeStatus\n");
	return(1);
    }

    if ( chdir($localdir)) {
	my $fh = new FileHandle("cvs status 2>&1 |");

	if ( defined($fh)) {
	    while ($_ = $fh->getline()) {
		chomp;

		# Skip white lines
		next if (m,^\s*$,);

		# Reset info
		if ( m,^=+$,) {
		    $file = $version = undef;
		    next;
		}

		# Get the current version in this tree
		if ( m,^\s+Working revision:\s*(\d+(\.\d+)+)\s*$,) {
		    $version = $1;
		    next;
		}

		if (m/^\s+Repository revision:\s*\d+(\.\d+)+\s+(.+),v\s*$/) {
		    if ( !defined($version)) {
			print("Error: no version for '$2'\n");
			$fh->getlines();
			$status = 1;
			last;
		    }
		    $file = $2;

		    # Remove the Attic from the repository filename
		    $file =~ s,/Attic/([^/]+)$,$1/,;
		    
		    # Remove repository path
		    if (defined($reppath)) {
			$file =~ s,^\Q$reppath\E,,;
			$file =~ s,^/+,,;
		    }
		    $info->{$file} = $version;
		    $file = $version = undef;
		    next;
		}
	    }
	    $fh->close();
	} else {
	    print("Error: can't run 'cvs stat' in '$localdir'\n");
	    return(1);
	}
    } else {
	print("Error: can't chdir into '$localdir'\n");
	return(1);
    }
    
    return($status);
}

1;
