package ZKFTP;
use FileHandle;
use strict;
use vars qw($VERSION);

$VERSION = "0.10";


sub new {
    my ($self, $host, $user, $passwd, $localdir, $remotedir) = @_;

    return(undef) if (!defined($host) || !defined($user) || !defined($passwd));

    bless {
	debug		=> 1,		# debug mode
	host		=> $host,	# the hostname
	localdir	=> defined($localdir) ? $localdir : undef,	# local directory
	mode		=> "binary",	# transfer mode
	passwd		=> $passwd,	# the user passwd
	remotedir	=> defined($remotedir) ? $remotedir : undef,	# remote directory
	user		=> $user,	# the user
    }, $self;
}

sub tmpdir {
    return($ENV{TEMP}) if (exists($ENV{TEMP}));
    return($ENV{TMP}) if (exists($ENV{TMP}));
    return(".");
}

sub ftp {
    my ($self, $cmd) = @_;
    my $tmpfile = &tmpdir() . "\\$$.lst";
    my $fh = new FileHandle;
    my $status = 0;

    # Build the commands file
    if ($fh->open(">$tmpfile")) {
	$fh->print("$self->{user}\n");
	$fh->print("$self->{passwd}\n");
	$fh->print("$self->{mode}\n");
	$fh->print("lcd $self->{localdir}\n") if (defined($self->{localdir}));
	$fh->print("cd $self->{remotedir}\n") if (defined($self->{remotedir}));
	$fh->print("$cmd\n");
	$fh->print("quit\n");
	$fh->close();
    } else {
	# Can not create temporary file
	return(1);
    }
    
    # Start ftp and feed it the commands file
    if ($fh->open("ftp -i -s:$tmpfile $self->{host} 2>&1 |")) {
	# Parse the result
	while(<$fh>) {
	    print("$_") if (defined($self->{debug}));
	    chomp;
	    if ((m,^.+: File not found\s*$,) ||
		(m,^(5\d\d|Invalid command),)) {
		next if (m,^521,);
		my @buf = <$fh>;

		$status = 1;
		last;
	    }
	}
	$fh->close();
	$status = 1 if ($?);
    } else {
	$status = 1;
    }

    unlink($tmpfile);
    return($status);
}

sub mkdir {
    my ($self, $dir) = @_;

    return($self->ftp("mkdir $dir"));
}

sub mput {
    my ($self, $files, $mode) = @_;
    my $status = 0;

    $files = [$files] if (ref($files) ne "ARRAY");
    $status = $self->ftp("mput " . join(" ", @{$files}));
    return($status);
}

sub remotedir {
    my ($self, $dir) = @_;

    $self->{remotedir} = $dir;
}

sub localdir {
    my ($self, $dir) = @_;

    $self->{localdir} = $dir;
}

sub mode {
    my ($self, $mode) = @_;

    if ($mode =~ m,^(binary|ascii),i) {
	$self->{mode} = lc($mode);
    }
}

