#!/usr/bin/perl -w
# 
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
# 
# The Original Code is Freedom(R) code.
# 
# The Initial Developer of the Original Code is Zero-Knowledge(R) Systems,
# Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
# Zero-Knowledge Systems, Inc.  All Rights Reserved.
# 
# Contributor(s):


use POSIX;
use Config;

sub Getopts {
    my $prefix = undef;

    while (@_) {
	$_ = shift(@_);

	if ( m,^-prefix$,i) {
	    if ( @_) {
		$prefix = shift(@_);
	    } else {
		print(STDERR "Error: missing arg to $_ option\n");
		exit(1);
	    }
	    next;
	}
    }
    if ( !defined($prefix)) {
	print(STDERR "Usage: $0 <-prefix prefix>\n");
	exit(1);
    }
    return($prefix);
}


if ( "MSWin32" eq $Config::Config{osname}) {
    my $prefix = Getopts(@ARGV);
    my $cwdrive = POSIX::getcwd(); $cwdrive =~ s,^(..).*,$1,;

    if(chdir("/freedom/client")) {
	foreach my $partner ("freeserve", "simtel", "tucows", "winsite", "zdnet") {
	    if (system("copy /y regfiles\\inituser.$partner.reg inituser.reg")) {
		print("Error copying inituser.reg for $partner\n");
		exit(1);
	    }
	    if (system("perl c:/scripts/CreateWin32Setup.pl -drive $cwdrive -path d:\\builds\\partners -prefix $prefix-$partner- -configs Release")) {
		print("Error generating setups for $partner\n");
		exit(1);
	    }
	}
    }
} else { 
    die "Environment variable ZK_ROOT must be set" if (!exists($ENV{ZK_ROOT}));

    my $zkinstallDir = "$ENV{ZK_ROOT}/freedom/client/zkinstall/linux";
    my $regfilesDir = "$ENV{ZK_ROOT}/freedom/client/regfiles";
    my $pkgFreedomDir = "$ENV{ZK_ROOT}/arch/linux-2.2-ix86/Release/pkg/client/usr/share/freedom";

    if(chdir($zkinstallDir)) {
	foreach my $partner ("freeserve", "simtel", "tucows", "winsite", "zdnet") {
	    if (system("/bin/cp -f $regfilesDir/inituser.$partner.sh $pkgFreedomDir/inituser.sh")) {
		print("Error copying inituser.sh for $partner\n");
		exit(1);
	    }
	    if ( open(CMD, "make package|")) {
		my $rpm = undef;

		while ($_ = <CMD>) {
		    chomp;
		    if ( m,^Wrote: (.+),) {
			$rpm = $1;
		    }
		}
		close(CMD);
		if ( !defined($rpm)) {
		    print("Error: Can't get name of the rpm generated\n");
		    exit(1);
		}

		if ( ! $?) {
		    if (!rename($rpm, "$rpm.$partner")) {
			print("Error: Can't rename to $rpm.$partner\n");
			exit(1);
		    }
		} else {
		    print("Error: in the rpm generation for partner $partner\n");
		    exit(1);
		}
		print("Error generating rpm for $partner\n");
		exit(1);
	    } else {
		print("Error: can't run make command\n");
		exit(1);
	    }
	}
    } else {
	print("Error: can't chdir into '$zkinstallDir'\n");
	exit(1);
    }
}
exit(0);
