#ifndef YCIPHER_H
#define YCIPHER_H

/* block cipher interface */

/* default to 3DES for yarrow 160 */

#if !defined(YARROW_CIPHER_3DES) && !defined(YARROW_CIPHER_BLOWFISH)
# if !defined(YARROW_CIPHER_IDEA)
#  define YARROW_CIPHER_3DES
# endif
#endif

#if defined(YARROW_CIPHER_3DES)

/* For yarrow160 use 3 key 3DES */

#include "openssl/des.h"

typedef struct 
{
  des_key_schedule ks1, ks2, ks3;
} CIPHER_CTX;

#define CIPHER_BLOCK_SIZE DES_KEY_SZ
#define CIPHER_KEY_SIZE (DES_KEY_SZ * 3)

#define CIPHER_Init(ctx, key)\
  des_key_sched(((des_cblock*) (key))    , (ctx)->ks1);\
  des_key_sched(((des_cblock*) (key)) + 1, (ctx)->ks2);\
  des_key_sched(((des_cblock*) (key)) + 2, (ctx)->ks3)

#define CIPHER_Encrypt_Block(ctx, in, out)\
  des_ecb3_encrypt((des_cblock*) in, (des_cblock*) out,\
            (ctx)->ks1, (ctx)->ks2, (ctx)->ks3, 1)

#elif defined(YARROW_CIPHER_BLOWFISH)

/* macros to allow blowfish */

#include "openssl/blowfish.h"

typedef struct 
{
  BF_KEY ks;
} CIPHER_CTX;

#define CIPHER_BLOCK_SIZE BF_BLOCK
#define CIPHER_KEY_SIZE 16

#define CIPHER_Init(ctx, key)\
  BF_set_key(&(ctx)->ks, CIPHER_KEY_SIZE, (void*)key)
#define CIPHER_Encrypt_Block(ctx, in, out)\
  BF_ecb_encrypt((void*) in, (void*) out, &(ctx)->ks, 1)

#elif defined(YARROW_CIPHER_IDEA)

/* macros to allow IDEA */

#include "openssl/idea.h"

typedef struct 
{
  IDEA_KEY_SCHEDULE ks;
} CIPHER_CTX;

#define CIPHER_BLOCK_SIZE IDEA_BLOCK
#define CIPHER_KEY_SIZE IDEA_KEY_LENGTH

#define CIPHER_Init(ctx, key)\
  idea_set_encrypt_key((void*) key, &(ctx)->ks)

#define CIPHER_Encrypt_Block(ctx, in, out)\
  idea_ecb_encrypt((void*)in, (void*)out, &(ctx)->ks)

#endif

#endif /* YCIPHER_H */
